(function(mod) {
  if (typeof exports == "object" && typeof module == "object") // CommonJS
    return mod(require("tern/lib/infer"), require("tern/lib/tern"));
  if (typeof define == "function" && define.amd) // AMD
    return define([ "tern/lib/infer", "tern/lib/tern" ], mod);
  mod(tern, tern);
})(function(infer, tern) {
  "use strict";

  function flattenPath(path) {
    if (!/(^|\/)(\.\/|[^\/]+\/\.\.\/)/.test(path)) return path;
    var parts = path.split("/");
    for (var i = 0; i < parts.length; ++i) {
      if (parts[i] == "." || !parts[i]) parts.splice(i--, 1);
      else if (i && parts[i] == "..") parts.splice(i-- - 1, 2);
    }
    return parts.join("/");
  }

  function resolveName(name, data) {
    var excl = name.indexOf("!");
    if (excl > -1) name = name.slice(0, excl);

    var opts = data.options;
    var hasExt = /\.js$/.test(name);
    if (hasExt || /^(?:\w+:|\/)/.test(name))
      return name + (hasExt ? "" : ".js");

    var base = opts.baseURL || "";
    if (base && base.charAt(base.length - 1) != "/") base += "/";
    if (opts.paths) {
      var known = opts.paths[name];
      if (known) return flattenPath(base + known + ".js");
      var dir = name.match(/^([^\/]+)(\/.*)$/);
      if (dir) {
        var known = opts.paths[dir[1]];
        if (known) return flattenPath(base + known + dir[2] + ".js");
      }
    }
    return flattenPath(base + name + ".js");
  }
  
  function getInterface(name, data) {
    var module = infer.cx().definitions["dojotoolkit"][name];
    if (module) return module;
 
    if (name == "require") return getRequire(data);
    if (name == "module") return infer.cx().definitions["dojotoolkit"].module;

    if (data.options.override && Object.prototype.hasOwnProperty.call(data.options.override, name)) {
      var over = data.options.override[name];
      if (typeof over == "string" && over.charAt(0) == "=") return infer.def.parsePath(over.slice(1));
      if (typeof over == "object") {
        var known = getKnownModule(name, data);
        if (known) return known;
        var scope = data.interfaces[stripJSExt(name)] = new infer.Obj(null, stripJSExt(name));
        infer.def.load(over, scope);
        return scope;
      }
      name = over;
    }

    if (!/^(https?:|\/)|\.js$/.test(name))
      name = resolveName(name, data);
    name = flattenPath(name);

    var known = getKnownModule(name, data);

    if (!known) {
      known = getModule(name, data);
      data.server.addFile(name, null, data.currentFile);
    }
    return known;
  }
  
  function getKnownModule(name, data) {
    return data.interfaces[stripJSExt(name)];
  }

  function getModule(name, data) {
    var known = getKnownModule(name, data);
    if (!known) {
      known = data.interfaces[stripJSExt(name)] = new infer.AVal;
      known.origin = name;
    }
    return known;
  }

  var EXPORT_OBJ_WEIGHT = 50;

  function stripJSExt(f) {
    return f.replace(/\.js$/, '');
  }
  
  var path = {
    dirname: function(path) {
      var lastSep = path.lastIndexOf("/");
      return lastSep == -1 ? "" : path.slice(0, lastSep);
    },
    relative: function(from, to) {
      if (to.indexOf(from) == 0) return to.slice(from.length);
      else return to;
    },
    join: function(a, b) {
      if (a && b) return a + "/" + b;
      else return (a || "") + (b || "");
    },
  };
    
  infer.registerFunction("requireDojo", function(_self, args, argNodes) {
    var server = infer.cx().parent, data = server && server._dojotoolkit;
    if (!data || !args.length) return infer.ANull;

    var name = data.currentFile;
    var out = getModule(name, data);

    var deps = [], fn;
    if (argNodes && args.length > 1) {
      var node = argNodes[args.length == 2 ? 0 : 1];
      var base = path.relative(server.options.projectDir, path.dirname(node.sourceFile.name));
      if (node.type == "Literal" && typeof node.value == "string") {
        deps.push(getInterface(path.join(base, node.value), data));
      } else if (node.type == "ArrayExpression") for (var i = 0; i < node.elements.length; ++i) {
        var elt = node.elements[i];
        if (elt.type == "Literal" && typeof elt.value == "string") {
          if (elt.value == "exports") {
            var exports = new infer.Obj(true);
            deps.push(exports);
            out.addType(exports, EXPORT_OBJ_WEIGHT);
          } else {
            deps.push(getInterface(path.join(base, elt.value), data));
          }
        }
      }
    } else if (argNodes && args.length == 1 && argNodes[0].type == "FunctionExpression" && argNodes[0].params.length) {
      // Simplified CommonJS call
      var exports = new infer.Obj(true);
      deps.push(getInterface("require", data), exports);
      out.addType(exports, EXPORT_OBJ_WEIGHT);
      fn = args[0];
    }

    if (!fn) {
      fn = args[Math.min(args.length - 1, 2)];
      if (!fn.isEmpty() && !fn.getFunctionType()) fn = null;
    }

    if (fn) fn.propagate(new infer.IsCallee(infer.ANull, deps, null, out));
    else if (args.length) args[0].propagate(out);

    return infer.ANull;
  });

  
  tern.registerPlugin("dojotoolkit1.9", function(server, options) {
    server._dojotoolkit = {
      interfaces: Object.create(null),
      options: options || {},
      currentFile: null,
      server: server
    };

    server.on("beforeLoad", function(file) {
      this._dojotoolkit.currentFile = file.name;
    });
    server.on("reset", function() {
      this._dojotoolkit.interfaces = Object.create(null);
      this._dojotoolkit.require = null;
    });
    return {
      defs : defs
    };
  });

  var defs = {"!name":"dojotoolkit","!define":{"dojo/AdapterRegistry":{"!doc":"<p>A registry to make contextual calling/searching easier.</p>\n","pairs":{},"returnWrappers":{},"prototype":{"constructor":{"!type":"fn(returnWrappers?: ?)","!doc":"<p>A registry to make contextual calling/searching easier.</p>\n"},"match":{"!type":"fn() -> ?","!doc":"<p>Find an adapter for the given arguments. If no suitable adapter\nis found, throws an exception. match() accepts any number of\narguments, all of which are passed to all matching functions\nfrom the registered pairs.</p>\n"},"register":{"!type":"fn(name: ?, check: ?, wrap: ?, directReturn?: ?, override?: ?)","!doc":"<p>register a check function to determine if the wrap function or\nobject gets selected</p>\n"},"unregister":{"!type":"fn(name: ?) -> bool","!doc":"<p>Remove a named adapter from the registry</p>\n"}}},"dojo/_base/kernel":{"!doc":"<p>This module is the foundational module of the dojo boot sequence; it defines the dojo object.</p>\n","_blockAsync":{},"_contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n"},"_defaultContext":{},"_hasResource":{},"_initFired":{},"_name":{},"_nodeDataCache":{},"_NodeListCtor":{},"_postLoad":{},"_windowUnloaders":{},"back":{"!doc":"<p>Browser history management resources</p>\n"},"baseUrl":{},"behavior":{},"cldr":{},"colors":{},"config":{"!doc":"<p>This module defines the user configuration during bootstrap.</p>\n"},"connectPublisher":{},"contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n"},"currency":{"!doc":"<p>localized formatting and parsing routines for currencies</p>\n"},"data":{},"date":{},"dijit":{},"dnd":{},"doc":{"!doc":"<p>Alias for the current document. &#39;doc&#39; can be modified\nfor temporary context shifting. See also withDoc().</p>\n"},"dojox":{},"fx":{"!doc":"<p>Effects library on top of Base animations</p>\n"},"gears":{"!doc":"<p>TODOC</p>\n"},"global":{"!doc":"<p>Alias for the current window. &#39;global&#39; can be modified\nfor temporary context shifting. See also withGlobal().</p>\n"},"html":{"!doc":"<p>TODOC</p>\n"},"i18n":{"!doc":"<p>This module implements the dojo/i18n! plugin and the v1.6- i18n API</p>\n"},"io":{},"isAir":{"!doc":"<p>True if client is Adobe Air</p>\n"},"isAndroid":{"!doc":"<p>Version as a Number if client is android browser. undefined otherwise.</p>\n"},"isAsync":{},"isBrowser":{},"isChrome":{"!doc":"<p>Version as a Number if client is Chrome browser. undefined otherwise.</p>\n"},"isCopyKey":{},"isFF":{},"isIE":{"!doc":"<p>Version as a Number if client is MSIE(PC). undefined otherwise. Corresponds to\nmajor detected IE version (6, 7, 8, etc.)</p>\n"},"isIos":{"!doc":"<p>Version as a Number if client is iPhone, iPod, or iPad. undefined otherwise.</p>\n"},"isKhtml":{"!doc":"<p>Version as a Number if client is a KHTML browser. undefined otherwise. Corresponds to major\ndetected version.</p>\n"},"isMac":{"!doc":"<p>True if the client runs on Mac</p>\n"},"isMoz":{},"isMozilla":{},"isOpera":{"!doc":"<p>Version as a Number if client is Opera. undefined otherwise. Corresponds to\nmajor detected version.</p>\n"},"isQuirks":{},"isSafari":{"!doc":"<p>Version as a Number if client is Safari or iPhone. undefined otherwise.</p>\n"},"isSpidermonkey":{},"isWebKit":{"!doc":"<p>Version as a Number if client is a WebKit-derived browser (Konqueror,\nSafari, Chrome, etc.). undefined otherwise.</p>\n"},"isWii":{"!doc":"<p>True if client is Wii</p>\n"},"keys":{"!doc":"<p>Definitions for common key values.  Client code should test keyCode against these named constants,\nas the actual codes can vary by browser.</p>\n"},"locale":{},"mouseButtons":{},"number":{"!doc":"<p>localized formatting and parsing routines for Number</p>\n"},"parser":{},"publish":{},"query":{},"regexp":{"!doc":"<p>Regular expressions and Builder resources</p>\n"},"rpc":{},"scopeMap":{},"store":{},"string":{"!doc":"<p>String utilities for Dojo</p>\n"},"subscribe":{},"tests":{"!doc":"<p>D.O.H. Test files for Dojo unit testing.</p>\n"},"toJsonIndentStr":{},"touch":{"!doc":"<p>This module provides unified touch event handlers by exporting\npress, move, release and cancel which can also run well on desktop.\nBased on <a href=\"http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html\">http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html</a>\nAlso, if the dojoClick property is set to truthy on a DOM node, dojo/touch generates\nclick events immediately for this node and its descendants (except for descendants that\nhave a dojoClick property set to falsy), to avoid the delay before native browser click events,\nand regardless of whether evt.preventDefault() was called in a touch.press event listener.\n</p>\n"},"version":{"!doc":"<p>Version number of the Dojo Toolkit</p>\n"},"window":{"!doc":"<p>TODOC</p>\n"},"AdapterRegistry":{"!type":"fn(returnWrappers?: ?)","!doc":"<p>A registry to make contextual calling/searching easier.</p>\n"},"addClass":{"!type":"fn(node: ?, classStr: ?)","!doc":"<p>Adds the specified classes to the end of the class list on the\npassed node. Will not re-apply duplicate classes.\n</p>\n"},"addOnLoad":{"!type":"fn(priority?: ?, context: ?, callback?: ?)","!doc":"<p>Add a function to execute on DOM content loaded and all requested modules have arrived and been evaluated.\nIn most cases, the <code>domReady</code> plug-in should suffice and this method should not be needed.\n\n</p>\n<p>When called in a non-browser environment, just checks that all requested modules have arrived and been\nevaluated.</p>\n"},"addOnUnload":{"!type":"fn(obj?: ?, functionName?: ?)","!doc":"<p>registers a function to be triggered when the page unloads.</p>\n"},"addOnWindowUnload":{"!type":"fn(obj?: ?, functionName?: ?)","!doc":"<p>registers a function to be triggered when window.onunload fires.\nBe careful trying to modify the DOM or access JavaScript properties\nduring this phase of page unloading: they may not always be available.\nConsider dojo.addOnUnload() if you need to modify the DOM or do heavy\nJavaScript work.</p>\n"},"anim":{"!type":"fn(node: ?, properties: ?, duration?: ?, easing?: ?, onEnd?: ?, delay?: ?) -> ?","!doc":"<p>A simpler interface to <code>animateProperty()</code>, also returns\nan instance of <code>Animation</code> but begins the animation\nimmediately, unlike nearly every other Dojo animation API.</p>\n"},"animateProperty":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation that will transition the properties of\nnode defined in <code>args</code> depending how they are defined in\n<code>args.properties</code>\n</p>\n"},"Animation":{"!type":"fn(args: ?)","!doc":"<p>A generic animation class that fires callbacks into its handlers\nobject at various states.</p>\n"},"attr":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Gets or sets an attribute on an HTML element.</p>\n"},"blendColors":{"!type":"fn(start: ?, end: ?, weight: ?, obj?: ?) -> ?","!doc":"<p>Blend colors end and start with weight from 0 to 1, 0.5 being a 50/50 blend,\ncan reuse a previously allocated Color object for the result</p>\n"},"body":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Return the body element of the specified document or of dojo/_base/window::doc.</p>\n"},"byId":{"!type":"fn(id: ?, doc?: ?) -> +instance","!doc":"<p>Returns DOM node with matching <code>id</code> attribute or falsy value (ex: null or undefined)\nif not found.  If <code>id</code> is a DomNode, this function is a no-op.\n</p>\n"},"cache":{"!type":"fn(module: ?, url: ?, value?: ?) -> ?","!doc":"<p>A getter and setter for storing the string content associated with the\nmodule and url arguments.</p>\n"},"clearCache":{"!type":"fn()"},"Color":{"!type":"fn(color: ?)","!doc":"<p>Takes a named string, hex string, array of rgb or rgba values,\nan object with r, g, b, and a properties, or another <code>Color</code> object\nand creates a new Color instance to work from.\n</p>\n"},"colorFromArray":{"!type":"fn(a: ?, obj?: ?) -> +any","!doc":"<p>Builds a <code>Color</code> from a 3 or 4 element array, mapping each\nelement in sequence to the rgb(a) values of the color.</p>\n"},"colorFromHex":{"!type":"fn(color: ?, obj?: ?) -> +any","!doc":"<p>Converts a hex string with a &#39;#&#39; prefix to a color object.\nSupports 12-bit #rgb shorthand. Optionally accepts a\n<code>Color</code> object to update with the parsed value.\n</p>\n"},"colorFromRgb":{"!type":"fn(color: ?, obj?: ?) -> +null","!doc":"<p>get rgb(a) array from css-style color declarations</p>\n"},"colorFromString":{"!type":"fn(str: ?, obj?: ?) -> +any","!doc":"<p>Parses <code>str</code> for a color value. Accepts hex, rgb, and rgba\nstyle color values.</p>\n"},"connect":{"!type":"fn(obj?: ?, event: ?, context: ?, method: ?, dontFix?: ?) -> ?","!doc":"<p><code>dojo.connect</code> is a deprecated event handling and delegation method in\nDojo. It allows one function to &quot;listen in&quot; on the execution of\nany other, triggering the second whenever the first is called. Many\nlisteners may be attached to a function, and source functions may\nbe either regular function calls or DOM events.\n</p>\n"},"contentBox":{"!type":"fn(node: ?, box?: ?) -> ?","!doc":"<p>Getter/setter for the content-box of node.</p>\n"},"cookie":{"!type":"fn(name: ?, value?: ?, props?: ?) -> ?","!doc":"<p>Get or set a cookie.</p>\n"},"coords":{"!type":"fn(node: ?, includeScroll?: ?) -> ?","!doc":"<p>Deprecated: Use position() for border-box x/y/w/h\nor marginBox() for margin-box w/h/l/t.\n\n</p>\n<p>Returns an object that measures margin-box (w)idth/(h)eight\nand absolute position x/y of the border-box. Also returned\nis computed (l)eft and (t)op values in pixels from the\nnode&#39;s offsetParent as returned from marginBox().\nReturn value will be in the form:\n\n</p>\n<pre><code>{ l: 50, t: 200, w: 300: h: 150, x: 100, y: 300 }</code></pre>\n<p>Does not act as a setter. If includeScroll is passed, the x and\n\n</p>\n<p>y params are affected as one would expect in dojo.position().</p>\n"},"create":{"!type":"fn(tag: ?, attrs: ?, refNode?: ?, pos?: ?) -> ?","!doc":"<p>Create an element, allowing for optional attribute decoration\nand placement.</p>\n"},"declare":{"!type":"fn(className?: ?, superclass: ?, props: ?) -> +dojo","!doc":"<p>Create a feature-rich constructor from compact notation.</p>\n"},"Deferred":{"!type":"fn(canceller?: ?)","!doc":"<p>Deprecated.   This module defines the legacy dojo/_base/Deferred API.\nNew code should use dojo/Deferred instead.</p>\n"},"DeferredList":{"!type":"fn(list: ?, fireOnOneCallback?: ?, fireOnOneErrback?: ?, consumeErrors?: ?, canceller?: ?)","!doc":"<p>Deprecated, use dojo/promise/all instead.\nProvides event handling for a group of Deferred objects.</p>\n"},"deprecated":{"!type":"fn(behaviour: ?, extra?: ?, removal?: ?)","!doc":"<p>Log a debug message to indicate that a behavior has been\ndeprecated.</p>\n"},"destroy":{"!type":"fn(node: ?)"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Remove a link created by dojo.connect.</p>\n"},"docScroll":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Returns an object with {node, x, y} with corresponding offsets.</p>\n"},"empty":{"!type":"fn(node: ?)"},"eval":{"!type":"fn(scriptText: ?) -> +any","!doc":"<p>A legacy method created for use exclusively by internal Dojo methods. Do not use this method\ndirectly unless you understand its possibly-different implications on the platforms your are targeting.</p>\n"},"every":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?) -> bool","!doc":"<p>Determines whether or not every item in arr satisfies the\ncondition implemented by callback.</p>\n"},"exit":{"!type":"fn(exitcode: ?)"},"experimental":{"!type":"fn(moduleName: ?, extra?: ?)","!doc":"<p>Marks code as experimental.</p>\n"},"fadeIn":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that will fade node defined in &#39;args&#39; from\nits current opacity to fully opaque.</p>\n"},"fadeOut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that will fade node defined in &#39;args&#39;\nfrom its current opacity to fully transparent.</p>\n"},"fieldToObject":{"!type":"fn(inputNode: ?) -> ?","!doc":"<p>Serialize a form field to a JavaScript object.</p>\n"},"filter":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?) -> []","!doc":"<p>Returns a new Array with those items from arr that match the\ncondition implemented by callback.</p>\n"},"fixEvent":{"!type":"fn(evt: ?, sender: ?) -> +Event","!doc":"<p>normalizes properties on the event object including event\nbubbling methods, keystroke normalization, and x/y positions</p>\n"},"fixIeBiDiScrollLeft":{"!type":"fn(scrollLeft: ?, doc?: ?) -> number","!doc":"<p>In RTL direction, scrollLeft should be a negative value, but IE\nreturns a positive one. All codes using documentElement.scrollLeft\nmust call this function to fix this error, otherwise the position\nwill offset to right when there is a horizontal scrollbar.</p>\n"},"forEach":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?)","!doc":"<p>for every item in arr, callback is invoked. Return values are ignored.\nIf you want to break out of the loop, consider using array.every() or array.some().\nforEach does not allow breaking out of the loop over the items in arr.</p>\n"},"formToJson":{"!type":"fn(formNode: ?, prettyPrint?: ?) -> string","!doc":"<p>Create a serialized JSON string from a form node or string\nID identifying the form to serialize</p>\n"},"formToObject":{"!type":"fn(formNode: ?) -> ?","!doc":"<p>Serialize a form node to a JavaScript object.</p>\n"},"formToQuery":{"!type":"fn(formNode: ?) -> string","!doc":"<p>Returns a URL-encoded string representing the form passed as either a\nnode or string ID identifying the form to serialize</p>\n"},"fromJson":{"!type":"fn(js: ?) -> ?","!doc":"<p>Parses a JavaScript expression and returns a JavaScript value.</p>\n"},"getAttr":{"!type":"fn(node: ?, name: ?) -> +any","!doc":"<p>Gets an attribute on an HTML element.</p>\n"},"getBorderExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns an object with properties useful for noting the border\ndimensions.</p>\n"},"getComputedStyle":{"!type":"fn(node: ?)","!doc":"<p>Returns a &quot;computed style&quot; object.\n</p>\n"},"getContentBox":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>Returns an object that encodes the width, height, left and top\npositions of the node&#39;s content box, irrespective of the\ncurrent box model.</p>\n"},"getIeDocumentElementOffset":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>returns the offset in x and y from the document body to the\nvisual edge of the page for IE</p>\n"},"getL10nName":{"!type":"fn(moduleName: ?, bundleName: ?, locale: ?) -> string"},"getMarginBox":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns an object that encodes the width, height, left and top\npositions of the node&#39;s margin box.</p>\n"},"getMarginExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns object with properties useful for box fitting with\nregards to box margins (i.e., the outer-box).\n\n</p>\n<ul>\n<li>l/t = marginLeft, marginTop, respectively</li>\n<li>w = total width, margin inclusive</li>\n<li>h = total height, margin inclusive</li>\n</ul>\n<p>The w/h are used for calculating boxes.\nNormally application code will not need to invoke this\ndirectly, and will use the ...box... functions instead.</p>\n"},"getMarginSize":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns an object that encodes the width and height of\nthe node&#39;s margin box</p>\n"},"getNodeProp":{"!type":"fn(node: ?, name: ?) -> +any","!doc":"<p>Returns an effective value of a property or an attribute.</p>\n"},"getPadBorderExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>Returns object with properties useful for box fitting with\nregards to padding.</p>\n"},"getPadExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>Returns object with special values specifically useful for node\nfitting.</p>\n"},"getProp":{"!type":"fn(node: ?, name: ?) -> +any","!doc":"<p>Gets a property on an HTML element.</p>\n"},"getStyle":{"!type":"fn(node: ?, name?: ?) -> ?","!doc":"<p>Accesses styles on a node.</p>\n"},"hasAttr":{"!type":"fn(node: ?, name: ?) -> bool","!doc":"<p>Returns true if the requested attribute is specified on the\ngiven element, and false otherwise.</p>\n"},"hasClass":{"!type":"fn(node: ?, classStr: ?) -> bool","!doc":"<p>Returns whether or not the specified classes are a portion of the\nclass list currently applied to the node.</p>\n"},"hash":{"!type":"fn(hash?: ?, replace?: ?) -> +any","!doc":"<p>Gets or sets the hash string in the browser URL.</p>\n"},"indexOf":{"!type":"fn(arr: ?, value: ?, fromIndex?: ?, findLast?: ?) -> number","!doc":"<p>locates the first index of the provided value in the\npassed array. If the value is not found, -1 is returned.</p>\n"},"isBodyLtr":{"!type":"fn(doc?: ?) -> bool","!doc":"<p>Returns true if the current language is left-to-right, and false otherwise.</p>\n"},"isDescendant":{"!type":"fn(node: ?, ancestor: ?) -> bool","!doc":"<p>Returns true if node is a descendant of ancestor</p>\n"},"lastIndexOf":{"!type":"fn(arr: ?, value: ?, fromIndex?: ?) -> number","!doc":"<p>locates the last index of the provided value in the passed\narray. If the value is not found, -1 is returned.</p>\n"},"loadInit":{"!type":"fn(f: ?)"},"map":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?, Ctr: ?) -> []","!doc":"<p>applies callback to each element of arr and returns\nan Array with the results</p>\n"},"marginBox":{"!type":"fn(node: ?, box?: ?) -> ?","!doc":"<p>Getter/setter for the margin-box of node.</p>\n"},"moduleUrl":{"!type":"fn(module: ?, url?: ?) -> string","!doc":"<p>Returns a URL relative to a module.</p>\n"},"NodeList":{"!type":"fn(array: ?) -> []","!doc":"<p>Array-like object which adds syntactic\nsugar for chaining, common iteration operations, animation, and\nnode manipulation. NodeLists are most often returned as the\nresult of dojo/query() calls.</p>\n"},"objectToQuery":{"!type":"fn(map: ?) -> ?","!doc":"<p>takes a name/value mapping object and returns a string representing\na URL-encoded version of that object.</p>\n"},"place":{"!type":"fn(node: ?, refNode: ?, position?: ?) -> +Node","!doc":"<p>Attempt to insert node into the DOM, choosing from various positioning options.\nReturns the first argument resolved to a DOM node.</p>\n"},"platformRequire":{"!type":"fn(modMap: ?)","!doc":"<p>require one or more modules based on which host environment\nDojo is currently operating in</p>\n"},"popContext":{"!type":"fn()","!doc":"<p>If the context stack contains elements, ensure that\nsubsequent code executes in the <em>previous</em> context to the\ncurrent context. The current context set ([global,\ndocument]) is returned.</p>\n"},"position":{"!type":"fn(node: ?, includeScroll?: ?) -> ?","!doc":"<p>Gets the position and size of the passed element relative to\nthe viewport (if includeScroll==false), or relative to the\ndocument root (if includeScroll==true).\n</p>\n"},"prop":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Gets or sets a property on an HTML element.</p>\n"},"provide":{"!type":"fn(mid: ?)"},"pushContext":{"!type":"fn(g?: ?, d?: ?)","!doc":"<p>causes subsequent calls to Dojo methods to assume the\npassed object and, optionally, document as the default\nscopes to use. A 2-element array of the previous global and\ndocument are returned.</p>\n"},"queryToObject":{"!type":"fn(str: ?) -> ?","!doc":"<p>Create an object representing a de-serialized query section of a\nURL. Query keys with multiple values are returned in an array.\n</p>\n"},"rawXhrPost":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP POST request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"},"rawXhrPut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP PUT request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"},"ready":{"!type":"fn(priority?: ?, context: ?, callback?: ?)","!doc":"<p>Add a function to execute on DOM content loaded and all requested modules have arrived and been evaluated.\nIn most cases, the <code>domReady</code> plug-in should suffice and this method should not be needed.\n\n</p>\n<p>When called in a non-browser environment, just checks that all requested modules have arrived and been\nevaluated.</p>\n"},"registerModulePath":{"!type":"fn(moduleName: ?, prefix: ?)","!doc":"<p>Maps a module name to a path</p>\n"},"removeAttr":{"!type":"fn(node: ?, name: ?)","!doc":"<p>Removes an attribute from an HTML element.</p>\n"},"removeClass":{"!type":"fn(node: ?, classStr?: ?)","!doc":"<p>Removes the specified classes from node. No <code>contains()</code>\ncheck is required.\n</p>\n"},"replaceClass":{"!type":"fn(node: ?, addClassStr: ?, removeClassStr?: ?)","!doc":"<p>Replaces one or more classes on a node if not present.\nOperates more quickly than calling dojo.removeClass and dojo.addClass\n</p>\n"},"require":{"!type":"fn(moduleName: ?, omitModuleCheck?: ?) -> +any","!doc":"<p>loads a Javascript module from the appropriate URI\n</p>\n"},"requireAfterIf":{"!type":"fn(condition: ?, moduleName: ?, omitModuleCheck?: ?)","!doc":"<p>If the condition is true then call <code>dojo.require()</code> for the specified\nresource\n</p>\n"},"requireIf":{"!type":"fn(condition: ?, moduleName: ?, omitModuleCheck?: ?)","!doc":"<p>If the condition is true then call <code>dojo.require()</code> for the specified\nresource\n</p>\n"},"requireLocalization":{"!type":"fn(moduleName: ?, bundleName: ?, locale?: ?)"},"safeMixin":{"!type":"fn(target: ?, source: ?) -> ?","!doc":"<p>Mix in properties skipping a constructor and decorating functions\nlike it is done by declare().</p>\n"},"setAttr":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Sets an attribute on an HTML element.</p>\n"},"setContentSize":{"!type":"fn(node: ?, box: ?, computedStyle?: ?)","!doc":"<p>Sets the size of the node&#39;s contents, irrespective of margins,\npadding, or borders.</p>\n"},"setContext":{"!type":"fn(globalObject: ?, globalDocument: ?)","!doc":"<p>changes the behavior of many core Dojo functions that deal with\nnamespace and DOM lookup, changing them to work in a new global\ncontext (e.g., an iframe). The varibles dojo.global and dojo.doc\nare modified as a result of calling this function and the result of\n<code>dojo.body()</code> likewise differs.</p>\n"},"setMarginBox":{"!type":"fn(node: ?, box: ?, computedStyle?: ?)","!doc":"<p>sets the size of the node&#39;s margin box and placement\n(left/top), irrespective of box model. Think of it as a\npassthrough to setBox that handles box-model vagaries for\nyou.</p>\n"},"setProp":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Sets a property on an HTML element.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)"},"setStyle":{"!type":"fn(node: ?, name: ?, value?: ?) -> string","!doc":"<p>Sets styles on a node.</p>\n"},"some":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?) -> bool","!doc":"<p>Determines whether or not any item in arr satisfies the\ncondition implemented by callback.</p>\n"},"Stateful":{"!type":"fn()"},"stopEvent":{"!type":"fn(evt: ?)","!doc":"<p>prevents propagation and clobbers the default action of the\npassed event</p>\n"},"style":{"!type":"fn(node: ?, name?: ?, value?: ?) -> +any","!doc":"<p>Accesses styles on a node. If 2 arguments are\npassed, acts as a getter. If 3 arguments are passed, acts\nas a setter.</p>\n"},"toDom":{"!type":"fn(frag: ?, doc?: ?) -> +any","!doc":"<p>instantiates an HTML fragment returning the corresponding DOM.</p>\n"},"toggleClass":{"!type":"fn(node: ?, classStr: ?, condition?: ?) -> bool","!doc":"<p>Adds a class to node if not present, or removes if present.\nPass a boolean condition if you want to explicitly add or remove.\nReturns the condition that was specified directly or indirectly.\n</p>\n"},"toJson":{"!type":"fn(it: ?, prettyPrint?: ?) -> +any","!doc":"<p>Returns a <a href=\"http://json.org\">JSON</a> serialization of an object.</p>\n"},"toPixelValue":{"!type":"fn(node: ?, value: ?) -> number","!doc":"<p>converts style value to pixels on IE or return a numeric value.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Remove a topic listener.</p>\n"},"when":{"!type":"fn(valueOrPromise: ?, callback?: ?, errback?: ?, progback?: ?) -> +dojo","!doc":"<p>Transparently applies callbacks to values and/or promises.</p>\n"},"windowUnloaded":{"!type":"fn()","!doc":"<p>signal fired by impending window destruction. You may use\ndojo.addOnWIndowUnload() or dojo.connect() to this method to perform\npage/application cleanup methods. See dojo.addOnWindowUnload for more info.</p>\n"},"withDoc":{"!type":"fn(documentObject: ?, callback: ?, thisObject?: ?, cbArguments?: ?) -> ?","!doc":"<p>Invoke callback with documentObject as dojo/_base/window::doc.</p>\n"},"withGlobal":{"!type":"fn(globalObject: ?, callback: ?, thisObject?: ?, cbArguments?: ?) -> ?","!doc":"<p>Invoke callback with globalObject as dojo.global and\nglobalObject.document as dojo.doc.</p>\n"},"xhr":{"!type":"fn(method: ?, args: ?) -> ?"},"xhrDelete":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP DELETE request to the server.</p>\n"},"xhrGet":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP GET request to the server.</p>\n"},"xhrPost":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP POST request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"},"xhrPut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP PUT request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"}},"dojo/_base/kernel.config":{"!doc":"<p>This module defines the user configuration during bootstrap.</p>\n","addOnLoad":{"!doc":"<p>Adds a callback via dojo/ready. Useful when Dojo is added after\nthe page loads and djConfig.afterOnLoad is true. Supports the same\narguments as dojo/ready. When using a function reference, use\n<code>djConfig.addOnLoad = function(){};</code>. For object with function name use\n<code>djConfig.addOnLoad = [myObject, &quot;functionName&quot;];</code> and for object with\nfunction reference use\n<code>djConfig.addOnLoad = [myObject, function(){}];</code></p>\n"},"afterOnLoad":{},"baseUrl":{},"callback":{"!doc":"<p>Defines a callback to be used when dependencies are defined before \nthe loader has been loaded. When provided, they cause the loader to \nexecute require(deps, callback) once it has finished loading. \nShould be used with deps.</p>\n"},"debugContainerId":{},"debugHeight":{},"defaultDuration":{"!doc":"<p>Default duration, in milliseconds, for wipe and fade animations within dijits.\nAssigned to dijit.defaultDuration.</p>\n"},"deferredInstrumentation":{"!doc":"<p>Whether deferred instrumentation should be loaded or included\nin builds.</p>\n"},"deps":{"!doc":"<p>Defines dependencies to be used before the loader has been loaded.\nWhen provided, they cause the loader to execute require(deps, callback) \nonce it has finished loading. Should be used with callback.</p>\n"},"dojoBlankHtmlUrl":{"!doc":"<p>Used by some modules to configure an empty iframe. Used by dojo/io/iframe and\ndojo/back, and dijit/popup support in IE where an iframe is needed to make sure native\ncontrols do not bleed through the popups. Normally this configuration variable\ndoes not need to be set, except when using cross-domain/CDN Dojo builds.\nSave dojo/resources/blank.html to your domain and set <code>djConfig.dojoBlankHtmlUrl</code>\nto the path on your domain your copy of blank.html.</p>\n"},"extraLocale":{"!doc":"<p>No default value. Specifies additional locales whose\nresources should also be loaded alongside the default locale when\ncalls to <code>dojo.requireLocalization()</code> are processed.</p>\n"},"ioPublish":{"!doc":"<p>Set this to true to enable publishing of topics for the different phases of\nIO operations. Publishing is done via dojo/topic.publish(). See dojo/main.__IoPublish for a list\nof topics that are published.</p>\n"},"isDebug":{"!doc":"<p>Defaults to <code>false</code>. If set to <code>true</code>, ensures that Dojo provides\nextended debugging feedback via Firebug. If Firebug is not available\non your platform, setting <code>isDebug</code> to <code>true</code> will force Dojo to\npull in (and display) the version of Firebug Lite which is\nintegrated into the Dojo distribution, thereby always providing a\ndebugging/logging console when <code>isDebug</code> is enabled. Note that\nFirebug&#39;s <code>console.*</code> methods are ALWAYS defined by Dojo. If\n<code>isDebug</code> is false and you are on a platform without Firebug, these\nmethods will be defined as no-ops.</p>\n"},"locale":{"!doc":"<p>The locale to assume for loading localized resources in this page,\nspecified according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a>.\nMust be specified entirely in lowercase, e.g. <code>en-us</code> and <code>zh-cn</code>.\nSee the documentation for <code>dojo.i18n</code> and <code>dojo.requireLocalization</code>\nfor details on loading localized resources. If no locale is specified,\nDojo assumes the locale of the user agent, according to <code>navigator.userLanguage</code>\nor <code>navigator.language</code> properties.</p>\n"},"modulePaths":{"!doc":"<p>A map of module names to paths relative to <code>dojo.baseUrl</code>. The\nkey/value pairs correspond directly to the arguments which\n<code>dojo.registerModulePath</code> accepts. Specifying\n<code>djConfig.modulePaths = { &quot;foo&quot;: &quot;../../bar&quot; }</code> is the equivalent\nof calling <code>dojo.registerModulePath(&quot;foo&quot;, &quot;../../bar&quot;);</code>. Multiple\nmodules may be configured via <code>djConfig.modulePaths</code>.</p>\n"},"parseOnLoad":{"!doc":"<p>Run the parser after the page is loaded</p>\n"},"require":{"!doc":"<p>An array of module names to be loaded immediately after dojo.js has been included\nin a page.</p>\n"},"transparentColor":{"!doc":"<p>Array containing the r, g, b components used as transparent color in dojo.Color;\nif undefined, [255,255,255] (white) will be used.</p>\n"},"urchin":{"!doc":"<p>Used by <code>dojox.analytics.Urchin</code> as the default UA-123456-7 account\nnumber used when being created. Alternately, you can pass an acct:&quot;&quot;\nparameter to the constructor a la: new dojox.analytics.Urchin({ acct:&quot;UA-123456-7&quot; });</p>\n"},"useCustomLogger":{"!doc":"<p>If set to a value that evaluates to true such as a string or array and\nisDebug is true and Firebug is not available or running, then it bypasses\nthe creation of Firebug Lite allowing you to define your own console object.</p>\n"},"useDeferredInstrumentation":{"!doc":"<p>Whether the deferred instrumentation should be used.\n\n</p>\n<ul>\n<li><code>&quot;report-rejections&quot;</code>: report each rejection as it occurs.</li>\n<li><code>true</code> or <code>1</code> or <code>&quot;report-unhandled-rejections&quot;</code>: wait 1 second\n  in an attempt to detect unhandled rejections.</li>\n</ul>\n"}},"dojo/_base/kernel.global":{"!doc":"<p>Alias for the current window. &#39;global&#39; can be modified\nfor temporary context shifting. See also withGlobal().</p>\n","$":{"!type":"fn() -> ?"},"GoogleSearchStoreCallback_undefined_NaN":{"!type":"fn(start: ?, data: ?, responseCode: ?, errorMsg: ?)"},"jQuery":{"!type":"fn() -> ?"},"swfIsInHTML":{"!type":"fn()"},"undefined_onload":{"!type":"fn()"}},"dojo/_base/kernel.dijit":{"_editor":{},"_tree":{},"form":{},"layout":{},"range":{"!doc":"<p>W3C range API</p>\n"},"registry":{},"tree":{},"byId":{"!type":"fn(id: ?) -> ?"},"Calendar":{"!type":"fn()"},"CalendarLite":{"!type":"fn()"},"CheckedMenuItem":{"!type":"fn()"},"ColorPalette":{"!type":"fn()"},"Declaration":{"!type":"fn()"},"Destroyable":{"!type":"fn()"},"Dialog":{"!type":"fn()"},"DialogUnderlay":{"!type":"fn()"},"DropDownMenu":{"!type":"fn()"},"Dye":{"!type":"fn()"},"Editor":{"!type":"fn()"},"Fieldset":{"!type":"fn()"},"InlineEditBox":{"!type":"fn()"},"Menu":{"!type":"fn()"},"MenuBar":{"!type":"fn()"},"MenuBarItem":{"!type":"fn()"},"MenuItem":{"!type":"fn()"},"MenuSeparator":{"!type":"fn()"},"PopupMenuBarItem":{"!type":"fn()"},"PopupMenuItem":{"!type":"fn()"},"ProgressBar":{"!type":"fn()"},"RadioButtonMenuItem":{"!type":"fn()"},"TitlePane":{"!type":"fn()"},"Toolbar":{"!type":"fn()"},"ToolbarSeparator":{"!type":"fn()"},"Tooltip":{"!type":"fn()"},"TooltipDialog":{"!type":"fn()"},"Tree":{"!type":"fn()"},"WidgetSet":{"!type":"fn()"}},"dojo/_base/kernel.dojox":{"analytics":{},"app":{},"atom":{},"av":{},"BidiComplex":{},"calc":{},"calendar":{},"charting":{},"collections":{},"color":{},"css3":{},"data":{},"date":{},"dgauges":{},"dnd":{},"drawing":{},"dtl":{},"editor":{},"embed":{},"encoding":{},"enhanced":{},"flash":{"!doc":"<p>Utilities to embed and communicate with the Flash player from Javascript</p>\n"},"form":{},"fx":{},"gantt":{},"gauges":{},"geo":{},"gesture":{},"gfx":{},"gfx3d":{},"grid":{},"help":{"!doc":"<p>Adds the help function to all variables.</p>\n"},"highlight":{},"html":{},"image":{},"io":{},"jq":{},"json":{},"jsonPath":{},"lang":{},"layout":{},"math":{},"mdnd":{},"mobile":{},"mvc":{},"openlayers":{},"rails":{},"robot":{},"rpc":{},"secure":{},"sketch":{},"sql":{},"storage":{},"string":{},"testing":{},"timing":{},"treemap":{},"uuid":{},"validate":{},"widget":{},"xml":{},"socket":{"!type":"fn(argsOrUrl: ?) -> +any","!doc":"<p>Provides a simple socket connection using WebSocket, or alternate\ncommunication mechanisms in legacy browsers for comet-style communication. This is based\non the WebSocket API and returns an object that implements the WebSocket interface:\n<a href=\"http://dev.w3.org/html5/websockets/#websocket\">http://dev.w3.org/html5/websockets/#websocket</a></p>\n"},"sprintf":{"!type":"fn(format: ?, filler: ?)"}},"dojo/_base/kernel.scopeMap":{"dijit":{},"dojo":{},"dojox":{}},"dojo/_base/kernel.version":{"!doc":"<p>Version number of the Dojo Toolkit</p>\n","flag":{},"major":{},"minor":{},"patch":{},"revision":{},"toString":{"!type":"fn() -> string"}},"dojo/_base/kernel._hasResource":{},"dojo/_base/kernel.doc":{"!doc":"<p>Alias for the current document. &#39;doc&#39; can be modified\nfor temporary context shifting. See also withDoc().</p>\n","documentElement":{},"dojoClick":{}},"dojo/_base/kernel.mouseButtons":{"LEFT":{"!doc":"<p>Numeric value of the left mouse button for the platform.</p>\n"},"MIDDLE":{"!doc":"<p>Numeric value of the middle mouse button for the platform.</p>\n"},"RIGHT":{"!doc":"<p>Numeric value of the right mouse button for the platform.</p>\n"},"isButton":{"!type":"fn(e: ?, button: ?) -> bool","!doc":"<p>Checks an event object for a pressed button</p>\n"},"isLeft":{"!type":"fn(e: ?) -> bool","!doc":"<p>Checks an event object for the pressed left button</p>\n"},"isMiddle":{"!type":"fn(e: ?) -> bool","!doc":"<p>Checks an event object for the pressed middle button</p>\n"},"isRight":{"!type":"fn(e: ?) -> bool","!doc":"<p>Checks an event object for the pressed right button</p>\n"}},"dojo/_base/kernel.keys":{"!doc":"<p>Definitions for common key values.  Client code should test keyCode against these named constants,\nas the actual codes can vary by browser.</p>\n","ALT":{},"BACKSPACE":{},"CAPS_LOCK":{},"CLEAR":{},"copyKey":{},"CTRL":{},"DELETE":{},"DOWN_ARROW":{},"DOWN_DPAD":{},"END":{},"ENTER":{},"ESCAPE":{},"F1":{},"F10":{},"F11":{},"F12":{},"F13":{},"F14":{},"F15":{},"F2":{},"F3":{},"F4":{},"F5":{},"F6":{},"F7":{},"F8":{},"F9":{},"HELP":{},"HOME":{},"INSERT":{},"LEFT_ARROW":{},"LEFT_DPAD":{},"LEFT_WINDOW":{},"META":{},"NUM_LOCK":{},"NUMPAD_0":{},"NUMPAD_1":{},"NUMPAD_2":{},"NUMPAD_3":{},"NUMPAD_4":{},"NUMPAD_5":{},"NUMPAD_6":{},"NUMPAD_7":{},"NUMPAD_8":{},"NUMPAD_9":{},"NUMPAD_DIVIDE":{},"NUMPAD_ENTER":{},"NUMPAD_MINUS":{},"NUMPAD_MULTIPLY":{},"NUMPAD_PERIOD":{},"NUMPAD_PLUS":{},"PAGE_DOWN":{},"PAGE_UP":{},"PAUSE":{},"RIGHT_ARROW":{},"RIGHT_DPAD":{},"RIGHT_WINDOW":{},"SCROLL_LOCK":{},"SELECT":{},"SHIFT":{},"SPACE":{},"TAB":{},"UP_ARROW":{},"UP_DPAD":{}},"dojo/_base/kernel._nodeDataCache":{},"dojo/_base/kernel.fx":{"!doc":"<p>Effects library on top of Base animations</p>\n","easing":{"!doc":"<p>Collection of easing functions to use beyond the default\n<code>dojo._defaultEasing</code> function.</p>\n"},"chain":{"!type":"fn(animations: ?) -> +instance","!doc":"<p>Chain a list of <code>dojo/_base/fx.Animation</code>s to run in sequence\n</p>\n"},"combine":{"!type":"fn(animations: ?) -> +instance","!doc":"<p>Combine a list of <code>dojo/_base/fx.Animation</code>s to run in parallel\n</p>\n"},"slideTo":{"!type":"fn(args: ?) -> ?","!doc":"<p>Slide a node to a new top/left position\n</p>\n"},"Toggler":{"!type":"fn()"},"wipeIn":{"!type":"fn(args: ?) -> ?","!doc":"<p>Expand a node to it&#39;s natural height.\n</p>\n"},"wipeOut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Shrink a node to nothing and hide it.\n</p>\n"}},"dojo/_base/kernel.date":{"stamp":{"!doc":"<p>TODOC</p>\n"},"add":{"!type":"fn(date: ?, interval: ?, amount: ?) -> +instance","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2?: ?, portion?: ?) -> number","!doc":"<p>Compare two date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?) -> ?","!doc":"<p>Get the difference in a specific unit of time (e.g., number of\nmonths, weeks, days, etc.) between two dates, rounded to the\nnearest integer.</p>\n"},"getDaysInMonth":{"!type":"fn(dateObject: ?) -> number","!doc":"<p>Returns the number of days in the month used by dateObject</p>\n"},"getTimezoneName":{"!type":"fn(dateObject: ?) -> ?","!doc":"<p>Get the user&#39;s time zone as provided by the browser</p>\n"},"isLeapYear":{"!type":"fn(dateObject: ?) -> bool","!doc":"<p>Determines if the year of the dateObject is a leap year</p>\n"}},"dojo/_base/kernel.html":{"!doc":"<p>TODOC</p>\n","set":{"!type":"fn(node: ?, cont: ?, params?: ?) -> ?","!doc":"<p>inserts (replaces) the given content into the given node. dojo/dom-construct.place(cont, node, &quot;only&quot;)\nmay be a better choice for simple HTML insertion.</p>\n"}},"dojo/_base/kernel.Stateful":{"!type":"fn()","prototype":{"!doc":"<p>Base class for objects that provide named properties with optional getter/setter\ncontrol and the ability to watch for property changes\n\n</p>\n<p>The class also provides the functionality to auto-magically manage getters\nand setters for object attributes/properties.\n\n</p>\n<p>Getters and Setters should follow the format of _xxxGetter or _xxxSetter where \nthe xxx is a name of the attribute to handle.  So an attribute of &quot;foo&quot; \nwould have a custom getter of _fooGetter and a custom setter of _fooSetter.\n</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojo/_base/kernel.contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n","auto":{"!type":"fn(xhr: ?)"},"javascript":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which evaluates the response data, expecting it to be valid JavaScript</p>\n"},"json":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which returns a JavaScript object created from the response data</p>\n"},"json-comment-filtered":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which expects comment-filtered JSON.</p>\n"},"json-comment-optional":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which checks the presence of comment-filtered JSON and\nalternates between the <code>json</code> and <code>json-comment-filtered</code> contentHandlers.</p>\n"},"olson-zoneinfo":{"!type":"fn(xhr: ?)"},"text":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which simply returns the plaintext response data</p>\n"},"xml":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler returning an XML Document parsed from the response data</p>\n"}},"dojo/_base/kernel._contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n","auto":{"!type":"fn(xhr: ?)"},"javascript":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which evaluates the response data, expecting it to be valid JavaScript</p>\n"},"json":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which returns a JavaScript object created from the response data</p>\n"},"json-comment-filtered":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which expects comment-filtered JSON.</p>\n"},"json-comment-optional":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which checks the presence of comment-filtered JSON and\nalternates between the <code>json</code> and <code>json-comment-filtered</code> contentHandlers.</p>\n"},"olson-zoneinfo":{"!type":"fn(xhr: ?)"},"text":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which simply returns the plaintext response data</p>\n"},"xml":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler returning an XML Document parsed from the response data</p>\n"}},"dojo/_base/kernel.__IoArgs":{"!type":"fn()","prototype":{"content":{"!doc":"<p>Contains properties with string values. These\nproperties will be serialized as name1=value2 and\npassed in the request.</p>\n"},"form":{"!doc":"<p>DOM node for a form. Used to extract the form values\nand send to the server.</p>\n"},"handleAs":{"!doc":"<p>Acceptable values depend on the type of IO\ntransport (see specific IO calls for more information).</p>\n"},"ioPublish":{"!doc":"<p>Set this explicitly to false to prevent publishing of topics related to\nIO operations. Otherwise, if djConfig.ioPublish is set to true, topics\nwill be published via dojo/topic.publish() for different phases of an IO operation.\nSee dojo/main.__IoPublish for a list of topics that are published.</p>\n"},"preventCache":{"!doc":"<p>Default is false. If true, then a\n&quot;dojo.preventCache&quot; parameter is sent in the request\nwith a value that changes with each request\n(timestamp). Useful only with GET-type requests.</p>\n"},"rawBody":{"!doc":"<p>Sets the raw body for an HTTP request. If this is used, then the content\nproperty is ignored. This is mostly useful for HTTP methods that have\na body to their requests, like PUT or POST. This property can be used instead\nof postData and putData for dojo/_base/xhr.rawXhrPost and dojo/_base/xhr.rawXhrPut respectively.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response. If this time\npasses, the then error callbacks are called.</p>\n"},"url":{"!doc":"<p>URL to server endpoint.</p>\n"},"error":{"!type":"fn(response: ?, ioArgs: ?)","!doc":"<p>This function will\nbe called when the request fails due to a network or server error, the url\nis invalid, etc. It will also be called if the load or handle callback throws an\nexception, unless djConfig.debugAtAllCosts is true.  This allows deployed applications\nto continue to run even when a logic error happens in the callback, while making\nit easier to troubleshoot while in debug mode.</p>\n"},"handle":{"!type":"fn(loadOrError: ?, response: ?, ioArgs: ?)","!doc":"<p>This function will\nbe called at the end of every request, whether or not an error occurs.</p>\n"},"load":{"!type":"fn(response: ?, ioArgs: ?)","!doc":"<p>This function will be\ncalled on a successful HTTP response code.</p>\n"}}},"dojo/_base/kernel.__IoCallbackArgs":{"!type":"fn()","prototype":{"args":{"!doc":"<p>the original object argument to the IO call.</p>\n"},"canDelete":{"!doc":"<p>For dojo/io/script calls only, indicates\nwhether the script tag that represents the\nrequest can be deleted after callbacks have\nbeen called. Used internally to know when\ncleanup can happen on JSONP-type requests.</p>\n"},"handleAs":{"!doc":"<p>The final indicator on how the response will be\nhandled.</p>\n"},"id":{"!doc":"<p>For dojo/io/script calls only, the internal\nscript ID used for the request.</p>\n"},"json":{"!doc":"<p>For dojo/io/script calls only: holds the JSON\nresponse for JSONP-type requests. Used\ninternally to hold on to the JSON responses.\nYou should not need to access it directly --\nthe same object should be passed to the success\ncallbacks directly.</p>\n"},"query":{"!doc":"<p>For non-GET requests, the\nname1=value1&amp;name2=value2 parameters sent up in\nthe request.</p>\n"},"url":{"!doc":"<p>The final URL used for the call. Many times it\nwill be different than the original args.url\nvalue.</p>\n"},"xhr":{"!doc":"<p>For XMLHttpRequest calls only, the\nXMLHttpRequest object that was used for the\nrequest.</p>\n"}}},"dojo/_base/kernel.__IoPublish":{"!type":"fn()","prototype":{"!doc":"<p>This is a list of IO topics that can be published\nif djConfig.ioPublish is set to true. IO topics can be\npublished for any Input/Output, network operation. So,\ndojo.xhr, dojo.io.script and dojo.io.iframe can all\ntrigger these topics to be published.</p>\n","done":{"!doc":"<p>&quot;/dojo/io/done&quot; is sent whenever an IO request has completed,\neither by loading or by erroring. It passes the error and\nthe dojo.Deferred for the request with the topic.</p>\n"},"error":{"!doc":"<p>&quot;/dojo/io/error&quot; is sent whenever an IO request has errored.\nIt passes the error and the dojo.Deferred\nfor the request with the topic.</p>\n"},"load":{"!doc":"<p>&quot;/dojo/io/load&quot; is sent whenever an IO request has loaded\nsuccessfully. It passes the response and the dojo.Deferred\nfor the request with the topic.</p>\n"},"send":{"!doc":"<p>&quot;/dojo/io/send&quot; is sent whenever a new IO request is started.\nIt passes the dojo.Deferred for the request with the topic.</p>\n"},"start":{"!doc":"<p>&quot;/dojo/io/start&quot; is sent when there are no outstanding IO\nrequests, and a new IO request is started. No arguments\nare passed with this topic.</p>\n"},"stop":{"!doc":"<p>&quot;/dojo/io/stop&quot; is sent when all outstanding IO requests have\nfinished. No arguments are passed with this topic.</p>\n"}}},"dojo/_base/kernel.__XhrArgs":{"!type":"fn()","prototype":{"!doc":"<p>In addition to the properties listed for the dojo._IoArgs type,\nthe following properties are allowed for dojo.xhr* methods.</p>\n","content":{"!doc":"<p>Contains properties with string values. These\nproperties will be serialized as name1=value2 and\npassed in the request.</p>\n"},"contentType":{"!doc":"<p>&quot;application/x-www-form-urlencoded&quot; is default. Set to false to\nprevent a Content-Type header from being sent, or to a string\nto send a different Content-Type.</p>\n"},"failOk":{"!doc":"<p>false is default. Indicates whether a request should be\nallowed to fail (and therefore no console error message in\nthe event of a failure)</p>\n"},"form":{"!doc":"<p>DOM node for a form. Used to extract the form values\nand send to the server.</p>\n"},"handleAs":{"!doc":"<p>Acceptable values are: text (default), json, json-comment-optional,\njson-comment-filtered, javascript, xml. See <code>dojo/_base/xhr.contentHandlers</code></p>\n"},"headers":{"!doc":"<p>Additional HTTP headers to send in the request.</p>\n"},"ioPublish":{"!doc":"<p>Set this explicitly to false to prevent publishing of topics related to\nIO operations. Otherwise, if djConfig.ioPublish is set to true, topics\nwill be published via dojo/topic.publish() for different phases of an IO operation.\nSee dojo/main.__IoPublish for a list of topics that are published.</p>\n"},"preventCache":{"!doc":"<p>Default is false. If true, then a\n&quot;dojo.preventCache&quot; parameter is sent in the request\nwith a value that changes with each request\n(timestamp). Useful only with GET-type requests.</p>\n"},"rawBody":{"!doc":"<p>Sets the raw body for an HTTP request. If this is used, then the content\nproperty is ignored. This is mostly useful for HTTP methods that have\na body to their requests, like PUT or POST. This property can be used instead\nof postData and putData for dojo/_base/xhr.rawXhrPost and dojo/_base/xhr.rawXhrPut respectively.</p>\n"},"sync":{"!doc":"<p>false is default. Indicates whether the request should\nbe a synchronous (blocking) request.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response. If this time\npasses, the then error callbacks are called.</p>\n"},"url":{"!doc":"<p>URL to server endpoint.</p>\n"},"error":{"!type":"fn(response: ?, ioArgs: ?)","!doc":"<p>This function will\nbe called when the request fails due to a network or server error, the url\nis invalid, etc. It will also be called if the load or handle callback throws an\nexception, unless djConfig.debugAtAllCosts is true.  This allows deployed applications\nto continue to run even when a logic error happens in the callback, while making\nit easier to troubleshoot while in debug mode.</p>\n"},"handle":{"!type":"fn(loadOrError: ?, response: ?, ioArgs: ?)","!doc":"<p>This function will\nbe called at the end of every request, whether or not an error occurs.</p>\n"},"load":{"!type":"fn(response: ?, ioArgs: ?)","!doc":"<p>This function will be\ncalled on a successful HTTP response code.</p>\n"}}},"dojo/_base/kernel.back":{"!doc":"<p>Browser history management resources</p>\n","addToHistory":{"!type":"fn(args: ?)","!doc":"<p>adds a state object (args) to the history list.</p>\n"},"getHash":{"!type":"fn() -> ?"},"goBack":{"!type":"fn()","!doc":"<p>private method. Do not call this directly.</p>\n"},"goForward":{"!type":"fn()","!doc":"<p>private method. Do not call this directly.</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initializes the undo stack. This must be called from a <script>\nblock that lives inside the <code>&lt;body&gt;</code> tag to prevent bugs on IE.\n\n</p>\n<p>Only call this method before the page&#39;s DOM is finished loading. Otherwise\nit will not work. Be careful with xdomain loading or djConfig.debugAtAllCosts scenarios,\nin order for this method to work, dojo/back will need to be part of a build layer.</p>\n"},"setHash":{"!type":"fn(h: ?)"},"setInitialState":{"!type":"fn(args: ?)","!doc":"<p>Sets the state object and back callback for the very first page\nthat is loaded.\n\n</p>\n<p>It is recommended that you call this method as part of an event\nlistener that is registered via dojo/ready.</p>\n"}},"dojo/_base/kernel.cldr":{"monetary":{"!doc":"<p>TODOC</p>\n"},"supplemental":{"!doc":"<p>TODOC</p>\n"}},"dojo/_base/kernel.i18n":{"!doc":"<p>This module implements the dojo/i18n! plugin and the v1.6- i18n API</p>\n","cache":{},"dynamic":{},"unitTests":{},"getL10nName":{"!type":"fn(moduleName: ?, bundleName: ?, locale: ?) -> string"},"getLocalization":{"!type":"fn(moduleName: ?, bundleName: ?, locale: ?) -> ?"},"load":{"!type":"fn(id: ?, require: ?, load: ?)","!doc":"<p>id is in one of the following formats\n\n</p>\n<ol>\n<li><p><path>/nls/<bundle>\n =&gt; load the bundle, localized to config.locale; load all bundles localized to\n config.extraLocale (if any); return the loaded bundle localized to config.locale.</p>\n</li>\n<li><p><path>/nls/<locale>/<bundle>\n =&gt; load then return the bundle localized to <locale></p>\n</li>\n<li><p><em>preload</em><path>/nls/<module>*<JSON array of available locales>\n =&gt; for config.locale and all config.extraLocale, load all bundles found\n in the best-matching bundle rollup. A value of 1 is returned, which\n is meaningless other than to say the plugin is executing the requested\n preloads</p>\n</li>\n</ol>\n<p>In cases 1 and 2, <path> is always normalized to an absolute module id upon entry; see\nnormalize. In case 3, it <path> is assumed to be absolute; this is arranged by the builder.\n\n</p>\n<p>To load a bundle means to insert the bundle into the plugin&#39;s cache and publish the bundle\nvalue to the loader. Given <path>, <bundle>, and a particular <locale>, the cache key\n\n</p>\n<pre><code>&lt;path&gt;/nls/&lt;bundle&gt;/&lt;locale&gt;</code></pre>\n<p>will hold the value. Similarly, then plugin will publish this value to the loader by\n\n</p>\n<pre><code>define(&quot;&lt;path&gt;/nls/&lt;bundle&gt;/&lt;locale&gt;&quot;, &lt;bundle-value&gt;);</code></pre>\n<p>Given this algorithm, other machinery can provide fast load paths be preplacing\nvalues in the plugin&#39;s cache, which is public. When a load is demanded the\ncache is inspected before starting any loading. Explicitly placing values in the plugin\ncache is an advanced/experimental feature that should not be needed; use at your own risk.\n\n</p>\n<p>For the normal AMD algorithm, the root bundle is loaded first, which instructs the\nplugin what additional localized bundles are required for a particular locale. These\nadditional locales are loaded and a mix of the root and each progressively-specific\nlocale is returned. For example:\n\n</p>\n<ol>\n<li><p>The client demands &quot;dojo/i18n!some/path/nls/someBundle</p>\n</li>\n<li><p>The loader demands load(some/path/nls/someBundle)</p>\n</li>\n<li><p>This plugin require&#39;s &quot;some/path/nls/someBundle&quot;, which is the root bundle.</p>\n</li>\n<li><p>Assuming config.locale is &quot;ab-cd-ef&quot; and the root bundle indicates that localizations\nare available for &quot;ab&quot; and &quot;ab-cd-ef&quot; (note the missing &quot;ab-cd&quot;, then the plugin\nrequires &quot;some/path/nls/ab/someBundle&quot; and &quot;some/path/nls/ab-cd-ef/someBundle&quot;</p>\n</li>\n<li><p>Upon receiving all required bundles, the plugin constructs the value of the bundle\nab-cd-ef as...</p>\n<pre><code> mixin(mixin(mixin({}, require(&quot;some/path/nls/someBundle&quot;),\n     require(&quot;some/path/nls/ab/someBundle&quot;)),\n     require(&quot;some/path/nls/ab-cd-ef/someBundle&quot;));</code></pre>\n</li>\n</ol>\n<p>This value is inserted into the cache and published to the loader at the\nkey/module-id some/path/nls/someBundle/ab-cd-ef.\n\n</p>\n<p>The special preload signature (case 3) instructs the plugin to stop servicing all normal requests\n(further preload requests will be serviced) until all ongoing preloading has completed.\n\n</p>\n<p>The preload signature instructs the plugin that a special rollup module is available that contains\none or more flattened, localized bundles. The JSON array of available locales indicates which locales\nare available. Here is an example:\n\n</p>\n<pre><code>*preload*some/path/nls/someModule*[&quot;root&quot;, &quot;ab&quot;, &quot;ab-cd-ef&quot;]</code></pre>\n<p>This indicates the following rollup modules are available:\n\n</p>\n<pre><code>some/path/nls/someModule_ROOT\nsome/path/nls/someModule_ab\nsome/path/nls/someModule_ab-cd-ef</code></pre>\n<p>Each of these modules is a normal AMD module that contains one or more flattened bundles in a hash.\nFor example, assume someModule contained the bundles some/bundle/path/someBundle and\nsome/bundle/path/someOtherBundle, then some/path/nls/someModule_ab would be expressed as follows:\n\n</p>\n<pre><code>define({\n    some/bundle/path/someBundle:&lt;value of someBundle, flattened with respect to locale ab&gt;,\n    some/bundle/path/someOtherBundle:&lt;value of someOtherBundle, flattened with respect to locale ab&gt;,\n});</code></pre>\n<p>E.g., given this design, preloading for locale==&quot;ab&quot; can execute the following algorithm:\n\n</p>\n<pre><code>require([&quot;some/path/nls/someModule_ab&quot;], function(rollup){\n    for(var p in rollup){\n        var id = p + &quot;/ab&quot;,\n        cache[id] = rollup[p];\n        define(id, rollup[p]);\n    }\n});</code></pre>\n<p>Similarly, if &quot;ab-cd&quot; is requested, the algorithm can determine that &quot;ab&quot; is the best available and\nload accordingly.\n\n</p>\n<p>The builder will write such rollups for every layer if a non-empty localeList  profile property is\nprovided. Further, the builder will include the following cache entry in the cache associated with\nany layer.\n\n</p>\n<pre><code>&quot;*now&quot;:function(r){r([&#39;dojo/i18n!*preload*&lt;path&gt;/nls/&lt;module&gt;*&lt;JSON array of available locales&gt;&#39;]);}</code></pre>\n<p>The *now special cache module instructs the loader to apply the provided function to context-require\nwith respect to the particular layer being defined. This causes the plugin to hold all normal service\nrequests until all preloading is complete.\n\n</p>\n<p>Notice that this algorithm is rarely better than the standard AMD load algorithm. Consider the normal case\nwhere the target locale has a single segment and a layer depends on a single bundle:\n\n</p>\n<p>Without Preloads:\n\n</p>\n<ol>\n<li>Layer loads root bundle.</li>\n<li>bundle is demanded; plugin loads single localized bundle.</li>\n</ol>\n<p>With Preloads:\n\n</p>\n<ol>\n<li>Layer causes preloading of target bundle.</li>\n<li>bundle is demanded; service is delayed until preloading complete; bundle is returned.</li>\n</ol>\n<p>In each case a single transaction is required to load the target bundle. In cases where multiple bundles\nare required and/or the locale has multiple segments, preloads still requires a single transaction whereas\nthe normal path requires an additional transaction for each additional bundle/locale-segment. However all\nof these additional transactions can be done concurrently. Owing to this analysis, the entire preloading\nalgorithm can be discard during a build by setting the has feature dojo-preload-i18n-Api to false.</p>\n"},"normalize":{"!type":"fn(id: ?, toAbsMid: ?) -> ?","!doc":"<p>id may be relative.\npreload has form <code>*preload*&lt;path&gt;/nls/&lt;module&gt;*&lt;flattened locales&gt;</code> and\ntherefore never looks like a relative</p>\n"},"normalizeLocale":{"!type":"fn(locale: ?) -> ?"}},"dojo/_base/kernel.colors":{"makeGrey":{"!type":"fn(g: ?, a?: ?)","!doc":"<p>creates a greyscale color with an optional alpha</p>\n"}},"dojo/_base/kernel.regexp":{"!doc":"<p>Regular expressions and Builder resources</p>\n","buildGroupRE":{"!type":"fn(arr: ?, re: ?, nonCapture?: ?) -> ?","!doc":"<p>Builds a regular expression that groups subexpressions</p>\n"},"escapeString":{"!type":"fn(str: ?, except?: ?) -> ?","!doc":"<p>Adds escape sequences for special characters in regular expressions</p>\n"},"group":{"!type":"fn(expression: ?, nonCapture?: ?) -> string","!doc":"<p>adds group match to expression</p>\n"}},"dojo/_base/kernel.string":{"!doc":"<p>String utilities for Dojo</p>\n","pad":{"!type":"fn(text: ?, size: ?, ch?: ?, end?: ?) -> number","!doc":"<p>Pad a string to guarantee that it is at least <code>size</code> length by\nfilling with the character <code>ch</code> at either the start or end of the\nstring. Pads at the start, by default.</p>\n"},"rep":{"!type":"fn(str: ?, num: ?) -> string","!doc":"<p>Efficiently replicate a string <code>n</code> times.</p>\n"},"substitute":{"!type":"fn(template: ?, map: ?, transform?: ?, thisObject?: ?) -> ?","!doc":"<p>Performs parameterized substitutions on a string. Throws an\nexception if any parameter is unmatched.</p>\n"},"trim":{"!type":"fn(str: ?) -> string","!doc":"<p>Trims whitespace from both sides of the string</p>\n"}},"dojo/_base/kernel.number":{"!doc":"<p>localized formatting and parsing routines for Number</p>\n","_numberPatternRE":{},"format":{"!type":"fn(value: ?, options?: ?) -> +null","!doc":"<p>Format a Number as a String, using locale-specific settings</p>\n"},"parse":{"!type":"fn(expression: ?, options?: ?) -> number","!doc":"<p>Convert a properly formatted string to a primitive Number, using\nlocale-specific settings.</p>\n"},"regexp":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a number</p>\n"},"round":{"!type":"fn(value: ?, places?: ?, increment?: ?) -> number","!doc":"<p>Rounds to the nearest value with the given number of decimal places, away from zero</p>\n"}},"dojo/_base/kernel.currency":{"!doc":"<p>localized formatting and parsing routines for currencies</p>\n","format":{"!type":"fn(value: ?, options?: ?) -> ?","!doc":"<p>Format a Number as a currency, using locale-specific settings\n</p>\n"},"parse":{"!type":"fn(expression: ?, options?: ?) -> ?"},"regexp":{"!type":"fn(options?: ?) -> ?"}},"dojo/_base/kernel.data":{"api":{},"util":{},"ItemFileReadStore":{"!type":"fn()"},"ItemFileWriteStore":{"!type":"fn()"},"ObjectStore":{"!type":"fn()"}},"dojo/_base/kernel.dnd":{"autoscroll":{"!doc":"<p>Used by dojo/dnd/Manager to scroll document or internal node when the user\ndrags near the edge of the viewport or a scrollable node</p>\n"},"move":{},"AutoSource":{"!type":"fn()"},"Avatar":{"!type":"fn()"},"Container":{"!type":"fn()"},"Manager":{"!type":"fn()"},"Moveable":{"!type":"fn()"},"Mover":{"!type":"fn()"},"Selector":{"!type":"fn()"},"Source":{"!type":"fn()"},"Target":{"!type":"fn()"},"TimedMoveable":{"!type":"fn()"}},"dojo/_base/kernel.touch":{"!doc":"<p>This module provides unified touch event handlers by exporting\npress, move, release and cancel which can also run well on desktop.\nBased on <a href=\"http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html\">http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html</a>\nAlso, if the dojoClick property is set to truthy on a DOM node, dojo/touch generates\nclick events immediately for this node and its descendants (except for descendants that\nhave a dojoClick property set to falsy), to avoid the delay before native browser click events,\nand regardless of whether evt.preventDefault() was called in a touch.press event listener.\n</p>\n","cancel":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;touchcancel&#39;|&#39;mouseleave&#39; for the given node</p>\n"},"enter":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to mouse.enter or touch equivalent for the given node</p>\n"},"leave":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to mouse.leave or touch equivalent for the given node</p>\n"},"move":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener that fires when the mouse cursor or a finger is dragged over the given node.</p>\n"},"out":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;mouseout&#39; or touch equivalent for the given node</p>\n"},"over":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;mouseover&#39; or touch equivalent for the given node</p>\n"},"press":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;touchstart&#39;|&#39;mousedown&#39; for the given node</p>\n"},"release":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to releasing the mouse button while the cursor is over the given node\n(i.e. &quot;mouseup&quot;) or for removing the finger from the screen while touching the given node.</p>\n"}},"dojo/_base/kernel.window":{"!doc":"<p>TODOC</p>\n","get":{"!type":"fn(doc: ?) -> ?","!doc":"<p>Get window object associated with document doc.</p>\n"},"getBox":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Returns the dimensions and scroll position of the viewable area of a browser window</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, pos?: ?)","!doc":"<p>Scroll the passed node into view using minimal movement, if it is not already.</p>\n"}},"dojo/_base/kernel.gears":{"!doc":"<p>TODOC</p>\n","available":{"!doc":"<p>True if client is using Google Gears</p>\n"}},"dojo/_base/kernel.io":{"iframe":{},"script":{"!doc":"<p>TODOC</p>\n"}},"dojo/_base/kernel.rpc":{"JsonpService":{"!type":"fn()"},"JsonService":{"!type":"fn()"},"RpcService":{"!type":"fn()"}},"dojo/_base/kernel.store":{"util":{},"Cache":{"!type":"fn(masterStore: ?, cachingStore: ?, options: ?) -> ?"},"DataStore":{"!type":"fn()"},"JsonRest":{"!type":"fn()"},"Memory":{"!type":"fn()"},"Observable":{"!type":"fn(store: ?) -> ?","!doc":"<p>The Observable store wrapper takes a store and sets an observe method on query()\nresults that can be used to monitor results for changes.\n</p>\n"}},"dojo/_base/kernel.tests":{"!doc":"<p>D.O.H. Test files for Dojo unit testing.</p>\n"},"dojo/has":{"!doc":"<p>Return the current value of the named feature.\n</p>\n","cache":{},"add":{"!type":"fn(name: ?, test: ?, now?: ?, force?: ?) -> ?","!doc":"<p>Register a new feature test for some named feature.</p>\n"},"clearElement":{"!type":"fn(element: ?)","!doc":"<p>Deletes the contents of the element passed to test functions.</p>\n"},"prototype":{"constructor":{"!type":"fn(name: ?) -> bool","!doc":"<p>Return the current value of the named feature.\n</p>\n"}},"load":{"!type":"fn(id: ?, parentRequire: ?, loaded: ?)","!doc":"<p>Conditional loading of AMD modules based on a has feature test value.</p>\n"},"normalize":{"!type":"fn(id: ?, toAbsMid: ?)","!doc":"<p>Resolves id into a module id based on possibly-nested tenary expression that branches on has feature test value(s).\n</p>\n"}},"dojo/_base/config":{"!doc":"<p>This module defines the user configuration during bootstrap.</p>\n","addOnLoad":{"!doc":"<p>Adds a callback via dojo/ready. Useful when Dojo is added after\nthe page loads and djConfig.afterOnLoad is true. Supports the same\narguments as dojo/ready. When using a function reference, use\n<code>djConfig.addOnLoad = function(){};</code>. For object with function name use\n<code>djConfig.addOnLoad = [myObject, &quot;functionName&quot;];</code> and for object with\nfunction reference use\n<code>djConfig.addOnLoad = [myObject, function(){}];</code></p>\n"},"afterOnLoad":{},"baseUrl":{},"callback":{"!doc":"<p>Defines a callback to be used when dependencies are defined before \nthe loader has been loaded. When provided, they cause the loader to \nexecute require(deps, callback) once it has finished loading. \nShould be used with deps.</p>\n"},"debugContainerId":{},"debugHeight":{},"defaultDuration":{"!doc":"<p>Default duration, in milliseconds, for wipe and fade animations within dijits.\nAssigned to dijit.defaultDuration.</p>\n"},"deferredInstrumentation":{"!doc":"<p>Whether deferred instrumentation should be loaded or included\nin builds.</p>\n"},"deps":{"!doc":"<p>Defines dependencies to be used before the loader has been loaded.\nWhen provided, they cause the loader to execute require(deps, callback) \nonce it has finished loading. Should be used with callback.</p>\n"},"dojoBlankHtmlUrl":{"!doc":"<p>Used by some modules to configure an empty iframe. Used by dojo/io/iframe and\ndojo/back, and dijit/popup support in IE where an iframe is needed to make sure native\ncontrols do not bleed through the popups. Normally this configuration variable\ndoes not need to be set, except when using cross-domain/CDN Dojo builds.\nSave dojo/resources/blank.html to your domain and set <code>djConfig.dojoBlankHtmlUrl</code>\nto the path on your domain your copy of blank.html.</p>\n"},"extraLocale":{"!doc":"<p>No default value. Specifies additional locales whose\nresources should also be loaded alongside the default locale when\ncalls to <code>dojo.requireLocalization()</code> are processed.</p>\n"},"ioPublish":{"!doc":"<p>Set this to true to enable publishing of topics for the different phases of\nIO operations. Publishing is done via dojo/topic.publish(). See dojo/main.__IoPublish for a list\nof topics that are published.</p>\n"},"isDebug":{"!doc":"<p>Defaults to <code>false</code>. If set to <code>true</code>, ensures that Dojo provides\nextended debugging feedback via Firebug. If Firebug is not available\non your platform, setting <code>isDebug</code> to <code>true</code> will force Dojo to\npull in (and display) the version of Firebug Lite which is\nintegrated into the Dojo distribution, thereby always providing a\ndebugging/logging console when <code>isDebug</code> is enabled. Note that\nFirebug&#39;s <code>console.*</code> methods are ALWAYS defined by Dojo. If\n<code>isDebug</code> is false and you are on a platform without Firebug, these\nmethods will be defined as no-ops.</p>\n"},"locale":{"!doc":"<p>The locale to assume for loading localized resources in this page,\nspecified according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a>.\nMust be specified entirely in lowercase, e.g. <code>en-us</code> and <code>zh-cn</code>.\nSee the documentation for <code>dojo.i18n</code> and <code>dojo.requireLocalization</code>\nfor details on loading localized resources. If no locale is specified,\nDojo assumes the locale of the user agent, according to <code>navigator.userLanguage</code>\nor <code>navigator.language</code> properties.</p>\n"},"modulePaths":{"!doc":"<p>A map of module names to paths relative to <code>dojo.baseUrl</code>. The\nkey/value pairs correspond directly to the arguments which\n<code>dojo.registerModulePath</code> accepts. Specifying\n<code>djConfig.modulePaths = { &quot;foo&quot;: &quot;../../bar&quot; }</code> is the equivalent\nof calling <code>dojo.registerModulePath(&quot;foo&quot;, &quot;../../bar&quot;);</code>. Multiple\nmodules may be configured via <code>djConfig.modulePaths</code>.</p>\n"},"parseOnLoad":{"!doc":"<p>Run the parser after the page is loaded</p>\n"},"require":{"!doc":"<p>An array of module names to be loaded immediately after dojo.js has been included\nin a page.</p>\n"},"transparentColor":{"!doc":"<p>Array containing the r, g, b components used as transparent color in dojo.Color;\nif undefined, [255,255,255] (white) will be used.</p>\n"},"urchin":{"!doc":"<p>Used by <code>dojox.analytics.Urchin</code> as the default UA-123456-7 account\nnumber used when being created. Alternately, you can pass an acct:&quot;&quot;\nparameter to the constructor a la: new dojox.analytics.Urchin({ acct:&quot;UA-123456-7&quot; });</p>\n"},"useCustomLogger":{"!doc":"<p>If set to a value that evaluates to true such as a string or array and\nisDebug is true and Firebug is not available or running, then it bypasses\nthe creation of Firebug Lite allowing you to define your own console object.</p>\n"},"useDeferredInstrumentation":{"!doc":"<p>Whether the deferred instrumentation should be used.\n\n</p>\n<ul>\n<li><code>&quot;report-rejections&quot;</code>: report each rejection as it occurs.</li>\n<li><code>true</code> or <code>1</code> or <code>&quot;report-unhandled-rejections&quot;</code>: wait 1 second\n  in an attempt to detect unhandled rejections.</li>\n</ul>\n"}},"dojo/_base/config.modulePaths":{"!doc":"<p>A map of module names to paths relative to <code>dojo.baseUrl</code>. The\nkey/value pairs correspond directly to the arguments which\n<code>dojo.registerModulePath</code> accepts. Specifying\n<code>djConfig.modulePaths = { &quot;foo&quot;: &quot;../../bar&quot; }</code> is the equivalent\nof calling <code>dojo.registerModulePath(&quot;foo&quot;, &quot;../../bar&quot;);</code>. Multiple\nmodules may be configured via <code>djConfig.modulePaths</code>.</p>\n"},"dojo/_base/lang":{"!doc":"<p>This module defines Javascript language extensions.</p>\n","_extraNames":{"!doc":"<p>Lists property names that must be explicitly processed during for-in iteration\nin environments that have has(&quot;bug-for-in-skips-shadowed&quot;) true.</p>\n"},"clone":{"!type":"fn(src: ?) -> +anything","!doc":"<p>Clones objects (including DOM nodes) and all children.\nWarning: do not clone cyclic structures.</p>\n"},"delegate":{"!type":"fn(obj: ?, props: ?) -> +any","!doc":"<p>Returns a new object which &quot;looks&quot; to obj for properties which it\ndoes not have a value for. Optionally takes a bag of properties to\nseed the returned object with initially.</p>\n"},"exists":{"!type":"fn(name: ?, obj?: ?) -> bool","!doc":"<p>determine if an object supports a given method</p>\n"},"extend":{"!type":"fn(ctor: ?, props: ?) -> ?","!doc":"<p>Adds all properties and methods of props to constructor&#39;s\nprototype, making them available to all instances created with\nconstructor.</p>\n"},"getObject":{"!type":"fn(name: ?, create?: ?, context?: ?) -> ?","!doc":"<p>Get a property from a dot-separated string, such as &quot;A.B.C&quot;</p>\n"},"hitch":{"!type":"fn(scope: ?, method: ?) -> ?","!doc":"<p>Returns a function that will only ever execute in the a given scope.\nThis allows for easy use of object member functions\nin callbacks and other places in which the &quot;this&quot; keyword may\notherwise not reference the expected scope.\nAny number of default positional arguments may be passed as parameters\nbeyond &quot;method&quot;.\nEach of these values will be used to &quot;placehold&quot; (similar to curry)\nfor the hitched function.</p>\n"},"isAlien":{"!type":"fn(it: ?) -> ?","!doc":"<p>Returns true if it is a built-in function or some other kind of\noddball that <em>should</em> report as a function but doesn&#39;t</p>\n"},"isArray":{"!type":"fn(it: ?) -> ?","!doc":"<p>Return true if it is an Array.\nDoes not work on Arrays created in other windows.</p>\n"},"isArrayLike":{"!type":"fn(it: ?) -> +any","!doc":"<p>similar to isArray() but more permissive</p>\n"},"isFunction":{"!type":"fn(it: ?) -> bool","!doc":"<p>Return true if it is a Function</p>\n"},"isObject":{"!type":"fn(it: ?) -> bool","!doc":"<p>Returns true if it is a JavaScript object (or an Array, a Function\nor null)</p>\n"},"isString":{"!type":"fn(it: ?) -> bool","!doc":"<p>Return true if it is a String</p>\n"},"mixin":{"!type":"fn(dest: ?, sources: ?) -> ?","!doc":"<p>Copies/adds all properties of one or more sources to dest; returns dest.</p>\n"},"partial":{"!type":"fn(method: ?) -> ?","!doc":"<p>similar to hitch() except that the scope object is left to be\nwhatever the execution context eventually becomes.</p>\n"},"replace":{"!type":"fn(tmpl: ?, map: ?, pattern?: ?) -> string","!doc":"<p>Performs parameterized substitutions on a string. Throws an\nexception if any parameter is unmatched.</p>\n"},"setObject":{"!type":"fn(name: ?, value: ?, context?: ?) -> ?","!doc":"<p>Set a property from a dot-separated string, such as &quot;A.B.C&quot;</p>\n"},"trim":{"!type":"fn(str: ?) -> string","!doc":"<p>Trims whitespace from both sides of the string</p>\n"}},"dojo/sniff":{"!doc":"<p>This module sets has() flags based on the current browser.\nIt returns the has() function.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>This module sets has() flags based on the current browser.\nIt returns the has() function.</p>\n"}}},"dojo/Deferred":{"!doc":"<p>Creates a new deferred. This API is preferred over\n<code>dojo/_base/Deferred</code>.</p>\n","promise":{},"cancel":{"!type":"fn(reason: ?, strict?: ?) -> +any","!doc":"<p>Inform the deferred it may cancel its asynchronous operation.</p>\n"},"prototype":{"constructor":{"!type":"fn(canceler?: ?)","!doc":"<p>Creates a new deferred. This API is preferred over\n<code>dojo/_base/Deferred</code>.</p>\n"},"toString":{"!type":"fn() -> string"}},"isCanceled":{"!type":"fn() -> bool","!doc":"<p>Checks whether the deferred has been canceled.</p>\n"},"isFulfilled":{"!type":"fn() -> bool","!doc":"<p>Checks whether the deferred has been resolved or rejected.</p>\n"},"isRejected":{"!type":"fn() -> bool","!doc":"<p>Checks whether the deferred has been rejected.</p>\n"},"isResolved":{"!type":"fn() -> bool","!doc":"<p>Checks whether the deferred has been resolved.</p>\n"},"progress":{"!type":"fn(update: ?, strict?: ?) -> +dojo","!doc":"<p>Emit a progress update on the deferred.</p>\n"},"reject":{"!type":"fn(error: ?, strict?: ?) -> +dojo","!doc":"<p>Reject the deferred.</p>\n"},"resolve":{"!type":"fn(value: ?, strict?: ?) -> +dojo","!doc":"<p>Resolve the deferred.</p>\n"},"then":{"!type":"fn(callback?: ?, errback?: ?, progback?: ?) -> +dojo","!doc":"<p>Add new callbacks to the deferred.</p>\n"}},"dojo/errors/CancelError":{"!doc":"<p>Default error if a promise is canceled without a reason.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Default error if a promise is canceled without a reason.</p>\n"}}},"dojo/errors/create":{"prototype":{"constructor":{"!type":"fn(name: ?, ctor: ?, base: ?, props: ?) -> fn"}}},"dojo/promise/Promise":{"!doc":"<p>The public interface to a deferred.</p>\n","prototype":{"always":{"!type":"fn(callbackOrErrback?: ?) -> +dojo","!doc":"<p>Add a callback to be invoked when the promise is resolved\nor rejected.</p>\n"},"cancel":{"!type":"fn(reason: ?, strict?: ?) -> +any","!doc":"<p>Inform the deferred it may cancel its asynchronous operation.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The public interface to a deferred.</p>\n"},"isCanceled":{"!type":"fn() -> bool","!doc":"<p>Checks whether the promise has been canceled.</p>\n"},"isFulfilled":{"!type":"fn() -> bool","!doc":"<p>Checks whether the promise has been resolved or rejected.</p>\n"},"isRejected":{"!type":"fn() -> bool","!doc":"<p>Checks whether the promise has been rejected.</p>\n"},"isResolved":{"!type":"fn() -> bool","!doc":"<p>Checks whether the promise has been resolved.</p>\n"},"otherwise":{"!type":"fn(errback?: ?) -> +dojo","!doc":"<p>Add new errbacks to the promise.</p>\n"},"then":{"!type":"fn(callback?: ?, errback?: ?, progback?: ?) -> +dojo","!doc":"<p>Add new callbacks to the promise.</p>\n"},"toString":{"!type":"fn() -> string"},"trace":{"!type":"fn() -> +dojo","!doc":"<p>Trace the promise.</p>\n"},"traceRejected":{"!type":"fn() -> +dojo","!doc":"<p>Trace rejection of the promise.</p>\n"}}},"dojo/DeferredList":{"!doc":"<p>Deprecated, use dojo/promise/all instead.\nProvides event handling for a group of Deferred objects.</p>\n","prototype":{"gatherResults":{"!type":"fn(deferredList: ?) -> +dojo","!doc":"<p>Gathers the results of the deferreds for packaging\nas the parameters to the Deferred Lists&#39; callback</p>\n"}}},"dojo/_base/Deferred":{"!doc":"<p>Deprecated.   This module defines the legacy dojo/_base/Deferred API.\nNew code should use dojo/Deferred instead.</p>\n","prototype":{"fired":{},"addBoth":{"!type":"fn(callback: ?) -> +any","!doc":"<p>Add handler as both successful callback and error callback for this deferred instance.</p>\n"},"addCallback":{"!type":"fn(callback: ?) -> +any","!doc":"<p>Adds successful callback for this deferred instance.</p>\n"},"addErrback":{"!type":"fn(errback: ?) -> +any","!doc":"<p>Adds error callback for this deferred instance.</p>\n"},"constructor":{"!type":"fn(canceller?: ?)","!doc":"<p>Deprecated.   This module defines the legacy dojo/_base/Deferred API.\nNew code should use dojo/Deferred instead.</p>\n"}},"promise":{},"addCallbacks":{"!type":"fn(callback?: ?, errback?: ?) -> +any","!doc":"<p>Adds callback and error callback for this deferred instance.</p>\n"},"callback":{"!type":"fn(value: ?)","!doc":"<p>Fulfills the Deferred instance successfully with the provide value</p>\n"},"cancel":{"!type":"fn()","!doc":"<p>Cancels the asynchronous operation</p>\n"},"errback":{"!type":"fn(error: ?)","!doc":"<p>Fulfills the Deferred instance as an error with the provided error</p>\n"},"isCanceled":{"!type":"fn() -> bool","!doc":"<p>Checks whether the deferred has been canceled.</p>\n"},"isFulfilled":{"!type":"fn() -> bool","!doc":"<p>Checks whether the deferred has been resolved or rejected.</p>\n"},"isRejected":{"!type":"fn() -> bool","!doc":"<p>Checks whether the deferred has been rejected.</p>\n"},"isResolved":{"!type":"fn() -> bool","!doc":"<p>Checks whether the deferred has been resolved.</p>\n"},"progress":{"!type":"fn(update: ?)","!doc":"<p>Send progress events to all listeners</p>\n"},"reject":{"!type":"fn(error: ?)","!doc":"<p>Fulfills the Deferred instance as an error with the provided error</p>\n"},"resolve":{"!type":"fn(value: ?)","!doc":"<p>Fulfills the Deferred instance successfully with the provide value</p>\n"},"then":{"!type":"fn(resolvedCallback?: ?, errorCallback?: ?, progressCallback?: ?) -> +any","!doc":"<p>Adds a fulfilledHandler, errorHandler, and progressHandler to be called for\ncompletion of a promise. The fulfilledHandler is called when the promise\nis fulfilled. The errorHandler is called when a promise fails. The\nprogressHandler is called for progress events. All arguments are optional\nand non-function values are ignored. The progressHandler is not only an\noptional argument, but progress events are purely optional. Promise\nproviders are not required to ever create progress events.\n\n</p>\n<p>This function will return a new promise that is fulfilled when the given\nfulfilledHandler or errorHandler callback is finished. This allows promise\noperations to be chained together. The value returned from the callback\nhandler is the fulfillment value for the returned promise. If the callback\nthrows an error, the returned promise will be moved to failed state.\n</p>\n"},"when":{"!type":"fn(valueOrPromise: ?, callback?: ?, errback?: ?, progback?: ?) -> +dojo","!doc":"<p>Transparently applies callbacks to values and/or promises.</p>\n"}},"dojo/when":{"!doc":"<p>Transparently applies callbacks to values and/or promises.</p>\n","prototype":{"constructor":{"!type":"fn(valueOrPromise: ?, callback?: ?, errback?: ?, progback?: ?) -> +dojo","!doc":"<p>Transparently applies callbacks to values and/or promises.</p>\n"}}},"dojo/_base/array":{"!doc":"<p>The Javascript v1.6 array extensions.</p>\n","clearCache":{"!type":"fn()"},"every":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?) -> bool","!doc":"<p>Determines whether or not every item in arr satisfies the\ncondition implemented by callback.</p>\n"},"filter":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?) -> []","!doc":"<p>Returns a new Array with those items from arr that match the\ncondition implemented by callback.</p>\n"},"forEach":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?)","!doc":"<p>for every item in arr, callback is invoked. Return values are ignored.\nIf you want to break out of the loop, consider using array.every() or array.some().\nforEach does not allow breaking out of the loop over the items in arr.</p>\n"},"indexOf":{"!type":"fn(arr: ?, value: ?, fromIndex?: ?, findLast?: ?) -> number","!doc":"<p>locates the first index of the provided value in the\npassed array. If the value is not found, -1 is returned.</p>\n"},"lastIndexOf":{"!type":"fn(arr: ?, value: ?, fromIndex?: ?) -> number","!doc":"<p>locates the last index of the provided value in the passed\narray. If the value is not found, -1 is returned.</p>\n"},"map":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?, Ctr: ?) -> []","!doc":"<p>applies callback to each element of arr and returns\nan Array with the results</p>\n"},"some":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?) -> bool","!doc":"<p>Determines whether or not any item in arr satisfies the\ncondition implemented by callback.</p>\n"}},"dojo/Evented":{"!doc":"<p>A class that can be used as a mixin or base class,\nto add on() and emit() methods to a class\nfor listening for events and emitting events:</p>\n","prototype":{"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojo/aspect":{"!doc":"<p>provides aspect oriented programming functionality, allowing for\none to add before, around, or after advice on existing methods.</p>\n","after":{"!type":"fn(target: ?, methodName: ?, advice: ?, receiveArguments?: ?) -> +any","!doc":"<p>The &quot;after&quot; export of the aspect module is a function that can be used to attach\n&quot;after&quot; advice to a method. This function will be executed after the original method\nis executed. By default the function will be called with a single argument, the return\nvalue of the original method, or the the return value of the last executed advice (if a previous one exists).\nThe fourth (optional) argument can be set to true to so the function receives the original\narguments (from when the original method was called) rather than the return value.\nIf there are multiple &quot;after&quot; advisors, they are executed in the order they were registered.</p>\n"},"around":{"!type":"fn(target: ?, methodName: ?, advice: ?)","!doc":"<p>The &quot;around&quot; export of the aspect module is a function that can be used to attach\n&quot;around&quot; advice to a method. The advisor function is immediately executed when\nthe around() is called, is passed a single argument that is a function that can be\ncalled to continue execution of the original method (or the next around advisor).\nThe advisor function should return a function, and this function will be called whenever\nthe method is called. It will be called with the arguments used to call the method.\nWhatever this function returns will be returned as the result of the method call (unless after advise changes it).</p>\n"},"before":{"!type":"fn(target: ?, methodName: ?, advice: ?)","!doc":"<p>The &quot;before&quot; export of the aspect module is a function that can be used to attach\n&quot;before&quot; advice to a method. This function will be executed before the original method\nis executed. This function will be called with the arguments used to call the method.\nThis function may optionally return an array as the new arguments to use to call\nthe original method (or the previous, next-to-execute before advice, if one exists).\nIf the before method doesn&#39;t return anything (returns undefined) the original arguments\nwill be preserved.\nIf there are multiple &quot;before&quot; advisors, they are executed in the reverse order they were registered.</p>\n"}},"dojo/on":{"!doc":"<p>A function that provides core event listening functionality. With this function\nyou can provide a target, event type, and listener to be notified of\nfuture matching events that are fired.</p>\n","prototype":{"constructor":{"!type":"fn(target: ?, type: ?, listener: ?, dontFix: ?) -> ?","!doc":"<p>A function that provides core event listening functionality. With this function\nyou can provide a target, event type, and listener to be notified of\nfuture matching events that are fired.</p>\n"}},"emit":{"!type":"fn(target: ?, type: ?, event: ?) -> ?"},"once":{"!type":"fn(target: ?, type: ?, listener: ?, dontFix: ?) -> ?","!doc":"<p>This function acts the same as on(), but will only call the listener once. The \nlistener will be called for the first\nevent that takes place and then listener will automatically be removed.</p>\n"},"parse":{"!type":"fn(target: ?, type: ?, listener: ?, addListener: ?, dontFix: ?, matchesTarget: ?) -> ?"},"pausable":{"!type":"fn(target: ?, type: ?, listener: ?, dontFix: ?) -> ?","!doc":"<p>This function acts the same as on(), but with pausable functionality. The\nreturned signal object has pause() and resume() functions. Calling the\npause() method will cause the listener to not be called for future events. Calling the\nresume() method will cause the listener to again be called for future events.</p>\n"},"selector":{"!type":"fn(selector: ?, eventType: ?, children: ?) -> fn","!doc":"<p>Creates a new extension event with event delegation. This is based on\nthe provided event type (can be extension event) that\nonly calls the listener when the CSS selector matches the target of the event.\n\n</p>\n<p>The application must require() an appropriate level of dojo/query to handle the selector.</p>\n"}},"dojo/NodeList-data":{"!doc":"<p>Adds data() and removeData() methods to NodeList, and returns NodeList constructor.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Adds data() and removeData() methods to NodeList, and returns NodeList constructor.</p>\n"}}},"dojo/query":{"!doc":"<p>This modules provides DOM querying functionality. The module export is a function\nthat can be used to query for DOM nodes by CSS selector and returns a NodeList\nrepresenting the matching nodes.</p>\n","prototype":{"constructor":{"!type":"fn(selector: ?, context?: ?) -> +instance","!doc":"<p>This modules provides DOM querying functionality. The module export is a function\nthat can be used to query for DOM nodes by CSS selector and returns a NodeList\nrepresenting the matching nodes.</p>\n"}},"load":{"!type":"fn(id: ?, parentRequire: ?, loaded: ?)","!doc":"<p>can be used as AMD plugin to conditionally load new query engine</p>\n"},"NodeList":{"!type":"fn(array: ?) -> []","!doc":"<p>Array-like object which adds syntactic\nsugar for chaining, common iteration operations, animation, and\nnode manipulation. NodeLists are most often returned as the\nresult of dojo/query() calls.</p>\n"}},"dojo/dom":{"!doc":"<p>This module defines the core dojo DOM API.</p>\n","byId":{"!type":"fn(id: ?, doc?: ?) -> +instance","!doc":"<p>Returns DOM node with matching <code>id</code> attribute or falsy value (ex: null or undefined)\nif not found.  If <code>id</code> is a DomNode, this function is a no-op.\n</p>\n"},"isDescendant":{"!type":"fn(node: ?, ancestor: ?) -> bool","!doc":"<p>Returns true if node is a descendant of ancestor</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)"}},"dojo/_base/window":{"!doc":"<p>API to save/set/restore the global/document scope.</p>\n","doc":{"!doc":"<p>Alias for the current document. &#39;doc&#39; can be modified\nfor temporary context shifting. See also withDoc().</p>\n"},"global":{"!doc":"<p>Alias for the current window. &#39;global&#39; can be modified\nfor temporary context shifting. See also withGlobal().</p>\n"},"body":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Return the body element of the specified document or of dojo/_base/window::doc.</p>\n"},"setContext":{"!type":"fn(globalObject: ?, globalDocument: ?)","!doc":"<p>changes the behavior of many core Dojo functions that deal with\nnamespace and DOM lookup, changing them to work in a new global\ncontext (e.g., an iframe). The varibles dojo.global and dojo.doc\nare modified as a result of calling this function and the result of\n<code>dojo.body()</code> likewise differs.</p>\n"},"withDoc":{"!type":"fn(documentObject: ?, callback: ?, thisObject?: ?, cbArguments?: ?) -> ?","!doc":"<p>Invoke callback with documentObject as dojo/_base/window::doc.</p>\n"},"withGlobal":{"!type":"fn(globalObject: ?, callback: ?, thisObject?: ?, cbArguments?: ?) -> ?","!doc":"<p>Invoke callback with globalObject as dojo.global and\nglobalObject.document as dojo.doc.</p>\n"}},"dojo/_base/window.global":{"!doc":"<p>Alias for the current window. &#39;global&#39; can be modified\nfor temporary context shifting. See also withGlobal().</p>\n","$":{"!type":"fn() -> ?"},"GoogleSearchStoreCallback_undefined_NaN":{"!type":"fn(start: ?, data: ?, responseCode: ?, errorMsg: ?)"},"jQuery":{"!type":"fn() -> ?"},"swfIsInHTML":{"!type":"fn()"},"undefined_onload":{"!type":"fn()"}},"dojo/_base/window.doc":{"!doc":"<p>Alias for the current document. &#39;doc&#39; can be modified\nfor temporary context shifting. See also withDoc().</p>\n","documentElement":{},"dojoClick":{}},"dojo/selector/_loader":{"!doc":"<p>This module handles loading the appropriate selector engine for the given browser</p>\n","load":{"!type":"fn(id: ?, parentRequire: ?, loaded: ?, config: ?) -> ?"}},"dojo/dom-attr":{"get":{"!type":"fn(node: ?, name: ?) -> +any","!doc":"<p>Gets an attribute on an HTML element.</p>\n"},"getNodeProp":{"!type":"fn(node: ?, name: ?) -> +any","!doc":"<p>Returns an effective value of a property or an attribute.</p>\n"},"has":{"!type":"fn(node: ?, name: ?) -> bool","!doc":"<p>Returns true if the requested attribute is specified on the\ngiven element, and false otherwise.</p>\n"},"remove":{"!type":"fn(node: ?, name: ?)","!doc":"<p>Removes an attribute from an HTML element.</p>\n"},"set":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Sets an attribute on an HTML element.</p>\n"}},"dojo/dom-style":{"!doc":"<p>This module defines the core dojo DOM style API.</p>\n","get":{"!type":"fn(node: ?, name: ?) -> ?"},"getComputedStyle":{"!type":"fn(node: ?)","!doc":"<p>Returns a &quot;computed style&quot; object.\n</p>\n"},"set":{"!type":"fn(node: ?, name: ?, value?: ?) -> ?"},"toPixelValue":{"!type":"fn(node: ?, value: ?) -> number","!doc":"<p>converts style value to pixels on IE or return a numeric value.</p>\n"}},"dojo/dom-prop":{"names":{},"get":{"!type":"fn(node: ?, name: ?) -> +any","!doc":"<p>Gets a property on an HTML element.</p>\n"},"set":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Sets a property on an HTML element.</p>\n"}},"dojo/dom-prop.names":{"class":{},"colspan":{},"for":{},"frameborder":{},"readonly":{},"rowspan":{},"tabindex":{},"valuetype":{}},"dojo/dom-construct":{"create":{"!type":"fn(tag: ?, attrs: ?, refNode?: ?, pos?: ?) -> ?","!doc":"<p>Create an element, allowing for optional attribute decoration\nand placement.</p>\n"},"destroy":{"!type":"fn(node: ?)","!doc":"<p>Removes a node from its parent, clobbering it and all of its\nchildren.\n</p>\n"},"empty":{"!type":"fn(node: ?)","!doc":"<p>safely removes all children of the node.</p>\n"},"place":{"!type":"fn(node: ?, refNode: ?, position?: ?) -> +Node","!doc":"<p>Attempt to insert node into the DOM, choosing from various positioning options.\nReturns the first argument resolved to a DOM node.</p>\n"},"toDom":{"!type":"fn(frag: ?, doc?: ?) -> +any","!doc":"<p>instantiates an HTML fragment returning the corresponding DOM.</p>\n"}},"dojo/_base/connect":{"!doc":"<p>This module defines the dojo.connect API.\nThis modules also provides keyboard event handling helpers.\nThis module exports an extension event for emulating Firefox&#39;s keypress handling.\nHowever, this extension event exists primarily for backwards compatibility and\nis not recommended. WebKit and IE uses an alternate keypress handling (only\nfiring for printable characters, to distinguish from keydown events), and most\nconsider the WebKit/IE behavior more desirable.</p>\n","connect":{"!type":"fn(obj?: ?, event: ?, context: ?, method: ?, dontFix?: ?) -> ?","!doc":"<p><code>dojo.connect</code> is a deprecated event handling and delegation method in\nDojo. It allows one function to &quot;listen in&quot; on the execution of\nany other, triggering the second whenever the first is called. Many\nlisteners may be attached to a function, and source functions may\nbe either regular function calls or DOM events.\n</p>\n"},"connectPublisher":{"!type":"fn(topic: ?, obj?: ?, event: ?) -> ?","!doc":"<p>Ensure that every time obj.event() is called, a message is published\non the topic. Returns a handle which can be passed to\ndojo.disconnect() to disable subsequent automatic publication on\nthe topic.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Remove a link created by dojo.connect.</p>\n"},"isCopyKey":{"!type":"fn(e: ?) -> ?","!doc":"<p>Checks an event for the copy key (meta on Mac, and ctrl anywhere else)</p>\n"},"publish":{"!type":"fn(topic: ?, args?: ?) -> ?","!doc":"<p>Invoke all listener method subscribed to topic.</p>\n"},"subscribe":{"!type":"fn(topic: ?, context?: ?, method: ?) -> ?","!doc":"<p>Attach a listener to a named topic. The listener function is invoked whenever the\nnamed topic is published (see: dojo.publish).\nReturns a handle which is needed to unsubscribe this listener.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Remove a topic listener.</p>\n"}},"dojo/topic":{"!doc":"<p>Pubsub hub.</p>\n","publish":{"!type":"fn(topic: ?, event: ?) -> ?","!doc":"<p>Publishes a message to a topic on the pub/sub hub. All arguments after\nthe first will be passed to the subscribers, so any number of arguments\ncan be provided (not just event).</p>\n"},"subscribe":{"!type":"fn(topic: ?, listener: ?) -> ?","!doc":"<p>Subscribes to a topic on the pub/sub hub</p>\n"}},"dojo/_base/event":{"!doc":"<p>This module defines dojo DOM event API.   Usually you should use dojo/on, and evt.stopPropagation() +\nevt.preventDefault(), rather than this module.</p>\n","fix":{"!type":"fn(evt: ?, sender: ?) -> +Event","!doc":"<p>normalizes properties on the event object including event\nbubbling methods, keystroke normalization, and x/y positions</p>\n"},"stop":{"!type":"fn(evt: ?)","!doc":"<p>prevents propagation and clobbers the default action of the\npassed event</p>\n"}},"dojo/dom-geometry":{"!doc":"<p>This module defines the core dojo DOM geometry API.</p>\n","boxModel":{},"docScroll":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Returns an object with {node, x, y} with corresponding offsets.</p>\n"},"fixIeBiDiScrollLeft":{"!type":"fn(scrollLeft: ?, doc?: ?) -> number","!doc":"<p>In RTL direction, scrollLeft should be a negative value, but IE\nreturns a positive one. All codes using documentElement.scrollLeft\nmust call this function to fix this error, otherwise the position\nwill offset to right when there is a horizontal scrollbar.</p>\n"},"getBorderExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns an object with properties useful for noting the border\ndimensions.</p>\n"},"getContentBox":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>Returns an object that encodes the width, height, left and top\npositions of the node&#39;s content box, irrespective of the\ncurrent box model.</p>\n"},"getIeDocumentElementOffset":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>returns the offset in x and y from the document body to the\nvisual edge of the page for IE</p>\n"},"getMarginBox":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns an object that encodes the width, height, left and top\npositions of the node&#39;s margin box.</p>\n"},"getMarginExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns object with properties useful for box fitting with\nregards to box margins (i.e., the outer-box).\n\n</p>\n<ul>\n<li>l/t = marginLeft, marginTop, respectively</li>\n<li>w = total width, margin inclusive</li>\n<li>h = total height, margin inclusive</li>\n</ul>\n<p>The w/h are used for calculating boxes.\nNormally application code will not need to invoke this\ndirectly, and will use the ...box... functions instead.</p>\n"},"getMarginSize":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns an object that encodes the width and height of\nthe node&#39;s margin box</p>\n"},"getPadBorderExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>Returns object with properties useful for box fitting with\nregards to padding.</p>\n"},"getPadExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>Returns object with special values specifically useful for node\nfitting.</p>\n"},"isBodyLtr":{"!type":"fn(doc?: ?) -> bool","!doc":"<p>Returns true if the current language is left-to-right, and false otherwise.</p>\n"},"normalizeEvent":{"!type":"fn(event: ?)","!doc":"<p>Normalizes the geometry of a DOM event, normalizing the pageX, pageY,\noffsetX, offsetY, layerX, and layerX properties</p>\n"},"position":{"!type":"fn(node: ?, includeScroll?: ?) -> ?","!doc":"<p>Gets the position and size of the passed element relative to\nthe viewport (if includeScroll==false), or relative to the\ndocument root (if includeScroll==true).\n</p>\n"},"setContentSize":{"!type":"fn(node: ?, box: ?, computedStyle?: ?)","!doc":"<p>Sets the size of the node&#39;s contents, irrespective of margins,\npadding, or borders.</p>\n"},"setMarginBox":{"!type":"fn(node: ?, box: ?, computedStyle?: ?)","!doc":"<p>sets the size of the node&#39;s margin box and placement\n(left/top), irrespective of box model. Think of it as a\npassthrough to setBox that handles box-model vagaries for\nyou.</p>\n"}},"dojo/mouse":{"!doc":"<p>This module provide mouse event handling utility functions and exports\nmouseenter and mouseleave event emulation.</p>\n","enter":{"!doc":"<p>This is an extension event for the mouseenter that IE provides, emulating the\nbehavior on other browsers.</p>\n"},"leave":{"!doc":"<p>This is an extension event for the mouseleave that IE provides, emulating the\nbehavior on other browsers.</p>\n"},"isLeft":{"!type":"fn()","!doc":"<p>Test an event object (from a mousedown event) to see if the left button was pressed.</p>\n"},"isMiddle":{"!type":"fn()","!doc":"<p>Test an event object (from a mousedown event) to see if the middle button was pressed.</p>\n"},"isRight":{"!type":"fn()","!doc":"<p>Test an event object (from a mousedown event) to see if the right button was pressed.</p>\n"},"wheel":{"!type":"fn(node: ?, listener: ?) -> ?","!doc":"<p>This is an extension event for the mousewheel that non-Mozilla browsers provide,\nemulating the behavior on Mozilla based browsers.</p>\n"}},"dojo/_base/sniff":{"!doc":"<p>Deprecated.   New code should use dojo/sniff.\nThis module populates the dojo browser version sniffing properties like dojo.isIE.</p>\n"},"dojo/keys":{"!doc":"<p>Definitions for common key values.  Client code should test keyCode against these named constants,\nas the actual codes can vary by browser.</p>\n","ALT":{},"BACKSPACE":{},"CAPS_LOCK":{},"CLEAR":{},"copyKey":{},"CTRL":{},"DELETE":{},"DOWN_ARROW":{},"DOWN_DPAD":{},"END":{},"ENTER":{},"ESCAPE":{},"F1":{},"F10":{},"F11":{},"F12":{},"F13":{},"F14":{},"F15":{},"F2":{},"F3":{},"F4":{},"F5":{},"F6":{},"F7":{},"F8":{},"F9":{},"HELP":{},"HOME":{},"INSERT":{},"LEFT_ARROW":{},"LEFT_DPAD":{},"LEFT_WINDOW":{},"META":{},"NUM_LOCK":{},"NUMPAD_0":{},"NUMPAD_1":{},"NUMPAD_2":{},"NUMPAD_3":{},"NUMPAD_4":{},"NUMPAD_5":{},"NUMPAD_6":{},"NUMPAD_7":{},"NUMPAD_8":{},"NUMPAD_9":{},"NUMPAD_DIVIDE":{},"NUMPAD_ENTER":{},"NUMPAD_MINUS":{},"NUMPAD_MULTIPLY":{},"NUMPAD_PERIOD":{},"NUMPAD_PLUS":{},"PAGE_DOWN":{},"PAGE_UP":{},"PAUSE":{},"RIGHT_ARROW":{},"RIGHT_DPAD":{},"RIGHT_WINDOW":{},"SCROLL_LOCK":{},"SELECT":{},"SHIFT":{},"SPACE":{},"TAB":{},"UP_ARROW":{},"UP_DPAD":{}},"dojo/NodeList-dom":{"!doc":"<p>Adds DOM related methods to NodeList, and returns NodeList constructor.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Adds DOM related methods to NodeList, and returns NodeList constructor.</p>\n"}}},"dojo/dom-class":{"!doc":"<p>This module defines the core dojo DOM class API.</p>\n","add":{"!type":"fn(node: ?, classStr: ?)","!doc":"<p>Adds the specified classes to the end of the class list on the\npassed node. Will not re-apply duplicate classes.\n</p>\n"},"contains":{"!type":"fn(node: ?, classStr: ?) -> bool","!doc":"<p>Returns whether or not the specified classes are a portion of the\nclass list currently applied to the node.</p>\n"},"remove":{"!type":"fn(node: ?, classStr?: ?)","!doc":"<p>Removes the specified classes from node. No <code>contains()</code>\ncheck is required.\n</p>\n"},"replace":{"!type":"fn(node: ?, addClassStr: ?, removeClassStr?: ?)","!doc":"<p>Replaces one or more classes on a node if not present.\nOperates more quickly than calling dojo.removeClass and dojo.addClass\n</p>\n"},"toggle":{"!type":"fn(node: ?, classStr: ?, condition?: ?) -> bool","!doc":"<p>Adds a class to node if not present, or removes if present.\nPass a boolean condition if you want to explicitly add or remove.\nReturns the condition that was specified directly or indirectly.\n</p>\n"}},"dojo/NodeList-fx":{"!doc":"<p>Adds dojo.fx animation support to dojo.query() by extending the NodeList class\nwith additional FX functions.  NodeList is the array-like object used to hold query results.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Adds dojo.fx animation support to dojo.query() by extending the NodeList class\nwith additional FX functions.  NodeList is the array-like object used to hold query results.</p>\n"},"fadeTo":{"!type":"fn(args: ?) -> ?","!doc":"<p>fade all elements of the node list to a specified opacity</p>\n"},"highlight":{"!type":"fn(args: ?) -> ?","!doc":"<p>highlight all elements of the node list.\nReturns an instance of dojo.Animation</p>\n"},"sizeTo":{"!type":"fn(args: ?) -> ?","!doc":"<p>size all elements of this NodeList. Returns an instance of dojo.Animation</p>\n"},"slideBy":{"!type":"fn(args: ?) -> ?","!doc":"<p>slide all elements of this NodeList. Returns an instance of dojo.Animation</p>\n"},"wipeTo":{"!type":"fn(args: ?) -> ?","!doc":"<p>Wipe all elements of the NodeList to a specified width: or height:</p>\n"}}},"dojo/_base/fx":{"!doc":"<p>This module defines the base dojo/_base/fx implementation.</p>\n","anim":{"!type":"fn(node: ?, properties: ?, duration?: ?, easing?: ?, onEnd?: ?, delay?: ?) -> ?","!doc":"<p>A simpler interface to <code>animateProperty()</code>, also returns\nan instance of <code>Animation</code> but begins the animation\nimmediately, unlike nearly every other Dojo animation API.</p>\n"},"animateProperty":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation that will transition the properties of\nnode defined in <code>args</code> depending how they are defined in\n<code>args.properties</code>\n</p>\n"},"Animation":{"!type":"fn(args: ?)","!doc":"<p>A generic animation class that fires callbacks into its handlers\nobject at various states.</p>\n"},"fadeIn":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that will fade node defined in &#39;args&#39; from\nits current opacity to fully opaque.</p>\n"},"fadeOut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that will fade node defined in &#39;args&#39;\nfrom its current opacity to fully transparent.</p>\n"}},"dojo/_base/declare":{"!doc":"<p>Create a feature-rich constructor from compact notation.</p>\n","__DeclareCreatedObject":{"!doc":"<p>dojo/_base/declare() returns a constructor <code>C</code>.   <code>new C()</code> returns an Object with the following\nmethods, in addition to the methods and properties specified via the arguments passed to declare().</p>\n"},"prototype":{"constructor":{"!type":"fn(className?: ?, superclass: ?, props: ?) -> +dojo","!doc":"<p>Create a feature-rich constructor from compact notation.</p>\n"}},"safeMixin":{"!type":"fn(target: ?, source: ?) -> ?","!doc":"<p>Mix in properties skipping a constructor and decorating functions\nlike it is done by declare().</p>\n"}},"dojo/_base/declare.__DeclareCreatedObject":{"!doc":"<p>dojo/_base/declare() returns a constructor <code>C</code>.   <code>new C()</code> returns an Object with the following\nmethods, in addition to the methods and properties specified via the arguments passed to declare().</p>\n","createSubclass":{"!type":"fn(mixins: ?, props?: ?) -> +dojo","!doc":"<p>Create a subclass of the declared class from a list of base classes.</p>\n"},"extend":{"!type":"fn(source: ?)","!doc":"<p>Adds all properties and methods of source to constructor&#39;s\nprototype, making them available to all instances created with\nconstructor. This method is specific to constructors created with\ndeclare().</p>\n"},"getInherited":{"!type":"fn(name?: ?, args: ?) -> +any","!doc":"<p>Returns a super method.</p>\n"},"inherited":{"!type":"fn(name?: ?, args: ?, newArgs?: ?) -> +any","!doc":"<p>Calls a super method.</p>\n"},"isInstanceOf":{"!type":"fn(cls: ?) -> +any","!doc":"<p>Checks the inheritance chain to see if it is inherited from this\nclass.</p>\n"}},"dojo/_base/Color":{"!doc":"<p>Takes a named string, hex string, array of rgb or rgba values,\nan object with r, g, b, and a properties, or another <code>Color</code> object\nand creates a new Color instance to work from.\n</p>\n","prototype":{"a":{},"b":{},"g":{},"r":{},"constructor":{"!type":"fn(color: ?)","!doc":"<p>Takes a named string, hex string, array of rgb or rgba values,\nan object with r, g, b, and a properties, or another <code>Color</code> object\nand creates a new Color instance to work from.\n</p>\n"},"sanitize":{"!type":"fn()","!doc":"<p>makes sure that the object has correct attributes</p>\n"},"setColor":{"!type":"fn(color: ?) -> fn","!doc":"<p>Takes a named string, hex string, array of rgb or rgba values,\nan object with r, g, b, and a properties, or another <code>Color</code> object\nand sets this color instance to that value.\n</p>\n"},"toCmy":{"!type":"fn() -> ?","!doc":"<p>Convert this Color to a CMY definition.</p>\n"},"toCmyk":{"!type":"fn() -> ?","!doc":"<p>Convert this Color to a CMYK definition.</p>\n"},"toCss":{"!type":"fn(includeAlpha?: ?) -> string","!doc":"<p>Returns a css color string in rgb(a) representation</p>\n"},"toHex":{"!type":"fn() -> string","!doc":"<p>Returns a CSS color string in hexadecimal representation</p>\n"},"toHsl":{"!type":"fn() -> ?","!doc":"<p>Convert this Color to an HSL definition.</p>\n"},"toHsv":{"!type":"fn() -> ?","!doc":"<p>Convert this Color to an HSV definition.</p>\n"},"toRgb":{"!type":"fn() -> []","!doc":"<p>Returns 3 component array of rgb values</p>\n"},"toRgba":{"!type":"fn() -> []","!doc":"<p>Returns a 4 component array of rgba values from the color\nrepresented by this object.</p>\n"},"toString":{"!type":"fn() -> ?","!doc":"<p>Returns a visual representation of the color</p>\n"}},"named":{"!doc":"<p>Dictionary list of all CSS named colors, by name. Values are 3-item arrays with corresponding RG and B values.</p>\n"},"blendColors":{"!type":"fn(start: ?, end: ?, weight: ?, obj?: ?) -> ?","!doc":"<p>Blend colors end and start with weight from 0 to 1, 0.5 being a 50/50 blend,\ncan reuse a previously allocated Color object for the result</p>\n"},"fromArray":{"!type":"fn(a: ?, obj?: ?) -> +any","!doc":"<p>Builds a <code>Color</code> from a 3 or 4 element array, mapping each\nelement in sequence to the rgb(a) values of the color.</p>\n"},"fromHex":{"!type":"fn(color: ?, obj?: ?) -> +any","!doc":"<p>Converts a hex string with a &#39;#&#39; prefix to a color object.\nSupports 12-bit #rgb shorthand. Optionally accepts a\n<code>Color</code> object to update with the parsed value.\n</p>\n"},"fromRgb":{"!type":"fn(color: ?, obj?: ?) -> +null","!doc":"<p>get rgb(a) array from css-style color declarations</p>\n"},"fromString":{"!type":"fn(str: ?, obj?: ?) -> +any","!doc":"<p>Parses <code>str</code> for a color value. Accepts hex, rgb, and rgba\nstyle color values.</p>\n"},"makeGrey":{"!type":"fn(g: ?, a?: ?)","!doc":"<p>creates a greyscale color with an optional alpha</p>\n"}},"dojo/_base/Color.named":{"!doc":"<p>Dictionary list of all CSS named colors, by name. Values are 3-item arrays with corresponding RG and B values.</p>\n","aliceblue":{},"antiquewhite":{},"aqua":{},"aquamarine":{},"azure":{},"beige":{},"bisque":{},"black":{},"blanchedalmond":{},"blue":{},"blueviolet":{},"brown":{},"burlywood":{},"cadetblue":{},"chartreuse":{},"chocolate":{},"coral":{},"cornflowerblue":{},"cornsilk":{},"crimson":{},"cyan":{},"darkblue":{},"darkcyan":{},"darkgoldenrod":{},"darkgray":{},"darkgreen":{},"darkgrey":{},"darkkhaki":{},"darkmagenta":{},"darkolivegreen":{},"darkorange":{},"darkorchid":{},"darkred":{},"darksalmon":{},"darkseagreen":{},"darkslateblue":{},"darkslategray":{},"darkslategrey":{},"darkturquoise":{},"darkviolet":{},"deeppink":{},"deepskyblue":{},"dimgray":{},"dimgrey":{},"dodgerblue":{},"firebrick":{},"floralwhite":{},"forestgreen":{},"fuchsia":{},"gainsboro":{},"ghostwhite":{},"gold":{},"goldenrod":{},"gray":{},"green":{},"greenyellow":{},"grey":{},"honeydew":{},"hotpink":{},"indianred":{},"indigo":{},"ivory":{},"khaki":{},"lavender":{},"lavenderblush":{},"lawngreen":{},"lemonchiffon":{},"lightblue":{},"lightcoral":{},"lightcyan":{},"lightgoldenrodyellow":{},"lightgray":{},"lightgreen":{},"lightgrey":{},"lightpink":{},"lightsalmon":{},"lightseagreen":{},"lightskyblue":{},"lightslategray":{},"lightslategrey":{},"lightsteelblue":{},"lightyellow":{},"lime":{},"limegreen":{},"linen":{},"magenta":{},"maroon":{},"mediumaquamarine":{},"mediumblue":{},"mediumorchid":{},"mediumpurple":{},"mediumseagreen":{},"mediumslateblue":{},"mediumspringgreen":{},"mediumturquoise":{},"mediumvioletred":{},"midnightblue":{},"mintcream":{},"mistyrose":{},"moccasin":{},"navajowhite":{},"navy":{},"oldlace":{},"olive":{},"olivedrab":{},"orange":{},"orangered":{},"orchid":{},"palegoldenrod":{},"palegreen":{},"paleturquoise":{},"palevioletred":{},"papayawhip":{},"peachpuff":{},"peru":{},"pink":{},"plum":{},"powderblue":{},"purple":{},"red":{},"rosybrown":{},"royalblue":{},"saddlebrown":{},"salmon":{},"sandybrown":{},"seagreen":{},"seashell":{},"sienna":{},"silver":{},"skyblue":{},"slateblue":{},"slategray":{},"slategrey":{},"snow":{},"springgreen":{},"steelblue":{},"tan":{},"teal":{},"thistle":{},"tomato":{},"transparent":{},"turquoise":{},"violet":{},"wheat":{},"white":{},"whitesmoke":{},"yellow":{},"yellowgreen":{}},"dojo/fx":{"!doc":"<p>Effects library on top of Base animations</p>\n","easing":{"!doc":"<p>Collection of easing functions to use beyond the default\n<code>dojo._defaultEasing</code> function.</p>\n"},"chain":{"!type":"fn(animations: ?) -> +instance","!doc":"<p>Chain a list of <code>dojo/_base/fx.Animation</code>s to run in sequence\n</p>\n"},"combine":{"!type":"fn(animations: ?) -> +instance","!doc":"<p>Combine a list of <code>dojo/_base/fx.Animation</code>s to run in parallel\n</p>\n"},"slideTo":{"!type":"fn(args: ?) -> ?","!doc":"<p>Slide a node to a new top/left position\n</p>\n"},"Toggler":{"!type":"fn()"},"wipeIn":{"!type":"fn(args: ?) -> ?","!doc":"<p>Expand a node to it&#39;s natural height.\n</p>\n"},"wipeOut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Shrink a node to nothing and hide it.\n</p>\n"}},"dojo/fx.Toggler":{"!type":"fn()","prototype":{"!doc":"<p>A simple <code>dojo.Animation</code> toggler API.</p>\n","_hideAnim":{},"_hideArgs":{},"_isHiding":{},"_isShowing":{},"_showAnim":{},"_showArgs":{},"hideDuration":{"!doc":"<p>Time in milliseconds to run the hide Animation</p>\n"},"node":{"!doc":"<p>the node to target for the showing and hiding animations</p>\n"},"showDuration":{"!doc":"<p>Time in milliseconds to run the show Animation</p>\n"},"constructor":{"!type":"fn(args: ?)"},"hide":{"!type":"fn(delay?: ?) -> ?","!doc":"<p>Toggle the node to hidden</p>\n"},"hideFunc":{"!type":"fn(args: ?) -> ?","!doc":"<p>The function that returns the <code>dojo.Animation</code> to hide the node</p>\n"},"show":{"!type":"fn(delay?: ?) -> ?","!doc":"<p>Toggle the node to showing</p>\n"},"showFunc":{"!type":"fn(args: ?) -> ?","!doc":"<p>The function that returns the <code>dojo.Animation</code> to show the node</p>\n"}}},"dojo/fx.easing":{"!doc":"<p>Collection of easing functions to use beyond the default\n<code>dojo._defaultEasing</code> function.</p>\n","backIn":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that starts away from the target,\nand quickly accelerates towards the end value.\n\n</p>\n<p>Use caution when the easing will cause values to become\nnegative as some properties cannot be set to negative values.</p>\n"},"backInOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function combining the effects of <code>backIn</code> and <code>backOut</code></p>\n"},"backOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that pops past the range briefly, and slowly comes back.</p>\n"},"bounceIn":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that &#39;bounces&#39; near the beginning of an Animation</p>\n"},"bounceInOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that &#39;bounces&#39; at the beginning and end of the Animation</p>\n"},"bounceOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that &#39;bounces&#39; near the end of an Animation</p>\n"},"circIn":{"!type":"fn(n?: ?) -> number"},"circInOut":{"!type":"fn(n?: ?) -> number"},"circOut":{"!type":"fn(n?: ?) -> ?"},"cubicIn":{"!type":"fn(n?: ?) -> ?"},"cubicInOut":{"!type":"fn(n?: ?) -> number"},"cubicOut":{"!type":"fn(n?: ?) -> number"},"elasticIn":{"!type":"fn(n?: ?) -> +Decimal","!doc":"<p>An easing function the elastically snaps from the start value</p>\n"},"elasticInOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that elasticly snaps around the value, near\nthe beginning and end of the Animation.</p>\n"},"elasticOut":{"!type":"fn(n?: ?) -> +Decimal","!doc":"<p>An easing function that elasticly snaps around the target value,\nnear the end of the Animation</p>\n"},"expoIn":{"!type":"fn(n?: ?) -> ?"},"expoInOut":{"!type":"fn(n?: ?) -> number"},"expoOut":{"!type":"fn(n?: ?) -> number"},"linear":{"!type":"fn(n?: ?) -> +Decimal","!doc":"<p>A linear easing function</p>\n"},"quadIn":{"!type":"fn(n?: ?) -> ?"},"quadInOut":{"!type":"fn(n?: ?) -> number"},"quadOut":{"!type":"fn(n?: ?) -> number"},"quartIn":{"!type":"fn(n?: ?) -> ?"},"quartInOut":{"!type":"fn(n?: ?) -> number"},"quartOut":{"!type":"fn(n?: ?) -> number"},"quintIn":{"!type":"fn(n?: ?) -> ?"},"quintInOut":{"!type":"fn(n?: ?) -> number"},"quintOut":{"!type":"fn(n?: ?) -> number"},"sineIn":{"!type":"fn(n?: ?) -> number"},"sineInOut":{"!type":"fn(n?: ?) -> number"},"sineOut":{"!type":"fn(n?: ?) -> ?"}},"dojo/ready":{"!doc":"<p>Add a function to execute on DOM content loaded and all requested modules have arrived and been evaluated.\nIn most cases, the <code>domReady</code> plug-in should suffice and this method should not be needed.\n\n</p>\n<p>When called in a non-browser environment, just checks that all requested modules have arrived and been\nevaluated.</p>\n","prototype":{"constructor":{"!type":"fn(priority?: ?, context: ?, callback?: ?)","!doc":"<p>Add a function to execute on DOM content loaded and all requested modules have arrived and been evaluated.\nIn most cases, the <code>domReady</code> plug-in should suffice and this method should not be needed.\n\n</p>\n<p>When called in a non-browser environment, just checks that all requested modules have arrived and been\nevaluated.</p>\n"}}},"dojo/NodeList-html":{"!doc":"<p>Adds a chainable html method to dojo/query() / NodeList instances for setting/replacing node content</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Adds a chainable html method to dojo/query() / NodeList instances for setting/replacing node content</p>\n"}}},"dojo/html":{"!doc":"<p>TODOC</p>\n","set":{"!type":"fn(node: ?, cont: ?, params?: ?) -> ?","!doc":"<p>inserts (replaces) the given content into the given node. dojo/dom-construct.place(cont, node, &quot;only&quot;)\nmay be a better choice for simple HTML insertion.</p>\n"}},"dojo/html._ContentSetter":{"!type":"fn()","prototype":{"cleanContent":{"!doc":"<p>Should the content be treated as a full html document,\nand the real content stripped of <html>, <body> wrapper before injection</p>\n"},"content":{"!doc":"<p>The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes</p>\n"},"extractContent":{"!doc":"<p>Should the content be treated as a full html document,\nand the real content stripped of <code>&lt;html&gt; &lt;body&gt;</code> wrapper before injection</p>\n"},"id":{"!doc":"<p>Usually only used internally, and auto-generated with each instance</p>\n"},"node":{"!doc":"<p>An node which will be the parent element that we set content into</p>\n"},"parseContent":{"!doc":"<p>Should the node by passed to the parser after the new content is set</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"startup":{"!doc":"<p>Start the child widgets after parsing them.   Only obeyed if parseContent is true.</p>\n"},"constructor":{"!type":"fn(params: ?, node: ?)","!doc":"<p>Provides a configurable, extensible object to wrap the setting on content on a node\ncall the set() method to actually set the content..</p>\n"},"empty":{"!type":"fn()","!doc":"<p>cleanly empty out existing content</p>\n"},"set":{"!type":"fn(cont?: ?, params?: ?) -> ?","!doc":"<p>front-end to the set-content sequence</p>\n"},"setContent":{"!type":"fn()","!doc":"<p>sets the content on the node</p>\n"},"tearDown":{"!type":"fn()","!doc":"<p>manually reset the Setter instance if its being re-used for example for another set()</p>\n"}}},"dojo/parser":{"!doc":"<p>The Dom/Widget parsing package</p>\n","_attrName":{},"_query":{},"construct":{"!type":"fn(ctor: ?, node: ?, mixin?: ?, options?: ?, scripts?: ?, inherited?: ?) -> +any","!doc":"<p>Calls new ctor(params, node), where params is the hash of parameters specified on the node,\nexcluding data-dojo-type and data-dojo-mixins.   Does not call startup().</p>\n"},"instantiate":{"!type":"fn(nodes: ?, mixin?: ?, options?: ?) -> +any","!doc":"<p>Takes array of nodes, and turns them into class instances and\npotentially calls a startup method to allow them to connect with\nany children.</p>\n"},"parse":{"!type":"fn(rootNode?: ?, options?: ?) -> +Mixed","!doc":"<p>Scan the DOM for class instances, and instantiate them.</p>\n"},"scan":{"!type":"fn(root?: ?, options: ?) -> +Promise","!doc":"<p>Scan a DOM tree and return an array of objects representing the DOMNodes\nthat need to be turned into widgets.</p>\n"}},"dojo/_base/url":{"authority":{},"fragment":{},"host":{},"password":{},"path":{},"port":{},"query":{},"scheme":{},"uri":{},"user":{},"prototype":{"constructor":{"!type":"fn()"},"toString":{"!type":"fn()"}}},"dojo/_base/url.scheme":{},"dojo/_base/url.authority":{},"dojo/_base/url.query":{},"dojo/_base/url.fragment":{},"dojo/_base/url.user":{},"dojo/_base/url.password":{},"dojo/_base/url.port":{},"dojo/promise/all":{"!doc":"<p>Takes multiple promises and returns a new promise that is fulfilled\nwhen all promises have been fulfilled.</p>\n","prototype":{"constructor":{"!type":"fn(objectOrArray?: ?) -> +dojo","!doc":"<p>Takes multiple promises and returns a new promise that is fulfilled\nwhen all promises have been fulfilled.</p>\n"}}},"dojo/date/stamp":{"!doc":"<p>TODOC</p>\n","_isoRegExp":{},"fromISOString":{"!type":"fn(formattedString: ?, defaultTime?: ?) -> +instance","!doc":"<p>Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.\n</p>\n"},"toISOString":{"!type":"fn(dateObject: ?, options?: ?) -> ?","!doc":"<p>Format a Date object as a string according a subset of the ISO-8601 standard\n</p>\n"}},"dojo/NodeList-manipulate":{"!doc":"<p>Adds chainable methods to dojo.query() / NodeList instances for manipulating HTML\nand DOM nodes and their properties.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Adds chainable methods to dojo.query() / NodeList instances for manipulating HTML\nand DOM nodes and their properties.</p>\n"}}},"dojo/NodeList-traverse":{"!doc":"<p>Adds chainable methods to dojo/query() / NodeList instances for traversing the DOM</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Adds chainable methods to dojo/query() / NodeList instances for traversing the DOM</p>\n"}}},"dojo/NodeList":{"!doc":"<p>Array-like object which adds syntactic\nsugar for chaining, common iteration operations, animation, and\nnode manipulation. NodeLists are most often returned as the\nresult of dojo/query() calls.</p>\n","_nodeDataCache":{},"events":{},"prototype":{"addClass":{"!type":"fn(className: ?)","!doc":"<p>adds the specified class to every node in the list</p>\n"},"addClassFx":{"!type":"fn(cssClass: ?, args: ?)","!doc":"<p>Animate the effects of adding a class to all nodes in this list.\nsee <code>dojox.fx.addClass</code></p>\n"},"addContent":{"!type":"fn(content: ?, position?: ?) -> fn","!doc":"<p>add a node, NodeList or some HTML as a string to every item in the\nlist.  Returns the original list.</p>\n"},"adopt":{"!type":"fn(queryOrListOrNode: ?, position?: ?) -> ?","!doc":"<p>places any/all elements in queryOrListOrNode at a\nposition relative to the first element in this list.\nReturns a dojo/NodeList of the adopted elements.</p>\n"},"after":{"!type":"fn(content: ?) -> +any","!doc":"<p>Places the content after every node in the NodeList.</p>\n"},"andSelf":{"!type":"fn() -> ?","!doc":"<p>Adds the nodes from the previous dojo/NodeList to the current dojo/NodeList.</p>\n"},"anim":{"!type":"fn(properties: ?, duration?: ?, easing?: ?, onEnd?: ?, delay?: ?) -> ?","!doc":"<p>Animate one or more CSS properties for all nodes in this list.\nThe returned animation object will already be playing when it\nis returned. See the docs for <code>dojo.anim</code> for full details.</p>\n"},"animateProperty":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>Animate all elements of this NodeList across the properties specified.\nsyntax identical to <code>dojo.animateProperty</code>\n</p>\n"},"append":{"!type":"fn(content: ?) -> +any","!doc":"<p>appends the content to every node in the NodeList.</p>\n"},"appendTo":{"!type":"fn(query: ?) -> +any","!doc":"<p>appends nodes in this NodeList to the nodes matched by\nthe query passed to appendTo.</p>\n"},"at":{"!type":"fn(index: ?) -> ?","!doc":"<p>Returns a new NodeList comprised of items in this NodeList\nat the given index or indices.\n</p>\n"},"attr":{"!type":"fn(property: ?, value?: ?) -> +any","!doc":"<p>gets or sets the DOM attribute for every element in the\nNodeList. See also <code>dojo/dom-attr</code></p>\n"},"before":{"!type":"fn(content: ?) -> +any","!doc":"<p>Places the content before every node in the NodeList.</p>\n"},"children":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all immediate child elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the child elements.</p>\n"},"clone":{"!type":"fn() -> +any","!doc":"<p>Clones all the nodes in this NodeList and returns them as a new NodeList.</p>\n"},"closest":{"!type":"fn(query: ?, root?: ?) -> +any","!doc":"<p>Returns closest parent that matches query, including current node in this\ndojo/NodeList if it matches the query.</p>\n"},"concat":{"!type":"fn(item?: ?) -> ?","!doc":"<p>Returns a new NodeList comprised of items in this NodeList\nas well as items passed in as parameters</p>\n"},"connect":{"!type":"fn(methodName: ?, objOrFunc: ?, funcName?: ?)","!doc":"<p>Attach event handlers to every item of the NodeList. Uses dojo.connect()\nso event properties are normalized.\n\n</p>\n<p>Application must manually require() &quot;dojo/_base/connect&quot; before using this method.</p>\n"},"constructor":{"!type":"fn(array: ?) -> []","!doc":"<p>Array-like object which adds syntactic\nsugar for chaining, common iteration operations, animation, and\nnode manipulation. NodeLists are most often returned as the\nresult of dojo/query() calls.</p>\n"},"coords":{"!type":"fn()","!doc":"<p>Deprecated: Use position() for border-box x/y/w/h\nor marginBox() for margin-box w/h/l/t.\nReturns the box objects of all elements in a node list as\nan Array (<em>not</em> a NodeList). Acts like <code>domGeom.coords</code>, though assumes\nthe node passed is each node in this list.</p>\n"},"data":{"!type":"fn(key?: ?, value?: ?) -> +Object","!doc":"<p>stash or get some arbitrary data on/from these nodes.\n</p>\n"},"delegate":{"!type":"fn(selector: ?, eventName: ?, fn: ?) -> ?","!doc":"<p>Monitor nodes in this NodeList for [bubbled] events on nodes that match selector.\nCalls fn(evt) for those events, where (inside of fn()), this == the node\nthat matches the selector.</p>\n"},"dtl":{"!type":"fn(template: ?, context: ?) -> fn","!doc":"<p>Renders the specified template in each of the NodeList entries.</p>\n"},"empty":{"!type":"fn() -> ?","!doc":"<p>clears all content from each node in the list. Effectively\nequivalent to removing all child nodes from every item in\nthe list.</p>\n"},"end":{"!type":"fn() -> ?","!doc":"<p>Ends use of the current <code>NodeList</code> by returning the previous NodeList\nthat generated the current NodeList.</p>\n"},"even":{"!type":"fn() -> +any","!doc":"<p>Returns the even nodes in this dojo/NodeList as a dojo/NodeList.</p>\n"},"every":{"!type":"fn(callback: ?, thisObject?: ?) -> ?","!doc":"<p>see <code>dojo/_base/array.every()</code> and the <a href=\"https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/every\">Array.every\ndocs</a>.\nTakes the same structure of arguments and returns as\ndojo/_base/array.every() with the caveat that the passed array is\nimplicitly this NodeList</p>\n"},"fadeIn":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>fade in all elements of this NodeList via <code>dojo.fadeIn</code>\n</p>\n"},"fadeOut":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>fade out all elements of this NodeList via <code>dojo.fadeOut</code>\n</p>\n"},"filter":{"!type":"fn(filter: ?) -> ?","!doc":"<p>&quot;masks&quot; the built-in javascript filter() method (supported\nin Dojo via <code>dojo.filter</code>) to support passing a simple\nstring filter in addition to supporting filtering function\nobjects.</p>\n"},"first":{"!type":"fn() -> +any","!doc":"<p>Returns the first node in this dojo/NodeList as a dojo/NodeList.</p>\n"},"forEach":{"!type":"fn(callback: ?, thisObj: ?) -> fn","!doc":"<p>see <code>dojo/_base/array.forEach()</code>. The primary difference is that the acted-on\narray is implicitly this NodeList. If you want the option to break out\nof the forEach loop, use every() or some() instead.</p>\n"},"html":{"!type":"fn(value?: ?) -> +any","!doc":"<p>allows setting the innerHTML of each node in the NodeList,\nif there is a value passed in, otherwise, reads the innerHTML value of the first node.</p>\n"},"indexOf":{"!type":"fn(value: ?, fromIndex?: ?) -> +any","!doc":"<p>see <code>dojo/_base/array.indexOf()</code>. The primary difference is that the acted-on\narray is implicitly this NodeList</p>\n"},"innerHTML":{"!type":"fn(value?: ?) -> +any","!doc":"<p>allows setting the innerHTML of each node in the NodeList,\nif there is a value passed in, otherwise, reads the innerHTML value of the first node.</p>\n"},"insertAfter":{"!type":"fn(query: ?) -> +any","!doc":"<p>The nodes in this NodeList will be placed after the nodes\nmatched by the query passed to insertAfter.</p>\n"},"insertBefore":{"!type":"fn(query: ?) -> +any","!doc":"<p>The nodes in this NodeList will be placed after the nodes\nmatched by the query passed to insertAfter.</p>\n"},"instantiate":{"!type":"fn(declaredClass: ?, properties?: ?) -> ?","!doc":"<p>Create a new instance of a specified class, using the\nspecified properties and each node in the NodeList as a\nsrcNodeRef.</p>\n"},"last":{"!type":"fn() -> +any","!doc":"<p>Returns the last node in this dojo/NodeList as a dojo/NodeList.</p>\n"},"lastIndexOf":{"!type":"fn(value: ?, fromIndex?: ?) -> +any","!doc":"<p>see <code>dojo/_base/array.lastIndexOf()</code>. The primary difference is that the\nacted-on array is implicitly this NodeList</p>\n"},"map":{"!type":"fn(func: ?, obj?: ?) -> ?","!doc":"<p>see <code>dojo/_base/array.map()</code>. The primary difference is that the acted-on\narray is implicitly this NodeList and the return is a\nNodeList (a subclass of Array)</p>\n"},"marginBox":{"!type":"fn()","!doc":"<p>Returns margin-box size of nodes</p>\n"},"next":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns the next element for nodes in this dojo/NodeList.\nOptionally takes a query to filter the next elements.</p>\n"},"nextAll":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all sibling elements that come after the nodes in this dojo/NodeList.\nOptionally takes a query to filter the sibling elements.</p>\n"},"odd":{"!type":"fn() -> +any","!doc":"<p>Returns the odd nodes in this dojo/NodeList as a dojo/NodeList.</p>\n"},"on":{"!type":"fn(eventName: ?, listener: ?) -> ?","!doc":"<p>Listen for events on the nodes in the NodeList. Basic usage is:\n</p>\n"},"orphan":{"!type":"fn(filter?: ?) -> +any","!doc":"<p>removes elements in this list that match the filter\nfrom their parents and returns them as a new NodeList.</p>\n"},"parent":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns immediate parent elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the parent elements.</p>\n"},"parents":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all parent elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the child elements.</p>\n"},"place":{"!type":"fn(queryOrNode: ?, position: ?) -> ?","!doc":"<p>places elements of this node list relative to the first element matched\nby queryOrNode. Returns the original NodeList. See: <code>dojo/dom-construct.place</code></p>\n"},"position":{"!type":"fn() -> ?","!doc":"<p>Returns border-box objects (x/y/w/h) of all elements in a node list\nas an Array (<em>not</em> a NodeList). Acts like <code>dojo/dom-geometry-position</code>, though\nassumes the node passed is each node in this list.</p>\n"},"prepend":{"!type":"fn(content: ?) -> +any","!doc":"<p>prepends the content to every node in the NodeList.</p>\n"},"prependTo":{"!type":"fn(query: ?) -> +any","!doc":"<p>prepends nodes in this NodeList to the nodes matched by\nthe query passed to prependTo.</p>\n"},"prev":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns the previous element for nodes in this dojo/NodeList.\nOptionally takes a query to filter the previous elements.</p>\n"},"prevAll":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all sibling elements that come before the nodes in this dojo/NodeList.\nOptionally takes a query to filter the sibling elements.</p>\n"},"query":{"!type":"fn(queryStr: ?) -> fn","!doc":"<p>Returns a new list whose members match the passed query,\nassuming elements of the current NodeList as the root for\neach search.</p>\n"},"remove":{"!type":"fn(filter?: ?) -> +any","!doc":"<p>removes elements in this list that match the filter\nfrom their parents and returns them as a new NodeList.</p>\n"},"removeAttr":{"!type":"fn(name: ?)","!doc":"<p>Removes an attribute from each node in the list.</p>\n"},"removeClass":{"!type":"fn(className?: ?) -> +any","!doc":"<p>removes the specified class from every node in the list</p>\n"},"removeClassFx":{"!type":"fn(cssClass: ?, args: ?)","!doc":"<p>Animate the effect of removing a class to all nodes in this list.\nsee <code>dojox.fx.removeClass</code></p>\n"},"removeData":{"!type":"fn(key?: ?)","!doc":"<p>Remove the data associated with these nodes.</p>\n"},"replaceAll":{"!type":"fn(query: ?) -> +any","!doc":"<p>replaces nodes matched by the query passed to replaceAll with the nodes\nin this NodeList.</p>\n"},"replaceClass":{"!type":"fn(addClassStr: ?, removeClassStr?: ?)","!doc":"<p>Replaces one or more classes on a node if not present.\nOperates more quickly than calling <code>removeClass()</code> and <code>addClass()</code></p>\n"},"replaceWith":{"!type":"fn(content: ?) -> +any","!doc":"<p>Replaces each node in ths NodeList with the content passed to replaceWith.</p>\n"},"siblings":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all sibling elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the sibling elements.</p>\n"},"slice":{"!type":"fn(begin: ?, end?: ?) -> ?","!doc":"<p>Returns a new NodeList, maintaining this one in place</p>\n"},"slideTo":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>slide all elements of the node list to the specified place via <code>dojo/fx.slideTo()</code>\n</p>\n"},"some":{"!type":"fn(callback: ?, thisObject?: ?) -> ?","!doc":"<p>Takes the same structure of arguments and returns as\n<code>dojo/_base/array.some()</code> with the caveat that the passed array is\nimplicitly this NodeList.  See <code>dojo/_base/array.some()</code> and Mozilla&#39;s\n<a href=\"https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/some\">Array.some\ndocumentation</a>.</p>\n"},"splice":{"!type":"fn(index: ?, howmany?: ?, item?: ?) -> ?","!doc":"<p>Returns a new NodeList, manipulating this NodeList based on\nthe arguments passed, potentially splicing in new elements\nat an offset, optionally deleting elements</p>\n"},"style":{"!type":"fn(property: ?, value?: ?) -> +any","!doc":"<p>gets or sets the CSS property for every element in the NodeList</p>\n"},"text":{"!type":"fn(value: ?) -> +any","!doc":"<p>allows setting the text value of each node in the NodeList,\nif there is a value passed in, otherwise, returns the text value for all the\nnodes in the NodeList in one string.</p>\n"},"toggleClass":{"!type":"fn(className: ?, condition?: ?)","!doc":"<p>Adds a class to node if not present, or removes if present.\nPass a boolean condition if you want to explicitly add or remove.</p>\n"},"toggleClassFx":{"!type":"fn(cssClass: ?, force: ?, args: ?)","!doc":"<p>Animate the effect of adding or removing a class to all nodes in this list.\nsee <code>dojox.fx.toggleClass</code></p>\n"},"toString":{"!type":"fn() -> ?"},"val":{"!type":"fn(value: ?) -> +any","!doc":"<p>If a value is passed, allows seting the value property of form elements in this\nNodeList, or properly selecting/checking the right value for radio/checkbox/select\nelements. If no value is passed, the value of the first node in this NodeList\nis returned.</p>\n"},"wipeIn":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>wipe in all elements of this NodeList via <code>dojo/fx.wipeIn()</code>\n</p>\n"},"wipeOut":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>wipe out all elements of this NodeList via <code>dojo/fx.wipeOut()</code>\n</p>\n"},"wrap":{"!type":"fn(html: ?) -> +any","!doc":"<p>Wrap each node in the NodeList with html passed to wrap.</p>\n"},"wrapAll":{"!type":"fn(html: ?) -> +any","!doc":"<p>Insert html where the first node in this NodeList lives, then place all\nnodes in this NodeList as the child of the html.</p>\n"},"wrapInner":{"!type":"fn(html: ?) -> +any","!doc":"<p>For each node in the NodeList, wrap all its children with the passed in html.</p>\n"}}},"dojo/NodeList._nodeDataCache":{},"dojo/Stateful":{"!type":"fn()","prototype":{"!doc":"<p>Base class for objects that provide named properties with optional getter/setter\ncontrol and the ability to watch for property changes\n\n</p>\n<p>The class also provides the functionality to auto-magically manage getters\nand setters for object attributes/properties.\n\n</p>\n<p>Getters and Setters should follow the format of _xxxGetter or _xxxSetter where \nthe xxx is a name of the attribute to handle.  So an attribute of &quot;foo&quot; \nwould have a custom getter of _fooGetter and a custom setter of _fooSetter.\n</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojo/_base/NodeList":{"!doc":"<p>This module extends dojo/NodeList with the legacy connect(), coords(),\nblur(), focus(), change(), click(), error(), keydown(), keypress(),\nkeyup(), load(), mousedown(), mouseenter(), mouseleave(), mousemove(),\nmouseout(), mouseover(), mouseup(), and submit() methods.</p>\n"},"dojo/_base/html":{"!doc":"<p>This module is a stub for the core dojo DOM API.</p>\n"},"dojo/_base/browser":{"!doc":"<p>This module causes the browser-only base modules to be loaded.</p>\n"},"dojo/_base/unload":{"!doc":"<p>This module contains the document and window unload detection API.</p>\n","addOnUnload":{"!type":"fn(obj?: ?, functionName?: ?)","!doc":"<p>registers a function to be triggered when the page unloads.</p>\n"},"addOnWindowUnload":{"!type":"fn(obj?: ?, functionName?: ?)","!doc":"<p>registers a function to be triggered when window.onunload\nfires.</p>\n"}},"dojo/_base/xhr":{"!doc":"<p>Deprecated.   Use dojo/request instead.</p>\n","contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n"},"prototype":{"constructor":{"!type":"fn(method: ?, args: ?, hasBody?: ?) -> ?","!doc":"<p>Deprecated.   Use dojo/request instead.</p>\n"}},"del":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP DELETE request to the server.</p>\n"},"fieldToObject":{"!type":"fn(inputNode: ?) -> ?","!doc":"<p>Serialize a form field to a JavaScript object.</p>\n"},"formToJson":{"!type":"fn(formNode: ?, prettyPrint?: ?) -> string","!doc":"<p>Create a serialized JSON string from a form node or string\nID identifying the form to serialize</p>\n"},"formToObject":{"!type":"fn(formNode: ?) -> ?","!doc":"<p>Serialize a form node to a JavaScript object.</p>\n"},"formToQuery":{"!type":"fn(formNode: ?) -> string","!doc":"<p>Returns a URL-encoded string representing the form passed as either a\nnode or string ID identifying the form to serialize</p>\n"},"get":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP GET request to the server.</p>\n"},"objectToQuery":{"!type":"fn(map: ?) -> ?","!doc":"<p>takes a name/value mapping object and returns a string representing\na URL-encoded version of that object.</p>\n"},"post":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP POST request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"},"put":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP PUT request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"},"queryToObject":{"!type":"fn(str: ?) -> ?","!doc":"<p>Create an object representing a de-serialized query section of a\nURL. Query keys with multiple values are returned in an array.\n</p>\n"}},"dojo/_base/xhr.contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n","auto":{"!type":"fn(xhr: ?)"},"javascript":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which evaluates the response data, expecting it to be valid JavaScript</p>\n"},"json":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which returns a JavaScript object created from the response data</p>\n"},"json-comment-filtered":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which expects comment-filtered JSON.</p>\n"},"json-comment-optional":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which checks the presence of comment-filtered JSON and\nalternates between the <code>json</code> and <code>json-comment-filtered</code> contentHandlers.</p>\n"},"olson-zoneinfo":{"!type":"fn(xhr: ?)"},"text":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which simply returns the plaintext response data</p>\n"},"xml":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler returning an XML Document parsed from the response data</p>\n"}},"dojo/io-query":{"!doc":"<p>This module defines query string processing functions.</p>\n","objectToQuery":{"!type":"fn(map: ?) -> ?","!doc":"<p>takes a name/value mapping object and returns a string representing\na URL-encoded version of that object.</p>\n"},"queryToObject":{"!type":"fn(str: ?) -> ?","!doc":"<p>Create an object representing a de-serialized query section of a\nURL. Query keys with multiple values are returned in an array.\n</p>\n"}},"dojo/dom-form":{"!doc":"<p>This module defines form-processing functions.</p>\n","fieldToObject":{"!type":"fn(inputNode: ?) -> ?","!doc":"<p>Serialize a form field to a JavaScript object.</p>\n"},"toJson":{"!type":"fn(formNode: ?, prettyPrint?: ?) -> string","!doc":"<p>Create a serialized JSON string from a form node or string\nID identifying the form to serialize</p>\n"},"toObject":{"!type":"fn(formNode: ?) -> ?","!doc":"<p>Serialize a form node to a JavaScript object.</p>\n"},"toQuery":{"!type":"fn(formNode: ?) -> string","!doc":"<p>Returns a URL-encoded string representing the form passed as either a\nnode or string ID identifying the form to serialize</p>\n"}},"dojo/json":{"!doc":"<p>Functions to parse and serialize JSON</p>\n","parse":{"!type":"fn(str: ?, strict: ?)","!doc":"<p>Parses a <a href=\"http://json.org\">JSON</a> string to return a JavaScript object.</p>\n"},"stringify":{"!type":"fn(value: ?, replacer: ?, spacer: ?)","!doc":"<p>Returns a <a href=\"http://json.org\">JSON</a> serialization of an object.</p>\n"}},"dojo/_base/json":{"!doc":"<p>This module defines the dojo JSON API.</p>\n"},"dojo/request/watch":{"!doc":"<p>Watches the io request represented by dfd to see if it completes.</p>\n","prototype":{"ioCheck":{"!doc":"<p>Function used to check if basic IO call worked. Gets the dfd\nobject as its only argument.</p>\n"},"resHandle":{"!doc":"<p>Function used to process response. Gets the dfd\nobject as its only argument.</p>\n"},"validCheck":{"!doc":"<p>Function used to check if the IO request is still valid. Gets the dfd\nobject as its only argument.</p>\n"},"constructor":{"!type":"fn(dfd: ?)","!doc":"<p>Watches the io request represented by dfd to see if it completes.</p>\n"}},"cancelAll":{"!type":"fn()","!doc":"<p>Cancels all pending IO requests, regardless of IO type</p>\n"}},"dojo/request/util":{"addCommonMethods":{"!type":"fn(provider: ?, methods: ?)"},"checkStatus":{"!type":"fn(stat: ?) -> bool"},"deepCopy":{"!type":"fn(target: ?, source: ?) -> ?"},"deepCreate":{"!type":"fn(source: ?, properties: ?) -> ?"},"deferred":{"!type":"fn(response: ?, cancel: ?, isValid: ?, isReady: ?, handleResponse: ?, last: ?) -> +instance"},"notify":{"!type":"fn(type?: ?, listener?: ?) -> +any","!doc":"<p>Register a listener to be notified when an event\nin dojo/request happens.</p>\n"},"parseArgs":{"!type":"fn(url: ?, options: ?, skipData: ?) -> ?"}},"dojo/errors/RequestError":{"!doc":"<p>TODOC</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>TODOC</p>\n"}}},"dojo/errors/RequestTimeoutError":{"!doc":"<p>TODOC</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>TODOC</p>\n"}}},"dojo/request/xhr":{"!doc":"<p>Sends a request using XMLHttpRequest with the given URL and options.</p>\n","prototype":{"constructor":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Sends a request using XMLHttpRequest with the given URL and options.</p>\n"}},"del":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP DELETE request using XMLHttpRequest with the given URL and options.</p>\n"},"get":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP GET request using XMLHttpRequest with the given URL and options.</p>\n"},"post":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP POST request using XMLHttpRequest with the given URL and options.</p>\n"},"put":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP PUT request using XMLHttpRequest with the given URL and options.</p>\n"}},"dojo/request/xhr.__BaseOptions":{"!type":"fn()","prototype":{"data":{"!doc":"<p>Data to transfer. This is ignored for GET and DELETE\nrequests.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"headers":{"!doc":"<p>Headers to use for the request.</p>\n"},"password":{"!doc":"<p>Password to use during the request.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"sync":{"!doc":"<p>Whether to make a synchronous request or not. Default\nis <code>false</code> (asynchronous).</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"},"user":{"!doc":"<p>Username to use during the request.</p>\n"},"withCredentials":{"!doc":"<p>For cross-site requests, whether to send credentials\nor not.</p>\n"}}},"dojo/request/xhr.__MethodOptions":{"!type":"fn()","prototype":{"method":{"!doc":"<p>The HTTP method to use to make the request. Must be\nuppercase. Default is <code>&quot;GET&quot;</code>.</p>\n"}}},"dojo/request/xhr.__Options":{"!type":"fn()","prototype":{"data":{"!doc":"<p>Data to transfer. This is ignored for GET and DELETE\nrequests.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"headers":{"!doc":"<p>Headers to use for the request.</p>\n"},"method":{"!doc":"<p>The HTTP method to use to make the request. Must be\nuppercase. Default is <code>&quot;GET&quot;</code>.</p>\n"},"password":{"!doc":"<p>Password to use during the request.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"sync":{"!doc":"<p>Whether to make a synchronous request or not. Default\nis <code>false</code> (asynchronous).</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"},"user":{"!doc":"<p>Username to use during the request.</p>\n"},"withCredentials":{"!doc":"<p>For cross-site requests, whether to send credentials\nor not.</p>\n"}}},"dojo/request/handlers":{"prototype":{"constructor":{"!type":"fn(response: ?) -> ?"}},"register":{"!type":"fn(name: ?, handler: ?)"}},"dojo/request":{"!doc":"<p>Send a request using the default transport for the current platform.</p>\n","prototype":{"constructor":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send a request using the default transport for the current platform.</p>\n"}},"del":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP DELETE request using the default transport for the current platform.</p>\n"},"get":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP GET request using the default transport for the current platform.</p>\n"},"post":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP POST request using the default transport for the current platform.</p>\n"},"put":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP POST request using the default transport for the current platform.</p>\n"}},"dojo/request.__Promise":{"!type":"fn()","prototype":{"!proto":"dojo/promise/Promise.prototype","response":{"!doc":"<p>A promise resolving to an object representing\nthe response from the server.</p>\n"},"always":{"!type":"fn(callbackOrErrback?: ?) -> +dojo","!doc":"<p>Add a callback to be invoked when the promise is resolved\nor rejected.</p>\n"},"cancel":{"!type":"fn(reason: ?, strict?: ?) -> +any","!doc":"<p>Inform the deferred it may cancel its asynchronous operation.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The public interface to a deferred.</p>\n"},"isCanceled":{"!type":"fn() -> bool","!doc":"<p>Checks whether the promise has been canceled.</p>\n"},"isFulfilled":{"!type":"fn() -> bool","!doc":"<p>Checks whether the promise has been resolved or rejected.</p>\n"},"isRejected":{"!type":"fn() -> bool","!doc":"<p>Checks whether the promise has been rejected.</p>\n"},"isResolved":{"!type":"fn() -> bool","!doc":"<p>Checks whether the promise has been resolved.</p>\n"},"otherwise":{"!type":"fn(errback?: ?) -> +dojo","!doc":"<p>Add new errbacks to the promise.</p>\n"},"then":{"!type":"fn(callback?: ?, errback?: ?, progback?: ?) -> +dojo","!doc":"<p>Add new callbacks to the promise.</p>\n"},"toString":{"!type":"fn() -> string"},"trace":{"!type":"fn() -> fn"},"traceRejected":{"!type":"fn() -> fn"}}},"dojo/request.__BaseOptions":{"!type":"fn()","prototype":{"data":{"!doc":"<p>Data to transfer.  This is ignored for GET and DELETE\nrequests.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"}}},"dojo/request.__MethodOptions":{"!type":"fn()","prototype":{"method":{"!doc":"<p>The HTTP method to use to make the request.  Must be\nuppercase.</p>\n"}}},"dojo/request.__Options":{"!type":"fn()","prototype":{"data":{"!doc":"<p>Data to transfer.  This is ignored for GET and DELETE\nrequests.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"method":{"!doc":"<p>The HTTP method to use to make the request.  Must be\nuppercase.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"}}},"dojo/_base/loader":{},"dojo/_base/query":{"!doc":"<p>Deprecated.   Use dojo/query instead.</p>\n"},"dojo/_firebug/firebug":{},"dojo/back":{"!doc":"<p>Browser history management resources</p>\n","addToHistory":{"!type":"fn(args: ?)","!doc":"<p>adds a state object (args) to the history list.</p>\n"},"getHash":{"!type":"fn() -> ?"},"goBack":{"!type":"fn()","!doc":"<p>private method. Do not call this directly.</p>\n"},"goForward":{"!type":"fn()","!doc":"<p>private method. Do not call this directly.</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initializes the undo stack. This must be called from a <script>\nblock that lives inside the <code>&lt;body&gt;</code> tag to prevent bugs on IE.\n\n</p>\n<p>Only call this method before the page&#39;s DOM is finished loading. Otherwise\nit will not work. Be careful with xdomain loading or djConfig.debugAtAllCosts scenarios,\nin order for this method to work, dojo/back will need to be part of a build layer.</p>\n"},"setHash":{"!type":"fn(h: ?)"},"setInitialState":{"!type":"fn(args: ?)","!doc":"<p>Sets the state object and back callback for the very first page\nthat is loaded.\n\n</p>\n<p>It is recommended that you call this method as part of an event\nlistener that is registered via dojo/ready.</p>\n"}},"dojo/behavior":{"!doc":"<p>Deprecated.   dojo/behavior&#39;s functionality can be achieved using event delegation using dojo/on\nand on.selector().</p>\n","_behaviors":{},"add":{"!type":"fn(behaviorObj: ?)","!doc":"<p>Add the specified behavior to the list of behaviors, ignoring existing\nmatches.</p>\n"},"apply":{"!type":"fn()","!doc":"<p>Applies all currently registered behaviors to the document.\n</p>\n"},"prototype":{"constructor":{"!type":"fn()","!doc":"<p>Deprecated.   dojo/behavior&#39;s functionality can be achieved using event delegation using dojo/on\nand on.selector().</p>\n"}}},"dojo/domReady":{"!doc":"<p>Plugin to delay require()/define() callback from firing until the DOM has finished loading.</p>\n","_Q":{},"prototype":{"constructor":{"!type":"fn(callback: ?)","!doc":"<p>Plugin to delay require()/define() callback from firing until the DOM has finished loading.</p>\n"}},"load":{"!type":"fn(id: ?, req: ?, load: ?)"}},"dojo/cache":{"!doc":"<p>A getter and setter for storing the string content associated with the\nmodule and url arguments.</p>\n","prototype":{"constructor":{"!type":"fn(module: ?, url: ?, value?: ?) -> ?","!doc":"<p>A getter and setter for storing the string content associated with the\nmodule and url arguments.</p>\n"}}},"dojo/text":{"!doc":"<p>This module implements the dojo/text! plugin and the dojo.cache API.</p>\n","dynamic":{},"load":{"!type":"fn(id: ?, require: ?, load: ?)"},"normalize":{"!type":"fn(id: ?, toAbsMid: ?) -> string"}},"dojo/cldr/monetary":{"!doc":"<p>TODOC</p>\n","getData":{"!type":"fn(code: ?) -> ?","!doc":"<p>A mapping of currency code to currency-specific formatting information. Returns a unique object with properties: places, round.</p>\n"}},"dojo/cldr/supplemental":{"!doc":"<p>TODOC</p>\n","getFirstDayOfWeek":{"!type":"fn(locale?: ?) -> number","!doc":"<p>Returns a zero-based index for first day of the week</p>\n"},"getWeekend":{"!type":"fn(locale?: ?) -> ?","!doc":"<p>Returns a hash containing the start and end days of the weekend</p>\n"}},"dojo/i18n":{"!doc":"<p>This module implements the dojo/i18n! plugin and the v1.6- i18n API</p>\n","cache":{},"dynamic":{},"unitTests":{},"getL10nName":{"!type":"fn(moduleName: ?, bundleName: ?, locale: ?) -> string"},"getLocalization":{"!type":"fn(moduleName: ?, bundleName: ?, locale: ?) -> ?"},"load":{"!type":"fn(id: ?, require: ?, load: ?)","!doc":"<p>id is in one of the following formats\n\n</p>\n<ol>\n<li><p><path>/nls/<bundle>\n =&gt; load the bundle, localized to config.locale; load all bundles localized to\n config.extraLocale (if any); return the loaded bundle localized to config.locale.</p>\n</li>\n<li><p><path>/nls/<locale>/<bundle>\n =&gt; load then return the bundle localized to <locale></p>\n</li>\n<li><p><em>preload</em><path>/nls/<module>*<JSON array of available locales>\n =&gt; for config.locale and all config.extraLocale, load all bundles found\n in the best-matching bundle rollup. A value of 1 is returned, which\n is meaningless other than to say the plugin is executing the requested\n preloads</p>\n</li>\n</ol>\n<p>In cases 1 and 2, <path> is always normalized to an absolute module id upon entry; see\nnormalize. In case 3, it <path> is assumed to be absolute; this is arranged by the builder.\n\n</p>\n<p>To load a bundle means to insert the bundle into the plugin&#39;s cache and publish the bundle\nvalue to the loader. Given <path>, <bundle>, and a particular <locale>, the cache key\n\n</p>\n<pre><code>&lt;path&gt;/nls/&lt;bundle&gt;/&lt;locale&gt;</code></pre>\n<p>will hold the value. Similarly, then plugin will publish this value to the loader by\n\n</p>\n<pre><code>define(&quot;&lt;path&gt;/nls/&lt;bundle&gt;/&lt;locale&gt;&quot;, &lt;bundle-value&gt;);</code></pre>\n<p>Given this algorithm, other machinery can provide fast load paths be preplacing\nvalues in the plugin&#39;s cache, which is public. When a load is demanded the\ncache is inspected before starting any loading. Explicitly placing values in the plugin\ncache is an advanced/experimental feature that should not be needed; use at your own risk.\n\n</p>\n<p>For the normal AMD algorithm, the root bundle is loaded first, which instructs the\nplugin what additional localized bundles are required for a particular locale. These\nadditional locales are loaded and a mix of the root and each progressively-specific\nlocale is returned. For example:\n\n</p>\n<ol>\n<li><p>The client demands &quot;dojo/i18n!some/path/nls/someBundle</p>\n</li>\n<li><p>The loader demands load(some/path/nls/someBundle)</p>\n</li>\n<li><p>This plugin require&#39;s &quot;some/path/nls/someBundle&quot;, which is the root bundle.</p>\n</li>\n<li><p>Assuming config.locale is &quot;ab-cd-ef&quot; and the root bundle indicates that localizations\nare available for &quot;ab&quot; and &quot;ab-cd-ef&quot; (note the missing &quot;ab-cd&quot;, then the plugin\nrequires &quot;some/path/nls/ab/someBundle&quot; and &quot;some/path/nls/ab-cd-ef/someBundle&quot;</p>\n</li>\n<li><p>Upon receiving all required bundles, the plugin constructs the value of the bundle\nab-cd-ef as...</p>\n<pre><code> mixin(mixin(mixin({}, require(&quot;some/path/nls/someBundle&quot;),\n     require(&quot;some/path/nls/ab/someBundle&quot;)),\n     require(&quot;some/path/nls/ab-cd-ef/someBundle&quot;));</code></pre>\n</li>\n</ol>\n<p>This value is inserted into the cache and published to the loader at the\nkey/module-id some/path/nls/someBundle/ab-cd-ef.\n\n</p>\n<p>The special preload signature (case 3) instructs the plugin to stop servicing all normal requests\n(further preload requests will be serviced) until all ongoing preloading has completed.\n\n</p>\n<p>The preload signature instructs the plugin that a special rollup module is available that contains\none or more flattened, localized bundles. The JSON array of available locales indicates which locales\nare available. Here is an example:\n\n</p>\n<pre><code>*preload*some/path/nls/someModule*[&quot;root&quot;, &quot;ab&quot;, &quot;ab-cd-ef&quot;]</code></pre>\n<p>This indicates the following rollup modules are available:\n\n</p>\n<pre><code>some/path/nls/someModule_ROOT\nsome/path/nls/someModule_ab\nsome/path/nls/someModule_ab-cd-ef</code></pre>\n<p>Each of these modules is a normal AMD module that contains one or more flattened bundles in a hash.\nFor example, assume someModule contained the bundles some/bundle/path/someBundle and\nsome/bundle/path/someOtherBundle, then some/path/nls/someModule_ab would be expressed as follows:\n\n</p>\n<pre><code>define({\n    some/bundle/path/someBundle:&lt;value of someBundle, flattened with respect to locale ab&gt;,\n    some/bundle/path/someOtherBundle:&lt;value of someOtherBundle, flattened with respect to locale ab&gt;,\n});</code></pre>\n<p>E.g., given this design, preloading for locale==&quot;ab&quot; can execute the following algorithm:\n\n</p>\n<pre><code>require([&quot;some/path/nls/someModule_ab&quot;], function(rollup){\n    for(var p in rollup){\n        var id = p + &quot;/ab&quot;,\n        cache[id] = rollup[p];\n        define(id, rollup[p]);\n    }\n});</code></pre>\n<p>Similarly, if &quot;ab-cd&quot; is requested, the algorithm can determine that &quot;ab&quot; is the best available and\nload accordingly.\n\n</p>\n<p>The builder will write such rollups for every layer if a non-empty localeList  profile property is\nprovided. Further, the builder will include the following cache entry in the cache associated with\nany layer.\n\n</p>\n<pre><code>&quot;*now&quot;:function(r){r([&#39;dojo/i18n!*preload*&lt;path&gt;/nls/&lt;module&gt;*&lt;JSON array of available locales&gt;&#39;]);}</code></pre>\n<p>The *now special cache module instructs the loader to apply the provided function to context-require\nwith respect to the particular layer being defined. This causes the plugin to hold all normal service\nrequests until all preloading is complete.\n\n</p>\n<p>Notice that this algorithm is rarely better than the standard AMD load algorithm. Consider the normal case\nwhere the target locale has a single segment and a layer depends on a single bundle:\n\n</p>\n<p>Without Preloads:\n\n</p>\n<ol>\n<li>Layer loads root bundle.</li>\n<li>bundle is demanded; plugin loads single localized bundle.</li>\n</ol>\n<p>With Preloads:\n\n</p>\n<ol>\n<li>Layer causes preloading of target bundle.</li>\n<li>bundle is demanded; service is delayed until preloading complete; bundle is returned.</li>\n</ol>\n<p>In each case a single transaction is required to load the target bundle. In cases where multiple bundles\nare required and/or the locale has multiple segments, preloads still requires a single transaction whereas\nthe normal path requires an additional transaction for each additional bundle/locale-segment. However all\nof these additional transactions can be done concurrently. Owing to this analysis, the entire preloading\nalgorithm can be discard during a build by setting the has feature dojo-preload-i18n-Api to false.</p>\n"},"normalize":{"!type":"fn(id: ?, toAbsMid: ?) -> ?","!doc":"<p>id may be relative.\npreload has form <code>*preload*&lt;path&gt;/nls/&lt;module&gt;*&lt;flattened locales&gt;</code> and\ntherefore never looks like a relative</p>\n"},"normalizeLocale":{"!type":"fn(locale: ?) -> ?"}},"dojo/i18n.cache":{},"dojo/colors":{"!doc":"<p>Color utilities, extending Base dojo.Color</p>\n","ThreeD":{}},"dojo/cookie":{"!doc":"<p>Get or set a cookie.</p>\n","prototype":{"constructor":{"!type":"fn(name: ?, value?: ?, props?: ?) -> ?","!doc":"<p>Get or set a cookie.</p>\n"}},"isSupported":{"!type":"fn() -> bool","!doc":"<p>Use to determine if the current browser supports cookies or not.\n\n</p>\n<p>Returns true if user allows cookies.\nReturns false if user doesn&#39;t allow cookies.</p>\n"}},"dojo/regexp":{"!doc":"<p>Regular expressions and Builder resources</p>\n","buildGroupRE":{"!type":"fn(arr: ?, re: ?, nonCapture?: ?) -> ?","!doc":"<p>Builds a regular expression that groups subexpressions</p>\n"},"escapeString":{"!type":"fn(str: ?, except?: ?) -> ?","!doc":"<p>Adds escape sequences for special characters in regular expressions</p>\n"},"group":{"!type":"fn(expression: ?, nonCapture?: ?) -> string","!doc":"<p>adds group match to expression</p>\n"}},"dojo/currency":{"!doc":"<p>localized formatting and parsing routines for currencies</p>\n","format":{"!type":"fn(value: ?, options?: ?) -> ?","!doc":"<p>Format a Number as a currency, using locale-specific settings\n</p>\n"},"parse":{"!type":"fn(expression: ?, options?: ?) -> ?"},"regexp":{"!type":"fn(options?: ?) -> ?"}},"dojo/currency.__FormatOptions":{"!type":"fn()","prototype":{"currency":{"!doc":"<p>an <a href=\"http://en.wikipedia.org/wiki/ISO_4217\">ISO4217</a> currency code, a three letter sequence like &quot;USD&quot;.\nFor use with dojo.currency only.</p>\n"},"fractional":{"!doc":"<p>If false, show no decimal places, overriding places and pattern settings.</p>\n"},"locale":{"!doc":"<p>override the locale used to determine formatting rules</p>\n"},"pattern":{"!doc":"<p>override <a href=\"http://www.unicode.org/reports/tr35/#Number_Format_Patterns\">formatting pattern</a>\nwith this string.  Default value is based on locale.  Overriding this property will defeat\nlocalization.  Literal characters in patterns are not supported.</p>\n"},"places":{"!doc":"<p>number of decimal places to show.  Default is defined based on which currency is used.</p>\n"},"round":{"!doc":"<p>5 rounds to nearest .5; 0 rounds to nearest whole (default). -1\nmeans do not round.</p>\n"},"symbol":{"!doc":"<p>localized currency symbol. The default will be looked up in table of supported currencies in <code>dojo.cldr</code>\nA <a href=\"http://en.wikipedia.org/wiki/ISO_4217\">ISO4217</a> currency code will be used if not found.</p>\n"},"type":{"!doc":"<p>Should not be set.  Value is assumed to be &quot;currency&quot;.</p>\n"}}},"dojo/currency.__ParseOptions":{"!type":"fn()","prototype":{"currency":{"!doc":"<p>an <a href=\"http://en.wikipedia.org/wiki/ISO_4217\">ISO4217</a> currency code, a three letter sequence like &quot;USD&quot;.\nFor use with dojo.currency only.</p>\n"},"fractional":{"!doc":"<p>Whether to include the fractional portion, where the number of decimal places are implied by the currency\nor explicit &#39;places&#39; parameter.  The value [true,false] makes the fractional portion optional.\nBy default for currencies, it the fractional portion is optional.</p>\n"},"locale":{"!doc":"<p>override the locale used to determine formatting rules</p>\n"},"pattern":{"!doc":"<p>override <a href=\"http://www.unicode.org/reports/tr35/#Number_Format_Patterns\">formatting pattern</a>\nwith this string.  Default value is based on locale.  Overriding this property will defeat\nlocalization.  Literal characters in patterns are not supported.</p>\n"},"places":{"!doc":"<p>fixed number of decimal places to accept.  The default is determined based on which currency is used.</p>\n"},"strict":{"!doc":"<p>strict parsing, false by default.  Strict parsing requires input as produced by the format() method.\nNon-strict is more permissive, e.g. flexible on white space, omitting thousands separators</p>\n"},"symbol":{"!doc":"<p>localized currency symbol. The default will be looked up in table of supported currencies in <code>dojo.cldr</code>\nA <a href=\"http://en.wikipedia.org/wiki/ISO_4217\">ISO4217</a> currency code will be used if not found.</p>\n"},"type":{"!doc":"<p>Should not be set.  Value is assumed to be currency.</p>\n"}}},"dojo/number":{"!doc":"<p>localized formatting and parsing routines for Number</p>\n","_numberPatternRE":{},"format":{"!type":"fn(value: ?, options?: ?) -> +null","!doc":"<p>Format a Number as a String, using locale-specific settings</p>\n"},"parse":{"!type":"fn(expression: ?, options?: ?) -> number","!doc":"<p>Convert a properly formatted string to a primitive Number, using\nlocale-specific settings.</p>\n"},"regexp":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a number</p>\n"},"round":{"!type":"fn(value: ?, places?: ?, increment?: ?) -> number","!doc":"<p>Rounds to the nearest value with the given number of decimal places, away from zero</p>\n"}},"dojo/number.__FormatOptions":{"!type":"fn()","prototype":{"fractional":{"!doc":"<p>If false, show no decimal places, overriding places and pattern settings.</p>\n"},"locale":{"!doc":"<p>override the locale used to determine formatting rules</p>\n"},"pattern":{"!doc":"<p>override <a href=\"http://www.unicode.org/reports/tr35/#Number_Format_Patterns\">formatting pattern</a>\nwith this string.  Default value is based on locale.  Overriding this property will defeat\nlocalization.  Literal characters in patterns are not supported.</p>\n"},"places":{"!doc":"<p>fixed number of decimal places to show.  This overrides any\ninformation in the provided pattern.</p>\n"},"round":{"!doc":"<p>5 rounds to nearest .5; 0 rounds to nearest whole (default). -1\nmeans do not round.</p>\n"},"type":{"!doc":"<p>choose a format type based on the locale from the following:\ndecimal, scientific (not yet supported), percent, currency. decimal by default.</p>\n"}}},"dojo/number.__FormatAbsoluteOptions":{"!type":"fn()","prototype":{"decimal":{"!doc":"<p>the decimal separator</p>\n"},"group":{"!doc":"<p>the group separator</p>\n"},"places":{"!doc":"<p>number of decimal places.  the range &quot;n,m&quot; will format to m places.</p>\n"},"round":{"!doc":"<p>5 rounds to nearest .5; 0 rounds to nearest whole (default). -1\nmeans don&#39;t round.</p>\n"}}},"dojo/number.__RegexpOptions":{"!type":"fn()","prototype":{"locale":{"!doc":"<p>override the locale used to determine formatting rules</p>\n"},"pattern":{"!doc":"<p>override <a href=\"http://www.unicode.org/reports/tr35/#Number_Format_Patterns\">formatting pattern</a>\nwith this string.  Default value is based on locale.  Overriding this property will defeat\nlocalization.</p>\n"},"places":{"!doc":"<p>number of decimal places to accept: Infinity, a positive number, or\na range &quot;n,m&quot;.  Defined by pattern or Infinity if pattern not provided.</p>\n"},"strict":{"!doc":"<p>strict parsing, false by default.  Strict parsing requires input as produced by the format() method.\nNon-strict is more permissive, e.g. flexible on white space, omitting thousands separators</p>\n"},"type":{"!doc":"<p>choose a format type based on the locale from the following:\ndecimal, scientific (not yet supported), percent, currency. decimal by default.</p>\n"}}},"dojo/number.__ParseOptions":{"!type":"fn()","prototype":{"fractional":{"!doc":"<p>Whether to include the fractional portion, where the number of decimal places are implied by pattern\nor explicit &#39;places&#39; parameter.  The value [true,false] makes the fractional portion optional.</p>\n"},"locale":{"!doc":"<p>override the locale used to determine formatting rules</p>\n"},"pattern":{"!doc":"<p>override <a href=\"http://www.unicode.org/reports/tr35/#Number_Format_Patterns\">formatting pattern</a>\nwith this string.  Default value is based on locale.  Overriding this property will defeat\nlocalization.  Literal characters in patterns are not supported.</p>\n"},"strict":{"!doc":"<p>strict parsing, false by default.  Strict parsing requires input as produced by the format() method.\nNon-strict is more permissive, e.g. flexible on white space, omitting thousands separators</p>\n"},"type":{"!doc":"<p>choose a format type based on the locale from the following:\ndecimal, scientific (not yet supported), percent, currency. decimal by default.</p>\n"}}},"dojo/number.__RealNumberRegexpFlags":{"!type":"fn()","prototype":{"decimal":{"!doc":"<p>A string for the character used as the decimal point.  Default\nis &quot;.&quot;.</p>\n"},"eSigned":{"!doc":"<p>The leading plus-or-minus sign on the exponent.  Can be true,\nfalse, or [true, false].  Default is [true, false], (i.e. will\nmatch if it is signed or unsigned).  flags in regexp.integer can be\napplied.</p>\n"},"exponent":{"!doc":"<p>Express in exponential notation.  Can be true, false, or [true,\nfalse]. Default is [true, false], (i.e. will match if the\nexponential part is present are not).</p>\n"},"fractional":{"!doc":"<p>Whether decimal places are used.  Can be true, false, or [true,\nfalse].  Default is [true, false] which means optional.</p>\n"},"places":{"!doc":"<p>The integer number of decimal places or a range given as &quot;n,m&quot;.  If\nnot given, the decimal part is optional and the number of places is\nunlimited.</p>\n"}}},"dojo/number.__IntegerRegexpFlags":{"!type":"fn()","prototype":{"groupSize":{"!doc":"<p>group size between separators</p>\n"},"groupSize2":{"!doc":"<p>second grouping, where separators 2..n have a different interval than the first separator (for India)</p>\n"},"separator":{"!doc":"<p>The character used as the thousands separator. Default is no\nseparator. For more than one symbol use an array, e.g. <code>[&quot;,&quot;, &quot;&quot;]</code>,\nmakes &#39;,&#39; optional.</p>\n"},"signed":{"!doc":"<p>The leading plus-or-minus sign. Can be true, false, or <code>[true,false]</code>.\nDefault is <code>[true, false]</code>, (i.e. will match if it is signed\nor unsigned).</p>\n"}}},"dojo/string":{"!doc":"<p>String utilities for Dojo</p>\n","pad":{"!type":"fn(text: ?, size: ?, ch?: ?, end?: ?) -> number","!doc":"<p>Pad a string to guarantee that it is at least <code>size</code> length by\nfilling with the character <code>ch</code> at either the start or end of the\nstring. Pads at the start, by default.</p>\n"},"rep":{"!type":"fn(str: ?, num: ?) -> string","!doc":"<p>Efficiently replicate a string <code>n</code> times.</p>\n"},"substitute":{"!type":"fn(template: ?, map: ?, transform?: ?, thisObject?: ?) -> ?","!doc":"<p>Performs parameterized substitutions on a string. Throws an\nexception if any parameter is unmatched.</p>\n"},"trim":{"!type":"fn(str: ?) -> string","!doc":"<p>Trims whitespace from both sides of the string</p>\n"}},"dojo/data/ItemFileReadStore":{"!type":"fn()","prototype":{"!proto":"dojo/Evented.prototype","!doc":"<p>The ItemFileReadStore implements the dojo/data/api/Read API and reads\ndata from JSON files that have contents in this format --\n\n</p>\n<pre><code>{ items: [\n    { name:&#39;Kermit&#39;, color:&#39;green&#39;, age:12, friends:[&#39;Gonzo&#39;, {_reference:{name:&#39;Fozzie Bear&#39;}}]},\n    { name:&#39;Fozzie Bear&#39;, wears:[&#39;hat&#39;, &#39;tie&#39;]},\n    { name:&#39;Miss Piggy&#39;, pets:&#39;Foo-Foo&#39;}\n]}</code></pre>\n<p>Note that it can also contain an &#39;identifier&#39; property that specified which attribute on the items\n\n</p>\n<p>in the array of items that acts as the unique identifier for that item.</p>\n","_ccUrl":{},"clearOnClose":{"!doc":"<p>Parameter to allow users to specify if a close call should force a reload or not.\nBy default, it retains the old behavior of not clearing if close is called.  But\nif set true, the store will be reset to default state.  Note that by doing this,\nall item handles will become invalid and a new fetch must be issued.</p>\n"},"data":{},"failOk":{"!doc":"<p>Parameter for specifying that it is OK for the xhrGet call to fail silently.</p>\n"},"hierarchical":{"!doc":"<p>Parameter to indicate to process data from the url as hierarchical\n(data items can contain other data items in js form).  Default is true\nfor backwards compatibility.  False means only root items are processed\nas items, all child objects outside of type-mapped objects and those in\nspecific reference format, are left straight JS data objects.</p>\n"},"typeMap":{},"url":{},"urlPreventCache":{"!doc":"<p>Parameter to allow specifying if preventCache should be passed to the xhrGet call or not when loading data from a url.\nNote this does not mean the store calls the server on each fetch, only that the data load has preventCache set as an option.\nAdded for tracker: #6072</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(keywordParameters: ?)","!doc":"<p>constructor</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"filter":{"!type":"fn(requestArgs: ?, arrayOfItems: ?, findCallback: ?)","!doc":"<p>This method handles the basic filtering needs for ItemFile* based stores.</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Identity.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +value","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojo/data/util/filter":{"patternToRegExp":{"!type":"fn(pattern: ?, ignoreCase?: ?) -> +instance","!doc":"<p>Helper function to convert a simple pattern to a regular expression for matching.</p>\n"}},"dojo/data/util/simpleFetch":{"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"}},"dojo/data/util/sorter":{"basicComparator":{"!type":"fn(a: ?, b: ?) -> number","!doc":"<p>Basic comparison function that compares if an item is greater or less than another item</p>\n"},"createSortFunction":{"!type":"fn(sortSpec: ?, store: ?)","!doc":"<p>Helper function to generate the sorting function based off the list of sort attributes.</p>\n"}},"dojo/data/ItemFileWriteStore":{"!type":"fn()","prototype":{"!proto":"dojo/data/ItemFileReadStore.prototype","!doc":"<p>TODOC</p>\n","_ccUrl":{},"clearOnClose":{"!doc":"<p>Parameter to allow users to specify if a close call should force a reload or not.\nBy default, it retains the old behavior of not clearing if close is called.  But\nif set true, the store will be reset to default state.  Note that by doing this,\nall item handles will become invalid and a new fetch must be issued.</p>\n"},"data":{},"failOk":{"!doc":"<p>Parameter for specifying that it is OK for the xhrGet call to fail silently.</p>\n"},"hierarchical":{"!doc":"<p>Parameter to indicate to process data from the url as hierarchical\n(data items can contain other data items in js form).  Default is true\nfor backwards compatibility.  False means only root items are processed\nas items, all child objects outside of type-mapped objects and those in\nspecific reference format, are left straight JS data objects.</p>\n"},"referenceIntegrity":{},"typeMap":{},"url":{},"urlPreventCache":{"!doc":"<p>Parameter to allow specifying if preventCache should be passed to the xhrGet call or not when loading data from a url.\nNote this does not mean the store calls the server on each fetch, only that the data load has preventCache set as an option.\nAdded for tracker: #6072</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>Over-ride of base close function of ItemFileReadStore to add in check for store state.</p>\n"},"constructor":{"!type":"fn(keywordParameters: ?)"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"deleteItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Write.deleteItem()</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"filter":{"!type":"fn(requestArgs: ?, arrayOfItems: ?, findCallback: ?)","!doc":"<p>This method handles the basic filtering needs for ItemFile* based stores.</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Identity.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +value","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isDirty":{"!type":"fn(item?: ?) -> ?","!doc":"<p>See dojo/data/api/Write.isDirty()</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"},"newItem":{"!type":"fn(keywordArgs?: ?, parentInfo?: ?) -> ?","!doc":"<p>See dojo/data/api/Write.newItem()</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"revert":{"!type":"fn() -> bool","!doc":"<p>See dojo/data/api/Write.revert()</p>\n"},"save":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Write.save()</p>\n"},"setValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Write.set()</p>\n"},"setValues":{"!type":"fn(item: ?, attribute: ?, values: ?) -> ?","!doc":"<p>See dojo/data/api/Write.setValues()</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Write.unsetAttribute()</p>\n"}}},"dojo/data/ObjectStore":{"!type":"fn()","prototype":{"!proto":"dojo/Evented.prototype","!doc":"<p>A Dojo Data implementation that wraps Dojo object stores for backwards\ncompatibility.</p>\n","labelProperty":{},"objectStore":{},"changing":{"!type":"fn(object: ?, _deleting: ?)","!doc":"<p>adds an object to the list of dirty objects.  This object\ncontains a reference to the object itself as well as a\ncloned and trimmed version of old object for use with\nrevert.</p>\n"},"close":{"!type":"fn(request: ?) -> ?","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(options: ?)"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>Checks to see if &#39;item&#39; has &#39;value&#39; at &#39;attribute&#39;</p>\n"},"deleteItem":{"!type":"fn(item: ?)","!doc":"<p>deletes item and any references to that item from the store.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"fetch":{"!type":"fn(args: ?) -> ?","!doc":"<p>See dojo/data/api/Read.fetch()</p>\n"},"fetchItemByIdentity":{"!type":"fn(args: ?) -> ?","!doc":"<p>fetch an item by its identity, by looking in our index of what we have loaded</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>Gets the available attributes of an item&#39;s &#39;property&#39; and returns\nit as an array.</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>return the store feature set</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>returns the identity of the given item\nSee dojo/data/api/Read.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns the attributes which are used to make up the\nidentity of an item.    Basically returns this.objectStore.idProperty\nSee dojo/data/api/Read.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, property: ?, defaultValue?: ?) -> +value","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39;</p>\n"},"getValues":{"!type":"fn(item: ?, property: ?) -> []","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39; and returns\nit. If this value is an array it is just returned,\nif not, the value is added to an array and that is returned.</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>Checks to see if item has attribute</p>\n"},"isDirty":{"!type":"fn(item: ?) -> bool","!doc":"<p>returns true if the item is marked as dirty or true if there are any dirty items</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>Checks to see if the argument is an item</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>Checks to see if the item is loaded.</p>\n"},"loadItem":{"!type":"fn(args: ?) -> ?","!doc":"<p>Loads an item and calls the callback handler. Note, that this will call the callback\nhandler even if the item is loaded. Consequently, you can use loadItem to ensure\nthat an item is loaded is situations when the item may or may not be loaded yet.\nIf you access a value directly through property access, you can use this to load\na lazy value as well (doesn&#39;t need to be an item).</p>\n"},"newItem":{"!type":"fn(data: ?, parentInfo: ?) -> ?","!doc":"<p>adds a new item to the store at the specified point.\nTakes two parameters, data, and options.</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"revert":{"!type":"fn()","!doc":"<p>returns any modified data to its original state prior to a save();</p>\n"},"save":{"!type":"fn(kwArgs: ?)","!doc":"<p>Saves the dirty data using object store provider. See dojo/data/api/Write for API.</p>\n"},"setValue":{"!type":"fn(item: ?, attribute: ?, value: ?)","!doc":"<p>sets &#39;attribute&#39; on &#39;item&#39; to &#39;value&#39;\nSee dojo/data/api/Write.setValue()</p>\n"},"setValues":{"!type":"fn(item: ?, attribute: ?, values: ?)","!doc":"<p>sets &#39;attribute&#39; on &#39;item&#39; to &#39;value&#39; value\nmust be an array.\nSee dojo/data/api/Write.setValues()</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, attribute: ?)","!doc":"<p>unsets &#39;attribute&#39; on &#39;item&#39;\nSee dojo/data/api/Write.unsetAttribute()</p>\n"}}},"dojo/data/api/Identity":{"!type":"fn()","prototype":{"!proto":"dojo/data/api/Read.prototype","!doc":"<p>This is an abstract API that data provider implementations conform to.\nThis file defines methods signatures and intentionally leaves all the\nmethods unimplemented.</p>\n","close":{"!type":"fn(request?: ?)","!doc":"<p>The close() method is intended for instructing the store to &#39;close&#39; out\nany information associated with a particular request.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?)","!doc":"<p>Returns true if the given <em>value</em> is one of the values that getValues()\nwould return.</p>\n"},"fetch":{"!type":"fn(keywordArgs: ?) -> +any","!doc":"<p>Given a query and set of defined options, such as a start and count of items to return,\nthis method executes the query and makes the results available as data items.\nThe format and expectations of stores is that they operate in a generally asynchronous\nmanner, therefore callbacks are always used to return items located by the fetch parameters.</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Given the identity of an item, this method returns the item that has\nthat identity through the onItem callback.  Conforming implementations\nshould return null if there is no item with the given identity.\nImplementations of fetchItemByIdentity() may sometimes return an item\nfrom a local cache and may sometimes fetch an item from a remote server,</p>\n"},"getAttributes":{"!type":"fn(item: ?)","!doc":"<p>Returns an array with all the attributes that this item has.  This\nmethod will always return an array; if the item has no attributes\nat all, getAttributes() will return an empty array: [].</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?)","!doc":"<p>Returns a unique identifier for an item.  The return value will be\neither a string or something that has a toString() method (such as,\nfor example, a dojox/uuid object).</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?)","!doc":"<p>Returns an array of attribute names that are used to generate the identity.\nFor most stores, this is a single attribute, but for some complex stores\nsuch as RDB backed stores that use compound (multi-attribute) identifiers\nit can be more than one.  If the identity is not composed of attributes\non the item, it will return null.  This function is intended to identify\nthe attributes that comprise the identity so that so that during a render\nof all attributes, the UI can hide the the identity information if it\nchooses.</p>\n"},"getLabel":{"!type":"fn(item: ?) -> +any","!doc":"<p>Method to inspect the item and return a user-readable &#39;label&#39; for the item\nthat provides a general/adequate description of what the item is.</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> +any","!doc":"<p>Method to inspect the item and return an array of what attributes of the item were used\nto generate its label, if any.</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +any","!doc":"<p>Returns a single attribute value.\nReturns defaultValue if and only if <em>item</em> does not have a value for <em>attribute</em>.\nReturns null if and only if null was explicitly set as the attribute value.\nReturns undefined if and only if the item does not have a value for the\ngiven attribute (which is the same as saying the item does not have the attribute).</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> +any","!doc":"<p>This getValues() method works just like the getValue() method, but getValues()\nalways returns an array rather than a single attribute value.  The array\nmay be empty, may contain a single attribute value, or may contain\nmany attribute values.\nIf the item does not have a value for the given attribute, then getValues()\nwill return an empty array: [].  (So, if store.hasAttribute(item, attribute)\nhas a return of false, then store.getValues(item, attribute) will return [].)</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?)","!doc":"<p>Returns true if the given <em>item</em> has a value for the given <em>attribute</em>.</p>\n"},"isItem":{"!type":"fn(something: ?)","!doc":"<p>Returns true if <em>something</em> is an item and came from the store instance.\nReturns false if <em>something</em> is a literal, an item from another store instance,\nor is any object other than an item.</p>\n"},"isItemLoaded":{"!type":"fn(something: ?)","!doc":"<p>Returns false if isItem(something) is false.  Returns false if\nif isItem(something) is true but the the item is not yet loaded\nin local memory (for example, if the item has not yet been read\nfrom the server).</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Given an item, this method loads the item so that a subsequent call\nto store.isItemLoaded(item) will return true.  If a call to\nisItemLoaded() returns true before loadItem() is even called,\nthen loadItem() need not do any work at all and will not even invoke\nthe callback handlers.  So, before invoking this method, check that\nthe item has not already been loaded.</p>\n"}}},"dojo/data/api/Read":{"!type":"fn()","prototype":{"!doc":"<p>This is an abstract API that data provider implementations conform to.\nThis file defines methods signatures and intentionally leaves all the\nmethods unimplemented.  For more information on the dojo.data APIs,\nplease visit: <a href=\"http://www.dojotoolkit.org/node/98\">http://www.dojotoolkit.org/node/98</a></p>\n","close":{"!type":"fn(request?: ?)","!doc":"<p>The close() method is intended for instructing the store to &#39;close&#39; out\nany information associated with a particular request.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?)","!doc":"<p>Returns true if the given <em>value</em> is one of the values that getValues()\nwould return.</p>\n"},"fetch":{"!type":"fn(keywordArgs: ?) -> +any","!doc":"<p>Given a query and set of defined options, such as a start and count of items to return,\nthis method executes the query and makes the results available as data items.\nThe format and expectations of stores is that they operate in a generally asynchronous\nmanner, therefore callbacks are always used to return items located by the fetch parameters.</p>\n"},"getAttributes":{"!type":"fn(item: ?)","!doc":"<p>Returns an array with all the attributes that this item has.  This\nmethod will always return an array; if the item has no attributes\nat all, getAttributes() will return an empty array: [].</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>The getFeatures() method returns an simple keyword values object\nthat specifies what interface features the datastore implements.\nA simple CsvStore may be read-only, and the only feature it\nimplements will be the &#39;dojo/data/api/Read&#39; interface, so the\ngetFeatures() method will return an object like this one:\n{&#39;dojo.data.api.Read&#39;: true}.\nA more sophisticated datastore might implement a variety of\ninterface features, like &#39;dojo.data.api.Read&#39;, &#39;dojo/data/api/Write&#39;,\n&#39;dojo.data.api.Identity&#39;, and &#39;dojo/data/api/Attribution&#39;.</p>\n"},"getLabel":{"!type":"fn(item: ?) -> +any","!doc":"<p>Method to inspect the item and return a user-readable &#39;label&#39; for the item\nthat provides a general/adequate description of what the item is.</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> +any","!doc":"<p>Method to inspect the item and return an array of what attributes of the item were used\nto generate its label, if any.</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +any","!doc":"<p>Returns a single attribute value.\nReturns defaultValue if and only if <em>item</em> does not have a value for <em>attribute</em>.\nReturns null if and only if null was explicitly set as the attribute value.\nReturns undefined if and only if the item does not have a value for the\ngiven attribute (which is the same as saying the item does not have the attribute).</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> +any","!doc":"<p>This getValues() method works just like the getValue() method, but getValues()\nalways returns an array rather than a single attribute value.  The array\nmay be empty, may contain a single attribute value, or may contain\nmany attribute values.\nIf the item does not have a value for the given attribute, then getValues()\nwill return an empty array: [].  (So, if store.hasAttribute(item, attribute)\nhas a return of false, then store.getValues(item, attribute) will return [].)</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?)","!doc":"<p>Returns true if the given <em>item</em> has a value for the given <em>attribute</em>.</p>\n"},"isItem":{"!type":"fn(something: ?)","!doc":"<p>Returns true if <em>something</em> is an item and came from the store instance.\nReturns false if <em>something</em> is a literal, an item from another store instance,\nor is any object other than an item.</p>\n"},"isItemLoaded":{"!type":"fn(something: ?)","!doc":"<p>Returns false if isItem(something) is false.  Returns false if\nif isItem(something) is true but the the item is not yet loaded\nin local memory (for example, if the item has not yet been read\nfrom the server).</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Given an item, this method loads the item so that a subsequent call\nto store.isItemLoaded(item) will return true.  If a call to\nisItemLoaded() returns true before loadItem() is even called,\nthen loadItem() need not do any work at all and will not even invoke\nthe callback handlers.  So, before invoking this method, check that\nthe item has not already been loaded.</p>\n"}}},"dojo/data/api/Item":{"!type":"fn()","prototype":{"!doc":"<p>An item in a dojo/data store\nClass for documentation purposes only. An item can take any form, so no \nproperties or methods are defined here.</p>\n"}},"dojo/data/api/Notification":{"!type":"fn()","prototype":{"!proto":"dojo/data/api/Read.prototype","!doc":"<p>This is an abstract API that data provider implementations conform to.\nThis file defines functions signatures and intentionally leaves all the\nfunctions unimplemented.</p>\n","close":{"!type":"fn(request?: ?)","!doc":"<p>The close() method is intended for instructing the store to &#39;close&#39; out\nany information associated with a particular request.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?)","!doc":"<p>Returns true if the given <em>value</em> is one of the values that getValues()\nwould return.</p>\n"},"fetch":{"!type":"fn(keywordArgs: ?) -> +any","!doc":"<p>Given a query and set of defined options, such as a start and count of items to return,\nthis method executes the query and makes the results available as data items.\nThe format and expectations of stores is that they operate in a generally asynchronous\nmanner, therefore callbacks are always used to return items located by the fetch parameters.</p>\n"},"getAttributes":{"!type":"fn(item: ?)","!doc":"<p>Returns an array with all the attributes that this item has.  This\nmethod will always return an array; if the item has no attributes\nat all, getAttributes() will return an empty array: [].</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> +any","!doc":"<p>Method to inspect the item and return a user-readable &#39;label&#39; for the item\nthat provides a general/adequate description of what the item is.</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> +any","!doc":"<p>Method to inspect the item and return an array of what attributes of the item were used\nto generate its label, if any.</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +any","!doc":"<p>Returns a single attribute value.\nReturns defaultValue if and only if <em>item</em> does not have a value for <em>attribute</em>.\nReturns null if and only if null was explicitly set as the attribute value.\nReturns undefined if and only if the item does not have a value for the\ngiven attribute (which is the same as saying the item does not have the attribute).</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> +any","!doc":"<p>This getValues() method works just like the getValue() method, but getValues()\nalways returns an array rather than a single attribute value.  The array\nmay be empty, may contain a single attribute value, or may contain\nmany attribute values.\nIf the item does not have a value for the given attribute, then getValues()\nwill return an empty array: [].  (So, if store.hasAttribute(item, attribute)\nhas a return of false, then store.getValues(item, attribute) will return [].)</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?)","!doc":"<p>Returns true if the given <em>item</em> has a value for the given <em>attribute</em>.</p>\n"},"isItem":{"!type":"fn(something: ?)","!doc":"<p>Returns true if <em>something</em> is an item and came from the store instance.\nReturns false if <em>something</em> is a literal, an item from another store instance,\nor is any object other than an item.</p>\n"},"isItemLoaded":{"!type":"fn(something: ?)","!doc":"<p>Returns false if isItem(something) is false.  Returns false if\nif isItem(something) is true but the the item is not yet loaded\nin local memory (for example, if the item has not yet been read\nfrom the server).</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Given an item, this method loads the item so that a subsequent call\nto store.isItemLoaded(item) will return true.  If a call to\nisItemLoaded() returns true before loadItem() is even called,\nthen loadItem() need not do any work at all and will not even invoke\nthe callback handlers.  So, before invoking this method, check that\nthe item has not already been loaded.</p>\n"}}},"dojo/data/api/Request":{"!type":"fn()","prototype":{"!doc":"<p>This class defines out the semantics of what a &#39;Request&#39; object looks like\nwhen returned from a fetch() method.  In general, a request object is\nnothing more than the original keywordArgs from fetch with an abort function\nattached to it to allow users to abort a particular request if they so choose.\nNo other functions are required on a general Request object return.  That does not\ninhibit other store implementations from adding extensions to it, of course.\n\n</p>\n<p>This is an abstract API that data provider implementations conform to.\nThis file defines methods signatures and intentionally leaves all the\nmethods unimplemented.\n\n</p>\n<p>For more details on fetch, see dojo/data/api/Read.fetch().</p>\n","abort":{"!type":"fn()","!doc":"<p>This function is a hook point for stores to provide as a way for\na fetch to be halted mid-processing.</p>\n"}}},"dojo/data/api/Write":{"!type":"fn()","prototype":{"!proto":"dojo/data/api/Read.prototype","!doc":"<p>This is an abstract API that data provider implementations conform to.\nThis file defines function signatures and intentionally leaves all the\nfunctions unimplemented.</p>\n","close":{"!type":"fn(request?: ?)","!doc":"<p>The close() method is intended for instructing the store to &#39;close&#39; out\nany information associated with a particular request.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?)","!doc":"<p>Returns true if the given <em>value</em> is one of the values that getValues()\nwould return.</p>\n"},"deleteItem":{"!type":"fn(item: ?)","!doc":"<p>Deletes an item from the store.</p>\n"},"fetch":{"!type":"fn(keywordArgs: ?) -> +any","!doc":"<p>Given a query and set of defined options, such as a start and count of items to return,\nthis method executes the query and makes the results available as data items.\nThe format and expectations of stores is that they operate in a generally asynchronous\nmanner, therefore callbacks are always used to return items located by the fetch parameters.</p>\n"},"getAttributes":{"!type":"fn(item: ?)","!doc":"<p>Returns an array with all the attributes that this item has.  This\nmethod will always return an array; if the item has no attributes\nat all, getAttributes() will return an empty array: [].</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> +any","!doc":"<p>Method to inspect the item and return a user-readable &#39;label&#39; for the item\nthat provides a general/adequate description of what the item is.</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> +any","!doc":"<p>Method to inspect the item and return an array of what attributes of the item were used\nto generate its label, if any.</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +any","!doc":"<p>Returns a single attribute value.\nReturns defaultValue if and only if <em>item</em> does not have a value for <em>attribute</em>.\nReturns null if and only if null was explicitly set as the attribute value.\nReturns undefined if and only if the item does not have a value for the\ngiven attribute (which is the same as saying the item does not have the attribute).</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> +any","!doc":"<p>This getValues() method works just like the getValue() method, but getValues()\nalways returns an array rather than a single attribute value.  The array\nmay be empty, may contain a single attribute value, or may contain\nmany attribute values.\nIf the item does not have a value for the given attribute, then getValues()\nwill return an empty array: [].  (So, if store.hasAttribute(item, attribute)\nhas a return of false, then store.getValues(item, attribute) will return [].)</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?)","!doc":"<p>Returns true if the given <em>item</em> has a value for the given <em>attribute</em>.</p>\n"},"isDirty":{"!type":"fn(item?: ?)","!doc":"<p>Given an item, isDirty() returns true if the item has been modified\nsince the last save().  If isDirty() is called with no <em>item</em> argument,\nthen this function returns true if any item has been modified since\nthe last save().</p>\n"},"isItem":{"!type":"fn(something: ?)","!doc":"<p>Returns true if <em>something</em> is an item and came from the store instance.\nReturns false if <em>something</em> is a literal, an item from another store instance,\nor is any object other than an item.</p>\n"},"isItemLoaded":{"!type":"fn(something: ?)","!doc":"<p>Returns false if isItem(something) is false.  Returns false if\nif isItem(something) is true but the the item is not yet loaded\nin local memory (for example, if the item has not yet been read\nfrom the server).</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Given an item, this method loads the item so that a subsequent call\nto store.isItemLoaded(item) will return true.  If a call to\nisItemLoaded() returns true before loadItem() is even called,\nthen loadItem() need not do any work at all and will not even invoke\nthe callback handlers.  So, before invoking this method, check that\nthe item has not already been loaded.</p>\n"},"newItem":{"!type":"fn(keywordArgs?: ?, parentInfo?: ?)","!doc":"<p>Returns a newly created item.  Sets the attributes of the new\nitem based on the <em>keywordArgs</em> provided.  In general, the attribute\nnames in the keywords become the attributes in the new item and as for\nthe attribute values in keywordArgs, they become the values of the attributes\nin the new item.  In addition, for stores that support hierarchical item\ncreation, an optional second parameter is accepted that defines what item is the parent\nof the new item and what attribute of that item should the new item be assigned to.\nIn general, this will assume that the attribute targeted is multi-valued and a new item\nis appended onto the list of values for that attribute.</p>\n"},"revert":{"!type":"fn()","!doc":"<p>Discards any unsaved changes.</p>\n"},"save":{"!type":"fn(keywordArgs: ?) -> +any","!doc":"<p>Saves to the server all the changes that have been made locally.\nThe save operation may take some time and is generally performed\nin an asynchronous fashion.  The outcome of the save action is\nis passed into the set of supported callbacks for the save.</p>\n"},"setValue":{"!type":"fn(item: ?, attribute: ?, value: ?)","!doc":"<p>Sets the value of an attribute on an item.\nReplaces any previous value or values.</p>\n"},"setValues":{"!type":"fn(item: ?, attribute: ?, values: ?)","!doc":"<p>Adds each value in the <em>values</em> array as a value of the given\nattribute on the given item.\nReplaces any previous value or values.\nCalling store.setValues(x, y, []) (with <em>values</em> as an empty array) has\nthe same effect as calling store.unsetAttribute(x, y).</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, attribute: ?)","!doc":"<p>Deletes all the values of an attribute on an item.</p>\n"}}},"dojo/date/locale":{"!doc":"<p>This modules defines dojo/date/locale, localization methods for Date.</p>\n","addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.\n</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?) -> ?","!doc":"<p>Format a Date object as a String, using locale-specific settings.\n</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?) -> ?","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.\n</p>\n"},"isWeekend":{"!type":"fn(dateObject?: ?, locale?: ?) -> bool","!doc":"<p>Determines if the date falls on a weekend, according to local custom.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?) -> +null","!doc":"<p>Convert a properly formatted string to a primitive Date object,\nusing locale-specific settings.\n</p>\n"},"regexp":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a localized date</p>\n"}},"dojo/date/locale.__FormatOptions":{"!type":"fn()","prototype":{"am":{"!doc":"<p>override strings for am in times</p>\n"},"datePattern":{"!doc":"<p>override pattern with this string</p>\n"},"formatLength":{"!doc":"<p>choice of long, short, medium or full (plus any custom additions).  Defaults to &#39;short&#39;</p>\n"},"fullYear":{"!doc":"<p>(format only) use 4 digit years whenever 2 digit years are called for</p>\n"},"locale":{"!doc":"<p>override the locale used to determine formatting rules</p>\n"},"pm":{"!doc":"<p>override strings for pm in times</p>\n"},"selector":{"!doc":"<p>choice of &#39;time&#39;,&#39;date&#39; (default: date and time)</p>\n"},"strict":{"!doc":"<p>(parse only) strict parsing, off by default</p>\n"},"timePattern":{"!doc":"<p>override pattern with this string</p>\n"}}},"dojo/date":{"!doc":"<p>Date manipulation utilities</p>\n","add":{"!type":"fn(date: ?, interval: ?, amount: ?) -> +instance","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2?: ?, portion?: ?) -> number","!doc":"<p>Compare two date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?) -> ?","!doc":"<p>Get the difference in a specific unit of time (e.g., number of\nmonths, weeks, days, etc.) between two dates, rounded to the\nnearest integer.</p>\n"},"getDaysInMonth":{"!type":"fn(dateObject: ?) -> number","!doc":"<p>Returns the number of days in the month used by dateObject</p>\n"},"getTimezoneName":{"!type":"fn(dateObject: ?) -> ?","!doc":"<p>Get the user&#39;s time zone as provided by the browser</p>\n"},"isLeapYear":{"!type":"fn(dateObject: ?) -> bool","!doc":"<p>Determines if the year of the dateObject is a leap year</p>\n"}},"dojo/dnd/AutoSource":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Source.prototype","!doc":"<p>a source that syncs its DnD nodes by default</p>\n","accept":{},"allowNested":{"!doc":"<p>Indicates whether to allow dnd item nodes to be nested within other elements.\nBy default this is false, indicating that only direct children of the container can\nbe draggable dnd item nodes</p>\n"},"autoSync":{},"copyOnly":{},"current":{"!doc":"<p>The DOM node the mouse is currently hovered over</p>\n"},"delay":{},"generateText":{},"horizontal":{},"isSource":{},"map":{"!doc":"<p>Map from an item&#39;s id (which is also the DOMNode&#39;s id) to\nthe dojo/dnd/Container.Item itself.</p>\n"},"selection":{"!doc":"<p>The set of id&#39;s that are currently selected, such that this.selection[id] == 1\nif the node w/that id is selected.  Can iterate over selected node&#39;s id&#39;s like:\n\n</p>\n<pre><code>for(var id in this.selection)</code></pre>\n"},"selfAccept":{},"selfCopy":{},"singular":{},"skipForm":{},"withHandles":{},"checkAcceptance":{"!type":"fn(source: ?, nodes: ?) -> bool","!doc":"<p>checks if the target can accept nodes from this source</p>\n"},"clearItems":{"!type":"fn()","!doc":"<p>removes all data items from the map</p>\n"},"constructor":{"!type":"fn(node: ?, params: ?)","!doc":"<p>constructor of the AutoSource --- see the Source constructor for details</p>\n"},"copyState":{"!type":"fn(keyPressed: ?, self?: ?) -> bool","!doc":"<p>Returns true if we need to copy items, false to move.\nIt is separated to be overwritten dynamically, if needed.</p>\n"},"creator":{"!type":"fn()","!doc":"<p>creator function, dummy at the moment</p>\n"},"deleteSelectedNodes":{"!type":"fn() -> fn","!doc":"<p>deletes all selected items</p>\n"},"delItem":{"!type":"fn(key: ?)","!doc":"<p>removes a data item from the map by its key (id)</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>prepares the object to be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"forInItems":{"!type":"fn(f: ?, o?: ?) -> string","!doc":"<p>iterates over a data map skipping members that\nare present in the empty object (IE and/or 3rd-party libraries).</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getAllNodes":{"!type":"fn() -> ?","!doc":"<p>returns a list (an array) of all valid child nodes</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>returns a data item by its key (id)</p>\n"},"getSelectedNodes":{"!type":"fn() -> +instance","!doc":"<p>returns a list (an array) of selected nodes</p>\n"},"insertNodes":{"!type":"fn(addSelected: ?, data: ?, before: ?, anchor: ?) -> fn","!doc":"<p>inserts new data items (see <code>dojo/dnd/Container.insertNodes()</code> method for details)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"selectAll":{"!type":"fn() -> ?","!doc":"<p>selects all items</p>\n"},"selectNone":{"!type":"fn() -> ?","!doc":"<p>unselects all items</p>\n"},"setItem":{"!type":"fn(key: ?, data: ?)","!doc":"<p>associates a data item with its key (id)</p>\n"},"startup":{"!type":"fn()","!doc":"<p>collects valid child items and populate the map</p>\n"},"sync":{"!type":"fn() -> fn","!doc":"<p>sync up the node list with the data map</p>\n"}}},"dojo/dnd/Source":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Selector.prototype","!doc":"<p>a Source object, which can be used as a DnD source, or a DnD target</p>\n","accept":{},"allowNested":{"!doc":"<p>Indicates whether to allow dnd item nodes to be nested within other elements.\nBy default this is false, indicating that only direct children of the container can\nbe draggable dnd item nodes</p>\n"},"autoSync":{},"copyOnly":{},"current":{"!doc":"<p>The DOM node the mouse is currently hovered over</p>\n"},"delay":{},"generateText":{},"horizontal":{},"isSource":{},"map":{"!doc":"<p>Map from an item&#39;s id (which is also the DOMNode&#39;s id) to\nthe dojo/dnd/Container.Item itself.</p>\n"},"selection":{"!doc":"<p>The set of id&#39;s that are currently selected, such that this.selection[id] == 1\nif the node w/that id is selected.  Can iterate over selected node&#39;s id&#39;s like:\n\n</p>\n<pre><code>for(var id in this.selection)</code></pre>\n"},"selfAccept":{},"selfCopy":{},"singular":{},"skipForm":{},"withHandles":{},"checkAcceptance":{"!type":"fn(source: ?, nodes: ?) -> bool","!doc":"<p>checks if the target can accept nodes from this source</p>\n"},"clearItems":{"!type":"fn()","!doc":"<p>removes all data items from the map</p>\n"},"constructor":{"!type":"fn(node: ?, params?: ?)","!doc":"<p>a constructor of the Source</p>\n"},"copyState":{"!type":"fn(keyPressed: ?, self?: ?) -> bool","!doc":"<p>Returns true if we need to copy items, false to move.\nIt is separated to be overwritten dynamically, if needed.</p>\n"},"creator":{"!type":"fn()","!doc":"<p>creator function, dummy at the moment</p>\n"},"deleteSelectedNodes":{"!type":"fn() -> fn","!doc":"<p>deletes all selected items</p>\n"},"delItem":{"!type":"fn(key: ?)","!doc":"<p>removes a data item from the map by its key (id)</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>prepares the object to be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"forInItems":{"!type":"fn(f: ?, o?: ?) -> string","!doc":"<p>iterates over a data map skipping members that\nare present in the empty object (IE and/or 3rd-party libraries).</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getAllNodes":{"!type":"fn() -> ?","!doc":"<p>returns a list (an array) of all valid child nodes</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>returns a data item by its key (id)</p>\n"},"getSelectedNodes":{"!type":"fn() -> +instance","!doc":"<p>returns a list (an array) of selected nodes</p>\n"},"insertNodes":{"!type":"fn(addSelected: ?, data: ?, before: ?, anchor: ?) -> fn","!doc":"<p>inserts new data items (see <code>dojo/dnd/Container.insertNodes()</code> method for details)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"selectAll":{"!type":"fn() -> ?","!doc":"<p>selects all items</p>\n"},"selectNone":{"!type":"fn() -> ?","!doc":"<p>unselects all items</p>\n"},"setItem":{"!type":"fn(key: ?, data: ?)","!doc":"<p>associates a data item with its key (id)</p>\n"},"startup":{"!type":"fn()","!doc":"<p>collects valid child items and populate the map</p>\n"},"sync":{"!type":"fn() -> fn","!doc":"<p>sync up the node list with the data map</p>\n"}}},"dojo/dnd/common":{"!doc":"<p>TODOC</p>\n","_defaultCreatorNodes":{},"_empty":{},"_manager":{},"_uniqueId":{},"getCopyKeyState":{"!type":"fn(evt: ?) -> ?"},"getUniqueId":{"!type":"fn() -> string","!doc":"<p>returns a unique string for use with any DOM element</p>\n"},"isFormElement":{"!type":"fn(e: ?)","!doc":"<p>returns true if user clicked on a form element</p>\n"},"manager":{"!type":"fn() -> +instance","!doc":"<p>Returns the current DnD manager.  Creates one if it is not created yet.</p>\n"}},"dojo/dnd/common._empty":{},"dojo/dnd/common._defaultCreatorNodes":{"div":{},"ol":{},"p":{},"ul":{}},"dojo/dnd/Selector":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Container.prototype","!doc":"<p>a Selector object, which knows how to select its children</p>\n","allowNested":{"!doc":"<p>Indicates whether to allow dnd item nodes to be nested within other elements.\nBy default this is false, indicating that only direct children of the container can\nbe draggable dnd item nodes</p>\n"},"current":{"!doc":"<p>The DOM node the mouse is currently hovered over</p>\n"},"map":{"!doc":"<p>Map from an item&#39;s id (which is also the DOMNode&#39;s id) to\nthe dojo/dnd/Container.Item itself.</p>\n"},"selection":{"!doc":"<p>The set of id&#39;s that are currently selected, such that this.selection[id] == 1\nif the node w/that id is selected.  Can iterate over selected node&#39;s id&#39;s like:\n\n</p>\n<pre><code>for(var id in this.selection)</code></pre>\n"},"singular":{},"skipForm":{},"clearItems":{"!type":"fn()","!doc":"<p>removes all data items from the map</p>\n"},"constructor":{"!type":"fn(node: ?, params?: ?)","!doc":"<p>constructor of the Selector</p>\n"},"creator":{"!type":"fn()","!doc":"<p>creator function, dummy at the moment</p>\n"},"deleteSelectedNodes":{"!type":"fn() -> fn","!doc":"<p>deletes all selected items</p>\n"},"delItem":{"!type":"fn(key: ?)","!doc":"<p>removes a data item from the map by its key (id)</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>prepares the object to be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"forInItems":{"!type":"fn(f: ?, o?: ?) -> string","!doc":"<p>iterates over a data map skipping members that\nare present in the empty object (IE and/or 3rd-party libraries).</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getAllNodes":{"!type":"fn() -> ?","!doc":"<p>returns a list (an array) of all valid child nodes</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>returns a data item by its key (id)</p>\n"},"getSelectedNodes":{"!type":"fn() -> +instance","!doc":"<p>returns a list (an array) of selected nodes</p>\n"},"insertNodes":{"!type":"fn(addSelected: ?, data: ?, before: ?, anchor: ?) -> fn","!doc":"<p>inserts new data items (see <code>dojo/dnd/Container.insertNodes()</code> method for details)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"selectAll":{"!type":"fn() -> ?","!doc":"<p>selects all items</p>\n"},"selectNone":{"!type":"fn() -> ?","!doc":"<p>unselects all items</p>\n"},"setItem":{"!type":"fn(key: ?, data: ?)","!doc":"<p>associates a data item with its key (id)</p>\n"},"startup":{"!type":"fn()","!doc":"<p>collects valid child items and populate the map</p>\n"},"sync":{"!type":"fn() -> fn","!doc":"<p>sync up the node list with the data map</p>\n"}}},"dojo/touch":{"!doc":"<p>This module provides unified touch event handlers by exporting\npress, move, release and cancel which can also run well on desktop.\nBased on <a href=\"http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html\">http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html</a>\nAlso, if the dojoClick property is set to truthy on a DOM node, dojo/touch generates\nclick events immediately for this node and its descendants (except for descendants that\nhave a dojoClick property set to falsy), to avoid the delay before native browser click events,\nand regardless of whether evt.preventDefault() was called in a touch.press event listener.\n</p>\n","cancel":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;touchcancel&#39;|&#39;mouseleave&#39; for the given node</p>\n"},"enter":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to mouse.enter or touch equivalent for the given node</p>\n"},"leave":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to mouse.leave or touch equivalent for the given node</p>\n"},"move":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener that fires when the mouse cursor or a finger is dragged over the given node.</p>\n"},"out":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;mouseout&#39; or touch equivalent for the given node</p>\n"},"over":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;mouseover&#39; or touch equivalent for the given node</p>\n"},"press":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;touchstart&#39;|&#39;mousedown&#39; for the given node</p>\n"},"release":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to releasing the mouse button while the cursor is over the given node\n(i.e. &quot;mouseup&quot;) or for removing the finger from the screen while touching the given node.</p>\n"}},"dojo/dnd/Container":{"!type":"fn()","prototype":{"!proto":"dojo/Evented.prototype","!doc":"<p>a Container object, which knows when mouse hovers over it,\nand over which element it hovers</p>\n","allowNested":{"!doc":"<p>Indicates whether to allow dnd item nodes to be nested within other elements.\nBy default this is false, indicating that only direct children of the container can\nbe draggable dnd item nodes</p>\n"},"current":{"!doc":"<p>The DOM node the mouse is currently hovered over</p>\n"},"map":{"!doc":"<p>Map from an item&#39;s id (which is also the DOMNode&#39;s id) to\nthe dojo/dnd/Container.Item itself.</p>\n"},"skipForm":{},"clearItems":{"!type":"fn()","!doc":"<p>removes all data items from the map</p>\n"},"constructor":{"!type":"fn(node: ?, params: ?)","!doc":"<p>a constructor of the Container</p>\n"},"creator":{"!type":"fn()","!doc":"<p>creator function, dummy at the moment</p>\n"},"delItem":{"!type":"fn(key: ?)","!doc":"<p>removes a data item from the map by its key (id)</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>prepares this object to be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"forInItems":{"!type":"fn(f: ?, o?: ?) -> string","!doc":"<p>iterates over a data map skipping members that\nare present in the empty object (IE and/or 3rd-party libraries).</p>\n"},"getAllNodes":{"!type":"fn() -> ?","!doc":"<p>returns a list (an array) of all valid child nodes</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>returns a data item by its key (id)</p>\n"},"insertNodes":{"!type":"fn(data: ?, before: ?, anchor: ?) -> fn","!doc":"<p>inserts an array of new nodes before/after an anchor node</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"setItem":{"!type":"fn(key: ?, data: ?)","!doc":"<p>associates a data item with its key (id)</p>\n"},"startup":{"!type":"fn()","!doc":"<p>collects valid child items and populate the map</p>\n"},"sync":{"!type":"fn() -> fn","!doc":"<p>sync up the node list with the data map</p>\n"}},"Item":{"!type":"fn()","!doc":"<p>Represents (one of) the source node(s) being dragged.\nContains (at least) the &quot;type&quot; and &quot;data&quot; attributes.</p>\n"}},"dojo/dnd/Container.__ContainerArgs":{"!type":"fn()","prototype":{"_skipStartup":{"!doc":"<p>skip startup(), which collects children, for deferred initialization\n(this is used in the markup mode)</p>\n"},"dropParent":{"!doc":"<p>node or node&#39;s id to use as the parent node for dropped items\n(must be underneath the &#39;node&#39; parameter in the DOM)</p>\n"},"skipForm":{"!doc":"<p>don&#39;t start the drag operation, if clicked on form elements</p>\n"},"creator":{"!type":"fn()","!doc":"<p>a creator function, which takes a data item, and returns an object like that:\n{node: newNode, data: usedData, type: arrayOfStrings}</p>\n"}}},"dojo/dnd/Manager":{"!type":"fn()","prototype":{"!proto":"dojo/Evented.prototype","!doc":"<p>the manager of DnD operations (usually a singleton)</p>\n","OFFSET_X":{},"OFFSET_Y":{},"canDrop":{"!type":"fn(flag: ?)","!doc":"<p>called to notify if the current target can accept items</p>\n"},"constructor":{"!type":"fn()"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"makeAvatar":{"!type":"fn() -> +instance","!doc":"<p>makes the avatar; it is separate to be overwritten dynamically, if needed</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"outSource":{"!type":"fn(source: ?)","!doc":"<p>called when a source detected a mouse-out condition</p>\n"},"overSource":{"!type":"fn(source: ?)","!doc":"<p>called when a source detected a mouse-over condition</p>\n"},"startDrag":{"!type":"fn(source: ?, nodes: ?, copy: ?)","!doc":"<p>called to initiate the DnD operation</p>\n"},"stopDrag":{"!type":"fn()","!doc":"<p>stop the DnD in progress</p>\n"},"updateAvatar":{"!type":"fn()","!doc":"<p>updates the avatar; it is separate to be overwritten dynamically, if needed</p>\n"}},"manager":{"!type":"fn() -> +instance","!doc":"<p>Returns the current DnD manager.  Creates one if it is not created yet.</p>\n"}},"dojo/dnd/autoscroll":{"!doc":"<p>Used by dojo/dnd/Manager to scroll document or internal node when the user\ndrags near the edge of the viewport or a scrollable node</p>\n","_validNodes":{},"_validOverflow":{},"H_AUTOSCROLL_VALUE":{},"H_TRIGGER_AUTOSCROLL":{},"V_AUTOSCROLL_VALUE":{},"V_TRIGGER_AUTOSCROLL":{},"autoScroll":{"!type":"fn(e: ?)","!doc":"<p>a handler for mousemove and touchmove events, which scrolls the window, if\nnecessary</p>\n"},"autoScrollNodes":{"!type":"fn(e: ?)","!doc":"<p>a handler for mousemove and touchmove events, which scrolls the first available\nDom element, it falls back to exports.autoScroll()</p>\n"},"autoScrollStart":{"!type":"fn(d: ?)","!doc":"<p>Called at the start of a drag.</p>\n"},"getViewport":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Returns the dimensions and scroll position of the viewable area of a browser window</p>\n"}},"dojo/dnd/autoscroll._validNodes":{"div":{},"p":{},"td":{}},"dojo/dnd/autoscroll._validOverflow":{"auto":{},"scroll":{}},"dojo/window":{"!doc":"<p>TODOC</p>\n","get":{"!type":"fn(doc: ?) -> ?","!doc":"<p>Get window object associated with document doc.</p>\n"},"getBox":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Returns the dimensions and scroll position of the viewable area of a browser window</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, pos?: ?)","!doc":"<p>Scroll the passed node into view using minimal movement, if it is not already.</p>\n"}},"dojo/dnd/Avatar":{"!type":"fn()","prototype":{"!doc":"<p>Object that represents transferred DnD items visually</p>\n","manager":{"!doc":"<p>a DnD manager object</p>\n"},"construct":{"!type":"fn()","!doc":"<p>constructor function;\nit is separate so it can be (dynamically) overwritten in case of need</p>\n"},"constructor":{"!type":"fn(manager: ?)"},"destroy":{"!type":"fn()","!doc":"<p>destructor for the avatar; called to remove all references so it can be garbage-collected</p>\n"},"update":{"!type":"fn()","!doc":"<p>updates the avatar to reflect the current DnD state</p>\n"}}},"dojo/hccss":{"!doc":"<p>Test if computer is in high contrast mode (i.e. if browser is not displaying background images).\nDefines <code>has(&quot;highcontrast&quot;)</code> and sets <code>dj_a11y</code> CSS class on <code>&lt;body&gt;</code> if machine is in high contrast mode.\nReturns <code>has()</code> method;</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Test if computer is in high contrast mode (i.e. if browser is not displaying background images).\nDefines <code>has(&quot;highcontrast&quot;)</code> and sets <code>dj_a11y</code> CSS class on <code>&lt;body&gt;</code> if machine is in high contrast mode.\nReturns <code>has()</code> method;</p>\n"}}},"dojo/dnd/Moveable":{"!type":"fn()","prototype":{"!proto":"dojo/Evented.prototype","!doc":"<p>an object, which makes a node movable</p>\n","delay":{},"handle":{},"skip":{},"constructor":{"!type":"fn(node: ?, params?: ?)"},"destroy":{"!type":"fn()","!doc":"<p>stops watching for possible move, deletes all references, so the object can be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojo/dnd/Moveable.__MoveableArgs":{"!type":"fn()","prototype":{"delay":{"!doc":"<p>delay move by this number of pixels</p>\n"},"handle":{"!doc":"<p>A node (or node&#39;s id), which is used as a mouse handle.\nIf omitted, the node itself is used as a handle.</p>\n"},"mover":{"!doc":"<p>a constructor of custom Mover</p>\n"},"skip":{"!doc":"<p>skip move of form elements</p>\n"}}},"dojo/dnd/Mover":{"!type":"fn()","prototype":{"!proto":"dojo/Evented.prototype","!doc":"<p>an object which makes a node follow the mouse, or touch-drag on touch devices.\nUsed as a default mover, and as a base class for custom movers.</p>\n","constructor":{"!type":"fn(node: ?, e: ?, host?: ?)"},"destroy":{"!type":"fn()","!doc":"<p>stops the move, deletes all references, so the object can be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojo/dnd/Target":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Source.prototype","!doc":"<p>a Target object, which can be used as a DnD target</p>\n","accept":{},"allowNested":{"!doc":"<p>Indicates whether to allow dnd item nodes to be nested within other elements.\nBy default this is false, indicating that only direct children of the container can\nbe draggable dnd item nodes</p>\n"},"autoSync":{},"copyOnly":{},"current":{"!doc":"<p>The DOM node the mouse is currently hovered over</p>\n"},"delay":{},"generateText":{},"horizontal":{},"isSource":{},"map":{"!doc":"<p>Map from an item&#39;s id (which is also the DOMNode&#39;s id) to\nthe dojo/dnd/Container.Item itself.</p>\n"},"selection":{"!doc":"<p>The set of id&#39;s that are currently selected, such that this.selection[id] == 1\nif the node w/that id is selected.  Can iterate over selected node&#39;s id&#39;s like:\n\n</p>\n<pre><code>for(var id in this.selection)</code></pre>\n"},"selfAccept":{},"selfCopy":{},"singular":{},"skipForm":{},"withHandles":{},"checkAcceptance":{"!type":"fn(source: ?, nodes: ?) -> bool","!doc":"<p>checks if the target can accept nodes from this source</p>\n"},"clearItems":{"!type":"fn()","!doc":"<p>removes all data items from the map</p>\n"},"constructor":{"!type":"fn(node: ?, params: ?)","!doc":"<p>a constructor of the Target --- see the <code>dojo/dnd/Source</code> constructor for details</p>\n"},"copyState":{"!type":"fn(keyPressed: ?, self?: ?) -> bool","!doc":"<p>Returns true if we need to copy items, false to move.\nIt is separated to be overwritten dynamically, if needed.</p>\n"},"creator":{"!type":"fn()","!doc":"<p>creator function, dummy at the moment</p>\n"},"deleteSelectedNodes":{"!type":"fn() -> fn","!doc":"<p>deletes all selected items</p>\n"},"delItem":{"!type":"fn(key: ?)","!doc":"<p>removes a data item from the map by its key (id)</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>prepares the object to be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"forInItems":{"!type":"fn(f: ?, o?: ?) -> string","!doc":"<p>iterates over a data map skipping members that\nare present in the empty object (IE and/or 3rd-party libraries).</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getAllNodes":{"!type":"fn() -> ?","!doc":"<p>returns a list (an array) of all valid child nodes</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>returns a data item by its key (id)</p>\n"},"getSelectedNodes":{"!type":"fn() -> +instance","!doc":"<p>returns a list (an array) of selected nodes</p>\n"},"insertNodes":{"!type":"fn(addSelected: ?, data: ?, before: ?, anchor: ?) -> fn","!doc":"<p>inserts new data items (see <code>dojo/dnd/Container.insertNodes()</code> method for details)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"selectAll":{"!type":"fn() -> ?","!doc":"<p>selects all items</p>\n"},"selectNone":{"!type":"fn() -> ?","!doc":"<p>unselects all items</p>\n"},"setItem":{"!type":"fn(key: ?, data: ?)","!doc":"<p>associates a data item with its key (id)</p>\n"},"startup":{"!type":"fn()","!doc":"<p>collects valid child items and populate the map</p>\n"},"sync":{"!type":"fn() -> fn","!doc":"<p>sync up the node list with the data map</p>\n"}}},"dojo/dnd/TimedMoveable":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Moveable.prototype","!doc":"<p>A specialized version of Moveable to support an FPS throttling.\nThis class puts an upper restriction on FPS, which may reduce\nthe CPU load. The additional parameter &quot;timeout&quot; regulates\nthe delay before actually moving the moveable object.</p>\n","delay":{},"handle":{},"skip":{},"timeout":{},"constructor":{"!type":"fn(node: ?, params: ?)","!doc":"<p>an object that makes a node moveable with a timer</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops watching for possible move, deletes all references, so the object can be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojo/dnd/move":{"!doc":"<p>TODOC</p>\n","boxConstrainedMoveable":{"!type":"fn()"},"constrainedMoveable":{"!type":"fn()"},"parentConstrainedMoveable":{"!type":"fn()"}},"dojo/dnd/move.constrainedMoveable":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Moveable.prototype","delay":{},"handle":{},"skip":{},"within":{},"constraints":{"!type":"fn()"},"constructor":{"!type":"fn(node: ?, params?: ?)","!doc":"<p>an object that makes a node moveable</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops watching for possible move, deletes all references, so the object can be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojo/dnd/move.boxConstrainedMoveable":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Moveable.prototype","box":{"!doc":"<p>object attributes (for markup)</p>\n"},"delay":{},"handle":{},"skip":{},"within":{},"constraints":{"!type":"fn()"},"constructor":{"!type":"fn(node: ?, params?: ?)","!doc":"<p>an object, which makes a node moveable</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops watching for possible move, deletes all references, so the object can be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojo/dnd/move.parentConstrainedMoveable":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Moveable.prototype","area":{"!doc":"<p>object attributes (for markup)</p>\n"},"delay":{},"handle":{},"skip":{},"within":{},"constraints":{"!type":"fn()"},"constructor":{"!type":"fn(node: ?, params?: ?)","!doc":"<p>an object, which makes a node moveable</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops watching for possible move, deletes all references, so the object can be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojo/fx/Toggler":{"!type":"fn()","prototype":{"!doc":"<p>A simple <code>dojo.Animation</code> toggler API.</p>\n","_hideAnim":{},"_hideArgs":{},"_isHiding":{},"_isShowing":{},"_showAnim":{},"_showArgs":{},"hideDuration":{"!doc":"<p>Time in milliseconds to run the hide Animation</p>\n"},"node":{"!doc":"<p>the node to target for the showing and hiding animations</p>\n"},"showDuration":{"!doc":"<p>Time in milliseconds to run the show Animation</p>\n"},"constructor":{"!type":"fn(args: ?)"},"hide":{"!type":"fn(delay?: ?) -> ?","!doc":"<p>Toggle the node to hidden</p>\n"},"hideFunc":{"!type":"fn(args: ?) -> ?","!doc":"<p>The function that returns the <code>dojo.Animation</code> to hide the node</p>\n"},"show":{"!type":"fn(delay?: ?) -> ?","!doc":"<p>Toggle the node to showing</p>\n"},"showFunc":{"!type":"fn(args: ?) -> ?","!doc":"<p>The function that returns the <code>dojo.Animation</code> to show the node</p>\n"}}},"dojo/fx/easing":{"!doc":"<p>Collection of easing functions to use beyond the default\n<code>dojo._defaultEasing</code> function.</p>\n","backIn":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that starts away from the target,\nand quickly accelerates towards the end value.\n\n</p>\n<p>Use caution when the easing will cause values to become\nnegative as some properties cannot be set to negative values.</p>\n"},"backInOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function combining the effects of <code>backIn</code> and <code>backOut</code></p>\n"},"backOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that pops past the range briefly, and slowly comes back.</p>\n"},"bounceIn":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that &#39;bounces&#39; near the beginning of an Animation</p>\n"},"bounceInOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that &#39;bounces&#39; at the beginning and end of the Animation</p>\n"},"bounceOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that &#39;bounces&#39; near the end of an Animation</p>\n"},"circIn":{"!type":"fn(n?: ?) -> number"},"circInOut":{"!type":"fn(n?: ?) -> number"},"circOut":{"!type":"fn(n?: ?) -> ?"},"cubicIn":{"!type":"fn(n?: ?) -> ?"},"cubicInOut":{"!type":"fn(n?: ?) -> number"},"cubicOut":{"!type":"fn(n?: ?) -> number"},"elasticIn":{"!type":"fn(n?: ?) -> +Decimal","!doc":"<p>An easing function the elastically snaps from the start value</p>\n"},"elasticInOut":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function that elasticly snaps around the value, near\nthe beginning and end of the Animation.</p>\n"},"elasticOut":{"!type":"fn(n?: ?) -> +Decimal","!doc":"<p>An easing function that elasticly snaps around the target value,\nnear the end of the Animation</p>\n"},"expoIn":{"!type":"fn(n?: ?) -> ?"},"expoInOut":{"!type":"fn(n?: ?) -> number"},"expoOut":{"!type":"fn(n?: ?) -> number"},"linear":{"!type":"fn(n?: ?) -> +Decimal","!doc":"<p>A linear easing function</p>\n"},"quadIn":{"!type":"fn(n?: ?) -> ?"},"quadInOut":{"!type":"fn(n?: ?) -> number"},"quadOut":{"!type":"fn(n?: ?) -> number"},"quartIn":{"!type":"fn(n?: ?) -> ?"},"quartInOut":{"!type":"fn(n?: ?) -> number"},"quartOut":{"!type":"fn(n?: ?) -> number"},"quintIn":{"!type":"fn(n?: ?) -> ?"},"quintInOut":{"!type":"fn(n?: ?) -> number"},"quintOut":{"!type":"fn(n?: ?) -> number"},"sineIn":{"!type":"fn(n?: ?) -> number"},"sineInOut":{"!type":"fn(n?: ?) -> number"},"sineOut":{"!type":"fn(n?: ?) -> ?"}},"dojo/gears":{"!doc":"<p>TODOC</p>\n","available":{"!doc":"<p>True if client is using Google Gears</p>\n"}},"dojo/gears.available":{"!doc":"<p>True if client is using Google Gears</p>\n"},"dojo/hash":{"!doc":"<p>Gets or sets the hash string in the browser URL.</p>\n","prototype":{"constructor":{"!type":"fn(hash?: ?, replace?: ?) -> +any","!doc":"<p>Gets or sets the hash string in the browser URL.</p>\n"}}},"dojo/io/iframe":{"!doc":"<p>Deprecated, use dojo/request/iframe instead.\nSends an Ajax I/O call using and Iframe (for instance, to upload files)</p>\n","create":{"!type":"fn(fname: ?, onloadstr: ?, uri: ?)","!doc":"<p>Creates a hidden iframe in the page. Used mostly for IO\ntransports.  You do not need to call this to start a\ndojo/io/iframe request. Just call send().</p>\n"},"doc":{"!type":"fn(iframeNode: ?)","!doc":"<p>Returns the document object associated with the iframe DOM Node argument.</p>\n"},"setSrc":{"!type":"fn(iframe: ?, src: ?, replace: ?)","!doc":"<p>Sets the URL that is loaded in an IFrame. The replace parameter\nindicates whether location.replace() should be used when\nchanging the location of the iframe.</p>\n"}},"dojo/request/iframe":{"!doc":"<p>Sends a request using an iframe element with the given URL and options.</p>\n","_currentDfd":{},"_dfdQueue":{},"_iframeName":{},"prototype":{"constructor":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Sends a request using an iframe element with the given URL and options.</p>\n"}},"create":{"!type":"fn(name: ?, onloadstr: ?, uri: ?) -> ?"},"doc":{"!type":"fn(iframeNode: ?) -> ?"},"get":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP GET request using an iframe element with the given URL and options.</p>\n"},"post":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP POST request using an iframe element with the given URL and options.</p>\n"},"setSrc":{"!type":"fn(_iframe: ?, src: ?, replace: ?)"}},"dojo/request/iframe.__BaseOptions":{"!type":"fn()","prototype":{"data":{"!doc":"<p>Data to transfer. When making a GET request, this will\nbe converted to key=value parameters and appended to the\nURL.</p>\n"},"form":{"!doc":"<p>A form node to use to submit data to the server.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"}}},"dojo/request/iframe.__MethodOptions":{"!type":"fn()","prototype":{"method":{"!doc":"<p>The HTTP method to use to make the request. Must be\nuppercase. Only <code>&quot;GET&quot;</code> and <code>&quot;POST&quot;</code> are accepted.\nDefault is <code>&quot;POST&quot;</code>.</p>\n"}}},"dojo/request/iframe.__Options":{"!type":"fn()","prototype":{"data":{"!doc":"<p>Data to transfer. When making a GET request, this will\nbe converted to key=value parameters and appended to the\nURL.</p>\n"},"form":{"!doc":"<p>A form node to use to submit data to the server.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"method":{"!doc":"<p>The HTTP method to use to make the request. Must be\nuppercase. Only <code>&quot;GET&quot;</code> and <code>&quot;POST&quot;</code> are accepted.\nDefault is <code>&quot;POST&quot;</code>.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"}}},"dojo/io/script":{"!doc":"<p>TODOC</p>\n","_counter":{},"_deadScripts":{},"attach":{"!type":"fn(id: ?, url: ?, frameDocument: ?)","!doc":"<p>creates a new <code>&lt;script&gt;</code> tag pointing to the specified URL and\nadds it to the document.</p>\n"},"get":{"!type":"fn(args: ?) -> ?","!doc":"<p>sends a get request using a dynamically created script tag.</p>\n"},"remove":{"!type":"fn(id: ?, frameDocument: ?)","!doc":"<p>removes the script element with the given id, from the given frameDocument.\nIf no frameDocument is passed, the current document is used.</p>\n"}},"dojo/request/script":{"!doc":"<p>Sends a request using a script element with the given URL and options.</p>\n","_callbacksProperty":{},"prototype":{"constructor":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Sends a request using a script element with the given URL and options.</p>\n"}},"get":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP GET request using a script element with the given URL and options.</p>\n"}},"dojo/request/script.__BaseOptions":{"!type":"fn()","prototype":{"checkString":{"!doc":"<p>A string of JavaScript that when evaluated like so:\n&quot;typeof(&quot; + checkString + &quot;) != &#39;undefined&#39;&quot;\nbeing true means that the script fetched has been loaded.\nDo not use this if doing a JSONP type of call (use <code>jsonp</code> instead).</p>\n"},"data":{"!doc":"<p>Data to transfer.  This is ignored for GET and DELETE\nrequests.</p>\n"},"frameDoc":{"!doc":"<p>The Document object of a child iframe. If this is passed in, the script\nwill be attached to that document. This can be helpful in some comet long-polling\nscenarios with Firefox and Opera.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"jsonp":{"!doc":"<p>The URL parameter name that indicates the JSONP callback string.\nFor instance, when using Yahoo JSONP calls it is normally,\njsonp: &quot;callback&quot;. For AOL JSONP calls it is normally\njsonp: &quot;c&quot;.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"}}},"dojo/request/script.__MethodOptions":{"!type":"fn()","prototype":{"method":{"!doc":"<p>This option is ignored. All requests using this transport are\nGET requests.</p>\n"}}},"dojo/request/script.__Options":{"!type":"fn()","prototype":{"checkString":{"!doc":"<p>A string of JavaScript that when evaluated like so:\n&quot;typeof(&quot; + checkString + &quot;) != &#39;undefined&#39;&quot;\nbeing true means that the script fetched has been loaded.\nDo not use this if doing a JSONP type of call (use <code>jsonp</code> instead).</p>\n"},"data":{"!doc":"<p>Data to transfer.  This is ignored for GET and DELETE\nrequests.</p>\n"},"frameDoc":{"!doc":"<p>The Document object of a child iframe. If this is passed in, the script\nwill be attached to that document. This can be helpful in some comet long-polling\nscenarios with Firefox and Opera.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"jsonp":{"!doc":"<p>The URL parameter name that indicates the JSONP callback string.\nFor instance, when using Yahoo JSONP calls it is normally,\njsonp: &quot;callback&quot;. For AOL JSONP calls it is normally\njsonp: &quot;c&quot;.</p>\n"},"method":{"!doc":"<p>This option is ignored. All requests using this transport are\nGET requests.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"}}},"dojo/loadInit":{"dynamic":{},"load":{},"normalize":{"!type":"fn(id: ?) -> ?"}},"dojo/main":{"!doc":"<p>This module is the foundational module of the dojo boot sequence; it defines the dojo object.</p>\n","_blockAsync":{},"_contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n"},"_defaultContext":{},"_hasResource":{},"_initFired":{},"_name":{},"_nodeDataCache":{},"_NodeListCtor":{},"_postLoad":{},"_windowUnloaders":{},"back":{"!doc":"<p>Browser history management resources</p>\n"},"baseUrl":{},"behavior":{},"cldr":{},"colors":{},"config":{"!doc":"<p>This module defines the user configuration during bootstrap.</p>\n"},"connectPublisher":{},"contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n"},"currency":{"!doc":"<p>localized formatting and parsing routines for currencies</p>\n"},"data":{},"date":{},"dijit":{},"dnd":{},"doc":{"!doc":"<p>Alias for the current document. &#39;doc&#39; can be modified\nfor temporary context shifting. See also withDoc().</p>\n"},"dojox":{},"fx":{"!doc":"<p>Effects library on top of Base animations</p>\n"},"gears":{"!doc":"<p>TODOC</p>\n"},"global":{"!doc":"<p>Alias for the current window. &#39;global&#39; can be modified\nfor temporary context shifting. See also withGlobal().</p>\n"},"html":{"!doc":"<p>TODOC</p>\n"},"i18n":{"!doc":"<p>This module implements the dojo/i18n! plugin and the v1.6- i18n API</p>\n"},"io":{},"isAir":{"!doc":"<p>True if client is Adobe Air</p>\n"},"isAndroid":{"!doc":"<p>Version as a Number if client is android browser. undefined otherwise.</p>\n"},"isAsync":{},"isBrowser":{},"isChrome":{"!doc":"<p>Version as a Number if client is Chrome browser. undefined otherwise.</p>\n"},"isCopyKey":{},"isFF":{},"isIE":{"!doc":"<p>Version as a Number if client is MSIE(PC). undefined otherwise. Corresponds to\nmajor detected IE version (6, 7, 8, etc.)</p>\n"},"isIos":{"!doc":"<p>Version as a Number if client is iPhone, iPod, or iPad. undefined otherwise.</p>\n"},"isKhtml":{"!doc":"<p>Version as a Number if client is a KHTML browser. undefined otherwise. Corresponds to major\ndetected version.</p>\n"},"isMac":{"!doc":"<p>True if the client runs on Mac</p>\n"},"isMoz":{},"isMozilla":{},"isOpera":{"!doc":"<p>Version as a Number if client is Opera. undefined otherwise. Corresponds to\nmajor detected version.</p>\n"},"isQuirks":{},"isSafari":{"!doc":"<p>Version as a Number if client is Safari or iPhone. undefined otherwise.</p>\n"},"isSpidermonkey":{},"isWebKit":{"!doc":"<p>Version as a Number if client is a WebKit-derived browser (Konqueror,\nSafari, Chrome, etc.). undefined otherwise.</p>\n"},"isWii":{"!doc":"<p>True if client is Wii</p>\n"},"keys":{"!doc":"<p>Definitions for common key values.  Client code should test keyCode against these named constants,\nas the actual codes can vary by browser.</p>\n"},"locale":{},"mouseButtons":{},"number":{"!doc":"<p>localized formatting and parsing routines for Number</p>\n"},"parser":{},"publish":{},"query":{},"regexp":{"!doc":"<p>Regular expressions and Builder resources</p>\n"},"rpc":{},"scopeMap":{},"store":{},"string":{"!doc":"<p>String utilities for Dojo</p>\n"},"subscribe":{},"tests":{"!doc":"<p>D.O.H. Test files for Dojo unit testing.</p>\n"},"toJsonIndentStr":{},"touch":{"!doc":"<p>This module provides unified touch event handlers by exporting\npress, move, release and cancel which can also run well on desktop.\nBased on <a href=\"http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html\">http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html</a>\nAlso, if the dojoClick property is set to truthy on a DOM node, dojo/touch generates\nclick events immediately for this node and its descendants (except for descendants that\nhave a dojoClick property set to falsy), to avoid the delay before native browser click events,\nand regardless of whether evt.preventDefault() was called in a touch.press event listener.\n</p>\n"},"version":{"!doc":"<p>Version number of the Dojo Toolkit</p>\n"},"window":{"!doc":"<p>TODOC</p>\n"},"AdapterRegistry":{"!type":"fn(returnWrappers?: ?)","!doc":"<p>A registry to make contextual calling/searching easier.</p>\n"},"addClass":{"!type":"fn(node: ?, classStr: ?)","!doc":"<p>Adds the specified classes to the end of the class list on the\npassed node. Will not re-apply duplicate classes.\n</p>\n"},"addOnLoad":{"!type":"fn(priority?: ?, context: ?, callback?: ?)","!doc":"<p>Add a function to execute on DOM content loaded and all requested modules have arrived and been evaluated.\nIn most cases, the <code>domReady</code> plug-in should suffice and this method should not be needed.\n\n</p>\n<p>When called in a non-browser environment, just checks that all requested modules have arrived and been\nevaluated.</p>\n"},"addOnUnload":{"!type":"fn(obj?: ?, functionName?: ?)","!doc":"<p>registers a function to be triggered when the page unloads.</p>\n"},"addOnWindowUnload":{"!type":"fn(obj?: ?, functionName?: ?)","!doc":"<p>registers a function to be triggered when window.onunload fires.\nBe careful trying to modify the DOM or access JavaScript properties\nduring this phase of page unloading: they may not always be available.\nConsider dojo.addOnUnload() if you need to modify the DOM or do heavy\nJavaScript work.</p>\n"},"anim":{"!type":"fn(node: ?, properties: ?, duration?: ?, easing?: ?, onEnd?: ?, delay?: ?) -> ?","!doc":"<p>A simpler interface to <code>animateProperty()</code>, also returns\nan instance of <code>Animation</code> but begins the animation\nimmediately, unlike nearly every other Dojo animation API.</p>\n"},"animateProperty":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation that will transition the properties of\nnode defined in <code>args</code> depending how they are defined in\n<code>args.properties</code>\n</p>\n"},"Animation":{"!type":"fn(args: ?)","!doc":"<p>A generic animation class that fires callbacks into its handlers\nobject at various states.</p>\n"},"attr":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Gets or sets an attribute on an HTML element.</p>\n"},"blendColors":{"!type":"fn(start: ?, end: ?, weight: ?, obj?: ?) -> ?","!doc":"<p>Blend colors end and start with weight from 0 to 1, 0.5 being a 50/50 blend,\ncan reuse a previously allocated Color object for the result</p>\n"},"body":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Return the body element of the specified document or of dojo/_base/window::doc.</p>\n"},"byId":{"!type":"fn(id: ?, doc?: ?) -> +instance","!doc":"<p>Returns DOM node with matching <code>id</code> attribute or falsy value (ex: null or undefined)\nif not found.  If <code>id</code> is a DomNode, this function is a no-op.\n</p>\n"},"cache":{"!type":"fn(module: ?, url: ?, value?: ?) -> ?","!doc":"<p>A getter and setter for storing the string content associated with the\nmodule and url arguments.</p>\n"},"clearCache":{"!type":"fn()"},"Color":{"!type":"fn(color: ?)","!doc":"<p>Takes a named string, hex string, array of rgb or rgba values,\nan object with r, g, b, and a properties, or another <code>Color</code> object\nand creates a new Color instance to work from.\n</p>\n"},"colorFromArray":{"!type":"fn(a: ?, obj?: ?) -> +any","!doc":"<p>Builds a <code>Color</code> from a 3 or 4 element array, mapping each\nelement in sequence to the rgb(a) values of the color.</p>\n"},"colorFromHex":{"!type":"fn(color: ?, obj?: ?) -> +any","!doc":"<p>Converts a hex string with a &#39;#&#39; prefix to a color object.\nSupports 12-bit #rgb shorthand. Optionally accepts a\n<code>Color</code> object to update with the parsed value.\n</p>\n"},"colorFromRgb":{"!type":"fn(color: ?, obj?: ?) -> +null","!doc":"<p>get rgb(a) array from css-style color declarations</p>\n"},"colorFromString":{"!type":"fn(str: ?, obj?: ?) -> +any","!doc":"<p>Parses <code>str</code> for a color value. Accepts hex, rgb, and rgba\nstyle color values.</p>\n"},"connect":{"!type":"fn(obj?: ?, event: ?, context: ?, method: ?, dontFix?: ?) -> ?","!doc":"<p><code>dojo.connect</code> is a deprecated event handling and delegation method in\nDojo. It allows one function to &quot;listen in&quot; on the execution of\nany other, triggering the second whenever the first is called. Many\nlisteners may be attached to a function, and source functions may\nbe either regular function calls or DOM events.\n</p>\n"},"contentBox":{"!type":"fn(node: ?, box?: ?) -> ?","!doc":"<p>Getter/setter for the content-box of node.</p>\n"},"cookie":{"!type":"fn(name: ?, value?: ?, props?: ?) -> ?","!doc":"<p>Get or set a cookie.</p>\n"},"coords":{"!type":"fn(node: ?, includeScroll?: ?) -> ?","!doc":"<p>Deprecated: Use position() for border-box x/y/w/h\nor marginBox() for margin-box w/h/l/t.\n\n</p>\n<p>Returns an object that measures margin-box (w)idth/(h)eight\nand absolute position x/y of the border-box. Also returned\nis computed (l)eft and (t)op values in pixels from the\nnode&#39;s offsetParent as returned from marginBox().\nReturn value will be in the form:\n\n</p>\n<pre><code>{ l: 50, t: 200, w: 300: h: 150, x: 100, y: 300 }</code></pre>\n<p>Does not act as a setter. If includeScroll is passed, the x and\n\n</p>\n<p>y params are affected as one would expect in dojo.position().</p>\n"},"create":{"!type":"fn(tag: ?, attrs: ?, refNode?: ?, pos?: ?) -> ?","!doc":"<p>Create an element, allowing for optional attribute decoration\nand placement.</p>\n"},"declare":{"!type":"fn(className?: ?, superclass: ?, props: ?) -> +dojo","!doc":"<p>Create a feature-rich constructor from compact notation.</p>\n"},"Deferred":{"!type":"fn(canceller?: ?)","!doc":"<p>Deprecated.   This module defines the legacy dojo/_base/Deferred API.\nNew code should use dojo/Deferred instead.</p>\n"},"DeferredList":{"!type":"fn(list: ?, fireOnOneCallback?: ?, fireOnOneErrback?: ?, consumeErrors?: ?, canceller?: ?)","!doc":"<p>Deprecated, use dojo/promise/all instead.\nProvides event handling for a group of Deferred objects.</p>\n"},"deprecated":{"!type":"fn(behaviour: ?, extra?: ?, removal?: ?)","!doc":"<p>Log a debug message to indicate that a behavior has been\ndeprecated.</p>\n"},"destroy":{"!type":"fn(node: ?)"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Remove a link created by dojo.connect.</p>\n"},"docScroll":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Returns an object with {node, x, y} with corresponding offsets.</p>\n"},"empty":{"!type":"fn(node: ?)"},"eval":{"!type":"fn(scriptText: ?) -> +any","!doc":"<p>A legacy method created for use exclusively by internal Dojo methods. Do not use this method\ndirectly unless you understand its possibly-different implications on the platforms your are targeting.</p>\n"},"every":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?) -> bool","!doc":"<p>Determines whether or not every item in arr satisfies the\ncondition implemented by callback.</p>\n"},"exit":{"!type":"fn(exitcode: ?)"},"experimental":{"!type":"fn(moduleName: ?, extra?: ?)","!doc":"<p>Marks code as experimental.</p>\n"},"fadeIn":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that will fade node defined in &#39;args&#39; from\nits current opacity to fully opaque.</p>\n"},"fadeOut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that will fade node defined in &#39;args&#39;\nfrom its current opacity to fully transparent.</p>\n"},"fieldToObject":{"!type":"fn(inputNode: ?) -> ?","!doc":"<p>Serialize a form field to a JavaScript object.</p>\n"},"filter":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?) -> []","!doc":"<p>Returns a new Array with those items from arr that match the\ncondition implemented by callback.</p>\n"},"fixEvent":{"!type":"fn(evt: ?, sender: ?) -> +Event","!doc":"<p>normalizes properties on the event object including event\nbubbling methods, keystroke normalization, and x/y positions</p>\n"},"fixIeBiDiScrollLeft":{"!type":"fn(scrollLeft: ?, doc?: ?) -> number","!doc":"<p>In RTL direction, scrollLeft should be a negative value, but IE\nreturns a positive one. All codes using documentElement.scrollLeft\nmust call this function to fix this error, otherwise the position\nwill offset to right when there is a horizontal scrollbar.</p>\n"},"forEach":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?)","!doc":"<p>for every item in arr, callback is invoked. Return values are ignored.\nIf you want to break out of the loop, consider using array.every() or array.some().\nforEach does not allow breaking out of the loop over the items in arr.</p>\n"},"formToJson":{"!type":"fn(formNode: ?, prettyPrint?: ?) -> string","!doc":"<p>Create a serialized JSON string from a form node or string\nID identifying the form to serialize</p>\n"},"formToObject":{"!type":"fn(formNode: ?) -> ?","!doc":"<p>Serialize a form node to a JavaScript object.</p>\n"},"formToQuery":{"!type":"fn(formNode: ?) -> string","!doc":"<p>Returns a URL-encoded string representing the form passed as either a\nnode or string ID identifying the form to serialize</p>\n"},"fromJson":{"!type":"fn(js: ?) -> ?","!doc":"<p>Parses a JavaScript expression and returns a JavaScript value.</p>\n"},"getAttr":{"!type":"fn(node: ?, name: ?) -> +any","!doc":"<p>Gets an attribute on an HTML element.</p>\n"},"getBorderExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns an object with properties useful for noting the border\ndimensions.</p>\n"},"getComputedStyle":{"!type":"fn(node: ?)","!doc":"<p>Returns a &quot;computed style&quot; object.\n</p>\n"},"getContentBox":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>Returns an object that encodes the width, height, left and top\npositions of the node&#39;s content box, irrespective of the\ncurrent box model.</p>\n"},"getIeDocumentElementOffset":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>returns the offset in x and y from the document body to the\nvisual edge of the page for IE</p>\n"},"getL10nName":{"!type":"fn(moduleName: ?, bundleName: ?, locale: ?) -> string"},"getMarginBox":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns an object that encodes the width, height, left and top\npositions of the node&#39;s margin box.</p>\n"},"getMarginExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns object with properties useful for box fitting with\nregards to box margins (i.e., the outer-box).\n\n</p>\n<ul>\n<li>l/t = marginLeft, marginTop, respectively</li>\n<li>w = total width, margin inclusive</li>\n<li>h = total height, margin inclusive</li>\n</ul>\n<p>The w/h are used for calculating boxes.\nNormally application code will not need to invoke this\ndirectly, and will use the ...box... functions instead.</p>\n"},"getMarginSize":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>returns an object that encodes the width and height of\nthe node&#39;s margin box</p>\n"},"getNodeProp":{"!type":"fn(node: ?, name: ?) -> +any","!doc":"<p>Returns an effective value of a property or an attribute.</p>\n"},"getPadBorderExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>Returns object with properties useful for box fitting with\nregards to padding.</p>\n"},"getPadExtents":{"!type":"fn(node: ?, computedStyle?: ?) -> ?","!doc":"<p>Returns object with special values specifically useful for node\nfitting.</p>\n"},"getProp":{"!type":"fn(node: ?, name: ?) -> +any","!doc":"<p>Gets a property on an HTML element.</p>\n"},"getStyle":{"!type":"fn(node: ?, name?: ?) -> ?","!doc":"<p>Accesses styles on a node.</p>\n"},"hasAttr":{"!type":"fn(node: ?, name: ?) -> bool","!doc":"<p>Returns true if the requested attribute is specified on the\ngiven element, and false otherwise.</p>\n"},"hasClass":{"!type":"fn(node: ?, classStr: ?) -> bool","!doc":"<p>Returns whether or not the specified classes are a portion of the\nclass list currently applied to the node.</p>\n"},"hash":{"!type":"fn(hash?: ?, replace?: ?) -> +any","!doc":"<p>Gets or sets the hash string in the browser URL.</p>\n"},"indexOf":{"!type":"fn(arr: ?, value: ?, fromIndex?: ?, findLast?: ?) -> number","!doc":"<p>locates the first index of the provided value in the\npassed array. If the value is not found, -1 is returned.</p>\n"},"isBodyLtr":{"!type":"fn(doc?: ?) -> bool","!doc":"<p>Returns true if the current language is left-to-right, and false otherwise.</p>\n"},"isDescendant":{"!type":"fn(node: ?, ancestor: ?) -> bool","!doc":"<p>Returns true if node is a descendant of ancestor</p>\n"},"lastIndexOf":{"!type":"fn(arr: ?, value: ?, fromIndex?: ?) -> number","!doc":"<p>locates the last index of the provided value in the passed\narray. If the value is not found, -1 is returned.</p>\n"},"loadInit":{"!type":"fn(f: ?)"},"map":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?, Ctr: ?) -> []","!doc":"<p>applies callback to each element of arr and returns\nan Array with the results</p>\n"},"marginBox":{"!type":"fn(node: ?, box?: ?) -> ?","!doc":"<p>Getter/setter for the margin-box of node.</p>\n"},"moduleUrl":{"!type":"fn(module: ?, url?: ?) -> string","!doc":"<p>Returns a URL relative to a module.</p>\n"},"NodeList":{"!type":"fn(array: ?) -> []","!doc":"<p>Array-like object which adds syntactic\nsugar for chaining, common iteration operations, animation, and\nnode manipulation. NodeLists are most often returned as the\nresult of dojo/query() calls.</p>\n"},"objectToQuery":{"!type":"fn(map: ?) -> ?","!doc":"<p>takes a name/value mapping object and returns a string representing\na URL-encoded version of that object.</p>\n"},"place":{"!type":"fn(node: ?, refNode: ?, position?: ?) -> +Node","!doc":"<p>Attempt to insert node into the DOM, choosing from various positioning options.\nReturns the first argument resolved to a DOM node.</p>\n"},"platformRequire":{"!type":"fn(modMap: ?)","!doc":"<p>require one or more modules based on which host environment\nDojo is currently operating in</p>\n"},"popContext":{"!type":"fn()","!doc":"<p>If the context stack contains elements, ensure that\nsubsequent code executes in the <em>previous</em> context to the\ncurrent context. The current context set ([global,\ndocument]) is returned.</p>\n"},"position":{"!type":"fn(node: ?, includeScroll?: ?) -> ?","!doc":"<p>Gets the position and size of the passed element relative to\nthe viewport (if includeScroll==false), or relative to the\ndocument root (if includeScroll==true).\n</p>\n"},"prop":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Gets or sets a property on an HTML element.</p>\n"},"provide":{"!type":"fn(mid: ?)"},"pushContext":{"!type":"fn(g?: ?, d?: ?)","!doc":"<p>causes subsequent calls to Dojo methods to assume the\npassed object and, optionally, document as the default\nscopes to use. A 2-element array of the previous global and\ndocument are returned.</p>\n"},"queryToObject":{"!type":"fn(str: ?) -> ?","!doc":"<p>Create an object representing a de-serialized query section of a\nURL. Query keys with multiple values are returned in an array.\n</p>\n"},"rawXhrPost":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP POST request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"},"rawXhrPut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP PUT request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"},"ready":{"!type":"fn(priority?: ?, context: ?, callback?: ?)","!doc":"<p>Add a function to execute on DOM content loaded and all requested modules have arrived and been evaluated.\nIn most cases, the <code>domReady</code> plug-in should suffice and this method should not be needed.\n\n</p>\n<p>When called in a non-browser environment, just checks that all requested modules have arrived and been\nevaluated.</p>\n"},"registerModulePath":{"!type":"fn(moduleName: ?, prefix: ?)","!doc":"<p>Maps a module name to a path</p>\n"},"removeAttr":{"!type":"fn(node: ?, name: ?)","!doc":"<p>Removes an attribute from an HTML element.</p>\n"},"removeClass":{"!type":"fn(node: ?, classStr?: ?)","!doc":"<p>Removes the specified classes from node. No <code>contains()</code>\ncheck is required.\n</p>\n"},"replaceClass":{"!type":"fn(node: ?, addClassStr: ?, removeClassStr?: ?)","!doc":"<p>Replaces one or more classes on a node if not present.\nOperates more quickly than calling dojo.removeClass and dojo.addClass\n</p>\n"},"require":{"!type":"fn(moduleName: ?, omitModuleCheck?: ?) -> +any","!doc":"<p>loads a Javascript module from the appropriate URI\n</p>\n"},"requireAfterIf":{"!type":"fn(condition: ?, moduleName: ?, omitModuleCheck?: ?)","!doc":"<p>If the condition is true then call <code>dojo.require()</code> for the specified\nresource\n</p>\n"},"requireIf":{"!type":"fn(condition: ?, moduleName: ?, omitModuleCheck?: ?)","!doc":"<p>If the condition is true then call <code>dojo.require()</code> for the specified\nresource\n</p>\n"},"requireLocalization":{"!type":"fn(moduleName: ?, bundleName: ?, locale?: ?)"},"safeMixin":{"!type":"fn(target: ?, source: ?) -> ?","!doc":"<p>Mix in properties skipping a constructor and decorating functions\nlike it is done by declare().</p>\n"},"setAttr":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Sets an attribute on an HTML element.</p>\n"},"setContentSize":{"!type":"fn(node: ?, box: ?, computedStyle?: ?)","!doc":"<p>Sets the size of the node&#39;s contents, irrespective of margins,\npadding, or borders.</p>\n"},"setContext":{"!type":"fn(globalObject: ?, globalDocument: ?)","!doc":"<p>changes the behavior of many core Dojo functions that deal with\nnamespace and DOM lookup, changing them to work in a new global\ncontext (e.g., an iframe). The varibles dojo.global and dojo.doc\nare modified as a result of calling this function and the result of\n<code>dojo.body()</code> likewise differs.</p>\n"},"setMarginBox":{"!type":"fn(node: ?, box: ?, computedStyle?: ?)","!doc":"<p>sets the size of the node&#39;s margin box and placement\n(left/top), irrespective of box model. Think of it as a\npassthrough to setBox that handles box-model vagaries for\nyou.</p>\n"},"setProp":{"!type":"fn(node: ?, name: ?, value?: ?) -> +any","!doc":"<p>Sets a property on an HTML element.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)"},"setStyle":{"!type":"fn(node: ?, name: ?, value?: ?) -> string","!doc":"<p>Sets styles on a node.</p>\n"},"some":{"!type":"fn(arr: ?, callback: ?, thisObject?: ?) -> bool","!doc":"<p>Determines whether or not any item in arr satisfies the\ncondition implemented by callback.</p>\n"},"Stateful":{"!type":"fn()"},"stopEvent":{"!type":"fn(evt: ?)","!doc":"<p>prevents propagation and clobbers the default action of the\npassed event</p>\n"},"style":{"!type":"fn(node: ?, name?: ?, value?: ?) -> +any","!doc":"<p>Accesses styles on a node. If 2 arguments are\npassed, acts as a getter. If 3 arguments are passed, acts\nas a setter.</p>\n"},"toDom":{"!type":"fn(frag: ?, doc?: ?) -> +any","!doc":"<p>instantiates an HTML fragment returning the corresponding DOM.</p>\n"},"toggleClass":{"!type":"fn(node: ?, classStr: ?, condition?: ?) -> bool","!doc":"<p>Adds a class to node if not present, or removes if present.\nPass a boolean condition if you want to explicitly add or remove.\nReturns the condition that was specified directly or indirectly.\n</p>\n"},"toJson":{"!type":"fn(it: ?, prettyPrint?: ?) -> +any","!doc":"<p>Returns a <a href=\"http://json.org\">JSON</a> serialization of an object.</p>\n"},"toPixelValue":{"!type":"fn(node: ?, value: ?) -> number","!doc":"<p>converts style value to pixels on IE or return a numeric value.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Remove a topic listener.</p>\n"},"when":{"!type":"fn(valueOrPromise: ?, callback?: ?, errback?: ?, progback?: ?) -> +dojo","!doc":"<p>Transparently applies callbacks to values and/or promises.</p>\n"},"windowUnloaded":{"!type":"fn()","!doc":"<p>signal fired by impending window destruction. You may use\ndojo.addOnWIndowUnload() or dojo.connect() to this method to perform\npage/application cleanup methods. See dojo.addOnWindowUnload for more info.</p>\n"},"withDoc":{"!type":"fn(documentObject: ?, callback: ?, thisObject?: ?, cbArguments?: ?) -> ?","!doc":"<p>Invoke callback with documentObject as dojo/_base/window::doc.</p>\n"},"withGlobal":{"!type":"fn(globalObject: ?, callback: ?, thisObject?: ?, cbArguments?: ?) -> ?","!doc":"<p>Invoke callback with globalObject as dojo.global and\nglobalObject.document as dojo.doc.</p>\n"},"xhr":{"!type":"fn(method: ?, args: ?) -> ?"},"xhrDelete":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP DELETE request to the server.</p>\n"},"xhrGet":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP GET request to the server.</p>\n"},"xhrPost":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP POST request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"},"xhrPut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP PUT request to the server. In addition to the properties\nlisted for the dojo.__XhrArgs type, the following property is allowed:</p>\n"}},"dojo/main.config":{"!doc":"<p>This module defines the user configuration during bootstrap.</p>\n","addOnLoad":{"!doc":"<p>Adds a callback via dojo/ready. Useful when Dojo is added after\nthe page loads and djConfig.afterOnLoad is true. Supports the same\narguments as dojo/ready. When using a function reference, use\n<code>djConfig.addOnLoad = function(){};</code>. For object with function name use\n<code>djConfig.addOnLoad = [myObject, &quot;functionName&quot;];</code> and for object with\nfunction reference use\n<code>djConfig.addOnLoad = [myObject, function(){}];</code></p>\n"},"afterOnLoad":{},"baseUrl":{},"callback":{"!doc":"<p>Defines a callback to be used when dependencies are defined before \nthe loader has been loaded. When provided, they cause the loader to \nexecute require(deps, callback) once it has finished loading. \nShould be used with deps.</p>\n"},"debugContainerId":{},"debugHeight":{},"defaultDuration":{"!doc":"<p>Default duration, in milliseconds, for wipe and fade animations within dijits.\nAssigned to dijit.defaultDuration.</p>\n"},"deferredInstrumentation":{"!doc":"<p>Whether deferred instrumentation should be loaded or included\nin builds.</p>\n"},"deps":{"!doc":"<p>Defines dependencies to be used before the loader has been loaded.\nWhen provided, they cause the loader to execute require(deps, callback) \nonce it has finished loading. Should be used with callback.</p>\n"},"dojoBlankHtmlUrl":{"!doc":"<p>Used by some modules to configure an empty iframe. Used by dojo/io/iframe and\ndojo/back, and dijit/popup support in IE where an iframe is needed to make sure native\ncontrols do not bleed through the popups. Normally this configuration variable\ndoes not need to be set, except when using cross-domain/CDN Dojo builds.\nSave dojo/resources/blank.html to your domain and set <code>djConfig.dojoBlankHtmlUrl</code>\nto the path on your domain your copy of blank.html.</p>\n"},"extraLocale":{"!doc":"<p>No default value. Specifies additional locales whose\nresources should also be loaded alongside the default locale when\ncalls to <code>dojo.requireLocalization()</code> are processed.</p>\n"},"ioPublish":{"!doc":"<p>Set this to true to enable publishing of topics for the different phases of\nIO operations. Publishing is done via dojo/topic.publish(). See dojo/main.__IoPublish for a list\nof topics that are published.</p>\n"},"isDebug":{"!doc":"<p>Defaults to <code>false</code>. If set to <code>true</code>, ensures that Dojo provides\nextended debugging feedback via Firebug. If Firebug is not available\non your platform, setting <code>isDebug</code> to <code>true</code> will force Dojo to\npull in (and display) the version of Firebug Lite which is\nintegrated into the Dojo distribution, thereby always providing a\ndebugging/logging console when <code>isDebug</code> is enabled. Note that\nFirebug&#39;s <code>console.*</code> methods are ALWAYS defined by Dojo. If\n<code>isDebug</code> is false and you are on a platform without Firebug, these\nmethods will be defined as no-ops.</p>\n"},"locale":{"!doc":"<p>The locale to assume for loading localized resources in this page,\nspecified according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a>.\nMust be specified entirely in lowercase, e.g. <code>en-us</code> and <code>zh-cn</code>.\nSee the documentation for <code>dojo.i18n</code> and <code>dojo.requireLocalization</code>\nfor details on loading localized resources. If no locale is specified,\nDojo assumes the locale of the user agent, according to <code>navigator.userLanguage</code>\nor <code>navigator.language</code> properties.</p>\n"},"modulePaths":{"!doc":"<p>A map of module names to paths relative to <code>dojo.baseUrl</code>. The\nkey/value pairs correspond directly to the arguments which\n<code>dojo.registerModulePath</code> accepts. Specifying\n<code>djConfig.modulePaths = { &quot;foo&quot;: &quot;../../bar&quot; }</code> is the equivalent\nof calling <code>dojo.registerModulePath(&quot;foo&quot;, &quot;../../bar&quot;);</code>. Multiple\nmodules may be configured via <code>djConfig.modulePaths</code>.</p>\n"},"parseOnLoad":{"!doc":"<p>Run the parser after the page is loaded</p>\n"},"require":{"!doc":"<p>An array of module names to be loaded immediately after dojo.js has been included\nin a page.</p>\n"},"transparentColor":{"!doc":"<p>Array containing the r, g, b components used as transparent color in dojo.Color;\nif undefined, [255,255,255] (white) will be used.</p>\n"},"urchin":{"!doc":"<p>Used by <code>dojox.analytics.Urchin</code> as the default UA-123456-7 account\nnumber used when being created. Alternately, you can pass an acct:&quot;&quot;\nparameter to the constructor a la: new dojox.analytics.Urchin({ acct:&quot;UA-123456-7&quot; });</p>\n"},"useCustomLogger":{"!doc":"<p>If set to a value that evaluates to true such as a string or array and\nisDebug is true and Firebug is not available or running, then it bypasses\nthe creation of Firebug Lite allowing you to define your own console object.</p>\n"},"useDeferredInstrumentation":{"!doc":"<p>Whether the deferred instrumentation should be used.\n\n</p>\n<ul>\n<li><code>&quot;report-rejections&quot;</code>: report each rejection as it occurs.</li>\n<li><code>true</code> or <code>1</code> or <code>&quot;report-unhandled-rejections&quot;</code>: wait 1 second\n  in an attempt to detect unhandled rejections.</li>\n</ul>\n"}},"dojo/main.global":{"!doc":"<p>Alias for the current window. &#39;global&#39; can be modified\nfor temporary context shifting. See also withGlobal().</p>\n","$":{"!type":"fn() -> ?"},"GoogleSearchStoreCallback_undefined_NaN":{"!type":"fn(start: ?, data: ?, responseCode: ?, errorMsg: ?)"},"jQuery":{"!type":"fn() -> ?"},"swfIsInHTML":{"!type":"fn()"},"undefined_onload":{"!type":"fn()"}},"dojo/main.dijit":{"_editor":{},"_tree":{},"form":{},"layout":{},"range":{"!doc":"<p>W3C range API</p>\n"},"registry":{},"tree":{},"byId":{"!type":"fn(id: ?) -> ?"},"Calendar":{"!type":"fn()"},"CalendarLite":{"!type":"fn()"},"CheckedMenuItem":{"!type":"fn()"},"ColorPalette":{"!type":"fn()"},"Declaration":{"!type":"fn()"},"Destroyable":{"!type":"fn()"},"Dialog":{"!type":"fn()"},"DialogUnderlay":{"!type":"fn()"},"DropDownMenu":{"!type":"fn()"},"Dye":{"!type":"fn()"},"Editor":{"!type":"fn()"},"Fieldset":{"!type":"fn()"},"InlineEditBox":{"!type":"fn()"},"Menu":{"!type":"fn()"},"MenuBar":{"!type":"fn()"},"MenuBarItem":{"!type":"fn()"},"MenuItem":{"!type":"fn()"},"MenuSeparator":{"!type":"fn()"},"PopupMenuBarItem":{"!type":"fn()"},"PopupMenuItem":{"!type":"fn()"},"ProgressBar":{"!type":"fn()"},"RadioButtonMenuItem":{"!type":"fn()"},"TitlePane":{"!type":"fn()"},"Toolbar":{"!type":"fn()"},"ToolbarSeparator":{"!type":"fn()"},"Tooltip":{"!type":"fn()"},"TooltipDialog":{"!type":"fn()"},"Tree":{"!type":"fn()"},"WidgetSet":{"!type":"fn()"}},"dojo/main.dojox":{"analytics":{},"app":{},"atom":{},"av":{},"BidiComplex":{},"calc":{},"calendar":{},"charting":{},"collections":{},"color":{},"css3":{},"data":{},"date":{},"dgauges":{},"dnd":{},"drawing":{},"dtl":{},"editor":{},"embed":{},"encoding":{},"enhanced":{},"flash":{"!doc":"<p>Utilities to embed and communicate with the Flash player from Javascript</p>\n"},"form":{},"fx":{},"gantt":{},"gauges":{},"geo":{},"gesture":{},"gfx":{},"gfx3d":{},"grid":{},"help":{"!doc":"<p>Adds the help function to all variables.</p>\n"},"highlight":{},"html":{},"image":{},"io":{},"jq":{},"json":{},"jsonPath":{},"lang":{},"layout":{},"math":{},"mdnd":{},"mobile":{},"mvc":{},"openlayers":{},"rails":{},"robot":{},"rpc":{},"secure":{},"sketch":{},"sql":{},"storage":{},"string":{},"testing":{},"timing":{},"treemap":{},"uuid":{},"validate":{},"widget":{},"xml":{},"socket":{"!type":"fn(argsOrUrl: ?) -> +any","!doc":"<p>Provides a simple socket connection using WebSocket, or alternate\ncommunication mechanisms in legacy browsers for comet-style communication. This is based\non the WebSocket API and returns an object that implements the WebSocket interface:\n<a href=\"http://dev.w3.org/html5/websockets/#websocket\">http://dev.w3.org/html5/websockets/#websocket</a></p>\n"},"sprintf":{"!type":"fn(format: ?, filler: ?)"}},"dojo/main.scopeMap":{"dijit":{},"dojo":{},"dojox":{}},"dojo/main.version":{"!doc":"<p>Version number of the Dojo Toolkit</p>\n","flag":{},"major":{},"minor":{},"patch":{},"revision":{},"toString":{"!type":"fn() -> string"}},"dojo/main._hasResource":{},"dojo/main.doc":{"!doc":"<p>Alias for the current document. &#39;doc&#39; can be modified\nfor temporary context shifting. See also withDoc().</p>\n","documentElement":{},"dojoClick":{}},"dojo/main.mouseButtons":{"LEFT":{"!doc":"<p>Numeric value of the left mouse button for the platform.</p>\n"},"MIDDLE":{"!doc":"<p>Numeric value of the middle mouse button for the platform.</p>\n"},"RIGHT":{"!doc":"<p>Numeric value of the right mouse button for the platform.</p>\n"},"isButton":{"!type":"fn(e: ?, button: ?) -> bool","!doc":"<p>Checks an event object for a pressed button</p>\n"},"isLeft":{"!type":"fn(e: ?) -> bool","!doc":"<p>Checks an event object for the pressed left button</p>\n"},"isMiddle":{"!type":"fn(e: ?) -> bool","!doc":"<p>Checks an event object for the pressed middle button</p>\n"},"isRight":{"!type":"fn(e: ?) -> bool","!doc":"<p>Checks an event object for the pressed right button</p>\n"}},"dojo/main.keys":{"!doc":"<p>Definitions for common key values.  Client code should test keyCode against these named constants,\nas the actual codes can vary by browser.</p>\n","ALT":{},"BACKSPACE":{},"CAPS_LOCK":{},"CLEAR":{},"copyKey":{},"CTRL":{},"DELETE":{},"DOWN_ARROW":{},"DOWN_DPAD":{},"END":{},"ENTER":{},"ESCAPE":{},"F1":{},"F10":{},"F11":{},"F12":{},"F13":{},"F14":{},"F15":{},"F2":{},"F3":{},"F4":{},"F5":{},"F6":{},"F7":{},"F8":{},"F9":{},"HELP":{},"HOME":{},"INSERT":{},"LEFT_ARROW":{},"LEFT_DPAD":{},"LEFT_WINDOW":{},"META":{},"NUM_LOCK":{},"NUMPAD_0":{},"NUMPAD_1":{},"NUMPAD_2":{},"NUMPAD_3":{},"NUMPAD_4":{},"NUMPAD_5":{},"NUMPAD_6":{},"NUMPAD_7":{},"NUMPAD_8":{},"NUMPAD_9":{},"NUMPAD_DIVIDE":{},"NUMPAD_ENTER":{},"NUMPAD_MINUS":{},"NUMPAD_MULTIPLY":{},"NUMPAD_PERIOD":{},"NUMPAD_PLUS":{},"PAGE_DOWN":{},"PAGE_UP":{},"PAUSE":{},"RIGHT_ARROW":{},"RIGHT_DPAD":{},"RIGHT_WINDOW":{},"SCROLL_LOCK":{},"SELECT":{},"SHIFT":{},"SPACE":{},"TAB":{},"UP_ARROW":{},"UP_DPAD":{}},"dojo/main._nodeDataCache":{},"dojo/main.fx":{"!doc":"<p>Effects library on top of Base animations</p>\n","easing":{"!doc":"<p>Collection of easing functions to use beyond the default\n<code>dojo._defaultEasing</code> function.</p>\n"},"chain":{"!type":"fn(animations: ?) -> +instance","!doc":"<p>Chain a list of <code>dojo/_base/fx.Animation</code>s to run in sequence\n</p>\n"},"combine":{"!type":"fn(animations: ?) -> +instance","!doc":"<p>Combine a list of <code>dojo/_base/fx.Animation</code>s to run in parallel\n</p>\n"},"slideTo":{"!type":"fn(args: ?) -> ?","!doc":"<p>Slide a node to a new top/left position\n</p>\n"},"Toggler":{"!type":"fn()"},"wipeIn":{"!type":"fn(args: ?) -> ?","!doc":"<p>Expand a node to it&#39;s natural height.\n</p>\n"},"wipeOut":{"!type":"fn(args: ?) -> ?","!doc":"<p>Shrink a node to nothing and hide it.\n</p>\n"}},"dojo/main.date":{"stamp":{"!doc":"<p>TODOC</p>\n"},"add":{"!type":"fn(date: ?, interval: ?, amount: ?) -> +instance","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2?: ?, portion?: ?) -> number","!doc":"<p>Compare two date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?) -> ?","!doc":"<p>Get the difference in a specific unit of time (e.g., number of\nmonths, weeks, days, etc.) between two dates, rounded to the\nnearest integer.</p>\n"},"getDaysInMonth":{"!type":"fn(dateObject: ?) -> number","!doc":"<p>Returns the number of days in the month used by dateObject</p>\n"},"getTimezoneName":{"!type":"fn(dateObject: ?) -> ?","!doc":"<p>Get the user&#39;s time zone as provided by the browser</p>\n"},"isLeapYear":{"!type":"fn(dateObject: ?) -> bool","!doc":"<p>Determines if the year of the dateObject is a leap year</p>\n"}},"dojo/main.html":{"!doc":"<p>TODOC</p>\n","set":{"!type":"fn(node: ?, cont: ?, params?: ?) -> ?","!doc":"<p>inserts (replaces) the given content into the given node. dojo/dom-construct.place(cont, node, &quot;only&quot;)\nmay be a better choice for simple HTML insertion.</p>\n"}},"dojo/main.Stateful":{"!type":"fn()","prototype":{"!doc":"<p>Base class for objects that provide named properties with optional getter/setter\ncontrol and the ability to watch for property changes\n\n</p>\n<p>The class also provides the functionality to auto-magically manage getters\nand setters for object attributes/properties.\n\n</p>\n<p>Getters and Setters should follow the format of _xxxGetter or _xxxSetter where \nthe xxx is a name of the attribute to handle.  So an attribute of &quot;foo&quot; \nwould have a custom getter of _fooGetter and a custom setter of _fooSetter.\n</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojo/main.contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n","auto":{"!type":"fn(xhr: ?)"},"javascript":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which evaluates the response data, expecting it to be valid JavaScript</p>\n"},"json":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which returns a JavaScript object created from the response data</p>\n"},"json-comment-filtered":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which expects comment-filtered JSON.</p>\n"},"json-comment-optional":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which checks the presence of comment-filtered JSON and\nalternates between the <code>json</code> and <code>json-comment-filtered</code> contentHandlers.</p>\n"},"olson-zoneinfo":{"!type":"fn(xhr: ?)"},"text":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which simply returns the plaintext response data</p>\n"},"xml":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler returning an XML Document parsed from the response data</p>\n"}},"dojo/main._contentHandlers":{"!doc":"<p>A map of available XHR transport handle types. Name matches the\n<code>handleAs</code> attribute passed to XHR calls.</p>\n","auto":{"!type":"fn(xhr: ?)"},"javascript":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which evaluates the response data, expecting it to be valid JavaScript</p>\n"},"json":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which returns a JavaScript object created from the response data</p>\n"},"json-comment-filtered":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which expects comment-filtered JSON.</p>\n"},"json-comment-optional":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which checks the presence of comment-filtered JSON and\nalternates between the <code>json</code> and <code>json-comment-filtered</code> contentHandlers.</p>\n"},"olson-zoneinfo":{"!type":"fn(xhr: ?)"},"text":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler which simply returns the plaintext response data</p>\n"},"xml":{"!type":"fn(xhr: ?) -> ?","!doc":"<p>A contentHandler returning an XML Document parsed from the response data</p>\n"}},"dojo/main.__IoArgs":{"!type":"fn()","prototype":{"content":{"!doc":"<p>Contains properties with string values. These\nproperties will be serialized as name1=value2 and\npassed in the request.</p>\n"},"form":{"!doc":"<p>DOM node for a form. Used to extract the form values\nand send to the server.</p>\n"},"handleAs":{"!doc":"<p>Acceptable values depend on the type of IO\ntransport (see specific IO calls for more information).</p>\n"},"ioPublish":{"!doc":"<p>Set this explicitly to false to prevent publishing of topics related to\nIO operations. Otherwise, if djConfig.ioPublish is set to true, topics\nwill be published via dojo/topic.publish() for different phases of an IO operation.\nSee dojo/main.__IoPublish for a list of topics that are published.</p>\n"},"preventCache":{"!doc":"<p>Default is false. If true, then a\n&quot;dojo.preventCache&quot; parameter is sent in the request\nwith a value that changes with each request\n(timestamp). Useful only with GET-type requests.</p>\n"},"rawBody":{"!doc":"<p>Sets the raw body for an HTTP request. If this is used, then the content\nproperty is ignored. This is mostly useful for HTTP methods that have\na body to their requests, like PUT or POST. This property can be used instead\nof postData and putData for dojo/_base/xhr.rawXhrPost and dojo/_base/xhr.rawXhrPut respectively.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response. If this time\npasses, the then error callbacks are called.</p>\n"},"url":{"!doc":"<p>URL to server endpoint.</p>\n"},"error":{"!type":"fn(response: ?, ioArgs: ?)","!doc":"<p>This function will\nbe called when the request fails due to a network or server error, the url\nis invalid, etc. It will also be called if the load or handle callback throws an\nexception, unless djConfig.debugAtAllCosts is true.  This allows deployed applications\nto continue to run even when a logic error happens in the callback, while making\nit easier to troubleshoot while in debug mode.</p>\n"},"handle":{"!type":"fn(loadOrError: ?, response: ?, ioArgs: ?)","!doc":"<p>This function will\nbe called at the end of every request, whether or not an error occurs.</p>\n"},"load":{"!type":"fn(response: ?, ioArgs: ?)","!doc":"<p>This function will be\ncalled on a successful HTTP response code.</p>\n"}}},"dojo/main.__IoCallbackArgs":{"!type":"fn()","prototype":{"args":{"!doc":"<p>the original object argument to the IO call.</p>\n"},"canDelete":{"!doc":"<p>For dojo/io/script calls only, indicates\nwhether the script tag that represents the\nrequest can be deleted after callbacks have\nbeen called. Used internally to know when\ncleanup can happen on JSONP-type requests.</p>\n"},"handleAs":{"!doc":"<p>The final indicator on how the response will be\nhandled.</p>\n"},"id":{"!doc":"<p>For dojo/io/script calls only, the internal\nscript ID used for the request.</p>\n"},"json":{"!doc":"<p>For dojo/io/script calls only: holds the JSON\nresponse for JSONP-type requests. Used\ninternally to hold on to the JSON responses.\nYou should not need to access it directly --\nthe same object should be passed to the success\ncallbacks directly.</p>\n"},"query":{"!doc":"<p>For non-GET requests, the\nname1=value1&amp;name2=value2 parameters sent up in\nthe request.</p>\n"},"url":{"!doc":"<p>The final URL used for the call. Many times it\nwill be different than the original args.url\nvalue.</p>\n"},"xhr":{"!doc":"<p>For XMLHttpRequest calls only, the\nXMLHttpRequest object that was used for the\nrequest.</p>\n"}}},"dojo/main.__IoPublish":{"!type":"fn()","prototype":{"!doc":"<p>This is a list of IO topics that can be published\nif djConfig.ioPublish is set to true. IO topics can be\npublished for any Input/Output, network operation. So,\ndojo.xhr, dojo.io.script and dojo.io.iframe can all\ntrigger these topics to be published.</p>\n","done":{"!doc":"<p>&quot;/dojo/io/done&quot; is sent whenever an IO request has completed,\neither by loading or by erroring. It passes the error and\nthe dojo.Deferred for the request with the topic.</p>\n"},"error":{"!doc":"<p>&quot;/dojo/io/error&quot; is sent whenever an IO request has errored.\nIt passes the error and the dojo.Deferred\nfor the request with the topic.</p>\n"},"load":{"!doc":"<p>&quot;/dojo/io/load&quot; is sent whenever an IO request has loaded\nsuccessfully. It passes the response and the dojo.Deferred\nfor the request with the topic.</p>\n"},"send":{"!doc":"<p>&quot;/dojo/io/send&quot; is sent whenever a new IO request is started.\nIt passes the dojo.Deferred for the request with the topic.</p>\n"},"start":{"!doc":"<p>&quot;/dojo/io/start&quot; is sent when there are no outstanding IO\nrequests, and a new IO request is started. No arguments\nare passed with this topic.</p>\n"},"stop":{"!doc":"<p>&quot;/dojo/io/stop&quot; is sent when all outstanding IO requests have\nfinished. No arguments are passed with this topic.</p>\n"}}},"dojo/main.__XhrArgs":{"!type":"fn()","prototype":{"!doc":"<p>In addition to the properties listed for the dojo._IoArgs type,\nthe following properties are allowed for dojo.xhr* methods.</p>\n","content":{"!doc":"<p>Contains properties with string values. These\nproperties will be serialized as name1=value2 and\npassed in the request.</p>\n"},"contentType":{"!doc":"<p>&quot;application/x-www-form-urlencoded&quot; is default. Set to false to\nprevent a Content-Type header from being sent, or to a string\nto send a different Content-Type.</p>\n"},"failOk":{"!doc":"<p>false is default. Indicates whether a request should be\nallowed to fail (and therefore no console error message in\nthe event of a failure)</p>\n"},"form":{"!doc":"<p>DOM node for a form. Used to extract the form values\nand send to the server.</p>\n"},"handleAs":{"!doc":"<p>Acceptable values are: text (default), json, json-comment-optional,\njson-comment-filtered, javascript, xml. See <code>dojo/_base/xhr.contentHandlers</code></p>\n"},"headers":{"!doc":"<p>Additional HTTP headers to send in the request.</p>\n"},"ioPublish":{"!doc":"<p>Set this explicitly to false to prevent publishing of topics related to\nIO operations. Otherwise, if djConfig.ioPublish is set to true, topics\nwill be published via dojo/topic.publish() for different phases of an IO operation.\nSee dojo/main.__IoPublish for a list of topics that are published.</p>\n"},"preventCache":{"!doc":"<p>Default is false. If true, then a\n&quot;dojo.preventCache&quot; parameter is sent in the request\nwith a value that changes with each request\n(timestamp). Useful only with GET-type requests.</p>\n"},"rawBody":{"!doc":"<p>Sets the raw body for an HTTP request. If this is used, then the content\nproperty is ignored. This is mostly useful for HTTP methods that have\na body to their requests, like PUT or POST. This property can be used instead\nof postData and putData for dojo/_base/xhr.rawXhrPost and dojo/_base/xhr.rawXhrPut respectively.</p>\n"},"sync":{"!doc":"<p>false is default. Indicates whether the request should\nbe a synchronous (blocking) request.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response. If this time\npasses, the then error callbacks are called.</p>\n"},"url":{"!doc":"<p>URL to server endpoint.</p>\n"},"error":{"!type":"fn(response: ?, ioArgs: ?)","!doc":"<p>This function will\nbe called when the request fails due to a network or server error, the url\nis invalid, etc. It will also be called if the load or handle callback throws an\nexception, unless djConfig.debugAtAllCosts is true.  This allows deployed applications\nto continue to run even when a logic error happens in the callback, while making\nit easier to troubleshoot while in debug mode.</p>\n"},"handle":{"!type":"fn(loadOrError: ?, response: ?, ioArgs: ?)","!doc":"<p>This function will\nbe called at the end of every request, whether or not an error occurs.</p>\n"},"load":{"!type":"fn(response: ?, ioArgs: ?)","!doc":"<p>This function will be\ncalled on a successful HTTP response code.</p>\n"}}},"dojo/main.back":{"!doc":"<p>Browser history management resources</p>\n","addToHistory":{"!type":"fn(args: ?)","!doc":"<p>adds a state object (args) to the history list.</p>\n"},"getHash":{"!type":"fn() -> ?"},"goBack":{"!type":"fn()","!doc":"<p>private method. Do not call this directly.</p>\n"},"goForward":{"!type":"fn()","!doc":"<p>private method. Do not call this directly.</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initializes the undo stack. This must be called from a <script>\nblock that lives inside the <code>&lt;body&gt;</code> tag to prevent bugs on IE.\n\n</p>\n<p>Only call this method before the page&#39;s DOM is finished loading. Otherwise\nit will not work. Be careful with xdomain loading or djConfig.debugAtAllCosts scenarios,\nin order for this method to work, dojo/back will need to be part of a build layer.</p>\n"},"setHash":{"!type":"fn(h: ?)"},"setInitialState":{"!type":"fn(args: ?)","!doc":"<p>Sets the state object and back callback for the very first page\nthat is loaded.\n\n</p>\n<p>It is recommended that you call this method as part of an event\nlistener that is registered via dojo/ready.</p>\n"}},"dojo/main.cldr":{"monetary":{"!doc":"<p>TODOC</p>\n"},"supplemental":{"!doc":"<p>TODOC</p>\n"}},"dojo/main.i18n":{"!doc":"<p>This module implements the dojo/i18n! plugin and the v1.6- i18n API</p>\n","cache":{},"dynamic":{},"unitTests":{},"getL10nName":{"!type":"fn(moduleName: ?, bundleName: ?, locale: ?) -> string"},"getLocalization":{"!type":"fn(moduleName: ?, bundleName: ?, locale: ?) -> ?"},"load":{"!type":"fn(id: ?, require: ?, load: ?)","!doc":"<p>id is in one of the following formats\n\n</p>\n<ol>\n<li><p><path>/nls/<bundle>\n =&gt; load the bundle, localized to config.locale; load all bundles localized to\n config.extraLocale (if any); return the loaded bundle localized to config.locale.</p>\n</li>\n<li><p><path>/nls/<locale>/<bundle>\n =&gt; load then return the bundle localized to <locale></p>\n</li>\n<li><p><em>preload</em><path>/nls/<module>*<JSON array of available locales>\n =&gt; for config.locale and all config.extraLocale, load all bundles found\n in the best-matching bundle rollup. A value of 1 is returned, which\n is meaningless other than to say the plugin is executing the requested\n preloads</p>\n</li>\n</ol>\n<p>In cases 1 and 2, <path> is always normalized to an absolute module id upon entry; see\nnormalize. In case 3, it <path> is assumed to be absolute; this is arranged by the builder.\n\n</p>\n<p>To load a bundle means to insert the bundle into the plugin&#39;s cache and publish the bundle\nvalue to the loader. Given <path>, <bundle>, and a particular <locale>, the cache key\n\n</p>\n<pre><code>&lt;path&gt;/nls/&lt;bundle&gt;/&lt;locale&gt;</code></pre>\n<p>will hold the value. Similarly, then plugin will publish this value to the loader by\n\n</p>\n<pre><code>define(&quot;&lt;path&gt;/nls/&lt;bundle&gt;/&lt;locale&gt;&quot;, &lt;bundle-value&gt;);</code></pre>\n<p>Given this algorithm, other machinery can provide fast load paths be preplacing\nvalues in the plugin&#39;s cache, which is public. When a load is demanded the\ncache is inspected before starting any loading. Explicitly placing values in the plugin\ncache is an advanced/experimental feature that should not be needed; use at your own risk.\n\n</p>\n<p>For the normal AMD algorithm, the root bundle is loaded first, which instructs the\nplugin what additional localized bundles are required for a particular locale. These\nadditional locales are loaded and a mix of the root and each progressively-specific\nlocale is returned. For example:\n\n</p>\n<ol>\n<li><p>The client demands &quot;dojo/i18n!some/path/nls/someBundle</p>\n</li>\n<li><p>The loader demands load(some/path/nls/someBundle)</p>\n</li>\n<li><p>This plugin require&#39;s &quot;some/path/nls/someBundle&quot;, which is the root bundle.</p>\n</li>\n<li><p>Assuming config.locale is &quot;ab-cd-ef&quot; and the root bundle indicates that localizations\nare available for &quot;ab&quot; and &quot;ab-cd-ef&quot; (note the missing &quot;ab-cd&quot;, then the plugin\nrequires &quot;some/path/nls/ab/someBundle&quot; and &quot;some/path/nls/ab-cd-ef/someBundle&quot;</p>\n</li>\n<li><p>Upon receiving all required bundles, the plugin constructs the value of the bundle\nab-cd-ef as...</p>\n<pre><code> mixin(mixin(mixin({}, require(&quot;some/path/nls/someBundle&quot;),\n     require(&quot;some/path/nls/ab/someBundle&quot;)),\n     require(&quot;some/path/nls/ab-cd-ef/someBundle&quot;));</code></pre>\n</li>\n</ol>\n<p>This value is inserted into the cache and published to the loader at the\nkey/module-id some/path/nls/someBundle/ab-cd-ef.\n\n</p>\n<p>The special preload signature (case 3) instructs the plugin to stop servicing all normal requests\n(further preload requests will be serviced) until all ongoing preloading has completed.\n\n</p>\n<p>The preload signature instructs the plugin that a special rollup module is available that contains\none or more flattened, localized bundles. The JSON array of available locales indicates which locales\nare available. Here is an example:\n\n</p>\n<pre><code>*preload*some/path/nls/someModule*[&quot;root&quot;, &quot;ab&quot;, &quot;ab-cd-ef&quot;]</code></pre>\n<p>This indicates the following rollup modules are available:\n\n</p>\n<pre><code>some/path/nls/someModule_ROOT\nsome/path/nls/someModule_ab\nsome/path/nls/someModule_ab-cd-ef</code></pre>\n<p>Each of these modules is a normal AMD module that contains one or more flattened bundles in a hash.\nFor example, assume someModule contained the bundles some/bundle/path/someBundle and\nsome/bundle/path/someOtherBundle, then some/path/nls/someModule_ab would be expressed as follows:\n\n</p>\n<pre><code>define({\n    some/bundle/path/someBundle:&lt;value of someBundle, flattened with respect to locale ab&gt;,\n    some/bundle/path/someOtherBundle:&lt;value of someOtherBundle, flattened with respect to locale ab&gt;,\n});</code></pre>\n<p>E.g., given this design, preloading for locale==&quot;ab&quot; can execute the following algorithm:\n\n</p>\n<pre><code>require([&quot;some/path/nls/someModule_ab&quot;], function(rollup){\n    for(var p in rollup){\n        var id = p + &quot;/ab&quot;,\n        cache[id] = rollup[p];\n        define(id, rollup[p]);\n    }\n});</code></pre>\n<p>Similarly, if &quot;ab-cd&quot; is requested, the algorithm can determine that &quot;ab&quot; is the best available and\nload accordingly.\n\n</p>\n<p>The builder will write such rollups for every layer if a non-empty localeList  profile property is\nprovided. Further, the builder will include the following cache entry in the cache associated with\nany layer.\n\n</p>\n<pre><code>&quot;*now&quot;:function(r){r([&#39;dojo/i18n!*preload*&lt;path&gt;/nls/&lt;module&gt;*&lt;JSON array of available locales&gt;&#39;]);}</code></pre>\n<p>The *now special cache module instructs the loader to apply the provided function to context-require\nwith respect to the particular layer being defined. This causes the plugin to hold all normal service\nrequests until all preloading is complete.\n\n</p>\n<p>Notice that this algorithm is rarely better than the standard AMD load algorithm. Consider the normal case\nwhere the target locale has a single segment and a layer depends on a single bundle:\n\n</p>\n<p>Without Preloads:\n\n</p>\n<ol>\n<li>Layer loads root bundle.</li>\n<li>bundle is demanded; plugin loads single localized bundle.</li>\n</ol>\n<p>With Preloads:\n\n</p>\n<ol>\n<li>Layer causes preloading of target bundle.</li>\n<li>bundle is demanded; service is delayed until preloading complete; bundle is returned.</li>\n</ol>\n<p>In each case a single transaction is required to load the target bundle. In cases where multiple bundles\nare required and/or the locale has multiple segments, preloads still requires a single transaction whereas\nthe normal path requires an additional transaction for each additional bundle/locale-segment. However all\nof these additional transactions can be done concurrently. Owing to this analysis, the entire preloading\nalgorithm can be discard during a build by setting the has feature dojo-preload-i18n-Api to false.</p>\n"},"normalize":{"!type":"fn(id: ?, toAbsMid: ?) -> ?","!doc":"<p>id may be relative.\npreload has form <code>*preload*&lt;path&gt;/nls/&lt;module&gt;*&lt;flattened locales&gt;</code> and\ntherefore never looks like a relative</p>\n"},"normalizeLocale":{"!type":"fn(locale: ?) -> ?"}},"dojo/main.colors":{"makeGrey":{"!type":"fn(g: ?, a?: ?)","!doc":"<p>creates a greyscale color with an optional alpha</p>\n"}},"dojo/main.regexp":{"!doc":"<p>Regular expressions and Builder resources</p>\n","buildGroupRE":{"!type":"fn(arr: ?, re: ?, nonCapture?: ?) -> ?","!doc":"<p>Builds a regular expression that groups subexpressions</p>\n"},"escapeString":{"!type":"fn(str: ?, except?: ?) -> ?","!doc":"<p>Adds escape sequences for special characters in regular expressions</p>\n"},"group":{"!type":"fn(expression: ?, nonCapture?: ?) -> string","!doc":"<p>adds group match to expression</p>\n"}},"dojo/main.string":{"!doc":"<p>String utilities for Dojo</p>\n","pad":{"!type":"fn(text: ?, size: ?, ch?: ?, end?: ?) -> number","!doc":"<p>Pad a string to guarantee that it is at least <code>size</code> length by\nfilling with the character <code>ch</code> at either the start or end of the\nstring. Pads at the start, by default.</p>\n"},"rep":{"!type":"fn(str: ?, num: ?) -> string","!doc":"<p>Efficiently replicate a string <code>n</code> times.</p>\n"},"substitute":{"!type":"fn(template: ?, map: ?, transform?: ?, thisObject?: ?) -> ?","!doc":"<p>Performs parameterized substitutions on a string. Throws an\nexception if any parameter is unmatched.</p>\n"},"trim":{"!type":"fn(str: ?) -> string","!doc":"<p>Trims whitespace from both sides of the string</p>\n"}},"dojo/main.number":{"!doc":"<p>localized formatting and parsing routines for Number</p>\n","_numberPatternRE":{},"format":{"!type":"fn(value: ?, options?: ?) -> +null","!doc":"<p>Format a Number as a String, using locale-specific settings</p>\n"},"parse":{"!type":"fn(expression: ?, options?: ?) -> number","!doc":"<p>Convert a properly formatted string to a primitive Number, using\nlocale-specific settings.</p>\n"},"regexp":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a number</p>\n"},"round":{"!type":"fn(value: ?, places?: ?, increment?: ?) -> number","!doc":"<p>Rounds to the nearest value with the given number of decimal places, away from zero</p>\n"}},"dojo/main.currency":{"!doc":"<p>localized formatting and parsing routines for currencies</p>\n","format":{"!type":"fn(value: ?, options?: ?) -> ?","!doc":"<p>Format a Number as a currency, using locale-specific settings\n</p>\n"},"parse":{"!type":"fn(expression: ?, options?: ?) -> ?"},"regexp":{"!type":"fn(options?: ?) -> ?"}},"dojo/main.data":{"api":{},"util":{},"ItemFileReadStore":{"!type":"fn()"},"ItemFileWriteStore":{"!type":"fn()"},"ObjectStore":{"!type":"fn()"}},"dojo/main.dnd":{"autoscroll":{"!doc":"<p>Used by dojo/dnd/Manager to scroll document or internal node when the user\ndrags near the edge of the viewport or a scrollable node</p>\n"},"move":{},"AutoSource":{"!type":"fn()"},"Avatar":{"!type":"fn()"},"Container":{"!type":"fn()"},"Manager":{"!type":"fn()"},"Moveable":{"!type":"fn()"},"Mover":{"!type":"fn()"},"Selector":{"!type":"fn()"},"Source":{"!type":"fn()"},"Target":{"!type":"fn()"},"TimedMoveable":{"!type":"fn()"}},"dojo/main.touch":{"!doc":"<p>This module provides unified touch event handlers by exporting\npress, move, release and cancel which can also run well on desktop.\nBased on <a href=\"http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html\">http://dvcs.w3.org/hg/webevents/raw-file/tip/touchevents.html</a>\nAlso, if the dojoClick property is set to truthy on a DOM node, dojo/touch generates\nclick events immediately for this node and its descendants (except for descendants that\nhave a dojoClick property set to falsy), to avoid the delay before native browser click events,\nand regardless of whether evt.preventDefault() was called in a touch.press event listener.\n</p>\n","cancel":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;touchcancel&#39;|&#39;mouseleave&#39; for the given node</p>\n"},"enter":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to mouse.enter or touch equivalent for the given node</p>\n"},"leave":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to mouse.leave or touch equivalent for the given node</p>\n"},"move":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener that fires when the mouse cursor or a finger is dragged over the given node.</p>\n"},"out":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;mouseout&#39; or touch equivalent for the given node</p>\n"},"over":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;mouseover&#39; or touch equivalent for the given node</p>\n"},"press":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to &#39;touchstart&#39;|&#39;mousedown&#39; for the given node</p>\n"},"release":{"!type":"fn(node: ?, listener: ?) -> +any","!doc":"<p>Register a listener to releasing the mouse button while the cursor is over the given node\n(i.e. &quot;mouseup&quot;) or for removing the finger from the screen while touching the given node.</p>\n"}},"dojo/main.window":{"!doc":"<p>TODOC</p>\n","get":{"!type":"fn(doc: ?) -> ?","!doc":"<p>Get window object associated with document doc.</p>\n"},"getBox":{"!type":"fn(doc?: ?) -> ?","!doc":"<p>Returns the dimensions and scroll position of the viewable area of a browser window</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, pos?: ?)","!doc":"<p>Scroll the passed node into view using minimal movement, if it is not already.</p>\n"}},"dojo/main.gears":{"!doc":"<p>TODOC</p>\n","available":{"!doc":"<p>True if client is using Google Gears</p>\n"}},"dojo/main.io":{"iframe":{},"script":{"!doc":"<p>TODOC</p>\n"}},"dojo/main.rpc":{"JsonpService":{"!type":"fn()"},"JsonService":{"!type":"fn()"},"RpcService":{"!type":"fn()"}},"dojo/main.store":{"util":{},"Cache":{"!type":"fn(masterStore: ?, cachingStore: ?, options: ?) -> ?"},"DataStore":{"!type":"fn()"},"JsonRest":{"!type":"fn()"},"Memory":{"!type":"fn()"},"Observable":{"!type":"fn(store: ?) -> ?","!doc":"<p>The Observable store wrapper takes a store and sets an observe method on query()\nresults that can be used to monitor results for changes.\n</p>\n"}},"dojo/main.tests":{"!doc":"<p>D.O.H. Test files for Dojo unit testing.</p>\n"},"dojo/node":{"!doc":"<p>This AMD plugin module allows native Node.js modules to be loaded by AMD modules using the Dojo\nloader. Note that this plugin will not work with AMD loaders other than the Dojo loader.</p>\n","load":{"!type":"fn(id: ?, require: ?, load: ?)","!doc":"<p>Standard AMD plugin interface. See <a href=\"https://github.com/amdjs/amdjs-api/wiki/Loader-Plugins\">https://github.com/amdjs/amdjs-api/wiki/Loader-Plugins</a>\nfor information.</p>\n"},"normalize":{"!type":"fn(id: ?, normalize: ?) -> ?","!doc":"<p>Produces a normalized id to be used by node.  Relative ids are resolved relative to the requesting\nmodule&#39;s location in the file system and will return an id with path separators appropriate for the\nlocal file system.</p>\n"}},"dojo/promise/first":{"!doc":"<p>Takes multiple promises and returns a new promise that is fulfilled\nwhen the first of these promises is fulfilled.</p>\n","prototype":{"constructor":{"!type":"fn(objectOrArray?: ?) -> +dojo","!doc":"<p>Takes multiple promises and returns a new promise that is fulfilled\nwhen the first of these promises is fulfilled.</p>\n"}}},"dojo/promise/instrumentation":{"!doc":"<p>Initialize instrumentation for the Deferred class.</p>\n","prototype":{"constructor":{"!type":"fn(Deferred: ?)","!doc":"<p>Initialize instrumentation for the Deferred class.</p>\n"}}},"dojo/promise/tracer":{"!doc":"<p>Trace promise fulfillment.</p>\n","on":{"!type":"fn(type: ?, listener: ?)","!doc":"<p>Subscribe to traces.</p>\n"}},"dojo/request/default":{"getPlatformDefaultId":{"!type":"fn()"},"load":{"!type":"fn(id: ?, parentRequire: ?, loaded: ?, config: ?)"}},"dojo/request/node":{"!doc":"<p>Sends a request using the included http or https interface from node.js\nwith the given URL and options.</p>\n","prototype":{"constructor":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Sends a request using the included http or https interface from node.js\nwith the given URL and options.</p>\n"}},"del":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP DELETE request using XMLHttpRequest with the given URL and options.</p>\n"},"get":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP GET request using XMLHttpRequest with the given URL and options.</p>\n"},"post":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP POST request using XMLHttpRequest with the given URL and options.</p>\n"},"put":{"!type":"fn(url: ?, options?: ?) -> +dojo","!doc":"<p>Send an HTTP PUT request using XMLHttpRequest with the given URL and options.</p>\n"}},"dojo/request/node.__BaseOptions":{"!type":"fn()","prototype":{"data":{"!doc":"<p>Data to transfer. This is ignored for GET and DELETE\nrequests.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"headers":{"!doc":"<p>Headers to use for the request.</p>\n"},"password":{"!doc":"<p>Password to use during the request.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"},"user":{"!doc":"<p>Username to use during the request.</p>\n"}}},"dojo/request/node.__MethodOptions":{"!type":"fn()","prototype":{"method":{"!doc":"<p>The HTTP method to use to make the request. Must be\nuppercase. Default is <code>&quot;GET&quot;</code>.</p>\n"}}},"dojo/request/node.__Options":{"!type":"fn()","prototype":{"data":{"!doc":"<p>Data to transfer. This is ignored for GET and DELETE\nrequests.</p>\n"},"handleAs":{"!doc":"<p>How to handle the response from the server.  Default is\n&#39;text&#39;.  Other values are &#39;json&#39;, &#39;javascript&#39;, and &#39;xml&#39;.</p>\n"},"headers":{"!doc":"<p>Headers to use for the request.</p>\n"},"method":{"!doc":"<p>The HTTP method to use to make the request. Must be\nuppercase. Default is <code>&quot;GET&quot;</code>.</p>\n"},"password":{"!doc":"<p>Password to use during the request.</p>\n"},"preventCache":{"!doc":"<p>Whether to append a cache-busting parameter to the URL.</p>\n"},"query":{"!doc":"<p>Query parameters to append to the URL.</p>\n"},"timeout":{"!doc":"<p>Milliseconds to wait for the response.  If this time\npasses, the then the promise is rejected.</p>\n"},"user":{"!doc":"<p>Username to use during the request.</p>\n"}}},"dojo/request/notify":{"!doc":"<p>Register a listener to be notified when an event\nin dojo/request happens.</p>\n","prototype":{"constructor":{"!type":"fn(type?: ?, listener?: ?) -> +any","!doc":"<p>Register a listener to be notified when an event\nin dojo/request happens.</p>\n"}},"emit":{"!type":"fn(type: ?, event: ?, cancel: ?)"}},"dojo/request/registry":{"prototype":{"constructor":{"!type":"fn(url: ?, options: ?) -> ?"}},"load":{"!type":"fn(id: ?, parentRequire: ?, loaded: ?, config: ?)"},"register":{"!type":"fn(url: ?, provider: ?, first: ?)"}},"dojo/require":{"dynamic":{},"load":{},"normalize":{"!type":"fn(id: ?) -> ?"}},"dojo/robot":{"_appletDead":{},"_loaded":{"!doc":"<p>Deferred that resolves when the _initRobot() has been called.\nNote to be confused with dojo/robotx.js, which defines initRobot() without an underscore</p>\n"},"_primePump":{},"_robotInitialized":{},"_robotLoaded":{},"_runsemaphore":{},"_spaceReceived":{},"_started":{"!doc":"<p>Deferred that resolves when startRobot() has signaled completing by typing on the keyboard,\nwhich in turn calls _run().</p>\n"},"doc":{},"mouseWheelSize":{},"window":{},"initRobot":{"!type":"fn(url: ?)","!doc":"<p>Opens the application at the specified URL for testing, redirecting dojo to point to the application\nenvironment instead of the test environment.</p>\n"},"keyDown":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Holds down a single key, like SHIFT or &#39;a&#39;.</p>\n"},"keyPress":{"!type":"fn(charOrCode: ?, delay?: ?, modifiers: ?, asynchronous: ?)","!doc":"<p>Types a key combination, like SHIFT-TAB.</p>\n"},"keyUp":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Releases a single key, like SHIFT or &#39;a&#39;.</p>\n"},"killRobot":{"!type":"fn()"},"mouseClick":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Convenience function to do a press/release.\nSee robot.mousePress for more info.</p>\n"},"mouseMove":{"!type":"fn(x: ?, y: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Moves the mouse to the specified x,y offset relative to the viewport.</p>\n"},"mouseMoveAt":{"!type":"fn(node: ?, delay: ?, duration: ?, offsetX: ?, offsetY: ?)","!doc":"<p>Moves the mouse over the specified node at the specified relative x,y offset.</p>\n"},"mouseMoveTo":{"!type":"fn(point: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Move the mouse from the current position to the specified point.\nDelays reading contents point until queued command starts running.\nSee mouseMove() for details.</p>\n"},"mousePress":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Presses mouse buttons.</p>\n"},"mouseRelease":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Releases mouse buttons.</p>\n"},"mouseWheel":{"!type":"fn(wheelAmt: ?, delay?: ?, duration?: ?)","!doc":"<p>Spins the mouse wheel.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, delay: ?)","!doc":"<p>Scroll the passed node into view, if it is not.</p>\n"},"sequence":{"!type":"fn(f: ?, delay?: ?, duration?: ?)","!doc":"<p>Defer an action by adding it to the robot&#39;s incrementally delayed queue of actions to execute.</p>\n"},"setClipboard":{"!type":"fn(data: ?, format?: ?)","!doc":"<p>Set clipboard content.</p>\n"},"startRobot":{"!type":"fn() -> ?"},"typeKeys":{"!type":"fn(chars: ?, delay?: ?, duration?: ?)","!doc":"<p>Types a string of characters in order, or types a dojo.keys.* constant.</p>\n"},"waitForPageToLoad":{"!type":"fn(submitActions: ?) -> +instance","!doc":"<p>Notifies DOH that the doh.robot is about to make a page change in the application it is driving,\nreturning a doh.Deferred object the user should return in their runTest function as part of a DOH test.</p>\n"}},"dojo/robot._runsemaphore":{"lock":{},"unlock":{"!type":"fn() -> ?"}},"doh/_browserRunner":{"!doc":"<p>Functions for registering and running automated tests.</p>\n","_currentGroup":{},"_currentTest":{},"_errorCount":{},"_failureCount":{},"_groupCount":{},"_groups":{},"_init":{},"_initRobotCalled":{},"_line":{},"_paused":{},"_registerUrl":{},"_setupGroupForRun":{},"_testCount":{},"_testTypes":{},"debug":{},"error":{},"isBrowser":{},"registerUrl":{},"robot":{},"assertEqual":{"!type":"fn(expected: ?, actual: ?, hint?: ?, doNotThrow: ?)","!doc":"<p>are the passed expected and actual objects/values deeply\nequivalent?</p>\n"},"assertError":{"!type":"fn(expectedError: ?, scope: ?, functionName: ?, args: ?, hint?: ?)","!doc":"<p>Test for a certain error to be thrown by the given function.</p>\n"},"assertFalse":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;falsey&quot;?</p>\n"},"assertNotEqual":{"!type":"fn(notExpected: ?, actual: ?, hint?: ?)","!doc":"<p>are the passed notexpected and actual objects/values deeply\nnot equivalent?</p>\n"},"assertTrue":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;truthy&quot;?</p>\n"},"Deferred":{"!type":"fn(canceller: ?)"},"e":{"!type":"fn(expectedError: ?, scope: ?, functionName: ?, args: ?, hint?: ?)","!doc":"<p>Test for a certain error to be thrown by the given function.</p>\n"},"f":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;falsey&quot;?</p>\n"},"is":{"!type":"fn(expected: ?, actual: ?, hint?: ?, doNotThrow: ?)","!doc":"<p>are the passed expected and actual objects/values deeply\nequivalent?</p>\n"},"isNot":{"!type":"fn(notExpected: ?, actual: ?, hint?: ?)","!doc":"<p>are the passed notexpected and actual objects/values deeply\nnot equivalent?</p>\n"},"pause":{"!type":"fn()","!doc":"<p>halt test run. Can be resumed.</p>\n"},"register":{"!type":"fn(a1: ?, a2: ?, a3: ?, a4: ?, a5: ?)"},"registerDocTests":{"!type":"fn(module: ?)","!doc":"<p>Deprecated.    Won&#39;t work unless you manually load dojox.testing.DocTest, and likely not even then.\nGets all the doctests from the given module and register each of them as a single test case here.</p>\n"},"registerGroup":{"!type":"fn(group: ?, tests: ?, setUp: ?, tearDown: ?, type: ?)","!doc":"<p>Deprecated.  Use doh.register(group/type, tests, setUp, tearDown) instead</p>\n"},"registerTest":{"!type":"fn(group: ?, test: ?, type: ?)","!doc":"<p>Deprecated.  Use doh.register(group/type, test) instead</p>\n"},"registerTestNs":{"!type":"fn(group: ?, ns: ?)","!doc":"<p>Deprecated.  Use doh.register(group, ns) instead</p>\n"},"registerTests":{"!type":"fn(group: ?, testArr: ?, type: ?)","!doc":"<p>Deprecated.  Use doh.register(group/type, testArr) instead</p>\n"},"registerTestType":{"!type":"fn(name: ?, initProc: ?)","!doc":"<p>Adds a test type and associates a function used to initialize each test of the given type</p>\n"},"run":{"!type":"fn()"},"runGroup":{"!type":"fn(groupName: ?, idx: ?)","!doc":"<p>runs the specified test group</p>\n"},"runOnLoad":{"!type":"fn()"},"showLogPage":{"!type":"fn()"},"showPerfTestsPage":{"!type":"fn()"},"showTestPage":{"!type":"fn()"},"t":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;truthy&quot;?</p>\n"},"togglePaused":{"!type":"fn()"},"toggleRunAll":{"!type":"fn()"}},"doh/_browserRunner._groups":{},"doh/_browserRunner._testTypes":{"perf":{"!type":"fn(group: ?, tObj: ?, type: ?)"}},"doh/_browserRunner.robot":{"_appletDead":{},"_loaded":{"!doc":"<p>Deferred that resolves when the _initRobot() has been called.\nNote to be confused with dojo/robotx.js, which defines initRobot() without an underscore</p>\n"},"_primePump":{},"_robotInitialized":{},"_robotLoaded":{},"_runsemaphore":{},"_spaceReceived":{},"_started":{"!doc":"<p>Deferred that resolves when startRobot() has signaled completing by typing on the keyboard,\nwhich in turn calls _run().</p>\n"},"doc":{},"mouseWheelSize":{},"window":{},"initRobot":{"!type":"fn(url: ?)","!doc":"<p>Opens the application at the specified URL for testing, redirecting dojo to point to the application\nenvironment instead of the test environment.</p>\n"},"keyDown":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Holds down a single key, like SHIFT or &#39;a&#39;.</p>\n"},"keyPress":{"!type":"fn(charOrCode: ?, delay?: ?, modifiers: ?, asynchronous: ?)","!doc":"<p>Types a key combination, like SHIFT-TAB.</p>\n"},"keyUp":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Releases a single key, like SHIFT or &#39;a&#39;.</p>\n"},"killRobot":{"!type":"fn()"},"mouseClick":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Convenience function to do a press/release.\nSee robot.mousePress for more info.</p>\n"},"mouseMove":{"!type":"fn(x: ?, y: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Moves the mouse to the specified x,y offset relative to the viewport.</p>\n"},"mouseMoveAt":{"!type":"fn(node: ?, delay: ?, duration: ?, offsetX: ?, offsetY: ?)","!doc":"<p>Moves the mouse over the specified node at the specified relative x,y offset.</p>\n"},"mouseMoveTo":{"!type":"fn(point: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Move the mouse from the current position to the specified point.\nDelays reading contents point until queued command starts running.\nSee mouseMove() for details.</p>\n"},"mousePress":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Presses mouse buttons.</p>\n"},"mouseRelease":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Releases mouse buttons.</p>\n"},"mouseWheel":{"!type":"fn(wheelAmt: ?, delay?: ?, duration?: ?)","!doc":"<p>Spins the mouse wheel.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, delay: ?)","!doc":"<p>Scroll the passed node into view, if it is not.</p>\n"},"sequence":{"!type":"fn(f: ?, delay?: ?, duration?: ?)","!doc":"<p>Defer an action by adding it to the robot&#39;s incrementally delayed queue of actions to execute.</p>\n"},"setClipboard":{"!type":"fn(data: ?, format?: ?)","!doc":"<p>Set clipboard content.</p>\n"},"startRobot":{"!type":"fn() -> ?"},"typeKeys":{"!type":"fn(chars: ?, delay?: ?, duration?: ?)","!doc":"<p>Types a string of characters in order, or types a dojo.keys.* constant.</p>\n"},"waitForPageToLoad":{"!type":"fn(submitActions: ?) -> +instance","!doc":"<p>Notifies DOH that the doh.robot is about to make a page change in the application it is driving,\nreturning a doh.Deferred object the user should return in their runTest function as part of a DOH test.</p>\n"}},"doh/runner":{"!doc":"<p>Functions for registering and running automated tests.</p>\n","_currentGroup":{},"_currentTest":{},"_errorCount":{},"_failureCount":{},"_groupCount":{},"_groups":{},"_init":{},"_initRobotCalled":{},"_line":{},"_paused":{},"_registerUrl":{},"_setupGroupForRun":{},"_testCount":{},"_testTypes":{},"debug":{},"error":{},"isBrowser":{},"registerUrl":{},"robot":{},"assertEqual":{"!type":"fn(expected: ?, actual: ?, hint?: ?, doNotThrow: ?)","!doc":"<p>are the passed expected and actual objects/values deeply\nequivalent?</p>\n"},"assertError":{"!type":"fn(expectedError: ?, scope: ?, functionName: ?, args: ?, hint?: ?)","!doc":"<p>Test for a certain error to be thrown by the given function.</p>\n"},"assertFalse":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;falsey&quot;?</p>\n"},"assertNotEqual":{"!type":"fn(notExpected: ?, actual: ?, hint?: ?)","!doc":"<p>are the passed notexpected and actual objects/values deeply\nnot equivalent?</p>\n"},"assertTrue":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;truthy&quot;?</p>\n"},"Deferred":{"!type":"fn(canceller: ?)"},"e":{"!type":"fn(expectedError: ?, scope: ?, functionName: ?, args: ?, hint?: ?)","!doc":"<p>Test for a certain error to be thrown by the given function.</p>\n"},"f":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;falsey&quot;?</p>\n"},"is":{"!type":"fn(expected: ?, actual: ?, hint?: ?, doNotThrow: ?)","!doc":"<p>are the passed expected and actual objects/values deeply\nequivalent?</p>\n"},"isNot":{"!type":"fn(notExpected: ?, actual: ?, hint?: ?)","!doc":"<p>are the passed notexpected and actual objects/values deeply\nnot equivalent?</p>\n"},"pause":{"!type":"fn()","!doc":"<p>halt test run. Can be resumed.</p>\n"},"register":{"!type":"fn(a1: ?, a2: ?, a3: ?, a4: ?, a5: ?)"},"registerDocTests":{"!type":"fn(module: ?)","!doc":"<p>Deprecated.    Won&#39;t work unless you manually load dojox.testing.DocTest, and likely not even then.\nGets all the doctests from the given module and register each of them as a single test case here.</p>\n"},"registerGroup":{"!type":"fn(group: ?, tests: ?, setUp: ?, tearDown: ?, type: ?)","!doc":"<p>Deprecated.  Use doh.register(group/type, tests, setUp, tearDown) instead</p>\n"},"registerTest":{"!type":"fn(group: ?, test: ?, type: ?)","!doc":"<p>Deprecated.  Use doh.register(group/type, test) instead</p>\n"},"registerTestNs":{"!type":"fn(group: ?, ns: ?)","!doc":"<p>Deprecated.  Use doh.register(group, ns) instead</p>\n"},"registerTests":{"!type":"fn(group: ?, testArr: ?, type: ?)","!doc":"<p>Deprecated.  Use doh.register(group/type, testArr) instead</p>\n"},"registerTestType":{"!type":"fn(name: ?, initProc: ?)","!doc":"<p>Adds a test type and associates a function used to initialize each test of the given type</p>\n"},"run":{"!type":"fn()"},"runGroup":{"!type":"fn(groupName: ?, idx: ?)","!doc":"<p>runs the specified test group</p>\n"},"runOnLoad":{"!type":"fn()"},"showLogPage":{"!type":"fn()"},"showPerfTestsPage":{"!type":"fn()"},"showTestPage":{"!type":"fn()"},"t":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;truthy&quot;?</p>\n"},"togglePaused":{"!type":"fn()"},"toggleRunAll":{"!type":"fn()"}},"doh/runner._groups":{},"doh/runner._testTypes":{"perf":{"!type":"fn(group: ?, tObj: ?, type: ?)"}},"doh/runner.robot":{"_appletDead":{},"_loaded":{"!doc":"<p>Deferred that resolves when the _initRobot() has been called.\nNote to be confused with dojo/robotx.js, which defines initRobot() without an underscore</p>\n"},"_primePump":{},"_robotInitialized":{},"_robotLoaded":{},"_runsemaphore":{},"_spaceReceived":{},"_started":{"!doc":"<p>Deferred that resolves when startRobot() has signaled completing by typing on the keyboard,\nwhich in turn calls _run().</p>\n"},"doc":{},"mouseWheelSize":{},"window":{},"initRobot":{"!type":"fn(url: ?)","!doc":"<p>Opens the application at the specified URL for testing, redirecting dojo to point to the application\nenvironment instead of the test environment.</p>\n"},"keyDown":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Holds down a single key, like SHIFT or &#39;a&#39;.</p>\n"},"keyPress":{"!type":"fn(charOrCode: ?, delay?: ?, modifiers: ?, asynchronous: ?)","!doc":"<p>Types a key combination, like SHIFT-TAB.</p>\n"},"keyUp":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Releases a single key, like SHIFT or &#39;a&#39;.</p>\n"},"killRobot":{"!type":"fn()"},"mouseClick":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Convenience function to do a press/release.\nSee robot.mousePress for more info.</p>\n"},"mouseMove":{"!type":"fn(x: ?, y: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Moves the mouse to the specified x,y offset relative to the viewport.</p>\n"},"mouseMoveAt":{"!type":"fn(node: ?, delay: ?, duration: ?, offsetX: ?, offsetY: ?)","!doc":"<p>Moves the mouse over the specified node at the specified relative x,y offset.</p>\n"},"mouseMoveTo":{"!type":"fn(point: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Move the mouse from the current position to the specified point.\nDelays reading contents point until queued command starts running.\nSee mouseMove() for details.</p>\n"},"mousePress":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Presses mouse buttons.</p>\n"},"mouseRelease":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Releases mouse buttons.</p>\n"},"mouseWheel":{"!type":"fn(wheelAmt: ?, delay?: ?, duration?: ?)","!doc":"<p>Spins the mouse wheel.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, delay: ?)","!doc":"<p>Scroll the passed node into view, if it is not.</p>\n"},"sequence":{"!type":"fn(f: ?, delay?: ?, duration?: ?)","!doc":"<p>Defer an action by adding it to the robot&#39;s incrementally delayed queue of actions to execute.</p>\n"},"setClipboard":{"!type":"fn(data: ?, format?: ?)","!doc":"<p>Set clipboard content.</p>\n"},"startRobot":{"!type":"fn() -> ?"},"typeKeys":{"!type":"fn(chars: ?, delay?: ?, duration?: ?)","!doc":"<p>Types a string of characters in order, or types a dojo.keys.* constant.</p>\n"},"waitForPageToLoad":{"!type":"fn(submitActions: ?) -> +instance","!doc":"<p>Notifies DOH that the doh.robot is about to make a page change in the application it is driving,\nreturning a doh.Deferred object the user should return in their runTest function as part of a DOH test.</p>\n"}},"doh/robot":{"_appletDead":{},"_loaded":{"!doc":"<p>Deferred that resolves when the _initRobot() has been called.\nNote to be confused with dojo/robotx.js, which defines initRobot() without an underscore</p>\n"},"_primePump":{},"_robotInitialized":{},"_robotLoaded":{},"_runsemaphore":{},"_spaceReceived":{},"_started":{"!doc":"<p>Deferred that resolves when startRobot() has signaled completing by typing on the keyboard,\nwhich in turn calls _run().</p>\n"},"doc":{},"mouseWheelSize":{},"window":{},"initRobot":{"!type":"fn(url: ?)","!doc":"<p>Opens the application at the specified URL for testing, redirecting dojo to point to the application\nenvironment instead of the test environment.</p>\n"},"keyDown":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Holds down a single key, like SHIFT or &#39;a&#39;.</p>\n"},"keyPress":{"!type":"fn(charOrCode: ?, delay?: ?, modifiers: ?, asynchronous: ?)","!doc":"<p>Types a key combination, like SHIFT-TAB.</p>\n"},"keyUp":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Releases a single key, like SHIFT or &#39;a&#39;.</p>\n"},"killRobot":{"!type":"fn()"},"mouseClick":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Convenience function to do a press/release.\nSee robot.mousePress for more info.</p>\n"},"mouseMove":{"!type":"fn(x: ?, y: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Moves the mouse to the specified x,y offset relative to the viewport.</p>\n"},"mouseMoveAt":{"!type":"fn(node: ?, delay: ?, duration: ?, offsetX: ?, offsetY: ?)","!doc":"<p>Moves the mouse over the specified node at the specified relative x,y offset.</p>\n"},"mouseMoveTo":{"!type":"fn(point: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Move the mouse from the current position to the specified point.\nDelays reading contents point until queued command starts running.\nSee mouseMove() for details.</p>\n"},"mousePress":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Presses mouse buttons.</p>\n"},"mouseRelease":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Releases mouse buttons.</p>\n"},"mouseWheel":{"!type":"fn(wheelAmt: ?, delay?: ?, duration?: ?)","!doc":"<p>Spins the mouse wheel.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, delay: ?)","!doc":"<p>Scroll the passed node into view, if it is not.</p>\n"},"sequence":{"!type":"fn(f: ?, delay?: ?, duration?: ?)","!doc":"<p>Defer an action by adding it to the robot&#39;s incrementally delayed queue of actions to execute.</p>\n"},"setClipboard":{"!type":"fn(data: ?, format?: ?)","!doc":"<p>Set clipboard content.</p>\n"},"startRobot":{"!type":"fn() -> ?"},"typeKeys":{"!type":"fn(chars: ?, delay?: ?, duration?: ?)","!doc":"<p>Types a string of characters in order, or types a dojo.keys.* constant.</p>\n"},"waitForPageToLoad":{"!type":"fn(submitActions: ?) -> +instance","!doc":"<p>Notifies DOH that the doh.robot is about to make a page change in the application it is driving,\nreturning a doh.Deferred object the user should return in their runTest function as part of a DOH test.</p>\n"}},"doh/robot._runsemaphore":{"lock":{},"unlock":{"!type":"fn() -> ?"}},"dojo/robotx":{"_appletDead":{},"_loaded":{"!doc":"<p>Deferred that resolves when the _initRobot() has been called.\nNote to be confused with dojo/robotx.js, which defines initRobot() without an underscore</p>\n"},"_primePump":{},"_robotInitialized":{},"_robotLoaded":{},"_runsemaphore":{},"_spaceReceived":{},"_started":{"!doc":"<p>Deferred that resolves when startRobot() has signaled completing by typing on the keyboard,\nwhich in turn calls _run().</p>\n"},"doc":{},"mouseWheelSize":{},"window":{},"initRobot":{"!type":"fn(url: ?)","!doc":"<p>Opens the application at the specified URL for testing, redirecting dojo to point to the application\nenvironment instead of the test environment.</p>\n"},"keyDown":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Holds down a single key, like SHIFT or &#39;a&#39;.</p>\n"},"keyPress":{"!type":"fn(charOrCode: ?, delay?: ?, modifiers: ?, asynchronous: ?)","!doc":"<p>Types a key combination, like SHIFT-TAB.</p>\n"},"keyUp":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Releases a single key, like SHIFT or &#39;a&#39;.</p>\n"},"killRobot":{"!type":"fn()"},"mouseClick":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Convenience function to do a press/release.\nSee robot.mousePress for more info.</p>\n"},"mouseMove":{"!type":"fn(x: ?, y: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Moves the mouse to the specified x,y offset relative to the viewport.</p>\n"},"mouseMoveAt":{"!type":"fn(node: ?, delay: ?, duration: ?, offsetX: ?, offsetY: ?)","!doc":"<p>Moves the mouse over the specified node at the specified relative x,y offset.</p>\n"},"mouseMoveTo":{"!type":"fn(point: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Move the mouse from the current position to the specified point.\nDelays reading contents point until queued command starts running.\nSee mouseMove() for details.</p>\n"},"mousePress":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Presses mouse buttons.</p>\n"},"mouseRelease":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Releases mouse buttons.</p>\n"},"mouseWheel":{"!type":"fn(wheelAmt: ?, delay?: ?, duration?: ?)","!doc":"<p>Spins the mouse wheel.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, delay: ?)","!doc":"<p>Scroll the passed node into view, if it is not.</p>\n"},"sequence":{"!type":"fn(f: ?, delay?: ?, duration?: ?)","!doc":"<p>Defer an action by adding it to the robot&#39;s incrementally delayed queue of actions to execute.</p>\n"},"setClipboard":{"!type":"fn(data: ?, format?: ?)","!doc":"<p>Set clipboard content.</p>\n"},"startRobot":{"!type":"fn() -> ?"},"typeKeys":{"!type":"fn(chars: ?, delay?: ?, duration?: ?)","!doc":"<p>Types a string of characters in order, or types a dojo.keys.* constant.</p>\n"},"waitForPageToLoad":{"!type":"fn(submitActions: ?) -> +instance","!doc":"<p>Notifies DOH that the doh.robot is about to make a page change in the application it is driving,\nreturning a doh.Deferred object the user should return in their runTest function as part of a DOH test.</p>\n"}},"dojo/robotx._runsemaphore":{"lock":{},"unlock":{"!type":"fn() -> ?"}},"doh/main":{"!doc":"<p>Functions for registering and running automated tests.</p>\n","_currentGroup":{},"_currentTest":{},"_errorCount":{},"_failureCount":{},"_groupCount":{},"_groups":{},"_init":{},"_initRobotCalled":{},"_line":{},"_paused":{},"_registerUrl":{},"_setupGroupForRun":{},"_testCount":{},"_testTypes":{},"debug":{},"error":{},"isBrowser":{},"registerUrl":{},"robot":{},"assertEqual":{"!type":"fn(expected: ?, actual: ?, hint?: ?, doNotThrow: ?)","!doc":"<p>are the passed expected and actual objects/values deeply\nequivalent?</p>\n"},"assertError":{"!type":"fn(expectedError: ?, scope: ?, functionName: ?, args: ?, hint?: ?)","!doc":"<p>Test for a certain error to be thrown by the given function.</p>\n"},"assertFalse":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;falsey&quot;?</p>\n"},"assertNotEqual":{"!type":"fn(notExpected: ?, actual: ?, hint?: ?)","!doc":"<p>are the passed notexpected and actual objects/values deeply\nnot equivalent?</p>\n"},"assertTrue":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;truthy&quot;?</p>\n"},"Deferred":{"!type":"fn(canceller: ?)"},"e":{"!type":"fn(expectedError: ?, scope: ?, functionName: ?, args: ?, hint?: ?)","!doc":"<p>Test for a certain error to be thrown by the given function.</p>\n"},"f":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;falsey&quot;?</p>\n"},"is":{"!type":"fn(expected: ?, actual: ?, hint?: ?, doNotThrow: ?)","!doc":"<p>are the passed expected and actual objects/values deeply\nequivalent?</p>\n"},"isNot":{"!type":"fn(notExpected: ?, actual: ?, hint?: ?)","!doc":"<p>are the passed notexpected and actual objects/values deeply\nnot equivalent?</p>\n"},"pause":{"!type":"fn()","!doc":"<p>halt test run. Can be resumed.</p>\n"},"register":{"!type":"fn(a1: ?, a2: ?, a3: ?, a4: ?, a5: ?)"},"registerDocTests":{"!type":"fn(module: ?)","!doc":"<p>Deprecated.    Won&#39;t work unless you manually load dojox.testing.DocTest, and likely not even then.\nGets all the doctests from the given module and register each of them as a single test case here.</p>\n"},"registerGroup":{"!type":"fn(group: ?, tests: ?, setUp: ?, tearDown: ?, type: ?)","!doc":"<p>Deprecated.  Use doh.register(group/type, tests, setUp, tearDown) instead</p>\n"},"registerTest":{"!type":"fn(group: ?, test: ?, type: ?)","!doc":"<p>Deprecated.  Use doh.register(group/type, test) instead</p>\n"},"registerTestNs":{"!type":"fn(group: ?, ns: ?)","!doc":"<p>Deprecated.  Use doh.register(group, ns) instead</p>\n"},"registerTests":{"!type":"fn(group: ?, testArr: ?, type: ?)","!doc":"<p>Deprecated.  Use doh.register(group/type, testArr) instead</p>\n"},"registerTestType":{"!type":"fn(name: ?, initProc: ?)","!doc":"<p>Adds a test type and associates a function used to initialize each test of the given type</p>\n"},"run":{"!type":"fn()"},"runGroup":{"!type":"fn(groupName: ?, idx: ?)","!doc":"<p>runs the specified test group</p>\n"},"runOnLoad":{"!type":"fn()"},"showLogPage":{"!type":"fn()"},"showPerfTestsPage":{"!type":"fn()"},"showTestPage":{"!type":"fn()"},"t":{"!type":"fn(condition: ?, hint?: ?)","!doc":"<p>is the passed item &quot;truthy&quot;?</p>\n"},"togglePaused":{"!type":"fn()"},"toggleRunAll":{"!type":"fn()"}},"doh/main._groups":{},"doh/main._testTypes":{"perf":{"!type":"fn(group: ?, tObj: ?, type: ?)"}},"doh/main.robot":{"_appletDead":{},"_loaded":{"!doc":"<p>Deferred that resolves when the _initRobot() has been called.\nNote to be confused with dojo/robotx.js, which defines initRobot() without an underscore</p>\n"},"_primePump":{},"_robotInitialized":{},"_robotLoaded":{},"_runsemaphore":{},"_spaceReceived":{},"_started":{"!doc":"<p>Deferred that resolves when startRobot() has signaled completing by typing on the keyboard,\nwhich in turn calls _run().</p>\n"},"doc":{},"mouseWheelSize":{},"window":{},"initRobot":{"!type":"fn(url: ?)","!doc":"<p>Opens the application at the specified URL for testing, redirecting dojo to point to the application\nenvironment instead of the test environment.</p>\n"},"keyDown":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Holds down a single key, like SHIFT or &#39;a&#39;.</p>\n"},"keyPress":{"!type":"fn(charOrCode: ?, delay?: ?, modifiers: ?, asynchronous: ?)","!doc":"<p>Types a key combination, like SHIFT-TAB.</p>\n"},"keyUp":{"!type":"fn(charOrCode: ?, delay?: ?)","!doc":"<p>Releases a single key, like SHIFT or &#39;a&#39;.</p>\n"},"killRobot":{"!type":"fn()"},"mouseClick":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Convenience function to do a press/release.\nSee robot.mousePress for more info.</p>\n"},"mouseMove":{"!type":"fn(x: ?, y: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Moves the mouse to the specified x,y offset relative to the viewport.</p>\n"},"mouseMoveAt":{"!type":"fn(node: ?, delay: ?, duration: ?, offsetX: ?, offsetY: ?)","!doc":"<p>Moves the mouse over the specified node at the specified relative x,y offset.</p>\n"},"mouseMoveTo":{"!type":"fn(point: ?, delay?: ?, duration?: ?, absolute: ?)","!doc":"<p>Move the mouse from the current position to the specified point.\nDelays reading contents point until queued command starts running.\nSee mouseMove() for details.</p>\n"},"mousePress":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Presses mouse buttons.</p>\n"},"mouseRelease":{"!type":"fn(buttons: ?, delay?: ?)","!doc":"<p>Releases mouse buttons.</p>\n"},"mouseWheel":{"!type":"fn(wheelAmt: ?, delay?: ?, duration?: ?)","!doc":"<p>Spins the mouse wheel.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, delay: ?)","!doc":"<p>Scroll the passed node into view, if it is not.</p>\n"},"sequence":{"!type":"fn(f: ?, delay?: ?, duration?: ?)","!doc":"<p>Defer an action by adding it to the robot&#39;s incrementally delayed queue of actions to execute.</p>\n"},"setClipboard":{"!type":"fn(data: ?, format?: ?)","!doc":"<p>Set clipboard content.</p>\n"},"startRobot":{"!type":"fn() -> ?"},"typeKeys":{"!type":"fn(chars: ?, delay?: ?, duration?: ?)","!doc":"<p>Types a string of characters in order, or types a dojo.keys.* constant.</p>\n"},"waitForPageToLoad":{"!type":"fn(submitActions: ?) -> +instance","!doc":"<p>Notifies DOH that the doh.robot is about to make a page change in the application it is driving,\nreturning a doh.Deferred object the user should return in their runTest function as part of a DOH test.</p>\n"}},"dojo/router/RouterBase":{"!type":"fn()","prototype":{"!doc":"<p>A module that allows one to easily map hash-based structures into\ncallbacks. The router module is a singleton, offering one central\npoint for all registrations of this type.</p>\n","_currentPath":{},"_routeIndex":{},"_routes":{},"_started":{},"globMatch":{},"globReplacement":{},"idMatch":{},"idReplacement":{},"constructor":{"!type":"fn(kwArgs: ?)"},"destroy":{"!type":"fn()"},"go":{"!type":"fn(path: ?, replace: ?) -> bool","!doc":"<p>A simple pass-through to make changing the hash easy,\nwithout having to require dojo/hash directly. It also\nsynchronously fires off any routes that match.</p>\n"},"register":{"!type":"fn(route: ?, callback: ?) -> ?","!doc":"<p>Registers a route to a handling callback</p>\n"},"registerBefore":{"!type":"fn(route: ?, callback: ?) -> ?","!doc":"<p>Registers a route to a handling callback, except before\nany previously registered callbacks</p>\n"},"startup":{"!type":"fn(defaultPath: ?)","!doc":"<p>This method must be called to activate the router. Until\nstartup is called, no hash changes will trigger route\ncallbacks.</p>\n"}}},"dojo/router":{"!doc":"<p>A singleton-style instance of dojo/router/RouterBase. See that\nmodule for specifics.</p>\n"},"dojo/rpc/JsonService":{"!type":"fn()","prototype":{"!proto":"dojo/rpc/RpcService.prototype","!doc":"<p>TODOC</p>\n","bustCache":{},"contentType":{},"lastSubmissionId":{},"serviceUrl":{},"strictArgChecks":{},"bind":{"!type":"fn(method: ?, parameters: ?, deferredRequestHandler: ?, url: ?)","!doc":"<p>JSON-RPC bind method. Takes remote method, parameters,\ndeferred, and a url, calls createRequest to make a JSON-RPC\nenvelope and passes that off with bind.</p>\n"},"callRemote":{"!type":"fn(method: ?, params: ?) -> +instance","!doc":"<p>call an arbitrary remote method without requiring it to be\npredefined with SMD</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Take a string as a url to retrieve an smd or an object that is an smd or partial smd to use\nas a definition for the service\n</p>\n"},"createRequest":{"!type":"fn(method: ?, params: ?) -> ?","!doc":"<p>create a JSON-RPC envelope for the request</p>\n"},"errorCallback":{"!type":"fn(deferredRequestHandler: ?) -> fn","!doc":"<p>create callback that calls the Deferred errback method</p>\n"},"generateMethod":{"!type":"fn(method: ?, parameters: ?, url: ?) -> ?","!doc":"<p>generate the local bind methods for the remote object</p>\n"},"parseResults":{"!type":"fn(obj: ?) -> ?","!doc":"<p>parse the result envelope and pass the results back to\nthe callback function</p>\n"},"processSmd":{"!type":"fn(object: ?)","!doc":"<p>callback method for receipt of a smd object.  Parse the smd\nand generate functions based on the description</p>\n"},"resultCallback":{"!type":"fn(deferredRequestHandler: ?) -> ?","!doc":"<p>create callback that calls the Deferred&#39;s callback method</p>\n"}}},"dojo/rpc/RpcService":{"!type":"fn()","prototype":{"!doc":"<p>TODOC</p>\n","serviceUrl":{},"strictArgChecks":{},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Take a string as a url to retrieve an smd or an object that is an smd or partial smd to use\nas a definition for the service\n</p>\n"},"errorCallback":{"!type":"fn(deferredRequestHandler: ?) -> fn","!doc":"<p>create callback that calls the Deferred errback method</p>\n"},"generateMethod":{"!type":"fn(method: ?, parameters: ?, url: ?) -> ?","!doc":"<p>generate the local bind methods for the remote object</p>\n"},"parseResults":{"!type":"fn(obj: ?) -> ?","!doc":"<p>parse the results coming back from an rpc request.  this\nbase implementation, just returns the full object\nsubclasses should parse and only return the actual results</p>\n"},"processSmd":{"!type":"fn(object: ?)","!doc":"<p>callback method for receipt of a smd object.  Parse the smd\nand generate functions based on the description</p>\n"},"resultCallback":{"!type":"fn(deferredRequestHandler: ?) -> ?","!doc":"<p>create callback that calls the Deferred&#39;s callback method</p>\n"}}},"dojo/rpc/JsonpService":{"!type":"fn()","prototype":{"!proto":"dojo/rpc/RpcService.prototype","!doc":"<p>Generic JSONP service.  Minimally extends RpcService to allow\neasy definition of nearly any JSONP style service. Example\nSMD files exist in dojox.data</p>\n","serviceUrl":{},"strictArgChecks":{},"bind":{"!type":"fn(method: ?, parameters: ?, deferredRequestHandler: ?, url: ?)","!doc":"<p>JSONP bind method. Takes remote method, parameters,\ndeferred, and a url, calls createRequest to make a JSON-RPC\nenvelope and passes that off with bind.</p>\n"},"constructor":{"!type":"fn(args: ?, requiredArgs: ?)"},"createRequest":{"!type":"fn(parameters: ?) -> ?","!doc":"<p>create a JSONP req</p>\n"},"errorCallback":{"!type":"fn(deferredRequestHandler: ?) -> fn","!doc":"<p>create callback that calls the Deferred errback method</p>\n"},"generateMethod":{"!type":"fn(method: ?, parameters: ?, url: ?) -> ?","!doc":"<p>generate the local bind methods for the remote object</p>\n"},"parseResults":{"!type":"fn(obj: ?) -> ?","!doc":"<p>parse the results coming back from an rpc request.  this\nbase implementation, just returns the full object\nsubclasses should parse and only return the actual results</p>\n"},"processSmd":{"!type":"fn(object: ?)","!doc":"<p>callback method for receipt of a smd object.  Parse the smd\nand generate functions based on the description</p>\n"},"resultCallback":{"!type":"fn(deferredRequestHandler: ?) -> ?","!doc":"<p>create callback that calls the Deferred&#39;s callback method</p>\n"}}},"dojo/selector/acme":{"!doc":"<p>Returns nodes which match the given CSS3 selector, searching the\nentire document by default but optionally taking a node to scope\nthe search by. Returns an array.</p>\n","prototype":{"constructor":{"!type":"fn(query: ?, root?: ?) -> []","!doc":"<p>Returns nodes which match the given CSS3 selector, searching the\nentire document by default but optionally taking a node to scope\nthe search by. Returns an array.</p>\n"}},"filter":{"!type":"fn(nodeList: ?, filter: ?, root?: ?)","!doc":"<p>function for filtering a NodeList based on a selector, optimized for simple selectors</p>\n"}},"dojo/selector/lite":{"!doc":"<p>A small lightweight query selector engine that implements CSS2.1 selectors\nminus pseudo-classes and the sibling combinator, plus CSS3 attribute selectors</p>\n","match":{},"prototype":{"constructor":{"!type":"fn(selector: ?, root: ?) -> ?","!doc":"<p>A small lightweight query selector engine that implements CSS2.1 selectors\nminus pseudo-classes and the sibling combinator, plus CSS3 attribute selectors</p>\n"}}},"dojo/store/Cache":{"!type":"fn()","prototype":{"!proto":"dojo/store/api/Store.prototype","!doc":"<p>The Cache store wrapper takes a master store and a caching store,\ncaches data from the master into the caching store for faster\nlookup. Normally one would use a memory store for the caching\nstore and a server store like JsonRest for the master store.</p>\n","idProperty":{"!doc":"<p>If the store has a single primary key, this indicates the property to use as the\nidentity property. The values of this property should be unique.</p>\n"},"queryEngine":{"!doc":"<p>If the store can be queried locally (on the client side in JS), this defines\nthe query engine to use for querying the data store.\nThis takes a query and query options and returns a function that can execute\nthe provided query on a JavaScript array. The queryEngine may be replace to\nprovide more sophisticated querying capabilities. For example:\n\n</p>\n<pre><code>var query = store.queryEngine({foo:&quot;bar&quot;}, {count:10});\nquery(someArray) -&gt; filtered array</code></pre>\n<p>The returned query function may have a &quot;matches&quot; property that can be\n\n</p>\n<p>used to determine if an object matches the query. For example:\n\n</p>\n<pre><code>query.matches({id:&quot;some-object&quot;, foo:&quot;bar&quot;}) -&gt; true\nquery.matches({id:&quot;some-object&quot;, foo:&quot;something else&quot;}) -&gt; false</code></pre>\n"},"add":{"!type":"fn(object: ?, directives?: ?) -> number","!doc":"<p>Add the given object to the store.</p>\n"},"constructor":{"!type":"fn(masterStore: ?, cachingStore: ?, options?: ?)"},"evict":{"!type":"fn(id: ?)","!doc":"<p>Remove the object with the given id from the underlying caching store.</p>\n"},"get":{"!type":"fn(id: ?, directives?: ?) -> +dojo","!doc":"<p>Get the object with the specific id.</p>\n"},"getChildren":{"!type":"fn(parent: ?, options?: ?) -> +dojo","!doc":"<p>Retrieves the children of an object.</p>\n"},"getIdentity":{"!type":"fn(object: ?) -> string","!doc":"<p>Returns an object&#39;s identity</p>\n"},"getMetadata":{"!type":"fn(object: ?) -> ?","!doc":"<p>Returns any metadata about the object. This may include attribution,\ncache directives, history, or version information.</p>\n"},"put":{"!type":"fn(object: ?, directives?: ?) -> number","!doc":"<p>Put the object into the store (similar to an HTTP PUT).</p>\n"},"query":{"!type":"fn(query: ?, directives?: ?) -> +dojo","!doc":"<p>Query the underlying master store and cache any results.</p>\n"},"remove":{"!type":"fn(id: ?)","!doc":"<p>Remove the object with the specific id.</p>\n"},"transaction":{"!type":"fn() -> +dojo","!doc":"<p>Starts a new transaction.\nNote that a store user might not call transaction() prior to using put,\ndelete, etc. in which case these operations effectively could be thought of\nas &quot;auto-commit&quot; style actions.</p>\n"}}},"dojo/store/api/Store":{"!type":"fn()","prototype":{"!doc":"<p>This is an abstract API that data provider implementations conform to.\nThis file defines methods signatures and intentionally leaves all the\nmethods unimplemented.  For more information on the ,\nplease visit: <a href=\"http://dojotoolkit.org/reference-guide/dojo/store.html\">http://dojotoolkit.org/reference-guide/dojo/store.html</a>\nEvery method and property is optional, and is only needed if the functionality\nit provides is required.\nEvery method may return a promise for the specified return value if the\nexecution of the operation is asynchronous (except\nfor query() which already defines an async return value).</p>\n","idProperty":{"!doc":"<p>If the store has a single primary key, this indicates the property to use as the\nidentity property. The values of this property should be unique.</p>\n"},"queryEngine":{"!doc":"<p>If the store can be queried locally (on the client side in JS), this defines\nthe query engine to use for querying the data store.\nThis takes a query and query options and returns a function that can execute\nthe provided query on a JavaScript array. The queryEngine may be replace to\nprovide more sophisticated querying capabilities. For example:\n\n</p>\n<pre><code>var query = store.queryEngine({foo:&quot;bar&quot;}, {count:10});\nquery(someArray) -&gt; filtered array</code></pre>\n<p>The returned query function may have a &quot;matches&quot; property that can be\n\n</p>\n<p>used to determine if an object matches the query. For example:\n\n</p>\n<pre><code>query.matches({id:&quot;some-object&quot;, foo:&quot;bar&quot;}) -&gt; true\nquery.matches({id:&quot;some-object&quot;, foo:&quot;something else&quot;}) -&gt; false</code></pre>\n"},"add":{"!type":"fn(object: ?, directives?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists</p>\n"},"get":{"!type":"fn(id: ?) -> ?","!doc":"<p>Retrieves an object by its identity</p>\n"},"getChildren":{"!type":"fn(parent: ?, options?: ?) -> +dojo","!doc":"<p>Retrieves the children of an object.</p>\n"},"getIdentity":{"!type":"fn(object: ?) -> string","!doc":"<p>Returns an object&#39;s identity</p>\n"},"getMetadata":{"!type":"fn(object: ?) -> ?","!doc":"<p>Returns any metadata about the object. This may include attribution,\ncache directives, history, or version information.</p>\n"},"put":{"!type":"fn(object: ?, directives?: ?) -> number","!doc":"<p>Stores an object</p>\n"},"query":{"!type":"fn(query: ?, options: ?) -> +dojo","!doc":"<p>Queries the store for objects. This does not alter the store, but returns a\nset of data from the store.</p>\n"},"remove":{"!type":"fn(id: ?)","!doc":"<p>Deletes an object by its identity</p>\n"},"transaction":{"!type":"fn() -> +dojo","!doc":"<p>Starts a new transaction.\nNote that a store user might not call transaction() prior to using put,\ndelete, etc. in which case these operations effectively could be thought of\nas &quot;auto-commit&quot; style actions.</p>\n"}},"PutDirectives":{"!type":"fn()"},"QueryOptions":{"!type":"fn()"},"QueryResults":{"!type":"fn()"},"SortInformation":{"!type":"fn()"},"Transaction":{"!type":"fn()"}},"dojo/store/api/Store.PutDirectives":{"!type":"fn()","prototype":{"!doc":"<p>Directives passed to put() and add() handlers for guiding the update and\ncreation of stored objects.</p>\n","before":{"!doc":"<p>If the collection of objects in the store has a natural ordering,\nthis indicates that the created or updated object should be placed before the\nobject specified by the value of this property. A value of null indicates that the\nobject should be last.</p>\n"},"id":{"!doc":"<p>Indicates the identity of the object if a new object is created</p>\n"},"overwrite":{"!doc":"<p>If this is provided as a boolean it indicates that the object should or should not\noverwrite an existing object. A value of true indicates that a new object\nshould not be created, the operation should update an existing object. A\nvalue of false indicates that an existing object should not be updated, a new\nobject should be created (which is the same as an add() operation). When\nthis property is not provided, either an update or creation is acceptable.</p>\n"},"parent":{"!doc":"<p>If the store is hierarchical (with single parenting) this property indicates the\nnew parent of the created or updated object.</p>\n"}}},"dojo/store/api/Store.SortInformation":{"!type":"fn()","prototype":{"!doc":"<p>An object describing what attribute to sort on, and the direction of the sort.</p>\n","attribute":{"!doc":"<p>The name of the attribute to sort on.</p>\n"},"descending":{"!doc":"<p>The direction of the sort.  Default is false.</p>\n"}}},"dojo/store/api/Store.QueryOptions":{"!type":"fn()","prototype":{"!doc":"<p>Optional object with additional parameters for query results.</p>\n","count":{"!doc":"<p>The number of how many results should be returned.</p>\n"},"sort":{"!doc":"<p>A list of attributes to sort on, as well as direction\nFor example:\n\n</p>\n<pre><code>[{attribute:&quot;price, descending: true}].</code></pre>\n<p>If the sort parameter is omitted, then the natural order of the store may be\n\n</p>\n<p>applied if there is a natural order.</p>\n"},"start":{"!doc":"<p>The first result to begin iteration on</p>\n"}}},"dojo/store/api/Store.QueryResults":{"!type":"fn()","prototype":{"!doc":"<p>This is an object returned from query() calls that provides access to the results\nof a query. Queries may be executed asynchronously.</p>\n","total":{"!doc":"<p>This property should be included in if the query options included the &quot;count&quot;\nproperty limiting the result set. This property indicates the total number of objects\nmatching the query (as if &quot;start&quot; and &quot;count&quot; weren&#39;t present). This may be\na promise if the query is asynchronous.</p>\n"},"filter":{"!type":"fn(callback: ?, thisObject: ?) -> +dojo","!doc":"<p>Filters the query results, based on\n<a href=\"https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/filter\">https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/filter</a>.\nNote that this may executed asynchronously. The callback may be called\nafter this function returns.</p>\n"},"forEach":{"!type":"fn(callback: ?, thisObject: ?)","!doc":"<p>Iterates over the query results, based on\n<a href=\"https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/forEach\">https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/forEach</a>.\nNote that this may executed asynchronously. The callback may be called\nafter this function returns.</p>\n"},"map":{"!type":"fn(callback: ?, thisObject: ?) -> +dojo","!doc":"<p>Maps the query results, based on\n<a href=\"https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/map\">https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/map</a>.\nNote that this may executed asynchronously. The callback may be called\nafter this function returns.</p>\n"},"observe":{"!type":"fn(listener: ?, includeAllUpdates: ?)","!doc":"<p>This registers a callback for notification of when data is modified in the query results.\nThis is an optional method, and is usually provided by dojo/store/Observable.</p>\n"},"then":{"!type":"fn(callback: ?, errorHandler: ?)","!doc":"<p>This registers a callback for when the query is complete, if the query is asynchronous.\nThis is an optional method, and may not be present for synchronous queries.</p>\n"}}},"dojo/store/api/Store.Transaction":{"!type":"fn()","prototype":{"!doc":"<p>This is an object returned from transaction() calls that represents the current\ntransaction.</p>\n","abort":{"!type":"fn(callback: ?, thisObject: ?)","!doc":"<p>Aborts the transaction. This may throw an error if it fails. Of if the operation\nis asynchronous, it may return a promise that represents the eventual success\nor failure of the abort.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Commits the transaction. This may throw an error if it fails. Of if the operation\nis asynchronous, it may return a promise that represents the eventual success\nor failure of the commit.</p>\n"}}},"dojo/store/DataStore":{"!type":"fn()","prototype":{"!proto":"dojo/store/api/Store.prototype","!doc":"<p>This is an adapter for using Dojo Data stores with an object store consumer.\nYou can provide a Dojo data store and use this adapter to interact with it through\nthe Dojo object store API</p>\n","idProperty":{"!doc":"<p>The object property to use to store the identity of the store items.</p>\n"},"store":{"!doc":"<p>The object store to convert to a data store</p>\n"},"target":{},"add":{"!type":"fn(object: ?, directives?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists</p>\n"},"constructor":{"!type":"fn(options?: ?)"},"get":{"!type":"fn(id?: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by it&#39;s identity. This will trigger a fetchItemByIdentity</p>\n"},"getChildren":{"!type":"fn(parent: ?, options?: ?) -> +dojo","!doc":"<p>Retrieves the children of an object.</p>\n"},"getIdentity":{"!type":"fn(object: ?) -> number","!doc":"<p>Fetch the identity for the given object.</p>\n"},"getMetadata":{"!type":"fn(object: ?) -> ?","!doc":"<p>Returns any metadata about the object. This may include attribution,\ncache directives, history, or version information.</p>\n"},"put":{"!type":"fn(object: ?, options?: ?)","!doc":"<p>Stores an object by its identity.</p>\n"},"query":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"queryEngine":{"!type":"fn(query: ?, options?: ?) -> fn()","!doc":"<p>Defines the query engine to use for querying the data store</p>\n"},"remove":{"!type":"fn(id: ?)","!doc":"<p>Deletes an object by its identity.</p>\n"},"transaction":{"!type":"fn() -> +dojo","!doc":"<p>Starts a new transaction.\nNote that a store user might not call transaction() prior to using put,\ndelete, etc. in which case these operations effectively could be thought of\nas &quot;auto-commit&quot; style actions.</p>\n"}}},"dojo/store/util/QueryResults":{"!doc":"<p>A function that wraps the results of a store query with additional\nmethods.</p>\n","prototype":{"constructor":{"!type":"fn(results: ?) -> +any","!doc":"<p>A function that wraps the results of a store query with additional\nmethods.</p>\n"}}},"dojo/store/util/SimpleQueryEngine":{"!doc":"<p>Simple query engine that matches using filter functions, named filter\nfunctions or objects by name-value on a query object hash\n</p>\n","prototype":{"constructor":{"!type":"fn(query: ?, options?: ?) -> fn()","!doc":"<p>Simple query engine that matches using filter functions, named filter\nfunctions or objects by name-value on a query object hash\n</p>\n"}}},"dojo/store/JsonRest":{"!type":"fn()","prototype":{"!proto":"dojo/store/api/Store.prototype","!doc":"<p>This is a basic store for RESTful communicating with a server through JSON\nformatted data. It implements dojo/store/api/Store.</p>\n","accepts":{"!doc":"<p>Defines the Accept header to use on HTTP requests</p>\n"},"ascendingPrefix":{"!doc":"<p>The prefix to apply to sort attribute names that are ascending</p>\n"},"descendingPrefix":{"!doc":"<p>The prefix to apply to sort attribute names that are ascending</p>\n"},"headers":{"!doc":"<p>Additional headers to pass in all requests to the server. These can be overridden\nby passing additional headers to calls to the store.</p>\n"},"idProperty":{"!doc":"<p>Indicates the property to use as the identity property. The values of this\nproperty should be unique.</p>\n"},"queryEngine":{"!doc":"<p>If the store can be queried locally (on the client side in JS), this defines\nthe query engine to use for querying the data store.\nThis takes a query and query options and returns a function that can execute\nthe provided query on a JavaScript array. The queryEngine may be replace to\nprovide more sophisticated querying capabilities. For example:\n\n</p>\n<pre><code>var query = store.queryEngine({foo:&quot;bar&quot;}, {count:10});\nquery(someArray) -&gt; filtered array</code></pre>\n<p>The returned query function may have a &quot;matches&quot; property that can be\n\n</p>\n<p>used to determine if an object matches the query. For example:\n\n</p>\n<pre><code>query.matches({id:&quot;some-object&quot;, foo:&quot;bar&quot;}) -&gt; true\nquery.matches({id:&quot;some-object&quot;, foo:&quot;something else&quot;}) -&gt; false</code></pre>\n"},"target":{"!doc":"<p>The target base URL to use for all requests to the server. This string will be\nprepended to the id to generate the URL (relative or absolute) for requests\nsent to the server</p>\n"},"add":{"!type":"fn(object: ?, options?: ?) -> ?","!doc":"<p>Adds an object. This will trigger a PUT request to the server\nif the object has an id, otherwise it will trigger a POST request.</p>\n"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>This is a basic store for RESTful communicating with a server through JSON\nformatted data.</p>\n"},"get":{"!type":"fn(id: ?, options?: ?) -> ?","!doc":"<p>Retrieves an object by its identity. This will trigger a GET request to the server using\nthe url <code>this.target + id</code>.</p>\n"},"getChildren":{"!type":"fn(parent: ?, options?: ?) -> +dojo","!doc":"<p>Retrieves the children of an object.</p>\n"},"getIdentity":{"!type":"fn(object: ?) -> number","!doc":"<p>Returns an object&#39;s identity</p>\n"},"getMetadata":{"!type":"fn(object: ?) -> ?","!doc":"<p>Returns any metadata about the object. This may include attribution,\ncache directives, history, or version information.</p>\n"},"put":{"!type":"fn(object: ?, options?: ?) -> +dojo","!doc":"<p>Stores an object. This will trigger a PUT request to the server\nif the object has an id, otherwise it will trigger a POST request.</p>\n"},"query":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects. This will trigger a GET request to the server, with the\nquery added as a query string.</p>\n"},"remove":{"!type":"fn(id: ?, options?: ?) -> ?","!doc":"<p>Deletes an object by its identity. This will trigger a DELETE request to the server.</p>\n"},"transaction":{"!type":"fn() -> +dojo","!doc":"<p>Starts a new transaction.\nNote that a store user might not call transaction() prior to using put,\ndelete, etc. in which case these operations effectively could be thought of\nas &quot;auto-commit&quot; style actions.</p>\n"}}},"dojo/store/Memory":{"!type":"fn()","prototype":{"!proto":"dojo/store/api/Store.prototype","!doc":"<p>This is a basic in-memory object store. It implements dojo/store/api/Store.</p>\n","data":{"!doc":"<p>The array of all the objects in the memory store</p>\n"},"idProperty":{"!doc":"<p>Indicates the property to use as the identity property. The values of this\nproperty should be unique.</p>\n"},"index":{"!doc":"<p>An index of data indices into the data array by id</p>\n"},"add":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists</p>\n"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>Creates a memory object store.</p>\n"},"get":{"!type":"fn(id: ?) -> ?","!doc":"<p>Retrieves an object by its identity</p>\n"},"getChildren":{"!type":"fn(parent: ?, options?: ?) -> +dojo","!doc":"<p>Retrieves the children of an object.</p>\n"},"getIdentity":{"!type":"fn(object: ?) -> number","!doc":"<p>Returns an object&#39;s identity</p>\n"},"getMetadata":{"!type":"fn(object: ?) -> ?","!doc":"<p>Returns any metadata about the object. This may include attribution,\ncache directives, history, or version information.</p>\n"},"put":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object</p>\n"},"query":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"queryEngine":{"!type":"fn(query: ?, options?: ?) -> fn()","!doc":"<p>Defines the query engine to use for querying the data store</p>\n"},"remove":{"!type":"fn(id: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Sets the given data as the source for this store, and indexes it</p>\n"},"transaction":{"!type":"fn() -> +dojo","!doc":"<p>Starts a new transaction.\nNote that a store user might not call transaction() prior to using put,\ndelete, etc. in which case these operations effectively could be thought of\nas &quot;auto-commit&quot; style actions.</p>\n"}}},"dojo/store/Observable":{"!doc":"<p>The Observable store wrapper takes a store and sets an observe method on query()\nresults that can be used to monitor results for changes.\n</p>\n","prototype":{"constructor":{"!type":"fn(store: ?) -> ?","!doc":"<p>The Observable store wrapper takes a store and sets an observe method on query()\nresults that can be used to monitor results for changes.\n</p>\n"}}},"dojo/uacss":{"!doc":"<p>Applies pre-set CSS classes to the top-level HTML node, based on:\n\n</p>\n<ul>\n<li>browser (ex: dj_ie)</li>\n<li>browser version (ex: dj_ie6)</li>\n<li>box model (ex: dj_contentBox)</li>\n<li>text direction (ex: dijitRtl)</li>\n</ul>\n<p>In addition, browser, browser version, and box model are\ncombined with an RTL flag when browser text is RTL. ex: dj_ie-rtl.\n\n</p>\n<p>Returns the has() method.</p>\n"},"dijit/BackgroundIframe":{"!doc":"<p>For IE/FF z-index shenanigans. id attribute is required.\n</p>\n","prototype":{"constructor":{"!type":"fn(node: ?)","!doc":"<p>For IE/FF z-index shenanigans. id attribute is required.\n</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>destroy the iframe</p>\n"},"resize":{"!type":"fn(node: ?)","!doc":"<p>Resize the iframe so it&#39;s the same size as node.\nNeeded on IE6 and IE/quirks because height:100% doesn&#39;t work right.</p>\n"}}},"dijit/main":{"!doc":"<p>The dijit package main module.\nDeprecated.   Users should access individual modules (ex: dijit/registry) directly.</p>\n","_activeStack":{"!doc":"<p>List of currently active widgets (focused widget and it&#39;s ancestors)</p>\n"},"_curFocus":{"!doc":"<p>Currently focused item on screen</p>\n"},"_masterTT":{},"_prevFocus":{"!doc":"<p>Previously focused item on screen</p>\n"},"_underlay":{},"defaultDuration":{"!doc":"<p>The default fx.animation speed (in ms) to use for all Dijit\ntransitional fx.animations, unless otherwise specified\non a per-instance basis. Defaults to 200, overrided by\n<code>djConfig.defaultDuration</code></p>\n"},"place":{"!doc":"<p>Code to place a DOMNode relative to another DOMNode.\nLoad using require([&quot;dijit/place&quot;], function(place){ ... }).</p>\n"},"popup":{},"registry":{"!doc":"<p>Registry of existing widget on page, plus some utility methods.</p>\n"},"typematic":{"!doc":"<p>These functions are used to repetitively call a user specified callback\nmethod when a specific key or mouse click over a specific DOM node is\nheld down for a specific amount of time.\nOnly 1 such event is allowed to occur on the browser page at 1 time.</p>\n"},"BackgroundIframe":{"!type":"fn(node: ?)","!doc":"<p>For IE/FF z-index shenanigans. id attribute is required.\n</p>\n"},"effectiveTabIndex":{"!type":"fn(elem: ?) -> ?","!doc":"<p>Returns effective tabIndex of an element, either a number, or undefined if element isn&#39;t focusable.</p>\n"},"focus":{"!type":"fn(node: ?)"},"getBookmark":{"!type":"fn() -> ?","!doc":"<p>Retrieves a bookmark that can be used with moveToBookmark to return to the same range</p>\n"},"getDocumentWindow":{"!type":"fn(doc: ?)"},"getFirstInTabbingOrder":{"!type":"fn(root: ?, doc?: ?) -> ?","!doc":"<p>Finds the descendant of the specified root node\nthat is first in the tabbing order</p>\n"},"getFocus":{"!type":"fn(menu?: ?, openedForWindow?: ?) -> +any","!doc":"<p>Called as getFocus(), this returns an Object showing the current focus\nand selected text.\n\n</p>\n<p>Called as getFocus(widget), where widget is a (widget representing) a button\nthat was just pressed, it returns where focus was before that button\nwas pressed.   (Pressing the button may have either shifted focus to the button,\nor removed focus altogether.)   In this case the selected text is not returned,\nsince it can&#39;t be accurately determined.\n</p>\n"},"getLastInTabbingOrder":{"!type":"fn(root: ?, doc?: ?) -> ?","!doc":"<p>Finds the descendant of the specified root node\nthat is last in the tabbing order</p>\n"},"getPopupAroundAlignment":{"!type":"fn(position: ?, leftToRight: ?)","!doc":"<p>Deprecated method, unneeded when using dijit/place directly.\nTransforms the passed array of preferred positions into a format suitable for\npassing as the aroundCorners argument to dijit/place.placeOnScreenAroundElement.</p>\n"},"getViewport":{"!type":"fn()","!doc":"<p>Deprecated method to return the dimensions and scroll position of the viewable area of a browser window.\nNew code should use windowUtils.getBox()</p>\n"},"getWaiRole":{"!type":"fn(elem: ?) -> +any","!doc":"<p>Gets the role for an element (which should be a wai role).</p>\n"},"getWaiState":{"!type":"fn(elem: ?, state: ?) -> +any","!doc":"<p>Gets the value of a state on an element.</p>\n"},"hasDefaultTabStop":{"!type":"fn(elem: ?) -> ?","!doc":"<p>Tests if element is tab-navigable even without an explicit tabIndex setting</p>\n"},"hasWaiRole":{"!type":"fn(elem: ?, role?: ?) -> +any","!doc":"<p>Determines if an element has a particular role.</p>\n"},"hasWaiState":{"!type":"fn(elem: ?, state: ?) -> +any","!doc":"<p>Determines if an element has a given state.</p>\n"},"hideTooltip":{"!type":"fn(aroundNode: ?) -> ?","!doc":"<p>Static method to hide the tooltip displayed via showTooltip()</p>\n"},"isCollapsed":{"!type":"fn() -> ?","!doc":"<p>Returns true if there is no text selected</p>\n"},"isFocusable":{"!type":"fn(elem: ?) -> bool","!doc":"<p>Tests if an element is focusable by tabbing to it, or clicking it with the mouse.</p>\n"},"isTabNavigable":{"!type":"fn(elem: ?) -> bool","!doc":"<p>Tests if an element is tab-navigable</p>\n"},"moveToBookmark":{"!type":"fn(bookmark: ?) -> ?","!doc":"<p>Moves current selection to a bookmark</p>\n"},"placeOnScreen":{"!type":"fn(node: ?, pos: ?, corners: ?, padding?: ?, layoutNode: ?) -> ?","!doc":"<p>Positions node kitty-corner to the rectangle centered at (pos.x, pos.y) with width and height of\npadding.x <em> 2 and padding.y </em> 2, or zero if padding not specified.  Picks first corner in corners[]\nwhere node is fully visible, or the corner where it&#39;s most visible.\n\n</p>\n<p>Node is assumed to be absolutely or relatively positioned.</p>\n"},"placeOnScreenAroundElement":{"!type":"fn(node: ?, aroundNode: ?, aroundCorners: ?, layoutNode: ?)","!doc":"<p>Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object\nfor the &quot;around&quot; argument and finds a proper processor to place a node.\nDeprecated, new code should use dijit/place.around() instead.</p>\n"},"placeOnScreenAroundNode":{"!type":"fn(node: ?, aroundNode: ?, aroundCorners: ?, layoutNode: ?)","!doc":"<p>Position node adjacent or kitty-corner to aroundNode\nsuch that it&#39;s fully visible in viewport.\nDeprecated, new code should use dijit/place.around() instead.</p>\n"},"placeOnScreenAroundRectangle":{"!type":"fn(node: ?, aroundRect: ?, aroundCorners: ?, layoutNode: ?)","!doc":"<p>Like dijit.placeOnScreenAroundNode(), except that the &quot;around&quot;\nparameter is an arbitrary rectangle on the screen (x, y, width, height)\ninstead of a dom node.\nDeprecated, new code should use dijit/place.around() instead.</p>\n"},"registerIframe":{"!type":"fn(iframe: ?) -> +any","!doc":"<p>Registers listeners on the specified iframe so that any click\nor focus event on that iframe (or anything in it) is reported\nas a focus/click event on the <code>&lt;iframe&gt;</code> itself.</p>\n"},"registerWin":{"!type":"fn(targetWindow?: ?, effectiveNode?: ?) -> +any","!doc":"<p>Registers listeners on the specified window (either the main\nwindow or an iframe&#39;s window) to detect when the user has clicked somewhere\nor focused somewhere.</p>\n"},"removeWaiRole":{"!type":"fn(elem: ?, role: ?)","!doc":"<p>Removes the specified role from an element.\nRemoves role attribute if no specific role provided (for backwards compat.)</p>\n"},"removeWaiState":{"!type":"fn(elem: ?, state: ?)","!doc":"<p>Removes a state from an element.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, pos?: ?)","!doc":"<p>Scroll the passed node into view, if it is not already.\nDeprecated, use <code>windowUtils.scrollIntoView</code> instead.</p>\n"},"selectInputText":{"!type":"fn(element: ?, start?: ?, stop?: ?)","!doc":"<p>Select text in the input element argument, from start (default 0), to stop (default end).</p>\n"},"setWaiRole":{"!type":"fn(elem: ?, role: ?)","!doc":"<p>Sets the role on an element.</p>\n"},"setWaiState":{"!type":"fn(elem: ?, state: ?, value: ?)","!doc":"<p>Sets a state on an element.</p>\n"},"showTooltip":{"!type":"fn(innerHTML: ?, aroundNode: ?, position?: ?, rtl?: ?, textDir?: ?) -> ?","!doc":"<p>Static method to display tooltip w/specified contents in specified position.\nSee description of dijit/Tooltip.defaultPosition for details on position parameter.\nIf position is not specified then dijit/Tooltip.defaultPosition is used.</p>\n"},"unregisterIframe":{"!type":"fn(handle: ?)","!doc":"<p>Unregisters listeners on the specified iframe created by registerIframe.\nAfter calling be sure to delete or null out the handle itself.</p>\n"},"unregisterWin":{"!type":"fn(handle: ?)","!doc":"<p>Unregisters listeners on the specified window (either the main\nwindow or an iframe&#39;s window) according to handle returned from registerWin().\nAfter calling be sure to delete or null out the handle itself.</p>\n"}},"dijit/main.registry":{"!doc":"<p>Registry of existing widget on page, plus some utility methods.</p>\n","_hash":{},"length":{"!doc":"<p>Number of registered widgets</p>\n"},"add":{"!type":"fn(widget: ?)","!doc":"<p>Add a widget to the registry. If a duplicate ID is detected, a error is thrown.</p>\n"},"byId":{"!type":"fn(id: ?) -> string","!doc":"<p>Find a widget by it&#39;s id.\nIf passed a widget then just returns the widget.</p>\n"},"byNode":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the widget corresponding to the given DOMNode</p>\n"},"findWidgets":{"!type":"fn(root: ?, skipNode: ?) -> []","!doc":"<p>Search subtree under root returning widgets found.\nDoesn&#39;t search for nested widgets (ie, widgets inside other widgets).</p>\n"},"getEnclosingWidget":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the widget whose DOM tree contains the specified DOMNode, or null if\nthe node is not contained within the DOM tree of any widget</p>\n"},"getUniqueId":{"!type":"fn(widgetType: ?) -> string","!doc":"<p>Generates a unique id for a given widgetType</p>\n"},"remove":{"!type":"fn(id: ?)","!doc":"<p>Remove a widget from the registry. Does not destroy the widget; simply\nremoves the reference.</p>\n"},"toArray":{"!type":"fn() -> []","!doc":"<p>Convert registry into a true Array\n</p>\n"}},"dijit/main.place":{"!doc":"<p>Code to place a DOMNode relative to another DOMNode.\nLoad using require([&quot;dijit/place&quot;], function(place){ ... }).</p>\n","__Position":{},"__Rectangle":{},"around":{"!type":"fn(node: ?, anchor: ?, positions: ?, leftToRight: ?, layoutNode?: ?) -> ?","!doc":"<p>Position node adjacent or kitty-corner to anchor\nsuch that it&#39;s fully visible in viewport.</p>\n"},"at":{"!type":"fn(node: ?, pos: ?, corners: ?, padding?: ?, layoutNode: ?) -> ?","!doc":"<p>Positions node kitty-corner to the rectangle centered at (pos.x, pos.y) with width and height of\npadding.x <em> 2 and padding.y </em> 2, or zero if padding not specified.  Picks first corner in corners[]\nwhere node is fully visible, or the corner where it&#39;s most visible.\n\n</p>\n<p>Node is assumed to be absolutely or relatively positioned.</p>\n"}},"dijit/main.typematic":{"!doc":"<p>These functions are used to repetitively call a user specified callback\nmethod when a specific key or mouse click over a specific DOM node is\nheld down for a specific amount of time.\nOnly 1 such event is allowed to occur on the browser page at 1 time.</p>\n","addKeyListener":{"!type":"fn(node: ?, keyObject: ?, _this: ?, callback: ?, subsequentDelay: ?, initialDelay: ?, minDelay?: ?) -> +any","!doc":"<p>Start listening for a specific typematic key.\nSee also the trigger method for other parameters.</p>\n"},"addListener":{"!type":"fn(mouseNode: ?, keyNode: ?, keyObject: ?, _this: ?, callback: ?, subsequentDelay: ?, initialDelay: ?, minDelay?: ?) -> +any","!doc":"<p>Start listening for a specific typematic key and mouseclick.\nThis is a thin wrapper to addKeyListener and addMouseListener.\nSee the addMouseListener and addKeyListener methods for other parameters.</p>\n"},"addMouseListener":{"!type":"fn(node: ?, _this: ?, callback: ?, subsequentDelay: ?, initialDelay: ?, minDelay?: ?) -> +any","!doc":"<p>Start listening for a typematic mouse click.\nSee the trigger method for other parameters.</p>\n"},"stop":{"!type":"fn()","!doc":"<p>Stop an ongoing timed, repeating callback sequence.</p>\n"},"trigger":{"!type":"fn(evt: ?, _this: ?, node: ?, callback: ?, obj: ?, subsequentDelay?: ?, initialDelay?: ?, minDelay?: ?)","!doc":"<p>Start a timed, repeating callback sequence.\nIf already started, the function call is ignored.\nThis method is not normally called by the user but can be\nwhen the normal listener code is insufficient.</p>\n"}},"dijit/main._Calendar":{"!type":"fn()","prototype":{"!proto":"dijit/CalendarLite.prototype","!doc":"<p>A simple GUI for choosing a date in the context of a monthly calendar.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setSummaryAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"currentFocus":{"!doc":"<p>Date object containing the currently focused date, or the date which would be focused\nif the calendar itself was focused.   Also indicates which year and month to display,\ni.e. the current &quot;page&quot; the calendar is on.</p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dateTemplateString":{},"dayWidth":{"!doc":"<p>How to represent the days of the week in the calendar header. See locale</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dowTemplateString":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The currently selected Date, initially set to invalid date to indicate no selection.</p>\n"},"weekTemplateString":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus the calendar by focusing one of the calendar cells</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()","!doc":"<p>Sets calendar&#39;s value to today&#39;s date</p>\n"},"handleKey":{"!type":"fn(evt: ?) -> +any","!doc":"<p>Provides keyboard navigation of calendar.</p>\n"},"isDisabledDate":{"!type":"fn(dateObject: ?, locale?: ?) -> bool","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.   Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/Calendar":{"!type":"fn()","prototype":{"!proto":"dijit/CalendarLite.prototype","!doc":"<p>A simple GUI for choosing a date in the context of a monthly calendar.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setSummaryAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"currentFocus":{"!doc":"<p>Date object containing the currently focused date, or the date which would be focused\nif the calendar itself was focused.   Also indicates which year and month to display,\ni.e. the current &quot;page&quot; the calendar is on.</p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dateTemplateString":{},"dayWidth":{"!doc":"<p>How to represent the days of the week in the calendar header. See locale</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dowTemplateString":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The currently selected Date, initially set to invalid date to indicate no selection.</p>\n"},"weekTemplateString":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus the calendar by focusing one of the calendar cells</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()","!doc":"<p>Sets calendar&#39;s value to today&#39;s date</p>\n"},"handleKey":{"!type":"fn(evt: ?) -> +any","!doc":"<p>Provides keyboard navigation of calendar.</p>\n"},"isDisabledDate":{"!type":"fn(dateObject: ?, locale?: ?) -> bool","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.   Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/Calendar._MonthDropDownButton":{"!type":"fn()","prototype":{"!proto":"dijit/form/DropDownButton.prototype","!doc":"<p>DropDownButton for the current month.    Displays name of current month\nand a list of month names in the drop down</p>\n","_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_setValueAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(callback: ?)"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/Calendar._MonthDropDown":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>The list-of-months drop down from the MonthDropDownButton</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"months":{"!doc":"<p>List of names of months, possibly w/some undefined entries for Hebrew leap months\n(ex: [&quot;January&quot;, &quot;February&quot;, undefined, &quot;April&quot;, ...])</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/CalendarLite":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Lightweight version of Calendar widget aimed towards mobile use\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setSummaryAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"currentFocus":{"!doc":"<p>Date object containing the currently focused date, or the date which would be focused\nif the calendar itself was focused.   Also indicates which year and month to display,\ni.e. the current &quot;page&quot; the calendar is on.</p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dateTemplateString":{},"dayWidth":{"!doc":"<p>How to represent the days of the week in the calendar header. See locale</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dowTemplateString":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The currently selected Date, initially set to invalid date to indicate no selection.</p>\n"},"weekTemplateString":{},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus the calendar by focusing one of the calendar cells</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()","!doc":"<p>Sets calendar&#39;s value to today&#39;s date</p>\n"},"isDisabledDate":{"!type":"fn(dateObject: ?, locale?: ?) -> bool","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/CalendarLite._MonthWidget":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Displays name of current month padded to the width of the month\nw/the longest name, so that changing months doesn&#39;t change width.\n\n</p>\n<p>Create as:\n\n</p>\n<pre><code>new Calendar._MonthWidget({\n        lang: ...,\n        dateLocaleModule: ...\n    })</code></pre>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_WidgetBase":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>Future base class for all Dijit widgets.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/Destroyable":{"!type":"fn()","prototype":{"!doc":"<p>Mixin to track handles and release them when instance is destroyed.</p>\n","destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this class, releasing any resources registered via own().</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"}}},"dijit/registry":{"!doc":"<p>Registry of existing widget on page, plus some utility methods.</p>\n","_hash":{},"length":{"!doc":"<p>Number of registered widgets</p>\n"},"add":{"!type":"fn(widget: ?)","!doc":"<p>Add a widget to the registry. If a duplicate ID is detected, a error is thrown.</p>\n"},"byId":{"!type":"fn(id: ?) -> string","!doc":"<p>Find a widget by it&#39;s id.\nIf passed a widget then just returns the widget.</p>\n"},"byNode":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the widget corresponding to the given DOMNode</p>\n"},"findWidgets":{"!type":"fn(root: ?, skipNode: ?) -> []","!doc":"<p>Search subtree under root returning widgets found.\nDoesn&#39;t search for nested widgets (ie, widgets inside other widgets).</p>\n"},"getEnclosingWidget":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the widget whose DOM tree contains the specified DOMNode, or null if\nthe node is not contained within the DOM tree of any widget</p>\n"},"getUniqueId":{"!type":"fn(widgetType: ?) -> string","!doc":"<p>Generates a unique id for a given widgetType</p>\n"},"remove":{"!type":"fn(id: ?)","!doc":"<p>Remove a widget from the registry. Does not destroy the widget; simply\nremoves the reference.</p>\n"},"toArray":{"!type":"fn() -> []","!doc":"<p>Convert registry into a true Array\n</p>\n"}},"dijit/registry._hash":{},"dijit/_TemplatedMixin":{"!type":"fn()","prototype":{"!proto":"dijit/_AttachMixin.prototype","!doc":"<p>Mixin for widgets that are instantiated from a template</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_skipNodeCache":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"searchContainerNode":{},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget from a template, setting this.domNode.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"destroyRendering":{"!type":"fn()"}},"_templateCache":{},"getCachedTemplate":{"!type":"fn(templateString: ?, alwaysUseString: ?, doc?: ?) -> +Mixed","!doc":"<p>Static method to get a template based on the templatePath or\ntemplateString key</p>\n"}},"dijit/_TemplatedMixin._templateCache":{},"dijit/_AttachMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for widgets to attach to dom nodes and setup events via\nconvenient data-dojo-attach-point and data-dojo-attach-event DOM attributes.\n\n</p>\n<p>Superclass of _TemplatedMixin, and can also be used standalone when templates are pre-rendered on the\nserver.\n\n</p>\n<p>Does not [yet] handle widgets like ContentPane with this.containerNode set.   It should skip\nscanning for data-dojo-attach-point and data-dojo-attach-event inside this.containerNode, but it\ndoesn&#39;t.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"searchContainerNode":{"!doc":"<p>Search descendants of this.containerNode for data-dojo-attach-point and data-dojo-attach-event.\nShould generally be left false (the default value) both for performance and to avoid failures when\nthis.containerNode holds other _AttachMixin instances with their own attach points and events.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Attach to DOM nodes marked with special attributes.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"destroyRendering":{"!type":"fn()"}}},"dijit/a11yclick":{"!doc":"<p>Custom press, release, and click synthetic events\nwhich trigger on a left mouse click, touch, or space/enter keyup.</p>\n","click":{"!type":"fn(node: ?, listener: ?)","!doc":"<p>Logical click operation for mouse, touch, or keyboard (space/enter key)</p>\n"},"move":{"!type":"fn(node: ?, listener: ?)","!doc":"<p>Mouse cursor or a finger is dragged over the given node.</p>\n"},"press":{"!type":"fn(node: ?, listener: ?)","!doc":"<p>Mousedown (left button), touchstart, or keydown (space or enter) corresponding to logical click operation.</p>\n"},"release":{"!type":"fn(node: ?, listener: ?)","!doc":"<p>Mouseup (left button), touchend, or keyup (space or enter) corresponding to logical click operation.</p>\n"}},"dijit/hccss":{"!doc":"<p>Test if computer is in high contrast mode, and sets <code>dijit_a11y</code> flag on <code>&lt;body&gt;</code> if it is.\nDeprecated, use <code>dojo/hccss</code> instead.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Test if computer is in high contrast mode, and sets <code>dijit_a11y</code> flag on <code>&lt;body&gt;</code> if it is.\nDeprecated, use <code>dojo/hccss</code> instead.</p>\n"}}},"dijit/_Widget":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Old base class for widgets.   New widgets should extend <code>dijit/_WidgetBase</code> instead</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dndType":{"!doc":"<p>Defines a type of widget.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"slideFrom":{"!doc":"<p>A parameter needed by RadioGroupSlide only. An optional paramter to force\nthe ContentPane to slide in from a set direction. Defaults\nto &quot;random&quot;, or specify one of &quot;top&quot;, &quot;left&quot;, &quot;right&quot;, &quot;bottom&quot;\nto slideFrom top, left, right, or bottom.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_OnDijitClickMixin":{"!type":"fn()","prototype":{"!doc":"<p>Deprecated.   New code should access the dijit/a11yclick event directly, ex:\n\n</p>\n<pre><code>this.own(on(node, a11yclick, function(){ ... }));</code></pre>\n<p>Mixing in this class will make _WidgetBase.connect(node, &quot;ondijitclick&quot;, ...) work.\nIt also used to be necessary to make templates with ondijitclick work, but now you can just require\ndijit/a11yclick.</p>\n","connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?"}},"a11yclick":{"!doc":"<p>Custom press, release, and click synthetic events\nwhich trigger on a left mouse click, touch, or space/enter keyup.</p>\n"}},"dijit/_OnDijitClickMixin.a11yclick":{"!doc":"<p>Custom press, release, and click synthetic events\nwhich trigger on a left mouse click, touch, or space/enter keyup.</p>\n","click":{"!type":"fn(node: ?, listener: ?)","!doc":"<p>Logical click operation for mouse, touch, or keyboard (space/enter key)</p>\n"},"move":{"!type":"fn(node: ?, listener: ?)","!doc":"<p>Mouse cursor or a finger is dragged over the given node.</p>\n"},"press":{"!type":"fn(node: ?, listener: ?)","!doc":"<p>Mousedown (left button), touchstart, or keydown (space or enter) corresponding to logical click operation.</p>\n"},"release":{"!type":"fn(node: ?, listener: ?)","!doc":"<p>Mouseup (left button), touchend, or keyup (space or enter) corresponding to logical click operation.</p>\n"}},"dijit/_FocusMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin to widget to provide _onFocus() and _onBlur() methods that\nfire when a widget or its descendants get/lose focus</p>\n","_focusManager":{}}},"dijit/focus":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>Tracks the currently focused node, and which widgets are currently &quot;active&quot;.\nAccess via require([&quot;dijit/focus&quot;], function(focus){ ... }).\n\n</p>\n<p>A widget is considered active if it or a descendant widget has focus,\nor if a non-focusable node of this widget or a descendant was recently clicked.\n\n</p>\n<p>Call focus.watch(&quot;curNode&quot;, callback) to track the current focused DOMNode,\nor focus.watch(&quot;activeStack&quot;, callback) to track the currently focused stack of widgets.\n\n</p>\n<p>Call focus.on(&quot;widget-blur&quot;, func) or focus.on(&quot;widget-focus&quot;, ...) to monitor when\nwhen widgets become active/inactive\n\n</p>\n<p>Finally, focus(node) will focus a node, suppressing errors if the node doesn&#39;t exist.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"activeStack":{"!doc":"<p>List of currently active widgets (focused widget and it&#39;s ancestors)</p>\n"},"curNode":{"!doc":"<p>Currently focused item on screen</p>\n"},"constructor":{"!type":"fn()"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"focus":{"!type":"fn(node: ?)","!doc":"<p>Focus the specified node, suppressing errors if they occur</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"postscript":{"!type":"fn(params?: ?)"},"registerIframe":{"!type":"fn(iframe: ?) -> +any","!doc":"<p>Registers listeners on the specified iframe so that any click\nor focus event on that iframe (or anything in it) is reported\nas a focus/click event on the <code>&lt;iframe&gt;</code> itself.</p>\n"},"registerWin":{"!type":"fn(targetWindow?: ?, effectiveNode?: ?) -> +any","!doc":"<p>Registers listeners on the specified window (either the main\nwindow or an iframe&#39;s window) to detect when the user has clicked somewhere\nor focused somewhere.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/a11y":{"!doc":"<p>Accessibility utility functions (keyboard, tab stops, etc.)</p>\n","effectiveTabIndex":{"!type":"fn(elem: ?) -> ?","!doc":"<p>Returns effective tabIndex of an element, either a number, or undefined if element isn&#39;t focusable.</p>\n"},"getFirstInTabbingOrder":{"!type":"fn(root: ?, doc?: ?) -> ?","!doc":"<p>Finds the descendant of the specified root node\nthat is first in the tabbing order</p>\n"},"getLastInTabbingOrder":{"!type":"fn(root: ?, doc?: ?) -> ?","!doc":"<p>Finds the descendant of the specified root node\nthat is last in the tabbing order</p>\n"},"hasDefaultTabStop":{"!type":"fn(elem: ?) -> ?","!doc":"<p>Tests if element is tab-navigable even without an explicit tabIndex setting</p>\n"},"isFocusable":{"!type":"fn(elem: ?) -> bool","!doc":"<p>Tests if an element is focusable by tabbing to it, or clicking it with the mouse.</p>\n"},"isTabNavigable":{"!type":"fn(elem: ?) -> bool","!doc":"<p>Tests if an element is tab-navigable</p>\n"}},"dijit/_CssStateMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus\nstate changes, and also higher-level state changes such becoming disabled or selected.\n</p>\n","active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"}}},"dijit/form/DropDownButton":{"!type":"fn()","prototype":{"!proto":"dijit/form/Button.prototype","!doc":"<p>A button with a drop down\n</p>\n","_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_setValueAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(callback: ?)"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/popup":{"!type":"fn()","prototype":{"!doc":"<p>Used to show drop downs (ex: the select list of a ComboBox)\nor popups (ex: right-click context menus).</p>\n","_beginZIndex":{"!doc":"<p>Z-index of the first popup.   (If first popup opens other\npopups they get a higher z-index.)</p>\n"},"_idGen":{},"_stack":{"!doc":"<p>Stack of currently popped up widgets.\n(someone opened _stack[0], and then it opened _stack[1], etc.)</p>\n"},"close":{"!type":"fn(popup?: ?)","!doc":"<p>Close specified popup and any popups that it parented.\nIf no popup is specified, closes all popups.</p>\n"},"getTopPopup":{"!type":"fn() -> ?","!doc":"<p>Compute the closest ancestor popup that&#39;s <em>not</em> a child of another popup.\nEx: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.</p>\n"},"hide":{"!type":"fn(widget: ?)","!doc":"<p>Hide this popup widget (until it is ready to be shown).\nInitialization for widgets that will be used as popups\n\n</p>\n<p>Also puts widget inside a wrapper DIV (if not already in one)\n\n</p>\n<p>If popup widget needs to layout it should\ndo so when it is made visible, and popup._onShow() is called.</p>\n"},"moveOffScreen":{"!type":"fn(widget: ?) -> ?","!doc":"<p>Moves the popup widget off-screen.\nDo not use this method to hide popups when not in use, because\nthat will create an accessibility issue: the offscreen popup is\nstill in the tabbing order.</p>\n"},"open":{"!type":"fn(args: ?) -> ?","!doc":"<p>Popup the widget at the specified position\n</p>\n"}}},"dijit/place":{"!doc":"<p>Code to place a DOMNode relative to another DOMNode.\nLoad using require([&quot;dijit/place&quot;], function(place){ ... }).</p>\n","__Position":{},"__Rectangle":{},"around":{"!type":"fn(node: ?, anchor: ?, positions: ?, leftToRight: ?, layoutNode?: ?) -> ?","!doc":"<p>Position node adjacent or kitty-corner to anchor\nsuch that it&#39;s fully visible in viewport.</p>\n"},"at":{"!type":"fn(node: ?, pos: ?, corners: ?, padding?: ?, layoutNode: ?) -> ?","!doc":"<p>Positions node kitty-corner to the rectangle centered at (pos.x, pos.y) with width and height of\npadding.x <em> 2 and padding.y </em> 2, or zero if padding not specified.  Picks first corner in corners[]\nwhere node is fully visible, or the corner where it&#39;s most visible.\n\n</p>\n<p>Node is assumed to be absolutely or relatively positioned.</p>\n"}},"dijit/place.__Position":{"x":{"!doc":"<p>horizontal coordinate in pixels, relative to document body</p>\n"},"y":{"!doc":"<p>vertical coordinate in pixels, relative to document body</p>\n"}},"dijit/place.__Rectangle":{"h":{"!doc":"<p>height in pixels.   Can also be specified as &quot;height&quot; for backwards-compatibility.</p>\n"},"w":{"!doc":"<p>width in pixels.   Can also be specified as &quot;width&quot; for backwards-compatibility.</p>\n"},"x":{"!doc":"<p>horizontal offset in pixels, relative to document body</p>\n"},"y":{"!doc":"<p>vertical offset in pixels, relative to document body</p>\n"}},"dijit/Viewport":{"!doc":"<p>Utility singleton to watch for viewport resizes, avoiding duplicate notifications\nwhich can lead to infinite loops.</p>\n"},"dijit/form/Button":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormWidget.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_setValueAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_FormWidget":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Base class for widgets corresponding to native HTML elements such as <code>&lt;checkbox&gt;</code> or <code>&lt;button&gt;</code>,\nwhich can be children of a <code>&lt;form&gt;</code> node or a <code>dijit/form/Form</code> widget.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_FormWidgetMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for widgets corresponding to native HTML elements such as <code>&lt;checkbox&gt;</code> or <code>&lt;button&gt;</code>,\nwhich can be children of a <code>&lt;form&gt;</code> node or a <code>dijit/form/Form</code> widget.\n</p>\n","_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setIdAttr":{},"_setTabIndexAttr":{},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"create":{"!type":"fn()"},"destroy":{"!type":"fn()"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Tells if this widget is focusable or not.  Used internally by dijit.</p>\n"}}},"dijit/form/_ButtonMixin":{"!type":"fn()","prototype":{"label":{"!doc":"<p>Content to display in button.</p>\n"},"type":{"!doc":"<p>Type of button (submit, reset, button, checkbox, radio)</p>\n"},"postCreate":{"!type":"fn()"}}},"dijit/_Container":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for widgets that contain HTML and/or a set of widget children.</p>\n","addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"}}},"dijit/_HasDropDown":{"!type":"fn()","prototype":{"!proto":"dijit/_FocusMixin.prototype","!doc":"<p>Mixin for widgets that need drop down ability.</p>\n","_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"destroy":{"!type":"fn()"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>set up nodes and connect our mouse and keyboard events</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"}}},"dijit/CheckedMenuItem":{"!type":"fn()","prototype":{"!proto":"dijit/MenuItem.prototype","!doc":"<p>A checkbox-like menu item for toggling on and off</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"accelKey":{"!doc":"<p>Text for the accelerator (shortcut) key combination, a control, alt, etc. modified keystroke meant to\nexecute the menu item regardless of where the focus is on the page.\n\n</p>\n<p>Note that although Menu can display accelerator keys, there is no infrastructure to actually catch and\nexecute those accelerators.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"checked":{"!doc":"<p>Our checked state</p>\n"},"checkedChar":{"!doc":"<p>Character (or string) used in place of checkbox icon when display in high contrast mode</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>If true, the menu item is disabled.\nIf false, the menu item is enabled.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>Menu text as HTML</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"role":{},"searchContainerNode":{},"shortcutKey":{"!doc":"<p>Single character (underlined when the parent Menu is focused) used to navigate directly to this widget,\nalso known as <a href=\"http://en.wikipedia.org/wiki/Mnemonics_(keyboard%29\">a mnemonic</a>.\nThis is denoted in the label by surrounding the single character with {}.\nFor example, if label=&quot;{F}ile&quot;, then shortcutKey=&quot;F&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus on this MenuItem</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.   Use set(&#39;disabled&#39;, bool) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.   Use set(&#39;label&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/MenuItem":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"accelKey":{"!doc":"<p>Text for the accelerator (shortcut) key combination, a control, alt, etc. modified keystroke meant to\nexecute the menu item regardless of where the focus is on the page.\n\n</p>\n<p>Note that although Menu can display accelerator keys, there is no infrastructure to actually catch and\nexecute those accelerators.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>If true, the menu item is disabled.\nIf false, the menu item is enabled.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode to make it display an icon.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>Menu text as HTML</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"shortcutKey":{"!doc":"<p>Single character (underlined when the parent Menu is focused) used to navigate directly to this widget,\nalso known as <a href=\"http://en.wikipedia.org/wiki/Mnemonics_(keyboard%29\">a mnemonic</a>.\nThis is denoted in the label by surrounding the single character with {}.\nFor example, if label=&quot;{F}ile&quot;, then shortcutKey=&quot;F&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus on this MenuItem</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.   Use set(&#39;disabled&#39;, bool) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.   Use set(&#39;label&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_Contained":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for widgets that are children of a container widget</p>\n","getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"}}},"dijit/ColorPalette":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A keyboard accessible color-picking widget</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_currentFocus":{"!doc":"<p>The currently focused cell (if the palette itself has focus), or otherwise\nthe cell to be focused when the palette itself gets focus.\nDifferent from value, which represents the selected (i.e. clicked) cell.</p>\n"},"_focusManager":{},"_palettes":{"!doc":"<p>This represents the value of the colors.\nThe first level is a hashmap of the different palettes available.\nThe next two dimensions represent the columns and rows of colors.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selectedCell":{"!doc":"<p>Index of the currently selected cell. Initially, none selected</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_xDim":{"!doc":"<p>This is the number of cells horizontally across.</p>\n"},"_yDim":{"!doc":"<p>This is the number of cells vertically down.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"cellClass":{"!doc":"<p>CSS class applied to each cell in the palette</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"defaultTimeout":{"!doc":"<p>Number of milliseconds before a held key or button becomes typematic</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dyeClass":{"!doc":"<p>Constructor for Object created for each cell of the palette.\ndyeClass should implement the dijit/_PaletteMixin.__Dye interface.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"palette":{"!doc":"<p>Size of grid, either &quot;7x10&quot; or &quot;3x4&quot;.</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Widget tab index.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template of this widget.</p>\n"},"timeoutChangeRate":{"!doc":"<p>Fraction of time used to change the typematic timer between events\n1.0 means that each typematic event fires at defaultTimeout intervals\nLess than 1.0 means that each typematic event fires at an increasing faster rate</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>Currently selected color/emoticon/etc.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus this widget.  Puts focus on the most recently focused cell.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/ColorPalette._Color":{"!type":"fn()","prototype":{"!proto":"dojo/_base/Color.prototype","!doc":"<p>Object associated with each cell in a ColorPalette palette.\nImplements dijit/Dye.</p>\n","_imagePaths":{"!doc":"<p>This is stores the path to the palette images used for high-contrast mode display</p>\n"},"a":{},"b":{},"g":{},"hcTemplate":{},"r":{},"template":{},"constructor":{"!type":"fn(alias: ?, row: ?, col: ?, title: ?)","!doc":"<p>Constructor for ColorPalette._Color</p>\n"},"fillCell":{"!type":"fn(cell: ?, blankGif: ?)"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Note that although dijit._Color is initialized with a value like &quot;white&quot; getValue() always\nreturns a hex value</p>\n"},"sanitize":{"!type":"fn()","!doc":"<p>makes sure that the object has correct attributes</p>\n"},"setColor":{"!type":"fn(color: ?) -> fn","!doc":"<p>Takes a named string, hex string, array of rgb or rgba values,\nan object with r, g, b, and a properties, or another <code>Color</code> object\nand sets this color instance to that value.\n</p>\n"},"toCss":{"!type":"fn(includeAlpha?: ?) -> string","!doc":"<p>Returns a css color string in rgb(a) representation</p>\n"},"toHex":{"!type":"fn() -> string","!doc":"<p>Returns a CSS color string in hexadecimal representation</p>\n"},"toRgb":{"!type":"fn() -> []","!doc":"<p>Returns 3 component array of rgb values</p>\n"},"toRgba":{"!type":"fn() -> []","!doc":"<p>Returns a 4 component array of rgba values from the color\nrepresented by this object.</p>\n"},"toString":{"!type":"fn() -> ?","!doc":"<p>Returns a visual representation of the color</p>\n"}}},"dijit/_PaletteMixin":{"!type":"fn()","prototype":{"!proto":"dijit/_CssStateMixin.prototype","!doc":"<p>A keyboard accessible palette, for picking a color/emoticon/etc.</p>\n","_currentFocus":{"!doc":"<p>The currently focused cell (if the palette itself has focus), or otherwise\nthe cell to be focused when the palette itself gets focus.\nDifferent from value, which represents the selected (i.e. clicked) cell.</p>\n"},"_selectedCell":{"!doc":"<p>Index of the currently selected cell. Initially, none selected</p>\n"},"_xDim":{"!doc":"<p>This is the number of cells horizontally across.</p>\n"},"_yDim":{"!doc":"<p>This is the number of cells vertically down.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"cellClass":{"!doc":"<p>CSS class applied to each cell in the palette</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"defaultTimeout":{"!doc":"<p>Number of milliseconds before a held key or button becomes typematic</p>\n"},"dyeClass":{"!doc":"<p>Constructor for Object created for each cell of the palette.\ndyeClass should implement the dijit/_PaletteMixin.__Dye interface.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"tabIndex":{"!doc":"<p>Widget tab index.</p>\n"},"timeoutChangeRate":{"!doc":"<p>Fraction of time used to change the typematic timer between events\n1.0 means that each typematic event fires at defaultTimeout intervals\nLess than 1.0 means that each typematic event fires at an increasing faster rate</p>\n"},"value":{"!doc":"<p>Currently selected color/emoticon/etc.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus this widget.  Puts focus on the most recently focused cell.</p>\n"},"postCreate":{"!type":"fn()"}}},"dijit/_PaletteMixin.__Dye":{"!type":"fn()","prototype":{"!doc":"<p>Interface for the JS Object associated with a palette cell (i.e. DOMNode)</p>\n","constructor":{"!type":"fn(alias: ?, row: ?, col: ?)","!doc":"<p>Initialize according to value or alias like &quot;white&quot;</p>\n"},"fillCell":{"!type":"fn(cell: ?, blankGif: ?)","!doc":"<p>Add cell DOMNode inner structure</p>\n"},"getValue":{"!type":"fn()","!doc":"<p>Return &quot;value&quot; of cell; meaning of &quot;value&quot; varies by subclass.</p>\n"}}},"dijit/typematic":{"!doc":"<p>These functions are used to repetitively call a user specified callback\nmethod when a specific key or mouse click over a specific DOM node is\nheld down for a specific amount of time.\nOnly 1 such event is allowed to occur on the browser page at 1 time.</p>\n","addKeyListener":{"!type":"fn(node: ?, keyObject: ?, _this: ?, callback: ?, subsequentDelay: ?, initialDelay: ?, minDelay?: ?) -> +any","!doc":"<p>Start listening for a specific typematic key.\nSee also the trigger method for other parameters.</p>\n"},"addListener":{"!type":"fn(mouseNode: ?, keyNode: ?, keyObject: ?, _this: ?, callback: ?, subsequentDelay: ?, initialDelay: ?, minDelay?: ?) -> +any","!doc":"<p>Start listening for a specific typematic key and mouseclick.\nThis is a thin wrapper to addKeyListener and addMouseListener.\nSee the addMouseListener and addKeyListener methods for other parameters.</p>\n"},"addMouseListener":{"!type":"fn(node: ?, _this: ?, callback: ?, subsequentDelay: ?, initialDelay: ?, minDelay?: ?) -> +any","!doc":"<p>Start listening for a typematic mouse click.\nSee the trigger method for other parameters.</p>\n"},"stop":{"!type":"fn()","!doc":"<p>Stop an ongoing timed, repeating callback sequence.</p>\n"},"trigger":{"!type":"fn(evt: ?, _this: ?, node: ?, callback: ?, obj: ?, subsequentDelay?: ?, initialDelay?: ?, minDelay?: ?)","!doc":"<p>Start a timed, repeating callback sequence.\nIf already started, the function call is ignored.\nThis method is not normally called by the user but can be\nwhen the normal listener code is insufficient.</p>\n"}},"dijit/Declaration":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>The Declaration widget allows a developer to declare new widget\nclasses directly from a snippet of markup.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_noScript":{"!doc":"<p>Flag to parser to leave alone the script tags contained inside of me</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaults":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"mixins":{"!doc":"<p>List containing the prototype for this widget, and also any mixins,\nex: [&quot;dijit._Widget&quot;, &quot;dijit._Container&quot;]</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside of me</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetClass":{"!doc":"<p>Name of class being declared, ex: &quot;acme.myWidget&quot;</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_WidgetsInTemplateMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin to supplement _TemplatedMixin when template contains widgets</p>\n","_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"startup":{"!type":"fn()"}}},"dijit/Dialog":{"!type":"fn()","prototype":{"!proto":"dijit/layout/ContentPane.prototype","!doc":"<p>A modal dialog Widget.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_firstFocusItem":{"!doc":"<p>The pointer to the first focusable node in the dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_focusManager":{},"_lastFocusItem":{"!doc":"<p>The pointer to which node has focus prior to our dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autofocus":{"!doc":"<p>A Toggle to modify the default focus behavior of a Dialog, which\nis to focus on the first dialog element after opening the dialog.\nFalse will disable autofocusing. Default: true</p>\n"},"baseClass":{},"class":{},"closable":{"!doc":"<p>Dialog show [x] icon to close itself, and ESC key will close the dialog.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"cssStateNodes":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"draggable":{"!doc":"<p>Toggles the movable aspect of the Dialog. If true, Dialog\ncan be dragged by it&#39;s title. If false it will remain centered\nin the viewport.</p>\n"},"duration":{"!doc":"<p>The time in milliseconds it takes the dialog to fade in and out</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"maxRatio":{"!doc":"<p>Maximum size to allow the dialog to expand to, relative to viewport size</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"open":{"!doc":"<p>True if Dialog is currently displayed on screen.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refocus":{"!doc":"<p>A Toggle to modify the default focus behavior of a Dialog, which\nis to re-focus the element which had focus before being opened.\nFalse will disable refocusing. Default: true</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Will be &quot;Error&quot; if one or more of the child widgets has an invalid value,\n&quot;Incomplete&quot; if not all of the required child widgets are filled in.  Otherwise, &quot;&quot;,\nwhich indicates that the form is ready to be submitted.</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectChildren":{"!type":"fn(inStartup: ?)","!doc":"<p>You can call this function directly, ex. in the event that you\nprogrammatically add a widget to the form <em>after</em> the form has been\ninitialized.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"disconnectChildren":{"!type":"fn()","!doc":"<p>Deprecated method.   Applications no longer need to call this.   Remove for 2.0.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"execute":{"!type":"fn(formContents: ?)","!doc":"<p>Callback when the user hits the submit button.\nOverride this method to handle Dialog execution.</p>\n"},"focus":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValues":{"!type":"fn() -> ?"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn() -> +dojo","!doc":"<p>Hide the dialog</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns true if all of the widgets are valid.\nDeprecated, will be removed in 2.0.  Use get(&quot;state&quot;) instead.</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"setValues":{"!type":"fn(val: ?) -> ?"},"show":{"!type":"fn() -> +dojo","!doc":"<p>Display the dialog</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?","!doc":"<p>returns if the form is valid - same as isValid - but\nprovides a few additional (ui-specific) features:\n\n</p>\n<ol>\n<li>it will highlight any sub-widgets that are not valid</li>\n<li>it will call focus() on the first invalid sub-widget</li>\n</ol>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"_DialogLevelManager":{"!doc":"<p>Controls the various active &quot;levels&quot; on the page, starting with the\nstuff initially visible on the page (at z-index 0), and then having an entry for\neach Dialog shown.</p>\n"},"_dialogStack":{}},"dijit/Dialog._DialogBase":{"!type":"fn()","prototype":{"!proto":"dijit/_TemplatedMixin.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_firstFocusItem":{"!doc":"<p>The pointer to the first focusable node in the dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_lastFocusItem":{"!doc":"<p>The pointer to which node has focus prior to our dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_skipNodeCache":{},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"autofocus":{"!doc":"<p>A Toggle to modify the default focus behavior of a Dialog, which\nis to focus on the first dialog element after opening the dialog.\nFalse will disable autofocusing. Default: true</p>\n"},"baseClass":{},"closable":{"!doc":"<p>Dialog show [x] icon to close itself, and ESC key will close the dialog.</p>\n"},"cssStateNodes":{},"doLayout":{"!doc":"<p>Don&#39;t change this parameter from the default value.\nThis ContentPane parameter doesn&#39;t make sense for Dialog, since Dialog\nis never a child of a layout container, nor can you specify the size of\nDialog in order to control the size of an inner widget.</p>\n"},"draggable":{"!doc":"<p>Toggles the movable aspect of the Dialog. If true, Dialog\ncan be dragged by it&#39;s title. If false it will remain centered\nin the viewport.</p>\n"},"duration":{"!doc":"<p>The time in milliseconds it takes the dialog to fade in and out</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"maxRatio":{"!doc":"<p>Maximum size to allow the dialog to expand to, relative to viewport size</p>\n"},"open":{"!doc":"<p>True if Dialog is currently displayed on screen.</p>\n"},"refocus":{"!doc":"<p>A Toggle to modify the default focus behavior of a Dialog, which\nis to re-focus the element which had focus before being opened.\nFalse will disable refocusing. Default: true</p>\n"},"searchContainerNode":{},"state":{"!doc":"<p>Will be &quot;Error&quot; if one or more of the child widgets has an invalid value,\n&quot;Incomplete&quot; if not all of the required child widgets are filled in.  Otherwise, &quot;&quot;,\nwhich indicates that the form is ready to be submitted.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget from a template, setting this.domNode.</p>\n"},"connectChildren":{"!type":"fn(inStartup: ?)","!doc":"<p>You can call this function directly, ex. in the event that you\nprogrammatically add a widget to the form <em>after</em> the form has been\ninitialized.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"destroy":{"!type":"fn()"},"destroyRendering":{"!type":"fn()"},"disconnectChildren":{"!type":"fn()","!doc":"<p>Deprecated method.   Applications no longer need to call this.   Remove for 2.0.</p>\n"},"execute":{"!type":"fn(formContents: ?)","!doc":"<p>Callback when the user hits the submit button.\nOverride this method to handle Dialog execution.</p>\n"},"focus":{"!type":"fn()"},"getValues":{"!type":"fn() -> ?"},"hide":{"!type":"fn() -> +dojo","!doc":"<p>Hide the dialog</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns true if all of the widgets are valid.\nDeprecated, will be removed in 2.0.  Use get(&quot;state&quot;) instead.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()"},"setValues":{"!type":"fn(val: ?) -> ?"},"show":{"!type":"fn() -> +dojo","!doc":"<p>Display the dialog</p>\n"},"startup":{"!type":"fn()"},"validate":{"!type":"fn() -> ?","!doc":"<p>returns if the form is valid - same as isValid - but\nprovides a few additional (ui-specific) features:\n\n</p>\n<ol>\n<li>it will highlight any sub-widgets that are not valid</li>\n<li>it will call focus() on the first invalid sub-widget</li>\n</ol>\n"}}},"dijit/Dialog._DialogLevelManager":{"!doc":"<p>Controls the various active &quot;levels&quot; on the page, starting with the\nstuff initially visible on the page (at z-index 0), and then having an entry for\neach Dialog shown.</p>\n","_beginZIndex":{},"hide":{"!type":"fn(dialog: ?)","!doc":"<p>Called when the specified dialog is hidden/destroyed, after the fade-out\nanimation ends, in order to reset page focus, fix the underlay, etc.\nIf the specified dialog isn&#39;t open then does nothing.\n\n</p>\n<p>Caller is responsible for either setting display:none on the dialog domNode,\nor calling dijit/popup.hide(), or removing it from the page DOM.</p>\n"},"isTop":{"!type":"fn(dialog: ?) -> bool","!doc":"<p>Returns true if specified Dialog is the top in the task</p>\n"},"show":{"!type":"fn(dialog: ?, underlayAttrs: ?)","!doc":"<p>Call right before fade-in animation for new dialog.\nSaves current focus, displays/adjusts underlay for new dialog,\nand sets the z-index of the dialog itself.\n\n</p>\n<p>New dialog will be displayed on top of all currently displayed dialogs.\n\n</p>\n<p>Caller is responsible for setting focus in new dialog after the fade-in\nanimation completes.</p>\n"}},"dijit/_base/manager":{"!doc":"<p>Deprecated.  Shim to methods on registry, plus a few other declarations.\nNew code should access dijit/registry directly when possible.</p>\n","defaultDuration":{"!doc":"<p>The default fx.animation speed (in ms) to use for all Dijit\ntransitional fx.animations, unless otherwise specified\non a per-instance basis. Defaults to 200, overrided by\n<code>djConfig.defaultDuration</code></p>\n"}},"dijit/form/_FormMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for containers of form widgets (i.e. widgets that represent a single value\nand can be children of a <code>&lt;form&gt;</code> node or <code>dijit/form/Form</code> widget)</p>\n","state":{"!doc":"<p>Will be &quot;Error&quot; if one or more of the child widgets has an invalid value,\n&quot;Incomplete&quot; if not all of the required child widgets are filled in.  Otherwise, &quot;&quot;,\nwhich indicates that the form is ready to be submitted.</p>\n"},"connectChildren":{"!type":"fn(inStartup: ?)","!doc":"<p>You can call this function directly, ex. in the event that you\nprogrammatically add a widget to the form <em>after</em> the form has been\ninitialized.</p>\n"},"destroy":{"!type":"fn()"},"disconnectChildren":{"!type":"fn()","!doc":"<p>Deprecated method.   Applications no longer need to call this.   Remove for 2.0.</p>\n"},"getValues":{"!type":"fn() -> ?"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns true if all of the widgets are valid.\nDeprecated, will be removed in 2.0.  Use get(&quot;state&quot;) instead.</p>\n"},"reset":{"!type":"fn()"},"setValues":{"!type":"fn(val: ?) -> ?"},"startup":{"!type":"fn()"},"validate":{"!type":"fn() -> ?","!doc":"<p>returns if the form is valid - same as isValid - but\nprovides a few additional (ui-specific) features:\n\n</p>\n<ol>\n<li>it will highlight any sub-widgets that are not valid</li>\n<li>it will call focus() on the first invalid sub-widget</li>\n</ol>\n"}}},"dijit/_DialogMixin":{"!type":"fn()","prototype":{"!doc":"<p>This provides functions useful to Dialog and TooltipDialog</p>\n","execute":{"!type":"fn(formContents: ?)","!doc":"<p>Callback when the user hits the submit button.\nOverride this method to handle Dialog execution.</p>\n"}}},"dijit/DialogUnderlay":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A component used to block input behind a <code>dijit/Dialog</code>.\n\n</p>\n<p>Normally this class should not be instantiated directly, but rather shown and hidden via\nDialogUnderlay.show() and DialogUnderlay.hide().  And usually the module is not accessed directly\nat all, since the underlay is shown and hidden by Dialog.DialogLevelManager.\n\n</p>\n<p>The underlay itself can be styled based on and id:\n\n</p>\n<pre><code>#myDialog_underlay { background-color:red; }</code></pre>\n<p>In the case of <code>dijit.Dialog</code>, this id is based on the id of the Dialog,\nsuffixed with _underlay.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_modalConnects":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dialogId":{"!doc":"<p>Id of the dialog.... DialogUnderlay&#39;s id is based on this id</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hides the dialog underlay</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"show":{"!type":"fn()","!doc":"<p>Show the dialog underlay</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"_singleton":{},"hide":{"!type":"fn()","!doc":"<p>Hide the underlay.</p>\n"},"show":{"!type":"fn(attrs: ?, zIndex: ?)","!doc":"<p>Display the underlay with the given attributes set.  If the underlay is already displayed,\nthen adjust it&#39;s attributes as specified.</p>\n"}},"dijit/layout/ContentPane":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A widget containing an HTML fragment, specified inline\nor by uri.  Fragment may include widgets.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/_ContentPaneResizeMixin":{"!type":"fn()","prototype":{"!doc":"<p>Resize() functionality of ContentPane.   If there&#39;s a single layout widget\nchild then it will call resize() with the same dimensions as the ContentPane.\nOtherwise just calls resize on each child.\n\n</p>\n<p>Also implements basic startup() functionality, where starting the parent\nwill start the children</p>\n","doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>See <code>dijit/layout/_LayoutWidget.startup()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"}}},"dijit/layout/utils":{"!doc":"<p>Utility functions for doing layout</p>\n","layoutChildren":{"!type":"fn(container: ?, dim: ?, children: ?, changedRegionId?: ?, changedRegionSize?: ?)","!doc":"<p>Layout a bunch of child dom nodes within a parent dom node</p>\n"},"marginBox2contentBox":{"!type":"fn(node: ?, mb: ?) -> ?","!doc":"<p>Given the margin-box size of a node, return its content box size.\nFunctions like domGeometry.contentBox() but is more reliable since it doesn&#39;t have\nto wait for the browser to compute sizes.</p>\n"}},"dijit/DropDownMenu":{"!type":"fn()","prototype":{"!proto":"dijit/_MenuBase.prototype","!doc":"<p>A menu, without features for context menu (Meaning, drop down menu)</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"activated":{"!doc":"<p>This Menu has been clicked (mouse or via space/arrow key) or opened as a submenu,\nso mere mouseover will open submenus.  Focusing a menu via TAB does NOT automatically make it active\nsince TAB is a navigation operation and not a selection one.\nFor Windows apps, pressing the ALT key focuses the menubar menus (similar to TAB navigation) but the\nmenu is not active (ie no dropdown) until an item is clicked.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoFocus":{"!doc":"<p>A toggle to control whether or not a Menu gets focused when opened as a drop down from a MenuBar\nor DropDownButton/ComboButton.   Note though that it always get focused when opened via the keyboard.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parentMenu":{"!doc":"<p>pointer to menu that displayed me</p>\n"},"passivePopupDelay":{"!doc":"<p>For a passive (unclicked) Menu, number of milliseconds before hovering (without clicking) will cause\nthe popup to open.  Default is Infinity, meaning you need to click the menu to open it.</p>\n"},"popupDelay":{"!doc":"<p>After a menu has been activated (by clicking on it etc.), number of milliseconds before hovering\n(without clicking) another MenuItem causes that MenuItem&#39;s popup to automatically open.</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>Currently selected (a.k.a. highlighted) MenuItem, or null if no MenuItem is selected.\nIf a submenu is open, will be set to MenuItem that displayed the submenu.   OTOH, if\nthis Menu is in passive mode (i.e. hasn&#39;t been clicked yet), will be null, because\n&quot;selected&quot; is not merely &quot;hovered&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"childSelector":{"!type":"fn(node: ?) -> ?","!doc":"<p>Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children\nlike MenuSeparator.  If subclass overrides to a string (ex: &quot;&gt; *&quot;), the subclass must require dojo/query.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the first child.</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"startupKeyNavChildren":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_MenuBase":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Abstract base class for Menu and MenuBar.\nSubclass should implement _onUpArrow(), _onDownArrow(), _onLeftArrow(), and _onRightArrow().</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"activated":{"!doc":"<p>This Menu has been clicked (mouse or via space/arrow key) or opened as a submenu,\nso mere mouseover will open submenus.  Focusing a menu via TAB does NOT automatically make it active\nsince TAB is a navigation operation and not a selection one.\nFor Windows apps, pressing the ALT key focuses the menubar menus (similar to TAB navigation) but the\nmenu is not active (ie no dropdown) until an item is clicked.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoFocus":{"!doc":"<p>A toggle to control whether or not a Menu gets focused when opened as a drop down from a MenuBar\nor DropDownButton/ComboButton.   Note though that it always get focused when opened via the keyboard.</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parentMenu":{"!doc":"<p>pointer to menu that displayed me</p>\n"},"passivePopupDelay":{"!doc":"<p>For a passive (unclicked) Menu, number of milliseconds before hovering (without clicking) will cause\nthe popup to open.  Default is Infinity, meaning you need to click the menu to open it.</p>\n"},"popupDelay":{"!doc":"<p>After a menu has been activated (by clicking on it etc.), number of milliseconds before hovering\n(without clicking) another MenuItem causes that MenuItem&#39;s popup to automatically open.</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>Currently selected (a.k.a. highlighted) MenuItem, or null if no MenuItem is selected.\nIf a submenu is open, will be set to MenuItem that displayed the submenu.   OTOH, if\nthis Menu is in passive mode (i.e. hasn&#39;t been clicked yet), will be null, because\n&quot;selected&quot; is not merely &quot;hovered&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"childSelector":{"!type":"fn(node: ?) -> ?","!doc":"<p>Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children\nlike MenuSeparator.  If subclass overrides to a string (ex: &quot;&gt; *&quot;), the subclass must require dojo/query.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the first child.</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"startupKeyNavChildren":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_KeyNavContainer":{"!type":"fn()","prototype":{"!proto":"dijit/_FocusMixin.prototype","!doc":"<p>A _Container with keyboard navigation of its children.</p>\n","_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_searchString":{},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"buildRendering":{"!type":"fn()"},"childSelector":{"!type":"fn(node: ?) -> ?"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the first child.</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"postCreate":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"startup":{"!type":"fn()"},"startupKeyNavChildren":{"!type":"fn()"}}},"dijit/_KeyNavMixin":{"!type":"fn()","prototype":{"!proto":"dijit/_FocusMixin.prototype","!doc":"<p>A mixin to allow arrow key and letter key navigation of child or descendant widgets.\nIt can be used by dijit/_Container based widgets with a flat list of children,\nor more complex widgets like dijit/Tree.\n\n</p>\n<p>To use this mixin, the subclass must:\n\n</p>\n<pre><code>- Implement  _getNext(), _getFirst(), _getLast(), _onLeftArrow(), _onRightArrow()\n  _onDownArrow(), _onUpArrow() methods to handle home/end/left/right/up/down keystrokes.\n  Next and previous in this context refer to a linear ordering of the descendants used\n  by letter key search.\n- Set all descendants&#39; initial tabIndex to &quot;-1&quot;; both initial descendants and any\n  descendants added later, by for example addChild()\n- Define childSelector to a function or string that identifies focusable descendant widgets</code></pre>\n<p>Also, child widgets must implement a focus() method.</p>\n","_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_searchString":{},"childSelector":{"!doc":"<p>Selector (passed to on.selector()) used to identify what to treat as a child widget.   Used to monitor\nfocus events and set this.focusedChild.   Must be set by implementing class.   If this is a string\n(ex: &quot;&gt; *&quot;) then the implementing class must require dojo/query.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the first child.</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"postCreate":{"!type":"fn()"}}},"dijit/Editor":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/RichText.prototype","!doc":"<p>A rich text Editing widget\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_editorCommandsLocalized":{},"_focusManager":{},"_local2NativeFormatNames":{},"_mozSettingProps":{},"_NAME_CONTENT_SEP":{"!doc":"<p>USed to separate name from content.  Just a colon isn&#39;t safe.</p>\n"},"_native2LocalFormatNames":{},"_qcaCache":{},"_SEPARATOR":{"!doc":"<p>Used to concat contents from multiple editors into a single string,\nso they can be saved into a single <code>&lt;textarea&gt;</code> node.  See &quot;name&quot; attribute.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_steps":{},"_undoedSteps":{},"_updateTimer":{},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"captureEvents":{"!doc":"<p>Events which should be connected to the underlying editing\narea, events in this array will be addListener with\ncapture=true.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"customUndo":{"!doc":"<p>Whether we shall use custom undo/redo support instead of the native\nbrowser support. By default, we now use custom undo.  It works better\nthan native browser support and provides a consistent behavior across\nbrowsers with a minimal performance hit.  We already had the hit on\nthe slowest browser, IE, anyway.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>The editor is disabled; the text cannot be changed.</p>\n"},"disableSpellCheck":{"!doc":"<p>When true, disables the browser&#39;s native spell checking, if supported.\nWorks only in Firefox.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editActionInterval":{"!doc":"<p>When using customUndo, not every keystroke will be saved as a step.\nInstead typing (including delete) will be grouped together: after\na user stops typing for editActionInterval seconds, a step will be\nsaved; if a user resume typing within editActionInterval seconds,\nthe timeout will be restarted. By default, editActionInterval is 3\nseconds.</p>\n"},"events":{"!doc":"<p>events which should be connected to the underlying editing area</p>\n"},"extraPlugins":{"!doc":"<p>A list of extra plugin names which will be appended to plugins array</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusOnLoad":{"!doc":"<p>Focus into this widget when the page is loaded</p>\n"},"height":{"!doc":"<p>Set height to fix the editor at a specific height, with scrolling.\nBy default, this is 300px.  If you want to have the editor always\nresizes to accommodate the content, use AlwaysShowToolbar plugin\nand set height=&quot;&quot;.  If this editor is used within a layout widget,\nset height=&quot;100%&quot;.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inheritWidth":{"!doc":"<p>whether to inherit the parent&#39;s width or simply use 100%</p>\n"},"isClosed":{},"isLoaded":{},"isTabIndent":{"!doc":"<p>Make tab key and shift-tab indent and outdent rather than navigating.\nCaution: sing this makes web pages inaccessible to users unable to use a mouse.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"minHeight":{"!doc":"<p>The minimum height that the editor should have.</p>\n"},"name":{"!doc":"<p>Specifies the name of a (hidden) <code>&lt;textarea&gt;</code> node on the page that&#39;s used to save\nthe editor content on page leave.   Used to restore editor contents after navigating\nto a new page and then hitting the back button.</p>\n"},"onLoadDeferred":{"!doc":"<p>Deferred which is fired when the editor finishes loading.\nCall myEditor.onLoadDeferred.then(callback) it to be informed\nwhen the rich-text area initialization is finalized.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"plugins":{"!doc":"<p>A list of plugin names (as strings) or instances (as objects)\nfor this widget.\n\n</p>\n<p>When declared in markup, it might look like:\n\n</p>\n<pre><code>plugins=&quot;[&#39;bold&#39;,{name:&#39;dijit._editor.plugins.FontChoice&#39;, command:&#39;fontName&#39;, generic:true}]&quot;</code></pre>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"styleSheets":{"!doc":"<p>semicolon (&quot;;&quot;) separated list of css files for the editing area</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"updateInterval":{},"addKeyHandler":{"!type":"fn(key: ?, ctrl: ?, shift: ?, handler: ?)","!doc":"<p>Add a handler for a keyboard shortcut</p>\n"},"addPlugin":{"!type":"fn(plugin: ?, index?: ?)","!doc":"<p>takes a plugin name as a string or a plugin instance and\nadds it to the toolbar and associates it with this editor\ninstance. The resulting plugin is added to the Editor&#39;s\nplugins array. If index is passed, it&#39;s placed in the plugins\narray at that index. No big magic, but a nice helper for\npassing in plugin names via markup.</p>\n"},"addStyleSheet":{"!type":"fn(uri: ?)","!doc":"<p>add an external stylesheet for the editing area</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"blur":{"!type":"fn()","!doc":"<p>Remove focus from this instance.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"execCommand":{"!type":"fn(cmd: ?) -> ?","!doc":"<p>Main handler for executing any commands to the editor, like paste, bold, etc.\nCalled by plugins, but not meant to be called by end users.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Move focus to this editor</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getFooterHeight":{"!type":"fn() -> ?","!doc":"<p>A function for obtaining the height of the footer node</p>\n"},"getHeaderHeight":{"!type":"fn() -> ?","!doc":"<p>A function for obtaining the height of the header node</p>\n"},"getNodeChildrenHtml":{"!type":"fn(dom: ?) -> ?","!doc":"<p>Deprecated.   Use dijit/_editor/html::getChildrenHtml() instead.</p>\n"},"getNodeHtml":{"!type":"fn(node: ?) -> ?","!doc":"<p>Deprecated.   Use dijit/_editor/html::_getNodeHtml() instead.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()","!doc":"<p>Called from <code>dijit/layout/_LayoutWidget.resize()</code>.  This shouldn&#39;t be called directly</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Extension to make sure a deferred is in place before certain functions\nexecute, like making sure all the plugins are properly inserted.</p>\n"},"queryCommandEnabled":{"!type":"fn(cmd: ?) -> bool","!doc":"<p>Returns true if specified editor command is enabled.\nUsed by the plugins to know when to highlight/not highlight buttons.</p>\n"},"queryCommandState":{"!type":"fn(command: ?) -> bool","!doc":"<p>Check the state of a given command and returns true or false.</p>\n"},"queryCommandValue":{"!type":"fn(command: ?) -> bool","!doc":"<p>Check the value of a given command. This matters most for\ncustom selections and complex values like font value setting.</p>\n"},"removeStyleSheet":{"!type":"fn(uri: ?)","!doc":"<p>remove an external stylesheet for the editing area</p>\n"},"replaceValue":{"!type":"fn(html: ?)","!doc":"<p>over-ride of replaceValue to support custom undo and stack maintenance.</p>\n"},"resize":{"!type":"fn(size: ?)","!doc":"<p>Resize the editor to the specified size, see <code>dijit/layout/_LayoutWidget.resize()</code></p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated, use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setupDefaultShortcuts":{"!type":"fn()","!doc":"<p>Add some default key handlers</p>\n"},"setValue":{"!type":"fn(html: ?)","!doc":"<p>This function sets the content. No undo history is preserved.\nUsers should use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/Toolbar":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A Toolbar widget, used to hold things like <code>dijit/Editor</code> buttons</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"childSelector":{"!type":"fn(node: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the first child.</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"startupKeyNavChildren":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/ToolbarSeparator":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A spacer between two <code>dijit.Toolbar</code> items</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> bool","!doc":"<p>This widget isn&#39;t focusable, so pass along that fact.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/_LayoutWidget":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Base class for a _Container widget which is responsible for laying out its children.\nWidgets which mixin this code must define layout() to manage placement and sizing of the children.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>This class name is applied to the widget&#39;s domNode\nand also may be used to generate names for sub nodes,\nfor example dijitTabContainer-content.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()","!doc":"<p>Widgets override this method to size and position their contents/children.\nWhen this is called this._contentBox is guaranteed to be set (see resize()).\n\n</p>\n<p>This is called after startup(), and also when the widget&#39;s size has been\nchanged.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(child: ?)"},"resize":{"!type":"fn(changeSize?: ?, resultSize?: ?)","!doc":"<p>Call this to resize a widget, or after its size has changed.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Called after all the widgets have been instantiated and their\ndom nodes have been inserted somewhere under <body>.\n\n</p>\n<p>Widgets should override this method to do any initialization\ndependent on other widgets existing, and then call\nthis superclass method to finish things off.\n\n</p>\n<p>startup() in subclasses shouldn&#39;t do anything\nsize related because the size of the widget hasn&#39;t been set yet.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/ToggleButton":{"!type":"fn()","prototype":{"!proto":"dijit/form/Button.prototype","!doc":"<p>A templated button widget that can be in two states (checked or not).\nCan be base class for things like tabs or checkbox or radio buttons.</p>\n","_aria_attr":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_setValueAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"checked":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.\nIn markup, specified as &quot;checked=&#39;checked&#39;&quot; or just &quot;checked&quot;.\nTrue if the button is depressed, or the checkbox is checked,\nor the radio button is selected, etc.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setChecked":{"!type":"fn(checked: ?)","!doc":"<p>Deprecated.  Use set(&#39;checked&#39;, true/false) instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_ToggleButtonMixin":{"!type":"fn()","prototype":{"!doc":"<p>A mixin to provide functionality to allow a button that can be in two states (checked or not).</p>\n","_aria_attr":{},"checked":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.\nIn markup, specified as &quot;checked=&#39;checked&#39;&quot; or just &quot;checked&quot;.\nTrue if the button is depressed, or the checkbox is checked,\nor the radio button is selected, etc.</p>\n"},"postCreate":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"}}},"dijit/_editor/_Plugin":{"!type":"fn()","prototype":{"!proto":"dijit/Destroyable.prototype","!doc":"<p>Base class for a &quot;plugin&quot; to the editor, which is usually\na single button on the Toolbar and some associated code</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}},"registry":{}},"dijit/_editor/_Plugin.registry":{"bold":{"!type":"fn(args: ?) -> +instance"},"colorTableCell":{"!type":"fn(args: ?)"},"copy":{"!type":"fn(args: ?) -> +instance"},"createLink":{"!type":"fn()"},"cut":{"!type":"fn(args: ?) -> +instance"},"delete":{"!type":"fn(args: ?) -> +instance"},"deleteTableColumn":{"!type":"fn(args: ?) -> +instance"},"deleteTableRow":{"!type":"fn(args: ?) -> +instance"},"foreColor":{"!type":"fn()"},"fullScreen":{"!type":"fn(args: ?)"},"fullscreen":{"!type":"fn(args: ?)"},"hiliteColor":{"!type":"fn()"},"indent":{"!type":"fn(args: ?) -> +instance"},"insertHorizontalRule":{"!type":"fn(args: ?) -> +instance"},"insertImage":{"!type":"fn()"},"insertOrderedList":{"!type":"fn(args: ?) -> +instance"},"insertTable":{"!type":"fn(args: ?)"},"insertTableColumnAfter":{"!type":"fn(args: ?) -> +instance"},"insertTableColumnBefore":{"!type":"fn(args: ?) -> +instance"},"insertTableRowAfter":{"!type":"fn(args: ?) -> +instance"},"insertTableRowBefore":{"!type":"fn(args: ?) -> +instance"},"insertUnorderedList":{"!type":"fn(args: ?) -> +instance"},"italic":{"!type":"fn(args: ?) -> +instance"},"justifyCenter":{"!type":"fn(args: ?) -> +instance"},"justifyFull":{"!type":"fn(args: ?) -> +instance"},"justifyLeft":{"!type":"fn(args: ?) -> +instance"},"justifyRight":{"!type":"fn(args: ?) -> +instance"},"LocalImage":{"!type":"fn(args: ?) -> +instance"},"localImage":{"!type":"fn(args: ?) -> +instance"},"localimage":{"!type":"fn(args: ?) -> +instance"},"modifyTable":{"!type":"fn(args: ?)"},"newPage":{"!type":"fn(args: ?)"},"newpage":{"!type":"fn(args: ?)"},"outdent":{"!type":"fn(args: ?) -> +instance"},"paste":{"!type":"fn(args: ?) -> +instance"},"print":{"!type":"fn()"},"redo":{"!type":"fn(args: ?) -> +instance"},"removeFormat":{"!type":"fn(args: ?) -> +instance"},"selectAll":{"!type":"fn(args: ?) -> +instance"},"strikethrough":{"!type":"fn(args: ?) -> +instance"},"subscript":{"!type":"fn(args: ?) -> +instance"},"superscript":{"!type":"fn(args: ?) -> +instance"},"tabIndent":{"!type":"fn()"},"tableContextMenu":{"!type":"fn(args: ?)"},"toggleDir":{"!type":"fn()"},"underline":{"!type":"fn(args: ?) -> +instance"},"undo":{"!type":"fn(args: ?) -> +instance"},"unlink":{"!type":"fn(args: ?) -> +instance"},"viewSource":{"!type":"fn(args: ?)"},"viewsource":{"!type":"fn(args: ?)"},"|":{"!type":"fn() -> +instance"}},"dijit/_editor/plugins/EnterKeyHandling":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin tries to make all browsers behave consistently with regard to\nhow ENTER behaves in the editor window.  It traps the ENTER key and alters\nthe way DOM is constructed in certain cases to try to commonize the generated\nDOM and behaviors across browsers.\n</p>\n","blockNodeForEnter":{"!doc":"<p>This property decides the behavior of Enter key. It can be either P,\nDIV, BR, or empty (which means disable this feature). Anything else\nwill trigger errors.  The default is &#39;BR&#39;\n\n</p>\n<p>See class description for more details.</p>\n"},"blockNodes":{"!doc":"<p>Regex for testing if a given tag is a block level (display:block) tag</p>\n"},"bogusHtmlContent":{"!doc":"<p>HTML to stick into a new empty block</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dijit/_editor/RichText":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>dijit/_editor/RichText is the core of dijit.Editor, which provides basic\nWYSIWYG editing features.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_editorCommandsLocalized":{},"_focusManager":{},"_local2NativeFormatNames":{},"_mozSettingProps":{},"_NAME_CONTENT_SEP":{"!doc":"<p>USed to separate name from content.  Just a colon isn&#39;t safe.</p>\n"},"_native2LocalFormatNames":{},"_qcaCache":{},"_SEPARATOR":{"!doc":"<p>Used to concat contents from multiple editors into a single string,\nso they can be saved into a single <code>&lt;textarea&gt;</code> node.  See &quot;name&quot; attribute.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_updateTimer":{},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"captureEvents":{"!doc":"<p>Events which should be connected to the underlying editing\narea, events in this array will be addListener with\ncapture=true.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>The editor is disabled; the text cannot be changed.</p>\n"},"disableSpellCheck":{"!doc":"<p>When true, disables the browser&#39;s native spell checking, if supported.\nWorks only in Firefox.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"events":{"!doc":"<p>events which should be connected to the underlying editing area</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusOnLoad":{"!doc":"<p>Focus into this widget when the page is loaded</p>\n"},"height":{"!doc":"<p>Set height to fix the editor at a specific height, with scrolling.\nBy default, this is 300px.  If you want to have the editor always\nresizes to accommodate the content, use AlwaysShowToolbar plugin\nand set height=&quot;&quot;.  If this editor is used within a layout widget,\nset height=&quot;100%&quot;.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inheritWidth":{"!doc":"<p>whether to inherit the parent&#39;s width or simply use 100%</p>\n"},"isClosed":{},"isLoaded":{},"isTabIndent":{"!doc":"<p>Make tab key and shift-tab indent and outdent rather than navigating.\nCaution: sing this makes web pages inaccessible to users unable to use a mouse.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"minHeight":{"!doc":"<p>The minimum height that the editor should have.</p>\n"},"name":{"!doc":"<p>Specifies the name of a (hidden) <code>&lt;textarea&gt;</code> node on the page that&#39;s used to save\nthe editor content on page leave.   Used to restore editor contents after navigating\nto a new page and then hitting the back button.</p>\n"},"onLoadDeferred":{"!doc":"<p>Deferred which is fired when the editor finishes loading.\nCall myEditor.onLoadDeferred.then(callback) it to be informed\nwhen the rich-text area initialization is finalized.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"styleSheets":{"!doc":"<p>semicolon (&quot;;&quot;) separated list of css files for the editing area</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"updateInterval":{},"addKeyHandler":{"!type":"fn(key: ?, ctrl: ?, shift: ?, handler: ?)","!doc":"<p>Add a handler for a keyboard shortcut</p>\n"},"addStyleSheet":{"!type":"fn(uri: ?)","!doc":"<p>add an external stylesheet for the editing area</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"blur":{"!type":"fn()","!doc":"<p>Remove focus from this instance.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"execCommand":{"!type":"fn(command: ?, argument: ?) -> ?","!doc":"<p>Executes a command in the Rich Text area</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Move focus to this editor</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getFooterHeight":{"!type":"fn() -> ?","!doc":"<p>A function for obtaining the height of the footer node</p>\n"},"getHeaderHeight":{"!type":"fn() -> ?","!doc":"<p>A function for obtaining the height of the header node</p>\n"},"getNodeChildrenHtml":{"!type":"fn(dom: ?) -> ?","!doc":"<p>Deprecated.   Use dijit/_editor/html::getChildrenHtml() instead.</p>\n"},"getNodeHtml":{"!type":"fn(node: ?) -> ?","!doc":"<p>Deprecated.   Use dijit/_editor/html::_getNodeHtml() instead.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"queryCommandEnabled":{"!type":"fn(command: ?) -> bool","!doc":"<p>Check whether a command is enabled or not.</p>\n"},"queryCommandState":{"!type":"fn(command: ?) -> bool","!doc":"<p>Check the state of a given command and returns true or false.</p>\n"},"queryCommandValue":{"!type":"fn(command: ?) -> bool","!doc":"<p>Check the value of a given command. This matters most for\ncustom selections and complex values like font value setting.</p>\n"},"removeStyleSheet":{"!type":"fn(uri: ?)","!doc":"<p>remove an external stylesheet for the editing area</p>\n"},"replaceValue":{"!type":"fn(html: ?)","!doc":"<p>This function set the content while trying to maintain the undo stack\n(now only works fine with Moz, this is identical to setValue in all\nother browsers)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated, use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setupDefaultShortcuts":{"!type":"fn()","!doc":"<p>Add some default key handlers</p>\n"},"setValue":{"!type":"fn(html: ?)","!doc":"<p>This function sets the content. No undo history is preserved.\nUsers should use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/selection":{"!doc":"<p>Class for monitoring / changing the selection (typically highlighted text) in a given window</p>\n","collapse":{"!type":"fn(beginning: ?)","!doc":"<p>Function to collapse (clear), the current selection</p>\n"},"prototype":{"constructor":{"!type":"fn(win: ?)","!doc":"<p>Class for monitoring / changing the selection (typically highlighted text) in a given window</p>\n"}},"getAncestorElement":{"!type":"fn(tagName: ?)","!doc":"<p>Return the parent element of the current selection which is of\ntype tagName (or one of the other specified tagName)</p>\n"},"getBookmark":{"!type":"fn()","!doc":"<p>Retrieves a bookmark that can be used with moveToBookmark to reselect the currently selected range.</p>\n"},"getParentElement":{"!type":"fn()","!doc":"<p>Get the parent element of the current selection</p>\n"},"getParentOfType":{"!type":"fn(node: ?, tags: ?)","!doc":"<p>Function to locate a parent node that matches one of a set of tags</p>\n"},"getSelectedElement":{"!type":"fn()","!doc":"<p>Retrieves the selected element (if any), just in the case that\na single element (object like and image or a table) is\nselected.</p>\n"},"getSelectedHtml":{"!type":"fn()","!doc":"<p>Return the html text of the current selection or null if unavailable</p>\n"},"getSelectedText":{"!type":"fn()","!doc":"<p>Return the text (no html tags) included in the current selection or null if no text is selected</p>\n"},"getType":{"!type":"fn()","!doc":"<p>Get the selection type (like doc.select.type in IE).</p>\n"},"hasAncestorElement":{"!type":"fn(tagName: ?)","!doc":"<p>Check whether current selection has a  parent element which is\nof type tagName (or one of the other specified tagName)</p>\n"},"inSelection":{"!type":"fn(node: ?)","!doc":"<p>This function determines if &#39;node&#39; is\nin the current selection.</p>\n"},"isCollapsed":{"!type":"fn()","!doc":"<p>Returns true if there is no text selected</p>\n"},"isTag":{"!type":"fn(node: ?, tags: ?)","!doc":"<p>Function to determine if a node is one of an array of tags.</p>\n"},"moveToBookmark":{"!type":"fn(bookmark: ?)","!doc":"<p>Moves current selection to a bookmark.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Function to delete the currently selected content from the document.</p>\n"},"selectElement":{"!type":"fn(element: ?, nochangefocus?: ?)","!doc":"<p>clear previous selection and select element (including all its children)</p>\n"},"selectElementChildren":{"!type":"fn(element: ?, nochangefocus?: ?)","!doc":"<p>clear previous selection and select the content of the node\n(excluding the node itself)</p>\n"}},"dijit/_editor/range":{"!doc":"<p>W3C range API</p>\n","BlockTagNames":{},"ie":{},"adjacentNoneTextNode":{"!type":"fn(startnode: ?, next: ?) -> []"},"atBeginningOfContainer":{"!type":"fn(container: ?, node: ?, offset: ?) -> bool"},"atEndOfContainer":{"!type":"fn(container: ?, node: ?, offset: ?) -> bool"},"create":{"!type":"fn(win?: ?) -> ?"},"getAncestor":{"!type":"fn(node: ?, regex?: ?, root?: ?) -> +Node"},"getBlockAncestor":{"!type":"fn(node: ?, regex?: ?, root?: ?) -> ?"},"getCommonAncestor":{"!type":"fn(n1: ?, n2: ?, root: ?) -> ?"},"getIndex":{"!type":"fn(node: ?, parent: ?) -> ?"},"getNode":{"!type":"fn(index: ?, parent: ?) -> +Node"},"getSelection":{"!type":"fn(window: ?, ignoreUpdate?: ?) -> ?"},"W3CRange":{"!type":"fn()"}},"dijit/_editor/range.ie":{"cachedSelection":{},"decomposeControlRange":{"!type":"fn(range: ?) -> []"},"decomposeTextRange":{"!type":"fn(range: ?) -> []"},"getEndPoint":{"!type":"fn(range: ?, end: ?) -> []"},"selection":{"!type":"fn(window: ?)"},"setEndPoint":{"!type":"fn(range: ?, container: ?, offset: ?) -> ?"},"setRange":{"!type":"fn(range: ?, startContainer: ?, startOffset: ?, endContainer: ?, endOffset: ?, collapsed: ?) -> ?"}},"dijit/_editor/range.W3CRange":{"!type":"fn()","prototype":{"cloneRange":{"!type":"fn() -> +instance"},"constructor":{"!type":"fn()"},"deleteContents":{"!type":"fn()"},"detach":{"!type":"fn()"},"getBookmark":{"!type":"fn() -> ?"},"setEnd":{"!type":"fn(node: ?, offset: ?)"},"setEndAfter":{"!type":"fn(node: ?, offset: ?)"},"setEndBefore":{"!type":"fn(node: ?, offset: ?)"},"setStart":{"!type":"fn(node: ?, offset: ?)"},"setStartAfter":{"!type":"fn(node: ?, offset: ?)"},"setStartBefore":{"!type":"fn(node: ?, offset: ?)"}}},"dijit/_editor/html":{"!doc":"<p>HTML serialization utility functions used by editor</p>\n","escapeXml":{"!type":"fn(str: ?, noSingleQuotes?: ?) -> ?","!doc":"<p>Adds escape sequences for special characters in XML: <code>&amp;&lt;&gt;&quot;&#39;</code>.\nOptionally skips escapes for single quotes.</p>\n"},"getChildrenHtml":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the html content of a DomNode&#39;s children</p>\n"},"getChildrenHtmlHelper":{"!type":"fn(dom: ?, output: ?)","!doc":"<p>Pushes the html content of a DomNode&#39;s children into out[]</p>\n"},"getNodeHtml":{"!type":"fn(node: ?) -> ?","!doc":"<p>Return string representing HTML for node and it&#39;s children</p>\n"},"getNodeHtmlHelper":{"!type":"fn(node: ?, output: ?) -> string","!doc":"<p>Pushes array of strings into output[] which represent HTML for node and it&#39;s children</p>\n"}},"dijit/_base/focus":{"!doc":"<p>Deprecated module to monitor currently focused node and stack of currently focused widgets.\nNew code should access dijit/focus directly.</p>\n","_activeStack":{"!doc":"<p>List of currently active widgets (focused widget and it&#39;s ancestors)</p>\n"},"_curFocus":{"!doc":"<p>Currently focused item on screen</p>\n"},"_prevFocus":{"!doc":"<p>Previously focused item on screen</p>\n"},"getBookmark":{"!type":"fn() -> ?","!doc":"<p>Retrieves a bookmark that can be used with moveToBookmark to return to the same range</p>\n"},"getFocus":{"!type":"fn(menu?: ?, openedForWindow?: ?) -> +any","!doc":"<p>Called as getFocus(), this returns an Object showing the current focus\nand selected text.\n\n</p>\n<p>Called as getFocus(widget), where widget is a (widget representing) a button\nthat was just pressed, it returns where focus was before that button\nwas pressed.   (Pressing the button may have either shifted focus to the button,\nor removed focus altogether.)   In this case the selected text is not returned,\nsince it can&#39;t be accurately determined.\n</p>\n"},"isCollapsed":{"!type":"fn() -> ?","!doc":"<p>Returns true if there is no text selected</p>\n"},"moveToBookmark":{"!type":"fn(bookmark: ?) -> ?","!doc":"<p>Moves current selection to a bookmark</p>\n"},"registerIframe":{"!type":"fn(iframe: ?) -> +any","!doc":"<p>Registers listeners on the specified iframe so that any click\nor focus event on that iframe (or anything in it) is reported\nas a focus/click event on the <code>&lt;iframe&gt;</code> itself.</p>\n"},"registerWin":{"!type":"fn(targetWindow?: ?, effectiveNode?: ?) -> +any","!doc":"<p>Registers listeners on the specified window (either the main\nwindow or an iframe&#39;s window) to detect when the user has clicked somewhere\nor focused somewhere.</p>\n"},"unregisterIframe":{"!type":"fn(handle: ?)","!doc":"<p>Unregisters listeners on the specified iframe created by registerIframe.\nAfter calling be sure to delete or null out the handle itself.</p>\n"},"unregisterWin":{"!type":"fn(handle: ?)","!doc":"<p>Unregisters listeners on the specified window (either the main\nwindow or an iframe&#39;s window) according to handle returned from registerWin().\nAfter calling be sure to delete or null out the handle itself.</p>\n"}},"dijit/Fieldset":{"!type":"fn()","prototype":{"!proto":"dijit/TitlePane.prototype","!doc":"<p>An accessible fieldset that can be expanded or collapsed via\nits legend.  Fieldset extends <code>dijit.TitlePane</code>.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_setTooltipAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The root className to use for the various states of this widget</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>Don&#39;t change this parameter from the default value.\nThis ContentPane parameter doesn&#39;t make sense for TitlePane, since TitlePane\nis never a child of a layout container, nor should TitlePane try to control\nthe size of an inner widget.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>Time in milliseconds to fade in/fade out</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"open":{"!doc":"<p>Whether fieldset is opened or closed.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the title (so users can tab to the title then\nuse space/enter to open/close the title pane)</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>Content of the legend tag. Overrides <legend> tag if not empty.</p>\n"},"toggleable":{"!doc":"<p>Whether pane can be opened or closed by clicking the title bar.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"setTitle":{"!type":"fn(title: ?)","!doc":"<p>Deprecated.  Use set(&#39;title&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/TitlePane":{"!type":"fn()","prototype":{"!proto":"dijit/layout/ContentPane.prototype","!doc":"<p>A pane with a title on top, that can be expanded or collapsed.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_setTooltipAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The root className to be placed on this widget&#39;s domNode.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>Don&#39;t change this parameter from the default value.\nThis ContentPane parameter doesn&#39;t make sense for TitlePane, since TitlePane\nis never a child of a layout container, nor should TitlePane try to control\nthe size of an inner widget.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>Time in milliseconds to fade in/fade out</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"open":{"!doc":"<p>Whether pane is opened or closed.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the title (so users can tab to the title then\nuse space/enter to open/close the title pane)</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>Title of the pane</p>\n"},"toggleable":{"!doc":"<p>Whether pane can be opened or closed by clicking the title bar.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"setTitle":{"!type":"fn(title: ?)","!doc":"<p>Deprecated.  Use set(&#39;title&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/InlineEditBox":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoSave":{"!doc":"<p>Changing the value automatically saves it; don&#39;t have to push save button\n(and save button isn&#39;t even displayed)</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"buttonCancel":{"!doc":"<p>Cancel button label</p>\n"},"buttonSave":{"!doc":"<p>Save button label</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>If true, clicking the InlineEditBox to edit it will have no effect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editing":{"!doc":"<p>Is the node currently in edit mode?</p>\n"},"editorParams":{"!doc":"<p>Set of parameters for editor, like {required: true}</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"noValueIndicator":{"!doc":"<p>The text that gets displayed when there is no value (so that the user has a place to click to edit)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"renderAsHtml":{"!doc":"<p>Set this to true if the specified Editor&#39;s value should be interpreted as HTML\nrather than plain text (ex: <code>dijit.Editor</code>)</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The display value of the widget in read-only mode</p>\n"},"width":{"!doc":"<p>Width of editor.  By default it&#39;s width=100% (ie, block mode).</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"editor":{"!type":"fn()","!doc":"<p>MID (ex: &quot;dijit/form/TextBox&quot;) or constructor for editor widget</p>\n"},"editorWrapper":{"!type":"fn()","!doc":"<p>Class name (or reference to the Class) for widget that wraps the editor widget, displaying save/cancel\nbuttons.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.   Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.   Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(val: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/InlineEditBox._InlineEditor":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Internal widget used by InlineEditBox, displayed when in editing mode\nto display the editor and maybe save/cancel buttons.  Calling code should\nconnect to save/cancel methods to detect when editing is finished\n\n</p>\n<p>Has mainly the same parameters as InlineEditBox, plus these values:\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>Set of CSS attributes of display node, to replicate in editor\n</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>Value as an HTML string or plain text string, depending on renderAsHTML flag</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"contextRequire":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"enableSave":{"!type":"fn() -> bool","!doc":"<p>User overridable function returning a Boolean to indicate\nif the Save button should be enabled or not - usually due to invalid conditions</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus the edit widget.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Return the [display] value of the edit widget</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_TextBoxMixin":{"!type":"fn()","prototype":{"__skipInputEvent":{},"_blankValue":{},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"postCreate":{"!type":"fn()"},"reset":{"!type":"fn()"}},"selectInputText":{"!type":"fn(element: ?, start?: ?, stop?: ?)","!doc":"<p>Select text in the input element argument, from start (default 0), to stop (default end).</p>\n"}},"dijit/form/TextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormValueWidget.prototype","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_FormValueWidget":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormWidget.prototype","!doc":"<p>Base class for widgets corresponding to native HTML elements such as <code>&lt;input&gt;</code> or <code>&lt;select&gt;</code>\nthat have user changeable values.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_FormValueMixin":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormWidgetMixin.prototype","!doc":"<p>Mixin for widgets corresponding to native HTML elements such as <code>&lt;input&gt;</code> or <code>&lt;select&gt;</code>\nthat have user changeable values.</p>\n","_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setIdAttr":{},"_setTabIndexAttr":{},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"create":{"!type":"fn()"},"destroy":{"!type":"fn()"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Tells if this widget is focusable or not.  Used internally by dijit.</p>\n"},"postCreate":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"}}},"dijit/Menu":{"!type":"fn()","prototype":{"!proto":"dijit/DropDownMenu.prototype","!doc":"<p>A context menu you can assign to multiple elements</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"activated":{"!doc":"<p>This Menu has been clicked (mouse or via space/arrow key) or opened as a submenu,\nso mere mouseover will open submenus.  Focusing a menu via TAB does NOT automatically make it active\nsince TAB is a navigation operation and not a selection one.\nFor Windows apps, pressing the ALT key focuses the menubar menus (similar to TAB navigation) but the\nmenu is not active (ie no dropdown) until an item is clicked.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoFocus":{"!doc":"<p>A toggle to control whether or not a Menu gets focused when opened as a drop down from a MenuBar\nor DropDownButton/ComboButton.   Note though that it always get focused when opened via the keyboard.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextMenuForWindow":{"!doc":"<p>If true, right clicking anywhere on the window will cause this context menu to open.\nIf false, must specify targetNodeIds.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"currentTarget":{"!doc":"<p>For context menus, set to the current node that the Menu is being displayed for.\nUseful so that the menu actions can be tailored according to the node</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"leftClickToOpen":{"!doc":"<p>If true, menu will open on left click instead of right click, similar to a file menu.</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parentMenu":{"!doc":"<p>pointer to menu that displayed me</p>\n"},"passivePopupDelay":{"!doc":"<p>For a passive (unclicked) Menu, number of milliseconds before hovering (without clicking) will cause\nthe popup to open.  Default is Infinity, meaning you need to click the menu to open it.</p>\n"},"popupDelay":{"!doc":"<p>After a menu has been activated (by clicking on it etc.), number of milliseconds before hovering\n(without clicking) another MenuItem causes that MenuItem&#39;s popup to automatically open.</p>\n"},"refocus":{"!doc":"<p>When this menu closes, re-focus the element which had focus before it was opened.</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>Currently selected (a.k.a. highlighted) MenuItem, or null if no MenuItem is selected.\nIf a submenu is open, will be set to MenuItem that displayed the submenu.   OTOH, if\nthis Menu is in passive mode (i.e. hasn&#39;t been clicked yet), will be null, because\n&quot;selected&quot; is not merely &quot;hovered&quot;.</p>\n"},"selector":{"!doc":"<p>CSS expression to apply this Menu to descendants of targetNodeIds, rather than to\nthe nodes specified by targetNodeIds themselves.    Useful for applying a Menu to\na range of rows in a table, tree, etc.\n\n</p>\n<p>The application must require() an appropriate level of dojo/query to handle the selector.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"targetNodeIds":{"!doc":"<p>Array of dom node ids of nodes to attach to.\nFill this with nodeIds upon widget creation and it becomes context menu for those nodes.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"bindDomNode":{"!type":"fn(node: ?)","!doc":"<p>Attach menu to given node</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"childSelector":{"!type":"fn(node: ?) -> ?","!doc":"<p>Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children\nlike MenuSeparator.  If subclass overrides to a string (ex: &quot;&gt; *&quot;), the subclass must require dojo/query.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the first child.</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPlaceholders":{"!type":"fn(label?: ?) -> +any","!doc":"<p>Returns an array of placeholders with the given label.  There\ncan be multiples.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"startupKeyNavChildren":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"unBindDomNode":{"!type":"fn(nodeName: ?)","!doc":"<p>Detach menu from given node</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/MenuBar":{"!type":"fn()","prototype":{"!proto":"dijit/_MenuBase.prototype","!doc":"<p>A menu bar, listing menu choices horizontally, like the &quot;File&quot; menu in most desktop applications</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_isMenuBar":{"!doc":"<p>This is a MenuBar widget, not a (vertical) Menu widget.</p>\n"},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_orient":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"activated":{"!doc":"<p>This Menu has been clicked (mouse or via space/arrow key) or opened as a submenu,\nso mere mouseover will open submenus.  Focusing a menu via TAB does NOT automatically make it active\nsince TAB is a navigation operation and not a selection one.\nFor Windows apps, pressing the ALT key focuses the menubar menus (similar to TAB navigation) but the\nmenu is not active (ie no dropdown) until an item is clicked.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoFocus":{"!doc":"<p>A toggle to control whether or not a Menu gets focused when opened as a drop down from a MenuBar\nor DropDownButton/ComboButton.   Note though that it always get focused when opened via the keyboard.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parentMenu":{"!doc":"<p>pointer to menu that displayed me</p>\n"},"passivePopupDelay":{"!doc":"<p>For a passive (unclicked) Menu, number of milliseconds before hovering (without clicking) will cause\nthe popup to open.  Default is Infinity, meaning you need to click the menu to open it.</p>\n"},"popupDelay":{},"searchContainerNode":{},"selected":{"!doc":"<p>Currently selected (a.k.a. highlighted) MenuItem, or null if no MenuItem is selected.\nIf a submenu is open, will be set to MenuItem that displayed the submenu.   OTOH, if\nthis Menu is in passive mode (i.e. hasn&#39;t been clicked yet), will be null, because\n&quot;selected&quot; is not merely &quot;hovered&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"childSelector":{"!type":"fn(node: ?) -> ?","!doc":"<p>Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children\nlike MenuSeparator.  If subclass overrides to a string (ex: &quot;&gt; *&quot;), the subclass must require dojo/query.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the first child.</p>\n"},"focusChild":{"!type":"fn(item: ?)"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"startupKeyNavChildren":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/MenuBarItem":{"!type":"fn()","prototype":{"!proto":"dijit/MenuItem.prototype","!doc":"<p>Item in a MenuBar that&#39;s clickable, and doesn&#39;t spawn a submenu when pressed (or hovered)</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"accelKey":{"!doc":"<p>Text for the accelerator (shortcut) key combination, a control, alt, etc. modified keystroke meant to\nexecute the menu item regardless of where the focus is on the page.\n\n</p>\n<p>Note that although Menu can display accelerator keys, there is no infrastructure to actually catch and\nexecute those accelerators.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>If true, the menu item is disabled.\nIf false, the menu item is enabled.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode to make it display an icon.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>Menu text as HTML</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"shortcutKey":{"!doc":"<p>Single character (underlined when the parent Menu is focused) used to navigate directly to this widget,\nalso known as <a href=\"http://en.wikipedia.org/wiki/Mnemonics_(keyboard%29\">a mnemonic</a>.\nThis is denoted in the label by surrounding the single character with {}.\nFor example, if label=&quot;{F}ile&quot;, then shortcutKey=&quot;F&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus on this MenuItem</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.   Use set(&#39;disabled&#39;, bool) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.   Use set(&#39;label&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/MenuBarItem._MenuBarItemMixin":{"!type":"fn()","prototype":{"_setIconClassAttr":{},"templateString":{}}},"dijit/MenuSeparator":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A line between two menu items</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> bool","!doc":"<p>Override to always return false</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/PopupMenuBarItem":{"!type":"fn()","prototype":{"!proto":"dijit/PopupMenuItem.prototype","!doc":"<p>Item in a MenuBar like &quot;File&quot; or &quot;Edit&quot;, that spawns a submenu when pressed (or hovered)</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"accelKey":{"!doc":"<p>Text for the accelerator (shortcut) key combination, a control, alt, etc. modified keystroke meant to\nexecute the menu item regardless of where the focus is on the page.\n\n</p>\n<p>Note that although Menu can display accelerator keys, there is no infrastructure to actually catch and\nexecute those accelerators.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>If true, the menu item is disabled.\nIf false, the menu item is enabled.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode to make it display an icon.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>Menu text as HTML</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"shortcutKey":{"!doc":"<p>Single character (underlined when the parent Menu is focused) used to navigate directly to this widget,\nalso known as <a href=\"http://en.wikipedia.org/wiki/Mnemonics_(keyboard%29\">a mnemonic</a>.\nThis is denoted in the label by surrounding the single character with {}.\nFor example, if label=&quot;{F}ile&quot;, then shortcutKey=&quot;F&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus on this MenuItem</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.   Use set(&#39;disabled&#39;, bool) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.   Use set(&#39;label&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/PopupMenuItem":{"!type":"fn()","prototype":{"!proto":"dijit/MenuItem.prototype","!doc":"<p>An item in a Menu that spawn a drop down (usually a drop down menu)</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"accelKey":{"!doc":"<p>Text for the accelerator (shortcut) key combination, a control, alt, etc. modified keystroke meant to\nexecute the menu item regardless of where the focus is on the page.\n\n</p>\n<p>Note that although Menu can display accelerator keys, there is no infrastructure to actually catch and\nexecute those accelerators.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>If true, the menu item is disabled.\nIf false, the menu item is enabled.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode to make it display an icon.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>Menu text as HTML</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"shortcutKey":{"!doc":"<p>Single character (underlined when the parent Menu is focused) used to navigate directly to this widget,\nalso known as <a href=\"http://en.wikipedia.org/wiki/Mnemonics_(keyboard%29\">a mnemonic</a>.\nThis is denoted in the label by surrounding the single character with {}.\nFor example, if label=&quot;{F}ile&quot;, then shortcutKey=&quot;F&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus on this MenuItem</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.   Use set(&#39;disabled&#39;, bool) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.   Use set(&#39;label&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/ProgressBar":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A progress indication widget, showing the amount completed\n(often the percentage completed) of a task.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_indeterminateHighContrastImagePath":{"!doc":"<p>URL to image to use for indeterminate progress bar when display is in high contrast mode</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indeterminate":{"!doc":"<p>If false: show progress value (number or percentage).\nIf true: show that a process is underway but that the amount completed is unknown.\nDeprecated.   Use &quot;value&quot; instead.</p>\n"},"label":{"!doc":"<p>HTML label on progress bar.   Defaults to percentage for determinate progress bar and\nblank for indeterminate progress bar.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maximum":{"!doc":"<p>Max sample number</p>\n"},"name":{"!doc":"<p>this is the field name (for a form) if set. This needs to be set if you want to use\nthis widget in a dijit/form/Form widget (such as dijit/Dialog)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"places":{"!doc":"<p>Number of places to show in values; 0 by default</p>\n"},"progress":{"!doc":"<p>Number or percentage indicating amount of task completed.\nDeprecated.   Use &quot;value&quot; instead.</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>Number or percentage indicating amount of task completed.\nWith &quot;%&quot;: percentage value, 0% &lt;= progress &lt;= 100%, or\nwithout &quot;%&quot;: absolute value, 0 &lt;= progress &lt;= maximum.\nInfinity means that the progress bar is indeterminate.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"report":{"!type":"fn(percent: ?) -> ?","!doc":"<p>Generates HTML message to show inside progress bar (normally indicating amount of task completed).\nMay be overridden.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/RadioMenuItem":{"!type":"fn()","prototype":{"!proto":"dijit/CheckedMenuItem.prototype","!doc":"<p>A radio-button-like menu item for toggling on and off</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_currentlyChecked":{},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"accelKey":{"!doc":"<p>Text for the accelerator (shortcut) key combination, a control, alt, etc. modified keystroke meant to\nexecute the menu item regardless of where the focus is on the page.\n\n</p>\n<p>Note that although Menu can display accelerator keys, there is no infrastructure to actually catch and\nexecute those accelerators.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"checked":{"!doc":"<p>Our checked state</p>\n"},"checkedChar":{"!doc":"<p>Character (or string) used in place of radio button icon when display in high contrast mode</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>If true, the menu item is disabled.\nIf false, the menu item is enabled.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"group":{"!doc":"<p>Toggling on a RadioMenuItem in a given group toggles off the other RadioMenuItems in that group.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>Menu text as HTML</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"role":{},"searchContainerNode":{},"shortcutKey":{"!doc":"<p>Single character (underlined when the parent Menu is focused) used to navigate directly to this widget,\nalso known as <a href=\"http://en.wikipedia.org/wiki/Mnemonics_(keyboard%29\">a mnemonic</a>.\nThis is denoted in the label by surrounding the single character with {}.\nFor example, if label=&quot;{F}ile&quot;, then shortcutKey=&quot;F&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus on this MenuItem</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.   Use set(&#39;disabled&#39;, bool) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.   Use set(&#39;label&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/Tooltip":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Pops up a tooltip (a help message) when you hover over a node.\nAlso provides static show() and hide() methods that can be used without instantiating a dijit/Tooltip.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"connectId":{"!doc":"<p>Id of domNode(s) to attach the tooltip to.\nWhen user hovers over specified dom node(s), the tooltip will appear.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>HTML to display in the tooltip.\nSpecified as innerHTML when creating the widget from markup.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"position":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on position parameter.</p>\n"},"selector":{"!doc":"<p>CSS expression to apply this Tooltip to descendants of connectIds, rather than to\nthe nodes specified by connectIds themselves.    Useful for applying a Tooltip to\na range of rows in a table, tree, etc.   Use in conjunction with getContent() parameter.\nEx: connectId: myTable, selector: &quot;tr&quot;, getContent: function(node){ return ...; }\n\n</p>\n<p>The application must require() an appropriate level of dojo/query to handle the selector.</p>\n"},"showDelay":{"!doc":"<p>Number of milliseconds to wait after hovering over/focusing on the object, before\nthe tooltip is displayed.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addTarget":{"!type":"fn(node: ?)","!doc":"<p>Attach tooltip to specified node if it&#39;s not already connected</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getContent":{"!type":"fn(node: ?) -> ?","!doc":"<p>User overridable function that return the text to display in the tooltip.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeTarget":{"!type":"fn(node: ?)","!doc":"<p>Detach tooltip from specified node</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"_masterTT":{},"defaultPosition":{"!doc":"<p>This variable controls the position of tooltips, if the position is not specified to\nthe Tooltip widget or *TextBox widget itself.  It&#39;s an array of strings with the values\npossible for <code>dijit/place.around()</code>.   The recommended values are:\n\n</p>\n<ul>\n<li>before-centered: centers tooltip to the left of the anchor node/widget, or to the right\nin the case of RTL scripts like Hebrew and Arabic</li>\n<li>after-centered: centers tooltip to the right of the anchor node/widget, or to the left\nin the case of RTL scripts like Hebrew and Arabic</li>\n<li>above-centered: tooltip is centered above anchor node</li>\n<li>below-centered: tooltip is centered above anchor node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the tooltip fits\nwithin the viewport.\n\n</p>\n<p>Be careful setting this parameter.  A value of &quot;above-centered&quot; may work fine until the user scrolls\nthe screen so that there&#39;s no room above the target node.   Nodes with drop downs, like\nDropDownButton or FilteringSelect, are especially problematic, in that you need to be sure\nthat the drop down and tooltip don&#39;t overlap, even when the viewport is scrolled so that there\nis only room below (or above) the target node, but not both.</p>\n"},"hide":{"!type":"fn(aroundNode: ?) -> ?","!doc":"<p>Static method to hide the tooltip displayed via showTooltip()</p>\n"},"show":{"!type":"fn(innerHTML: ?, aroundNode: ?, position?: ?, rtl?: ?, textDir?: ?) -> ?","!doc":"<p>Static method to display tooltip w/specified contents in specified position.\nSee description of dijit/Tooltip.defaultPosition for details on position parameter.\nIf position is not specified then dijit/Tooltip.defaultPosition is used.</p>\n"}},"dijit/Tooltip._MasterTooltip":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Internal widget that holds the actual tooltip markup,\nwhich occurs once per page.\nCalled by Tooltip widgets which are just containers to hold\nthe markup</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>Milliseconds to fade in/fade out</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn(aroundNode: ?)","!doc":"<p>Hide the tooltip</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"orient":{"!type":"fn(node: ?, aroundCorner: ?, tooltipCorner: ?, spaceAvailable: ?, aroundNodeCoords: ?) -> ?","!doc":"<p>Private function to set CSS for tooltip node based on which position it&#39;s in.\nThis is called by the dijit popup code.   It will also reduce the tooltip&#39;s\nwidth to whatever width is available</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"show":{"!type":"fn(innerHTML: ?, aroundNode: ?, position?: ?, rtl?: ?, textDir?: ?)","!doc":"<p>Display tooltip w/specified contents to right of specified node\n(To left if there&#39;s no space on the right, or if rtl == true)</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/TooltipDialog":{"!type":"fn()","prototype":{"!proto":"dijit/layout/ContentPane.prototype","!doc":"<p>Pops up a dialog that appears like a Tooltip</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_firstFocusItem":{"!doc":"<p>The pointer to the first focusable node in the dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_focusManager":{},"_lastFocusItem":{"!doc":"<p>The pointer to which node has focus prior to our dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autofocus":{"!doc":"<p>A Toggle to modify the default focus behavior of a Dialog, which\nis to focus on the first dialog element after opening the dialog.\nFalse will disable autofocusing.  Default: true.</p>\n"},"baseClass":{"!doc":"<p>The root className to use for the various states of this widget</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>Don&#39;t change this parameter from the default value.\nThis ContentPane parameter doesn&#39;t make sense for TooltipDialog, since TooltipDialog\nis never a child of a layout container, nor can you specify the size of\nTooltipDialog in order to control the size of an inner widget.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Will be &quot;Error&quot; if one or more of the child widgets has an invalid value,\n&quot;Incomplete&quot; if not all of the required child widgets are filled in.  Otherwise, &quot;&quot;,\nwhich indicates that the form is ready to be submitted.</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>Description of tooltip dialog (required for a11y)</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectChildren":{"!type":"fn(inStartup: ?)","!doc":"<p>You can call this function directly, ex. in the event that you\nprogrammatically add a widget to the form <em>after</em> the form has been\ninitialized.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"disconnectChildren":{"!type":"fn()","!doc":"<p>Deprecated method.   Applications no longer need to call this.   Remove for 2.0.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"execute":{"!type":"fn(formContents: ?)","!doc":"<p>Callback when the user hits the submit button.\nOverride this method to handle Dialog execution.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus on first field</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValues":{"!type":"fn() -> ?"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns true if all of the widgets are valid.\nDeprecated, will be removed in 2.0.  Use get(&quot;state&quot;) instead.</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"orient":{"!type":"fn(node: ?, aroundCorner: ?, tooltipCorner: ?)","!doc":"<p>Configure widget to be displayed in given position relative to the button.\nThis is called from the dijit.popup code, and should not be called\ndirectly.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"setValues":{"!type":"fn(val: ?) -> ?"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?","!doc":"<p>returns if the form is valid - same as isValid - but\nprovides a few additional (ui-specific) features:\n\n</p>\n<ol>\n<li>it will highlight any sub-widgets that are not valid</li>\n<li>it will call focus() on the first invalid sub-widget</li>\n</ol>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/Tree":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>This widget displays hierarchical data from a store.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_nodePixelIndent":{"!doc":"<p>Number of pixels to indent tree nodes (relative to parent node).\nDefault is 19 but can be overridden by setting CSS class dijitTreeIndent\nand calling resize() or startup() on tree after it&#39;s in the DOM.</p>\n"},"_outstandingPaintOperations":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoExpand":{"!doc":"<p>Fully expand the tree on load.   Overrides <code>persist</code>.</p>\n"},"baseClass":{},"betweenThreshold":{"!doc":"<p>Set to a positive value to allow drag and drop &quot;between&quot; nodes.\n\n</p>\n<p>If during DnD mouse is over a (target) node but less than betweenThreshold\npixels from the bottom edge, dropping the the dragged node will make it\nthe next sibling of the target node, rather than the child.\n\n</p>\n<p>Similarly, if mouse is over a target node but less that betweenThreshold\npixels from the top edge, dropping the dragged node will make it\nthe target node&#39;s previous sibling rather than the target node&#39;s child.</p>\n"},"checkAcceptance":{},"checkItemAcceptance":{},"childrenAttr":{"!doc":"<p>Deprecated.   This information should be specified in the model.\nOne ore more attributes that holds children of a tree node</p>\n"},"childSelector":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dndParams":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dragThreshold":{"!doc":"<p>Number of pixels mouse moves before it&#39;s considered the start of a drag operation</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>Deprecated.  Use dijit/tree/ForestStoreModel directly instead.\nUsed in conjunction with query parameter.\nIf a query is specified (rather than a root node id), and a label is also specified,\nthen a fake root node is created and displayed, with this label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"model":{"!doc":"<p>Interface to read tree data, get notifications of changes to tree data,\nand for handling drop operations (i.e drag and drop onto the tree)</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"onDndCancel":{"!doc":"<p>Parameter to dndController, see <code>dijit/tree/dndSource.onDndCancel()</code>.\nGenerally this doesn&#39;t need to be set.</p>\n"},"onDndDrop":{"!doc":"<p>Parameter to dndController, see <code>dijit/tree/dndSource.onDndDrop()</code>.\nGenerally this doesn&#39;t need to be set.</p>\n"},"openOnClick":{"!doc":"<p>If true, clicking a folder node&#39;s label will open it, rather than calling onClick()</p>\n"},"openOnDblClick":{"!doc":"<p>If true, double-clicking a folder node&#39;s label will open it, rather than calling onDblClick()</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"path":{"!doc":"<p>Backward compatible singular variant of paths.</p>\n"},"paths":{"!doc":"<p>Full paths from rootNode to selected nodes expressed as array of items or array of ids.\nSince setting the paths may be asynchronous (because of waiting on dojo.data), set(&quot;paths&quot;, ...)\nreturns a Promise to indicate when the set is complete.</p>\n"},"persist":{"!doc":"<p>Enables/disables use of cookies for state saving.</p>\n"},"query":{"!doc":"<p>Deprecated.  User should specify query to the model directly instead.\nSpecifies datastore query to return the root item or top items for the tree.</p>\n"},"searchContainerNode":{},"selectedItem":{"!doc":"<p>Backward compatible singular variant of selectedItems.</p>\n"},"selectedItems":{"!doc":"<p>The currently selected items in this tree.\nThis property can only be set (via set(&#39;selectedItems&#39;, ...)) when that item is already\nvisible in the tree.   (I.e. the tree has already been expanded to show that node.)\nShould generally use <code>paths</code> attribute to set the selected items instead.</p>\n"},"showRoot":{"!doc":"<p>Should the root node be displayed, or hidden?</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>Deprecated.  Use &quot;model&quot; parameter instead.\nThe store to get data to display in the tree.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"collapseAll":{"!type":"fn() -> +any","!doc":"<p>Collapse all nodes in the tree</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn()"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dndController":{"!type":"fn()","!doc":"<p>Class to use as as the dnd controller.  Specifying this class enables DnD.\nGenerally you should specify this as dijit/tree/dndSource.\nSetting of dijit/tree/_dndSelector handles selection only (no actual DnD).</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"expandAll":{"!type":"fn() -> +any","!doc":"<p>Expand all nodes in the tree</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the previously focused child, or first child.\nSome applications may want to change this method to focus the [first] selected child.</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>End key pressed; go to last visible node.</p>\n"},"focusNode":{"!type":"fn(node: ?)","!doc":"<p>Focus on the specified node (which must be visible)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIconClass":{"!type":"fn(item: ?, opened: ?) -> string","!doc":"<p>Overridable function to return CSS class name to display icon</p>\n"},"getIconStyle":{"!type":"fn(item: ?, opened: ?) -> ?","!doc":"<p>Overridable function to return CSS styles to display icon</p>\n"},"getItemChildren":{"!type":"fn(parentItem: ?, onComplete: ?)","!doc":"<p>Deprecated.   This should be specified on the model itself.\n\n</p>\n<p>Overridable function that return array of child items of given parent item,\nor if parentItem==null then return top items in tree</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>Overridable function to get the label for a tree node (given the item)</p>\n"},"getLabelClass":{"!type":"fn(item: ?, opened: ?) -> string","!doc":"<p>Overridable function to return CSS class name to display label</p>\n"},"getLabelStyle":{"!type":"fn(item: ?, opened: ?) -> +any","!doc":"<p>Overridable function to return CSS styles to display label</p>\n"},"getNodesByItem":{"!type":"fn(item: ?) -> +any","!doc":"<p>Returns all tree nodes that refer to an item</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowClass":{"!type":"fn(item: ?, opened: ?) -> string","!doc":"<p>Overridable function to return CSS class name to display row</p>\n"},"getRowStyle":{"!type":"fn(item: ?, opened: ?) -> +any","!doc":"<p>Overridable function to return CSS styles to display row</p>\n"},"getTooltip":{"!type":"fn(item: ?) -> string","!doc":"<p>Overridable function to get the tooltip for a tree node (given the item)</p>\n"},"isExpandoNode":{"!type":"fn(node: ?, widget: ?) -> ?","!doc":"<p>check whether a dom node is the expandoNode for a particular TreeNode widget</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"itemCreator":{"!type":"fn(nodes: ?, target: ?, source: ?) -> [+Object]","!doc":"<p>Returns objects passed to <code>Tree.model.newItem()</code> based on DnD nodes\ndropped onto the tree.   Developer must override this method to enable\ndropping from external sources onto this Tree, unless the Tree.model&#39;s items\nhappen to look like {id: 123, name: &quot;Apple&quot; } with no other attributes.\n\n</p>\n<p>For each node in nodes[], which came from source, create a hash of name/value\npairs to be passed to Tree.model.newItem().  Returns array of those hashes.</p>\n"},"mayHaveChildren":{"!type":"fn(item: ?)","!doc":"<p>Deprecated.   This should be specified on the model itself.\n\n</p>\n<p>Overridable function to tell if an item has or may have children.\nControls whether or not +/- expando icon is shown.\n(For efficiency reasons we may not want to check if an element actually\nhas children until user clicks the expando node)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"resize":{"!type":"fn(changeSize: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"PathError":{}},"dijit/Tree._TreeNode":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Single node within a tree.   This class is used internally\nby Tree and should not be accessed directly.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTooltipAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indent":{"!doc":"<p>Levels from this node to the root node</p>\n"},"isExpandable":{"!doc":"<p>This node has children, so show the expando node (+ sign)</p>\n"},"isExpanded":{"!doc":"<p>This node is currently expanded (ie, opened)</p>\n"},"isTreeNode":{"!doc":"<p>Indicates that this is a TreeNode.   Used by <code>dijit.Tree</code> only,\nshould not be accessed directly.</p>\n"},"item":{"!doc":"<p>the dojo.data entry this tree represents</p>\n"},"label":{"!doc":"<p>Text of this tree node</p>\n"},"labelType":{"!doc":"<p>Specifies how to interpret the label.  Can be &quot;html&quot; or &quot;text&quot;.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Dynamic loading-related stuff.\nWhen an empty folder node appears, it is &quot;NotLoaded&quot; first,\nthen after dojo.data query it becomes &quot;Loading&quot; and, finally &quot;Loaded&quot;</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"collapse":{"!type":"fn() -> +any","!doc":"<p>Collapse this node (if it&#39;s expanded)</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"expand":{"!type":"fn() -> +any","!doc":"<p>Show my children</p>\n"},"focus":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIdentity":{"!type":"fn() -> ?"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getTreePath":{"!type":"fn() -> []"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeExpandable":{"!type":"fn()","!doc":"<p>if this node wasn&#39;t already showing the expando node,\nturn it into one and call _setExpando()</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(node: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setChildItems":{"!type":"fn(items: ?) -> +any","!doc":"<p>Sets the child items of this node, removing/adding nodes\nfrom current children to match specified items[] array.\nAlso, if this.persist == true, expands any children that were previously\nopened.</p>\n"},"setSelected":{"!type":"fn(selected: ?)","!doc":"<p>A Tree has a (single) currently selected node.\nMark that this node is/isn&#39;t that currently selected node.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/tree/TreeStoreModel":{"!type":"fn()","prototype":{"!doc":"<p>Implements dijit/Tree/model connecting to a dojo.data store with a single\nroot item.  Any methods passed into the constructor will override\nthe ones defined here.</p>\n","childrenAttrs":{"!doc":"<p>One or more attribute names (attributes in the dojo.data item) that specify that item&#39;s children</p>\n"},"deferItemLoadingUntilExpand":{"!doc":"<p>Setting this to true will cause the TreeStoreModel to defer calling loadItem on nodes\nuntil they are expanded. This allows for lazying loading where only one\nloadItem (and generally one network call, consequently) per expansion\n(rather than one for each child).\nThis relies on partial loading of the children items; each children item of a\nfully loaded item should contain the label and info about having children.</p>\n"},"labelAttr":{"!doc":"<p>If specified, get label for tree node from this attribute, rather\nthan by calling store.getLabel()</p>\n"},"newItemIdAttr":{"!doc":"<p>Name of attribute in the Object passed to newItem() that specifies the id.\n\n</p>\n<p>If newItemIdAttr is set then it&#39;s used when newItem() is called to see if an\nitem with the same id already exists, and if so just links to the old item\n(so that the old item ends up with two parents).\n\n</p>\n<p>Setting this to null or &quot;&quot; will make every drop create a new item.</p>\n"},"query":{"!doc":"<p>Specifies datastore query to return the root item for the tree.\nMust only return a single item.   Alternately can just pass in pointer\nto root item.</p>\n"},"root":{"!doc":"<p>Pointer to the root item (read only, not a parameter)</p>\n"},"store":{"!doc":"<p>Underlying store</p>\n"},"destroy":{"!type":"fn()"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Given the identity of an item, this method returns the item that has\nthat identity through the onItem callback.  Conforming implementations\nshould return null if there is no item with the given identity.\nImplementations of fetchItemByIdentity() may sometimes return an item\nfrom a local cache and may sometimes fetch an item from a remote server.</p>\n"},"getChildren":{"!type":"fn(parentItem: ?, onComplete: ?, onError: ?)","!doc":"<p>Calls onComplete() with array of child items of given parent item, all loaded.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>Get the label for an item</p>\n"},"getRoot":{"!type":"fn(onItem: ?, onError: ?)","!doc":"<p>Calls onItem with the root item for the tree, possibly a fabricated item.\nCalls onError on error.</p>\n"},"isItem":{"!type":"fn(something: ?) -> ?"},"mayHaveChildren":{"!type":"fn(item: ?) -> ?","!doc":"<p>Tells if an item has or may have children.  Implementing logic here\navoids showing +/- expando icon for nodes that we know don&#39;t have children.\n(For efficiency reasons we may not want to check if an element actually\nhas children until user clicks the expando node)</p>\n"},"newItem":{"!type":"fn(args: ?, parent: ?, insertIndex?: ?)","!doc":"<p>Creates a new item.   See <code>dojo/data/api/Write</code> for details on args.\nUsed in drag &amp; drop when item from external source dropped onto tree.</p>\n"},"pasteItem":{"!type":"fn(childItem: ?, oldParentItem: ?, newParentItem: ?, bCopy: ?, insertIndex?: ?)","!doc":"<p>Move or copy an item from one parent item to another.\nUsed in drag &amp; drop</p>\n"}}},"dijit/tree/ForestStoreModel":{"!type":"fn()","prototype":{"!proto":"dijit/tree/TreeStoreModel.prototype","!doc":"<p>Interface between a dijit.Tree and a dojo.data store that doesn&#39;t have a root item,\na.k.a. a store that has multiple &quot;top level&quot; items.\n</p>\n","childrenAttrs":{"!doc":"<p>One or more attribute names (attributes in the dojo.data item) that specify that item&#39;s children</p>\n"},"deferItemLoadingUntilExpand":{"!doc":"<p>Setting this to true will cause the TreeStoreModel to defer calling loadItem on nodes\nuntil they are expanded. This allows for lazying loading where only one\nloadItem (and generally one network call, consequently) per expansion\n(rather than one for each child).\nThis relies on partial loading of the children items; each children item of a\nfully loaded item should contain the label and info about having children.</p>\n"},"labelAttr":{"!doc":"<p>If specified, get label for tree node from this attribute, rather\nthan by calling store.getLabel()</p>\n"},"newItemIdAttr":{"!doc":"<p>Name of attribute in the Object passed to newItem() that specifies the id.\n\n</p>\n<p>If newItemIdAttr is set then it&#39;s used when newItem() is called to see if an\nitem with the same id already exists, and if so just links to the old item\n(so that the old item ends up with two parents).\n\n</p>\n<p>Setting this to null or &quot;&quot; will make every drop create a new item.</p>\n"},"query":{"!doc":"<p>Specifies the set of children of the root item.</p>\n"},"root":{"!doc":"<p>Pointer to the root item (read only, not a parameter)</p>\n"},"rootId":{"!doc":"<p>ID of fabricated root item</p>\n"},"rootLabel":{"!doc":"<p>Label of fabricated root item</p>\n"},"store":{"!doc":"<p>Underlying store</p>\n"},"destroy":{"!type":"fn()"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)"},"getChildren":{"!type":"fn(parentItem: ?, callback: ?, onError: ?)","!doc":"<p>Calls onComplete() with array of child items of given parent item, all loaded.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getLabel":{"!type":"fn(item: ?) -> ?"},"getRoot":{"!type":"fn(onItem: ?, onError: ?)","!doc":"<p>Calls onItem with the root item for the tree, possibly a fabricated item.\nCalls onError on error.</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool"},"mayHaveChildren":{"!type":"fn(item: ?) -> bool","!doc":"<p>Tells if an item has or may have children.  Implementing logic here\navoids showing +/- expando icon for nodes that we know don&#39;t have children.\n(For efficiency reasons we may not want to check if an element actually\nhas children until user clicks the expando node)</p>\n"},"newItem":{"!type":"fn(args: ?, parent: ?, insertIndex?: ?) -> ?","!doc":"<p>Creates a new item.   See dojo/data/api/Write for details on args.\nUsed in drag &amp; drop when item from external source dropped onto tree.</p>\n"},"pasteItem":{"!type":"fn(childItem: ?, oldParentItem: ?, newParentItem: ?, bCopy: ?, insertIndex?: ?)","!doc":"<p>Move or copy an item from one parent item to another.\nUsed in drag &amp; drop</p>\n"}}},"dijit/tree/_dndSelector":{"!type":"fn()","prototype":{"!proto":"dijit/tree/_dndContainer.prototype","!doc":"<p>This is a base class for <code>dijit/tree/dndSource</code>, and isn&#39;t meant to be used directly.\nIt&#39;s based on <code>dojo/dnd/Selector</code>.</p>\n","current":{"!doc":"<p>The currently hovered TreeNode.  Not set to anything for keyboard operation.  (TODO: change?)</p>\n"},"selection":{"!doc":"<p>(id to DomNode) map for every TreeNode that&#39;s currently selected.\nThe DOMNode is the TreeNode.rowNode.</p>\n"},"singular":{"!doc":"<p>Allows selection of only one element, if true.\nTree hasn&#39;t been tested in singular=true mode, unclear if it works.</p>\n"},"addTreeNode":{"!type":"fn(node: ?, isAnchor?: ?) -> +Node","!doc":"<p>add node to current selection</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Prepares the object to be garbage-collected</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>Iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>Returns the dojo/dnd/Container._Item (representing a dragged node) by it&#39;s key (id).\nCalled by dojo/dnd/Source.checkAcceptance().</p>\n"},"getSelectedTreeNodes":{"!type":"fn() -> []","!doc":"<p>Returns a list of selected node(s).\nUsed by dndSource on the start of a drag.</p>\n"},"isTreeNodeSelected":{"!type":"fn(node: ?) -> ?","!doc":"<p>return true if node is currently selected</p>\n"},"removeTreeNode":{"!type":"fn(node: ?) -> +Node","!doc":"<p>remove node and it&#39;s descendants from current selection</p>\n"},"setSelection":{"!type":"fn(newSelection: ?)","!doc":"<p>set the list of selected nodes to be exactly newSelection. All changes to the\nselection should be passed through this function, which ensures that derived\nattributes are kept up to date. Anchor will be deleted if it has been removed\nfrom the selection, but no new anchor will be added by this function.</p>\n"},"userSelect":{"!type":"fn(node: ?, multi: ?, range: ?)","!doc":"<p>Add or remove the given node from selection, responding\nto a user action such as a click or keypress.</p>\n"}}},"dijit/tree/_dndContainer":{"!type":"fn()","prototype":{"!doc":"<p>This is a base class for <code>dijit/tree/_dndSelector</code>, and isn&#39;t meant to be used directly.\nIt&#39;s modeled after <code>dojo/dnd/Container</code>.</p>\n","current":{"!doc":"<p>The currently hovered TreeNode.  Not set to anything for keyboard operation.  (TODO: change?)</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Prepares this object to be garbage-collected</p>\n"}}},"dijit/WidgetSet":{"!type":"fn()","prototype":{"!doc":"<p>A set of widgets indexed by id.\nDeprecated, will be removed in 2.0.\n</p>\n","add":{"!type":"fn(widget: ?)","!doc":"<p>Add a widget to this list. If a duplicate ID is detected, a error is thrown.\n</p>\n"},"byClass":{"!type":"fn(cls: ?) -> +instance","!doc":"<p>Reduce this widgetset to a new WidgetSet of a particular <code>declaredClass</code>\n</p>\n"},"byId":{"!type":"fn(id: ?) -> ?","!doc":"<p>Find a widget in this list by it&#39;s id.</p>\n"},"constructor":{"!type":"fn()"},"every":{"!type":"fn(func: ?, thisObj?: ?) -> bool","!doc":"<p>A synthetic clone of <code>array.every</code> acting explicitly on this WidgetSet\n</p>\n"},"filter":{"!type":"fn(filter: ?, thisObj?: ?) -> +instance","!doc":"<p>Filter down this WidgetSet to a smaller new WidgetSet\nWorks the same as <code>array.filter</code> and <code>NodeList.filter</code>\n</p>\n"},"forEach":{"!type":"fn(func: ?, thisObj?: ?) -> +any","!doc":"<p>Call specified function for each widget in this set.\n</p>\n"},"map":{"!type":"fn(func: ?, thisObj?: ?) -> +any","!doc":"<p>Create a new Array from this WidgetSet, following the same rules as <code>array.map</code></p>\n"},"remove":{"!type":"fn(id: ?)","!doc":"<p>Remove a widget from this WidgetSet. Does not destroy the widget; simply\nremoves the reference.</p>\n"},"some":{"!type":"fn(func: ?, thisObj?: ?) -> bool","!doc":"<p>A synthetic clone of <code>array.some</code> acting explicitly on this WidgetSet\n</p>\n"},"toArray":{"!type":"fn() -> []","!doc":"<p>Convert this WidgetSet into a true Array\n</p>\n"}}},"dijit/_BidiMixin":{"!doc":"<p>When has(&quot;dojo-bidi&quot;) is true, _WidgetBase will mixin this class.   It enables support for the textdir\nproperty to control text direction independently from the GUI direction.</p>\n","textDir":{"!doc":"<p>Bi-directional support, the main variable which is responsible for the direction of the text.\nThe text direction can be different than the GUI direction by using this parameter in creation\nof a widget.\n\n</p>\n<p>Allowed values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot;</li>\n<li>&quot;rtl&quot;</li>\n<li>&quot;auto&quot; - contextual the direction of a text defined by first strong letter.</li>\n</ol>\n<p>By default is as the page direction.</p>\n"},"applyTextDir":{"!type":"fn(element: ?, text?: ?)","!doc":"<p>Set element.dir according to this.textDir, assuming this.textDir has a value.</p>\n"},"enforceTextDirWithUcc":{"!type":"fn(option: ?, text: ?) -> string","!doc":"<p>Wraps by UCC (Unicode control characters) option&#39;s text according to this.textDir</p>\n"},"getTextDir":{"!type":"fn(text: ?) -> ?","!doc":"<p>Gets the right direction of text.</p>\n"},"restoreOriginalText":{"!type":"fn(origObj: ?) -> ?","!doc":"<p>Restores the text of origObj, if needed, after enforceTextDirWithUcc, e.g. set(&quot;textDir&quot;, textDir).</p>\n"}},"dijit/_BidiSupport":{"!doc":"<p>Deprecated module for enabling textdir support in the dijit widgets.   New code should just define\nhas(&quot;dojo-bidi&quot;) to return true, rather than manually requiring this module.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Deprecated module for enabling textdir support in the dijit widgets.   New code should just define\nhas(&quot;dojo-bidi&quot;) to return true, rather than manually requiring this module.</p>\n"}}},"dijit/_Calendar":{"!doc":"<p>Deprecated widget, used dijit/Calendar instead.   Will be removed in 2.0.</p>\n"},"dijit/_Templated":{"!type":"fn()","prototype":{"!proto":"dijit/_TemplatedMixin.prototype","!doc":"<p>Deprecated mixin for widgets that are instantiated from a template.\nWidgets should use _TemplatedMixin plus if necessary _WidgetsInTemplateMixin instead.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_skipNodeCache":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"searchContainerNode":{},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget from a template, setting this.domNode.</p>\n"},"constructor":{"!type":"fn()"},"destroyRendering":{"!type":"fn()"},"startup":{"!type":"fn()"}}},"dijit/_TimePicker":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A time picker dropdown, used by dijit/form/TimeTextBox.\nThis widget is not available as a standalone widget due to lack of accessibility support.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_clickableIncrement":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_totalIncrements":{},"_visibleIncrement":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The root className to use for the various states of this widget</p>\n"},"class":{},"clickableIncrement":{"!doc":"<p>ISO-8601 string representing the amount by which\nevery clickable element in the time picker increases.\nSet in local time, without a time zone.\nExample: <code>T00:15:00</code> creates 15 minute increments\nMust divide dijit/_TimePicker.visibleIncrement evenly</p>\n"},"constraints":{"!doc":"<p>Specifies valid range of times (start time, end time)</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"filterString":{"!doc":"<p>The string to filter by</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"selected":{"!doc":"<p>currently selected node</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>Date to display.\nDefaults to current time and date.\nCan be a Date object or an ISO-8601 string.\nIf you specify the GMT time zone (<code>-01:00</code>),\nthe time will be converted to the local time in the local time zone.\nOtherwise, the time is considered to be in the local time zone.\nIf you specify the date and isDate is true, the date is used.\nExample: if your local time zone is <code>GMT -05:00</code>,\n<code>T10:00:00</code> becomes <code>T10:00:00-05:00</code> (considered to be local time),\n<code>T10:00:00-01:00</code> becomes <code>T06:00:00-05:00</code> (4 hour difference),\n<code>T10:00:00Z</code> becomes <code>T05:00:00-05:00</code> (5 hour difference between Zulu and local time)\n<code>yyyy-mm-ddThh:mm:ss</code> is the format to set the date and time\nExample: <code>2007-06-01T09:00:00</code></p>\n"},"visibleIncrement":{"!doc":"<p>ISO-8601 string representing the amount by which\nevery element with a visible time in the time picker increases.\nSet in local time, without a time zone.\nExample: <code>T01:00:00</code> creates text in every 1 hour increment</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleKey":{"!type":"fn(e: ?) -> bool","!doc":"<p>Called from <code>dijit/form/_DateTimeTextBox</code> to pass a keypress event\nfrom the <code>dijit/form/TimeTextBox</code> to be handled in this widget</p>\n"},"isDisabledDate":{"!type":"fn(dateObject: ?, locale?: ?) -> bool","!doc":"<p>May be overridden to disable certain dates in the TimePicker e.g. <code>isDisabledDate=locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()"},"selectFirstNode":{"!type":"fn()","!doc":"<p>Select the first displayed item in the list.</p>\n"},"selectLastNode":{"!type":"fn()","!doc":"<p>Select the last displayed item in the list</p>\n"},"selectNextNode":{"!type":"fn()","!doc":"<p>Select the item just below the current selection.\nIf nothing selected, select first node.</p>\n"},"selectPreviousNode":{"!type":"fn()","!doc":"<p>Select the item just above the current selection.\nIf nothing selected, select last node (if\nyou select Previous and try to keep scrolling up the list).</p>\n"},"serialize":{"!type":"fn(dateObject: ?, options?: ?) -> ?","!doc":"<p>Format a Date object as a string according a subset of the ISO-8601 standard\n</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Used set(&#39;value&#39;) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_TimePicker.__Constraints":{"!type":"fn()","prototype":{"am":{"!doc":"<p>override strings for am in times</p>\n"},"clickableIncrement":{"!doc":"<p>See <code>dijit/_TimePicker.clickableIncrement</code></p>\n"},"datePattern":{"!doc":"<p>override pattern with this string</p>\n"},"formatLength":{"!doc":"<p>choice of long, short, medium or full (plus any custom additions).  Defaults to &#39;short&#39;</p>\n"},"fullYear":{"!doc":"<p>(format only) use 4 digit years whenever 2 digit years are called for</p>\n"},"locale":{"!doc":"<p>override the locale used to determine formatting rules</p>\n"},"pm":{"!doc":"<p>override strings for pm in times</p>\n"},"selector":{"!doc":"<p>choice of &#39;time&#39;,&#39;date&#39; (default: date and time)</p>\n"},"strict":{"!doc":"<p>(parse only) strict parsing, off by default</p>\n"},"timePattern":{"!doc":"<p>override pattern with this string</p>\n"}}},"dijit/form/_ListMouseMixin":{"!type":"fn()","prototype":{"!proto":"dijit/form/_ListBase.prototype","!doc":"<p>A mixin to handle mouse or touch events for a focus-less menu\nAbstract methods that must be defined externally:\n\n</p>\n<ul>\n<li>onClick: item was chosen (mousedown somewhere on the menu and mouseup somewhere on the menu)</li>\n</ul>\n","selected":{"!doc":"<p>currently selected node</p>\n"},"postCreate":{"!type":"fn()"},"selectFirstNode":{"!type":"fn()","!doc":"<p>Select the first displayed item in the list.</p>\n"},"selectLastNode":{"!type":"fn()","!doc":"<p>Select the last displayed item in the list</p>\n"},"selectNextNode":{"!type":"fn()","!doc":"<p>Select the item just below the current selection.\nIf nothing selected, select first node.</p>\n"},"selectPreviousNode":{"!type":"fn()","!doc":"<p>Select the item just above the current selection.\nIf nothing selected, select last node (if\nyou select Previous and try to keep scrolling up the list).</p>\n"}}},"dijit/form/_ListBase":{"!type":"fn()","prototype":{"!doc":"<p>Focus-less menu to handle UI events consistently.\nAbstract methods that must be defined externally:\n\n</p>\n<ul>\n<li>onSelect: item is active (mousedown but not yet mouseup, or keyboard arrow selected but no Enter)</li>\n<li>onDeselect:  cancels onSelect</li>\n</ul>\n","selected":{"!doc":"<p>currently selected node</p>\n"},"selectFirstNode":{"!type":"fn()","!doc":"<p>Select the first displayed item in the list.</p>\n"},"selectLastNode":{"!type":"fn()","!doc":"<p>Select the last displayed item in the list</p>\n"},"selectNextNode":{"!type":"fn()","!doc":"<p>Select the item just below the current selection.\nIf nothing selected, select first node.</p>\n"},"selectPreviousNode":{"!type":"fn()","!doc":"<p>Select the item just above the current selection.\nIf nothing selected, select last node (if\nyou select Previous and try to keep scrolling up the list).</p>\n"}}},"dijit/_base/place":{"!doc":"<p>Deprecated back compatibility module, new code should use dijit/place directly instead of using this module.</p>\n","getPopupAroundAlignment":{"!type":"fn(position: ?, leftToRight: ?)","!doc":"<p>Deprecated method, unneeded when using dijit/place directly.\nTransforms the passed array of preferred positions into a format suitable for\npassing as the aroundCorners argument to dijit/place.placeOnScreenAroundElement.</p>\n"},"getViewport":{"!type":"fn()","!doc":"<p>Deprecated method to return the dimensions and scroll position of the viewable area of a browser window.\nNew code should use windowUtils.getBox()</p>\n"},"placeOnScreen":{"!type":"fn(node: ?, pos: ?, corners: ?, padding?: ?, layoutNode: ?) -> ?","!doc":"<p>Positions node kitty-corner to the rectangle centered at (pos.x, pos.y) with width and height of\npadding.x <em> 2 and padding.y </em> 2, or zero if padding not specified.  Picks first corner in corners[]\nwhere node is fully visible, or the corner where it&#39;s most visible.\n\n</p>\n<p>Node is assumed to be absolutely or relatively positioned.</p>\n"},"placeOnScreenAroundElement":{"!type":"fn(node: ?, aroundNode: ?, aroundCorners: ?, layoutNode: ?)","!doc":"<p>Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object\nfor the &quot;around&quot; argument and finds a proper processor to place a node.\nDeprecated, new code should use dijit/place.around() instead.</p>\n"},"placeOnScreenAroundNode":{"!type":"fn(node: ?, aroundNode: ?, aroundCorners: ?, layoutNode: ?)","!doc":"<p>Position node adjacent or kitty-corner to aroundNode\nsuch that it&#39;s fully visible in viewport.\nDeprecated, new code should use dijit/place.around() instead.</p>\n"},"placeOnScreenAroundRectangle":{"!type":"fn(node: ?, aroundRect: ?, aroundCorners: ?, layoutNode: ?)","!doc":"<p>Like dijit.placeOnScreenAroundNode(), except that the &quot;around&quot;\nparameter is an arbitrary rectangle on the screen (x, y, width, height)\ninstead of a dom node.\nDeprecated, new code should use dijit/place.around() instead.</p>\n"}},"dijit/_base/popup":{"!doc":"<p>Deprecated.   Old module for popups, new code should use dijit/popup directly.</p>\n"},"dijit/_base/scroll":{"!doc":"<p>Back compatibility module, new code should use windowUtils directly instead of using this module.</p>\n"},"dijit/_base/sniff":{"!doc":"<p>Deprecated, back compatibility module, new code should require dojo/uacss directly instead of this module.</p>\n"},"dijit/_base/typematic":{"!doc":"<p>Deprecated, for back-compat, just loads top level module</p>\n"},"dijit/_base/wai":{"!doc":"<p>Deprecated methods for setting/getting wai roles and states.\nNew code should call setAttribute()/getAttribute() directly.\n\n</p>\n<p>Also loads hccss to apply dj_a11y class to root node if machine is in high-contrast mode.</p>\n","getWaiRole":{"!type":"fn(elem: ?) -> +any","!doc":"<p>Gets the role for an element (which should be a wai role).</p>\n"},"getWaiState":{"!type":"fn(elem: ?, state: ?) -> +any","!doc":"<p>Gets the value of a state on an element.</p>\n"},"hasWaiRole":{"!type":"fn(elem: ?, role?: ?) -> +any","!doc":"<p>Determines if an element has a particular role.</p>\n"},"hasWaiState":{"!type":"fn(elem: ?, state: ?) -> +any","!doc":"<p>Determines if an element has a given state.</p>\n"},"removeWaiRole":{"!type":"fn(elem: ?, role: ?)","!doc":"<p>Removes the specified role from an element.\nRemoves role attribute if no specific role provided (for backwards compat.)</p>\n"},"removeWaiState":{"!type":"fn(elem: ?, state: ?)","!doc":"<p>Removes a state from an element.</p>\n"},"setWaiRole":{"!type":"fn(elem: ?, role: ?)","!doc":"<p>Sets the role on an element.</p>\n"},"setWaiState":{"!type":"fn(elem: ?, state: ?, value: ?)","!doc":"<p>Sets a state on an element.</p>\n"}},"dijit/_base/window":{"!doc":"<p>Back compatibility module, new code should use windowUtils directly instead of using this module.</p>\n"},"dijit/_base":{"!doc":"<p>Includes all the modules in dijit/_base</p>\n"},"dijit/_editor/plugins/AlwaysShowToolbar":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin is required for Editors in auto-expand mode.\nIt handles the auto-expansion as the user adds/deletes text,\nand keeps the editor&#39;s toolbar visible even when the top of the editor\nhas scrolled off the top of the viewport (usually when editing a long\ndocument).</p>\n","_handleScroll":{"!doc":"<p>Enables/disables the handler for scroll events</p>\n"},"_lastHeight":{"!doc":"<p>Height in px of the editor at the last time we did sizing</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(e: ?)"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dijit/_editor/plugins/FontChoice":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides three drop downs for setting style in the editor\n(font, font size, and format block), as controlled by command.\n</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>Override _Plugin.useDefaultCommand...\nprocessing is handled by this plugin, not by dijit/Editor.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Overrides _Plugin.updateState().  This controls updating the menu\noptions to the right values on state changes in the document (that trigger a\ntest of the actions.)\nIt set value of drop down in toolbar to reflect font/font size/format block\nof text at current caret position.</p>\n"}}},"dijit/_editor/plugins/FontChoice._FontDropDown":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Base class for widgets that contains a label (like &quot;Font:&quot;)\nand a FilteringSelect drop down to pick a value.\nUsed as Toolbar entry.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>The label to apply to this particular FontDropDown.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"plainText":{"!doc":"<p>Flag to indicate that the returned label should be plain text\ninstead of an example.</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template used to construct the labeled dropdown.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"contextRequire":{"!type":"fn()","!doc":"<p>The context require that is used to resolve modules in template.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Over-ride for focus control of this widget.  Delegates focus down to the\nfiltering select.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Over-ride for the default postCreate action\nThis establishes the filtering selects and the like.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Over-ride to set specific properties.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_editor/plugins/FontChoice._FontNameDropDown":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Dropdown to select a font; goes in editor toolbar.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"comboClass":{},"command":{"!doc":"<p>The editor &#39;command&#39; implemented by this plugin.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"generic":{"!doc":"<p>Use generic (web standard) font names</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>The label to apply to this particular FontDropDown.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"plainText":{"!doc":"<p>Flag to indicate that the returned label should be plain text\ninstead of an example.</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template used to construct the labeled dropdown.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"contextRequire":{"!type":"fn()","!doc":"<p>The context require that is used to resolve modules in template.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Over-ride for focus control of this widget.  Delegates focus down to the\nfiltering select.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getLabel":{"!type":"fn(value: ?, name: ?) -> string","!doc":"<p>Function used to generate the labels of the format dropdown\nwill return a formatted, or plain label based on the value\nof the plainText option.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Over-ride for the default postCreate action\nThis establishes the filtering selects and the like.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Over-ride for the default posr mixin control</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_editor/plugins/FontChoice._FontSizeDropDown":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Dropdown to select a font size; goes in editor toolbar.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"comboClass":{},"command":{"!doc":"<p>The editor &#39;command&#39; implemented by this plugin.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>The label to apply to this particular FontDropDown.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"plainText":{"!doc":"<p>Flag to indicate that the returned label should be plain text\ninstead of an example.</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template used to construct the labeled dropdown.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"values":{"!doc":"<p>The HTML font size values supported by this plugin</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"contextRequire":{"!type":"fn()","!doc":"<p>The context require that is used to resolve modules in template.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Over-ride for focus control of this widget.  Delegates focus down to the\nfiltering select.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getLabel":{"!type":"fn(value: ?, name: ?) -> string","!doc":"<p>Function used to generate the labels of the format dropdown\nwill return a formatted, or plain label based on the value\nof the plainText option.\nWe&#39;re stuck using the deprecated FONT tag to correspond\nwith the size measurements used by the editor</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Over-ride for the default postCreate action\nThis establishes the filtering selects and the like.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Over-ride to set specific properties.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/_editor/plugins/FontChoice._FormatBlockDropDown":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Dropdown to select a format (like paragraph or heading); goes in editor toolbar.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"comboClass":{},"command":{"!doc":"<p>The editor &#39;command&#39; implemented by this plugin.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>The label to apply to this particular FontDropDown.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"plainText":{"!doc":"<p>Flag to indicate that the returned label should be plain text\ninstead of an example.</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template used to construct the labeled dropdown.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"values":{"!doc":"<p>The HTML format tags supported by this plugin</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"contextRequire":{"!type":"fn()","!doc":"<p>The context require that is used to resolve modules in template.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Over-ride for focus control of this widget.  Delegates focus down to the\nfiltering select.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getLabel":{"!type":"fn(value: ?, name: ?) -> string","!doc":"<p>Function used to generate the labels of the format dropdown\nwill return a formatted, or plain label based on the value\nof the plainText option.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Over-ride to set specific properties.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/FilteringSelect":{"!type":"fn()","prototype":{"!proto":"dijit/form/MappedTextBox.prototype","!doc":"<p>An enhanced version of the HTML SELECT tag, populated dynamically\n</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_lastDisplayedValue":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoComplete":{"!doc":"<p>If user types in a partial string, and then tab out of the <code>&lt;input&gt;</code> box,\nautomatically copy the first entry displayed in the drop down list to\nthe <code>&lt;input&gt;</code> field</p>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"class":{},"constraints":{"!doc":"<p>user-defined object needed to pass parameters to the validator functions</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"fetchProperties":{"!doc":"<p>Mixin to the store&#39;s fetch.\nFor example, to set the sort order of the ComboBox menu, pass:\n\n</p>\n<pre><code>{ sort: [{attribute:&quot;name&quot;,descending: true}] }</code></pre>\n<p>To override the default queryOptions so that deep=false, do:\n\n\n</p>\n<pre><code>{ queryOptions: {ignoreCase: true, deep: false} }</code></pre>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to have a down arrow button, to display the drop down list.\nDefaults to true.</p>\n"},"highlightMatch":{"!doc":"<p>One of: &quot;first&quot;, &quot;all&quot; or &quot;none&quot;.\n\n</p>\n<p>If the ComboBox/FilteringSelect opens with the search results and the searched\nstring can be found, it will be highlighted.  If set to &quot;all&quot;\nthen will probably want to change <code>queryExpr</code> parameter to &#39;<em>${0}</em>&#39;\n\n</p>\n<p>Highlighting is only performed when <code>labelType</code> is &quot;text&quot;, so as to not\ninterfere with any HTML markup an HTML label might contain.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ignoreCase":{"!doc":"<p>Set true if the query should ignore case when matching possible items</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"item":{"!doc":"<p>This is the item returned by the dojo/store/api/Store implementation that\nprovides the data for this ComboBox, it&#39;s the currently selected item.</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the\ndojo.data items.\nIf not specified, the searchAttr attribute is used instead.</p>\n"},"labelType":{"!doc":"<p>Specifies how to interpret the labelAttr in the data store items.\nCan be &quot;html&quot; or &quot;text&quot;.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageSize":{"!doc":"<p>Argument to data provider.\nSpecifies maximum number of search results to return per query</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"query":{"!doc":"<p>A query that can be passed to <code>store</code> to initially filter the items.\nComboBox overwrites any reference to the <code>searchAttr</code> and sets it to the <code>queryExpr</code> with the user&#39;s input substituted.</p>\n"},"queryExpr":{"!doc":"<p>This specifies what query is sent to the data store,\nbased on what the user has typed.  Changing this expression will modify\nwhether the results are only exact matches, a &quot;starting with&quot; match,\netc.\n<code>${0}</code> will be substituted for the user text.\n<code>*</code> is used for wildcards.\n<code>${0}*</code> means &quot;starts with&quot;, <code>*${0}*</code> means &quot;contains&quot;, <code>${0}</code> means &quot;is&quot;</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>True (default) if user is required to enter a value into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchAttr":{"!doc":"<p>Search for items in the data store where this attribute (in the item)\nmatches what the user typed</p>\n"},"searchContainerNode":{},"searchDelay":{"!doc":"<p>Delay in milliseconds between when user types something and we start\nsearching based on that value</p>\n"},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this ComboBox.\n\n</p>\n<p>Should be dojo/store/api/Store, but dojo/data/api/Read supported\nfor backwards compatibility.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"doHighlight":{"!type":"fn(label: ?, find: ?) -> ?","!doc":"<p>Highlights the string entered by the user in the menu.  By default this\nhighlights the first occurrence found. Override this method\nto implement your custom highlighting.</p>\n"},"dropDownClass":{"!type":"fn()","!doc":"<p>Dropdown widget class used to select a date/time.\nSubclasses should specify this.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return an error message to show if appropriate</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn() -> bool"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> string","!doc":"<p>Overridable function used to convert the get(&#39;value&#39;) result to a canonical\n(non-localized) string.  For example, will print dates in ISO format, and\nnumbers the same way as they are represented in javascript.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/MappedTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/ValidationTextBox.prototype","!doc":"<p>A dijit/form/ValidationTextBox subclass which provides a base class for widgets that have\na visible formatted display value, and a serializable\nvalue in a hidden input field which is actually sent to the server.</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraints":{"!doc":"<p>user-defined object needed to pass parameters to the validator functions</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return an error message to show if appropriate</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if value is valid.\nCan override with your own routine in a subclass.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> string","!doc":"<p>Overridable function used to convert the get(&#39;value&#39;) result to a canonical\n(non-localized) string.  For example, will print dates in ISO format, and\nnumbers the same way as they are represented in javascript.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/ValidationTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/TextBox.prototype","!doc":"<p>Base class for textbox widgets with the ability to validate content of various types and provide user feedback.</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraints":{"!doc":"<p>user-defined object needed to pass parameters to the validator functions</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"isValid":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return an error message to show if appropriate</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Called by oninit, onblur, and onkeypress.</p>\n"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/ComboBoxMixin":{"!type":"fn()","prototype":{"!proto":"dijit/_HasDropDown.prototype","!doc":"<p>Provides main functionality of ComboBox widget</p>\n","_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"autoComplete":{"!doc":"<p>If user types in a partial string, and then tab out of the <code>&lt;input&gt;</code> box,\nautomatically copy the first entry displayed in the drop down list to\nthe <code>&lt;input&gt;</code> field</p>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"cssStateNodes":{},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"fetchProperties":{"!doc":"<p>Mixin to the store&#39;s fetch.\nFor example, to set the sort order of the ComboBox menu, pass:\n\n</p>\n<pre><code>{ sort: [{attribute:&quot;name&quot;,descending: true}] }</code></pre>\n<p>To override the default queryOptions so that deep=false, do:\n\n\n</p>\n<pre><code>{ queryOptions: {ignoreCase: true, deep: false} }</code></pre>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to have a down arrow button, to display the drop down list.\nDefaults to true.</p>\n"},"highlightMatch":{"!doc":"<p>One of: &quot;first&quot;, &quot;all&quot; or &quot;none&quot;.\n\n</p>\n<p>If the ComboBox/FilteringSelect opens with the search results and the searched\nstring can be found, it will be highlighted.  If set to &quot;all&quot;\nthen will probably want to change <code>queryExpr</code> parameter to &#39;<em>${0}</em>&#39;\n\n</p>\n<p>Highlighting is only performed when <code>labelType</code> is &quot;text&quot;, so as to not\ninterfere with any HTML markup an HTML label might contain.</p>\n"},"ignoreCase":{"!doc":"<p>Set true if the query should ignore case when matching possible items</p>\n"},"item":{"!doc":"<p>This is the item returned by the dojo/store/api/Store implementation that\nprovides the data for this ComboBox, it&#39;s the currently selected item.</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the\ndojo.data items.\nIf not specified, the searchAttr attribute is used instead.</p>\n"},"labelType":{"!doc":"<p>Specifies how to interpret the labelAttr in the data store items.\nCan be &quot;html&quot; or &quot;text&quot;.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"pageSize":{"!doc":"<p>Argument to data provider.\nSpecifies maximum number of search results to return per query</p>\n"},"query":{"!doc":"<p>A query that can be passed to <code>store</code> to initially filter the items.\nComboBox overwrites any reference to the <code>searchAttr</code> and sets it to the <code>queryExpr</code> with the user&#39;s input substituted.</p>\n"},"queryExpr":{"!doc":"<p>This specifies what query is sent to the data store,\nbased on what the user has typed.  Changing this expression will modify\nwhether the results are only exact matches, a &quot;starting with&quot; match,\netc.\n<code>${0}</code> will be substituted for the user text.\n<code>*</code> is used for wildcards.\n<code>${0}*</code> means &quot;starts with&quot;, <code>*${0}*</code> means &quot;contains&quot;, <code>${0}</code> means &quot;is&quot;</p>\n"},"searchAttr":{"!doc":"<p>Search for items in the data store where this attribute (in the item)\nmatches what the user typed</p>\n"},"searchDelay":{"!doc":"<p>Delay in milliseconds between when user types something and we start\nsearching based on that value</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this ComboBox.\n\n</p>\n<p>Should be dojo/store/api/Store, but dojo/data/api/Read supported\nfor backwards compatibility.</p>\n"},"templateString":{},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"doHighlight":{"!type":"fn(label: ?, find: ?) -> ?","!doc":"<p>Highlights the string entered by the user in the menu.  By default this\nhighlights the first occurrence found. Override this method\nto implement your custom highlighting.</p>\n"},"dropDownClass":{"!type":"fn()","!doc":"<p>Dropdown widget class used to select a date/time.\nSubclasses should specify this.</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>set up nodes and connect our mouse and keyboard events</p>\n"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"}}},"dijit/form/_AutoCompleterMixin":{"!type":"fn()","prototype":{"!proto":"dijit/form/_SearchMixin.prototype","!doc":"<p>A mixin that implements the base functionality for <code>dijit/form/ComboBox</code>/<code>dijit/form/FilteringSelect</code></p>\n","_stopClickEvents":{},"autoComplete":{"!doc":"<p>If user types in a partial string, and then tab out of the <code>&lt;input&gt;</code> box,\nautomatically copy the first entry displayed in the drop down list to\nthe <code>&lt;input&gt;</code> field</p>\n"},"fetchProperties":{"!doc":"<p>Mixin to the store&#39;s fetch.\nFor example, to set the sort order of the ComboBox menu, pass:\n\n</p>\n<pre><code>{ sort: [{attribute:&quot;name&quot;,descending: true}] }</code></pre>\n<p>To override the default queryOptions so that deep=false, do:\n\n\n</p>\n<pre><code>{ queryOptions: {ignoreCase: true, deep: false} }</code></pre>\n"},"highlightMatch":{"!doc":"<p>One of: &quot;first&quot;, &quot;all&quot; or &quot;none&quot;.\n\n</p>\n<p>If the ComboBox/FilteringSelect opens with the search results and the searched\nstring can be found, it will be highlighted.  If set to &quot;all&quot;\nthen will probably want to change <code>queryExpr</code> parameter to &#39;<em>${0}</em>&#39;\n\n</p>\n<p>Highlighting is only performed when <code>labelType</code> is &quot;text&quot;, so as to not\ninterfere with any HTML markup an HTML label might contain.</p>\n"},"ignoreCase":{"!doc":"<p>Set true if the query should ignore case when matching possible items</p>\n"},"item":{"!doc":"<p>This is the item returned by the dojo/store/api/Store implementation that\nprovides the data for this ComboBox, it&#39;s the currently selected item.</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the\ndojo.data items.\nIf not specified, the searchAttr attribute is used instead.</p>\n"},"labelType":{"!doc":"<p>Specifies how to interpret the labelAttr in the data store items.\nCan be &quot;html&quot; or &quot;text&quot;.</p>\n"},"maxHeight":{},"pageSize":{"!doc":"<p>Argument to data provider.\nSpecifies maximum number of search results to return per query</p>\n"},"query":{"!doc":"<p>A query that can be passed to <code>store</code> to initially filter the items.\nComboBox overwrites any reference to the <code>searchAttr</code> and sets it to the <code>queryExpr</code> with the user&#39;s input substituted.</p>\n"},"queryExpr":{"!doc":"<p>This specifies what query is sent to the data store,\nbased on what the user has typed.  Changing this expression will modify\nwhether the results are only exact matches, a &quot;starting with&quot; match,\netc.\n<code>${0}</code> will be substituted for the user text.\n<code>*</code> is used for wildcards.\n<code>${0}*</code> means &quot;starts with&quot;, <code>*${0}*</code> means &quot;contains&quot;, <code>${0}</code> means &quot;is&quot;</p>\n"},"searchAttr":{"!doc":"<p>Search for items in the data store where this attribute (in the item)\nmatches what the user typed</p>\n"},"searchDelay":{"!doc":"<p>Delay in milliseconds between when user types something and we start\nsearching based on that value</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this ComboBox.\nThe store must accept an object hash of properties for its query. See <code>query</code> and <code>queryExpr</code> for details.</p>\n"},"closeDropDown":{"!type":"fn()"},"constructor":{"!type":"fn()"},"doHighlight":{"!type":"fn(label: ?, find: ?) -> ?","!doc":"<p>Highlights the string entered by the user in the menu.  By default this\nhighlights the first occurrence found. Override this method\nto implement your custom highlighting.</p>\n"},"isLoaded":{"!type":"fn() -> bool"},"loadDropDown":{"!type":"fn(callback: ?)"},"postCreate":{"!type":"fn()","!doc":"<p>Subclasses must call this method from their postCreate() methods</p>\n"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()"}}},"dijit/form/DataList":{"!type":"fn()","prototype":{"!proto":"dojo/store/Memory.prototype","!doc":"<p>Inefficient but small data store specialized for inlined data via OPTION tags\n</p>\n","data":{"!doc":"<p>The array of all the objects in the memory store</p>\n"},"idProperty":{"!doc":"<p>Indicates the property to use as the identity property. The values of this\nproperty should be unique.</p>\n"},"index":{"!doc":"<p>An index of data indices into the data array by id</p>\n"},"add":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef: ?)","!doc":"<p>Create the widget.</p>\n"},"destroy":{"!type":"fn()"},"fetchSelectedItem":{"!type":"fn() -> ?","!doc":"<p>Get the option marked as selected, like <code>&lt;option selected&gt;</code>.\nNot part of dojo.data API.</p>\n"},"get":{"!type":"fn(id: ?) -> ?","!doc":"<p>Retrieves an object by its identity</p>\n"},"getChildren":{"!type":"fn(parent: ?, options?: ?) -> +dojo","!doc":"<p>Retrieves the children of an object.</p>\n"},"getIdentity":{"!type":"fn(object: ?) -> number","!doc":"<p>Returns an object&#39;s identity</p>\n"},"getMetadata":{"!type":"fn(object: ?) -> ?","!doc":"<p>Returns any metadata about the object. This may include attribution,\ncache directives, history, or version information.</p>\n"},"put":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object</p>\n"},"query":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"queryEngine":{"!type":"fn(query: ?, options?: ?) -> fn()","!doc":"<p>Defines the query engine to use for querying the data store</p>\n"},"remove":{"!type":"fn(id: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Sets the given data as the source for this store, and indexes it</p>\n"},"transaction":{"!type":"fn() -> +dojo","!doc":"<p>Starts a new transaction.\nNote that a store user might not call transaction() prior to using put,\ndelete, etc. in which case these operations effectively could be thought of\nas &quot;auto-commit&quot; style actions.</p>\n"}}},"dijit/form/_SearchMixin":{"!type":"fn()","prototype":{"!doc":"<p>A mixin that implements the base functionality to search a store based upon user-entered text such as\nwith <code>dijit/form/ComboBox</code> or <code>dijit/form/FilteringSelect</code></p>\n","fetchProperties":{"!doc":"<p>Mixin to the store&#39;s fetch.\nFor example, to set the sort order of the ComboBox menu, pass:\n\n</p>\n<pre><code>{ sort: [{attribute:&quot;name&quot;,descending: true}] }</code></pre>\n<p>To override the default queryOptions so that deep=false, do:\n\n\n</p>\n<pre><code>{ queryOptions: {ignoreCase: true, deep: false} }</code></pre>\n"},"ignoreCase":{"!doc":"<p>Set true if the query should ignore case when matching possible items</p>\n"},"pageSize":{"!doc":"<p>Argument to data provider.\nSpecifies maximum number of search results to return per query</p>\n"},"query":{"!doc":"<p>A query that can be passed to <code>store</code> to initially filter the items.\nComboBox overwrites any reference to the <code>searchAttr</code> and sets it to the <code>queryExpr</code> with the user&#39;s input substituted.</p>\n"},"queryExpr":{"!doc":"<p>This specifies what query is sent to the data store,\nbased on what the user has typed.  Changing this expression will modify\nwhether the results are only exact matches, a &quot;starting with&quot; match,\netc.\n<code>${0}</code> will be substituted for the user text.\n<code>*</code> is used for wildcards.\n<code>${0}*</code> means &quot;starts with&quot;, <code>*${0}*</code> means &quot;contains&quot;, <code>${0}</code> means &quot;is&quot;</p>\n"},"searchAttr":{"!doc":"<p>Search for items in the data store where this attribute (in the item)\nmatches what the user typed</p>\n"},"searchDelay":{"!doc":"<p>Delay in milliseconds between when user types something and we start\nsearching based on that value</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this ComboBox.\nThe store must accept an object hash of properties for its query. See <code>query</code> and <code>queryExpr</code> for details.</p>\n"},"constructor":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"}}},"dijit/form/_ComboBoxMenu":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Focus-less menu for internal use in <code>dijit/form/ComboBox</code>\nAbstract methods that must be defined externally:\n\n</p>\n<ul>\n<li>onChange: item was explicitly chosen (mousedown somewhere on the menu and mouseup somewhere on the menu)</li>\n<li>onPage: next(1) or previous(-1) button pressed</li>\n</ul>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_messages":{"!doc":"<p>Holds &quot;next&quot; and &quot;previous&quot; text for paging buttons on drop down</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>currently selected node</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clearResultList":{"!type":"fn()","!doc":"<p>Clears the entries in the drop down list, but of course keeps the previous and next buttons.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createOptions":{"!type":"fn(results: ?, options: ?, labelFunc: ?)","!doc":"<p>Fills in the items in the drop down list</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getHighlightedOption":{"!type":"fn() -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleKey":{"!type":"fn(evt: ?) -> bool","!doc":"<p>Handle keystroke event forwarded from ComboBox, returning false if it&#39;s\na keystroke I recognize and process, true otherwise.</p>\n"},"highlightFirstOption":{"!type":"fn()","!doc":"<p>Highlight the first real item in the list (not Previous Choices).</p>\n"},"highlightLastOption":{"!type":"fn()","!doc":"<p>Highlight the last real item in the list (not More Choices).</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"selectFirstNode":{"!type":"fn()","!doc":"<p>Select the first displayed item in the list.</p>\n"},"selectLastNode":{"!type":"fn()","!doc":"<p>Select the last displayed item in the list</p>\n"},"selectNextNode":{"!type":"fn()","!doc":"<p>Select the item just below the current selection.\nIf nothing selected, select first node.</p>\n"},"selectPreviousNode":{"!type":"fn()","!doc":"<p>Select the item just above the current selection.\nIf nothing selected, select last node (if\nyou select Previous and try to keep scrolling up the list).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_ComboBoxMenuMixin":{"!type":"fn()","prototype":{"_messages":{"!doc":"<p>Holds &quot;next&quot; and &quot;previous&quot; text for paging buttons on drop down</p>\n"},"buildRendering":{"!type":"fn()"},"clearResultList":{"!type":"fn()","!doc":"<p>Clears the entries in the drop down list, but of course keeps the previous and next buttons.</p>\n"},"createOptions":{"!type":"fn(results: ?, options: ?, labelFunc: ?)","!doc":"<p>Fills in the items in the drop down list</p>\n"},"getHighlightedOption":{"!type":"fn() -> ?"},"highlightFirstOption":{"!type":"fn()","!doc":"<p>Highlight the first real item in the list (not Previous Choices).</p>\n"},"highlightLastOption":{"!type":"fn()","!doc":"<p>Highlight the last real item in the list (not More Choices).</p>\n"},"postMixInProperties":{"!type":"fn()"},"selectFirstNode":{"!type":"fn()"},"selectLastNode":{"!type":"fn()"}}},"dijit/_editor/plugins/FullScreen":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides FullScreen capability to the editor.  When\ntoggled on, it will render the editor into the full window and\noverlay everything.  It also binds to the hotkey: CTRL-SHIFT-F11\nfor toggling fullscreen mode.</p>\n","_origiFrameState":{"!doc":"<p>The original view state of the iframe of the editor.</p>\n"},"_origState":{"!doc":"<p>The original view state of the editor.</p>\n"},"_resizeHandle":{"!doc":"<p>Connection point used for handling resize when window resizes.</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"isFullscreen":{"!doc":"<p>Read-Only variable used to denote of the editor is in fullscreen mode or not.</p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"zIndex":{"!doc":"<p>zIndex value used for overlaying the full page.\ndefault is 500.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Over-ride to ensure the resize handle gets cleaned up.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"toggle":{"!type":"fn()","!doc":"<p>Function to allow programmatic toggling of the view.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dijit/_editor/plugins/LinkDialog":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides the basis for an &#39;anchor&#39; (link) dialog and an extension of it\nprovides the image link dialog.</p>\n","_hostRxp":{},"_userAtRxp":{},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"emailRegExp":{"!doc":"<p>Used for validating input as correct email address.  Taken from dojox.validate</p>\n"},"htmlTemplate":{"!doc":"<p>String used for templating the HTML to insert at the desired point.</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"linkDialogTemplate":{"!doc":"<p>Template for contents of TooltipDialog to pick URL</p>\n"},"tag":{"!doc":"<p>Tag used for the link type.</p>\n"},"urlRegExp":{"!doc":"<p>Used for validating input as correct URL.  While file:// urls are not terribly\nuseful, they are technically valid.</p>\n"},"useDefaultCommand":{},"buttonClass":{"!type":"fn()"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}},"ImgLinkDialog":{"!type":"fn()"}},"dijit/_editor/plugins/LinkDialog.ImgLinkDialog":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin extends LinkDialog and adds in a plugin for handling image links.\nprovides the image link dialog.</p>\n","_hostRxp":{},"_userAtRxp":{},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"emailRegExp":{"!doc":"<p>Used for validating input as correct email address.  Taken from dojox.validate</p>\n"},"htmlTemplate":{"!doc":"<p>String used for templating the <code>&lt;img&gt;</code> HTML to insert at the desired point.</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"linkDialogTemplate":{"!doc":"<p>Over-ride for template since img dialog doesn&#39;t need target that anchor tags may.</p>\n"},"tag":{"!doc":"<p>Tag used for the link type (img).</p>\n"},"urlRegExp":{"!doc":"<p>Used for validating input as correct URL.  While file:// urls are not terribly\nuseful, they are technically valid.</p>\n"},"useDefaultCommand":{},"buttonClass":{"!type":"fn()"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dijit/_editor/plugins/NewPage":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides a simple &#39;new page&#39; capability.  In other\nwords, set content to some default user defined string.</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"content":{"!doc":"<p>The default content to insert into the editor as the new page.\nThe default is the <code>&lt;br&gt;</code> tag, a single blank line.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dijit/_editor/plugins/Print":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides Print capability to the editor.  When\nclicked, the document in the editor frame will be printed.</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dijit/_editor/plugins/TabIndent":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin is used to allow the use of the tab and shift-tab keys\nto indent/outdent list items.  This overrides the default behavior\nof moving focus from/to the toolbar</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{},"buttonClass":{"!type":"fn()"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()"}}},"dijit/_editor/plugins/TextColor":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides dropdown color pickers for setting text color and background color</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>False as we do not use the default editor command/click behavior.</p>\n"},"buttonClass":{"!type":"fn()"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Overrides _Plugin.updateState().  This updates the ColorPalette\nto show the color of the currently selected text.</p>\n"}}},"dijit/_editor/plugins/ToggleDir":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin is used to toggle direction of the edited document,\nindependent of what direction the whole page is.</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{},"buttonClass":{"!type":"fn()"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dijit/_editor/plugins/ViewSource":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides a simple view source capability.  When view\nsource mode is enabled, it disables all other buttons/plugins on the RTE.\nIt also binds to the hotkey: CTRL-SHIFT-F11 for toggling ViewSource mode.</p>\n","_fsPlugin":{"!doc":"<p>Reference to a registered fullscreen plugin so that viewSource knows\nhow to scale.</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"readOnly":{"!doc":"<p>Boolean flag used to indicate if the source view should be readonly or not.\nCannot be changed after initialization of the plugin.\nDefaults to false.</p>\n"},"stripComments":{"!doc":"<p>Boolean flag used to indicate if comment tags should be stripped from the document.\nDefaults to true.</p>\n"},"stripIFrames":{},"stripScripts":{"!doc":"<p>Boolean flag used to indicate if script tags should be stripped from the document.\nDefaults to true.</p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Over-ride to remove the node used to correct for IE&#39;s\nzoom bug.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setSourceAreaCaret":{"!type":"fn()","!doc":"<p>Internal function to set the caret in the sourceArea\nto 0x0</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"toggle":{"!type":"fn()","!doc":"<p>Function to allow programmatic toggling of the view.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dijit/_editor/selection":{"!doc":"<p>Deprecated text selection API.  Will be removed in 2.0.  New code should use dijit/selection.</p>\n","collapse":{"!type":"fn(beginning: ?)","!doc":"<p>Function to collapse (clear), the current selection</p>\n"},"getAncestorElement":{"!type":"fn(tagName: ?) -> ?","!doc":"<p>Return the parent element of the current selection which is of\ntype tagName (or one of the other specified tagName)</p>\n"},"getParentElement":{"!type":"fn() -> ?","!doc":"<p>Get the parent element of the current selection</p>\n"},"getParentOfType":{"!type":"fn(node: ?, tags: ?) -> +Node","!doc":"<p>Function to locate a parent node that matches one of a set of tags</p>\n"},"getSelectedElement":{"!type":"fn() -> ?","!doc":"<p>Retrieves the selected element (if any), just in the case that\na single element (object like and image or a table) is\nselected.</p>\n"},"getSelectedHtml":{"!type":"fn() -> string","!doc":"<p>Return the html text of the current selection or null if unavailable</p>\n"},"getSelectedText":{"!type":"fn() -> string","!doc":"<p>Return the text (no html tags) included in the current selection or null if no text is selected</p>\n"},"getType":{"!type":"fn() -> string","!doc":"<p>Get the selection type (like win.doc.select.type in IE).</p>\n"},"hasAncestorElement":{"!type":"fn(tagName: ?) -> bool","!doc":"<p>Check whether current selection has a  parent element which is\nof type tagName (or one of the other specified tagName)</p>\n"},"inSelection":{"!type":"fn(node: ?) -> bool","!doc":"<p>This function determines if &#39;node&#39; is\nin the current selection.</p>\n"},"isTag":{"!type":"fn(node: ?, tags: ?) -> ?","!doc":"<p>Function to determine if a node is one of an array of tags.</p>\n"},"remove":{"!type":"fn() -> ?","!doc":"<p>Function to delete the currently selected content from the document.</p>\n"},"selectElement":{"!type":"fn(element: ?, nochangefocus?: ?)","!doc":"<p>clear previous selection and select element (including all its children)</p>\n"},"selectElementChildren":{"!type":"fn(element: ?, nochangefocus?: ?)","!doc":"<p>clear previous selection and select the content of the node\n(excluding the node itself)</p>\n"}},"dijit/_tree/dndSource":{"!doc":"<p>Deprecated module, use dijit/tree/dndSource instead.</p>\n"},"dijit/tree/dndSource":{"!type":"fn()","prototype":{"!proto":"dijit/tree/_dndSelector.prototype","!doc":"<p>Handles drag and drop operations (as a source or a target) for <code>dijit.Tree</code></p>\n","accept":{"!doc":"<p>List of accepted types (text strings) for the Tree; defaults to\n[&quot;text&quot;]</p>\n"},"betweenThreshold":{"!doc":"<p>Distance from upper/lower edge of node to allow drop to reorder nodes</p>\n"},"copyOnly":{"!doc":"<p>Copy items, if true, use a state of Ctrl key otherwise</p>\n"},"current":{"!doc":"<p>The currently hovered TreeNode.  Not set to anything for keyboard operation.  (TODO: change?)</p>\n"},"dragThreshold":{"!doc":"<p>The move delay in pixels before detecting a drag; 5 by default</p>\n"},"generateText":{},"isSource":{"!doc":"<p>Can be used as a DnD source.</p>\n"},"selection":{"!doc":"<p>(id to DomNode) map for every TreeNode that&#39;s currently selected.\nThe DOMNode is the TreeNode.rowNode.</p>\n"},"singular":{"!doc":"<p>Allows selection of only one element, if true.\nTree hasn&#39;t been tested in singular=true mode, unclear if it works.</p>\n"},"addTreeNode":{"!type":"fn(node: ?, isAnchor?: ?) -> +Node","!doc":"<p>add node to current selection</p>\n"},"checkAcceptance":{"!type":"fn(source: ?, nodes: ?) -> bool","!doc":"<p>Checks if the target can accept nodes from this source</p>\n"},"checkItemAcceptance":{"!type":"fn(target: ?, source: ?, position: ?) -> bool","!doc":"<p>Stub function to be overridden if one wants to check for the ability to drop at the node/item level</p>\n"},"copyState":{"!type":"fn(keyPressed: ?) -> ?","!doc":"<p>Returns true, if we need to copy items, false to move.\nIt is separated to be overwritten dynamically, if needed.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Prepares the object to be garbage-collected.</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>Iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>Returns the dojo/dnd/Container._Item (representing a dragged node) by it&#39;s key (id).\nCalled by dojo/dnd/Source.checkAcceptance().</p>\n"},"getSelectedTreeNodes":{"!type":"fn() -> []","!doc":"<p>Returns a list of selected node(s).\nUsed by dndSource on the start of a drag.</p>\n"},"isTreeNodeSelected":{"!type":"fn(node: ?) -> ?","!doc":"<p>return true if node is currently selected</p>\n"},"itemCreator":{"!type":"fn(nodes: ?, target: ?, source: ?) -> [+__Item]","!doc":"<p>Returns objects passed to <code>Tree.model.newItem()</code> based on DnD nodes\ndropped onto the tree.   Developer must override this method to enable\ndropping from external sources onto this Tree, unless the Tree.model&#39;s items\nhappen to look like {id: 123, name: &quot;Apple&quot; } with no other attributes.</p>\n"},"removeTreeNode":{"!type":"fn(node: ?) -> +Node","!doc":"<p>remove node and it&#39;s descendants from current selection</p>\n"},"setSelection":{"!type":"fn(newSelection: ?)","!doc":"<p>set the list of selected nodes to be exactly newSelection. All changes to the\nselection should be passed through this function, which ensures that derived\nattributes are kept up to date. Anchor will be deleted if it has been removed\nfrom the selection, but no new anchor will be added by this function.</p>\n"},"userSelect":{"!type":"fn(node: ?, multi: ?, range: ?)","!doc":"<p>Add or remove the given node from selection, responding\nto a user action such as a click or keypress.</p>\n"}},"__Item":{"!doc":"<p>New item to be added to the Tree, like:</p>\n"}},"dijit/tree/dndSource.__Item":{"!doc":"<p>New item to be added to the Tree, like:</p>\n","id":{},"name":{}},"dijit/dijit-all":{"!doc":"<p>A rollup that includes every dijit. You probably don&#39;t need this.</p>\n"},"dijit/dijit":{"!doc":"<p>A roll-up for common dijit methods\nAll the stuff in _base (these are the function that are guaranteed available without an explicit dojo.require)\nAnd some other stuff that we tend to pull in all the time anyway</p>\n"},"dijit/form/Form":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Widget corresponding to HTML form tag, for validation and serialization\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"accept":{"!doc":"<p>List of MIME types for file upload.</p>\n"},"accept-charset":{},"action":{"!doc":"<p>Server-side form handler.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"encType":{"!doc":"<p>Encoding type for the form, ex: application/x-www-form-urlencoded.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"method":{"!doc":"<p>HTTP method used to submit the form, either &quot;GET&quot; or &quot;POST&quot;.</p>\n"},"name":{"!doc":"<p>Name of form for scripting.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Will be &quot;Error&quot; if one or more of the child widgets has an invalid value,\n&quot;Incomplete&quot; if not all of the required child widgets are filled in.  Otherwise, &quot;&quot;,\nwhich indicates that the form is ready to be submitted.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"target":{"!doc":"<p>Target frame for the document to be opened in.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectChildren":{"!type":"fn(inStartup: ?)","!doc":"<p>You can call this function directly, ex. in the event that you\nprogrammatically add a widget to the form <em>after</em> the form has been\ninitialized.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"disconnectChildren":{"!type":"fn()","!doc":"<p>Deprecated method.   Applications no longer need to call this.   Remove for 2.0.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"execute":{"!type":"fn(formContents: ?)","!doc":"<p>Deprecated: use submit()</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValues":{"!type":"fn() -> ?"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns true if all of the widgets are valid.\nDeprecated, will be removed in 2.0.  Use get(&quot;state&quot;) instead.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn(e?: ?)","!doc":"<p>restores all widget values back to their init values,\ncalls onReset() which can cancel the reset by returning false</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setValues":{"!type":"fn(val: ?) -> ?"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"submit":{"!type":"fn()","!doc":"<p>programmatically submit form if and only if the <code>onSubmit</code> returns true</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?","!doc":"<p>returns if the form is valid - same as isValid - but\nprovides a few additional (ui-specific) features:\n\n</p>\n<ol>\n<li>it will highlight any sub-widgets that are not valid</li>\n<li>it will call focus() on the first invalid sub-widget</li>\n</ol>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/ComboButton":{"!type":"fn()","prototype":{"!proto":"dijit/form/DropDownButton.prototype","!doc":"<p>A combination button and drop-down button.\nUsers can click one side to &quot;press&quot; the button, or click an arrow\nicon to display the drop down.\n</p>\n","_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusedNode":{},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTitleAttr":{},"_setTypeAttr":{},"_setValueAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"optionsTitle":{"!doc":"<p>Text that describes the options menu (accessibility)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn(position: ?)","!doc":"<p>Focuses this widget to according to position, if specified,\notherwise on arrow node</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(callback: ?)"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/CheckBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/ToggleButton.prototype","!doc":"<p>Same as an HTML checkbox, but with fancy styling.\n</p>\n","_aria_attr":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"checked":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.\nIn markup, specified as &quot;checked=&#39;checked&#39;&quot; or just &quot;checked&quot;.\nTrue if the button is depressed, or the checkbox is checked,\nor the radio button is selected, etc.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setChecked":{"!type":"fn(checked: ?)","!doc":"<p>Deprecated.  Use set(&#39;checked&#39;, true/false) instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_CheckBoxMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin to provide widget functionality corresponding to an HTML checkbox\n</p>\n","_aria_attr":{},"_setLabelAttr":{},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"type":{"!doc":"<p>type attribute on <code>&lt;input&gt;</code> node.\nOverrides <code>dijit/form/Button.type</code>.  Users should not change this value.</p>\n"},"value":{"!doc":"<p>As an initialization parameter, equivalent to value field on normal checkbox\n(if checked, the value is passed as the value when form is submitted).</p>\n"},"reset":{"!type":"fn()"}}},"dijit/form/RadioButton":{"!type":"fn()","prototype":{"!proto":"dijit/form/CheckBox.prototype","!doc":"<p>Same as an HTML radio, but with fancy styling.</p>\n","_aria_attr":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"checked":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.\nIn markup, specified as &quot;checked=&#39;checked&#39;&quot; or just &quot;checked&quot;.\nTrue if the button is depressed, or the checkbox is checked,\nor the radio button is selected, etc.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setChecked":{"!type":"fn(checked: ?)","!doc":"<p>Deprecated.  Use set(&#39;checked&#39;, true/false) instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_RadioButtonMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin to provide widget functionality for an HTML radio button</p>\n","type":{"!doc":"<p>type attribute on <code>&lt;input&gt;</code> node.\nUsers should not change this value.</p>\n"}}},"dijit/form/CurrencyTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/NumberTextBox.prototype","!doc":"<p>A validating currency textbox</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraints":{"!doc":"<p>Despite the name, this parameter specifies both constraints on the input\n(including minimum/maximum allowed values) as well as\nformatting options.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"currency":{"!doc":"<p>the <a href=\"http://en.wikipedia.org/wiki/ISO_4217\">ISO4217</a> currency code, a three letter sequence like &quot;USD&quot;</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editOptions":{"!doc":"<p>Properties to mix into constraints when the value is being edited.\nThis is here because we edit the number in the format &quot;12345&quot;, which is\ndifferent than the display value (ex: &quot;12,345&quot;)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Parses string value as a Currency, according to the constraints object</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> string","!doc":"<p>Overridable function used to convert the get(&#39;value&#39;) result to a canonical\n(non-localized) string.  For example, will print dates in ISO format, and\nnumbers the same way as they are represented in javascript.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/NumberTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/RangeBoundTextBox.prototype","!doc":"<p>A TextBox for entering numbers, with formatting and range checking</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editOptions":{"!doc":"<p>Properties to mix into constraints when the value is being edited.\nThis is here because we edit the number in the format &quot;12345&quot;, which is\ndifferent than the display value (ex: &quot;12,345&quot;)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"isValid":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> string","!doc":"<p>Overridable function used to convert the get(&#39;value&#39;) result to a canonical\n(non-localized) string.  For example, will print dates in ISO format, and\nnumbers the same way as they are represented in javascript.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"Mixin":{"!type":"fn()"}},"dijit/form/NumberTextBox.Mixin":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for all number textboxes</p>\n","constraints":{"!doc":"<p>Despite the name, this parameter specifies both constraints on the input\n(including minimum/maximum allowed values) as well as\nformatting options like places (the number of digits to display after\nthe decimal point).</p>\n"},"editOptions":{"!doc":"<p>Properties to mix into constraints when the value is being edited.\nThis is here because we edit the number in the format &quot;12345&quot;, which is\ndifferent than the display value (ex: &quot;12,345&quot;)</p>\n"},"value":{"!doc":"<p>The value of this NumberTextBox as a Javascript Number (i.e., not a String).\nIf the displayed value is blank, the value is NaN, and if the user types in\nan gibberish value (like &quot;hello world&quot;), the value is undefined\n(i.e. get(&#39;value&#39;) returns undefined).\n\n</p>\n<p>Symmetrically, set(&#39;value&#39;, NaN) will clear the displayed value,\nwhereas set(&#39;value&#39;, undefined) will have no effect.</p>\n"},"filter":{"!type":"fn(value: ?) -> number","!doc":"<p>This is called with both the display value (string), and the actual value (a number).\nWhen called with the actual value it does corrections so that &#39;&#39; etc. are represented as NaN.\nOtherwise it dispatches to the superclass&#39;s filter() method.\n\n</p>\n<p>See <code>dijit/form/TextBox.filter()</code> for more details.</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Formats the value as a Number, according to constraints.</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"parse":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Replaceable function to convert a formatted string to a number value</p>\n"},"pattern":{"!type":"fn(constraints: ?) -> string"},"postMixInProperties":{"!type":"fn()"},"serialize":{"!type":"fn(value: ?, options?: ?) -> string","!doc":"<p>Convert value (a Number) into a canonical string (ie, how the number literal is written in javascript/java/C/etc.)</p>\n"}}},"dijit/form/NumberTextBox.__Constraints":{"!type":"fn()","prototype":{"!doc":"<p>Specifies both the rules on valid/invalid values (minimum, maximum,\nnumber of required decimal places), and also formatting options for\ndisplaying the value when the field is not focused.</p>\n","fractional":{"!doc":"<p>If false, show no decimal places, overriding places and pattern settings.</p>\n"},"locale":{"!doc":"<p>override the locale used to determine formatting rules</p>\n"},"max":{"!doc":"<p>Maximum signed value.  Default is +Infinity</p>\n"},"min":{"!doc":"<p>Minimum signed value.  Default is -Infinity</p>\n"},"pattern":{"!doc":"<p>override <a href=\"http://www.unicode.org/reports/tr35/#Number_Format_Patterns\">formatting pattern</a>\nwith this string.  Default value is based on locale.  Overriding this property will defeat\nlocalization.  Literal characters in patterns are not supported.</p>\n"},"places":{"!doc":"<p>fixed number of decimal places to show.  This overrides any\ninformation in the provided pattern.</p>\n"},"round":{"!doc":"<p>5 rounds to nearest .5; 0 rounds to nearest whole (default). -1\nmeans do not round.</p>\n"},"strict":{"!doc":"<p>strict parsing, false by default.  Strict parsing requires input as produced by the format() method.\nNon-strict is more permissive, e.g. flexible on white space, omitting thousands separators</p>\n"},"type":{"!doc":"<p>choose a format type based on the locale from the following:\ndecimal, scientific (not yet supported), percent, currency. decimal by default.</p>\n"}}},"dijit/form/RangeBoundTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/MappedTextBox.prototype","!doc":"<p>Base class for textbox form widgets which defines a range of valid values.</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> string","!doc":"<p>Overridable function used to convert the get(&#39;value&#39;) result to a canonical\n(non-localized) string.  For example, will print dates in ISO format, and\nnumbers the same way as they are represented in javascript.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/RangeBoundTextBox.__Constraints":{"!type":"fn()","prototype":{"max":{"!doc":"<p>Maximum signed value.  Default is +Infinity</p>\n"},"min":{"!doc":"<p>Minimum signed value.  Default is -Infinity</p>\n"}}},"dijit/form/DateTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/_DateTimeTextBox.prototype","!doc":"<p>A validating, serializable, range-bound date text box with a drop down calendar</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selector":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{},"baseClass":{},"class":{},"constraints":{"!doc":"<p>Despite the name, this parameter specifies both constraints on the input\n(including starting/ending dates/times allowed) as well as\nformatting options like whether the date is displayed in long (ex: December 25, 2005)\nor short (ex: 12/25/2005) format.  See <code>dijit/form/_DateTimeTextBox.__Constraints</code> for details.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownDefaultValue":{"!doc":"<p>The default value to focus in the popupClass widget when the textbox value is empty.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to display a down arrow button, to open the drop down list.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>The value of this widget as a JavaScript Date object, with only year/month/day specified.\nIf specified in markup, use the format specified in <code>stamp.fromISOString</code>.\nset(&quot;value&quot;, ...) accepts either a Date object or a string.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Formats the value as a Date, according to specified locale (second argument)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn(callback: ?)"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Parses as string as a Date, according to constraints</p>\n"},"pattern":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a localized date</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"popupClass":{"!type":"fn()"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> ?"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_DateTimeTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/RangeBoundTextBox.prototype","!doc":"<p>Base class for validating, serializable, range-bound date or time text box.</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selector":{"!doc":"<p>Specifies constraints.selector passed to dojo.date functions, should be either\n&quot;date&quot; or &quot;time&quot;.\nSubclass must specify this.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{},"baseClass":{},"class":{},"constraints":{"!doc":"<p>Despite the name, this parameter specifies both constraints on the input\n(including starting/ending dates/times allowed) as well as\nformatting options like whether the date is displayed in long (ex: December 25, 2005)\nor short (ex: 12/25/2005) format.  See <code>dijit/form/_DateTimeTextBox.__Constraints</code> for details.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownDefaultValue":{"!doc":"<p>The default value to focus in the popupClass widget when the textbox value is empty.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to display a down arrow button, to open the drop down list.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"popupClass":{"!doc":"<p>Name of the popup widget class used to select a date/time.\nSubclasses should specify this.</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>The value of this widget as a JavaScript Date object.  Use get(&quot;value&quot;) / set(&quot;value&quot;, val) to manipulate.\nWhen passed to the parser in markup, must be specified according to <code>dojo/date/stamp.fromISOString()</code></p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Formats the value as a Date, according to specified locale (second argument)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn(callback: ?)"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Parses as string as a Date, according to constraints</p>\n"},"pattern":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a localized date</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> ?"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_DateTimeTextBox.__Constraints":{"!type":"fn()","prototype":{"!doc":"<p>Specifies both the rules on valid/invalid values (first/last date/time allowed),\nand also formatting options for how the date/time is displayed.</p>\n","am":{"!doc":"<p>override strings for am in times</p>\n"},"datePattern":{"!doc":"<p>override pattern with this string</p>\n"},"formatLength":{"!doc":"<p>choice of long, short, medium or full (plus any custom additions).  Defaults to &#39;short&#39;</p>\n"},"fullYear":{"!doc":"<p>(format only) use 4 digit years whenever 2 digit years are called for</p>\n"},"locale":{"!doc":"<p>override the locale used to determine formatting rules</p>\n"},"max":{"!doc":"<p>Maximum signed value.  Default is +Infinity</p>\n"},"min":{"!doc":"<p>Minimum signed value.  Default is -Infinity</p>\n"},"pm":{"!doc":"<p>override strings for pm in times</p>\n"},"selector":{"!doc":"<p>choice of &#39;time&#39;,&#39;date&#39; (default: date and time)</p>\n"},"strict":{"!doc":"<p>(parse only) strict parsing, off by default</p>\n"},"timePattern":{"!doc":"<p>override pattern with this string</p>\n"}}},"dijit/form/TimeTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/_DateTimeTextBox.prototype","!doc":"<p>A validating, serializable, range-bound time text box with a drop down time picker</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selector":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{},"baseClass":{},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownDefaultValue":{"!doc":"<p>The default value to focus in the popupClass widget when the textbox value is empty.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to display a down arrow button, to open the drop down list.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>The value of this widget as a JavaScript Date object.  Note that the date portion implies time zone and daylight savings rules.\n\n</p>\n<p>Example:\n\n</p>\n<pre><code>new dijit/form/TimeTextBox({value: stamp.fromISOString(&quot;T12:59:59&quot;, new Date())})</code></pre>\n<p>When passed to the parser in markup, must be specified according to locale-independent\n<code>stamp.fromISOString</code> format.\n\n</p>\n<p>Example:\n\n</p>\n<pre><code>&lt;input data-dojo-type=&#39;dijit/form/TimeTextBox&#39; value=&#39;T12:34:00&#39;&gt;</code></pre>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Formats the value as a Date, according to specified locale (second argument)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn(callback: ?)"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Parses as string as a Date, according to constraints</p>\n"},"pattern":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a localized date</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"popupClass":{"!type":"fn()"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> ?"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/NumberSpinner":{"!type":"fn()","prototype":{"!proto":"dijit/form/_Spinner.prototype","!doc":"<p>Extends NumberTextBox to add up/down arrows and pageup/pagedown for incremental change to the value\n</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_wheelTimer":{},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"defaultTimeout":{"!doc":"<p>Number of milliseconds before a held arrow key or up/down button becomes typematic</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editOptions":{"!doc":"<p>Properties to mix into constraints when the value is being edited.\nThis is here because we edit the number in the format &quot;12345&quot;, which is\ndifferent than the display value (ex: &quot;12,345&quot;)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"largeDelta":{"!doc":"<p>Adjust the value by this much when spinning using the PgUp/Dn keys</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"minimumTimeout":{"!doc":"<p>minimum number of milliseconds that typematic event fires when held key or button is held</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"smallDelta":{"!doc":"<p>Adjust the value by this much when spinning using the arrow keys/buttons</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"timeoutChangeRate":{"!doc":"<p>Fraction of time used to change the typematic timer between events.\n1.0 means that each typematic event fires at defaultTimeout intervals.\nLess than 1.0 means that each typematic event fires at an increasing faster rate.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"adjust":{"!type":"fn(val: ?, delta: ?) -> ?","!doc":"<p>Change Number val by the given amount</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> string","!doc":"<p>Overridable function used to convert the get(&#39;value&#39;) result to a canonical\n(non-localized) string.  For example, will print dates in ISO format, and\nnumbers the same way as they are represented in javascript.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_Spinner":{"!type":"fn()","prototype":{"!proto":"dijit/form/RangeBoundTextBox.prototype","!doc":"<p>Mixin for validation widgets with a spinner.</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_wheelTimer":{},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"defaultTimeout":{"!doc":"<p>Number of milliseconds before a held arrow key or up/down button becomes typematic</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"largeDelta":{"!doc":"<p>Adjust the value by this much when spinning using the PgUp/Dn keys</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"minimumTimeout":{"!doc":"<p>minimum number of milliseconds that typematic event fires when held key or button is held</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"smallDelta":{"!doc":"<p>Adjust the value by this much when spinning using the arrow keys/buttons</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"timeoutChangeRate":{"!doc":"<p>Fraction of time used to change the typematic timer between events.\n1.0 means that each typematic event fires at defaultTimeout intervals.\nLess than 1.0 means that each typematic event fires at an increasing faster rate.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"adjust":{"!type":"fn(val: ?, delta: ?) -> ?","!doc":"<p>Overridable function used to adjust a primitive value(Number/Date/...) by the delta amount specified.\nThe val is adjusted in a way that makes sense to the object type.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> string","!doc":"<p>Overridable function used to convert the get(&#39;value&#39;) result to a canonical\n(non-localized) string.  For example, will print dates in ISO format, and\nnumbers the same way as they are represented in javascript.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/ComboBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/ValidationTextBox.prototype","!doc":"<p>Auto-completing text box\n</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoComplete":{"!doc":"<p>If user types in a partial string, and then tab out of the <code>&lt;input&gt;</code> box,\nautomatically copy the first entry displayed in the drop down list to\nthe <code>&lt;input&gt;</code> field</p>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"class":{},"constraints":{"!doc":"<p>user-defined object needed to pass parameters to the validator functions</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"fetchProperties":{"!doc":"<p>Mixin to the store&#39;s fetch.\nFor example, to set the sort order of the ComboBox menu, pass:\n\n</p>\n<pre><code>{ sort: [{attribute:&quot;name&quot;,descending: true}] }</code></pre>\n<p>To override the default queryOptions so that deep=false, do:\n\n\n</p>\n<pre><code>{ queryOptions: {ignoreCase: true, deep: false} }</code></pre>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to have a down arrow button, to display the drop down list.\nDefaults to true.</p>\n"},"highlightMatch":{"!doc":"<p>One of: &quot;first&quot;, &quot;all&quot; or &quot;none&quot;.\n\n</p>\n<p>If the ComboBox/FilteringSelect opens with the search results and the searched\nstring can be found, it will be highlighted.  If set to &quot;all&quot;\nthen will probably want to change <code>queryExpr</code> parameter to &#39;<em>${0}</em>&#39;\n\n</p>\n<p>Highlighting is only performed when <code>labelType</code> is &quot;text&quot;, so as to not\ninterfere with any HTML markup an HTML label might contain.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ignoreCase":{"!doc":"<p>Set true if the query should ignore case when matching possible items</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"item":{"!doc":"<p>This is the item returned by the dojo/store/api/Store implementation that\nprovides the data for this ComboBox, it&#39;s the currently selected item.</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the\ndojo.data items.\nIf not specified, the searchAttr attribute is used instead.</p>\n"},"labelType":{"!doc":"<p>Specifies how to interpret the labelAttr in the data store items.\nCan be &quot;html&quot; or &quot;text&quot;.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageSize":{"!doc":"<p>Argument to data provider.\nSpecifies maximum number of search results to return per query</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"query":{"!doc":"<p>A query that can be passed to <code>store</code> to initially filter the items.\nComboBox overwrites any reference to the <code>searchAttr</code> and sets it to the <code>queryExpr</code> with the user&#39;s input substituted.</p>\n"},"queryExpr":{"!doc":"<p>This specifies what query is sent to the data store,\nbased on what the user has typed.  Changing this expression will modify\nwhether the results are only exact matches, a &quot;starting with&quot; match,\netc.\n<code>${0}</code> will be substituted for the user text.\n<code>*</code> is used for wildcards.\n<code>${0}*</code> means &quot;starts with&quot;, <code>*${0}*</code> means &quot;contains&quot;, <code>${0}</code> means &quot;is&quot;</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchAttr":{"!doc":"<p>Search for items in the data store where this attribute (in the item)\nmatches what the user typed</p>\n"},"searchContainerNode":{},"searchDelay":{"!doc":"<p>Delay in milliseconds between when user types something and we start\nsearching based on that value</p>\n"},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this ComboBox.\n\n</p>\n<p>Should be dojo/store/api/Store, but dojo/data/api/Read supported\nfor backwards compatibility.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"doHighlight":{"!type":"fn(label: ?, find: ?) -> ?","!doc":"<p>Highlights the string entered by the user in the menu.  By default this\nhighlights the first occurrence found. Override this method\nto implement your custom highlighting.</p>\n"},"dropDownClass":{"!type":"fn()","!doc":"<p>Dropdown widget class used to select a date/time.\nSubclasses should specify this.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return an error message to show if appropriate</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if value is valid.\nCan override with your own routine in a subclass.</p>\n"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Called by oninit, onblur, and onkeypress.</p>\n"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/MultiSelect":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormValueWidget.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiple":{},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"size":{"!doc":"<p>Number of elements to display on a page\nNOTE: may be removed in version 2.0, since elements may have variable height;\nset the size via style=&quot;...&quot; or CSS class names instead.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"addSelected":{"!type":"fn(select: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getSelected":{"!type":"fn() -> ?","!doc":"<p>Access the NodeList of the selected options directly</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"invertSelection":{"!type":"fn(onChange?: ?)","!doc":"<p>Invert the selection</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"resize":{"!type":"fn(size: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/Select":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormSelectWidget.prototype","_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_childrenLoaded":{"!doc":"<p>Whether or not our children have been loaded</p>\n"},"_focusManager":{},"_isLoaded":{"!doc":"<p>Whether or not we have been loaded</p>\n"},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_onContainerKeydown":{},"_onContainerKeypress":{},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"emptyLabel":{"!doc":"<p>What to display in an &quot;empty&quot; dropdown</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the dojo.store items.\nIf <code>store</code> is set, labelAttr must be set too, unless store is an old-style\ndojo.data store rather than a new dojo/store.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadChildrenOnOpen":{"!doc":"<p>By default loadChildren is called when the items are fetched from the\nstore.  This property allows delaying loadChildren (and the creation\nof the options/menuitems) until the user clicks the button to open the\ndropdown.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"message":{"!doc":"<p>Currently displayed error/prompt message</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"multiple":{"!doc":"<p>Whether or not we are multi-valued</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"onFetch":{"!doc":"<p>A callback to do with an onFetch - but before any items are actually\niterated over (i.e. to filter even further what you want to add)</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by setStore().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior setStore completes.</p>\n"},"options":{"!doc":"<p>The set of options for our select item.  Roughly corresponds to\nthe html <code>&lt;option&gt;</code> tag.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query to use when fetching items from our store</p>\n"},"queryOptions":{"!doc":"<p>Query options to use when fetching from the store</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"required":{"!doc":"<p>Can be true or false, default is false.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"sortByLabel":{"!doc":"<p>Flag to sort the options returned from a store by the label of\nthe store.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>&quot;Incomplete&quot; if this select is required but unset (i.e. blank value), &quot;&quot; otherwise</p>\n"},"store":{"!doc":"<p>A store to use for getting our list of options - rather than reading them\nfrom the <code>&lt;option&gt;</code> html tags.   Should support getIdentity().\nFor back-compat store can also be a dojo/data/api/Identity.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"addOption":{"!type":"fn(option: ?)","!doc":"<p>Adds an option or options to the end of the select.  If value\nof the option is empty or missing, a separator is created instead.\nPassing in an array of options will yield slightly better performance\nsince the children are only loaded once.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"childSelector":{"!type":"fn(node: ?) -> ?"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()"},"focusChild":{"!type":"fn(widget: ?)","!doc":"<p>Sets the value to the given option, used during search by letter.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getOptions":{"!type":"fn(valueOrIdx: ?) -> +any","!doc":"<p>Returns a given option (or options).</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> ?"},"isValid":{"!type":"fn(isFocused: ?) -> bool","!doc":"<p>Whether or not this is a valid value.  The only way a Select\ncan be invalid is when it&#39;s required but nothing is selected.</p>\n"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>populates the menu</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>stop mousemove from selecting text on IE to be consistent with other browsers</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>set the missing message</p>\n"},"removeOption":{"!type":"fn(valueOrIdx: ?)","!doc":"<p>Removes the given option or options.  You can remove by string\n(in which case the value is removed), number (in which case the\nindex in the options array is removed), or select option (in\nwhich case, the select option with a matching value is removed).\nYou can also pass in an array of those values for a slightly\nbetter performance since the children are only loaded once.\nFor numeric option values, specify {value: number} as the argument.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Overridden so that the state will be cleared.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setStore":{"!type":"fn(store: ?, selectedValue?: ?, fetchArgs?: ?) -> ?","!doc":"<p>Sets the store you would like to use with this select widget.\nThe selected value is the value of the new store to set.  This\nfunction returns the original store, in case you want to reuse\nit or something.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateOption":{"!type":"fn(newOption: ?)","!doc":"<p>Updates the values of the given option.  The option to update\nis matched based on the value of the entered option.  Passing\nin an array of new options will yield better performance since\nthe children will only be loaded once.</p>\n"},"validate":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Called by oninit, onblur, and onkeypress, and whenever required/disabled state changes</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/Select._Menu":{"!type":"fn()","prototype":{"!proto":"dijit/DropDownMenu.prototype","!doc":"<p>An internally-used menu for dropdown that allows us a vertical scrollbar</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"activated":{"!doc":"<p>This Menu has been clicked (mouse or via space/arrow key) or opened as a submenu,\nso mere mouseover will open submenus.  Focusing a menu via TAB does NOT automatically make it active\nsince TAB is a navigation operation and not a selection one.\nFor Windows apps, pressing the ALT key focuses the menubar menus (similar to TAB navigation) but the\nmenu is not active (ie no dropdown) until an item is clicked.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoFocus":{},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parentMenu":{"!doc":"<p>pointer to menu that displayed me</p>\n"},"passivePopupDelay":{"!doc":"<p>For a passive (unclicked) Menu, number of milliseconds before hovering (without clicking) will cause\nthe popup to open.  Default is Infinity, meaning you need to click the menu to open it.</p>\n"},"popupDelay":{"!doc":"<p>After a menu has been activated (by clicking on it etc.), number of milliseconds before hovering\n(without clicking) another MenuItem causes that MenuItem&#39;s popup to automatically open.</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>Currently selected (a.k.a. highlighted) MenuItem, or null if no MenuItem is selected.\nIf a submenu is open, will be set to MenuItem that displayed the submenu.   OTOH, if\nthis Menu is in passive mode (i.e. hasn&#39;t been clicked yet), will be null, because\n&quot;selected&quot; is not merely &quot;hovered&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"childSelector":{"!type":"fn(node: ?) -> ?","!doc":"<p>Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children\nlike MenuSeparator.  If subclass overrides to a string (ex: &quot;&gt; *&quot;), the subclass must require dojo/query.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Overridden so that the previously selected value will be focused instead of only the first item</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>stop mousemove from selecting text on IE to be consistent with other browsers</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"startupKeyNavChildren":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_FormSelectWidget":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormValueWidget.prototype","!doc":"<p>Extends _FormValueWidget in order to provide &quot;select-specific&quot;\nvalues - i.e., those values that are unique to <code>&lt;select&gt;</code> elements.\nThis also provides the mechanism for reading the elements from\na store, if desired.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the dojo.store items.\nIf <code>store</code> is set, labelAttr must be set too, unless store is an old-style\ndojo.data store rather than a new dojo/store.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadChildrenOnOpen":{"!doc":"<p>By default loadChildren is called when the items are fetched from the\nstore.  This property allows delaying loadChildren (and the creation\nof the options/menuitems) until the user clicks the button to open the\ndropdown.</p>\n"},"multiple":{"!doc":"<p>Whether or not we are multi-valued</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"onFetch":{"!doc":"<p>A callback to do with an onFetch - but before any items are actually\niterated over (i.e. to filter even further what you want to add)</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by setStore().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior setStore completes.</p>\n"},"options":{"!doc":"<p>The set of options for our select item.  Roughly corresponds to\nthe html <code>&lt;option&gt;</code> tag.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query to use when fetching items from our store</p>\n"},"queryOptions":{"!doc":"<p>Query options to use when fetching from the store</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"sortByLabel":{"!doc":"<p>Flag to sort the options returned from a store by the label of\nthe store.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>A store to use for getting our list of options - rather than reading them\nfrom the <code>&lt;option&gt;</code> html tags.   Should support getIdentity().\nFor back-compat store can also be a dojo/data/api/Identity.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"addOption":{"!type":"fn(option: ?)","!doc":"<p>Adds an option or options to the end of the select.  If value\nof the option is empty or missing, a separator is created instead.\nPassing in an array of options will yield slightly better performance\nsince the children are only loaded once.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up our connections</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getOptions":{"!type":"fn(valueOrIdx: ?) -> +any","!doc":"<p>Returns a given option (or options).</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>sets up our event handling that we need for functioning\nas a select</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeOption":{"!type":"fn(valueOrIdx: ?)","!doc":"<p>Removes the given option or options.  You can remove by string\n(in which case the value is removed), number (in which case the\nindex in the options array is removed), or select option (in\nwhich case, the select option with a matching value is removed).\nYou can also pass in an array of those values for a slightly\nbetter performance since the children are only loaded once.\nFor numeric option values, specify {value: number} as the argument.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setStore":{"!type":"fn(store: ?, selectedValue?: ?, fetchArgs?: ?) -> ?","!doc":"<p>Sets the store you would like to use with this select widget.\nThe selected value is the value of the new store to set.  This\nfunction returns the original store, in case you want to reuse\nit or something.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateOption":{"!type":"fn(newOption: ?)","!doc":"<p>Updates the values of the given option.  The option to update\nis matched based on the value of the entered option.  Passing\nin an array of new options will yield better performance since\nthe children will only be loaded once.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"__SelectOption":{}},"dijit/form/_FormSelectWidget.__SelectOption":{"disabled":{"!doc":"<p>Whether or not this specific option is disabled</p>\n"},"label":{"!doc":"<p>The label for our option.  It can contain html tags.</p>\n"},"selected":{"!doc":"<p>Whether or not we are a selected option</p>\n"},"value":{"!doc":"<p>The value of the option.  Setting to empty (or missing) will\nplace a separator at that location</p>\n"}},"dijit/form/HorizontalSlider":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormValueWidget.prototype","!doc":"<p>A form widget that allows one to select a value with a horizontally draggable handle</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_handleOffsetCoord":{},"_mousePixelCoord":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_pixelCount":{},"_progressPixelSize":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_startingPixelCoord":{},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"clickSelect":{"!doc":"<p>If clicking the slider bar changes the value or not</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"discreteValues":{"!doc":"<p>If specified, indicates that the slider handle has only &#39;discreteValues&#39; possible positions,\nand that after dragging the handle, it will snap to the nearest possible position.\nThus, the slider has only &#39;discreteValues&#39; possible values.\n\n</p>\n<p>For example, if minimum=10, maxiumum=30, and discreteValues=3, then the slider handle has\nthree possible positions, representing values 10, 20, or 30.\n\n</p>\n<p>If discreteValues is not specified or if it&#39;s value is higher than the number of pixels\nin the slider bar, then the slider handle can be moved freely, and the slider&#39;s value will be\ncomputed/reported based on pixel position (in this case it will likely be fractional,\nsuch as 123.456789).</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maximum":{"!doc":"<p>The maximum value the slider can be set to.</p>\n"},"minimum":{"!doc":"<p>The minimum value the slider can be set to.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageIncrement":{"!doc":"<p>If discreteValues is also specified, this indicates the amount of clicks (ie, snap positions)\nthat the slider handle is moved via pageup/pagedown keys.\nIf discreteValues is not specified, it indicates the number of pixels.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showButtons":{"!doc":"<p>Show increment/decrement buttons at the ends of the slider?</p>\n"},"slideDuration":{"!doc":"<p>The time in ms to take to animate the slider handle from 0% to 100%,\nwhen clicking the slider bar to make the handle move.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/HorizontalSlider._Mover":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Mover.prototype","constructor":{"!type":"fn(node: ?, e: ?, host?: ?)"},"destroy":{"!type":"fn(e: ?)"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dijit/form/VerticalSlider":{"!type":"fn()","prototype":{"!proto":"dijit/form/HorizontalSlider.prototype","!doc":"<p>A form widget that allows one to select a value with a vertically draggable handle</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_descending":{"!doc":"<p>Specifies if the slider values go from high-on-top (true), or low-on-top (false)\nTODO: expose this in 1.2 - the css progress/remaining bar classes need to be reversed</p>\n"},"_focusManager":{},"_handleOffsetCoord":{},"_mousePixelCoord":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_pixelCount":{},"_progressPixelSize":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_startingPixelCoord":{},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"clickSelect":{"!doc":"<p>If clicking the slider bar changes the value or not</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"discreteValues":{"!doc":"<p>If specified, indicates that the slider handle has only &#39;discreteValues&#39; possible positions,\nand that after dragging the handle, it will snap to the nearest possible position.\nThus, the slider has only &#39;discreteValues&#39; possible values.\n\n</p>\n<p>For example, if minimum=10, maxiumum=30, and discreteValues=3, then the slider handle has\nthree possible positions, representing values 10, 20, or 30.\n\n</p>\n<p>If discreteValues is not specified or if it&#39;s value is higher than the number of pixels\nin the slider bar, then the slider handle can be moved freely, and the slider&#39;s value will be\ncomputed/reported based on pixel position (in this case it will likely be fractional,\nsuch as 123.456789).</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maximum":{"!doc":"<p>The maximum value the slider can be set to.</p>\n"},"minimum":{"!doc":"<p>The minimum value the slider can be set to.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageIncrement":{"!doc":"<p>If discreteValues is also specified, this indicates the amount of clicks (ie, snap positions)\nthat the slider handle is moved via pageup/pagedown keys.\nIf discreteValues is not specified, it indicates the number of pixels.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showButtons":{"!doc":"<p>Show increment/decrement buttons at the ends of the slider?</p>\n"},"slideDuration":{"!doc":"<p>The time in ms to take to animate the slider handle from 0% to 100%,\nwhen clicking the slider bar to make the handle move.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/HorizontalRule":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Hash marks for <code>dijit/form/HorizontalSlider</code></p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_isHorizontal":{"!doc":"<p>VerticalRule will override this...</p>\n"},"_positionPrefix":{},"_positionSuffix":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_suffix":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"container":{"!doc":"<p>For HorizontalSlider, this is either &quot;topDecoration&quot; or &quot;bottomDecoration&quot;,\nand indicates whether this rule goes above or below the slider.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"count":{"!doc":"<p>Number of hash marks to generate</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"ruleStyle":{"!doc":"<p>CSS style to apply to individual hash marks</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/VerticalRule":{"!type":"fn()","prototype":{"!proto":"dijit/form/HorizontalRule.prototype","!doc":"<p>Hash marks for the <code>dijit/form/VerticalSlider</code></p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_isHorizontal":{},"_positionPrefix":{},"_positionSuffix":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_suffix":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"container":{"!doc":"<p>This is either &quot;leftDecoration&quot; or &quot;rightDecoration&quot;,\nto indicate whether this rule goes to the left or to the right of the slider.\nNote that on RTL system, &quot;leftDecoration&quot; would actually go to the right, and vice-versa.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"count":{"!doc":"<p>Number of hash marks to generate</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"ruleStyle":{"!doc":"<p>CSS style to apply to individual hash marks</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/HorizontalRuleLabels":{"!type":"fn()","prototype":{"!proto":"dijit/form/HorizontalRule.prototype","!doc":"<p>Labels for <code>dijit/form/HorizontalSlider</code></p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_isHorizontal":{"!doc":"<p>VerticalRule will override this...</p>\n"},"_labelPrefix":{},"_positionPrefix":{},"_positionSuffix":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_suffix":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{"!doc":"<p>pattern, places, lang, et al (see dojo.number) for generated numeric labels when labels[] are not specified</p>\n"},"container":{"!doc":"<p>For HorizontalSlider, this is either &quot;topDecoration&quot; or &quot;bottomDecoration&quot;,\nand indicates whether this rule goes above or below the slider.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"count":{"!doc":"<p>Number of hash marks to generate</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"labels":{"!doc":"<p>Array of text labels to render - evenly spaced from left-to-right or bottom-to-top.\nAlternately, minimum and maximum can be specified, to get numeric labels.</p>\n"},"labelStyle":{"!doc":"<p>CSS style to apply to individual text labels</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maximum":{},"minimum":{},"numericMargin":{"!doc":"<p>Number of generated numeric labels that should be rendered as &#39;&#39; on the ends when labels[] are not specified</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"ruleStyle":{"!doc":"<p>CSS style to apply to individual hash marks</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getLabels":{"!type":"fn() -> ?","!doc":"<p>Overridable function to return array of labels to use for this slider.\nCan specify a getLabels() method instead of a labels[] array, or min/max attributes.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/VerticalRuleLabels":{"!type":"fn()","prototype":{"!proto":"dijit/form/HorizontalRuleLabels.prototype","!doc":"<p>Labels for the <code>dijit/form/VerticalSlider</code></p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_isHorizontal":{},"_labelPrefix":{},"_positionPrefix":{},"_positionSuffix":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_suffix":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{"!doc":"<p>pattern, places, lang, et al (see dojo.number) for generated numeric labels when labels[] are not specified</p>\n"},"container":{"!doc":"<p>For HorizontalSlider, this is either &quot;topDecoration&quot; or &quot;bottomDecoration&quot;,\nand indicates whether this rule goes above or below the slider.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"count":{"!doc":"<p>Number of hash marks to generate</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"labels":{"!doc":"<p>Array of text labels to render - evenly spaced from left-to-right or bottom-to-top.\nAlternately, minimum and maximum can be specified, to get numeric labels.</p>\n"},"labelStyle":{"!doc":"<p>CSS style to apply to individual text labels</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maximum":{},"minimum":{},"numericMargin":{"!doc":"<p>Number of generated numeric labels that should be rendered as &#39;&#39; on the ends when labels[] are not specified</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"ruleStyle":{"!doc":"<p>CSS style to apply to individual hash marks</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getLabels":{"!type":"fn() -> ?","!doc":"<p>Overridable function to return array of labels to use for this slider.\nCan specify a getLabels() method instead of a labels[] array, or min/max attributes.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/SimpleTextarea":{"!type":"fn()","prototype":{"!proto":"dijit/form/TextBox.prototype","!doc":"<p>A simple textarea that degrades, and responds to\nminimal LayoutContainer usage, and works with dijit/form/Form.\nDoesn&#39;t automatically size according to input, like Textarea.\n</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"cols":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"rows":{"!doc":"<p>The number of rows of text.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(value: ?) -> ?"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/Textarea":{"!type":"fn()","prototype":{"!proto":"dijit/form/SimpleTextarea.prototype","!doc":"<p>A textarea widget that adjusts it&#39;s height according to the amount of data.\n</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"cols":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"rows":{"!doc":"<p>The number of rows of text.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(value: ?) -> ?"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Resizes the textarea vertically (should be called after a style/value change)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/_ExpandingTextAreaMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for textarea widgets to add auto-expanding capability</p>\n","postCreate":{"!type":"fn()"},"resize":{"!type":"fn()","!doc":"<p>Resizes the textarea vertically (should be called after a style/value change)</p>\n"},"startup":{"!type":"fn()"}}},"dijit/layout/AccordionContainer":{"!type":"fn()","prototype":{"!proto":"dijit/layout/StackContainer.prototype","!doc":"<p>Holds a set of panes where every pane&#39;s title is visible, but only one pane&#39;s content is visible at a time,\nand switching between panes is visualized by sliding the other panes up/down.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_verticalSpace":{"!doc":"<p>Pixels of space available for the open pane\n(my content box size minus the cumulative size of all the title bars)</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>If true, change the size of my currently displayed child to match my size</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>Amount of time (in ms) it takes to slide panes</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"persist":{"!doc":"<p>Remembers the selected child across sessions</p>\n"},"selectedChildWidget":{"!doc":"<p>References the currently selected child widget, if any.\nAdjust selected child with selectChild() method.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"back":{"!type":"fn() -> ?","!doc":"<p>Go back to previous page.</p>\n"},"buildRendering":{"!type":"fn()"},"buttonWidget":{"!type":"fn()","!doc":"<p>The name of the widget used to display the title of each pane</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"forward":{"!type":"fn() -> ?","!doc":"<p>Advance to next page.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> ?"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(child: ?)"},"resize":{"!type":"fn()"},"selectChild":{"!type":"fn(page: ?, animate: ?) -> ?","!doc":"<p>Show the given widget (which must be one of my children)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/AccordionContainer._InnerContainer":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"buttonWidget":{"!doc":"<p>Class to use to instantiate title\n(Wish we didn&#39;t have a separate widget for just the title but maintaining it\nfor backwards compatibility, is it worth it?)</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contentWidget":{"!doc":"<p>Pointer to the real child widget</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/AccordionContainer._Button":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>The title bar to click to open up an accordion pane.\nInternal widget used by AccordionContainer.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLabelAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClassAttr":{"!doc":"<p>CSS class for icon to left of label</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>Title of the pane</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>Tooltip that appears on hover</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getTitleHeight":{"!type":"fn() -> ?","!doc":"<p>Returns the height of the title dom node.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/StackContainer":{"!type":"fn()","prototype":{"!proto":"dijit/layout/_LayoutWidget.prototype","!doc":"<p>A container that has multiple children, but shows only\none child at a time\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>If true, change the size of my currently displayed child to match my size</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"persist":{"!doc":"<p>Remembers the selected child across sessions</p>\n"},"selectedChildWidget":{"!doc":"<p>References the currently selected child widget, if any.\nAdjust selected child with selectChild() method.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"back":{"!type":"fn() -> ?","!doc":"<p>Go back to previous page.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"forward":{"!type":"fn() -> ?","!doc":"<p>Advance to next page.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(page: ?)"},"resize":{"!type":"fn()"},"selectChild":{"!type":"fn(page: ?, animate: ?) -> ?","!doc":"<p>Show the given widget (which must be one of my children)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a StackContainer.</p>\n"}},"dijit/layout/StackContainer.ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a StackContainer.</p>\n","closable":{"!doc":"<p>True if user can close (destroy) this child, such as (for example) clicking the X on the tab.</p>\n"},"disabled":{"!doc":"<p>Specifies that the button to select this pane should be disabled.\nDoesn&#39;t affect programmatic selection of the pane, nor does it deselect the pane if it is currently selected.</p>\n"},"iconClass":{"!doc":"<p>CSS Class specifying icon to use in label associated with this pane.</p>\n"},"selected":{"!doc":"<p>Specifies that this widget should be the initially displayed pane.\nNote: to change the selected child use <code>dijit/layout/StackContainer.selectChild</code></p>\n"},"showTitle":{"!doc":"<p>When true, display title of this widget as tab label etc., rather than just using\nicon specified in iconClass</p>\n"}},"dijit/layout/BorderContainer":{"!type":"fn()","prototype":{"!proto":"dijit/layout/LayoutContainer.prototype","!doc":"<p>A BorderContainer is a <code>dijit/LayoutContainer</code> that can have draggable splitters between the children,\nin order to adjust their sizes.\n\n</p>\n<p>In addition, it automatically adds some space between the children even\nif they don&#39;t have a draggable splitter between them, and space between the edge of the BorderContainer\nand the children that are adjacent to the edge.  Note that the intended style is that all the children\nhave borders, but (despite the name) the BorderContainer itself does not.\n\n</p>\n<p>See <code>BorderContainer.ChildWidgetProperties</code> for details on the properties that can be set on\nchildren of a <code>BorderContainer</code>.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"design":{"!doc":"<p>Which design is used for the layout:\n\n</p>\n<ul>\n<li>&quot;headline&quot; (default) where the top and bottom extend the full width of the container</li>\n<li>&quot;sidebar&quot; where the left and right sides extend from top to bottom.</li>\n</ul>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"gutters":{"!doc":"<p>Give each pane a border and margin.\nMargin determined by domNode.paddingLeft.\nWhen false, only resizable panes have a gutter (i.e. draggable splitter) for resizing.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"liveSplitters":{"!doc":"<p>Specifies whether splitters resize as you drag (true) or only upon mouseup (false)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"persist":{"!doc":"<p>Save splitter positions in a cookie.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn()"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> ?"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSplitter":{"!type":"fn(region: ?) -> ?","!doc":"<p>Returns the widget responsible for rendering the splitter associated with region</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(child: ?)"},"resize":{"!type":"fn(newSize: ?, currentSize: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a BorderContainer.</p>\n"}},"dijit/layout/BorderContainer.ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a BorderContainer.</p>\n","layoutAlign":{"!doc":"<p>Synonym for region, except using &quot;client&quot; instead of &quot;center&quot;.  Deprecated; use region instead.</p>\n"},"layoutPriority":{"!doc":"<p>Children with a higher layoutPriority will be placed closer to the LayoutContainer center,\nbetween children with a lower layoutPriority.</p>\n"},"maxSize":{"!doc":"<p>Specifies a maximum size (in pixels) for this widget when resized by a splitter.</p>\n"},"minSize":{"!doc":"<p>Specifies a minimum size (in pixels) for this widget when resized by a splitter.</p>\n"},"region":{"!doc":"<p>Values: &quot;top&quot;, &quot;bottom&quot;, &quot;leading&quot;, &quot;trailing&quot;, &quot;left&quot;, &quot;right&quot;, &quot;center&quot;.\nSee the <code>dijit/layout/LayoutContainer</code> description for details.</p>\n"},"splitter":{"!doc":"<p>Parameter for children where region != &quot;center&quot;.\nIf true, enables user to resize the widget by putting a draggable splitter between\nthis widget and the region=center widget.</p>\n"}},"dijit/layout/BorderContainer._Splitter":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A draggable spacer between two items in a <code>dijit/layout/BorderContainer</code>.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"child":{"!doc":"<p>Pointer to the pane associated with this splitter</p>\n"},"class":{},"container":{"!doc":"<p>Pointer to the parent BorderContainer</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"live":{"!doc":"<p>If true, the child&#39;s size changes and the child widget is redrawn as you drag the splitter;\notherwise, the size doesn&#39;t change until you drop the splitter (by mouse-up)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"region":{"!doc":"<p>Region of pane associated with this splitter.\n&quot;top&quot;, &quot;bottom&quot;, &quot;left&quot;, &quot;right&quot;.</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/BorderContainer._Gutter":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Just a spacer div to separate side pane from center pane.\nBasically a trick to lookup the gutter/splitter width from the theme.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/LayoutContainer":{"!type":"fn()","prototype":{"!proto":"dijit/layout/_LayoutWidget.prototype","!doc":"<p>A LayoutContainer is a box with a specified size, such as style=&quot;width: 500px; height: 500px;&quot;,\nthat contains a child widget marked region=&quot;center&quot; and optionally children widgets marked\nregion equal to &quot;top&quot;, &quot;bottom&quot;, &quot;leading&quot;, &quot;trailing&quot;, &quot;left&quot; or &quot;right&quot;.\nChildren along the edges will be laid out according to width or height dimensions. The remaining\nspace is designated for the center region.\n\n</p>\n<p>The outer size must be specified on the LayoutContainer node.  Width must be specified for the sides\nand height for the top and bottom, respectively.  No dimensions should be specified on the center;\nit will fill the remaining space.  Regions named &quot;leading&quot; and &quot;trailing&quot; may be used just like\n&quot;left&quot; and &quot;right&quot; except that they will be reversed in right-to-left environments.\n\n</p>\n<p>For complex layouts, multiple children can be specified for a single region.   In this case, the\nlayoutPriority flag on the children determines which child is closer to the edge (low layoutPriority)\nand which child is closer to the center (high layoutPriority).   layoutPriority can also be used\ninstead of the design attribute to control layout precedence of horizontal vs. vertical panes.\n\n</p>\n<p>See <code>LayoutContainer.ChildWidgetProperties</code> for details on the properties that can be set on\nchildren of a <code>LayoutContainer</code>.\n\n</p>\n<p>If layoutPriority is not set, lays out each child in the natural order the children occur in.\nBasically each child is laid out into the &quot;remaining space&quot;, where &quot;remaining space&quot; is initially\nthe content area of this widget, but is reduced to a smaller rectangle each time a child is added.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"design":{"!doc":"<p>Which design is used for the layout:\n\n</p>\n<ul>\n<li>&quot;headline&quot; (default) where the top and bottom extend the full width of the container</li>\n<li>&quot;sidebar&quot; where the left and right sides extend from top to bottom.</li>\n</ul>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(child: ?)"},"resize":{"!type":"fn(changeSize?: ?, resultSize?: ?)","!doc":"<p>Call this to resize a widget, or after its size has changed.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a LayoutContainer.</p>\n"}},"dijit/layout/LayoutContainer.ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a LayoutContainer.</p>\n","layoutAlign":{"!doc":"<p>Synonym for region, except using &quot;client&quot; instead of &quot;center&quot;.  Deprecated; use region instead.</p>\n"},"layoutPriority":{"!doc":"<p>Children with a higher layoutPriority will be placed closer to the LayoutContainer center,\nbetween children with a lower layoutPriority.</p>\n"},"region":{"!doc":"<p>Values: &quot;top&quot;, &quot;bottom&quot;, &quot;leading&quot;, &quot;trailing&quot;, &quot;left&quot;, &quot;right&quot;, &quot;center&quot;.\nSee the <code>dijit/layout/LayoutContainer</code> description for details.</p>\n"}},"dijit/layout/LinkPane":{"!type":"fn()","prototype":{"!proto":"dijit/layout/ContentPane.prototype","!doc":"<p>A ContentPane with an href where (when declared in markup)\nthe title is specified as innerHTML rather than as a title attribute.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/SplitContainer":{"!type":"fn()","prototype":{"!proto":"dijit/layout/_LayoutWidget.prototype","!doc":"<p>Deprecated.  Use <code>dijit/layout/BorderContainer</code> instead.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"activeSizing":{"!doc":"<p>If true, the children&#39;s size changes as you drag the bar;\notherwise, the sizes don&#39;t change until you drop the bar (by mouse-up)</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"orientation":{"!doc":"<p>either &#39;horizontal&#39; or vertical; indicates whether the children are\narranged side-by-side or up/down.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"persist":{"!doc":"<p>Save splitter positions in a cookie</p>\n"},"sizerWidth":{"!doc":"<p>Size in pixels of the bar between each child</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)","!doc":"<p>Add a child widget to the container</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"beginSizing":{"!type":"fn(e: ?, i: ?)","!doc":"<p>Begin dragging the splitter between child[i] and child[i+1]</p>\n"},"buildRendering":{"!type":"fn()"},"changeSizing":{"!type":"fn(e: ?)","!doc":"<p>Called on mousemove while dragging the splitter</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"endSizing":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()","!doc":"<p>Do layout of panels</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Remove sizer, but only if widget is really our child and\nwe have at least one sizer to throw away</p>\n"},"resize":{"!type":"fn(changeSize?: ?, resultSize?: ?)","!doc":"<p>Call this to resize a widget, or after its size has changed.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a SplitContainer.</p>\n"}},"dijit/layout/SplitContainer.ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a SplitContainer.</p>\n","sizeMin":{"!doc":"<p>Minimum size (width or height) of a child of a SplitContainer.\nThe value is relative to other children&#39;s sizeShare properties.</p>\n"},"sizeShare":{"!doc":"<p>Size (width or height) of a child of a SplitContainer.\nThe value is relative to other children&#39;s sizeShare properties.\nFor example, if there are two children and each has sizeShare=10, then\neach takes up 50% of the available space.</p>\n"}},"dijit/layout/TabContainer":{"!type":"fn()","prototype":{"!proto":"dijit/layout/_TabContainerBase.prototype","!doc":"<p>A Container with tabs to select each child (only one of which is displayed at a time).</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"controllerWidget":{"!doc":"<p>An optional parameter to override the widget used to display the tab labels</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>If true, change the size of my currently displayed child to match my size</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"nested":{"!doc":"<p>If true, use styling for a TabContainer nested inside another TabContainer.\nFor tundra etc., makes tabs look like links, and hides the outer\nborder since the outer TabContainer already has a border.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"persist":{"!doc":"<p>Remembers the selected child across sessions</p>\n"},"searchContainerNode":{},"selectedChildWidget":{"!doc":"<p>References the currently selected child widget, if any.\nAdjust selected child with selectChild() method.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabPosition":{"!doc":"<p>Defines where tabs go relative to tab content.\n&quot;top&quot;, &quot;bottom&quot;, &quot;left-h&quot;, &quot;right-h&quot;</p>\n"},"tabStrip":{"!doc":"<p>Defines whether the tablist gets an extra class for layouting, putting a border/shading\naround the set of tabs.   Not supported by claro theme.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useMenu":{"!doc":"<p>True if a menu should be used to select tabs when they are too\nwide to fit the TabContainer, false otherwise.</p>\n"},"useSlider":{"!doc":"<p>True if a slider should be used to select tabs when they are too\nwide to fit the TabContainer, false otherwise.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"back":{"!type":"fn() -> ?","!doc":"<p>Go back to previous page.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"forward":{"!type":"fn() -> ?","!doc":"<p>Advance to next page.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(page: ?)"},"resize":{"!type":"fn()"},"selectChild":{"!type":"fn(page: ?, animate: ?) -> ?","!doc":"<p>Show the given widget (which must be one of my children)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/_TabContainerBase":{"!type":"fn()","prototype":{"!proto":"dijit/layout/StackContainer.prototype","!doc":"<p>Abstract base class for TabContainer.   Must define _makeController() to instantiate\nand return the widget that displays the tab labels</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>If true, change the size of my currently displayed child to match my size</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"nested":{"!doc":"<p>If true, use styling for a TabContainer nested inside another TabContainer.\nFor tundra etc., makes tabs look like links, and hides the outer\nborder since the outer TabContainer already has a border.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"persist":{"!doc":"<p>Remembers the selected child across sessions</p>\n"},"searchContainerNode":{},"selectedChildWidget":{"!doc":"<p>References the currently selected child widget, if any.\nAdjust selected child with selectChild() method.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabPosition":{"!doc":"<p>Defines where tabs go relative to tab content.\n&quot;top&quot;, &quot;bottom&quot;, &quot;left-h&quot;, &quot;right-h&quot;</p>\n"},"tabStrip":{"!doc":"<p>Defines whether the tablist gets an extra class for layouting, putting a border/shading\naround the set of tabs.   Not supported by claro theme.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"back":{"!type":"fn() -> ?","!doc":"<p>Go back to previous page.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"forward":{"!type":"fn() -> ?","!doc":"<p>Advance to next page.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(page: ?)"},"resize":{"!type":"fn()"},"selectChild":{"!type":"fn(page: ?, animate: ?) -> ?","!doc":"<p>Show the given widget (which must be one of my children)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/TabController":{"!type":"fn()","prototype":{"!proto":"dijit/layout/StackController.prototype","!doc":"<p>Set of tabs (the things with titles and a close button, that you click to show a tab panel).\nUsed internally by <code>dijit/layout/TabContainer</code>.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"buttonWidgetCloseClass":{"!doc":"<p>Class of [x] close icon, used by event delegation code to tell when close button was clicked</p>\n"},"class":{},"containerId":{"!doc":"<p>The id of the page container that I point to</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabPosition":{"!doc":"<p>Defines where tabs go relative to the content.\n&quot;top&quot;, &quot;bottom&quot;, &quot;left-h&quot;, &quot;right-h&quot;</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"buttonWidget":{"!type":"fn()","!doc":"<p>The tab widget to create to correspond to each page</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"pane2button":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns the button corresponding to the pane w/the given id.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"TabButton":{"!type":"fn()"}},"dijit/layout/TabController.TabButton":{"!type":"fn()","prototype":{"!proto":"dijit/form/ToggleButton.prototype","_aria_attr":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_setValueAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The CSS class applied to the domNode.</p>\n"},"checked":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.\nIn markup, specified as &quot;checked=&#39;checked&#39;&quot; or just &quot;checked&quot;.\nTrue if the button is depressed, or the checkbox is checked,\nor the radio button is selected, etc.</p>\n"},"class":{},"closeButton":{"!doc":"<p>When true, display close button for this tab</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setChecked":{"!type":"fn(checked: ?)","!doc":"<p>Deprecated.  Use set(&#39;checked&#39;, true/false) instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/StackController":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Set of buttons to select a page in a <code>dijit/layout/StackContainer</code></p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"buttonWidgetCloseClass":{"!doc":"<p>CSS class of [x] close icon, used by event delegation code to tell when close button was clicked</p>\n"},"class":{},"containerId":{"!doc":"<p>The id of the page container that I point to</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"buttonWidget":{"!type":"fn()","!doc":"<p>The button widget to create to correspond to each page</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"pane2button":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns the button corresponding to the pane w/the given id.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"StackButton":{"!type":"fn()"}},"dijit/layout/StackController.StackButton":{"!type":"fn()","prototype":{"!proto":"dijit/form/ToggleButton.prototype","!doc":"<p>Internal widget used by StackContainer.</p>\n","_aria_attr":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_setValueAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"checked":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.\nIn markup, specified as &quot;checked=&#39;checked&#39;&quot; or just &quot;checked&quot;.\nTrue if the button is depressed, or the checkbox is checked,\nor the radio button is selected, etc.</p>\n"},"class":{},"closeButton":{"!doc":"<p>When true, display close button for this tab</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn(evt: ?)"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setChecked":{"!type":"fn(checked: ?)","!doc":"<p>Deprecated.  Use set(&#39;checked&#39;, true/false) instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/layout/ScrollingTabController":{"!type":"fn()","prototype":{"!proto":"dijit/layout/TabController.prototype","!doc":"<p>Set of tabs with left/right arrow keys and a menu to switch between tabs not\nall fitting on a single row.\nWorks only for horizontal tabs (either above or below the content, not to the left\nor right).</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_minScroll":{"!doc":"<p>The distance in pixels from the edge of the tab strip which,\nif a scroll animation is less than, forces the scroll to\ngo all the way to the left/right.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"buttonWidgetCloseClass":{"!doc":"<p>Class of [x] close icon, used by event delegation code to tell when close button was clicked</p>\n"},"class":{},"containerId":{"!doc":"<p>The id of the page container that I point to</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabPosition":{"!doc":"<p>Defines where tabs go relative to the content.\n&quot;top&quot;, &quot;bottom&quot;, &quot;left-h&quot;, &quot;right-h&quot;</p>\n"},"tabStripClass":{"!doc":"<p>The css class to apply to the tab strip, if it is visible.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useMenu":{"!doc":"<p>True if a menu should be used to select tabs when they are too\nwide to fit the TabContainer, false otherwise.</p>\n"},"useSlider":{"!doc":"<p>True if a slider should be used to select tabs when they are too\nwide to fit the TabContainer, false otherwise.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"buttonWidget":{"!type":"fn()","!doc":"<p>The tab widget to create to correspond to each page</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createSmoothScroll":{"!type":"fn(x?: ?) -> +instance","!doc":"<p>Creates a dojo._Animation object that smoothly scrolls the tab list\neither to a fixed horizontal pixel value, or to the selected tab.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"doSlide":{"!type":"fn(direction: ?, node: ?)","!doc":"<p>Scrolls the tab list to the left or right by 75% of the widget width.</p>\n"},"doSlideLeft":{"!type":"fn(e: ?)","!doc":"<p>Scrolls the menu to the left.</p>\n"},"doSlideRight":{"!type":"fn(e: ?)","!doc":"<p>Scrolls the menu to the right.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"pane2button":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns the button corresponding to the pane w/the given id.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(dim: ?) -> ?","!doc":"<p>Hides or displays the buttons used to scroll the tab list and launch the menu\nthat selects tabs.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/form/Slider":{"!doc":"<p>Rollup of all the the Slider related widgets\nFor back-compat, remove for 2.0</p>\n"},"dijit/layout/AccordionPane":{"!type":"fn()","prototype":{"!proto":"dijit/layout/ContentPane.prototype","!doc":"<p>Deprecated widget.   Use <code>dijit/layout/ContentPane</code> instead.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dijit/robot":{},"dijit/robotx":{},"dijit/tree/ObjectStoreModel":{"!type":"fn()","prototype":{"!doc":"<p>Implements dijit/tree/model connecting dijit/Tree to a dojo/store/api/Store that implements\ngetChildren().\n\n</p>\n<p>If getChildren() returns an array with an observe() method, then it will be leveraged to reflect\nstore updates to the tree.   So, this class will work best when:\n\n</p>\n<pre><code>1. the store implements dojo/store/Observable\n2. getChildren() is implemented as a query to the server (i.e. it calls store.query())</code></pre>\n<p>Drag and Drop: To support drag and drop, besides implementing getChildren()\nand dojo/store/Observable, the store must support the parent option to put().\nAnd in order to have child elements ordered according to how the user dropped them,\nput() must support the before option.</p>\n","labelAttr":{"!doc":"<p>Get label for tree node from this attribute</p>\n"},"labelType":{"!doc":"<p>Specifies how to interpret the labelAttr in the data store items.\nCan be &quot;html&quot; or &quot;text&quot;.</p>\n"},"query":{"!doc":"<p>Specifies datastore query to return the root item for the tree.\nMust only return a single item.   Alternately can just pass in pointer\nto root item.</p>\n"},"root":{"!doc":"<p>Pointer to the root item from the dojo/store/api/Store (read only, not a parameter)</p>\n"},"store":{"!doc":"<p>Underlying store</p>\n"},"destroy":{"!type":"fn()"},"getChildren":{"!type":"fn(parentItem: ?, onComplete: ?, onError: ?)","!doc":"<p>Calls onComplete() with array of child items of given parent item.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>Get the label for an item</p>\n"},"getRoot":{"!type":"fn(onItem: ?, onError: ?)","!doc":"<p>Calls onItem with the root item for the tree, possibly a fabricated item.\nCalls onError on error.</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool"},"mayHaveChildren":{"!type":"fn(item: ?) -> bool","!doc":"<p>Tells if an item has or might have children.  Implementing logic here\navoids showing +/- expando icon for nodes that we know won&#39;t have children.\n(For efficiency reasons we may not want to check if an element actually\nhas children until user clicks the expando node).\n\n</p>\n<p>Application code should override this method based on the data, for example\nit could be <code>return item.leaf == true;</code>.\n\n</p>\n<p>Note that mayHaveChildren() must return true for an item if it could possibly\nhave children in the future, due to drag-an-drop or some other data store update.\nAlso note that it may return true if it&#39;s just too expensive to check during tree\ncreation whether or not the item has children.</p>\n"},"newItem":{"!type":"fn(args: ?, parent: ?, insertIndex?: ?, before: ?) -> ?","!doc":"<p>Creates a new item.   See <code>dojo/data/api/Write</code> for details on args.\nUsed in drag &amp; drop when item from external source dropped onto tree.</p>\n"},"pasteItem":{"!type":"fn(childItem: ?, oldParentItem: ?, newParentItem: ?, bCopy: ?, insertIndex?: ?, before: ?) -> ?","!doc":"<p>Move or copy an item from one parent item to another.\nUsed in drag &amp; drop</p>\n"}}},"dijit/tree/model":{"!type":"fn()","prototype":{"!doc":"<p>Contract for any data provider object for the tree.</p>\n","destroy":{"!type":"fn()","!doc":"<p>Destroys this object, releasing connections to the store</p>\n"},"getChildren":{"!type":"fn(parentItem: ?, onComplete: ?)","!doc":"<p>Calls onComplete() with array of child items of given parent item, all loaded.\nThrows exception on error.</p>\n"},"getIdentity":{"!type":"fn(item: ?)","!doc":"<p>Returns identity for an item</p>\n"},"getLabel":{"!type":"fn(item: ?)","!doc":"<p>Get the label for an item</p>\n"},"getRoot":{"!type":"fn(onItem: ?)","!doc":"<p>Calls onItem with the root item for the tree, possibly a fabricated item.\nThrows exception on error.</p>\n"},"isItem":{"!type":"fn(something: ?)","!doc":"<p>Returns true if <em>something</em> is an item and came from this model instance.\nReturns false if <em>something</em> is a literal, an item from another model instance,\nor is any object other than an item.</p>\n"},"mayHaveChildren":{"!type":"fn(item: ?)","!doc":"<p>Tells if an item has or may have children.  Implementing logic here\navoids showing +/- expando icon for nodes that we know don&#39;t have children.\n(For efficiency reasons we may not want to check if an element actually\nhas children until user clicks the expando node)</p>\n"},"newItem":{"!type":"fn(args: ?, parent: ?, insertIndex?: ?, before?: ?)","!doc":"<p>Creates a new item.   See <code>dojo/data/api/Write</code> for details on args.</p>\n"},"pasteItem":{"!type":"fn(childItem: ?, oldParentItem: ?, newParentItem: ?, bCopy: ?, insertIndex?: ?, before?: ?)","!doc":"<p>Move or copy an item from one parent item to another.\nUsed in drag &amp; drop.\nIf oldParentItem is specified and bCopy is false, childItem is removed from oldParentItem.\nIf newParentItem is specified, childItem is attached to newParentItem.</p>\n"}}},"dojox/NodeList/delegate":{"!doc":"<p>Array-like object which adds syntactic\nsugar for chaining, common iteration operations, animation, and\nnode manipulation. NodeLists are most often returned as the\nresult of dojo/query() calls.</p>\n","_nodeDataCache":{},"events":{},"prototype":{"addClass":{"!type":"fn(className: ?)","!doc":"<p>adds the specified class to every node in the list</p>\n"},"addClassFx":{"!type":"fn(cssClass: ?, args: ?)","!doc":"<p>Animate the effects of adding a class to all nodes in this list.\nsee <code>dojox.fx.addClass</code></p>\n"},"addContent":{"!type":"fn(content: ?, position?: ?) -> fn","!doc":"<p>add a node, NodeList or some HTML as a string to every item in the\nlist.  Returns the original list.</p>\n"},"adopt":{"!type":"fn(queryOrListOrNode: ?, position?: ?) -> ?","!doc":"<p>places any/all elements in queryOrListOrNode at a\nposition relative to the first element in this list.\nReturns a dojo/NodeList of the adopted elements.</p>\n"},"after":{"!type":"fn(content: ?) -> +any","!doc":"<p>Places the content after every node in the NodeList.</p>\n"},"andSelf":{"!type":"fn() -> ?","!doc":"<p>Adds the nodes from the previous dojo/NodeList to the current dojo/NodeList.</p>\n"},"anim":{"!type":"fn(properties: ?, duration?: ?, easing?: ?, onEnd?: ?, delay?: ?) -> ?","!doc":"<p>Animate one or more CSS properties for all nodes in this list.\nThe returned animation object will already be playing when it\nis returned. See the docs for <code>dojo.anim</code> for full details.</p>\n"},"animateProperty":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>Animate all elements of this NodeList across the properties specified.\nsyntax identical to <code>dojo.animateProperty</code>\n</p>\n"},"append":{"!type":"fn(content: ?) -> +any","!doc":"<p>appends the content to every node in the NodeList.</p>\n"},"appendTo":{"!type":"fn(query: ?) -> +any","!doc":"<p>appends nodes in this NodeList to the nodes matched by\nthe query passed to appendTo.</p>\n"},"at":{"!type":"fn(index: ?) -> ?","!doc":"<p>Returns a new NodeList comprised of items in this NodeList\nat the given index or indices.\n</p>\n"},"attr":{"!type":"fn(property: ?, value?: ?) -> +any","!doc":"<p>gets or sets the DOM attribute for every element in the\nNodeList. See also <code>dojo/dom-attr</code></p>\n"},"before":{"!type":"fn(content: ?) -> +any","!doc":"<p>Places the content before every node in the NodeList.</p>\n"},"children":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all immediate child elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the child elements.</p>\n"},"clone":{"!type":"fn() -> +any","!doc":"<p>Clones all the nodes in this NodeList and returns them as a new NodeList.</p>\n"},"closest":{"!type":"fn(query: ?, root?: ?) -> +any","!doc":"<p>Returns closest parent that matches query, including current node in this\ndojo/NodeList if it matches the query.</p>\n"},"concat":{"!type":"fn(item?: ?) -> ?","!doc":"<p>Returns a new NodeList comprised of items in this NodeList\nas well as items passed in as parameters</p>\n"},"connect":{"!type":"fn(methodName: ?, objOrFunc: ?, funcName?: ?)","!doc":"<p>Attach event handlers to every item of the NodeList. Uses dojo.connect()\nso event properties are normalized.\n\n</p>\n<p>Application must manually require() &quot;dojo/_base/connect&quot; before using this method.</p>\n"},"constructor":{"!type":"fn(array: ?) -> []","!doc":"<p>Array-like object which adds syntactic\nsugar for chaining, common iteration operations, animation, and\nnode manipulation. NodeLists are most often returned as the\nresult of dojo/query() calls.</p>\n"},"coords":{"!type":"fn()","!doc":"<p>Deprecated: Use position() for border-box x/y/w/h\nor marginBox() for margin-box w/h/l/t.\nReturns the box objects of all elements in a node list as\nan Array (<em>not</em> a NodeList). Acts like <code>domGeom.coords</code>, though assumes\nthe node passed is each node in this list.</p>\n"},"data":{"!type":"fn(key?: ?, value?: ?) -> +Object","!doc":"<p>stash or get some arbitrary data on/from these nodes.\n</p>\n"},"delegate":{"!type":"fn(selector: ?, eventName: ?, fn: ?) -> ?","!doc":"<p>Monitor nodes in this NodeList for [bubbled] events on nodes that match selector.\nCalls fn(evt) for those events, where (inside of fn()), this == the node\nthat matches the selector.</p>\n"},"dtl":{"!type":"fn(template: ?, context: ?) -> fn","!doc":"<p>Renders the specified template in each of the NodeList entries.</p>\n"},"empty":{"!type":"fn() -> ?","!doc":"<p>clears all content from each node in the list. Effectively\nequivalent to removing all child nodes from every item in\nthe list.</p>\n"},"end":{"!type":"fn() -> ?","!doc":"<p>Ends use of the current <code>NodeList</code> by returning the previous NodeList\nthat generated the current NodeList.</p>\n"},"even":{"!type":"fn() -> +any","!doc":"<p>Returns the even nodes in this dojo/NodeList as a dojo/NodeList.</p>\n"},"every":{"!type":"fn(callback: ?, thisObject?: ?) -> ?","!doc":"<p>see <code>dojo/_base/array.every()</code> and the <a href=\"https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/every\">Array.every\ndocs</a>.\nTakes the same structure of arguments and returns as\ndojo/_base/array.every() with the caveat that the passed array is\nimplicitly this NodeList</p>\n"},"fadeIn":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>fade in all elements of this NodeList via <code>dojo.fadeIn</code>\n</p>\n"},"fadeOut":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>fade out all elements of this NodeList via <code>dojo.fadeOut</code>\n</p>\n"},"filter":{"!type":"fn(filter: ?) -> ?","!doc":"<p>&quot;masks&quot; the built-in javascript filter() method (supported\nin Dojo via <code>dojo.filter</code>) to support passing a simple\nstring filter in addition to supporting filtering function\nobjects.</p>\n"},"first":{"!type":"fn() -> +any","!doc":"<p>Returns the first node in this dojo/NodeList as a dojo/NodeList.</p>\n"},"forEach":{"!type":"fn(callback: ?, thisObj: ?) -> fn","!doc":"<p>see <code>dojo/_base/array.forEach()</code>. The primary difference is that the acted-on\narray is implicitly this NodeList. If you want the option to break out\nof the forEach loop, use every() or some() instead.</p>\n"},"html":{"!type":"fn(value?: ?) -> +any","!doc":"<p>allows setting the innerHTML of each node in the NodeList,\nif there is a value passed in, otherwise, reads the innerHTML value of the first node.</p>\n"},"indexOf":{"!type":"fn(value: ?, fromIndex?: ?) -> +any","!doc":"<p>see <code>dojo/_base/array.indexOf()</code>. The primary difference is that the acted-on\narray is implicitly this NodeList</p>\n"},"innerHTML":{"!type":"fn(value?: ?) -> +any","!doc":"<p>allows setting the innerHTML of each node in the NodeList,\nif there is a value passed in, otherwise, reads the innerHTML value of the first node.</p>\n"},"insertAfter":{"!type":"fn(query: ?) -> +any","!doc":"<p>The nodes in this NodeList will be placed after the nodes\nmatched by the query passed to insertAfter.</p>\n"},"insertBefore":{"!type":"fn(query: ?) -> +any","!doc":"<p>The nodes in this NodeList will be placed after the nodes\nmatched by the query passed to insertAfter.</p>\n"},"instantiate":{"!type":"fn(declaredClass: ?, properties?: ?) -> ?","!doc":"<p>Create a new instance of a specified class, using the\nspecified properties and each node in the NodeList as a\nsrcNodeRef.</p>\n"},"last":{"!type":"fn() -> +any","!doc":"<p>Returns the last node in this dojo/NodeList as a dojo/NodeList.</p>\n"},"lastIndexOf":{"!type":"fn(value: ?, fromIndex?: ?) -> +any","!doc":"<p>see <code>dojo/_base/array.lastIndexOf()</code>. The primary difference is that the\nacted-on array is implicitly this NodeList</p>\n"},"map":{"!type":"fn(func: ?, obj?: ?) -> ?","!doc":"<p>see <code>dojo/_base/array.map()</code>. The primary difference is that the acted-on\narray is implicitly this NodeList and the return is a\nNodeList (a subclass of Array)</p>\n"},"marginBox":{"!type":"fn()","!doc":"<p>Returns margin-box size of nodes</p>\n"},"next":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns the next element for nodes in this dojo/NodeList.\nOptionally takes a query to filter the next elements.</p>\n"},"nextAll":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all sibling elements that come after the nodes in this dojo/NodeList.\nOptionally takes a query to filter the sibling elements.</p>\n"},"odd":{"!type":"fn() -> +any","!doc":"<p>Returns the odd nodes in this dojo/NodeList as a dojo/NodeList.</p>\n"},"on":{"!type":"fn(eventName: ?, listener: ?) -> ?","!doc":"<p>Listen for events on the nodes in the NodeList. Basic usage is:\n</p>\n"},"orphan":{"!type":"fn(filter?: ?) -> +any","!doc":"<p>removes elements in this list that match the filter\nfrom their parents and returns them as a new NodeList.</p>\n"},"parent":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns immediate parent elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the parent elements.</p>\n"},"parents":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all parent elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the child elements.</p>\n"},"place":{"!type":"fn(queryOrNode: ?, position: ?) -> ?","!doc":"<p>places elements of this node list relative to the first element matched\nby queryOrNode. Returns the original NodeList. See: <code>dojo/dom-construct.place</code></p>\n"},"position":{"!type":"fn() -> ?","!doc":"<p>Returns border-box objects (x/y/w/h) of all elements in a node list\nas an Array (<em>not</em> a NodeList). Acts like <code>dojo/dom-geometry-position</code>, though\nassumes the node passed is each node in this list.</p>\n"},"prepend":{"!type":"fn(content: ?) -> +any","!doc":"<p>prepends the content to every node in the NodeList.</p>\n"},"prependTo":{"!type":"fn(query: ?) -> +any","!doc":"<p>prepends nodes in this NodeList to the nodes matched by\nthe query passed to prependTo.</p>\n"},"prev":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns the previous element for nodes in this dojo/NodeList.\nOptionally takes a query to filter the previous elements.</p>\n"},"prevAll":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all sibling elements that come before the nodes in this dojo/NodeList.\nOptionally takes a query to filter the sibling elements.</p>\n"},"query":{"!type":"fn(queryStr: ?) -> fn","!doc":"<p>Returns a new list whose members match the passed query,\nassuming elements of the current NodeList as the root for\neach search.</p>\n"},"remove":{"!type":"fn(filter?: ?) -> +any","!doc":"<p>removes elements in this list that match the filter\nfrom their parents and returns them as a new NodeList.</p>\n"},"removeAttr":{"!type":"fn(name: ?)","!doc":"<p>Removes an attribute from each node in the list.</p>\n"},"removeClass":{"!type":"fn(className?: ?) -> +any","!doc":"<p>removes the specified class from every node in the list</p>\n"},"removeClassFx":{"!type":"fn(cssClass: ?, args: ?)","!doc":"<p>Animate the effect of removing a class to all nodes in this list.\nsee <code>dojox.fx.removeClass</code></p>\n"},"removeData":{"!type":"fn(key?: ?)","!doc":"<p>Remove the data associated with these nodes.</p>\n"},"replaceAll":{"!type":"fn(query: ?) -> +any","!doc":"<p>replaces nodes matched by the query passed to replaceAll with the nodes\nin this NodeList.</p>\n"},"replaceClass":{"!type":"fn(addClassStr: ?, removeClassStr?: ?)","!doc":"<p>Replaces one or more classes on a node if not present.\nOperates more quickly than calling <code>removeClass()</code> and <code>addClass()</code></p>\n"},"replaceWith":{"!type":"fn(content: ?) -> +any","!doc":"<p>Replaces each node in ths NodeList with the content passed to replaceWith.</p>\n"},"siblings":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all sibling elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the sibling elements.</p>\n"},"slice":{"!type":"fn(begin: ?, end?: ?) -> ?","!doc":"<p>Returns a new NodeList, maintaining this one in place</p>\n"},"slideTo":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>slide all elements of the node list to the specified place via <code>dojo/fx.slideTo()</code>\n</p>\n"},"some":{"!type":"fn(callback: ?, thisObject?: ?) -> ?","!doc":"<p>Takes the same structure of arguments and returns as\n<code>dojo/_base/array.some()</code> with the caveat that the passed array is\nimplicitly this NodeList.  See <code>dojo/_base/array.some()</code> and Mozilla&#39;s\n<a href=\"https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/some\">Array.some\ndocumentation</a>.</p>\n"},"splice":{"!type":"fn(index: ?, howmany?: ?, item?: ?) -> ?","!doc":"<p>Returns a new NodeList, manipulating this NodeList based on\nthe arguments passed, potentially splicing in new elements\nat an offset, optionally deleting elements</p>\n"},"style":{"!type":"fn(property: ?, value?: ?) -> +any","!doc":"<p>gets or sets the CSS property for every element in the NodeList</p>\n"},"text":{"!type":"fn(value: ?) -> +any","!doc":"<p>allows setting the text value of each node in the NodeList,\nif there is a value passed in, otherwise, returns the text value for all the\nnodes in the NodeList in one string.</p>\n"},"toggleClass":{"!type":"fn(className: ?, condition?: ?)","!doc":"<p>Adds a class to node if not present, or removes if present.\nPass a boolean condition if you want to explicitly add or remove.</p>\n"},"toggleClassFx":{"!type":"fn(cssClass: ?, force: ?, args: ?)","!doc":"<p>Animate the effect of adding or removing a class to all nodes in this list.\nsee <code>dojox.fx.toggleClass</code></p>\n"},"toString":{"!type":"fn() -> ?"},"val":{"!type":"fn(value: ?) -> +any","!doc":"<p>If a value is passed, allows seting the value property of form elements in this\nNodeList, or properly selecting/checking the right value for radio/checkbox/select\nelements. If no value is passed, the value of the first node in this NodeList\nis returned.</p>\n"},"wipeIn":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>wipe in all elements of this NodeList via <code>dojo/fx.wipeIn()</code>\n</p>\n"},"wipeOut":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>wipe out all elements of this NodeList via <code>dojo/fx.wipeOut()</code>\n</p>\n"},"wrap":{"!type":"fn(html: ?) -> +any","!doc":"<p>Wrap each node in the NodeList with html passed to wrap.</p>\n"},"wrapAll":{"!type":"fn(html: ?) -> +any","!doc":"<p>Insert html where the first node in this NodeList lives, then place all\nnodes in this NodeList as the child of the html.</p>\n"},"wrapInner":{"!type":"fn(html: ?) -> +any","!doc":"<p>For each node in the NodeList, wrap all its children with the passed in html.</p>\n"}}},"dojox/NodeList/delegate._nodeDataCache":{},"dojox/analytics/Urchin":{"!type":"fn()","prototype":{"!doc":"<p>A Google-analytics helper, for post-onLoad inclusion of the tracker, and\ndynamic tracking during long-lived page cycles.</p>\n","acct":{"!doc":"<p>your GA urchin tracker account number. Overrides <code>djConfig.urchin</code></p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initialize this Urchin instance. Immediately starts the load\nsequence, so defer construction until (ideally) after onLoad and\npotentially widget parsing.</p>\n"},"GAonLoad":{"!type":"fn()","!doc":"<p>Stub function to fire when urchin is complete</p>\n"},"trackPageView":{"!type":"fn(url: ?)","!doc":"<p>A public API attached to this widget instance, allowing you\nAjax-like notification of updates.</p>\n"}}},"dojox/analytics/_base":{"!doc":"<p>where we store data until we&#39;re ready to send it off.</p>\n","_data":{},"_id":{},"dataUrl":{},"inTransitRetry":{},"maxRequestSize":{},"sendInterval":{},"sendMethod":{},"prototype":{"addData":{"!type":"fn(dataType: ?, data: ?)","!doc":"<p>add data to the queue. Will be pusshed to the server on the next\ndata push</p>\n"},"checkData":{"!type":"fn()","!doc":"<p>TODOC?</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>where we store data until we&#39;re ready to send it off.</p>\n"},"getQueryPacket":{"!type":"fn() -> string"},"pushData":{"!type":"fn() -> bool","!doc":"<p>pushes data to the server if any exists.  If a push is done, return\nthe deferred after hooking up completion callbacks.  If there is no data\nto be pushed, return false;</p>\n"},"schedulePusher":{"!type":"fn(interval: ?)","!doc":"<p>Schedule the data pushing routines to happen in interval ms</p>\n"}}},"dojox/analytics/_base.plugins":{"consoleMessages":{},"dojo":{},"gestureEvents":{},"idle":{},"mouseClick":{},"mouseOver":{},"touchMove":{},"touchPress":{},"window":{}},"dojox/analytics/plugins/consoleMessages":{},"dojox/analytics/plugins/dojo":{"!doc":"<p>plugin to have analyitcs return the base info dojo collects</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>plugin to have analyitcs return the base info dojo collects</p>\n"}}},"dojox/analytics/plugins/gestureEvents":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/gesture/tap":{"!type":"fn()","prototype":{"!proto":"dojox/gesture/Base.prototype","_elements":{"!doc":"<p>List of elements that wraps target node and gesture data</p>\n"},"_events":{"!doc":"<p>The complete list of supported gesture events with full name space\ne.g [&#39;tap&#39;, &#39;tap.hold&#39;, &#39;tap.doubletap&#39;]</p>\n"},"_lock":{"!doc":"<p>The dom node whose descendants are all locked for processing</p>\n"},"defaultEvent":{"!doc":"<p>Default event - &#39;tap&#39;</p>\n"},"doubleTapTimeout":{},"holdThreshold":{"!doc":"<p>Threshold(in milliseconds) for &#39;tap.hold&#39;</p>\n"},"subEvents":{"!doc":"<p>List of sub events, used by being\ncombined with defaultEvent as &#39;tap.hold&#39;, &#39;tap.doubletap&#39;.</p>\n"},"tapRadius":{"!doc":"<p>Valid tap radius from previous touch point</p>\n"},"touchOnly":{"!doc":"<p>Whether the gesture is touch-device only</p>\n"},"cancel":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Process the &#39;cancel&#39; phase of a gesture</p>\n"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Release all handlers and resources</p>\n"},"fire":{"!type":"fn(node: ?, event: ?)","!doc":"<p>Fire a gesture event and invoke registered listeners\na simulated GestureEvent will also be sent along</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initialization works</p>\n"},"isLocked":{"!type":"fn(node: ?) -> bool","!doc":"<p>Check if the node is locked, isLocked(node) means\nwhether it&#39;s a descendant of the currently locked node.</p>\n"},"lock":{"!type":"fn(node: ?)","!doc":"<p>Lock all descendants of the node.</p>\n"},"move":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Process the &#39;move&#39; phase of a gesture</p>\n"},"press":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, record initial tap info and register a timeout checker for &#39;tap.hold&#39;</p>\n"},"release":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, fire matched &#39;tap&#39; or &#39;tap.doubletap&#39; during touchend</p>\n"},"unLock":{"!type":"fn()","!doc":"<p>Release the lock</p>\n"}}},"dojox/gesture/tap.Tap":{"!type":"fn()","prototype":{"!proto":"dojox/gesture/Base.prototype","_elements":{"!doc":"<p>List of elements that wraps target node and gesture data</p>\n"},"_events":{"!doc":"<p>The complete list of supported gesture events with full name space\ne.g [&#39;tap&#39;, &#39;tap.hold&#39;, &#39;tap.doubletap&#39;]</p>\n"},"_lock":{"!doc":"<p>The dom node whose descendants are all locked for processing</p>\n"},"defaultEvent":{"!doc":"<p>Default event - &#39;tap&#39;</p>\n"},"doubleTapTimeout":{},"holdThreshold":{"!doc":"<p>Threshold(in milliseconds) for &#39;tap.hold&#39;</p>\n"},"subEvents":{"!doc":"<p>List of sub events, used by being\ncombined with defaultEvent as &#39;tap.hold&#39;, &#39;tap.doubletap&#39;.</p>\n"},"tapRadius":{"!doc":"<p>Valid tap radius from previous touch point</p>\n"},"touchOnly":{"!doc":"<p>Whether the gesture is touch-device only</p>\n"},"cancel":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Process the &#39;cancel&#39; phase of a gesture</p>\n"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Release all handlers and resources</p>\n"},"fire":{"!type":"fn(node: ?, event: ?)","!doc":"<p>Fire a gesture event and invoke registered listeners\na simulated GestureEvent will also be sent along</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initialization works</p>\n"},"isLocked":{"!type":"fn(node: ?) -> bool","!doc":"<p>Check if the node is locked, isLocked(node) means\nwhether it&#39;s a descendant of the currently locked node.</p>\n"},"lock":{"!type":"fn(node: ?)","!doc":"<p>Lock all descendants of the node.</p>\n"},"move":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Process the &#39;move&#39; phase of a gesture</p>\n"},"press":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, record initial tap info and register a timeout checker for &#39;tap.hold&#39;</p>\n"},"release":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, fire matched &#39;tap&#39; or &#39;tap.doubletap&#39; during touchend</p>\n"},"unLock":{"!type":"fn()","!doc":"<p>Release the lock</p>\n"}}},"dojox/gesture/Base":{"!type":"fn()","prototype":{"_elements":{"!doc":"<p>List of elements that wraps target node and gesture data</p>\n"},"_events":{"!doc":"<p>The complete list of supported gesture events with full name space\ne.g [&#39;tap&#39;, &#39;tap.hold&#39;, &#39;tap.doubletap&#39;]</p>\n"},"_lock":{"!doc":"<p>The dom node whose descendants are all locked for processing</p>\n"},"defaultEvent":{"!doc":"<p>Default event e.g. &#39;tap&#39; is a default event of dojox.gesture.tap</p>\n"},"subEvents":{"!doc":"<p>A list of sub events e.g [&#39;hold&#39;, &#39;doubletap&#39;],\nused by being combined with defaultEvent like &#39;tap.hold&#39;, &#39;tap.doubletap&#39; etc.</p>\n"},"touchOnly":{"!doc":"<p>Whether the gesture is touch-device only</p>\n"},"cancel":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Process the &#39;cancel&#39; phase of a gesture</p>\n"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Release all handlers and resources</p>\n"},"fire":{"!type":"fn(node: ?, event: ?)","!doc":"<p>Fire a gesture event and invoke registered listeners\na simulated GestureEvent will also be sent along</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initialization works</p>\n"},"isLocked":{"!type":"fn(node: ?) -> bool","!doc":"<p>Check if the node is locked, isLocked(node) means\nwhether it&#39;s a descendant of the currently locked node.</p>\n"},"lock":{"!type":"fn(node: ?)","!doc":"<p>Lock all descendants of the node.</p>\n"},"move":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Process the &#39;move&#39; phase of a gesture</p>\n"},"press":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Process the &#39;press&#39; phase of a gesture</p>\n"},"release":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Process the &#39;release&#39; phase of a gesture</p>\n"},"unLock":{"!type":"fn()","!doc":"<p>Release the lock</p>\n"}}},"dojox/main":{"!doc":"<p>The dojox package main module; dojox package is somewhat unusual in that the main module currently just provides an empty object.\nApps should require modules from the dojox packages directly, rather than loading this module.</p>\n","_reqSeqId":{},"buddhist":{},"charting":{},"data":{},"functional":{},"gesture":{},"grid":{},"hebrew":{},"highlight":{},"image":{},"islamic":{},"json":{},"languages":{},"math":{},"persian":{},"regexp":{},"relative":{},"rpc":{},"umalqura":{},"util":{},"utils":{},"validate":{}},"dojox/main.utils":{"coerceType":{"!type":"fn(target: ?, source: ?) -> ?","!doc":"<p>Coerces one object to the type of another.</p>\n"},"merge":{"!type":"fn(object: ?, mixin: ?) -> +instance","!doc":"<p>Merge two objects structurally, mixin properties will override object&#39;s properties.</p>\n"},"updateWithObject":{"!type":"fn(target: ?, source: ?, conv?: ?) -> ?","!doc":"<p>Updates an existing object in place with properties from an &quot;source&quot; object.</p>\n"},"updateWithPattern":{"!type":"fn(target: ?, source: ?, pattern: ?, conv?: ?) -> ?","!doc":"<p>Updates an existing object in place with properties from an &quot;source&quot; object.</p>\n"}},"dojox/main.functional":{"buildLambda":{"!type":"fn(s: ?) -> string","!doc":"<p>builds a function from a snippet, returns a string, which\nrepresents the function.</p>\n"},"clearLambdaCache":{"!type":"fn()","!doc":"<p>clears internal cache of lambdas</p>\n"},"every":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"everyRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"filter":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"filterIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with all attributes that pass the test\nimplemented by the provided function.</p>\n"},"filterRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"foldl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns the final value.</p>\n"},"foldr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns the final value.</p>\n"},"forEach":{"!type":"fn(a: ?, f: ?, o?: ?) -> string","!doc":"<p>executes a provided function once per array element.</p>\n"},"forEachRev":{"!type":"fn(a: ?, f: ?, o?: ?)","!doc":"<p>executes a provided function once per array element.</p>\n"},"forIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> string","!doc":"<p>iterates over all object attributes.</p>\n"},"keys":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all keys in the object</p>\n"},"lambda":{"!type":"fn(s: ?) -> fn()","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns a function object; functions are passed through\nunmodified.</p>\n"},"map":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"mapIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with the results of calling\na provided function on every attribute in this object.</p>\n"},"mapRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"rawLambda":{"!type":"fn(s: ?) -> ?","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns an object describing the function; functions are\npassed through unmodified.</p>\n"},"reduce":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from left-to-right) as to reduce it to a single value.</p>\n"},"reduceRight":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from right-to-left) as to reduce it to a single value.</p>\n"},"repeat":{"!type":"fn(n: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>builds an array by repeatedly applying a unary function N times\nwith a seed value Z. N should be greater than 0.</p>\n"},"scanl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns an array\nof values produced by foldl() at that point.</p>\n"},"scanl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns an array of values produced by foldl1() at that\npoint.</p>\n"},"scanr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns an array\nof values produced by foldr() at that point.</p>\n"},"scanr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns an array of values produced by foldr1() at that\npoint.</p>\n"},"some":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"someRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"unfold":{"!type":"fn(pr: ?, f: ?, g: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by unfolding a value</p>\n"},"until":{"!type":"fn(pr: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by repeatedly applying a unary function with\na seed value Z until the predicate is satisfied.</p>\n"},"values":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all values in the object</p>\n"}},"dojox/main.buddhist":{"locale":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2: ?, portion?: ?)","!doc":"<p>Compare two buddhist date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(dateObject: ?)"},"isLeapYear":{"!type":"fn(dateObject: ?)"}},"dojox/main.hebrew":{"locale":{},"numerals":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(dateheb1: ?, dateheb2: ?, portion?: ?)","!doc":"<p>Compare two hebrew date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(month: ?)"}},"dojox/main.islamic":{"locale":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2: ?, portion?: ?)","!doc":"<p>Compare two islamic date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(month: ?)"}},"dojox/main.persian":{"locale":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2: ?, portion?: ?)","!doc":"<p>Compare two persian date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(month: ?)"}},"dojox/main.relative":{"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using locale-specific settings,\nrelative to the current date or some other date.\n</p>\n"}},"dojox/main.umalqura":{"locale":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2: ?, portion?: ?)","!doc":"<p>Compare two umalqura date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(month: ?)"}},"dojox/main.charting":{"hideTooltip":{"!type":"fn(gfxObject: ?)","!doc":"<p>Hides the tooltip displayed around the given shape.</p>\n"},"showTooltip":{"!type":"fn(innerHTML: ?, gfxObject: ?, positions?: ?)","!doc":"<p>Show a Tooltip displaying the given HTML message around the given gfx shape.</p>\n"}},"dojox/main.util":{},"dojox/main.languages":{"pygments":{}},"dojox/main.regexp":{},"dojox/gesture/swipe":{"!type":"fn()","prototype":{"!proto":"dojox/gesture/Base.prototype","_elements":{"!doc":"<p>List of elements that wraps target node and gesture data</p>\n"},"_events":{"!doc":"<p>The complete list of supported gesture events with full name space\ne.g [&#39;tap&#39;, &#39;tap.hold&#39;, &#39;tap.doubletap&#39;]</p>\n"},"_lock":{"!doc":"<p>The dom node whose descendants are all locked for processing</p>\n"},"defaultEvent":{"!doc":"<p>Default event - &#39;swipe&#39;</p>\n"},"subEvents":{"!doc":"<p>List of sub events, used by \nbeing combined with defaultEvent as &#39;swipe.end&#39;</p>\n"},"touchOnly":{"!doc":"<p>Whether the gesture is touch-device only</p>\n"},"cancel":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten</p>\n"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Release all handlers and resources</p>\n"},"fire":{"!type":"fn(node: ?, event: ?)","!doc":"<p>Fire a gesture event and invoke registered listeners\na simulated GestureEvent will also be sent along</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initialization works</p>\n"},"isLocked":{"!type":"fn(node: ?) -> bool","!doc":"<p>Check if the node is locked, isLocked(node) means\nwhether it&#39;s a descendant of the currently locked node.</p>\n"},"lock":{"!type":"fn(node: ?)","!doc":"<p>Lock all descendants of the node.</p>\n"},"move":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, fire matched &#39;swipe&#39; during touchmove</p>\n"},"press":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, set initial swipe info</p>\n"},"release":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, fire matched &#39;swipe.end&#39; when touchend</p>\n"},"unLock":{"!type":"fn()","!doc":"<p>Release the lock</p>\n"}}},"dojox/gesture/swipe.Swipe":{"!type":"fn()","prototype":{"!proto":"dojox/gesture/Base.prototype","_elements":{"!doc":"<p>List of elements that wraps target node and gesture data</p>\n"},"_events":{"!doc":"<p>The complete list of supported gesture events with full name space\ne.g [&#39;tap&#39;, &#39;tap.hold&#39;, &#39;tap.doubletap&#39;]</p>\n"},"_lock":{"!doc":"<p>The dom node whose descendants are all locked for processing</p>\n"},"defaultEvent":{"!doc":"<p>Default event - &#39;swipe&#39;</p>\n"},"subEvents":{"!doc":"<p>List of sub events, used by \nbeing combined with defaultEvent as &#39;swipe.end&#39;</p>\n"},"touchOnly":{"!doc":"<p>Whether the gesture is touch-device only</p>\n"},"cancel":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten</p>\n"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Release all handlers and resources</p>\n"},"fire":{"!type":"fn(node: ?, event: ?)","!doc":"<p>Fire a gesture event and invoke registered listeners\na simulated GestureEvent will also be sent along</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initialization works</p>\n"},"isLocked":{"!type":"fn(node: ?) -> bool","!doc":"<p>Check if the node is locked, isLocked(node) means\nwhether it&#39;s a descendant of the currently locked node.</p>\n"},"lock":{"!type":"fn(node: ?)","!doc":"<p>Lock all descendants of the node.</p>\n"},"move":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, fire matched &#39;swipe&#39; during touchmove</p>\n"},"press":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, set initial swipe info</p>\n"},"release":{"!type":"fn(data: ?, e: ?)","!doc":"<p>Overwritten, fire matched &#39;swipe.end&#39; when touchend</p>\n"},"unLock":{"!type":"fn()","!doc":"<p>Release the lock</p>\n"}}},"dojox/analytics/plugins/idle":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/analytics/plugins/mouseClick":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/analytics/plugins/mouseOver":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/analytics/plugins/touchMove":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/analytics/plugins/touchPress":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/analytics/plugins/window":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/analytics":{"!doc":"<p>Deprecated.  Should require dojox/analytics modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/app/Controller":{"!type":"fn()","prototype":{"bind":{"!type":"fn(evented: ?, event: ?, handler: ?) -> fn","!doc":"<p>Bind event on dojo/Evented instance, document, domNode or window.\nSave event signal in controller instance. If no parameter is provided\nautomatically bind all events registered in controller events property.\n</p>\n"},"constructor":{"!type":"fn(app: ?, events: ?)","!doc":"<p>bind events on application dojo/Evented instance.\nbind css selector events on document.\n</p>\n"},"unbind":{"!type":"fn(evented: ?, event: ?) -> fn","!doc":"<p>remove a binded event signal.\n</p>\n"}}},"dojox/app/View":{"!type":"fn()","prototype":{"!proto":"dijit/_TemplatedMixin.prototype","!doc":"<p>View class inheriting from ViewBase adding templating &amp; globalization capabilities.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_skipNodeCache":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"searchContainerNode":{},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"afterActivate":{"!type":"fn()","!doc":"<p>view life cycle afterActivate()</p>\n"},"afterDeactivate":{"!type":"fn()","!doc":"<p>view life cycle afterDeactivate()</p>\n"},"beforeActivate":{"!type":"fn()","!doc":"<p>view life cycle beforeActivate()</p>\n"},"beforeDeactivate":{"!type":"fn()","!doc":"<p>view life cycle beforeDeactivate()</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget from a template, setting this.domNode.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?"},"constructor":{"!type":"fn(params: ?)","!doc":"<p>Constructs a View instance either from a configuration or programmatically.\n</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this class, releasing any resources registered via own().</p>\n"},"destroyRendering":{"!type":"fn()"},"init":{"!type":"fn()","!doc":"<p>view life cycle init()</p>\n"},"load":{"!type":"fn() -> +instance"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"start":{"!type":"fn() -> fn","!doc":"<p>start view object.\nload view template, view controller implement and startup all widgets in view template.</p>\n"},"startup":{"!type":"fn()"}}},"dojox/app/ViewBase":{"!type":"fn()","prototype":{"!doc":"<p>View base class with model &amp; controller capabilities. Subclass must implement rendering capabilities.</p>\n","afterActivate":{"!type":"fn()","!doc":"<p>view life cycle afterActivate()</p>\n"},"afterDeactivate":{"!type":"fn()","!doc":"<p>view life cycle afterDeactivate()</p>\n"},"beforeActivate":{"!type":"fn()","!doc":"<p>view life cycle beforeActivate()</p>\n"},"beforeDeactivate":{"!type":"fn()","!doc":"<p>view life cycle beforeDeactivate()</p>\n"},"constructor":{"!type":"fn(params: ?)","!doc":"<p>Constructs a ViewBase instance.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>view life cycle destroy()</p>\n"},"init":{"!type":"fn()","!doc":"<p>view life cycle init()</p>\n"},"load":{"!type":"fn() -> ?"},"start":{"!type":"fn() -> fn","!doc":"<p>start view object.\nload view template, view controller implement and startup all widgets in view template.</p>\n"}}},"dojox/app/utils/model":{"!doc":"<p>model is called to create all of the models for the app, and all models for a view, it will\ncreate and call the appropriate model utility based upon the modelLoader set in the model in the config</p>\n","prototype":{"constructor":{"!type":"fn(config: ?, parent: ?, app: ?) -> +loadedModels","!doc":"<p>model is called to create all of the models for the app, and all models for a view, it will\ncreate and call the appropriate model utility based upon the modelLoader set in the model in the config</p>\n"}}},"dojox/app/utils/constraints":{"getAllSelectedChildren":{"!type":"fn(view: ?, selChildren: ?) -> +any","!doc":"<p>get current all selected children for this view and it&#39;s selected subviews\n</p>\n"},"getSelectedChild":{"!type":"fn(view: ?, constraint: ?) -> +any","!doc":"<p>get current selected child according to the constraint\n</p>\n"},"register":{"!type":"fn(constraint: ?)"},"setSelectedChild":{"!type":"fn(view: ?, constraint: ?, child: ?)","!doc":"<p>set current selected child according to the constraint\n</p>\n"}},"dojox/app/utils/nls":{"!doc":"<p>nsl is called to create to load the nls all for the app, or for a view.</p>\n","prototype":{"constructor":{"!type":"fn(config: ?, parent: ?) -> bool","!doc":"<p>nsl is called to create to load the nls all for the app, or for a view.</p>\n"}}},"dojox/app/controllers/BorderLayout":{"!type":"fn()","prototype":{"!proto":"dojox/app/controllers/LayoutBase.prototype","bind":{"!type":"fn(evented: ?, event: ?, handler: ?) -> fn","!doc":"<p>Bind event on dojo/Evented instance, document, domNode or window.\nSave event signal in controller instance. If no parameter is provided\nautomatically bind all events registered in controller events property.\n</p>\n"},"constructor":{"!type":"fn(app: ?, events: ?)","!doc":"<p>bind &quot;app-initLayout&quot;, &quot;app-layoutView&quot; and &quot;app-resize&quot; events on application instance.\n</p>\n"},"hideView":{"!type":"fn(view: ?)"},"initLayout":{"!type":"fn(event: ?)","!doc":"<p>Response to dojox/app &quot;app-initLayout&quot; event which is setup in LayoutBase.\nThe initLayout event is called once when the View is being created the first time.\n</p>\n"},"layoutView":{"!type":"fn(event: ?)","!doc":"<p>Response to dojox/app &quot;app-layoutView&quot; event.\n</p>\n"},"showView":{"!type":"fn(view: ?)"},"unbind":{"!type":"fn(evented: ?, event: ?) -> fn","!doc":"<p>remove a binded event signal.\n</p>\n"}}},"dojox/app/controllers/LayoutBase":{"!type":"fn()","prototype":{"!proto":"dojox/app/Controller.prototype","bind":{"!type":"fn(evented: ?, event: ?, handler: ?) -> fn","!doc":"<p>Bind event on dojo/Evented instance, document, domNode or window.\nSave event signal in controller instance. If no parameter is provided\nautomatically bind all events registered in controller events property.\n</p>\n"},"constructor":{"!type":"fn(app: ?, events: ?)","!doc":"<p>bind &quot;app-initLayout&quot;, &quot;app-layoutView&quot; and &quot;app-resize&quot; events on application instance.\n</p>\n"},"hideView":{"!type":"fn(view: ?)"},"initLayout":{"!type":"fn(event: ?)","!doc":"<p>Response to dojox/app &quot;app-initLayout&quot; event.\n</p>\n"},"layoutView":{"!type":"fn(event: ?)","!doc":"<p>Response to dojox/app &quot;app-layoutView&quot; event.\n</p>\n"},"showView":{"!type":"fn(view: ?)"},"unbind":{"!type":"fn(evented: ?, event: ?) -> fn","!doc":"<p>remove a binded event signal.\n</p>\n"}}},"dojox/app/controllers/History":{"!type":"fn()","prototype":{"!proto":"dojox/app/Controller.prototype","_currentPosition":{"!doc":"<p>Persistent variable which indicates the current position/index in the history\n(so as to be able to figure out whether the popState event was triggerd by\na backward or forward action).</p>\n"},"currentState":{"!doc":"<p>Current state</p>\n"},"bind":{"!type":"fn(evented: ?, event: ?, handler: ?) -> fn","!doc":"<p>Bind event on dojo/Evented instance, document, domNode or window.\nSave event signal in controller instance. If no parameter is provided\nautomatically bind all events registered in controller events property.\n</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Bind &quot;app-domNode&quot; event on dojox/app application instance.\nBind &quot;startTransition&quot; event on dojox/app application domNode.\nBind &quot;popstate&quot; event on window object.\n</p>\n"},"unbind":{"!type":"fn(evented: ?, event: ?) -> fn","!doc":"<p>remove a binded event signal.\n</p>\n"}}},"dojox/app/utils/hash":{"!doc":"<p>This module contains the hash</p>\n","addViewParams":{"!type":"fn(hash: ?, view: ?, params: ?) -> +any","!doc":"<p>add the view specific params to the hash for example (view1&amp;param1=value1)</p>\n"},"buildWithParams":{"!type":"fn(hash: ?, params: ?) -> +any","!doc":"<p>build up the url hash adding the params</p>\n"},"getParamObj":{"!type":"fn(params: ?, viewPart: ?) -> +any","!doc":"<p>called to handle a view specific params object</p>\n"},"getParams":{"!type":"fn(hash: ?) -> +any","!doc":"<p>get the params from the hash\n</p>\n"},"getParamString":{"!type":"fn(params: ?) -> +any","!doc":"<p>return the param string</p>\n"},"getTarget":{"!type":"fn(hash: ?, defaultView?: ?) -> +any","!doc":"<p>return the target string</p>\n"}},"dojox/app/controllers/HistoryHash":{"!type":"fn()","prototype":{"!proto":"dojox/app/Controller.prototype","bind":{"!type":"fn(evented: ?, event: ?, handler: ?) -> fn","!doc":"<p>Bind event on dojo/Evented instance, document, domNode or window.\nSave event signal in controller instance. If no parameter is provided\nautomatically bind all events registered in controller events property.\n</p>\n"},"constructor":{"!type":"fn(app: ?)","!doc":"<p>Bind &quot;app-domNode&quot; event on dojox/app application instance,\nBind &quot;startTransition&quot; event on dojox/app application domNode,\nsubscribe &quot;/dojo/hashchange&quot; event.\n</p>\n"},"unbind":{"!type":"fn(evented: ?, event: ?) -> fn","!doc":"<p>remove a binded event signal.\n</p>\n"}}},"dojox/app/controllers/Layout":{"!type":"fn()","prototype":{"!proto":"dojox/app/controllers/LayoutBase.prototype","bind":{"!type":"fn(evented: ?, event: ?, handler: ?) -> fn","!doc":"<p>Bind event on dojo/Evented instance, document, domNode or window.\nSave event signal in controller instance. If no parameter is provided\nautomatically bind all events registered in controller events property.\n</p>\n"},"constructor":{"!type":"fn(app: ?, events: ?)","!doc":"<p>bind &quot;app-initLayout&quot; and &quot;app-layoutView&quot; events on application instance.\n</p>\n"},"hideView":{"!type":"fn(view: ?)"},"initLayout":{"!type":"fn(event: ?)","!doc":"<p>Response to dojox/app &quot;app-initLayout&quot; event.\n</p>\n"},"layoutView":{"!type":"fn(event: ?)","!doc":"<p>Response to dojox/app &quot;app-layoutView&quot; event.\n</p>\n"},"resizeSelectedChildren":{"!type":"fn(w: ?)"},"showView":{"!type":"fn(view: ?)"},"unbind":{"!type":"fn(evented: ?, event: ?) -> fn","!doc":"<p>remove a binded event signal.\n</p>\n"}}},"dojox/app/utils/layout":{"layoutChildren":{"!type":"fn(container: ?, dim: ?, children: ?, changedRegionId?: ?, changedRegionSize?: ?)","!doc":"<p>Layout a bunch of child dom nodes within a parent dom node</p>\n"},"marginBox2contentBox":{"!type":"fn(node: ?, mb: ?) -> ?","!doc":"<p>Given the margin-box size of a node, return its content box size.\nFunctions like domGeometry.contentBox() but is more reliable since it doesn&#39;t have\nto wait for the browser to compute sizes.</p>\n"}},"dojox/app/controllers/Load":{"!type":"fn()","prototype":{"!proto":"dojox/app/Controller.prototype","_waitingQueue":{},"bind":{"!type":"fn(evented: ?, event: ?, handler: ?) -> fn","!doc":"<p>Bind event on dojo/Evented instance, document, domNode or window.\nSave event signal in controller instance. If no parameter is provided\nautomatically bind all events registered in controller events property.\n</p>\n"},"constructor":{"!type":"fn(app: ?, events: ?)","!doc":"<p>bind &quot;app-load&quot; event on application instance.\n</p>\n"},"createChild":{"!type":"fn(parent: ?, childId: ?, subIds: ?, params: ?) -> +any","!doc":"<p>Create a view instance if not already loaded by calling createView. This is typically a\ndojox/app/View.\n</p>\n"},"createView":{"!type":"fn(parent: ?, id: ?, name: ?, mixin: ?, params: ?, type: ?) -> +any","!doc":"<p>Create a dojox/app/View instance. Can be overridden to create different type of views.</p>\n"},"init":{"!type":"fn(event: ?)"},"load":{"!type":"fn(event: ?) -> +any","!doc":"<p>Response to dojox/app &quot;loadArray&quot; event.\n</p>\n"},"loadChild":{"!type":"fn(parent: ?, childId: ?, subIds: ?, params: ?, loadEvent: ?) -> +any","!doc":"<p>Load child and sub children views recursively.\n</p>\n"},"loadView":{"!type":"fn(loadEvent: ?) -> +any","!doc":"<p>Response to dojox/app &quot;app-load&quot; event.\n</p>\n"},"proceedLoadView":{"!type":"fn(loadEvt: ?)","!doc":"<p>Proceed load queue by FIFO by default.\nIf load is in proceeding, add the next load to waiting queue.\n</p>\n"},"unbind":{"!type":"fn(evented: ?, event: ?) -> fn","!doc":"<p>remove a binded event signal.\n</p>\n"}}},"dojox/app/controllers/Transition":{"!type":"fn()","prototype":{"!proto":"dojox/app/Controller.prototype","proceeding":{},"waitingQueue":{},"bind":{"!type":"fn(evented: ?, event: ?, handler: ?) -> fn","!doc":"<p>Bind event on dojo/Evented instance, document, domNode or window.\nSave event signal in controller instance. If no parameter is provided\nautomatically bind all events registered in controller events property.\n</p>\n"},"constructor":{"!type":"fn(app: ?, events: ?)","!doc":"<p>bind &quot;app-transition&quot; event on application instance.\n</p>\n"},"proceedTransition":{"!type":"fn(transitionEvt: ?)","!doc":"<p>Proceed transition queue by FIFO by default.\nIf transition is in proceeding, add the next transition to waiting queue.\n</p>\n"},"transition":{"!type":"fn(event: ?)","!doc":"<p>Response to dojox/app &quot;app-transition&quot; event.\n</p>\n"},"unbind":{"!type":"fn(evented: ?, event: ?) -> fn","!doc":"<p>remove a binded event signal.\n</p>\n"}}},"dojox/app/main":{"prototype":{"constructor":{"!type":"fn(config: ?, node: ?)"}}},"dojox/app/module/lifecycle":{"!type":"fn()","prototype":{"_status":{},"lifecycle":{},"getStatus":{"!type":"fn() -> ?"},"setStatus":{"!type":"fn(newStatus: ?)"}}},"dojox/app/utils/config":{"!doc":"<p>This module contains the config</p>\n","configMerge":{"!type":"fn(target: ?, source: ?) -> +any","!doc":"<p>does a deep copy of the source into the target to merge the config from the source into the target</p>\n"},"configProcessHas":{"!type":"fn(source: ?) -> +any","!doc":"<p>scan the source config for has checks and call configMerge to merge has sections, and remove the has sections from the source.</p>\n"}},"dojox/app/module/env":{"!type":"fn()","prototype":{"mode":{},"init":{"!type":"fn()"}}},"dojox/app/utils/mvcModel":{"!doc":"<p>mvcModel is called for each mvc model, to create the mvc model based upon the type and params.\nIt will also load models and return the either the loadedModels or a promise.</p>\n","prototype":{"constructor":{"!type":"fn(config: ?, params: ?, item: ?) -> +model","!doc":"<p>mvcModel is called for each mvc model, to create the mvc model based upon the type and params.\nIt will also load models and return the either the loadedModels or a promise.</p>\n"}}},"dojox/mvc/getStateful":{"!doc":"<p>Create a dojo/Stateful object from a raw value.</p>\n","prototype":{"constructor":{"!type":"fn(value: ?, options: ?) -> +Anything","!doc":"<p>Create a dojo/Stateful object from a raw value.</p>\n"}},"getStatefulArray":{"!type":"fn(a: ?) -> +instance","!doc":"<p>Returns the stateful version of the given array.</p>\n"},"getStatefulObject":{"!type":"fn(o: ?) -> +instance","!doc":"<p>Returns the stateful version of the given object.</p>\n"},"getStatefulValue":{"!type":"fn(v: ?) -> +Anything","!doc":"<p>Just returns the given value.</p>\n"},"getType":{"!type":"fn(v: ?) -> string","!doc":"<p>Returns the type of the given value.</p>\n"}},"dojox/mvc/StatefulArray":{"!doc":"<p>An inheritance of native JavaScript array, that adds dojo/Stateful capability.</p>\n","_meta":{},"prototype":{"constructor":{"!type":"fn(a: ?) -> ?","!doc":"<p>An inheritance of native JavaScript array, that adds dojo/Stateful capability.</p>\n"}}},"dojox/mvc/StatefulArray._meta":{"bases":{}},"dojox/app/utils/simpleModel":{"!doc":"<p>simpleModel is called for each simple model, to create the simple model from the DataStore \nbased upon the store and query params.\nIt will also load models and return the either the loadedModels or a promise.</p>\n","prototype":{"constructor":{"!type":"fn(config: ?, params: ?, item: ?) -> +model","!doc":"<p>simpleModel is called for each simple model, to create the simple model from the DataStore \nbased upon the store and query params.\nIt will also load models and return the either the loadedModels or a promise.</p>\n"}}},"dojox/app/widgets/Container":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_moved":{"!doc":"<p>Flag that signals if the user have moved in (one of) the scroll\ndirection(s) since touch start (a move under the threshold is ignored).</p>\n"},"_parentPadBorderExtentsBottom":{"!doc":"<p>For Tooltip.js.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"allowNestedScrolls":{"!doc":"<p>e.g. Allow ScrollableView in a SwapView.</p>\n"},"appBars":{"!doc":"<p>Enables the search for application-specific bars (header or footer).</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraint":{"!doc":"<p>bounce back to the content area</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dirLock":{"!doc":"<p>disable the move handler if scroll starts in the unexpected direction</p>\n"},"disableFlashScrollBar":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fadeScrollBar":{},"fixedFooter":{},"fixedFooterHeight":{"!doc":"<p>height of a fixed footer</p>\n"},"fixedHeader":{},"fixedHeaderHeight":{"!doc":"<p>height of a fixed header</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>explicitly specified height of this widget (ex. &quot;300px&quot;)</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLocalFooter":{"!doc":"<p>footer is view-local (as opposed to application-wide)</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"propagatable":{"!doc":"<p>let touchstart event propagate up</p>\n"},"scrollable":{},"scrollableParams":{"!doc":"<p>Parameters for dojox/mobile/scrollable.init().</p>\n"},"scrollBar":{"!doc":"<p>show scroll bar or not</p>\n"},"scrollDir":{"!doc":"<p>v: vertical, h: horizontal, vh: both, f: flip</p>\n"},"scrollType":{"!doc":"<ul>\n<li>1: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)animation for slide animation</li>\n<li>2: use top/left style,</li>\n<li>3: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)transition for slide animation</li>\n<li>0: use default value (3 for Android, iOS6+, and BlackBerry; otherwise 1)</li>\n</ul>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"threshold":{"!doc":"<p>drag threshold value in pixels</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"touchNode":{"!doc":"<p>a node that will have touch event handlers</p>\n"},"weight":{"!doc":"<p>frictional drag</p>\n"},"abort":{"!type":"fn()","!doc":"<p>Aborts scrolling.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"adjustDestination":{"!type":"fn(to: ?, pos: ?, dim: ?) -> bool","!doc":"<p>A stub function to be overridden by subclasses.</p>\n"},"buildRendering":{"!type":"fn()"},"calcScrollBarPos":{"!type":"fn(to: ?) -> ?","!doc":"<p>Calculates the scroll bar position.</p>\n"},"calcSpeed":{"!type":"fn(distance: ?, time: ?) -> number","!doc":"<p>Calculate the speed given the distance and time.</p>\n"},"checkFixedBar":{"!type":"fn(node: ?, local: ?) -> ?","!doc":"<p>Checks if the given node is a fixed bar or not.</p>\n"},"cleanup":{"!type":"fn()","!doc":"<p>Uninitialize the module.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createMask":{"!type":"fn()","!doc":"<p>Creates a mask for a scroll bar edge.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"findAppBars":{"!type":"fn()","!doc":"<p>Search for application-specific header or footer.</p>\n"},"findDisp":{"!type":"fn(node: ?) -> +null","!doc":"<p>Finds the currently displayed view node from my sibling nodes.</p>\n"},"flashScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar instantly.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDim":{"!type":"fn() -> ?","!doc":"<p>Returns various internal dimensional information needed for calculation.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPos":{"!type":"fn() -> ?","!doc":"<p>Gets the top position in the midst of animation.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getScreenSize":{"!type":"fn() -> ?","!doc":"<p>Returns the dimensions of the browser window.</p>\n"},"getSpeed":{"!type":"fn() -> ?","!doc":"<p>Returns an object that indicates the scrolling speed.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hideScrollBar":{"!type":"fn()","!doc":"<p>Hides the scroll bar.</p>\n"},"init":{"!type":"fn(params?: ?)","!doc":"<p>Initialize according to the given params.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isFormElement":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the given node is a form control.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isTopLevel":{"!type":"fn() -> bool","!doc":"<p>Returns true if this is a top-level widget.</p>\n"},"layout":{"!type":"fn()","!doc":"<p>layout container</p>\n"},"makeTranslateStr":{"!type":"fn(to: ?) -> string","!doc":"<p>Constructs a string value that is passed to the -webkit-transform property.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"reparent":{"!type":"fn()","!doc":"<p>Moves all the children to containerNode.</p>\n"},"resetScrollBar":{"!type":"fn()","!doc":"<p>Resets the scroll bar length, position, etc.</p>\n"},"resize":{"!type":"fn(changeSize?: ?, resultSize?: ?)","!doc":"<p>Call this to resize a widget, or after its size has changed.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, alignWithTop?: ?, duration?: ?)","!doc":"<p>Scrolls the pane until the searching node is in the view.</p>\n"},"scrollScrollBarTo":{"!type":"fn(to: ?)","!doc":"<p>Moves the scroll bar(s) to the given position without animation.</p>\n"},"scrollTo":{"!type":"fn(to: ?, doNotMoveScrollBar?: ?, node?: ?)","!doc":"<p>Scrolls to the given position immediately without animation.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setKeyframes":{"!type":"fn(from: ?, to: ?, idx: ?)","!doc":"<p>Programmatically sets key frames for the scroll animation.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)","!doc":"<p>Sets the given node as selectable or unselectable.</p>\n"},"showScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar.</p>\n"},"slideScrollBarTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Moves the scroll bar(s) to the given position with the slide animation.</p>\n"},"slideTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Scrolls to the given position with the slide animation.</p>\n"},"startup":{"!type":"fn()"},"stopAnimation":{"!type":"fn()","!doc":"<p>Stops the currently running animation.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/app/widgets/_ScrollableMixin":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/scrollable.prototype","!doc":"<p>Mixin for widgets to have a touch scrolling capability.</p>\n","_moved":{"!doc":"<p>Flag that signals if the user have moved in (one of) the scroll\ndirection(s) since touch start (a move under the threshold is ignored).</p>\n"},"_parentPadBorderExtentsBottom":{"!doc":"<p>For Tooltip.js.</p>\n"},"allowNestedScrolls":{"!doc":"<p>e.g. Allow ScrollableView in a SwapView.</p>\n"},"appBars":{"!doc":"<p>Enables the search for application-specific bars (header or footer).</p>\n"},"constraint":{"!doc":"<p>bounce back to the content area</p>\n"},"dirLock":{"!doc":"<p>disable the move handler if scroll starts in the unexpected direction</p>\n"},"disableFlashScrollBar":{},"fadeScrollBar":{},"fixedFooterHeight":{"!doc":"<p>height of a fixed footer</p>\n"},"fixedHeaderHeight":{"!doc":"<p>height of a fixed header</p>\n"},"height":{"!doc":"<p>explicitly specified height of this widget (ex. &quot;300px&quot;)</p>\n"},"isLocalFooter":{"!doc":"<p>footer is view-local (as opposed to application-wide)</p>\n"},"propagatable":{"!doc":"<p>let touchstart event propagate up</p>\n"},"scrollableParams":{"!doc":"<p>Parameters for dojox/mobile/scrollable.init().</p>\n"},"scrollBar":{"!doc":"<p>show scroll bar or not</p>\n"},"scrollDir":{"!doc":"<p>v: vertical, h: horizontal, vh: both, f: flip</p>\n"},"scrollType":{"!doc":"<ul>\n<li>1: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)animation for slide animation</li>\n<li>2: use top/left style,</li>\n<li>3: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)transition for slide animation</li>\n<li>0: use default value (3 for Android, iOS6+, and BlackBerry; otherwise 1)</li>\n</ul>\n"},"threshold":{"!doc":"<p>drag threshold value in pixels</p>\n"},"touchNode":{"!doc":"<p>a node that will have touch event handlers</p>\n"},"weight":{"!doc":"<p>frictional drag</p>\n"},"abort":{"!type":"fn()","!doc":"<p>Aborts scrolling.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"adjustDestination":{"!type":"fn(to: ?, pos: ?, dim: ?) -> bool","!doc":"<p>A stub function to be overridden by subclasses.</p>\n"},"buildRendering":{"!type":"fn()"},"calcScrollBarPos":{"!type":"fn(to: ?) -> ?","!doc":"<p>Calculates the scroll bar position.</p>\n"},"calcSpeed":{"!type":"fn(distance: ?, time: ?) -> number","!doc":"<p>Calculate the speed given the distance and time.</p>\n"},"checkFixedBar":{"!type":"fn(node: ?, local: ?) -> ?","!doc":"<p>Checks if the given node is a fixed bar or not.</p>\n"},"cleanup":{"!type":"fn()","!doc":"<p>Uninitialize the module.</p>\n"},"constructor":{"!type":"fn()"},"createMask":{"!type":"fn()","!doc":"<p>Creates a mask for a scroll bar edge.</p>\n"},"destroy":{"!type":"fn()"},"findAppBars":{"!type":"fn()","!doc":"<p>Search for application-specific header or footer.</p>\n"},"findDisp":{"!type":"fn(node: ?) -> +null","!doc":"<p>Finds the currently displayed view node from my sibling nodes.</p>\n"},"flashScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar instantly.</p>\n"},"getDim":{"!type":"fn() -> ?","!doc":"<p>Returns various internal dimensional information needed for calculation.</p>\n"},"getPos":{"!type":"fn() -> ?","!doc":"<p>Gets the top position in the midst of animation.</p>\n"},"getScreenSize":{"!type":"fn() -> ?","!doc":"<p>Returns the dimensions of the browser window.</p>\n"},"getSpeed":{"!type":"fn() -> ?","!doc":"<p>Returns an object that indicates the scrolling speed.</p>\n"},"hideScrollBar":{"!type":"fn()","!doc":"<p>Hides the scroll bar.</p>\n"},"init":{"!type":"fn(params?: ?)","!doc":"<p>Initialize according to the given params.</p>\n"},"isFormElement":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the given node is a form control.</p>\n"},"isTopLevel":{"!type":"fn() -> bool","!doc":"<p>Returns true if this is a top-level widget.</p>\n"},"makeTranslateStr":{"!type":"fn(to: ?) -> string","!doc":"<p>Constructs a string value that is passed to the -webkit-transform property.</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"reparent":{"!type":"fn()","!doc":"<p>Moves all the children to containerNode.</p>\n"},"resetScrollBar":{"!type":"fn()","!doc":"<p>Resets the scroll bar length, position, etc.</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, alignWithTop?: ?, duration?: ?)","!doc":"<p>Scrolls the pane until the searching node is in the view.</p>\n"},"scrollScrollBarTo":{"!type":"fn(to: ?)","!doc":"<p>Moves the scroll bar(s) to the given position without animation.</p>\n"},"scrollTo":{"!type":"fn(to: ?, doNotMoveScrollBar?: ?, node?: ?)","!doc":"<p>Scrolls to the given position immediately without animation.</p>\n"},"setKeyframes":{"!type":"fn(from: ?, to: ?, idx: ?)","!doc":"<p>Programmatically sets key frames for the scroll animation.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)","!doc":"<p>Sets the given node as selectable or unselectable.</p>\n"},"showScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar.</p>\n"},"slideScrollBarTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Moves the scroll bar(s) to the given position with the slide animation.</p>\n"},"slideTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Scrolls to the given position with the slide animation.</p>\n"},"startup":{"!type":"fn()"},"stopAnimation":{"!type":"fn()","!doc":"<p>Stops the currently running animation.</p>\n"}}},"dojox/mobile/scrollable":{"!doc":"<p>Mixin for enabling touch scrolling capability.</p>\n","prototype":{"_moved":{"!doc":"<p>Flag that signals if the user have moved in (one of) the scroll\ndirection(s) since touch start (a move under the threshold is ignored).</p>\n"},"_parentPadBorderExtentsBottom":{"!doc":"<p>For Tooltip.js.</p>\n"},"constraint":{"!doc":"<p>bounce back to the content area</p>\n"},"dirLock":{"!doc":"<p>disable the move handler if scroll starts in the unexpected direction</p>\n"},"disableFlashScrollBar":{},"fadeScrollBar":{},"fixedFooterHeight":{"!doc":"<p>height of a fixed footer</p>\n"},"fixedHeaderHeight":{"!doc":"<p>height of a fixed header</p>\n"},"height":{"!doc":"<p>explicitly specified height of this widget (ex. &quot;300px&quot;)</p>\n"},"isLocalFooter":{"!doc":"<p>footer is view-local (as opposed to application-wide)</p>\n"},"propagatable":{"!doc":"<p>let touchstart event propagate up</p>\n"},"scrollBar":{"!doc":"<p>show scroll bar or not</p>\n"},"scrollDir":{"!doc":"<p>v: vertical, h: horizontal, vh: both, f: flip</p>\n"},"scrollType":{"!doc":"<ul>\n<li>1: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)animation for slide animation</li>\n<li>2: use top/left style,</li>\n<li>3: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)transition for slide animation</li>\n<li>0: use default value (3 for Android, iOS6+, and BlackBerry; otherwise 1)</li>\n</ul>\n"},"threshold":{"!doc":"<p>drag threshold value in pixels</p>\n"},"touchNode":{"!doc":"<p>a node that will have touch event handlers</p>\n"},"weight":{"!doc":"<p>frictional drag</p>\n"},"abort":{"!type":"fn()","!doc":"<p>Aborts scrolling.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"adjustDestination":{"!type":"fn(to: ?, pos: ?, dim: ?) -> bool","!doc":"<p>A stub function to be overridden by subclasses.</p>\n"},"calcScrollBarPos":{"!type":"fn(to: ?) -> ?","!doc":"<p>Calculates the scroll bar position.</p>\n"},"calcSpeed":{"!type":"fn(distance: ?, time: ?) -> number","!doc":"<p>Calculate the speed given the distance and time.</p>\n"},"cleanup":{"!type":"fn()","!doc":"<p>Uninitialize the module.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Mixin for enabling touch scrolling capability.</p>\n"},"createMask":{"!type":"fn()","!doc":"<p>Creates a mask for a scroll bar edge.</p>\n"},"findDisp":{"!type":"fn(node: ?) -> +null","!doc":"<p>Finds the currently displayed view node from my sibling nodes.</p>\n"},"flashScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar instantly.</p>\n"},"getDim":{"!type":"fn() -> ?","!doc":"<p>Returns various internal dimensional information needed for calculation.</p>\n"},"getPos":{"!type":"fn() -> ?","!doc":"<p>Gets the top position in the midst of animation.</p>\n"},"getScreenSize":{"!type":"fn() -> ?","!doc":"<p>Returns the dimensions of the browser window.</p>\n"},"getSpeed":{"!type":"fn() -> ?","!doc":"<p>Returns an object that indicates the scrolling speed.</p>\n"},"hideScrollBar":{"!type":"fn()","!doc":"<p>Hides the scroll bar.</p>\n"},"init":{"!type":"fn(params?: ?)","!doc":"<p>Initialize according to the given params.</p>\n"},"isFormElement":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the given node is a form control.</p>\n"},"isTopLevel":{"!type":"fn() -> bool","!doc":"<p>Returns true if this is a top-level widget.</p>\n"},"makeTranslateStr":{"!type":"fn(to: ?) -> string","!doc":"<p>Constructs a string value that is passed to the -webkit-transform property.</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"resetScrollBar":{"!type":"fn()","!doc":"<p>Resets the scroll bar length, position, etc.</p>\n"},"resize":{"!type":"fn(e: ?)","!doc":"<p>Adjusts the height of the widget.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, alignWithTop?: ?, duration?: ?)","!doc":"<p>Scrolls the pane until the searching node is in the view.</p>\n"},"scrollScrollBarTo":{"!type":"fn(to: ?)","!doc":"<p>Moves the scroll bar(s) to the given position without animation.</p>\n"},"scrollTo":{"!type":"fn(to: ?, doNotMoveScrollBar?: ?, node?: ?)","!doc":"<p>Scrolls to the given position immediately without animation.</p>\n"},"setKeyframes":{"!type":"fn(from: ?, to: ?, idx: ?)","!doc":"<p>Programmatically sets key frames for the scroll animation.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)","!doc":"<p>Sets the given node as selectable or unselectable.</p>\n"},"showScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar.</p>\n"},"slideScrollBarTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Moves the scroll bar(s) to the given position with the slide animation.</p>\n"},"slideTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Scrolls to the given position with the slide animation.</p>\n"},"stopAnimation":{"!type":"fn()","!doc":"<p>Stops the currently running animation.</p>\n"}}},"dojox/mobile/sniff":{"!doc":"<p>Deprecated: use dojo/sniff instead.\nOn iOS, dojox/mobile/sniff sets &quot;iphone&quot; to the same value as &quot;ios&quot;\nfor compatibility with earlier versions, but this should be considered deprecated.\nIn future versions, &quot;iphone&quot; will be set only when running on an iPhone (not iPad on iPod).</p>\n"},"dojox/mobile/_css3":{"!doc":"<p>This module provide some cross-browser support for CSS3 properties.</p>\n","add":{"!type":"fn(styles: ?, css3Styles: ?) -> +any","!doc":"<p>Prefixes all property names in &quot;css3Styles&quot; and adds the prefixed properties in &quot;styles&quot;.\nUsed as a convenience when an object is passed to domStyle.set to set multiple styles.</p>\n"},"name":{"!type":"fn(p: ?, hyphen?: ?) -> string","!doc":"<p>Returns the name of a CSS3 property with the correct prefix depending on the browser.</p>\n"}},"dojox/mobile/_maskUtils":{"!doc":"<p>Utility methods to clip rounded corners of various elements (Switch, ScrollablePane, scrollbars in scrollable widgets).\nUses -webkit-mask-image on webkit, or SVG on other browsers.</p>\n","createRoundMask":{"!type":"fn(node: ?, x: ?, y: ?, r: ?, b: ?, w: ?, h: ?, rx: ?, ry: ?, e: ?)","!doc":"<p>Creates and sets a mask for the specified node.</p>\n"}},"dojox/atom/io/Connection":{"!type":"fn()","prototype":{"!doc":"<p>This object implements a transport layer for working with ATOM feeds and ATOM publishing protocols.</p>\n","alertsEnabled":{},"preventCache":{},"addEntry":{"!type":"fn(entry: ?, url: ?, callback: ?, errorCallback: ?, retrieveEntry: ?, scope: ?) -> +any","!doc":"<p>Function to add a new ATOM entry by posting the new entry via APP.</p>\n"},"constructor":{"!type":"fn(sync: ?, preventCache: ?)","!doc":"<p>initializer</p>\n"},"deleteEntry":{"!type":"fn(entry: ?, callback: ?, errorCallback: ?, xmethod: ?, scope: ?) -> +any","!doc":"<p>Function to delete a specific ATOM entry via APP.</p>\n"},"getEntry":{"!type":"fn(url: ?, callback: ?, errorCallback: ?, scope: ?) -> +any","!doc":"<p>Function to retrieve a single entry from an ATOM feed from the given URL.</p>\n"},"getFeed":{"!type":"fn(url: ?, callback: ?, errorCallback: ?, scope: ?) -> +any","!doc":"<p>Function to obtain a s specific ATOM feed from a given ATOM Feed url.</p>\n"},"getService":{"!type":"fn(url: ?, callback: ?, errorCallback: ?, scope: ?) -> +any","!doc":"<p>Function to retrieve an introspection document from the given URL.</p>\n"},"updateEntry":{"!type":"fn(entry: ?, callback: ?, errorCallback: ?, retrieveUpdated: ?, xmethod: ?, scope: ?) -> +any","!doc":"<p>Function to update a specific ATOM entry by putting the new changes via APP.</p>\n"}}},"dojox/atom/io/model":{"_actions":{"!doc":"<p>Container for tag handling functions.</p>\n"},"_Constants":{"!doc":"<p>Container for general constants.</p>\n"},"util":{},"AtomItem":{"!type":"fn()"},"Category":{"!type":"fn()"},"Collection":{"!type":"fn()"},"Content":{"!type":"fn()"},"Entry":{"!type":"fn()"},"Feed":{"!type":"fn()"},"Generator":{"!type":"fn()"},"Link":{"!type":"fn()"},"Node":{"!type":"fn()"},"Person":{"!type":"fn()"},"Service":{"!type":"fn()"},"Workspace":{"!type":"fn()"}},"dojox/atom/io/model._Constants":{"!doc":"<p>Container for general constants.</p>\n","APP_NS":{},"ATOM_NS":{},"ATOM_URI":{},"PURL_NS":{}},"dojox/atom/io/model._actions":{"!doc":"<p>Container for tag handling functions.</p>\n","node":{"!doc":"<p>The dom node containing the data</p>\n"},"obj":{"!doc":"<p>The object to insert data into.</p>\n"},"author":{"!type":"fn(obj: ?, node: ?)"},"category":{"!type":"fn(obj: ?, node: ?)"},"content":{"!type":"fn(obj: ?, node: ?)"},"contributor":{"!type":"fn(obj: ?, node: ?)"},"email":{"!type":"fn(obj: ?, node: ?)"},"entry":{"!type":"fn(obj: ?, node: ?)"},"generator":{"!type":"fn(obj: ?, node: ?)"},"icon":{"!type":"fn(obj: ?, node: ?)"},"id":{"!type":"fn(obj: ?, node: ?)"},"issued":{"!type":"fn(obj: ?, node: ?)"},"link":{"!type":"fn(obj: ?, node: ?)"},"modified":{"!type":"fn(obj: ?, node: ?)"},"name":{"!type":"fn(obj: ?, node: ?)"},"published":{"!type":"fn(obj: ?, node: ?)"},"rights":{"!type":"fn(obj: ?, node: ?)"},"subtitle":{"!type":"fn(obj: ?, node: ?)"},"summary":{"!type":"fn(obj: ?, node: ?)"},"title":{"!type":"fn(obj: ?, node: ?)"},"updated":{"!type":"fn(obj: ?, node: ?)"},"uri":{"!type":"fn(obj: ?, node: ?)"}},"dojox/atom/io/model.util":{"createDate":{"!type":"fn(node: ?) -> +any","!doc":"<p>Utility function to create a date from a DOM node&#39;s text content.</p>\n"},"escapeHtml":{"!type":"fn(str: ?) -> +any","!doc":"<p>Utility function to escape XML special characters in an HTML string.</p>\n"},"getNodename":{"!type":"fn(node: ?) -> string","!doc":"<p>Utility function to get a node name and deal with IE&#39;s bad handling of namespaces\non tag names.</p>\n"},"unEscapeHtml":{"!type":"fn(str: ?) -> +any","!doc":"<p>Utility function to un-escape XML special characters in an HTML string.</p>\n"}},"dojox/atom/io/model.Node":{"!type":"fn()","prototype":{"addAttribute":{"!type":"fn(name: ?, value: ?)"},"addContent":{"!type":"fn(content: ?)"},"buildFromDom":{"!type":"fn(node: ?)"},"constructor":{"!type":"fn(name_space: ?, name: ?, attributes: ?, content: ?, shortNs: ?)"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"toString":{"!type":"fn() -> ?"}}},"dojox/atom/io/model.AtomItem":{"!type":"fn()","prototype":{"!doc":"<p>Class container for generic Atom items.</p>\n","_accepts":{},"accept":{"!type":"fn(tag: ?) -> ?"},"addAttribute":{"!type":"fn(name: ?, value: ?)"},"addAuthor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addCategory":{"!type":"fn(scheme: ?, term: ?, label: ?)","!doc":"<p>Function to add in a category to the list of categories.</p>\n"},"addContent":{"!type":"fn(content: ?)"},"addContributor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addExtension":{"!type":"fn(name_space: ?, name: ?, attributes: ?, content: ?, shortNS: ?)","!doc":"<p>Function to add in an extension namespace into the item.</p>\n"},"addLink":{"!type":"fn(href: ?, rel: ?, hrefLang: ?, title: ?, type: ?)","!doc":"<p>Function to add in a link to the list of links.</p>\n"},"addNamespace":{"!type":"fn(fullName: ?, shortName: ?)"},"buildFromDom":{"!type":"fn(node: ?)"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"getCategories":{"!type":"fn(scheme: ?) -> ?","!doc":"<p>Function to get all categories that match a particular scheme.</p>\n"},"getExtensions":{"!type":"fn(name_space: ?, name: ?) -> []","!doc":"<p>Function to get extensions that match a namespace and name.</p>\n"},"removeBasicLinks":{"!type":"fn() -> number","!doc":"<p>Function to remove all basic links from the list of links.</p>\n"},"removeCategories":{"!type":"fn(scheme: ?, term: ?) -> number","!doc":"<p>Function to remove all categories that match a particular scheme and term.</p>\n"},"removeExtensions":{"!type":"fn(name_space: ?, name: ?)","!doc":"<p>Function to remove extensions that match a namespace and name.</p>\n"},"removeLink":{"!type":"fn(href: ?, rel: ?) -> number","!doc":"<p>Function to remove a link from the list of links.</p>\n"},"setTitle":{"!type":"fn(str: ?, type: ?)","!doc":"<p>Function to set the title of the item.</p>\n"},"toString":{"!type":"fn() -> ?"}}},"dojox/atom/io/model.Category":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;Category&#39; types.</p>\n","addAttribute":{"!type":"fn(name: ?, value: ?)"},"addContent":{"!type":"fn(content: ?)"},"buildFromDom":{"!type":"fn(node: ?)","!doc":"<p>Function to do construction of the Category data from the DOM node containing it.</p>\n"},"constructor":{"!type":"fn(scheme: ?, term: ?, label: ?)"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"toString":{"!type":"fn() -> ?","!doc":"<p>Function to construct string form of the category tag, which is an XML structure.</p>\n"}}},"dojox/atom/io/model.Content":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;Content&#39; types. Such as summary, content, username, and so on types of data.</p>\n","addAttribute":{"!type":"fn(name: ?, value: ?)"},"addContent":{"!type":"fn(content: ?)"},"buildFromDom":{"!type":"fn(node: ?)","!doc":"<p>Function to do construction of the Content data from the DOM node containing it.</p>\n"},"constructor":{"!type":"fn(tagName: ?, value: ?, src: ?, type: ?, xmlLang: ?)"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"toString":{"!type":"fn() -> ?","!doc":"<p>Function to construct string form of the content tag, which is an XML structure.</p>\n"}}},"dojox/atom/io/model.Link":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;link&#39; types.</p>\n","addAttribute":{"!type":"fn(name: ?, value: ?)"},"addContent":{"!type":"fn(content: ?)"},"buildFromDom":{"!type":"fn(node: ?)","!doc":"<p>Function to do construction of the link data from the DOM node containing it.</p>\n"},"constructor":{"!type":"fn(href: ?, rel: ?, hrefLang: ?, title: ?, type: ?)"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"toString":{"!type":"fn() -> ?","!doc":"<p>Function to construct string form of the link tag, which is an XML structure.</p>\n"}}},"dojox/atom/io/model.Person":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;person&#39; types, such as Author, contributors, and so on.</p>\n","_accepts":{},"accept":{"!type":"fn(tag: ?) -> ?"},"addAttribute":{"!type":"fn(name: ?, value: ?)"},"addContent":{"!type":"fn(content: ?)"},"buildFromDom":{"!type":"fn(node: ?)","!doc":"<p>Function to do construction of the person data from the DOM node containing it.</p>\n"},"constructor":{"!type":"fn(personType: ?, name: ?, email: ?, uri: ?)"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"toString":{"!type":"fn() -> ?","!doc":"<p>Function to construct string form of the Person tag, which is an XML structure.</p>\n"}}},"dojox/atom/io/model.Generator":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;Generator&#39; types.</p>\n","addAttribute":{"!type":"fn(name: ?, value: ?)"},"addContent":{"!type":"fn(content: ?)"},"buildFromDom":{"!type":"fn(node: ?)","!doc":"<p>Function to do construction of the generator data from the DOM node containing it.</p>\n"},"constructor":{"!type":"fn(uri: ?, version: ?, value: ?)"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"toString":{"!type":"fn() -> ?","!doc":"<p>Function to construct string form of the Generator tag, which is an XML structure.</p>\n"}}},"dojox/atom/io/model.Entry":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;Entry&#39; types.</p>\n","_accepts":{},"accept":{"!type":"fn(tag: ?) -> ?"},"addAttribute":{"!type":"fn(name: ?, value: ?)"},"addAuthor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addCategory":{"!type":"fn(scheme: ?, term: ?, label: ?)","!doc":"<p>Function to add in a category to the list of categories.</p>\n"},"addContent":{"!type":"fn(content: ?)"},"addContributor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addExtension":{"!type":"fn(name_space: ?, name: ?, attributes: ?, content: ?, shortNS: ?)","!doc":"<p>Function to add in an extension namespace into the item.</p>\n"},"addLink":{"!type":"fn(href: ?, rel: ?, hrefLang: ?, title: ?, type: ?)","!doc":"<p>Function to add in a link to the list of links.</p>\n"},"addNamespace":{"!type":"fn(fullName: ?, shortName: ?)"},"buildFromDom":{"!type":"fn(node: ?)"},"constructor":{"!type":"fn(id: ?)"},"destroy":{"!type":"fn()"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"getCategories":{"!type":"fn(scheme: ?) -> ?","!doc":"<p>Function to get all categories that match a particular scheme.</p>\n"},"getEditHref":{"!type":"fn() -> +any","!doc":"<p>Function to get the href that allows editing of this feed entry.</p>\n"},"getExtensions":{"!type":"fn(name_space: ?, name: ?) -> []","!doc":"<p>Function to get extensions that match a namespace and name.</p>\n"},"removeBasicLinks":{"!type":"fn() -> number","!doc":"<p>Function to remove all basic links from the list of links.</p>\n"},"removeCategories":{"!type":"fn(scheme: ?, term: ?) -> number","!doc":"<p>Function to remove all categories that match a particular scheme and term.</p>\n"},"removeExtensions":{"!type":"fn(name_space: ?, name: ?)","!doc":"<p>Function to remove extensions that match a namespace and name.</p>\n"},"removeLink":{"!type":"fn(href: ?, rel: ?) -> number","!doc":"<p>Function to remove a link from the list of links.</p>\n"},"setEditHref":{"!type":"fn(url: ?)"},"setTitle":{"!type":"fn(str: ?, type: ?)","!doc":"<p>Function to set the title of the item.</p>\n"},"toString":{"!type":"fn(amPrimary: ?) -> ?","!doc":"<p>Function to construct string form of the entry tag, which is an XML structure.</p>\n"}}},"dojox/atom/io/model.Feed":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;Feed&#39; types.</p>\n","_accepts":{},"accept":{"!type":"fn(tag: ?) -> ?"},"addAttribute":{"!type":"fn(name: ?, value: ?)"},"addAuthor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addCategory":{"!type":"fn(scheme: ?, term: ?, label: ?)","!doc":"<p>Function to add in a category to the list of categories.</p>\n"},"addContent":{"!type":"fn(content: ?)"},"addContributor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addEntry":{"!type":"fn(entry: ?)","!doc":"<p>Function to add an entry to this feed.</p>\n"},"addExtension":{"!type":"fn(name_space: ?, name: ?, attributes: ?, content: ?, shortNS: ?)","!doc":"<p>Function to add in an extension namespace into the item.</p>\n"},"addLink":{"!type":"fn(href: ?, rel: ?, hrefLang: ?, title: ?, type: ?)","!doc":"<p>Function to add in a link to the list of links.</p>\n"},"addNamespace":{"!type":"fn(fullName: ?, shortName: ?)"},"buildFromDom":{"!type":"fn(node: ?)"},"constructor":{"!type":"fn(args: ?)"},"createEntry":{"!type":"fn() -> +any","!doc":"<p>Function to Create a new entry object in the feed.</p>\n"},"destroy":{"!type":"fn()"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"getCategories":{"!type":"fn(scheme: ?) -> ?","!doc":"<p>Function to get all categories that match a particular scheme.</p>\n"},"getEntry":{"!type":"fn(entryId: ?) -> +any","!doc":"<p>Function to get an entry by its id.</p>\n"},"getExtensions":{"!type":"fn(name_space: ?, name: ?) -> []","!doc":"<p>Function to get extensions that match a namespace and name.</p>\n"},"getFirstEntry":{"!type":"fn() -> +any","!doc":"<p>Function to get the first entry of the feed.</p>\n"},"getSelfHref":{"!type":"fn() -> +any","!doc":"<p>Function to get the href that refers to this feed.</p>\n"},"removeBasicLinks":{"!type":"fn() -> number","!doc":"<p>Function to remove all basic links from the list of links.</p>\n"},"removeCategories":{"!type":"fn(scheme: ?, term: ?) -> number","!doc":"<p>Function to remove all categories that match a particular scheme and term.</p>\n"},"removeEntry":{"!type":"fn(entry: ?) -> number","!doc":"<p>Function to remove an entry from the list of links.</p>\n"},"removeExtensions":{"!type":"fn(name_space: ?, name: ?)","!doc":"<p>Function to remove extensions that match a namespace and name.</p>\n"},"removeLink":{"!type":"fn(href: ?, rel: ?) -> number","!doc":"<p>Function to remove a link from the list of links.</p>\n"},"setEntries":{"!type":"fn(arrayOfEntry: ?)","!doc":"<p>Function to add a set of entries to the feed.</p>\n"},"setTitle":{"!type":"fn(str: ?, type: ?)","!doc":"<p>Function to set the title of the item.</p>\n"},"toString":{"!type":"fn() -> ?","!doc":"<p>Function to construct string form of the feed tag, which is an XML structure.</p>\n"}}},"dojox/atom/io/model.Service":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;Feed&#39; types.</p>\n","_accepts":{},"accept":{"!type":"fn(tag: ?) -> ?"},"addAttribute":{"!type":"fn(name: ?, value: ?)"},"addAuthor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addCategory":{"!type":"fn(scheme: ?, term: ?, label: ?)","!doc":"<p>Function to add in a category to the list of categories.</p>\n"},"addContent":{"!type":"fn(content: ?)"},"addContributor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addExtension":{"!type":"fn(name_space: ?, name: ?, attributes: ?, content: ?, shortNS: ?)","!doc":"<p>Function to add in an extension namespace into the item.</p>\n"},"addLink":{"!type":"fn(href: ?, rel: ?, hrefLang: ?, title: ?, type: ?)","!doc":"<p>Function to add in a link to the list of links.</p>\n"},"addNamespace":{"!type":"fn(fullName: ?, shortName: ?)"},"buildFromDom":{"!type":"fn(node: ?)","!doc":"<p>Function to do construction of the Service data from the DOM node containing it.</p>\n"},"constructor":{"!type":"fn(href: ?)"},"destroy":{"!type":"fn()"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"getCategories":{"!type":"fn(scheme: ?) -> ?","!doc":"<p>Function to get all categories that match a particular scheme.</p>\n"},"getCollection":{"!type":"fn(url: ?) -> ?","!doc":"<p>Function to collections that match a specific url.</p>\n"},"getExtensions":{"!type":"fn(name_space: ?, name: ?) -> []","!doc":"<p>Function to get extensions that match a namespace and name.</p>\n"},"removeBasicLinks":{"!type":"fn() -> number","!doc":"<p>Function to remove all basic links from the list of links.</p>\n"},"removeCategories":{"!type":"fn(scheme: ?, term: ?) -> number","!doc":"<p>Function to remove all categories that match a particular scheme and term.</p>\n"},"removeExtensions":{"!type":"fn(name_space: ?, name: ?)","!doc":"<p>Function to remove extensions that match a namespace and name.</p>\n"},"removeLink":{"!type":"fn(href: ?, rel: ?) -> number","!doc":"<p>Function to remove a link from the list of links.</p>\n"},"setTitle":{"!type":"fn(str: ?, type: ?)","!doc":"<p>Function to set the title of the item.</p>\n"},"toString":{"!type":"fn() -> ?"}}},"dojox/atom/io/model.Workspace":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;Workspace&#39; types.</p>\n","_accepts":{},"accept":{"!type":"fn(tag: ?) -> ?"},"addAttribute":{"!type":"fn(name: ?, value: ?)"},"addAuthor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addCategory":{"!type":"fn(scheme: ?, term: ?, label: ?)","!doc":"<p>Function to add in a category to the list of categories.</p>\n"},"addContent":{"!type":"fn(content: ?)"},"addContributor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addExtension":{"!type":"fn(name_space: ?, name: ?, attributes: ?, content: ?, shortNS: ?)","!doc":"<p>Function to add in an extension namespace into the item.</p>\n"},"addLink":{"!type":"fn(href: ?, rel: ?, hrefLang: ?, title: ?, type: ?)","!doc":"<p>Function to add in a link to the list of links.</p>\n"},"addNamespace":{"!type":"fn(fullName: ?, shortName: ?)"},"buildFromDom":{"!type":"fn(node: ?)","!doc":"<p>Function to do construction of the Workspace data from the DOM node containing it.</p>\n"},"constructor":{"!type":"fn(title: ?)"},"destroy":{"!type":"fn()"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"getCategories":{"!type":"fn(scheme: ?) -> ?","!doc":"<p>Function to get all categories that match a particular scheme.</p>\n"},"getExtensions":{"!type":"fn(name_space: ?, name: ?) -> []","!doc":"<p>Function to get extensions that match a namespace and name.</p>\n"},"removeBasicLinks":{"!type":"fn() -> number","!doc":"<p>Function to remove all basic links from the list of links.</p>\n"},"removeCategories":{"!type":"fn(scheme: ?, term: ?) -> number","!doc":"<p>Function to remove all categories that match a particular scheme and term.</p>\n"},"removeExtensions":{"!type":"fn(name_space: ?, name: ?)","!doc":"<p>Function to remove extensions that match a namespace and name.</p>\n"},"removeLink":{"!type":"fn(href: ?, rel: ?) -> number","!doc":"<p>Function to remove a link from the list of links.</p>\n"},"setTitle":{"!type":"fn(str: ?, type: ?)","!doc":"<p>Function to set the title of the item.</p>\n"},"toString":{"!type":"fn() -> ?"}}},"dojox/atom/io/model.Collection":{"!type":"fn()","prototype":{"!doc":"<p>Class container for &#39;Collection&#39; types.</p>\n","_accepts":{},"accept":{"!type":"fn(tag: ?) -> ?"},"addAttribute":{"!type":"fn(name: ?, value: ?)"},"addAuthor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addCategory":{"!type":"fn(scheme: ?, term: ?, label: ?)","!doc":"<p>Function to add in a category to the list of categories.</p>\n"},"addContent":{"!type":"fn(content: ?)"},"addContributor":{"!type":"fn(name: ?, email: ?, uri: ?)","!doc":"<p>Function to add in an author to the list of authors.</p>\n"},"addExtension":{"!type":"fn(name_space: ?, name: ?, attributes: ?, content: ?, shortNS: ?)","!doc":"<p>Function to add in an extension namespace into the item.</p>\n"},"addLink":{"!type":"fn(href: ?, rel: ?, hrefLang: ?, title: ?, type: ?)","!doc":"<p>Function to add in a link to the list of links.</p>\n"},"addNamespace":{"!type":"fn(fullName: ?, shortName: ?)"},"buildFromDom":{"!type":"fn(node: ?)","!doc":"<p>Function to do construction of the Collection data from the DOM node containing it.</p>\n"},"constructor":{"!type":"fn(href: ?, title: ?)"},"destroy":{"!type":"fn()"},"getAttribute":{"!type":"fn(name: ?) -> ?"},"getCategories":{"!type":"fn(scheme: ?) -> ?","!doc":"<p>Function to get all categories that match a particular scheme.</p>\n"},"getExtensions":{"!type":"fn(name_space: ?, name: ?) -> []","!doc":"<p>Function to get extensions that match a namespace and name.</p>\n"},"removeBasicLinks":{"!type":"fn() -> number","!doc":"<p>Function to remove all basic links from the list of links.</p>\n"},"removeCategories":{"!type":"fn(scheme: ?, term: ?) -> number","!doc":"<p>Function to remove all categories that match a particular scheme and term.</p>\n"},"removeExtensions":{"!type":"fn(name_space: ?, name: ?)","!doc":"<p>Function to remove extensions that match a namespace and name.</p>\n"},"removeLink":{"!type":"fn(href: ?, rel: ?) -> number","!doc":"<p>Function to remove a link from the list of links.</p>\n"},"setTitle":{"!type":"fn(str: ?, type: ?)","!doc":"<p>Function to set the title of the item.</p>\n"},"toString":{"!type":"fn() -> ?"}}},"dojox/xml/parser":{"innerXML":{"!type":"fn(node: ?) -> ?","!doc":"<p>Implementation of MS&#39;s innerXML function.</p>\n"},"parse":{"!type":"fn(str?: ?, mimetype?: ?) -> ?","!doc":"<p>cross-browser implementation of creating an XML document object from null, empty string, and XML text..\n</p>\n"},"removeChildren":{"!type":"fn(node: ?) -> ?","!doc":"<p>removes all children from node and returns the count of children removed.\nThe children nodes are not destroyed. Be sure to call dojo.destroy on them\nafter they are not used anymore.</p>\n"},"replaceChildren":{"!type":"fn(node: ?, newChildren: ?) -> string","!doc":"<p>Removes all children of node and appends newChild. All the existing\nchildren will be destroyed.</p>\n"},"textContent":{"!type":"fn(node: ?, text?: ?) -> ?","!doc":"<p>Implementation of the DOM Level 3 attribute; scan node for text</p>\n"}},"dojox/atom/widget/FeedEntryEditor":{"!type":"fn()","prototype":{"!proto":"dojox/atom/widget/FeedEntryViewer.prototype","!doc":"<p>An ATOM feed entry editor that allows viewing of the individual attributes of an entry.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_contentEditor":{},"_contentEditorCreator":{},"_displayEntrySections":{},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_editable":{},"_editMode":{},"_editors":{},"_entry":{},"_feed":{},"_focusManager":{},"_oldContent":{},"_optionButtonDisplayed":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setObject":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_validEntryFields":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayEntrySections":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"enableEdit":{},"enableMenu":{},"enableMenuFade":{},"entryNewButton":{},"entrySelectionTopic":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"cancelEdits":{"!type":"fn() -> +any","!doc":"<p>Cancels edits and reverts the editor to its previous state (display mode)</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Clears the editor, destroys all editors, leaving the editor completely clear</p>\n"},"clearEditors":{"!type":"fn()"},"clearNodes":{"!type":"fn()","!doc":"<p>Function to clear all the display nodes for the ATOM entry from the viewer.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getEntry":{"!type":"fn() -> ?"},"getFeed":{"!type":"fn() -> ?"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFieldValid":{"!type":"fn(field: ?) -> +any","!doc":"<p>Function to return if a displayable field is valid or not</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"saveEdits":{"!type":"fn() -> +any","!doc":"<p>Saves edits submitted when the &#39;save&#39; button is pressed.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setAuthors":{"!type":"fn(authorsAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the author node in the template to some value from the entry.</p>\n"},"setAuthorsHeader":{"!type":"fn(authorHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the title format for the authors section of the author row in the template to some value from the entry.</p>\n"},"setContent":{"!type":"fn(contentAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the content node in the template to some value from the entry.</p>\n"},"setContentHeader":{"!type":"fn(contentHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the content node in the template to some value from the entry.</p>\n"},"setContributors":{"!type":"fn(contributorsAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the contributor node in the template to some value from the entry.</p>\n"},"setContributorsHeader":{"!type":"fn(contributorsHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the contributor header node in the template to some value from the entry.</p>\n"},"setDisplaySections":{"!type":"fn(sectionsArray: ?) -> +any","!doc":"<p>Function for setting which sections of the entry should be displayed.</p>\n"},"setEntry":{"!type":"fn(entry: ?, feed: ?, leaveMenuState: ?)","!doc":"<p>Function to set the current entry that is being edited.</p>\n"},"setFieldValidity":{"!type":"fn(field: ?, isValid: ?) -> +any","!doc":"<p>Function to set whether a field in the view is valid and displayable.</p>\n"},"setId":{"!type":"fn(idAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the ID  node in the template to some value from the entry.</p>\n"},"setIdHeader":{"!type":"fn(idHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the ID  node in the template to some value from the entry.</p>\n"},"setSummary":{"!type":"fn(summaryAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the summary  node in the template to some value from the entry.</p>\n"},"setSummaryHeader":{"!type":"fn(summaryHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the summary  node in the template to some value from the entry.</p>\n"},"setTitle":{"!type":"fn(titleAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the title node in the template to some value from the entry.</p>\n"},"setTitleHeader":{"!type":"fn(titleHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the title header node in the template to some value.</p>\n"},"setUpdated":{"!type":"fn(updatedAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the updated  node in the template to some value from the entry.</p>\n"},"setUpdatedHeader":{"!type":"fn(updatedHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the updated header node in the template to some value from the entry.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/atom/widget/FeedEntryViewer":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>An ATOM feed entry editor for publishing updated ATOM entries, or viewing non-editable entries.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_displayEntrySections":{},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_editMode":{},"_entry":{},"_feed":{},"_focusManager":{},"_optionButtonDisplayed":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_validEntryFields":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayEntrySections":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"enableMenu":{},"enableMenuFade":{},"entrySelectionTopic":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Function to clear the state of the widget.</p>\n"},"clearNodes":{"!type":"fn()","!doc":"<p>Function to clear all the display nodes for the ATOM entry from the viewer.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getEntry":{"!type":"fn() -> ?"},"getFeed":{"!type":"fn() -> ?"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFieldValid":{"!type":"fn(field: ?) -> +any","!doc":"<p>Function to return if a displayable field is valid or not</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setAuthors":{"!type":"fn(authorsAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the author node in the template to some value from the entry.</p>\n"},"setAuthorsHeader":{"!type":"fn(authorHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the title format for the authors section of the author row in the template to some value from the entry.</p>\n"},"setContent":{"!type":"fn(contentAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the content node in the template to some value from the entry.</p>\n"},"setContentHeader":{"!type":"fn(contentHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the content node in the template to some value from the entry.</p>\n"},"setContributors":{"!type":"fn(contributorsAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the contributor node in the template to some value from the entry.</p>\n"},"setContributorsHeader":{"!type":"fn(contributorsHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the contributor header node in the template to some value from the entry.</p>\n"},"setDisplaySections":{"!type":"fn(sectionsArray: ?) -> +any","!doc":"<p>Function for setting which sections of the entry should be displayed.</p>\n"},"setEntry":{"!type":"fn(entry: ?, feed: ?, leaveMenuState: ?)","!doc":"<p>Function to set the current entry that is being edited.</p>\n"},"setFieldValidity":{"!type":"fn(field: ?, isValid: ?) -> +any","!doc":"<p>Function to set whether a field in the view is valid and displayable.</p>\n"},"setId":{"!type":"fn(idAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the ID  node in the template to some value from the entry.</p>\n"},"setIdHeader":{"!type":"fn(idHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the ID  node in the template to some value from the entry.</p>\n"},"setSummary":{"!type":"fn(summaryAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the summary  node in the template to some value from the entry.</p>\n"},"setSummaryHeader":{"!type":"fn(summaryHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the summary  node in the template to some value from the entry.</p>\n"},"setTitle":{"!type":"fn(titleAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the title node in the template to some value from the entry.</p>\n"},"setTitleHeader":{"!type":"fn(titleHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the title header node in the template to some value.</p>\n"},"setUpdated":{"!type":"fn(updatedAnchorNode: ?, editMode: ?, entry: ?)","!doc":"<p>Function to set the contents of the updated  node in the template to some value from the entry.</p>\n"},"setUpdatedHeader":{"!type":"fn(updatedHeaderNode: ?, entry: ?)","!doc":"<p>Function to set the contents of the updated header node in the template to some value from the entry.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"EntryHeader":{"!type":"fn()"}},"dojox/atom/widget/FeedEntryViewer.EntryHeader":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Widget representing a header in a FeedEntryViewer/Editor</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clear":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setListHeader":{"!type":"fn(title: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/atom/widget/FeedViewer":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>An ATOM feed viewer that allows for viewing a feed, deleting entries, and editing entries.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_currentSelection":{},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_feed":{},"_focusManager":{},"_includeFilters":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alertsEnabled":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"entrySelectionTopic":{},"feedViewerTable":{},"feedViewerTableBody":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"localSaveOnly":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"url":{},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"xmethod":{},"addCategoryIncludeFilter":{"!type":"fn(filter: ?) -> +any","!doc":"<p>Function to add a filter for entry inclusion in the feed view.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"appendEntry":{"!type":"fn(entry: ?) -> +any","!doc":"<p>Function for appending an entry to the feed view.</p>\n"},"appendGrouping":{"!type":"fn(titleText: ?) -> +any","!doc":"<p>Function for appending a new grouping of entries to the feed view.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clear":{"!type":"fn() -> +any","!doc":"<p>Function clearing all current entries in the feed view.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteEntry":{"!type":"fn(entryRow: ?)","!doc":"<p>Function for deleting a row from the view</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this widget, including all descendants and subscriptions.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>The postCreate function.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeCategoryIncludeFilter":{"!type":"fn(filter: ?) -> +any","!doc":"<p>Function to remove a filter for entry inclusion in the feed view.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setFeed":{"!type":"fn(feed: ?) -> +any","!doc":"<p>Function setting the dojox.atom.io.model.Feed data into the view.</p>\n"},"setFeedFromUrl":{"!type":"fn(url: ?) -> +any","!doc":"<p>Function setting the feed from a URL which to get the feed.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>The startup function.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"AtomEntryCategoryFilter":{"!type":"fn()"},"CategoryIncludeFilter":{"!type":"fn()"},"FeedViewerEntry":{"!type":"fn()"},"FeedViewerGrouping":{"!type":"fn()"}},"dojox/atom/widget/FeedViewer.FeedViewerEntry":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Widget for handling the display of an entry and specific events associated with it.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"deleteButton":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"entry":{},"entryNode":{},"feed":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"timeNode":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteEntry":{"!type":"fn(event: ?) -> +any","!doc":"<p>Function to handle the delete event and delete the entry.</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disableDelete":{"!type":"fn() -> +any","!doc":"<p>Function to disable the delete action on this entry.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"enableDelete":{"!type":"fn() -> +any","!doc":"<p>Function to enable the delete action on this entry.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setTime":{"!type":"fn(timeText: ?) -> +any","!doc":"<p>Function to set the time of the entry.</p>\n"},"setTitle":{"!type":"fn(text: ?) -> +any","!doc":"<p>Function to set the title of the entry.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/atom/widget/FeedViewer.FeedViewerGrouping":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Grouping of feed entries.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"groupingNode":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"titleNode":{},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setText":{"!type":"fn(text: ?)","!doc":"<p>Sets the text to be shown above this grouping.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/atom/widget/FeedViewer.AtomEntryCategoryFilter":{"!type":"fn()","prototype":{"!doc":"<p>A filter to be applied to the list of entries.</p>\n","isFilter":{},"label":{},"scheme":{},"term":{}}},"dojox/atom/widget/FeedViewer.CategoryIncludeFilter":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(scheme: ?, term: ?, label: ?)","!doc":"<p>The initializer function.</p>\n"},"match":{"!type":"fn(entry: ?) -> +any","!doc":"<p>Function to determine if this category filter matches against a category on an atom entry</p>\n"}}},"dojox/av/FLAudio":{"!type":"fn()","prototype":{"!doc":"<p>Play MP3 files through the Flash SWF built in the\nDEFT project.</p>\n","_swfPath":{"!doc":"<p>The path to the video player SWF resource</p>\n"},"allowNetworking":{"!doc":"<p>Whether SWF is restricted to a domain</p>\n"},"allowScriptAccess":{"!doc":"<p>Whether the SWF can access the container JS</p>\n"},"id":{"!doc":"<p>The id of this widget and the id of the SWF movie.</p>\n"},"initialPan":{"!doc":"<p>From -1 to 1 (-1 is left, 1 is right, 0 is middle)\nSets pan for all files unless changed with play\nor setPan</p>\n"},"initialVolume":{"!doc":"<p>From 0-1\nSets volume for all files unless changed with doPlay\nor setVolume</p>\n"},"isDebug":{"!doc":"<p>Setting to true tells the SWF to output log messages to Firebug.</p>\n"},"statusInterval":{"!doc":"<p>How often in milliseconds that the status of the\nplayer is checked - both load and play</p>\n"},"constructor":{"!type":"fn(options: ?)"},"destroy":{"!type":"fn()","!doc":"<p>destroys flash</p>\n"},"doPlay":{"!type":"fn(options: ?)","!doc":"<p>Tell media to play, based on\nthe options passed.</p>\n"},"getPan":{"!type":"fn(options: ?) -> ?","!doc":"<p>Set media pan, based on identifier in\nthe options passed.</p>\n"},"getPosition":{"!type":"fn(options: ?) -> ?","!doc":"<p>Get the current time.</p>\n"},"getVolume":{"!type":"fn(options: ?) -> ?","!doc":"<p>Get media volume, based on identifier in\nthe options passed.</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initialize the media.</p>\n"},"load":{"!type":"fn(options: ?) -> +any","!doc":"<p>Adds a media object to the playlist\n<em>*</em>This can be called repeatedly to add multiple items.</p>\n"},"pause":{"!type":"fn(options: ?)","!doc":"<p>Tell media to pause, based on identifier in\nthe options passed.</p>\n"},"setPan":{"!type":"fn(options: ?)","!doc":"<p>Set media pan, based on identifier in\nthe options passed.</p>\n"},"setVolume":{"!type":"fn(options: ?)","!doc":"<p>Set media volume, based on identifier in\nthe options passed.</p>\n"},"stop":{"!type":"fn(options: ?)","!doc":"<p>Tell media to stop, based on identifier in\nthe options passed.</p>\n"}}},"dojox/embed/Flash":{"!doc":"<p>Create a wrapper object around a Flash movie; this is the DojoX equivilent\nto SWFObject.\n</p>\n","available":{},"domNode":{},"id":{},"initialized":{},"minimumVersion":{},"movie":{},"prototype":{"constructor":{"!type":"fn(kwArgs: ?, node: ?)","!doc":"<p>Create a wrapper object around a Flash movie; this is the DojoX equivilent\nto SWFObject.\n</p>\n"}},"place":{"!type":"fn(kwArgs: ?, node: ?)"}},"dojox/timing/doLater":{"!doc":"<p>Check if a parameter is ready, and if not,\n&quot;do later&quot;. doLater will ping the parameter\nuntil it evaluates to something (truthy).\nIt thens calls the caller with original\narguments, using the supplied context or\nwindow.</p>\n","prototype":{"constructor":{"!type":"fn(conditional: ?, context?: ?, interval?: ?)","!doc":"<p>Check if a parameter is ready, and if not,\n&quot;do later&quot;. doLater will ping the parameter\nuntil it evaluates to something (truthy).\nIt thens calls the caller with original\narguments, using the supplied context or\nwindow.</p>\n"}}},"dojox/timing/_base":{"doLater":{"!type":"fn(conditional: ?, context?: ?, interval?: ?)","!doc":"<p>Check if a parameter is ready, and if not,\n&quot;do later&quot;. doLater will ping the parameter\nuntil it evaluates to something (truthy).\nIt thens calls the caller with original\narguments, using the supplied context or\nwindow.</p>\n"},"Timer":{"!type":"fn(interval: ?)","!doc":"<p>Timer object executes an &quot;onTick()&quot; method repeatedly at a specified interval.\nrepeatedly at a given interval.</p>\n"}},"dojox/av/FLVideo":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Inserts a Flash FLV video into the HTML page and provides methods\nand events for controlling the video. Also plays the H264/M4V codec\nwith a little trickery: change the &#39;.M4V&#39; extension to &#39;.flv&#39;.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_flashObject":{"!doc":"<p>The dojox.embed object</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_swfPath":{"!doc":"<p>The path to the video player SWF resource</p>\n"},"allowFullScreen":{"!doc":"<p>Whether to allow the SWF to go to fullscreen</p>\n"},"allowNetworking":{"!doc":"<p>Whether SWF is restricted to a domain</p>\n"},"allowScriptAccess":{"!doc":"<p>Whether the SWF can access the container JS</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoPlay":{"!doc":"<p>Whether the video automatically plays on load or not.</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"bufferTime":{"!doc":"<p>Time in milliseconds that the video should be loaded before it will\nplay. May pause and resume to build up buffer. Prevents stuttering.\n\n</p>\n<p>Note: Older FLVs, without a duration, cannot be buffered.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"flashMedia":{"!doc":"<p>The SWF object. Methods are passed to this.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"initialVolume":{"!doc":"<p>The initial volume setting of the player. Acccepts between 0 and 1.</p>\n"},"isDebug":{"!doc":"<p>Setting to true tells the SWF to output log messages to Firebug.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"mediaUrl":{},"minBufferTime":{"!doc":"<p>Time in milliseconds between the playhead time and loaded time that\nwill trigger the buffer. When buffer is triggered, video will pause\nuntil the bufferTime amount is buffered.\nNote: Should be a small number, greater than zero.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"percentDownloaded":{"!doc":"<p>The percentage the media has downloaded; from 0-100</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"updateTime":{"!doc":"<p>How often, in milliseconds to get an update of the video position.</p>\n"},"wmode":{"!doc":"<p>The render type of the SWF</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(options: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getTime":{"!type":"fn() -> ?","!doc":"<p>Returns the current time of the video</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"pause":{"!type":"fn()","!doc":"<p>Pauses the video</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"play":{"!type":"fn(newUrl?: ?)","!doc":"<p>Plays the video. If an url is passed in, plays the new link.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Initialize the media.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"seek":{"!type":"fn(time: ?)","!doc":"<p>Goes to the time passed in the argument</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"volume":{"!type":"fn(vol: ?) -> ?","!doc":"<p>Sets the volume of the video to the time in the</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/av/_Media":{"!type":"fn()","prototype":{"!doc":"<p>Used as a mixin for dojox and AIR media</p>\n","_flashObject":{"!doc":"<p>The dojox.embed object</p>\n"},"allowFullScreen":{"!doc":"<p>Whether to allow the SWF to go to fullscreen</p>\n"},"allowNetworking":{"!doc":"<p>Whether SWF is restricted to a domain</p>\n"},"allowScriptAccess":{"!doc":"<p>Whether the SWF can access the container JS</p>\n"},"autoPlay":{"!doc":"<p>Whether the video automatically plays on load or not.</p>\n"},"bufferTime":{"!doc":"<p>Time in milliseconds that the video should be loaded before it will\nplay. May pause and resume to build up buffer. Prevents stuttering.\n\n</p>\n<p>Note: Older FLVs, without a duration, cannot be buffered.</p>\n"},"flashMedia":{"!doc":"<p>The SWF object. Methods are passed to this.</p>\n"},"id":{"!doc":"<p>The id of this widget and the id of the SWF movie.</p>\n"},"initialVolume":{"!doc":"<p>The initial volume setting of the player. Acccepts between 0 and 1.</p>\n"},"isDebug":{"!doc":"<p>Setting to true tells the SWF to output log messages to Firebug.</p>\n"},"mediaUrl":{},"minBufferTime":{"!doc":"<p>Time in milliseconds between the playhead time and loaded time that\nwill trigger the buffer. When buffer is triggered, video will pause\nuntil the bufferTime amount is buffered.\nNote: Should be a small number, greater than zero.</p>\n"},"percentDownloaded":{"!doc":"<p>The percentage the media has downloaded; from 0-100</p>\n"},"updateTime":{"!doc":"<p>How often, in milliseconds to get an update of the video position.</p>\n"},"wmode":{"!doc":"<p>The render type of the SWF</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>destroys flash</p>\n"},"getTime":{"!type":"fn() -> ?","!doc":"<p>Returns the current time of the video</p>\n"}}},"dojox/av/widget/PlayButton":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A Play/Pause button widget to use with dojox.av.widget.Player</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Initialize button.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setMedia":{"!type":"fn(med: ?)","!doc":"<p>A common method to set the media in all Player widgets.\nMay do connections and initializations.</p>\n"},"showPause":{"!type":"fn()","!doc":"<p>Toggles the play button invisible and the pause\nbutton visible.</p>\n"},"showPlay":{"!type":"fn()","!doc":"<p>Toggles the pause button invisible and the play\nbutton visible..</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/av/widget/Player":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A Media Player UI widget for all types of dojox.av and AIR media.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"playerWidth":{"!doc":"<p>Sets the width of the player (not the video size)\nNumber will be converted to pixels\nString will be used literally. EX: &quot;320px&quot; or &quot;100%&quot;</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Do player styling, and place child widgets in the proper location.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Fired when all children are ready. Set the media in\nall children with setMedia()</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/av/widget/ProgressSlider":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A custom slider widget to use with dojox.av.widget.Player.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"endDrag":{"!type":"fn()","!doc":"<p>Fired on document.onmouseup.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleOut":{"!type":"fn()","!doc":"<p>Unhighlights handle onmouseover, or on endDrag.</p>\n"},"handleOver":{"!type":"fn()","!doc":"<p>Highlights the slider handle on mouseover, and\nstays highlighted during drag.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Initialize slider.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setHandle":{"!type":"fn(time: ?)","!doc":"<p>Sets the slider handle (when it is not being dragged)</p>\n"},"setLoadedPosition":{"!type":"fn(decimal: ?)","!doc":"<p>Sets the download progress bar to the percentage of how much\nthe media has been downloaded.</p>\n"},"setMedia":{"!type":"fn(med: ?, playerWidget: ?)","!doc":"<p>A common method to set the media in all Player widgets.\nMay do connections and initializations.</p>\n"},"startDrag":{"!type":"fn()","!doc":"<p>Fired onmousedown of the slider handle.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/av/widget/Status":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A Status widget to use with dojox.av.widget.Player</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setMedia":{"!type":"fn(med: ?)","!doc":"<p>A common method to set the media in all Player widgets.\nMay do connections and initializations.</p>\n"},"setStatus":{"!type":"fn(str: ?, isError: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toSeconds":{"!type":"fn(time: ?) -> ?"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/av/widget/VolumeButton":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A volume widget to use with dojox.av.widget.Player</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"endDrag":{"!type":"fn()","!doc":"<p>Fired on mouseup of the slider handle.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleOut":{"!type":"fn()","!doc":"<p>Unhighlights handle onmouseover, or on endDrag.\n</p>\n"},"handleOver":{"!type":"fn()","!doc":"<p>Highlights the slider handle on mouseover, and\nstays highlighted during drag.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Initialize the widget.\n</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setMedia":{"!type":"fn(med: ?)","!doc":"<p>A common method to set the media in all Player widgets.\nMay do connections and initializations.\n</p>\n"},"startDrag":{"!type":"fn()","!doc":"<p>Fired on mousedown of the slider handle.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateIcon":{"!type":"fn(vol: ?)","!doc":"<p>Changes the icon on the button according to volume level.\n</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/FuncGen":{"approx":{"!type":"fn(r: ?) -> number","!doc":"<p>Return a less exact approximation of r such that approx(r * (1 +- eps)) == approx(r)</p>\n"},"draw":{"!type":"fn(chart: ?, functionToGraph: ?, params: ?) -> ?","!doc":"<p>graph a chart with the given function.</p>\n"},"FuncGen":{"!type":"fn()"},"generatePoints":{"!type":"fn(funcToGraph: ?, x: ?, y: ?, width: ?, minX: ?, maxX: ?, minY: ?, maxY: ?) -> ?","!doc":"<p>create the points with information about the graph.</p>\n"},"Grapher":{"!type":"fn()"},"pow":{"!type":"fn(base: ?, exponent: ?) -> ?"},"toFrac":{"!type":"fn(number: ?) -> ?"}},"dojox/calc/FuncGen._Executor":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A graphing, scientific calculator\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteFunction":{"!type":"fn(name: ?)"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"eval":{"!type":"fn(text: ?) -> ?","!doc":"<p>create an anonymous function to run the code the parser generates from the user input.</p>\n"},"Function":{"!type":"fn(name: ?, args: ?, body: ?) -> ?","!doc":"<p>create an anonymous function to run the code the parser generates from the user input.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"normalizedFunction":{"!type":"fn(name: ?, args: ?, body: ?) -> ?"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/FuncGen.FuncGen":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dialog layout for making functions</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"functions":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readStore":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"writeStore":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>clear the name, arguments, and body</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteFunction":{"!type":"fn(name: ?)"},"deleteThing":{"!type":"fn(item: ?)","!doc":"<p>delete an item in the writestore</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"readyStatus":{"!type":"fn()","!doc":"<p>set the status in the template to ready</p>\n"},"reset":{"!type":"fn()","!doc":"<p>set the arguments and body to match a function selected if it exists in the function list</p>\n"},"saveFunction":{"!type":"fn(name: ?, args: ?, body: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>make sure the parent has a close button if it needs to be able to close\nlink the write store too</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/FuncGen.Grapher":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dialog layout for making graphs</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"chartIndex":{},"checkboxIndex":{},"class":{},"colorBoxFieldsetIndex":{},"colorIndex":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDownIndex":{},"evaluatedExpression":{},"expressionIndex":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"funcNumberIndex":{},"functionMode":{},"functionRef":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"statusIndex":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipIndex":{},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addXYAxes":{"!type":"fn(chart: ?) -> ?","!doc":"<p>add or re-add the default x/y axes to the Chart provided</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"changedColor":{"!type":"fn()","!doc":"<p>Make the color of the chart the new color.\nThe context is changed to the colorPalette, and a reference to chart was added to it a an attribute.</p>\n"},"checkDirty":{"!type":"fn()","!doc":"<p>adjust all charts in this.array according to any changes in window options</p>\n"},"checkDirty1":{"!type":"fn()","!doc":"<p>to stay in sync with onChange, checkDirty is called with a timeout</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createFunction":{"!type":"fn()","!doc":"<p>create a new row in the table with all of the dojo objects.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>deselect all checkboxes inside the function table</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawOne":{"!type":"fn(i: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"erase":{"!type":"fn(i: ?)","!doc":"<p>erase the chart inside this.array with the index i</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeDirty":{"!type":"fn()","!doc":"<p>if something in the window options is changed, this is called</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>add Event handlers, some additional attributes, etc</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"selectAll":{"!type":"fn()","!doc":"<p>select all checkboxes inside the function table</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setStatus":{"!type":"fn(i: ?, status: ?)","!doc":"<p>set the status of the row i to be status</p>\n"},"startup":{"!type":"fn()","!doc":"<p>make sure the parent has a close button if it needs to be able to close</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/math/_base":{},"dojox/calc/_Executor":{"approx":{"!type":"fn(r: ?) -> number","!doc":"<p>Return a less exact approximation of r such that approx(r * (1 +- eps)) == approx(r)</p>\n"},"draw":{"!type":"fn(chart: ?, functionToGraph: ?, params: ?) -> ?","!doc":"<p>graph a chart with the given function.</p>\n"},"FuncGen":{"!type":"fn()"},"generatePoints":{"!type":"fn(funcToGraph: ?, x: ?, y: ?, width: ?, minX: ?, maxX: ?, minY: ?, maxY: ?) -> ?","!doc":"<p>create the points with information about the graph.</p>\n"},"Grapher":{"!type":"fn()"},"pow":{"!type":"fn(base: ?, exponent: ?) -> ?"},"toFrac":{"!type":"fn(number: ?) -> ?"}},"dojox/calc/_Executor._Executor":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A graphing, scientific calculator\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteFunction":{"!type":"fn(name: ?)"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"eval":{"!type":"fn(text: ?) -> ?","!doc":"<p>create an anonymous function to run the code the parser generates from the user input.</p>\n"},"Function":{"!type":"fn(name: ?, args: ?, body: ?) -> ?","!doc":"<p>create an anonymous function to run the code the parser generates from the user input.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"normalizedFunction":{"!type":"fn(name: ?, args: ?, body: ?) -> ?"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/_Executor.FuncGen":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dialog layout for making functions</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"functions":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readStore":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"writeStore":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>clear the name, arguments, and body</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteFunction":{"!type":"fn(name: ?)"},"deleteThing":{"!type":"fn(item: ?)","!doc":"<p>delete an item in the writestore</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"readyStatus":{"!type":"fn()","!doc":"<p>set the status in the template to ready</p>\n"},"reset":{"!type":"fn()","!doc":"<p>set the arguments and body to match a function selected if it exists in the function list</p>\n"},"saveFunction":{"!type":"fn(name: ?, args: ?, body: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>make sure the parent has a close button if it needs to be able to close\nlink the write store too</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/_Executor.Grapher":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dialog layout for making graphs</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"chartIndex":{},"checkboxIndex":{},"class":{},"colorBoxFieldsetIndex":{},"colorIndex":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDownIndex":{},"evaluatedExpression":{},"expressionIndex":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"funcNumberIndex":{},"functionMode":{},"functionRef":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"statusIndex":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipIndex":{},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addXYAxes":{"!type":"fn(chart: ?) -> ?","!doc":"<p>add or re-add the default x/y axes to the Chart provided</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"changedColor":{"!type":"fn()","!doc":"<p>Make the color of the chart the new color.\nThe context is changed to the colorPalette, and a reference to chart was added to it a an attribute.</p>\n"},"checkDirty":{"!type":"fn()","!doc":"<p>adjust all charts in this.array according to any changes in window options</p>\n"},"checkDirty1":{"!type":"fn()","!doc":"<p>to stay in sync with onChange, checkDirty is called with a timeout</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createFunction":{"!type":"fn()","!doc":"<p>create a new row in the table with all of the dojo objects.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>deselect all checkboxes inside the function table</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawOne":{"!type":"fn(i: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"erase":{"!type":"fn(i: ?)","!doc":"<p>erase the chart inside this.array with the index i</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeDirty":{"!type":"fn()","!doc":"<p>if something in the window options is changed, this is called</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>add Event handlers, some additional attributes, etc</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"selectAll":{"!type":"fn()","!doc":"<p>select all checkboxes inside the function table</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setStatus":{"!type":"fn(i: ?, status: ?)","!doc":"<p>set the status of the row i to be status</p>\n"},"startup":{"!type":"fn()","!doc":"<p>make sure the parent has a close button if it needs to be able to close</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/GraphPro":{"!type":"fn()","prototype":{"!proto":"dojox/calc/Standard.prototype","!doc":"<p>The dialog widget for a graphing, scientific calculator</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"aFloatingPane":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"funcMaker":{},"functions":{},"grapher":{},"hasDisplay":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readStore":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"writeStore":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clearText":{"!type":"fn()","!doc":"<p>this clears the input box if it has content, but if it does not it clears the display</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"cycleCommandDown":{"!type":"fn()","!doc":"<p>cycle down through the list of commands the user has entered already</p>\n"},"cycleCommands":{"!type":"fn(count: ?, node: ?, event: ?)","!doc":"<p>Cycle through the commands that the user has entered.\nIt does not wrap around.</p>\n"},"cycleCommandUp":{"!type":"fn()","!doc":"<p>cycle up through the list of commands the user has entered already</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"executorLoaded":{"!type":"fn()","!doc":"<p>when executor loads check to see if the writestore is there</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"insertMinus":{"!type":"fn()","!doc":"<p>insert a minus sign when they press (-) in the combo button</p>\n"},"insertOperator":{"!type":"fn(newText: ?)","!doc":"<p>insert an operator with a button</p>\n"},"insertText":{"!type":"fn(newText: ?)"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"loadStore":{"!type":"fn(store: ?, isReadOnly: ?)","!doc":"<p>load an entire store, and make it publicly editable/viewable based on isReadOnly</p>\n"},"makeFunctionWindow":{"!type":"fn()","!doc":"<p>use this function to create a function window (with the button on the layout)</p>\n"},"makeGrapherWindow":{"!type":"fn()","!doc":"<p>use this to make a Grapher window appear with a button</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseTextbox":{"!type":"fn()","!doc":"<p>parse the contents of the textboxWidget and display the answer somewhere (depending on the layout)</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>run startup, see if there is an upper display box, etc</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"print":{"!type":"fn(text: ?, isRight: ?)","!doc":"<p>print the answer (typically) to the display or the input box</p>\n"},"putInAnsIfTextboxIsHighlighted":{"!type":"fn(node: ?) -> bool","!doc":"<p>try seeing if the textbox is highlighted completely so you know if Ans should be put in for an operator like +</p>\n"},"saveFunction":{"!type":"fn(name: ?, args: ?, body: ?)","!doc":"<p>make the function with executor</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setTextboxValue":{"!type":"fn(widget: ?, val: ?)","!doc":"<p>set a widget&#39;s value</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/Standard":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dialog layout for a standard 4 function/algebraic calculator</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"functions":{},"hasDisplay":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readStore":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"writeStore":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clearText":{"!type":"fn()","!doc":"<p>this clears the input box if it has content, but if it does not it clears the display</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"cycleCommandDown":{"!type":"fn()","!doc":"<p>cycle down through the list of commands the user has entered already</p>\n"},"cycleCommands":{"!type":"fn(count: ?, node: ?, event: ?)","!doc":"<p>Cycle through the commands that the user has entered.\nIt does not wrap around.</p>\n"},"cycleCommandUp":{"!type":"fn()","!doc":"<p>cycle up through the list of commands the user has entered already</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"executorLoaded":{"!type":"fn()","!doc":"<p>load in the stores after executor is loaded (the stores need executor to be loaded because it parses them)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"insertMinus":{"!type":"fn()","!doc":"<p>insert a minus sign when they press (-) in the combo button</p>\n"},"insertOperator":{"!type":"fn(newText: ?)","!doc":"<p>insert an operator with a button</p>\n"},"insertText":{"!type":"fn(newText: ?)"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"loadStore":{"!type":"fn(store: ?, isReadOnly: ?)","!doc":"<p>load an entire store, and make it publicly editable/viewable based on isReadOnly</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseTextbox":{"!type":"fn()","!doc":"<p>parse the contents of the textboxWidget and display the answer somewhere (depending on the layout)</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>run startup, see if there is an upper display box, etc</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"print":{"!type":"fn(text: ?, isRight: ?)","!doc":"<p>print the answer (typically) to the display or the input box</p>\n"},"putInAnsIfTextboxIsHighlighted":{"!type":"fn(node: ?) -> bool","!doc":"<p>try seeing if the textbox is highlighted completely so you know if Ans should be put in for an operator like +</p>\n"},"saveFunction":{"!type":"fn(name: ?, args: ?, body: ?)","!doc":"<p>make the function with executor</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setTextboxValue":{"!type":"fn(widget: ?, val: ?)","!doc":"<p>set a widget&#39;s value</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/Grapher":{"approx":{"!type":"fn(r: ?) -> number","!doc":"<p>Return a less exact approximation of r such that approx(r * (1 +- eps)) == approx(r)</p>\n"},"draw":{"!type":"fn(chart: ?, functionToGraph: ?, params: ?) -> ?","!doc":"<p>graph a chart with the given function.</p>\n"},"FuncGen":{"!type":"fn()"},"generatePoints":{"!type":"fn(funcToGraph: ?, x: ?, y: ?, width: ?, minX: ?, maxX: ?, minY: ?, maxY: ?) -> ?","!doc":"<p>create the points with information about the graph.</p>\n"},"Grapher":{"!type":"fn()"},"pow":{"!type":"fn(base: ?, exponent: ?) -> ?"},"toFrac":{"!type":"fn(number: ?) -> ?"}},"dojox/calc/Grapher._Executor":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A graphing, scientific calculator\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteFunction":{"!type":"fn(name: ?)"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"eval":{"!type":"fn(text: ?) -> ?","!doc":"<p>create an anonymous function to run the code the parser generates from the user input.</p>\n"},"Function":{"!type":"fn(name: ?, args: ?, body: ?) -> ?","!doc":"<p>create an anonymous function to run the code the parser generates from the user input.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"normalizedFunction":{"!type":"fn(name: ?, args: ?, body: ?) -> ?"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/Grapher.FuncGen":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dialog layout for making functions</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"functions":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readStore":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"writeStore":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>clear the name, arguments, and body</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteFunction":{"!type":"fn(name: ?)"},"deleteThing":{"!type":"fn(item: ?)","!doc":"<p>delete an item in the writestore</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"readyStatus":{"!type":"fn()","!doc":"<p>set the status in the template to ready</p>\n"},"reset":{"!type":"fn()","!doc":"<p>set the arguments and body to match a function selected if it exists in the function list</p>\n"},"saveFunction":{"!type":"fn(name: ?, args: ?, body: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>make sure the parent has a close button if it needs to be able to close\nlink the write store too</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/Grapher.Grapher":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dialog layout for making graphs</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"chartIndex":{},"checkboxIndex":{},"class":{},"colorBoxFieldsetIndex":{},"colorIndex":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDownIndex":{},"evaluatedExpression":{},"expressionIndex":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"funcNumberIndex":{},"functionMode":{},"functionRef":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"statusIndex":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipIndex":{},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addXYAxes":{"!type":"fn(chart: ?) -> ?","!doc":"<p>add or re-add the default x/y axes to the Chart provided</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"changedColor":{"!type":"fn()","!doc":"<p>Make the color of the chart the new color.\nThe context is changed to the colorPalette, and a reference to chart was added to it a an attribute.</p>\n"},"checkDirty":{"!type":"fn()","!doc":"<p>adjust all charts in this.array according to any changes in window options</p>\n"},"checkDirty1":{"!type":"fn()","!doc":"<p>to stay in sync with onChange, checkDirty is called with a timeout</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createFunction":{"!type":"fn()","!doc":"<p>create a new row in the table with all of the dojo objects.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>deselect all checkboxes inside the function table</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawOne":{"!type":"fn(i: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"erase":{"!type":"fn(i: ?)","!doc":"<p>erase the chart inside this.array with the index i</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeDirty":{"!type":"fn()","!doc":"<p>if something in the window options is changed, this is called</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>add Event handlers, some additional attributes, etc</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"selectAll":{"!type":"fn()","!doc":"<p>select all checkboxes inside the function table</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setStatus":{"!type":"fn(i: ?, status: ?)","!doc":"<p>set the status of the row i to be status</p>\n"},"startup":{"!type":"fn()","!doc":"<p>make sure the parent has a close button if it needs to be able to close</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/charting/Chart":{"!type":"fn()","prototype":{"!doc":"<p>The main chart object in dojox.charting.  This will create a two dimensional\nchart based on dojox.gfx.\n</p>\n","addAxis":{"!type":"fn(name: ?, kwArgs?: ?) -> +dojox","!doc":"<p>Add an axis to the chart, for rendering.</p>\n"},"addPlot":{"!type":"fn(name: ?, kwArgs: ?) -> +dojox","!doc":"<p>Add a new plot to the chart, defined by name and using the optional keyword arguments object.\nNote that dojox.charting assumes the main plot to be called &quot;default&quot;; if you do not have\na plot called &quot;default&quot; and attempt to add data series to the chart without specifying the\nplot to be rendered on, you WILL get errors.</p>\n"},"addSeries":{"!type":"fn(name: ?, data: ?, kwArgs?: ?) -> +dojox","!doc":"<p>Add a data series to the chart for rendering.</p>\n"},"calculateGeometry":{"!type":"fn() -> +dojox","!doc":"<p>Calculate the geometry of the chart based on the defined axes of\na chart.</p>\n"},"connectToPlot":{"!type":"fn(name: ?, object: ?, method: ?) -> []","!doc":"<p>A convenience method to connect a function to a plot.</p>\n"},"constructor":{"!type":"fn(node: ?, kwArgs?: ?) -> +dojox","!doc":"<p>The constructor for a new Chart.  Initializes all parameters used for a chart.</p>\n"},"delayedRender":{"!type":"fn() -> +dojox","!doc":"<p>Delayed render, which is used to collect multiple updates\nwithin a delayInMs time window.</p>\n"},"destroy":{"!type":"fn() -> +void","!doc":"<p>Cleanup when a chart is to be destroyed.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?) -> +dojox","!doc":"<p>Fires a synthetic event for a series item.</p>\n"},"formatTruncatedLabel":{"!type":"fn(element: ?, label: ?, labelType: ?)"},"fullGeometry":{"!type":"fn() -> +dojox","!doc":"<p>Calculate the full geometry of the chart.  This includes passing\nover all major elements of a chart (plots, axes, series, container)\nin order to ensure proper rendering.</p>\n"},"fullRender":{"!type":"fn() -> +dojox","!doc":"<p>Force a full rendering of the chart, including full resets on the chart itself.\nYou should not call this method directly unless absolutely necessary.</p>\n"},"getAxis":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Get the given axis, by name.</p>\n"},"getCoords":{"!type":"fn() -> ?","!doc":"<p>Get the coordinates and dimensions of the containing DOMNode, as\nreturned by dojo.coords.</p>\n"},"getGeometry":{"!type":"fn() -> ?","!doc":"<p>Returns a map of information about all axes in a chart and what they represent\nin terms of scaling (see dojox.charting.axis2d.Default.getScaler).</p>\n"},"getPlot":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Get the given plot, by name.</p>\n"},"getPlotOrder":{"!type":"fn() -> []","!doc":"<p>Returns an array of plot names in the current order\n(the top-most plot is the first).</p>\n"},"getSeries":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Get the given series, by name.</p>\n"},"getSeriesOrder":{"!type":"fn(plotName: ?) -> []","!doc":"<p>Returns an array of series names in the current order\n(the top-most series is the first) within a plot.</p>\n"},"movePlotToBack":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given plot to back.</p>\n"},"movePlotToFront":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given plot to front.</p>\n"},"moveSeriesToBack":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given series to back of a plot.</p>\n"},"moveSeriesToFront":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given series to front of a plot.</p>\n"},"removeAxis":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Remove the axis that was defined using name.</p>\n"},"removePlot":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Remove the plot defined using name from the chart&#39;s plot stack.</p>\n"},"removeSeries":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Remove the series defined by name from the chart.</p>\n"},"render":{"!type":"fn() -> +dojox","!doc":"<p>Render the chart according to the current information defined.  This should\nbe the last call made when defining/creating a chart, or if data within the\nchart has been changed.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the chart to the dimensions of width and height.</p>\n"},"setAxisWindow":{"!type":"fn(name: ?, scale: ?, offset: ?, zoom?: ?) -> +dojox","!doc":"<p>Zooms an axis and all dependent plots. Can be used to zoom in 1D.</p>\n"},"setDir":{"!type":"fn(dir: ?) -> fn"},"setPlotOrder":{"!type":"fn(newOrder: ?) -> +dojox","!doc":"<p>Sets new order of plots. newOrder cannot add or remove\nplots. Wrong names, or dups are ignored.</p>\n"},"setSeriesOrder":{"!type":"fn(newOrder: ?) -> +dojox","!doc":"<p>Sets new order of series within a plot. newOrder cannot add\nor remove series. Wrong names, or dups are ignored.</p>\n"},"setTheme":{"!type":"fn(theme: ?) -> +dojox","!doc":"<p>Set a theme of the chart.</p>\n"},"setWindow":{"!type":"fn(sx: ?, sy: ?, dx: ?, dy: ?, zoom?: ?) -> +dojox","!doc":"<p>Zooms in or out any plots in two dimensions.</p>\n"},"updateSeries":{"!type":"fn(name: ?, data: ?, offsets?: ?) -> +dojox","!doc":"<p>Update the given series with a new set of data points.</p>\n"},"zoomIn":{"!type":"fn(name: ?, range: ?, delayed: ?)","!doc":"<p>Zoom the chart to a specific range on one axis.  This calls render()\ndirectly as a convenience method.</p>\n"}}},"dojox/charting/Element":{"!type":"fn()","prototype":{"!doc":"<p>A base class that is used to build other elements of a chart, such as\na series.</p>\n","chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"cleanGroup":{"!type":"fn(creator?: ?) -> +dojox","!doc":"<p>Clean any elements (HTML or GFX-based) out of our group, and create a new one.</p>\n"},"constructor":{"!type":"fn(chart: ?)","!doc":"<p>Creates a new charting element.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"purgeGroup":{"!type":"fn() -> +dojox","!doc":"<p>Clear any elements out of our group, and destroy the group.</p>\n"}}},"dojox/gfx":{"_base":{},"_gfxBidiSupport":{},"_svgFontCache":{},"_vectorFontCache":{},"arc":{"!doc":"<p>This module contains the core graphics Arc functions.</p>\n"},"bezierutils":{},"canvas":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of this module alone will only add in drawing support.\nThe additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.\nThe id of the Canvas renderer is &#39;canvas&#39;.  This id can be used when switch Dojo&#39;s\ngraphics context between renderer implementations.  See dojox/gfx/_base.switchRenderer\nAPI.</p>\n"},"canvas_attach":{},"canvasext":{},"canvasWithEvents":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers which extends\nthe basic canvas drawing renderer bridge to add additional support for graphics events\non Shapes.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of the canvas module alone will only add in drawing support.\nThis additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.</p>\n"},"cm_in_pt":{"!doc":"<p>points per centimeter (constant)</p>\n"},"defaultCircle":{"!doc":"<p>An object defining the default Circle prototype.</p>\n"},"defaultEllipse":{"!doc":"<p>Defines the default Ellipse prototype.</p>\n"},"defaultFont":{"!doc":"<p>An object specifying the default properties for a Font used in text operations.</p>\n"},"defaultImage":{"!doc":"<p>Defines the default Image prototype.</p>\n"},"defaultLine":{"!doc":"<p>An object defining the default Line prototype.</p>\n"},"defaultLinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n"},"defaultPath":{"!doc":"<p>Defines the default Path prototype object.</p>\n"},"defaultPattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n"},"defaultPolyline":{"!doc":"<p>Defines the default PolyLine prototype.</p>\n"},"defaultRadialGradient":{"!doc":"<p>An object specifying the default properties for RadialGradients using in fills patterns.</p>\n"},"defaultRect":{"!doc":"<p>Defines the default Rect prototype.</p>\n"},"defaultStroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.\nThis object defines the default Stroke prototype.</p>\n"},"defaultText":{"!doc":"<p>Defines the default Text prototype.</p>\n"},"defaultTextPath":{"!doc":"<p>Defines the default TextPath prototype.</p>\n"},"defaultVectorFont":{},"defaultVectorText":{},"Fill":{"!doc":"<p>Defines how to fill a shape. Four types of fills can be used: solid, linear gradient, radial gradient and pattern.\nSee dojox/gfx.LinearGradient, dojox/gfx.RadialGradient and dojox/gfx.Pattern respectively for more information about the properties supported by each type.</p>\n"},"Font":{"!doc":"<p>An object specifying the properties for a Font used in text operations.</p>\n"},"fx":{},"getDefault":{},"gradient":{},"gradutils":{},"LinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n"},"matrix":{},"mm_in_pt":{"!doc":"<p>points per millimeter (constant)</p>\n"},"move":{},"path":{"!doc":"<p>This module contains the core graphics Path API.\nPath command format follows the W3C SVG 1.0 Path api.</p>\n"},"pathSvgRegExp":{},"pathVmlRegExp":{"!doc":"<p>a constant regular expression used to split a SVG/VML path into primitive components</p>\n"},"Pattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n"},"RadialGradient":{"!doc":"<p>Specifies the properties for RadialGradients using in fills patterns.</p>\n"},"renderer":{"!doc":"<p>Either the string name of a renderer (eg. &#39;canvas&#39;, &#39;svg, ...) or the renderer\nobject to switch to.</p>\n"},"shape":{"!doc":"<p>This module contains the core graphics Shape API.\nDifferent graphics renderer implementation modules (svg, canvas, vml, silverlight, etc.) extend this\nbasic api to provide renderer-specific implementations for each shape.</p>\n"},"silverlight":{"!doc":"<p>This the graphics rendering bridge for the Microsoft Silverlight plugin.\nSilverlight is a faster implementation on IE6-8 than the default 2d graphics, VML</p>\n"},"silverlight_attach":{},"Stroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.</p>\n"},"svg":{"!doc":"<p>This the graphics rendering bridge for browsers compliant with W3C SVG1.0.\nThis is the preferred renderer to use for interactive and accessible graphics.</p>\n"},"svgext":{},"utils":{},"vectorFontFitting":{},"VectorText":{},"vml":{"!doc":"<p>This the default graphics rendering bridge for IE6-7.\nThis renderer is very slow.  For best performance on IE6-8, use Silverlight plugin.\nIE9+ defaults to the standard W3C SVG renderer.</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +dojox","!doc":"<p>creates a surface</p>\n"},"decompose":{"!type":"fn(matrix: ?)","!doc":"<p>Decompose a 2D matrix into translation, scaling, and rotation components.</p>\n"},"Ellipse":{"!type":"fn()"},"equalSources":{"!type":"fn(a: ?, b: ?)","!doc":"<p>compares event sources, returns true if they are equal</p>\n"},"formatNumber":{"!type":"fn(x: ?, addSpace: ?) -> string","!doc":"<p>converts a number to a string using a fixed notation</p>\n"},"getVectorFont":{"!type":"fn(url: ?) -> ?"},"Group":{"!type":"fn()"},"Line":{"!type":"fn()"},"makeFontString":{"!type":"fn(font: ?) -> string","!doc":"<p>converts a font object to a CSS font string</p>\n"},"makeParameters":{"!type":"fn(defaults: ?, update: ?) -> ?","!doc":"<p>copies the original object, and all copied properties from the\n&#39;update&#39; object</p>\n"},"Matrix2D":{"!type":"fn(arg: ?)","!doc":"<p>a 2D matrix object</p>\n"},"Moveable":{"!type":"fn()"},"Mover":{"!type":"fn()"},"normalizeColor":{"!type":"fn(color: ?) -> +instance","!doc":"<p>converts any legal color representation to normalized\ndojo/Color object</p>\n"},"normalizedLength":{"!type":"fn(len: ?) -> number","!doc":"<p>converts any length value to pixels</p>\n"},"normalizeParameters":{"!type":"fn(existed: ?, update: ?) -> ?","!doc":"<p>updates an existing object with properties from an &#39;update&#39;\nobject</p>\n"},"Path":{"!type":"fn()"},"Point":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"pt2px":{"!type":"fn(len: ?) -> number","!doc":"<p>converts points to pixels</p>\n"},"px2pt":{"!type":"fn(len: ?) -> number","!doc":"<p>converts pixels to points</p>\n"},"px_in_pt":{"!type":"fn() -> number","!doc":"<p>returns the current number of pixels per point.</p>\n"},"Rect":{"!type":"fn()"},"Rectangle":{"!type":"fn()"},"splitFontString":{"!type":"fn(str: ?) -> ?","!doc":"<p>converts a CSS font string to a font object</p>\n"},"Surface":{"!type":"fn()"},"switchTo":{"!type":"fn(renderer: ?)","!doc":"<p>switch the graphics implementation to the specified renderer.</p>\n"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"},"VectorFont":{"!type":"fn()"}},"dojox/gfx._base":{},"dojox/gfx.Stroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.</p>\n","cap":{"!doc":"<p>The endcap style of the path. One of &#39;butt&#39;, &#39;round&#39;, ... . Default value &#39;butt&#39;.</p>\n"},"color":{"!doc":"<p>The color of the stroke, default value &#39;black&#39;.</p>\n"},"join":{"!doc":"<p>The join style to use when combining path segments. Default value 4.</p>\n"},"style":{"!doc":"<p>The style of the stroke, one of &#39;solid&#39;, ... . Default value &#39;solid&#39;.</p>\n"},"width":{"!doc":"<p>The width of a stroke, default value 1.</p>\n"}},"dojox/gfx.Fill":{"!doc":"<p>Defines how to fill a shape. Four types of fills can be used: solid, linear gradient, radial gradient and pattern.\nSee dojox/gfx.LinearGradient, dojox/gfx.RadialGradient and dojox/gfx.Pattern respectively for more information about the properties supported by each type.</p>\n","color":{"!doc":"<p>The color of a solid fill type.</p>\n"},"type":{"!doc":"<p>The type of fill. One of &#39;linear&#39;, &#39;radial&#39;, &#39;pattern&#39; or undefined. If not specified, a solid fill is assumed.</p>\n"}},"dojox/gfx.LinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the start of the line).  The start of the line is\ndefined at offest 0 with the end of the line at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Linear Gradient, value &#39;linear&#39;</p>\n"},"x1":{"!doc":"<p>The X coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"x2":{"!doc":"<p>The X coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"},"y1":{"!doc":"<p>The Y coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"y2":{"!doc":"<p>The Y coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"}},"dojox/gfx.RadialGradient":{"!doc":"<p>Specifies the properties for RadialGradients using in fills patterns.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the center of the radial gradient).\nThe center is defined at offest 0 with the outer edge of the gradient at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"cx":{"!doc":"<p>The X coordinate of the center of the radial gradient, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the radial gradient, default value 0.</p>\n"},"r":{"!doc":"<p>The radius to the end of the radial gradient, default value 100.</p>\n"},"type":{"!doc":"<p>Specifies this is a RadialGradient, value &#39;radial&#39;</p>\n"}},"dojox/gfx.Pattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n","height":{"!doc":"<p>The height of the pattern image, default value is 0.</p>\n"},"src":{"!doc":"<p>A url specifying the image to use for the pattern.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Pattern, value &#39;pattern&#39;.</p>\n"},"width":{"!doc":"<p>The width of the pattern image, default value is 0.</p>\n"},"x":{"!doc":"<p>The X coordinate of the position of the pattern, default value is 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the position of the pattern, default value is 0.</p>\n"}},"dojox/gfx.Text":{"!type":"fn()","prototype":{"!doc":"<p>a generic text (do not instantiate it directly)</p>\n","textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx.Font":{"!doc":"<p>An object specifying the properties for a Font used in text operations.</p>\n","family":{"!doc":"<p>The font family, one of &#39;serif&#39;, &#39;sanserif&#39;, ..., default value &#39;serif&#39;.</p>\n"},"size":{"!doc":"<p>The font size (including units), default value &#39;10pt&#39;.</p>\n"},"style":{"!doc":"<p>The font style, one of &#39;normal&#39;, &#39;bold&#39;, default value &#39;normal&#39;.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Font, value &#39;font&#39;.</p>\n"},"variant":{"!doc":"<p>The font variant, one of &#39;normal&#39;, ... , default value &#39;normal&#39;.</p>\n"},"weight":{"!doc":"<p>The font weight, one of &#39;normal&#39;, ..., default value &#39;normal&#39;.</p>\n"}},"dojox/gfx.defaultPath":{"!doc":"<p>Defines the default Path prototype object.</p>\n","path":{"!doc":"<p>The path commands. See W32C SVG 1.0 specification.\nDefaults to empty string value.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Path, default value &#39;path&#39;.</p>\n"}},"dojox/gfx.defaultPolyline":{"!doc":"<p>Defines the default PolyLine prototype.</p>\n","points":{"!doc":"<p>An array of point objects [{x:0,y:0},...] defining the default polyline&#39;s line segments. Value is an empty array [].</p>\n"},"type":{"!doc":"<p>Specifies this object is a PolyLine, default value &#39;polyline&#39;.</p>\n"}},"dojox/gfx.defaultRect":{"!doc":"<p>Defines the default Rect prototype.</p>\n","height":{"!doc":"<p>The height of the default rectangle, value 100.</p>\n"},"r":{"!doc":"<p>The corner radius for the default rectangle, value 0.</p>\n"},"type":{"!doc":"<p>Specifies this default object is a type of Rect. Value is &#39;rect&#39;</p>\n"},"width":{"!doc":"<p>The width of the default rectangle, value 100.</p>\n"},"x":{"!doc":"<p>The X coordinate of the default rectangles position, value 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the default rectangle&#39;s position, value 0.</p>\n"}},"dojox/gfx.defaultEllipse":{"!doc":"<p>Defines the default Ellipse prototype.</p>\n","cx":{"!doc":"<p>The X coordinate of the center of the ellipse, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the ellipse, default value 0.</p>\n"},"rx":{"!doc":"<p>The radius of the ellipse in the X direction, default value 200.</p>\n"},"ry":{"!doc":"<p>The radius of the ellipse in the Y direction, default value 200.</p>\n"},"type":{"!doc":"<p>Specifies that this object is a type of Ellipse, value is &#39;ellipse&#39;</p>\n"}},"dojox/gfx.defaultCircle":{"!doc":"<p>An object defining the default Circle prototype.</p>\n","cx":{"!doc":"<p>The X coordinate of the center of the circle, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the circle, default value 0.</p>\n"},"r":{"!doc":"<p>The radius, default value 100.</p>\n"},"type":{"!doc":"<p>Specifies this object is a circle, value &#39;circle&#39;</p>\n"}},"dojox/gfx.defaultLine":{"!doc":"<p>An object defining the default Line prototype.</p>\n","type":{"!doc":"<p>Specifies this is a Line, value &#39;line&#39;</p>\n"},"x1":{"!doc":"<p>The X coordinate of the start of the line, default value 0.</p>\n"},"x2":{"!doc":"<p>The X coordinate of the end of the line, default value 100.</p>\n"},"y1":{"!doc":"<p>The Y coordinate of the start of the line, default value 0.</p>\n"},"y2":{"!doc":"<p>The Y coordinate of the end of the line, default value 100.</p>\n"}},"dojox/gfx.defaultImage":{"!doc":"<p>Defines the default Image prototype.</p>\n","height":{"!doc":"<p>The height of the image, default value 0.</p>\n"},"src":{"!doc":"<p>The src url of the image, defaults to empty string.</p>\n"},"type":{"!doc":"<p>Specifies this object is an image, value &#39;image&#39;.</p>\n"},"width":{"!doc":"<p>The width of the image, default value 0.</p>\n"},"x":{"!doc":"<p>The X coordinate of the image&#39;s position, default value 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the image&#39;s position, default value 0.</p>\n"}},"dojox/gfx.defaultText":{"!doc":"<p>Defines the default Text prototype.</p>\n","align":{"!doc":"<p>The horizontal text alignment, one of &#39;start&#39;, &#39;end&#39;, &#39;center&#39;. Default value &#39;start&#39;.</p>\n"},"decoration":{"!doc":"<p>The text decoration , one of &#39;none&#39;, ... . Default value &#39;none&#39;.</p>\n"},"kerning":{"!doc":"<p>Whether kerning is used on the text, boolean default value true.</p>\n"},"rotated":{"!doc":"<p>Whether the text is rotated, boolean default value false.</p>\n"},"text":{"!doc":"<p>The text to be displayed, default value empty string.</p>\n"},"type":{"!doc":"<p>Specifies this is a Text shape, value &#39;text&#39;.</p>\n"},"x":{"!doc":"<p>The X coordinate of the text position, default value 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the text position, default value 0.</p>\n"}},"dojox/gfx.defaultTextPath":{"!doc":"<p>Defines the default TextPath prototype.</p>\n","align":{"!doc":"<p>The horizontal text alignment, one of &#39;start&#39;, &#39;end&#39;, &#39;center&#39;. Default value &#39;start&#39;.</p>\n"},"decoration":{"!doc":"<p>The text decoration , one of &#39;none&#39;, ... . Default value &#39;none&#39;.</p>\n"},"kerning":{"!doc":"<p>Whether kerning is used on the text, boolean default value true.</p>\n"},"rotated":{"!doc":"<p>Whether the text is rotated, boolean default value false.</p>\n"},"text":{"!doc":"<p>The text to be displayed, default value empty string.</p>\n"},"type":{"!doc":"<p>Specifies this is a TextPath, value &#39;textpath&#39;.</p>\n"}},"dojox/gfx.defaultStroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.\nThis object defines the default Stroke prototype.</p>\n","cap":{"!doc":"<p>The endcap style of the path. One of &#39;butt&#39;, &#39;round&#39;, ... . Default value &#39;butt&#39;.</p>\n"},"color":{"!doc":"<p>The color of the stroke, default value &#39;black&#39;.</p>\n"},"join":{"!doc":"<p>The join style to use when combining path segments. Default value 4.</p>\n"},"style":{"!doc":"<p>The style of the stroke, one of &#39;solid&#39;, ... . Default value &#39;solid&#39;.</p>\n"},"type":{"!doc":"<p>Specifies this object is a type of Stroke, value &#39;stroke&#39;.</p>\n"},"width":{"!doc":"<p>The width of a stroke, default value 1.</p>\n"}},"dojox/gfx.defaultLinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the start of the line).  The start of the line is\ndefined at offest 0 with the end of the line at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Linear Gradient, value &#39;linear&#39;</p>\n"},"x1":{"!doc":"<p>The X coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"x2":{"!doc":"<p>The X coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"},"y1":{"!doc":"<p>The Y coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"y2":{"!doc":"<p>The Y coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"}},"dojox/gfx.defaultRadialGradient":{"!doc":"<p>An object specifying the default properties for RadialGradients using in fills patterns.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the center of the radial gradient).\nThe center is defined at offest 0 with the outer edge of the gradient at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"cx":{"!doc":"<p>The X coordinate of the center of the radial gradient, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the radial gradient, default value 0.</p>\n"},"r":{"!doc":"<p>The radius to the end of the radial gradient, default value 100.</p>\n"},"type":{"!doc":"<p>Specifies this is a RadialGradient, value &#39;radial&#39;</p>\n"}},"dojox/gfx.defaultPattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n","height":{"!doc":"<p>The height of the pattern image, default value is 0.</p>\n"},"src":{"!doc":"<p>A url specifying the image to use for the pattern.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Pattern, value &#39;pattern&#39;.</p>\n"},"width":{"!doc":"<p>The width of the pattern image, default value is 0.</p>\n"},"x":{"!doc":"<p>The X coordinate of the position of the pattern, default value is 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the position of the pattern, default value is 0.</p>\n"}},"dojox/gfx.defaultFont":{"!doc":"<p>An object specifying the default properties for a Font used in text operations.</p>\n","family":{"!doc":"<p>The font family, one of &#39;serif&#39;, &#39;sanserif&#39;, ..., default value &#39;serif&#39;.</p>\n"},"size":{"!doc":"<p>The font size (including units), default value &#39;10pt&#39;.</p>\n"},"style":{"!doc":"<p>The font style, one of &#39;normal&#39;, &#39;bold&#39;, default value &#39;normal&#39;.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Font, value &#39;font&#39;.</p>\n"},"variant":{"!doc":"<p>The font variant, one of &#39;normal&#39;, ... , default value &#39;normal&#39;.</p>\n"},"weight":{"!doc":"<p>The font weight, one of &#39;normal&#39;, ..., default value &#39;normal&#39;.</p>\n"}},"dojox/gfx.matrix":{"flipX":{"!doc":"<p>a matrix, which reflects points at x = 0 line: flipX * (x, y) == (-x, y)</p>\n"},"flipXY":{"!doc":"<p>a matrix, which reflects points at the origin of coordinates: flipXY * (x, y) == (-x, -y)</p>\n"},"flipY":{"!doc":"<p>a matrix, which reflects points at y = 0 line: flipY * (x, y) == (x, -y)</p>\n"},"identity":{"!doc":"<p>an identity matrix constant: identity * (x, y) == (x, y)</p>\n"},"clone":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>creates a copy of a 2D matrix</p>\n"},"invert":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>inverts a 2D matrix</p>\n"},"isIdentity":{"!type":"fn(matrix: ?) -> bool","!doc":"<p>returns whether the specified matrix is the identity.</p>\n"},"Matrix2D":{"!type":"fn(arg: ?)","!doc":"<p>a 2D matrix object</p>\n"},"multiply":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>combines matrices by multiplying them sequentially in the given order</p>\n"},"multiplyPoint":{"!type":"fn(matrix: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>applies a matrix to a point</p>\n"},"multiplyRectangle":{"!type":"fn(matrix: ?, rect: ?) -> +dojox","!doc":"<p>Applies a matrix to a rectangle.</p>\n"},"normalize":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>converts an object to a matrix, if necessary</p>\n"},"project":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms an orthogonal projection matrix</p>\n"},"reflect":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a reflection matrix</p>\n"},"rotate":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms a rotating matrix</p>\n"},"rotateAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>rotates a picture using a specified point as a center of rotation</p>\n"},"rotateg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms a rotating matrix</p>\n"},"rotategAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>rotates a picture using a specified point as a center of rotation</p>\n"},"scale":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a scaling matrix</p>\n"},"scaleAt":{"!type":"fn(a: ?, b?: ?, c: ?, d: ?) -> +dojox","!doc":"<p>scales a picture using a specified point as a center of scaling</p>\n"},"skewX":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms an x skewing matrix</p>\n"},"skewXAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the x axis using a specified point as a center of skewing</p>\n"},"skewXg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms an x skewing matrix</p>\n"},"skewXgAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the x axis using a specified point as a center of skewing</p>\n"},"skewY":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms a y skewing matrix</p>\n"},"skewYAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the y axis using a specified point as a center of skewing</p>\n"},"skewYg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms a y skewing matrix</p>\n"},"skewYgAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the y axis using a specified point as a center of skewing</p>\n"},"translate":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a translation matrix</p>\n"}},"dojox/gfx.shape":{"!doc":"<p>This module contains the core graphics Shape API.\nDifferent graphics renderer implementation modules (svg, canvas, vml, silverlight, etc.) extend this\nbasic api to provide renderer-specific implementations for each shape.</p>\n","_eventsProcessing":{},"Container":{"!doc":"<p>a container of shapes, which can be used\nas a foundation for renderer-specific groups, or as a way\nto logically group shapes (e.g, to propagate matricies)</p>\n"},"Creator":{"!doc":"<p>shape creators</p>\n"},"byId":{"!type":"fn(id: ?) -> +dojox","!doc":"<p>Returns the shape that matches the specified id.</p>\n"},"Circle":{"!type":"fn()"},"dispose":{"!type":"fn(s: ?, recurse?: ?)","!doc":"<p>Removes the specified shape from the registry.</p>\n"},"Ellipse":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"register":{"!type":"fn(s: ?) -> number","!doc":"<p>Register the specified shape into the graphics registry.</p>\n"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"}},"dojox/gfx.Point":{"!type":"fn()","prototype":{"!doc":"<p>2D point for drawings - {x, y}</p>\n"}},"dojox/gfx.Rectangle":{"!type":"fn()","prototype":{"!doc":"<p>rectangle - {x, y, width, height}</p>\n"}},"dojox/gfx.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape, which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","textDir":{"!doc":"<p>Will be used for inheritance, or as default for text objects\nthat textDir wasn&#39;t directly specified for them but the bidi support was required.</p>\n"},"add":{"!type":"fn(shape: ?) -> ?","!doc":"<p>adds a shape to the list</p>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn(destroy?: ?) -> fn","!doc":"<p>removes all shapes from a group/surface.</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a generic rectangle</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box (its shape in this case)</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a generic circle</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>a generic ellipse</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx.Line":{"!type":"fn()","prototype":{"!doc":"<p>a generic line (do not instantiate it directly)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a generic polyline/polygon (do not instantiate it directly)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(points: ?, closed: ?) -> fn","!doc":"<p>sets a polyline/polygon shape object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> ?","!doc":"<p>adds a shape to the list</p>\n"},"clear":{"!type":"fn(destroy?: ?) -> fn","!doc":"<p>removes all shapes from a group/surface.</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()","!doc":"<p>destroy all relevant external resources and release all\nexternal references to make this object garbage-collectible</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> ?","!doc":"<p>gets current width and height in pixels</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx.gradutils":{"getColor":{"!type":"fn(fill: ?, pt: ?)","!doc":"<p>sample a color from a gradient using a point</p>\n"},"reverse":{"!type":"fn(fill: ?)","!doc":"<p>reverses a gradient</p>\n"}},"dojox/gfx.fx":{"animateFill":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation which will change fill color over time.\nOnly solid fill color is supported at the moment</p>\n"},"animateFont":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation which will change font properties over time.</p>\n"},"animateStroke":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation which will change stroke properties over time.</p>\n"},"animateTransform":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation which will change transformation over time.</p>\n"}},"dojox/gfx.utils":{"_gfxSvgProxy":{},"_initSvgSerializerDeferred":{},"deserialize":{"!type":"fn(parent: ?, object: ?) -> ?","!doc":"<p>Takes a surface or a shape and populates it with an object produced by serialize().</p>\n"},"forEach":{"!type":"fn(object: ?, f: ?, o?: ?)","!doc":"<p>Takes a shape or a surface and applies a function &quot;f&quot; to in the context of &quot;o&quot;\n(or global, if missing). If &quot;shape&quot; was a surface or a group, it applies the same\nfunction to all children recursively effectively visiting all shapes of the underlying scene graph.</p>\n"},"fromJson":{"!type":"fn(parent: ?, json: ?) -> ?","!doc":"<p>Works just like deserialize() but takes a JSON representation of the object.</p>\n"},"serialize":{"!type":"fn(object: ?) -> ?","!doc":"<p>Takes a shape or a surface and returns an object, which describes underlying shapes.</p>\n"},"toJson":{"!type":"fn(object: ?, prettyPrint?: ?) -> string","!doc":"<p>Works just like serialize() but returns a JSON string. If prettyPrint is true, the string is pretty-printed to make it more human-readable.</p>\n"},"toSvg":{"!type":"fn(surface: ?) -> +any","!doc":"<p>Function to serialize a GFX surface to SVG text.</p>\n"}},"dojox/gfx.path":{"!doc":"<p>This module contains the core graphics Path API.\nPath command format follows the W3C SVG 1.0 Path api.</p>\n","Path":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx.Path":{"!type":"fn()","prototype":{"!doc":"<p>a generalized path shape</p>\n","_2PI":{},"_validSegments":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)","!doc":"<p>a path constructor</p>\n"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns the bounding box {x, y, width, height} or null</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a generalized TextPath shape</p>\n","_2PI":{},"_validSegments":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)","!doc":"<p>a TextPath shape constructor</p>\n"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns the bounding box {x, y, width, height} or null</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx._gfxBidiSupport":{},"dojox/gfx.Mover":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(shape: ?, e: ?, host?: ?)","!doc":"<p>an object, which makes a shape follow the mouse,\nused as a default mover, and as a base class for custom movers</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops the move, deletes all references, so the object can be garbage-collected</p>\n"}}},"dojox/gfx.__MoveableCtorArgs":{"!type":"fn()","prototype":{"!doc":"<p>The arguments used for dojox/gfx/Moveable constructor.</p>\n","delay":{"!doc":"<p>delay move by this number of pixels</p>\n"},"mover":{"!type":"fn()","!doc":"<p>a constructor of custom Mover</p>\n"}}},"dojox/gfx.Moveable":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(shape: ?, params: ?)","!doc":"<p>an object, which makes a shape moveable</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops watching for possible move, deletes all references, so the object can be garbage-collected</p>\n"}}},"dojox/gfx.VectorText":{},"dojox/gfx.vectorFontFitting":{"FIT":{},"FLOW":{},"NONE":{}},"dojox/gfx.defaultVectorText":{"align":{},"decoration":{},"fitting":{},"height":{},"leading":{},"text":{},"type":{},"width":{},"x":{},"y":{}},"dojox/gfx.defaultVectorFont":{"family":{},"size":{},"type":{}},"dojox/gfx._vectorFontCache":{},"dojox/gfx._svgFontCache":{},"dojox/gfx.VectorFont":{"!type":"fn()","prototype":{"!doc":"<p>An implementation of the SVG Font 1.1 spec, using dojox/gfx.\n\n</p>\n<p>Basic interface:\n\n</p>\n<pre><code>var f = new gfx.Font(url|string);\nsurface||group.createVectorText(text)\n.setFill(fill)\n.setStroke(stroke)\n.setFont(fontStyleObject);</code></pre>\n<p>The arguments passed to createVectorText are the same as you would\npass to surface||group.createText; the difference is that this\nis entirely renderer-agnostic, and the return value is a subclass\nof dojox/gfx.Group.\n\n</p>\n<p>Note also that the &quot;defaultText&quot; object is slightly different:\n{ type:&quot;vectortext&quot;, x:0, y:0, width:null, height: null,\ntext: &quot;&quot;, align: &quot;start&quot;, decoration: &quot;none&quot; }\n\n</p>\n<p>...as well as the &quot;defaultVectorFont&quot; object:\n{ type:&quot;vectorfont&quot;, size:&quot;10pt&quot; }\n\n</p>\n<p>The reason for this should be obvious: most of the style for the font is defined\nby the font object itself.\n\n</p>\n<p>Note that this will only render IF and WHEN you set the font.</p>\n","_entityRe":{},"constructor":{"!type":"fn(url: ?)","!doc":"<p>Create this font object based on the SVG Font definition at url.</p>\n"},"draw":{"!type":"fn(group: ?, textArgs: ?, fontArgs: ?, fillArgs: ?, strokeArgs: ?) -> +dojox","!doc":"<p>based on the passed parameters, draw the given text using paths\ndefined by this font.</p>\n"},"getBaseline":{"!type":"fn(scale?: ?) -> number","!doc":"<p>Find the baseline coord for alignment; adjust for scale if passed.</p>\n"},"getCenterline":{"!type":"fn(scale?: ?) -> number","!doc":"<p>return the y coordinate that is the center of the viewbox.</p>\n"},"getLineHeight":{"!type":"fn(scale?: ?) -> number","!doc":"<p>return the height of a single line, sans leading, based on scale.</p>\n"},"getWidth":{"!type":"fn(text: ?, scale?: ?) -> number","!doc":"<p>Get the width of the rendered text without actually rendering it.</p>\n"},"initialized":{"!type":"fn() -> bool","!doc":"<p>Return if we&#39;ve loaded a font def, and the parsing was successful.</p>\n"},"load":{"!type":"fn(url: ?) -> fn","!doc":"<p>Load the passed SVG and send it to the parser for parsing.</p>\n"}}},"dojox/gfx.arc":{"!doc":"<p>This module contains the core graphics Arc functions.</p>\n","curvePI4":{"!doc":"<p>an object with properties of an arc around a unit circle from 0 to pi/4</p>\n"},"arcAsBezier":{"!type":"fn(last: ?, rx: ?, ry: ?, xRotg: ?, large: ?, sweep: ?, x: ?, y: ?) -> []","!doc":"<p>calculates an arc as a series of Bezier curves\ngiven the last point and a standard set of SVG arc parameters,\nit returns an array of arrays of parameters to form a series of\nabsolute Bezier curves.</p>\n"},"unitArcAsBezier":{"!type":"fn(alpha: ?)","!doc":"<p>return a start point, 1st and 2nd control points, and an end point of\na an arc, which is reflected on the x axis</p>\n"}},"dojox/gfx.bezierutils":{"computeLength":{"!type":"fn(points: ?) -> number","!doc":"<p>Returns the length of the given bezier curve.</p>\n"},"distance":{"!type":"fn(x1: ?, y1: ?, x2: ?, y2: ?) -> ?","!doc":"<p>Returns the distance between the specified points.</p>\n"},"splitBezierAtT":{"!type":"fn(points: ?, t: ?) -> ?"},"tAtLength":{"!type":"fn(points: ?, length: ?) -> number","!doc":"<p>Returns the t corresponding to the given length for the specified bezier curve.</p>\n"}},"dojox/gfx.canvas":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of this module alone will only add in drawing support.\nThe additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.\nThe id of the Canvas renderer is &#39;canvas&#39;.  This id can be used when switch Dojo&#39;s\ngraphics context between renderer implementations.  See dojox/gfx/_base.switchRenderer\nAPI.</p>\n","attachNode":{"!type":"fn()"},"attachSurface":{"!type":"fn()"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +instance","!doc":"<p>creates a surface (Canvas)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx.canvasWithEvents":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers which extends\nthe basic canvas drawing renderer bridge to add additional support for graphics events\non Shapes.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of the canvas module alone will only add in drawing support.\nThis additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.</p>\n","Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)","!doc":"<p>creates a surface (Canvas)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx.canvas_attach":{},"dojox/gfx.canvasext":{},"dojox/gfx.gradient":{"project":{"!type":"fn(matrix: ?, gradient: ?, tl: ?, rb: ?, ttl: ?, trb: ?) -> ?","!doc":"<p>Returns a new gradient using the &quot;VML algorithm&quot; and suitable for VML.</p>\n"},"rescale":{"!type":"fn(stops: ?, from: ?, to: ?) -> []","!doc":"<p>Recalculates a gradient from 0-1 window to\n&quot;from&quot;-&quot;to&quot; window blending and replicating colors,\nif necessary.</p>\n"}},"dojox/gfx.move":{},"dojox/gfx.silverlight":{"!doc":"<p>This the graphics rendering bridge for the Microsoft Silverlight plugin.\nSilverlight is a faster implementation on IE6-8 than the default 2d graphics, VML</p>\n","attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)","!doc":"<p>creates a surface (Silverlight)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx.silverlight_attach":{},"dojox/gfx.svg":{"!doc":"<p>This the graphics rendering bridge for browsers compliant with W3C SVG1.0.\nThis is the preferred renderer to use for interactive and accessible graphics.</p>\n","dasharray":{},"useSvgWeb":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"getRef":{"!type":"fn(name: ?) -> +any","!doc":"<p>looks up a node by its external name</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx.svgext":{},"dojox/gfx.vml":{"!doc":"<p>This the default graphics rendering bridge for IE6-7.\nThis renderer is very slow.  For best performance on IE6-8, use Silverlight plugin.\nIE9+ defaults to the standard W3C SVG renderer.</p>\n","_bool":{},"text_alignment":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +any","!doc":"<p>creates a surface (VML)</p>\n"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_base":{"_base":{},"_gfxBidiSupport":{},"_svgFontCache":{},"_vectorFontCache":{},"arc":{"!doc":"<p>This module contains the core graphics Arc functions.</p>\n"},"bezierutils":{},"canvas":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of this module alone will only add in drawing support.\nThe additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.\nThe id of the Canvas renderer is &#39;canvas&#39;.  This id can be used when switch Dojo&#39;s\ngraphics context between renderer implementations.  See dojox/gfx/_base.switchRenderer\nAPI.</p>\n"},"canvas_attach":{},"canvasext":{},"canvasWithEvents":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers which extends\nthe basic canvas drawing renderer bridge to add additional support for graphics events\non Shapes.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of the canvas module alone will only add in drawing support.\nThis additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.</p>\n"},"cm_in_pt":{"!doc":"<p>points per centimeter (constant)</p>\n"},"defaultCircle":{"!doc":"<p>An object defining the default Circle prototype.</p>\n"},"defaultEllipse":{"!doc":"<p>Defines the default Ellipse prototype.</p>\n"},"defaultFont":{"!doc":"<p>An object specifying the default properties for a Font used in text operations.</p>\n"},"defaultImage":{"!doc":"<p>Defines the default Image prototype.</p>\n"},"defaultLine":{"!doc":"<p>An object defining the default Line prototype.</p>\n"},"defaultLinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n"},"defaultPath":{"!doc":"<p>Defines the default Path prototype object.</p>\n"},"defaultPattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n"},"defaultPolyline":{"!doc":"<p>Defines the default PolyLine prototype.</p>\n"},"defaultRadialGradient":{"!doc":"<p>An object specifying the default properties for RadialGradients using in fills patterns.</p>\n"},"defaultRect":{"!doc":"<p>Defines the default Rect prototype.</p>\n"},"defaultStroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.\nThis object defines the default Stroke prototype.</p>\n"},"defaultText":{"!doc":"<p>Defines the default Text prototype.</p>\n"},"defaultTextPath":{"!doc":"<p>Defines the default TextPath prototype.</p>\n"},"defaultVectorFont":{},"defaultVectorText":{},"Fill":{"!doc":"<p>Defines how to fill a shape. Four types of fills can be used: solid, linear gradient, radial gradient and pattern.\nSee dojox/gfx.LinearGradient, dojox/gfx.RadialGradient and dojox/gfx.Pattern respectively for more information about the properties supported by each type.</p>\n"},"Font":{"!doc":"<p>An object specifying the properties for a Font used in text operations.</p>\n"},"fx":{},"getDefault":{},"gradient":{},"gradutils":{},"LinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n"},"matrix":{},"mm_in_pt":{"!doc":"<p>points per millimeter (constant)</p>\n"},"move":{},"path":{"!doc":"<p>This module contains the core graphics Path API.\nPath command format follows the W3C SVG 1.0 Path api.</p>\n"},"pathSvgRegExp":{},"pathVmlRegExp":{"!doc":"<p>a constant regular expression used to split a SVG/VML path into primitive components</p>\n"},"Pattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n"},"RadialGradient":{"!doc":"<p>Specifies the properties for RadialGradients using in fills patterns.</p>\n"},"renderer":{"!doc":"<p>Either the string name of a renderer (eg. &#39;canvas&#39;, &#39;svg, ...) or the renderer\nobject to switch to.</p>\n"},"shape":{"!doc":"<p>This module contains the core graphics Shape API.\nDifferent graphics renderer implementation modules (svg, canvas, vml, silverlight, etc.) extend this\nbasic api to provide renderer-specific implementations for each shape.</p>\n"},"silverlight":{"!doc":"<p>This the graphics rendering bridge for the Microsoft Silverlight plugin.\nSilverlight is a faster implementation on IE6-8 than the default 2d graphics, VML</p>\n"},"silverlight_attach":{},"Stroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.</p>\n"},"svg":{"!doc":"<p>This the graphics rendering bridge for browsers compliant with W3C SVG1.0.\nThis is the preferred renderer to use for interactive and accessible graphics.</p>\n"},"svgext":{},"utils":{},"vectorFontFitting":{},"VectorText":{},"vml":{"!doc":"<p>This the default graphics rendering bridge for IE6-7.\nThis renderer is very slow.  For best performance on IE6-8, use Silverlight plugin.\nIE9+ defaults to the standard W3C SVG renderer.</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +dojox","!doc":"<p>creates a surface</p>\n"},"decompose":{"!type":"fn(matrix: ?)","!doc":"<p>Decompose a 2D matrix into translation, scaling, and rotation components.</p>\n"},"Ellipse":{"!type":"fn()"},"equalSources":{"!type":"fn(a: ?, b: ?)","!doc":"<p>compares event sources, returns true if they are equal</p>\n"},"formatNumber":{"!type":"fn(x: ?, addSpace: ?) -> string","!doc":"<p>converts a number to a string using a fixed notation</p>\n"},"getVectorFont":{"!type":"fn(url: ?) -> ?"},"Group":{"!type":"fn()"},"Line":{"!type":"fn()"},"makeFontString":{"!type":"fn(font: ?) -> string","!doc":"<p>converts a font object to a CSS font string</p>\n"},"makeParameters":{"!type":"fn(defaults: ?, update: ?) -> ?","!doc":"<p>copies the original object, and all copied properties from the\n&#39;update&#39; object</p>\n"},"Matrix2D":{"!type":"fn(arg: ?)","!doc":"<p>a 2D matrix object</p>\n"},"Moveable":{"!type":"fn()"},"Mover":{"!type":"fn()"},"normalizeColor":{"!type":"fn(color: ?) -> +instance","!doc":"<p>converts any legal color representation to normalized\ndojo/Color object</p>\n"},"normalizedLength":{"!type":"fn(len: ?) -> number","!doc":"<p>converts any length value to pixels</p>\n"},"normalizeParameters":{"!type":"fn(existed: ?, update: ?) -> ?","!doc":"<p>updates an existing object with properties from an &#39;update&#39;\nobject</p>\n"},"Path":{"!type":"fn()"},"Point":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"pt2px":{"!type":"fn(len: ?) -> number","!doc":"<p>converts points to pixels</p>\n"},"px2pt":{"!type":"fn(len: ?) -> number","!doc":"<p>converts pixels to points</p>\n"},"px_in_pt":{"!type":"fn() -> number","!doc":"<p>returns the current number of pixels per point.</p>\n"},"Rect":{"!type":"fn()"},"Rectangle":{"!type":"fn()"},"splitFontString":{"!type":"fn(str: ?) -> ?","!doc":"<p>converts a CSS font string to a font object</p>\n"},"Surface":{"!type":"fn()"},"switchTo":{"!type":"fn(renderer: ?)","!doc":"<p>switch the graphics implementation to the specified renderer.</p>\n"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"},"VectorFont":{"!type":"fn()"}},"dojox/gfx/_base._base":{},"dojox/gfx/_base.Stroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.</p>\n","cap":{"!doc":"<p>The endcap style of the path. One of &#39;butt&#39;, &#39;round&#39;, ... . Default value &#39;butt&#39;.</p>\n"},"color":{"!doc":"<p>The color of the stroke, default value &#39;black&#39;.</p>\n"},"join":{"!doc":"<p>The join style to use when combining path segments. Default value 4.</p>\n"},"style":{"!doc":"<p>The style of the stroke, one of &#39;solid&#39;, ... . Default value &#39;solid&#39;.</p>\n"},"width":{"!doc":"<p>The width of a stroke, default value 1.</p>\n"}},"dojox/gfx/_base.Fill":{"!doc":"<p>Defines how to fill a shape. Four types of fills can be used: solid, linear gradient, radial gradient and pattern.\nSee dojox/gfx.LinearGradient, dojox/gfx.RadialGradient and dojox/gfx.Pattern respectively for more information about the properties supported by each type.</p>\n","color":{"!doc":"<p>The color of a solid fill type.</p>\n"},"type":{"!doc":"<p>The type of fill. One of &#39;linear&#39;, &#39;radial&#39;, &#39;pattern&#39; or undefined. If not specified, a solid fill is assumed.</p>\n"}},"dojox/gfx/_base.LinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the start of the line).  The start of the line is\ndefined at offest 0 with the end of the line at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Linear Gradient, value &#39;linear&#39;</p>\n"},"x1":{"!doc":"<p>The X coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"x2":{"!doc":"<p>The X coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"},"y1":{"!doc":"<p>The Y coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"y2":{"!doc":"<p>The Y coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"}},"dojox/gfx/_base.RadialGradient":{"!doc":"<p>Specifies the properties for RadialGradients using in fills patterns.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the center of the radial gradient).\nThe center is defined at offest 0 with the outer edge of the gradient at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"cx":{"!doc":"<p>The X coordinate of the center of the radial gradient, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the radial gradient, default value 0.</p>\n"},"r":{"!doc":"<p>The radius to the end of the radial gradient, default value 100.</p>\n"},"type":{"!doc":"<p>Specifies this is a RadialGradient, value &#39;radial&#39;</p>\n"}},"dojox/gfx/_base.Pattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n","height":{"!doc":"<p>The height of the pattern image, default value is 0.</p>\n"},"src":{"!doc":"<p>A url specifying the image to use for the pattern.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Pattern, value &#39;pattern&#39;.</p>\n"},"width":{"!doc":"<p>The width of the pattern image, default value is 0.</p>\n"},"x":{"!doc":"<p>The X coordinate of the position of the pattern, default value is 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the position of the pattern, default value is 0.</p>\n"}},"dojox/gfx/_base.Text":{"!type":"fn()","prototype":{"!doc":"<p>a generic text (do not instantiate it directly)</p>\n","textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_base.Font":{"!doc":"<p>An object specifying the properties for a Font used in text operations.</p>\n","family":{"!doc":"<p>The font family, one of &#39;serif&#39;, &#39;sanserif&#39;, ..., default value &#39;serif&#39;.</p>\n"},"size":{"!doc":"<p>The font size (including units), default value &#39;10pt&#39;.</p>\n"},"style":{"!doc":"<p>The font style, one of &#39;normal&#39;, &#39;bold&#39;, default value &#39;normal&#39;.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Font, value &#39;font&#39;.</p>\n"},"variant":{"!doc":"<p>The font variant, one of &#39;normal&#39;, ... , default value &#39;normal&#39;.</p>\n"},"weight":{"!doc":"<p>The font weight, one of &#39;normal&#39;, ..., default value &#39;normal&#39;.</p>\n"}},"dojox/gfx/_base.defaultPath":{"!doc":"<p>Defines the default Path prototype object.</p>\n","path":{"!doc":"<p>The path commands. See W32C SVG 1.0 specification.\nDefaults to empty string value.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Path, default value &#39;path&#39;.</p>\n"}},"dojox/gfx/_base.defaultPolyline":{"!doc":"<p>Defines the default PolyLine prototype.</p>\n","points":{"!doc":"<p>An array of point objects [{x:0,y:0},...] defining the default polyline&#39;s line segments. Value is an empty array [].</p>\n"},"type":{"!doc":"<p>Specifies this object is a PolyLine, default value &#39;polyline&#39;.</p>\n"}},"dojox/gfx/_base.defaultRect":{"!doc":"<p>Defines the default Rect prototype.</p>\n","height":{"!doc":"<p>The height of the default rectangle, value 100.</p>\n"},"r":{"!doc":"<p>The corner radius for the default rectangle, value 0.</p>\n"},"type":{"!doc":"<p>Specifies this default object is a type of Rect. Value is &#39;rect&#39;</p>\n"},"width":{"!doc":"<p>The width of the default rectangle, value 100.</p>\n"},"x":{"!doc":"<p>The X coordinate of the default rectangles position, value 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the default rectangle&#39;s position, value 0.</p>\n"}},"dojox/gfx/_base.defaultEllipse":{"!doc":"<p>Defines the default Ellipse prototype.</p>\n","cx":{"!doc":"<p>The X coordinate of the center of the ellipse, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the ellipse, default value 0.</p>\n"},"rx":{"!doc":"<p>The radius of the ellipse in the X direction, default value 200.</p>\n"},"ry":{"!doc":"<p>The radius of the ellipse in the Y direction, default value 200.</p>\n"},"type":{"!doc":"<p>Specifies that this object is a type of Ellipse, value is &#39;ellipse&#39;</p>\n"}},"dojox/gfx/_base.defaultCircle":{"!doc":"<p>An object defining the default Circle prototype.</p>\n","cx":{"!doc":"<p>The X coordinate of the center of the circle, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the circle, default value 0.</p>\n"},"r":{"!doc":"<p>The radius, default value 100.</p>\n"},"type":{"!doc":"<p>Specifies this object is a circle, value &#39;circle&#39;</p>\n"}},"dojox/gfx/_base.defaultLine":{"!doc":"<p>An object defining the default Line prototype.</p>\n","type":{"!doc":"<p>Specifies this is a Line, value &#39;line&#39;</p>\n"},"x1":{"!doc":"<p>The X coordinate of the start of the line, default value 0.</p>\n"},"x2":{"!doc":"<p>The X coordinate of the end of the line, default value 100.</p>\n"},"y1":{"!doc":"<p>The Y coordinate of the start of the line, default value 0.</p>\n"},"y2":{"!doc":"<p>The Y coordinate of the end of the line, default value 100.</p>\n"}},"dojox/gfx/_base.defaultImage":{"!doc":"<p>Defines the default Image prototype.</p>\n","height":{"!doc":"<p>The height of the image, default value 0.</p>\n"},"src":{"!doc":"<p>The src url of the image, defaults to empty string.</p>\n"},"type":{"!doc":"<p>Specifies this object is an image, value &#39;image&#39;.</p>\n"},"width":{"!doc":"<p>The width of the image, default value 0.</p>\n"},"x":{"!doc":"<p>The X coordinate of the image&#39;s position, default value 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the image&#39;s position, default value 0.</p>\n"}},"dojox/gfx/_base.defaultText":{"!doc":"<p>Defines the default Text prototype.</p>\n","align":{"!doc":"<p>The horizontal text alignment, one of &#39;start&#39;, &#39;end&#39;, &#39;center&#39;. Default value &#39;start&#39;.</p>\n"},"decoration":{"!doc":"<p>The text decoration , one of &#39;none&#39;, ... . Default value &#39;none&#39;.</p>\n"},"kerning":{"!doc":"<p>Whether kerning is used on the text, boolean default value true.</p>\n"},"rotated":{"!doc":"<p>Whether the text is rotated, boolean default value false.</p>\n"},"text":{"!doc":"<p>The text to be displayed, default value empty string.</p>\n"},"type":{"!doc":"<p>Specifies this is a Text shape, value &#39;text&#39;.</p>\n"},"x":{"!doc":"<p>The X coordinate of the text position, default value 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the text position, default value 0.</p>\n"}},"dojox/gfx/_base.defaultTextPath":{"!doc":"<p>Defines the default TextPath prototype.</p>\n","align":{"!doc":"<p>The horizontal text alignment, one of &#39;start&#39;, &#39;end&#39;, &#39;center&#39;. Default value &#39;start&#39;.</p>\n"},"decoration":{"!doc":"<p>The text decoration , one of &#39;none&#39;, ... . Default value &#39;none&#39;.</p>\n"},"kerning":{"!doc":"<p>Whether kerning is used on the text, boolean default value true.</p>\n"},"rotated":{"!doc":"<p>Whether the text is rotated, boolean default value false.</p>\n"},"text":{"!doc":"<p>The text to be displayed, default value empty string.</p>\n"},"type":{"!doc":"<p>Specifies this is a TextPath, value &#39;textpath&#39;.</p>\n"}},"dojox/gfx/_base.defaultStroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.\nThis object defines the default Stroke prototype.</p>\n","cap":{"!doc":"<p>The endcap style of the path. One of &#39;butt&#39;, &#39;round&#39;, ... . Default value &#39;butt&#39;.</p>\n"},"color":{"!doc":"<p>The color of the stroke, default value &#39;black&#39;.</p>\n"},"join":{"!doc":"<p>The join style to use when combining path segments. Default value 4.</p>\n"},"style":{"!doc":"<p>The style of the stroke, one of &#39;solid&#39;, ... . Default value &#39;solid&#39;.</p>\n"},"type":{"!doc":"<p>Specifies this object is a type of Stroke, value &#39;stroke&#39;.</p>\n"},"width":{"!doc":"<p>The width of a stroke, default value 1.</p>\n"}},"dojox/gfx/_base.defaultLinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the start of the line).  The start of the line is\ndefined at offest 0 with the end of the line at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Linear Gradient, value &#39;linear&#39;</p>\n"},"x1":{"!doc":"<p>The X coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"x2":{"!doc":"<p>The X coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"},"y1":{"!doc":"<p>The Y coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"y2":{"!doc":"<p>The Y coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"}},"dojox/gfx/_base.defaultRadialGradient":{"!doc":"<p>An object specifying the default properties for RadialGradients using in fills patterns.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the center of the radial gradient).\nThe center is defined at offest 0 with the outer edge of the gradient at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"cx":{"!doc":"<p>The X coordinate of the center of the radial gradient, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the radial gradient, default value 0.</p>\n"},"r":{"!doc":"<p>The radius to the end of the radial gradient, default value 100.</p>\n"},"type":{"!doc":"<p>Specifies this is a RadialGradient, value &#39;radial&#39;</p>\n"}},"dojox/gfx/_base.defaultPattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n","height":{"!doc":"<p>The height of the pattern image, default value is 0.</p>\n"},"src":{"!doc":"<p>A url specifying the image to use for the pattern.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Pattern, value &#39;pattern&#39;.</p>\n"},"width":{"!doc":"<p>The width of the pattern image, default value is 0.</p>\n"},"x":{"!doc":"<p>The X coordinate of the position of the pattern, default value is 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the position of the pattern, default value is 0.</p>\n"}},"dojox/gfx/_base.defaultFont":{"!doc":"<p>An object specifying the default properties for a Font used in text operations.</p>\n","family":{"!doc":"<p>The font family, one of &#39;serif&#39;, &#39;sanserif&#39;, ..., default value &#39;serif&#39;.</p>\n"},"size":{"!doc":"<p>The font size (including units), default value &#39;10pt&#39;.</p>\n"},"style":{"!doc":"<p>The font style, one of &#39;normal&#39;, &#39;bold&#39;, default value &#39;normal&#39;.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Font, value &#39;font&#39;.</p>\n"},"variant":{"!doc":"<p>The font variant, one of &#39;normal&#39;, ... , default value &#39;normal&#39;.</p>\n"},"weight":{"!doc":"<p>The font weight, one of &#39;normal&#39;, ..., default value &#39;normal&#39;.</p>\n"}},"dojox/gfx/_base.matrix":{"flipX":{"!doc":"<p>a matrix, which reflects points at x = 0 line: flipX * (x, y) == (-x, y)</p>\n"},"flipXY":{"!doc":"<p>a matrix, which reflects points at the origin of coordinates: flipXY * (x, y) == (-x, -y)</p>\n"},"flipY":{"!doc":"<p>a matrix, which reflects points at y = 0 line: flipY * (x, y) == (x, -y)</p>\n"},"identity":{"!doc":"<p>an identity matrix constant: identity * (x, y) == (x, y)</p>\n"},"clone":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>creates a copy of a 2D matrix</p>\n"},"invert":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>inverts a 2D matrix</p>\n"},"isIdentity":{"!type":"fn(matrix: ?) -> bool","!doc":"<p>returns whether the specified matrix is the identity.</p>\n"},"Matrix2D":{"!type":"fn(arg: ?)","!doc":"<p>a 2D matrix object</p>\n"},"multiply":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>combines matrices by multiplying them sequentially in the given order</p>\n"},"multiplyPoint":{"!type":"fn(matrix: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>applies a matrix to a point</p>\n"},"multiplyRectangle":{"!type":"fn(matrix: ?, rect: ?) -> +dojox","!doc":"<p>Applies a matrix to a rectangle.</p>\n"},"normalize":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>converts an object to a matrix, if necessary</p>\n"},"project":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms an orthogonal projection matrix</p>\n"},"reflect":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a reflection matrix</p>\n"},"rotate":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms a rotating matrix</p>\n"},"rotateAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>rotates a picture using a specified point as a center of rotation</p>\n"},"rotateg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms a rotating matrix</p>\n"},"rotategAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>rotates a picture using a specified point as a center of rotation</p>\n"},"scale":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a scaling matrix</p>\n"},"scaleAt":{"!type":"fn(a: ?, b?: ?, c: ?, d: ?) -> +dojox","!doc":"<p>scales a picture using a specified point as a center of scaling</p>\n"},"skewX":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms an x skewing matrix</p>\n"},"skewXAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the x axis using a specified point as a center of skewing</p>\n"},"skewXg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms an x skewing matrix</p>\n"},"skewXgAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the x axis using a specified point as a center of skewing</p>\n"},"skewY":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms a y skewing matrix</p>\n"},"skewYAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the y axis using a specified point as a center of skewing</p>\n"},"skewYg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms a y skewing matrix</p>\n"},"skewYgAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the y axis using a specified point as a center of skewing</p>\n"},"translate":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a translation matrix</p>\n"}},"dojox/gfx/_base.shape":{"!doc":"<p>This module contains the core graphics Shape API.\nDifferent graphics renderer implementation modules (svg, canvas, vml, silverlight, etc.) extend this\nbasic api to provide renderer-specific implementations for each shape.</p>\n","_eventsProcessing":{},"Container":{"!doc":"<p>a container of shapes, which can be used\nas a foundation for renderer-specific groups, or as a way\nto logically group shapes (e.g, to propagate matricies)</p>\n"},"Creator":{"!doc":"<p>shape creators</p>\n"},"byId":{"!type":"fn(id: ?) -> +dojox","!doc":"<p>Returns the shape that matches the specified id.</p>\n"},"Circle":{"!type":"fn()"},"dispose":{"!type":"fn(s: ?, recurse?: ?)","!doc":"<p>Removes the specified shape from the registry.</p>\n"},"Ellipse":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"register":{"!type":"fn(s: ?) -> number","!doc":"<p>Register the specified shape into the graphics registry.</p>\n"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"}},"dojox/gfx/_base.Point":{"!type":"fn()","prototype":{"!doc":"<p>2D point for drawings - {x, y}</p>\n"}},"dojox/gfx/_base.Rectangle":{"!type":"fn()","prototype":{"!doc":"<p>rectangle - {x, y, width, height}</p>\n"}},"dojox/gfx/_base.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape, which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","textDir":{"!doc":"<p>Will be used for inheritance, or as default for text objects\nthat textDir wasn&#39;t directly specified for them but the bidi support was required.</p>\n"},"add":{"!type":"fn(shape: ?) -> ?","!doc":"<p>adds a shape to the list</p>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn(destroy?: ?) -> fn","!doc":"<p>removes all shapes from a group/surface.</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_base.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a generic rectangle</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box (its shape in this case)</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_base.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a generic circle</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_base.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>a generic ellipse</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_base.Line":{"!type":"fn()","prototype":{"!doc":"<p>a generic line (do not instantiate it directly)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_base.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a generic polyline/polygon (do not instantiate it directly)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(points: ?, closed: ?) -> fn","!doc":"<p>sets a polyline/polygon shape object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_base.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> ?","!doc":"<p>adds a shape to the list</p>\n"},"clear":{"!type":"fn(destroy?: ?) -> fn","!doc":"<p>removes all shapes from a group/surface.</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()","!doc":"<p>destroy all relevant external resources and release all\nexternal references to make this object garbage-collectible</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> ?","!doc":"<p>gets current width and height in pixels</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/_base.gradutils":{"getColor":{"!type":"fn(fill: ?, pt: ?)","!doc":"<p>sample a color from a gradient using a point</p>\n"},"reverse":{"!type":"fn(fill: ?)","!doc":"<p>reverses a gradient</p>\n"}},"dojox/gfx/_base.fx":{"animateFill":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation which will change fill color over time.\nOnly solid fill color is supported at the moment</p>\n"},"animateFont":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation which will change font properties over time.</p>\n"},"animateStroke":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation which will change stroke properties over time.</p>\n"},"animateTransform":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation which will change transformation over time.</p>\n"}},"dojox/gfx/_base.utils":{"_gfxSvgProxy":{},"_initSvgSerializerDeferred":{},"deserialize":{"!type":"fn(parent: ?, object: ?) -> ?","!doc":"<p>Takes a surface or a shape and populates it with an object produced by serialize().</p>\n"},"forEach":{"!type":"fn(object: ?, f: ?, o?: ?)","!doc":"<p>Takes a shape or a surface and applies a function &quot;f&quot; to in the context of &quot;o&quot;\n(or global, if missing). If &quot;shape&quot; was a surface or a group, it applies the same\nfunction to all children recursively effectively visiting all shapes of the underlying scene graph.</p>\n"},"fromJson":{"!type":"fn(parent: ?, json: ?) -> ?","!doc":"<p>Works just like deserialize() but takes a JSON representation of the object.</p>\n"},"serialize":{"!type":"fn(object: ?) -> ?","!doc":"<p>Takes a shape or a surface and returns an object, which describes underlying shapes.</p>\n"},"toJson":{"!type":"fn(object: ?, prettyPrint?: ?) -> string","!doc":"<p>Works just like serialize() but returns a JSON string. If prettyPrint is true, the string is pretty-printed to make it more human-readable.</p>\n"},"toSvg":{"!type":"fn(surface: ?) -> +any","!doc":"<p>Function to serialize a GFX surface to SVG text.</p>\n"}},"dojox/gfx/_base.path":{"!doc":"<p>This module contains the core graphics Path API.\nPath command format follows the W3C SVG 1.0 Path api.</p>\n","Path":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_base.Path":{"!type":"fn()","prototype":{"!doc":"<p>a generalized path shape</p>\n","_2PI":{},"_validSegments":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)","!doc":"<p>a path constructor</p>\n"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns the bounding box {x, y, width, height} or null</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/_base.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a generalized TextPath shape</p>\n","_2PI":{},"_validSegments":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)","!doc":"<p>a TextPath shape constructor</p>\n"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns the bounding box {x, y, width, height} or null</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/_base._gfxBidiSupport":{},"dojox/gfx/_base.Mover":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(shape: ?, e: ?, host?: ?)","!doc":"<p>an object, which makes a shape follow the mouse,\nused as a default mover, and as a base class for custom movers</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops the move, deletes all references, so the object can be garbage-collected</p>\n"}}},"dojox/gfx/_base.__MoveableCtorArgs":{"!type":"fn()","prototype":{"!doc":"<p>The arguments used for dojox/gfx/Moveable constructor.</p>\n","delay":{"!doc":"<p>delay move by this number of pixels</p>\n"},"mover":{"!type":"fn()","!doc":"<p>a constructor of custom Mover</p>\n"}}},"dojox/gfx/_base.Moveable":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(shape: ?, params: ?)","!doc":"<p>an object, which makes a shape moveable</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops watching for possible move, deletes all references, so the object can be garbage-collected</p>\n"}}},"dojox/gfx/_base.VectorText":{},"dojox/gfx/_base.vectorFontFitting":{"FIT":{},"FLOW":{},"NONE":{}},"dojox/gfx/_base.defaultVectorText":{"align":{},"decoration":{},"fitting":{},"height":{},"leading":{},"text":{},"type":{},"width":{},"x":{},"y":{}},"dojox/gfx/_base.defaultVectorFont":{"family":{},"size":{},"type":{}},"dojox/gfx/_base._vectorFontCache":{},"dojox/gfx/_base._svgFontCache":{},"dojox/gfx/_base.VectorFont":{"!type":"fn()","prototype":{"!doc":"<p>An implementation of the SVG Font 1.1 spec, using dojox/gfx.\n\n</p>\n<p>Basic interface:\n\n</p>\n<pre><code>var f = new gfx.Font(url|string);\nsurface||group.createVectorText(text)\n.setFill(fill)\n.setStroke(stroke)\n.setFont(fontStyleObject);</code></pre>\n<p>The arguments passed to createVectorText are the same as you would\npass to surface||group.createText; the difference is that this\nis entirely renderer-agnostic, and the return value is a subclass\nof dojox/gfx.Group.\n\n</p>\n<p>Note also that the &quot;defaultText&quot; object is slightly different:\n{ type:&quot;vectortext&quot;, x:0, y:0, width:null, height: null,\ntext: &quot;&quot;, align: &quot;start&quot;, decoration: &quot;none&quot; }\n\n</p>\n<p>...as well as the &quot;defaultVectorFont&quot; object:\n{ type:&quot;vectorfont&quot;, size:&quot;10pt&quot; }\n\n</p>\n<p>The reason for this should be obvious: most of the style for the font is defined\nby the font object itself.\n\n</p>\n<p>Note that this will only render IF and WHEN you set the font.</p>\n","_entityRe":{},"constructor":{"!type":"fn(url: ?)","!doc":"<p>Create this font object based on the SVG Font definition at url.</p>\n"},"draw":{"!type":"fn(group: ?, textArgs: ?, fontArgs: ?, fillArgs: ?, strokeArgs: ?) -> +dojox","!doc":"<p>based on the passed parameters, draw the given text using paths\ndefined by this font.</p>\n"},"getBaseline":{"!type":"fn(scale?: ?) -> number","!doc":"<p>Find the baseline coord for alignment; adjust for scale if passed.</p>\n"},"getCenterline":{"!type":"fn(scale?: ?) -> number","!doc":"<p>return the y coordinate that is the center of the viewbox.</p>\n"},"getLineHeight":{"!type":"fn(scale?: ?) -> number","!doc":"<p>return the height of a single line, sans leading, based on scale.</p>\n"},"getWidth":{"!type":"fn(text: ?, scale?: ?) -> number","!doc":"<p>Get the width of the rendered text without actually rendering it.</p>\n"},"initialized":{"!type":"fn() -> bool","!doc":"<p>Return if we&#39;ve loaded a font def, and the parsing was successful.</p>\n"},"load":{"!type":"fn(url: ?) -> fn","!doc":"<p>Load the passed SVG and send it to the parser for parsing.</p>\n"}}},"dojox/gfx/_base.arc":{"!doc":"<p>This module contains the core graphics Arc functions.</p>\n","curvePI4":{"!doc":"<p>an object with properties of an arc around a unit circle from 0 to pi/4</p>\n"},"arcAsBezier":{"!type":"fn(last: ?, rx: ?, ry: ?, xRotg: ?, large: ?, sweep: ?, x: ?, y: ?) -> []","!doc":"<p>calculates an arc as a series of Bezier curves\ngiven the last point and a standard set of SVG arc parameters,\nit returns an array of arrays of parameters to form a series of\nabsolute Bezier curves.</p>\n"},"unitArcAsBezier":{"!type":"fn(alpha: ?)","!doc":"<p>return a start point, 1st and 2nd control points, and an end point of\na an arc, which is reflected on the x axis</p>\n"}},"dojox/gfx/_base.bezierutils":{"computeLength":{"!type":"fn(points: ?) -> number","!doc":"<p>Returns the length of the given bezier curve.</p>\n"},"distance":{"!type":"fn(x1: ?, y1: ?, x2: ?, y2: ?) -> ?","!doc":"<p>Returns the distance between the specified points.</p>\n"},"splitBezierAtT":{"!type":"fn(points: ?, t: ?) -> ?"},"tAtLength":{"!type":"fn(points: ?, length: ?) -> number","!doc":"<p>Returns the t corresponding to the given length for the specified bezier curve.</p>\n"}},"dojox/gfx/_base.canvas":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of this module alone will only add in drawing support.\nThe additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.\nThe id of the Canvas renderer is &#39;canvas&#39;.  This id can be used when switch Dojo&#39;s\ngraphics context between renderer implementations.  See dojox/gfx/_base.switchRenderer\nAPI.</p>\n","attachNode":{"!type":"fn()"},"attachSurface":{"!type":"fn()"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +instance","!doc":"<p>creates a surface (Canvas)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_base.canvasWithEvents":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers which extends\nthe basic canvas drawing renderer bridge to add additional support for graphics events\non Shapes.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of the canvas module alone will only add in drawing support.\nThis additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.</p>\n","Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)","!doc":"<p>creates a surface (Canvas)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_base.canvas_attach":{},"dojox/gfx/_base.canvasext":{},"dojox/gfx/_base.gradient":{"project":{"!type":"fn(matrix: ?, gradient: ?, tl: ?, rb: ?, ttl: ?, trb: ?) -> ?","!doc":"<p>Returns a new gradient using the &quot;VML algorithm&quot; and suitable for VML.</p>\n"},"rescale":{"!type":"fn(stops: ?, from: ?, to: ?) -> []","!doc":"<p>Recalculates a gradient from 0-1 window to\n&quot;from&quot;-&quot;to&quot; window blending and replicating colors,\nif necessary.</p>\n"}},"dojox/gfx/_base.move":{},"dojox/gfx/_base.silverlight":{"!doc":"<p>This the graphics rendering bridge for the Microsoft Silverlight plugin.\nSilverlight is a faster implementation on IE6-8 than the default 2d graphics, VML</p>\n","attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)","!doc":"<p>creates a surface (Silverlight)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_base.silverlight_attach":{},"dojox/gfx/_base.svg":{"!doc":"<p>This the graphics rendering bridge for browsers compliant with W3C SVG1.0.\nThis is the preferred renderer to use for interactive and accessible graphics.</p>\n","dasharray":{},"useSvgWeb":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"getRef":{"!type":"fn(name: ?) -> +any","!doc":"<p>looks up a node by its external name</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_base.svgext":{},"dojox/gfx/_base.vml":{"!doc":"<p>This the default graphics rendering bridge for IE6-7.\nThis renderer is very slow.  For best performance on IE6-8, use Silverlight plugin.\nIE9+ defaults to the standard W3C SVG renderer.</p>\n","_bool":{},"text_alignment":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +any","!doc":"<p>creates a surface (VML)</p>\n"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/shape":{"!doc":"<p>This module contains the core graphics Shape API.\nDifferent graphics renderer implementation modules (svg, canvas, vml, silverlight, etc.) extend this\nbasic api to provide renderer-specific implementations for each shape.</p>\n","_eventsProcessing":{},"Container":{"!doc":"<p>a container of shapes, which can be used\nas a foundation for renderer-specific groups, or as a way\nto logically group shapes (e.g, to propagate matricies)</p>\n"},"Creator":{"!doc":"<p>shape creators</p>\n"},"byId":{"!type":"fn(id: ?) -> +dojox","!doc":"<p>Returns the shape that matches the specified id.</p>\n"},"Circle":{"!type":"fn()"},"dispose":{"!type":"fn(s: ?, recurse?: ?)","!doc":"<p>Removes the specified shape from the registry.</p>\n"},"Ellipse":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"register":{"!type":"fn(s: ?) -> number","!doc":"<p>Register the specified shape into the graphics registry.</p>\n"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"}},"dojox/gfx/shape.Shape":{"!type":"fn()","prototype":{"!doc":"<p>a Shape object, which knows how to apply\ngraphical attributes and transformations</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/shape._eventsProcessing":{"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"}},"dojox/gfx/shape.Container":{"!doc":"<p>a container of shapes, which can be used\nas a foundation for renderer-specific groups, or as a way\nto logically group shapes (e.g, to propagate matricies)</p>\n","add":{"!type":"fn(shape: ?) -> ?","!doc":"<p>adds a shape to the list</p>\n"},"clear":{"!type":"fn(destroy?: ?) -> fn","!doc":"<p>removes all shapes from a group/surface.</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"}},"dojox/gfx/shape.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> ?","!doc":"<p>adds a shape to the list</p>\n"},"clear":{"!type":"fn(destroy?: ?) -> fn","!doc":"<p>removes all shapes from a group/surface.</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()","!doc":"<p>destroy all relevant external resources and release all\nexternal references to make this object garbage-collectible</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> ?","!doc":"<p>gets current width and height in pixels</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/shape.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a generic rectangle</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box (its shape in this case)</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/shape.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>a generic ellipse</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/shape.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a generic circle</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/shape.Line":{"!type":"fn()","prototype":{"!doc":"<p>a generic line (do not instantiate it directly)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/shape.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a generic polyline/polygon (do not instantiate it directly)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(points: ?, closed: ?) -> fn","!doc":"<p>sets a polyline/polygon shape object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/shape.Image":{"!type":"fn()","prototype":{"!doc":"<p>a generic image (do not instantiate it directly)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box (its shape in this case)</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn() -> fn","!doc":"<p>ignore setting a fill style</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn() -> fn","!doc":"<p>ignore setting a stroke style</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/shape.Text":{"!type":"fn()","prototype":{"!doc":"<p>a generic text (do not instantiate it directly)</p>\n","textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/shape.Creator":{"!doc":"<p>shape creators</p>\n","createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"}},"dojox/gfx/matrix":{"flipX":{"!doc":"<p>a matrix, which reflects points at x = 0 line: flipX * (x, y) == (-x, y)</p>\n"},"flipXY":{"!doc":"<p>a matrix, which reflects points at the origin of coordinates: flipXY * (x, y) == (-x, -y)</p>\n"},"flipY":{"!doc":"<p>a matrix, which reflects points at y = 0 line: flipY * (x, y) == (x, -y)</p>\n"},"identity":{"!doc":"<p>an identity matrix constant: identity * (x, y) == (x, y)</p>\n"},"clone":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>creates a copy of a 2D matrix</p>\n"},"invert":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>inverts a 2D matrix</p>\n"},"isIdentity":{"!type":"fn(matrix: ?) -> bool","!doc":"<p>returns whether the specified matrix is the identity.</p>\n"},"Matrix2D":{"!type":"fn(arg: ?)","!doc":"<p>a 2D matrix object</p>\n"},"multiply":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>combines matrices by multiplying them sequentially in the given order</p>\n"},"multiplyPoint":{"!type":"fn(matrix: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>applies a matrix to a point</p>\n"},"multiplyRectangle":{"!type":"fn(matrix: ?, rect: ?) -> +dojox","!doc":"<p>Applies a matrix to a rectangle.</p>\n"},"normalize":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>converts an object to a matrix, if necessary</p>\n"},"project":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms an orthogonal projection matrix</p>\n"},"reflect":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a reflection matrix</p>\n"},"rotate":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms a rotating matrix</p>\n"},"rotateAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>rotates a picture using a specified point as a center of rotation</p>\n"},"rotateg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms a rotating matrix</p>\n"},"rotategAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>rotates a picture using a specified point as a center of rotation</p>\n"},"scale":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a scaling matrix</p>\n"},"scaleAt":{"!type":"fn(a: ?, b?: ?, c: ?, d: ?) -> +dojox","!doc":"<p>scales a picture using a specified point as a center of scaling</p>\n"},"skewX":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms an x skewing matrix</p>\n"},"skewXAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the x axis using a specified point as a center of skewing</p>\n"},"skewXg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms an x skewing matrix</p>\n"},"skewXgAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the x axis using a specified point as a center of skewing</p>\n"},"skewY":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms a y skewing matrix</p>\n"},"skewYAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the y axis using a specified point as a center of skewing</p>\n"},"skewYg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms a y skewing matrix</p>\n"},"skewYgAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the y axis using a specified point as a center of skewing</p>\n"},"translate":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a translation matrix</p>\n"}},"dojox/charting/SimpleTheme":{"!type":"fn()","prototype":{"!doc":"<p>A SimpleTheme or Theme is a pre-defined object, primarily JSON-based, that makes up the definitions to\nstyle a chart.\n</p>\n","shapeSpaces":{},"addMarker":{"!type":"fn(name: ?, segment: ?)","!doc":"<p>Add a custom marker to this theme.</p>\n"},"addMixin":{"!type":"fn(theme: ?, elementType: ?, mixin: ?, doPost: ?) -> +dojox","!doc":"<p>Add a mixin object to the passed theme and process.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Clear and reset the internal pointer to start fresh.</p>\n"},"clone":{"!type":"fn() -> +dojox.charting.SimpleTheme","!doc":"<p>Clone the current theme.</p>\n"},"constructor":{"!type":"fn(kwArgs: ?)","!doc":"<p>Initialize a theme using the keyword arguments.  Note that the arguments\nlook like the example (above), and may include a few more parameters.</p>\n"},"getTick":{"!type":"fn(name: ?, mixin?: ?) -> ?","!doc":"<p>Calculates and merges tick parameters.</p>\n"},"inspectObjects":{"!type":"fn(f: ?)"},"next":{"!type":"fn(elementType?: ?, mixin?: ?, doPost?: ?) -> ?","!doc":"<p>Get the next color or series theme.</p>\n"},"post":{"!type":"fn(theme: ?, elementType: ?) -> +dojox","!doc":"<p>Process any post-shape fills.</p>\n"},"reverseFills":{"!type":"fn()"},"setMarkers":{"!type":"fn(obj: ?)","!doc":"<p>Set all the markers of this theme at once.  obj should be a\ndictionary of keys and path segments.\n</p>\n"},"skip":{"!type":"fn()","!doc":"<p>Skip the next internal color.</p>\n"}},"defaultColors":{},"defaultMarkers":{},"defaultTheme":{}},"dojox/charting/SimpleTheme.defaultMarkers":{"CIRCLE":{},"CROSS":{},"DIAMOND":{},"SQUARE":{},"TRIANGLE":{},"TRIANGLE_INVERTED":{},"X":{}},"dojox/charting/SimpleTheme.defaultTheme":{"axis":{},"chart":{},"indicator":{},"marker":{},"plotarea":{},"series":{}},"dojox/lang/utils":{"coerceType":{"!type":"fn(target: ?, source: ?) -> ?","!doc":"<p>Coerces one object to the type of another.</p>\n"},"merge":{"!type":"fn(object: ?, mixin: ?) -> +instance","!doc":"<p>Merge two objects structurally, mixin properties will override object&#39;s properties.</p>\n"},"updateWithObject":{"!type":"fn(target: ?, source: ?, conv?: ?) -> ?","!doc":"<p>Updates an existing object in place with properties from an &quot;source&quot; object.</p>\n"},"updateWithPattern":{"!type":"fn(target: ?, source: ?, pattern: ?, conv?: ?) -> ?","!doc":"<p>Updates an existing object in place with properties from an &quot;source&quot; object.</p>\n"}},"dojox/gfx/gradutils":{"getColor":{"!type":"fn(fill: ?, pt: ?)","!doc":"<p>sample a color from a gradient using a point</p>\n"},"reverse":{"!type":"fn(fill: ?)","!doc":"<p>reverses a gradient</p>\n"}},"dojox/charting/Series":{"!type":"fn()","prototype":{"!proto":"dojox/charting/Element.prototype","!doc":"<p>An object representing a series of data for plotting on a chart.</p>\n","chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"cleanGroup":{"!type":"fn(creator?: ?) -> +dojox","!doc":"<p>Clean any elements (HTML or GFX-based) out of our group, and create a new one.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Clear the calculated additional parameters set on this series.</p>\n"},"constructor":{"!type":"fn(chart: ?, data: ?, kwArgs?: ?)","!doc":"<p>Create a new data series object for use within charting.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"purgeGroup":{"!type":"fn() -> +dojox","!doc":"<p>Clear any elements out of our group, and destroy the group.</p>\n"},"update":{"!type":"fn(data: ?)","!doc":"<p>Set data and make this object dirty, so it can be redrawn.</p>\n"}}},"dojox/charting/axis2d/common":{"createText":{}},"dojox/charting/axis2d/common.createText":{"gfx":{"!type":"fn(chart: ?, creator: ?, x: ?, y: ?, align: ?, text: ?, font: ?, fontColor: ?) -> +dojox.gfx.Text","!doc":"<p>Use dojox.gfx to create any text.</p>\n"},"html":{"!type":"fn(chart: ?, creator: ?, x: ?, y: ?, align: ?, text: ?, font: ?, fontColor: ?, labelWidth?: ?) -> +Node","!doc":"<p>Use the HTML DOM to create any text.</p>\n"}},"dojox/lang/functional":{"buildLambda":{"!type":"fn(s: ?) -> string","!doc":"<p>builds a function from a snippet, returns a string, which\nrepresents the function.</p>\n"},"clearLambdaCache":{"!type":"fn()","!doc":"<p>clears internal cache of lambdas</p>\n"},"every":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"everyRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"filter":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"filterIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with all attributes that pass the test\nimplemented by the provided function.</p>\n"},"filterRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"foldl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns the final value.</p>\n"},"foldr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns the final value.</p>\n"},"forEach":{"!type":"fn(a: ?, f: ?, o?: ?) -> string","!doc":"<p>executes a provided function once per array element.</p>\n"},"forEachRev":{"!type":"fn(a: ?, f: ?, o?: ?)","!doc":"<p>executes a provided function once per array element.</p>\n"},"forIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> string","!doc":"<p>iterates over all object attributes.</p>\n"},"keys":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all keys in the object</p>\n"},"lambda":{"!type":"fn(s: ?) -> fn()","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns a function object; functions are passed through\nunmodified.</p>\n"},"map":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"mapIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with the results of calling\na provided function on every attribute in this object.</p>\n"},"mapRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"rawLambda":{"!type":"fn(s: ?) -> ?","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns an object describing the function; functions are\npassed through unmodified.</p>\n"},"reduce":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from left-to-right) as to reduce it to a single value.</p>\n"},"reduceRight":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from right-to-left) as to reduce it to a single value.</p>\n"},"repeat":{"!type":"fn(n: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>builds an array by repeatedly applying a unary function N times\nwith a seed value Z. N should be greater than 0.</p>\n"},"scanl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns an array\nof values produced by foldl() at that point.</p>\n"},"scanl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns an array of values produced by foldl1() at that\npoint.</p>\n"},"scanr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns an array\nof values produced by foldr() at that point.</p>\n"},"scanr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns an array of values produced by foldr1() at that\npoint.</p>\n"},"some":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"someRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"unfold":{"!type":"fn(pr: ?, f: ?, g: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by unfolding a value</p>\n"},"until":{"!type":"fn(pr: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by repeatedly applying a unary function with\na seed value Z until the predicate is satisfied.</p>\n"},"values":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all values in the object</p>\n"}},"dojox/lang/functional/lambda":{"buildLambda":{"!type":"fn(s: ?) -> string","!doc":"<p>builds a function from a snippet, returns a string, which\nrepresents the function.</p>\n"},"clearLambdaCache":{"!type":"fn()","!doc":"<p>clears internal cache of lambdas</p>\n"},"every":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"everyRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"filter":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"filterIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with all attributes that pass the test\nimplemented by the provided function.</p>\n"},"filterRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"foldl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns the final value.</p>\n"},"foldr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns the final value.</p>\n"},"forEach":{"!type":"fn(a: ?, f: ?, o?: ?) -> string","!doc":"<p>executes a provided function once per array element.</p>\n"},"forEachRev":{"!type":"fn(a: ?, f: ?, o?: ?)","!doc":"<p>executes a provided function once per array element.</p>\n"},"forIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> string","!doc":"<p>iterates over all object attributes.</p>\n"},"keys":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all keys in the object</p>\n"},"lambda":{"!type":"fn(s: ?) -> fn()","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns a function object; functions are passed through\nunmodified.</p>\n"},"map":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"mapIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with the results of calling\na provided function on every attribute in this object.</p>\n"},"mapRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"rawLambda":{"!type":"fn(s: ?) -> ?","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns an object describing the function; functions are\npassed through unmodified.</p>\n"},"reduce":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from left-to-right) as to reduce it to a single value.</p>\n"},"reduceRight":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from right-to-left) as to reduce it to a single value.</p>\n"},"repeat":{"!type":"fn(n: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>builds an array by repeatedly applying a unary function N times\nwith a seed value Z. N should be greater than 0.</p>\n"},"scanl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns an array\nof values produced by foldl() at that point.</p>\n"},"scanl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns an array of values produced by foldl1() at that\npoint.</p>\n"},"scanr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns an array\nof values produced by foldr() at that point.</p>\n"},"scanr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns an array of values produced by foldr1() at that\npoint.</p>\n"},"some":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"someRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"unfold":{"!type":"fn(pr: ?, f: ?, g: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by unfolding a value</p>\n"},"until":{"!type":"fn(pr: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by repeatedly applying a unary function with\na seed value Z until the predicate is satisfied.</p>\n"},"values":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all values in the object</p>\n"}},"dojox/lang/functional/array":{"buildLambda":{"!type":"fn(s: ?) -> string","!doc":"<p>builds a function from a snippet, returns a string, which\nrepresents the function.</p>\n"},"clearLambdaCache":{"!type":"fn()","!doc":"<p>clears internal cache of lambdas</p>\n"},"every":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"everyRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"filter":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"filterIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with all attributes that pass the test\nimplemented by the provided function.</p>\n"},"filterRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"foldl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns the final value.</p>\n"},"foldr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns the final value.</p>\n"},"forEach":{"!type":"fn(a: ?, f: ?, o?: ?) -> string","!doc":"<p>executes a provided function once per array element.</p>\n"},"forEachRev":{"!type":"fn(a: ?, f: ?, o?: ?)","!doc":"<p>executes a provided function once per array element.</p>\n"},"forIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> string","!doc":"<p>iterates over all object attributes.</p>\n"},"keys":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all keys in the object</p>\n"},"lambda":{"!type":"fn(s: ?) -> fn()","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns a function object; functions are passed through\nunmodified.</p>\n"},"map":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"mapIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with the results of calling\na provided function on every attribute in this object.</p>\n"},"mapRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"rawLambda":{"!type":"fn(s: ?) -> ?","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns an object describing the function; functions are\npassed through unmodified.</p>\n"},"reduce":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from left-to-right) as to reduce it to a single value.</p>\n"},"reduceRight":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from right-to-left) as to reduce it to a single value.</p>\n"},"repeat":{"!type":"fn(n: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>builds an array by repeatedly applying a unary function N times\nwith a seed value Z. N should be greater than 0.</p>\n"},"scanl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns an array\nof values produced by foldl() at that point.</p>\n"},"scanl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns an array of values produced by foldl1() at that\npoint.</p>\n"},"scanr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns an array\nof values produced by foldr() at that point.</p>\n"},"scanr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns an array of values produced by foldr1() at that\npoint.</p>\n"},"some":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"someRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"unfold":{"!type":"fn(pr: ?, f: ?, g: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by unfolding a value</p>\n"},"until":{"!type":"fn(pr: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by repeatedly applying a unary function with\na seed value Z until the predicate is satisfied.</p>\n"},"values":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all values in the object</p>\n"}},"dojox/lang/functional/object":{"buildLambda":{"!type":"fn(s: ?) -> string","!doc":"<p>builds a function from a snippet, returns a string, which\nrepresents the function.</p>\n"},"clearLambdaCache":{"!type":"fn()","!doc":"<p>clears internal cache of lambdas</p>\n"},"every":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"everyRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"filter":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"filterIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with all attributes that pass the test\nimplemented by the provided function.</p>\n"},"filterRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"foldl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns the final value.</p>\n"},"foldr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns the final value.</p>\n"},"forEach":{"!type":"fn(a: ?, f: ?, o?: ?) -> string","!doc":"<p>executes a provided function once per array element.</p>\n"},"forEachRev":{"!type":"fn(a: ?, f: ?, o?: ?)","!doc":"<p>executes a provided function once per array element.</p>\n"},"forIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> string","!doc":"<p>iterates over all object attributes.</p>\n"},"keys":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all keys in the object</p>\n"},"lambda":{"!type":"fn(s: ?) -> fn()","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns a function object; functions are passed through\nunmodified.</p>\n"},"map":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"mapIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with the results of calling\na provided function on every attribute in this object.</p>\n"},"mapRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"rawLambda":{"!type":"fn(s: ?) -> ?","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns an object describing the function; functions are\npassed through unmodified.</p>\n"},"reduce":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from left-to-right) as to reduce it to a single value.</p>\n"},"reduceRight":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from right-to-left) as to reduce it to a single value.</p>\n"},"repeat":{"!type":"fn(n: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>builds an array by repeatedly applying a unary function N times\nwith a seed value Z. N should be greater than 0.</p>\n"},"scanl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns an array\nof values produced by foldl() at that point.</p>\n"},"scanl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns an array of values produced by foldl1() at that\npoint.</p>\n"},"scanr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns an array\nof values produced by foldr() at that point.</p>\n"},"scanr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns an array of values produced by foldr1() at that\npoint.</p>\n"},"some":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"someRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"unfold":{"!type":"fn(pr: ?, f: ?, g: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by unfolding a value</p>\n"},"until":{"!type":"fn(pr: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by repeatedly applying a unary function with\na seed value Z until the predicate is satisfied.</p>\n"},"values":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all values in the object</p>\n"}},"dojox/lang/functional/fold":{},"dojox/lang/functional/reversed":{"buildLambda":{"!type":"fn(s: ?) -> string","!doc":"<p>builds a function from a snippet, returns a string, which\nrepresents the function.</p>\n"},"clearLambdaCache":{"!type":"fn()","!doc":"<p>clears internal cache of lambdas</p>\n"},"every":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"everyRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"filter":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"filterIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with all attributes that pass the test\nimplemented by the provided function.</p>\n"},"filterRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"foldl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns the final value.</p>\n"},"foldr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns the final value.</p>\n"},"forEach":{"!type":"fn(a: ?, f: ?, o?: ?) -> string","!doc":"<p>executes a provided function once per array element.</p>\n"},"forEachRev":{"!type":"fn(a: ?, f: ?, o?: ?)","!doc":"<p>executes a provided function once per array element.</p>\n"},"forIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> string","!doc":"<p>iterates over all object attributes.</p>\n"},"keys":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all keys in the object</p>\n"},"lambda":{"!type":"fn(s: ?) -> fn()","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns a function object; functions are passed through\nunmodified.</p>\n"},"map":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"mapIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with the results of calling\na provided function on every attribute in this object.</p>\n"},"mapRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"rawLambda":{"!type":"fn(s: ?) -> ?","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns an object describing the function; functions are\npassed through unmodified.</p>\n"},"reduce":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from left-to-right) as to reduce it to a single value.</p>\n"},"reduceRight":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from right-to-left) as to reduce it to a single value.</p>\n"},"repeat":{"!type":"fn(n: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>builds an array by repeatedly applying a unary function N times\nwith a seed value Z. N should be greater than 0.</p>\n"},"scanl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns an array\nof values produced by foldl() at that point.</p>\n"},"scanl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns an array of values produced by foldl1() at that\npoint.</p>\n"},"scanr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns an array\nof values produced by foldr() at that point.</p>\n"},"scanr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns an array of values produced by foldr1() at that\npoint.</p>\n"},"some":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"someRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"unfold":{"!type":"fn(pr: ?, f: ?, g: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by unfolding a value</p>\n"},"until":{"!type":"fn(pr: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by repeatedly applying a unary function with\na seed value Z until the predicate is satisfied.</p>\n"},"values":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all values in the object</p>\n"}},"dojox/charting/axis2d/Default":{"!type":"fn()","prototype":{"!proto":"dojox/charting/axis2d/Invisible.prototype","!doc":"<p>The default axis object used in dojox.charting.  See dojox.charting.Chart.addAxis for details.</p>\n","chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"offset":{},"opt":{"!doc":"<p>The actual options used to define this axis, created at initialization.</p>\n"},"optionalParams":{},"scale":{},"scaler":{},"ticks":{},"calculate":{"!type":"fn(min: ?, max: ?, span: ?) -> fn"},"cleanGroup":{"!type":"fn(creator: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all calculated properties on this axis;</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for an axis.</p>\n"},"createLine":{"!type":"fn(creator: ?, params: ?) -> ?"},"createText":{"!type":"fn(labelType: ?, creator: ?, x: ?, y: ?, align: ?, textContent: ?, font: ?, fontColor: ?, labelWidth: ?) -> ?"},"dependOnData":{"!type":"fn() -> bool","!doc":"<p>Find out whether or not the axis options depend on the data in the axis.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getOffsets":{"!type":"fn() -> ?","!doc":"<p>Get the physical offset values for this axis (used in drawing data series). This method is not\nsupposed to be called by the users but internally.</p>\n"},"getScaler":{"!type":"fn() -> ?","!doc":"<p>Get the pre-calculated scaler object.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"getTicks":{"!type":"fn() -> ?","!doc":"<p>Get the pre-calculated ticks object.</p>\n"},"getWindowOffset":{"!type":"fn() -> number","!doc":"<p>Get the current windowing offset for the axis.</p>\n"},"getWindowScale":{"!type":"fn() -> number","!doc":"<p>Get the current windowing scale of the axis.</p>\n"},"initialized":{"!type":"fn() -> bool","!doc":"<p>Finds out if this axis has been initialized or not.</p>\n"},"labelTooltip":{"!type":"fn(elem: ?, chart: ?, label: ?, truncatedLabel: ?, font: ?, elemType: ?)"},"purgeGroup":{"!type":"fn() -> +dojox","!doc":"<p>Clear any elements out of our group, and destroy the group.</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render/draw the axis.</p>\n"},"setWindow":{"!type":"fn(scale: ?, offset: ?) -> +dojox","!doc":"<p>Set the drawing &quot;window&quot; for the axis.</p>\n"}}},"dojox/charting/axis2d/Invisible":{"!type":"fn()","prototype":{"!proto":"dojox/charting/axis2d/Base.prototype","!doc":"<p>A axis object used in dojox.charting.  You can use that axis if you want the axis to be invisible.\nSee dojox.charting.Chart.addAxis for details.\n</p>\n","chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{"!doc":"<p>The default parameters used to define any axis.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{"!doc":"<p>Any optional parameters needed to define an axis.</p>\n"},"calculate":{"!type":"fn(min: ?, max: ?, span: ?) -> +dojox","!doc":"<p>Perform all calculations needed to render this axis.</p>\n"},"cleanGroup":{"!type":"fn(creator?: ?) -> +dojox","!doc":"<p>Clean any elements (HTML or GFX-based) out of our group, and create a new one.</p>\n"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all calculated properties on this axis;</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for an invisible axis.</p>\n"},"dependOnData":{"!type":"fn() -> bool","!doc":"<p>Find out whether or not the axis options depend on the data in the axis.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getOffsets":{"!type":"fn() -> ?","!doc":"<p>A stub function to return any offsets needed for axis and series rendering.</p>\n"},"getScaler":{"!type":"fn() -> ?","!doc":"<p>Get the pre-calculated scaler object.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"getTicks":{"!type":"fn() -> ?","!doc":"<p>Get the pre-calculated ticks object.</p>\n"},"getWindowOffset":{"!type":"fn() -> number","!doc":"<p>Get the current windowing offset for the axis.</p>\n"},"getWindowScale":{"!type":"fn() -> number","!doc":"<p>Get the current windowing scale of the axis.</p>\n"},"initialized":{"!type":"fn() -> bool","!doc":"<p>Finds out if this axis has been initialized or not.</p>\n"},"purgeGroup":{"!type":"fn() -> +dojox","!doc":"<p>Clear any elements out of our group, and destroy the group.</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Stub function to render this axis.</p>\n"},"setWindow":{"!type":"fn(scale: ?, offset: ?) -> +dojox","!doc":"<p>Set the drawing &quot;window&quot; for the axis.</p>\n"}}},"dojox/charting/axis2d/Base":{"!type":"fn()","prototype":{"!proto":"dojox/charting/Element.prototype","!doc":"<p>The base class for any axis.  This is more of an interface/API\ndefinition than anything else; see dojox.charting.axis2d.Default\nfor more details.</p>\n","chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"calculate":{"!type":"fn(min: ?, max: ?, span: ?) -> +dojox","!doc":"<p>Stub function to run the calcuations needed for drawing this axis.</p>\n"},"cleanGroup":{"!type":"fn(creator?: ?) -> +dojox","!doc":"<p>Clean any elements (HTML or GFX-based) out of our group, and create a new one.</p>\n"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Stub function for clearing the axis.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Return a new base axis.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getOffsets":{"!type":"fn() -> ?","!doc":"<p>A stub function to return any offsets needed for axis and series rendering.</p>\n"},"getScaler":{"!type":"fn() -> ?","!doc":"<p>A stub function to return the scaler object created during calculate.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"getTicks":{"!type":"fn() -> ?","!doc":"<p>A stub function to return the object that helps define how ticks are rendered.</p>\n"},"initialized":{"!type":"fn() -> bool","!doc":"<p>Return a flag as to whether or not this axis has been initialized.</p>\n"},"purgeGroup":{"!type":"fn() -> +dojox","!doc":"<p>Clear any elements out of our group, and destroy the group.</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Stub function to render this axis.</p>\n"}}},"dojox/charting/scaler/linear":{"buildScaler":{"!type":"fn(min: ?, max: ?, span: ?, kwArgs: ?, delta?: ?, minorDelta?: ?) -> ?"},"buildTicks":{"!type":"fn(scaler: ?, kwArgs: ?) -> +null"},"getTransformerFromModel":{"!type":"fn(scaler: ?) -> fn"},"getTransformerFromPlot":{"!type":"fn(scaler: ?) -> fn"}},"dojox/charting/scaler/common":{"doIfLoaded":{"!type":"fn(moduleName: ?, ifloaded: ?, ifnotloaded: ?) -> ?"},"getNumericLabel":{"!type":"fn(number: ?, precision: ?, kwArgs: ?) -> ?"}},"dojox/charting/plot2d/Default":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/CartesianBase.prototype","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{"!doc":"<p>The default parameters of this plot.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{"!doc":"<p>The optional parameters of this plot.</p>\n"},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"buildSegments":{"!type":"fn(i: ?, indexed: ?) -> []"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Return a new plot.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createPath":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render/draw everything on this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/CartesianBase":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Base.prototype","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Create a cartesian base plot for cartesian charts.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"purgeGroup":{"!type":"fn()"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render the plot on the chart.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Base":{"!type":"fn()","prototype":{"!proto":"dojox/charting/Element.prototype","!doc":"<p>Base class for all plot types.</p>\n","chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator?: ?) -> +dojox","!doc":"<p>Clean any elements (HTML or GFX-based) out of our group, and create a new one.</p>\n"},"clear":{"!type":"fn() -> +dojox.charting.plot2d.Base","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Create a base plot for charting.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn() -> fn","!doc":"<p>Does nothing.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"purgeGroup":{"!type":"fn() -> +dojox","!doc":"<p>Clear any elements out of our group, and destroy the group.</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render the plot on the chart.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"}}},"dojox/charting/plot2d/common":{"defaultStats":{},"augmentColor":{"!type":"fn(target: ?, color: ?) -> +instance"},"augmentFill":{"!type":"fn(fill: ?, color: ?) -> ?"},"augmentStroke":{"!type":"fn(stroke: ?, color: ?) -> ?"},"calculateBarSize":{"!type":"fn(availableSize: ?, opt: ?, clusterSize?: ?) -> ?"},"collectSimpleStats":{"!type":"fn(series: ?) -> ?"},"collectStackedStats":{"!type":"fn(series: ?) -> ?"},"curve":{"!type":"fn(a: ?, tension: ?) -> ?"},"doIfLoaded":{"!type":"fn(moduleName: ?, ifloaded: ?, ifnotloaded: ?) -> ?"},"getLabel":{"!type":"fn(number: ?, fixed: ?, precision: ?) -> ?"},"makeStroke":{"!type":"fn(stroke: ?) -> ?"}},"dojox/charting/plot2d/common.defaultStats":{"hmax":{},"hmin":{},"vmax":{},"vmin":{}},"dojox/charting/scaler/primitive":{"buildScaler":{"!type":"fn(min: ?, max: ?, span: ?, kwArgs: ?) -> ?"},"buildTicks":{"!type":"fn(scaler: ?, kwArgs: ?) -> ?"},"getTransformerFromModel":{"!type":"fn(scaler: ?) -> fn"},"getTransformerFromPlot":{"!type":"fn(scaler: ?) -> fn"}},"dojox/gfx/fx":{"animateFill":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation which will change fill color over time.\nOnly solid fill color is supported at the moment</p>\n"},"animateFont":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation which will change font properties over time.</p>\n"},"animateStroke":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation which will change stroke properties over time.</p>\n"},"animateTransform":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation which will change transformation over time.</p>\n"}},"dojox/charting/plot2d/_PlotEvents":{"!type":"fn()","prototype":{"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Destroy any internal elements and event handlers.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"}}},"dojox/charting/plot2d/Lines":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Default.prototype","!doc":"<p>A convenience constructor to create a typical line chart.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{"!doc":"<p>The default parameters of this plot.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{"!doc":"<p>The optional parameters of this plot.</p>\n"},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"buildSegments":{"!type":"fn(i: ?, indexed: ?) -> []"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Preset our default plot to be line-based.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createPath":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render/draw everything on this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/themes/Tufte":{},"dojox/charting/themes/common":{"Adobebricks":{},"Algae":{},"Bahamation":{},"base":{},"blue":{},"BlueDusk":{},"Charged":{},"Chris":{},"Claro":{},"CubanShirts":{},"cyan":{},"Desert":{},"Distinctive":{},"Dollar":{},"Electric":{},"Grasshopper":{},"Grasslands":{},"green":{},"GreySkies":{},"Harmony":{},"IndigoNation":{},"Ireland":{},"Julie":{},"MiamiNice":{},"Midwest":{},"Minty":{},"orange":{},"PrimaryColors":{},"purple":{},"PurpleRain":{},"red":{},"Renkoo":{},"RoyalPurples":{},"SageToLime":{},"Shrooms":{},"Tom":{},"Tufte":{},"WatersEdge":{},"Wetland":{},"generateFills":{"!type":"fn(colors: ?, fillPattern: ?, lumFrom: ?, lumTo: ?)","!doc":"<p>generates 2-color gradients using pure colors, a fill pattern, and two luminance values</p>\n"},"generateGradientByIntensity":{"!type":"fn(color: ?, intensityMap: ?)","!doc":"<p>generates gradient colors using an intensity map</p>\n"},"generateMiniTheme":{"!type":"fn(colors: ?, fillPattern: ?, lumFrom: ?, lumTo: ?, lumStroke: ?)","!doc":"<p>generates mini-themes with 2-color gradients using colors, a fill pattern, and three luminance values</p>\n"},"updateFills":{"!type":"fn(themes: ?, fillPattern: ?, lumFrom: ?, lumTo: ?)","!doc":"<p>transforms solid color fills into 2-color gradients using a fill pattern, and two luminance values</p>\n"}},"dojox/layout/FloatingPane":{"!type":"fn()","prototype":{"!proto":"dojox/layout/ContentPane.prototype","!doc":"<p>A non-modal Floating window.</p>\n","_allFPs":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_dockNode":{},"_focusManager":{},"_hideAnim":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_restoreState":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_showAnim":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_startZ":{},"adjustPaths":{"!doc":"<p>Adjust relative paths in html string content to point to this page.\nOnly useful if you grab content from a another folder then the current one</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{},"baseClass":{},"class":{},"cleanContent":{"!doc":"<p>Cleans content to make it less likely to generate DOM/JS errors.\nUseful if you send ContentPane a complete page, instead of a html fragment\nscans for:\n\n</p>\n<ul>\n<li>title Node, remove</li>\n<li>DOCTYPE tag, remove</li>\n</ul>\n"},"closable":{"!doc":"<p>Allow closure of this Node</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"contentClass":{"!doc":"<p>The className to give to the inner node which has the content</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dockable":{"!doc":"<p>Allow minimizing of pane if true</p>\n"},"dockTo":{"!doc":"<p>if empty, will create private layout.Dock that scrolls with viewport\non bottom span of viewport.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>Time is MS to spend toggling in/out node</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"executeScripts":{"!doc":"<p>Execute (eval) scripts that is found in the content</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"iconSrc":{"!doc":"<p>[not implemented yet] will be either icon in titlepane to left\nof Title, and/or icon show when docked in a fisheye-like dock\nor maybe dockIcon would be better?</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"maxable":{"!doc":"<p>Horrible param name for &quot;Can you maximize this floating pane?&quot;</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"renderStyles":{"!doc":"<p>trigger/load styles in the content</p>\n"},"resizable":{"!doc":"<p>Allow resizing of pane true if true</p>\n"},"resizeAxis":{"!doc":"<p>One of: x | xy | y to limit pane&#39;s sizing direction</p>\n"},"scriptHasHooks":{"!doc":"<p>replace keyword &#39;<em>container</em>&#39; in scripts with &#39;dijit.byId(this.id)&#39;\nNOTE this name might change in the near future</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>Title to use in the header</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"bringToTop":{"!type":"fn()","!doc":"<p>bring this FloatingPane above all other panes</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"close":{"!type":"fn()","!doc":"<p>Close and destroy this widget</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroy this FloatingPane completely</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn(callback?: ?)","!doc":"<p>Close, but do not destroy this FloatingPane</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP GET request to the server.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"maximize":{"!type":"fn()","!doc":"<p>Make this FloatingPane full-screen (viewport)</p>\n"},"minimize":{"!type":"fn()","!doc":"<p>Hide and dock the FloatingPane</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(dim: ?)","!doc":"<p>Size the FloatingPane and place accordingly</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"setTitle":{"!type":"fn(title: ?)","!doc":"<p>Update the Title bar with a new string</p>\n"},"show":{"!type":"fn(callback?: ?)","!doc":"<p>Show the FloatingPane</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/layout/ContentPane":{"!type":"fn()","prototype":{"!proto":"dijit/layout/ContentPane.prototype","!doc":"<p>An extended version of dijit.layout.ContentPane.\nSupports infile scripts and external ones declared by <code>&lt;script src=&#39;&#39;...&gt;</code>\nrelative path adjustments (content fetched from a different folder)\n<code>&lt;style&gt;</code> and <code>&lt;link rel=&#39;stylesheet&#39; href=&#39;..&#39;&gt;</code> tags,\ncss paths inside cssText is adjusted (if you set adjustPaths = true)\n\n</p>\n<p>NOTE that dojo.require in script in the fetched file isn&#39;t recommended\nMany widgets need to be required at page load to work properly</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"adjustPaths":{"!doc":"<p>Adjust relative paths in html string content to point to this page.\nOnly useful if you grab content from a another folder then the current one</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"cleanContent":{"!doc":"<p>Cleans content to make it less likely to generate DOM/JS errors.\nUseful if you send ContentPane a complete page, instead of a html fragment\nscans for:\n\n</p>\n<ul>\n<li>title Node, remove</li>\n<li>DOCTYPE tag, remove</li>\n</ul>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"executeScripts":{"!doc":"<p>Execute (eval) scripts that is found in the content</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"renderStyles":{"!doc":"<p>trigger/load styles in the content</p>\n"},"scriptHasHooks":{"!doc":"<p>replace keyword &#39;<em>container</em>&#39; in scripts with &#39;dijit.byId(this.id)&#39;\nNOTE this name might change in the near future</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP GET request to the server.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/html/_base":{"evalInGlobal":{"!type":"fn(code: ?, appendNode: ?)"},"set":{"!type":"fn(node: ?, cont: ?, params?: ?)","!doc":"<p>inserts (replaces) the given content into the given node</p>\n"}},"dojox/html/_base._ContentSetter":{"!type":"fn()","prototype":{"adjustPaths":{"!doc":"<p>Adjust relative paths in html string content to point to this page\nOnly useful if you grab content from a another folder than the current one</p>\n"},"cleanContent":{"!doc":"<p>Should the content be treated as a full html document,\nand the real content stripped of <html>, <body> wrapper before injection</p>\n"},"content":{"!doc":"<p>The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes</p>\n"},"executeScripts":{},"extractContent":{"!doc":"<p>Should the content be treated as a full html document,\nand the real content stripped of <code>&lt;html&gt; &lt;body&gt;</code> wrapper before injection</p>\n"},"id":{"!doc":"<p>Usually only used internally, and auto-generated with each instance</p>\n"},"node":{"!doc":"<p>An node which will be the parent element that we set content into</p>\n"},"parseContent":{"!doc":"<p>Should the node by passed to the parser after the new content is set</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"referencePath":{},"renderStyles":{},"scriptHasHooks":{},"scriptHookReplacement":{},"startup":{"!doc":"<p>Start the child widgets after parsing them.   Only obeyed if parseContent is true.</p>\n"},"constructor":{"!type":"fn(params: ?, node: ?)","!doc":"<p>Provides a configurable, extensible object to wrap the setting on content on a node\ncall the set() method to actually set the content..</p>\n"},"empty":{"!type":"fn()"},"set":{"!type":"fn(cont?: ?, params?: ?) -> ?","!doc":"<p>front-end to the set-content sequence</p>\n"},"setContent":{"!type":"fn()","!doc":"<p>sets the content on the node</p>\n"},"tearDown":{"!type":"fn()"}}},"dojox/layout/ResizeHandle":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A draggable handle used to resize an attached node.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"activeResize":{"!doc":"<p>if true, node will size realtime with mouse movement,\nif false, node will create virtual node, and only resize target on mouseUp</p>\n"},"activeResizeClass":{"!doc":"<p>css class applied to virtual resize node.</p>\n"},"animateDuration":{"!doc":"<p>time in MS to run sizing animation. if animateMethod=&quot;chain&quot;, total animation\nplaytime is 2*animateDuration</p>\n"},"animateMethod":{"!doc":"<p>one of &quot;chain&quot; or &quot;combine&quot; ... visual effect only. combine will &quot;scale&quot;\nnode to size, &quot;chain&quot; will alter width, then height</p>\n"},"animateSizing":{"!doc":"<p>only applicable if activeResize = false. onMouseup, animate the node to the\nnew size</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constrainMax":{"!doc":"<p>Toggle if this widget cares about the maxHeight and maxWidth\nparameters.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"endTopic":{"!doc":"<p>The name of the topic this resizehandle publishes when resize is complete</p>\n"},"fixedAspect":{"!doc":"<p>Toggle to enable this widget to maintain the aspect\nratio of the attached node.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Toggle to enable/disable this widget from firing onResize\nevents at every step of a resize. If <code>activeResize</code> is true,\nand this is false, onResize only fires <em>after</em> the drop\noperation. Animated resizing is not affected by this setting.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maxHeight":{"!doc":"<p>Largest height size in px the resize node can become.</p>\n"},"maxWidth":{"!doc":"<p>Largest width size in px the resize node can become.</p>\n"},"minHeight":{"!doc":"<p>smallest height in px resized node can be</p>\n"},"minWidth":{"!doc":"<p>smallest width in px resize node can be</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"resizeAxis":{"!doc":"<p>one of: x|y|xy limit resizing to a single axis, default to xy ...</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startTopic":{"!doc":"<p>The name of the topic this resizehandle publishes when resize is starting</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"targetContainer":{"!doc":"<p>over-ride targetId and attch this handle directly to a reference of a DomNode</p>\n"},"targetId":{"!doc":"<p>id of the Widget OR DomNode that I will size</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>setup our one major listener upon creation</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/layout/Dock":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget that attaches to a node and keeps track of incoming / outgoing FloatingPanes\nand handles layout</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_docked":{"!doc":"<p>array of panes currently in our dock</p>\n"},"_inPositioning":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoPosition":{},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addNode":{"!type":"fn(refNode: ?) -> +instance","!doc":"<p>Insert a dockNode reference into the dock</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/toFrac":{"approx":{"!type":"fn(r: ?) -> number","!doc":"<p>Return a less exact approximation of r such that approx(r * (1 +- eps)) == approx(r)</p>\n"},"draw":{"!type":"fn(chart: ?, functionToGraph: ?, params: ?) -> ?","!doc":"<p>graph a chart with the given function.</p>\n"},"FuncGen":{"!type":"fn()"},"generatePoints":{"!type":"fn(funcToGraph: ?, x: ?, y: ?, width: ?, minX: ?, maxX: ?, minY: ?, maxY: ?) -> ?","!doc":"<p>create the points with information about the graph.</p>\n"},"Grapher":{"!type":"fn()"},"pow":{"!type":"fn(base: ?, exponent: ?) -> ?"},"toFrac":{"!type":"fn(number: ?) -> ?"}},"dojox/calc/toFrac._Executor":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A graphing, scientific calculator\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteFunction":{"!type":"fn(name: ?)"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"eval":{"!type":"fn(text: ?) -> ?","!doc":"<p>create an anonymous function to run the code the parser generates from the user input.</p>\n"},"Function":{"!type":"fn(name: ?, args: ?, body: ?) -> ?","!doc":"<p>create an anonymous function to run the code the parser generates from the user input.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"normalizedFunction":{"!type":"fn(name: ?, args: ?, body: ?) -> ?"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/toFrac.FuncGen":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dialog layout for making functions</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"functions":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readStore":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"writeStore":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>clear the name, arguments, and body</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deleteFunction":{"!type":"fn(name: ?)"},"deleteThing":{"!type":"fn(item: ?)","!doc":"<p>delete an item in the writestore</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"readyStatus":{"!type":"fn()","!doc":"<p>set the status in the template to ready</p>\n"},"reset":{"!type":"fn()","!doc":"<p>set the arguments and body to match a function selected if it exists in the function list</p>\n"},"saveFunction":{"!type":"fn(name: ?, args: ?, body: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>make sure the parent has a close button if it needs to be able to close\nlink the write store too</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calc/toFrac.Grapher":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dialog layout for making graphs</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"chartIndex":{},"checkboxIndex":{},"class":{},"colorBoxFieldsetIndex":{},"colorIndex":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDownIndex":{},"evaluatedExpression":{},"expressionIndex":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"funcNumberIndex":{},"functionMode":{},"functionRef":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"statusIndex":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipIndex":{},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addXYAxes":{"!type":"fn(chart: ?) -> ?","!doc":"<p>add or re-add the default x/y axes to the Chart provided</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"changedColor":{"!type":"fn()","!doc":"<p>Make the color of the chart the new color.\nThe context is changed to the colorPalette, and a reference to chart was added to it a an attribute.</p>\n"},"checkDirty":{"!type":"fn()","!doc":"<p>adjust all charts in this.array according to any changes in window options</p>\n"},"checkDirty1":{"!type":"fn()","!doc":"<p>to stay in sync with onChange, checkDirty is called with a timeout</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createFunction":{"!type":"fn()","!doc":"<p>create a new row in the table with all of the dojo objects.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>deselect all checkboxes inside the function table</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawOne":{"!type":"fn(i: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"erase":{"!type":"fn(i: ?)","!doc":"<p>erase the chart inside this.array with the index i</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeDirty":{"!type":"fn()","!doc":"<p>if something in the window options is changed, this is called</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>add Event handlers, some additional attributes, etc</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"selectAll":{"!type":"fn()","!doc":"<p>select all checkboxes inside the function table</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setStatus":{"!type":"fn(i: ?, status: ?)","!doc":"<p>set the status of the row i to be status</p>\n"},"startup":{"!type":"fn()","!doc":"<p>make sure the parent has a close button if it needs to be able to close</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/Calendar":{"!type":"fn()","prototype":{"!proto":"dojox/calendar/CalendarBase.prototype","_animRangeOutDir":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_calendar":{},"_currentViewIndex":{},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_timeInterval":{},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"animateRange":{"!doc":"<p>Indicates that the previous/next range method will be animated.</p>\n"},"animationRangeDuration":{"!doc":"<p>The duration of the next/previous range animation.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"columnView":{"!doc":"<p>The column view is displaying one day to seven days time intervals.</p>\n"},"columnViewProps":{"!doc":"<p>Map of property/value passed to the constructor of the column view.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"createItemFunc":{"!doc":"<p>A user supplied function that creates a new event.\nThis function is used when createOnGridClick is set to true and the user is clicking and dragging on the grid.\nThis view takes two parameters:\n\n</p>\n<ul>\n<li>view: the current view,</li>\n<li>d: the date at the clicked location.</li>\n</ul>\n"},"createOnGridClick":{"!doc":"<p>Indicates whether the user can create new event by clicking and dragging the grid.\nA createItem function must be defined on the view or the calendar object.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"currentView":{"!doc":"<p>The current view displayed by the Calendar object.\nThe currentViewChange event can be used to react on a view change.</p>\n"},"date":{"!doc":"<p>The reference date used to determine along with the <code>dateInterval</code> \nand <code>dateIntervalSteps</code> properties the time interval to display.</p>\n"},"dateInterval":{"!doc":"<p>The date interval used to compute along with the <code>date</code> and \n<code>dateIntervalSteps</code> the time interval to display.\nValid values are &quot;day&quot;, &quot;week&quot; (default value) and &quot;month&quot;.</p>\n"},"dateIntervalSteps":{"!doc":"<p>The number of date intervals used to compute along with the <code>date</code> and \n<code>dateInterval</code> the time interval to display.\nDefault value is 1.     </p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines. Uses Gregorian Calendar routines at dojo.date by default.</p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>A flag that indicates whether or not the user can edit\nitems in the data provider.\nIf <code>true</code>, the item renderers in the control are editable.\nThe user can click on an item renderer, or use the keyboard or touch devices, to move or resize the associated event.</p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endDate":{"!doc":"<p>The end date of the displayed time interval (included).     </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"firstDayOfWeek":{"!doc":"<p>(Optional) The first day of week override. By default the first day of week is determined \nfor the current locale (extracted from the CLDR).\nSpecial value -1 (default value), means use locale dependent value.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatItemTimeFunc":{"!doc":"<p>Optional function to format the time of day of the item renderers.\nThe function takes the date and render data object as arguments and returns a String.</p>\n"},"hoveredItem":{"!doc":"<p>Current render item which is under the mouse cursor.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"matrixView":{"!doc":"<p>The column view is displaying time intervals that lasts more than seven days.</p>\n"},"matrixViewProps":{"!doc":"<p>Map of property/value passed to the constructor of the matrix view.</p>\n"},"moveEnabled":{"!doc":"<p>A flag that indicates whether the user can move items displayed.\nIf <code>true</code>, the user can move the items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"resizeEnabled":{"!doc":"<p>A flag that indicates whether the items can be resized.\nIf <code>true</code>, the control supports resizing of items.</p>\n"},"searchContainerNode":{},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startDate":{"!doc":"<p>The start date of the displayed time interval.</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"viewChangeDuration":{},"viewContainer":{"!doc":"<p>The DOM node that will contains the views.</p>\n"},"views":{"!doc":"<p>The views displayed by the widget.\nTo add/remove only one view, prefer, respectively, the addView() or removeView() methods.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"addView":{"!type":"fn(view: ?, index: ?)","!doc":"<p>Add a view to the calendar&#39;s view list.</p>\n"},"buildRendering":{"!type":"fn()"},"columnViewColumnHeaderClick":{"!type":"fn(e: ?)","!doc":"<p>Function called when the cell of a column header of the column view is clicked.\nShow the time range defined by the clicked date.</p>\n"},"computeTimeInterval":{"!type":"fn() -> []","!doc":"<p>Computes the displayed time interval according to the date, dateInterval and \ndateIntervalSteps if date is not null or startDate and endDate properties otherwise.</p>\n"},"configureButtons":{"!type":"fn()","!doc":"<p>Set the localized labels of the buttons and the event handlers.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(args: ?)"},"dayButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggerred when the day button is clicked.\nBy default, sets the dateInterval property to &quot;day&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"floorDate":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToWeek":{"!type":"fn(d: ?) -> ?","!doc":"<p>Floors the specified date to the beginning of week.</p>\n"},"fourDaysButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggerred when the 4 days button is clicked.\nBy default, sets the dateInterval property to &quot;day&quot; and \nthe &quot;dateIntervalSteps&quot; to 4.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> +null"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToday":{"!type":"fn()","!doc":"<p>Changes the displayed time interval to show the current day.\nSets the date property to the current day, the dateInterval property to &quot;day&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"},"installDefaultViewsActions":{"!type":"fn(views: ?)","!doc":"<p>Installs the default actions on newly created default views.\nBy default this action is registering:\n- the matrixViewRowHeaderClick method on the rowHeaderClick event of the matrix view.\n- the columnViewColumnHeaderClick method on the columnHeaderClick event of the column view.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemEditable":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be edited.\nBy default it is using the editable property value.</p>\n"},"isItemHovered":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is hovered or not.</p>\n"},"isItemMoveEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be moved.\nBy default it is using the moveEnabled property value.</p>\n"},"isItemResizeEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be resized.\nBy default it is using the resizedEnabled property value.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns wether an item is selected or not.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isStartOfDay":{"!type":"fn(d: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(date: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"matrixViewRowHeaderClick":{"!type":"fn(e: ?)","!doc":"<p>Function called when the cell of a row header of the matrix view is clicked.\nThe implementation is doing the foolowing actions:\n- If another row is already expanded, collapse it and then expand the clicked row.\n- If the clicked row is already expadned, collapse it.\n- If no row is expanded, expand the click row.</p>\n"},"monthButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggered when the month button is clicked.\nBy default, sets the dateInterval property to &quot;month&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"},"newDate":{"!type":"fn(obj: ?) -> +Date","!doc":"<p>Creates a new Date object.</p>\n"},"nextRange":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"previousRange":{"!type":"fn()"},"refreshRendering":{"!type":"fn()","!doc":"<p>Refreshes all the visual rendering of the calendar. </p>\n"},"removeView":{"!type":"fn(view: ?)","!doc":"<p>Removes a view from the calendar&#39;s view list.</p>\n"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"resize":{"!type":"fn(changeSize: ?)"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?) -> bool","!doc":"<p>Applies selection triggered by an user interaction</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)","!doc":"<p>Change the selection state of an item.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"todayButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggered when the today button is clicked.\nBy default, calls the goToday() method.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(obj: ?, stateOnly: ?)"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"},"weekButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggered when the week button is clicked.\nBy default, sets the dateInterval property to &quot;week&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"}}},"dojox/calendar/CalendarBase":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>This class defines a generic calendar widget that manages several views to display event in time.</p>\n","_animRangeOutDir":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_calendar":{},"_currentViewIndex":{},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_timeInterval":{},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"animateRange":{"!doc":"<p>Indicates that the previous/next range method will be animated.</p>\n"},"animationRangeDuration":{"!doc":"<p>The duration of the next/previous range animation.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"columnView":{"!doc":"<p>The column view is displaying one day to seven days time intervals.</p>\n"},"columnViewProps":{"!doc":"<p>Map of property/value passed to the constructor of the column view.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"createItemFunc":{"!doc":"<p>A user supplied function that creates a new event.\nThis function is used when createOnGridClick is set to true and the user is clicking and dragging on the grid.\nThis view takes two parameters:\n\n</p>\n<ul>\n<li>view: the current view,</li>\n<li>d: the date at the clicked location.</li>\n</ul>\n"},"createOnGridClick":{"!doc":"<p>Indicates whether the user can create new event by clicking and dragging the grid.\nA createItem function must be defined on the view or the calendar object.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"currentView":{"!doc":"<p>The current view displayed by the Calendar object.\nThe currentViewChange event can be used to react on a view change.</p>\n"},"date":{"!doc":"<p>The reference date used to determine along with the <code>dateInterval</code> \nand <code>dateIntervalSteps</code> properties the time interval to display.</p>\n"},"dateInterval":{"!doc":"<p>The date interval used to compute along with the <code>date</code> and \n<code>dateIntervalSteps</code> the time interval to display.\nValid values are &quot;day&quot;, &quot;week&quot; (default value) and &quot;month&quot;.</p>\n"},"dateIntervalSteps":{"!doc":"<p>The number of date intervals used to compute along with the <code>date</code> and \n<code>dateInterval</code> the time interval to display.\nDefault value is 1.     </p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines. Uses Gregorian Calendar routines at dojo.date by default.</p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>A flag that indicates whether or not the user can edit\nitems in the data provider.\nIf <code>true</code>, the item renderers in the control are editable.\nThe user can click on an item renderer, or use the keyboard or touch devices, to move or resize the associated event.</p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endDate":{"!doc":"<p>The end date of the displayed time interval (included).     </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"firstDayOfWeek":{"!doc":"<p>(Optional) The first day of week override. By default the first day of week is determined \nfor the current locale (extracted from the CLDR).\nSpecial value -1 (default value), means use locale dependent value.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatItemTimeFunc":{"!doc":"<p>Optional function to format the time of day of the item renderers.\nThe function takes the date and render data object as arguments and returns a String.</p>\n"},"hoveredItem":{"!doc":"<p>Current render item which is under the mouse cursor.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"matrixView":{"!doc":"<p>The column view is displaying time intervals that lasts more than seven days.</p>\n"},"matrixViewProps":{"!doc":"<p>Map of property/value passed to the constructor of the matrix view.</p>\n"},"moveEnabled":{"!doc":"<p>A flag that indicates whether the user can move items displayed.\nIf <code>true</code>, the user can move the items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"resizeEnabled":{"!doc":"<p>A flag that indicates whether the items can be resized.\nIf <code>true</code>, the control supports resizing of items.</p>\n"},"searchContainerNode":{},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startDate":{"!doc":"<p>The start date of the displayed time interval.</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"viewChangeDuration":{},"viewContainer":{"!doc":"<p>The DOM node that will contains the views.</p>\n"},"views":{"!doc":"<p>The views displayed by the widget.\nTo add/remove only one view, prefer, respectively, the addView() or removeView() methods.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"addView":{"!type":"fn(view: ?, index: ?)","!doc":"<p>Add a view to the calendar&#39;s view list.</p>\n"},"buildRendering":{"!type":"fn()"},"columnViewColumnHeaderClick":{"!type":"fn(e: ?)","!doc":"<p>Function called when the cell of a column header of the column view is clicked.\nShow the time range defined by the clicked date.</p>\n"},"computeTimeInterval":{"!type":"fn() -> []","!doc":"<p>Computes the displayed time interval according to the date, dateInterval and \ndateIntervalSteps if date is not null or startDate and endDate properties otherwise.</p>\n"},"configureButtons":{"!type":"fn()","!doc":"<p>Set the localized labels of the buttons and the event handlers.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(args: ?)"},"dayButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggerred when the day button is clicked.\nBy default, sets the dateInterval property to &quot;day&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"floorDate":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToWeek":{"!type":"fn(d: ?) -> ?","!doc":"<p>Floors the specified date to the beginning of week.</p>\n"},"fourDaysButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggerred when the 4 days button is clicked.\nBy default, sets the dateInterval property to &quot;day&quot; and \nthe &quot;dateIntervalSteps&quot; to 4.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> +null"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToday":{"!type":"fn()","!doc":"<p>Changes the displayed time interval to show the current day.\nSets the date property to the current day, the dateInterval property to &quot;day&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemEditable":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be edited.\nBy default it is using the editable property value.</p>\n"},"isItemHovered":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is hovered or not.</p>\n"},"isItemMoveEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be moved.\nBy default it is using the moveEnabled property value.</p>\n"},"isItemResizeEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be resized.\nBy default it is using the resizedEnabled property value.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns wether an item is selected or not.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isStartOfDay":{"!type":"fn(d: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(date: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"matrixViewRowHeaderClick":{"!type":"fn(e: ?)","!doc":"<p>Function called when the cell of a row header of the matrix view is clicked.\nThe implementation is doing the foolowing actions:\n- If another row is already expanded, collapse it and then expand the clicked row.\n- If the clicked row is already expadned, collapse it.\n- If no row is expanded, expand the click row.</p>\n"},"monthButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggered when the month button is clicked.\nBy default, sets the dateInterval property to &quot;month&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"},"newDate":{"!type":"fn(obj: ?) -> +Date","!doc":"<p>Creates a new Date object.</p>\n"},"nextRange":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"previousRange":{"!type":"fn()"},"refreshRendering":{"!type":"fn()","!doc":"<p>Refreshes all the visual rendering of the calendar. </p>\n"},"removeView":{"!type":"fn(view: ?)","!doc":"<p>Removes a view from the calendar&#39;s view list.</p>\n"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"resize":{"!type":"fn(changeSize: ?)"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?) -> bool","!doc":"<p>Applies selection triggered by an user interaction</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)","!doc":"<p>Change the selection state of an item.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"todayButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggered when the today button is clicked.\nBy default, calls the goToday() method.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(obj: ?, stateOnly: ?)"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"},"weekButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggered when the week button is clicked.\nBy default, sets the dateInterval property to &quot;week&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"}}},"dojox/calendar/StoreMixin":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>This mixin contains the store management.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/_Invalidating":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>Base class for classes (usually widgets) that watch invalidated properties and delay the rendering\nafter these properties modifications to the next execution frame.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"postscript":{"!type":"fn(mixin: ?)"},"refreshRendering":{"!type":"fn()","!doc":"<p>Actually refresh the rendering. Implementation should implement that method.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/Selection":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>Base class for widgets that manage a list of selected data items.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"constructor":{"!type":"fn()"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"getIdentity":{"!type":"fn(item: ?)","!doc":"<p>This function must be implemented to return the id of a item.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns wether an item is selected or not.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?) -> bool","!doc":"<p>Applies selection triggered by an user interaction</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)","!doc":"<p>Change the selection state of an item.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/time":{"floor":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?, dateClassObj?: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(d: ?, reuse: ?, dateClassObj?: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToMonth":{"!type":"fn(d: ?, reuse: ?, dateClassObj?: ?) -> +Date","!doc":"<p>Floors the specified date to the start of the date&#39;s month.</p>\n"},"floorToWeek":{"!type":"fn(d: ?, dateClassObj?: ?, dateModule?: ?, firstDayOfWeek?: ?, locale?: ?) -> ?","!doc":"<p>Floors the specified date to the beginning of week.</p>\n"},"isStartOfDay":{"!type":"fn(d: ?, dateClassObj?: ?, dateModule?: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(d: ?, dateClassObj?: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"newDate":{"!type":"fn(obj: ?, dateClassObj?: ?) -> +instance","!doc":"<p>Creates a new Date object.</p>\n"}},"dojox/calendar/ColumnView":{"!type":"fn()","prototype":{"!proto":"dojox/calendar/SimpleColumnView.prototype","!doc":"<p>This class defines a simple column view that also uses a secondary \nsheet to display long or all day events. \nBy default an dojox.calendar.ColumnViewSecondarySheet instance is created.\nSet the secondarySheetClass property to define the class to instantiate,\nfor example to mix the default class with Mouse, Keyboard or Touch plugins. </p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_calendar":{},"_columnHeaderHandlers":{},"_cssDays":{},"_DAY_IN_MILLISECONDS":{},"_gridMouseDown":{},"_isEditing":{"!doc":"<p>Whether an item is being edited or not.</p>\n"},"_layoutStep":{"!doc":"<p>The number of units displayed by a visual layout unit (i.e. a column or a row)</p>\n"},"_layoutUnit":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_rendererHandles":{},"_scrollPos":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tempIdCount":{},"_tempItemsMap":{},"_viewHandles":{},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"allowResizeLessThan24H":{},"allowStartEndSwap":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoScroll":{"!doc":"<p>Indicates whether the view can be scrolled automatically. \nAuto scrolling is used when moving focus to a non visible renderer using keyboard \nand while editing an item. </p>\n"},"baseClass":{},"class":{},"columnCount":{"!doc":"<p>The number of column to display (from the startDate).</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"createOnGridClick":{"!doc":"<p>Indicates whether the user can create new event by clicking and dragging the grid.\nA createItem function must be defined on the view or the calendar object.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines. Uses Gregorian Calendar routines at dojo.date by default.</p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"doubleTapDelay":{"!doc":"<p>The maximum delay between two taps needed to trigger an &quot;itemDoubleClick&quot; event, in touch context.      </p>\n"},"editable":{"!doc":"<p>A flag that indicates whether or not the user can edit\nitems in the data provider.\nIf <code>true</code>, the item renderers in the control are editable.\nThe user can click on an item renderer, or use the keyboard or touch devices, to move or resize the associated event.</p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatItemTimeFunc":{"!doc":"<p>Optional function to format the time of day of the item renderers.\nThe function takes the date and render data object as arguments and returns a String.</p>\n"},"headerPadding":{"!doc":"<p>Padding between the header (composed of the secondary sheet and the column header) \nand the primary sheet.</p>\n"},"horizontalGap":{"!doc":"<p>The number of pixels between two item renderers that are overlapping each other if the percentOverlap property is 0.</p>\n"},"hourSize":{"!doc":"<p>The desired size in pixels of an hour on the screen.\nNote that the effective size may be different as the time slot size must be an integer.</p>\n"},"hoveredItem":{"!doc":"<p>The currently hovered data item.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"itemToRenderer":{"!doc":"<p>The associated array item to renderer list.</p>\n"},"itemToRendererKindFunc":{"!doc":"<p>An optional function to associate a kind of renderer (&quot;horizontal&quot;, &quot;label&quot; or null) with the specified item.\nBy default, if an item is lasting more that 24 hours an horizontal item is used, otherwise a label is used.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"layoutPriorityFunction":{"!doc":"<p>An optional comparison function use to determine the order the item will be laid out\nThe function is used to sort an array and must, as any sorting function, take two items \nas argument and must return an integer whose sign define order between arguments.\nBy default, a comparison by start time then end time is used.</p>\n"},"liveLayout":{},"maxHours":{"!doc":"<p>The maximum hour to be displayed. It must be in the [1,24] interval and must be greater than the minHours.  </p>\n"},"minDurationSteps":{},"minDurationUnit":{},"minHours":{"!doc":"<p>The minimum hour to be displayed. It must be in the [0,23] interval and must be lower than the maxHours.</p>\n"},"moveEnabled":{"!doc":"<p>A flag that indicates whether the user can move items displayed.\nIf <code>true</code>, the user can move the items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"percentOverlap":{"!doc":"<p>The percentage of the renderer width used to superimpose one item renderer on another \nwhen two events are overlapping.</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"renderData":{"!doc":"<p>The render data is the object that contains all the properties needed to render the component.</p>\n"},"rendererList":{"!doc":"<p>The list of used renderers</p>\n"},"rendererPool":{"!doc":"<p>The stack of recycled renderers available.</p>\n"},"resizeCursor":{"!doc":"<p>CSS value to apply to the cursor while resizing an item renderer. </p>\n"},"resizeEnabled":{"!doc":"<p>A flag that indicates whether the items can be resized.\nIf <code>true</code>, the control supports resizing of items.</p>\n"},"rowHeaderFirstLabelOffset":{"!doc":"<p>Offset of the first row label from the top of the first row header cell in pixels.</p>\n"},"rowHeaderGridSlotDuration":{"!doc":"<p>Duration of the time slot in minutes in the row header. Must be a divisor of 60 and a multiple/divisor of timeSlotDuration.</p>\n"},"rowHeaderLabelOffset":{"!doc":"<p>Offset of the row label from the top of the row header cell in pixels.</p>\n"},"rowHeaderLabelSlotDuration":{"!doc":"<p>Duration of the time slot in minutes in the row header labels. Must be a divisor of 60 and a multiple/divisor of timeSlotDuration.</p>\n"},"scrollable":{"!doc":"<p>Indicates whether the view can be scrolled or not.</p>\n"},"scrollBarRTLPosition":{"!doc":"<p>Position of the scroll bar in right-to-left display.\nValid values are &quot;left&quot; and &quot;right&quot;, default value is &quot;left&quot;.</p>\n"},"scrollMethod":{"!doc":"<p>Method used to scroll the view, for example the scroll of column view.\nValid value are:\n\n</p>\n<ul>\n<li>&quot;auto&quot;: let the view decide (default),</li>\n<li>&quot;css&quot;: use css 3d transform,</li>\n<li>&quot;dom&quot;: use the scrollTop property.</li>\n</ul>\n"},"searchContainerNode":{},"secondarySheetProps":{"!doc":"<p>Secondary sheet constructor parameters.</p>\n"},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"snapSteps":{},"snapUnit":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startDate":{"!doc":"<p>The start date of the time interval displayed.\nIf not set at initialization time, will be set to current day.</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"startTimeOfDay":{"!doc":"<p>First time (hour/minute) of day displayed, if reachable. \nAn object containing &quot;hours&quot; and &quot;minutes&quot; properties.</p>\n"},"stayInView":{},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"styleGridCellFunc":{"!doc":"<p>Custom function to customize the appearance of a grid cell by installing custom CSS class on the node.\nThe signature of the function must be the same then the styleGridCell one.\nBy default the defaultStyleGridCell function is used.</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"timeSlotDuration":{"!doc":"<p>Duration of the time slot in minutes. Must be a divisor of 60.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"verticalRenderer":{"!doc":"<p>The class use to create vertical renderers.</p>\n"},"viewKind":{"!doc":"<p>Type of the view. Used by the calendar widget to determine how to configure the view.\nThis view kind is &quot;columns&quot;.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"applyRendererZIndex":{"!type":"fn(item: ?, renderer: ?, hovered: ?, selected: ?, edited: ?, focused: ?)","!doc":"<p>Applies the z-index to the renderer based on the state of the item.\nThis methods is setting a z-index of 20 is the item is selected or edited \nand the current lane value computed by the overlap layout (i.e. the renderers \nare stacked according to their lane).</p>\n"},"buildRendering":{"!type":"fn()"},"computeOverlapping":{"!type":"fn(layoutItems: ?, func: ?) -> ?","!doc":"<p>Computes the overlap layout of a list of items. A lane and extent properties are added to each layout item.</p>\n"},"computeProjectionOnDate":{"!type":"fn(renderData: ?, refDate: ?, date: ?, max: ?) -> number","!doc":"<p>Computes the time to pixel projection in a day.</p>\n"},"computeRangeOverlap":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> [+Date]","!doc":"<p>Computes the overlap time range of the time ranges.\nReturns a vector of Date with at index 0 the start time and at index 1 the end time.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"createItemFunc":{"!type":"fn(view: ?, d: ?, e: ?)","!doc":"<p>A user supplied function that creates a new event.</p>\n"},"defaultStyleGridCell":{"!type":"fn(node: ?, date: ?, hours: ?, minutes: ?, renderData: ?) -> ?","!doc":"<p>Styles the CSS classes to the node that displays a cell.\nBy default this method is setting:\n- &quot;dojoxCalendarToday&quot; class name if the date displayed is the current date,\n- &quot;dojoxCalendarWeekend&quot; if the date represents a weekend,\n- the CSS class corresponding of the displayed day of week (&quot;Sun&quot;, &quot;Mon&quot; and so on),\n- the CSS classes corresponfing to the time of day (e.g. &quot;H14&quot; and &quot;M30&quot; for for 2:30pm).   </p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"ensureMinimalDuration":{"!type":"fn(renderData: ?, item: ?, unit: ?, steps: ?, editKind: ?)","!doc":"<p>During the resize editing gesture, ensures that the item has the specified minimal duration.</p>\n"},"ensureVisibility":{"!type":"fn(start: ?, end: ?, visibilityTarget: ?, margin: ?, duration: ?)","!doc":"<p>Scrolls the view if the [start, end] time range is not visible or only partially visible.</p>\n"},"floorDate":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToMonth":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of the date&#39;s month.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getCSSPrefix":{"!type":"fn() -> string","!doc":"<p>Utility method that return the specific CSS prefix\nfor non standard CSS properties. Ex: -moz-border-radius.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRenderers":{"!type":"fn(item: ?) -> [+Object]","!doc":"<p>Returns the renderers that are currently used to displayed the speficied item.\nReturns an array of objects that contains two properties:\n- container: The DOM node that contains the renderer.\n- renderer: The dojox.calendar._RendererMixin instance.\nDo not keep references on the renderers are they are recycled and reused for other items.</p>\n"},"getSecondarySheet":{"!type":"fn() -> +dojox","!doc":"<p>Returns the secondary sheet</p>\n"},"getTime":{"!type":"fn(e: ?, x: ?, y: ?, touchIndex: ?) -> +Date","!doc":"<p>Returns the time displayed at the specified point by this component.</p>\n"},"getTimeOfDay":{"!type":"fn(pos: ?, rd: ?) -> ?","!doc":"<p>Return the time of day associated to the specified position.</p>\n"},"getWeekNumberLabel":{"!type":"fn(date: ?) -> ?","!doc":"<p>Returns the week number string from dojo.date.locale.format() method as\ndojox.date.XXXX calendar are not supporting the &quot;w&quot; pattern.</p>\n"},"invalidateRendering":{"!type":"fn()"},"isAscendantHasClass":{"!type":"fn(node: ?, ancestor: ?, className: ?) -> bool","!doc":"<p>Determines if a node has an ascendant node that has the css class specified.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemBeingEdited":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether an item is being edited or not.</p>\n"},"isItemEditable":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be edited or not.\nBy default it is using the editable property value.</p>\n"},"isItemFocused":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is focused or not.</p>\n"},"isItemHovered":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is hovered or not.</p>\n"},"isItemMoveEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be moved.\nBy default it is using the moveEnabled property value.</p>\n"},"isItemResizeEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be resized.\nBy default it is using the resizedEnabled property value.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> ?"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isOverlapping":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> bool","!doc":"<p>Computes if the first time range defined by the start1 and end1 parameters \nis overlapping the second time range defined by the start2 and end2 parameters.</p>\n"},"isSameDay":{"!type":"fn(date1: ?, date2: ?) -> bool","!doc":"<p>Tests if the specified dates are in the same day.</p>\n"},"isStartOfDay":{"!type":"fn(d: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(date: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"isWeekEnd":{"!type":"fn(date: ?) -> ?","!doc":"<p>Determines whether the specified date is a week-end.\nThis method is using dojo.date.locale.isWeekend() method as\ndojox.date.XXXX calendars are not supporting this method.</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"newDate":{"!type":"fn(obj: ?) -> +Date","!doc":"<p>Creates a new Date object.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"postscript":{"!type":"fn()"},"refreshRendering":{"!type":"fn()"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"resizeSecondarySheet":{"!type":"fn(height: ?)","!doc":"<p>Resizes the secondary sheet header and relayout the other sub components according this new height.\nWarning: this method is only available for the default template and default CSS.</p>\n"},"scrollView":{"!type":"fn(dir: ?)","!doc":"<p>Scrolls the view to the specified direction of one time slot duration.</p>\n"},"secondarySheetClass":{"!type":"fn()","!doc":"<p>The secondary sheet class, by default dojox.calendar.ColumnViewSecondarySheet.</p>\n"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleColumnHeaderCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a column header cell.\nBy default this method is setting:\n- &quot;dojoxCalendarToday&quot; class name if the date displayed is the current date,\n- &quot;dojoxCalendarWeekend&quot; if the date represents a weekend,\n- the CSS class corresponding of the displayed day of week (&quot;Sun&quot;, &quot;Mon&quot; and so on).</p>\n"},"styleGridCell":{"!type":"fn(node: ?, date: ?, hours: ?, minutes: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a cell.\nDelegates to styleGridCellFunc if defined or defaultStyleGridCell otherwise.</p>\n"},"styleRowHeaderCell":{"!type":"fn(node: ?, h: ?, m: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a row header cell.\nBy default this method is doing nothing.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(obj: ?, stateOnly: ?)"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/html/metrics":{"_fontResizeNode":{},"getCachedFontMeasurements":{"!type":"fn(recalculate: ?) -> ?"},"getFontMeasurements":{"!type":"fn() -> ?","!doc":"<p>Returns an object that has pixel equivilents of standard font size values.</p>\n"},"getScrollbar":{"!type":"fn() -> ?"},"getTextBox":{"!type":"fn(text: ?, style: ?, className?: ?)"},"initOnFontResize":{"!type":"fn()"}},"dojox/calendar/SimpleColumnView":{"!type":"fn()","prototype":{"!proto":"dojox/calendar/ViewBase.prototype","!doc":"<p>The simple column view is displaying a day per column. Each cell of a column is a time slot.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_calendar":{},"_columnHeaderHandlers":{},"_cssDays":{},"_DAY_IN_MILLISECONDS":{},"_gridMouseDown":{},"_isEditing":{"!doc":"<p>Whether an item is being edited or not.</p>\n"},"_layoutStep":{"!doc":"<p>The number of units displayed by a visual layout unit (i.e. a column or a row)</p>\n"},"_layoutUnit":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_rendererHandles":{},"_scrollPos":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tempIdCount":{},"_tempItemsMap":{},"_viewHandles":{},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"allowResizeLessThan24H":{},"allowStartEndSwap":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoScroll":{"!doc":"<p>Indicates whether the view can be scrolled automatically. \nAuto scrolling is used when moving focus to a non visible renderer using keyboard \nand while editing an item. </p>\n"},"baseClass":{},"class":{},"columnCount":{"!doc":"<p>The number of column to display (from the startDate).</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"createOnGridClick":{"!doc":"<p>Indicates whether the user can create new event by clicking and dragging the grid.\nA createItem function must be defined on the view or the calendar object.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines. Uses Gregorian Calendar routines at dojo.date by default.</p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"doubleTapDelay":{"!doc":"<p>The maximum delay between two taps needed to trigger an &quot;itemDoubleClick&quot; event, in touch context.      </p>\n"},"editable":{"!doc":"<p>A flag that indicates whether or not the user can edit\nitems in the data provider.\nIf <code>true</code>, the item renderers in the control are editable.\nThe user can click on an item renderer, or use the keyboard or touch devices, to move or resize the associated event.</p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatItemTimeFunc":{"!doc":"<p>Optional function to format the time of day of the item renderers.\nThe function takes the date and render data object as arguments and returns a String.</p>\n"},"horizontalGap":{"!doc":"<p>The number of pixels between two item renderers that are overlapping each other if the percentOverlap property is 0.</p>\n"},"hourSize":{"!doc":"<p>The desired size in pixels of an hour on the screen.\nNote that the effective size may be different as the time slot size must be an integer.</p>\n"},"hoveredItem":{"!doc":"<p>The currently hovered data item.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"itemToRenderer":{"!doc":"<p>The associated array item to renderer list.</p>\n"},"itemToRendererKindFunc":{"!doc":"<p>An optional function to associate a kind of renderer (&quot;horizontal&quot;, &quot;label&quot; or null) with the specified item.\nBy default, if an item is lasting more that 24 hours an horizontal item is used, otherwise a label is used.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"layoutPriorityFunction":{"!doc":"<p>An optional comparison function use to determine the order the item will be laid out\nThe function is used to sort an array and must, as any sorting function, take two items \nas argument and must return an integer whose sign define order between arguments.\nBy default, a comparison by start time then end time is used.</p>\n"},"liveLayout":{},"maxHours":{"!doc":"<p>The maximum hour to be displayed. It must be in the [1,24] interval and must be greater than the minHours.  </p>\n"},"minDurationSteps":{},"minDurationUnit":{},"minHours":{"!doc":"<p>The minimum hour to be displayed. It must be in the [0,23] interval and must be lower than the maxHours.</p>\n"},"moveEnabled":{"!doc":"<p>A flag that indicates whether the user can move items displayed.\nIf <code>true</code>, the user can move the items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"percentOverlap":{"!doc":"<p>The percentage of the renderer width used to superimpose one item renderer on another \nwhen two events are overlapping.</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"renderData":{"!doc":"<p>The render data is the object that contains all the properties needed to render the component.</p>\n"},"rendererList":{"!doc":"<p>The list of used renderers</p>\n"},"rendererPool":{"!doc":"<p>The stack of recycled renderers available.</p>\n"},"resizeCursor":{"!doc":"<p>CSS value to apply to the cursor while resizing an item renderer. </p>\n"},"resizeEnabled":{"!doc":"<p>A flag that indicates whether the items can be resized.\nIf <code>true</code>, the control supports resizing of items.</p>\n"},"rowHeaderFirstLabelOffset":{"!doc":"<p>Offset of the first row label from the top of the first row header cell in pixels.</p>\n"},"rowHeaderGridSlotDuration":{"!doc":"<p>Duration of the time slot in minutes in the row header. Must be a divisor of 60 and a multiple/divisor of timeSlotDuration.</p>\n"},"rowHeaderLabelOffset":{"!doc":"<p>Offset of the row label from the top of the row header cell in pixels.</p>\n"},"rowHeaderLabelSlotDuration":{"!doc":"<p>Duration of the time slot in minutes in the row header labels. Must be a divisor of 60 and a multiple/divisor of timeSlotDuration.</p>\n"},"scrollable":{"!doc":"<p>Indicates whether the view can be scrolled or not.</p>\n"},"scrollBarRTLPosition":{"!doc":"<p>Position of the scroll bar in right-to-left display.\nValid values are &quot;left&quot; and &quot;right&quot;, default value is &quot;left&quot;.</p>\n"},"scrollMethod":{"!doc":"<p>Method used to scroll the view, for example the scroll of column view.\nValid value are:\n\n</p>\n<ul>\n<li>&quot;auto&quot;: let the view decide (default),</li>\n<li>&quot;css&quot;: use css 3d transform,</li>\n<li>&quot;dom&quot;: use the scrollTop property.</li>\n</ul>\n"},"searchContainerNode":{},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"snapSteps":{},"snapUnit":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startDate":{"!doc":"<p>The start date of the time interval displayed.\nIf not set at initialization time, will be set to current day.</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"startTimeOfDay":{"!doc":"<p>First time (hour/minute) of day displayed, if reachable. \nAn object containing &quot;hours&quot; and &quot;minutes&quot; properties.</p>\n"},"stayInView":{},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"styleGridCellFunc":{"!doc":"<p>Custom function to customize the appearance of a grid cell by installing custom CSS class on the node.\nThe signature of the function must be the same then the styleGridCell one.\nBy default the defaultStyleGridCell function is used.</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"timeSlotDuration":{"!doc":"<p>Duration of the time slot in minutes. Must be a divisor of 60.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"verticalRenderer":{"!doc":"<p>The class use to create vertical renderers.</p>\n"},"viewKind":{"!doc":"<p>Type of the view. Used by the calendar widget to determine how to configure the view.\nThis view kind is &quot;columns&quot;.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"applyRendererZIndex":{"!type":"fn(item: ?, renderer: ?, hovered: ?, selected: ?, edited: ?, focused: ?)","!doc":"<p>Applies the z-index to the renderer based on the state of the item.\nThis methods is setting a z-index of 20 is the item is selected or edited \nand the current lane value computed by the overlap layout (i.e. the renderers \nare stacked according to their lane).</p>\n"},"buildRendering":{"!type":"fn()"},"computeOverlapping":{"!type":"fn(layoutItems: ?, func: ?) -> ?","!doc":"<p>Computes the overlap layout of a list of items. A lane and extent properties are added to each layout item.</p>\n"},"computeProjectionOnDate":{"!type":"fn(renderData: ?, refDate: ?, date: ?, max: ?) -> number","!doc":"<p>Computes the time to pixel projection in a day.</p>\n"},"computeRangeOverlap":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> [+Date]","!doc":"<p>Computes the overlap time range of the time ranges.\nReturns a vector of Date with at index 0 the start time and at index 1 the end time.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"createItemFunc":{"!type":"fn(view: ?, d: ?, e: ?)","!doc":"<p>A user supplied function that creates a new event.</p>\n"},"defaultStyleGridCell":{"!type":"fn(node: ?, date: ?, hours: ?, minutes: ?, renderData: ?) -> ?","!doc":"<p>Styles the CSS classes to the node that displays a cell.\nBy default this method is setting:\n- &quot;dojoxCalendarToday&quot; class name if the date displayed is the current date,\n- &quot;dojoxCalendarWeekend&quot; if the date represents a weekend,\n- the CSS class corresponding of the displayed day of week (&quot;Sun&quot;, &quot;Mon&quot; and so on),\n- the CSS classes corresponfing to the time of day (e.g. &quot;H14&quot; and &quot;M30&quot; for for 2:30pm).   </p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"ensureMinimalDuration":{"!type":"fn(renderData: ?, item: ?, unit: ?, steps: ?, editKind: ?)","!doc":"<p>During the resize editing gesture, ensures that the item has the specified minimal duration.</p>\n"},"ensureVisibility":{"!type":"fn(start: ?, end: ?, visibilityTarget: ?, margin: ?, duration: ?)","!doc":"<p>Scrolls the view if the [start, end] time range is not visible or only partially visible.</p>\n"},"floorDate":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToMonth":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of the date&#39;s month.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getCSSPrefix":{"!type":"fn() -> string","!doc":"<p>Utility method that return the specific CSS prefix\nfor non standard CSS properties. Ex: -moz-border-radius.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRenderers":{"!type":"fn(item: ?) -> [+Object]","!doc":"<p>Returns the renderers that are currently used to displayed the speficied item.\nReturns an array of objects that contains two properties:\n- container: The DOM node that contains the renderer.\n- renderer: The dojox.calendar._RendererMixin instance.\nDo not keep references on the renderers are they are recycled and reused for other items.</p>\n"},"getTime":{"!type":"fn(e: ?, x: ?, y: ?, touchIndex: ?) -> +Date","!doc":"<p>Returns the time displayed at the specified point by this component.</p>\n"},"getTimeOfDay":{"!type":"fn(pos: ?, rd: ?) -> ?","!doc":"<p>Return the time of day associated to the specified position.</p>\n"},"getWeekNumberLabel":{"!type":"fn(date: ?) -> ?","!doc":"<p>Returns the week number string from dojo.date.locale.format() method as\ndojox.date.XXXX calendar are not supporting the &quot;w&quot; pattern.</p>\n"},"invalidateLayout":{"!type":"fn()","!doc":"<p>Triggers a re-layout of the renderers.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isAscendantHasClass":{"!type":"fn(node: ?, ancestor: ?, className: ?) -> bool","!doc":"<p>Determines if a node has an ascendant node that has the css class specified.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemBeingEdited":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether an item is being edited or not.</p>\n"},"isItemEditable":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be edited or not.\nBy default it is using the editable property value.</p>\n"},"isItemFocused":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is focused or not.</p>\n"},"isItemHovered":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is hovered or not.</p>\n"},"isItemMoveEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be moved.\nBy default it is using the moveEnabled property value.</p>\n"},"isItemResizeEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be resized.\nBy default it is using the resizedEnabled property value.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> ?"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isOverlapping":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> bool","!doc":"<p>Computes if the first time range defined by the start1 and end1 parameters \nis overlapping the second time range defined by the start2 and end2 parameters.</p>\n"},"isSameDay":{"!type":"fn(date1: ?, date2: ?) -> bool","!doc":"<p>Tests if the specified dates are in the same day.</p>\n"},"isStartOfDay":{"!type":"fn(d: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(date: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"isWeekEnd":{"!type":"fn(date: ?) -> ?","!doc":"<p>Determines whether the specified date is a week-end.\nThis method is using dojo.date.locale.isWeekend() method as\ndojox.date.XXXX calendars are not supporting this method.</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"newDate":{"!type":"fn(obj: ?) -> +Date","!doc":"<p>Creates a new Date object.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"postscript":{"!type":"fn()"},"refreshRendering":{"!type":"fn()"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"resize":{"!type":"fn(changeSize: ?)","!doc":"<p>Function to call when the view is resized. \nIf the view is in a Dijit container or in a Dojo mobile container, it will be automatically called.\nOn other use cases, this method must called when the window is resized and/or when the orientation has changed.</p>\n"},"scrollView":{"!type":"fn(dir: ?)","!doc":"<p>Scrolls the view to the specified direction of one time slot duration.</p>\n"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleColumnHeaderCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a column header cell.\nBy default this method is setting:\n- &quot;dojoxCalendarToday&quot; class name if the date displayed is the current date,\n- &quot;dojoxCalendarWeekend&quot; if the date represents a weekend,\n- the CSS class corresponding of the displayed day of week (&quot;Sun&quot;, &quot;Mon&quot; and so on).</p>\n"},"styleGridCell":{"!type":"fn(node: ?, date: ?, hours: ?, minutes: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a cell.\nDelegates to styleGridCellFunc if defined or defaultStyleGridCell otherwise.</p>\n"},"styleRowHeaderCell":{"!type":"fn(node: ?, h: ?, m: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a row header cell.\nBy default this method is doing nothing.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(obj: ?, stateOnly: ?)","!doc":"<p>Updates all the renderers that represents the specified item(s).</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/ViewBase":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The dojox.calendar.ViewBase widget is the base of calendar view widgets</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_calendar":{},"_cssDays":{},"_DAY_IN_MILLISECONDS":{},"_gridMouseDown":{},"_isEditing":{"!doc":"<p>Whether an item is being edited or not.</p>\n"},"_layoutStep":{"!doc":"<p>The number of units displayed by a visual layout unit (i.e. a column or a row)</p>\n"},"_layoutUnit":{},"_rendererHandles":{},"_scrollPos":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tempIdCount":{},"_tempItemsMap":{},"_viewHandles":{},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"allowResizeLessThan24H":{"!doc":"<p>If an event has a duration greater than 24 hours, indicates if using a resize gesture, it can be resized to last less than 24 hours.\nThis flag is usually used when two different kind of renderers are used (MatrixView) to prevent changing the kind of renderer during an editing gesture.</p>\n"},"allowStartEndSwap":{"!doc":"<p>Specifies if the start and end time of an item can be swapped during an editing gesture. Note that using the keyboard this property is ignored. </p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoScroll":{"!doc":"<p>Indicates whether the view can be scrolled automatically. \nAuto scrolling is used when moving focus to a non visible renderer using keyboard \nand while editing an item. </p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"createOnGridClick":{"!doc":"<p>Indicates whether the user can create new event by clicking and dragging the grid.\nA createItem function must be defined on the view or the calendar object.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines. Uses Gregorian Calendar routines at dojo.date by default.</p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"doubleTapDelay":{"!doc":"<p>The maximum delay between two taps needed to trigger an &quot;itemDoubleClick&quot; event, in touch context.      </p>\n"},"editable":{"!doc":"<p>A flag that indicates whether or not the user can edit\nitems in the data provider.\nIf <code>true</code>, the item renderers in the control are editable.\nThe user can click on an item renderer, or use the keyboard or touch devices, to move or resize the associated event.</p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatItemTimeFunc":{"!doc":"<p>Optional function to format the time of day of the item renderers.\nThe function takes the date and render data object as arguments and returns a String.</p>\n"},"hoveredItem":{"!doc":"<p>The currently hovered data item.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"itemToRenderer":{"!doc":"<p>The associated array item to renderer list.</p>\n"},"itemToRendererKindFunc":{"!doc":"<p>An optional function to associate a kind of renderer (&quot;horizontal&quot;, &quot;label&quot; or null) with the specified item.\nBy default, if an item is lasting more that 24 hours an horizontal item is used, otherwise a label is used.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"layoutPriorityFunction":{"!doc":"<p>An optional comparison function use to determine the order the item will be laid out\nThe function is used to sort an array and must, as any sorting function, take two items \nas argument and must return an integer whose sign define order between arguments.\nBy default, a comparison by start time then end time is used.</p>\n"},"liveLayout":{"!doc":"<p>If true, all the events are laid out during the editing gesture. If false, only the edited event is laid out.</p>\n"},"minDurationSteps":{"!doc":"<p>The number of units used to define the minimal duration of the edited item.</p>\n"},"minDurationUnit":{"!doc":"<p>The unit used to define the minimal duration of the edited item.\n&quot;day&quot;, &quot;hour&quot; and &quot;minute&quot; are valid values.</p>\n"},"moveEnabled":{"!doc":"<p>A flag that indicates whether the user can move items displayed.\nIf <code>true</code>, the user can move the items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"rendererList":{"!doc":"<p>The list of used renderers</p>\n"},"rendererPool":{"!doc":"<p>The stack of recycled renderers available.</p>\n"},"resizeCursor":{"!doc":"<p>CSS value to apply to the cursor while resizing an item renderer. </p>\n"},"resizeEnabled":{"!doc":"<p>A flag that indicates whether the items can be resized.\nIf <code>true</code>, the control supports resizing of items.</p>\n"},"scrollable":{"!doc":"<p>Indicates whether the view can be scrolled or not.</p>\n"},"scrollMethod":{"!doc":"<p>Method used to scroll the view, for example the scroll of column view.\nValid value are:\n\n</p>\n<ul>\n<li>&quot;auto&quot;: let the view decide (default),</li>\n<li>&quot;css&quot;: use css 3d transform,</li>\n<li>&quot;dom&quot;: use the scrollTop property.</li>\n</ul>\n"},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"snapSteps":{"!doc":"<p>The number of units used to compute the snapping of the edited item.</p>\n"},"snapUnit":{"!doc":"<p>The unit of the snapping to apply during the editing of an event.\n&quot;day&quot;, &quot;hour&quot; and &quot;minute&quot; are valid values. </p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"stayInView":{"!doc":"<p>Specifies during editing, if the item is already in view, if the item must stay in the time range defined by the view or not.       </p>\n"},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"viewKind":{"!doc":"<p>Kind of the view. Used by the calendar widget to determine how to configure the view.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"applyRendererZIndex":{"!type":"fn(item: ?, renderer: ?, hovered: ?, selected: ?, edited: ?, focused: ?)","!doc":"<p>Applies the z-index to the renderer based on the state of the item.\nThis methods is setting a z-index of 20 is the item is selected or edited \nand the current lane value computed by the overlap layout (i.e. the renderers \nare stacked according to their lane).</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"computeOverlapping":{"!type":"fn(layoutItems: ?, func: ?) -> ?","!doc":"<p>Computes the overlap layout of a list of items. A lane and extent properties are added to each layout item.</p>\n"},"computeProjectionOnDate":{"!type":"fn(renderData: ?, refDate: ?, date: ?, max: ?) -> number","!doc":"<p>Computes the time to pixel projection in a day.</p>\n"},"computeRangeOverlap":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> [+Date]","!doc":"<p>Computes the overlap time range of the time ranges.\nReturns a vector of Date with at index 0 the start time and at index 1 the end time.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(args: ?)"},"createItemFunc":{"!type":"fn(view: ?, d: ?, e: ?)","!doc":"<p>A user supplied function that creates a new event.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"ensureMinimalDuration":{"!type":"fn(renderData: ?, item: ?, unit: ?, steps: ?, editKind: ?)","!doc":"<p>During the resize editing gesture, ensures that the item has the specified minimal duration.</p>\n"},"ensureVisibility":{"!type":"fn(start: ?, end: ?, margin: ?, visibilityTarget: ?, duration: ?)","!doc":"<p>Scrolls the view if the [start, end] time range is not visible or only partially visible.</p>\n"},"floorDate":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToMonth":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of the date&#39;s month.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getCSSPrefix":{"!type":"fn() -> string","!doc":"<p>Utility method that return the specific CSS prefix\nfor non standard CSS properties. Ex: -moz-border-radius.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRenderers":{"!type":"fn(item: ?) -> [+Object]","!doc":"<p>Returns the renderers that are currently used to displayed the speficied item.\nReturns an array of objects that contains two properties:\n- container: The DOM node that contains the renderer.\n- renderer: The dojox.calendar._RendererMixin instance.\nDo not keep references on the renderers are they are recycled and reused for other items.</p>\n"},"getTime":{"!type":"fn(e: ?, x: ?, y: ?, touchIndex: ?) -> +Date","!doc":"<p>Returns the time displayed at the specified point by this component.</p>\n"},"getWeekNumberLabel":{"!type":"fn(date: ?) -> ?","!doc":"<p>Returns the week number string from dojo.date.locale.format() method as\ndojox.date.XXXX calendar are not supporting the &quot;w&quot; pattern.</p>\n"},"invalidateLayout":{"!type":"fn()","!doc":"<p>Triggers a re-layout of the renderers.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isAscendantHasClass":{"!type":"fn(node: ?, ancestor: ?, className: ?) -> bool","!doc":"<p>Determines if a node has an ascendant node that has the css class specified.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemBeingEdited":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether an item is being edited or not.</p>\n"},"isItemEditable":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be edited or not.\nBy default it is using the editable property value.</p>\n"},"isItemFocused":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is focused or not.</p>\n"},"isItemHovered":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is hovered or not.</p>\n"},"isItemMoveEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be moved.\nBy default it is using the moveEnabled property value.</p>\n"},"isItemResizeEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be resized.\nBy default it is using the resizedEnabled property value.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> ?"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isOverlapping":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> bool","!doc":"<p>Computes if the first time range defined by the start1 and end1 parameters \nis overlapping the second time range defined by the start2 and end2 parameters.</p>\n"},"isSameDay":{"!type":"fn(date1: ?, date2: ?) -> bool","!doc":"<p>Tests if the specified dates are in the same day.</p>\n"},"isStartOfDay":{"!type":"fn(d: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(date: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"isWeekEnd":{"!type":"fn(date: ?) -> ?","!doc":"<p>Determines whether the specified date is a week-end.\nThis method is using dojo.date.locale.isWeekend() method as\ndojox.date.XXXX calendars are not supporting this method.</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"newDate":{"!type":"fn(obj: ?) -> +Date","!doc":"<p>Creates a new Date object.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()","!doc":"<p>Actually refresh the rendering. Implementation should implement that method.</p>\n"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"resize":{"!type":"fn(changeSize: ?)","!doc":"<p>Function to call when the view is resized. \nIf the view is in a Dijit container or in a Dojo mobile container, it will be automatically called.\nOn other use cases, this method must called when the window is resized and/or when the orientation has changed.</p>\n"},"scrollView":{"!type":"fn(dir: ?)","!doc":"<p>If the view is scrollable, scrolls it to the specified direction.</p>\n"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(obj: ?, stateOnly: ?)","!doc":"<p>Updates all the renderers that represents the specified item(s).</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/_VerticalScrollBarBase":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_scrollHandle":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scroll bar.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scroll bar.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The value of the scroll bar in pixel offset.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/ColumnViewSecondarySheet":{"!type":"fn()","prototype":{"!proto":"dojox/calendar/MatrixView.prototype","!doc":"<p>This class defines a matrix view designed to be embedded in a column view, \nusually to display long or all day events on one row. </p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_calendar":{},"_cssDays":{},"_DAY_IN_MILLISECONDS":{},"_ddRendererList":{},"_ddRendererPool":{},"_defaultHeight":{},"_gridMouseDown":{},"_isEditing":{"!doc":"<p>Whether an item is being edited or not.</p>\n"},"_layoutStep":{"!doc":"<p>The number of units displayed by a visual layout unit (i.e. a column or a row)</p>\n"},"_layoutUnit":{},"_offsets":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_rendererHandles":{},"_rowHeaderHandles":{},"_scrollPos":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tempIdCount":{},"_tempItemsMap":{},"_viewHandles":{},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"allowResizeLessThan24H":{},"allowStartEndSwap":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoScroll":{"!doc":"<p>Indicates whether the view can be scrolled automatically. \nAuto scrolling is used when moving focus to a non visible renderer using keyboard \nand while editing an item. </p>\n"},"baseClass":{},"cellPaddingTop":{},"class":{},"columnCount":{"!doc":"<p>The number of column to display (from the startDate).</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"createOnGridClick":{"!doc":"<p>Indicates whether the user can create new event by clicking and dragging the grid.\nA createItem function must be defined on the view or the calendar object.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines. Uses Gregorian Calendar routines at dojo.date by default.</p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"doubleTapDelay":{"!doc":"<p>The maximum delay between two taps needed to trigger an &quot;itemDoubleClick&quot; event, in touch context.      </p>\n"},"editable":{"!doc":"<p>A flag that indicates whether or not the user can edit\nitems in the data provider.\nIf <code>true</code>, the item renderers in the control are editable.\nThe user can click on an item renderer, or use the keyboard or touch devices, to move or resize the associated event.</p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"expandDuration":{"!doc":"<p>Duration of the animation when expanding or collapsing a row.</p>\n"},"expandEasing":{"!doc":"<p>Easing function of the animation when expanding or collapsing a row (null by default).</p>\n"},"expandRenderer":{"!doc":"<p>The class use to create drill down renderers.       </p>\n"},"expandRendererHeight":{"!doc":"<p>The height in pixels of the expand/collapse renderers that is applied by the layout.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatItemTimeFunc":{"!doc":"<p>Optional function to format the time of day of the item renderers.\nThe function takes the date and render data object as arguments and returns a String.</p>\n"},"horizontalRenderer":{"!doc":"<p>The class use to create horizontal renderers.</p>\n"},"horizontalRendererHeight":{"!doc":"<p>The height in pixels of the horizontal and label renderers that is applied by the layout.</p>\n"},"hoveredItem":{"!doc":"<p>The currently hovered data item.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"itemToRenderer":{"!doc":"<p>The associated array item to renderer list.</p>\n"},"itemToRendererKindFunc":{"!doc":"<p>An optional function to associate a kind of renderer (&quot;horizontal&quot;, &quot;label&quot; or null) with the specified item.\nBy default, if an item is lasting more that 24 hours an horizontal item is used, otherwise a label is used.</p>\n"},"labelRenderer":{"!doc":"<p>The class use to create label renderers.</p>\n"},"labelRendererHeight":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"layoutDuringResize":{},"layoutPriorityFunction":{"!doc":"<p>An optional comparison function use to determine the order the item will be laid out\nThe function is used to sort an array and must, as any sorting function, take two items \nas argument and must return an integer whose sign define order between arguments.\nBy default, a comparison by start time then end time is used.</p>\n"},"liveLayout":{},"minDurationSteps":{},"minDurationUnit":{},"moveEnabled":{"!doc":"<p>A flag that indicates whether the user can move items displayed.\nIf <code>true</code>, the user can move the items.</p>\n"},"naturalRowsHeight":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"percentOverlap":{"!doc":"<p>The percentage of the renderer width used to superimpose one item renderers on another \nwhen two events are overlapping. By default 0.</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"refEndTime":{},"refStartTime":{"!doc":"<p>(Optional) Start of the time interval of interest. \nIt is used to style differently the displayed rows out of the \ntime interval of interest.          </p>\n"},"renderData":{"!doc":"<p>The render data object contains all the data needed to render the widget.</p>\n"},"rendererList":{"!doc":"<p>The list of used renderers</p>\n"},"rendererPool":{"!doc":"<p>The stack of recycled renderers available.</p>\n"},"resizeAnimationDuration":{"!doc":"<p>Duration, in milliseconds, of the fade animation showing the item renderers after a widget resize.</p>\n"},"resizeCursor":{},"resizeEnabled":{"!doc":"<p>A flag that indicates whether the items can be resized.\nIf <code>true</code>, the control supports resizing of items.</p>\n"},"roundToDay":{},"rowCount":{},"scrollable":{},"scrollMethod":{"!doc":"<p>Method used to scroll the view, for example the scroll of column view.\nValid value are:\n\n</p>\n<ul>\n<li>&quot;auto&quot;: let the view decide (default),</li>\n<li>&quot;css&quot;: use css 3d transform,</li>\n<li>&quot;dom&quot;: use the scrollTop property.</li>\n</ul>\n"},"searchContainerNode":{},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"showCellLabel":{"!doc":"<p>Whether display or not the grid cells label (usually the day of month).</p>\n"},"snapSteps":{},"snapUnit":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startDate":{"!doc":"<p>The start date of the time interval displayed.\nIf not set at initialization time, will be set to current day.</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"stayInView":{},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"styleGridCellFunc":{"!doc":"<p>Custom function to customize the appearance of a grid cell by installing custom CSS class on the node.\nThe signature of the function must be the same then the styleGridCell one.\nBy default the defaultStyleGridCell function is used.</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"triggerExtent":{},"verticalGap":{"!doc":"<p>The number of pixels between two item renderers that are overlapping each other if the percentOverlap property is 0.</p>\n"},"viewKind":{"!doc":"<p>Type of the view. Used by the calendar widget to determine how to configure the view.\nThis view kind is &quot;matrix&quot;.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"collapseRow":{"!type":"fn(duration: ?, easing: ?, apply: ?)","!doc":"<p>Collapses the expanded row, if any.</p>\n"},"computeOverlapping":{"!type":"fn(layoutItems: ?, func: ?) -> ?","!doc":"<p>Computes the overlap layout of a list of items. A lane and extent properties are added to each layout item.</p>\n"},"computeProjectionOnDate":{"!type":"fn(renderData: ?, refDate: ?, date: ?, max: ?) -> number","!doc":"<p>Computes the time to pixel projection in a day.</p>\n"},"computeRangeOverlap":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> [+Date]","!doc":"<p>Computes the overlap time range of the time ranges.\nReturns a vector of Date with at index 0 the start time and at index 1 the end time.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"createItemFunc":{"!type":"fn(view: ?, d: ?, e: ?)","!doc":"<p>A user supplied function that creates a new event.</p>\n"},"defaultStyleGridCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a cell.\nBy default this method is setting the following CSS classes:\n- &quot;dojoxCalendarToday&quot; class name if the date displayed is the current date, \n- &quot;dojoxCalendarWeekend&quot; if the date represents a weekend or\n- &quot;dojoxCalendarDayDisabled&quot; if the date is out of the [refStartTime, refEndTime] interval.\n- the CSS class corresponding of the displayed day of week (&quot;Sun&quot;, &quot;Mon&quot; and so on).            </p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"ensureMinimalDuration":{"!type":"fn(renderData: ?, item: ?, unit: ?, steps: ?, editKind: ?)","!doc":"<p>During the resize editing gesture, ensures that the item has the specified minimal duration.</p>\n"},"ensureVisibility":{"!type":"fn(start: ?, end: ?, margin: ?, visibilityTarget: ?, duration: ?)","!doc":"<p>Scrolls the view if the [start, end] time range is not visible or only partially visible.</p>\n"},"expandRendererClickHandler":{"!type":"fn(e: ?, renderer: ?)","!doc":"<p>Default action when an expand renderer is clicked.\nThis method will expand the secondary sheet to show all the events.</p>\n"},"expandRow":{"!type":"fn(rowIndex: ?, colIndex?: ?, duration?: ?, easing?: ?, apply: ?) -> number","!doc":"<p>Expands the specified row.</p>\n"},"floorDate":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToMonth":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of the date&#39;s month.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getCSSPrefix":{"!type":"fn() -> string","!doc":"<p>Utility method that return the specific CSS prefix\nfor non standard CSS properties. Ex: -moz-border-radius.</p>\n"},"getExpandedRowIndex":{"!type":"fn() -> number","!doc":"<p>Returns the index of the expanded row or -1 if there&#39;s no row expanded.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRenderers":{"!type":"fn(item: ?) -> [+Object]","!doc":"<p>Returns the renderers that are currently used to displayed the speficied item.\nReturns an array of objects that contains two properties:\n- container: The DOM node that contains the renderer.\n- renderer: The dojox.calendar._RendererMixin instance.\nDo not keep references on the renderers are they are recycled and reused for other items.</p>\n"},"getTime":{"!type":"fn(e: ?, x: ?, y: ?, touchIndex: ?) -> +Date","!doc":"<p>Returns the time displayed at the specified point by this component.</p>\n"},"getWeekNumberLabel":{"!type":"fn(date: ?) -> ?","!doc":"<p>Returns the week number string from dojo.date.locale.format() method as\ndojox.date.XXXX calendar are not supporting the &quot;w&quot; pattern.</p>\n"},"invalidateLayout":{"!type":"fn()","!doc":"<p>Triggers a re-layout of the renderers.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isAscendantHasClass":{"!type":"fn(node: ?, ancestor: ?, className: ?) -> bool","!doc":"<p>Determines if a node has an ascendant node that has the css class specified.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemBeingEdited":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether an item is being edited or not.</p>\n"},"isItemEditable":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be edited or not.\nBy default it is using the editable property value.</p>\n"},"isItemFocused":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is focused or not.</p>\n"},"isItemHovered":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is hovered or not.</p>\n"},"isItemMoveEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be moved.\nBy default it is using the moveEnabled property value.</p>\n"},"isItemResizeEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be resized.\nBy default it is using the resizedEnabled property value.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> ?"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isOverlapping":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> bool","!doc":"<p>Computes if the first time range defined by the start1 and end1 parameters \nis overlapping the second time range defined by the start2 and end2 parameters.</p>\n"},"isSameDay":{"!type":"fn(date1: ?, date2: ?) -> bool","!doc":"<p>Tests if the specified dates are in the same day.</p>\n"},"isStartOfDay":{"!type":"fn(d: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(date: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"isWeekEnd":{"!type":"fn(date: ?) -> ?","!doc":"<p>Determines whether the specified date is a week-end.\nThis method is using dojo.date.locale.isWeekend() method as\ndojox.date.XXXX calendars are not supporting this method.</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"newDate":{"!type":"fn(obj: ?) -> +Date","!doc":"<p>Creates a new Date object.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"resize":{"!type":"fn(changeSize: ?)"},"scrollView":{"!type":"fn(dir: ?)","!doc":"<p>If the view is scrollable, scrolls it to the specified direction.</p>\n"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleColumnHeaderCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a column header cell.\nBy default this method is setting the &quot;dojoxCalendarWeekend&quot; if the day of week represents a weekend.</p>\n"},"styleGridCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a cell.\nDelegates to styleGridCellFunc if defined or defaultStyleGridCell otherwise.</p>\n"},"styleRowHeaderCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a row header cell.\nBy default this method is doing nothing.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(obj: ?, stateOnly: ?)","!doc":"<p>Updates all the renderers that represents the specified item(s).</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/MatrixView":{"!type":"fn()","prototype":{"!proto":"dojox/calendar/ViewBase.prototype","!doc":"<p>The matrix view is a calendar view that displaying a matrix where each cell is a day.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_calendar":{},"_cssDays":{},"_DAY_IN_MILLISECONDS":{},"_ddRendererList":{},"_ddRendererPool":{},"_gridMouseDown":{},"_isEditing":{"!doc":"<p>Whether an item is being edited or not.</p>\n"},"_layoutStep":{"!doc":"<p>The number of units displayed by a visual layout unit (i.e. a column or a row)</p>\n"},"_layoutUnit":{},"_offsets":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_rendererHandles":{},"_rowHeaderHandles":{},"_scrollPos":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tempIdCount":{},"_tempItemsMap":{},"_viewHandles":{},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"allowResizeLessThan24H":{},"allowStartEndSwap":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoScroll":{"!doc":"<p>Indicates whether the view can be scrolled automatically. \nAuto scrolling is used when moving focus to a non visible renderer using keyboard \nand while editing an item. </p>\n"},"baseClass":{},"cellPaddingTop":{"!doc":"<p>The top offset in pixels of each cell applied by the layout.</p>\n"},"class":{},"columnCount":{"!doc":"<p>The number of column to display (from the startDate).</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"createOnGridClick":{"!doc":"<p>Indicates whether the user can create new event by clicking and dragging the grid.\nA createItem function must be defined on the view or the calendar object.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines. Uses Gregorian Calendar routines at dojo.date by default.</p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"doubleTapDelay":{"!doc":"<p>The maximum delay between two taps needed to trigger an &quot;itemDoubleClick&quot; event, in touch context.      </p>\n"},"editable":{"!doc":"<p>A flag that indicates whether or not the user can edit\nitems in the data provider.\nIf <code>true</code>, the item renderers in the control are editable.\nThe user can click on an item renderer, or use the keyboard or touch devices, to move or resize the associated event.</p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"expandDuration":{"!doc":"<p>Duration of the animation when expanding or collapsing a row.</p>\n"},"expandEasing":{"!doc":"<p>Easing function of the animation when expanding or collapsing a row (null by default).</p>\n"},"expandRenderer":{"!doc":"<p>The class use to create drill down renderers.       </p>\n"},"expandRendererHeight":{"!doc":"<p>The height in pixels of the expand/collapse renderers that is applied by the layout.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatItemTimeFunc":{"!doc":"<p>Optional function to format the time of day of the item renderers.\nThe function takes the date and render data object as arguments and returns a String.</p>\n"},"horizontalRenderer":{"!doc":"<p>The class use to create horizontal renderers.</p>\n"},"horizontalRendererHeight":{"!doc":"<p>The height in pixels of the horizontal and label renderers that is applied by the layout.</p>\n"},"hoveredItem":{"!doc":"<p>The currently hovered data item.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"itemToRenderer":{"!doc":"<p>The associated array item to renderer list.</p>\n"},"itemToRendererKindFunc":{"!doc":"<p>An optional function to associate a kind of renderer (&quot;horizontal&quot;, &quot;label&quot; or null) with the specified item.\nBy default, if an item is lasting more that 24 hours an horizontal item is used, otherwise a label is used.</p>\n"},"labelRenderer":{"!doc":"<p>The class use to create label renderers.</p>\n"},"labelRendererHeight":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"layoutDuringResize":{"!doc":"<p>Indicates if the item renderers&#39; position and size is updated or if they are hidden during a resize of the widget. </p>\n"},"layoutPriorityFunction":{"!doc":"<p>An optional comparison function use to determine the order the item will be laid out\nThe function is used to sort an array and must, as any sorting function, take two items \nas argument and must return an integer whose sign define order between arguments.\nBy default, a comparison by start time then end time is used.</p>\n"},"liveLayout":{},"minDurationSteps":{},"minDurationUnit":{},"moveEnabled":{"!doc":"<p>A flag that indicates whether the user can move items displayed.\nIf <code>true</code>, the user can move the items.</p>\n"},"naturalRowsHeight":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"percentOverlap":{"!doc":"<p>The percentage of the renderer width used to superimpose one item renderers on another \nwhen two events are overlapping. By default 0.</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"refEndTime":{},"refStartTime":{"!doc":"<p>(Optional) Start of the time interval of interest. \nIt is used to style differently the displayed rows out of the \ntime interval of interest.          </p>\n"},"renderData":{"!doc":"<p>The render data object contains all the data needed to render the widget.</p>\n"},"rendererList":{"!doc":"<p>The list of used renderers</p>\n"},"rendererPool":{"!doc":"<p>The stack of recycled renderers available.</p>\n"},"resizeAnimationDuration":{"!doc":"<p>Duration, in milliseconds, of the fade animation showing the item renderers after a widget resize.</p>\n"},"resizeCursor":{},"resizeEnabled":{"!doc":"<p>A flag that indicates whether the items can be resized.\nIf <code>true</code>, the control supports resizing of items.</p>\n"},"roundToDay":{"!doc":"<p>For horizontal renderers that are not filling entire days, whether fill the day or not.</p>\n"},"rowCount":{"!doc":"<p>The number of rows to display (from the startDate).</p>\n"},"scrollable":{},"scrollMethod":{"!doc":"<p>Method used to scroll the view, for example the scroll of column view.\nValid value are:\n\n</p>\n<ul>\n<li>&quot;auto&quot;: let the view decide (default),</li>\n<li>&quot;css&quot;: use css 3d transform,</li>\n<li>&quot;dom&quot;: use the scrollTop property.</li>\n</ul>\n"},"searchContainerNode":{},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"showCellLabel":{"!doc":"<p>Whether display or not the grid cells label (usually the day of month).</p>\n"},"snapSteps":{},"snapUnit":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startDate":{"!doc":"<p>The start date of the time interval displayed.\nIf not set at initialization time, will be set to current day.</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"stayInView":{},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"styleGridCellFunc":{"!doc":"<p>Custom function to customize the appearance of a grid cell by installing custom CSS class on the node.\nThe signature of the function must be the same then the styleGridCell one.\nBy default the defaultStyleGridCell function is used.</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"triggerExtent":{},"verticalGap":{"!doc":"<p>The number of pixels between two item renderers that are overlapping each other if the percentOverlap property is 0.</p>\n"},"viewKind":{"!doc":"<p>Type of the view. Used by the calendar widget to determine how to configure the view.\nThis view kind is &quot;matrix&quot;.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"collapseRow":{"!type":"fn(duration: ?, easing: ?, apply: ?)","!doc":"<p>Collapses the expanded row, if any.</p>\n"},"computeOverlapping":{"!type":"fn(layoutItems: ?, func: ?) -> ?","!doc":"<p>Computes the overlap layout of a list of items. A lane and extent properties are added to each layout item.</p>\n"},"computeProjectionOnDate":{"!type":"fn(renderData: ?, refDate: ?, date: ?, max: ?) -> number","!doc":"<p>Computes the time to pixel projection in a day.</p>\n"},"computeRangeOverlap":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> [+Date]","!doc":"<p>Computes the overlap time range of the time ranges.\nReturns a vector of Date with at index 0 the start time and at index 1 the end time.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"createItemFunc":{"!type":"fn(view: ?, d: ?, e: ?)","!doc":"<p>A user supplied function that creates a new event.</p>\n"},"defaultStyleGridCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a cell.\nBy default this method is setting the following CSS classes:\n- &quot;dojoxCalendarToday&quot; class name if the date displayed is the current date, \n- &quot;dojoxCalendarWeekend&quot; if the date represents a weekend or\n- &quot;dojoxCalendarDayDisabled&quot; if the date is out of the [refStartTime, refEndTime] interval.\n- the CSS class corresponding of the displayed day of week (&quot;Sun&quot;, &quot;Mon&quot; and so on).            </p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"ensureMinimalDuration":{"!type":"fn(renderData: ?, item: ?, unit: ?, steps: ?, editKind: ?)","!doc":"<p>During the resize editing gesture, ensures that the item has the specified minimal duration.</p>\n"},"ensureVisibility":{"!type":"fn(start: ?, end: ?, margin: ?, visibilityTarget: ?, duration: ?)","!doc":"<p>Scrolls the view if the [start, end] time range is not visible or only partially visible.</p>\n"},"expandRendererClickHandler":{"!type":"fn(e: ?, renderer: ?)","!doc":"<p>Default action when an expand renderer is clicked.</p>\n"},"expandRow":{"!type":"fn(rowIndex: ?, colIndex?: ?, duration?: ?, easing?: ?, apply: ?) -> number","!doc":"<p>Expands the specified row.</p>\n"},"floorDate":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToMonth":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of the date&#39;s month.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getCSSPrefix":{"!type":"fn() -> string","!doc":"<p>Utility method that return the specific CSS prefix\nfor non standard CSS properties. Ex: -moz-border-radius.</p>\n"},"getExpandedRowIndex":{"!type":"fn() -> number","!doc":"<p>Returns the index of the expanded row or -1 if there&#39;s no row expanded.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRenderers":{"!type":"fn(item: ?) -> [+Object]","!doc":"<p>Returns the renderers that are currently used to displayed the speficied item.\nReturns an array of objects that contains two properties:\n- container: The DOM node that contains the renderer.\n- renderer: The dojox.calendar._RendererMixin instance.\nDo not keep references on the renderers are they are recycled and reused for other items.</p>\n"},"getTime":{"!type":"fn(e: ?, x: ?, y: ?, touchIndex: ?) -> +Date","!doc":"<p>Returns the time displayed at the specified point by this component.</p>\n"},"getWeekNumberLabel":{"!type":"fn(date: ?) -> ?","!doc":"<p>Returns the week number string from dojo.date.locale.format() method as\ndojox.date.XXXX calendar are not supporting the &quot;w&quot; pattern.</p>\n"},"invalidateLayout":{"!type":"fn()","!doc":"<p>Triggers a re-layout of the renderers.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isAscendantHasClass":{"!type":"fn(node: ?, ancestor: ?, className: ?) -> bool","!doc":"<p>Determines if a node has an ascendant node that has the css class specified.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemBeingEdited":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether an item is being edited or not.</p>\n"},"isItemEditable":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be edited or not.\nBy default it is using the editable property value.</p>\n"},"isItemFocused":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is focused or not.</p>\n"},"isItemHovered":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is hovered or not.</p>\n"},"isItemMoveEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be moved.\nBy default it is using the moveEnabled property value.</p>\n"},"isItemResizeEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be resized.\nBy default it is using the resizedEnabled property value.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> ?"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isOverlapping":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> bool","!doc":"<p>Computes if the first time range defined by the start1 and end1 parameters \nis overlapping the second time range defined by the start2 and end2 parameters.</p>\n"},"isSameDay":{"!type":"fn(date1: ?, date2: ?) -> bool","!doc":"<p>Tests if the specified dates are in the same day.</p>\n"},"isStartOfDay":{"!type":"fn(d: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(date: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"isWeekEnd":{"!type":"fn(date: ?) -> ?","!doc":"<p>Determines whether the specified date is a week-end.\nThis method is using dojo.date.locale.isWeekend() method as\ndojox.date.XXXX calendars are not supporting this method.</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"newDate":{"!type":"fn(obj: ?) -> +Date","!doc":"<p>Creates a new Date object.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"resize":{"!type":"fn(changeSize: ?)"},"scrollView":{"!type":"fn(dir: ?)","!doc":"<p>If the view is scrollable, scrolls it to the specified direction.</p>\n"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleColumnHeaderCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a column header cell.\nBy default this method is setting the &quot;dojoxCalendarWeekend&quot; if the day of week represents a weekend.</p>\n"},"styleGridCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a cell.\nDelegates to styleGridCellFunc if defined or defaultStyleGridCell otherwise.</p>\n"},"styleRowHeaderCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a row header cell.\nBy default this method is doing nothing.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(obj: ?, stateOnly: ?)","!doc":"<p>Updates all the renderers that represents the specified item(s).</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/VerticalRenderer":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The default item vertical renderer.     </p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_displayValue":{},"_displayValueMap":{},"_orientation":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"edited":{"!doc":"<p>Indicates that the item displayed by this renderer is in editing mode.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovered":{"!doc":"<p>Indicates that the item displayed by this renderer is hovered.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"item":{"!doc":"<p>The layout item displayed by this renderer.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"moveEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be moved.</p>\n"},"owner":{"!doc":"<p>The view that contains this renderer.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"resizeEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be resized.</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>Indicates that the item displayed by this renderer is selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"storeState":{"!doc":"<p>Indicates that the item displayed by this renderer is not in the store, being saved to the store or in the store.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"visibilityLimits":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDisplayValue":{"!type":"fn(part: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRendering":{"!type":"fn(w?: ?, h?: ?)","!doc":"<p>Updates the visual appearance of the renderer according the new values of the properties and the new size of the component.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/_RendererMixin":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>This class is the base class of calendar renderers.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_displayValue":{},"_displayValueMap":{},"_orientation":{},"edited":{"!doc":"<p>Indicates that the item displayed by this renderer is in editing mode.</p>\n"},"focused":{"!doc":"<p>Indicates that the item displayed by this renderer is focused.</p>\n"},"hovered":{"!doc":"<p>Indicates that the item displayed by this renderer is hovered.</p>\n"},"item":{"!doc":"<p>The layout item displayed by this renderer.</p>\n"},"moveEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be moved.</p>\n"},"owner":{"!doc":"<p>The view that contains this renderer.</p>\n"},"resizeEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be resized.</p>\n"},"selected":{"!doc":"<p>Indicates that the item displayed by this renderer is selected.</p>\n"},"storeState":{"!doc":"<p>Indicates that the item displayed by this renderer is not in the store, being saved to the store or in the store.</p>\n"},"visibilityLimits":{},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"getDisplayValue":{"!type":"fn(part: ?) -> ?"},"postscript":{"!type":"fn(params?: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"updateRendering":{"!type":"fn(w?: ?, h?: ?)","!doc":"<p>Updates the visual appearance of the renderer according the new values of the properties and the new size of the component.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/HorizontalRenderer":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The default item horizontal renderer. </p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_displayValue":{},"_displayValueMap":{},"_orientation":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"arrowPadding":{"!doc":"<p>The padding size in pixels to apply to the label container on left and/or right side, to show the arrows correctly.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"edited":{"!doc":"<p>Indicates that the item displayed by this renderer is in editing mode.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovered":{"!doc":"<p>Indicates that the item displayed by this renderer is hovered.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"item":{"!doc":"<p>The layout item displayed by this renderer.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"moveEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be moved.</p>\n"},"owner":{"!doc":"<p>The view that contains this renderer.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"resizeEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be resized.</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>Indicates that the item displayed by this renderer is selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"storeState":{"!doc":"<p>Indicates that the item displayed by this renderer is not in the store, being saved to the store or in the store.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"visibilityLimits":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDisplayValue":{"!type":"fn(part: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRendering":{"!type":"fn(w?: ?, h?: ?)","!doc":"<p>Updates the visual appearance of the renderer according the new values of the properties and the new size of the component.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/LabelRenderer":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The default item label renderer. </p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_displayValue":{},"_displayValueMap":{},"_orientation":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"edited":{"!doc":"<p>Indicates that the item displayed by this renderer is in editing mode.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovered":{"!doc":"<p>Indicates that the item displayed by this renderer is hovered.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"item":{"!doc":"<p>The layout item displayed by this renderer.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"moveEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be moved.</p>\n"},"owner":{"!doc":"<p>The view that contains this renderer.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"resizeEnabled":{},"searchContainerNode":{},"selected":{"!doc":"<p>Indicates that the item displayed by this renderer is selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"storeState":{"!doc":"<p>Indicates that the item displayed by this renderer is not in the store, being saved to the store or in the store.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"visibilityLimits":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDisplayValue":{"!type":"fn(part: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRendering":{"!type":"fn(w?: ?, h?: ?)","!doc":"<p>Updates the visual appearance of the renderer according the new values of the properties and the new size of the component.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/ExpandRenderer":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The default renderer display in MatrixView cells where some item renderers cannot be displayed because of size constraints.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"columnIndex":{"!doc":"<p>Column index where this renderer is used.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"date":{"!doc":"<p>The date displayed by the cell where this renderer is used.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"down":{"!doc":"<p>Indicates that the renderer is pressed.</p>\n"},"focused":{"!doc":"<p>Indicates that the renderer is focused.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"items":{"!doc":"<p>List of items that are not displayed in the cell because of the size constraints.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"owner":{"!doc":"<p>The view that contains this renderer.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"rowIndex":{"!doc":"<p>Row index where this renderer is used.</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"up":{"!doc":"<p>Indicates that the mouse cursor is over renderer.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/Keyboard":{"!type":"fn()","prototype":{"!doc":"<p>This mixin is managing the keyboard interactions on a calendar view.</p>\n","allDayKeyboardLeftRightSteps":{"!doc":"<p>Unit used during editing of an all day event using the keyboard and the left or right keys were pressed. Valid values are &quot;week&quot;, &quot;day&quot;, &quot;hours&quot; &quot;minute&quot;.</p>\n"},"allDayKeyboardLeftRightUnit":{"!doc":"<p>Steps used during editing of an all day event using the keyboard and the up or down keys were pressed.</p>\n"},"allDayKeyboardUpDownSteps":{"!doc":"<p>Unit used during editing of an all day event using the keyboard and the up or down keys were pressed. Valid values are &quot;week&quot;, &quot;day&quot;, &quot;hours&quot; &quot;minute&quot;.     </p>\n"},"allDayKeyboardUpDownUnit":{"!doc":"<p>Steps used during editing of an all day event using the keyboard and the up or down keys were pressed.</p>\n"},"focusedItem":{"!doc":"<p>The data item that currently has the focus.</p>\n"},"keyboardLeftRightSteps":{"!doc":"<p>Unit used during editing of an event using the keyboard and the left or right keys were pressed.        </p>\n"},"keyboardLeftRightUnit":{"!doc":"<p>Unit used during editing of an event using the keyboard and the left or right keys were pressed. Valid values are &quot;week&quot;, &quot;day&quot;, &quot;hours&quot; &quot;minute&quot;.</p>\n"},"keyboardUpDownSteps":{"!doc":"<p>Steps used during editing of an event using the keyboard and the up or down keys were pressed.      </p>\n"},"keyboardUpDownUnit":{"!doc":"<p>Unit used during editing of an event using the keyboard and the up or down keys were pressed. Valid values are &quot;week&quot;, &quot;day&quot;, &quot;hours&quot; &quot;minute&quot;.</p>\n"},"maxScrollAnimationDuration":{"!doc":"<p>The duration in milliseconds to scroll the entire view. \nThe scroll speed is constant when scrolling to show an item renderer. </p>\n"},"resizeModifier":{},"showFocus":{"!doc":"<p>Show or hide the focus graphic feedback on item renderers.</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"postCreate":{"!type":"fn()"}}},"dojox/calendar/Mouse":{"!type":"fn()","prototype":{"!doc":"<p>This plugin is managing the mouse interactions on item renderers displayed by a calendar view.      </p>\n","triggerExtent":{"!doc":"<p>The distance in pixels along the vertical or horizontal axis to cover with the \nmouse button down before triggering the editing gesture.</p>\n"},"postMixInProperties":{"!type":"fn()"}}},"dojox/calendar/MobileCalendar":{"!type":"fn()","prototype":{"!proto":"dojox/calendar/CalendarBase.prototype","!doc":"<p>This class defines a calendar widget that display events in time designed to be used in mobile environment.</p>\n","_animRangeOutDir":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_calendar":{},"_currentViewIndex":{},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_timeInterval":{},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"animateRange":{"!doc":"<p>Indicates that the previous/next range method will be animated.</p>\n"},"animationRangeDuration":{"!doc":"<p>The duration of the next/previous range animation.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"columnView":{"!doc":"<p>The column view is displaying one day to seven days time intervals.</p>\n"},"columnViewProps":{"!doc":"<p>Map of property/value passed to the constructor of the column view.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"createItemFunc":{"!doc":"<p>A user supplied function that creates a new event.\nThis function is used when createOnGridClick is set to true and the user is clicking and dragging on the grid.\nThis view takes two parameters:\n\n</p>\n<ul>\n<li>view: the current view,</li>\n<li>d: the date at the clicked location.</li>\n</ul>\n"},"createOnGridClick":{"!doc":"<p>Indicates whether the user can create new event by clicking and dragging the grid.\nA createItem function must be defined on the view or the calendar object.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"currentView":{"!doc":"<p>The current view displayed by the Calendar object.\nThe currentViewChange event can be used to react on a view change.</p>\n"},"date":{"!doc":"<p>The reference date used to determine along with the <code>dateInterval</code> \nand <code>dateIntervalSteps</code> properties the time interval to display.</p>\n"},"dateInterval":{"!doc":"<p>The date interval used to compute along with the <code>date</code> and \n<code>dateIntervalSteps</code> the time interval to display.\nValid values are &quot;day&quot;, &quot;week&quot; (default value) and &quot;month&quot;.</p>\n"},"dateIntervalSteps":{"!doc":"<p>The number of date intervals used to compute along with the <code>date</code> and \n<code>dateInterval</code> the time interval to display.\nDefault value is 1.     </p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines. Uses Gregorian Calendar routines at dojo.date by default.</p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>A flag that indicates whether or not the user can edit\nitems in the data provider.\nIf <code>true</code>, the item renderers in the control are editable.\nThe user can click on an item renderer, or use the keyboard or touch devices, to move or resize the associated event.</p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endDate":{"!doc":"<p>The end date of the displayed time interval (included).     </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"firstDayOfWeek":{"!doc":"<p>(Optional) The first day of week override. By default the first day of week is determined \nfor the current locale (extracted from the CLDR).\nSpecial value -1 (default value), means use locale dependent value.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatItemTimeFunc":{"!doc":"<p>Optional function to format the time of day of the item renderers.\nThe function takes the date and render data object as arguments and returns a String.</p>\n"},"hoveredItem":{"!doc":"<p>Current render item which is under the mouse cursor.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"matrixView":{"!doc":"<p>The column view is displaying time intervals that lasts more than seven days.</p>\n"},"matrixViewProps":{"!doc":"<p>Map of property/value passed to the constructor of the matrix view.</p>\n"},"moveEnabled":{"!doc":"<p>A flag that indicates whether the user can move items displayed.\nIf <code>true</code>, the user can move the items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"resizeEnabled":{"!doc":"<p>A flag that indicates whether the items can be resized.\nIf <code>true</code>, the control supports resizing of items.</p>\n"},"searchContainerNode":{},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startDate":{"!doc":"<p>The start date of the displayed time interval.</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"viewChangeDuration":{},"viewContainer":{"!doc":"<p>The DOM node that will contains the views.</p>\n"},"views":{"!doc":"<p>The views displayed by the widget.\nTo add/remove only one view, prefer, respectively, the addView() or removeView() methods.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"addView":{"!type":"fn(view: ?, index: ?)","!doc":"<p>Add a view to the calendar&#39;s view list.</p>\n"},"buildRendering":{"!type":"fn()"},"columnViewColumnHeaderClick":{"!type":"fn(e: ?)","!doc":"<p>Function called when the cell of a column header of the column view is clicked.\nShow the time range defined by the clicked date.</p>\n"},"computeTimeInterval":{"!type":"fn() -> []","!doc":"<p>Computes the displayed time interval according to the date, dateInterval and \ndateIntervalSteps if date is not null or startDate and endDate properties otherwise.</p>\n"},"configureButtons":{"!type":"fn()","!doc":"<p>Set the localized labels of the buttons and the event handlers.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(args: ?)"},"dayButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggerred when the day button is clicked.\nBy default, sets the dateInterval property to &quot;day&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"floorDate":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToWeek":{"!type":"fn(d: ?) -> ?","!doc":"<p>Floors the specified date to the beginning of week.</p>\n"},"fourDaysButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggerred when the 4 days button is clicked.\nBy default, sets the dateInterval property to &quot;day&quot; and \nthe &quot;dateIntervalSteps&quot; to 4.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> +null"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToday":{"!type":"fn()","!doc":"<p>Changes the displayed time interval to show the current day.\nSets the date property to the current day, the dateInterval property to &quot;day&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"},"installDefaultViewsActions":{"!type":"fn(views: ?)","!doc":"<p>Installs the default actions on newly created default views.\nBy default this action is registering:\n- the matrixViewRowHeaderClick method   on the rowHeaderClick event of the matrix view.\n- the columnViewColumnHeaderClick method    on the columnHeaderClick event of the column view.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemEditable":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be edited.\nBy default it is using the editable property value.</p>\n"},"isItemHovered":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is hovered or not.</p>\n"},"isItemMoveEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be moved.\nBy default it is using the moveEnabled property value.</p>\n"},"isItemResizeEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be resized.\nBy default it is using the resizedEnabled property value.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns wether an item is selected or not.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isStartOfDay":{"!type":"fn(d: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(date: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"matrixViewRowHeaderClick":{"!type":"fn(e: ?)","!doc":"<p>Function called when the cell of a row header of the matrix view is clicked.\nThe implementation is doing the foolowing actions:\n- If another row is already expanded, collapse it and then expand the clicked row.\n- If the clicked row is already expadned, collapse it.\n- If no row is expanded, expand the click row.</p>\n"},"monthButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggered when the month button is clicked.\nBy default, sets the dateInterval property to &quot;month&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"},"newDate":{"!type":"fn(obj: ?) -> +Date","!doc":"<p>Creates a new Date object.</p>\n"},"nextRange":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"previousRange":{"!type":"fn()"},"refreshRendering":{"!type":"fn()","!doc":"<p>Refreshes all the visual rendering of the calendar. </p>\n"},"removeView":{"!type":"fn(view: ?)","!doc":"<p>Removes a view from the calendar&#39;s view list.</p>\n"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"resize":{"!type":"fn(changeSize: ?)"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?) -> bool","!doc":"<p>Applies selection triggered by an user interaction</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)","!doc":"<p>Change the selection state of an item.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"todayButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggered when the today button is clicked.\nBy default, calls the goToday() method.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(obj: ?, stateOnly: ?)"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"},"weekButtonClick":{"!type":"fn(e: ?)","!doc":"<p>The action triggered when the week button is clicked.\nBy default, sets the dateInterval property to &quot;week&quot; and \nthe &quot;dateIntervalSteps&quot; to 1.</p>\n"}}},"dojox/calendar/MobileVerticalRenderer":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The mobile specific item vertical renderer.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_displayValue":{},"_displayValueMap":{},"_orientation":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"edited":{"!doc":"<p>Indicates that the item displayed by this renderer is in editing mode.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovered":{"!doc":"<p>Indicates that the item displayed by this renderer is hovered.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"item":{"!doc":"<p>The layout item displayed by this renderer.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"mobile":{},"moveEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be moved.</p>\n"},"owner":{"!doc":"<p>The view that contains this renderer.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"resizeEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be resized.</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>Indicates that the item displayed by this renderer is selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"storeState":{"!doc":"<p>Indicates that the item displayed by this renderer is not in the store, being saved to the store or in the store.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"visibilityLimits":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDisplayValue":{"!type":"fn(part: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRendering":{"!type":"fn(w?: ?, h?: ?)","!doc":"<p>Updates the visual appearance of the renderer according the new values of the properties and the new size of the component.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/MobileHorizontalRenderer":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The mobile specific item horizontal renderer.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_displayValue":{},"_displayValueMap":{},"_orientation":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"arrowPadding":{"!doc":"<p>The padding size in pixels to apply to the label container on left and/or right side, to show the arrows correctly.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"edited":{"!doc":"<p>Indicates that the item displayed by this renderer is in editing mode.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovered":{"!doc":"<p>Indicates that the item displayed by this renderer is hovered.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"item":{"!doc":"<p>The layout item displayed by this renderer.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"mobile":{},"moveEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be moved.</p>\n"},"owner":{"!doc":"<p>The view that contains this renderer.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"resizeEnabled":{"!doc":"<p>Whether the event displayed by this renderer can be resized.</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>Indicates that the item displayed by this renderer is selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"storeState":{"!doc":"<p>Indicates that the item displayed by this renderer is not in the store, being saved to the store or in the store.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"visibilityLimits":{},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDisplayValue":{"!type":"fn(part: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRendering":{"!type":"fn(w?: ?, h?: ?)","!doc":"<p>Updates the visual appearance of the renderer according the new values of the properties and the new size of the component.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/Touch":{"!type":"fn()","prototype":{"!doc":"<p>This plugin is managing the touch interactions on item renderers displayed by a calendar view.      </p>\n","autoScrollTouchMargin":{"!doc":"<p>The minimum number of minutes of margin around the edited event. </p>\n"},"touchEndEditingTimer":{"!doc":"<p>The delay after which the item is leaving the editing mode after the previous editing gesture, in touch context.</p>\n"},"touchStartEditingTimer":{"!doc":"<p>The delay of one touch over the renderer before setting the item in editing mode.       </p>\n"},"postMixInProperties":{"!type":"fn()"}}},"dojox/mobile/Button":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Non-templated BUTTON widget with a thin API wrapper for click \nevents and for setting the label.\n\n</p>\n<p>Buttons can display a label, an icon, or both.\nA label should always be specified (through innerHTML) or the label\nattribute.  It can be hidden via showLabel=false.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{"!doc":"<p>Overrides the automatic assignment of type to nodes, because it causes\nexception on IE. Instead, the type must be specified as this.type\nwhen the node is created, as part of the original DOM.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>The duration of selection, in milliseconds, or -1 for no post-click CSS styling.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>The label of the button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> bool"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/calendar/MonthColumnView":{"!type":"fn()","prototype":{"!proto":"dojox/calendar/ViewBase.prototype","!doc":"<p>The month column view is a calendar view used to display a month per column where each cell of the column is a day.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_calendar":{},"_columnHeaderHandlers":{},"_cssDays":{},"_DAY_IN_MILLISECONDS":{},"_gridMouseDown":{},"_isEditing":{"!doc":"<p>Whether an item is being edited or not.</p>\n"},"_layoutStep":{"!doc":"<p>The number of units displayed by a visual layout unit (i.e. a column or a row)</p>\n"},"_layoutUnit":{"!doc":"<p>Unit of layout: each column is displaying a month. </p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_rendererHandles":{},"_scrollPos":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tempIdCount":{},"_tempItemsMap":{},"_viewHandles":{},"allDayAttr":{"!doc":"<p>The attribute of the store item that contains the all day state of \nthe events represented by this item.    Default is &quot;allDay&quot;.</p>\n"},"allowResizeLessThan24H":{},"allowStartEndSwap":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoScroll":{"!doc":"<p>Indicates whether the view can be scrolled automatically. \nAuto scrolling is used when moving focus to a non visible renderer using keyboard \nand while editing an item. </p>\n"},"baseClass":{},"class":{},"columnCount":{"!doc":"<p>The number of column to display (from the startDate).</p>\n"},"columnHeaderFormatLength":{"!doc":"<p>Length of the column labels. Valid values are &quot;wide&quot; or &quot;abbr&quot;.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"createOnGridClick":{"!doc":"<p>Indicates whether the user can create new event by clicking and dragging the grid.\nA createItem function must be defined on the view or the calendar object.</p>\n"},"cssClassFunc":{"!doc":"<p>Optional function that returns a css class name to apply to item renderers that are displaying the specified item in parameter. </p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines. Uses Gregorian Calendar routines at dojo.date by default.</p>\n"},"daySize":{"!doc":"<p>The desired size in pixels of an hour on the screen.\nNote that the effective size may be different as the time slot size must be an integer.</p>\n"},"decodeDate":{"!doc":"<p>An optional function to transform store date into Date objects. Default is null. </p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedItemsInvalidated":{"!doc":"<p>Whether the data items displayed must be recomputed, usually after the displayed \ntime range has changed. </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"doubleTapDelay":{"!doc":"<p>The maximum delay between two taps needed to trigger an &quot;itemDoubleClick&quot; event, in touch context.      </p>\n"},"editable":{"!doc":"<p>A flag that indicates whether or not the user can edit\nitems in the data provider.\nIf <code>true</code>, the item renderers in the control are editable.\nThe user can click on an item renderer, or use the keyboard or touch devices, to move or resize the associated event.</p>\n"},"encodeDate":{"!doc":"<p>An optional function to transform Date objects into store date. Default is null. </p>\n"},"endTimeAttr":{"!doc":"<p>The attribute of the store item that contains the end time of \nthe events represented by this item.    Default is &quot;endTime&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatItemTimeFunc":{"!doc":"<p>Optional function to format the time of day of the item renderers.\nThe function takes the date and render data object as arguments and returns a String.</p>\n"},"gridCellDatePattern":{"!doc":"<p>The date pattern of the cell labels. By default a custom function is used to compute the label.</p>\n"},"horizontalGap":{"!doc":"<p>The number of pixels between two item renderers.</p>\n"},"hoveredItem":{"!doc":"<p>The currently hovered data item.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"itemToRenderer":{"!doc":"<p>The associated array item to renderer list.</p>\n"},"itemToRendererKindFunc":{"!doc":"<p>An optional function to associate a kind of renderer (&quot;horizontal&quot;, &quot;label&quot; or null) with the specified item.\nBy default, if an item is lasting more that 24 hours an horizontal item is used, otherwise a label is used.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"layoutPriorityFunction":{"!doc":"<p>An optional comparison function use to determine the order the item will be laid out\nThe function is used to sort an array and must, as any sorting function, take two items \nas argument and must return an integer whose sign define order between arguments.\nBy default, a comparison by start time then end time is used.</p>\n"},"liveLayout":{},"minDurationSteps":{},"minDurationUnit":{},"moveEnabled":{"!doc":"<p>A flag that indicates whether the user can move items displayed.\nIf <code>true</code>, the user can move the items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"percentOverlap":{"!doc":"<p>The percentage of the renderer width used to superimpose one item renderer on another \nwhen two events are overlapping.</p>\n"},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"renderData":{"!doc":"<p>The render data is the object that contains all the properties needed to render the component.</p>\n"},"rendererList":{"!doc":"<p>The list of used renderers</p>\n"},"rendererPool":{"!doc":"<p>The stack of recycled renderers available.</p>\n"},"resizeCursor":{"!doc":"<p>CSS value to apply to the cursor while resizing an item renderer. </p>\n"},"resizeEnabled":{"!doc":"<p>A flag that indicates whether the items can be resized.\nIf <code>true</code>, the control supports resizing of items.</p>\n"},"roundToDay":{},"scrollable":{"!doc":"<p>Indicates whether the view can be scrolled or not.</p>\n"},"scrollBarRTLPosition":{"!doc":"<p>Position of the scroll bar in right-to-left display.\nValid values are &quot;left&quot; and &quot;right&quot;, default value is &quot;left&quot;.</p>\n"},"scrollMethod":{"!doc":"<p>Method used to scroll the view, for example the scroll of column view.\nValid value are:\n\n</p>\n<ul>\n<li>&quot;auto&quot;: let the view decide (default),</li>\n<li>&quot;css&quot;: use css 3d transform,</li>\n<li>&quot;dom&quot;: use the scrollTop property.</li>\n</ul>\n"},"scrollPosition":{"!doc":"<p>The scroll position of the view. </p>\n"},"searchContainerNode":{},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"showCellLabel":{"!doc":"<p>Whether display or not the grid cells label (usually the day of month).</p>\n"},"showHiddenItems":{"!doc":"<p>Whether show or not the hidden items.\nBy default the events that are shorter than a day are not displayed using vertical renderers by this widget.\nBut the grid cells that contains one or several hidden items display a decoration.</p>\n"},"snapSteps":{},"snapUnit":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startDate":{"!doc":"<p>The start date of the time interval displayed.\nIf not set at initialization time, will be set to current day.</p>\n"},"startTimeAttr":{"!doc":"<p>The attribute of the store item that contains the start time of \nthe events represented by this item.    Default is &quot;startTime&quot;. </p>\n"},"stayInView":{},"store":{"!doc":"<p>The store that contains the events to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"styleGridCellFunc":{"!doc":"<p>Custom function to customize the appearance of a grid cell by installing custom CSS class on the node.\nThe signature of the function must be the same then the styleGridCell one.\nBy default the defaultStyleGridCell function is used.</p>\n"},"summaryAttr":{"!doc":"<p>The attribute of the store item that contains the summary of \nthe events represented by this item.    Default is &quot;summary&quot;.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"verticalRenderer":{"!doc":"<p>The class use to create vertical renderers.</p>\n"},"viewKind":{"!doc":"<p>Type of the view. Used by the calendar widget to determine how to configure the view.\nThis view kind is &quot;columns&quot;.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"applyRendererZIndex":{"!type":"fn(item: ?, renderer: ?, hovered: ?, selected: ?, edited: ?, focused: ?)","!doc":"<p>Applies the z-index to the renderer based on the state of the item.\nThis methods is setting a z-index of 20 is the item is selected or edited \nand the current lane value computed by the overlap layout (i.e. the renderers \nare stacked according to their lane).</p>\n"},"computeOverlapping":{"!type":"fn(layoutItems: ?, func: ?) -> ?","!doc":"<p>Computes the overlap layout of a list of items. A lane and extent properties are added to each layout item.</p>\n"},"computeProjectionOnDate":{"!type":"fn(renderData: ?, refDate: ?, date: ?, max: ?) -> number","!doc":"<p>Computes the time to pixel projection in a day.</p>\n"},"computeRangeOverlap":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> [+Date]","!doc":"<p>Computes the overlap time range of the time ranges.\nReturns a vector of Date with at index 0 the start time and at index 1 the end time.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"createItemFunc":{"!type":"fn(view: ?, d: ?, e: ?)","!doc":"<p>A user supplied function that creates a new event.</p>\n"},"defaultStyleGridCell":{"!type":"fn(node: ?, date: ?, col: ?, row: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a column.\nBy default this method is setting the following CSS classes:\n- &quot;dojoxCalendarToday&quot; class name if the date displayed is the current date,\n- &quot;dojoxCalendarWeekend&quot; if the date represents a weekend,\n- the CSS class corresponding of the displayed day of week (&quot;Sun&quot;, &quot;Mon&quot; and so on),</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"ensureMinimalDuration":{"!type":"fn(renderData: ?, item: ?, unit: ?, steps: ?, editKind: ?)","!doc":"<p>During the resize editing gesture, ensures that the item has the specified minimal duration.</p>\n"},"ensureVisibility":{"!type":"fn(start: ?, end: ?, visibilityTarget: ?, margin: ?, duration: ?)","!doc":"<p>Scrolls the view if the [start, end] time range is not visible or only partially visible.</p>\n"},"floorDate":{"!type":"fn(date: ?, unit: ?, steps: ?, reuse: ?) -> +Date","!doc":"<p>floors the date to the unit.</p>\n"},"floorToDay":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of day.</p>\n"},"floorToMonth":{"!type":"fn(date: ?, reuse: ?) -> +Date","!doc":"<p>Floors the specified date to the start of the date&#39;s month.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getCSSPrefix":{"!type":"fn() -> string","!doc":"<p>Utility method that return the specific CSS prefix\nfor non standard CSS properties. Ex: -moz-border-radius.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getItemStoreState":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the creation state of an item. \nThis state is changing during the interactive creation of an item.\nValid values are:\n- &quot;unstored&quot;: The event is being interactively created. It is not in the store yet.\n- &quot;storing&quot;: The creation gesture has ended, the event is being added to the store.\n- &quot;stored&quot;: The event is not in the two previous states, and is assumed to be in the store \n(not checking because of performance reasons, use store API for testing existence in store).\nitem: Object\nThe item.\nreturns: String</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRenderers":{"!type":"fn(item: ?) -> [+Object]","!doc":"<p>Returns the renderers that are currently used to displayed the speficied item.\nReturns an array of objects that contains two properties:\n- container: The DOM node that contains the renderer.\n- renderer: The dojox.calendar._RendererMixin instance.\nDo not keep references on the renderers are they are recycled and reused for other items.</p>\n"},"getTime":{"!type":"fn(e: ?, x: ?, y: ?, touchIndex: ?) -> +Date","!doc":"<p>Returns the time displayed at the specified point by this component.</p>\n"},"getWeekNumberLabel":{"!type":"fn(date: ?) -> ?","!doc":"<p>Returns the week number string from dojo.date.locale.format() method as\ndojox.date.XXXX calendar are not supporting the &quot;w&quot; pattern.</p>\n"},"invalidateLayout":{"!type":"fn()"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isAscendantHasClass":{"!type":"fn(node: ?, ancestor: ?, className: ?) -> bool","!doc":"<p>Determines if a node has an ascendant node that has the css class specified.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemBeingEdited":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether an item is being edited or not.</p>\n"},"isItemEditable":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be edited or not.\nBy default it is using the editable property value.</p>\n"},"isItemFocused":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is focused or not.</p>\n"},"isItemHovered":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether the specified item is hovered or not.</p>\n"},"isItemMoveEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be moved.\nBy default it is using the moveEnabled property value.</p>\n"},"isItemResizeEnabled":{"!type":"fn(item: ?, rendererKind: ?) -> bool","!doc":"<p>Computes whether particular item renderer can be resized.\nBy default it is using the resizedEnabled property value.</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> ?"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isOverlapping":{"!type":"fn(renderData: ?, start1: ?, end1: ?, start2: ?, end2: ?, includeLimits: ?) -> bool","!doc":"<p>Computes if the first time range defined by the start1 and end1 parameters \nis overlapping the second time range defined by the start2 and end2 parameters.</p>\n"},"isSameDay":{"!type":"fn(date1: ?, date2: ?) -> bool","!doc":"<p>Tests if the specified dates are in the same day.</p>\n"},"isStartOfDay":{"!type":"fn(d: ?) -> bool","!doc":"<p>Tests if the specified date represents the starts of day. </p>\n"},"isToday":{"!type":"fn(date: ?) -> bool","!doc":"<p>Returns whether the specified date is in the current day.</p>\n"},"isWeekEnd":{"!type":"fn(date: ?) -> ?","!doc":"<p>Determines whether the specified date is a week-end.\nThis method is using dojo.date.locale.isWeekend() method as\ndojox.date.XXXX calendars are not supporting this method.</p>\n"},"itemToRenderItem":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>Creates the render item based on the dojo.store item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object,\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the store id, the summaryAttr, \n\n</p>\n<p>startTimeAttr and endTimeAttr properties as well as decodeDate property if not null. \nOther fields or way to query fields can be used if needed.</p>\n"},"newDate":{"!type":"fn(obj: ?) -> +Date","!doc":"<p>Creates a new Date object.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"postscript":{"!type":"fn()"},"refreshRendering":{"!type":"fn()"},"renderItemToItem":{"!type":"fn(renderItem: ?, store: ?) -> ?","!doc":"<p>Create a store item based on the render item. It must be of the form:\n\n</p>\n<pre><code>{\n    id: Object\n    startTime: Date,\n    endTime: Date,\n    summary: String\n}</code></pre>\n<p>By default it is building an object using the summaryAttr, startTimeAttr and endTimeAttr properties\n\n</p>\n<p>and encodeDate property if not null. If the encodeDate property is null a Date object will be set in the start and end time.\nWhen using a JsonRest store, for example, it is recommended to transfer dates using the ISO format (see dojo.date.stamp).\nIn that case, provide a custom function to the encodeDate property that is using the date ISO encoding provided by Dojo. </p>\n"},"resize":{"!type":"fn(changeSize: ?)","!doc":"<p>Function to call when the view is resized. \nIf the view is in a Dijit container or in a Dojo mobile container, it will be automatically called.\nOn other use cases, this method must called when the window is resized and/or when the orientation has changed.</p>\n"},"scrollView":{"!type":"fn(dir: ?)","!doc":"<p>Scrolls the view to the specified direction of one time slot duration.</p>\n"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleColumnHeaderCell":{"!type":"fn(node: ?, date: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a column header cell.\nBy default this method is does nothing and is designed to be overridden.</p>\n"},"styleGridCell":{"!type":"fn(node: ?, date: ?, col: ?, row: ?, renderData: ?)","!doc":"<p>Styles the CSS classes to the node that displays a column.\nDelegates to styleGridCellFunc if defined or defaultStyleGridCell otherwise.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(obj: ?, stateOnly: ?)","!doc":"<p>Updates all the renderers that represents the specified item(s).</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/charting/BidiSupport":{},"dojox/charting/BidiSupport3D":{},"dojox/charting/Chart2D":{"!type":"fn()","prototype":{"!doc":"<p>The main chart object in dojox.charting.  This will create a two dimensional\nchart based on dojox.gfx.\n</p>\n","addAxis":{"!type":"fn(name: ?, kwArgs?: ?) -> +dojox","!doc":"<p>Add an axis to the chart, for rendering.</p>\n"},"addPlot":{"!type":"fn(name: ?, kwArgs: ?) -> +dojox","!doc":"<p>Add a new plot to the chart, defined by name and using the optional keyword arguments object.\nNote that dojox.charting assumes the main plot to be called &quot;default&quot;; if you do not have\na plot called &quot;default&quot; and attempt to add data series to the chart without specifying the\nplot to be rendered on, you WILL get errors.</p>\n"},"addSeries":{"!type":"fn(name: ?, data: ?, kwArgs?: ?) -> +dojox","!doc":"<p>Add a data series to the chart for rendering.</p>\n"},"calculateGeometry":{"!type":"fn() -> +dojox","!doc":"<p>Calculate the geometry of the chart based on the defined axes of\na chart.</p>\n"},"connectToPlot":{"!type":"fn(name: ?, object: ?, method: ?) -> []","!doc":"<p>A convenience method to connect a function to a plot.</p>\n"},"constructor":{"!type":"fn(node: ?, kwArgs?: ?) -> +dojox","!doc":"<p>The constructor for a new Chart.  Initializes all parameters used for a chart.</p>\n"},"delayedRender":{"!type":"fn() -> +dojox","!doc":"<p>Delayed render, which is used to collect multiple updates\nwithin a delayInMs time window.</p>\n"},"destroy":{"!type":"fn() -> +void","!doc":"<p>Cleanup when a chart is to be destroyed.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?) -> +dojox","!doc":"<p>Fires a synthetic event for a series item.</p>\n"},"formatTruncatedLabel":{"!type":"fn(element: ?, label: ?, labelType: ?)"},"fullGeometry":{"!type":"fn() -> +dojox","!doc":"<p>Calculate the full geometry of the chart.  This includes passing\nover all major elements of a chart (plots, axes, series, container)\nin order to ensure proper rendering.</p>\n"},"fullRender":{"!type":"fn() -> +dojox","!doc":"<p>Force a full rendering of the chart, including full resets on the chart itself.\nYou should not call this method directly unless absolutely necessary.</p>\n"},"getAxis":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Get the given axis, by name.</p>\n"},"getCoords":{"!type":"fn() -> ?","!doc":"<p>Get the coordinates and dimensions of the containing DOMNode, as\nreturned by dojo.coords.</p>\n"},"getGeometry":{"!type":"fn() -> ?","!doc":"<p>Returns a map of information about all axes in a chart and what they represent\nin terms of scaling (see dojox.charting.axis2d.Default.getScaler).</p>\n"},"getPlot":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Get the given plot, by name.</p>\n"},"getPlotOrder":{"!type":"fn() -> []","!doc":"<p>Returns an array of plot names in the current order\n(the top-most plot is the first).</p>\n"},"getSeries":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Get the given series, by name.</p>\n"},"getSeriesOrder":{"!type":"fn(plotName: ?) -> []","!doc":"<p>Returns an array of series names in the current order\n(the top-most series is the first) within a plot.</p>\n"},"movePlotToBack":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given plot to back.</p>\n"},"movePlotToFront":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given plot to front.</p>\n"},"moveSeriesToBack":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given series to back of a plot.</p>\n"},"moveSeriesToFront":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given series to front of a plot.</p>\n"},"removeAxis":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Remove the axis that was defined using name.</p>\n"},"removePlot":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Remove the plot defined using name from the chart&#39;s plot stack.</p>\n"},"removeSeries":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Remove the series defined by name from the chart.</p>\n"},"render":{"!type":"fn() -> +dojox","!doc":"<p>Render the chart according to the current information defined.  This should\nbe the last call made when defining/creating a chart, or if data within the\nchart has been changed.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the chart to the dimensions of width and height.</p>\n"},"setAxisWindow":{"!type":"fn(name: ?, scale: ?, offset: ?, zoom?: ?) -> +dojox","!doc":"<p>Zooms an axis and all dependent plots. Can be used to zoom in 1D.</p>\n"},"setDir":{"!type":"fn(dir: ?) -> fn"},"setPlotOrder":{"!type":"fn(newOrder: ?) -> +dojox","!doc":"<p>Sets new order of plots. newOrder cannot add or remove\nplots. Wrong names, or dups are ignored.</p>\n"},"setSeriesOrder":{"!type":"fn(newOrder: ?) -> +dojox","!doc":"<p>Sets new order of series within a plot. newOrder cannot add\nor remove series. Wrong names, or dups are ignored.</p>\n"},"setTheme":{"!type":"fn(theme: ?) -> +dojox","!doc":"<p>Set a theme of the chart.</p>\n"},"setWindow":{"!type":"fn(sx: ?, sy: ?, dx: ?, dy: ?, zoom?: ?) -> +dojox","!doc":"<p>Zooms in or out any plots in two dimensions.</p>\n"},"updateSeries":{"!type":"fn(name: ?, data: ?, offsets?: ?) -> +dojox","!doc":"<p>Update the given series with a new set of data points.</p>\n"},"zoomIn":{"!type":"fn(name: ?, range: ?, delayed: ?)","!doc":"<p>Zoom the chart to a specific range on one axis.  This calls render()\ndirectly as a convenience method.</p>\n"}}},"dojox/charting/plot2d/Areas":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Default.prototype","!doc":"<p>Represents an area chart.  See dojox/charting/plot2d/Default for details.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{"!doc":"<p>The default parameters of this plot.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{"!doc":"<p>The optional parameters of this plot.</p>\n"},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"buildSegments":{"!type":"fn(i: ?, indexed: ?) -> []"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The constructor for an Area chart.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createPath":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render/draw everything on this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Markers":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Default.prototype","!doc":"<p>A convenience plot to draw a line chart with markers.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{"!doc":"<p>The default parameters of this plot.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{"!doc":"<p>The optional parameters of this plot.</p>\n"},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"buildSegments":{"!type":"fn(i: ?, indexed: ?) -> []"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Set up the plot for lines and markers.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createPath":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render/draw everything on this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/MarkersOnly":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Default.prototype","!doc":"<p>A convenience object to draw only markers (like a scatter but not quite).</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{"!doc":"<p>The default parameters of this plot.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{"!doc":"<p>The optional parameters of this plot.</p>\n"},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"buildSegments":{"!type":"fn(i: ?, indexed: ?) -> []"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Set up our default plot to only have markers and no lines.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createPath":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render/draw everything on this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Scatter":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/CartesianBase.prototype","!doc":"<p>A plot object representing a typical scatter chart.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Create the scatter plot.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Stacked":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Default.prototype","!doc":"<p>Like the default plot, Stacked sets up lines, areas and markers\nin a stacked fashion (values on the y axis added to each other)\nas opposed to a direct one.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{"!doc":"<p>The default parameters of this plot.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{"!doc":"<p>The optional parameters of this plot.</p>\n"},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"buildSegments":{"!type":"fn(i: ?, indexed: ?) -> []"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Return a new plot.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createPath":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render/draw everything on this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/commonStacked":{"collectStats":{"!type":"fn(series: ?) -> ?"},"getIndexValue":{"!type":"fn(series: ?, i: ?, index: ?) -> []"},"getValue":{"!type":"fn(series: ?, i: ?, x: ?) -> []"}},"dojox/charting/plot2d/StackedLines":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Stacked.prototype","!doc":"<p>A convenience object to create a stacked line chart.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{"!doc":"<p>The default parameters of this plot.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{"!doc":"<p>The optional parameters of this plot.</p>\n"},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"buildSegments":{"!type":"fn(i: ?, indexed: ?) -> []"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Force our Stacked base to be lines only.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createPath":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render/draw everything on this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/StackedAreas":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Stacked.prototype","!doc":"<p>A convenience object to set up a stacked area plot.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{"!doc":"<p>The default parameters of this plot.</p>\n"},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{"!doc":"<p>The optional parameters of this plot.</p>\n"},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"buildSegments":{"!type":"fn(i: ?, indexed: ?) -> []"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Force our Stacked plotter to include both lines and areas.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createPath":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render/draw everything on this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Columns":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/CartesianBase.prototype","!doc":"<p>The plot object representing a column chart (vertical bars).</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for a columns chart.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createRect":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getBarProperties":{"!type":"fn() -> ?"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"getValue":{"!type":"fn(value: ?, j: ?, seriesIndex: ?, indexed: ?) -> ?"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/StackedColumns":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Columns.prototype","!doc":"<p>The plot object representing a stacked column chart (vertical bars).</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for a columns chart.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createRect":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getBarProperties":{"!type":"fn() -> ?"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"getValue":{"!type":"fn(value: ?, index: ?, seriesIndex: ?, indexed: ?) -> ?"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/ClusteredColumns":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Columns.prototype","!doc":"<p>A plot representing grouped or clustered columns (vertical bars)</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for a columns chart.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createRect":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getBarProperties":{"!type":"fn() -> ?"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"getValue":{"!type":"fn(value: ?, j: ?, seriesIndex: ?, indexed: ?) -> ?"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Bars":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/CartesianBase.prototype","!doc":"<p>The plot object representing a bar chart (horizontal bars).</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for a bar chart.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createRect":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getBarProperties":{"!type":"fn() -> ?"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"getValue":{"!type":"fn(value: ?, j: ?, seriesIndex: ?, indexed: ?) -> ?"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/StackedBars":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Bars.prototype","!doc":"<p>The plot object representing a stacked bar chart (horizontal bars).</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for a bar chart.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createRect":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getBarProperties":{"!type":"fn() -> ?"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"getValue":{"!type":"fn(value: ?, index: ?, seriesIndex: ?, indexed: ?) -> ?"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/ClusteredBars":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Bars.prototype","!doc":"<p>A plot representing grouped or clustered bars (horizontal bars)</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for a bar chart.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createRect":{"!type":"fn(run: ?, creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getBarProperties":{"!type":"fn() -> ?"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"getValue":{"!type":"fn(value: ?, j: ?, seriesIndex: ?, indexed: ?) -> ?"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Grid":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/CartesianBase.prototype","!doc":"<p>A &quot;faux&quot; plot that can be placed behind other plots to represent\na grid against which other plots can be easily measured.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Ignored but included as a dummy method.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn()"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Create the faux Grid plot.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"createLine":{"!type":"fn(creator: ?, params: ?) -> ?"},"createRect":{"!type":"fn(creator: ?, params: ?) -> ?"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Returns default stats (irrelevant for this type of plot).</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"purgeGroup":{"!type":"fn()"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render the plot on the chart.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Pie":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Base.prototype","!doc":"<p>The plot that represents a typical pie chart.</p>\n","chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a series of data to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator?: ?) -> +dojox","!doc":"<p>Clean any elements (HTML or GFX-based) out of our group, and create a new one.</p>\n"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs: ?)","!doc":"<p>Create a pie plot.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Return the number of colors needed to draw this plot.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Returns default stats (irrelevant for this type of plot).</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn() -> fn","!doc":"<p>Does nothing.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn() -> +dojox","!doc":"<p>Clear any elements out of our group, and destroy the group.</p>\n"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render the plot on the chart.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Dummy method, since axes are irrelevant with a Pie chart.</p>\n"}}},"dojox/charting/plot2d/Bubble":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/CartesianBase.prototype","!doc":"<p>A plot representing bubbles.  Note that data for Bubbles requires 3 parameters,\nin the form of:  { x, y, size }, where size determines the size of the bubble.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Create a plot of bubbles.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Candlesticks":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/CartesianBase.prototype","!doc":"<p>A plot that represents typical candlesticks (financial reporting, primarily).\nUnlike most charts, the Candlestick expects data points to be represented by\nan object of the form { x?, open, close, high, low, mid? }, where both\nx and mid are optional parameters.  If x is not provided, the index of the\ndata array is used.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"collectStats":{"!type":"fn(series: ?) -> ?","!doc":"<p>Collect all statistics for drawing this chart.  Since the common\nfunctionality only assumes x and y, Candlesticks must create it&#39;s own\nstats (since data has no y value, but open/close/high/low instead).</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for a candlestick chart.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"tooltipFunc":{"!type":"fn(o: ?) -> string"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/OHLC":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/CartesianBase.prototype","!doc":"<p>A plot that represents typical open/high/low/close (financial reporting, primarily).\nUnlike most charts, the Candlestick expects data points to be represented by\nan object of the form { x?, open, close, high, low, mid? }, where both\nx and mid are optional parameters.  If x is not provided, the index of the\ndata array is used.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"collectStats":{"!type":"fn(series: ?) -> ?","!doc":"<p>Collect all statistics for drawing this chart.  Since the common\nfunctionality only assumes x and y, OHLC must create it&#39;s own\nstats (since data has no y value, but open/close/high/low instead).</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>The constructor for a candlestick chart.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Run the calculations for any axes for this plot.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Spider":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Base.prototype","!doc":"<p>The plot that represents a typical Spider chart.</p>\n","chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?) -> +dojox","!doc":"<p>Add a data series to this plot.</p>\n"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator?: ?) -> +dojox","!doc":"<p>Clean any elements (HTML or GFX-based) out of our group, and create a new one.</p>\n"},"clear":{"!type":"fn() -> +dojox","!doc":"<p>Clear out all of the information tied to this plot.</p>\n"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Create a Spider plot.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Calculate the min/max on all attached series in both directions.</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn() -> fn","!doc":"<p>Does nothing.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn() -> +dojox","!doc":"<p>Clear any elements out of our group, and destroy the group.</p>\n"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Render the plot on the chart.</p>\n"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Optionally set axis min and max property.</p>\n"},"tooltipFunc":{"!type":"fn(o: ?) -> string"}}},"dojox/charting/Chart3D":{"!type":"fn()","prototype":{"addPlot":{"!type":"fn(plot: ?) -> ?"},"addWall":{"!type":"fn(wall: ?) -> ?"},"constructor":{"!type":"fn(node: ?, lights: ?, camera: ?, theme: ?)"},"generate":{"!type":"fn() -> ?"},"invalidate":{"!type":"fn() -> fn"},"removePlot":{"!type":"fn(plot: ?) -> ?"},"removeWall":{"!type":"fn(wall: ?) -> ?"},"render":{"!type":"fn() -> fn"},"setDir":{"!type":"fn(dir: ?) -> fn"}}},"dojox/gfx3d":{"!doc":"<p>Deprecated.  Should require dojox/gfx3d modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/gfx3d/matrix":{"identity":{"!doc":"<p>an identity matrix constant: identity * (x, y, z) == (x, y, z)</p>\n"},"cameraRotateX":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the x axis) in cameraTransform manner</p>\n"},"cameraRotateXg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the x axis)in cameraTransform manner</p>\n"},"cameraRotateY":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the y axis) in cameraTransform manner</p>\n"},"cameraRotateYg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the y axis) in cameraTransform manner</p>\n"},"cameraRotateZ":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the z axis) in cameraTransform manner</p>\n"},"cameraRotateZg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the z axis) in cameraTransform manner</p>\n"},"cameraTranslate":{"!type":"fn(a: ?, b?: ?, c?: ?) -> +instance","!doc":"<p>forms a translation matrix</p>\n"},"clone":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>creates a copy of a 3D matrix</p>\n"},"invert":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>inverts a 2D matrix</p>\n"},"Matrix3D":{"!type":"fn(arg: ?)","!doc":"<p>a 3D matrix object</p>\n"},"multiply":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>combines matrices by multiplying them sequentially in the given order</p>\n"},"multiplyPoint":{"!type":"fn(matrix: ?, a: ?, b?: ?, c?: ?) -> ?","!doc":"<p>applies a matrix to a point</p>\n"},"normalize":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>converts an object to a matrix, if necessary</p>\n"},"project":{"!type":"fn(matrix: ?, a: ?, b?: ?, c?: ?) -> ?","!doc":"<p>applies a matrix to a point</p>\n"},"rotateX":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the x axis)</p>\n"},"rotateXg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the x axis)</p>\n"},"rotateY":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the y axis)</p>\n"},"rotateYg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the y axis)</p>\n"},"rotateZ":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the z axis)</p>\n"},"rotateZg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the z axis)</p>\n"},"scale":{"!type":"fn(a: ?, b?: ?, c?: ?) -> +instance","!doc":"<p>forms a scaling matrix</p>\n"},"translate":{"!type":"fn(a: ?, b?: ?, c?: ?) -> +instance","!doc":"<p>forms a translation matrix</p>\n"}},"dojox/gfx3d/_base":{"_creators":{"!doc":"<p>object creators</p>\n"},"defaultCube":{},"defaultCylinder":{},"defaultEdges":{},"defaultOrbit":{},"defaultPath3d":{},"defaultPolygon":{},"defaultQuads":{},"defaultTriangles":{},"drawer":{},"lighting":{},"matrix":{},"scheduler":{},"vector":{},"Cube":{"!type":"fn()"},"Cylinder":{"!type":"fn()"},"Edges":{"!type":"fn()"},"gradient":{"!type":"fn(model: ?, material: ?, center: ?, radius: ?, from: ?, to: ?, matrix: ?) -> ?","!doc":"<p>calculate a cylindrical gradient</p>\n"},"Matrix3D":{"!type":"fn(arg: ?)","!doc":"<p>a 3D matrix object</p>\n"},"Object":{"!type":"fn()"},"Orbit":{"!type":"fn()"},"Path3d":{"!type":"fn()"},"Polygon":{"!type":"fn()"},"Quads":{"!type":"fn()"},"Scene":{"!type":"fn()"},"Triangles":{"!type":"fn()"},"Viewport":{"!type":"fn()"}},"dojox/gfx3d/_base.defaultEdges":{"points":{},"style":{},"type":{}},"dojox/gfx3d/_base.defaultTriangles":{"points":{},"style":{},"type":{}},"dojox/gfx3d/_base.defaultQuads":{"points":{},"style":{},"type":{}},"dojox/gfx3d/_base.defaultOrbit":{"center":{},"radius":{},"type":{}},"dojox/gfx3d/_base.defaultPath3d":{"path":{},"type":{}},"dojox/gfx3d/_base.defaultPolygon":{"path":{},"type":{}},"dojox/gfx3d/_base.defaultCube":{"bottom":{},"top":{},"type":{}},"dojox/gfx3d/_base.defaultCylinder":{"center":{},"height":{},"radius":{},"type":{}},"dojox/gfx3d/_base.matrix":{"identity":{"!doc":"<p>an identity matrix constant: identity * (x, y, z) == (x, y, z)</p>\n"},"cameraRotateX":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the x axis) in cameraTransform manner</p>\n"},"cameraRotateXg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the x axis)in cameraTransform manner</p>\n"},"cameraRotateY":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the y axis) in cameraTransform manner</p>\n"},"cameraRotateYg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the y axis) in cameraTransform manner</p>\n"},"cameraRotateZ":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the z axis) in cameraTransform manner</p>\n"},"cameraRotateZg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the z axis) in cameraTransform manner</p>\n"},"cameraTranslate":{"!type":"fn(a: ?, b?: ?, c?: ?) -> +instance","!doc":"<p>forms a translation matrix</p>\n"},"clone":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>creates a copy of a 3D matrix</p>\n"},"invert":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>inverts a 2D matrix</p>\n"},"Matrix3D":{"!type":"fn(arg: ?)","!doc":"<p>a 3D matrix object</p>\n"},"multiply":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>combines matrices by multiplying them sequentially in the given order</p>\n"},"multiplyPoint":{"!type":"fn(matrix: ?, a: ?, b?: ?, c?: ?) -> ?","!doc":"<p>applies a matrix to a point</p>\n"},"normalize":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>converts an object to a matrix, if necessary</p>\n"},"project":{"!type":"fn(matrix: ?, a: ?, b?: ?, c?: ?) -> ?","!doc":"<p>applies a matrix to a point</p>\n"},"rotateX":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the x axis)</p>\n"},"rotateXg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the x axis)</p>\n"},"rotateY":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the y axis)</p>\n"},"rotateYg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the y axis)</p>\n"},"rotateZ":{"!type":"fn(angle: ?) -> +instance","!doc":"<p>forms a rotating matrix (about the z axis)</p>\n"},"rotateZg":{"!type":"fn(degree: ?) -> ?","!doc":"<p>forms a rotating matrix (about the z axis)</p>\n"},"scale":{"!type":"fn(a: ?, b?: ?, c?: ?) -> +instance","!doc":"<p>forms a scaling matrix</p>\n"},"translate":{"!type":"fn(a: ?, b?: ?, c?: ?) -> +instance","!doc":"<p>forms a translation matrix</p>\n"}},"dojox/gfx3d/_base.vector":{"center":{"!type":"fn() -> ?","!doc":"<p>center of the vectors</p>\n"},"crossProduct":{"!type":"fn(a: ?, b: ?, c: ?, d: ?, e: ?, f: ?) -> ?","!doc":"<p>applies a matrix to a point</p>\n"},"dotProduct":{"!type":"fn(a: ?, b: ?, c?: ?, d?: ?, e?: ?, f?: ?) -> ?","!doc":"<p>applies a matrix to a point</p>\n"},"normalize":{"!type":"fn(a: ?, b: ?, c: ?) -> ?","!doc":"<p>find the normal of the implicit surface</p>\n"},"substract":{"!type":"fn(a: ?, b: ?) -> ?"},"sum":{"!type":"fn() -> ?","!doc":"<p>sum of the vectors</p>\n"}},"dojox/gfx3d/_base.scheduler":{"BinarySearchTree":{"!type":"fn()"},"bsp":{"!type":"fn(buffer: ?, outline: ?) -> ?"},"order":{"!type":"fn(it: ?) -> ?"},"outline":{"!type":"fn(it: ?) -> ?"},"zOrder":{"!type":"fn(buffer: ?, order: ?) -> ?"}},"dojox/gfx3d/_base.drawer":{"chart":{"!type":"fn(todos: ?, objects: ?, viewport: ?)"},"conservative":{"!type":"fn(todos: ?, objects: ?, viewport: ?)"}},"dojox/gfx3d/_base.lighting":{"finish":{},"add":{"!type":"fn(a: ?, b: ?) -> ?"},"addColor":{"!type":"fn(a: ?, b: ?) -> ?"},"black":{"!type":"fn() -> ?"},"diff2Color":{"!type":"fn(c1: ?, c2: ?) -> number"},"diffuse":{"!type":"fn(normal: ?, lights: ?) -> ?"},"dot":{"!type":"fn(a: ?, b: ?) -> number"},"faceforward":{"!type":"fn(n: ?, i: ?) -> ?"},"fromStdColor":{"!type":"fn(c: ?) -> +instance"},"length":{"!type":"fn(v: ?) -> ?"},"length2Color":{"!type":"fn(c: ?) -> number"},"mixColor":{"!type":"fn(c1: ?, c2: ?, s: ?) -> ?"},"Model":{"!type":"fn()"},"multiplyColor":{"!type":"fn(a: ?, b: ?) -> ?"},"normalize":{"!type":"fn(v: ?) -> ?"},"phong":{"!type":"fn(normal: ?, v: ?, size: ?, lights: ?) -> ?"},"reflect":{"!type":"fn(i: ?, n: ?) -> ?"},"saturate":{"!type":"fn(v: ?) -> ?"},"saturateColor":{"!type":"fn(c: ?) -> ?"},"scale":{"!type":"fn(s: ?, v: ?) -> ?"},"scaleColor":{"!type":"fn(s: ?, c: ?) -> ?"},"specular":{"!type":"fn(normal: ?, v: ?, roughness: ?, lights: ?) -> ?"},"toStdColor":{"!type":"fn(c: ?) -> ?"},"white":{"!type":"fn() -> ?"}},"dojox/gfx3d/_base.Object":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a Object object, which knows how to map\n3D objects to 2D shapes.</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(lighting: ?)"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?) -> fn","!doc":"<p>sets a Object object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Scene":{"!type":"fn()","prototype":{"!doc":"<p>the Scene is just a container.</p>\n","abandon":{"!type":"fn(obj: ?, silently?: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"addTodo":{"!type":"fn(newObject: ?)"},"adopt":{"!type":"fn(obj: ?) -> fn","!doc":"<p>adds a shape to the list</p>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a container of other 3D objects</p>\n"},"create3DObject":{"!type":"fn(objectType: ?, rawObject: ?, style: ?) -> +instance","!doc":"<p>creates an instance of the passed objectType class</p>\n"},"createCube":{"!type":"fn(cube: ?) -> ?","!doc":"<p>creates an cube object</p>\n"},"createCylinder":{"!type":"fn(cylinder: ?) -> ?","!doc":"<p>creates an cylinder object</p>\n"},"createEdges":{"!type":"fn(edges: ?, style: ?) -> ?","!doc":"<p>creates an edge object</p>\n"},"createOrbit":{"!type":"fn(orbit: ?) -> ?","!doc":"<p>creates an Orbit object</p>\n"},"createPath3d":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates an 3d path object</p>\n"},"createPolygon":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates an polygon object</p>\n"},"createQuads":{"!type":"fn(quads: ?, style: ?) -> ?","!doc":"<p>creates an quads object</p>\n"},"createScene":{"!type":"fn() -> ?","!doc":"<p>creates a scene object</p>\n"},"createTriangles":{"!type":"fn(tris: ?, style: ?) -> ?","!doc":"<p>creates an triangle object</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(lighting: ?)"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?, deep: ?)"},"setDrawer":{"!type":"fn(drawer: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn"},"setObject":{"!type":"fn(newObject: ?) -> fn","!doc":"<p>sets a Object object</p>\n"},"setScheduler":{"!type":"fn(scheduler: ?)"},"setStroke":{"!type":"fn(stroke: ?) -> fn"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Edges":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a generic edge in 3D viewport</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn()"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?, style?: ?) -> fn","!doc":"<p>setup the object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Orbit":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a generic edge in 3D viewport</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(lighting: ?)"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?) -> fn","!doc":"<p>sets a Object object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Path3d":{"!type":"fn()","prototype":{"_validSegments":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a generic line</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(lighting: ?)"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?) -> fn","!doc":"<p>sets a Object object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Triangles":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a generic triangle</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(lighting: ?)"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?, style?: ?) -> fn","!doc":"<p>setup the object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Quads":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a generic quad</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(lighting: ?)"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?, style?: ?) -> fn","!doc":"<p>setup the object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Polygon":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a generic polygon</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(lighting: ?)"},"getOutline":{"!type":"fn() -> ?"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?) -> fn","!doc":"<p>setup the object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Cube":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a generic cube</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(lighting: ?)"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?)","!doc":"<p>setup the object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Cylinder":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn()"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?) -> fn","!doc":"<p>sets a Object object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/_base.Viewport":{"!type":"fn()","prototype":{"abandon":{"!type":"fn(obj: ?, silently?: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"add":{"!type":"fn(shape: ?) -> ?","!doc":"<p>adds a shape to the list</p>\n"},"addLights":{"!type":"fn(lights: ?) -> ?","!doc":"<p>add new light/lights to the viewport.</p>\n"},"addTodo":{"!type":"fn(newObject: ?)"},"adopt":{"!type":"fn(obj: ?) -> fn","!doc":"<p>adds a shape to the list</p>\n"},"applyCameraLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyCameraRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyCameraTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>a shortcut for dojox.gfx3d.Object.applyRightTransform</p>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn(destroy?: ?) -> fn","!doc":"<p>removes all shapes from a group/surface.</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a viewport/container for 3D objects, which knows\nthe camera and lightings</p>\n"},"create3DObject":{"!type":"fn(objectType: ?, rawObject: ?, style: ?) -> +instance","!doc":"<p>creates an instance of the passed objectType class</p>\n"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createCube":{"!type":"fn(cube: ?) -> ?","!doc":"<p>creates an cube object</p>\n"},"createCylinder":{"!type":"fn(cylinder: ?) -> ?","!doc":"<p>creates an cylinder object</p>\n"},"createEdges":{"!type":"fn(edges: ?, style: ?) -> ?","!doc":"<p>creates an edge object</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createOrbit":{"!type":"fn(orbit: ?) -> ?","!doc":"<p>creates an Orbit object</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPath3d":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates an 3d path object</p>\n"},"createPolygon":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates an polygon object</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createQuads":{"!type":"fn(quads: ?, style: ?) -> ?","!doc":"<p>creates an quads object</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createScene":{"!type":"fn() -> ?","!doc":"<p>creates a scene object</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTriangles":{"!type":"fn(tris: ?, style: ?) -> ?","!doc":"<p>creates an triangle object</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"invalidate":{"!type":"fn()"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"render":{"!type":"fn()","!doc":"<p>iterate all children and call their render callback function.</p>\n"},"setCameraTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setDimensions":{"!type":"fn(dim: ?)"},"setDrawer":{"!type":"fn(drawer: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setLights":{"!type":"fn(lights: ?, ambient?: ?, specular?: ?) -> fn","!doc":"<p>set the lights</p>\n"},"setScheduler":{"!type":"fn(scheduler: ?)"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx3d/_base._creators":{"!doc":"<p>object creators</p>\n","abandon":{"!type":"fn(obj: ?, silently?: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"adopt":{"!type":"fn(obj: ?) -> fn","!doc":"<p>adds a shape to the list</p>\n"},"create3DObject":{"!type":"fn(objectType: ?, rawObject: ?, style: ?) -> +instance","!doc":"<p>creates an instance of the passed objectType class</p>\n"},"createCube":{"!type":"fn(cube: ?) -> ?","!doc":"<p>creates an cube object</p>\n"},"createCylinder":{"!type":"fn(cylinder: ?) -> ?","!doc":"<p>creates an cylinder object</p>\n"},"createEdges":{"!type":"fn(edges: ?, style: ?) -> ?","!doc":"<p>creates an edge object</p>\n"},"createOrbit":{"!type":"fn(orbit: ?) -> ?","!doc":"<p>creates an Orbit object</p>\n"},"createPath3d":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates an 3d path object</p>\n"},"createPolygon":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates an polygon object</p>\n"},"createQuads":{"!type":"fn(quads: ?, style: ?) -> ?","!doc":"<p>creates an quads object</p>\n"},"createScene":{"!type":"fn() -> ?","!doc":"<p>creates a scene object</p>\n"},"createTriangles":{"!type":"fn(tris: ?, style: ?) -> ?","!doc":"<p>creates an triangle object</p>\n"},"setDrawer":{"!type":"fn(drawer: ?)"},"setScheduler":{"!type":"fn(scheduler: ?)"}},"dojox/gfx3d/object":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox.gfx.Shape.applyRightTransform</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>a Object object, which knows how to map\n3D objects to 2D shapes.</p>\n"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(lighting: ?)"},"getOutline":{"!type":"fn() -> +null"},"getZOrder":{"!type":"fn() -> number"},"invalidate":{"!type":"fn()"},"render":{"!type":"fn(camera: ?)"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation is to delegate to\nthe underlying 2D shape).</p>\n"},"setObject":{"!type":"fn(newObject: ?) -> fn","!doc":"<p>sets a Object object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>sets a transformation matrix</p>\n"},"toStdFill":{"!type":"fn(lighting: ?, normal: ?) -> ?"}}},"dojox/gfx3d/scheduler":{"drawer":{},"scheduler":{},"BinarySearchTree":{"!type":"fn()"}},"dojox/gfx3d/scheduler.scheduler":{"BinarySearchTree":{"!type":"fn()"},"bsp":{"!type":"fn(buffer: ?, outline: ?) -> ?"},"order":{"!type":"fn(it: ?) -> ?"},"outline":{"!type":"fn(it: ?) -> ?"},"zOrder":{"!type":"fn(buffer: ?, order: ?) -> ?"}},"dojox/gfx3d/scheduler.drawer":{"chart":{"!type":"fn(todos: ?, objects: ?, viewport: ?)"},"conservative":{"!type":"fn(todos: ?, objects: ?, viewport: ?)"}},"dojox/gfx3d/scheduler.BinarySearchTree":{"!type":"fn()","prototype":{"add":{"!type":"fn(obj: ?, outline: ?)"},"constructor":{"!type":"fn(obj: ?, outline: ?)","!doc":"<p>build the binary search tree, using binary space partition algorithm.\nThe idea is for any polygon, for example, (a, b, c), the space is divided by\nthe plane into two space: plus and minus.\n\n</p>\n<p>for any arbitrary vertex p, if(p - a) dotProduct n = 0, p is inside the plane,\n</p>\n<blockquote>\n<p>0, p is in the plus space, vice versa for minus space.\nn is the normal vector that is perpendicular the plate, defined as:\n\n</p>\n</blockquote>\n<pre><code>n = ( b - a) crossProduct ( c - a )</code></pre>\n<p>in this implementation, n is declared as normal, ,a is declared as orient.</p>\n"},"iterate":{"!type":"fn(outline: ?) -> ?"}}},"dojox/gfx3d/vector":{"center":{"!type":"fn() -> ?","!doc":"<p>center of the vectors</p>\n"},"crossProduct":{"!type":"fn(a: ?, b: ?, c: ?, d: ?, e: ?, f: ?) -> ?","!doc":"<p>applies a matrix to a point</p>\n"},"dotProduct":{"!type":"fn(a: ?, b: ?, c?: ?, d?: ?, e?: ?, f?: ?) -> ?","!doc":"<p>applies a matrix to a point</p>\n"},"normalize":{"!type":"fn(a: ?, b: ?, c: ?) -> ?","!doc":"<p>find the normal of the implicit surface</p>\n"},"substract":{"!type":"fn(a: ?, b: ?) -> ?"},"sum":{"!type":"fn() -> ?","!doc":"<p>sum of the vectors</p>\n"}},"dojox/gfx3d/gradient":{"!doc":"<p>calculate a cylindrical gradient</p>\n","prototype":{"tolerance":{"!doc":"<p>tolerable difference in colors between gradient steps</p>\n"},"constructor":{"!type":"fn(model: ?, material: ?, center: ?, radius: ?, from: ?, to: ?, matrix: ?) -> ?","!doc":"<p>calculate a cylindrical gradient</p>\n"}}},"dojox/gfx3d/lighting":{"finish":{},"add":{"!type":"fn(a: ?, b: ?) -> ?"},"addColor":{"!type":"fn(a: ?, b: ?) -> ?"},"black":{"!type":"fn() -> ?"},"diff2Color":{"!type":"fn(c1: ?, c2: ?) -> number"},"diffuse":{"!type":"fn(normal: ?, lights: ?) -> ?"},"dot":{"!type":"fn(a: ?, b: ?) -> number"},"faceforward":{"!type":"fn(n: ?, i: ?) -> ?"},"fromStdColor":{"!type":"fn(c: ?) -> +instance"},"length":{"!type":"fn(v: ?) -> ?"},"length2Color":{"!type":"fn(c: ?) -> number"},"mixColor":{"!type":"fn(c1: ?, c2: ?, s: ?) -> ?"},"Model":{"!type":"fn()"},"multiplyColor":{"!type":"fn(a: ?, b: ?) -> ?"},"normalize":{"!type":"fn(v: ?) -> ?"},"phong":{"!type":"fn(normal: ?, v: ?, size: ?, lights: ?) -> ?"},"reflect":{"!type":"fn(i: ?, n: ?) -> ?"},"saturate":{"!type":"fn(v: ?) -> ?"},"saturateColor":{"!type":"fn(c: ?) -> ?"},"scale":{"!type":"fn(s: ?, v: ?) -> ?"},"scaleColor":{"!type":"fn(s: ?, c: ?) -> ?"},"specular":{"!type":"fn(normal: ?, v: ?, roughness: ?, lights: ?) -> ?"},"toStdColor":{"!type":"fn(c: ?) -> ?"},"white":{"!type":"fn() -> ?"}},"dojox/gfx3d/lighting.Model":{"!type":"fn()","prototype":{"constant":{"!type":"fn(normal: ?, finish: ?, pigment: ?) -> ?"},"constructor":{"!type":"fn(incident: ?, lights: ?, ambient: ?, specular: ?)"},"matte":{"!type":"fn(normal: ?, finish: ?, pigment: ?) -> ?"},"metal":{"!type":"fn(normal: ?, finish: ?, pigment: ?) -> ?"},"npr":{"!type":"fn(normal: ?, finish: ?, pigment: ?) -> ?"},"plastic":{"!type":"fn(normal: ?, finish: ?, pigment: ?) -> ?"}}},"dojox/gfx3d/lighting.finish":{"defaults":{},"dull":{},"glossy":{},"luminous":{},"metalA":{},"metalB":{},"metalC":{},"metalD":{},"metalE":{},"phong_dull":{},"phong_glossy":{},"phong_shiny":{},"shiny":{}},"dojox/charting/DataChart":{"!type":"fn()","prototype":{"!proto":"dojox/charting/Chart2D.prototype","!doc":"<p>Extension to the 2D chart that connects to a data store in\na simple manner. Convenience methods have been added for\nconnecting store item labels to the chart labels.</p>\n","chartTheme":{"!doc":"<p>The theme to style the chart. Defaults to PlotKit.blue.</p>\n"},"comparative":{"!doc":"<p>If false, all items are each their own series.\nIf true, the items are combined into one series\nso that their charted properties can be compared.</p>\n"},"displayRange":{"!doc":"<p>The number of major ticks to show on the xaxis</p>\n"},"fieldName":{"!doc":"<p>The field in the store item that is getting charted</p>\n"},"label":{"!doc":"<p>The name field of the store item\nDO NOT SET: Set from store.labelAttribute</p>\n"},"minHeight":{"!doc":"<p>The the smallest the chart height can be</p>\n"},"minWidth":{"!doc":"<p>The the smallest the chart width can be</p>\n"},"query":{"!doc":"<p>Used for fetching items. Will vary depending upon store.</p>\n"},"queryOptions":{"!doc":"<p>Option used for fetching items</p>\n"},"scroll":{"!doc":"<p>Whether live data updates and changes display, like columns moving\nup and down, or whether it scrolls to the left as data is added</p>\n"},"showing":{"!doc":"<p>Whether the chart is showing (default) on\ninitialization or hidden.</p>\n"},"stretchToFit":{"!doc":"<p>If true, chart is sized to data. If false, chart is a\nfixed size. Note, is overridden by displayRange.\nTODO: Stretch for the y-axis?</p>\n"},"addAxis":{"!type":"fn(name: ?, kwArgs?: ?) -> +dojox","!doc":"<p>Add an axis to the chart, for rendering.</p>\n"},"addPlot":{"!type":"fn(name: ?, kwArgs: ?) -> +dojox","!doc":"<p>Add a new plot to the chart, defined by name and using the optional keyword arguments object.\nNote that dojox.charting assumes the main plot to be called &quot;default&quot;; if you do not have\na plot called &quot;default&quot; and attempt to add data series to the chart without specifying the\nplot to be rendered on, you WILL get errors.</p>\n"},"addSeries":{"!type":"fn(name: ?, data: ?, kwArgs?: ?) -> +dojox","!doc":"<p>Add a data series to the chart for rendering.</p>\n"},"calculateGeometry":{"!type":"fn() -> +dojox","!doc":"<p>Calculate the geometry of the chart based on the defined axes of\na chart.</p>\n"},"connectToPlot":{"!type":"fn(name: ?, object: ?, method: ?) -> []","!doc":"<p>A convenience method to connect a function to a plot.</p>\n"},"constructor":{"!type":"fn(node: ?, kwArgs: ?)","!doc":"<p>Set up properties and initialize chart build.</p>\n"},"convertLabels":{"!type":"fn(axis: ?) -> +null","!doc":"<p>Convenience method to convert a label array of strings\ninto an array of objects\n</p>\n"},"delayedRender":{"!type":"fn() -> +dojox","!doc":"<p>Delayed render, which is used to collect multiple updates\nwithin a delayInMs time window.</p>\n"},"destroy":{"!type":"fn()"},"fetch":{"!type":"fn()","!doc":"<p>Fetches initial data. Subsequent changes\nare received via onSet in data store.\n</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?) -> +dojox","!doc":"<p>Fires a synthetic event for a series item.</p>\n"},"formatTruncatedLabel":{"!type":"fn(element: ?, label: ?, labelType: ?)"},"fullGeometry":{"!type":"fn() -> +dojox","!doc":"<p>Calculate the full geometry of the chart.  This includes passing\nover all major elements of a chart (plots, axes, series, container)\nin order to ensure proper rendering.</p>\n"},"fullRender":{"!type":"fn() -> +dojox","!doc":"<p>Force a full rendering of the chart, including full resets on the chart itself.\nYou should not call this method directly unless absolutely necessary.</p>\n"},"getAxis":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Get the given axis, by name.</p>\n"},"getCoords":{"!type":"fn() -> ?","!doc":"<p>Get the coordinates and dimensions of the containing DOMNode, as\nreturned by dojo.coords.</p>\n"},"getGeometry":{"!type":"fn() -> ?","!doc":"<p>Returns a map of information about all axes in a chart and what they represent\nin terms of scaling (see dojox.charting.axis2d.Default.getScaler).</p>\n"},"getPlot":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Get the given plot, by name.</p>\n"},"getPlotOrder":{"!type":"fn() -> []","!doc":"<p>Returns an array of plot names in the current order\n(the top-most plot is the first).</p>\n"},"getProperty":{"!type":"fn(item: ?, prop: ?) -> ?","!doc":"<p>The main use of this function is to determine\nbetween a single value and an array of values.\nOther property types included for convenience.\n</p>\n"},"getSeries":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Get the given series, by name.</p>\n"},"getSeriesOrder":{"!type":"fn(plotName: ?) -> []","!doc":"<p>Returns an array of series names in the current order\n(the top-most series is the first) within a plot.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>If chart is showing, hide it\nPrevents rendering while hidden</p>\n"},"movePlotToBack":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given plot to back.</p>\n"},"movePlotToFront":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given plot to front.</p>\n"},"moveSeriesToBack":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given series to back of a plot.</p>\n"},"moveSeriesToFront":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Moves a given series to front of a plot.</p>\n"},"removeAxis":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Remove the axis that was defined using name.</p>\n"},"removePlot":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Remove the plot defined using name from the chart&#39;s plot stack.</p>\n"},"removeSeries":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Remove the series defined by name from the chart.</p>\n"},"render":{"!type":"fn() -> +dojox","!doc":"<p>Render the chart according to the current information defined.  This should\nbe the last call made when defining/creating a chart, or if data within the\nchart has been changed.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the chart to the dimensions of width and height.</p>\n"},"resizeChart":{"!type":"fn(dim: ?)","!doc":"<p>Call this function to change the chart size.\nCan be connected to a layout widget that calls\nresize.</p>\n"},"seriesLabels":{"!type":"fn(val: ?) -> string","!doc":"<p>Convenience method that sets series labels based on item labels.</p>\n"},"setAxisWindow":{"!type":"fn(name: ?, scale: ?, offset: ?, zoom?: ?) -> +dojox","!doc":"<p>Zooms an axis and all dependent plots. Can be used to zoom in 1D.</p>\n"},"setDir":{"!type":"fn(dir: ?) -> fn"},"setPlotOrder":{"!type":"fn(newOrder: ?) -> +dojox","!doc":"<p>Sets new order of plots. newOrder cannot add or remove\nplots. Wrong names, or dups are ignored.</p>\n"},"setSeriesOrder":{"!type":"fn(newOrder: ?) -> +dojox","!doc":"<p>Sets new order of series within a plot. newOrder cannot add\nor remove series. Wrong names, or dups are ignored.</p>\n"},"setStore":{"!type":"fn(store: ?, query: ?, fieldName: ?, queryOptions: ?)","!doc":"<p>Sets the chart store and query\nthen does the first fetch and\nconnects to subsequent changes.</p>\n"},"setTheme":{"!type":"fn(theme: ?) -> +dojox","!doc":"<p>Set a theme of the chart.</p>\n"},"setWindow":{"!type":"fn(sx: ?, sy: ?, dx: ?, dy: ?, zoom?: ?) -> +dojox","!doc":"<p>Zooms in or out any plots in two dimensions.</p>\n"},"show":{"!type":"fn()","!doc":"<p>If chart is hidden, show it</p>\n"},"updateSeries":{"!type":"fn(name: ?, data: ?, offsets?: ?) -> +dojox","!doc":"<p>Update the given series with a new set of data points.</p>\n"},"zoomIn":{"!type":"fn(name: ?, range: ?, delayed: ?)","!doc":"<p>Zoom the chart to a specific range on one axis.  This calls render()\ndirectly as a convenience method.</p>\n"}}},"dojox/charting/themes/PlotKit/blue":{"chart":{},"colors":{},"plotarea":{}},"dojox/charting/themes/PlotKit/base":{"Adobebricks":{},"Algae":{},"Bahamation":{},"base":{},"blue":{},"BlueDusk":{},"Charged":{},"Chris":{},"Claro":{},"CubanShirts":{},"cyan":{},"Desert":{},"Distinctive":{},"Dollar":{},"Electric":{},"Grasshopper":{},"Grasslands":{},"green":{},"GreySkies":{},"Harmony":{},"IndigoNation":{},"Ireland":{},"Julie":{},"MiamiNice":{},"Midwest":{},"Minty":{},"orange":{},"PrimaryColors":{},"purple":{},"PurpleRain":{},"red":{},"Renkoo":{},"RoyalPurples":{},"SageToLime":{},"Shrooms":{},"Tom":{},"Tufte":{},"WatersEdge":{},"Wetland":{},"generateFills":{"!type":"fn(colors: ?, fillPattern: ?, lumFrom: ?, lumTo: ?)","!doc":"<p>generates 2-color gradients using pure colors, a fill pattern, and two luminance values</p>\n"},"generateGradientByIntensity":{"!type":"fn(color: ?, intensityMap: ?)","!doc":"<p>generates gradient colors using an intensity map</p>\n"},"generateMiniTheme":{"!type":"fn(colors: ?, fillPattern: ?, lumFrom: ?, lumTo: ?, lumStroke: ?)","!doc":"<p>generates mini-themes with 2-color gradients using colors, a fill pattern, and three luminance values</p>\n"},"updateFills":{"!type":"fn(themes: ?, fillPattern: ?, lumFrom: ?, lumTo: ?)","!doc":"<p>transforms solid color fills into 2-color gradients using a fill pattern, and two luminance values</p>\n"}},"dojox/charting/Theme":{"!type":"fn()","prototype":{"!proto":"dojox/charting/SimpleTheme.prototype","!doc":"<p>A Theme is a pre-defined object, primarily JSON-based, that makes up the definitions to\nstyle a chart. It extends SimpleTheme with additional features like color definition by\npalettes and gradients definition.</p>\n","shapeSpaces":{},"addMarker":{"!type":"fn(name: ?, segment: ?)","!doc":"<p>Add a custom marker to this theme.</p>\n"},"addMixin":{"!type":"fn(theme: ?, elementType: ?, mixin: ?, doPost: ?) -> +dojox","!doc":"<p>Add a mixin object to the passed theme and process.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Clear and reset the internal pointer to start fresh.</p>\n"},"clone":{"!type":"fn() -> +dojox.charting.SimpleTheme","!doc":"<p>Clone the current theme.</p>\n"},"constructor":{"!type":"fn(kwArgs: ?)","!doc":"<p>Initialize a theme using the keyword arguments.  Note that the arguments\nlook like the example (above), and may include a few more parameters.</p>\n"},"getTick":{"!type":"fn(name: ?, mixin?: ?) -> ?","!doc":"<p>Calculates and merges tick parameters.</p>\n"},"inspectObjects":{"!type":"fn(f: ?)"},"next":{"!type":"fn(elementType?: ?, mixin?: ?, doPost?: ?) -> ?","!doc":"<p>Get the next color or series theme.</p>\n"},"post":{"!type":"fn(theme: ?, elementType: ?) -> +dojox","!doc":"<p>Process any post-shape fills.</p>\n"},"reverseFills":{"!type":"fn()"},"setMarkers":{"!type":"fn(obj: ?)","!doc":"<p>Set all the markers of this theme at once.  obj should be a\ndictionary of keys and path segments.\n</p>\n"},"skip":{"!type":"fn()","!doc":"<p>Skip the next internal color.</p>\n"}},"defaultColors":{},"defaultMarkers":{},"defaultTheme":{},"defineColors":{"!type":"fn(kwArgs: ?) -> +dojo","!doc":"<p>Generate a set of colors for the theme based on keyword\narguments.</p>\n"},"generateGradient":{"!type":"fn(fillPattern: ?, colorFrom: ?, colorTo: ?) -> ?"},"generateHslColor":{"!type":"fn(color: ?, luminance: ?) -> ?"},"generateHslGradient":{"!type":"fn(color: ?, fillPattern: ?, lumFrom: ?, lumTo: ?) -> ?"}},"dojox/charting/Theme.defaultMarkers":{"CIRCLE":{},"CROSS":{},"DIAMOND":{},"SQUARE":{},"TRIANGLE":{},"TRIANGLE_INVERTED":{},"X":{}},"dojox/charting/Theme.defaultTheme":{"axis":{},"chart":{},"indicator":{},"marker":{},"plotarea":{},"series":{}},"dojox/color/_base":{"Colorspace":{},"greyscale":{},"blend":{"!type":"fn(start: ?, end: ?, weight: ?, obj?: ?) -> ?","!doc":"<p>Blend colors end and start with weight from 0 to 1, 0.5 being a 50/50 blend,\ncan reuse a previously allocated Color object for the result</p>\n"},"Color":{"!type":"fn(color: ?)","!doc":"<p>Takes a named string, hex string, array of rgb or rgba values,\nan object with r, g, b, and a properties, or another <code>Color</code> object\nand creates a new Color instance to work from.\n</p>\n"},"fromArray":{"!type":"fn(a: ?, obj?: ?) -> +any","!doc":"<p>Builds a <code>Color</code> from a 3 or 4 element array, mapping each\nelement in sequence to the rgb(a) values of the color.</p>\n"},"fromCmy":{"!type":"fn(cyan: ?, magenta: ?, yellow: ?) -> +instance","!doc":"<p>Create a dojox.color.Color from a CMY defined color.\nAll colors should be expressed as 0-100 (percentage)</p>\n"},"fromCmyk":{"!type":"fn(cyan: ?, magenta: ?, yellow: ?, black: ?) -> +instance","!doc":"<p>Create a dojox.color.Color from a CMYK defined color.\nAll colors should be expressed as 0-100 (percentage)</p>\n"},"fromHex":{"!type":"fn(color: ?, obj?: ?) -> +any","!doc":"<p>Converts a hex string with a &#39;#&#39; prefix to a color object.\nSupports 12-bit #rgb shorthand. Optionally accepts a\n<code>Color</code> object to update with the parsed value.\n</p>\n"},"fromHsl":{"!type":"fn(hue: ?, saturation: ?, luminosity: ?) -> +instance","!doc":"<p>Create a dojox.color.Color from an HSL defined color.\nhue from 0-359 (degrees), saturation and luminosity 0-100.</p>\n"},"fromHsv":{"!type":"fn(hue: ?, saturation: ?, value: ?) -> +instance","!doc":"<p>Create a dojox.color.Color from an HSV defined color.\nhue from 0-359 (degrees), saturation and value 0-100.</p>\n"},"fromRgb":{"!type":"fn(color: ?, obj?: ?) -> +null","!doc":"<p>get rgb(a) array from css-style color declarations</p>\n"},"fromString":{"!type":"fn(str: ?, obj?: ?) -> +any","!doc":"<p>Parses <code>str</code> for a color value. Accepts hex, rgb, and rgba\nstyle color values.</p>\n"},"Palette":{"!type":"fn(base: ?)","!doc":"<p>An object that represents a palette of colors.</p>\n"}},"dojox/color/Palette":{"!doc":"<p>An object that represents a palette of colors.</p>\n","generators":{},"prototype":{"clone":{"!type":"fn() -> +instance","!doc":"<p>Clones the current palette.</p>\n"},"constructor":{"!type":"fn(base: ?)","!doc":"<p>An object that represents a palette of colors.</p>\n"},"transform":{"!type":"fn(kwArgs: ?) -> ?","!doc":"<p>Transform the palette using a specific transformation function\nand a set of transformation parameters.</p>\n"}},"generate":{"!type":"fn(base: ?, type: ?) -> ?","!doc":"<p>Generate a new Palette using any of the named functions in\ndojox.color.Palette.generators or an optional function definition.  Current\ngenerators include &quot;analogous&quot;, &quot;monochromatic&quot;, &quot;triadic&quot;, &quot;complementary&quot;,\n&quot;splitComplementary&quot;, and &quot;shades&quot;.</p>\n"}},"dojox/color/Palette.generators":{"analogous":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Create a 5 color palette based on the analogous rules as implemented at\n<a href=\"http://kuler.adobe.com\">http://kuler.adobe.com</a>.</p>\n"},"complementary":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Create a 5 color palette based on the complementary rules as implemented at\n<a href=\"http://kuler.adobe.com\">http://kuler.adobe.com</a>.</p>\n"},"compound":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Create a 5 color palette based on the compound rules as implemented at\n<a href=\"http://kuler.adobe.com\">http://kuler.adobe.com</a>.</p>\n"},"monochromatic":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Create a 5 color palette based on the monochromatic rules as implemented at\n<a href=\"http://kuler.adobe.com\">http://kuler.adobe.com</a>.</p>\n"},"shades":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Create a 5 color palette based on the shades rules as implemented at\n<a href=\"http://kuler.adobe.com\">http://kuler.adobe.com</a>.</p>\n"},"splitComplementary":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Create a 5 color palette based on the split complementary rules as implemented at\n<a href=\"http://kuler.adobe.com\">http://kuler.adobe.com</a>.</p>\n"},"triadic":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Create a 5 color palette based on the triadic rules as implemented at\n<a href=\"http://kuler.adobe.com\">http://kuler.adobe.com</a>.</p>\n"}},"dojox/charting/DataSeries":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(store: ?, kwArgs: ?, value: ?)","!doc":"<p>Series adapter for dojo.data stores.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up before GC.</p>\n"},"fetch":{"!type":"fn()","!doc":"<p>Fetches data from the store and updates a chart.</p>\n"},"setSeriesObject":{"!type":"fn(series: ?)","!doc":"<p>Sets a dojox.charting.Series object we will be working with.</p>\n"}}},"dojox/charting/StoreSeries":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(store: ?, kwArgs: ?, value: ?)","!doc":"<p>Series adapter for dojo object stores (dojo.store).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up before GC.</p>\n"},"fetch":{"!type":"fn()","!doc":"<p>Fetches data from the store and updates a chart.</p>\n"},"setSeriesObject":{"!type":"fn(series: ?)","!doc":"<p>Sets a dojox.charting.Series object we will be working with.</p>\n"}}},"dojox/charting/action2d/Base":{"!type":"fn()","prototype":{"!proto":"dojo/Evented.prototype","!doc":"<p>Base action class for plot and chart actions.</p>\n","connect":{"!type":"fn()","!doc":"<p>Connect this action to the plot or the chart.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot?: ?)","!doc":"<p>Create a new base action.  This can either be a plot or a chart action.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the plot or the chart.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojox/charting/action2d/ChartAction":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/Base.prototype","!doc":"<p>Base action class for chart actions.</p>\n","connect":{"!type":"fn()","!doc":"<p>Connect this action to the chart.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot?: ?)","!doc":"<p>Create a new base chart action.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the chart.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojox/charting/action2d/Highlight":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/PlotAction.prototype","!doc":"<p>Creates a highlighting action on a plot, where an element on that plot\nhas a highlight on it.</p>\n","defaultParams":{},"optionalParams":{},"overOutEvents":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the given plot.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot?: ?, kwArgs?: ?)","!doc":"<p>Create the highlighting action and connect it to the plot.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the given plot, if connected.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"process":{"!type":"fn(o: ?)","!doc":"<p>Process the action on the given object.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the action.</p>\n"}}},"dojox/charting/action2d/PlotAction":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/Base.prototype","!doc":"<p>Base action class for plot actions.</p>\n","overOutEvents":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the given plot.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot?: ?, kwargs?: ?)","!doc":"<p>Create a new base PlotAction.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the given plot, if connected.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"reset":{"!type":"fn()","!doc":"<p>Reset the action.</p>\n"}}},"dojox/charting/action2d/Magnify":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/PlotAction.prototype","!doc":"<p>Create an action that magnifies the object the action is applied to.</p>\n","defaultParams":{},"optionalParams":{},"overOutEvents":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the given plot.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot?: ?, kwArgs?: ?)","!doc":"<p>Create the magnifying action.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the given plot, if connected.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"process":{"!type":"fn(o: ?)","!doc":"<p>Process the action on the given object.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the action.</p>\n"}}},"dojox/charting/action2d/MouseIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/ChartAction.prototype","!doc":"<p>Create a mouse indicator action. You can drag mouse over the chart to display a data indicator.</p>\n","defaultParams":{},"optionalParams":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the chart. This adds a indicator plot\nto the chart that&#39;s why Chart.render() must be called after connect.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot: ?, kwArgs?: ?)","!doc":"<p>Create an mouse indicator action and connect it.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the chart.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojox/charting/action2d/_IndicatorElement":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/Indicator.prototype","!doc":"<p>Internal element used by indicator actions.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?)"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator?: ?) -> +dojox","!doc":"<p>Clean any elements (HTML or GFX-based) out of our group, and create a new one.</p>\n"},"clear":{"!type":"fn()"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs: ?)"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Returns default stats (irrelevant for this type of plot).</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Return whether or not this plot needs to be redrawn.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn()"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"stopTrack":{"!type":"fn()"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/plot2d/Indicator":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot2d/CartesianBase.prototype","!doc":"<p>A &quot;faux&quot; plot that can be placed behind or above other plots to represent a line or multi-line\nthreshold on the chart.</p>\n","baseParams":{},"chart":{"!doc":"<p>The parent chart for this element.</p>\n"},"defaultParams":{},"dirty":{"!doc":"<p>A flag indicating whether or not this element needs to be rendered.</p>\n"},"group":{"!doc":"<p>The visual GFX group representing this element.</p>\n"},"htmlElement":{"!doc":"<p>Any DOMNodes used as a part of this element (such as HTML-based labels).</p>\n"},"htmlElements":{},"optionalParams":{},"addSeries":{"!type":"fn(run: ?)"},"assignAxes":{"!type":"fn(axes: ?)","!doc":"<p>From an array of axes pick the ones that correspond to this plot and\nassign them to the plot using setAxis method.</p>\n"},"calculateAxes":{"!type":"fn(dim: ?) -> +dojox","!doc":"<p>Stub function for running the axis calculations (deprecated).</p>\n"},"cleanGroup":{"!type":"fn(creator: ?, noClip: ?)"},"clear":{"!type":"fn()"},"connect":{"!type":"fn(object: ?, method: ?) -> []","!doc":"<p>Helper function to connect any object&#39;s method to our plotEvent.</p>\n"},"constructor":{"!type":"fn(chart: ?, kwArgs?: ?)","!doc":"<p>Create the faux Grid plot.</p>\n"},"createLabel":{"!type":"fn(group: ?, value: ?, bbox: ?, theme: ?)"},"destroy":{"!type":"fn()","!doc":"<p>API addition to conform to the rest of the Dojo Toolkit&#39;s standard.</p>\n"},"destroyHtmlElements":{"!type":"fn()","!doc":"<p>Destroy any DOMNodes that may have been created as a part of this element.</p>\n"},"events":{"!type":"fn() -> bool","!doc":"<p>Find out if any event handlers have been connected to our plotEvent.</p>\n"},"fireEvent":{"!type":"fn(seriesName: ?, eventName: ?, index: ?, eventObject?: ?)","!doc":"<p>Emulates firing an event for a given data value (specified by\nan index) of a given series.</p>\n"},"getGroup":{"!type":"fn() -> ?"},"getRequiredColors":{"!type":"fn() -> number","!doc":"<p>Get how many data series we have, so we know how many colors to use.</p>\n"},"getSeriesStats":{"!type":"fn() -> ?","!doc":"<p>Returns default stats (irrelevant for this type of plot).</p>\n"},"getTextWidth":{"!type":"fn(s: ?, font: ?) -> number"},"getTextWithLimitCharCount":{"!type":"fn(s?: ?, font?: ?, wcLimit?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited character count(dichotomy algorithm)</p>\n"},"getTextWithLimitLength":{"!type":"fn(s?: ?, font?: ?, limitWidth?: ?, truncated?: ?) -> ?","!doc":"<p>Get the truncated string based on the limited width in px(dichotomy algorithm)</p>\n"},"initializeScalers":{"!type":"fn(dim: ?, stats: ?) -> +dojox","!doc":"<p>Initializes scalers using attached axes.</p>\n"},"isDataDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not any of this plot&#39;s data series need to be rendered.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not this plot needs to be rendered.</p>\n"},"performZoom":{"!type":"fn(dim: ?, offsets: ?) -> +dojox","!doc":"<p>Create/alter any zooming windows on this plot.</p>\n"},"plotEvent":{"!type":"fn(o: ?)","!doc":"<p>Stub function for use by specific plots.</p>\n"},"purgeGroup":{"!type":"fn()"},"raiseEvent":{"!type":"fn(o: ?)","!doc":"<p>Raises events in predefined order</p>\n"},"render":{"!type":"fn(dim: ?, offsets: ?) -> ?"},"renderLabel":{"!type":"fn(group: ?, x: ?, y: ?, label: ?, theme: ?, block: ?, align: ?) -> ?"},"resetEvents":{"!type":"fn()","!doc":"<p>Reset all events attached to our plotEvent (i.e. disconnect).</p>\n"},"setAxis":{"!type":"fn(axis: ?) -> +dojox","!doc":"<p>Set an axis for this plot.</p>\n"},"toData":{"!type":"fn(coord: ?) -> ?","!doc":"<p>Compute plot axis data coordinates from page coordinates.</p>\n"},"toPage":{"!type":"fn(coord?: ?) -> ?","!doc":"<p>Compute page coordinates from plot axis data coordinates.</p>\n"}}},"dojox/charting/action2d/MouseZoomAndPan":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/ChartAction.prototype","!doc":"<p>Create an mouse zoom and pan action.\nYou can zoom in or out the data window with mouse wheel. You can scroll using mouse drag gesture. \nYou can toggle between zoom and fit view using double click on the chart.</p>\n","defaultParams":{},"optionalParams":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the chart.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot: ?, kwArgs?: ?)","!doc":"<p>Create an mouse zoom and pan action and connect it.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the chart.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojox/charting/action2d/MoveSlice":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/PlotAction.prototype","!doc":"<p>Create an action for a pie chart that moves and scales a pie slice.</p>\n","defaultParams":{},"optionalParams":{},"overOutEvents":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the given plot.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot?: ?, kwArgs?: ?)","!doc":"<p>Create the slice moving action and connect it to the plot.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the given plot, if connected.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"process":{"!type":"fn(o: ?)","!doc":"<p>Process the action on the given object.</p>\n"},"reset":{"!type":"fn()"}}},"dojox/lang/functional/scan":{},"dojox/charting/action2d/Shake":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/PlotAction.prototype","!doc":"<p>Create a shaking action for use on an element in a chart.</p>\n","defaultParams":{},"optionalParams":{},"overOutEvents":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the given plot.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot?: ?, kwArgs?: ?)","!doc":"<p>Create the shaking action and connect it to the plot.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the given plot, if connected.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"process":{"!type":"fn(o: ?)","!doc":"<p>Process the action on the given object.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the action.</p>\n"}}},"dojox/charting/action2d/Tooltip":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/PlotAction.prototype","!doc":"<p>Create an action on a plot where a tooltip is shown when hovering over an element.</p>\n","defaultParams":{},"optionalParams":{},"overOutEvents":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the given plot.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot?: ?, kwArgs?: ?)","!doc":"<p>Create the tooltip action and connect it to the plot.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the given plot, if connected.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"process":{"!type":"fn(o: ?)","!doc":"<p>Process the action on the given object.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the action.</p>\n"}}},"dojox/charting/action2d/TouchIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/ChartAction.prototype","!doc":"<p>Create a touch indicator action. You can touch over the chart to display a data indicator.</p>\n","defaultParams":{},"optionalParams":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the chart. This adds a indicator plot\nto the chart that&#39;s why Chart.render() must be called after connect.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot: ?, kwArgs?: ?)","!doc":"<p>Create a new touch indicator action and connect it.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the chart.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojox/charting/action2d/TouchZoomAndPan":{"!type":"fn()","prototype":{"!proto":"dojox/charting/action2d/ChartAction.prototype","!doc":"<p>Create a touch zoom and pan action.\nYou can zoom out or in the data window with pinch and spread gestures except on Android 2.x and WP8 devices.\nYou can scroll using drag gesture.\nFinally this is possible to navigate between a fit window and a zoom one using double tap gesture.</p>\n","defaultParams":{},"optionalParams":{},"connect":{"!type":"fn()","!doc":"<p>Connect this action to the chart. On iOS this adds a new glass view plot\nto the chart that&#39;s why Chart.render() must be called after connect.</p>\n"},"constructor":{"!type":"fn(chart: ?, plot: ?, kwArgs?: ?)","!doc":"<p>Create a new touch zoom and pan action and connect it.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Do any cleanup needed when destroying parent elements.</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>Disconnect this action from the chart.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojox/charting/bidi/Chart":{"!type":"fn()","prototype":{"dir":{"!doc":"<p>Mirroring support,  the main variable which is responsible for the direction of the chart.\n\n</p>\n<p>Allowed values:\n1. &quot;ltr&quot;\n2. &quot;rtl&quot;\n\n</p>\n<p>By default is ltr.</p>\n"},"isMirrored":{},"textDir":{"!doc":"<p>Bi-directional support, the main variable which is responsible for the direction of the text.\nThe text direction can be different than the GUI direction by using this parameter.\nAllowed values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot;</li>\n<li>&quot;rtl&quot;</li>\n<li>&quot;auto&quot; - contextual the direction of a text defined by first strong letter.</li>\n</ol>\n<p>By default is as the page direction.</p>\n"},"applyMirroring":{"!type":"fn(plot: ?, dim: ?, offsets: ?) -> fn","!doc":"<p>apply the mirroring operation to the current chart plots.\n</p>\n"},"formatTruncatedLabel":{"!type":"fn(element: ?, label: ?, labelType: ?)"},"getTextDir":{"!type":"fn(text: ?) -> ?","!doc":"<p>Return direction of the text. </p>\n"},"isRightToLeft":{"!type":"fn() -> bool","!doc":"<p>check the direction of the chart.</p>\n"},"render":{"!type":"fn() -> fn"},"setDir":{"!type":"fn(dir: ?) -> fn","!doc":"<p>Setter for the dir attribute.</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?, obj: ?) -> fn","!doc":"<p>Setter for the textDir attribute.</p>\n"}}},"dojox/gfx/_gfxBidiSupport":{"_base":{},"_gfxBidiSupport":{},"_svgFontCache":{},"_vectorFontCache":{},"arc":{"!doc":"<p>This module contains the core graphics Arc functions.</p>\n"},"bezierutils":{},"canvas":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of this module alone will only add in drawing support.\nThe additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.\nThe id of the Canvas renderer is &#39;canvas&#39;.  This id can be used when switch Dojo&#39;s\ngraphics context between renderer implementations.  See dojox/gfx/_base.switchRenderer\nAPI.</p>\n"},"canvas_attach":{},"canvasext":{},"canvasWithEvents":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers which extends\nthe basic canvas drawing renderer bridge to add additional support for graphics events\non Shapes.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of the canvas module alone will only add in drawing support.\nThis additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.</p>\n"},"cm_in_pt":{"!doc":"<p>points per centimeter (constant)</p>\n"},"defaultCircle":{"!doc":"<p>An object defining the default Circle prototype.</p>\n"},"defaultEllipse":{"!doc":"<p>Defines the default Ellipse prototype.</p>\n"},"defaultFont":{"!doc":"<p>An object specifying the default properties for a Font used in text operations.</p>\n"},"defaultImage":{"!doc":"<p>Defines the default Image prototype.</p>\n"},"defaultLine":{"!doc":"<p>An object defining the default Line prototype.</p>\n"},"defaultLinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n"},"defaultPath":{"!doc":"<p>Defines the default Path prototype object.</p>\n"},"defaultPattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n"},"defaultPolyline":{"!doc":"<p>Defines the default PolyLine prototype.</p>\n"},"defaultRadialGradient":{"!doc":"<p>An object specifying the default properties for RadialGradients using in fills patterns.</p>\n"},"defaultRect":{"!doc":"<p>Defines the default Rect prototype.</p>\n"},"defaultStroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.\nThis object defines the default Stroke prototype.</p>\n"},"defaultText":{"!doc":"<p>Defines the default Text prototype.</p>\n"},"defaultTextPath":{"!doc":"<p>Defines the default TextPath prototype.</p>\n"},"defaultVectorFont":{},"defaultVectorText":{},"Fill":{"!doc":"<p>Defines how to fill a shape. Four types of fills can be used: solid, linear gradient, radial gradient and pattern.\nSee dojox/gfx.LinearGradient, dojox/gfx.RadialGradient and dojox/gfx.Pattern respectively for more information about the properties supported by each type.</p>\n"},"Font":{"!doc":"<p>An object specifying the properties for a Font used in text operations.</p>\n"},"fx":{},"getDefault":{},"gradient":{},"gradutils":{},"LinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n"},"matrix":{},"mm_in_pt":{"!doc":"<p>points per millimeter (constant)</p>\n"},"move":{},"path":{"!doc":"<p>This module contains the core graphics Path API.\nPath command format follows the W3C SVG 1.0 Path api.</p>\n"},"pathSvgRegExp":{},"pathVmlRegExp":{"!doc":"<p>a constant regular expression used to split a SVG/VML path into primitive components</p>\n"},"Pattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n"},"RadialGradient":{"!doc":"<p>Specifies the properties for RadialGradients using in fills patterns.</p>\n"},"renderer":{"!doc":"<p>Either the string name of a renderer (eg. &#39;canvas&#39;, &#39;svg, ...) or the renderer\nobject to switch to.</p>\n"},"shape":{"!doc":"<p>This module contains the core graphics Shape API.\nDifferent graphics renderer implementation modules (svg, canvas, vml, silverlight, etc.) extend this\nbasic api to provide renderer-specific implementations for each shape.</p>\n"},"silverlight":{"!doc":"<p>This the graphics rendering bridge for the Microsoft Silverlight plugin.\nSilverlight is a faster implementation on IE6-8 than the default 2d graphics, VML</p>\n"},"silverlight_attach":{},"Stroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.</p>\n"},"svg":{"!doc":"<p>This the graphics rendering bridge for browsers compliant with W3C SVG1.0.\nThis is the preferred renderer to use for interactive and accessible graphics.</p>\n"},"svgext":{},"utils":{},"vectorFontFitting":{},"VectorText":{},"vml":{"!doc":"<p>This the default graphics rendering bridge for IE6-7.\nThis renderer is very slow.  For best performance on IE6-8, use Silverlight plugin.\nIE9+ defaults to the standard W3C SVG renderer.</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +dojox","!doc":"<p>creates a surface</p>\n"},"decompose":{"!type":"fn(matrix: ?)","!doc":"<p>Decompose a 2D matrix into translation, scaling, and rotation components.</p>\n"},"Ellipse":{"!type":"fn()"},"equalSources":{"!type":"fn(a: ?, b: ?)","!doc":"<p>compares event sources, returns true if they are equal</p>\n"},"formatNumber":{"!type":"fn(x: ?, addSpace: ?) -> string","!doc":"<p>converts a number to a string using a fixed notation</p>\n"},"getVectorFont":{"!type":"fn(url: ?) -> ?"},"Group":{"!type":"fn()"},"Line":{"!type":"fn()"},"makeFontString":{"!type":"fn(font: ?) -> string","!doc":"<p>converts a font object to a CSS font string</p>\n"},"makeParameters":{"!type":"fn(defaults: ?, update: ?) -> ?","!doc":"<p>copies the original object, and all copied properties from the\n&#39;update&#39; object</p>\n"},"Matrix2D":{"!type":"fn(arg: ?)","!doc":"<p>a 2D matrix object</p>\n"},"Moveable":{"!type":"fn()"},"Mover":{"!type":"fn()"},"normalizeColor":{"!type":"fn(color: ?) -> +instance","!doc":"<p>converts any legal color representation to normalized\ndojo/Color object</p>\n"},"normalizedLength":{"!type":"fn(len: ?) -> number","!doc":"<p>converts any length value to pixels</p>\n"},"normalizeParameters":{"!type":"fn(existed: ?, update: ?) -> ?","!doc":"<p>updates an existing object with properties from an &#39;update&#39;\nobject</p>\n"},"Path":{"!type":"fn()"},"Point":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"pt2px":{"!type":"fn(len: ?) -> number","!doc":"<p>converts points to pixels</p>\n"},"px2pt":{"!type":"fn(len: ?) -> number","!doc":"<p>converts pixels to points</p>\n"},"px_in_pt":{"!type":"fn() -> number","!doc":"<p>returns the current number of pixels per point.</p>\n"},"Rect":{"!type":"fn()"},"Rectangle":{"!type":"fn()"},"splitFontString":{"!type":"fn(str: ?) -> ?","!doc":"<p>converts a CSS font string to a font object</p>\n"},"Surface":{"!type":"fn()"},"switchTo":{"!type":"fn(renderer: ?)","!doc":"<p>switch the graphics implementation to the specified renderer.</p>\n"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"},"VectorFont":{"!type":"fn()"}},"dojox/gfx/_gfxBidiSupport._base":{},"dojox/gfx/_gfxBidiSupport.Stroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.</p>\n","cap":{"!doc":"<p>The endcap style of the path. One of &#39;butt&#39;, &#39;round&#39;, ... . Default value &#39;butt&#39;.</p>\n"},"color":{"!doc":"<p>The color of the stroke, default value &#39;black&#39;.</p>\n"},"join":{"!doc":"<p>The join style to use when combining path segments. Default value 4.</p>\n"},"style":{"!doc":"<p>The style of the stroke, one of &#39;solid&#39;, ... . Default value &#39;solid&#39;.</p>\n"},"width":{"!doc":"<p>The width of a stroke, default value 1.</p>\n"}},"dojox/gfx/_gfxBidiSupport.Fill":{"!doc":"<p>Defines how to fill a shape. Four types of fills can be used: solid, linear gradient, radial gradient and pattern.\nSee dojox/gfx.LinearGradient, dojox/gfx.RadialGradient and dojox/gfx.Pattern respectively for more information about the properties supported by each type.</p>\n","color":{"!doc":"<p>The color of a solid fill type.</p>\n"},"type":{"!doc":"<p>The type of fill. One of &#39;linear&#39;, &#39;radial&#39;, &#39;pattern&#39; or undefined. If not specified, a solid fill is assumed.</p>\n"}},"dojox/gfx/_gfxBidiSupport.LinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the start of the line).  The start of the line is\ndefined at offest 0 with the end of the line at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Linear Gradient, value &#39;linear&#39;</p>\n"},"x1":{"!doc":"<p>The X coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"x2":{"!doc":"<p>The X coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"},"y1":{"!doc":"<p>The Y coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"y2":{"!doc":"<p>The Y coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"}},"dojox/gfx/_gfxBidiSupport.RadialGradient":{"!doc":"<p>Specifies the properties for RadialGradients using in fills patterns.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the center of the radial gradient).\nThe center is defined at offest 0 with the outer edge of the gradient at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"cx":{"!doc":"<p>The X coordinate of the center of the radial gradient, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the radial gradient, default value 0.</p>\n"},"r":{"!doc":"<p>The radius to the end of the radial gradient, default value 100.</p>\n"},"type":{"!doc":"<p>Specifies this is a RadialGradient, value &#39;radial&#39;</p>\n"}},"dojox/gfx/_gfxBidiSupport.Pattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n","height":{"!doc":"<p>The height of the pattern image, default value is 0.</p>\n"},"src":{"!doc":"<p>A url specifying the image to use for the pattern.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Pattern, value &#39;pattern&#39;.</p>\n"},"width":{"!doc":"<p>The width of the pattern image, default value is 0.</p>\n"},"x":{"!doc":"<p>The X coordinate of the position of the pattern, default value is 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the position of the pattern, default value is 0.</p>\n"}},"dojox/gfx/_gfxBidiSupport.Text":{"!type":"fn()","prototype":{"!doc":"<p>a generic text (do not instantiate it directly)</p>\n","textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_gfxBidiSupport.Font":{"!doc":"<p>An object specifying the properties for a Font used in text operations.</p>\n","family":{"!doc":"<p>The font family, one of &#39;serif&#39;, &#39;sanserif&#39;, ..., default value &#39;serif&#39;.</p>\n"},"size":{"!doc":"<p>The font size (including units), default value &#39;10pt&#39;.</p>\n"},"style":{"!doc":"<p>The font style, one of &#39;normal&#39;, &#39;bold&#39;, default value &#39;normal&#39;.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Font, value &#39;font&#39;.</p>\n"},"variant":{"!doc":"<p>The font variant, one of &#39;normal&#39;, ... , default value &#39;normal&#39;.</p>\n"},"weight":{"!doc":"<p>The font weight, one of &#39;normal&#39;, ..., default value &#39;normal&#39;.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultPath":{"!doc":"<p>Defines the default Path prototype object.</p>\n","path":{"!doc":"<p>The path commands. See W32C SVG 1.0 specification.\nDefaults to empty string value.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Path, default value &#39;path&#39;.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultPolyline":{"!doc":"<p>Defines the default PolyLine prototype.</p>\n","points":{"!doc":"<p>An array of point objects [{x:0,y:0},...] defining the default polyline&#39;s line segments. Value is an empty array [].</p>\n"},"type":{"!doc":"<p>Specifies this object is a PolyLine, default value &#39;polyline&#39;.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultRect":{"!doc":"<p>Defines the default Rect prototype.</p>\n","height":{"!doc":"<p>The height of the default rectangle, value 100.</p>\n"},"r":{"!doc":"<p>The corner radius for the default rectangle, value 0.</p>\n"},"type":{"!doc":"<p>Specifies this default object is a type of Rect. Value is &#39;rect&#39;</p>\n"},"width":{"!doc":"<p>The width of the default rectangle, value 100.</p>\n"},"x":{"!doc":"<p>The X coordinate of the default rectangles position, value 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the default rectangle&#39;s position, value 0.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultEllipse":{"!doc":"<p>Defines the default Ellipse prototype.</p>\n","cx":{"!doc":"<p>The X coordinate of the center of the ellipse, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the ellipse, default value 0.</p>\n"},"rx":{"!doc":"<p>The radius of the ellipse in the X direction, default value 200.</p>\n"},"ry":{"!doc":"<p>The radius of the ellipse in the Y direction, default value 200.</p>\n"},"type":{"!doc":"<p>Specifies that this object is a type of Ellipse, value is &#39;ellipse&#39;</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultCircle":{"!doc":"<p>An object defining the default Circle prototype.</p>\n","cx":{"!doc":"<p>The X coordinate of the center of the circle, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the circle, default value 0.</p>\n"},"r":{"!doc":"<p>The radius, default value 100.</p>\n"},"type":{"!doc":"<p>Specifies this object is a circle, value &#39;circle&#39;</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultLine":{"!doc":"<p>An object defining the default Line prototype.</p>\n","type":{"!doc":"<p>Specifies this is a Line, value &#39;line&#39;</p>\n"},"x1":{"!doc":"<p>The X coordinate of the start of the line, default value 0.</p>\n"},"x2":{"!doc":"<p>The X coordinate of the end of the line, default value 100.</p>\n"},"y1":{"!doc":"<p>The Y coordinate of the start of the line, default value 0.</p>\n"},"y2":{"!doc":"<p>The Y coordinate of the end of the line, default value 100.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultImage":{"!doc":"<p>Defines the default Image prototype.</p>\n","height":{"!doc":"<p>The height of the image, default value 0.</p>\n"},"src":{"!doc":"<p>The src url of the image, defaults to empty string.</p>\n"},"type":{"!doc":"<p>Specifies this object is an image, value &#39;image&#39;.</p>\n"},"width":{"!doc":"<p>The width of the image, default value 0.</p>\n"},"x":{"!doc":"<p>The X coordinate of the image&#39;s position, default value 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the image&#39;s position, default value 0.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultText":{"!doc":"<p>Defines the default Text prototype.</p>\n","align":{"!doc":"<p>The horizontal text alignment, one of &#39;start&#39;, &#39;end&#39;, &#39;center&#39;. Default value &#39;start&#39;.</p>\n"},"decoration":{"!doc":"<p>The text decoration , one of &#39;none&#39;, ... . Default value &#39;none&#39;.</p>\n"},"kerning":{"!doc":"<p>Whether kerning is used on the text, boolean default value true.</p>\n"},"rotated":{"!doc":"<p>Whether the text is rotated, boolean default value false.</p>\n"},"text":{"!doc":"<p>The text to be displayed, default value empty string.</p>\n"},"type":{"!doc":"<p>Specifies this is a Text shape, value &#39;text&#39;.</p>\n"},"x":{"!doc":"<p>The X coordinate of the text position, default value 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the text position, default value 0.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultTextPath":{"!doc":"<p>Defines the default TextPath prototype.</p>\n","align":{"!doc":"<p>The horizontal text alignment, one of &#39;start&#39;, &#39;end&#39;, &#39;center&#39;. Default value &#39;start&#39;.</p>\n"},"decoration":{"!doc":"<p>The text decoration , one of &#39;none&#39;, ... . Default value &#39;none&#39;.</p>\n"},"kerning":{"!doc":"<p>Whether kerning is used on the text, boolean default value true.</p>\n"},"rotated":{"!doc":"<p>Whether the text is rotated, boolean default value false.</p>\n"},"text":{"!doc":"<p>The text to be displayed, default value empty string.</p>\n"},"type":{"!doc":"<p>Specifies this is a TextPath, value &#39;textpath&#39;.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultStroke":{"!doc":"<p>A stroke defines stylistic properties that are used when drawing a path.\nThis object defines the default Stroke prototype.</p>\n","cap":{"!doc":"<p>The endcap style of the path. One of &#39;butt&#39;, &#39;round&#39;, ... . Default value &#39;butt&#39;.</p>\n"},"color":{"!doc":"<p>The color of the stroke, default value &#39;black&#39;.</p>\n"},"join":{"!doc":"<p>The join style to use when combining path segments. Default value 4.</p>\n"},"style":{"!doc":"<p>The style of the stroke, one of &#39;solid&#39;, ... . Default value &#39;solid&#39;.</p>\n"},"type":{"!doc":"<p>Specifies this object is a type of Stroke, value &#39;stroke&#39;.</p>\n"},"width":{"!doc":"<p>The width of a stroke, default value 1.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultLinearGradient":{"!doc":"<p>An object defining the default stylistic properties used for Linear Gradient fills.\nLinear gradients are drawn along a virtual line, which results in appearance of a rotated pattern in a given direction/orientation.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the start of the line).  The start of the line is\ndefined at offest 0 with the end of the line at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Linear Gradient, value &#39;linear&#39;</p>\n"},"x1":{"!doc":"<p>The X coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"x2":{"!doc":"<p>The X coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"},"y1":{"!doc":"<p>The Y coordinate of the start of the virtual line along which the gradient is drawn, default value 0.</p>\n"},"y2":{"!doc":"<p>The Y coordinate of the end of the virtual line along which the gradient is drawn, default value 100.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultRadialGradient":{"!doc":"<p>An object specifying the default properties for RadialGradients using in fills patterns.</p>\n","colors":{"!doc":"<p>An array of colors at given offsets (from the center of the radial gradient).\nThe center is defined at offest 0 with the outer edge of the gradient at offset 1.\nDefault value, [{ offset: 0, color: &#39;black&#39;},{offset: 1, color: &#39;white&#39;}], is a gradient from black to white.</p>\n"},"cx":{"!doc":"<p>The X coordinate of the center of the radial gradient, default value 0.</p>\n"},"cy":{"!doc":"<p>The Y coordinate of the center of the radial gradient, default value 0.</p>\n"},"r":{"!doc":"<p>The radius to the end of the radial gradient, default value 100.</p>\n"},"type":{"!doc":"<p>Specifies this is a RadialGradient, value &#39;radial&#39;</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultPattern":{"!doc":"<p>An object specifying the default properties for a Pattern using in fill operations.</p>\n","height":{"!doc":"<p>The height of the pattern image, default value is 0.</p>\n"},"src":{"!doc":"<p>A url specifying the image to use for the pattern.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Pattern, value &#39;pattern&#39;.</p>\n"},"width":{"!doc":"<p>The width of the pattern image, default value is 0.</p>\n"},"x":{"!doc":"<p>The X coordinate of the position of the pattern, default value is 0.</p>\n"},"y":{"!doc":"<p>The Y coordinate of the position of the pattern, default value is 0.</p>\n"}},"dojox/gfx/_gfxBidiSupport.defaultFont":{"!doc":"<p>An object specifying the default properties for a Font used in text operations.</p>\n","family":{"!doc":"<p>The font family, one of &#39;serif&#39;, &#39;sanserif&#39;, ..., default value &#39;serif&#39;.</p>\n"},"size":{"!doc":"<p>The font size (including units), default value &#39;10pt&#39;.</p>\n"},"style":{"!doc":"<p>The font style, one of &#39;normal&#39;, &#39;bold&#39;, default value &#39;normal&#39;.</p>\n"},"type":{"!doc":"<p>Specifies this object is a Font, value &#39;font&#39;.</p>\n"},"variant":{"!doc":"<p>The font variant, one of &#39;normal&#39;, ... , default value &#39;normal&#39;.</p>\n"},"weight":{"!doc":"<p>The font weight, one of &#39;normal&#39;, ..., default value &#39;normal&#39;.</p>\n"}},"dojox/gfx/_gfxBidiSupport.matrix":{"flipX":{"!doc":"<p>a matrix, which reflects points at x = 0 line: flipX * (x, y) == (-x, y)</p>\n"},"flipXY":{"!doc":"<p>a matrix, which reflects points at the origin of coordinates: flipXY * (x, y) == (-x, -y)</p>\n"},"flipY":{"!doc":"<p>a matrix, which reflects points at y = 0 line: flipY * (x, y) == (x, -y)</p>\n"},"identity":{"!doc":"<p>an identity matrix constant: identity * (x, y) == (x, y)</p>\n"},"clone":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>creates a copy of a 2D matrix</p>\n"},"invert":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>inverts a 2D matrix</p>\n"},"isIdentity":{"!type":"fn(matrix: ?) -> bool","!doc":"<p>returns whether the specified matrix is the identity.</p>\n"},"Matrix2D":{"!type":"fn(arg: ?)","!doc":"<p>a 2D matrix object</p>\n"},"multiply":{"!type":"fn(matrix: ?) -> +instance","!doc":"<p>combines matrices by multiplying them sequentially in the given order</p>\n"},"multiplyPoint":{"!type":"fn(matrix: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>applies a matrix to a point</p>\n"},"multiplyRectangle":{"!type":"fn(matrix: ?, rect: ?) -> +dojox","!doc":"<p>Applies a matrix to a rectangle.</p>\n"},"normalize":{"!type":"fn(matrix: ?) -> +dojox","!doc":"<p>converts an object to a matrix, if necessary</p>\n"},"project":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms an orthogonal projection matrix</p>\n"},"reflect":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a reflection matrix</p>\n"},"rotate":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms a rotating matrix</p>\n"},"rotateAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>rotates a picture using a specified point as a center of rotation</p>\n"},"rotateg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms a rotating matrix</p>\n"},"rotategAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>rotates a picture using a specified point as a center of rotation</p>\n"},"scale":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a scaling matrix</p>\n"},"scaleAt":{"!type":"fn(a: ?, b?: ?, c: ?, d: ?) -> +dojox","!doc":"<p>scales a picture using a specified point as a center of scaling</p>\n"},"skewX":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms an x skewing matrix</p>\n"},"skewXAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the x axis using a specified point as a center of skewing</p>\n"},"skewXg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms an x skewing matrix</p>\n"},"skewXgAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the x axis using a specified point as a center of skewing</p>\n"},"skewY":{"!type":"fn(angle: ?) -> +dojox","!doc":"<p>forms a y skewing matrix</p>\n"},"skewYAt":{"!type":"fn(angle: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the y axis using a specified point as a center of skewing</p>\n"},"skewYg":{"!type":"fn(degree: ?) -> +dojox","!doc":"<p>forms a y skewing matrix</p>\n"},"skewYgAt":{"!type":"fn(degree: ?, a: ?, b?: ?) -> +dojox","!doc":"<p>skews a picture along the y axis using a specified point as a center of skewing</p>\n"},"translate":{"!type":"fn(a: ?, b?: ?) -> +dojox","!doc":"<p>forms a translation matrix</p>\n"}},"dojox/gfx/_gfxBidiSupport.shape":{"!doc":"<p>This module contains the core graphics Shape API.\nDifferent graphics renderer implementation modules (svg, canvas, vml, silverlight, etc.) extend this\nbasic api to provide renderer-specific implementations for each shape.</p>\n","_eventsProcessing":{},"Container":{"!doc":"<p>a container of shapes, which can be used\nas a foundation for renderer-specific groups, or as a way\nto logically group shapes (e.g, to propagate matricies)</p>\n"},"Creator":{"!doc":"<p>shape creators</p>\n"},"byId":{"!type":"fn(id: ?) -> +dojox","!doc":"<p>Returns the shape that matches the specified id.</p>\n"},"Circle":{"!type":"fn()"},"dispose":{"!type":"fn(s: ?, recurse?: ?)","!doc":"<p>Removes the specified shape from the registry.</p>\n"},"Ellipse":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"register":{"!type":"fn(s: ?) -> number","!doc":"<p>Register the specified shape into the graphics registry.</p>\n"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"}},"dojox/gfx/_gfxBidiSupport.Point":{"!type":"fn()","prototype":{"!doc":"<p>2D point for drawings - {x, y}</p>\n"}},"dojox/gfx/_gfxBidiSupport.Rectangle":{"!type":"fn()","prototype":{"!doc":"<p>rectangle - {x, y, width, height}</p>\n"}},"dojox/gfx/_gfxBidiSupport.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape, which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","textDir":{"!doc":"<p>Will be used for inheritance, or as default for text objects\nthat textDir wasn&#39;t directly specified for them but the bidi support was required.</p>\n"},"add":{"!type":"fn(shape: ?) -> ?","!doc":"<p>adds a shape to the list</p>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn(destroy?: ?) -> fn","!doc":"<p>removes all shapes from a group/surface.</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_gfxBidiSupport.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a generic rectangle</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box (its shape in this case)</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_gfxBidiSupport.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a generic circle</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_gfxBidiSupport.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>a generic ellipse</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_gfxBidiSupport.Line":{"!type":"fn()","prototype":{"!doc":"<p>a generic line (do not instantiate it directly)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_gfxBidiSupport.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a generic polyline/polygon (do not instantiate it directly)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>returns the bounding box</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(points: ?, closed: ?) -> fn","!doc":"<p>sets a polyline/polygon shape object</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/_gfxBidiSupport.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> ?","!doc":"<p>adds a shape to the list</p>\n"},"clear":{"!type":"fn(destroy?: ?) -> fn","!doc":"<p>removes all shapes from a group/surface.</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()","!doc":"<p>destroy all relevant external resources and release all\nexternal references to make this object garbage-collectible</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> ?","!doc":"<p>gets current width and height in pixels</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> fn","!doc":"<p>removes a shape from the list</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/_gfxBidiSupport.gradutils":{"getColor":{"!type":"fn(fill: ?, pt: ?)","!doc":"<p>sample a color from a gradient using a point</p>\n"},"reverse":{"!type":"fn(fill: ?)","!doc":"<p>reverses a gradient</p>\n"}},"dojox/gfx/_gfxBidiSupport.fx":{"animateFill":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation which will change fill color over time.\nOnly solid fill color is supported at the moment</p>\n"},"animateFont":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation which will change font properties over time.</p>\n"},"animateStroke":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation which will change stroke properties over time.</p>\n"},"animateTransform":{"!type":"fn(args: ?) -> +instance","!doc":"<p>Returns an animation which will change transformation over time.</p>\n"}},"dojox/gfx/_gfxBidiSupport.utils":{"_gfxSvgProxy":{},"_initSvgSerializerDeferred":{},"deserialize":{"!type":"fn(parent: ?, object: ?) -> ?","!doc":"<p>Takes a surface or a shape and populates it with an object produced by serialize().</p>\n"},"forEach":{"!type":"fn(object: ?, f: ?, o?: ?)","!doc":"<p>Takes a shape or a surface and applies a function &quot;f&quot; to in the context of &quot;o&quot;\n(or global, if missing). If &quot;shape&quot; was a surface or a group, it applies the same\nfunction to all children recursively effectively visiting all shapes of the underlying scene graph.</p>\n"},"fromJson":{"!type":"fn(parent: ?, json: ?) -> ?","!doc":"<p>Works just like deserialize() but takes a JSON representation of the object.</p>\n"},"serialize":{"!type":"fn(object: ?) -> ?","!doc":"<p>Takes a shape or a surface and returns an object, which describes underlying shapes.</p>\n"},"toJson":{"!type":"fn(object: ?, prettyPrint?: ?) -> string","!doc":"<p>Works just like serialize() but returns a JSON string. If prettyPrint is true, the string is pretty-printed to make it more human-readable.</p>\n"},"toSvg":{"!type":"fn(surface: ?) -> +any","!doc":"<p>Function to serialize a GFX surface to SVG text.</p>\n"}},"dojox/gfx/_gfxBidiSupport.path":{"!doc":"<p>This module contains the core graphics Path API.\nPath command format follows the W3C SVG 1.0 Path api.</p>\n","Path":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_gfxBidiSupport.Path":{"!type":"fn()","prototype":{"!doc":"<p>a generalized path shape</p>\n","_2PI":{},"_validSegments":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)","!doc":"<p>a path constructor</p>\n"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns the bounding box {x, y, width, height} or null</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/_gfxBidiSupport.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a generalized TextPath shape</p>\n","_2PI":{},"_validSegments":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)","!doc":"<p>a TextPath shape constructor</p>\n"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns the bounding box {x, y, width, height} or null</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/_gfxBidiSupport._gfxBidiSupport":{},"dojox/gfx/_gfxBidiSupport.Mover":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(shape: ?, e: ?, host?: ?)","!doc":"<p>an object, which makes a shape follow the mouse,\nused as a default mover, and as a base class for custom movers</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops the move, deletes all references, so the object can be garbage-collected</p>\n"}}},"dojox/gfx/_gfxBidiSupport.__MoveableCtorArgs":{"!type":"fn()","prototype":{"!doc":"<p>The arguments used for dojox/gfx/Moveable constructor.</p>\n","delay":{"!doc":"<p>delay move by this number of pixels</p>\n"},"mover":{"!type":"fn()","!doc":"<p>a constructor of custom Mover</p>\n"}}},"dojox/gfx/_gfxBidiSupport.Moveable":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(shape: ?, params: ?)","!doc":"<p>an object, which makes a shape moveable</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops watching for possible move, deletes all references, so the object can be garbage-collected</p>\n"}}},"dojox/gfx/_gfxBidiSupport.VectorText":{},"dojox/gfx/_gfxBidiSupport.vectorFontFitting":{"FIT":{},"FLOW":{},"NONE":{}},"dojox/gfx/_gfxBidiSupport.defaultVectorText":{"align":{},"decoration":{},"fitting":{},"height":{},"leading":{},"text":{},"type":{},"width":{},"x":{},"y":{}},"dojox/gfx/_gfxBidiSupport.defaultVectorFont":{"family":{},"size":{},"type":{}},"dojox/gfx/_gfxBidiSupport._vectorFontCache":{},"dojox/gfx/_gfxBidiSupport._svgFontCache":{},"dojox/gfx/_gfxBidiSupport.VectorFont":{"!type":"fn()","prototype":{"!doc":"<p>An implementation of the SVG Font 1.1 spec, using dojox/gfx.\n\n</p>\n<p>Basic interface:\n\n</p>\n<pre><code>var f = new gfx.Font(url|string);\nsurface||group.createVectorText(text)\n.setFill(fill)\n.setStroke(stroke)\n.setFont(fontStyleObject);</code></pre>\n<p>The arguments passed to createVectorText are the same as you would\npass to surface||group.createText; the difference is that this\nis entirely renderer-agnostic, and the return value is a subclass\nof dojox/gfx.Group.\n\n</p>\n<p>Note also that the &quot;defaultText&quot; object is slightly different:\n{ type:&quot;vectortext&quot;, x:0, y:0, width:null, height: null,\ntext: &quot;&quot;, align: &quot;start&quot;, decoration: &quot;none&quot; }\n\n</p>\n<p>...as well as the &quot;defaultVectorFont&quot; object:\n{ type:&quot;vectorfont&quot;, size:&quot;10pt&quot; }\n\n</p>\n<p>The reason for this should be obvious: most of the style for the font is defined\nby the font object itself.\n\n</p>\n<p>Note that this will only render IF and WHEN you set the font.</p>\n","_entityRe":{},"constructor":{"!type":"fn(url: ?)","!doc":"<p>Create this font object based on the SVG Font definition at url.</p>\n"},"draw":{"!type":"fn(group: ?, textArgs: ?, fontArgs: ?, fillArgs: ?, strokeArgs: ?) -> +dojox","!doc":"<p>based on the passed parameters, draw the given text using paths\ndefined by this font.</p>\n"},"getBaseline":{"!type":"fn(scale?: ?) -> number","!doc":"<p>Find the baseline coord for alignment; adjust for scale if passed.</p>\n"},"getCenterline":{"!type":"fn(scale?: ?) -> number","!doc":"<p>return the y coordinate that is the center of the viewbox.</p>\n"},"getLineHeight":{"!type":"fn(scale?: ?) -> number","!doc":"<p>return the height of a single line, sans leading, based on scale.</p>\n"},"getWidth":{"!type":"fn(text: ?, scale?: ?) -> number","!doc":"<p>Get the width of the rendered text without actually rendering it.</p>\n"},"initialized":{"!type":"fn() -> bool","!doc":"<p>Return if we&#39;ve loaded a font def, and the parsing was successful.</p>\n"},"load":{"!type":"fn(url: ?) -> fn","!doc":"<p>Load the passed SVG and send it to the parser for parsing.</p>\n"}}},"dojox/gfx/_gfxBidiSupport.arc":{"!doc":"<p>This module contains the core graphics Arc functions.</p>\n","curvePI4":{"!doc":"<p>an object with properties of an arc around a unit circle from 0 to pi/4</p>\n"},"arcAsBezier":{"!type":"fn(last: ?, rx: ?, ry: ?, xRotg: ?, large: ?, sweep: ?, x: ?, y: ?) -> []","!doc":"<p>calculates an arc as a series of Bezier curves\ngiven the last point and a standard set of SVG arc parameters,\nit returns an array of arrays of parameters to form a series of\nabsolute Bezier curves.</p>\n"},"unitArcAsBezier":{"!type":"fn(alpha: ?)","!doc":"<p>return a start point, 1st and 2nd control points, and an end point of\na an arc, which is reflected on the x axis</p>\n"}},"dojox/gfx/_gfxBidiSupport.bezierutils":{"computeLength":{"!type":"fn(points: ?) -> number","!doc":"<p>Returns the length of the given bezier curve.</p>\n"},"distance":{"!type":"fn(x1: ?, y1: ?, x2: ?, y2: ?) -> ?","!doc":"<p>Returns the distance between the specified points.</p>\n"},"splitBezierAtT":{"!type":"fn(points: ?, t: ?) -> ?"},"tAtLength":{"!type":"fn(points: ?, length: ?) -> number","!doc":"<p>Returns the t corresponding to the given length for the specified bezier curve.</p>\n"}},"dojox/gfx/_gfxBidiSupport.canvas":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of this module alone will only add in drawing support.\nThe additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.\nThe id of the Canvas renderer is &#39;canvas&#39;.  This id can be used when switch Dojo&#39;s\ngraphics context between renderer implementations.  See dojox/gfx/_base.switchRenderer\nAPI.</p>\n","attachNode":{"!type":"fn()"},"attachSurface":{"!type":"fn()"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +instance","!doc":"<p>creates a surface (Canvas)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_gfxBidiSupport.canvasWithEvents":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers which extends\nthe basic canvas drawing renderer bridge to add additional support for graphics events\non Shapes.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of the canvas module alone will only add in drawing support.\nThis additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.</p>\n","Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)","!doc":"<p>creates a surface (Canvas)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_gfxBidiSupport.canvas_attach":{},"dojox/gfx/_gfxBidiSupport.canvasext":{},"dojox/gfx/_gfxBidiSupport.gradient":{"project":{"!type":"fn(matrix: ?, gradient: ?, tl: ?, rb: ?, ttl: ?, trb: ?) -> ?","!doc":"<p>Returns a new gradient using the &quot;VML algorithm&quot; and suitable for VML.</p>\n"},"rescale":{"!type":"fn(stops: ?, from: ?, to: ?) -> []","!doc":"<p>Recalculates a gradient from 0-1 window to\n&quot;from&quot;-&quot;to&quot; window blending and replicating colors,\nif necessary.</p>\n"}},"dojox/gfx/_gfxBidiSupport.move":{},"dojox/gfx/_gfxBidiSupport.silverlight":{"!doc":"<p>This the graphics rendering bridge for the Microsoft Silverlight plugin.\nSilverlight is a faster implementation on IE6-8 than the default 2d graphics, VML</p>\n","attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)","!doc":"<p>creates a surface (Silverlight)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_gfxBidiSupport.silverlight_attach":{},"dojox/gfx/_gfxBidiSupport.svg":{"!doc":"<p>This the graphics rendering bridge for browsers compliant with W3C SVG1.0.\nThis is the preferred renderer to use for interactive and accessible graphics.</p>\n","dasharray":{},"useSvgWeb":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"getRef":{"!type":"fn(name: ?) -> +any","!doc":"<p>looks up a node by its external name</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/_gfxBidiSupport.svgext":{},"dojox/gfx/_gfxBidiSupport.vml":{"!doc":"<p>This the default graphics rendering bridge for IE6-7.\nThis renderer is very slow.  For best performance on IE6-8, use Silverlight plugin.\nIE9+ defaults to the standard W3C SVG renderer.</p>\n","_bool":{},"text_alignment":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +any","!doc":"<p>creates a surface (VML)</p>\n"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/utils":{"_gfxSvgProxy":{},"_initSvgSerializerDeferred":{},"deserialize":{"!type":"fn(parent: ?, object: ?) -> ?","!doc":"<p>Takes a surface or a shape and populates it with an object produced by serialize().</p>\n"},"forEach":{"!type":"fn(object: ?, f: ?, o?: ?)","!doc":"<p>Takes a shape or a surface and applies a function &quot;f&quot; to in the context of &quot;o&quot;\n(or global, if missing). If &quot;shape&quot; was a surface or a group, it applies the same\nfunction to all children recursively effectively visiting all shapes of the underlying scene graph.</p>\n"},"fromJson":{"!type":"fn(parent: ?, json: ?) -> ?","!doc":"<p>Works just like deserialize() but takes a JSON representation of the object.</p>\n"},"serialize":{"!type":"fn(object: ?) -> ?","!doc":"<p>Takes a shape or a surface and returns an object, which describes underlying shapes.</p>\n"},"toJson":{"!type":"fn(object: ?, prettyPrint?: ?) -> string","!doc":"<p>Works just like serialize() but returns a JSON string. If prettyPrint is true, the string is pretty-printed to make it more human-readable.</p>\n"},"toSvg":{"!type":"fn(surface: ?) -> +any","!doc":"<p>Function to serialize a GFX surface to SVG text.</p>\n"}},"dojox/gfx/path":{"!doc":"<p>This module contains the core graphics Path API.\nPath command format follows the W3C SVG 1.0 Path api.</p>\n","Path":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/path.Path":{"!type":"fn()","prototype":{"!doc":"<p>a generalized path shape</p>\n","_2PI":{},"_validSegments":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)","!doc":"<p>a path constructor</p>\n"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns the bounding box {x, y, width, height} or null</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/path.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a generalized TextPath shape</p>\n","_2PI":{},"_validSegments":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)","!doc":"<p>a TextPath shape constructor</p>\n"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns the bounding box {x, y, width, height} or null</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?)","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/string/BidiEngine":{"!type":"fn()","prototype":{"!doc":"<p>This class provides a bidi transformation engine, i.e.\nfunctions for reordering and shaping bidi text.</p>\n","bidiTransform":{"!type":"fn(text: ?, formatIn: ?, formatOut: ?) -> string","!doc":"<p>Central public API for Bidi engine. Transforms the text according to formatIn, formatOut parameters.\nIf formatIn or formatOut parametrs are not valid throws an exception.</p>\n"},"checkContextual":{"!type":"fn(text: ?) -> +any","!doc":"<p>Determine the base direction of a bidi text according\nto its first strong directional character.</p>\n"},"hasBidiChar":{"!type":"fn(text: ?) -> +any","!doc":"<p>Return true if text contains RTL directed character.</p>\n"}}},"dojox/charting/bidi/_bidiutils":{"reverseMatrix":{"!type":"fn(plot: ?, dim: ?, offsets: ?, rtl: ?)"}},"dojox/charting/bidi/Chart3D":{"!type":"fn()","prototype":{"direction":{"!doc":"<p>Mirroring support,  the main variable which is responsible for the direction of the chart.\n\n</p>\n<p>Allowed values:\n1. &quot;ltr&quot;\n2. &quot;rtl&quot;\n\n</p>\n<p>By default is ltr.</p>\n"},"isMirrored":{},"applyMirroring":{"!type":"fn(plot: ?, dim: ?, offsets: ?) -> fn","!doc":"<p>apply the mirroring operation to the current chart plots.\n</p>\n"},"generate":{"!type":"fn() -> fn"},"isRightToLeft":{"!type":"fn() -> bool","!doc":"<p>check the Direction of the chart.</p>\n"},"postscript":{"!type":"fn(node: ?, lights: ?, camera: ?, theme: ?, direction: ?)","!doc":"<p>The keyword arguments that can be passed in a Chart constructor.\n</p>\n"},"setDir":{"!type":"fn(dir: ?) -> fn","!doc":"<p>Setter for the chartBaseDirection attribute.</p>\n"}}},"dojox/charting/bidi/action2d/Tooltip":{"!type":"fn()","prototype":{}},"dojox/charting/bidi/action2d/ZoomAndPan":{"!type":"fn()","prototype":{}},"dojox/charting/bidi/axis2d/Default":{"!type":"fn()","prototype":{"labelTooltip":{"!type":"fn(elem: ?, chart: ?, label: ?, truncatedLabel: ?, font: ?, elemType: ?)"}}},"dojox/charting/bidi/widget/Chart":{"!type":"fn()","prototype":{"postMixInProperties":{"!type":"fn()"}}},"dojox/charting/bidi/widget/Legend":{"!type":"fn()","prototype":{}},"dojox/charting/plot3d/Bars":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot3d/Base.prototype","constructor":{"!type":"fn(width: ?, height: ?, kwArgs: ?)"},"generate":{"!type":"fn(chart: ?, creator: ?) -> fn"},"getDepth":{"!type":"fn() -> number"},"setData":{"!type":"fn(data: ?) -> fn"}}},"dojox/charting/plot3d/Base":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(width: ?, height: ?, kwArgs: ?)"},"generate":{"!type":"fn(chart: ?, creator: ?)"},"getDepth":{"!type":"fn() -> ?"},"setData":{"!type":"fn(data: ?) -> fn"}}},"dojox/charting/plot3d/Cylinders":{"!type":"fn()","prototype":{"!proto":"dojox/charting/plot3d/Base.prototype","constructor":{"!type":"fn(width: ?, height: ?, kwArgs: ?)"},"generate":{"!type":"fn(chart: ?, creator: ?) -> fn"},"getDepth":{"!type":"fn() -> number"},"setData":{"!type":"fn(data: ?) -> fn"}}},"dojox/charting/themes/Adobebricks":{},"dojox/charting/themes/Algae":{},"dojox/charting/themes/Bahamation":{},"dojox/charting/themes/BlueDusk":{},"dojox/charting/themes/Charged":{"next":{"!type":"fn(elementType: ?, mixin: ?, doPost: ?)"},"post":{"!type":"fn(theme: ?, elementType: ?)"}},"dojox/charting/themes/Chris":{"next":{"!type":"fn(elementType: ?, mixin: ?, doPost: ?)"},"post":{"!type":"fn(theme: ?, elementType: ?)"}},"dojox/charting/themes/Claro":{"next":{"!type":"fn(elementType: ?, mixin: ?, doPost: ?)"},"post":{"!type":"fn(theme: ?, elementType: ?)"}},"dojox/charting/themes/CubanShirts":{},"dojox/charting/themes/Desert":{},"dojox/charting/themes/Distinctive":{},"dojox/charting/themes/Dollar":{},"dojox/charting/themes/Electric":{"next":{"!type":"fn(elementType: ?, mixin: ?, doPost: ?)"},"post":{"!type":"fn(theme: ?, elementType: ?)"}},"dojox/charting/themes/Grasshopper":{},"dojox/charting/themes/Grasslands":{},"dojox/charting/themes/GreySkies":{},"dojox/charting/themes/Harmony":{},"dojox/charting/themes/IndigoNation":{},"dojox/charting/themes/Ireland":{},"dojox/charting/themes/Julie":{"next":{"!type":"fn(elementType: ?, mixin: ?, doPost: ?)"},"post":{"!type":"fn(theme: ?, elementType: ?)"}},"dojox/charting/themes/MiamiNice":{},"dojox/charting/themes/Midwest":{},"dojox/charting/themes/Minty":{},"dojox/charting/themes/PlotKit/cyan":{"chart":{},"colors":{},"plotarea":{}},"dojox/charting/themes/PlotKit/green":{"chart":{},"colors":{},"plotarea":{}},"dojox/charting/themes/PlotKit/orange":{"chart":{},"colors":{},"plotarea":{}},"dojox/charting/themes/PlotKit/purple":{"chart":{},"colors":{},"plotarea":{}},"dojox/charting/themes/PlotKit/red":{"chart":{},"colors":{},"plotarea":{}},"dojox/charting/themes/PrimaryColors":{},"dojox/charting/themes/gradientGenerator":{"Adobebricks":{},"Algae":{},"Bahamation":{},"base":{},"blue":{},"BlueDusk":{},"Charged":{},"Chris":{},"Claro":{},"CubanShirts":{},"cyan":{},"Desert":{},"Distinctive":{},"Dollar":{},"Electric":{},"Grasshopper":{},"Grasslands":{},"green":{},"GreySkies":{},"Harmony":{},"IndigoNation":{},"Ireland":{},"Julie":{},"MiamiNice":{},"Midwest":{},"Minty":{},"orange":{},"PrimaryColors":{},"purple":{},"PurpleRain":{},"red":{},"Renkoo":{},"RoyalPurples":{},"SageToLime":{},"Shrooms":{},"Tom":{},"Tufte":{},"WatersEdge":{},"Wetland":{},"generateFills":{"!type":"fn(colors: ?, fillPattern: ?, lumFrom: ?, lumTo: ?)","!doc":"<p>generates 2-color gradients using pure colors, a fill pattern, and two luminance values</p>\n"},"generateGradientByIntensity":{"!type":"fn(color: ?, intensityMap: ?)","!doc":"<p>generates gradient colors using an intensity map</p>\n"},"generateMiniTheme":{"!type":"fn(colors: ?, fillPattern: ?, lumFrom: ?, lumTo: ?, lumStroke: ?)","!doc":"<p>generates mini-themes with 2-color gradients using colors, a fill pattern, and three luminance values</p>\n"},"updateFills":{"!type":"fn(themes: ?, fillPattern: ?, lumFrom: ?, lumTo: ?)","!doc":"<p>transforms solid color fills into 2-color gradients using a fill pattern, and two luminance values</p>\n"}},"dojox/charting/themes/PurpleRain":{},"dojox/charting/themes/Renkoo":{"next":{"!type":"fn(elementType: ?, mixin: ?, doPost: ?)"},"post":{"!type":"fn(theme: ?, elementType: ?)"}},"dojox/charting/themes/RoyalPurples":{},"dojox/charting/themes/SageToLime":{},"dojox/charting/themes/Shrooms":{},"dojox/charting/themes/ThreeD":{"series":{},"next":{"!type":"fn(elementType: ?, mixin: ?, doPost: ?)"}},"dojox/charting/themes/Tom":{"next":{"!type":"fn(elementType: ?, mixin: ?, doPost: ?)"},"post":{"!type":"fn(theme: ?, elementType: ?)"}},"dojox/charting/themes/WatersEdge":{},"dojox/charting/themes/Wetland":{},"dojox/charting/widget/BidiSupport":{},"dojox/charting/widget/Chart":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A chart widget.  This is leveraging dojox/charting/Chart as a Dijit widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"margins":{"!doc":"<p>The margins around the chart. Default is { l:10, t:10, r:10, b:10 }.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"theme":{"!doc":"<p>An optional theme to use for styling the chart.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>properly destroy the widget</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"fill":{"!type":"fn() -> ?","!doc":"<p>The color for the chart.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"resize":{"!type":"fn(box: ?)","!doc":"<p>Resize the widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"stroke":{"!type":"fn() -> ?","!doc":"<p>The outline of the chart (stroke in vector graphics terms).</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/charting/widget/Chart2D":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A chart widget.  This is leveraging dojox/charting/Chart as a Dijit widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"margins":{"!doc":"<p>The margins around the chart. Default is { l:10, t:10, r:10, b:10 }.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"theme":{"!doc":"<p>An optional theme to use for styling the chart.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>properly destroy the widget</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"fill":{"!type":"fn() -> ?","!doc":"<p>The color for the chart.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"resize":{"!type":"fn(box: ?)","!doc":"<p>Resize the widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"stroke":{"!type":"fn() -> ?","!doc":"<p>The outline of the chart (stroke in vector graphics terms).</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/charting/widget/Legend":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A legend for a chart. A legend contains summary labels for\neach series of data contained in the chart.\n\n</p>\n<p>Set the horizontal attribute to boolean false to layout legend labels vertically.\nSet the horizontal attribute to a number to layout legend labels in horizontal\nrows each containing that number of labels (except possibly the last row).\n\n</p>\n<p>(Line or Scatter charts (colored lines with shape symbols) )\n-o- Series1     -X- Series2     -v- Series3\n\n</p>\n<p>(Area/Bar/Pie charts (letters represent colors))\n[a] Series1     [b] Series2     [c] Series3</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"chartRef":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"horizontal":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"legendBody":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"swatchSize":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn()","!doc":"<p>regenerates the legend to reflect changes to the chart</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/charting/widget/SelectableLegend":{"!type":"fn()","prototype":{"!proto":"dojox/charting/widget/Legend.prototype","!doc":"<p>An enhanced chart legend supporting interactive events on data series</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"chartRef":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"horizontal":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"legendBody":{},"outline":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"swatchSize":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transitionFill":{},"transitionStroke":{},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/charting/widget/Sparkline":{},"dojox/collections/ArrayList":{"!doc":"<p>Returns a new object of type dojox.collections.ArrayList</p>\n","prototype":{"constructor":{"!type":"fn(arr?: ?)","!doc":"<p>Returns a new object of type dojox.collections.ArrayList</p>\n"}}},"dojox/collections/_base":{"Set":{},"ArrayList":{"!type":"fn(arr?: ?)","!doc":"<p>Returns a new object of type dojox.collections.ArrayList</p>\n"},"BinaryTree":{"!type":"fn(data: ?)"},"Dictionary":{"!type":"fn(dictionary?: ?)","!doc":"<p>Returns an object of type dojox.collections.Dictionary</p>\n"},"DictionaryEntry":{"!type":"fn(k: ?, v: ?)","!doc":"<p>return an object of type dojox.collections.DictionaryEntry</p>\n"},"DictionaryIterator":{"!type":"fn(obj: ?)","!doc":"<p>return an object of type dojox.collections.DictionaryIterator</p>\n"},"Iterator":{"!type":"fn(a: ?)","!doc":"<p>return an object of type dojox.collections.Iterator</p>\n"},"Queue":{"!type":"fn(arr?: ?)","!doc":"<p>return an object of type dojox.collections.Queue</p>\n"},"SortedList":{"!type":"fn(dictionary?: ?)","!doc":"<p>creates a collection that acts like a dictionary but is also internally sorted.\nNote that the act of adding any elements forces an internal resort, making this object potentially slow.</p>\n"},"Stack":{"!type":"fn(arr?: ?)","!doc":"<p>returns an object of type dojox.collections.Stack</p>\n"}},"dojox/collections/BinaryTree":{"TraversalMethods":{},"prototype":{"constructor":{"!type":"fn(data: ?)"}}},"dojox/collections/BinaryTree.TraversalMethods":{"Inorder":{},"Postorder":{},"Preorder":{}},"dojox/collections/Dictionary":{"!doc":"<p>Returns an object of type dojox.collections.Dictionary</p>\n","prototype":{"constructor":{"!type":"fn(dictionary?: ?)","!doc":"<p>Returns an object of type dojox.collections.Dictionary</p>\n"}}},"dojox/collections/Queue":{"!doc":"<p>return an object of type dojox.collections.Queue</p>\n","prototype":{"constructor":{"!type":"fn(arr?: ?)","!doc":"<p>return an object of type dojox.collections.Queue</p>\n"}}},"dojox/collections/Set":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/collections/SortedList":{"!doc":"<p>creates a collection that acts like a dictionary but is also internally sorted.\nNote that the act of adding any elements forces an internal resort, making this object potentially slow.</p>\n","prototype":{"constructor":{"!type":"fn(dictionary?: ?)","!doc":"<p>creates a collection that acts like a dictionary but is also internally sorted.\nNote that the act of adding any elements forces an internal resort, making this object potentially slow.</p>\n"}}},"dojox/collections/Stack":{"!doc":"<p>returns an object of type dojox.collections.Stack</p>\n","prototype":{"constructor":{"!type":"fn(arr?: ?)","!doc":"<p>returns an object of type dojox.collections.Stack</p>\n"}}},"dojox/collections":{"!doc":"<p>Deprecated.  Should require dojox/collections modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/color/Colorspace":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/math/matrix":{},"dojox/color/MeanColorModel":{"!type":"fn()","prototype":{"!proto":"dojox/color/NeutralColorModel.prototype","!doc":"<p>A color model that returns a color from a data value\nusing an interpolation between two extremum colors around the mean value.</p>\n","_e":{},"_endColor":{},"_max":{},"_min":{},"_startColor":{},"computeNeutral":{"!type":"fn(min: ?, max: ?, sum: ?, values: ?) -> ?","!doc":"<p>Return the neutral value in this case the mean value of the data values.</p>\n"},"constructor":{"!type":"fn(startColor: ?, endColor?: ?)"},"getColor":{"!type":"fn(value: ?) -> ?","!doc":"<p>return the color for a given data value.</p>\n"},"getNormalizedValue":{"!type":"fn(value: ?) -> number","!doc":"<p>Return the normalized (between 0 and 1) value for a given data value.\nThis implementation uses an power function to map neutral value to 0.5\nand distribute other values around it.</p>\n"},"initialize":{"!type":"fn(items: ?, colorFunc: ?)","!doc":"<p>Initialize the color model from a list of data items and using a function\nthat returns the value used to compute the color for a given item.</p>\n"}}},"dojox/color/NeutralColorModel":{"!type":"fn()","prototype":{"!proto":"dojox/color/SimpleColorModel.prototype","!doc":"<p>Base class for color models that return a color from a data value\nusing an interpolation between two extremum colors around a neutral value.</p>\n","_e":{},"_endColor":{},"_max":{},"_min":{},"_startColor":{},"computeNeutral":{"!type":"fn(min: ?, max: ?, sum: ?, values: ?)","!doc":"<p>Return the neutral value. This can be for example the mean or average value.\nThis function must be implemented by implementations.</p>\n"},"constructor":{"!type":"fn(startColor: ?, endColor?: ?)"},"getColor":{"!type":"fn(value: ?) -> ?","!doc":"<p>return the color for a given data value.</p>\n"},"getNormalizedValue":{"!type":"fn(value: ?) -> number","!doc":"<p>Return the normalized (between 0 and 1) value for a given data value.\nThis implementation uses an power function to map neutral value to 0.5\nand distribute other values around it.</p>\n"},"initialize":{"!type":"fn(items: ?, colorFunc: ?)","!doc":"<p>Initialize the color model from a list of data items and using a function\nthat returns the value used to compute the color for a given item.</p>\n"}}},"dojox/color/SimpleColorModel":{"!type":"fn()","prototype":{"!doc":"<p>Base class for color models that return a color from a data value\nusing an interpolation between two extremum colors.</p>\n","_endColor":{},"_startColor":{},"constructor":{"!type":"fn(startColor: ?, endColor?: ?)","!doc":"<p>Construct a color model interpolating between start and end color.\nIf only start color is provided use it to compute reasonable start and end\ncolors from it.</p>\n"},"getColor":{"!type":"fn(value: ?) -> ?","!doc":"<p>return the color for a given data value.</p>\n"},"getNormalizedValue":{"!type":"fn(value: ?)","!doc":"<p>Return the normalized (between 0 and 1) value for a given data value.\nThis function must be implemented by implementations.</p>\n"}}},"dojox/color":{"!doc":"<p>Deprecated.  Should require dojox/color modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/color/api/ColorModel":{"!type":"fn()","prototype":{"!doc":"<p>API for classes that implement a color model that returns a color from a data value.</p>\n","constructor":{"!type":"fn()","!doc":"<p>Constructor.</p>\n"},"getColor":{"!type":"fn(value: ?)","!doc":"<p>return the color for a given data value.</p>\n"},"initialize":{"!type":"fn(items: ?, colorFunc: ?)","!doc":"<p>Optionally initialize the color model from a list of data items and using a function\nthat returns the value used to compute the color for a given item.</p>\n"}}},"dojox/css3/fx":{"!doc":"<p>Utilities for animation effects.</p>\n","bounce":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that does a &quot;bounce&quot; effect on args.node.\n</p>\n"},"expand":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that expands args.node.\n</p>\n"},"flip":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that flips an element around his y axis.\n</p>\n"},"puff":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that will do a &quot;puff&quot; effect on the given node.\n</p>\n"},"rotate":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that rotates an element.\n</p>\n"},"shrink":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns an animation that shrinks args.node.\n</p>\n"}},"dojox/html/ext-dojo/style":{},"dojox/fx/ext-dojo/complex":{"!doc":"<p>Extends dojo/_base/fx.animateProperty to animate a &quot;complex property&quot;. The primary example is the\nclip style: rect(10px 30px 10px 50px).\nNote this can also be used with (and is actually intended for)\nCSS3 properties, such as transform:\ntransform: rotate(10deg) translateX(0px)</p>\n"},"dojox/css3/transit":{"!doc":"<p>Performs a transition to hide a node and show another node.</p>\n","prototype":{"constructor":{"!type":"fn(from: ?, to: ?, options?: ?)","!doc":"<p>Performs a transition to hide a node and show another node.</p>\n"}}},"dojox/css3/transition":{"!doc":"<p>This module defines the transition utilities which can be used\nto perform transition effects based on the CSS Transition standard.</p>\n","autoClear":{},"deferred":{},"direction":{},"duration":{},"endState":{},"in":{},"node":{},"playing":{},"startState":{},"prototype":{"beforeClear":{"!type":"fn()","!doc":"<p>The callback which will be called right after the end\nof the transition effect and before the final state is\ncleared.</p>\n"},"beforeStart":{"!type":"fn()","!doc":"<p>The callback which will be called right before the start\nof the transition effect.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Method to clear the state after a transition.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>This module defines the transition utilities which can be used\nto perform transition effects based on the CSS Transition standard.</p>\n"},"initState":{"!type":"fn()","!doc":"<p>Method to initialize the state for a transition.</p>\n"},"play":{"!type":"fn()","!doc":"<p>Plays the transition effect defined by this transition object.</p>\n"},"start":{"!type":"fn()","!doc":"<p>Method to start the transition.</p>\n"}},"chainedPlay":{"!type":"fn(args: ?)","!doc":"<p>The method which plays multiple transitions one by one.</p>\n"},"fade":{"!type":"fn(node: ?, config: ?)","!doc":"<p>Method which is used to create the transition object of fade effect.</p>\n"},"flip":{"!type":"fn(node: ?, config: ?)","!doc":"<p>Method which is used to create the transition object of flip effect.</p>\n"},"getWaitingList":{"!type":"fn(nodes: ?) -> ?"},"groupedPlay":{"!type":"fn(args: ?) -> ?","!doc":"<p>The method which groups multiple transitions and plays \nthem together.</p>\n"},"slide":{"!type":"fn(node: ?, config: ?)","!doc":"<p>Method which is used to create the transition object of a slide effect.</p>\n"}},"dojox/css3/transition.startState":{},"dojox/css3/transition.endState":{},"dojox/css3/transition.playing":{},"dojox/data/AndOrReadStore":{"!type":"fn()","prototype":{"!proto":"dojo/data/ItemFileReadStore.prototype","!doc":"<p>AndOrReadStore uses ItemFileReadStore as a base, modifying only the query (_fetchItems) section.\nSupports queries of the form: query:&quot;id:1<em> OR dept:&#39;Sales Department&#39; || (id:2</em> &amp;&amp; NOT dept:S*)&quot;\nIncludes legacy/widget support via:\n\n</p>\n<pre><code>query:{complexQuery:&quot;id:1* OR dept:&#39;Sales Department&#39; || (id:2* &amp;&amp; NOT dept:S*)&quot;}</code></pre>\n<p>The ItemFileReadStore implements the dojo/data/api/Read API and reads\n\n</p>\n<p>data from JSON files that have contents in this format --\n\n</p>\n<pre><code>{ items: [\n    { name:&#39;Kermit&#39;, color:&#39;green&#39;, age:12, friends:[&#39;Gonzo&#39;, {_reference:{name:&#39;Fozzie Bear&#39;}}]},\n    { name:&#39;Fozzie Bear&#39;, wears:[&#39;hat&#39;, &#39;tie&#39;]},\n    { name:&#39;Miss Piggy&#39;, pets:&#39;Foo-Foo&#39;}\n]}</code></pre>\n<p>Note that it can also contain an &#39;identifier&#39; property that specified which attribute on the items\n\n</p>\n<p>in the array of items that acts as the unique identifier for that item.</p>\n","_ccUrl":{},"clearOnClose":{"!doc":"<p>Parameter to allow users to specify if a close call should force a reload or not.\nBy default, it retains the old behavior of not clearing if close is called.  But\nif set true, the store will be reset to default state.  Note that by doing this,\nall item handles will become invalid and a new fetch must be issued.</p>\n"},"data":{},"failOk":{"!doc":"<p>Parameter for specifying that it is OK for the xhrGet call to fail silently.</p>\n"},"hierarchical":{"!doc":"<p>Parameter to indicate to process data from the url as hierarchical\n(data items can contain other data items in js form).  Default is true\nfor backwards compatibility.  False means only root items are processed\nas items, all child objects outside of type-mapped objects and those in\nspecific reference format, are left straight JS data objects.</p>\n"},"typeMap":{},"url":{},"urlPreventCache":{"!doc":"<p>Parameter to allow specifying if preventCache should be passed to the xhrGet call or not when loading data from a url.\nNote this does not mean the store calls the server on each fetch, only that the data load has preventCache set as an option.\nAdded for tracker: #6072</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(keywordParameters: ?)","!doc":"<p>constructor</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"filter":{"!type":"fn(requestArgs: ?, arrayOfItems: ?, findCallback: ?)"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Identity.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +value","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"}}},"dojox/data/AndOrWriteStore":{"!type":"fn()","prototype":{"!proto":"dojo/data/ItemFileWriteStore.prototype","_ccUrl":{},"clearOnClose":{"!doc":"<p>Parameter to allow users to specify if a close call should force a reload or not.\nBy default, it retains the old behavior of not clearing if close is called.  But\nif set true, the store will be reset to default state.  Note that by doing this,\nall item handles will become invalid and a new fetch must be issued.</p>\n"},"data":{},"failOk":{"!doc":"<p>Parameter for specifying that it is OK for the xhrGet call to fail silently.</p>\n"},"hierarchical":{"!doc":"<p>Parameter to indicate to process data from the url as hierarchical\n(data items can contain other data items in js form).  Default is true\nfor backwards compatibility.  False means only root items are processed\nas items, all child objects outside of type-mapped objects and those in\nspecific reference format, are left straight JS data objects.</p>\n"},"referenceIntegrity":{},"typeMap":{},"url":{},"urlPreventCache":{"!doc":"<p>Parameter to allow specifying if preventCache should be passed to the xhrGet call or not when loading data from a url.\nNote this does not mean the store calls the server on each fetch, only that the data load has preventCache set as an option.\nAdded for tracker: #6072</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>Over-ride of base close function of ItemFileReadStore to add in check for store state.</p>\n"},"constructor":{"!type":"fn(keywordParameters: ?)"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"deleteItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Write.deleteItem()</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"filter":{"!type":"fn(requestArgs: ?, arrayOfItems: ?, findCallback: ?)","!doc":"<p>This method handles the basic filtering needs for ItemFile* based stores.</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Identity.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +value","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isDirty":{"!type":"fn(item?: ?) -> ?","!doc":"<p>See dojo/data/api/Write.isDirty()</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"},"newItem":{"!type":"fn(keywordArgs?: ?, parentInfo?: ?) -> ?","!doc":"<p>See dojo/data/api/Write.newItem()</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"revert":{"!type":"fn() -> bool","!doc":"<p>See dojo/data/api/Write.revert()</p>\n"},"save":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Write.save()</p>\n"},"setValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Write.set()</p>\n"},"setValues":{"!type":"fn(item: ?, attribute: ?, values: ?) -> ?","!doc":"<p>See dojo/data/api/Write.setValues()</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Write.unsetAttribute()</p>\n"}}},"dojox/data/AppStore":{"!type":"fn()","prototype":{"_adds":{},"_atomIO":{},"_deletes":{},"_feed":{},"_processing":{},"_requests":{},"_updates":{},"url":{"!doc":"<p>So the parser can instantiate the store via markup.</p>\n"},"urlPreventCache":{"!doc":"<p>Whether or not to pass the preventCache parameter to the connection</p>\n"},"xmethod":{"!doc":"<p>Whether to use X-Method-Override for PUT/DELETE.</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>The APP data store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"deleteItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Write.deleteItem()</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Identity.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +value","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isDirty":{"!type":"fn(item?: ?) -> bool","!doc":"<p>See dojo/data/api/Write.isDirty()</p>\n"},"isItem":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"},"newItem":{"!type":"fn(keywordArgs?: ?) -> bool","!doc":"<p>See dojo/data/api/Write.newItem()</p>\n"},"revert":{"!type":"fn() -> bool","!doc":"<p>See dojo/data/api/Write.revert()</p>\n"},"save":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Write.save()</p>\n"},"setValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Write.setValue()</p>\n"},"setValues":{"!type":"fn(item: ?, attribute: ?, values: ?) -> ?","!doc":"<p>See dojo/data/api/Write.setValues()</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Write.unsetAttribute()</p>\n"}}},"dojox/data/AtomReadStore":{"!type":"fn()","prototype":{"!doc":"<p>A read only data store for Atom XML based services or documents</p>\n","label":{},"sendQuery":{},"unescapeHTML":{},"url":{},"urlPreventCache":{"!doc":"<p>Configurable preventCache option for the URL.</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Constructor for the AtomRead store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> +any","!doc":"<p>Check whether the attribute values contain the value</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> +any","!doc":"<p>Return an array of attribute names</p>\n"},"getFeatures":{"!type":"fn() -> +any","!doc":"<p>Return supported data APIs</p>\n"},"getFeedValue":{"!type":"fn(attribute: ?, defaultValue: ?) -> ?","!doc":"<p>Non-API method for retrieving values regarding the Atom feed,\nrather than the Atom entries.</p>\n"},"getFeedValues":{"!type":"fn(attribute: ?, defaultValue: ?) -> ?","!doc":"<p>Non-API method for retrieving values regarding the Atom feed,\nrather than the Atom entries.</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +any","!doc":"<p>Return an attribute value</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> +any","!doc":"<p>Return an attribute value</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> +any","!doc":"<p>Check whether an element has the attribute</p>\n"},"isItem":{"!type":"fn(something: ?) -> +any","!doc":"<p>Check whether the object is an item (XML element)</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> +any","!doc":"<p>Check whether the object is an item (XML element) and loaded</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Load an item (XML element)</p>\n"}}},"dojox/data/CdfStore":{"!type":"fn()","prototype":{"!doc":"<p>IMPORTANT: The CDF Store is designed to work with Tibco GI, and references Tibco&#39;s\nJSX3 JavaScript library and will not work without it.\n\n</p>\n<p>The CDF Store implements dojo.data.Read, Write, and Identity api&#39;s.  It is a local\n(in memory) store that handles XML documents formatted according to the\nCommon Data Format (CDF) spec:\n<a href=\"http://www.tibco.com/devnet/resources/gi/3_1/tips_and_techniques/CommonDataFormatCDF.pdf\">http://www.tibco.com/devnet/resources/gi/3_1/tips_and_techniques/CommonDataFormatCDF.pdf</a>\n\n</p>\n<p>The purpose of this store is to provide a glue between a jsx3 CDF file and a Dijit.\n\n</p>\n<p>While a CDF document is an XML file, other than the initial input, all data returned\nfrom and written to this store should be in object format.</p>\n","data":{"!doc":"<p>A object that will be converted into the xmlStr property, and then parsed into a CDF.</p>\n"},"identity":{"!doc":"<p>The unique identifier for each item. Defaults to &quot;jsxid&quot; which is standard for a CDF\ndocument. Should not be changed.</p>\n"},"label":{"!doc":"<p>The property within each item used to define the item.</p>\n"},"mode":{},"url":{},"xmlStr":{"!doc":"<p>A string that can be parsed into an XML document and should be formatted according\nto the CDF spec.</p>\n"},"byId":{"!type":"fn(args: ?)"},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Constructor for the CDF store. Instantiate a new CdfStore.</p>\n"},"containsValue":{"!type":"fn(item: ?, property: ?, value: ?) -> bool","!doc":"<p>Check whether an item contains a value</p>\n"},"deleteItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>Delete an jsx3.xml.Entity (wrapper to a XML element).</p>\n"},"fetch":{"!type":"fn(request?: ?) -> bool","!doc":"<p>Returns an Array of items based on the request arguments.</p>\n"},"fetchItemByIdentity":{"!type":"fn(args: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity(keywordArgs).\n\n</p>\n<p>Note:\nThis method can be synchronous if mode is set.\nAlso, there is a more finger friendly alias of this method, byId();</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>Return an array of property names</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>Return supported data APIs</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the identifier for an item.</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>Returns the property used for the identity.</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns an array of what properties of the item that were used\nto generate its label\nSee dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, property: ?, defaultValue?: ?) -> ?","!doc":"<p>Return an property value of an item</p>\n"},"getValues":{"!type":"fn(item: ?, property: ?) -> []","!doc":"<p>Return an array of values</p>\n"},"hasAttribute":{"!type":"fn(item: ?, property: ?) -> bool","!doc":"<p>Check whether an item has a property</p>\n"},"hasProperty":{"!type":"fn(item: ?, property: ?) -> ?","!doc":"<p>Alias for hasAttribute</p>\n"},"isDirty":{"!type":"fn(item?: ?) -> bool","!doc":"<p>Check whether an item is new, modified or deleted.\nIf no item is passed, checks if anything in the store has changed.</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool","!doc":"<p>Check whether the object is an item (jsx3.xml.Entity)</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>Check whether the object is a jsx3.xml.Entity object and loaded</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Load an item</p>\n"},"newItem":{"!type":"fn(keywordArgs?: ?, parentInfo?: ?) -> ?","!doc":"<p>Creates a jsx3.xml.Entity item and inserts it either inside the\nparent or appends it to the root</p>\n"},"revert":{"!type":"fn() -> bool","!doc":"<p>Invalidate changes (new and/or modified elements)\nResets data by simply deleting the reference to the cdfDoc.\nSubsequent fetches will load the new data.\n\n</p>\n<p>Note:\nAny items outside the store will no longer be valid and may cause errors.</p>\n"},"setValue":{"!type":"fn(item: ?, property: ?, value: ?) -> bool","!doc":"<p>Set an property value</p>\n"},"setValues":{"!type":"fn(item: ?, property: ?, values: ?) -> ?","!doc":"<p>Set property values.</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, property: ?) -> bool","!doc":"<p>Remove an property</p>\n"}}},"dojox/data/ClientFilter":{"!type":"fn()","prototype":{"cacheByDefault":{},"serverVersion":{},"cachingFetch":{"!type":"fn(args: ?) -> ?"},"clearCache":{"!type":"fn()","!doc":"<p>Clears the cache of client side queries</p>\n"},"clientSideFetch":{"!type":"fn(request: ?, baseResults: ?) -> ?","!doc":"<p>Performs a query on the client side and returns the results as an array</p>\n"},"clientSidePaging":{"!type":"fn(request: ?, baseResults: ?) -> []"},"constructor":{"!type":"fn()","!doc":"<p>This is an abstract class that data stores can extend to add updateable result set functionality\nas well as client side querying capabilities. This enables\nwidgets to be aware of how active results change in response to the modifications/notifications.</p>\n"},"isUpdateable":{"!type":"fn(request: ?) -> bool","!doc":"<p>Returns whether the provide fetch arguments can be used to update an existing list</p>\n"},"makeComparator":{"!type":"fn(sort: ?) -> fn","!doc":"<p>returns a comparator function for the given sort order array</p>\n"},"matchesQuery":{"!type":"fn(item: ?, request: ?) -> bool"},"querySuperSet":{"!type":"fn(argsSuper: ?, argsSub: ?) -> ?","!doc":"<p>Determines whether the provided arguments are super/sub sets of each other</p>\n"},"updateResultSet":{"!type":"fn(resultSet: ?, request: ?) -> +any","!doc":"<p>Attempts to update the given result set based on previous notifications</p>\n"}}},"dojox/data/CouchDBRestStore":{"!type":"fn()","prototype":{"!doc":"<p>A CouchDBRestStore is an extension of JsonRestStore to handle CouchDB&#39;s idiosyncrasies, special features,\nand deviations from standard HTTP Rest.\nNOTE: CouchDB is not designed to be run on a public facing network. There is no access control\non database documents, and you should NOT rely on client side control to implement security.</p>\n","fetch":{"!type":"fn(args: ?) -> ?","!doc":"<p>This only differs from JsonRestStore in that it, will put the query string the query part of the URL and it handles start and count</p>\n"},"save":{"!type":"fn(kwArgs: ?)"}},"getStores":{"!type":"fn(couchServerUrl: ?)"}},"dojox/data/JsonRestStore":{"!type":"fn()","prototype":{"!proto":"dojox/data/ServiceStore.prototype","_currentId":{},"allowNoTrailingSlash":{"!doc":"<p>Allow no trailing slash on target paths. This is generally discouraged since\nit creates prevents simple scalar values from being used a relative URLs.\nDisabled by default.</p>\n"},"estimateCountFactor":{"!doc":"<p>This parameter is used by the ServiceStore to estimate the total count. When\npaging is indicated in a fetch and the response includes the full number of items\nrequested by the fetch&#39;s count parameter, then the total count will be estimated\nto be estimateCountFactor multiplied by the provided count. If this is 1, then it is assumed that the server\ndoes not support paging, and the response is the full set of items, where the\ntotal count is equal to the numer of items returned. If the server does support\npaging, an estimateCountFactor of 2 is a good value for estimating the total count\nIt is also possible to override _processResults if the server can provide an exact\ntotal count.    </p>\n"},"idAsRef":{"!doc":"<p>Treat objects in queries as partially loaded objects</p>\n"},"idAttribute":{"!doc":"<p>Defaults to &#39;id&#39;. The name of the attribute that holds an objects id.\nThis can be a preexisting id provided by the server.\nIf an ID isn&#39;t already provided when an object\nis fetched or added to the store, the autoIdentity system\nwill generate an id for it and add it to the index.</p>\n"},"labelAttribute":{},"loadLazyValues":{},"loadReferencedSchema":{"!doc":"<p>Will load any schemas referenced content-type header or in Link headers</p>\n"},"referenceIntegrity":{},"schema":{"!doc":"<p>This is a schema object for this store. This should be JSON Schema format.</p>\n"},"service":{"!doc":"<p>This is the service object that is used to retrieve lazy data and save results\nThe function should be directly callable with a single parameter of an object id to be loaded</p>\n"},"syncMode":{"!doc":"<p>Setting this to true will set the store to using synchronous calls by default.\nSync calls return their data immediately from the calling function, so\ncallbacks are unnecessary. This will only work with a synchronous capable service.</p>\n"},"target":{},"cancelChanging":{"!type":"fn(object: ?)","!doc":"<p>Removes an object from the list of dirty objects\nThis will prevent that object from being saved to the server on the next save</p>\n"},"changing":{"!type":"fn(item: ?, _deleting: ?)","!doc":"<p>adds an item to the list of dirty items.    This item\ncontains a reference to the item itself as well as a\ncloned and trimmed version of old item for use with\nrevert.</p>\n"},"close":{"!type":"fn(request: ?) -> ?"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>JsonRestStore is a Dojo Data store interface to JSON HTTP/REST web\nstorage services that support read and write through GET, PUT, POST, and DELETE.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>Checks to see if &#39;item&#39; has &#39;value&#39; at &#39;attribute&#39;</p>\n"},"deleteItem":{"!type":"fn(item: ?)","!doc":"<p>deletes item and any references to that item from the store.</p>\n"},"fetch":{"!type":"fn(args: ?) -> string","!doc":"<p>See dojo/data/api/Read.fetch</p>\n"},"fetchItemByIdentity":{"!type":"fn(args: ?) -> ?"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>Gets the available attributes of an item&#39;s &#39;property&#39; and returns\nit as an array.</p>\n"},"getConstructor":{"!type":"fn() -> ?","!doc":"<p>Gets the constructor for objects from this store</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>return the store feature set</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns the attributes which are used to make up the\nidentity of an item.    Basically returns this.idAttribute</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>returns the label for an item. Just gets the &quot;label&quot; attribute.</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns an array of attributes that are used to create the label of an item</p>\n"},"getParent":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the parent item (or query) for the given item</p>\n"},"getSchema":{"!type":"fn() -> ?","!doc":"<p>Returns a reference to the JSON Schema</p>\n"},"getValue":{"!type":"fn(item: ?, property: ?, defaultValue?: ?)","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39;</p>\n"},"getValues":{"!type":"fn(item: ?, property: ?) -> []","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39; and returns\nit. If this value is an array it is just returned,\nif not, the value is added to an array and that is returned.</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>Checks to see if item has attribute</p>\n"},"isDirty":{"!type":"fn(item: ?) -> ?","!doc":"<p>returns true if the item is marked as dirty.</p>\n"},"isItem":{"!type":"fn(item: ?, anyStore: ?) -> ?","!doc":"<p>Checks to see if a passed &#39;item&#39;\nreally belongs to this JsonRestStore.</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>Checks to see if the item is loaded.</p>\n"},"loadItem":{"!type":"fn(args: ?) -> ?","!doc":"<p>Loads an item and calls the callback handler. Note, that this will call the callback\nhandler even if the item is loaded. Consequently, you can use loadItem to ensure\nthat an item is loaded is situations when the item may or may not be loaded yet.\nIf you access a value directly through property access, you can use this to load\na lazy value as well (doesn&#39;t need to be an item).</p>\n"},"newItem":{"!type":"fn(data: ?, parentInfo: ?) -> +instance","!doc":"<p>adds a new item to the store at the specified point.\nTakes two parameters, data, and options.</p>\n"},"revert":{"!type":"fn(kwArgs: ?)","!doc":"<p>returns any modified data to its original state prior to a save();</p>\n"},"save":{"!type":"fn(kwArgs: ?) -> ?","!doc":"<p>Saves the dirty data using REST Ajax methods. See dojo/data/api/Write for API.</p>\n"},"setValue":{"!type":"fn(item: ?, attribute: ?, value: ?)","!doc":"<p>sets &#39;attribute&#39; on &#39;item&#39; to &#39;value&#39;</p>\n"},"setValues":{"!type":"fn(item: ?, attribute: ?, values: ?)","!doc":"<p>sets &#39;attribute&#39; on &#39;item&#39; to &#39;value&#39; value\nmust be an array.</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, attribute: ?)","!doc":"<p>unsets &#39;attribute&#39; on &#39;item&#39;</p>\n"}},"getStore":{"!type":"fn(options: ?, Class: ?)","!doc":"<p>Will retrieve or create a store using the given options (the same options\nthat are passed to JsonRestStore constructor. Returns a JsonRestStore instance</p>\n"}},"dojox/rpc/Rest":{"!doc":"<p>This provides a HTTP REST service with full range REST verbs include PUT,POST, and DELETE.</p>\n","_index":{},"_timeStamps":{},"prototype":{"constructor":{"!type":"fn(path: ?, isJson?: ?, schema?: ?, getRequest?: ?) -> fn","!doc":"<p>This provides a HTTP REST service with full range REST verbs include PUT,POST, and DELETE.</p>\n"}}},"dojox/rpc/Rest._index":{},"dojox/rpc/Rest._timeStamps":{},"dojox/rpc/JsonRest":{"_saveNotNeeded":{},"conflictDateHeader":{},"schemas":{},"services":{},"byId":{"!type":"fn(service: ?, id: ?) -> +instance"},"changing":{"!type":"fn(object: ?, _deleting: ?)","!doc":"<p>adds an object to the list of dirty objects.  This object\ncontains a reference to the object itself as well as a\ncloned and trimmed version of old object for use with\nrevert.</p>\n"},"commit":{"!type":"fn(kwArgs: ?) -> []","!doc":"<p>Saves the dirty data using REST Ajax methods</p>\n"},"deleteObject":{"!type":"fn(object: ?)","!doc":"<p>deletes an object</p>\n"},"fetch":{"!type":"fn(absoluteId: ?) -> ?","!doc":"<p>Fetches a resource by an absolute path/id and returns a dojo.Deferred.</p>\n"},"getConstructor":{"!type":"fn(service: ?, schema: ?) -> ?","!doc":"<p>Creates or gets a constructor for objects from this service</p>\n"},"getDirtyObjects":{"!type":"fn() -> []"},"getIdAttribute":{"!type":"fn(service: ?) -> string","!doc":"<p>Return the ids attribute used by this service (based on it&#39;s schema).\nDefaults to &quot;id&quot;, if not other id is defined</p>\n"},"getServiceAndId":{"!type":"fn(absoluteId: ?) -> ?","!doc":"<p>Returns the REST service and the local id for the given absolute id. The result\nis returned as an object with a service property and an id property</p>\n"},"isDirty":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>returns true if the item is marked as dirty or true if there are any dirty items</p>\n"},"query":{"!type":"fn(service: ?, id: ?, args: ?) -> ?"},"registerService":{"!type":"fn(service: ?, servicePath: ?, schema?: ?)","!doc":"<p>Registers a service for as a JsonRest service, mapping it to a path and schema</p>\n"},"revert":{"!type":"fn(service: ?)","!doc":"<p>Reverts all the changes made to JSON/REST data</p>\n"},"sendToServer":{"!type":"fn(actions: ?, kwArgs: ?)"},"serviceClass":{"!type":"fn(path: ?, isJson?: ?, schema?: ?, getRequest?: ?) -> fn","!doc":"<p>This provides a HTTP REST service with full range REST verbs include PUT,POST, and DELETE.</p>\n"}},"dojox/rpc/JsonRest.services":{},"dojox/rpc/JsonRest.schemas":{},"dojox/json/ref":{"!doc":"<p>Adds advanced JSON {de}serialization capabilities to the base json library.\nThis enhances the capabilities of dojo.toJson and dojo.fromJson,\nadding referencing support, date handling, and other extra format handling.\nOn parsing, references are resolved. When references are made to\nids/objects that have been loaded yet, the loader function will be set to\n_loadObject to denote a lazy loading (not loaded yet) object.</p>\n","_useRefs":{},"refAttribute":{},"serializeFunctions":{},"fromJson":{"!type":"fn(str: ?, args?: ?) -> +any","!doc":"<p>evaluates the passed string-form of a JSON object.</p>\n"},"resolveJson":{"!type":"fn(root: ?, args?: ?) -> +any","!doc":"<p>Indexes and resolves references in the JSON object.</p>\n"},"toJson":{"!type":"fn(it: ?, prettyPrint?: ?, idPrefix?: ?, indexSubObjects?: ?) -> +any","!doc":"<p>Create a JSON serialization of an object.\nThis has support for referencing, including circular references, duplicate references, and out-of-message references\nid and path-based referencing is supported as well and is based on <a href=\"http://www.json.com/2007/10/19/json-referencing-proposal-and-library/\">http://www.json.com/2007/10/19/json-referencing-proposal-and-library/</a>.</p>\n"}},"dojox/json/schema":{"checkPropertyChange":{"!type":"fn(value: ?, schema: ?, property: ?) -> +any","!doc":"<p>The checkPropertyChange method will check to see if an value can legally be in property with the given schema\nThis is slightly different than the validate method in that it will fail if the schema is readonly and it will\nnot check for self-validation, it is assumed that the passed in value is already internally valid.\nThe checkPropertyChange method will return the same object type as validate, see JSONSchema.validate for\ninformation.</p>\n"},"mustBeValid":{"!type":"fn(result: ?)","!doc":"<p>This checks to ensure that the result is valid and will throw an appropriate error message if it is not</p>\n"},"validate":{"!type":"fn(instance: ?, schema: ?)","!doc":"<p>To use the validator call this with an instance object and an optional schema object.\nIf a schema is provided, it will be used to validate. If the instance object refers to a schema (self-validating),\nthat schema will be used to validate and the schema parameter is not necessary (if both exist,\nboth validations will occur).</p>\n"}},"dojox/data/ServiceStore":{"!type":"fn()","prototype":{"!doc":"<p>note that dojox.rpc.Service is not required, you can create your own services\nA ServiceStore is a readonly data store that provides a data.data interface to an RPC service.\n\n</p>\n<pre><code>var myServices = new dojox.rpc.Service(dojo.moduleUrl(&quot;dojox.rpc.tests.resources&quot;, &quot;test.smd&quot;));\nvar serviceStore = new dojox.data.ServiceStore({service:myServices.ServiceStore});</code></pre>\n<p>The ServiceStore also supports lazy loading. References can be made to objects that have not been loaded.\nFor example if a service returned:\n\n</p>\n<pre><code>{&quot;name&quot;:&quot;Example&quot;,&quot;lazyLoadedObject&quot;:{&quot;$ref&quot;:&quot;obj2&quot;}}</code></pre>\n<p>And this object has accessed using the dojo.data API:\n\n</p>\n<pre><code>var obj = serviceStore.getValue(myObject,&quot;lazyLoadedObject&quot;);</code></pre>\n<p>The object would automatically be requested from the server (with an object id of &quot;obj2&quot;).\n</p>\n","_currentId":{},"estimateCountFactor":{"!doc":"<p>This parameter is used by the ServiceStore to estimate the total count. When\npaging is indicated in a fetch and the response includes the full number of items\nrequested by the fetch&#39;s count parameter, then the total count will be estimated\nto be estimateCountFactor multiplied by the provided count. If this is 1, then it is assumed that the server\ndoes not support paging, and the response is the full set of items, where the\ntotal count is equal to the numer of items returned. If the server does support\npaging, an estimateCountFactor of 2 is a good value for estimating the total count\nIt is also possible to override _processResults if the server can provide an exact\ntotal count.    </p>\n"},"idAttribute":{"!doc":"<p>Defaults to &#39;id&#39;. The name of the attribute that holds an objects id.\nThis can be a preexisting id provided by the server.\nIf an ID isn&#39;t already provided when an object\nis fetched or added to the store, the autoIdentity system\nwill generate an id for it and add it to the index.</p>\n"},"labelAttribute":{},"loadLazyValues":{},"schema":{"!doc":"<p>This is a schema object for this store. This should be JSON Schema format.</p>\n"},"service":{"!doc":"<p>This is the service object that is used to retrieve lazy data and save results\nThe function should be directly callable with a single parameter of an object id to be loaded</p>\n"},"syncMode":{"!doc":"<p>Setting this to true will set the store to using synchronous calls by default.\nSync calls return their data immediately from the calling function, so\ncallbacks are unnecessary. This will only work with a synchronous capable service.</p>\n"},"close":{"!type":"fn(request: ?) -> ?"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>ServiceStore constructor, instantiate a new ServiceStore\nA ServiceStore can be configured from a JSON Schema. Queries are just\npassed through to the underlying services</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>Checks to see if &#39;item&#39; has &#39;value&#39; at &#39;attribute&#39;</p>\n"},"fetch":{"!type":"fn(args: ?) -> string","!doc":"<p>See dojo/data/api/Read.fetch</p>\n"},"fetchItemByIdentity":{"!type":"fn(args: ?) -> ?","!doc":"<p>fetch an item by its identity, by looking in our index of what we have loaded</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>Gets the available attributes of an item&#39;s &#39;property&#39; and returns\nit as an array.</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>return the store feature set</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns the attributes which are used to make up the\nidentity of an item.    Basically returns this.idAttribute</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>returns the label for an item. Just gets the &quot;label&quot; attribute.</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns an array of attributes that are used to create the label of an item</p>\n"},"getSchema":{"!type":"fn() -> ?","!doc":"<p>Returns a reference to the JSON Schema</p>\n"},"getValue":{"!type":"fn(item: ?, property: ?, defaultValue?: ?)","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39;</p>\n"},"getValues":{"!type":"fn(item: ?, property: ?) -> []","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39; and returns\nit. If this value is an array it is just returned,\nif not, the value is added to an array and that is returned.</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>Checks to see if item has attribute</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>Checks to see if the argument is an item</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>Checks to see if the item is loaded.</p>\n"},"loadItem":{"!type":"fn(args: ?) -> ?","!doc":"<p>Loads an item and calls the callback handler. Note, that this will call the callback\nhandler even if the item is loaded. Consequently, you can use loadItem to ensure\nthat an item is loaded is situations when the item may or may not be loaded yet.\nIf you access a value directly through property access, you can use this to load\na lazy value as well (doesn&#39;t need to be an item).</p>\n"}}},"dojox/data/CssClassStore":{"!type":"fn()","prototype":{"!proto":"dojox/data/CssRuleStore.prototype","!doc":"<p>Basic store to display CSS information.</p>\n","_browserMap":{},"_cache":{},"_cName":{},"_idAttribute":{},"_labelAttribute":{"!doc":"<p>text representation of the Item [label and identifier may need to stay due to method names]</p>\n"},"_storeRef":{},"close":{"!type":"fn()","!doc":"<p>See dojo/data/api/Read.close().\nClears out the cache and allItems objects, meaning all future fetches will requery\nthe stylesheets.</p>\n"},"constructor":{"!type":"fn(keywordParameters: ?)"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?) -> string","!doc":"<p>See dojo/data/api/Read.fetch()</p>\n"},"fetchItemByIdentity":{"!type":"fn(request: ?) -> string","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Identity.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"},"setContext":{"!type":"fn(context: ?)"}}},"dojox/data/CssRuleStore":{"!type":"fn()","prototype":{"!doc":"<p>Basic store to display CSS information.</p>\n","_browserMap":{},"_cache":{},"_cName":{},"_labelAttribute":{},"_storeRef":{},"close":{"!type":"fn()","!doc":"<p>See dojo/data/api/Read.close().\nClears out the cache and allItems objects, meaning all future fetches will requery\nthe stylesheets.</p>\n"},"constructor":{"!type":"fn(keywordParameters: ?)"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?) -> string","!doc":"<p>See dojo/data/api/Read.fetch()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"},"setContext":{"!type":"fn(context: ?)"}}},"dojox/data/css":{"rules":{},"determineContext":{"!type":"fn(initialStylesheets: ?) -> []"},"findStyleSheet":{"!type":"fn(sheet: ?) -> []"},"findStyleSheets":{"!type":"fn(sheets: ?) -> []"}},"dojox/data/css.rules":{"forEach":{"!type":"fn(fn: ?, ctx: ?, context: ?)"}},"dojox/data/CsvStore":{"!type":"fn()","prototype":{"!doc":"<p>The CsvStore implements the dojo/data/api/Read API and reads\ndata from files in CSV (Comma Separated Values) format.\nAll values are simple string values. References to other items\nare not supported as attribute values in this datastore.\n\n</p>\n<p>Example data file:\nname, color, age, tagline\nKermit, green, 12, &quot;Hi, I&#39;m Kermit the Frog.&quot;\nFozzie Bear, orange, 10, &quot;Wakka Wakka Wakka!&quot;\nMiss Piggy, pink, 11, &quot;Kermie!&quot;\n\n</p>\n<p>Note that values containing a comma must be enclosed with quotes (&quot;&quot;)\nAlso note that values containing quotes must be escaped with two consecutive quotes (&quot;&quot;quoted&quot;&quot;)\n</p>\n","identifier":{"!doc":"<p>Declarative hook for setting the identifier.</p>\n"},"label":{"!doc":"<p>Declarative hook for setting the label attribute.</p>\n"},"separator":{"!doc":"<p>Declatative and programmatic hook for defining the separator\ncharacter used in the Csv style file.</p>\n"},"url":{"!doc":"<p>Declarative hook for setting Csv source url.</p>\n"},"urlPreventCache":{},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(keywordParameters: ?)","!doc":"<p>initializer</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Identity.getIdentifierAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()\nNote that for the CsvStore, an empty string value is the same as no value,\nso the defaultValue would be returned instead of an empty string.</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValues()\nCSV syntax does not support multi-valued attributes, so this is just a\nwrapper function for getValue().</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()\nThe hasAttribute test is true if attribute has an index number within the item&#39;s array length\nAND if the item has a value for that attribute. Note that for the CsvStore, an\nempty string value is the same as no value.</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()\nThe CsvStore always loads all items, so if it&#39;s an item, then it&#39;s loaded.</p>\n"},"loadItem":{"!type":"fn(item: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/FileStore":{"!type":"fn()","prototype":{"_attributes":{"!doc":"<p>Internal variable of attributes all file items should have.</p>\n"},"_identifier":{"!doc":"<p>Default attribute to use to represent the item&#39;s identifier.\nPath should always be unique in the store instance.</p>\n"},"_storeRef":{"!doc":"<p>Internal variable used to denote an item came from this store instance.</p>\n"},"failOk":{"!doc":"<p>Flag to pass on to xhr functions to check if we are OK to fail the call silently</p>\n"},"label":{"!doc":"<p>Default attribute to use to represent the item as a user-readable\nstring.  Public, so users can change it.</p>\n"},"options":{"!doc":"<p>Array of options to always send when doing requests.\nBack end service controls this, like &#39;dirsOnly&#39;, &#39;showHiddenFiles&#39;, &#39;expandChildren&#39;, etc.</p>\n"},"pathAsQueryParam":{},"pathSeparator":{"!doc":"<p>The path separator to use when chaining requests for children\nCan be overriden by the server on initial load</p>\n"},"url":{"!doc":"<p>The URL to the file path service.</p>\n"},"urlPreventCache":{"!doc":"<p>Flag to dennote if preventCache should be passed to xhrGet.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>A simple store that provides a datastore interface to a filesystem.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch  items that match to a query</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/FlickrRestStore":{"!type":"fn()","prototype":{"!proto":"dojox/data/FlickrStore.prototype","_apikey":{"!doc":"<p>The users API key to be used when accessing Flickr REST services.</p>\n"},"_cache":{"!doc":"<p>An Array of all previously downloaded picture info.</p>\n"},"_flickrRestUrl":{"!doc":"<p>The URL to the Flickr REST services.</p>\n"},"_handlers":{"!doc":"<p>A HashMap used to record the handlers registered for a single remote request.  Multiple\nrequests may be made for the same information before the first request has finished.\nEach element of this Object is an array of handlers to call back when the request finishes.\nThis prevents multiple requests being made for the same information.</p>\n"},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_prevRequests":{"!doc":"<p>A HashMap used to record the signature of a request to prevent duplicate\nrequest being made.</p>\n"},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that Flickr will use.</p>\n"},"_sortAttributes":{"!doc":"<p>A quick lookup of valid attribute names in a sort query.</p>\n"},"_storeRef":{"!doc":"<p>A key used to mark an data store item as belonging to this store.</p>\n"},"label":{},"urlPreventCache":{},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the FlickrRestStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/FlickrStore":{"!type":"fn()","prototype":{"_storeRef":{},"label":{},"urlPreventCache":{},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the FlickrStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}},"urlRegistry":{}},"dojox/data/GoogleFeedStore":{"!type":"fn()","prototype":{"!doc":"<p>A data store for retrieving RSS and Atom feeds from Google. The\nfeeds can come from any source, which is specified in the &quot;url&quot;\nparameter of the query passed to the &quot;fetch&quot; function.\nThe following attributes are supported on each item:\n\n</p>\n<ul>\n<li>title - The feed entry title. </li>\n<li>link - The URL for the HTML version of the feed entry. </li>\n<li>content - The full content of the blog post, in HTML format </li>\n<li>summary - A snippet of information about the feed entry, in plain text </li>\n<li>published - The string date on which the entry was published.<pre><code>      You can parse the date with new Date(store.getValue(item, &quot;published&quot;) </code></pre>\n</li>\n<li>categories - An array of string tags for the entry </li>\n</ul>\n<p>The query accepts one parameter: url - The URL of the feed to retrieve</p>\n","_aggregatedAttributes":{},"_attributes":{},"_googleUrl":{},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_queryAttrs":{},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that GoogleSearchStore will use.</p>\n"},"_storeRef":{"!doc":"<p>The internal reference added to each item pointing at the store which owns it.</p>\n"},"_type":{},"label":{"!doc":"<p>The default attribute which acts as a label for each item.</p>\n"},"urlPreventCache":{"!doc":"<p>Sets whether or not to pass preventCache to dojo.io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the GoogleSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch Google search items that match to a query</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getFeedValue":{"!type":"fn(attribute: ?, defaultValue: ?) -> ?","!doc":"<p>Non-API method for retrieving values regarding the Atom feed,\nrather than the Atom entries.</p>\n"},"getFeedValues":{"!type":"fn(attribute: ?, defaultValue: ?) -> ?","!doc":"<p>Non-API method for retrieving values regarding the Atom feed,\nrather than the Atom entries.</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/GoogleSearchStore":{"BlogSearch":{"!type":"fn()"},"BookSearch":{"!type":"fn()"},"ImageSearch":{"!type":"fn()"},"LocalSearch":{"!type":"fn()"},"NewsSearch":{"!type":"fn()"},"Search":{"!type":"fn()"},"VideoSearch":{"!type":"fn()"},"WebSearch":{"!type":"fn()"}},"dojox/data/GoogleSearchStore.Search":{"!type":"fn()","prototype":{"!doc":"<p>A data store for retrieving search results from Google.\nThis data store acts as a base class for Google searches,\nand has a number of child data stores that implement different\nsearches. This store defaults to searching the web, and is functionally\nidentical to the dojox.data.GoogleWebSearchStore object.\nThe following attributes are supported on each item:\n\n</p>\n<ul>\n<li>url - The URL for the item</li>\n<li>unescapedUrl - The URL for the item, with URL escaping. This is often more readable</li>\n<li>visibleUrl - The URL with no protocol specified.</li>\n<li>cacheUrl - The URL to the copy of the document cached by Google</li>\n<li>title - The page title in HTML format.</li>\n<li>titleNoFormatting - The page title in plain text</li>\n<li>content - A snippet of information about the page</li>\n</ul>\n<p>The query accepts one parameter: text - The string to search for</p>\n","_aggregatedAttributes":{},"_attributes":{"!doc":"<p>The list of attributes that this store supports</p>\n"},"_googleUrl":{"!doc":"<p>The URL to Googles search web service.</p>\n"},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_queryAttrs":{"!doc":"<p>Maps query hash keys to Google query parameters.</p>\n"},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that GoogleSearchStore will use.</p>\n"},"_storeRef":{"!doc":"<p>The internal reference added to each item pointing at the store which owns it.</p>\n"},"_type":{},"label":{"!doc":"<p>The default attribute which acts as a label for each item.</p>\n"},"urlPreventCache":{"!doc":"<p>Sets whether or not to pass preventCache to dojo.io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the GoogleSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch Google search items that match to a query</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/GoogleSearchStore.ImageSearch":{"!type":"fn()","prototype":{"!doc":"<p>A data store for retrieving search results from Google.\nThe following attributes are supported on each item:\n\n</p>\n<ul>\n<li>title - The image title in HTML format.</li>\n<li>titleNoFormatting - The image title in plain text</li>\n<li>url - The URL for the image</li>\n<li>unescapedUrl - The URL for the image, with URL escaping. This is often more readable</li>\n<li>tbUrl - The URL for the image thumbnail</li>\n<li>visibleUrl - A shortened version of the URL associated with the result, stripped of a protocol and path</li>\n<li>originalContextUrl - The URL of the page containing the image.</li>\n<li>width - The width of the image in pixels.</li>\n<li>height - The height of the image in pixels.</li>\n<li>tbWidth - The width of the image thumbnail in pixels.</li>\n<li>tbHeight - The height of the image thumbnail in pixels.</li>\n<li>content - A snippet of information about the image, in HTML format</li>\n<li>contentNoFormatting - A snippet of information about the image, in plain text</li>\n</ul>\n<p>The query accepts one parameter: text - The string to search for</p>\n","_aggregatedAttributes":{},"_attributes":{},"_googleUrl":{"!doc":"<p>The URL to Googles search web service.</p>\n"},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_queryAttrs":{"!doc":"<p>Maps query hash keys to Google query parameters.</p>\n"},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that GoogleSearchStore will use.</p>\n"},"_storeRef":{"!doc":"<p>The internal reference added to each item pointing at the store which owns it.</p>\n"},"_type":{},"label":{"!doc":"<p>The default attribute which acts as a label for each item.</p>\n"},"urlPreventCache":{"!doc":"<p>Sets whether or not to pass preventCache to dojo.io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the GoogleSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch Google search items that match to a query</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/GoogleSearchStore.BookSearch":{"!type":"fn()","prototype":{"!doc":"<p>A data store for retrieving search results from Google.\nThe following attributes are supported on each item:\n\n</p>\n<ul>\n<li>title - The book title in HTML format.</li>\n<li>titleNoFormatting - The book title in plain text</li>\n<li>authors - An array of authors</li>\n<li>url - The URL for the item</li>\n<li>unescapedUrl - The URL for the item, with URL escaping. This is often more readable</li>\n<li>bookId - An identifier for the book, usually an ISBN.</li>\n<li>pageCount - The number of pages in the book.</li>\n<li>publishedYear - The year of publication.</li>\n</ul>\n<p>The query accepts one parameter: text - The string to search for</p>\n","_aggregatedAttributes":{},"_attributes":{},"_googleUrl":{"!doc":"<p>The URL to Googles search web service.</p>\n"},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_queryAttrs":{"!doc":"<p>Maps query hash keys to Google query parameters.</p>\n"},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that GoogleSearchStore will use.</p>\n"},"_storeRef":{"!doc":"<p>The internal reference added to each item pointing at the store which owns it.</p>\n"},"_type":{},"label":{"!doc":"<p>The default attribute which acts as a label for each item.</p>\n"},"urlPreventCache":{"!doc":"<p>Sets whether or not to pass preventCache to dojo.io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the GoogleSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch Google search items that match to a query</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/GoogleSearchStore.NewsSearch":{"!type":"fn()","prototype":{"!doc":"<p>A data store for retrieving search results from Google.\nThe following attributes are supported on each item:\n\n</p>\n<ul>\n<li>title - The news story title in HTML format.</li>\n<li>titleNoFormatting - The news story title in plain text</li>\n<li>content - A snippet of information about the news story</li>\n<li>url - The URL for the item</li>\n<li>unescapedUrl - The URL for the item, with URL escaping. This is often more readable</li>\n<li>publisher - The name of the publisher</li>\n<li>clusterUrl - A URL pointing to a page listing related storied.</li>\n<li>location - The location of the news story.</li>\n<li>publishedDate - The date of publication, in RFC-822 format.</li>\n<li>relatedStories - An optional array of objects specifying related stories.\n  Each object has the following subset of properties:\n  &quot;title&quot;, &quot;titleNoFormatting&quot;, &quot;url&quot;, &quot;unescapedUrl&quot;, &quot;publisher&quot;, &quot;location&quot;, &quot;publishedDate&quot;.</li>\n</ul>\n<p>The query accepts one parameter: text - The string to search for</p>\n","_aggregatedAttributes":{},"_attributes":{},"_googleUrl":{"!doc":"<p>The URL to Googles search web service.</p>\n"},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_queryAttrs":{"!doc":"<p>Maps query hash keys to Google query parameters.</p>\n"},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that GoogleSearchStore will use.</p>\n"},"_storeRef":{"!doc":"<p>The internal reference added to each item pointing at the store which owns it.</p>\n"},"_type":{},"label":{"!doc":"<p>The default attribute which acts as a label for each item.</p>\n"},"urlPreventCache":{"!doc":"<p>Sets whether or not to pass preventCache to dojo.io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the GoogleSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch Google search items that match to a query</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/GoogleSearchStore.VideoSearch":{"!type":"fn()","prototype":{"!doc":"<p>A data store for retrieving search results from Google.\nThe following attributes are supported on each item:\n\n</p>\n<ul>\n<li>title - The blog post title in HTML format.</li>\n<li>titleNoFormatting - The  blog post title in plain text</li>\n<li>content - A snippet of information about the blog post</li>\n<li>url - The URL for the item</li>\n<li>published - The published date, in RFC-822 format.</li>\n<li>publisher - The name of the publisher.</li>\n<li>duration - The approximate duration, in seconds, of the video.</li>\n<li>tbWidth - The width in pixels of the video.</li>\n<li>tbHeight - The height in pixels of the video</li>\n<li>tbUrl - The URL to a thumbnail representation of the video.</li>\n<li>playUrl - If present, supplies the url of the flash version of the video that can be played inline on your page. To play this video simply create and <embed> element on your page using this value as the src attribute and using application/x-shockwave-flash as the type attribute. If you want the video to play right away, make sure to append &amp;autoPlay=true to the url..</li>\n</ul>\n<p>The query accepts one parameter: text - The string to search for</p>\n","_aggregatedAttributes":{},"_attributes":{},"_googleUrl":{"!doc":"<p>The URL to Googles search web service.</p>\n"},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_queryAttrs":{"!doc":"<p>Maps query hash keys to Google query parameters.</p>\n"},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that GoogleSearchStore will use.</p>\n"},"_storeRef":{"!doc":"<p>The internal reference added to each item pointing at the store which owns it.</p>\n"},"_type":{},"label":{"!doc":"<p>The default attribute which acts as a label for each item.</p>\n"},"urlPreventCache":{"!doc":"<p>Sets whether or not to pass preventCache to dojo.io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the GoogleSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch Google search items that match to a query</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/GoogleSearchStore.LocalSearch":{"!type":"fn()","prototype":{"!doc":"<p>A data store for retrieving search results from Google.\nThe following attributes are supported on each item:\n\n</p>\n<ul>\n<li>title - The blog post title in HTML format.</li>\n<li>titleNoFormatting - The  blog post title in plain text</li>\n<li>content - A snippet of information about the blog post</li>\n<li>url - The URL for the item</li>\n<li>lat - The latitude.</li>\n<li>lng - The longtitude.</li>\n<li>streetAddress - The street address</li>\n<li>city - The city</li>\n<li>region - The region</li>\n<li>country - The country</li>\n<li>phoneNumbers - Phone numbers associated with this address. Can be one or more.</li>\n<li>ddUrl - A URL that can be used to provide driving directions from the center of the search results to this search results</li>\n<li>ddUrlToHere - A URL that can be used to provide driving directions from this search result to a user specified location</li>\n<li>staticMapUrl - The published date, in RFC-822 format</li>\n<li>viewport - Recommended viewport for the query results (same for all results in a query)<ul>\n<li>center - contains lat, lng properties</li>\n<li>span - lat, lng properties for the viewport span</li>\n<li>ne, sw - lat, lng properties for the viewport corners</li>\n</ul>\n</li>\n</ul>\n<p>The query accepts the following parameters:\n\n</p>\n<ul>\n<li>text - The string to search for</li>\n<li>centerLatLong - Comma-separated lat &amp; long for the center of the search (e.g. &quot;48.8565,2.3509&quot;)</li>\n<li>searchSpan - Comma-separated lat &amp; long degrees indicating the size of the desired search area (e.g. &quot;0.065165,0.194149&quot;)</li>\n</ul>\n","_aggregatedAttributes":{},"_attributes":{},"_googleUrl":{"!doc":"<p>The URL to Googles search web service.</p>\n"},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_queryAttrs":{},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that GoogleSearchStore will use.</p>\n"},"_storeRef":{"!doc":"<p>The internal reference added to each item pointing at the store which owns it.</p>\n"},"_type":{},"label":{"!doc":"<p>The default attribute which acts as a label for each item.</p>\n"},"urlPreventCache":{"!doc":"<p>Sets whether or not to pass preventCache to dojo.io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the GoogleSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch Google search items that match to a query</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/GoogleSearchStore.BlogSearch":{"!type":"fn()","prototype":{"!doc":"<p>A data store for retrieving search results from Google.\nThe following attributes are supported on each item:\n\n</p>\n<ul>\n<li>title - The blog post title in HTML format.</li>\n<li>titleNoFormatting - The  blog post title in plain text</li>\n<li>content - A snippet of information about the blog post</li>\n<li>blogUrl - The URL for the blog</li>\n<li>postUrl - The URL for the a single blog post</li>\n<li>visibleUrl - The URL with no protocol specified.</li>\n<li>cacheUrl - The URL to the copy of the document cached by Google</li>\n<li>author - The author of the blog post</li>\n<li>publishedDate - The published date, in RFC-822 format</li>\n</ul>\n<p>The query accepts one parameter: text - The string to search for</p>\n","_aggregatedAttributes":{},"_attributes":{},"_googleUrl":{"!doc":"<p>The URL to Googles search web service.</p>\n"},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_queryAttrs":{"!doc":"<p>Maps query hash keys to Google query parameters.</p>\n"},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that GoogleSearchStore will use.</p>\n"},"_storeRef":{"!doc":"<p>The internal reference added to each item pointing at the store which owns it.</p>\n"},"_type":{},"label":{"!doc":"<p>The default attribute which acts as a label for each item.</p>\n"},"urlPreventCache":{"!doc":"<p>Sets whether or not to pass preventCache to dojo.io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the GoogleSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch Google search items that match to a query</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/GoogleSearchStore.WebSearch":{"!type":"fn()","prototype":{"!doc":"<p>A data store for retrieving search results from Google.\nThe following attributes are supported on each item:\n\n</p>\n<ul>\n<li>title - The page title in HTML format.</li>\n<li>titleNoFormatting - The page title in plain text</li>\n<li>content - A snippet of information about the page</li>\n<li>url - The URL for the item</li>\n<li>unescapedUrl - The URL for the item, with URL escaping. This is often more readable</li>\n<li>visibleUrl - The URL with no protocol specified.</li>\n<li>cacheUrl - The URL to the copy of the document cached by Google</li>\n<li>estimatedResultCount - (aggregated per-query) estimated number of results</li>\n</ul>\n<p>The query accepts one parameter: text - The string to search for</p>\n","_aggregatedAttributes":{},"_attributes":{"!doc":"<p>The list of attributes that this store supports</p>\n"},"_googleUrl":{"!doc":"<p>The URL to Googles search web service.</p>\n"},"_id":{"!doc":"<p>A unique identifier for this store.</p>\n"},"_queryAttrs":{"!doc":"<p>Maps query hash keys to Google query parameters.</p>\n"},"_requestCount":{"!doc":"<p>A counter for the number of requests made. This is used to define\nthe callback function that GoogleSearchStore will use.</p>\n"},"_storeRef":{"!doc":"<p>The internal reference added to each item pointing at the store which owns it.</p>\n"},"_type":{},"label":{"!doc":"<p>The default attribute which acts as a label for each item.</p>\n"},"urlPreventCache":{"!doc":"<p>Sets whether or not to pass preventCache to dojo.io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the GoogleSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?)","!doc":"<p>Fetch Google search items that match to a query</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/HtmlStore":{"!type":"fn()","prototype":{"dataId":{"!doc":"<p>The id in the document for an element from which to get the data.</p>\n"},"fetchOnCreate":{"!doc":"<p>Flag to denote if it should try to load from a data id (nested in the page)\nThe moment the store is created, instead of waiting for first\nfetch call.</p>\n"},"trimWhitespace":{},"url":{"!doc":"<p>The URL from which to load an HTML document for data loading</p>\n"},"urlPreventCache":{"!doc":"<p>Flag to denote if peventCache should be used on xhrGet calls.</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the HTML table store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Identity.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> string","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +value","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/HtmlTableStore":{"!type":"fn()","prototype":{"tableId":{"!doc":"<p>The id of the table to load as store contents.</p>\n"},"url":{"!doc":"<p>The URL from which to load an HTML document for data loading</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Identity.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> string","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +value","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/JsonQueryRestStore":{"!type":"fn()","prototype":{"!proto":"dojox/data/JsonRestStore.prototype","_currentId":{},"allowNoTrailingSlash":{"!doc":"<p>Allow no trailing slash on target paths. This is generally discouraged since\nit creates prevents simple scalar values from being used a relative URLs.\nDisabled by default.</p>\n"},"estimateCountFactor":{"!doc":"<p>This parameter is used by the ServiceStore to estimate the total count. When\npaging is indicated in a fetch and the response includes the full number of items\nrequested by the fetch&#39;s count parameter, then the total count will be estimated\nto be estimateCountFactor multiplied by the provided count. If this is 1, then it is assumed that the server\ndoes not support paging, and the response is the full set of items, where the\ntotal count is equal to the numer of items returned. If the server does support\npaging, an estimateCountFactor of 2 is a good value for estimating the total count\nIt is also possible to override _processResults if the server can provide an exact\ntotal count.    </p>\n"},"idAsRef":{"!doc":"<p>Treat objects in queries as partially loaded objects</p>\n"},"idAttribute":{"!doc":"<p>Defaults to &#39;id&#39;. The name of the attribute that holds an objects id.\nThis can be a preexisting id provided by the server.\nIf an ID isn&#39;t already provided when an object\nis fetched or added to the store, the autoIdentity system\nwill generate an id for it and add it to the index.</p>\n"},"jsonQueryPagination":{},"labelAttribute":{},"loadLazyValues":{},"loadReferencedSchema":{"!doc":"<p>Will load any schemas referenced content-type header or in Link headers</p>\n"},"referenceIntegrity":{},"schema":{"!doc":"<p>This is a schema object for this store. This should be JSON Schema format.</p>\n"},"service":{"!doc":"<p>This is the service object that is used to retrieve lazy data and save results\nThe function should be directly callable with a single parameter of an object id to be loaded</p>\n"},"syncMode":{"!doc":"<p>Setting this to true will set the store to using synchronous calls by default.\nSync calls return their data immediately from the calling function, so\ncallbacks are unnecessary. This will only work with a synchronous capable service.</p>\n"},"target":{},"useFullIdInQueries":{},"cancelChanging":{"!type":"fn(object: ?)","!doc":"<p>Removes an object from the list of dirty objects\nThis will prevent that object from being saved to the server on the next save</p>\n"},"changing":{"!type":"fn(item: ?, _deleting: ?)","!doc":"<p>adds an item to the list of dirty items.    This item\ncontains a reference to the item itself as well as a\ncloned and trimmed version of old item for use with\nrevert.</p>\n"},"clientSideFetch":{"!type":"fn(request: ?, baseResults: ?) -> ?"},"close":{"!type":"fn(request: ?) -> ?"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>JsonRestStore is a Dojo Data store interface to JSON HTTP/REST web\nstorage services that support read and write through GET, PUT, POST, and DELETE.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>Checks to see if &#39;item&#39; has &#39;value&#39; at &#39;attribute&#39;</p>\n"},"deleteItem":{"!type":"fn(item: ?)","!doc":"<p>deletes item and any references to that item from the store.</p>\n"},"fetch":{"!type":"fn(args: ?) -> string","!doc":"<p>See dojo/data/api/Read.fetch</p>\n"},"fetchItemByIdentity":{"!type":"fn(args: ?) -> ?"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>Gets the available attributes of an item&#39;s &#39;property&#39; and returns\nit as an array.</p>\n"},"getConstructor":{"!type":"fn() -> ?","!doc":"<p>Gets the constructor for objects from this store</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>return the store feature set</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns the attributes which are used to make up the\nidentity of an item.    Basically returns this.idAttribute</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>returns the label for an item. Just gets the &quot;label&quot; attribute.</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns an array of attributes that are used to create the label of an item</p>\n"},"getParent":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the parent item (or query) for the given item</p>\n"},"getSchema":{"!type":"fn() -> ?","!doc":"<p>Returns a reference to the JSON Schema</p>\n"},"getValue":{"!type":"fn(item: ?, property: ?, defaultValue?: ?)","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39;</p>\n"},"getValues":{"!type":"fn(item: ?, property: ?) -> []","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39; and returns\nit. If this value is an array it is just returned,\nif not, the value is added to an array and that is returned.</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>Checks to see if item has attribute</p>\n"},"isDirty":{"!type":"fn(item: ?) -> ?","!doc":"<p>returns true if the item is marked as dirty.</p>\n"},"isItem":{"!type":"fn(item: ?, anyStore: ?) -> ?","!doc":"<p>Checks to see if a passed &#39;item&#39;\nreally belongs to this JsonRestStore.</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>Checks to see if the item is loaded.</p>\n"},"isUpdateable":{"!type":"fn() -> bool"},"loadItem":{"!type":"fn(args: ?) -> ?","!doc":"<p>Loads an item and calls the callback handler. Note, that this will call the callback\nhandler even if the item is loaded. Consequently, you can use loadItem to ensure\nthat an item is loaded is situations when the item may or may not be loaded yet.\nIf you access a value directly through property access, you can use this to load\na lazy value as well (doesn&#39;t need to be an item).</p>\n"},"matchesQuery":{"!type":"fn(item: ?, request: ?) -> ?"},"newItem":{"!type":"fn(data: ?, parentInfo: ?) -> +instance","!doc":"<p>adds a new item to the store at the specified point.\nTakes two parameters, data, and options.</p>\n"},"querySuperSet":{"!type":"fn(argsSuper: ?, argsSub: ?) -> ?"},"revert":{"!type":"fn(kwArgs: ?)","!doc":"<p>returns any modified data to its original state prior to a save();</p>\n"},"save":{"!type":"fn(kwArgs: ?) -> ?","!doc":"<p>Saves the dirty data using REST Ajax methods. See dojo/data/api/Write for API.</p>\n"},"setValue":{"!type":"fn(item: ?, attribute: ?, value: ?)","!doc":"<p>sets &#39;attribute&#39; on &#39;item&#39; to &#39;value&#39;</p>\n"},"setValues":{"!type":"fn(item: ?, attribute: ?, values: ?)","!doc":"<p>sets &#39;attribute&#39; on &#39;item&#39; to &#39;value&#39; value\nmust be an array.</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, attribute: ?)","!doc":"<p>unsets &#39;attribute&#39; on &#39;item&#39;</p>\n"}}},"dojox/data/util/JsonQuery":{"!type":"fn()","prototype":{"jsonQueryPagination":{},"useFullIdInQueries":{},"clientSideFetch":{"!type":"fn(request: ?, baseResults: ?) -> ?"},"fetch":{"!type":"fn(args: ?) -> ?"},"isUpdateable":{"!type":"fn() -> bool"},"matchesQuery":{"!type":"fn(item: ?, request: ?) -> ?"},"querySuperSet":{"!type":"fn(argsSuper: ?, argsSub: ?) -> ?"}}},"dojox/json/query":{"!doc":"<p>Performs a JSONQuery on the provided object and returns the results.\nIf no object is provided (just a query), it returns a &quot;compiled&quot; function that evaluates objects\naccording to the provided query.</p>\n","prototype":{"constructor":{"!type":"fn(query: ?, obj?: ?)","!doc":"<p>Performs a JSONQuery on the provided object and returns the results.\nIf no object is provided (just a query), it returns a &quot;compiled&quot; function that evaluates objects\naccording to the provided query.</p>\n"}}},"dojox/data/KeyValueStore":{"!type":"fn()","prototype":{"!doc":"<p>This is a dojo.data store implementation.  It can take in either a Javascript\narray, JSON string, or URL as the data source.  Data is expected to be in the\nfollowing format:\n\n</p>\n<pre><code>[\n    { &quot;key1&quot;: &quot;value1&quot; },\n    { &quot;key2&quot;: &quot;value2&quot; }\n]</code></pre>\n<p>This is to mimic the Java Properties file format.  Each &#39;item&#39; from this store\n\n</p>\n<p>is a JS object representing a key-value pair.  If an item in the above array has\nmore than one key/value pair, only the first will be used/accessed.</p>\n","data":{},"url":{},"urlPreventCache":{"!doc":"<p>Controls if urlPreventCache should be used with underlying xhrGet.</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(keywordParameters: ?)","!doc":"<p>constructor</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Identity.getIdentifierAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +value","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValues()\nKey/Value syntax does not support multi-valued attributes, so this is just a\nwrapper function for getValue().</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()\nThe KeyValueStore always loads all items, so if it&#39;s an item, then it&#39;s loaded.</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/OpenSearchStore":{"!type":"fn()","prototype":{"_storeRef":{},"ATOM_CONTENT_TYPE":{},"ATOM_CONTENT_TYPE_STRING":{},"iframeElement":{},"itemPath":{},"RSS_CONTENT_TYPE":{},"RSS_CONTENT_TYPE_STRING":{},"url":{},"urlElement":{},"urlPreventCache":{"!doc":"<p>Flag denoting if xhrGet calls should use the preventCache option.</p>\n"},"XML_CONTENT_TYPE":{},"XML_CONTENT_TYPE_STRING":{},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the OpenSearchStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"},"process":{"!type":"fn(data: ?) -> ?"},"processItem":{"!type":"fn(item: ?, attribute: ?) -> ?"}}},"dojox/data/OpmlStore":{"!type":"fn()","prototype":{"!doc":"<p>The OpmlStore implements the dojo/data/api/Read API.</p>\n","label":{"!doc":"<p>The attribute of the Opml item to act as a label.</p>\n"},"url":{"!doc":"<p>The location from which to fetch the Opml document.</p>\n"},"urlPreventCache":{"!doc":"<p>Flag to denote if the underlying xhrGet call should set preventCache.</p>\n"},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(keywordParameters: ?)","!doc":"<p>constructor</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> ?","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Identity.getIdentifierAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValues()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded().\nOpmlStore loads every item, so if it&#39;s an item, then it&#39;s loaded.</p>\n"},"loadItem":{"!type":"fn(item: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/PersevereStore":{"!type":"fn()","prototype":{"jsonQueryPagination":{},"useFullIdInQueries":{}},"addProxy":{"!type":"fn()","!doc":"<p>Invokes the XHR proxy plugin. Call this if you will be using x-site data.</p>\n"},"getStores":{"!type":"fn(path?: ?, sync?: ?) -> +any","!doc":"<p>Creates Dojo data stores for all the table/classes on a Persevere server</p>\n"}},"dojox/rpc/Client":{"clientId":{}},"dojox/data/PicasaStore":{"!type":"fn()","prototype":{"_picasaUrl":{},"_storeRef":{},"label":{"!doc":"<p>The attribute to use from the picasa item as its label.</p>\n"},"maxResults":{"!doc":"<p>Define out how many results to return for a fetch.</p>\n"},"urlPreventCache":{"!doc":"<p>Flag denoting if preventCache should be passed to io.script.</p>\n"},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initializer for the PicasaStore store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/QueryReadStore":{"!type":"fn()","prototype":{"!doc":"<p>This class provides a store that is mainly intended to be used\nfor loading data dynamically from the server, used i.e. for\nretrieving chunks of data from huge data stores on the server (by server-side filtering!).\nUpon calling the fetch() method of this store the data are requested from\nthe server if they are not yet loaded for paging (or cached).\n\n</p>\n<p>For example used for a combobox which works on lots of data. It\ncan be used to retrieve the data partially upon entering the\nletters &quot;ac&quot; it returns only items like &quot;action&quot;, &quot;acting&quot;, etc.\n\n</p>\n<p>note:\nThe field name &quot;id&quot; in a query is reserved for looking up data\nby id. This is necessary as before the first fetch, the store\nhas no way of knowing which field the server will declare as\nidentifier.\n</p>\n","_className":{},"_features":{},"_identifier":{},"_items":{},"_itemsByIdentity":{},"_labelAttr":{},"_lastServerQuery":{},"_numRows":{},"doClientPaging":{"!doc":"<p>By default every request for paging is sent to the server.</p>\n"},"doClientSorting":{"!doc":"<p>By default all the sorting is done serverside before the data is returned\nwhich is the proper place to be doing it for really large datasets.</p>\n"},"lastRequestHash":{},"requestMethod":{},"url":{},"close":{"!type":"fn(request?: ?)"},"constructor":{"!type":"fn(params: ?)"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool"},"fetch":{"!type":"fn(request?: ?) -> string","!doc":"<p>See dojo.data.util.simpleFetch.fetch() this is just a copy and I adjusted\nonly the paging, since it happens on the server if doClientPaging is\nfalse, thx to <a href=\"http://trac.dojotoolkit.org/ticket/4761\">http://trac.dojotoolkit.org/ticket/4761</a> reporting this.\nWould be nice to be able to use simpleFetch() to reduce copied code,\nbut i dont know how yet. Ideas please!</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []"},"getFeatures":{"!type":"fn() -> ?"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Identity.getIdentity()</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Identity.getIdentityAttributes()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +value"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> ?","!doc":"<p>See dojo/data/api/Read.hasAttribute()</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool"},"isItemLoaded":{"!type":"fn(something: ?) -> ?"},"loadItem":{"!type":"fn(args: ?)"}}},"dojox/data/RailsStore":{"!type":"fn()","prototype":{"constructor":{"!type":"fn()","!doc":"<p>RailsStore is a data store for interacting with RESTful Rails controllers</p>\n"},"fetch":{"!type":"fn(args: ?) -> ?"},"preamble":{"!type":"fn(options: ?)"}}},"dojox/data/S3Store":{"!type":"fn()","prototype":{"!proto":"dojox/data/JsonRestStore.prototype","!doc":"<p>S3JsonRestStore is an extension of JsonRestStore to handle\nAmazon&#39;s S3 service using JSON data</p>\n","_currentId":{},"allowNoTrailingSlash":{"!doc":"<p>Allow no trailing slash on target paths. This is generally discouraged since\nit creates prevents simple scalar values from being used a relative URLs.\nDisabled by default.</p>\n"},"estimateCountFactor":{"!doc":"<p>This parameter is used by the ServiceStore to estimate the total count. When\npaging is indicated in a fetch and the response includes the full number of items\nrequested by the fetch&#39;s count parameter, then the total count will be estimated\nto be estimateCountFactor multiplied by the provided count. If this is 1, then it is assumed that the server\ndoes not support paging, and the response is the full set of items, where the\ntotal count is equal to the numer of items returned. If the server does support\npaging, an estimateCountFactor of 2 is a good value for estimating the total count\nIt is also possible to override _processResults if the server can provide an exact\ntotal count.    </p>\n"},"idAsRef":{"!doc":"<p>Treat objects in queries as partially loaded objects</p>\n"},"idAttribute":{"!doc":"<p>Defaults to &#39;id&#39;. The name of the attribute that holds an objects id.\nThis can be a preexisting id provided by the server.\nIf an ID isn&#39;t already provided when an object\nis fetched or added to the store, the autoIdentity system\nwill generate an id for it and add it to the index.</p>\n"},"labelAttribute":{},"loadLazyValues":{},"loadReferencedSchema":{"!doc":"<p>Will load any schemas referenced content-type header or in Link headers</p>\n"},"referenceIntegrity":{},"schema":{"!doc":"<p>This is a schema object for this store. This should be JSON Schema format.</p>\n"},"service":{"!doc":"<p>This is the service object that is used to retrieve lazy data and save results\nThe function should be directly callable with a single parameter of an object id to be loaded</p>\n"},"syncMode":{"!doc":"<p>Setting this to true will set the store to using synchronous calls by default.\nSync calls return their data immediately from the calling function, so\ncallbacks are unnecessary. This will only work with a synchronous capable service.</p>\n"},"target":{},"cancelChanging":{"!type":"fn(object: ?)","!doc":"<p>Removes an object from the list of dirty objects\nThis will prevent that object from being saved to the server on the next save</p>\n"},"changing":{"!type":"fn(item: ?, _deleting: ?)","!doc":"<p>adds an item to the list of dirty items.    This item\ncontains a reference to the item itself as well as a\ncloned and trimmed version of old item for use with\nrevert.</p>\n"},"close":{"!type":"fn(request: ?) -> ?"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>JsonRestStore is a Dojo Data store interface to JSON HTTP/REST web\nstorage services that support read and write through GET, PUT, POST, and DELETE.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>Checks to see if &#39;item&#39; has &#39;value&#39; at &#39;attribute&#39;</p>\n"},"deleteItem":{"!type":"fn(item: ?)","!doc":"<p>deletes item and any references to that item from the store.</p>\n"},"fetch":{"!type":"fn(args: ?) -> string","!doc":"<p>See dojo/data/api/Read.fetch</p>\n"},"fetchItemByIdentity":{"!type":"fn(args: ?) -> ?"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>Gets the available attributes of an item&#39;s &#39;property&#39; and returns\nit as an array.</p>\n"},"getConstructor":{"!type":"fn() -> ?","!doc":"<p>Gets the constructor for objects from this store</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>return the store feature set</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns the attributes which are used to make up the\nidentity of an item.    Basically returns this.idAttribute</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>returns the label for an item. Just gets the &quot;label&quot; attribute.</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns an array of attributes that are used to create the label of an item</p>\n"},"getParent":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the parent item (or query) for the given item</p>\n"},"getSchema":{"!type":"fn() -> ?","!doc":"<p>Returns a reference to the JSON Schema</p>\n"},"getValue":{"!type":"fn(item: ?, property: ?, defaultValue?: ?)","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39;</p>\n"},"getValues":{"!type":"fn(item: ?, property: ?) -> []","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39; and returns\nit. If this value is an array it is just returned,\nif not, the value is added to an array and that is returned.</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>Checks to see if item has attribute</p>\n"},"isDirty":{"!type":"fn(item: ?) -> ?","!doc":"<p>returns true if the item is marked as dirty.</p>\n"},"isItem":{"!type":"fn(item: ?, anyStore: ?) -> ?","!doc":"<p>Checks to see if a passed &#39;item&#39;\nreally belongs to this JsonRestStore.</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>Checks to see if the item is loaded.</p>\n"},"loadItem":{"!type":"fn(args: ?) -> ?","!doc":"<p>Loads an item and calls the callback handler. Note, that this will call the callback\nhandler even if the item is loaded. Consequently, you can use loadItem to ensure\nthat an item is loaded is situations when the item may or may not be loaded yet.\nIf you access a value directly through property access, you can use this to load\na lazy value as well (doesn&#39;t need to be an item).</p>\n"},"newItem":{"!type":"fn(data: ?, parentInfo: ?) -> +instance","!doc":"<p>adds a new item to the store at the specified point.\nTakes two parameters, data, and options.</p>\n"},"revert":{"!type":"fn(kwArgs: ?)","!doc":"<p>returns any modified data to its original state prior to a save();</p>\n"},"save":{"!type":"fn(kwArgs: ?) -> ?","!doc":"<p>Saves the dirty data using REST Ajax methods. See dojo/data/api/Write for API.</p>\n"},"setValue":{"!type":"fn(item: ?, attribute: ?, value: ?)","!doc":"<p>sets &#39;attribute&#39; on &#39;item&#39; to &#39;value&#39;</p>\n"},"setValues":{"!type":"fn(item: ?, attribute: ?, values: ?)","!doc":"<p>sets &#39;attribute&#39; on &#39;item&#39; to &#39;value&#39; value\nmust be an array.</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, attribute: ?)","!doc":"<p>unsets &#39;attribute&#39; on &#39;item&#39;</p>\n"}}},"dojox/rpc/ProxiedPath":{},"dojox/rpc/Service":{"_nextId":{}},"dojox/data/SnapLogicStore":{"!type":"fn()","prototype":{"Parts":{},"url":{},"close":{"!type":"fn(request: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Initialize a SnapLogicStore object.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>See dojo/data/api/Read.containsValue()</p>\n"},"fetch":{"!type":"fn(request: ?) -> ?","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getAttributes()</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>See dojo/data/api/Read.getFeatures()</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> +null","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> []","!doc":"<p>See dojo/data/api/Read.getValue()</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>See dojo/data/api/Read.hasAttributes()</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>See dojo/data/api/Read.isItem()</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.isItemLoaded()</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Read.loadItem()</p>\n"}}},"dojox/data/WikipediaStore":{"!type":"fn()","prototype":{"!proto":"dojox/data/ServiceStore.prototype","!doc":"<p>Initializer for the Wikipedia data store interface.</p>\n","_currentId":{},"estimateCountFactor":{"!doc":"<p>This parameter is used by the ServiceStore to estimate the total count. When\npaging is indicated in a fetch and the response includes the full number of items\nrequested by the fetch&#39;s count parameter, then the total count will be estimated\nto be estimateCountFactor multiplied by the provided count. If this is 1, then it is assumed that the server\ndoes not support paging, and the response is the full set of items, where the\ntotal count is equal to the numer of items returned. If the server does support\npaging, an estimateCountFactor of 2 is a good value for estimating the total count\nIt is also possible to override _processResults if the server can provide an exact\ntotal count.    </p>\n"},"idAttribute":{"!doc":"<p>Defaults to &#39;id&#39;. The name of the attribute that holds an objects id.\nThis can be a preexisting id provided by the server.\nIf an ID isn&#39;t already provided when an object\nis fetched or added to the store, the autoIdentity system\nwill generate an id for it and add it to the index.</p>\n"},"labelAttribute":{},"loadLazyValues":{},"schema":{"!doc":"<p>This is a schema object for this store. This should be JSON Schema format.</p>\n"},"service":{"!doc":"<p>This is the service object that is used to retrieve lazy data and save results\nThe function should be directly callable with a single parameter of an object id to be loaded</p>\n"},"syncMode":{"!doc":"<p>Setting this to true will set the store to using synchronous calls by default.\nSync calls return their data immediately from the calling function, so\ncallbacks are unnecessary. This will only work with a synchronous capable service.</p>\n"},"close":{"!type":"fn(request: ?) -> ?"},"constructor":{"!type":"fn(options: ?)"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> bool","!doc":"<p>Checks to see if &#39;item&#39; has &#39;value&#39; at &#39;attribute&#39;</p>\n"},"fetch":{"!type":"fn(request: ?) -> ?","!doc":"<p>Fetch a page or some partially-loaded search results from\nWikipedia. Note that there isn&#39;t a way to sort data coming\nin from the API, so we just ignore the <em>sort</em> parameter.</p>\n"},"fetchItemByIdentity":{"!type":"fn(args: ?) -> ?","!doc":"<p>fetch an item by its identity, by looking in our index of what we have loaded</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>Gets the available attributes of an item&#39;s &#39;property&#39; and returns\nit as an array.</p>\n"},"getFeatures":{"!type":"fn() -> ?","!doc":"<p>return the store feature set</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns the attributes which are used to make up the\nidentity of an item.    Basically returns this.idAttribute</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>returns the label for an item. Just gets the &quot;label&quot; attribute.</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>returns an array of attributes that are used to create the label of an item</p>\n"},"getSchema":{"!type":"fn() -> ?","!doc":"<p>Returns a reference to the JSON Schema</p>\n"},"getValue":{"!type":"fn(item: ?, property: ?, defaultValue?: ?)","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39;</p>\n"},"getValues":{"!type":"fn(item: ?, property: ?) -> []","!doc":"<p>Gets the value of an item&#39;s &#39;property&#39; and returns\nit. If this value is an array it is just returned,\nif not, the value is added to an array and that is returned.</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> bool","!doc":"<p>Checks to see if item has attribute</p>\n"},"isItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>Checks to see if the argument is an item</p>\n"},"isItemLoaded":{"!type":"fn(item: ?) -> ?","!doc":"<p>Checks to see if the item is loaded.</p>\n"},"loadItem":{"!type":"fn(args: ?) -> ?","!doc":"<p>Loads an item and calls the callback handler. Note, that this will call the callback\nhandler even if the item is loaded. Consequently, you can use loadItem to ensure\nthat an item is loaded is situations when the item may or may not be loaded yet.\nIf you access a value directly through property access, you can use this to load\na lazy value as well (doesn&#39;t need to be an item).</p>\n"}}},"dojox/data/XmlItem":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(element: ?, store: ?, query: ?)","!doc":"<p>Initialize with an XML element</p>\n"},"toString":{"!type":"fn() -> +any","!doc":"<p>Return a value of the first text child of the element</p>\n"}}},"dojox/data/XmlStore":{"!type":"fn()","prototype":{"!doc":"<p>A data store for XML based services or documents</p>\n","attributeMap":{},"keyAttribute":{},"label":{},"rootItem":{},"sendQuery":{},"url":{},"urlPreventCache":{},"close":{"!type":"fn(request?: ?)","!doc":"<p>See dojo/data/api/Read.close()</p>\n"},"constructor":{"!type":"fn(args: ?)","!doc":"<p>Constructor for the XML store.</p>\n"},"containsValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> +any","!doc":"<p>Check whether the attribute values contain the value</p>\n"},"deleteItem":{"!type":"fn(item: ?) -> +any","!doc":"<p>Delete an dojox.data.XmlItem (wrapper to a XML element).</p>\n"},"errorHandler":{"!type":"fn(errorData: ?, requestObject: ?)","!doc":"<p>The error handler when there is an error fetching items.  This function should not be called\ndirectly and is used by simpleFetch.fetch().</p>\n"},"fetch":{"!type":"fn(request?: ?)","!doc":"<p>The simpleFetch mixin is designed to serve as a set of function(s) that can\nbe mixed into other datastore implementations to accelerate their development.</p>\n"},"fetchHandler":{"!type":"fn(items: ?, requestObject: ?)","!doc":"<p>The handler when items are sucessfully fetched.  This function should not be called directly\nand is used by simpleFetch.fetch().</p>\n"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)","!doc":"<p>See dojo/data/api/Identity.fetchItemByIdentity(keywordArgs)</p>\n"},"getAttributes":{"!type":"fn(item: ?) -> +any","!doc":"<p>Return an array of attribute names</p>\n"},"getFeatures":{"!type":"fn() -> +any","!doc":"<p>Return supported data APIs</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns a unique identifier for an item.</p>\n"},"getIdentityAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>Returns an array of attribute names that are used to generate the identity.</p>\n"},"getLabel":{"!type":"fn(item: ?) -> ?","!doc":"<p>See dojo/data/api/Read.getLabel()</p>\n"},"getLabelAttributes":{"!type":"fn(item: ?) -> []","!doc":"<p>See dojo/data/api/Read.getLabelAttributes()</p>\n"},"getValue":{"!type":"fn(item: ?, attribute: ?, defaultValue?: ?) -> +any","!doc":"<p>Return an attribute value</p>\n"},"getValues":{"!type":"fn(item: ?, attribute: ?) -> +any","!doc":"<p>Return an array of attribute values</p>\n"},"hasAttribute":{"!type":"fn(item: ?, attribute: ?) -> +any","!doc":"<p>Check whether an element has the attribute</p>\n"},"isDirty":{"!type":"fn(item?: ?) -> +any","!doc":"<p>Check whether an item is new, modified or deleted</p>\n"},"isItem":{"!type":"fn(something: ?) -> +any","!doc":"<p>Check whether the object is an item (XML element)</p>\n"},"isItemLoaded":{"!type":"fn(something: ?) -> +any","!doc":"<p>Check whether the object is an item (XML element) and loaded</p>\n"},"loadItem":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Load an item (XML element)</p>\n"},"newItem":{"!type":"fn(keywordArgs?: ?, parentInfo: ?) -> +any","!doc":"<p>Return a new dojox.data.XmlItem</p>\n"},"revert":{"!type":"fn() -> +any","!doc":"<p>Invalidate changes (new and/or modified elements)</p>\n"},"save":{"!type":"fn(keywordArgs: ?)","!doc":"<p>Save new and/or modified items (XML elements)</p>\n"},"setValue":{"!type":"fn(item: ?, attribute: ?, value: ?) -> +any","!doc":"<p>Set an attribute value</p>\n"},"setValues":{"!type":"fn(item: ?, attribute: ?, values: ?) -> +any","!doc":"<p>Set attribute values</p>\n"},"unsetAttribute":{"!type":"fn(item: ?, attribute: ?) -> +any","!doc":"<p>Remove an attribute</p>\n"}}},"dojox/data/dom":{"createDocument":{"!type":"fn(str?: ?, mimetype?: ?) -> ?","!doc":"<p>cross-browser implementation of creating an XML document object.</p>\n"},"innerXML":{"!type":"fn(node: ?)","!doc":"<p>Implementation of MS&#39;s innerXML function.</p>\n"},"removeChildren":{"!type":"fn(node: ?)","!doc":"<p>removes all children from node and returns the count of children removed.\nThe children nodes are not destroyed. Be sure to call dojo._destroyElement on them\nafter they are not used anymore.</p>\n"},"replaceChildren":{"!type":"fn(node: ?, newChildren: ?)","!doc":"<p>Removes all children of node and appends newChild. All the existing\nchildren will be destroyed.</p>\n"},"textContent":{"!type":"fn(node: ?, text?: ?)","!doc":"<p>Implementation of the DOM Level 3 attribute; scan node for text</p>\n"}},"dojox/data/restListener":{"!doc":"<p>this function can be used to receive REST notifications, from Comet or from another frame</p>\n","prototype":{"constructor":{"!type":"fn(message: ?)","!doc":"<p>this function can be used to receive REST notifications, from Comet or from another frame</p>\n"}}},"dojox/date/buddhist/Date":{"!type":"fn()","prototype":{"_date":{},"_day":{},"_hours":{},"_milliseconds":{},"_minutes":{},"_month":{},"_seconds":{},"_year":{},"constructor":{"!type":"fn()","!doc":"<p>This is the constructor</p>\n"},"fromGregorian":{"!type":"fn(gdate: ?) -> fn","!doc":"<p>This function sets this Date to the Hebrew Date corresponding to the Gregorian Date</p>\n"},"getDate":{"!type":"fn(isNumber?: ?) -> ?","!doc":"<p>This function returns the date value (0 - 30)</p>\n"},"getDay":{"!type":"fn() -> ?","!doc":"<p>This function return Week Day value ( 0 - 6 )</p>\n"},"getFullYear":{"!type":"fn() -> ?","!doc":"<p>This function return the Year value</p>\n"},"getHours":{"!type":"fn() -> ?","!doc":"<p>returns the Hour value</p>\n"},"getMilliseconds":{"!type":"fn() -> ?","!doc":"<p>returns the Milliseconds value</p>\n"},"getMinutes":{"!type":"fn() -> ?","!doc":"<p>returns the Minutes value</p>\n"},"getMonth":{"!type":"fn() -> ?","!doc":"<p>This function return the month value ( 0 - 11 )</p>\n"},"getSeconds":{"!type":"fn() -> ?","!doc":"<p>returns the Seconds value</p>\n"},"setDate":{"!type":"fn(date: ?) -> fn","!doc":"<p>This function sets the Date</p>\n"},"setFullYear":{"!type":"fn(year: ?, month?: ?, date?: ?)","!doc":"<p>This function set Year</p>\n"},"setHours":{"!type":"fn()","!doc":"<p>set the Hours  0-23</p>\n"},"setMilliseconds":{"!type":"fn(milliseconds: ?) -> fn"},"setMinutes":{"!type":"fn(minutes: ?) -> fn","!doc":"<p>sets the minutes (0-59) only.</p>\n"},"setMonth":{"!type":"fn(month: ?)","!doc":"<p>This function set Month</p>\n"},"setSeconds":{"!type":"fn(seconds: ?) -> fn","!doc":"<p>sets the seconds (0-59) only.</p>\n"},"toGregorian":{"!type":"fn() -> +instance","!doc":"<p>This returns the equivalent Gregorian date value as a Date object</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>This returns a string representation of the date in &quot;dd, MM, YYYY HH:MM:SS&quot; format</p>\n"},"valueOf":{"!type":"fn()"}}},"dojox/date/buddhist/locale":{"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?)","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parses string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a buddhist.Date</p>\n"}},"dojox/date/buddhist":{"locale":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2: ?, portion?: ?)","!doc":"<p>Compare two buddhist date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(dateObject: ?)"},"isLeapYear":{"!type":"fn(dateObject: ?)"}},"dojox/date/buddhist.locale":{"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?)","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parses string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a buddhist.Date</p>\n"}},"dojox/date/hebrew/Date":{"!type":"fn()","prototype":{"!doc":"<p>A Date-like object which implements the Hebrew calendar</p>\n","_date":{},"_day":{},"_GREGORIAN_MONTH_COUNT":{},"_hours":{},"_LEAP_MONTH_START":{},"_milliseconds":{},"_minutes":{},"_month":{},"_MONTH_LENGTH":{},"_MONTH_START":{},"_seconds":{},"_year":{},"constructor":{"!type":"fn()","!doc":"<p>initialize the date object value</p>\n"},"fromGregorian":{"!type":"fn(gdate: ?) -> fn","!doc":"<p>This function sets this Date to the Hebrew Date corresponding to the Gregorian Date</p>\n"},"getDate":{"!type":"fn() -> ?","!doc":"<p>returns the date value (1 - 30)</p>\n"},"getDateLocalized":{"!type":"fn(locale?: ?) -> ?","!doc":"<p>returns the date value as hebrew numerals for the Hebrew locale,\na number for all others.</p>\n"},"getDay":{"!type":"fn() -> number","!doc":"<p>returns weekday value (0 - 6)</p>\n"},"getDaysInHebrewMonth":{"!type":"fn(month: ?, year: ?) -> ?","!doc":"<p>returns the number of days in the given month and year</p>\n"},"getFullYear":{"!type":"fn() -> ?","!doc":"<p>returns the Year value</p>\n"},"getHours":{"!type":"fn() -> ?","!doc":"<p>returns the hour value</p>\n"},"getMilliseconds":{"!type":"fn() -> ?","!doc":"<p>returns the milliseconds value</p>\n"},"getMinutes":{"!type":"fn() -> ?","!doc":"<p>returns the minutes value</p>\n"},"getMonth":{"!type":"fn() -> ?","!doc":"<p>returns the month value (0 - 12)</p>\n"},"getSeconds":{"!type":"fn() -> ?","!doc":"<p>returns the seconds value</p>\n"},"isLeapYear":{"!type":"fn(year: ?) -> bool","!doc":"<p>Determines if the year (argument) is a leap year</p>\n"},"setDate":{"!type":"fn(date: ?) -> fn","!doc":"<p>sets the date number for a given month</p>\n"},"setFullYear":{"!type":"fn(year: ?, month?: ?, date?: ?) -> fn","!doc":"<p>set the year</p>\n"},"setHours":{"!type":"fn() -> fn","!doc":"<p>sets the hour</p>\n"},"setMilliseconds":{"!type":"fn(milliseconds: ?) -> fn"},"setMinutes":{"!type":"fn(minutes: ?) -> fn","!doc":"<p>sets the minutes (0-59) only.</p>\n"},"setMonth":{"!type":"fn(month: ?) -> fn","!doc":"<p>sets the month.  You should use &quot;absolute&quot; index in the month array:\n\n</p>\n<ol>\n<li>Tishri</li>\n<li>Heshvan</li>\n<li>Kislev</li>\n<li>Tevet</li>\n<li>Shevat</li>\n<li>Adar I (leap years only)</li>\n<li>Adar</li>\n<li>Nisan</li>\n<li>Iyar</li>\n<li>Sivan</li>\n<li>Tammuz</li>\n<li>Av</li>\n<li>Elul - 12</li>\n</ol>\n<p>For non leap years, for months after Shevat, the actual position of\nthe month in the year (used for short format) is less than\nthe &quot;absolute&quot; index by 1.</p>\n"},"setSeconds":{"!type":"fn(seconds: ?) -> fn","!doc":"<p>sets the seconds (0-59) only.</p>\n"},"toGregorian":{"!type":"fn() -> +instance","!doc":"<p>returns the equivalent Gregorian date value as a native Date object</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>returns a string representation of the date in &quot;dd, MM, yyyy HH:mm:ss&quot; format</p>\n"},"valueOf":{"!type":"fn()"}}},"dojox/date/hebrew/numerals":{"getDayHebrewLetters":{"!type":"fn(day: ?, nogrsh?: ?)","!doc":"<p>converts an integer to a String representing the number in Hebrew numerals.\nCan be formatted with or without geresh &#x05f3;</p>\n"},"getMonthHebrewLetters":{"!type":"fn(month: ?)","!doc":"<p>converts an integer representing a  month to a String written in Hebrew numerals</p>\n"},"getYearHebrewLetters":{"!type":"fn(year: ?)","!doc":"<p>converts the year from an integer to Hebrew numerals.</p>\n"},"parseDayHebrewLetters":{"!type":"fn(day: ?)","!doc":"<p>converts the string containing a Hebrew numeral to an integer</p>\n"},"parseMonthHebrewLetters":{"!type":"fn(monthStr: ?)","!doc":"<p>converts a Hebrew numeral string representing\na month to an integer.  The returned value\nis indexed in the month name array.  To use it for\nsetMonth, do correction for leap year</p>\n"},"parseYearHebrewLetters":{"!type":"fn(year: ?)","!doc":"<p>converts the year written in Hebrew numerals to an integer</p>\n"}},"dojox/date/hebrew/locale":{"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?)","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parse string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a hebrew.Date</p>\n"}},"dojox/date/hebrew":{"locale":{},"numerals":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(dateheb1: ?, dateheb2: ?, portion?: ?)","!doc":"<p>Compare two hebrew date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(month: ?)"}},"dojox/date/hebrew.numerals":{"getDayHebrewLetters":{"!type":"fn(day: ?, nogrsh?: ?)","!doc":"<p>converts an integer to a String representing the number in Hebrew numerals.\nCan be formatted with or without geresh &#x05f3;</p>\n"},"getMonthHebrewLetters":{"!type":"fn(month: ?)","!doc":"<p>converts an integer representing a  month to a String written in Hebrew numerals</p>\n"},"getYearHebrewLetters":{"!type":"fn(year: ?)","!doc":"<p>converts the year from an integer to Hebrew numerals.</p>\n"},"parseDayHebrewLetters":{"!type":"fn(day: ?)","!doc":"<p>converts the string containing a Hebrew numeral to an integer</p>\n"},"parseMonthHebrewLetters":{"!type":"fn(monthStr: ?)","!doc":"<p>converts a Hebrew numeral string representing\na month to an integer.  The returned value\nis indexed in the month name array.  To use it for\nsetMonth, do correction for leap year</p>\n"},"parseYearHebrewLetters":{"!type":"fn(year: ?)","!doc":"<p>converts the year written in Hebrew numerals to an integer</p>\n"}},"dojox/date/hebrew.locale":{"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?)","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parse string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a hebrew.Date</p>\n"}},"dojox/date/islamic/Date":{"!type":"fn()","prototype":{"!doc":"<p>The component defines the Islamic (Hijri) Calendar Object</p>\n","_date":{},"_day":{},"_GREGORIAN_EPOCH":{},"_hours":{},"_ISLAMIC_EPOCH":{},"_milliseconds":{},"_minutes":{},"_month":{},"_seconds":{},"_year":{},"constructor":{"!type":"fn()","!doc":"<p>This is the constructor</p>\n"},"fromGregorian":{"!type":"fn(gdate: ?) -> fn","!doc":"<p>This function returns the equivalent Islamic Date value for the Gregorian Date</p>\n"},"getDate":{"!type":"fn() -> ?","!doc":"<p>This function returns the date value (1 - 30)</p>\n"},"getDay":{"!type":"fn() -> ?","!doc":"<p>This function return Week Day value ( 0 - 6 )</p>\n"},"getDaysInIslamicMonth":{"!type":"fn(month: ?, year: ?) -> number","!doc":"<p>returns the number of days in the given Islamic Month</p>\n"},"getFullYear":{"!type":"fn() -> ?","!doc":"<p>This function return the Year value</p>\n"},"getHours":{"!type":"fn() -> ?","!doc":"<p>returns the Hour value</p>\n"},"getMilliseconds":{"!type":"fn() -> ?","!doc":"<p>returns the Milliseconds value</p>\n"},"getMinutes":{"!type":"fn() -> ?","!doc":"<p>returns the Minutes value</p>\n"},"getMonth":{"!type":"fn() -> ?","!doc":"<p>This function return the month value ( 0 - 11 )</p>\n"},"getSeconds":{"!type":"fn() -> ?","!doc":"<p>returns the seconds value</p>\n"},"setDate":{"!type":"fn(date: ?) -> fn","!doc":"<p>This function sets the Date</p>\n"},"setFullYear":{"!type":"fn(year: ?)","!doc":"<p>This function set Year</p>\n"},"setHours":{"!type":"fn()","!doc":"<p>set the Hours</p>\n"},"setMilliseconds":{"!type":"fn(milliseconds: ?) -> fn"},"setMinutes":{"!type":"fn(minutes: ?) -> fn","!doc":"<p>sets the minutes (0-59) only.</p>\n"},"setMonth":{"!type":"fn(month: ?)","!doc":"<p>This function set Month</p>\n"},"setSeconds":{"!type":"fn(seconds: ?) -> fn","!doc":"<p>sets the seconds (0-59) only.</p>\n"},"toGregorian":{"!type":"fn() -> +instance","!doc":"<p>This returns the equevalent Grogorian date value in Date object</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>This returns a string representation of the date in &quot;DDDD MMMM DD YYYY HH:MM:SS&quot; format</p>\n"},"valueOf":{"!type":"fn() -> ?","!doc":"<p>This function returns The stored time value in milliseconds\nsince midnight, January 1, 1970 UTC</p>\n"}},"getDaysInIslamicMonth":{"!type":"fn(month: ?)"}},"dojox/date/islamic/locale":{"months":{},"weekDays":{},"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?) -> ?","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parse string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a islamic.Date</p>\n"}},"dojox/date/islamic":{"locale":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2: ?, portion?: ?)","!doc":"<p>Compare two islamic date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(month: ?)"}},"dojox/date/islamic.locale":{"months":{},"weekDays":{},"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?) -> ?","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parse string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a islamic.Date</p>\n"}},"dojox/date/persian/Date":{"!type":"fn()","prototype":{"!doc":"<p>The component defines the Persian (Hijri) Calendar Object</p>\n","_date":{},"_day":{},"_GREGORIAN_EPOCH":{},"_hours":{},"_milliseconds":{},"_minutes":{},"_month":{},"_PERSIAN_EPOCH":{},"_seconds":{},"_year":{},"daysInMonth":{},"calcGregorian":{"!type":"fn(year: ?, month: ?, day: ?) -> +instance"},"constructor":{"!type":"fn()","!doc":"<p>This is the constructor</p>\n"},"fromGregorian":{"!type":"fn(gdate: ?) -> fn","!doc":"<p>This function returns the equivalent Persian Date value for the Gregorian Date</p>\n"},"getDate":{"!type":"fn() -> ?","!doc":"<p>This function returns the date value (1 - 30)</p>\n"},"getDay":{"!type":"fn() -> ?","!doc":"<p>This function return Week Day value ( 0 - 6 )</p>\n"},"getDaysInPersianMonth":{"!type":"fn(month: ?, year: ?) -> ?","!doc":"<p>returns the number of days in the given Persian Month</p>\n"},"getFullYear":{"!type":"fn() -> ?","!doc":"<p>This function return the Year value</p>\n"},"getHours":{"!type":"fn() -> ?","!doc":"<p>returns the Hour value</p>\n"},"getMilliseconds":{"!type":"fn() -> ?","!doc":"<p>returns the Milliseconds value</p>\n"},"getMinutes":{"!type":"fn() -> ?","!doc":"<p>returns the Minutes value</p>\n"},"getMonth":{"!type":"fn() -> ?","!doc":"<p>This function return the month value ( 0 - 11 )</p>\n"},"getSeconds":{"!type":"fn() -> ?","!doc":"<p>returns the seconds value</p>\n"},"gregorian_to_jd":{"!type":"fn(year: ?, month: ?, day: ?) -> number"},"isLeapYear":{"!type":"fn(j_y: ?, j_m: ?, j_d: ?) -> bool","!doc":"<p>return Boolean value if Persian leap year</p>\n"},"jd_to_gregorian":{"!type":"fn(jd: ?, jmonth: ?) -> +instance"},"jd_to_persian":{"!type":"fn(jd: ?) -> +instance"},"jwday":{"!type":"fn(j: ?) -> ?"},"leap_gregorian":{"!type":"fn(year: ?) -> bool"},"persian_to_jd":{"!type":"fn(year: ?, month: ?, day: ?) -> number"},"setDate":{"!type":"fn(date: ?) -> fn","!doc":"<p>This function sets the Date</p>\n"},"setFullYear":{"!type":"fn(year: ?)","!doc":"<p>This function set Year</p>\n"},"setHours":{"!type":"fn()","!doc":"<p>set the Hours</p>\n"},"setMilliseconds":{"!type":"fn(milliseconds: ?) -> fn"},"setMinutes":{"!type":"fn(minutes: ?) -> fn","!doc":"<p>sets the minutes (0-59) only.</p>\n"},"setMonth":{"!type":"fn(month: ?)","!doc":"<p>This function set Month</p>\n"},"setSeconds":{"!type":"fn(seconds: ?) -> fn","!doc":"<p>sets the seconds (0-59) only.</p>\n"},"toGregorian":{"!type":"fn() -> +instance","!doc":"<p>This returns the equevalent Grogorian date value in Date object</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>This returns a string representation of the date in &quot;DDDD MMMM DD YYYY HH:MM:SS&quot; format</p>\n"},"valueOf":{"!type":"fn() -> ?","!doc":"<p>This function returns The stored time value in milliseconds\nsince midnight, January 1, 1970 UTC</p>\n"}},"getDaysInPersianMonth":{"!type":"fn(month: ?)"}},"dojox/date/persian/locale":{"months":{},"weekDays":{},"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?) -> ?","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parse string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a persian.Date</p>\n"}},"dojox/date/persian":{"locale":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2: ?, portion?: ?)","!doc":"<p>Compare two persian date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(month: ?)"}},"dojox/date/persian.locale":{"months":{},"weekDays":{},"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?) -> ?","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parse string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a persian.Date</p>\n"}},"dojox/date/php":{"DateFormat":{"!type":"fn(format: ?)","!doc":"<p>Format the internal date object</p>\n"},"format":{"!type":"fn(date: ?, format: ?)","!doc":"<p>Get a formatted string for a given date object</p>\n"}},"dojox/string/tokenize":{"!doc":"<p>Split a string by a regular expression with the ability to capture the delimeters</p>\n","prototype":{"constructor":{"!type":"fn(str: ?, re: ?, parseDelim?: ?, instance?: ?) -> []","!doc":"<p>Split a string by a regular expression with the ability to capture the delimeters</p>\n"}}},"dojox/date/posix":{"getIsoWeekOfYear":{"!type":"fn(dateObject: ?)","!doc":"<p>Get the ISO8601 week number of the given date.\nThe week containing January 4th is the first week of the year.\nSee <a href=\"http://en.wikipedia.org/wiki/ISO_week_date\">http://en.wikipedia.org/wiki/ISO_week_date</a></p>\n"},"getIsoWeeksInYear":{"!type":"fn(dateObject: ?)","!doc":"<p>Determine the number of ISO8601 weeks in the year of the given\ndate. Most years have 52 but some have 53.\nSee <a href=\"http://www.phys.uu.nl/~vgent/calendar/isocalendar_text3.htm\">http://www.phys.uu.nl/~vgent/calendar/isocalendar_text3.htm</a></p>\n"},"getStartOfWeek":{"!type":"fn(dateObject: ?, firstDay: ?)","!doc":"<p>Return a date object representing the first day of the given\ndate&#39;s week.</p>\n"},"setIsoWeekOfYear":{"!type":"fn(dateObject: ?, week: ?)","!doc":"<p>Set the ISO8601 week number of the given date.\nThe week containing January 4th is the first week of the year.</p>\n"},"strftime":{"!type":"fn(dateObject: ?, format: ?, locale?: ?)"}},"dojox/date/relative":{"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using locale-specific settings,\nrelative to the current date or some other date.\n</p>\n"}},"dojox/date/timezone":{},"dojox/date/umalqura/Date":{"!type":"fn()","prototype":{"!doc":"<p>The component defines the UmAlqura (Hijri) Calendar Object according to Umalqura calculations\nThis module is similar to the Date() object provided by JavaScript</p>\n","_date":{},"_day":{},"_hijriBegin":{},"_hijriEnd":{},"_hours":{},"_milliseconds":{},"_minutes":{},"_month":{},"_MONTH_LENGTH":{},"_seconds":{},"_year":{},"constructor":{"!type":"fn()","!doc":"<p>This function initialize the date object values</p>\n"},"fromGregorian":{"!type":"fn(gdate: ?) -> fn","!doc":"<p>This function returns the equivalent UmAlqura Date value for the Gregorian Date</p>\n"},"getDate":{"!type":"fn() -> ?","!doc":"<p>This function returns the date value (1 - 30)</p>\n"},"getDay":{"!type":"fn() -> ?","!doc":"<p>This function returns the week day value ( 0 - 6 )\nsunday is 0, monday is 1,...etc</p>\n"},"getDaysInIslamicMonth":{"!type":"fn(month: ?, year: ?) -> ?","!doc":"<p>returns the number of days in the given Islamic month</p>\n"},"getFullYear":{"!type":"fn() -> ?","!doc":"<p>This function return the year value</p>\n"},"getHours":{"!type":"fn() -> ?","!doc":"<p>returns the hour value</p>\n"},"getMilliseconds":{"!type":"fn() -> ?","!doc":"<p>returns the milliseconds value</p>\n"},"getMinutes":{"!type":"fn() -> ?","!doc":"<p>returns the minutes value</p>\n"},"getMonth":{"!type":"fn() -> ?","!doc":"<p>This function return the month value ( 0 - 11 )</p>\n"},"getSeconds":{"!type":"fn() -> ?","!doc":"<p>returns the seconds value</p>\n"},"setDate":{"!type":"fn(date: ?) -> fn","!doc":"<p>This function sets the date</p>\n"},"setFullYear":{"!type":"fn(year: ?)","!doc":"<p>This function set Year</p>\n"},"setHours":{"!type":"fn()","!doc":"<p>set the hours</p>\n"},"setMilliseconds":{"!type":"fn(milliseconds: ?)","!doc":"<p>set the milliseconds</p>\n"},"setMinutes":{"!type":"fn(minutes: ?)","!doc":"<p>set the minutes</p>\n"},"setMonth":{"!type":"fn(month: ?)","!doc":"<p>This function sets the month</p>\n"},"setSeconds":{"!type":"fn(seconds: ?)","!doc":"<p>set seconds</p>\n"},"toGregorian":{"!type":"fn() -> +instance","!doc":"<p>This returns the equivalent gregorian date value in Date object</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>This returns a string representation of the date in &quot;DDDD MMMM DD YYYY HH:MM:SS&quot; format</p>\n"},"valueOf":{"!type":"fn() -> ?","!doc":"<p>This function returns the stored time value in milliseconds\nsince midnight, January 1, 1970 UTC</p>\n"}},"getDaysInIslamicMonth":{"!type":"fn(month: ?)"}},"dojox/date/umalqura/locale":{"months":{},"weekDays":{},"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?) -> ?","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parse string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a umalqura.Date</p>\n"}},"dojox/date/umalqura":{"locale":{},"add":{"!type":"fn(date: ?, interval: ?, amount: ?)","!doc":"<p>Add to a Date in intervals of different size, from milliseconds to years</p>\n"},"compare":{"!type":"fn(date1: ?, date2: ?, portion?: ?)","!doc":"<p>Compare two umalqura date objects by date, time, or both.</p>\n"},"difference":{"!type":"fn(date1: ?, date2?: ?, interval?: ?)","!doc":"<p>date2 - date1</p>\n"},"getDaysInMonth":{"!type":"fn(month: ?)"}},"dojox/date/umalqura.locale":{"months":{},"weekDays":{},"addCustomFormats":{"!type":"fn(packageName: ?, bundleName: ?)","!doc":"<p>Add a reference to a bundle containing localized custom formats to be\nused by date/time formatting and parsing routines.</p>\n"},"format":{"!type":"fn(dateObject: ?, options?: ?)","!doc":"<p>Format a Date object as a String, using  settings.</p>\n"},"getNames":{"!type":"fn(item: ?, type: ?, context?: ?, locale?: ?, date?: ?) -> ?","!doc":"<p>Used to get localized strings from dojo.cldr for day or month names.</p>\n"},"parse":{"!type":"fn(value: ?, options?: ?)","!doc":"<p>This function parse string date value according to options</p>\n"},"regexp":{"!type":"fn(options?: ?)","!doc":"<p>Builds the regular needed to parse a umalqura.Date</p>\n"}},"dojox/dgauges/CircularGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/GaugeBase.prototype","!doc":"<p>The base class for circular gauges.\nYou can create custom circular or semi-circular gauges by extending this class.\nSee dojox.dgauges.components.default.CircularLinearGauge.js for an example of circular gauge.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(args: ?, node: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/GaugeBase":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>This class is the base class for the circular and \nrectangular (horizontal and vertical) gauge components.\nA gauge is a composition of elements added to the gauge using the addElement method.\nElements are drawn from back to front in the same order they are added (using addElement).\nAn elements can be: \n\n</p>\n<ul>\n<li>A GFX drawing functions typically used for defining the style of the gauge.</li>\n<li>A scale: CircularScale or RectangularScale depending on the type of gauge.</li>\n<li>A text, using the TextIndicator\nNote: Indicator classes (value indicators, range indicators) are sub-elements of scales\nTo create a custom gauge, subclass CircularGauge or RectangularGauge and\nconfigure its elements in the constructor.\nReady to use, predefined gauges are available in dojox/dgauges/components/\nThey are good examples of gauges built on top of the framework.</li>\n</ul>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_widgetBox":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(args: ?, node: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()","!doc":"<p>Actually refresh the rendering. Implementation should implement that method.</p>\n"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/ScaleBase":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_Invalidating.prototype","!doc":"<p>The ScaleBase class is the base class for the circular and rectangular scales.\nA scaler must be set to use this class. A scaler is responsible for\ntick generation and various data-transform operations.  </p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_bgGroup":{},"_fgGroup":{},"_gauge":{},"_gfxGroup":{},"_indicators":{},"_indicatorsIndex":{},"_indicatorsRenderers":{},"_watchers":{},"font":{"!doc":"<p>The font used for the ticks labels.\nThis is null by default which means this scale use the font defined \non the gauge.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"labelGap":{"!doc":"<p>The label gap between the ticks and their labels. Default value is 1.</p>\n"},"labelPosition":{"!doc":"<p>See CircularScale and RectangularScale for valid values.</p>\n"},"scaler":{"!doc":"<p>The scaler used for tick generation and data-transform operations.\nThis property is mandatory for using the scale.</p>\n"},"tickStroke":{"!doc":"<p>The GFX stroke used by the default tickShapeFunc implementation.</p>\n"},"addIndicator":{"!type":"fn(name: ?, indicator: ?, behindScale: ?) -> ?","!doc":"<p>Add an indicator to the scale. Before calling this function, ensure \nthis scale has already been added to a gauge using the addElement method\nof the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"getIndicator":{"!type":"fn(name: ?) -> +IndicatorBase","!doc":"<p>Get an indicator instance.</p>\n"},"getIndicatorRenderer":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Gets the GFX shape of an indicator.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"positionForValue":{"!type":"fn(value: ?) -> number","!doc":"<p>See CircularScale and Rectangular for more informations.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeIndicator":{"!type":"fn(name: ?) -> +IndicatorBase","!doc":"<p>Removes an indicator.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"tickLabelFunc":{"!type":"fn(tickItem: ?) -> string","!doc":"<p>Customize the text of ticks labels.</p>\n"},"tickShapeFunc":{"!type":"fn(group: ?, scale: ?, tickItem: ?) -> ?","!doc":"<p>Customize the shape of ticks.</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"valueForPosition":{"!type":"fn(position: ?) -> number","!doc":"<p>See CircularScale and Rectangular for more informations.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/CircularRangeIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/ScaleIndicatorBase.prototype","!doc":"<p>A CircularRangeIndicator is used to represent a range of values on a scale.\nUse the addIndicator method of CircularScale to use it.\nIt is represented as a donut slice.</p>\n","_animation":{},"_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_cursorListeners":{},"_downListeners":{},"_indicatorShapeFuncFlag":{},"_interactionAreaFlag":{},"_moveAndUpListeners":{},"_preventAnimation":{},"_transitionValue":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"animationEaser":{"!doc":"<p>The easer function of the value change animation. Default is fx._defaultEasing.</p>\n"},"endThickness":{"!doc":"<p>The end thickness of the donut slice in pixels. </p>\n"},"fill":{"!doc":"<p>A fill object that will be passed to the setFill method of GFX.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot;, &quot;area&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf set to &quot;area&quot;, the whole bounding box of the widget reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Deprecated. Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"radius":{"!doc":"<p>The outer radius in pixels of the range indicator.</p>\n"},"scale":{},"start":{"!doc":"<p>The start value of the range indicator.</p>\n"},"startThickness":{"!doc":"<p>The start thickness of the donut slice in pixels. </p>\n"},"stroke":{"!doc":"<p>A stroke object that will be passed to the setStroke method of GFX.</p>\n"},"value":{},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"postscript":{"!type":"fn(mixin: ?)"},"refreshRendering":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"valueChanged":{"!type":"fn(indicator: ?)","!doc":"<p>Invoked when the value of the indicator changes.\nUser can connect an listener on this function: \n\n</p>\n<pre><code>theIndicator.on(&quot;valueChanged&quot;, lang.hitch(this, function(){\n    //do something\n}));</code></pre>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/ScaleIndicatorBase":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/IndicatorBase.prototype","!doc":"<p>The base class for indicators that rely on a scale for their rendering.\nTypically, value indicators and range indicators are subclasses of ScaleIndicatorBase.</p>\n","_animation":{},"_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_cursorListeners":{},"_downListeners":{},"_indicatorShapeFuncFlag":{},"_interactionAreaFlag":{},"_moveAndUpListeners":{},"_preventAnimation":{},"_transitionValue":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"animationEaser":{"!doc":"<p>The easer function of the value change animation. Default is fx._defaultEasing.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot;, &quot;area&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf set to &quot;area&quot;, the whole bounding box of the widget reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Deprecated. Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"scale":{},"value":{},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"postscript":{"!type":"fn(mixin: ?)"},"refreshRendering":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"valueChanged":{"!type":"fn(indicator: ?)","!doc":"<p>Invoked when the value of the indicator changes.\nUser can connect an listener on this function: \n\n</p>\n<pre><code>theIndicator.on(&quot;valueChanged&quot;, lang.hitch(this, function(){\n    //do something\n}));</code></pre>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/IndicatorBase":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_Invalidating.prototype","!doc":"<p>The base class for indicators. Basically, an indicator is used to render a value.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"value":{"!doc":"<p>The value of this indicator.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"postscript":{"!type":"fn(mixin: ?)"},"refreshRendering":{"!type":"fn()","!doc":"<p>Actually refresh the rendering. Implementation should implement that method.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/_circularUtils":{},"dojox/dgauges/CircularScale":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/ScaleBase.prototype","!doc":"<p>The circular scale. A scaler must be set to use this class.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_bgGroup":{},"_fgGroup":{},"_gauge":{},"_gfxGroup":{},"_indicators":{},"_indicatorsIndex":{},"_indicatorsRenderers":{},"_watchers":{},"endAngle":{"!doc":"<p>The end angle of the scale in degrees.</p>\n"},"font":{"!doc":"<p>The font used for the ticks labels.\nThis is null by default which means this scale use the font defined \non the gauge.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"labelGap":{"!doc":"<p>The label gap between the ticks and their labels. Default value is 1.</p>\n"},"labelPosition":{"!doc":"<p>See CircularScale and RectangularScale for valid values.</p>\n"},"orientation":{"!doc":"<p>The orientation of the scale. Can be &quot;clockwise&quot; or &quot;cclockwise&quot;.\nThe default value is &quot;clockwise&quot;.</p>\n"},"originX":{"!doc":"<p>The origin x-coordinate of the scale in pixels.</p>\n"},"originY":{"!doc":"<p>The origin y-coordinate of the scale in pixels.</p>\n"},"radius":{"!doc":"<p>The outer radius in pixels of the scale.</p>\n"},"scaler":{"!doc":"<p>The scaler used for tick generation and data-transform operations.\nThis property is mandatory for using the scale.</p>\n"},"startAngle":{"!doc":"<p>The start angle of the scale in degrees.</p>\n"},"tickStroke":{"!doc":"<p>The GFX stroke used by the default tickShapeFunc implementation.</p>\n"},"addIndicator":{"!type":"fn(name: ?, indicator: ?, behindScale: ?) -> ?","!doc":"<p>Add an indicator to the scale. Before calling this function, ensure \nthis scale has already been added to a gauge using the addElement method\nof the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"getIndicator":{"!type":"fn(name: ?) -> +IndicatorBase","!doc":"<p>Get an indicator instance.</p>\n"},"getIndicatorRenderer":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Gets the GFX shape of an indicator.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"positionForValue":{"!type":"fn(value: ?) -> number","!doc":"<p>Transforms a value into an angle using the associated scaler.</p>\n"},"positionInRange":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns true if the value parameter is between the accepted scale positions.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeIndicator":{"!type":"fn(name: ?) -> +IndicatorBase","!doc":"<p>Removes an indicator.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"tickLabelFunc":{"!type":"fn(tickItem: ?) -> string","!doc":"<p>Customize the text of ticks labels.</p>\n"},"tickShapeFunc":{"!type":"fn(group: ?, scale: ?, tickItem: ?) -> ?","!doc":"<p>Customize the shape of ticks.</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"valueForPosition":{"!type":"fn(angle: ?) -> number","!doc":"<p>Transforms an angle in degrees into a value using the associated scaler.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/CircularValueIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/ScaleIndicatorBase.prototype","!doc":"<p>The circular value indicator, typically used for creating needles.</p>\n","_animation":{},"_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_cursorListeners":{},"_downListeners":{},"_indicatorShapeFuncFlag":{},"_interactionAreaFlag":{},"_moveAndUpListeners":{},"_preventAnimation":{},"_transitionValue":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"animationEaser":{"!doc":"<p>The easer function of the value change animation. Default is fx._defaultEasing.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot;, &quot;area&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf set to &quot;area&quot;, the whole bounding box of the widget reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Deprecated. Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"scale":{},"value":{},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"indicatorShapeFunc":{"!type":"fn(group: ?, indicator: ?) -> +dojox","!doc":"<p>Draws the indicator. The rotation center is at (0, 0).</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"postscript":{"!type":"fn(mixin: ?)"},"refreshRendering":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"valueChanged":{"!type":"fn(indicator: ?)","!doc":"<p>Invoked when the value of the indicator changes.\nUser can connect an listener on this function: \n\n</p>\n<pre><code>theIndicator.on(&quot;valueChanged&quot;, lang.hitch(this, function(){\n    //do something\n}));</code></pre>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/LinearScaler":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>The linear scaler. This scaler creates major and minor ticks regularly between \na minimum and a maximum.\nScalers are responsible for tick generation and various data-transform operations.      </p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_computedMajorTickInterval":{},"_computedMinorTickInterval":{},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"majorTicks":{"!doc":"<p>The array of generated major ticks. You should not set this\nproperty when using the scaler.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicks":{"!doc":"<p>The array of generated minor ticks. You should not set this\nproperty when using the scaler.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"computeTicks":{"!type":"fn() -> []","!doc":"<p>Creates or re-creates the ticks for this scaler.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"getComputedMajorTickInterval":{"!type":"fn() -> number","!doc":"<p>The computed or user defined major tick interval.</p>\n"},"getComputedMinorTickInterval":{"!type":"fn() -> number","!doc":"<p>The computed or user defined minor tick interval.</p>\n"},"positionForValue":{"!type":"fn(value: ?) -> number","!doc":"<p>Transforms a value into a relative position between 0 and 1.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"valueForPosition":{"!type":"fn(position: ?) -> number","!doc":"<p>Transforms a relative position (between 0 and 1) into a value.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/LogScaler":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>The LogScaler maps numeric values evenly\nbetween a minimum and a maximum value along a gauge scale.\nIf no multiplier is specified, the scale will place\na tick on each power of 10 value (1, 10, 100, 1000, and so on) between\nthe minimum and maximum values.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_computedMultiplier":{},"majorTicks":{"!doc":"<p>The array of generated major ticks. You should not set this\nproperty when using the scaler.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 1000.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"multiplier":{"!doc":"<p>The interval between two major ticks.</p>\n"},"computeTicks":{"!type":"fn() -> []","!doc":"<p>Creates or re-creates the ticks for this scaler.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"getComputedMaximum":{"!type":"fn() -> ?","!doc":"<p>The computed maximum value of the scale. If the maximum value is not\nan even power of 10, the scale computes a new maximum so that it maps to \nan even power of 10.</p>\n"},"getComputedMinimum":{"!type":"fn() -> ?","!doc":"<p>The computed minimum value of the scale. If the minimum value is not\nan even power of 10, the scale computes a new minimum so that it maps to \nan even power of 10.</p>\n"},"getComputedMultiplier":{"!type":"fn() -> ?","!doc":"<p>The computed multiplier value of the scale. If the multiplier value is not\nan even power of 10, the scale computes a new multiplier so that it maps to \nan even power of 10.</p>\n"},"positionForValue":{"!type":"fn(value: ?) -> number","!doc":"<p>Transforms a value into a relative position between 0 and 1.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"valueForPosition":{"!type":"fn(position: ?) -> number","!doc":"<p>Transforms a relative position (between 0 and 1) into a value.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/MultiLinearScaler":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>The multi-linear scaler. This scaler maps numeric values according \nto the majorTickValues content. \nThis allows display of very large value intervals that are difficult to render \nwith a linear scale. For example, if majorTickValues contains [0, 10, 50, 500, 2000], \nthe scale will show five major ticks with these values. \nNote that this is not a logarithmic scale, the interpolation is linear between \ntwo contiguous major ticks.\nScalers are responsible for tick generation and various data-transform operations.  </p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_snapCount":{},"_snapIntervalPrecision":{},"majorTicks":{"!doc":"<p>The array of generated major ticks. You should not set this\nproperty when using the scaler.</p>\n"},"majorTickValues":{"!doc":"<p>An array of Number for creating major ticks.\nThis array must be sorted in ascendant order.</p>\n"},"minorTickCount":{"!doc":"<p>The number of minor ticks between two contiguous major ticks.\nThe default value is 4.</p>\n"},"minorTicks":{"!doc":"<p>The array of generated minor ticks. You should not set this\nproperty when using the scaler.</p>\n"},"computeTicks":{"!type":"fn() -> []","!doc":"<p>Creates or re-creates the ticks for this scaler.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"positionForValue":{"!type":"fn(value: ?) -> number","!doc":"<p>Transforms a value into a relative position between 0 and 1.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"valueForPosition":{"!type":"fn(position: ?) -> number","!doc":"<p>Transforms a relative position (between 0 and 1) into a value.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/RectangularGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/GaugeBase.prototype","!doc":"<p>The base class for rectangular gauges.\nYou can create custom horizontal or vertical gauges by extending this class.\nSee dojox/dgauges/components/default/HorinzontalLinearGauge.js for an example of rectangular gauge.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/RectangularRangeIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/ScaleIndicatorBase.prototype","!doc":"<p>A RectangularRangeIndicator is used to represent a range of values on a scale.\nFor adding this kind of indicator instance to the gauge, use the addIndicator \nmethod of RectangularScale.</p>\n","_animation":{},"_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_cursorListeners":{},"_downListeners":{},"_indicatorShapeFuncFlag":{},"_interactionAreaFlag":{},"_moveAndUpListeners":{},"_preventAnimation":{},"_transitionValue":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"animationEaser":{"!doc":"<p>The easer function of the value change animation. Default is fx._defaultEasing.</p>\n"},"endThickness":{"!doc":"<p>The thickness in pixels of the shape at the position defined by the value property.\nDefault is 10.</p>\n"},"fill":{"!doc":"<p>A fill object that will be passed to the setFill method of GFX.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot;, &quot;area&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf set to &quot;area&quot;, the whole bounding box of the widget reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Deprecated. Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"paddingBottom":{"!doc":"<p>The bottom padding. Not used for vertical gauges.</p>\n"},"paddingLeft":{"!doc":"<p>The left padding. Not used for horizontal gauges.</p>\n"},"paddingRight":{"!doc":"<p>The right padding. Not used for horizontal gauges.</p>\n"},"paddingTop":{"!doc":"<p>The top padding. Not used for vertical gauges.</p>\n"},"scale":{},"start":{"!doc":"<p>The start value of the range. Default is 0.</p>\n"},"startThickness":{"!doc":"<p>The thickness in pixels of the shape at the position defined by the start property.\nDefault is 10.</p>\n"},"stroke":{"!doc":"<p>A stroke object that will be passed to the setStroke method of GFX.</p>\n"},"value":{},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"postscript":{"!type":"fn(mixin: ?)"},"refreshRendering":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"valueChanged":{"!type":"fn(indicator: ?)","!doc":"<p>Invoked when the value of the indicator changes.\nUser can connect an listener on this function: \n\n</p>\n<pre><code>theIndicator.on(&quot;valueChanged&quot;, lang.hitch(this, function(){\n    //do something\n}));</code></pre>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/RectangularScale":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/ScaleBase.prototype","!doc":"<p>The rectangular scale. A scaler must be set to use this class.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_bgGroup":{},"_contentBox":{},"_fgGroup":{},"_gauge":{},"_gfxGroup":{},"_indicators":{},"_indicatorsIndex":{},"_indicatorsRenderers":{},"_watchers":{},"font":{"!doc":"<p>The font used for the ticks labels.\nThis is null by default which means this scale use the font defined \non the gauge.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"labelGap":{"!doc":"<p>The label gap between the ticks and their labels. Default value is 1.</p>\n"},"labelPosition":{"!doc":"<p>See CircularScale and RectangularScale for valid values.</p>\n"},"paddingBottom":{"!doc":"<p>The bottom padding.</p>\n"},"paddingLeft":{"!doc":"<p>The left padding.</p>\n"},"paddingRight":{"!doc":"<p>The right padding.</p>\n"},"paddingTop":{"!doc":"<p>The top padding.</p>\n"},"scaler":{"!doc":"<p>The scaler used for tick generation and data-transform operations.\nThis property is mandatory for using the scale.</p>\n"},"tickStroke":{"!doc":"<p>The GFX stroke used by the default tickShapeFunc implementation.</p>\n"},"addIndicator":{"!type":"fn(name: ?, indicator: ?, behindScale: ?) -> ?","!doc":"<p>Add an indicator to the scale. Before calling this function, ensure \nthis scale has already been added to a gauge using the addElement method\nof the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"getIndicator":{"!type":"fn(name: ?) -> +IndicatorBase","!doc":"<p>Get an indicator instance.</p>\n"},"getIndicatorRenderer":{"!type":"fn(name: ?) -> +dojox","!doc":"<p>Gets the GFX shape of an indicator.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"positionForValue":{"!type":"fn(value: ?) -> number","!doc":"<p>Transforms a value into a position using the associated scaler.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeIndicator":{"!type":"fn(name: ?) -> +IndicatorBase","!doc":"<p>Removes an indicator.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"tickLabelFunc":{"!type":"fn(tickItem: ?) -> string","!doc":"<p>Customize the text of ticks labels.</p>\n"},"tickShapeFunc":{"!type":"fn(group: ?, scale: ?, tickItem: ?) -> ?","!doc":"<p>Customize the shape of ticks.</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"valueForPosition":{"!type":"fn(pos: ?) -> number","!doc":"<p>Transforms a position in pixels into a value using the associated scaler.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/RectangularSegmentedRangeIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/IndicatorBase.prototype","!doc":"<p>A segmented-style range indicator for rectangular gauges.\nThis class will be replaced by a cleaner implementation in a future version.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"endThickness":{},"fill":{},"invalidatingProperties":{},"invalidRendering":{},"paddingBottom":{},"paddingLeft":{},"paddingRight":{},"paddingTop":{},"ranges":{"!doc":"<p>An array containing objects to define color ranges. Example [{color:&quot;#FF0000&quot;, size:20}, {color:&quot;#FF8800&quot;, size:50}].</p>\n"},"rounded":{"!doc":"<p>Indicates if the extremity segments are rounded.\nDefault is true.        </p>\n"},"segments":{"!doc":"<p>The number of segments making the indicator.\nBy default it is 10.</p>\n"},"segmentSpacing":{"!doc":"<p>The blank space between two segments. The default value is 2</p>\n"},"start":{},"startThickness":{},"stroke":{},"value":{"!doc":"<p>The value of this indicator.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Constructor.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"postscript":{"!type":"fn(mixin: ?)"},"refreshRendering":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/RectangularValueIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/ScaleIndicatorBase.prototype","!doc":"<p>The rectangular value indicator, typically used for creating markers or thumbs.</p>\n","_animation":{},"_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_cursorListeners":{},"_downListeners":{},"_indicatorShapeFuncFlag":{},"_interactionAreaFlag":{},"_moveAndUpListeners":{},"_preventAnimation":{},"_transitionValue":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"animationEaser":{"!doc":"<p>The easer function of the value change animation. Default is fx._defaultEasing.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot;, &quot;area&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf set to &quot;area&quot;, the whole bounding box of the widget reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Deprecated. Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"paddingBottom":{"!doc":"<p>The bottom padding.</p>\n"},"paddingLeft":{"!doc":"<p>The left padding.</p>\n"},"paddingRight":{"!doc":"<p>The right padding.</p>\n"},"paddingTop":{"!doc":"<p>The top padding.</p>\n"},"scale":{},"value":{},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"indicatorShapeFunc":{"!type":"fn(group: ?, indicator: ?) -> +dojox","!doc":"<p>Draws the indicator.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"postscript":{"!type":"fn(mixin: ?)"},"refreshRendering":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"valueChanged":{"!type":"fn(indicator: ?)","!doc":"<p>Invoked when the value of the indicator changes.\nUser can connect an listener on this function: \n\n</p>\n<pre><code>theIndicator.on(&quot;valueChanged&quot;, lang.hitch(this, function(){\n    //do something\n}));</code></pre>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/TextIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/IndicatorBase.prototype","!doc":"<p>This type of indicator is used to render text.\nTo render an arbitrary text, set the value property.\nTo render the value of a value indicator or a range indicator, set the indicator property.\nSetting the indicator property takes precedence on setting the value property.\nWhen the indicator property is set, the text is automatically updated on value changes.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_textCreated":{},"_textInstance":{},"_valueWatcher":{},"align":{"!doc":"<p>An alignment of a text in regards to the anchor position:\n\n</p>\n<ul>\n<li>&quot;start&quot;: A text&#39;s baseline starts at the anchor. \nThis is the default value of the align attribute.</li>\n<li>&quot;middle&quot;: A text&#39;s baseline is centered on the anchor point.</li>\n<li>&quot;end&quot;: A text&#39;s baseline ends at the anchor point.</li>\n</ul>\n"},"color":{"!doc":"<p>The color of the text.</p>\n"},"font":{"!doc":"<p>Font used by this element.</p>\n"},"indicator":{"!doc":"<p>If this property is set, the value of the indicator is automatically\nrendered by this text element.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"labelFunc":{"!doc":"<p>If set, this method allows to format the value of this text indicator.\nA label function takes the text to render as argument and returns a String. </p>\n"},"value":{"!doc":"<p>The value of this indicator.</p>\n"},"x":{"!doc":"<p>The text anchor x-position. Default is 0.</p>\n"},"y":{"!doc":"<p>The text anchor y-position. Default is 0.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"refreshRendering":{"!type":"fn() -> ?"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/DefaultPropertiesMixin":{"!type":"fn()","prototype":{"!doc":"<p>This class defines default properties of predefined gauges.</p>\n","animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"value":{}}},"dojox/dgauges/components/black/CircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#000000&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#000000&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#A4A4A4&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/black/HorizontalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A horizontal gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#000000&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#000000&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#A4A4A4&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/black/SemiCircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A semi circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#000000&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#000000&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#A4A4A4&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/black/VerticalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A vertical gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#000000&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#000000&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#A4A4A4&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/classic/CircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#797E86&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#9498A1&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#FFFFFF&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/classic/HorizontalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A horizontal gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#797E86&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#9498A1&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#FFFFFF&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/classic/SemiCircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A semi circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#797E86&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#9498A1&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#FFFFFF&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/classic/VerticalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A vertical gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#797E86&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#9498A1&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#FFFFFF&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/default/CircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_radius":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#C9DFF2&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#FCFCFF&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#F01E28&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/utils":{"brightness":{"!type":"fn(col: ?, b: ?) -> number","!doc":"<p>Adjusts the brightness of a color.</p>\n"},"createGradient":{"!type":"fn(entries: ?) -> number","!doc":"<p>Creates a gradient object</p>\n"},"genericCircularGauge":{"!type":"fn(scale: ?, indicator: ?, originX: ?, originY: ?, radius: ?, startAngle: ?, endAngle: ?, orientation?: ?, font?: ?, labelPosition?: ?, tickShapeFunc?: ?) -> number","!doc":"<p>A helper method for configuring a circular gauge.</p>\n"}},"dojox/dgauges/components/default/HorizontalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A horizontal gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#C9DFF2&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#FCFCFF&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#F01E28&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/default/SemiCircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A semi circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_height":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_radius":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"_width":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#C9DFF2&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#FCFCFF&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#F01E28&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/default/VerticalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A vertical gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#C9DFF2&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#FCFCFF&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#F01E28&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/green/CircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#323232&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The fill color. Default is &quot;#6DB713&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#000000&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/green/HorizontalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A horizontal gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#323232&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#6DB713&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#000000&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/green/SemiCircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A semi circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#323232&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The fill color. Default is &quot;#6DB713&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#000000&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/green/VerticalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A vertical gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#323232&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#6DB713&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#000000&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/grey/CircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#9498A1&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#9498A1&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#3F3F3F&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(args: ?, node: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/grey/HorizontalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A horizontal gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#9498A1&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#9498A1&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#3F3F3F&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/grey/SemiCircularLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/CircularGauge.prototype","!doc":"<p>A semi circular gauge widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_mainIndicator":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_transformProperties":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#9498A1&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#9498A1&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#3F3F3F&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?) -> ?","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(args: ?, node: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"drawForeground":{"!type":"fn(g: ?)","!doc":"<p>Draws the foreground shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dgauges/components/grey/VerticalLinearGauge":{"!type":"fn()","prototype":{"!proto":"dojox/dgauges/RectangularGauge.prototype","!doc":"<p>A vertical gauge widget.</p>\n","_attrPairNames":{},"_baseParts":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_classParts":{},"_elements":{},"_elementsIndex":{},"_elementsRenderers":{},"_gfxGroup":{},"_layoutInfos":{},"_leadingParts":{},"_mainIndicator":{},"_middleParts":{},"_mouseShield":{},"_node":{},"_scales":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_trailingParts":{},"_widgetBox":{},"animationDuration":{"!doc":"<p>The duration of the value change animation in milliseconds. Default is 0.\nThe animation occurs on both user interactions and programmatic value changes.\nSet this property to 0 to disable animation.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"borderColor":{"!doc":"<p>The border color. Default is &quot;#9498A1&quot;.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fillColor":{"!doc":"<p>The background color. Default is &quot;#9498A1&quot;.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge used by elements if not overridden.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"indicatorColor":{"!doc":"<p>The indicator fill color. Default is &quot;#3F3F3F&quot;.</p>\n"},"interactionArea":{"!doc":"<p>How to interact with the indicator using mouse or touch interactions.\nCan be &quot;indicator&quot;, &quot;gauge&quot; or &quot;none&quot;. The default value is &quot;gauge&quot;.\nIf set to &quot;indicator&quot;, the indicator shape reacts to mouse and touch events.\nIf set to &quot;gauge&quot;, the whole gauge reacts to mouse and touch events.\nIf &quot;none&quot;, interactions are disabled.</p>\n"},"interactionMode":{"!doc":"<p>Can be &quot;mouse&quot; or &quot;touch&quot;.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTickInterval":{"!doc":"<p>The interval between two major ticks.</p>\n"},"maximum":{"!doc":"<p>The maximum value of the scaler. Default is 100.</p>\n"},"minimum":{"!doc":"<p>The minimum value of the scaler. Default is 0.</p>\n"},"minorTickInterval":{"!doc":"<p>The interval between two minor ticks.</p>\n"},"minorTicksEnabled":{"!doc":"<p>If false, minor ticks are not generated. Default is true.</p>\n"},"orientation":{"!doc":"<p>The orientation of the gauge. Default is &quot;horizontal&quot;.  </p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"snapInterval":{"!doc":"<p>Specifies the increment value to be used as snap values on this scale \nduring user interaction.\nDefault is 1.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>This property acts as a top-level wrapper for the value of the first indicator added to \nits scale with the name &quot;indicator&quot;, i.e. myScale.addIndicator(&quot;indicator&quot;, myIndicator).\nThis property must be manipulated with get(&quot;value&quot;) and set(&quot;value&quot;, xxx).</p>\n"},"addElement":{"!type":"fn(name: ?, element: ?, location: ?)","!doc":"<p>Adds a element to the gauge.</p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup when a gauge is to be destroyed.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"drawBackground":{"!type":"fn(g: ?, w: ?, h: ?)","!doc":"<p>Draws the background shape of the gauge.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element, by name.</p>\n"},"getElementRenderer":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get the given element renderer, by name.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refreshRendering":{"!type":"fn()"},"removeElement":{"!type":"fn(name: ?) -> ?","!doc":"<p>Remove the element defined by name from the gauge.</p>\n"},"resize":{"!type":"fn(width?: ?, height?: ?) -> +dojox","!doc":"<p>Resize the gauge to the dimensions of width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dnd/BoundingBoxController":{"!type":"fn()","prototype":{"!doc":"<p>Allows the user draw bounding boxes around nodes on the page.\nPublishes to the &quot;/dojox/dnd/bounding&quot; topic to tell the selector to check\nto see whether any dnd items fall within the coordinates of the bounding box</p>\n","_endX":{},"_endY":{},"_startX":{},"_startY":{},"boundingBoxIsViable":{"!type":"fn(evt: ?) -> bool","!doc":"<p>Override-able by the client as an extra check to ensure that a bounding\nbox is viable. In some instances, it might not make sense that\na mouse down -&gt; mouse move -&gt; mouse up interaction represents a bounding box.\nFor example, if a dialog is open the client might want to suppress a bounding\nbox. This function could be used by the client to ensure that a bounding box is only\ndrawn on the document when certain conditions are met.</p>\n"},"constructor":{"!type":"fn(sources: ?, domNode: ?)","!doc":"<p>Sets mouse handlers for the document to capture when a user\nis trying to draw a bounding box.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>prepares this object to be garbage-collected</p>\n"},"shouldStartDrawingBox":{"!type":"fn(evt: ?) -> bool","!doc":"<p>Override-able by the client as an extra check to ensure that a bounding\nbox should begin to be drawn. If the client has any preconditions to when a\nbounding box should be drawn, they should be included in this method.</p>\n"}}},"dojox/dnd/Selector":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Selector.prototype","allowNested":{"!doc":"<p>Indicates whether to allow dnd item nodes to be nested within other elements.\nBy default this is false, indicating that only direct children of the container can\nbe draggable dnd item nodes</p>\n"},"conservative":{},"current":{"!doc":"<p>The DOM node the mouse is currently hovered over</p>\n"},"map":{"!doc":"<p>Map from an item&#39;s id (which is also the DOMNode&#39;s id) to\nthe dojo/dnd/Container.Item itself.</p>\n"},"selection":{"!doc":"<p>The set of id&#39;s that are currently selected, such that this.selection[id] == 1\nif the node w/that id is selected.  Can iterate over selected node&#39;s id&#39;s like:\n\n</p>\n<pre><code>for(var id in this.selection)</code></pre>\n"},"singular":{},"skipForm":{},"clearItems":{"!type":"fn()","!doc":"<p>removes all data items from the map</p>\n"},"constructor":{"!type":"fn(node: ?, params?: ?)","!doc":"<p>constructor of the Selector</p>\n"},"creator":{"!type":"fn()","!doc":"<p>creator function, dummy at the moment</p>\n"},"deleteSelectedNodes":{"!type":"fn() -> fn","!doc":"<p>deletes all selected items</p>\n"},"delItem":{"!type":"fn(key: ?)","!doc":"<p>removes a data item from the map by its key (id)</p>\n"},"deselectNode":{"!type":"fn(node: ?) -> fn","!doc":"<p>deselects a node</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>prepares the object to be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"forInItems":{"!type":"fn(f: ?, o?: ?) -> string","!doc":"<p>iterates over a data map skipping members that\nare present in the empty object (IE and/or 3rd-party libraries).</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getAllNodes":{"!type":"fn() -> ?","!doc":"<p>returns a list (an array) of all valid child nodes</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>returns a data item by its key (id)</p>\n"},"getSelectedNodes":{"!type":"fn() -> +instance","!doc":"<p>returns a list (an array) of selected nodes</p>\n"},"insertNodes":{"!type":"fn(addSelected: ?, data: ?, before: ?, anchor: ?) -> fn","!doc":"<p>inserts new data items (see <code>dojo/dnd/Container.insertNodes()</code> method for details)</p>\n"},"isSelected":{"!type":"fn(node: ?) -> ?","!doc":"<p>checks if node is selected</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"selectAll":{"!type":"fn() -> ?","!doc":"<p>selects all items</p>\n"},"selectByBBox":{"!type":"fn(left: ?, top: ?, right: ?, bottom: ?, add?: ?) -> fn","!doc":"<p>selects nodes by bounding box</p>\n"},"selectNode":{"!type":"fn(node: ?, add?: ?) -> fn","!doc":"<p>selects a node</p>\n"},"selectNone":{"!type":"fn() -> ?","!doc":"<p>unselects all items</p>\n"},"setItem":{"!type":"fn(key: ?, data: ?)","!doc":"<p>associates a data item with its key (id)</p>\n"},"shift":{"!type":"fn(toNext: ?, add?: ?)","!doc":"<p>shifts the currently selected dnd item forwards and backwards.\nOne possible use would be to allow a user select different\ndnd items using the right and left keys.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>collects valid child items and populate the map</p>\n"},"sync":{"!type":"fn() -> fn","!doc":"<p>sync up the node list with the data map</p>\n"}}},"dojox/drawing/Drawing":{"!type":"fn()","prototype":{"!doc":"<p>Drawing is a project that sits on top of DojoX GFX and uses SVG and\nVML vector graphics to draw and display.</p>\n","height":{"!doc":"<p>Height of the canvas</p>\n"},"mode":{"!doc":"<p>Changes the functionality of the drawing</p>\n"},"ready":{"!doc":"<p>Whether or not the canvas has been created and Stencils can be added</p>\n"},"width":{"!doc":"<p>Width of the canvas</p>\n"},"addPlugin":{"!type":"fn(plugin: ?)","!doc":"<p>Add a toolbar plugin object to plugins array\nto be parsed</p>\n"},"addStencil":{"!type":"fn(type: ?, options: ?) -> ?","!doc":"<p>Use this method to programmatically add Stencils that display on\nthe canvas.\n\n</p>\n<p>FIXME: Currently only supports Stencils that have been registered,\nwhich is items in the toolbar, and the additional Stencils at the\nend of onSurfaceReady. This covers all Stencils, but you can&#39;t\nuse &#39;display only&#39; Stencils for Line, Rect, and Ellipse.</p>\n"},"addUI":{"!type":"fn(type: ?, options: ?) -> +null","!doc":"<p>Use this method to programmatically add Stencils that display on\nthe canvas.\n\n</p>\n<p>FIXME: Currently only supports Stencils that have been registered,\nwhich is items in the toolbar, and the additional Stencils at the\nend of onSurfaceReady. This covers all Stencils, but you can&#39;t\nuse &#39;display only&#39; Stencils for Line, Rect, and Ellipse.</p>\n"},"changeDefaults":{"!type":"fn(newStyle: ?, value: ?)","!doc":"<p>Change the defaults so that all Stencils from this\npoint on will use the newly changed style.</p>\n"},"constructor":{"!type":"fn(props: ?, node: ?)","!doc":"<p>Drawing is not a Dijit. This is the master method.\n\n</p>\n<p>NOTE:\nprops is always null since this is not a real widget\nWill change when Drawing can be created programmatically.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Collects all Stencil data and returns an\nArray of objects.</p>\n"},"get":{"!type":"fn(name: ?)"},"getConstructor":{"!type":"fn(abbr: ?) -> ?","!doc":"<p>Returns a Stencil constructor base on\nabbreviation</p>\n"},"getShapeProps":{"!type":"fn(data: ?, mode: ?) -> ?","!doc":"<p>The common objects that are mixed into\na new Stencil. Mostly internal, but could be used.</p>\n"},"importer":{"!type":"fn(objects: ?)","!doc":"<p>Handles an Array of stencil data and imports the objects\nto the drawing.</p>\n"},"initPlugins":{"!type":"fn()","!doc":"<p>Called from Toolbar after a plugin has been loaded\nThe call to this coming from toolbar is a bit funky as the timing\nof IE for canvas load is different than other browsers</p>\n"},"registerTool":{"!type":"fn(type: ?)","!doc":"<p>Registers a tool that can be accessed. Internal.</p>\n"},"removeAll":{"!type":"fn()","!doc":"<p>Deletes all Stencils on the canvas.</p>\n"},"removeStencil":{"!type":"fn(stencil: ?)","!doc":"<p>Use this method to programmatically remove Stencils from the canvas.</p>\n"},"resize":{"!type":"fn(box: ?)","!doc":"<p>Resizes the canvas.\nIf within a ContentPane this will get called automatically.\nCan also be called directly.</p>\n"},"selectAll":{"!type":"fn()","!doc":"<p>Selects all stencils</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Drawing registers as a widget and needs to support\nwidget&#39;s api.</p>\n"},"setTool":{"!type":"fn(type: ?)","!doc":"<p>Sets up a new class to be used to draw. Called from Toolbar,\nand this class... after a tool is used a new one of the same\ntype is initialized. Could be called externally.</p>\n"},"startup":{"!type":"fn()"},"toSelected":{"!type":"fn(func: ?)","!doc":"<p>Call a function within all selected Stencils\nlike attr()</p>\n"},"unSetTool":{"!type":"fn()","!doc":"<p>Destroys current tool</p>\n"}}},"dojox/drawing/defaults":{"!doc":"<p>Styles and defaults used for Drawing stencils and text.</p>\n","anchors":{"!doc":"<p>Style for the anchor resize-points</p>\n"},"angleSnap":{"!doc":"<p>Line, arrows, vector and axes will all snap to this angle on mouse up\nshown angle also reflects the snap\ncurrently cannot accept less than 1 degree</p>\n"},"arrows":{"!doc":"<p>Size of arrows on vectors.\nlength is in pixels\nwidth is actually an angle\nbut is close to pixels in size</p>\n"},"button":{},"clickable":{},"clickMode":{"!doc":"<p>Determines whether in draw or edit mode (whether stencils\nare clickable.  If clickMode is false, the original\nfunctionality of silently switching between select modes\nis enabled.  If clickMode is true, it allows powerpoint-\nlike functionality.  Clickable is used by powerpoint to\ndistinguish when things can be selected and when they can&#39;t</p>\n"},"current":{"!doc":"<p>current will point to either null or selected</p>\n"},"currentHit":{"!doc":"<p>currentHit will point to either hitNorm or hitSelected</p>\n"},"disabled":{"!doc":"<p>Disabled or &quot;locked&quot; or &quot;fade&quot; style of all shapes</p>\n"},"highlighted":{"!doc":"<p>Highlighted style of all shapes\nNOT CURRENTLY BEING USED</p>\n"},"hitHighlighted":{"!doc":"<p>Highlighted style of a hit area</p>\n"},"hitNorm":{"!doc":"<p>Normal style of a hit area</p>\n"},"hitSelected":{"!doc":"<p>Selected style of a hit area</p>\n"},"labelSameColor":{"!doc":"<p>If true, the label text color will be the same as the\nStencil&#39;s line color.</p>\n"},"norm":{"!doc":"<p>Normal style of all shapes\nwill get overridden by\nabove andes styles</p>\n"},"renderHitLayer":{"!doc":"<p>If true, renders a second layer for each Stencil, one\nacting as a &#39;hit&#39; object for a wider mouse-click area.\nIt also doubles as a hilight. If true, overrides\nrenderHitLines setting.</p>\n"},"renderHitLines":{"!doc":"<p>If true, renders a second, larger layer for lines to make\nthem more easily clickable.</p>\n"},"selected":{"!doc":"<p>Selected style of all shapes\nstyles not shown will used from\nnorm</p>\n"},"text":{"!doc":"<p>Style of text</p>\n"},"textDisabled":{"!doc":"<p>Style of disabled text</p>\n"},"textMode":{"!doc":"<p>These styles apply to the containing\ntext box (edit mode), and not the text itself</p>\n"},"useSelectedStyle":{},"zAngle":{},"zAxis":{"!doc":"<p>If true, draw current object in z-direction.</p>\n"},"zAxisEnabled":{},"copy":{"!type":"fn() -> ?","!doc":"<p>Each shape gets its own copy\nof these styles so that instances\ndo not change each other&#39;s styles</p>\n"}},"dojox/drawing/defaults.norm":{"!doc":"<p>Normal style of all shapes\nwill get overridden by\nabove andes styles</p>\n","cap":{},"color":{},"fill":{},"style":{},"width":{}},"dojox/drawing/defaults.selected":{"!doc":"<p>Selected style of all shapes\nstyles not shown will used from\nnorm</p>\n","color":{},"width":{}},"dojox/drawing/defaults.highlighted":{"!doc":"<p>Highlighted style of all shapes\nNOT CURRENTLY BEING USED</p>\n","cap":{},"color":{},"fill":{},"style":{},"width":{}},"dojox/drawing/defaults.disabled":{"!doc":"<p>Disabled or &quot;locked&quot; or &quot;fade&quot; style of all shapes</p>\n","cap":{},"color":{},"fill":{},"style":{},"width":{}},"dojox/drawing/defaults.hitNorm":{"!doc":"<p>Normal style of a hit area</p>\n","cap":{},"color":{},"fill":{},"style":{},"width":{}},"dojox/drawing/defaults.hitSelected":{"!doc":"<p>Selected style of a hit area</p>\n","cap":{},"color":{},"fill":{},"style":{},"width":{}},"dojox/drawing/defaults.hitHighlighted":{"!doc":"<p>Highlighted style of a hit area</p>\n","cap":{},"color":{},"fill":{},"style":{},"width":{}},"dojox/drawing/defaults.anchors":{"!doc":"<p>Style for the anchor resize-points</p>\n","cap":{},"color":{},"fill":{},"marginZero":{},"minSize":{},"size":{},"style":{},"width":{}},"dojox/drawing/defaults.arrows":{"!doc":"<p>Size of arrows on vectors.\nlength is in pixels\nwidth is actually an angle\nbut is close to pixels in size</p>\n","length":{},"width":{}},"dojox/drawing/defaults.text":{"!doc":"<p>Style of text</p>\n","color":{},"deleteEmptyCreate":{},"deleteEmptyModify":{},"family":{},"minWidth":{},"pad":{},"size":{},"weight":{}},"dojox/drawing/defaults.textDisabled":{"!doc":"<p>Style of disabled text</p>\n","color":{},"family":{},"size":{},"weight":{}},"dojox/drawing/defaults.textMode":{"!doc":"<p>These styles apply to the containing\ntext box (edit mode), and not the text itself</p>\n","create":{},"edit":{}},"dojox/drawing/defaults.button":{"down":{},"icon":{},"norm":{},"over":{},"selected":{}},"dojox/drawing/manager/_registry":{"getRegistered":{"!type":"fn(type: ?, id: ?) -> ?"},"register":{"!type":"fn(item: ?, type: ?)"}},"dojox/drawing/manager/keys":{"!doc":"<p>A singleton, master object that detects\nkeyboard keys and events\nConnect to it like:\n\n</p>\n<pre><code>dojo.connect(this.keys, &quot;onEnter&quot;, ....);</code></pre>\n","alt":{"!doc":"<p>Indicates whether the Alt or Option key is currently pressed</p>\n"},"arrowIncrement":{"!doc":"<p>The amount, in pixels, a selected Stencil will\nmove on an arrow key event</p>\n"},"arrowShiftIncrement":{"!doc":"<p>The amount, in pixels, a selected Stencil will\nmove on an arrow key + SHIFT event</p>\n"},"cmmd":{"!doc":"<p>Indicates whether the Apple Command key is currently pressed</p>\n"},"ctrl":{"!doc":"<p>Indicates whether the Control key is currently pressed</p>\n"},"listeners":{},"meta":{"!doc":"<p>Indicates whether any &#39;meta&#39; key is currently pressed:\nshift || ctrl || cmd || alt</p>\n"},"shift":{"!doc":"<p>Indicates whether the Shift key is currently pressed</p>\n"},"editMode":{"!type":"fn(_isedit: ?)","!doc":"<p>Relinquishes control of events to another portion\nof Drawing; namely the TextBlock.</p>\n"},"enable":{"!type":"fn(_enabled: ?)","!doc":"<p>Enables or disables key events, to relinquish\ncontrol to something outside of Drawing; input\nfields for example.\nYou may need to call this directly if you are\nusing textareas or contenteditables.\nNOTE: See scanForFields</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initialize the keys object</p>\n"},"register":{"!type":"fn(options: ?)","!doc":"<p>Register an object and callback to be notified\nof events.\nNOTE: Not really used in code, but should work.\nSee manager.mouse for similar usage</p>\n"},"scanForFields":{"!type":"fn()","!doc":"<p>Scans the document for inputs\nand calls this automatically. However you may need\nto call this if you create inputs after the fact.</p>\n"}},"dojox/drawing/util/common":{"!doc":"<p>A collection of common methods used for DojoX Drawing.\nThis singleton is accessible in most Drawing classes\nas this.util</p>\n","objects":{},"abbr":{"!type":"fn(type: ?) -> number","!doc":"<p>Converts a namespace (typically a tool or a stencil) into\nan abbreviation</p>\n"},"angle":{"!type":"fn(obj: ?, snap?: ?) -> ?","!doc":"<p>Return angle based on mouse object</p>\n"},"argsToObj":{"!type":"fn() -> ?","!doc":"<p>Attempts to determine in a Mouse Object\nwas passed or indiviual numbers. Returns\nan object.</p>\n"},"attr":{"!type":"fn(elem: ?, prop: ?, value: ?, squelchErrors: ?) -> bool","!doc":"<p>Helper function to attach attributes to SVG and VML raw nodes.</p>\n"},"byId":{"!type":"fn(id: ?) -> ?","!doc":"<p>Get an object that was registered with util.register</p>\n"},"constrainAngle":{"!type":"fn(obj: ?, min: ?, max: ?) -> +EventObject","!doc":"<p>Ensures the angle in the Mouse Object is within the\nmin and max limits. If not one of those limits is used.\nReturns an x,y point for the angle used.</p>\n"},"degToRad":{"!type":"fn(n: ?) -> number","!doc":"<p>Convert the passed number to radians.</p>\n"},"distance":{"!type":"fn() -> ?","!doc":"<p>Return the length derived from the coordinates\nin the Mouse object. Different from util.length\nin that this always returns an absolute value.</p>\n"},"idSetStart":{"!type":"fn(num: ?)"},"length":{"!type":"fn(o: ?) -> ?","!doc":"<p>Return the length derived from the coordinates\nin the Mouse object.</p>\n"},"lineSub":{"!type":"fn(x1: ?, y1: ?, x2: ?, y2: ?, amt: ?) -> ?","!doc":"<p>Subtract an amount from a line</p>\n"},"mixin":{"!type":"fn(o1: ?, o2: ?)"},"oppAngle":{"!type":"fn(ang: ?) -> +Angle"},"pointOnCircle":{"!type":"fn(cx: ?, cy: ?, radius: ?, angle: ?) -> ?","!doc":"<p>A <em>very</em> helpful method. If you know the center\n(or starting) point, length and angle, find the\nx,y point at the end of that line.</p>\n"},"radians":{"!type":"fn(o: ?) -> ?","!doc":"<p>Return the radians derived from the coordinates\nin the Mouse object.</p>\n"},"radToDeg":{"!type":"fn(n: ?) -> number","!doc":"<p>Convert the passed number to degrees.</p>\n"},"register":{"!type":"fn(obj: ?)","!doc":"<p>Since util is the only Singleton in Drawing (besides\nkeys) it is used to help connect the Drawing object\nthe Toolbar. Since multiple drawings can be on one\npage, this function serves a little more use than\non first apearance.</p>\n"},"slope":{"!type":"fn(p1: ?, p2: ?) -> number","!doc":"<p>Given two poits of a line, returns the slope.</p>\n"},"snapAngle":{"!type":"fn(obj: ?, ca: ?) -> ?","!doc":"<p>Snaps a line to the nearest angle</p>\n"},"uid":{"!type":"fn(str?: ?) -> string","!doc":"<p>Creates a unique ID.</p>\n"}},"dojox/drawing/util/common.objects":{},"dojox/math/round":{"prototype":{"constructor":{"!type":"fn(v: ?, p: ?, m: ?) -> number"}}},"dojox/drawing/manager/Mouse":{"!type":"fn()","prototype":{"!doc":"<p>Master object (instance) that tracks mouse\nevents. A new instance is created for each\nDrawing object.</p>\n","__reg":{},"_downOnCanvas":{},"_lastx":{},"_lasty":{},"CustomEventMethod":{"!doc":"<p>The custom event methods that an Object that has\nregistered with manager.Mouse can receive.\nCan contain any or all of the following methods\nand they will be called on mouse events. All events\nwill be sent a EventObject event object.\n\n</p>\n<p>NOTE:\nEvents happen anywhere in the document unless\notherwise noted.</p>\n"},"doublClickSpeed":{"!doc":"<p>Milliseconds between clicks to\nregister as for onDoubleClick</p>\n"},"zoom":{},"constructor":{"!type":"fn(options: ?)"},"create":{"!type":"fn(evt: ?, squelchErrors: ?) -> ?","!doc":"<p>Internal. Create EventObject</p>\n"},"down":{"!type":"fn(evt: ?)","!doc":"<p>Internal. Create onDown event</p>\n"},"drag":{"!type":"fn(evt: ?)","!doc":"<p>Internal. Create onDrag event</p>\n"},"eventName":{"!type":"fn(name: ?) -> string","!doc":"<p>Internal. Determine the event name</p>\n"},"EventObject":{"!type":"fn()","!doc":"<p>The custom event object that is sent to registered objects\nand their respective methods.\n\n</p>\n<p>NOTE: Most event objects are the same with the exception\nof the onDown events, which have fewer.</p>\n"},"init":{"!type":"fn(node: ?)","!doc":"<p>Internal. Initializes mouse.</p>\n"},"move":{"!type":"fn(evt: ?)","!doc":"<p>Internal.</p>\n"},"out":{"!type":"fn(obj: ?)","!doc":"<p>Internal.</p>\n"},"over":{"!type":"fn(obj: ?)","!doc":"<p>Internal.</p>\n"},"overName":{"!type":"fn(obj: ?, evt: ?) -> string"},"register":{"!type":"fn(scope: ?) -> +handle","!doc":"<p>All objects (Stencils) should register here if they\nuse mouse events. When registering, the object will\nbe called if it has that method.\nSee: CustomEventMethod and EventObject</p>\n"},"resize":{"!type":"fn(width: ?, height: ?)"},"scrollOffset":{"!type":"fn() -> ?","!doc":"<p>Gets scroll offset of canvas</p>\n"},"setCanvas":{"!type":"fn()","!doc":"<p>Internal. Sets canvas position</p>\n"},"setCursor":{"!type":"fn(cursor: ?, node: ?)","!doc":"<p>Sets the cursor for  a given node.  If no\nnode is specified the containing node is used.</p>\n"},"setEventMode":{"!type":"fn(mode: ?)","!doc":"<p>Sets the mouse mode s that custom events can be called.\nAlso can &#39;disable&#39; events by using a bogus mode:\n\n</p>\n<pre><code>mouse.setEventMode(&quot;DISABLED&quot;)</code></pre>\n<p>(unless any object subscribes to this event,\n\n</p>\n<p>it is effectively disabled)</p>\n"},"setZoom":{"!type":"fn(zoom: ?)","!doc":"<p>Internal. Sets the mouse zoom percentage to\nthat of the canvas</p>\n"},"unregister":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects object. Mouse events are no longer\ncalled for it.</p>\n"},"up":{"!type":"fn(evt: ?)","!doc":"<p>Internal. Create onUp event</p>\n"}}},"dojox/drawing/util/oo":{"!doc":"<p>Inheritance utilities used in DojoX Drawing</p>\n","declare":{"!type":"fn() -> ?","!doc":"<p>Creates a constructor Function from a\nFunction, and collection of methods, and\nmore Functions that are extended.</p>\n"},"extend":{"!type":"fn() -> fn","!doc":"<p>Extends constructors to inherit from other\nconstructors .</p>\n"}},"dojox/drawing/manager/Canvas":{"!type":"fn()","prototype":{"!doc":"<p>Creates a dojox.gfx.surface to be used for Drawing. Note that\nThe &#39;surface&#39; that Drawing uses is actually a dojox.gfx.group.\nThis allows for more versatility.\n\n</p>\n<p>Called internally from a dojox.Drawing.\n\n</p>\n<p>Note: Surface creation is asynchronous. Connect to\nonSurfaceReady in Drawing.</p>\n","baseClass":{},"useScrollbars":{},"zoom":{"!doc":"<p>The amount the canvas is zoomed</p>\n"},"constructor":{"!type":"fn(options: ?)"},"getScrollOffset":{"!type":"fn() -> ?","!doc":"<p>Get the scroll position of the canvas</p>\n"},"getScrollWidth":{"!type":"fn() -> number","!doc":"<p>Special method used to detect the width (and height)\nof the browser scrollbars. Becomes memoized.</p>\n"},"resize":{"!type":"fn(width: ?, height: ?)","!doc":"<p>Method used to change size of canvas. Potentially\ncalled from a container like ContentPane. May be\ncalled directly.</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?, scrollx: ?, scrolly: ?)","!doc":"<p>Internal. Changes canvas size and sets scroll position.\nDo not call this, use resize().</p>\n"},"setZoom":{"!type":"fn(zoom: ?)","!doc":"<p>Internal. Zooms canvas in and out.</p>\n"}}},"dojox/drawing/manager/Undo":{"!type":"fn()","prototype":{"!doc":"<p>Handles the Undo in drawing.\nNOTE: Only partially implemented!!! There is very\nlittle actual undo functionality!</p>\n","add":{"!type":"fn(stack: ?)"},"apply":{"!type":"fn(scope: ?, method: ?, args: ?)"},"constructor":{"!type":"fn(options: ?)"},"redo":{"!type":"fn()"},"undo":{"!type":"fn()"}}},"dojox/drawing/manager/Anchors":{"!type":"fn()","prototype":{"add":{"!type":"fn(item: ?)","!doc":"<p>Creates anchor points on a Stencil, based on the\nStencil&#39;s points.</p>\n"},"constructor":{"!type":"fn(options: ?)"},"remove":{"!type":"fn(item: ?)","!doc":"<p>Destroys the anchor points for a Stencil.</p>\n"}}},"dojox/drawing/manager/Stencil":{"!type":"fn()","prototype":{"!doc":"<p>The main class for tracking Stencils that are cretaed, added,\nselected, or deleted. Also handles selections, multiple\nselections, adding and removing from selections, and dragging\nselections. It&#39;s this class that triggers the anchors to\nappear on a Stencil and whther there are anchor on a multiple\nselect or not (currently not)</p>\n","_dragBegun":{},"_isBusy":{},"_lastmxx":{},"_lastmxy":{},"_secondClick":{},"_throttle":{},"_throttleVrl":{},"_wasDragged":{},"throttleTime":{},"constructor":{"!type":"fn(options: ?)"},"deleteItem":{"!type":"fn(stencil: ?)","!doc":"<p>Deletes a stencil.\nNOTE: supports limited undo.</p>\n"},"deselect":{"!type":"fn()"},"deselectItem":{"!type":"fn(stencil: ?)","!doc":"<p>Deselect passed stencil</p>\n"},"exporter":{"!type":"fn() -> []","!doc":"<p>Collects all Stencil data and returns an\nArray of objects.</p>\n"},"getRecentStencil":{"!type":"fn() -> ?","!doc":"<p>Returns the stencil most recently interacted\nwith whether it&#39;s last created or last selected</p>\n"},"hasSelected":{"!type":"fn() -> number","!doc":"<p>Returns number of selected (generally used\nas truthy or falsey)</p>\n"},"isSelected":{"!type":"fn(stencil: ?) -> bool","!doc":"<p>Returns if passed stencil is selected or not\nbased on internal collection, not on stencil\nboolean</p>\n"},"listStencils":{"!type":"fn() -> ?"},"register":{"!type":"fn(stencil: ?) -> bool","!doc":"<p>Key method for adding Stencils. Stencils\ncan be added to the canvas without adding\nthem to this, but they won&#39;t have selection\nor drag ability.</p>\n"},"removeAll":{"!type":"fn()","!doc":"<p>Deletes all Stencils on the canvas.</p>\n"},"saveMoveState":{"!type":"fn()","!doc":"<p>Internal. Used for the prototype undo stack.\nSaves selection position.</p>\n"},"saveThrottledState":{"!type":"fn()","!doc":"<p>Internal. Used for the prototype undo stack.\nPrevents an undo point on every mouse move.\nOnly does a point when the mouse hesitates.</p>\n"},"selectAll":{"!type":"fn()","!doc":"<p>Selects all items</p>\n"},"selectItem":{"!type":"fn(idOrItem: ?)","!doc":"<p>Method used to select a stencil.</p>\n"},"setConstraint":{"!type":"fn()","!doc":"<p>Internal. Gets all selected stencils&#39; coordinates\nand determines how far left and up the selection\ncan go without going below zero</p>\n"},"setRecentStencil":{"!type":"fn(stencil: ?)","!doc":"<p>Keeps track of the most recent stencil interacted\nwith, whether created or selected.</p>\n"},"setSelectionGroup":{"!type":"fn()","!doc":"<p>Internal. Creates a new selection group\nused to hold selected stencils.</p>\n"},"toSelected":{"!type":"fn(func: ?)","!doc":"<p>Convenience function calls function <em>within</em>\nall selected stencils</p>\n"},"unDelete":{"!type":"fn(stencils: ?)","!doc":"<p>Undeletes a stencil. Used in undo stack.</p>\n"},"unregister":{"!type":"fn(stencil: ?)","!doc":"<p>Method for removing Stencils from the manager.\nThis doesn&#39;t delete them, only removes them from\nthe list.</p>\n"},"withSelected":{"!type":"fn(func: ?)","!doc":"<p>Convenience function calls function on\nall selected stencils</p>\n"},"withStencils":{"!type":"fn(func: ?)","!doc":"<p>Convenience function calls function on\nall stencils</p>\n"},"withUnselected":{"!type":"fn(func: ?)","!doc":"<p>Convenience function calls function on\nall stencils that are not selected</p>\n"}}},"dojox/drawing/manager/StencilUI":{"!type":"fn()","prototype":{"!doc":"<p>Used for handling Stencils as UI components.</p>\n","constructor":{"!type":"fn(options: ?)"},"register":{"!type":"fn(stencil: ?) -> ?"}}},"dojox/drawing/_base":{"!type":"fn()","prototype":{"!doc":"<p>Drawing is a project that sits on top of DojoX GFX and uses SVG and\nVML vector graphics to draw and display.</p>\n","height":{"!doc":"<p>Height of the canvas</p>\n"},"mode":{"!doc":"<p>Changes the functionality of the drawing</p>\n"},"ready":{"!doc":"<p>Whether or not the canvas has been created and Stencils can be added</p>\n"},"width":{"!doc":"<p>Width of the canvas</p>\n"},"addPlugin":{"!type":"fn(plugin: ?)","!doc":"<p>Add a toolbar plugin object to plugins array\nto be parsed</p>\n"},"addStencil":{"!type":"fn(type: ?, options: ?) -> ?","!doc":"<p>Use this method to programmatically add Stencils that display on\nthe canvas.\n\n</p>\n<p>FIXME: Currently only supports Stencils that have been registered,\nwhich is items in the toolbar, and the additional Stencils at the\nend of onSurfaceReady. This covers all Stencils, but you can&#39;t\nuse &#39;display only&#39; Stencils for Line, Rect, and Ellipse.</p>\n"},"addUI":{"!type":"fn(type: ?, options: ?) -> +null","!doc":"<p>Use this method to programmatically add Stencils that display on\nthe canvas.\n\n</p>\n<p>FIXME: Currently only supports Stencils that have been registered,\nwhich is items in the toolbar, and the additional Stencils at the\nend of onSurfaceReady. This covers all Stencils, but you can&#39;t\nuse &#39;display only&#39; Stencils for Line, Rect, and Ellipse.</p>\n"},"changeDefaults":{"!type":"fn(newStyle: ?, value: ?)","!doc":"<p>Change the defaults so that all Stencils from this\npoint on will use the newly changed style.</p>\n"},"constructor":{"!type":"fn(props: ?, node: ?)","!doc":"<p>Drawing is not a Dijit. This is the master method.\n\n</p>\n<p>NOTE:\nprops is always null since this is not a real widget\nWill change when Drawing can be created programmatically.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Collects all Stencil data and returns an\nArray of objects.</p>\n"},"get":{"!type":"fn(name: ?)"},"getConstructor":{"!type":"fn(abbr: ?) -> ?","!doc":"<p>Returns a Stencil constructor base on\nabbreviation</p>\n"},"getShapeProps":{"!type":"fn(data: ?, mode: ?) -> ?","!doc":"<p>The common objects that are mixed into\na new Stencil. Mostly internal, but could be used.</p>\n"},"importer":{"!type":"fn(objects: ?)","!doc":"<p>Handles an Array of stencil data and imports the objects\nto the drawing.</p>\n"},"initPlugins":{"!type":"fn()","!doc":"<p>Called from Toolbar after a plugin has been loaded\nThe call to this coming from toolbar is a bit funky as the timing\nof IE for canvas load is different than other browsers</p>\n"},"registerTool":{"!type":"fn(type: ?)","!doc":"<p>Registers a tool that can be accessed. Internal.</p>\n"},"removeAll":{"!type":"fn()","!doc":"<p>Deletes all Stencils on the canvas.</p>\n"},"removeStencil":{"!type":"fn(stencil: ?)","!doc":"<p>Use this method to programmatically remove Stencils from the canvas.</p>\n"},"resize":{"!type":"fn(box: ?)","!doc":"<p>Resizes the canvas.\nIf within a ContentPane this will get called automatically.\nCan also be called directly.</p>\n"},"selectAll":{"!type":"fn()","!doc":"<p>Selects all stencils</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Drawing registers as a widget and needs to support\nwidget&#39;s api.</p>\n"},"setTool":{"!type":"fn(type: ?)","!doc":"<p>Sets up a new class to be used to draw. Called from Toolbar,\nand this class... after a tool is used a new one of the same\ntype is initialized. Could be called externally.</p>\n"},"startup":{"!type":"fn()"},"toSelected":{"!type":"fn(func: ?)","!doc":"<p>Call a function within all selected Stencils\nlike attr()</p>\n"},"unSetTool":{"!type":"fn()","!doc":"<p>Destroys current tool</p>\n"}}},"dojox/drawing/annotations/Label":{"Label":{"!type":"fn()"}},"dojox/drawing/annotations/Label.Label":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/Text.prototype","!doc":"<p>An annotation called internally to label an Stencil.</p>\n","_align":{},"_lineHeight":{"!doc":"<p>The height of each line of text. Based on style information\n    and font size.</p>\n"},"align":{"!doc":"<p>Text horizontal alignment.\nOptions: start, middle, end</p>\n"},"anchorType":{},"baseRender":{},"drawingType":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"StencilData":{"!doc":"<p>The data used to create the dojox.gfx Text</p>\n"},"StencilPoints":{},"type":{},"valign":{"!doc":"<p>Text vertical alignment\nOptions: top, middle, bottom (FIXME: bottom not supported)</p>\n"},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"beforeAttr":{"!type":"fn(key: ?, value: ?)"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>Getter for text.</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"makeFit":{"!type":"fn(text: ?, w: ?) -> ?"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn(text: ?)","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object). Text is slightly different than other\nimplementations. Instead of calling render twice, it calls\n_createHilite for the &#39;hit&#39;</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Sets the text of the label. Not called directly. Should\nbe called within Stencil. See stencil._Base</p>\n"},"setLabelCustom":{"!type":"fn(text: ?)","!doc":"<p>Attaches to custom positioning within a Stencil</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setText":{"!type":"fn(text: ?)","!doc":"<p>Setter for text.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"typesetter":{"!type":"fn(text: ?) -> ?","!doc":"<p>Register raw text, returning typeset form.\nUses function dojox.drawing.stencil.Text.typeset\nfor typesetting, if it exists.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}}},"dojox/drawing/stencil/Text":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/_Base.prototype","!doc":"<p>Creates a dojox.gfx Text (SVG or VML) based on data provided.</p>\n","_lineHeight":{"!doc":"<p>The height of each line of text. Based on style information\n    and font size.</p>\n"},"align":{"!doc":"<p>Text horizontal alignment.\nOptions: start, middle, end</p>\n"},"anchorType":{},"baseRender":{},"drawingType":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"StencilData":{"!doc":"<p>The data used to create the dojox.gfx Text</p>\n"},"StencilPoints":{},"type":{},"valign":{"!doc":"<p>Text vertical alignment\nOptions: top, middle, bottom (FIXME: bottom not supported)</p>\n"},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>constructor.</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>Getter for text.</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"makeFit":{"!type":"fn(text: ?, w: ?) -> ?"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn(text: ?)","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object). Text is slightly different than other\nimplementations. Instead of calling render twice, it calls\n_createHilite for the &#39;hit&#39;</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setText":{"!type":"fn(text: ?)","!doc":"<p>Setter for text.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"typesetter":{"!type":"fn(text: ?) -> ?","!doc":"<p>Register raw text, returning typeset form.\nUses function dojox.drawing.stencil.Text.typeset\nfor typesetting, if it exists.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}}},"dojox/drawing/stencil/_Base":{"!type":"fn()","prototype":{"!doc":"<p>The base class used for all Stencils.</p>\n","drawingType":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"type":{"!doc":"<p>The type of Stencil this is. Should be overridden\nby extending classes.\nFIXME: should this be declaredClass?</p>\n"},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)"},"dataToPoints":{"!type":"fn(data: ?)","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(points: ?)","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>This Stencil&#39;s render description. Often\ncalls &#39;sub render&#39; methods.</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}}},"dojox/drawing/annotations/BoxShadow":{"!type":"fn()","prototype":{"!doc":"<p>Creates a box shadow under solid objects. Can change the\nshadow direction, color, size, and intensity. Can center\nthe shadow and make it a Glow.</p>\n","showing":{},"arrowPoints":{"!type":"fn() -> []","!doc":"<p>Creates data used to draw arrow head.</p>\n"},"constructor":{"!type":"fn(options: ?)"},"createForEllipse":{"!type":"fn(o: ?, size: ?, mult: ?, d: ?, r: ?, p: ?, c: ?)"},"createForLine":{"!type":"fn(o: ?, size: ?, mult: ?, d: ?, r: ?, p: ?, c: ?)"},"createForPath":{"!type":"fn(o: ?, size: ?, mult: ?, pts: ?, r: ?, p: ?, c: ?)"},"createForRect":{"!type":"fn(o: ?, size: ?, mult: ?, d: ?, r: ?, p: ?, c: ?)"},"createForZArrow":{"!type":"fn(o: ?, size: ?, mult: ?, pts: ?, r: ?, p: ?, c: ?)"},"destroy":{"!type":"fn()"},"hide":{"!type":"fn()"},"render":{"!type":"fn()"},"show":{"!type":"fn()"}}},"dojox/drawing/annotations/Angle":{"!type":"fn()","prototype":{"!doc":"<p>When initiated, an HTML box will hover near the Stencil,\ndisplaying it&#39;s angle while drawn or modified. Currently\nonly works with Vector, Line, Arrow, and Axes.</p>\n","angle":{},"type":{},"constructor":{"!type":"fn(options: ?)"},"getAngleNode":{"!type":"fn() -> ?","!doc":"<p>Gets or creates HTMLNode used for display</p>\n"},"hideAngle":{"!type":"fn()","!doc":"<p>Turns display off.</p>\n"},"showAngle":{"!type":"fn()","!doc":"<p>Called to display angle</p>\n"}}},"dojox/drawing/util/positioning":{"angle":{"!type":"fn(start: ?, end: ?) -> ?","!doc":"<p>Returns the optimal position for annotations.Angle.</p>\n"},"label":{"!type":"fn(start: ?, end: ?) -> ?","!doc":"<p>Returns the optimal text positions for annotations.Label.</p>\n"}},"dojox/drawing/util/typeset":{"convertHTML":{"!type":"fn(inText: ?) -> ?"},"convertLaTeX":{"!type":"fn(inText: ?) -> ?"}},"dojox/drawing/library/greek":{"!doc":"<p>Greek characters used by typesetter and greekPalette.</p>\n","alpha":{},"Alpha":{},"beta":{},"Beta":{},"Chi":{},"chi":{},"Delta":{},"delta":{},"epsilon":{},"Epsilon":{},"eta":{},"Eta":{},"Gamma":{},"gamma":{},"iota":{},"Iota":{},"kappa":{},"Kappa":{},"Lambda":{},"lambda":{},"mu":{},"Mu":{},"nu":{},"Nu":{},"omega":{},"Omega":{},"omicron":{},"Omicron":{},"Phi":{},"phi":{},"Pi":{},"pi":{},"piv":{},"psi":{},"Psi":{},"Rho":{},"rho":{},"sigma":{},"Sigma":{},"sigmaf":{},"Tau":{},"tau":{},"theta":{},"Theta":{},"thetasym":{},"upsih":{},"Upsilon":{},"upsilon":{},"Xi":{},"xi":{},"Zeta":{},"zeta":{}},"dojox/drawing/annotations/Arrow":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/Path.prototype","!doc":"<p>An annotation called internally to put an arrowhead\non ether end of a Line. Initiated in Arrow (and Vector)\nwith the optional params: arrowStart and arrowEnd. Both\ndefault true for Axes.</p>\n","baseRender":{},"closeColor":{},"closePath":{},"closeRadius":{},"drawingType":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"idx1":{},"idx2":{},"minimumSize":{},"StencilData":{},"StencilPoints":{},"subShape":{},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"arrowHead":{"!type":"fn(x1: ?, y1: ?, x2: ?, y2: ?, style: ?) -> []","!doc":"<p>Creates data used to draw arrow head.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"checkClosePoint":{"!type":"fn(firstPt: ?, currPt: ?, remove: ?) -> bool","!doc":"<p>Checks if points are close enough to indicate that\npath should be close. Provides a visual cue.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)"},"dataToPoints":{"!type":"fn(data: ?)","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Overwriting _Base.getBounds. Not sure how absolute should\nwork for a path.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(points: ?)","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}}},"dojox/drawing/stencil/Path":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/_Base.prototype","!doc":"<p>Creates a dojox.gfx Path based on points provided.</p>\n","baseRender":{},"closeColor":{},"closePath":{},"closeRadius":{},"drawingType":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"StencilData":{},"StencilPoints":{},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"checkClosePoint":{"!type":"fn(firstPt: ?, currPt: ?, remove: ?) -> bool","!doc":"<p>Checks if points are close enough to indicate that\npath should be close. Provides a visual cue.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)"},"dataToPoints":{"!type":"fn(data: ?)","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Overwriting _Base.getBounds. Not sure how absolute should\nwork for a path.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(points: ?)","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}}},"dojox/drawing/library/icons":{"!doc":"<p>A collection of icon Stencils for use with the buttons in the ui/Toolbar.</p>\n","arrow":{},"axes":{},"ellipse":{},"equation":{},"iconize":{},"line":{},"pan":{},"path":{},"pencil":{},"plus":{},"rect":{},"textBlock":{},"triangle":{},"vector":{},"zoom100":{},"zoomIn":{},"zoomOut":{}},"dojox/drawing/library/icons.line":{"borderWidth":{},"type":{},"x1":{},"x2":{},"y1":{},"y2":{}},"dojox/drawing/library/icons.ellipse":{"borderWidth":{},"cx":{},"cy":{},"rx":{},"ry":{},"type":{}},"dojox/drawing/library/icons.rect":{"borderWidth":{},"height":{},"type":{},"width":{},"x":{},"y":{}},"dojox/drawing/library/icons.triangle":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.path":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.arrow":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.textBlock":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.equation":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.axes":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.vector":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.pan":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.plus":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.zoomIn":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.zoomOut":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.zoom100":{"text":{},"type":{}},"dojox/drawing/library/icons.iconize":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/library/icons.pencil":{"borderWidth":{},"closePath":{},"points":{},"type":{}},"dojox/drawing/plugins/_Plugin":{"!type":"fn()","prototype":{"!doc":"<p>Base class for plugins.</p>\n","anchors":{},"button":{},"canvas":{},"drawing":{},"keys":{},"mouse":{},"node":{},"stencils":{},"type":{},"util":{},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"}}},"dojox/drawing/plugins/drawing/GreekPalette":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>This plugin uses the palette dijit in order to give tips for\nnon-english (mostly greek for now) letters.\n\n</p>\n<p>IMPORTANT!  Because it is a full blown dijit it is NOT loaded\nlike the other plugins.  INSTEAD currently it is instantiated\nin markup.  TextBlock LOOKS FOR IT by ID - &quot;greekPalette&quot;\nand if it finds it does the necessary initialization/connections.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_currentFocus":{"!doc":"<p>The currently focused cell (if the palette itself has focus), or otherwise\nthe cell to be focused when the palette itself gets focus.\nDifferent from value, which represents the selected (i.e. clicked) cell.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selectedCell":{"!doc":"<p>Index of the currently selected cell. Initially, none selected</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_xDim":{"!doc":"<p>This is the number of cells horizontally across.</p>\n"},"_yDim":{"!doc":"<p>This is the number of cells vertically down.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"cellClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"defaultTimeout":{"!doc":"<p>Number of milliseconds before a held key or button becomes typematic</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"paletteClass":{},"searchContainerNode":{},"showPreview":{"!doc":"<p>Whether the preview pane will be displayed, to show details about the selected entity.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Widget tab index.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template of this widget.  Using dojoxEntityPalette classes\nin order to allow easy transfer of css</p>\n"},"timeoutChangeRate":{"!doc":"<p>Fraction of time used to change the typematic timer between events\n1.0 means that each typematic event fires at defaultTimeout intervals\nLess than 1.0 means that each typematic event fires at an increasing faster rate</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>Currently selected color/emoticon/etc.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dyeClass":{"!type":"fn()"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus this widget.  Puts focus on the most recently focused cell.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"show":{"!type":"fn(obj: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/drawing/plugins/drawing/Grid":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/plugins/_Plugin.prototype","!doc":"<p>Plugin that displays a grid on the Drawing canvas.</p>\n","anchors":{},"button":{},"canvas":{},"drawing":{},"gap":{"!doc":"<p>How far apart to set the grid lines</p>\n"},"keys":{},"major":{},"majorColor":{"!doc":"<p>Major lines color</p>\n"},"minor":{},"minorColor":{"!doc":"<p>Minor lines color</p>\n"},"mouse":{},"node":{},"stencils":{},"type":{},"util":{},"zoom":{"!doc":"<p>The current zoom of the grid</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"setGrid":{"!type":"fn(options: ?) -> ?","!doc":"<p>Renders grid</p>\n"},"setZoom":{"!type":"fn(zoom: ?)","!doc":"<p>Set&#39;s the zoom of the canvas</p>\n"}}},"dojox/drawing/plugins/drawing/Silverlight":{},"dojox/drawing/plugins/tools/Iconize":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/plugins/_Plugin.prototype","!doc":"<p>Somewhat of internal use...\nOutputs a path to be used as an icon. Will end up being a\nsub-icon under Export options</p>\n","anchors":{},"button":{},"canvas":{},"drawing":{},"keys":{},"mouse":{},"node":{},"stencils":{},"type":{},"util":{},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"makeIcon":{"!type":"fn(p: ?)"}},"setup":{}},"dojox/drawing/plugins/tools/Iconize.setup":{"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/plugins/tools/Pan":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/plugins/_Plugin.prototype","!doc":"<p>A plugin that allows for a scrolling canvas. An action\ntool is added to the toolbar that allows for panning. Holding\nthe space bar is a shortcut to that action. The canvas will\nonly pan and scroll if there are objects out of the viewable\narea.</p>\n","anchors":{},"button":{},"canvas":{},"drawing":{},"interval":{},"keys":{},"keyScroll":{},"mouse":{},"node":{},"selected":{},"stencils":{},"type":{},"util":{},"checkBounds":{"!type":"fn()"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"}},"setup":{}},"dojox/drawing/plugins/tools/Pan.setup":{"button":{},"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/plugins/tools/Zoom":{"Zoom100":{"!type":"fn()"},"ZoomIn":{"!type":"fn()"},"ZoomOut":{"!type":"fn()"}},"dojox/drawing/plugins/tools/Zoom.ZoomIn":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/plugins/_Plugin.prototype","anchors":{},"button":{},"canvas":{},"drawing":{},"keys":{},"mouse":{},"node":{},"stencils":{},"type":{},"util":{},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"}},"setup":{}},"dojox/drawing/plugins/tools/Zoom.Zoom100":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/plugins/_Plugin.prototype","anchors":{},"button":{},"canvas":{},"drawing":{},"keys":{},"mouse":{},"node":{},"stencils":{},"type":{},"util":{},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"}},"setup":{}},"dojox/drawing/plugins/tools/Zoom.ZoomOut":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/plugins/_Plugin.prototype","anchors":{},"button":{},"canvas":{},"drawing":{},"keys":{},"mouse":{},"node":{},"stencils":{},"type":{},"util":{},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"}},"setup":{}},"dojox/drawing/stencil/Ellipse":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/_Base.prototype","!doc":"<p>Creates a dojox.gfx Ellipse based on data or points provided.</p>\n","anchorType":{},"baseRender":{},"drawingType":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>constructor</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}}},"dojox/drawing/stencil/Image":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/_Base.prototype","!doc":"<p>Creates an dojox.gfx Image based on the data\nprovided.</p>\n","anchorType":{},"baseRender":{},"drawingType":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"StencilData":{"!doc":"<p>The data used to create the dojox.gfx Shape</p>\n"},"StencilPoints":{},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>constructor</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getImageSize":{"!type":"fn(render: ?)","!doc":"<p>Internal. If no image size is passed in with the data\ncreate a dom node, insert and image, gets its dimensions\nrecord them - then destroy everything.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn(dbg: ?)","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object). Image is slightly different than other\nimplementations. Instead of calling render twice, it calls\n_createHilite for the &#39;hit&#39;</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}}},"dojox/drawing/stencil/Line":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/_Base.prototype","!doc":"<p>Creates a dojox.gfx Line based on data or points provided.</p>\n","anchorType":{},"baseRender":{},"drawingType":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"StencilData":{"!doc":"<p>The data used to create the dojox.gfx Shape, specify\nx1,y1,x2,y2, or x,y,angle,radius</p>\n"},"StencilPoints":{},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>constructor</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}}},"dojox/drawing/stencil/Rect":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/_Base.prototype","!doc":"<p>Creates a dojox.gfx rectangle based on data or points provided.</p>\n","anchorType":{},"baseRender":{},"drawingType":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>constructor</p>\n"},"dataToPoints":{"!type":"fn(d: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}}},"dojox/drawing/tools/Arrow":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/tools/Line.prototype","!doc":"<p>Extends stencil.Line and adds an arrow head\nto the end and or start.</p>\n","anchorType":{},"arrowEnd":{"!doc":"<p>Whether or not to place an arrow on end.</p>\n"},"arrowStart":{"!doc":"<p>Whether or not to place an arrow on start.</p>\n"},"baseRender":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"showAngle":{},"StencilData":{"!doc":"<p>The data used to create the dojox.gfx Shape, specify\nx1,y1,x2,y2, or x,y,angle,radius</p>\n"},"StencilPoints":{},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>constructor</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"labelPosition":{"!type":"fn() -> ?","!doc":"<p>The custom position used for the label</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}},"setup":{}},"dojox/drawing/tools/Arrow.setup":{"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/tools/Line":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/Line.prototype","!doc":"<p>Class for a drawable Line</p>\n","anchorType":{},"baseRender":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"showAngle":{},"StencilData":{"!doc":"<p>The data used to create the dojox.gfx Shape, specify\nx1,y1,x2,y2, or x,y,angle,radius</p>\n"},"StencilPoints":{},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>constructor</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}},"setup":{}},"dojox/drawing/tools/Line.setup":{"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/tools/Ellipse":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/Ellipse.prototype","!doc":"<p>A drawable Ellipse.</p>\n","anchorType":{},"baseRender":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>constructor</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}},"setup":{}},"dojox/drawing/tools/Ellipse.setup":{"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/tools/Path":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/Path.prototype","!doc":"<p>Class for a drawable Path</p>\n","baseRender":{},"closeColor":{},"closePath":{},"closeRadius":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"StencilData":{},"StencilPoints":{},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"checkClosePoint":{"!type":"fn(firstPt: ?, currPt: ?, remove: ?) -> bool","!doc":"<p>Checks if points are close enough to indicate that\npath should be close. Provides a visual cue.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>constructor</p>\n"},"createGuide":{"!type":"fn(obj: ?)"},"dataToPoints":{"!type":"fn(data: ?)","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Overwriting _Base.getBounds. Not sure how absolute should\nwork for a path.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"makeSubPath":{"!type":"fn(_closePath: ?)"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(points: ?)","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}},"setup":{}},"dojox/drawing/tools/Path.setup":{"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/tools/Pencil":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/Path.prototype","!doc":"<p>Class for a drawable, continuous Path</p>\n","baseRender":{},"closeColor":{},"closePath":{},"closeRadius":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minDist":{"!doc":"<p>The distance the mouse must travel before rendering\na path segment. Lower number is a higher definition\npath but more points.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"StencilData":{},"StencilPoints":{},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"checkClosePoint":{"!type":"fn(firstPt: ?, currPt: ?, remove: ?) -> bool","!doc":"<p>Checks if points are close enough to indicate that\npath should be close. Provides a visual cue.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>constructor</p>\n"},"dataToPoints":{"!type":"fn(data: ?)","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Overwriting _Base.getBounds. Not sure how absolute should\nwork for a path.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(points: ?)","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}},"setup":{}},"dojox/drawing/tools/Pencil.setup":{"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/tools/Rect":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/Rect.prototype","!doc":"<p>Class for a drawable rectangle</p>\n","anchorType":{},"baseRender":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"type":{},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>constructor</p>\n"},"dataToPoints":{"!type":"fn(d: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}},"setup":{}},"dojox/drawing/tools/Rect.setup":{"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/tools/TextBlock":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/Text.prototype","!doc":"<p>A tool to create text fields on a canvas.</p>\n","_blockExec":{},"_caretEnd":{},"_caretStart":{},"_downOnCanvas":{},"_lineHeight":{"!doc":"<p>The height of each line of text. Based on style information\n    and font size.</p>\n"},"align":{"!doc":"<p>Text horizontal alignment.\nOptions: start, middle, end</p>\n"},"anchorType":{},"baseRender":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"selectOnExec":{"!doc":"<p>Whether the Stencil is selected when the text field\nis executed or not</p>\n"},"showEmpty":{"!doc":"<p>If true and there is no text in the data, the TextBlock\nIs displayed and focused and awaits input.</p>\n"},"StencilData":{"!doc":"<p>The data used to create the dojox.gfx Text</p>\n"},"StencilPoints":{},"type":{},"valign":{"!doc":"<p>Text vertical alignment\nOptions: top, middle, bottom (FIXME: bottom not supported)</p>\n"},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"cleanText":{"!type":"fn(txt: ?, removeBreaks: ?) -> ?","!doc":"<p>Cleans text. Strings HTML chars and double spaces\nand optionally removes line breaks.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"connectTextField":{"!type":"fn()","!doc":"<p>Internal. Creates the connections to the\ncontenteditable HTML node.</p>\n"},"constructor":{"!type":"fn(options: ?)","!doc":"<p>constructor</p>\n"},"createAnchors":{"!type":"fn()","!doc":"<p>Internal. Creates HTML nodes at each corner\nof the contenteditable div. These nodes are\ndraggable and will resize the div horizontally.</p>\n"},"createTextField":{"!type":"fn(txt: ?) -> ?","!doc":"<p>Internal. Inserts the contenteditable HTML node\ninto its parent node, and styles it.</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"destroyAnchors":{"!type":"fn()","!doc":"<p>Internal. Destroys HTML anchors.</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"edit":{"!type":"fn()","!doc":"<p>Internal?\nMethod used to instantiate the contenteditable HTML node.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"execText":{"!type":"fn()","!doc":"<p>Internal. Method fired when text is executed,\nvia mouse-click-off, ESC key or Enter key.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getSavedCaret":{"!type":"fn() -> ?"},"getSelection":{"!type":"fn(node: ?)","!doc":"<p>This gets and stores the caret position\nin the contentEditable div (conEdit).\nNOTE: Doesn&#39;t work with html nodes inside\nthe div.</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>Getter for text.</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"insertText":{"!type":"fn(node: ?, val: ?)","!doc":"<p>Uses saved caret position to insert text\ninto position and place caret at the end of\ninsertion</p>\n"},"makeFit":{"!type":"fn(text: ?, w: ?) -> ?"},"measureText":{"!type":"fn(str: ?, width: ?) -> ?","!doc":"<p>Mechanism for measuring text.\nSVG nor VML have a way of determining the width or\nheight of a block of text. This method creates an\nHTML text block and those measurements are used for\ndisplaying the SVG/VML text.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn(text: ?)","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object). Text is slightly different than other\nimplementations. Instead of calling render twice, it calls\n_createHilite for the &#39;hit&#39;</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setSavedCaret":{"!type":"fn(val: ?)","!doc":"<p>Internal, called when caret needs to\nbe moved into position after text is added</p>\n"},"setSelection":{"!type":"fn(node: ?, what: ?)","!doc":"<p>Used for placing the cursor during edits and character help.\nTakes the values: end, beg, start, all or any numerical value\n(in which case the number will constitute the caret position)</p>\n"},"setText":{"!type":"fn(text: ?)","!doc":"<p>Setter for text.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"showParent":{"!type":"fn(obj: ?)","!doc":"<p>Internal. Builds the parent node for the\ncontenteditable HTML node.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"typesetter":{"!type":"fn(text: ?) -> ?","!doc":"<p>Register raw text, returning typeset form.\nUses function dojox.drawing.stencil.Text.typeset\nfor typesetting, if it exists.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}},"setup":{}},"dojox/drawing/tools/TextBlock.setup":{"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/tools/custom/Axes":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/stencil/Path.prototype","!doc":"<p>Draws a right-angle Axes (shaped like an L, not a +)</p>\n","baseRender":{},"closeColor":{},"closePath":{},"closeRadius":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{},"showAngle":{},"StencilData":{},"StencilPoints":{},"type":{},"zScale":{},"addShadow":{"!type":"fn(args: ?)"},"anchorPositionCheck":{"!type":"fn(x: ?, y: ?, anchor: ?) -> ?","!doc":"<p>Gets called from anchor to check if its current\nposition is ok. If not, its x or y transform will\nbe changed until this passes.</p>\n"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"checkClosePoint":{"!type":"fn(firstPt: ?, currPt: ?, remove: ?) -> bool","!doc":"<p>Checks if points are close enough to indicate that\npath should be close. Provides a visual cue.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)"},"createLabels":{"!type":"fn()","!doc":"<p>Creates the label for each axis.</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Custom getBounds overwrites _Base.getBounds</p>\n"},"getLabel":{"!type":"fn() -> +null","!doc":"<p>Getter for the labels. returns an object.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Possibility of z-axis makes bounds unreliable.\nUse these points instead.</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn() -> ?","!doc":"<p>Converts points to data.</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object).</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(value: ?)","!doc":"<p>Set the text of the labels. The text would be\nbroken up into the two labels.</p>\n"},"setLabelX":{"!type":"fn() -> ?","!doc":"<p>Custom placement for x-axis label</p>\n"},"setLabelY":{"!type":"fn() -> ?","!doc":"<p>Custom placement for y-axis label</p>\n"},"setLabelZ":{"!type":"fn() -> ?","!doc":"<p>Custom placement for z-axis label</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"},"zPoint":{"!type":"fn(obj: ?) -> ?","!doc":"<p>Finds the point for the z axis.</p>\n"},"zSet":{"!type":"fn()"}},"setup":{"!doc":"<p>See stencil._Base ToolsSetup</p>\n"}},"dojox/drawing/tools/custom/Axes.setup":{"!doc":"<p>See stencil._Base ToolsSetup</p>\n","iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/tools/custom/Vector":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/tools/Arrow.prototype","!doc":"<p>Creates a Vector Stencil.</p>\n","anchorType":{},"arrowEnd":{"!doc":"<p>Whether or not to place an arrow on end.</p>\n"},"arrowStart":{"!doc":"<p>Whether or not to place an arrow on start.</p>\n"},"baseRender":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{},"showAngle":{},"StencilData":{"!doc":"<p>The data used to create the dojox.gfx Shape, specify\nx1,y1,x2,y2, or x,y,angle,radius</p>\n"},"StencilPoints":{},"type":{},"addShadow":{"!type":"fn(args: ?)"},"anchorConstrain":{"!type":"fn(x: ?, y: ?) -> +null","!doc":"<p>Called from anchor point mouse drag</p>\n"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"changeAxis":{"!type":"fn(cosphi: ?)","!doc":"<p>Converts a vector to and from the z axis.\nIf passed a cosphi value that is used to set\nthe axis, otherwise it is the opp of what it is.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"constructor":{"!type":"fn(options: ?)"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"labelPosition":{"!type":"fn() -> ?","!doc":"<p>The custom position used for the label</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn()","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object). Additionally checks if Vector should be\ndrawn as an arrow or a circle (zero-length)</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"},"zPoint":{"!type":"fn(obj: ?) -> +null","!doc":"<p>Takes any point and converts it to\nbe on the z-axis.</p>\n"}},"setup":{"!doc":"<p>See stencil._Base ToolsSetup</p>\n"}},"dojox/drawing/tools/custom/Vector.setup":{"!doc":"<p>See stencil._Base ToolsSetup</p>\n","iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/tools/custom/Equation":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/tools/TextBlock.prototype","_blockExec":{},"_caretEnd":{},"_caretStart":{},"_downOnCanvas":{},"_lineHeight":{"!doc":"<p>The height of each line of text. Based on style information\n    and font size.</p>\n"},"align":{"!doc":"<p>Text horizontal alignment.\nOptions: start, middle, end</p>\n"},"anchorType":{},"baseRender":{},"customType":{},"drawingType":{},"draws":{},"enabled":{"!doc":"<p>Whether the Stencil is enabled or not.</p>\n"},"minimumSize":{"!doc":"<p>The minimum size allowed for a render. If the size\nis less, the shape is destroyed.</p>\n"},"selectOnExec":{"!doc":"<p>Whether the Stencil is selected when the text field\nis executed or not</p>\n"},"showEmpty":{"!doc":"<p>If true and there is no text in the data, the TextBlock\nIs displayed and focused and awaits input.</p>\n"},"StencilData":{"!doc":"<p>The data used to create the dojox.gfx Text</p>\n"},"StencilPoints":{},"type":{},"valign":{"!doc":"<p>Text vertical alignment\nOptions: top, middle, bottom (FIXME: bottom not supported)</p>\n"},"addShadow":{"!type":"fn(args: ?)"},"animate":{"!type":"fn(options: ?, create: ?)"},"applyTransform":{"!type":"fn(mx: ?)","!doc":"<p>Applies the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"attr":{"!type":"fn(key: ?, value?: ?)","!doc":"<p>Changes properties in the style or disabled styles,\ndepending on whether the object is enabled.\nAlso can be used to change most position and size props.</p>\n"},"cleanText":{"!type":"fn(txt: ?, removeBreaks: ?) -> ?","!doc":"<p>Cleans text. Strings HTML chars and double spaces\nand optionally removes line breaks.</p>\n"},"connect":{"!type":"fn(o: ?, e: ?, s: ?, m: ?, once: ?) -> ?","!doc":"<p>Convenience method for quick connects\nSee comments below for possiblities\nfunctions can be strings</p>\n"},"connectMouse":{"!type":"fn()","!doc":"<p>Internal. Registers this Stencil to receive\nmouse events.</p>\n"},"connectMult":{"!type":"fn()","!doc":"<p>Convenience method for batches of quick connects\nHandles are not returned and therefore cannot be\ndisconnected until Shape destroy time</p>\n"},"connectTextField":{"!type":"fn()","!doc":"<p>Internal. Creates the connections to the\ncontenteditable HTML node.</p>\n"},"constructor":{"!type":"fn(options: ?)"},"createAnchors":{"!type":"fn()","!doc":"<p>Internal. Creates HTML nodes at each corner\nof the contenteditable div. These nodes are\ndraggable and will resize the div horizontally.</p>\n"},"createTextField":{"!type":"fn(txt: ?) -> ?","!doc":"<p>Internal. Inserts the contenteditable HTML node\ninto its parent node, and styles it.</p>\n"},"dataToPoints":{"!type":"fn(o: ?) -> []","!doc":"<p>Converts data to points.</p>\n"},"deselect":{"!type":"fn(useDelay: ?)","!doc":"<p>Called when the Stencil is deselected.\nNOTE: Calling this will not deselect the Stencil\ncalling this just sets the style to the current\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nand deselecting Stencils.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys this Stencil</p>\n"},"destroyAnchors":{"!type":"fn()","!doc":"<p>Internal. Destroys HTML anchors.</p>\n"},"disable":{"!type":"fn()","!doc":"<p>Disables Stencil so it is not selectable.\nChanges the color to the disabled style.</p>\n"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"disconnectMouse":{"!type":"fn()","!doc":"<p>Internal. Unregisters this Stencil from receiving\nmouse events.</p>\n"},"edit":{"!type":"fn()","!doc":"<p>Internal?\nMethod used to instantiate the contenteditable HTML node.</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables Stencil so it is not selectable (if\nit was selectable to begin with). Changes the\ncolor to the current style.</p>\n"},"execText":{"!type":"fn()","!doc":"<p>Internal. Method fired when text is executed,\nvia mouse-click-off, ESC key or Enter key.</p>\n"},"exporter":{"!type":"fn() -> ?","!doc":"<p>Exports Stencil data</p>\n"},"getAngle":{"!type":"fn() -> ?","!doc":"<p>Gets angle of Stencil\nNOTE: Only works for Lines, Arrows, Vectors and Axes\n(works on points, not transforms)</p>\n"},"getBounds":{"!type":"fn(absolute: ?) -> ?","!doc":"<p>Returns the coordinates of the Stencil. This is often\ndifferent than the data or the points.\n\n</p>\n<p>NOTE: Won&#39;t work for paths or annotations (labels, Axes, arrow tips)\nThey should overwrite.\n\n</p>\n<p>NOTE: Primarily used for checking for if shape is off\ncanvas. Therefore Lines could get flipped. Use absolute\nto prevent this.</p>\n"},"getLabel":{"!type":"fn() -> ?","!doc":"<p>Get the text of the label.</p>\n"},"getRadius":{"!type":"fn() -> ?","!doc":"<p>Gets radius (length) of Stencil.\n\n</p>\n<p>NOTE: Only works for Lines, Arrows and Vectors\n(not for Ellipse, Axes has its own version)</p>\n"},"getSavedCaret":{"!type":"fn() -> ?"},"getSelection":{"!type":"fn(node: ?)","!doc":"<p>This gets and stores the caret position\nin the contentEditable div (conEdit).\nNOTE: Doesn&#39;t work with html nodes inside\nthe div.</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>Getter for text.</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transform (position) of the Stencil&#39;s\ncontainer</p>\n"},"highlight":{"!type":"fn()","!doc":"<p>Changes style to the highlight theme.</p>\n"},"insertText":{"!type":"fn(node: ?, val: ?)","!doc":"<p>Uses saved caret position to insert text\ninto position and place caret at the end of\ninsertion</p>\n"},"makeFit":{"!type":"fn(text: ?, w: ?) -> ?"},"measureText":{"!type":"fn(str: ?, width: ?) -> ?","!doc":"<p>Mechanism for measuring text.\nSVG nor VML have a way of determining the width or\nheight of a block of text. This method creates an\nHTML text block and those measurements are used for\ndisplaying the SVG/VML text.</p>\n"},"moveToBack":{"!type":"fn()","!doc":"<p>Moves Stencil to the back of all other items\non the canvas.</p>\n"},"moveToFront":{"!type":"fn()","!doc":"<p>Moves Stencil to the front of all other items\non the canvas.</p>\n"},"pointsToData":{"!type":"fn(p: ?) -> ?","!doc":"<p>Converts points to data</p>\n"},"preventNegativePos":{"!type":"fn()","!doc":"<p>Internal. Prevent item from being drawn/rendered less\nthan zero on the X or Y.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes shape(s), typically before a re-render\nNo args defaults to this.shape\nPass in multiple args to remove multiple shapes</p>\n"},"removeShadow":{"!type":"fn()"},"render":{"!type":"fn(text: ?)","!doc":"<p>Renders the &#39;hit&#39; object (the shape used for an expanded\nhit area and for highlighting) and the&#39;shape&#39; (the actual\ndisplay object). Text is slightly different than other\nimplementations. Instead of calling render twice, it calls\n_createHilite for the &#39;hit&#39;</p>\n"},"select":{"!type":"fn()","!doc":"<p>Called when the Stencil is selected.\nNOTE: Calling this will not select the Stencil\ncalling this just sets the style to the &#39;selected&#39;\ntheme. &#39;manager.Stencil&#39; should be used for selecting\nStencils.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Setter for Stencil data; also converts\ndata to points. See individual Stencils\nfor specific data properties.</p>\n"},"setLabel":{"!type":"fn(text: ?)","!doc":"<p>Creates and sets a label annotation for the Stencil.\nIf Stencil contains a labelPosition method, that will\nbe used for positioning. Otherwise\ndojox.drawing.util.positioning.label is used.</p>\n"},"setPoints":{"!type":"fn(points: ?)","!doc":"<p>Setter for Stencil points; also converts\npoints to data. See individual Stencils\nfor specific points properties.</p>\n"},"setSavedCaret":{"!type":"fn(val: ?)","!doc":"<p>Internal, called when caret needs to\nbe moved into position after text is added</p>\n"},"setSelection":{"!type":"fn(node: ?, what: ?)","!doc":"<p>Used for placing the cursor during edits and character help.\nTakes the values: end, beg, start, all or any numerical value\n(in which case the number will constitute the caret position)</p>\n"},"setText":{"!type":"fn(text: ?)","!doc":"<p>Setter for text.</p>\n"},"setTransform":{"!type":"fn(mx: ?)","!doc":"<p>Sets the transform to the stencil\n\n</p>\n<p>NOTE: PARTIALLY IMPLEMENTED.  Only applies x y coords.</p>\n"},"showParent":{"!type":"fn(obj: ?)","!doc":"<p>Internal. Builds the parent node for the\ncontenteditable HTML node.</p>\n"},"transformPoints":{"!type":"fn(mx: ?)","!doc":"<p>Moves object to a new X Y location\nmx is additive. So mx.dx=1 will move the stencil\n1 pixel to the right from wherever it was.</p>\n"},"typesetter":{"!type":"fn(text: ?) -> ?","!doc":"<p>Register raw text, returning typeset form.\nUses function dojox.drawing.stencil.Text.typeset\nfor typesetting, if it exists.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Changes style to the current theme.</p>\n"}},"setup":{"!doc":"<p>See stencil._Base ToolsSetup</p>\n"}},"dojox/drawing/tools/custom/Equation.setup":{"!doc":"<p>See stencil._Base ToolsSetup</p>\n","iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/ui/Button":{"!type":"fn()","prototype":{"!doc":"<p>Creates a clickable button in &quot;UI&quot; mode of the drawing.</p>\n","callback":{},"enabled":{},"hitched":{},"scope":{},"selected":{},"toolType":{},"type":{},"attr":{"!type":"fn(options: ?)"},"constructor":{"!type":"fn(options: ?)"},"deselect":{"!type":"fn()"},"disable":{"!type":"fn()"},"enable":{"!type":"fn()"},"makeOptions":{"!type":"fn(d: ?, s: ?) -> ?"},"select":{"!type":"fn()"}}},"dojox/drawing/ui/Toolbar":{"!type":"fn()","prototype":{"!doc":"<p>A Toolbar used for holding buttons; typically representing the Stencils\nused for a DojoX Drawing.</p>\n","margin":{"!doc":"<p>The space between each button.</p>\n"},"padding":{"!doc":"<p>The amount of spce between the top and left of the toolbar and the buttons.</p>\n"},"radius":{"!doc":"<p>The size of the button&#39;s rounded corner</p>\n"},"size":{"!doc":"<p>The width and height of the button</p>\n"},"strPlugs":{"!doc":"<p>A comma delineated list of the plugins to include in the Toolbar.\nIf &quot;all&quot; is used, all registered plugins are included.</p>\n"},"strSelected":{"!doc":"<p>The button that should be selected at startup.</p>\n"},"strTools":{"!doc":"<p>A comma delineated list of the Stencil-tools to include in the Toolbar.\nIf &quot;all&quot; is used, all registered tools are included.</p>\n"},"toolPlugGap":{"!doc":"<p>The distance between the tool buttons and plug buttons</p>\n"},"addBack":{"!type":"fn()","!doc":"<p>Internal. Adds the back, behind the toolbar.</p>\n"},"addPlugin":{"!type":"fn()"},"addTool":{"!type":"fn()"},"constructor":{"!type":"fn(props: ?, node: ?)"},"makeButtons":{"!type":"fn()","!doc":"<p>Internal. create buttons.</p>\n"}}},"dojox/drawing/ui/Tooltip":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/plugins/_Plugin.prototype","anchors":{},"button":{},"canvas":{},"drawing":{},"height":{},"keys":{},"mouse":{},"node":{},"stencils":{},"type":{},"util":{},"width":{},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"}}},"dojox/drawing/ui/dom/Pan":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/plugins/_Plugin.prototype","anchors":{},"button":{},"canvas":{},"drawing":{},"keys":{},"mouse":{},"node":{},"selected":{},"stencils":{},"type":{},"util":{},"checkBounds":{"!type":"fn()","!doc":"<p>Scans all items on the canvas and checks if they are out of\nbounds. If so, a scroll bar (in Canvas) is shown. If the position\nis left or top, the canvas is scrolled all items are relocated\nthe distance of the scroll. Ideally, it should look as if the\nitems do not move.</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"}},"setup":{}},"dojox/drawing/ui/dom/Pan.setup":{"iconClass":{},"name":{},"tooltip":{}},"dojox/drawing/ui/dom/Toolbar":{"!type":"fn()","prototype":{"baseClass":{"!doc":"<p>The CSS style to apply to the toolbar node</p>\n"},"buttonClass":{"!doc":"<p>The CSS style to apply to each button node</p>\n"},"iconClass":{"!doc":"<p>The CSS style to apply to each button icon node</p>\n"},"constructor":{"!type":"fn(props: ?, node: ?)"},"createIcon":{"!type":"fn(node: ?, constr?: ?)","!doc":"<p>Internal. Creates an icon node for each button.</p>\n"},"createTool":{"!type":"fn(node: ?)","!doc":"<p>Creates a button on the Toolbar that is\na Tool, not a Plugin. Tools draw Stencils,\nPlugins do actions.</p>\n"},"parse":{"!type":"fn()","!doc":"<p>Initializing method that reads the dom node and its\nchildren for tools and plugins.</p>\n"}}},"dojox/drawing/ui/dom/Zoom":{"!type":"fn()","prototype":{"!proto":"dojox/drawing/plugins/_Plugin.prototype","anchors":{},"baseClass":{"!doc":"<p>The CSS class added to the Toolbar buttons</p>\n"},"botClass":{"!doc":"<p>The CSS class added to the bottom (or right) Toolbar button</p>\n"},"button":{},"canvas":{},"drawing":{},"keys":{},"maxZoom":{"!doc":"<p>The maximum the canvas can be zoomed in. 10 = 1000%</p>\n"},"midClass":{"!doc":"<p>The CSS class added to the middle Toolbar button</p>\n"},"minZoom":{"!doc":"<p>The most the canvas can be zoomed out. .1 = 10%</p>\n"},"mouse":{},"node":{},"stencils":{},"topClass":{"!doc":"<p>The CSS class added to the top (or left) Toolbar button</p>\n"},"type":{},"util":{},"zoomFactor":{"!doc":"<p>The current zoom amount</p>\n"},"zoomInc":{"!doc":"<p>The amount of zoom that will occur upon each click.</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn(options: ?)"},"disconnect":{"!type":"fn(handles: ?)","!doc":"<p>Removes connections based on passed\nhandles arguments</p>\n"},"makeButton":{"!type":"fn(name: ?, cls: ?)","!doc":"<p>Internal. Creates one of the buttons in the zoom-button set.</p>\n"}}},"dojox/drawing":{"!doc":"<p>Deprecated.  Should require dojox/drawing modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/dtl/Context":{"!doc":"<p>Represents a runtime context used by DTL templates.</p>\n","prototype":{"constructor":{"!type":"fn(dict: ?)","!doc":"<p>Pass one of these when rendering a template to tell the template what values to use.</p>\n"},"extend":{"!type":"fn(obj: ?) -> ?","!doc":"<p>Returns a clone of this context object, with the items from the passed objecct mixed in.</p>\n"},"filter":{"!type":"fn(filter: ?) -> +instance","!doc":"<p>Returns a clone of this context, only containing the items defined in the filter.</p>\n"},"get":{"!type":"fn(key: ?, otherwise: ?) -> ?"},"getKeys":{"!type":"fn() -> []","!doc":"<p>Returns the set of keys exported by this context.</p>\n"},"getThis":{"!type":"fn() -> ?","!doc":"<p>Gets the object on which to perform operations. </p>\n"},"hasKey":{"!type":"fn(key: ?) -> bool","!doc":"<p>Indicates whether the specified key is defined on this context.</p>\n"},"pop":{"!type":"fn()"},"push":{"!type":"fn() -> ?"},"setThis":{"!type":"fn(scope: ?)","!doc":"<p>Sets the object on which to perform operations. </p>\n"},"update":{"!type":"fn(dict: ?) -> ?"}}},"dojox/dtl/_base":{"_base":{},"_noOpNode":{},"BOOLS":{},"data":{},"date":{},"dates":{},"dijit":{},"dom":{},"html":{},"htmlstrings":{},"integers":{},"lists":{},"loader":{},"logic":{},"loop":{},"misc":{},"objects":{},"register":{"!doc":"<p>A register for filters and tags.</p>\n"},"strings":{},"text":{},"TOKEN_ATTR":{},"TOKEN_BLOCK":{},"TOKEN_CHANGE":{},"TOKEN_COMMENT":{},"TOKEN_CUSTOM":{},"TOKEN_NODE":{},"TOKEN_TEXT":{},"TOKEN_VAR":{},"AttributeNode":{"!type":"fn(key: ?, value: ?)"},"ChangeNode":{"!type":"fn(node: ?, up?: ?, root: ?)","!doc":"<p>Changes the parent during render/unrender</p>\n"},"Context":{"!type":"fn(dict: ?)","!doc":"<p>Represents a runtime context used by DTL templates.</p>\n"},"DomBuffer":{"!type":"fn(parent: ?)","!doc":"<p>Allows the manipulation of DOM</p>\n"},"DomInline":{"!type":"fn(args: ?, node: ?)"},"DomTemplate":{"!type":"fn(obj: ?)","!doc":"<p>The template class for DOM templating.</p>\n"},"Inline":{"!type":"fn(args: ?, node: ?)"},"mark_safe":{"!type":"fn(value: ?)"},"quickFilter":{"!type":"fn(str: ?)"},"Template":{"!type":"fn(template: ?, isString: ?)","!doc":"<p>The base class for text-based templates.</p>\n"}},"dojox/dtl/_base._base":{"escape":{"!type":"fn(value: ?)","!doc":"<p>Escapes a string&#39;s HTML</p>\n"},"safe":{"!type":"fn(value: ?)"}},"dojox/dtl/_base.text":{"_re":{},"getFilter":{"!type":"fn(name: ?, errorless: ?) -> ?"},"getTag":{"!type":"fn(name: ?, errorless: ?) -> ?"},"getTemplate":{"!type":"fn(file: ?) -> +instance"},"getTemplateString":{"!type":"fn(file: ?) -> string"},"tokenize":{"!type":"fn(str: ?) -> ?"}},"dojox/dtl/_base.register":{"!doc":"<p>A register for filters and tags.</p>\n","_registry":{},"filters":{"!type":"fn(base: ?, locations: ?)","!doc":"<p>Register the specified filter libraries.</p>\n"},"tags":{"!type":"fn(base: ?, locations: ?)","!doc":"<p>Register the specified tag libraries.</p>\n"}},"dojox/dtl/_base.BOOLS":{"checked":{},"disabled":{},"readonly":{}},"dojox/dtl/_base.dom":{"_attributes":{},"_re4":{},"_reSplit":{},"_reTrim":{},"_swallowed":{},"_uppers":{},"getTemplate":{"!type":"fn(text: ?) -> ?"},"tokenize":{"!type":"fn(nodes: ?) -> []"}},"dojox/dtl/_base.dijit":{},"dojox/dtl/_base.data":{},"dojox/dtl/_base.objects":{},"dojox/dtl/_base.date":{},"dojox/dtl/_base.dates":{},"dojox/dtl/_base.htmlstrings":{},"dojox/dtl/_base.integers":{},"dojox/dtl/_base.lists":{},"dojox/dtl/_base.logic":{},"dojox/dtl/_base.misc":{},"dojox/dtl/_base.strings":{},"dojox/dtl/_base.html":{},"dojox/dtl/_base.loader":{},"dojox/dtl/_base.loop":{},"dojox/string/Builder":{"!doc":"<p>A fast buffer for creating large strings.</p>\n","length":{},"append":{"!type":"fn(s: ?)","!doc":"<p>Append all arguments to the end of the buffer</p>\n"},"appendArray":{"!type":"fn(strings: ?)","!doc":"<p>Append an array of items to the internal buffer.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Remove all characters from the buffer.</p>\n"},"concat":{"!type":"fn(s: ?)","!doc":"<p>Alias for append.</p>\n"},"prototype":{"constructor":{"!type":"fn(str?: ?)","!doc":"<p>A fast buffer for creating large strings.</p>\n"}},"insert":{"!type":"fn(index: ?, str: ?)","!doc":"<p>Insert string str starting at index.</p>\n"},"remove":{"!type":"fn(start: ?, len?: ?)","!doc":"<p>Remove len characters starting at index start.  If len\nis not provided, the end of the string is assumed.</p>\n"},"replace":{"!type":"fn(oldStr: ?, newStr: ?)","!doc":"<p>Replace instances of one string with another in the buffer.</p>\n"},"toString":{"!type":"fn()","!doc":"<p>Return the string representation of the internal buffer.</p>\n"}},"dojox/dtl/DomInline":{"prototype":{"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"context":{},"declaredClass":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()"},"render":{"!type":"fn(context?: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dtl/dom":{"_attributes":{},"_re4":{},"_reSplit":{},"_reTrim":{},"_swallowed":{},"_uppers":{},"getTemplate":{"!type":"fn(text: ?) -> ?"},"tokenize":{"!type":"fn(nodes: ?) -> []"}},"dojox/dtl/dom._attributes":{},"dojox/dtl/dom._uppers":{},"dojox/dtl/Inline":{"prototype":{"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"context":{},"declaredClass":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()"},"render":{"!type":"fn(context?: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/dtl/_DomTemplated":{"!doc":"<p>The base class for DOM-based templating.</p>\n","prototype":{"_dijitTemplateCompat":{},"buildRendering":{"!type":"fn()","!doc":"<p>Constructs the DOM representation.</p>\n"},"render":{"!type":"fn(context?: ?, tpl?: ?)","!doc":"<p>Renders this template.</p>\n"},"setTemplate":{"!type":"fn(template: ?, context?: ?)","!doc":"<p>Quickly switch between templated by location</p>\n"}}},"dojox/dtl":{"!doc":"<p>Deprecated.  Should require dojox/dtl modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/dtl/contrib/dijit":{"widgetsInTemplate":{},"AttachNode":{"!type":"fn(keys: ?, object: ?)"},"data-dojo-attach-event":{"!type":"fn(parser: ?, token: ?) -> +instance"},"data-dojo-attach-point":{"!type":"fn(parser: ?, token: ?) -> +instance"},"data-dojo-type":{"!type":"fn(parser: ?, token: ?) -> +instance"},"dojoAttachEvent":{"!type":"fn(parser: ?, token: ?) -> +instance"},"dojoAttachPoint":{"!type":"fn(parser: ?, token: ?) -> +instance"},"dojoType":{"!type":"fn(parser: ?, token: ?) -> +instance"},"DojoTypeNode":{"!type":"fn(node: ?, parsed: ?)"},"EventNode":{"!type":"fn(command: ?, obj: ?)"},"on":{"!type":"fn(parser: ?, token: ?) -> +instance","!doc":"<p>Associates an event type to a function (on the current widget) by name</p>\n"}},"dojox/dtl/render/dom":{"Render":{"!type":"fn(attachPoint?: ?, tpl?: ?)"}},"dojox/dtl/_Templated":{"!type":"fn()","prototype":{"!proto":"dijit/_TemplatedMixin.prototype","!doc":"<p>The base-class for DTL-templated widgets.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_dijitTemplateCompat":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_skipNodeCache":{},"_templateCache":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"searchContainerNode":{},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"buildRendering":{"!type":"fn()"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"destroyRendering":{"!type":"fn()"},"getCachedTemplate":{"!type":"fn(templatePath: ?, templateString: ?, alwaysUseString: ?) -> ?","!doc":"<p>Layer for dijit._Templated.getCachedTemplate</p>\n"},"render":{"!type":"fn()","!doc":"<p>Renders the widget.</p>\n"},"startup":{"!type":"fn()"}}},"dojox/dtl/contrib/data":{"bind_data":{"!type":"fn(parser: ?, token: ?) -> +instance","!doc":"<p>Turns a list of data store items into DTL compatible items</p>\n"},"bind_query":{"!type":"fn(parser: ?, token: ?) -> +instance","!doc":"<p>Queries a data store and makes the returned items DTL compatible</p>\n"},"BindDataNode":{"!type":"fn(items: ?, query: ?, store: ?, alias: ?)"}},"dojox/dtl/contrib/dom":{"buffer":{"!type":"fn(parser: ?, token: ?) -> +instance","!doc":"<p>Buffer large DOM manipulations during re-render.</p>\n"},"BufferNode":{"!type":"fn(nodelist: ?, options: ?)"},"html":{"!type":"fn(parser: ?, token: ?) -> ?"},"style_":{"!type":"fn(parser: ?, token: ?) -> +instance"},"StyleNode":{"!type":"fn(styles: ?)"}},"dojox/dtl/contrib/objects":{"key":{"!type":"fn(value: ?, arg: ?) -> ?"}},"dojox/dtl/ext-dojo/NodeList":{"!doc":"<p>Array-like object which adds syntactic\nsugar for chaining, common iteration operations, animation, and\nnode manipulation. NodeLists are most often returned as the\nresult of dojo/query() calls.</p>\n","_nodeDataCache":{},"events":{},"prototype":{"addClass":{"!type":"fn(className: ?)","!doc":"<p>adds the specified class to every node in the list</p>\n"},"addClassFx":{"!type":"fn(cssClass: ?, args: ?)","!doc":"<p>Animate the effects of adding a class to all nodes in this list.\nsee <code>dojox.fx.addClass</code></p>\n"},"addContent":{"!type":"fn(content: ?, position?: ?) -> fn","!doc":"<p>add a node, NodeList or some HTML as a string to every item in the\nlist.  Returns the original list.</p>\n"},"adopt":{"!type":"fn(queryOrListOrNode: ?, position?: ?) -> ?","!doc":"<p>places any/all elements in queryOrListOrNode at a\nposition relative to the first element in this list.\nReturns a dojo/NodeList of the adopted elements.</p>\n"},"after":{"!type":"fn(content: ?) -> +any","!doc":"<p>Places the content after every node in the NodeList.</p>\n"},"andSelf":{"!type":"fn() -> ?","!doc":"<p>Adds the nodes from the previous dojo/NodeList to the current dojo/NodeList.</p>\n"},"anim":{"!type":"fn(properties: ?, duration?: ?, easing?: ?, onEnd?: ?, delay?: ?) -> ?","!doc":"<p>Animate one or more CSS properties for all nodes in this list.\nThe returned animation object will already be playing when it\nis returned. See the docs for <code>dojo.anim</code> for full details.</p>\n"},"animateProperty":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>Animate all elements of this NodeList across the properties specified.\nsyntax identical to <code>dojo.animateProperty</code>\n</p>\n"},"append":{"!type":"fn(content: ?) -> +any","!doc":"<p>appends the content to every node in the NodeList.</p>\n"},"appendTo":{"!type":"fn(query: ?) -> +any","!doc":"<p>appends nodes in this NodeList to the nodes matched by\nthe query passed to appendTo.</p>\n"},"at":{"!type":"fn(index: ?) -> ?","!doc":"<p>Returns a new NodeList comprised of items in this NodeList\nat the given index or indices.\n</p>\n"},"attr":{"!type":"fn(property: ?, value?: ?) -> +any","!doc":"<p>gets or sets the DOM attribute for every element in the\nNodeList. See also <code>dojo/dom-attr</code></p>\n"},"before":{"!type":"fn(content: ?) -> +any","!doc":"<p>Places the content before every node in the NodeList.</p>\n"},"children":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all immediate child elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the child elements.</p>\n"},"clone":{"!type":"fn() -> +any","!doc":"<p>Clones all the nodes in this NodeList and returns them as a new NodeList.</p>\n"},"closest":{"!type":"fn(query: ?, root?: ?) -> +any","!doc":"<p>Returns closest parent that matches query, including current node in this\ndojo/NodeList if it matches the query.</p>\n"},"concat":{"!type":"fn(item?: ?) -> ?","!doc":"<p>Returns a new NodeList comprised of items in this NodeList\nas well as items passed in as parameters</p>\n"},"connect":{"!type":"fn(methodName: ?, objOrFunc: ?, funcName?: ?)","!doc":"<p>Attach event handlers to every item of the NodeList. Uses dojo.connect()\nso event properties are normalized.\n\n</p>\n<p>Application must manually require() &quot;dojo/_base/connect&quot; before using this method.</p>\n"},"constructor":{"!type":"fn(array: ?) -> []","!doc":"<p>Array-like object which adds syntactic\nsugar for chaining, common iteration operations, animation, and\nnode manipulation. NodeLists are most often returned as the\nresult of dojo/query() calls.</p>\n"},"coords":{"!type":"fn()","!doc":"<p>Deprecated: Use position() for border-box x/y/w/h\nor marginBox() for margin-box w/h/l/t.\nReturns the box objects of all elements in a node list as\nan Array (<em>not</em> a NodeList). Acts like <code>domGeom.coords</code>, though assumes\nthe node passed is each node in this list.</p>\n"},"data":{"!type":"fn(key?: ?, value?: ?) -> +Object","!doc":"<p>stash or get some arbitrary data on/from these nodes.\n</p>\n"},"delegate":{"!type":"fn(selector: ?, eventName: ?, fn: ?) -> ?","!doc":"<p>Monitor nodes in this NodeList for [bubbled] events on nodes that match selector.\nCalls fn(evt) for those events, where (inside of fn()), this == the node\nthat matches the selector.</p>\n"},"dtl":{"!type":"fn(template: ?, context: ?) -> fn","!doc":"<p>Renders the specified template in each of the NodeList entries.</p>\n"},"empty":{"!type":"fn() -> ?","!doc":"<p>clears all content from each node in the list. Effectively\nequivalent to removing all child nodes from every item in\nthe list.</p>\n"},"end":{"!type":"fn() -> ?","!doc":"<p>Ends use of the current <code>NodeList</code> by returning the previous NodeList\nthat generated the current NodeList.</p>\n"},"even":{"!type":"fn() -> +any","!doc":"<p>Returns the even nodes in this dojo/NodeList as a dojo/NodeList.</p>\n"},"every":{"!type":"fn(callback: ?, thisObject?: ?) -> ?","!doc":"<p>see <code>dojo/_base/array.every()</code> and the <a href=\"https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/every\">Array.every\ndocs</a>.\nTakes the same structure of arguments and returns as\ndojo/_base/array.every() with the caveat that the passed array is\nimplicitly this NodeList</p>\n"},"fadeIn":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>fade in all elements of this NodeList via <code>dojo.fadeIn</code>\n</p>\n"},"fadeOut":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>fade out all elements of this NodeList via <code>dojo.fadeOut</code>\n</p>\n"},"filter":{"!type":"fn(filter: ?) -> ?","!doc":"<p>&quot;masks&quot; the built-in javascript filter() method (supported\nin Dojo via <code>dojo.filter</code>) to support passing a simple\nstring filter in addition to supporting filtering function\nobjects.</p>\n"},"first":{"!type":"fn() -> +any","!doc":"<p>Returns the first node in this dojo/NodeList as a dojo/NodeList.</p>\n"},"forEach":{"!type":"fn(callback: ?, thisObj: ?) -> fn","!doc":"<p>see <code>dojo/_base/array.forEach()</code>. The primary difference is that the acted-on\narray is implicitly this NodeList. If you want the option to break out\nof the forEach loop, use every() or some() instead.</p>\n"},"html":{"!type":"fn(value?: ?) -> +any","!doc":"<p>allows setting the innerHTML of each node in the NodeList,\nif there is a value passed in, otherwise, reads the innerHTML value of the first node.</p>\n"},"indexOf":{"!type":"fn(value: ?, fromIndex?: ?) -> +any","!doc":"<p>see <code>dojo/_base/array.indexOf()</code>. The primary difference is that the acted-on\narray is implicitly this NodeList</p>\n"},"innerHTML":{"!type":"fn(value?: ?) -> +any","!doc":"<p>allows setting the innerHTML of each node in the NodeList,\nif there is a value passed in, otherwise, reads the innerHTML value of the first node.</p>\n"},"insertAfter":{"!type":"fn(query: ?) -> +any","!doc":"<p>The nodes in this NodeList will be placed after the nodes\nmatched by the query passed to insertAfter.</p>\n"},"insertBefore":{"!type":"fn(query: ?) -> +any","!doc":"<p>The nodes in this NodeList will be placed after the nodes\nmatched by the query passed to insertAfter.</p>\n"},"instantiate":{"!type":"fn(declaredClass: ?, properties?: ?) -> ?","!doc":"<p>Create a new instance of a specified class, using the\nspecified properties and each node in the NodeList as a\nsrcNodeRef.</p>\n"},"last":{"!type":"fn() -> +any","!doc":"<p>Returns the last node in this dojo/NodeList as a dojo/NodeList.</p>\n"},"lastIndexOf":{"!type":"fn(value: ?, fromIndex?: ?) -> +any","!doc":"<p>see <code>dojo/_base/array.lastIndexOf()</code>. The primary difference is that the\nacted-on array is implicitly this NodeList</p>\n"},"map":{"!type":"fn(func: ?, obj?: ?) -> ?","!doc":"<p>see <code>dojo/_base/array.map()</code>. The primary difference is that the acted-on\narray is implicitly this NodeList and the return is a\nNodeList (a subclass of Array)</p>\n"},"marginBox":{"!type":"fn()","!doc":"<p>Returns margin-box size of nodes</p>\n"},"next":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns the next element for nodes in this dojo/NodeList.\nOptionally takes a query to filter the next elements.</p>\n"},"nextAll":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all sibling elements that come after the nodes in this dojo/NodeList.\nOptionally takes a query to filter the sibling elements.</p>\n"},"odd":{"!type":"fn() -> +any","!doc":"<p>Returns the odd nodes in this dojo/NodeList as a dojo/NodeList.</p>\n"},"on":{"!type":"fn(eventName: ?, listener: ?) -> ?","!doc":"<p>Listen for events on the nodes in the NodeList. Basic usage is:\n</p>\n"},"orphan":{"!type":"fn(filter?: ?) -> +any","!doc":"<p>removes elements in this list that match the filter\nfrom their parents and returns them as a new NodeList.</p>\n"},"parent":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns immediate parent elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the parent elements.</p>\n"},"parents":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all parent elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the child elements.</p>\n"},"place":{"!type":"fn(queryOrNode: ?, position: ?) -> ?","!doc":"<p>places elements of this node list relative to the first element matched\nby queryOrNode. Returns the original NodeList. See: <code>dojo/dom-construct.place</code></p>\n"},"position":{"!type":"fn() -> ?","!doc":"<p>Returns border-box objects (x/y/w/h) of all elements in a node list\nas an Array (<em>not</em> a NodeList). Acts like <code>dojo/dom-geometry-position</code>, though\nassumes the node passed is each node in this list.</p>\n"},"prepend":{"!type":"fn(content: ?) -> +any","!doc":"<p>prepends the content to every node in the NodeList.</p>\n"},"prependTo":{"!type":"fn(query: ?) -> +any","!doc":"<p>prepends nodes in this NodeList to the nodes matched by\nthe query passed to prependTo.</p>\n"},"prev":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns the previous element for nodes in this dojo/NodeList.\nOptionally takes a query to filter the previous elements.</p>\n"},"prevAll":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all sibling elements that come before the nodes in this dojo/NodeList.\nOptionally takes a query to filter the sibling elements.</p>\n"},"query":{"!type":"fn(queryStr: ?) -> fn","!doc":"<p>Returns a new list whose members match the passed query,\nassuming elements of the current NodeList as the root for\neach search.</p>\n"},"remove":{"!type":"fn(filter?: ?) -> +any","!doc":"<p>removes elements in this list that match the filter\nfrom their parents and returns them as a new NodeList.</p>\n"},"removeAttr":{"!type":"fn(name: ?)","!doc":"<p>Removes an attribute from each node in the list.</p>\n"},"removeClass":{"!type":"fn(className?: ?) -> +any","!doc":"<p>removes the specified class from every node in the list</p>\n"},"removeClassFx":{"!type":"fn(cssClass: ?, args: ?)","!doc":"<p>Animate the effect of removing a class to all nodes in this list.\nsee <code>dojox.fx.removeClass</code></p>\n"},"removeData":{"!type":"fn(key?: ?)","!doc":"<p>Remove the data associated with these nodes.</p>\n"},"replaceAll":{"!type":"fn(query: ?) -> +any","!doc":"<p>replaces nodes matched by the query passed to replaceAll with the nodes\nin this NodeList.</p>\n"},"replaceClass":{"!type":"fn(addClassStr: ?, removeClassStr?: ?)","!doc":"<p>Replaces one or more classes on a node if not present.\nOperates more quickly than calling <code>removeClass()</code> and <code>addClass()</code></p>\n"},"replaceWith":{"!type":"fn(content: ?) -> +any","!doc":"<p>Replaces each node in ths NodeList with the content passed to replaceWith.</p>\n"},"siblings":{"!type":"fn(query?: ?) -> +any","!doc":"<p>Returns all sibling elements for nodes in this dojo/NodeList.\nOptionally takes a query to filter the sibling elements.</p>\n"},"slice":{"!type":"fn(begin: ?, end?: ?) -> ?","!doc":"<p>Returns a new NodeList, maintaining this one in place</p>\n"},"slideTo":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>slide all elements of the node list to the specified place via <code>dojo/fx.slideTo()</code>\n</p>\n"},"some":{"!type":"fn(callback: ?, thisObject?: ?) -> ?","!doc":"<p>Takes the same structure of arguments and returns as\n<code>dojo/_base/array.some()</code> with the caveat that the passed array is\nimplicitly this NodeList.  See <code>dojo/_base/array.some()</code> and Mozilla&#39;s\n<a href=\"https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Array/some\">Array.some\ndocumentation</a>.</p>\n"},"splice":{"!type":"fn(index: ?, howmany?: ?, item?: ?) -> ?","!doc":"<p>Returns a new NodeList, manipulating this NodeList based on\nthe arguments passed, potentially splicing in new elements\nat an offset, optionally deleting elements</p>\n"},"style":{"!type":"fn(property: ?, value?: ?) -> +any","!doc":"<p>gets or sets the CSS property for every element in the NodeList</p>\n"},"text":{"!type":"fn(value: ?) -> +any","!doc":"<p>allows setting the text value of each node in the NodeList,\nif there is a value passed in, otherwise, returns the text value for all the\nnodes in the NodeList in one string.</p>\n"},"toggleClass":{"!type":"fn(className: ?, condition?: ?)","!doc":"<p>Adds a class to node if not present, or removes if present.\nPass a boolean condition if you want to explicitly add or remove.</p>\n"},"toggleClassFx":{"!type":"fn(cssClass: ?, force: ?, args: ?)","!doc":"<p>Animate the effect of adding or removing a class to all nodes in this list.\nsee <code>dojox.fx.toggleClass</code></p>\n"},"toString":{"!type":"fn() -> ?"},"val":{"!type":"fn(value: ?) -> +any","!doc":"<p>If a value is passed, allows seting the value property of form elements in this\nNodeList, or properly selecting/checking the right value for radio/checkbox/select\nelements. If no value is passed, the value of the first node in this NodeList\nis returned.</p>\n"},"wipeIn":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>wipe in all elements of this NodeList via <code>dojo/fx.wipeIn()</code>\n</p>\n"},"wipeOut":{"!type":"fn(args?: ?) -> +dojo/_base/fx.Animation","!doc":"<p>wipe out all elements of this NodeList via <code>dojo/fx.wipeOut()</code>\n</p>\n"},"wrap":{"!type":"fn(html: ?) -> +any","!doc":"<p>Wrap each node in the NodeList with html passed to wrap.</p>\n"},"wrapAll":{"!type":"fn(html: ?) -> +any","!doc":"<p>Insert html where the first node in this NodeList lives, then place all\nnodes in this NodeList as the child of the html.</p>\n"},"wrapInner":{"!type":"fn(html: ?) -> +any","!doc":"<p>For each node in the NodeList, wrap all its children with the passed in html.</p>\n"}}},"dojox/dtl/ext-dojo/NodeList._nodeDataCache":{},"dojox/dtl/filter/dates":{"date":{"!type":"fn(value: ?, arg: ?) -> string","!doc":"<p>Formats a date according to the given format</p>\n"},"time":{"!type":"fn(value: ?, arg: ?) -> string","!doc":"<p>Formats a time according to the given format</p>\n"},"timesince":{"!type":"fn(value: ?, arg: ?) -> string","!doc":"<p>Formats a date as the time since that date (i.e. &quot;4 days, 6 hours&quot;)</p>\n"},"timeuntil":{"!type":"fn(value: ?, arg: ?) -> string","!doc":"<p>Formats a date as the time until that date (i.e. &quot;4 days, 6 hours&quot;)</p>\n"}},"dojox/dtl/utils/date":{"DateFormat":{"!type":"fn(format: ?)","!doc":"<p>Format the internal date object</p>\n"}},"dojox/dtl/filter/htmlstrings":{"_linebreaksbr":{},"_linebreaksn":{},"_linebreaksrn":{},"_linebreakss":{},"_removetagsfind":{},"_striptags":{},"linebreaks":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts newlines into <code>&lt;p&gt;</code> and <code>&lt;br /&gt;</code>s</p>\n"},"linebreaksbr":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts newlines into <code>&lt;br /&gt;</code>s</p>\n"},"removetags":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>Removes a space separated list of [X]HTML tags from the output&quot;</p>\n"},"striptags":{"!type":"fn(value: ?) -> ?","!doc":"<p>Strips all [X]HTML tags</p>\n"}},"dojox/dtl/filter/integers":{"add":{"!type":"fn(value: ?, arg: ?) -> number"},"get_digit":{"!type":"fn(value: ?, arg: ?) -> number","!doc":"<p>Given a whole number, returns the 1-based requested digit of it</p>\n"}},"dojox/dtl/filter/lists":{"dictsort":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>Takes a list of dicts, returns that list sorted by the property given in the argument.</p>\n"},"dictsortreversed":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>Takes a list of dicts, returns that list sorted in reverse order by the property given in the argument.</p>\n"},"first":{"!type":"fn(value: ?) -> string","!doc":"<p>Returns the first item in a list</p>\n"},"join":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>Joins a list with a string, like Python&#39;s <code>str.join(list)</code></p>\n"},"length":{"!type":"fn(value: ?) -> ?","!doc":"<p>Returns the length of the value - useful for lists</p>\n"},"length_is":{"!type":"fn(value: ?, arg: ?) -> bool","!doc":"<p>Returns a boolean of whether the value&#39;s length is the argument</p>\n"},"random":{"!type":"fn(value: ?) -> ?","!doc":"<p>Returns a random item from the list</p>\n"},"slice":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>Returns a slice of the list.</p>\n"},"unordered_list":{"!type":"fn(value: ?) -> ?","!doc":"<p>Recursively takes a self-nested list and returns an HTML unordered list --\nWITHOUT opening and closing <code>&lt;ul&gt;</code> tags.</p>\n"}},"dojox/dtl/filter/logic":{"_yesno":{},"default_":{"!type":"fn(value: ?, arg: ?) -> string","!doc":"<p>If value is unavailable, use given default</p>\n"},"default_if_none":{"!type":"fn(value: ?, arg: ?) -> string","!doc":"<p>If value is null, use given default</p>\n"},"divisibleby":{"!type":"fn(value: ?, arg: ?) -> bool","!doc":"<p>Returns true if the value is divisible by the argument&quot;</p>\n"},"yesno":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>arg being a comma-delimited string, value of true/false/none\nchooses the appropriate item from the string</p>\n"}},"dojox/dtl/filter/misc":{"_phone2numeric":{},"filesizeformat":{"!type":"fn(value: ?) -> string","!doc":"<p>Format the value like a &#39;human-readable&#39; file size (i.e. 13 KB, 4.1 MB, 102bytes, etc).</p>\n"},"phone2numeric":{"!type":"fn(value: ?) -> string","!doc":"<p>Takes a phone number and converts it in to its numerical equivalent</p>\n"},"pluralize":{"!type":"fn(value: ?, arg: ?) -> string","!doc":"<p>Returns a plural suffix if the value is not 1, for &#39;1 vote&#39; vs. &#39;2 votes&#39;</p>\n"},"pprint":{"!type":"fn(value: ?) -> ?","!doc":"<p>A wrapper around toJson unless something better comes along</p>\n"}},"dojox/dtl/filter/misc._phone2numeric":{"a":{},"b":{},"c":{},"d":{},"e":{},"f":{},"g":{},"h":{},"i":{},"j":{},"k":{},"l":{},"m":{},"n":{},"o":{},"p":{},"r":{},"s":{},"t":{},"u":{},"v":{},"w":{},"x":{},"y":{}},"dojox/dtl/filter/strings":{"_fix_ampersands":{},"_strings":{},"_truncate_singlets":{},"_truncate_tag":{},"_truncate_words":{},"_truncatewords":{},"_urlize":{},"_urlize2":{},"addslashes":{"!type":"fn(value: ?) -> ?","!doc":"<p>Adds slashes - useful for passing strings to JavaScript, for example.</p>\n"},"capfirst":{"!type":"fn(value: ?) -> number","!doc":"<p>Capitalizes the first character of the value</p>\n"},"center":{"!type":"fn(value: ?, arg: ?) -> string","!doc":"<p>Centers the value in a field of a given width</p>\n"},"cut":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>Removes all values of arg from the given string</p>\n"},"fix_ampersands":{"!type":"fn(value: ?) -> ?","!doc":"<p>Replaces ampersands with <code>&amp;amp;</code> entities</p>\n"},"floatformat":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>Format a number according to arg</p>\n"},"iriencode":{"!type":"fn(value: ?) -> ?"},"linenumbers":{"!type":"fn(value: ?) -> ?","!doc":"<p>Displays text with line numbers</p>\n"},"ljust":{"!type":"fn(value: ?, arg: ?) -> string"},"lower":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts a string into all lowercase</p>\n"},"make_list":{"!type":"fn(value: ?) -> []","!doc":"<p>Returns the value turned into a list. For an integer, it&#39;s a list of\ndigits. For a string, it&#39;s a list of characters.</p>\n"},"rjust":{"!type":"fn(value: ?, arg: ?) -> string"},"slugify":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts to lowercase, removes\nnon-alpha chars and converts spaces to hyphens</p>\n"},"stringformat":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>Formats the variable according to the argument, a string formatting specifier.\nThis specifier uses Python string formatting syntax, with the exception that\nthe leading &quot;%&quot; is dropped.</p>\n"},"title":{"!type":"fn(value: ?) -> string","!doc":"<p>Converts a string into titlecase</p>\n"},"truncatewords":{"!type":"fn(value: ?, arg: ?) -> ?","!doc":"<p>Truncates a string after a certain number of words</p>\n"},"truncatewords_html":{"!type":"fn(value: ?, arg: ?) -> string"},"upper":{"!type":"fn(value: ?) -> ?"},"urlencode":{"!type":"fn(value: ?) -> ?"},"urlize":{"!type":"fn(value: ?) -> ?"},"urlizetrunc":{"!type":"fn(value: ?, arg: ?) -> ?"},"wordcount":{"!type":"fn(value: ?) -> number"},"wordwrap":{"!type":"fn(value: ?, arg: ?) -> ?"}},"dojox/dtl/filter/strings._strings":{},"dojox/dtl/filter/strings._truncate_singlets":{"area":{},"base":{},"br":{},"col":{},"hr":{},"img":{},"input":{},"link":{},"param":{}},"dojox/string/sprintf":{"prototype":{"constructor":{"!type":"fn(format: ?, filler: ?)"}},"Formatter":{"!type":"fn(format: ?)"}},"dojox/dtl/render/html":{"Render":{"!type":"fn(attachPoint?: ?, tpl?: ?)"}},"dojox/dtl/tag/date":{"now":{"!type":"fn(parser: ?, token: ?)"},"NowNode":{"!type":"fn(format: ?, node: ?)"}},"dojox/dtl/tag/loader":{"block":{"!type":"fn(parser: ?, token: ?) -> +instance"},"BlockNode":{"!type":"fn(name: ?, nodelist: ?)"},"extends_":{"!type":"fn(parser: ?, token: ?) -> +instance"},"ExtendsNode":{"!type":"fn(getTemplate: ?, nodelist: ?, shared: ?, parent: ?, key: ?)"},"include":{"!type":"fn(parser: ?, token: ?) -> +instance"},"IncludeNode":{"!type":"fn(path: ?, constant: ?, getTemplate: ?, text: ?, parsed: ?)"},"ssi":{"!type":"fn(parser: ?, token: ?) -> ?"}},"dojox/dtl/tag/logic":{"for_":{"!type":"fn(parser: ?, token: ?) -> +instance"},"ForNode":{"!type":"fn(assign: ?, loop: ?, reversed: ?, nodelist: ?)"},"if_":{"!type":"fn(parser: ?, token: ?) -> +instance"},"ifequal":{"!type":"fn(parser: ?, token: ?) -> ?"},"IfEqualNode":{"!type":"fn(var1: ?, var2: ?, trues: ?, falses: ?, negate: ?)"},"IfNode":{"!type":"fn(bools: ?, trues: ?, falses: ?, type: ?)"},"ifnotequal":{"!type":"fn(parser: ?, token: ?) -> ?"}},"dojox/dtl/tag/loop":{"cycle":{"!type":"fn(parser: ?, token: ?) -> ?","!doc":"<p>Cycle among the given strings each time this tag is encountered</p>\n"},"CycleNode":{"!type":"fn(cyclevars: ?, name: ?, text: ?, shared: ?)"},"ifchanged":{"!type":"fn(parser: ?, token: ?) -> +instance"},"IfChangedNode":{"!type":"fn(nodes: ?, vars: ?, shared: ?)"},"regroup":{"!type":"fn(parser: ?, token: ?) -> +instance"},"RegroupNode":{"!type":"fn(expression: ?, key: ?, alias: ?)"}},"dojox/dtl/tag/misc":{"comment":{"!type":"fn(parser: ?, token: ?) -> +instance","!doc":"<p>Ignore everything between {% comment %} and {% endcomment %}</p>\n"},"debug":{"!type":"fn(parser: ?, token: ?) -> +instance","!doc":"<p>Output the current context, maybe add more stuff later.</p>\n"},"DebugNode":{"!type":"fn(text: ?)"},"filter":{"!type":"fn(parser: ?, token: ?) -> +instance","!doc":"<p>Filter the contents of the blog through variable filters.</p>\n"},"FilterNode":{"!type":"fn(varnode: ?, nodelist: ?)"},"firstof":{"!type":"fn(parser: ?, token: ?) -> +instance"},"FirstOfNode":{"!type":"fn(vars: ?, text: ?)"},"spaceless":{"!type":"fn(parser: ?, token: ?) -> +instance"},"SpacelessNode":{"!type":"fn(nodelist: ?, text: ?)"},"templatetag":{"!type":"fn(parser: ?, token: ?) -> +instance"},"TemplateTagNode":{"!type":"fn(tag: ?, text: ?)"},"widthratio":{"!type":"fn(parser: ?, token: ?) -> +instance"},"WidthRatioNode":{"!type":"fn(current: ?, max: ?, width: ?, text: ?)"},"with_":{"!type":"fn(parser: ?, token: ?) -> +instance"},"WithNode":{"!type":"fn(target: ?, alias: ?, nodelist: ?)"}},"dojox/editor/plugins/AutoSave":{"!type":"fn()","prototype":{"!proto":"dojox/editor/plugins/Save.prototype","!doc":"<p>This plugin provides the auto save capability to the editor. The\nplugin saves the content of the editor in interval. When\nthe save action is performed, the document in the editor frame\nwill be posted to the URL provided, or none, if none provided.</p>\n","_iconClassPrefix":{"!doc":"<p>This prefix of the CSS class</p>\n"},"_MIN":{"!doc":"<p>Default 1 minute</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code>\nand <code>command</code></p>\n"},"interval":{"!doc":"<p>The interval to perform the save action.</p>\n"},"logResults":{"!doc":"<p>Boolean flag to indicate that the default action for save and\nerror handlers is to just log to console.  Default is true.</p>\n"},"url":{"!doc":"<p>The URL to POST the content back to.  Used by the save function.</p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup of our plugin.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"save":{"!type":"fn(content: ?)","!doc":"<p>User over-ridable save function for the editor content.\nPlease note that the service URL provided should do content\nfiltering of the posted content to avoid XSS injection via\nthe data from the editor.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor. No toggle button for\nthis plugin. And start to save the content of the editor in\ninterval</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/editor/plugins/AutoSave._AutoSaveSettingDialog":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"btnCancel":{},"btnOk":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dialogDescription":{},"dialogTitle":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"paramLabel":{},"paramName":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hide the setting dialog.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"show":{"!type":"fn()","!doc":"<p>Display the setting dialog. If the internal interval value is &quot;&quot;\nset it to zero</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/Save":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides Save capability to the editor.  When\nclicked, the document in the editor frame will be posted to the URL\nprovided, or none, if none provided.  Users who desire a different save\nfunction can extend this plugin (via dojo.extend) and over-ride the\nsave method while save is in process, the save button is disabled.</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code>\nand <code>command</code></p>\n"},"logResults":{"!doc":"<p>Boolean flag to indicate that the default action for save and\nerror handlers is to just log to console.  Default is true.</p>\n"},"url":{"!doc":"<p>The URL to POST the content back to.  Used by the save function.</p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"save":{"!type":"fn(content: ?)","!doc":"<p>User over-ridable save function for the editor content.\nPlease note that the service URL provided should do content\nfiltering of the posted content to avoid XSS injection via\nthe data from the editor.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/editor/plugins/AutoUrlLink":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin can recognize a URL like string\n(such as <a href=\"http://www.website.com\">http://www.website.com</a>) and turn it into\na hyperlink that points to that URL.</p>\n","_template":{"!doc":"<p>The link template</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Called by the editor it belongs to.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dojox/editor/plugins/Blockquote":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides Blockquote capability to the editor.\nwindow/tab</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node icon.</p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Overrides _Plugin.updateState().  This controls whether or not the current\ncursor position should toggle on the quote button or not.</p>\n"}}},"dojox/editor/plugins/Breadcrumb":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides Breadcrumb capability to the editor. As you move\naround the editor, it updates with your current indention depth.</p>\n","_menu":{"!doc":"<p>The popup menu that is displayed.</p>\n"},"breadcrumbBar":{"!doc":"<p>The toolbar containing the breadcrumb.</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Over-ride to clean up the breadcrumb toolbar.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride of updateState to hide the toolbar when the iframe is not visible.\nAlso triggers the breadcrumb update.</p>\n"}}},"dojox/editor/plugins/Breadcrumb._BreadcrumbMenuTitle":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Simple internal, non-clickable, menu entry to act as a menu title bar.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"menuTitle":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/CollapsibleToolbar":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides a weappable toolbar container to allow expand/collapse\nof the editor toolbars.  This plugin should be registered first in most cases to\navoid conflicts in toolbar construction.</p>\n","_myWidgets":{"!doc":"<p>Container for widgets I allocate that will need to be destroyed.</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Over-ride of destroy method for cleanup.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dojox/editor/plugins/CollapsibleToolbar._CollapsibleToolbarButton":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Simple internal widget for representing a clickable button for expand/collapse\nwith A11Y support.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"buttonClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"text":{},"textClass":{},"title":{},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/EntityPalette":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A keyboard accessible HTML entity-picking widget (for inserting symbol characters)</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_currentFocus":{"!doc":"<p>The currently focused cell (if the palette itself has focus), or otherwise\nthe cell to be focused when the palette itself gets focus.\nDifferent from value, which represents the selected (i.e. clicked) cell.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selectedCell":{"!doc":"<p>Index of the currently selected cell. Initially, none selected</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_xDim":{"!doc":"<p>This is the number of cells horizontally across.</p>\n"},"_yDim":{"!doc":"<p>This is the number of cells vertically down.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"cellClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"defaultTimeout":{"!doc":"<p>Number of milliseconds before a held key or button becomes typematic</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dyeClass":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"palette":{"!doc":"<p>The symbol pallete to display.  The only current one is &#39;latin&#39;.</p>\n"},"paletteClass":{},"searchContainerNode":{},"showCode":{"!doc":"<p>Show the character code for the entity.</p>\n"},"showEntityName":{"!doc":"<p>Show the entity name for the entity.</p>\n"},"showPreview":{"!doc":"<p>Whether the preview pane will be displayed, to show details about the selected entity.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Widget tab index.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The basic template used to render the palette.\nShould generally be over-ridden to define different classes.</p>\n"},"timeoutChangeRate":{"!doc":"<p>Fraction of time used to change the typematic timer between events\n1.0 means that each typematic event fires at defaultTimeout intervals\nLess than 1.0 means that each typematic event fires at an increasing faster rate</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>Currently selected color/emoticon/etc.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Instantiate the template, which makes a skeleton table which we&#39;ll insert the entities</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus this widget.  Puts focus on the most recently focused cell.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Convert hash of entities into two-dimensional rows/columns table (array of arrays)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"LatinEntity":{"!type":"fn()"}},"dojox/editor/plugins/EntityPalette.LatinEntity":{"!type":"fn()","prototype":{"!doc":"<p>Represents a character.\nInitialized using an alias for the character (like cent) rather\nthan with the character itself.</p>\n","constructor":{"!type":"fn(alias: ?)","!doc":"<p>Construct JS object representing an entity (associated w/a cell\n   in the palette)</p>\n"},"fillCell":{"!type":"fn(cell: ?)"},"getValue":{"!type":"fn() -> string","!doc":"<p>Returns HTML representing the character, like &amp;</p>\n"}}},"dojox/editor/plugins/FindReplace":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides a Find/Replace capability for the editor.\nNote that this plugin is NOT supported on Opera currently, as opera\ndoes not implement a window.find or equiv function.</p>\n","_backwards":{"!doc":"<p>The backwards checkbox</p>\n"},"_caseSensitive":{"!doc":"<p>The case sensitive checkbox</p>\n"},"_closeBox":{"!doc":"<p>The close button of the F/R toolbar</p>\n"},"_findButton":{"!doc":"<p>The Find button of the F/R toolbar</p>\n"},"_findField":{"!doc":"<p>The Find field of the F/R toolbar</p>\n"},"_frToolbar":{"!doc":"<p>The toolbar that contain all the entries and buttons</p>\n"},"_promDialog":{"!doc":"<p>The prompt message box that shows the user some messages\nsuch as the end of a search, the end of a replacement, etc.</p>\n"},"_promDialogTimeout":{},"_replaceAllButton":{"!doc":"<p>The ReplaceAll button of the F/R toolbar</p>\n"},"_replaceButton":{"!doc":"<p>The Replace button of the F/R toolbar</p>\n"},"_replaceField":{"!doc":"<p>The Replace field of the F/R toolbar</p>\n"},"_strings":{"!doc":"<p>The array that contains globalized strings</p>\n"},"button":{"!doc":"<p>The toggle button</p>\n"},"buttonClass":{"!doc":"<p>Define the class of button the editor uses.</p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>The editor this plugin belongs to</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup of our custom toolbar.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>This is a callback handler that set a reference to the editor this plugin\nhosts in</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Over-ride so that find/replace toolbar is appended after the current toolbar.</p>\n"},"toggle":{"!type":"fn()","!doc":"<p>Function to allow programmatic toggling of the find toolbar.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/editor/plugins/FindReplace._FindReplaceCloseBox":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Base class for widgets that contains a button labeled X\nto close the tool bar.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"btnId":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widget":{},"widgetsInTemplate":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/FindReplace._FindReplaceTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Base class for widgets that contains a label (like &quot;Font:&quot;)\nand a TextBox to pick a value.\nUsed as Toolbar entry.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>The label of the enhanced textbox</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"textId":{"!doc":"<p>The id of the enhanced textbox</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"toolTip":{},"widget":{},"widgetsInTemplate":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/FindReplace._FindReplaceCheckBox":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Base class for widgets that contains a label (like &quot;Match case: &quot;)\nand a checkbox to indicate if it is checked or not.\nUsed as Toolbar entry.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"checkId":{"!doc":"<p>The id of the enhanced checkbox</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>The label of the enhanced checkbox</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>The tooltip of the enhanced checkbox when the mouse is hovering it</p>\n"},"widget":{},"widgetsInTemplate":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/FindReplace._FindReplaceToolbar":{"!type":"fn()","prototype":{"!proto":"dijit/Toolbar.prototype","!doc":"<p>A toolbar that derived from dijit.Toolbar, which\neliminates some unnecessary event response such as LEFT_ARROW pressing\nand click bubbling.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Add a child to our _Container and prevent the default\narrow key navigation function. This function may bring in\nside effect</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"childSelector":{"!type":"fn(node: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the first child.</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"startupKeyNavChildren":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/ToolbarLineBreak":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A &#39;line break&#39; between two <code>dijit.Toolbar</code> items so that very\nlong toolbars can be organized a bit.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> bool","!doc":"<p>This widget isn&#39;t focusable, so pass along that fact.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/InsertAnchor":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides the basis for an insert anchor dialog for the\ndijit.Editor\n</p>\n","_template":{},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"htmlTemplate":{"!doc":"<p>String used for templating the HTML to insert at the desired point.</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node icon.</p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"getAnchorStyle":{"!type":"fn() -> ?","!doc":"<p>Over-ridable function for getting the style to apply to the anchor.\nThe default is a dashed border with an anchor symbol.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/editor/plugins/InsertEntity":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin allows the user to select from standard Symbols (HTML Entities)\nto insert at the current cursor position.  It binds to the key pattern:\nctrl-shift-s for opening the insert symbol dropdown.\n</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/html/entities":{"html":{},"latin":{},"decode":{"!type":"fn(str: ?, m: ?)","!doc":"<p>Function to obtain an entity encoding for a specified character</p>\n"},"encode":{"!type":"fn(str: ?, m: ?)","!doc":"<p>Function to obtain an entity encoding for a specified character</p>\n"}},"dojox/editor/plugins/LocalImage":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides an enhanced image link dialog that\nnot only insert the online images, but upload the local image files onto\nto server then insert them as well.\n\n</p>\n<p>Dependencies:\nThis plugin depends on dojox.form.FileUploader to upload the images on the local driver.\nDo the regression test whenever FileUploader is upgraded.</p>\n","_closable":{"!doc":"<p>Indicate if the tooltip dialog can be closed. Used to workaround Safari 5 bug\nwhere the file dialog doesn&#39;t pop up in modal until after the first click.</p>\n"},"_cssPrefix":{"!doc":"<p>The prefix of the CSS style</p>\n"},"_hostRxp":{},"_isLocalFile":{"!doc":"<p>Indicate if a local file is to be uploaded to the server\nIf false, the text of _urlInput field is regarded as the\nURL of the online image</p>\n"},"_messages":{"!doc":"<p>Contains i18n strings.</p>\n"},"_userAtRxp":{},"baseImageUrl":{"!doc":"<p>The prefix of the image url on the server.\nFor example, an image is uploaded and stored at\n<code>http://www.myhost.com/images/uploads/test.jpg</code>.\nWhen the image is uploaded, the server returns &quot;uploads/test.jpg&quot; as the\nrelative path. So the baseImageUrl should be set to &quot;<a href=\"http://www.myhost.com/images/\">http://www.myhost.com/images/</a>&quot;\nso that the client can retrieve the image from the server.\nIf the image file is located on the same domain as that of the current web page,\nbaseImageUrl can be a relative path. For example:\n\n</p>\n<pre><code>baseImageUrl = images/</code></pre>\n<p>and the server returns uploads/test.jpg\n\n</p>\n<p>The complete URL of the image file is images/upload/test.jpg</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"emailRegExp":{"!doc":"<p>Used for validating input as correct email address.  Taken from dojox.validate</p>\n"},"fileMask":{"!doc":"<p>Specify the types of images that are allowed to be uploaded.\nNote that the type checking on server is also very important!</p>\n"},"htmlFieldName":{},"htmlTemplate":{"!doc":"<p>String used for templating the <code>&lt;img&gt;</code> HTML to insert at the desired point.</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"linkDialogTemplate":{"!doc":"<p>Over-ride for template since this is an enhanced image dialog.</p>\n"},"tag":{"!doc":"<p>Tag used for the link type (img).</p>\n"},"uploadable":{"!doc":"<p>Indicate whether the user can upload a local image file onto the server.\nIf it is set to true, the Browse button will be available.</p>\n"},"uploadUrl":{"!doc":"<p>The url targeted for uploading. Both absolute and relative URLs are OK.</p>\n"},"urlRegExp":{"!doc":"<p>Used to validate if the input is a valid image URL.</p>\n"},"useDefaultCommand":{},"buttonClass":{"!type":"fn()"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Cleanup of the plugin.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dojox/form/FileUploader":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Handles File Uploading to a server (PHP script included for testing)\n\n</p>\n<p>FileUploader is now a WIDGET. You do not have to pass a button\nin. Passing a button is still supported until version 1.5 to maintain\nbackwards compatibility, but it is not recommended. Just create your\nuploader like any other widget.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"activeClass":{"!doc":"<p>The name of the class that will style the button in a &quot;press&quot; state. A specific\nclass should be made to do this. Do not rely on a target like button:active{...}</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the class that will style the button in a &quot;normal&quot; state.\nIf baseClass is not defined, &#39;class&#39; will be used.\nNOTE: By default the uploader will be styled like a dijit buttons and\nadhere to the the themes. Tundra, Soria, and Nihilo are supported.\nYou can cascade the existing style by using &#39;class&#39; or &#39;style&#39;. If you\noverwrite baseClass, you should overwrite the remaing state classes\nthat follow) as well.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"deferredUploading":{"!doc":"<p>(Flash only) throttles the upload to a certain amount of files at a time.\nBy default, Flash uploads file one at a time to the server, but in parallel.\nFirefox will try to queue all files at once, leading to problems. Set this\nto the amount to upload in parallel at a time.\nGenerally, 1 should work fine, but you can experiment with queuing more than\none at a time.\nThis is of course ignored if selectMultipleFiles equals false.</p>\n"},"devMode":{"!doc":"<p>Re-implemented. devMode increases the logging, adding style tracing from the SWF.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabledClass":{"!doc":"<p>The name of the class that will style the button when its disabled.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fileListId":{"!doc":"<p>The id of a dom node to be used as a container for the pending file list.</p>\n"},"fileMask":{"!doc":"<p>(an array, or an array of arrays)\nRestrict file selection to certain file types\nEmpty array defaults to &quot;All Files&quot;\n\n</p>\n<p>example:\n\n\n</p>\n<pre><code>fileMask = [&quot;Images&quot;, &quot;*.jpg;*.jpeg;*.gif;*.png&quot;]</code></pre>\n<p>or\n\n\n</p>\n<pre><code>fileMask = [\n    [&quot;Jpeg File&quot;,   &quot;*.jpg;*.jpeg&quot;],\n    [&quot;GIF File&quot;,    &quot;*.gif&quot;],\n    [&quot;PNG File&quot;,    &quot;*.png&quot;],\n    [&quot;All Images&quot;,  &quot;*.jpg;*.jpeg;*.gif;*.png&quot;],\n]</code></pre>\n<p>NOTE: MacType is not supported, as it does not work very well.\nfileMask will work on a Mac, but differently than\nWindows.</p>\n"},"flashFieldName":{"!doc":"<p>The name of the field of the flash uploaded files that the server is expecting</p>\n"},"flashMovie":{"!doc":"<p>The SWF. Mostly Internal.</p>\n"},"flashObject":{"!doc":"<p>The object that creates the SWF embed object. Mostly Internal.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"force":{"!doc":"<p>Use &quot;flash&quot; to always use Flash (and hopefully force the user to download the plugin\nif they don&#39;t have it). Use &quot;html&quot; to always use the HTML uploader. An empty string\n(default) will check for the right version of Flash and use HTML if not available.</p>\n"},"hoverClass":{"!doc":"<p>The name of the class that will style the button in a &quot;hover&quot; state. A specific\nclass should be made to do this. Do not rely on a target like button:hover{...}</p>\n"},"htmlFieldName":{"!doc":"<p>The name of the field of the fileInput that the server is expecting</p>\n"},"id":{"!doc":"<p>The object id, just like any other widget in Dojo. However, this id\nis also used as a reference for the SWF</p>\n"},"insideNode":{"!doc":"<p>The div that holds the SWF and form/fileInput</p>\n"},"isDebug":{"!doc":"<p>If true, outputs traces from the SWF to console. What exactly gets passed\nis very relative, and depends upon what traces have been left in the DEFT SWF.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"minFlashVersion":{"!doc":"<p>The minimum of version of Flash player to target. 0 would always install Flash, 100\nwould never install it. The Flash Player has supported multiple uploads since\nversion 8, so it could go as low as that safely.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"progressBackgroundColor":{"!doc":"<p>The background color to use for the button-progress</p>\n"},"progressBackgroundUrl":{"!doc":"<p>The background image to use for the button-progress</p>\n"},"progressMessage":{"!doc":"<p>The message shown while the button is changed to a progress bar</p>\n"},"progressWidgetId":{"!doc":"<p>The widget id of a Dijit Progress bar. The Uploader will bind to it and update it\nautomatically.</p>\n"},"searchContainerNode":{},"selectMultipleFiles":{"!doc":"<p>If true and flash mode, multiple files may be selected from the dialog.\nIf html mode, files are not uploaded until upload() is called. The references\nto each file is incremented:uploadedfile0, uploadedfile1, uploadedfile2... etc.</p>\n"},"serverTimeout":{"!doc":"<p>The amount of time given to the uploaded file\nto wait for a server response. After this amount\nof time, the onComplete is fired but with a &#39;server timeout&#39;\nerror in the returned item.</p>\n"},"showProgress":{"!doc":"<p>If true, the button changes to a progress bar during upload.</p>\n"},"skipServerCheck":{"!doc":"<p>If true, will not verify that the server was sent the correct format.\nThis can be safely set to true. The purpose of the server side check\nis mainly to show the dev if they&#39;ve implemented the different returns\ncorrectly.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"swfPath":{},"tabIndex":{"!doc":"<p>The tab order in the DOM. Only supported by Flash. HTML Uploaders have security\nprotection to prevent you from tabbing to the uploader. Stupid.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"uploaderType":{"!doc":"<p>Internal. What type of uploader is being used: &quot;flash&quot; or &quot;html&quot;</p>\n"},"uploadOnChange":{"!doc":"<p>If true, uploads immediately after a file has been selected. If false,\nwaits for upload() to be called.</p>\n"},"uploadUrl":{"!doc":"<p>The url targeted for upload. An absolute URL is preferred. Relative URLs are\nchanged to absolute.</p>\n"},"version":{"!doc":"<p>1.5 (deprecated)</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"createFlashUploader":{"!type":"fn()","!doc":"<p>Internal. Creates Flash Uploader</p>\n"},"createHtmlUploader":{"!type":"fn()","!doc":"<p>Internal. Fires of methods to build HTML Uploader.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys uploader button</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getButtonStyle":{"!type":"fn()","!doc":"<p>Internal.\nGet necessary style information from srcRefNode and\nassigned styles\n</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getHiddenNode":{"!type":"fn(node: ?) -> +null","!doc":"<p>Internal.\nIf a parent node is styled as display:none,\nreturns that node. This node will be temporarilly\nchanged to display:block. Note if the node is in\na widget that has an onShow event, this is\noverridden.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getStyle":{"!type":"fn(node: ?) -> ?"},"getTempNodeStyle":{"!type":"fn(node: ?, _class: ?, isDijitButton: ?) -> ?"},"getText":{"!type":"fn(node: ?) -> ?"},"getTextStyle":{"!type":"fn(node: ?) -> ?"},"isButton":{"!type":"fn(node: ?) -> bool"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"log":{"!type":"fn()","!doc":"<p>Due to the excessive logging necessary to make this code happen,\nIt&#39;s easier to turn it on and off here in one place.\nAlso helpful if there are multiple uploaders on one page.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeFile":{"!type":"fn(name: ?, noListEdit: ?)","!doc":"<p>Removes a file from the pending file list.\nRemoves pending data from the Flash movie\nand fileInputes from the HTML uploader.\nIf a file container node is bound, the file\nwill also be removed.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setButtonStyle":{"!type":"fn()","!doc":"<p>Internal.\nSet up internal dom nodes for button construction.</p>\n"},"startup":{"!type":"fn()"},"submit":{"!type":"fn(form?: ?) -> bool","!doc":"<p>If FileUploader is in a form, and other data should be sent\nalong with the files, use this instead of form submit.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"upload":{"!type":"fn(data?: ?) -> bool","!doc":"<p>When called, begins file upload</p>\n"},"uploadFlash":{"!type":"fn()","!doc":"<p>Internal. You should use upload() or submit();</p>\n"},"uploadHTML":{"!type":"fn()","!doc":"<p>Internal. You could use this, but you should use upload() or submit();\nwhich can also handle the post data.</p>\n"},"urlencode":{"!type":"fn(url: ?) -> bool"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/embed/flashVars":{"!doc":"<p>Handles flashvar serialization\nConverting complex objects into a simple, clear string that can be appended\nto the swf as a query: myMovie.swf?flashvars=foo.\nNote this needs to work with the SWF, which must know what variables to expect.\nTherefore this is something of an &quot;internal&quot; class - unless you know how to\nmodify or create SWFs.\n</p>\n","See":{"!doc":"<p>Deft/common/flashVars.as\n</p>\n"},"serialize":{"!type":"fn(n: ?, o: ?) -> ?","!doc":"<p>Key method. Serializes an object.</p>\n"}},"dojox/html/styles":{"entities":{},"ext-dojo":{},"metrics":{},"activeStyleSheet":{"!type":"fn(title?: ?)","!doc":"<p>Getter/Setter</p>\n"},"disableStyleSheet":{"!type":"fn(styleSheetName: ?)","!doc":"<p>Disables the dynamic style sheet with the name passed in the\nargument. If no arg is passed, defaults to the default style sheet.</p>\n"},"enableStyleSheet":{"!type":"fn(styleSheetName: ?)","!doc":"<p>Enables the style sheet with the name passed in the\nargument. Deafults to the default style sheet.\n</p>\n"},"getDynamicStyleSheet":{"!type":"fn(styleSheetName?: ?) -> ?","!doc":"<p>Creates and returns a dynamically created style sheet\nused for dynamic styles</p>\n"},"getPreferredStyleSheet":{"!type":"fn()","!doc":"<p>Returns the style sheet that was initially enabled\non document launch.\nTODO, does not work.</p>\n"},"getStyleSheet":{"!type":"fn(styleSheetName?: ?)","!doc":"<p>Returns a style sheet based on the argument.\nSearches dynamic style sheets first. If no matches,\nsearches document style sheets.</p>\n"},"getStyleSheets":{"!type":"fn() -> +any","!doc":"<p>Collects all the style sheets referenced in the HTML page,\nincluding any included via @import.</p>\n"},"getToggledStyleSheets":{"!type":"fn() -> +any","!doc":"<p>Searches HTML for style sheets that are &quot;toggle-able&quot; -\ncan be enabled and disabled. These would include sheets\nwith the title attribute, as well as the REL attribute.</p>\n"},"insertCssRule":{"!type":"fn(selector: ?, declaration: ?, styleSheetName?: ?) -> string","!doc":"<p>Creates a style and attaches it to a dynamically created stylesheet</p>\n"},"modifyCssRule":{"!type":"fn(selector: ?, declaration: ?, styleSheetName: ?)","!doc":"<p>Not implemented - it seems to have some merit for changing some complex\nselectors. It&#39;s not much use for changing simple ones like &quot;span&quot;.\nFor now, simply write a new rule which will cascade over the first.\n\n</p>\n<p>Modifies an existing cssRule</p>\n"},"removeCssRule":{"!type":"fn(selector: ?, declaration: ?, styleSheetName: ?)","!doc":"<p>Removes a cssRule base on the selector and declaration passed\nThe declaration is needed for cases of dupe selectors</p>\n"}},"dojox/html/styles._ContentSetter":{"!type":"fn()","prototype":{"adjustPaths":{"!doc":"<p>Adjust relative paths in html string content to point to this page\nOnly useful if you grab content from a another folder than the current one</p>\n"},"cleanContent":{"!doc":"<p>Should the content be treated as a full html document,\nand the real content stripped of <html>, <body> wrapper before injection</p>\n"},"content":{"!doc":"<p>The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes</p>\n"},"executeScripts":{},"extractContent":{"!doc":"<p>Should the content be treated as a full html document,\nand the real content stripped of <code>&lt;html&gt; &lt;body&gt;</code> wrapper before injection</p>\n"},"id":{"!doc":"<p>Usually only used internally, and auto-generated with each instance</p>\n"},"node":{"!doc":"<p>An node which will be the parent element that we set content into</p>\n"},"parseContent":{"!doc":"<p>Should the node by passed to the parser after the new content is set</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"referencePath":{},"renderStyles":{},"scriptHasHooks":{},"scriptHookReplacement":{},"startup":{"!doc":"<p>Start the child widgets after parsing them.   Only obeyed if parseContent is true.</p>\n"},"constructor":{"!type":"fn(params: ?, node: ?)","!doc":"<p>Provides a configurable, extensible object to wrap the setting on content on a node\ncall the set() method to actually set the content..</p>\n"},"empty":{"!type":"fn()"},"set":{"!type":"fn(cont?: ?, params?: ?) -> ?","!doc":"<p>front-end to the set-content sequence</p>\n"},"setContent":{"!type":"fn()","!doc":"<p>sets the content on the node</p>\n"},"tearDown":{"!type":"fn()"}}},"dojox/html/styles.metrics":{"_fontResizeNode":{},"getCachedFontMeasurements":{"!type":"fn(recalculate: ?) -> ?"},"getFontMeasurements":{"!type":"fn() -> ?","!doc":"<p>Returns an object that has pixel equivilents of standard font size values.</p>\n"},"getScrollbar":{"!type":"fn() -> ?"},"getTextBox":{"!type":"fn(text: ?, style: ?, className?: ?)"},"initOnFontResize":{"!type":"fn()"}},"dojox/html/styles.ext-dojo":{"style":{}},"dojox/html/styles.entities":{"html":{},"latin":{},"decode":{"!type":"fn(str: ?, m: ?)","!doc":"<p>Function to obtain an entity encoding for a specified character</p>\n"},"encode":{"!type":"fn(str: ?, m: ?)","!doc":"<p>Function to obtain an entity encoding for a specified character</p>\n"}},"dojox/editor/plugins/NormalizeIndentOutdent":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides improved indent and outdent handling to\nthe editor.  It tries to generate valid HTML, as well as be\nconsistent about how it indents and outdents lists and blocks/elements.</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"indentBy":{"!doc":"<p>The amount to indent by.  Valid values are 1+.  This is combined with\nthe indentUnits parameter to determine how much to indent or outdent\nby for regular text.  It does not affect lists.</p>\n"},"indentUnits":{"!doc":"<p>The units to apply to the indent amount.  Usually &#39;px&#39;, but can also\nbe em.</p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dojox/editor/plugins/NormalizeStyle":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides NormalizeStyle capability to the editor.  It is\na headless plugin that tries to normalize how content is styled when\nit comes out of th editor (&#39;b&#39; or css).   It also auto-converts\nincoming content to the proper one expected by the browser as well so\nthat the native styling buttons work.</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"condenseSpans":{"!doc":"<p>A boolean variable indicating if it should try to condense\n&#39;span&#39;&#39;span&#39;&#39;span&#39; styles  when in css mode\nThe default is true, it will try to combine where it can.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"mode":{"!doc":"<p>A String variable indicating if it should use semantic tags &#39;b&#39;, &#39;i&#39;, etc, or\nCSS styling.  The default is semantic.</p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dojox/editor/plugins/PageBreak":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides a simple CSS page break plugin that\nlets your insert browser print recognizable page breaks in\nthe document.\nThis plugin registers the hotkey command: CTRL-SHIFT-ENTER</p>\n","_pbContent":{"!doc":"<p>The markup used for the pagebreak insert.</p>\n"},"_unbreakableNodes":{"!doc":"<p>The nodes that should not allow page breaks to be inserted into them.</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from\n<code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/editor/plugins/PasteFromWord":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides PasteFromWord capability to the editor.  When\nclicked, a dialog opens with a spartan RichText instance to paste\nword content into via the keyboard commands.  The contents are\nthen filtered to remove word style classes and other meta-junk\nthat tends to cause issues.</p>\n","_filters":{"!doc":"<p>The filters is an array of regular expressions to try and strip out a lot\nof style data MS Word likes to insert when pasting into a contentEditable.\nPrettymuch all of it is junk and not good html.  The hander is a place to put a function\nfor match handling.  In most cases, it just handles it as empty string.  But the option is\nthere for more complex handling.</p>\n"},"_template":{},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"height":{"!doc":"<p>The height to use for the rich text area in the copy/pate dialog, in px.  Default is 300px.</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code>\nand <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"width":{"!doc":"<p>The width to use for the rich text area in the copy/pate dialog, in px.  Default is 400px.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/html/format":{"prettyPrint":{"!type":"fn(html: ?, indentBy: ?, maxLineLength: ?, map: ?, xhtml: ?)","!doc":"<p>Function for providing a &#39;pretty print&#39; version of HTML content from\nthe provided string.  It&#39;s nor perfect by any means, but it does\na &#39;reasonable job&#39;.</p>\n"}},"dojox/editor/plugins/PrettyPrint":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides a mechanism by which to &#39;beautify HTML&#39;\ngenerated by the editor.  It is by no means perfect.</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"entityMap":{},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"indentBy":{"!doc":"<p>The indentBy property configures if the plugin should use a\nset number of spaces for indent (between 1-5), or just tab.\nThe default value is -1, which means tab.</p>\n"},"lineLength":{"!doc":"<p>The lineLength property configures if the plugin should break up long\ntext lines into N lines of &#39;lineLength&#39; length.  This parameter does not\ntake into account indention depth, only text line length.  The default is -1\nwhich means unlimited line length.</p>\n"},"useDefaultCommand":{},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride to take over getValue of editor so that\nwe can &#39;pretty&#39; the output.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Over-ride to do nothing.\nWe don&#39;t want to append a button, we take over getValue.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dojox/editor/plugins/Preview":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides Preview capability to the editor.  When\nclicked, the document in the editor frame will displayed in a separate\nwindow/tab</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node icon.</p>\n"},"styles":{"!doc":"<p>A string of CSS styles to apply to the previewed content, if any.</p>\n"},"stylesheets":{"!doc":"<p>An array of stylesheets to import into the preview, if any.</p>\n"},"useDefaultCommand":{},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/editor/plugins/SafePaste":{"!type":"fn()","prototype":{"!proto":"dojox/editor/plugins/PasteFromWord.prototype","!doc":"<p>This plugin extends from the PasteFromWord plugin and provides\n&#39;safe pasting&#39;, meaning that it will not allow keyboard/menu pasting\ninto the dijit editor.  It still runs all of the word cleanup code, \nincluding script strippers.  If you use this plugin, you don&#39;t need to \nuse the &#39;PasteFromWord Plugin&#39;</p>\n","_filters":{"!doc":"<p>The filters is an array of regular expressions to try and strip out a lot\nof style data MS Word likes to insert when pasting into a contentEditable.\nPrettymuch all of it is junk and not good html.  The hander is a place to put a function\nfor match handling.  In most cases, it just handles it as empty string.  But the option is\nthere for more complex handling.</p>\n"},"_template":{},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"height":{"!doc":"<p>The height to use for the rich text area in the copy/pate dialog, in px.  Default is 300px.</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code>\nand <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"width":{"!doc":"<p>The width to use for the rich text area in the copy/pate dialog, in px.  Default is 400px.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Overrides _Plugin.updateState(). </p>\n"}}},"dojox/editor/plugins/ShowBlockNodes":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides ShowBlockNodes capability to the editor.  When\nclicked, the document in the editor will apply a class to specific\nblock nodes to make them visible in the layout.  This info is not\nexposed/extracted when the editor value is obtained, it is purely for help\nwhile working on the page.</p>\n","_styled":{"!doc":"<p>Flag indicating the document has had the style updates applied.</p>\n"},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"toggle":{"!type":"fn()","!doc":"<p>Function to allow programmatic toggling of the view.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/editor/plugins/Smiley":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin allows the user to select from emoticons or &quot;smileys&quot;\nto insert at the current cursor position.</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"emoticonImageClass":{},"emoticonMarker":{"!doc":"<p>a marker for emoticon wrap like [:-)] for regexp convienent\nwhen a message which contains an emoticon stored in a database or view source, this marker include also\nbut when user enter an emoticon by key board, user don&#39;t need to enter this marker.\nalso emoticon definition character set can not contains this marker</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/editor/plugins/_SmileyPalette":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A keyboard accessible emoticon-picking widget (for inserting smiley characters)</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_currentFocus":{"!doc":"<p>The currently focused cell (if the palette itself has focus), or otherwise\nthe cell to be focused when the palette itself gets focus.\nDifferent from value, which represents the selected (i.e. clicked) cell.</p>\n"},"_focusManager":{},"_palette":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selectedCell":{"!doc":"<p>Index of the currently selected cell. Initially, none selected</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_xDim":{"!doc":"<p>This is the number of cells horizontally across.</p>\n"},"_yDim":{"!doc":"<p>This is the number of cells vertically down.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"cellClass":{"!doc":"<p>CSS class applied to each cell in the palette</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"defaultTimeout":{"!doc":"<p>Number of milliseconds before a held key or button becomes typematic</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Widget tab index.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template of this widget.</p>\n"},"timeoutChangeRate":{"!doc":"<p>Fraction of time used to change the typematic timer between events\n1.0 means that each typematic event fires at defaultTimeout intervals\nLess than 1.0 means that each typematic event fires at an increasing faster rate</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>Currently selected color/emoticon/etc.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dyeClass":{"!type":"fn()"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus this widget.  Puts focus on the most recently focused cell.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"Emoticon":{"!type":"fn()"}},"dojox/editor/plugins/_SmileyPalette.Emoticon":{"!type":"fn()","prototype":{"!doc":"<p>JS Object representing an emoticon</p>\n","constructor":{"!type":"fn(id: ?)","!doc":"<p>Create emoticon object from an id (like &quot;smile&quot;)</p>\n"},"fillCell":{"!type":"fn(cell: ?, blankGif: ?)"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Returns a emoticon string in ascii representation, ex: :-)</p>\n"},"imgHtml":{"!type":"fn(clazz: ?) -> ?","!doc":"<p>Return the HTML string for an <code>&lt;img&gt;</code> node that shows this smiley</p>\n"}},"ascii":{},"fromAscii":{"!type":"fn(str: ?)","!doc":"<p>Factory to create Emoticon object based on string like &quot;:-)&quot; rather than id like &quot;smile&quot;</p>\n"}},"dojox/editor/plugins/StatusBar":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides StatusBar capability to the editor.\nBasically a footer bar where status can be published.  It also\nputs a resize handle on the status bar, allowing you to resize the\neditor via mouse.</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"resizer":{"!doc":"<p>Flag indicating that a resizer should be shown or not.  Default is true.\nThere are cases (such as using center pane border container to autoresize the editor\nThat a resizer is not valued.</p>\n"},"statusBar":{"!doc":"<p>The status bar and resizer.</p>\n"},"useDefaultCommand":{"!doc":"<p>If true, this plugin executes by calling Editor.execCommand() with the argument specified in <code>command</code>.</p>\n"},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Over-ride to clean up the breadcrumb toolbar.</p>\n"},"get":{"!type":"fn(attr: ?) -> ?","!doc":"<p>Quick and dirty implementation of &#39;get&#39; pattern</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(attr: ?, val: ?)","!doc":"<p>Quick and dirty implementation of &#39;set&#39; pattern</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Over-ride for the setting of the editor.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Change state of the plugin to respond to events in the editor.</p>\n"}}},"dojox/editor/plugins/StatusBar._StatusBar":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>Template for the widget.  Currently using table to get the alignment behavior and\nbordering I wanted.  Would prefer not to use table, though.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/TablePlugins":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>A collection of Plugins for inserting and modifying tables in the Editor\nSee end of this document for all available plugs\nand dojox/editorPlugins/tests/editorTablePlugs.html for an example\n\n</p>\n<p>NOT IMPLEMENTED: Not handling cell merge, span or split\n</p>\n","alwaysAvailable":{},"button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"commandName":{},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{},"label":{},"undoEnabled":{},"useDefaultCommand":{},"begEdit":{"!type":"fn()"},"buttonClass":{"!type":"fn()"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Over-ridden destroy to do some cleanup.</p>\n"},"endEdit":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"getSelectedCells":{"!type":"fn() -> +any","!doc":"<p>Gets the selected cells from the passed table.</p>\n"},"getTableInfo":{"!type":"fn(forceNewData: ?) -> ?","!doc":"<p>Gets the table in focus\nCollects info on the table - see return params\n</p>\n"},"makeColumnsEven":{"!type":"fn()","!doc":"<p>After changing column amount, change widths to\nkeep columns even</p>\n"},"modTable":{"!type":"fn(cmd: ?, args: ?)","!doc":"<p>Where each plugin performs its action.\nNote: not using execCommand. In spite of their presence in the\nEditor as query-able plugins, I was not able to find any evidence\nthat they are supported (especially in NOT IE). If they are\nsupported in other browsers, it may help with the undo problem.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"selectTable":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/widget/ColorPicker":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormWidget.prototype","!doc":"<p>a HSV color picker - similar to Photoshop picker</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_huePickerPointer":{},"_huePickerPointerAlly":{},"_hueUnderlay":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_pickerPointer":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_underlay":{},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"animatePoint":{"!doc":"<p>toggle to use slideTo (true) or just place the cursor (false) on click</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"liveUpdate":{"!doc":"<p>Set to true to fire onChange in an indeterminate way</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"PICKER_HUE_H":{"!doc":"<p>Height of the hue picker, used to calculate positions</p>\n"},"PICKER_HUE_SELECTOR_H":{"!doc":"<p>Height of the hue selector DOM node, used to calc offsets so that selection\nis center of the image node.</p>\n"},"PICKER_SAT_SELECTOR_H":{"!doc":"<p>Height of the saturation selector DOM node, used to calc offsets so that selection\nis center of the image node.</p>\n"},"PICKER_SAT_SELECTOR_W":{"!doc":"<p>Width of the saturation selector DOM node, used to calc offsets so that selection\nis center of the image node.</p>\n"},"PICKER_SAT_VAL_H":{"!doc":"<p>Height of the 2d picker, used to calculate positions</p>\n"},"PICKER_SAT_VAL_W":{"!doc":"<p>Width of the 2d picker, used to calculate positions</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showHex":{"!doc":"<p>show/update Hex value field</p>\n"},"showHsv":{"!doc":"<p>show/update HSV input nodes</p>\n"},"showRgb":{"!doc":"<p>show/update RGB input nodes</p>\n"},"slideDuration":{"!doc":"<p>time in ms picker node will slide to next location (non-dragging) when animatePoint=true</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Default color for this component. Only hex values are accepted as incoming/returned\nvalues. Adjust this value with <code>.attr</code>, eg: dijit.byId(&quot;myPicker&quot;).attr(&quot;value&quot;, &quot;#ededed&quot;);\nto cause the points to adjust and the values to reflect the current color.</p>\n"},"webSafe":{"!doc":"<p>deprecated? or just use a toggle to show/hide that node, too?</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Over-ride to clean up subscriptions, etc.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget, only if focus isn&#39;t set on it already.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>As quickly as we can, set up ie6 alpha-filter support for our\nunderlay.  we don&#39;t do image handles (done in css), just the &#39;core&#39;\nof this widget: the underlay.</p>\n"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setColor":{"!type":"fn(col: ?, force: ?)","!doc":"<p>Set a color on a picker. Usually used to set\ninitial color as an alternative to passing defaultColor option\nto the constructor.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/TextColor":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>This plugin provides dropdown color pickers for setting text color and background color\nand makes use of the nicer-looking (though not entirely accessible), dojox.widget.ColorPicker.\n</p>\n","button":{"!doc":"<p>Pointer to <code>dijit/form/Button</code> or other widget (ex: <code>dijit/form/FilteringSelect</code>)\nthat is added to the toolbar to control this plugin.\nIf not specified, will be created on initialization according to <code>buttonClass</code></p>\n"},"buttonClass":{},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{"!doc":"<p>The CSS class name for the button node is formed from <code>iconClassPrefix</code> and <code>command</code></p>\n"},"useDefaultCommand":{},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Over-ride cleanup function.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)","!doc":"<p>Tell the plugin to add it&#39;s controller widget (often a button)\nto the toolbar.  Does nothing if there is no controller widget.</p>\n"},"updateState":{"!type":"fn()","!doc":"<p>Overrides _Plugin.updateState().  This updates the ColorPalette\nto show the color of the currently selected text.</p>\n"}}},"dojox/editor/plugins/TextColor._TextColorDropDown":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A sample widget that uses/creates a dropdown with a dojox.widget.ColorPicker.  Also provides\npassthroughs to the value of the color picker and convenient hook points.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template used to create the ColorPicker.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Flag denoting widgets are contained in the template.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Constructor over-ride so that the translated strings are mixsed in so\nthe template fills out.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Over-ride of startup to do the basic connect setups and such.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/editor/plugins/UploadImage":{"!type":"fn()","prototype":{"!proto":"dijit/_editor/_Plugin.prototype","!doc":"<p>Adds an icon to the Editor toolbar that when clicked, opens a system dialog\nAlthough the toolbar icon is a tiny &quot;image&quot; the uploader could be used for\nany file type</p>\n","button":{},"command":{"!doc":"<p>String like &quot;insertUnorderedList&quot;, &quot;outdent&quot;, &quot;justifyCenter&quot;, etc. that represents an editor command.\nPassed to editor.execCommand() if <code>useDefaultCommand</code> is true.</p>\n"},"disabled":{"!doc":"<p>Flag to indicate if this plugin has been disabled and should do nothing\nhelps control button state, among other things.  Set via the setter api.</p>\n"},"editor":{"!doc":"<p>Points to the parent editor</p>\n"},"iconClassPrefix":{},"label":{},"tempImageUrl":{},"uploadUrl":{},"useDefaultCommand":{},"buttonClass":{"!type":"fn()","!doc":"<p>Class of widget (ex: dijit.form.Button or dijit/form/FilteringSelect)\nthat is added to the toolbar to control this plugin.\nThis is used to instantiate the button, unless <code>button</code> itself is specified directly.</p>\n"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)","!doc":"<p>Deprecated.  Use this.own() with dojo/on or dojo/aspect.instead.\n\n</p>\n<p>Make a connect.connect() that is automatically disconnected when this plugin is destroyed.\nSimilar to <code>dijit/_Widget.connect()</code>.</p>\n"},"constructor":{"!type":"fn(args?: ?)","!doc":"<p>Create the plugin.</p>\n"},"createFileInput":{"!type":"fn()"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a plugin.</p>\n"},"insertTempImage":{"!type":"fn()","!doc":"<p>inserting a &quot;busy&quot; image to show something is hapening\nduring upload and download of the image.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a plugin</p>\n"},"setEditor":{"!type":"fn(editor: ?)","!doc":"<p>Tell the plugin which Editor it is associated with.</p>\n"},"setToolbar":{"!type":"fn(toolbar: ?)"},"updateState":{"!type":"fn()","!doc":"<p>Over-ride for button state control for disabled to work.</p>\n"}}},"dojox/editor/plugins/_SpellCheckParser":{"!type":"fn()","prototype":{"lang":{},"getIndices":{"!type":"fn() -> +any","!doc":"<p>Get the indices of the words. They are in one-to-one correspondence</p>\n"},"parseIntoWords":{"!type":"fn(text: ?) -> +any","!doc":"<p>Parse the text into words</p>\n"}}},"dojox/embed/Object":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A widget you can use to embed either a Flash or Quicktime\nmovie.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>The height of the movie. If not provided, the height of this.domNode is used.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"movie":{"!doc":"<p>The eventual reference to the movie embedded.  If you are looking to script\ncontrol over the movie, you&#39;d access it this way.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"params":{"!doc":"<p>A property bag that is created postCreate.  Any additional attributes you\ndefine on your domNode will be collected and placed into this, which will\nthen be passed to the movie constructor.</p>\n"},"reFlash":{"!doc":"<p>Expression used on the src property to determine if this is Flash or Quicktime.</p>\n"},"reQtAudio":{"!doc":"<p>Expression used on the src property to determine if this is Flash or Quicktime.</p>\n"},"reQtMovie":{"!doc":"<p>Expression used on the src property to determine if this is Flash or Quicktime.</p>\n"},"src":{"!doc":"<p>The URL of the movie to embed.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"width":{"!doc":"<p>The width of the movie. If not provided, the width of this.domNode is used.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Constructs the movie and places it in the document.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/embed/Quicktime":{"!doc":"<p>Returns a reference to the HTMLObject/HTMLEmbed that is created to\nplace the movie in the document.  You can use this either with or\nwithout the new operator.  Note that with any other DOM manipulation,\nyou must wait until the document is finished loading before trying\nto use this.\n</p>\n","initialized":{},"prototype":{"constructor":{"!type":"fn(kwArgs: ?, node: ?) -> ?","!doc":"<p>Returns a reference to the HTMLObject/HTMLEmbed that is created to\nplace the movie in the document.  You can use this either with or\nwithout the new operator.  Note that with any other DOM manipulation,\nyou must wait until the document is finished loading before trying\nto use this.\n</p>\n"}}},"dojox/encoding/_base":{},"dojox/encoding/ascii85":{"decode":{"!type":"fn(input: ?)","!doc":"<p>decodes the input string back to array of numbers</p>\n"},"encode":{"!type":"fn(input: ?)","!doc":"<p>encodes input data in ascii85 string</p>\n"}},"dojox/encoding/base64":{"decode":{"!type":"fn(str: ?)","!doc":"<p>Convert a base64-encoded string to an array of bytes</p>\n"},"encode":{"!type":"fn(ba: ?)","!doc":"<p>Encode an array of bytes as a base64-encoded string</p>\n"}},"dojox/encoding/bits":{"InputStream":{"!type":"fn(buffer: ?, width: ?)"},"OutputStream":{"!type":"fn()"}},"dojox/encoding/compression/lzw":{"Decoder":{"!type":"fn(n: ?)"},"Encoder":{"!type":"fn(n: ?)"}},"dojox/encoding/compression/splay":{"prototype":{"constructor":{"!type":"fn(n: ?)"},"decode":{"!type":"fn(stream: ?) -> number"},"encode":{"!type":"fn(value: ?, stream: ?) -> ?"},"reset":{"!type":"fn()"},"splay":{"!type":"fn(i: ?)"}}},"dojox/encoding/crypto/Blowfish":{"!doc":"<p>Object for doing Blowfish encryption/decryption.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Object for doing Blowfish encryption/decryption.</p>\n"}}},"dojox/encoding/crypto/_base":{"Blowfish":{},"cipherModes":{"!doc":"<p>Enumeration for various cipher modes.</p>\n"},"outputTypes":{"!doc":"<p>Enumeration for input and output encodings.</p>\n"},"SimpleAES":{},"RSAKey":{"!type":"fn()"}},"dojox/encoding/crypto/_base.cipherModes":{"!doc":"<p>Enumeration for various cipher modes.</p>\n","CBC":{},"CFB":{},"CTR":{},"ECB":{},"OFB":{},"PCBC":{}},"dojox/encoding/crypto/_base.outputTypes":{"!doc":"<p>Enumeration for input and output encodings.</p>\n","Base64":{},"Hex":{},"Raw":{},"String":{}},"dojox/encoding/crypto/_base.RSAKey":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(rngf: ?)","!doc":"<p>&quot;empty&quot; RSA key constructor</p>\n"},"decrypt":{"!type":"fn(ctext: ?) -> +any","!doc":"<p>Return the PKCS#1 RSA decryption of &quot;ctext&quot;.</p>\n"},"encrypt":{"!type":"fn(text: ?) -> +null"},"generate":{"!type":"fn(B: ?, E: ?)","!doc":"<p>Generate a new random private key B bits long, using public expt E</p>\n"},"setPrivate":{"!type":"fn(N: ?, E: ?, D: ?)","!doc":"<p>Set the private key fields N, e, d and CRT params from hex strings</p>\n"},"setPrivateEx":{"!type":"fn(N: ?, E: ?, D: ?, P: ?, Q: ?, DP: ?, DQ: ?, C: ?)","!doc":"<p>Set the private key fields N, e, d and CRT params from hex strings</p>\n"},"setPublic":{"!type":"fn(N: ?, E: ?)","!doc":"<p>Set the public key fields N and e from hex strings</p>\n"}}},"dojox/encoding/crypto/RSAKey-ext":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(rngf: ?)","!doc":"<p>&quot;empty&quot; RSA key constructor</p>\n"},"decrypt":{"!type":"fn(ctext: ?) -> +any","!doc":"<p>Return the PKCS#1 RSA decryption of &quot;ctext&quot;.</p>\n"},"encrypt":{"!type":"fn(text: ?) -> +null"},"generate":{"!type":"fn(B: ?, E: ?)","!doc":"<p>Generate a new random private key B bits long, using public expt E</p>\n"},"setPrivate":{"!type":"fn(N: ?, E: ?, D: ?)","!doc":"<p>Set the private key fields N, e, d and CRT params from hex strings</p>\n"},"setPrivateEx":{"!type":"fn(N: ?, E: ?, D: ?, P: ?, Q: ?, DP: ?, DQ: ?, C: ?)","!doc":"<p>Set the private key fields N, e, d and CRT params from hex strings</p>\n"},"setPublic":{"!type":"fn(N: ?, E: ?)","!doc":"<p>Set the public key fields N and e from hex strings</p>\n"}}},"dojox/encoding/crypto/RSAKey":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(rngf: ?)","!doc":"<p>&quot;empty&quot; RSA key constructor</p>\n"},"decrypt":{"!type":"fn(ctext: ?) -> +any","!doc":"<p>Return the PKCS#1 RSA decryption of &quot;ctext&quot;.</p>\n"},"encrypt":{"!type":"fn(text: ?) -> +null"},"generate":{"!type":"fn(B: ?, E: ?)","!doc":"<p>Generate a new random private key B bits long, using public expt E</p>\n"},"setPrivate":{"!type":"fn(N: ?, E: ?, D: ?)","!doc":"<p>Set the private key fields N, e, d and CRT params from hex strings</p>\n"},"setPrivateEx":{"!type":"fn(N: ?, E: ?, D: ?, P: ?, Q: ?, DP: ?, DQ: ?, C: ?)","!doc":"<p>Set the private key fields N, e, d and CRT params from hex strings</p>\n"},"setPublic":{"!type":"fn(N: ?, E: ?)","!doc":"<p>Set the public key fields N and e from hex strings</p>\n"}}},"dojox/math/BigInteger":{"prototype":{"am":{"!type":"fn(i: ?, x: ?, w: ?, j: ?, c: ?, n: ?) -> number"},"constructor":{"!type":"fn(a: ?, b: ?, c: ?)"}}},"dojox/math/random/Simple":{"!type":"fn()","prototype":{"!doc":"<p>Super simple implementation of a random number generator,\nwhich relies on Math.random().</p>\n","destroy":{"!type":"fn()","!doc":"<p>Prepares the object for GC. (empty in this case)</p>\n"},"nextBytes":{"!type":"fn(byteArray: ?)","!doc":"<p>Fills in an array of bytes with random numbers</p>\n"}}},"dojox/math/BigInteger-ext":{"prototype":{"am":{"!type":"fn(i: ?, x: ?, w: ?, j: ?, c: ?, n: ?) -> number"},"constructor":{"!type":"fn(a: ?, b: ?, c: ?)"}}},"dojox/encoding/crypto/SimpleAES":{"!doc":"<p>SimpleAES, ported from dojox.sql, and done without the need for\na Google Gears worker pool.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>SimpleAES, ported from dojox.sql, and done without the need for\na Google Gears worker pool.</p>\n"}}},"dojox/encoding/digests/MD5":{"!doc":"<p>computes the digest of data, and returns the result according to type outputType</p>\n","prototype":{"constructor":{"!type":"fn(data: ?, outputType?: ?)","!doc":"<p>computes the digest of data, and returns the result according to type outputType</p>\n"}}},"dojox/encoding/digests/_base":{"outputTypes":{"!doc":"<p>Enumeration for input and output encodings.</p>\n"},"addWords":{"!type":"fn(a: ?, b: ?) -> number","!doc":"<p>add a pair of words together with rollover</p>\n"},"MD5":{"!type":"fn(data: ?, outputType?: ?)","!doc":"<p>computes the digest of data, and returns the result according to type outputType</p>\n"},"SHA1":{"!type":"fn(data: ?, outputType?: ?)","!doc":"<p>Computes the SHA1 digest of the data, and returns the result according to output type.</p>\n"},"stringToUtf8":{"!type":"fn(input: ?)"},"stringToWord":{"!type":"fn(s: ?) -> []","!doc":"<p>convert a string to a word array</p>\n"},"wordToBase64":{"!type":"fn(wa: ?)","!doc":"<p>convert an array of words to base64 encoding, should be more efficient\nthan using dojox.encoding.base64</p>\n"},"wordToHex":{"!type":"fn(wa: ?)","!doc":"<p>convert an array of words to a hex tab</p>\n"},"wordToString":{"!type":"fn(wa: ?)","!doc":"<p>convert an array of words to a string</p>\n"}},"dojox/encoding/digests/_base.outputTypes":{"!doc":"<p>Enumeration for input and output encodings.</p>\n","Base64":{},"Hex":{},"Raw":{},"String":{}},"dojox/encoding/digests/SHA1":{"!doc":"<p>Computes the SHA1 digest of the data, and returns the result according to output type.</p>\n","prototype":{"constructor":{"!type":"fn(data: ?, outputType?: ?)","!doc":"<p>Computes the SHA1 digest of the data, and returns the result according to output type.</p>\n"}}},"dojox/encoding/digests/SHA224":{"prototype":{"constructor":{"!type":"fn(data: ?, outputType?: ?) -> ?"}}},"dojox/encoding/digests/_sha-32":{"digest":{"!type":"fn(msg: ?, length: ?, hash: ?, depth: ?) -> ?"},"toBase64":{"!type":"fn(wa: ?) -> ?"},"toHex":{"!type":"fn(wa: ?) -> ?"},"toWord":{"!type":"fn(s: ?) -> ?"}},"dojox/encoding/digests/SHA256":{"prototype":{"constructor":{"!type":"fn(data: ?, outputType?: ?) -> ?"}}},"dojox/encoding/digests/SHA384":{"prototype":{"constructor":{"!type":"fn(data: ?, outputType?: ?) -> ?"}}},"dojox/encoding/digests/_sha-64":{"outputTypes":{},"digest":{"!type":"fn(msg: ?, length: ?, hash: ?, depth: ?) -> []"},"stringToUtf8":{"!type":"fn(s: ?) -> ?"},"toBase64":{"!type":"fn(wa: ?) -> ?"},"toHex":{"!type":"fn(wa: ?) -> ?"},"toWord":{"!type":"fn(s: ?) -> ?"}},"dojox/encoding/digests/SHA512":{"prototype":{"constructor":{"!type":"fn(data: ?, outputType?: ?) -> ?"}}},"dojox/encoding/easy64":{"decode":{"!type":"fn(input: ?)","!doc":"<p>decodes the input string back to array of numbers</p>\n"},"encode":{"!type":"fn(input: ?)","!doc":"<p>encodes input data in easy64 string</p>\n"}},"dojox/flash/_base":{},"dojox/flash":{"!doc":"<p>Deprecated.  Should require dojox/flash modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/form/BusyButton":{"!type":"fn()","prototype":{"!proto":"dijit/form/Button.prototype","!doc":"<p>BusyButton is a simple widget which provides implementing more \nuser friendly form submission.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_setValueAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"busyLabel":{"!doc":"<p>text while button is busy</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"isBusy":{},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"timeout":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"useIcon":{"!doc":"<p>use a busy icon</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"cancel":{"!type":"fn()","!doc":"<p>if no timeout is set or for other reason the user can put the button back\nto being idle</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeBusy":{"!type":"fn()","!doc":"<p>sets state from idle to busy</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"resetTimeout":{"!type":"fn(timeout: ?)","!doc":"<p>to reset existing timeout and setting a new timeout</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/CheckedMultiSelect":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormSelectWidget.prototype","!doc":"<p>Extends the core dijit MultiSelect to provide a &quot;checkbox&quot; selector</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_message":{"!doc":"<p>Currently displayed message</p>\n"},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>Drop down version or not</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the dojo.store items.\nIf <code>store</code> is set, labelAttr must be set too, unless store is an old-style\ndojo.data store rather than a new dojo/store.</p>\n"},"labelText":{"!doc":"<p>Label of the drop down button</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadChildrenOnOpen":{"!doc":"<p>By default loadChildren is called when the items are fetched from the\nstore.  This property allows delaying loadChildren (and the creation\nof the options/menuitems) until the user clicks the button to open the\ndropdown.</p>\n"},"multiple":{"!doc":"<p>Whether or not we are multi-valued</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"onFetch":{"!doc":"<p>A callback to do with an onFetch - but before any items are actually\niterated over (i.e. to filter even further what you want to add)</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by setStore().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior setStore completes.</p>\n"},"options":{"!doc":"<p>The set of options for our select item.  Roughly corresponds to\nthe html <code>&lt;option&gt;</code> tag.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query to use when fetching items from our store</p>\n"},"queryOptions":{"!doc":"<p>Query options to use when fetching from the store</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"required":{"!doc":"<p>User is required to check at least one item.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"sortByLabel":{"!doc":"<p>Flag to sort the options returned from a store by the label of\nthe store.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>A store to use for getting our list of options - rather than reading them\nfrom the <code>&lt;option&gt;</code> html tags.   Should support getIdentity().\nFor back-compat store can also be a dojo/data/api/Identity.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"addOption":{"!type":"fn(option: ?)","!doc":"<p>Adds an option or options to the end of the select.  If value\nof the option is empty or missing, a separator is created instead.\nPassing in an array of options will yield slightly better performance\nsince the children are only loaded once.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up our connections</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return an error message to show if appropriate</p>\n"},"getOptions":{"!type":"fn(valueOrIdx: ?) -> +any","!doc":"<p>Returns a given option (or options).</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"invertSelection":{"!type":"fn(onChange: ?)","!doc":"<p>Invert the selection</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the required items are selected.\nCan override with your own routine in a subclass.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>sets up our event handling that we need for functioning\nas a select</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeOption":{"!type":"fn(valueOrIdx: ?)","!doc":"<p>Removes the given option or options.  You can remove by string\n(in which case the value is removed), number (in which case the\nindex in the options array is removed), or select option (in\nwhich case, the select option with a matching value is removed).\nYou can also pass in an array of those values for a slightly\nbetter performance since the children are only loaded once.\nFor numeric option values, specify {value: number} as the argument.</p>\n"},"reset":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setStore":{"!type":"fn(store: ?, selectedValue?: ?, fetchArgs?: ?) -> ?","!doc":"<p>Sets the store you would like to use with this select widget.\nThe selected value is the value of the new store to set.  This\nfunction returns the original store, in case you want to reuse\nit or something.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Set the value to be the first, or the selected index</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn()"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateOption":{"!type":"fn(newOption: ?)","!doc":"<p>Updates the values of the given option.  The option to update\nis matched based on the value of the entered option.  Passing\nin an array of new options will yield better performance since\nthe children will only be loaded once.</p>\n"},"validate":{"!type":"fn(isFocused: ?) -> ?"},"validator":{"!type":"fn() -> bool","!doc":"<p>Overridable function used to validate that an item is selected if required =\ntrue.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/DateTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/_DateTimeTextBox.prototype","!doc":"<p>A validating, serializable, range-bound date text box with a popup calendar</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selector":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{},"baseClass":{},"class":{},"constraints":{"!doc":"<p>Despite the name, this parameter specifies both constraints on the input\n(including starting/ending dates/times allowed) as well as\nformatting options like whether the date is displayed in long (ex: December 25, 2005)\nor short (ex: 12/25/2005) format.  See <code>dijit/form/_DateTimeTextBox.__Constraints</code> for details.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownDefaultValue":{"!doc":"<p>The default value to focus in the popupClass widget when the textbox value is empty.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to display a down arrow button, to open the drop down list.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>The value of this widget as a JavaScript Date object.  Use get(&quot;value&quot;) / set(&quot;value&quot;, val) to manipulate.\nWhen passed to the parser in markup, must be specified according to <code>dojo/date/stamp.fromISOString()</code></p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Formats the value as a Date, according to specified locale (second argument)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn()"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Parses as string as a Date, according to constraints</p>\n"},"pattern":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a localized date</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"popupClass":{"!type":"fn()","!doc":"<p>The popup widget to use. In this case, a calendar with Day, Month and Year views.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> ?"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/Calendar":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarBase.prototype","!doc":"<p>The standard Calendar. It includes day and month/year views.\nNo visual effects are included.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parent":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/_CalendarBase":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>The Root class for all _Calendar extensions</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/_CalendarDay":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for the dojox.widget.Calendar which provides\nthe standard day-view. A single month is shown at a time.</p>\n","parent":{},"constructor":{"!type":"fn()"}}},"dojox/widget/_CalendarDayView":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarView.prototype","!doc":"<p>View class for the dojox/widget/Calendar.\nAdds a view showing every day of a single month to the calendar.\nThis should not be mixed in directly with dojox/widget._CalendarBase.\nInstead, use dojox/widget._CalendarDay</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"datePart":{"!doc":"<p>Specifies how much to increment the displayed date when the user\nclicks the array button to increment of decrement the view.</p>\n"},"dayWidth":{"!doc":"<p>Specifies the type of day name to display.  &quot;narrow&quot; causes just one letter to be shown.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"headerClass":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useHeader":{},"adjustDate":{"!type":"fn(date: ?, amount: ?) -> ?","!doc":"<p>Adds or subtracts values from a date.\nThe unit, e.g. &quot;day&quot;, &quot;month&quot; or &quot;year&quot;, is\nspecified in the &quot;datePart&quot; property of the\ncalendar view mixin.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"cloneClass":{"!type":"fn(clazz: ?, n: ?, before: ?)","!doc":"<p>Clones all nodes with the class &#39;clazz&#39; in a widget</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getHeader":{"!type":"fn() -> ?","!doc":"<p>Returns the header node of a view. If none exists,\nan empty DIV is created and returned.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Constructs the calendar view.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/_CalendarView":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Base implementation for all view mixins.\nAll calendar views should extend this widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"headerClass":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useHeader":{},"adjustDate":{"!type":"fn(date: ?, amount: ?) -> ?","!doc":"<p>Adds or subtracts values from a date.\nThe unit, e.g. &quot;day&quot;, &quot;month&quot; or &quot;year&quot;, is\nspecified in the &quot;datePart&quot; property of the\ncalendar view mixin.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"cloneClass":{"!type":"fn(clazz: ?, n: ?, before: ?)","!doc":"<p>Clones all nodes with the class &#39;clazz&#39; in a widget</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getHeader":{"!type":"fn() -> ?","!doc":"<p>Returns the header node of a view. If none exists,\nan empty DIV is created and returned.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/_CalendarMonthYear":{"!type":"fn()","prototype":{"!doc":"<p>Mixin class for adding a view listing all 12\nmonths of the year to the dojox/widget/_CalendarBase</p>\n","constructor":{"!type":"fn()","!doc":"<p>Adds a dojox/widget/_CalendarMonthView view to the calendar widget.</p>\n"}}},"dojox/widget/_CalendarMonthYearView":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarView.prototype","!doc":"<p>A Calendar view listing the 12 months of the year</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"datePart":{"!doc":"<p>Specifies how much to increment the displayed date when the user\nclicks the array button to increment of decrement the view.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedYears":{"!doc":"<p>The number of years to display at once.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"headerClass":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useHeader":{},"adjustDate":{"!type":"fn(date: ?, amount: ?) -> ?","!doc":"<p>Adds or subtracts values from a date.\nThe unit, e.g. &quot;day&quot;, &quot;month&quot; or &quot;year&quot;, is\nspecified in the &quot;datePart&quot; property of the\ncalendar view mixin.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"cloneClass":{"!type":"fn(clazz: ?, n: ?, before: ?)","!doc":"<p>Clones all nodes with the class &#39;clazz&#39; in a widget</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getHeader":{"!type":"fn() -> +null"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/DayTextBox":{"!type":"fn()","prototype":{"!proto":"dojox/form/DateTextBox.prototype","!doc":"<p>A validating, serializable, range-bound date text box with a popup calendar that contains just months.</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selector":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{},"baseClass":{},"class":{},"constraints":{"!doc":"<p>Despite the name, this parameter specifies both constraints on the input\n(including starting/ending dates/times allowed) as well as\nformatting options like whether the date is displayed in long (ex: December 25, 2005)\nor short (ex: 12/25/2005) format.  See <code>dijit/form/_DateTimeTextBox.__Constraints</code> for details.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownDefaultValue":{"!doc":"<p>The default value to focus in the popupClass widget when the textbox value is empty.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to display a down arrow button, to open the drop down list.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>The value of this widget as a JavaScript Date object.  Use get(&quot;value&quot;) / set(&quot;value&quot;, val) to manipulate.\nWhen passed to the parser in markup, must be specified according to <code>dojo/date/stamp.fromISOString()</code></p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?) -> ?"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn()"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(displayVal: ?) -> ?"},"pattern":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a localized date</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"popupClass":{"!type":"fn()","!doc":"<p>The popup widget to use. In this case, a calendar with just a Month view.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> ?"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?) -> bool"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/DailyCalendar":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarBase.prototype","!doc":"<p>A calendar with only a daily view.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parent":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/DropDownSelect":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormSelectWidget.prototype","_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_childrenLoaded":{"!doc":"<p>Whether or not our children have been loaded</p>\n"},"_focusManager":{},"_isLoaded":{"!doc":"<p>Whether or not we have been loaded</p>\n"},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_onContainerKeydown":{},"_onContainerKeypress":{},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"emptyLabel":{"!doc":"<p>What to display in an &quot;empty&quot; dropdown</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the dojo.store items.\nIf <code>store</code> is set, labelAttr must be set too, unless store is an old-style\ndojo.data store rather than a new dojo/store.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadChildrenOnOpen":{"!doc":"<p>By default loadChildren is called when the items are fetched from the\nstore.  This property allows delaying loadChildren (and the creation\nof the options/menuitems) until the user clicks the button to open the\ndropdown.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"message":{"!doc":"<p>Currently displayed error/prompt message</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"multiple":{"!doc":"<p>Whether or not we are multi-valued</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"onFetch":{"!doc":"<p>A callback to do with an onFetch - but before any items are actually\niterated over (i.e. to filter even further what you want to add)</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by setStore().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior setStore completes.</p>\n"},"options":{"!doc":"<p>The set of options for our select item.  Roughly corresponds to\nthe html <code>&lt;option&gt;</code> tag.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query to use when fetching items from our store</p>\n"},"queryOptions":{"!doc":"<p>Query options to use when fetching from the store</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"required":{"!doc":"<p>Can be true or false, default is false.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"sortByLabel":{"!doc":"<p>Flag to sort the options returned from a store by the label of\nthe store.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>&quot;Incomplete&quot; if this select is required but unset (i.e. blank value), &quot;&quot; otherwise</p>\n"},"store":{"!doc":"<p>A store to use for getting our list of options - rather than reading them\nfrom the <code>&lt;option&gt;</code> html tags.   Should support getIdentity().\nFor back-compat store can also be a dojo/data/api/Identity.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"addOption":{"!type":"fn(option: ?)","!doc":"<p>Adds an option or options to the end of the select.  If value\nof the option is empty or missing, a separator is created instead.\nPassing in an array of options will yield slightly better performance\nsince the children are only loaded once.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"childSelector":{"!type":"fn(node: ?) -> ?"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()"},"focusChild":{"!type":"fn(widget: ?)","!doc":"<p>Sets the value to the given option, used during search by letter.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getOptions":{"!type":"fn(valueOrIdx: ?) -> +any","!doc":"<p>Returns a given option (or options).</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> ?"},"isValid":{"!type":"fn(isFocused: ?) -> bool","!doc":"<p>Whether or not this is a valid value.  The only way a Select\ncan be invalid is when it&#39;s required but nothing is selected.</p>\n"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>populates the menu</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>stop mousemove from selecting text on IE to be consistent with other browsers</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>set the missing message</p>\n"},"removeOption":{"!type":"fn(valueOrIdx: ?)","!doc":"<p>Removes the given option or options.  You can remove by string\n(in which case the value is removed), number (in which case the\nindex in the options array is removed), or select option (in\nwhich case, the select option with a matching value is removed).\nYou can also pass in an array of those values for a slightly\nbetter performance since the children are only loaded once.\nFor numeric option values, specify {value: number} as the argument.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Overridden so that the state will be cleared.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setStore":{"!type":"fn(store: ?, selectedValue?: ?, fetchArgs?: ?) -> ?","!doc":"<p>Sets the store you would like to use with this select widget.\nThe selected value is the value of the new store to set.  This\nfunction returns the original store, in case you want to reuse\nit or something.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateOption":{"!type":"fn(newOption: ?)","!doc":"<p>Updates the values of the given option.  The option to update\nis matched based on the value of the entered option.  Passing\nin an array of new options will yield better performance since\nthe children will only be loaded once.</p>\n"},"validate":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Called by oninit, onblur, and onkeypress, and whenever required/disabled state changes</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/DropDownSelect._Menu":{"!type":"fn()","prototype":{"!proto":"dijit/DropDownMenu.prototype","!doc":"<p>An internally-used menu for dropdown that allows us a vertical scrollbar</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"activated":{"!doc":"<p>This Menu has been clicked (mouse or via space/arrow key) or opened as a submenu,\nso mere mouseover will open submenus.  Focusing a menu via TAB does NOT automatically make it active\nsince TAB is a navigation operation and not a selection one.\nFor Windows apps, pressing the ALT key focuses the menubar menus (similar to TAB navigation) but the\nmenu is not active (ie no dropdown) until an item is clicked.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoFocus":{},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parentMenu":{"!doc":"<p>pointer to menu that displayed me</p>\n"},"passivePopupDelay":{"!doc":"<p>For a passive (unclicked) Menu, number of milliseconds before hovering (without clicking) will cause\nthe popup to open.  Default is Infinity, meaning you need to click the menu to open it.</p>\n"},"popupDelay":{"!doc":"<p>After a menu has been activated (by clicking on it etc.), number of milliseconds before hovering\n(without clicking) another MenuItem causes that MenuItem&#39;s popup to automatically open.</p>\n"},"searchContainerNode":{},"selected":{"!doc":"<p>Currently selected (a.k.a. highlighted) MenuItem, or null if no MenuItem is selected.\nIf a submenu is open, will be set to MenuItem that displayed the submenu.   OTOH, if\nthis Menu is in passive mode (i.e. hasn&#39;t been clicked yet), will be null, because\n&quot;selected&quot; is not merely &quot;hovered&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"childSelector":{"!type":"fn(node: ?) -> ?","!doc":"<p>Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children\nlike MenuSeparator.  If subclass overrides to a string (ex: &quot;&gt; *&quot;), the subclass must require dojo/query.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Overridden so that the previously selected value will be focused instead of only the first item</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>stop mousemove from selecting text on IE to be consistent with other browsers</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"startupKeyNavChildren":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/DropDownStack":{"!doc":"<p>A dropdown-based select stack.</p>\n"},"dojox/form/_SelectStackMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mix this class in to a dijit.form._FormSelectWidget in order to\nprovide support for &quot;selectable&quot; multiforms.  The widget is pointed\nto a dijit.layout.StackContainer and will handle displaying and\nsubmitting the values of only the appropriate pane.\n\n</p>\n<p>The options for this widget will be automatically set - based on\nthe panes that are in the stack container.  The &quot;title&quot; attribute of\nthe pane will be used for the display of the option.  The &quot;id&quot; attribute\nof the pane will be used as the value of the option.  In order to\navoid running into unique ID constraint issues, a stackPrefix mechanism\nis provided.</p>\n","stackId":{"!doc":"<p>The id of the stack that this widget is supposed to control</p>\n"},"stackPrefix":{"!doc":"<p>A prefix to remove from our stack pane ids when setting our options.\nThis exists so that we won&#39;t run into unique ID constraints.  For\nexample, if stackPrefix is set to &quot;foo_&quot;, and there are three panes\nin our stack with ids of &quot;foo_a&quot;, &quot;foo_b&quot;, and &quot;foo_c&quot;, then the values\nof the options created for the stack controller widget will be &quot;a&quot;,\n&quot;b&quot;, and &quot;c&quot;.  This allows you to have multiple select stack widgets\nwith the same values - without having to have the panes require the\nsame ids.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?"},"destroy":{"!type":"fn()"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"}}},"dojox/form/FileInput":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormWidget.prototype","!doc":"<p>A styled input type=&quot;file&quot;\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"cancelText":{"!doc":"<p>the title of the &quot;Cancel&quot; button</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>the title text of the &quot;Browse&quot; button</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>ugh, this should be pulled from this.domNode</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn(e: ?)","!doc":"<p>on click of cancel button, since we can&#39;t clear the input because of\nsecurity reasons, we destroy it, and add a new one in it&#39;s place.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(label: ?, cssClass?: ?)","!doc":"<p>method to allow use to change button label</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>listen for changes on our real file input</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/FileInputAuto":{"!type":"fn()","prototype":{"!proto":"dojox/form/FileInput.prototype","!doc":"<p>An extension on FileInput providing background upload progress\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_sent":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"blurDelay":{"!doc":"<p>time in ms before an un-focused widget will wait before uploading the file to the url=&quot;&quot; specified\ndefault: 2 seconds</p>\n"},"cancelText":{"!doc":"<p>the title of the &quot;Cancel&quot; button</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>The time in ms to use as the generic timing mechanism for the animations\nset to 1 or 0 for &quot;immediate response&quot;</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>the title text of the &quot;Browse&quot; button</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>ugh, this should be pulled from this.domNode</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"triggerEvent":{"!doc":"<p>Event which triggers the upload. Defaults to onblur, sending the file selected\n&#39;blurDelay&#39; milliseconds after losing focus. Set to &quot;onchange&quot; with a low blurDelay\nto send files immediately after uploading.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uploadMessage":{"!doc":"<p>FIXME: i18n somehow?</p>\n"},"url":{"!doc":"<p>the URL where our background FileUpload will be sent</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn(e: ?)","!doc":"<p>accommodate our extra focusListeners</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(label: ?, cssClass?: ?)","!doc":"<p>method to allow use to change button label</p>\n"},"setMessage":{"!type":"fn(title: ?)","!doc":"<p>set the text of the progressbar</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>add our extra blur listeners</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/FileInputBlind":{"!type":"fn()","prototype":{"!proto":"dojox/form/FileInput.prototype","!doc":"<p>An extension on FileInput providing background upload progress\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_sent":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"blurDelay":{"!doc":"<p>time in ms before an un-focused widget will wait before uploading the file to the url=&quot;&quot; specified\ndefault: 2 seconds</p>\n"},"cancelText":{"!doc":"<p>the title of the &quot;Cancel&quot; button</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>The time in ms to use as the generic timing mechanism for the animations\nset to 1 or 0 for &quot;immediate response&quot;</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>the title text of the &quot;Browse&quot; button</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>ugh, this should be pulled from this.domNode</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"triggerEvent":{"!doc":"<p>Event which triggers the upload. Defaults to onblur, sending the file selected\n&#39;blurDelay&#39; milliseconds after losing focus. Set to &quot;onchange&quot; with a low blurDelay\nto send files immediately after uploading.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uploadMessage":{"!doc":"<p>FIXME: i18n somehow?</p>\n"},"url":{"!doc":"<p>the URL where our background FileUpload will be sent</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn(e: ?)","!doc":"<p>accommodate our extra focusListeners</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(label: ?, cssClass?: ?)","!doc":"<p>method to allow use to change button label</p>\n"},"setMessage":{"!type":"fn(title: ?)","!doc":"<p>set the text of the progressbar</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>add our extra blur listeners</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/FilePickerTextBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/ValidationTextBox.prototype","!doc":"<p>A validating text box tied to a file picker popup</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"class":{},"constraints":{"!doc":"<p>user-defined object needed to pass parameters to the validator functions</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"numPanes":{"!doc":"<p>The number of panes to display in our box (if we don&#39;t have any\nminPaneWidth specified by our constraints)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"searchDelay":{"!doc":"<p>Delay in milliseconds between when user types something and we start\nsearching based on that value</p>\n"},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"valueItem":{"!doc":"<p>The item, in our store, of the directory relating to our value</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return an error message to show if appropriate</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if value is valid.\nCan override with your own routine in a subclass.</p>\n"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn()"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Function to convert a formatted string to a value - we use\nit to verify that it <em>really</em> is a valid value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Called by oninit, onblur, and onkeypress.</p>\n"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/FilePicker":{"!type":"fn()","prototype":{"!proto":"dojox/widget/RollingList.prototype","!doc":"<p>a specialized version of RollingList that handles file information\nin a store</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"cancelButtonLabel":{"!doc":"<p>The string to use for the Cancel button - will use dijit&#39;s common\n&quot;Cancel&quot; string if not set</p>\n"},"childrenAttrs":{"!doc":"<p>one ore more attributes that holds children of a node</p>\n"},"class":{},"className":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"executeOnDblClick":{"!doc":"<p>Set to true if you want to call onExecute when an item is\ndouble-clicked, false if you want to call onExecute yourself. (mainly\nused for popups to control how they want to be handled)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"minPaneWidth":{"!doc":"<p>the minimum pane width (in px) for all child panes.  If they are narrower,\nthe width will be increased to this value.</p>\n"},"okButtonLabel":{"!doc":"<p>The string to use for the OK button - will use dijit&#39;s common &quot;OK&quot; string\nif not set</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parentAttr":{"!doc":"<p>the attribute to read for finding our parent directory</p>\n"},"pathAttr":{"!doc":"<p>the attribute to read for getting the full path of our file</p>\n"},"pathSeparator":{"!doc":"<p>Our file separator - it will be guessed if not set</p>\n"},"preloadItems":{"!doc":"<p>Set this to a sane number - since we expect to mostly be using the\ndojox.data.FileStore - which doesn&#39;t like loading lots of items\nall at once.</p>\n"},"query":{"!doc":"<p>a query to pass to the datastore.  This is only used if items are null</p>\n"},"queryOptions":{"!doc":"<p>query options to be passed to the datastore</p>\n"},"searchContainerNode":{},"selectDirectories":{"!doc":"<p>whether or not we allow selection of directories - that is, whether or\nour value can be set to a directory.</p>\n"},"selectFiles":{"!doc":"<p>whether or not we allow selection of files - that is, we will disable\nthe file entries.</p>\n"},"showButtons":{"!doc":"<p>if set to true, then buttons for &quot;OK&quot; and &quot;Cancel&quot; will be provided</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>the store we must use</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"topDir":{"!doc":"<p>The top directory string - it will be guessed if not set</p>\n"},"value":{"!doc":"<p>The value that has been selected</p>\n"},"widgetsInTemplate":{},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>adds a child to this rolling list - if passed an insertIndex,\nthen all children from that index on will be removed and destroyed\nbefore adding the child.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>sets the focus state of this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildItems":{"!type":"fn(item: ?) -> []"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getMenuItemForItem":{"!type":"fn(item: ?, parentPane: ?, children?: ?) -> +instance"},"getPaneForItem":{"!type":"fn(item: ?, parentPane: ?, children?: ?) -> +instance"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleKey":{"!type":"fn(e: ?) -> bool","!doc":"<p>handle the key for the given event - called by dropdown\nwidgets</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Mix in our labels, if they are not set</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(args: ?)"},"scrollIntoView":{"!type":"fn(childWidget: ?)","!doc":"<p>scrolls the given widget into view</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/RollingList":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>a rolling list that can be tied to a data store with children</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"cancelButtonLabel":{"!doc":"<p>The string to use for the Cancel button - will use dijit&#39;s common\n&quot;Cancel&quot; string if not set</p>\n"},"childrenAttrs":{"!doc":"<p>one ore more attributes that holds children of a node</p>\n"},"class":{},"className":{"!doc":"<p>an additional class (or space-separated classes) to add for our widget</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"executeOnDblClick":{"!doc":"<p>Set to true if you want to call onExecute when an item is\ndouble-clicked, false if you want to call onExecute yourself. (mainly\nused for popups to control how they want to be handled)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"minPaneWidth":{"!doc":"<p>the minimum pane width (in px) for all child panes.  If they are narrower,\nthe width will be increased to this value.</p>\n"},"okButtonLabel":{"!doc":"<p>The string to use for the OK button - will use dijit&#39;s common &quot;OK&quot; string\nif not set</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parentAttr":{"!doc":"<p>the attribute to read for finding our parent item (if any)</p>\n"},"preloadItems":{"!doc":"<p>if set to true, then onItems will be called only <em>after</em> all items have\nbeen loaded (ie store.isLoaded will return true for all of them).  If\nfalse, then no preloading will occur.  If set to an integer, preloading\nwill occur if the number of items is less than or equal to the value\nof the integer.  The onItems function will need to be aware of handling\nitems that may not be loaded</p>\n"},"query":{"!doc":"<p>a query to pass to the datastore.  This is only used if items are null</p>\n"},"queryOptions":{"!doc":"<p>query options to be passed to the datastore</p>\n"},"searchContainerNode":{},"showButtons":{"!doc":"<p>if set to true, then buttons for &quot;OK&quot; and &quot;Cancel&quot; will be provided</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>the store we must use</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The value that has been selected</p>\n"},"widgetsInTemplate":{},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>adds a child to this rolling list - if passed an insertIndex,\nthen all children from that index on will be removed and destroyed\nbefore adding the child.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>sets the focus state of this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildItems":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the child items for the given store item</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getMenuItemForItem":{"!type":"fn(item: ?, parentPane: ?, children?: ?) -> +instance","!doc":"<p>user overridable function to return a widget for the given item\nand its children.</p>\n"},"getPaneForItem":{"!type":"fn(item?: ?, parentPane?: ?, children?: ?) -> +instance","!doc":"<p>user-overridable function to return a pane that corresponds\nto the given item in the store.  It can return null to not add a new pane\n(ie, you are planning on doing something else with it in onItemClick)</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleKey":{"!type":"fn(e: ?) -> bool","!doc":"<p>handle the key for the given event - called by dropdown\nwidgets</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Mix in our labels, if they are not set</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(args: ?)"},"scrollIntoView":{"!type":"fn(childWidget: ?)","!doc":"<p>scrolls the given widget into view</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/ListInput":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormValueWidget.prototype","!doc":"<p>An automatic list maker</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_count":{"!doc":"<p>Count items</p>\n"},"_currentItem":{"!doc":"<p>Widget currently in edition</p>\n"},"_focusManager":{},"_input":{"!doc":"<p>Widget use for input box</p>\n"},"_items":{"!doc":"<p>Array of widget.\nContain all reference to _ListInputInputItem</p>\n"},"_lastAddedItem":{"!doc":"<p>Contain a reference to the last created item</p>\n"},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraints":{"!doc":"<p>user-defined object needed to pass parameters to the validator functions</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"delimiter":{"!doc":"<p>delimiter for the string. Every match will be splitted.\nThe string can contain only one delimiter.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>Animation duration</p>\n"},"easingIn":{"!doc":"<p>function used to easing on fadeIn end</p>\n"},"easingOut":{"!doc":"<p>function used to easing on fadeOut end</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inputClass":{"!doc":"<p>Class which will be used to create the input box. You can implements yours.\nIt must be a widget, focusNode or domNode must have &quot;onkeydown&quot; event\nIt must have .attr(&quot;value&quot;) to get value\nIt also must implement an (or more) handler for the &quot;onChange&quot; method</p>\n"},"inputHandler":{"!doc":"<p>The widget will connect on all handler to check input value\nYou can use comma separated list</p>\n"},"inputProperties":{"!doc":"<p>Properties used to create input box\nIf String, it must be a valid JSON</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maxItems":{"!doc":"<p>Specify max item the list can have\nnull = infinity</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"readOnlyInput":{"!doc":"<p>if false, the list will be editable\nCan only be set when instantiate</p>\n"},"readOnlyItem":{"!doc":"<p>If true, items can be edited\nCan only be set when instantiate</p>\n"},"regExp":{"!doc":"<p>regular expression string used to validate the input\nDo not specify both regExp and regExpGen</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showCloseButtonWhenInvalid":{"!doc":"<p>if true, a close button will be added on invalid item</p>\n"},"showCloseButtonWhenValid":{"!doc":"<p>if true, a close button will be added on valid item</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"submitOnlyValidValue":{"!doc":"<p>If true, only valid value will be submitted with form</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{},"useAnim":{"!doc":"<p>If true, then item will use an animation to show hide itself</p>\n"},"useArrowForEdit":{"!doc":"<p>If true, arraow left and right can be used for editing\nCan only be set when instantiate</p>\n"},"useOnBlur":{"!doc":"<p>If true, onBlur event do a validate (like pressing ENTER)</p>\n"},"value":{},"add":{"!type":"fn(values: ?)","!doc":"<p>Create new list element</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by attr(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all widget</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>If closeButton is used, add a class</p>\n"},"postMixInProperties":{"!type":"fn()"},"regExpGen":{"!type":"fn(constraints: ?) -> ?","!doc":"<p>Overridable function used to generate regExp when dependent on constraints.\nDo not specify both regExp and regExpGen.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/Manager":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>The widget to orchestrate dynamic forms.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"accept":{},"accept-charset":{},"action":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"encType":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"method":{},"name":{},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"target":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"watching":{},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addClass":{"!type":"fn(className: ?, names?: ?) -> fn","!doc":"<p>Add a class to nodes according to the supplied set of names</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disable":{"!type":"fn(state?: ?) -> ?","!doc":"<p>Disable form controls according to the supplied state object\nreturning the previous state.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"elementValue":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>Set or get a form widget/element or an attached point node by name.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"enable":{"!type":"fn(state?: ?, defaultState: ?) -> fn","!doc":"<p>Enable form controls according to the supplied state object.</p>\n"},"formNodeValue":{"!type":"fn(elem: ?, value?: ?) -> ?","!doc":"<p>Set or get a form element by name.</p>\n"},"formPointValue":{"!type":"fn(elem: ?, value?: ?) -> ?","!doc":"<p>Set or get a node context by name (using dojoAttachPoint).</p>\n"},"formWidgetValue":{"!type":"fn(elem: ?, value?: ?) -> ?","!doc":"<p>Set or get a form widget by name.</p>\n"},"gatherClassState":{"!type":"fn(className: ?, names?: ?) -> ?","!doc":"<p>Gather the presence of a certain class in all controlled elements.</p>\n"},"gatherDisplayState":{"!type":"fn(names?: ?) -> ?","!doc":"<p>Gather display state of all attached elements and return as a dictionary.</p>\n"},"gatherEnableState":{"!type":"fn(names?: ?) -> ?","!doc":"<p>Gather enable state of all form elements and return as a dictionary.</p>\n"},"gatherFormValues":{"!type":"fn(names?: ?) -> ?","!doc":"<p>Collect form values.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn(state?: ?) -> ?","!doc":"<p>Hide attached nodes according to the supplied state object.</p>\n"},"inspect":{"!type":"fn(inspector: ?, state?: ?, defaultValue?: ?) -> ?","!doc":"<p>Run an inspector function on controlled elements returning a result object.</p>\n"},"inspectAttachedPoints":{"!type":"fn(inspector: ?, state?: ?, defaultValue?: ?) -> ?","!doc":"<p>Run an inspector function on &quot;dojoAttachPoint&quot; nodes returning a result object.</p>\n"},"inspectFormNodes":{"!type":"fn(inspector: ?, state?: ?, defaultValue?: ?) -> ?","!doc":"<p>Run an inspector function on controlled form elements returning a result object.</p>\n"},"inspectFormWidgets":{"!type":"fn(inspector: ?, state?: ?, defaultValue?: ?) -> ?","!doc":"<p>Run an inspector function on controlled widgets returning a result object.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Make sure that every widget that has a validator function returns true.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"registerNode":{"!type":"fn(node: ?) -> ?","!doc":"<p>Register a node with the form manager</p>\n"},"registerNodeDescendants":{"!type":"fn(node: ?) -> ?","!doc":"<p>Register node&#39;s descendants (form nodes) with the form manager</p>\n"},"registerWidget":{"!type":"fn(widget: ?) -> ?","!doc":"<p>Register a widget with the form manager</p>\n"},"registerWidgetDescendants":{"!type":"fn(widget: ?) -> ?","!doc":"<p>Register widget&#39;s descendants with the form manager</p>\n"},"removeClass":{"!type":"fn(className: ?, names?: ?) -> fn","!doc":"<p>Remove a class from nodes according to the supplied set of names</p>\n"},"reset":{"!type":"fn() -> fn","!doc":"<p>Resets form widget values.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setFormValues":{"!type":"fn(values: ?) -> fn","!doc":"<p>Set values to form elements</p>\n"},"show":{"!type":"fn(state?: ?, defaultState?: ?) -> fn","!doc":"<p>Show attached nodes according to the supplied state object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"submit":{"!type":"fn()","!doc":"<p>programmatically submit form if and only if the <code>onSubmit</code> returns true</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unregisterNode":{"!type":"fn(name: ?) -> ?","!doc":"<p>Removes the node by name from internal tables unregistering\nconnected observers</p>\n"},"unregisterNodeDescendants":{"!type":"fn(node: ?) -> ?","!doc":"<p>Unregister node&#39;s descendants (form nodes) with the form manager</p>\n"},"unregisterWidget":{"!type":"fn(name: ?) -> ?","!doc":"<p>Removes the widget by name from internal tables unregistering\nconnected observers</p>\n"},"unregisterWidgetDescendants":{"!type":"fn(widget: ?) -> ?","!doc":"<p>Unregister widget&#39;s descendants with the form manager</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> bool","!doc":"<p>validate() returns if the form is valid - same as isValid - \nbut provides a few additional (ui-specific) features: it \nwill highlight any sub-widgets that are not valid it will \ncall focus() on the first invalid sub-widget</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/manager/_Mixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin to orchestrate dynamic forms.</p>\n","watching":{},"destroy":{"!type":"fn()","!doc":"<p>Called when the widget is being destroyed</p>\n"},"formPointValue":{"!type":"fn(elem: ?, value?: ?) -> ?","!doc":"<p>Set or get a node context by name (using dojoAttachPoint).</p>\n"},"formWidgetValue":{"!type":"fn(elem: ?, value?: ?) -> ?","!doc":"<p>Set or get a form widget by name.</p>\n"},"inspect":{"!type":"fn(inspector: ?, state?: ?, defaultValue?: ?) -> ?","!doc":"<p>Run an inspector function on controlled elements returning a result object.</p>\n"},"inspectAttachedPoints":{"!type":"fn(inspector: ?, state?: ?, defaultValue?: ?) -> ?","!doc":"<p>Run an inspector function on &quot;dojoAttachPoint&quot; nodes returning a result object.</p>\n"},"inspectFormWidgets":{"!type":"fn(inspector: ?, state?: ?, defaultValue?: ?) -> ?","!doc":"<p>Run an inspector function on controlled widgets returning a result object.</p>\n"},"registerWidget":{"!type":"fn(widget: ?) -> ?","!doc":"<p>Register a widget with the form manager</p>\n"},"registerWidgetDescendants":{"!type":"fn(widget: ?) -> ?","!doc":"<p>Register widget&#39;s descendants with the form manager</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Called after all the widgets have been instantiated and their\ndom nodes have been inserted somewhere under win.doc.body.</p>\n"},"unregisterWidget":{"!type":"fn(name: ?) -> ?","!doc":"<p>Removes the widget by name from internal tables unregistering\nconnected observers</p>\n"},"unregisterWidgetDescendants":{"!type":"fn(widget: ?) -> ?","!doc":"<p>Unregister widget&#39;s descendants with the form manager</p>\n"}}},"dojox/form/manager/_NodeMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin to orchestrate dynamic forms (works with DOM nodes).</p>\n","destroy":{"!type":"fn()","!doc":"<p>Called when the widget is being destroyed</p>\n"},"formNodeValue":{"!type":"fn(elem: ?, value?: ?) -> ?","!doc":"<p>Set or get a form element by name.</p>\n"},"inspectFormNodes":{"!type":"fn(inspector: ?, state?: ?, defaultValue?: ?) -> ?","!doc":"<p>Run an inspector function on controlled form elements returning a result object.</p>\n"},"registerNode":{"!type":"fn(node: ?) -> ?","!doc":"<p>Register a node with the form manager</p>\n"},"registerNodeDescendants":{"!type":"fn(node: ?) -> ?","!doc":"<p>Register node&#39;s descendants (form nodes) with the form manager</p>\n"},"unregisterNode":{"!type":"fn(name: ?) -> ?","!doc":"<p>Removes the node by name from internal tables unregistering\nconnected observers</p>\n"},"unregisterNodeDescendants":{"!type":"fn(node: ?) -> ?","!doc":"<p>Unregister node&#39;s descendants (form nodes) with the form manager</p>\n"}}},"dojox/form/manager/_FormMixin":{"!type":"fn()","prototype":{"!doc":"<p>Form manager&#39;s mixin for form-specific functionality.</p>\n","accept":{},"accept-charset":{},"action":{},"encType":{},"method":{},"name":{},"target":{},"isValid":{"!type":"fn() -> bool","!doc":"<p>Make sure that every widget that has a validator function returns true.</p>\n"},"reset":{"!type":"fn() -> fn","!doc":"<p>Resets form widget values.</p>\n"},"startup":{"!type":"fn()"},"submit":{"!type":"fn()","!doc":"<p>programmatically submit form if and only if the <code>onSubmit</code> returns true</p>\n"},"validate":{"!type":"fn() -> bool","!doc":"<p>validate() returns if the form is valid - same as isValid - \nbut provides a few additional (ui-specific) features: it \nwill highlight any sub-widgets that are not valid it will \ncall focus() on the first invalid sub-widget</p>\n"}}},"dojox/form/manager/_ValueMixin":{"!type":"fn()","prototype":{"!doc":"<p>Form manager&#39;s mixin for getting/setting form values in the unified manner.</p>\n","elementValue":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>Set or get a form widget/element or an attached point node by name.</p>\n"},"gatherFormValues":{"!type":"fn(names?: ?) -> ?","!doc":"<p>Collect form values.</p>\n"},"setFormValues":{"!type":"fn(values: ?) -> fn","!doc":"<p>Set values to form elements</p>\n"}}},"dojox/form/manager/_EnableMixin":{"!type":"fn()","prototype":{"!doc":"<p>Form manager&#39;s mixin for controlling enable/disable state of\nform elements.</p>\n","disable":{"!type":"fn(state?: ?) -> ?","!doc":"<p>Disable form controls according to the supplied state object\nreturning the previous state.</p>\n"},"enable":{"!type":"fn(state?: ?, defaultState: ?) -> fn","!doc":"<p>Enable form controls according to the supplied state object.</p>\n"},"gatherEnableState":{"!type":"fn(names?: ?) -> ?","!doc":"<p>Gather enable state of all form elements and return as a dictionary.</p>\n"}}},"dojox/form/manager/_DisplayMixin":{"!type":"fn()","prototype":{"!doc":"<p>Form manager&#39;s mixin for controlling show/hide state of\ncontrolled elements (defined by dojoAttachPoint attributes).</p>\n","gatherDisplayState":{"!type":"fn(names?: ?) -> ?","!doc":"<p>Gather display state of all attached elements and return as a dictionary.</p>\n"},"hide":{"!type":"fn(state?: ?) -> ?","!doc":"<p>Hide attached nodes according to the supplied state object.</p>\n"},"show":{"!type":"fn(state?: ?, defaultState?: ?) -> fn","!doc":"<p>Show attached nodes according to the supplied state object.</p>\n"}}},"dojox/form/manager/_ClassMixin":{"!type":"fn()","prototype":{"!doc":"<p>Form manager&#39;s mixin for testing/assigning/removing\nclasses of controlled elements.</p>\n","addClass":{"!type":"fn(className: ?, names?: ?) -> fn","!doc":"<p>Add a class to nodes according to the supplied set of names</p>\n"},"gatherClassState":{"!type":"fn(className: ?, names?: ?) -> ?","!doc":"<p>Gather the presence of a certain class in all controlled elements.</p>\n"},"removeClass":{"!type":"fn(className: ?, names?: ?) -> fn","!doc":"<p>Remove a class from nodes according to the supplied set of names</p>\n"}}},"dojox/form/MonthTextBox":{"!type":"fn()","prototype":{"!proto":"dojox/form/DateTextBox.prototype","!doc":"<p>A validating, serializable, range-bound date text box with a popup calendar that contains only months</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selector":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{},"baseClass":{},"class":{},"constraints":{"!doc":"<p>Despite the name, this parameter specifies both constraints on the input\n(including starting/ending dates/times allowed) as well as\nformatting options like whether the date is displayed in long (ex: December 25, 2005)\nor short (ex: 12/25/2005) format.  See <code>dijit/form/_DateTimeTextBox.__Constraints</code> for details.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownDefaultValue":{"!doc":"<p>The default value to focus in the popupClass widget when the textbox value is empty.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to display a down arrow button, to open the drop down list.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"selector":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>The value of this widget as a JavaScript Date object.  Use get(&quot;value&quot;) / set(&quot;value&quot;, val) to manipulate.\nWhen passed to the parser in markup, must be specified according to <code>dojo/date/stamp.fromISOString()</code></p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?) -> number"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn()"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> number"},"pattern":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a localized date</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"popupClass":{"!type":"fn()","!doc":"<p>The popup widget to use. In this case, a calendar with just a Month view.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(value: ?, constraints: ?) -> ?"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?) -> bool"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/MonthlyCalendar":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarBase.prototype","!doc":"<p>A calendar with only a month view.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/_CalendarMonth":{"!type":"fn()","prototype":{"!doc":"<p>Mixin class for adding a view listing all 12 months of the year to the\ndojox/widget/_CalendarBase</p>\n","constructor":{"!type":"fn()","!doc":"<p>Adds a dojox/widget/_CalendarMonthView view to the calendar widget.</p>\n"}}},"dojox/widget/_CalendarMonthView":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarView.prototype","!doc":"<p>A Calendar view listing the 12 months of the year</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"datePart":{"!doc":"<p>Specifies how much to increment the displayed date when the user\nclicks the array button to increment of decrement the view.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedYear":{"!doc":"<p>The current year being displayed </p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"headerClass":{"!doc":"<p>Specifies the CSS class to apply to the header node for this view.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useHeader":{},"adjustDate":{"!type":"fn(date: ?, amount: ?) -> ?","!doc":"<p>Adds or subtracts values from a date.\nThe unit, e.g. &quot;day&quot;, &quot;month&quot; or &quot;year&quot;, is\nspecified in the &quot;datePart&quot; property of the\ncalendar view mixin.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"cloneClass":{"!type":"fn(clazz: ?, n: ?, before: ?)","!doc":"<p>Clones all nodes with the class &#39;clazz&#39; in a widget</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getHeader":{"!type":"fn() -> ?","!doc":"<p>Returns the header node of a view. If none exists,\nan empty DIV is created and returned.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Constructs the view</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/MultiComboBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/ValidationTextBox.prototype","!doc":"<p>A ComboBox that accepts multiple inputs on a single line</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_previousMatches":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoComplete":{"!doc":"<p>If user types in a partial string, and then tab out of the <code>&lt;input&gt;</code> box,\nautomatically copy the first entry displayed in the drop down list to\nthe <code>&lt;input&gt;</code> field</p>\n"},"autoWidth":{"!doc":"<p>Set to true to make the drop down at least as wide as this\nwidget.  Set to false if the drop down should just be its\ndefault width.</p>\n"},"baseClass":{},"class":{},"constraints":{"!doc":"<p>user-defined object needed to pass parameters to the validator functions</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"delimiter":{"!doc":"<p>The character to use to separate items in the ComboBox input</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"fetchProperties":{"!doc":"<p>Mixin to the store&#39;s fetch.\nFor example, to set the sort order of the ComboBox menu, pass:\n\n</p>\n<pre><code>{ sort: [{attribute:&quot;name&quot;,descending: true}] }</code></pre>\n<p>To override the default queryOptions so that deep=false, do:\n\n\n</p>\n<pre><code>{ queryOptions: {ignoreCase: true, deep: false} }</code></pre>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to have a down arrow button, to display the drop down list.\nDefaults to true.</p>\n"},"highlightMatch":{"!doc":"<p>One of: &quot;first&quot;, &quot;all&quot; or &quot;none&quot;.\n\n</p>\n<p>If the ComboBox/FilteringSelect opens with the search results and the searched\nstring can be found, it will be highlighted.  If set to &quot;all&quot;\nthen will probably want to change <code>queryExpr</code> parameter to &#39;<em>${0}</em>&#39;\n\n</p>\n<p>Highlighting is only performed when <code>labelType</code> is &quot;text&quot;, so as to not\ninterfere with any HTML markup an HTML label might contain.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ignoreCase":{"!doc":"<p>Set true if the query should ignore case when matching possible items</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"item":{"!doc":"<p>This is the item returned by the dojo/store/api/Store implementation that\nprovides the data for this ComboBox, it&#39;s the currently selected item.</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the\ndojo.data items.\nIf not specified, the searchAttr attribute is used instead.</p>\n"},"labelType":{"!doc":"<p>Specifies how to interpret the labelAttr in the data store items.\nCan be &quot;html&quot; or &quot;text&quot;.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageSize":{"!doc":"<p>Argument to data provider.\nSpecifies maximum number of search results to return per query</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"query":{"!doc":"<p>A query that can be passed to <code>store</code> to initially filter the items.\nComboBox overwrites any reference to the <code>searchAttr</code> and sets it to the <code>queryExpr</code> with the user&#39;s input substituted.</p>\n"},"queryExpr":{"!doc":"<p>This specifies what query is sent to the data store,\nbased on what the user has typed.  Changing this expression will modify\nwhether the results are only exact matches, a &quot;starting with&quot; match,\netc.\n<code>${0}</code> will be substituted for the user text.\n<code>*</code> is used for wildcards.\n<code>${0}*</code> means &quot;starts with&quot;, <code>*${0}*</code> means &quot;contains&quot;, <code>${0}</code> means &quot;is&quot;</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchAttr":{"!doc":"<p>Search for items in the data store where this attribute (in the item)\nmatches what the user typed</p>\n"},"searchContainerNode":{},"searchDelay":{"!doc":"<p>Delay in milliseconds between when user types something and we start\nsearching based on that value</p>\n"},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this ComboBox.\n\n</p>\n<p>Should be dojo/store/api/Store, but dojo/data/api/Read supported\nfor backwards compatibility.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"doHighlight":{"!type":"fn(label: ?, find: ?) -> ?","!doc":"<p>Highlights the string entered by the user in the menu.  By default this\nhighlights the first occurrence found. Override this method\nto implement your custom highlighting.</p>\n"},"dropDownClass":{"!type":"fn()","!doc":"<p>Dropdown widget class used to select a date/time.\nSubclasses should specify this.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return an error message to show if appropriate</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if value is valid.\nCan override with your own routine in a subclass.</p>\n"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget.   To be called only when this.dropDown\nhas been created and is ready to display (ie, it&#39;s data is loaded).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Called by oninit, onblur, and onkeypress.</p>\n"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/PasswordValidator":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormValueWidget.prototype","!doc":"<p>A password validation widget that simplifies the &quot;old/new/verify&quot;\nstyle of requesting passwords.  You will probably want to override\nthis class and implement your own pwCheck function.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_hasBeenBlurred":{},"_inputWidgets":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"oldName":{"!doc":"<p>The name to send our old password as (when form is posted)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"required":{"!doc":"<p>Whether or not it is required for form submission</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>places focus on the first invalid input widget - if all\ninput widgets are valid, the first widget is focused.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>we are valid if ALL our children are valid</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Sets up the correct widgets.  You <em>MUST</em> specify one child\ntext box (a simple HTML <code>&lt;input&gt;</code> element) with pwType=&quot;new&quot;\n<em>and</em> one child text box with pwType=&quot;verify&quot;.  You <em>MAY</em>\nspecify a third child text box with pwType=&quot;old&quot; in order to\nprompt the user to enter in their old password before the\nwidget returns that it is valid.</p>\n"},"postMixInProperties":{"!type":"fn()"},"pwCheck":{"!type":"fn(password: ?) -> bool","!doc":"<p>Overridable function for validation of the old password box.\n\n</p>\n<p>This function is called and passed the old password.  Return\ntrue if it&#39;s OK to continue, and false if it is not.\n\n</p>\n<p>IMPORTANT SECURITY NOTE:  Do NOT EVER EVER EVER check this in\nHTML or JavaScript!!!\n\n</p>\n<p>You will probably want to override this function to callback\nto a server to verify the password (the callback will need to\nbe synchronous) - and it&#39;s probably a good idea to validate\nit again on form submission before actually doing\nanything destructive - that&#39;s why the &quot;oldName&quot; value\nis available.\n\n</p>\n<p>And don&#39;t just fetch the password from the server\neither :)  Send the test password (probably hashed, for\nsecurity) and return from the server a status instead.\n\n</p>\n<p>Again - DON&#39;T BE INSECURE!!!  Security is left as an exercise\nfor the reader :)</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resetting this widget resets all our children</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Validating this widget validates all our children</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/RadioStack":{"!doc":"<p>A radio-based select stack.</p>\n"},"dojox/form/RangeSlider":{"!type":"fn()","prototype":{"value":{},"destroy":{"!type":"fn()"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"}}},"dojox/fx":{"!doc":"<p>Deprecated.  Should require dojox/fx modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/fx/_base":{"!doc":"<p>Experimental and extended Animations beyond Dojo Core / Base functionality.\nProvides advanced Lines, Animations, and convenience aliases.</p>\n","_allowedProperties":{},"addClass":{"!type":"fn(node: ?, cssClass: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of adding a class to a node</p>\n"},"animateTimeline":{"!type":"fn(options: ?, node: ?)","!doc":"<p>An add-on to dojo.fx that provides the ability to create\na complex property animation based on an array of &quot;keyframes&quot;.</p>\n"},"flip":{"!type":"fn(args: ?)","!doc":"<p>Animate a node flipping following a specific direction</p>\n"},"flipCube":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a more 3d-like rotation</p>\n"},"flipGrid":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a decomposition in rows * cols flipping elements</p>\n"},"flipPage":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a page flip like animation.</p>\n"},"removeClass":{"!type":"fn(node: ?, cssClass: ?, args: ?) -> ?","!doc":"<p>Animate the effects of removing a class from a node</p>\n"},"smoothScroll":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation that will smooth-scroll to a node</p>\n"},"toggleClass":{"!type":"fn(node: ?, cssClass: ?, condition?: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of Toggling a class on a Node</p>\n"}},"dojox/form/Rating":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormWidget.prototype","!doc":"<p>A widget for rating using stars.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"numStars":{"!doc":"<p>The number of stars to show, default is 3.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"required":{"!doc":"<p>TODO: Can be true or false, default is false.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>The current value of the Rating</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn(params: ?)","!doc":"<p>Build the templateString. The number of stars is given by this.numStars,\nwhich is normally an attribute to the widget node.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(key: ?, value: ?)","!doc":"<p>Deprecated.   Use set(&quot;value&quot;, ...) instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/TimeSpinner":{"!type":"fn()","prototype":{"!proto":"dijit/form/_Spinner.prototype","!doc":"<p>Time Spinner</p>\n","__skipInputEvent":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_wheelTimer":{},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"defaultTimeout":{"!doc":"<p>Number of milliseconds before a held arrow key or up/down button becomes typematic</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"largeDelta":{},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"minimumTimeout":{"!doc":"<p>minimum number of milliseconds that typematic event fires when held key or button is held</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pattern":{"!doc":"<p>This defines the regular expression used to validate the input.\nDo not add leading ^ or $ characters since the widget adds these.\nA function may be used to generate a valid pattern when dependent on constraints or other runtime factors.\nset(&#39;pattern&#39;, String|Function).</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"smallDelta":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"timeoutChangeRate":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{},"adjust":{"!type":"fn(val: ?, delta: ?) -> ?"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(time: ?, locale: ?) -> ?"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn() -> bool"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(time: ?, locale: ?) -> ?"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(dateObject: ?, options?: ?) -> ?","!doc":"<p>Format a Date object as a string according a subset of the ISO-8601 standard\n</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?, constraints: ?) -> ?","!doc":"<p>Overridable function used to validate the text input against the regular expression.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/TriStateCheckBox":{"!type":"fn()","prototype":{"!proto":"dijit/form/Button.prototype","!doc":"<p>Checkbox with three states</p>\n","_aria_attr":{},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_currentState":{"!doc":"<p>The current state of the TriStateCheckBox</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stateLabels":{"!doc":"<p>These characters are used to replace the image to show\ncurrent state of TriStateCheckBox in high contrast mode. This is an associate array of\nstates with their corresponding replacing characters. State can either be &quot;False&quot;, &quot;True&quot; or &quot;Mixed&quot;.</p>\n"},"_stateType":{"!doc":"<p>The current state type of the TriStateCheckBox\nCould be &quot;False&quot;, &quot;True&quot; or &quot;Mixed&quot;</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"checked":{"!doc":"<p>Current check state of the check box.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>Content to display in button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"states":{"!doc":"<p>States of TriStateCheckBox.\nThe value of This.checked should be one of these three states:\n[false, true, &quot;mixed&quot;]</p>\n"},"stateValue":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>type attribute on <code>&lt;input&gt;</code> node.\nOverrides <code>dijit/form/Button.type</code>.  Users should not change this value.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"click":{"!type":"fn()","!doc":"<p>Emulate a click on the check box, but will not trigger the\nonClick method.</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"fixState":{"!type":"fn()","!doc":"<p>Fix _currentState property if it&#39;s out of bound.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setChecked":{"!type":"fn(checked: ?)","!doc":"<p>Deprecated.  Use set(&#39;checked&#39;, true/false) instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/Uploader":{"!type":"fn()","prototype":{"!proto":"dojox/form/uploader/_Base.prototype","!doc":"<p>A widget that creates a stylable file-input button, with optional multi-file selection,\nusing only HTML elements. Non-HTML5 browsers have fallback options of Flash or an iframe.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_nameIndex":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"deferredUploading":{"!doc":"<p>(Flash only) throttles the upload to a certain amount of files at a time.\nBy default, Flash uploads file one at a time to the server, but in parallel.\nFirefox will try to queue all files at once, leading to problems. Set this\nto the amount to upload in parallel at a time.\nGenerally, 1 should work fine, but you can experiment with queuing more than\none at a time.\nThis is of course ignored if selectMultipleFiles equals false.</p>\n"},"devMode":{"!doc":"<p>Re-implemented. devMode increases the logging, adding style tracing from the SWF.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"errMsg":{},"flashFieldName":{"!doc":"<p>If set, this will be the name of the field of the flash uploaded files that the server\nis expecting. If not set, &quot;Flash&quot; is appended to the &quot;name&quot; property.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedClass":{"!doc":"<p>The class applied to the button when it is focused (via TAB key)</p>\n"},"force":{"!doc":"<p>options: form, html5, iframe, flash\nEmpty string defaults to html5 if available, and iframe if not.\nUse &quot;flash&quot; to always use Flash (and hopefully force the user to download the plugin\nif they don&#39;t have it).\nUse &quot;iframe&quot; to always use an iframe, and never flash nor html5. Sometimes preferred\nfor consistent results.\nUse &quot;form&quot; to not use ajax and post to a page.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode in button to make it display an icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"isDebug":{"!doc":"<p>If true, outputs traces from the SWF to console. What exactly gets passed\nis very relative, and depends upon what traces have been left in the DEFT SWF.</p>\n"},"label":{"!doc":"<p>The text used in the button that when clicked, opens a system Browse Dialog.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiple":{"!doc":"<p>If true and flash mode, multiple files may be selected from the dialog.</p>\n"},"name":{"!doc":"<p>The name attribute needs to end with square brackets: [] as this is the standard way\nof handling an attribute &quot;array&quot;. This requires a slightly different technique on the\nserver.</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"preventCache":{"!doc":"<p>If true, then flash request is sent with a value that changes with each request (timestamp)</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"serverTimeout":{"!doc":"<p>The amount of time given to the uploaded file\nto wait for a server response. After this amount\nof time, the onComplete is fired but with a &#39;server timeout&#39;\nerror in the returned item.</p>\n"},"showInput":{"!doc":"<p>Position to show an input which shows selected filename(s). Possible\nvalues are &quot;before&quot;, &quot;after&quot;, which specifies where the input should\nbe placed with reference to the containerNode which contains the\nlabel). By default, this is empty string (no such input will be\nshown). Specify showInput=&quot;before&quot; to mimic the look&amp;feel of a\nnative file input element.</p>\n"},"showLabel":{"!doc":"<p>Set this to true to hide the label text and display only the icon.\n(If showLabel=false then iconClass must be specified.)\nEspecially useful for toolbars.\nIf showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.\n\n</p>\n<p>The exception case is for computers in high-contrast mode, where the label\nwill still be displayed, since the icon doesn&#39;t appear.</p>\n"},"skipServerCheck":{"!doc":"<p>If true, will not verify that the server was sent the correct format.\nThis can be safely set to true. The purpose of the server side check\nis mainly to show the dev if they&#39;ve implemented the different returns\ncorrectly.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"swfPath":{"!doc":"<p>Path to SWF. Can be overwritten or provided in djConfig.</p>\n"},"tabIndex":{"!doc":"<p>The tab order in the DOM.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uploadOnSelect":{"!doc":"<p>If true, uploads immediately after a file has been selected. If false,\nwaits for upload() to be called.</p>\n"},"uploadType":{"!doc":"<p>The type of uploader being used. As an alternative to determining the upload type on the\nserver based on the fieldName, this property could be sent to the server to help\ndetermine what type of parsing should be used.\nThis is set based on the force property and what features are available in the browser.</p>\n"},"url":{"!doc":"<p>The url targeted for upload. An absolute URL is preferred. Relative URLs are\nchanged to absolute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addDropTarget":{"!type":"fn(node: ?, onlyConnectDrop?: ?)","!doc":"<p>Add a dom node which will act as the drop target area so user\ncan drop files to this node.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectForm":{"!type":"fn()","!doc":"<p>Internal. Connects to form if there is one.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"convertBytes":{"!type":"fn(bytes: ?) -> ?","!doc":"<p>Converts bytes. Returns an object with all conversions. The &quot;value&quot; property is\nconsidered the most likely desired result.</p>\n"},"createXhr":{"!type":"fn() -> +instance"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"flashReset":{"!type":"fn()"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getFileList":{"!type":"fn() -> []","!doc":"<p>Returns a list of selected files.</p>\n"},"getFileType":{"!type":"fn(name: ?) -> ?","!doc":"<p>Gets the extension of a file</p>\n"},"getFlashFileList":{"!type":"fn() -> ?","!doc":"<p>Returns list of currently selected files</p>\n"},"getForm":{"!type":"fn() -> +Node;","!doc":"<p>Finds the parent form of the Uploader, if it exists.</p>\n"},"getMimeType":{"!type":"fn() -> string","!doc":"<p>Returns the mime type that should be used in an HTML5 upload form. Return result\nmay change as the current use is very generic.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getUrl":{"!type":"fn() -> ?","!doc":"<p>Finds the URL to upload to, whether it be the action in the parent form, this.url or\nthis.uploadUrl</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Resets entire input, clearing all files.\nNOTE:\nRemoving individual files is not yet supported, because the HTML5 uploaders can&#39;t\nbe edited.\nTODO:\nAdd this ability by effectively, not uploading them\n</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.  Use set(&#39;label&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"submit":{"!type":"fn(form?: ?)","!doc":"<p>If Uploader is in a form, and other data should be sent along with the files, use\nthis instead of form submit.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"supports":{"!type":"fn(what: ?) -> bool","!doc":"<p>Does feature testing for uploader capabilities. (No browser sniffing - yay)</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"upload":{"!type":"fn(formData?: ?)","!doc":"<p>When called, begins file upload. Only supported with plugins.</p>\n"},"uploadIFrame":{"!type":"fn(data: ?)","!doc":"<p>Internal. You could use this, but you should use upload() or submit();\nwhich can also handle the post data.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/uploader/_Base":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>The Base class used for dojox/form/Uploader and dojox/form/uploader/FileList.\n\n</p>\n<p>Should not be used as a standalone. To be mixed in with other classes.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectForm":{"!type":"fn()","!doc":"<p>Internal. Connects to form if there is one.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"convertBytes":{"!type":"fn(bytes: ?) -> ?","!doc":"<p>Converts bytes. Returns an object with all conversions. The &quot;value&quot; property is\nconsidered the most likely desired result.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getFileType":{"!type":"fn(name: ?) -> ?","!doc":"<p>Gets the extension of a file</p>\n"},"getForm":{"!type":"fn() -> +Node;","!doc":"<p>Finds the parent form of the Uploader, if it exists.</p>\n"},"getMimeType":{"!type":"fn() -> string","!doc":"<p>Returns the mime type that should be used in an HTML5 upload form. Return result\nmay change as the current use is very generic.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getUrl":{"!type":"fn() -> ?","!doc":"<p>Finds the URL to upload to, whether it be the action in the parent form, this.url or\nthis.uploadUrl</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"supports":{"!type":"fn(what: ?) -> bool","!doc":"<p>Does feature testing for uploader capabilities. (No browser sniffing - yay)</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/uploader/_HTML5":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for dojox/form/Uploader that adds HTML5 multiple-file upload capabilities and\nprogress events.\n</p>\n","errMsg":{},"uploadType":{},"addDropTarget":{"!type":"fn(node: ?, onlyConnectDrop?: ?)","!doc":"<p>Add a dom node which will act as the drop target area so user\ncan drop files to this node.</p>\n"},"createXhr":{"!type":"fn() -> +instance"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"upload":{"!type":"fn(formData?: ?)","!doc":"<p>See: dojox.form.Uploader.upload</p>\n"}}},"dojox/form/uploader/_IFrame":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for dojox/form/Uploader that adds Ajax upload capabilities via an iframe.\n</p>\n","postMixInProperties":{"!type":"fn()"},"uploadIFrame":{"!type":"fn(data: ?)","!doc":"<p>Internal. You could use this, but you should use upload() or submit();\nwhich can also handle the post data.</p>\n"}}},"dojox/form/uploader/_Flash":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for dojox/form/Uploader that utilizes a Flash SWF for handling to upload in IE.\nAll other browsers will use the HTML5 plugin, unless force=&quot;flash&quot; is used, then Flash\nwill be used in all browsers. force=&quot;flash&quot; is provided because Flash has some features\nthat HTML5 does not yet have. But it is still not recommended because of the many problems\nthat Firefox and Webkit have with the Flash plugin.\n</p>\n","deferredUploading":{"!doc":"<p>(Flash only) throttles the upload to a certain amount of files at a time.\nBy default, Flash uploads file one at a time to the server, but in parallel.\nFirefox will try to queue all files at once, leading to problems. Set this\nto the amount to upload in parallel at a time.\nGenerally, 1 should work fine, but you can experiment with queuing more than\none at a time.\nThis is of course ignored if selectMultipleFiles equals false.</p>\n"},"devMode":{"!doc":"<p>Re-implemented. devMode increases the logging, adding style tracing from the SWF.</p>\n"},"isDebug":{"!doc":"<p>If true, outputs traces from the SWF to console. What exactly gets passed\nis very relative, and depends upon what traces have been left in the DEFT SWF.</p>\n"},"preventCache":{"!doc":"<p>If true, then flash request is sent with a value that changes with each request (timestamp)</p>\n"},"serverTimeout":{"!doc":"<p>The amount of time given to the uploaded file\nto wait for a server response. After this amount\nof time, the onComplete is fired but with a &#39;server timeout&#39;\nerror in the returned item.</p>\n"},"skipServerCheck":{"!doc":"<p>If true, will not verify that the server was sent the correct format.\nThis can be safely set to true. The purpose of the server side check\nis mainly to show the dev if they&#39;ve implemented the different returns\ncorrectly.</p>\n"},"swfPath":{"!doc":"<p>Path to SWF. Can be overwritten or provided in djConfig.</p>\n"},"flashReset":{"!type":"fn()"},"getFlashFileList":{"!type":"fn() -> ?","!doc":"<p>Returns list of currently selected files</p>\n"},"postMixInProperties":{"!type":"fn()"}}},"dojox/form/YearTextBox":{"!type":"fn()","prototype":{"!proto":"dojox/form/DateTextBox.prototype","!doc":"<p>A validating, serializable, range-bound date text box with a popup calendar that contains only years</p>\n","__skipInputEvent":{},"_aroundNode":{"!doc":"<p>The node to display the popup around.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then domNode will be used.</p>\n"},"_arrowWrapperNode":{"!doc":"<p>Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending\non where the drop down is set to be positioned.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then _buttonNode will be used.</p>\n"},"_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_buttonInputDisabled":{},"_buttonNode":{"!doc":"<p>The button/icon/node to click to display the drop down.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then either focusNode or domNode (if focusNode is also missing) will be used.</p>\n"},"_focusManager":{},"_maskValidSubsetError":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_popupStateNode":{"!doc":"<p>The node to set the aria-expanded class on.\nAlso sets popupActive class but that will be removed in 2.0.\nCan be set via a data-dojo-attach-point assignment.\nIf missing, then focusNode or _buttonNode (if focusNode is missing) will be used.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_selector":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setNameAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_singleNodeTemplate":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{"!doc":"<p>When set to false, the click events will not be stopped, in\ncase you want to use them in your subclass</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoWidth":{},"baseClass":{},"class":{},"constraints":{"!doc":"<p>Despite the name, this parameter specifies both constraints on the input\n(including starting/ending dates/times allowed) as well as\nformatting options like whether the date is displayed in long (ex: December 25, 2005)\nor short (ex: 12/25/2005) format.  See <code>dijit/form/_DateTimeTextBox.__Constraints</code> for details.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{},"datePackage":{"!doc":"<p>JavaScript namespace to find calendar routines.  If unspecified, uses Gregorian calendar routines\nat dojo/date and dojo/date/locale.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup.  This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownDefaultValue":{"!doc":"<p>The default value to focus in the popupClass widget when the textbox value is empty.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop down.\nIt&#39;s an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.\n</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"forceWidth":{"!doc":"<p>Set to true to make the drop down exactly as wide as this\nwidget.  Overrides autoWidth.</p>\n"},"hasDownArrow":{"!doc":"<p>Set this textbox to display a down arrow button, to open the drop down list.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"invalidMessage":{"!doc":"<p>The message to display if value is invalid.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the promptMessage instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The max height for our dropdown.\nAny dropdown taller than this will have scrollbars.\nSet to 0 for no max height, or -1 to limit height to available space in viewport</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"message":{"!doc":"<p>Currently error/prompt message.\nWhen using the default tooltip implementation, this will only be\ndisplayed when the field is focused.</p>\n"},"missingMessage":{"!doc":"<p>The message to display if value is empty and the field is required.\nThe translated string value is read from the message file by default.\nSet to &quot;&quot; to use the invalidMessage instead.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"promptMessage":{"!doc":"<p>If defined, display this hint string immediately on focus to the textbox, if empty.\nAlso displays if the textbox value is Incomplete (not yet valid but will be with additional input).\nThink of this like a tooltip that tells the user what to do, not an error message\nthat tells the user what they&#39;ve done wrong.\n\n</p>\n<p>Message disappears when user starts typing.</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"rangeMessage":{"!doc":"<p>The message to display if value is out-of-range</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"regExp":{},"required":{"!doc":"<p>User is required to enter data into this field.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Shows current state (ie, validation result) of input (&quot;&quot;=Normal, Incomplete, or Error)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipPosition":{"!doc":"<p>See description of <code>dijit/Tooltip.defaultPosition</code> for details on this parameter.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>The value of this widget as a JavaScript Date object.  Use get(&quot;value&quot;) / set(&quot;value&quot;, val) to manipulate.\nWhen passed to the parser in markup, must be specified according to <code>dojo/date/stamp.fromISOString()</code></p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn(focus: ?)","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"displayMessage":{"!type":"fn(message: ?)","!doc":"<p>Overridable method to display validation errors/hints.\nBy default uses a tooltip.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?) -> ?"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getDisplayedValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;displayedValue&#39;) instead.</p>\n"},"getErrorMessage":{"!type":"fn(isFocused: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPromptMessage":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Return a hint message to show when widget is first focused</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isInRange":{"!type":"fn(isFocused: ?) -> ?","!doc":"<p>Tests if the value is in the min/max range specified in constraints</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool","!doc":"<p>Returns true if the dropdown exists and it&#39;s data is loaded.  This can\nbe overridden in order to force a call to loadDropDown().</p>\n"},"isValid":{"!type":"fn(isFocused: ?) -> ?"},"loadAndOpenDropDown":{"!type":"fn() -> +Deferred","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and\nthen opens the drop down.  This is basically a callback when the\nuser presses the down arrow button to open the drop down.</p>\n"},"loadDropDown":{"!type":"fn(loadCallback: ?)","!doc":"<p>Creates the drop down if it doesn&#39;t exist, loads the data\nif there&#39;s an href and it hasn&#39;t been loaded yet, and then calls\nthe given callback.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"openDropDown":{"!type":"fn()"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string"},"pattern":{"!type":"fn(options?: ?) -> ?","!doc":"<p>Builds the regular needed to parse a localized date</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"popupClass":{"!type":"fn()"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"rangeCheck":{"!type":"fn(primitive: ?, constraints: ?) -> bool","!doc":"<p>Overridable function used to validate the range of the numeric input value.</p>\n"},"regExpGen":{"!type":"fn(constraints: ?)","!doc":"<p>Deprecated.  Use set(&#39;pattern&#39;, Function) instead.</p>\n"},"reset":{"!type":"fn()"},"serialize":{"!type":"fn(val: ?, options?: ?) -> ?"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setDisplayedValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;displayedValue&#39;, ...) instead.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDropDown":{"!type":"fn()","!doc":"<p>Callback when the user presses the down arrow button or presses\nthe down arrow key to open/close the drop down.\nToggle the drop-down widget; if it is up, close it, if not, open it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns widget as a printable string using the widget&#39;s value</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?"},"validator":{"!type":"fn(value: ?) -> bool"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/YearlyCalendar":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarBase.prototype","!doc":"<p>A calendar with only a year view.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parent":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/_CalendarYear":{"!type":"fn()","prototype":{"!doc":"<p>Mixin class for adding a view listing 12 years to the\ndojox/widget/_CalendarBase</p>\n","parent":{},"constructor":{"!type":"fn()","!doc":"<p>Adds a dojox/widget/_CalendarYearView view to the\ndojo/widget/_CalendarBase widget.</p>\n"}}},"dojox/widget/_CalendarYearView":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarView.prototype","!doc":"<p>A Calendar view listing 12 years</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"displayedYears":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"headerClass":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useHeader":{},"adjustDate":{"!type":"fn(date: ?, amount: ?) -> ?","!doc":"<p>Adjusts the value of a date. It moves it by 12 years each time.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"cloneClass":{"!type":"fn(clazz: ?, n: ?, before: ?)","!doc":"<p>Clones all nodes with the class &#39;clazz&#39; in a widget</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getHeader":{"!type":"fn() -> ?","!doc":"<p>Returns the header node of a view. If none exists,\nan empty DIV is created and returned.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Constructs the view</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/_FormSelectWidget":{"!type":"fn()","prototype":{"!proto":"dijit/form/_FormValueWidget.prototype","!doc":"<p>Extends _FormValueWidget in order to provide &quot;select-specific&quot;\nvalues - i.e., those values that are unique to <code>&lt;select&gt;</code> elements.\nThis also provides the mechanism for reading the elements from\na store, if desired.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the dojo.store items.\nIf <code>store</code> is set, labelAttr must be set too, unless store is an old-style\ndojo.data store rather than a new dojo/store.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadChildrenOnOpen":{"!doc":"<p>By default loadChildren is called when the items are fetched from the\nstore.  This property allows delaying loadChildren (and the creation\nof the options/menuitems) until the user clicks the button to open the\ndropdown.</p>\n"},"multiple":{"!doc":"<p>Whether or not we are multi-valued</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"onFetch":{"!doc":"<p>A callback to do with an onFetch - but before any items are actually\niterated over (i.e. to filter even further what you want to add)</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by setStore().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior setStore completes.</p>\n"},"options":{"!doc":"<p>The set of options for our select item.  Roughly corresponds to\nthe html <code>&lt;option&gt;</code> tag.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query to use when fetching items from our store</p>\n"},"queryOptions":{"!doc":"<p>Query options to use when fetching from the store</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchContainerNode":{},"sortByLabel":{"!doc":"<p>Flag to sort the options returned from a store by the label of\nthe store.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>A store to use for getting our list of options - rather than reading them\nfrom the <code>&lt;option&gt;</code> html tags.   Should support getIdentity().\nFor back-compat store can also be a dojo/data/api/Identity.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"addOption":{"!type":"fn(option: ?)","!doc":"<p>Adds an option or options to the end of the select.  If value\nof the option is empty or missing, a separator is created instead.\nPassing in an array of options will yield slightly better performance\nsince the children are only loaded once.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up our connections</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getOptions":{"!type":"fn(valueOrIdx: ?) -> +any","!doc":"<p>Returns a given option (or options).</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Deprecated.  Use get(&#39;value&#39;) instead.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>sets up our event handling that we need for functioning\nas a select</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeOption":{"!type":"fn(valueOrIdx: ?)","!doc":"<p>Removes the given option or options.  You can remove by string\n(in which case the value is removed), number (in which case the\nindex in the options array is removed), or select option (in\nwhich case, the select option with a matching value is removed).\nYou can also pass in an array of those values for a slightly\nbetter performance since the children are only loaded once.\nFor numeric option values, specify {value: number} as the argument.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.  Use set(&#39;disabled&#39;, ...) instead.</p>\n"},"setStore":{"!type":"fn(store: ?, selectedValue?: ?, fetchArgs?: ?) -> ?","!doc":"<p>Sets the store you would like to use with this select widget.\nThe selected value is the value of the new store to set.  This\nfunction returns the original store, in case you want to reuse\nit or something.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Deprecated.  Use set(&#39;value&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateOption":{"!type":"fn(newOption: ?)","!doc":"<p>Updates the values of the given option.  The option to update\nis matched based on the value of the entered option.  Passing\nin an array of new options will yield better performance since\nthe children will only be loaded once.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"__SelectOption":{}},"dojox/form/_FormSelectWidget.__SelectOption":{"disabled":{"!doc":"<p>Whether or not this specific option is disabled</p>\n"},"label":{"!doc":"<p>The label for our option.  It can contain html tags.</p>\n"},"selected":{"!doc":"<p>Whether or not we are a selected option</p>\n"},"value":{"!doc":"<p>The value of the option.  Setting to empty (or missing) will\nplace a separator at that location</p>\n"}},"dojox/form/_HasDropDown":{"!doc":"<p>This module defines Javascript language extensions.</p>\n","_extraNames":{"!doc":"<p>Lists property names that must be explicitly processed during for-in iteration\nin environments that have has(&quot;bug-for-in-skips-shadowed&quot;) true.</p>\n"},"clone":{"!type":"fn(src: ?) -> +anything","!doc":"<p>Clones objects (including DOM nodes) and all children.\nWarning: do not clone cyclic structures.</p>\n"},"delegate":{"!type":"fn(obj: ?, props: ?) -> +any","!doc":"<p>Returns a new object which &quot;looks&quot; to obj for properties which it\ndoes not have a value for. Optionally takes a bag of properties to\nseed the returned object with initially.</p>\n"},"exists":{"!type":"fn(name: ?, obj?: ?) -> bool","!doc":"<p>determine if an object supports a given method</p>\n"},"extend":{"!type":"fn(ctor: ?, props: ?) -> ?","!doc":"<p>Adds all properties and methods of props to constructor&#39;s\nprototype, making them available to all instances created with\nconstructor.</p>\n"},"getObject":{"!type":"fn(name: ?, create?: ?, context?: ?) -> ?","!doc":"<p>Get a property from a dot-separated string, such as &quot;A.B.C&quot;</p>\n"},"hitch":{"!type":"fn(scope: ?, method: ?) -> ?","!doc":"<p>Returns a function that will only ever execute in the a given scope.\nThis allows for easy use of object member functions\nin callbacks and other places in which the &quot;this&quot; keyword may\notherwise not reference the expected scope.\nAny number of default positional arguments may be passed as parameters\nbeyond &quot;method&quot;.\nEach of these values will be used to &quot;placehold&quot; (similar to curry)\nfor the hitched function.</p>\n"},"isAlien":{"!type":"fn(it: ?) -> ?","!doc":"<p>Returns true if it is a built-in function or some other kind of\noddball that <em>should</em> report as a function but doesn&#39;t</p>\n"},"isArray":{"!type":"fn(it: ?) -> ?","!doc":"<p>Return true if it is an Array.\nDoes not work on Arrays created in other windows.</p>\n"},"isArrayLike":{"!type":"fn(it: ?) -> +any","!doc":"<p>similar to isArray() but more permissive</p>\n"},"isFunction":{"!type":"fn(it: ?) -> bool","!doc":"<p>Return true if it is a Function</p>\n"},"isObject":{"!type":"fn(it: ?) -> bool","!doc":"<p>Returns true if it is a JavaScript object (or an Array, a Function\nor null)</p>\n"},"isString":{"!type":"fn(it: ?) -> bool","!doc":"<p>Return true if it is a String</p>\n"},"mixin":{"!type":"fn(dest: ?, sources: ?) -> ?","!doc":"<p>Copies/adds all properties of one or more sources to dest; returns dest.</p>\n"},"partial":{"!type":"fn(method: ?) -> ?","!doc":"<p>similar to hitch() except that the scope object is left to be\nwhatever the execution context eventually becomes.</p>\n"},"replace":{"!type":"fn(tmpl: ?, map: ?, pattern?: ?) -> string","!doc":"<p>Performs parameterized substitutions on a string. Throws an\nexception if any parameter is unmatched.</p>\n"},"setObject":{"!type":"fn(name: ?, value: ?, context?: ?) -> ?","!doc":"<p>Set a property from a dot-separated string, such as &quot;A.B.C&quot;</p>\n"},"trim":{"!type":"fn(str: ?) -> string","!doc":"<p>Trims whitespace from both sides of the string</p>\n"}},"dojox/form/uploader/FileList":{"!type":"fn()","prototype":{"!proto":"dojox/form/uploader/_Base.prototype","!doc":"<p>A simple widget that provides a list of the files currently selected by\ndojox/form/Uploader</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_upCheckCnt":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"headerFilename":{"!doc":"<p>The label for the file name column.</p>\n"},"headerFilesize":{"!doc":"<p>The label for the file size column.</p>\n"},"headerIndex":{"!doc":"<p>The label for the index column.</p>\n"},"headerType":{"!doc":"<p>The label for the file type column.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"rowAmt":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"uploader":{"!doc":"<p>The dojox.form.Uploader to connect to. Use either this property of unploaderId. This\nproperty is populated if uploaderId is used.</p>\n"},"uploaderId":{"!doc":"<p>The id of the dojox.form.Uploader to connect to.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectForm":{"!type":"fn()","!doc":"<p>Internal. Connects to form if there is one.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"convertBytes":{"!type":"fn(bytes: ?) -> ?","!doc":"<p>Converts bytes. Returns an object with all conversions. The &quot;value&quot; property is\nconsidered the most likely desired result.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getFileType":{"!type":"fn(name: ?) -> ?","!doc":"<p>Gets the extension of a file</p>\n"},"getForm":{"!type":"fn() -> +Node;","!doc":"<p>Finds the parent form of the Uploader, if it exists.</p>\n"},"getMimeType":{"!type":"fn() -> string","!doc":"<p>Returns the mime type that should be used in an HTML5 upload form. Return result\nmay change as the current use is very generic.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getUrl":{"!type":"fn() -> ?","!doc":"<p>Finds the URL to upload to, whether it be the action in the parent form, this.url or\nthis.uploadUrl</p>\n"},"hideProgress":{"!type":"fn(animate: ?)"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Clears all rows of items. Happens automatically if Uploader is reset, but you\ncould call this directly.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setUploader":{"!type":"fn()","!doc":"<p>Connects to the Uploader based on the uploader or the uploaderId properties.</p>\n"},"showProgress":{"!type":"fn(animate: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"supports":{"!type":"fn(what: ?) -> bool","!doc":"<p>Does feature testing for uploader capabilities. (No browser sniffing - yay)</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/form/uploader/plugins/Flash":{},"dojox/form/uploader/plugins/HTML5":{},"dojox/form/uploader/plugins/IFrame":{},"dojox/fx/Shadow":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Adds a drop-shadow to a node.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"animate":{"!doc":"<p>A toggle to disable animated transitions</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"node":{"!doc":"<p>The node we will be applying this shadow to</p>\n"},"observer":{},"opacity":{"!doc":"<p>Overall opacity of the shadow</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"shadowOffset":{"!doc":"<p>How deep to make the shadow appear to be</p>\n"},"shadowPng":{"!doc":"<p>Base location for drop-shadow images</p>\n"},"shadowThickness":{"!doc":"<p>How wide (in px) to make the shadow</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"resize":{"!type":"fn(args: ?)","!doc":"<p>Resizes the shadow based on width and height.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>enable / disable the shadow</p>\n"},"setOpacity":{"!type":"fn(n: ?, animArgs?: ?)","!doc":"<p>set the opacity of the underlay</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Initializes the shadow.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/fx/Timeline":{"!doc":"<p>Experimental and extended Animations beyond Dojo Core / Base functionality.\nProvides advanced Lines, Animations, and convenience aliases.</p>\n","_allowedProperties":{},"addClass":{"!type":"fn(node: ?, cssClass: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of adding a class to a node</p>\n"},"animateTimeline":{"!type":"fn(options: ?, node: ?)","!doc":"<p>An add-on to dojo.fx that provides the ability to create\na complex property animation based on an array of &quot;keyframes&quot;.</p>\n"},"flip":{"!type":"fn(args: ?)","!doc":"<p>Animate a node flipping following a specific direction</p>\n"},"flipCube":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a more 3d-like rotation</p>\n"},"flipGrid":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a decomposition in rows * cols flipping elements</p>\n"},"flipPage":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a page flip like animation.</p>\n"},"removeClass":{"!type":"fn(node: ?, cssClass: ?, args: ?) -> ?","!doc":"<p>Animate the effects of removing a class from a node</p>\n"},"smoothScroll":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation that will smooth-scroll to a node</p>\n"},"toggleClass":{"!type":"fn(node: ?, cssClass: ?, condition?: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of Toggling a class on a Node</p>\n"}},"dojox/fx/_arg":{"ShadowResizeArgs":{"!type":"fn(args: ?)","!doc":"<p>The odd way to document object parameters.</p>\n"},"StyleArgs":{"!type":"fn(args: ?)","!doc":"<p>The node and CSS class to use for style manipulations.</p>\n"}},"dojox/fx/_core":{"!doc":"<p>a custom _Line to accommodate multi-dimensional values</p>\n","end":{"!doc":"<p>An Integer (or an Array of integers) to use as an ending point</p>\n"},"start":{"!doc":"<p>An Integer (or an Array of integers) to use as a starting point</p>\n"},"prototype":{"constructor":{"!type":"fn(start: ?, end: ?)","!doc":"<p>a custom _Line to accommodate multi-dimensional values</p>\n"}},"getValue":{"!type":"fn(n: ?) -> +Mixed","!doc":"<p>Returns the point on the line, or an array of points</p>\n"}},"dojox/fx/easing":{"!doc":"<p>An Alias to <code>dojo.fx.easing</code>. Moved to Core in Dojo 1.2.</p>\n"},"dojox/fx/ext-dojo/NodeList-style":{"!doc":"<p>Core extensions to <code>dojo.NodeList</code> providing additional fx to <code>dojo.NodeList-fx</code>\nfrom <code>dojox.fx.style</code></p>\n"},"dojox/fx/style":{"!doc":"<p>dojox.fx CSS Class Animations</p>\n","_allowedProperties":{},"addClass":{"!type":"fn(node: ?, cssClass: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of adding a class to a node</p>\n"},"removeClass":{"!type":"fn(node: ?, cssClass: ?, args: ?) -> ?","!doc":"<p>Animate the effects of removing a class from a node</p>\n"},"toggleClass":{"!type":"fn(node: ?, cssClass: ?, condition?: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of Toggling a class on a Node</p>\n"}},"dojox/fx/ext-dojo/NodeList":{"!doc":"<p>Core extensions to dojo/NodeList providing additional fx to dojo.NodeList-fx</p>\n"},"dojox/fx/ext-dojo/reverse":{"!doc":"<p>A generic animation class that fires callbacks into its handlers\nobject at various states.</p>\n","prototype":{"_percent":{},"_reversed":{},"_startRepeatCount":{},"beforeBegin":{"!doc":"<p>Synthetic event fired before a Animation begins playing (synchronous)</p>\n"},"curve":{"!doc":"<p>A two element array of start and end values, or a <code>_Line</code> instance to be\nused in the Animation.</p>\n"},"delay":{"!doc":"<p>The time in milliseconds to wait before starting animation after it\nhas been .play()&#39;ed</p>\n"},"duration":{"!doc":"<p>The time in milliseconds the animation will take to run</p>\n"},"easing":{"!doc":"<p>A Function to adjust the acceleration (or deceleration) of the progress\nacross a _Line</p>\n"},"onAnimate":{"!doc":"<p>Synthetic event fired at each interval of the Animation</p>\n"},"onBegin":{"!doc":"<p>Synthetic event fired as a Animation begins playing (useful?)</p>\n"},"onEnd":{"!doc":"<p>Synthetic event fired after the final frame of the Animation</p>\n"},"onPause":{"!doc":"<p>Synthetic event fired when the Animation is paused</p>\n"},"onPlay":{"!doc":"<p>Synthetic event fired any time the Animation is play()&#39;ed</p>\n"},"onStop":{"!doc":"<p>Synthetic event fires when the Animation is stopped</p>\n"},"rate":{"!doc":"<p>the time in milliseconds to wait before advancing to next frame\n(used as a fps timer: 1000/rate = fps)</p>\n"},"repeat":{"!doc":"<p>The number of times to loop the animation</p>\n"},"gotoPercent":{"!type":"fn(percent: ?, andPlay?: ?) -> fn","!doc":"<p>Sets the progress of the animation.</p>\n"},"pause":{"!type":"fn() -> fn","!doc":"<p>Pauses a running animation.</p>\n"},"play":{"!type":"fn(delay?: ?, gotoStart?: ?) -> +Animation","!doc":"<p>Start the animation.</p>\n"},"reverse":{"!type":"fn(keepPaused: ?, reverseEase?: ?) -> fn","!doc":"<p>The key method added to an animation to enable reversal.</p>\n"},"status":{"!type":"fn() -> string","!doc":"<p>Returns a string token representation of the status of\nthe animation, one of: &quot;paused&quot;, &quot;playing&quot;, &quot;stopped&quot;</p>\n"},"stop":{"!type":"fn(gotoEnd?: ?) -> fn","!doc":"<p>Stops a running animation.</p>\n"}},"curve":{}},"dojox/fx/flip":{"!doc":"<p>Experimental and extended Animations beyond Dojo Core / Base functionality.\nProvides advanced Lines, Animations, and convenience aliases.</p>\n","_allowedProperties":{},"addClass":{"!type":"fn(node: ?, cssClass: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of adding a class to a node</p>\n"},"animateTimeline":{"!type":"fn(options: ?, node: ?)","!doc":"<p>An add-on to dojo.fx that provides the ability to create\na complex property animation based on an array of &quot;keyframes&quot;.</p>\n"},"flip":{"!type":"fn(args: ?)","!doc":"<p>Animate a node flipping following a specific direction</p>\n"},"flipCube":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a more 3d-like rotation</p>\n"},"flipGrid":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a decomposition in rows * cols flipping elements</p>\n"},"flipPage":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a page flip like animation.</p>\n"},"removeClass":{"!type":"fn(node: ?, cssClass: ?, args: ?) -> ?","!doc":"<p>Animate the effects of removing a class from a node</p>\n"},"smoothScroll":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation that will smooth-scroll to a node</p>\n"},"toggleClass":{"!type":"fn(node: ?, cssClass: ?, condition?: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of Toggling a class on a Node</p>\n"}},"dojox/fx/scroll":{"!doc":"<p>Returns an animation that will smooth-scroll to a node</p>\n","prototype":{"constructor":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation that will smooth-scroll to a node</p>\n"}}},"dojox/fx/split":{"!doc":"<p>Experimental and extended Animations beyond Dojo Core / Base functionality.\nProvides advanced Lines, Animations, and convenience aliases.</p>\n","_allowedProperties":{},"addClass":{"!type":"fn(node: ?, cssClass: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of adding a class to a node</p>\n"},"animateTimeline":{"!type":"fn(options: ?, node: ?)","!doc":"<p>An add-on to dojo.fx that provides the ability to create\na complex property animation based on an array of &quot;keyframes&quot;.</p>\n"},"flip":{"!type":"fn(args: ?)","!doc":"<p>Animate a node flipping following a specific direction</p>\n"},"flipCube":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a more 3d-like rotation</p>\n"},"flipGrid":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a decomposition in rows * cols flipping elements</p>\n"},"flipPage":{"!type":"fn(args: ?)","!doc":"<p>An extension to <code>dojox.fx.flip</code> providing a page flip like animation.</p>\n"},"removeClass":{"!type":"fn(node: ?, cssClass: ?, args: ?) -> ?","!doc":"<p>Animate the effects of removing a class from a node</p>\n"},"smoothScroll":{"!type":"fn(args: ?)","!doc":"<p>Returns an animation that will smooth-scroll to a node</p>\n"},"toggleClass":{"!type":"fn(node: ?, cssClass: ?, condition?: ?, args?: ?) -> ?","!doc":"<p>Animate the effects of Toggling a class on a Node</p>\n"}},"dojox/fx/text":{"backspace":{"!type":"fn(args: ?)","!doc":"<p>Split a block of text into words or letters and backspace them in sequence</p>\n"},"blockFadeIn":{"!type":"fn(args: ?)"},"blockFadeOut":{"!type":"fn(args: ?)","!doc":"<p>Split a block of text into words or letters and fade them</p>\n"},"build":{"!type":"fn(args: ?)"},"converge":{"!type":"fn(args: ?)"},"disintegrate":{"!type":"fn(args: ?)","!doc":"<p>Split a block of text into words or letters and let them fall</p>\n"},"explode":{"!type":"fn(args: ?)","!doc":"<p>Explode a block of text into words or letters</p>\n"},"type":{"!type":"fn(args: ?)"}},"dojox/gantt/GanttChart":{"!type":"fn()","prototype":{"addDayInPanelTime":{"!type":"fn(row: ?)"},"addHourInPanelTime":{"!type":"fn(row: ?)"},"addMonthInPanelTime":{"!type":"fn(row: ?, count: ?, month: ?, year: ?)"},"addProject":{"!type":"fn(projectItem: ?)"},"addWeekInPanelTime":{"!type":"fn(row: ?, count: ?, week: ?)"},"addYearInPanelTime":{"!type":"fn(row: ?, count: ?, year: ?)"},"adjustPanelTime":{"!type":"fn()"},"buildChildTasksData":{"!type":"fn(parentTask: ?, childTaskItems: ?)"},"buildUIContent":{"!type":"fn()"},"checkHeighPanelTasks":{"!type":"fn()"},"checkPosition":{"!type":"fn()"},"checkPosParentTask":{"!type":"fn(parentTask: ?, task: ?) -> bool"},"checkPosParentTaskInTree":{"!type":"fn(parentTask: ?) -> bool"},"checkPosPreviousTask":{"!type":"fn(predTask: ?, task: ?) -> bool"},"clearAll":{"!type":"fn()"},"clearData":{"!type":"fn()"},"clearEvents":{"!type":"fn()"},"clearItems":{"!type":"fn()"},"constructor":{"!type":"fn(configuration: ?, node: ?)"},"correctPosParentTask":{"!type":"fn(parentTask: ?, ctask: ?)"},"correctPosPreviousTask":{"!type":"fn(predTask: ?, ctask: ?, ctaskObj: ?)"},"createChildItemControls":{"!type":"fn(arrChildTasks: ?, project: ?)"},"createPanelNamesTasks":{"!type":"fn() -> ?"},"createPanelNamesTasksHeader":{"!type":"fn() -> ?"},"createPanelTasks":{"!type":"fn() -> ?"},"createPanelTime":{"!type":"fn() -> ?"},"createTasks":{"!type":"fn(project: ?)"},"deleteProject":{"!type":"fn(id: ?)"},"getChildTasksData":{"!type":"fn(childTasks: ?) -> []"},"getCountDays":{"!type":"fn() -> ?"},"getJSONData":{"!type":"fn() -> ?"},"getLastChildTask":{"!type":"fn(task: ?) -> ?"},"getLastCloseParent":{"!type":"fn(task: ?) -> ?"},"getPosOnDate":{"!type":"fn(startTime: ?) -> number"},"getProject":{"!type":"fn(id: ?) -> ?"},"getProjectItemById":{"!type":"fn(id: ?) -> ?"},"getStartDate":{"!type":"fn() -> +instance"},"getWidthOnDuration":{"!type":"fn(duration: ?) -> ?"},"incHeightPanelNames":{"!type":"fn(height: ?)"},"incHeightPanelTasks":{"!type":"fn(height: ?)"},"init":{"!type":"fn() -> fn"},"insertProject":{"!type":"fn(id: ?, name: ?, startDate: ?) -> bool"},"loadJSONData":{"!type":"fn(filename: ?)"},"loadJSONString":{"!type":"fn(content: ?)"},"openNode":{"!type":"fn(parentTask: ?)"},"openTree":{"!type":"fn(parentTask: ?)"},"postBindEvents":{"!type":"fn()"},"postLoadData":{"!type":"fn()"},"refresh":{"!type":"fn(count: ?, current: ?, multi: ?)"},"refreshController":{"!type":"fn()"},"refreshParams":{"!type":"fn(pixelsPerDay: ?)"},"removeCell":{"!type":"fn(row: ?)"},"saveJSONData":{"!type":"fn(fileName: ?)"},"setPreviousTask":{"!type":"fn(project: ?) -> bool"},"setPreviousTaskInTree":{"!type":"fn(parentTask: ?) -> bool"},"setStartTimeChild":{"!type":"fn(parentTask: ?)"},"sortChildTasks":{"!type":"fn(parenttask: ?) -> ?"},"sortProjStartDate":{"!type":"fn(a: ?, b: ?) -> number"},"sortTasksByStartTime":{"!type":"fn(project: ?)"},"sortTaskStartTime":{"!type":"fn(a: ?, b: ?) -> number"},"switchTeleMicroView":{"!type":"fn(dip: ?)"}}},"dojox/gantt/GanttProjectItem":{"!type":"fn()","prototype":{"!proto":"dojox/gantt/GanttTaskItem.prototype","addChildTask":{"!type":"fn(task: ?)"},"addTask":{"!type":"fn(task: ?)"},"constructor":{"!type":"fn(configuration: ?)"},"deleteTask":{"!type":"fn(id: ?)"},"getTaskById":{"!type":"fn(id: ?) -> ?"},"getTaskByIdInTree":{"!type":"fn(parentTask: ?, id: ?) -> ?"},"setProject":{"!type":"fn(project: ?)"}}},"dojox/gantt/GanttTaskItem":{"!type":"fn()","prototype":{"addChildTask":{"!type":"fn(task: ?)"},"constructor":{"!type":"fn(configuration: ?)"},"setProject":{"!type":"fn(project: ?)"}}},"dojox/gantt/GanttTaskControl":{"!type":"fn()","prototype":{"adjustPanelTime":{"!type":"fn()"},"buildResourceInfo":{"!type":"fn(resourceInfo: ?)"},"checkPos":{"!type":"fn(startTime: ?) -> ?"},"checkPosition":{"!type":"fn()"},"checkWidthTaskNameItem":{"!type":"fn()"},"clearPredTask":{"!type":"fn()"},"constructor":{"!type":"fn(taskInfo: ?, project: ?, chart: ?)"},"create":{"!type":"fn() -> fn"},"createConnectingLinesDS":{"!type":"fn() -> []"},"createConnectingLinesPN":{"!type":"fn() -> []"},"createTaskDescItem":{"!type":"fn() -> ?"},"createTaskItem":{"!type":"fn() -> ?"},"createTaskNameItem":{"!type":"fn() -> ?"},"createTreeImg":{"!type":"fn() -> ?"},"endMove":{"!type":"fn()"},"endResizeItem":{"!type":"fn()"},"getDateOnPosition":{"!type":"fn(position?: ?) -> +instance"},"getMaxPosPredChildTaskItem":{"!type":"fn() -> ?"},"getMaxPosPredChildTaskItemInTree":{"!type":"fn(task: ?) -> number"},"getMoveInfo":{"!type":"fn()"},"getResizeInfo":{"!type":"fn()"},"getTaskOwner":{"!type":"fn() -> ?"},"hideChildTasks":{"!type":"fn(task: ?)"},"hideDescTask":{"!type":"fn()"},"moveChildTaskItems":{"!type":"fn(task: ?, width: ?, moveChild: ?)"},"moveCurrentTaskItem":{"!type":"fn(width: ?, moveChild: ?)"},"moveDescTask":{"!type":"fn()"},"moveItem":{"!type":"fn(event: ?)"},"moveTaskItem":{"!type":"fn(posX: ?)"},"objKeyToStr":{"!type":"fn(obj: ?, delm: ?) -> string"},"postLoadData":{"!type":"fn()"},"refresh":{"!type":"fn() -> fn"},"refreshConnectingLinesDS":{"!type":"fn(arrLines: ?) -> ?"},"refreshTaskDesc":{"!type":"fn(divDesc: ?) -> ?"},"refreshTaskItem":{"!type":"fn(itemControl: ?) -> ?"},"resizeItem":{"!type":"fn(event: ?)"},"resizeTaskItem":{"!type":"fn(width: ?)"},"setDuration":{"!type":"fn(duration: ?) -> bool"},"setName":{"!type":"fn(name: ?)"},"setPercentCompleted":{"!type":"fn(percentage: ?) -> bool"},"setPreviousTask":{"!type":"fn(previousTaskId: ?) -> bool"},"setStartTime":{"!type":"fn(startTime: ?, shiftChild: ?) -> bool"},"setTaskOwner":{"!type":"fn(owner: ?) -> bool"},"shiftChildTask":{"!type":"fn(task: ?, height: ?)"},"shiftCurrentTasks":{"!type":"fn(task: ?, height: ?)"},"shiftNextTask":{"!type":"fn(task: ?, height: ?)"},"shiftTask":{"!type":"fn(task: ?, height: ?)"},"showChildTasks":{"!type":"fn(task: ?, isOpen: ?)"},"showDescTask":{"!type":"fn()"},"startMove":{"!type":"fn(event: ?)"},"startResize":{"!type":"fn(event: ?)"}}},"dojox/gantt/GanttProjectControl":{"!type":"fn()","prototype":{"adjustPanelTime":{"!type":"fn()"},"checkWidthProjectNameItem":{"!type":"fn()"},"constructor":{"!type":"fn(ganttChart: ?, projectItem: ?)"},"create":{"!type":"fn()"},"createDescrProject":{"!type":"fn() -> ?"},"createProjectItem":{"!type":"fn() -> ?"},"createProjectNameItem":{"!type":"fn() -> ?"},"deleteChildTask":{"!type":"fn(task: ?)"},"deleteTask":{"!type":"fn(id: ?)"},"getDescStr":{"!type":"fn() -> string"},"getDuration":{"!type":"fn() -> number"},"getPercentCompleted":{"!type":"fn() -> ?"},"getTaskById":{"!type":"fn(id: ?) -> ?"},"hideDescrProject":{"!type":"fn()"},"insertTask":{"!type":"fn(id: ?, name: ?, startTime: ?, duration: ?, percentage: ?, previousTaskId: ?, taskOwner: ?, parentTaskId: ?) -> bool"},"postLoadData":{"!type":"fn()"},"refresh":{"!type":"fn() -> fn"},"refreshDescrProject":{"!type":"fn(divDesc: ?) -> ?"},"refreshProjectItem":{"!type":"fn(projectItem: ?) -> ?"},"resizeProjectItem":{"!type":"fn(width: ?)"},"searchTaskInTree":{"!type":"fn(task: ?, id: ?) -> ?"},"setName":{"!type":"fn(name: ?)"},"setPercentCompleted":{"!type":"fn(percentage: ?) -> bool"},"shiftChildTasks":{"!type":"fn(task: ?, height: ?)"},"shiftDescrProject":{"!type":"fn()"},"shiftNextParentTask":{"!type":"fn(task: ?, height: ?)"},"shiftNextProject":{"!type":"fn(project: ?, height: ?)"},"shiftProject":{"!type":"fn(height: ?)"},"shiftProjectItem":{"!type":"fn()"},"shiftTask":{"!type":"fn(task: ?, height: ?)"},"showDescrProject":{"!type":"fn()"}}},"dojox/gantt/GanttResourceItem":{"!type":"fn()","prototype":{"buildOwnerTimeConsume":{"!type":"fn() -> ?"},"buildResource":{"!type":"fn() -> ?"},"checkWidthTaskNameItem":{"!type":"fn(taskNameItem: ?)"},"clearAll":{"!type":"fn()"},"clearData":{"!type":"fn()"},"clearItems":{"!type":"fn()"},"constructor":{"!type":"fn(ganttchart: ?)"},"create":{"!type":"fn()"},"createConnectingLinesPN":{"!type":"fn(parentNode: ?, currentNode: ?) -> []"},"createDetailedTaskEntry":{"!type":"fn(owner: ?, parentNode: ?, task: ?) -> []"},"createOwnerEntry":{"!type":"fn(owner: ?) -> fn"},"createOwnerItem":{"!type":"fn(owner: ?, posY: ?) -> ?"},"createOwnerNameItem":{"!type":"fn(owner: ?, posY: ?) -> ?"},"createPanelNamesOwners":{"!type":"fn() -> ?"},"createPanelOwners":{"!type":"fn() -> ?"},"createTaskItem":{"!type":"fn(task: ?, posY: ?) -> ?"},"createTaskNameItem":{"!type":"fn(owner: ?, posY: ?) -> ?"},"createTreeImg":{"!type":"fn(ownerNameItem: ?) -> ?"},"postAdjustment":{"!type":"fn()"},"reConstruct":{"!type":"fn()"},"refresh":{"!type":"fn()"},"refreshDetailedTaskEntry":{"!type":"fn(owner: ?, item: ?, task: ?)"},"refreshOwnerEntry":{"!type":"fn(owner: ?)"},"refreshOwnerItem":{"!type":"fn(owner: ?)"},"refreshTaskItem":{"!type":"fn(item: ?, task: ?)"},"styleOwnerItem":{"!type":"fn(tItem: ?, owner: ?, displayType: ?, topOffset: ?)"}}},"dojox/gantt/TabMenu":{"!type":"fn()","prototype":{"addItemMenuPanel":{"!type":"fn(tab: ?)"},"buildContent":{"!type":"fn()"},"clear":{"!type":"fn()"},"constructor":{"!type":"fn(chart: ?)"},"createMenuPanel":{"!type":"fn()"},"createTab":{"!type":"fn(id: ?, desc: ?, type: ?, showOInfo: ?, menu: ?, withDefaultValue: ?) -> +instance"},"createTabPanel":{"!type":"fn()"},"hide":{"!type":"fn()"},"show":{"!type":"fn(elem: ?, object: ?)"}}},"dojox/gantt/contextMenuTab":{"!type":"fn()","prototype":{"addAction":{"!type":"fn(handler: ?)"},"addChildTaskAction":{"!type":"fn()"},"addItem":{"!type":"fn(id: ?, name: ?, key: ?, required: ?)"},"addProjectAction":{"!type":"fn()"},"addSuccessorTaskAction":{"!type":"fn()"},"addTaskAction":{"!type":"fn()"},"constructor":{"!type":"fn(id: ?, description: ?, type: ?, showOInfo: ?, tabMenu: ?, withDefaultValue: ?)"},"cpProjectAction":{"!type":"fn()"},"cpUpdateAction":{"!type":"fn()"},"decodeDate":{"!type":"fn(dateStr: ?) -> +instance"},"deleteAction":{"!type":"fn()"},"deleteProjectAction":{"!type":"fn()"},"durationUpdateAction":{"!type":"fn()"},"encodeDate":{"!type":"fn(date: ?) -> string"},"hide":{"!type":"fn()"},"insertData":{"!type":"fn(content: ?, name: ?, value: ?)"},"ownerUpdateAction":{"!type":"fn()"},"preValueValidation":{"!type":"fn(items: ?) -> bool"},"ptUpdateAction":{"!type":"fn()"},"renameProjectAction":{"!type":"fn()"},"renameTaskAction":{"!type":"fn()"},"show":{"!type":"fn()"}}},"dojox/gauges/AnalogArcIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/AnalogIndicatorBase.prototype","!doc":"<p>An indicator for the AnalogGauge that draws a segment of arc.\nThe segment of arc starts at the start angle of the gauge and ends at the\nangle that corresponds to the value of the indicator.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{"!doc":"<p>The width of the indicator.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/AnalogIndicatorBase":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/_Indicator.prototype","!doc":"<p>An abstract base class for indicators that can be used in an AnalogGauge.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{"!doc":"<p>The width of the indicator.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/_Indicator":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>An indicator to be used in a gauge\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{"!doc":"<p>The width of the indicator.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(dontAnimate?: ?)","!doc":"<p>Performs the initial drawing of the indicator.</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/AnalogArrowIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/AnalogIndicatorBase.prototype","!doc":"<p>An indicator for the AnalogGauge that draws an arrow. The arrow is drawn on the angle that corresponds\nto the value of the indicator.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{"!doc":"<p>The width of the indicator.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/AnalogCircleIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/AnalogIndicatorBase.prototype","!doc":"<p>An indicator for the AnalogGauge that draws a circle. The center of the circle is positioned\non the circular gauge according to the value of the indicator. The circle has for radius the \nlength of the indicator. This indicator is mainly used to draw round ticks for the scale.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{"!doc":"<p>The width of the indicator.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/AnalogGauge":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/_Gauge.prototype","!doc":"<p>a gauge built using the dojox.gfx package.\n</p>\n","_attrPairNames":{},"_backgroundDefault":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_drag":{},"_focusManager":{},"_img":{},"_indicatorData":{},"_lastHover":{},"_overOverlay":{},"_rangeData":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"background":{"!doc":"<p>The color of the background.  This must be an object of one of two forms:\n{&#39;color&#39;: &#39;color-name&#39;}\nOR\n(for a gradient:)\n{&#39;type&#39;: &#39;linear&#39;, &#39;x1&#39;: 0, &#39;x2&#39;: 0, &#39;y1&#39;: 0, &#39;y2&#39;: 200, &#39;colors&#39;: [{offset: 0, color:&#39;#C0C0C0&#39;}, {offset: 1, color: &#39;#E0E0E0&#39;}] }</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cx":{"!doc":"<p>center of gauge x coordinate (default is gauge width / 2)</p>\n"},"cy":{"!doc":"<p>center of gauge x coordinate (default is gauge height / 2)</p>\n"},"defaultColors":{"!doc":"<p>Set of default colors to color ranges with.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"endAngle":{"!doc":"<p>angle (in degrees) for end of gauge (default is 90)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"gaugeContent":{},"height":{"!doc":"<p>The height of the gauge (default is 200)</p>\n"},"hideValues":{"!doc":"<p>Indicates whether the text boxes showing the value of the indicator (as text\ncontent) should be hidden or shown.  Default is not hidden, aka shown.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"image":{"!doc":"<p>Background image for gauge (default is no image)</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTicks":{"!doc":"<p>An object representing the tick marks that should be added to the gauge. Major tick marks have a text label\nindicating the value.  The object can have the following attributes (required are marked with a *):\n\n</p>\n<ul>\n<li>offset: the distance from the &#39;center&#39; of the gauge.  Used differently for Analog vs. Bar</li>\n<li>width: The width of the mark</li>\n<li>length: The length of the mark</li>\n<li>interval: The interval the ticks should be added on</li>\n<li>color: The color of the mark and text</li>\n<li><p>font: an object with any/all of the following parameters:</p>\n<p>  {family: &quot;Helvetica&quot;, style: &quot;italic&quot;, variant: &#39;small-caps&#39;, weight: &#39;bold&#39;, size: &quot;18pt&quot;}</p>\n</li>\n</ul>\n"},"max":{"!doc":"<p>The maximum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"min":{"!doc":"<p>The minimum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"minorTicks":{"!doc":"<p>An object of the same format as majorTicks, indicating where the minor (label-less) marks should be placed\nThe font parameter is ignored if provided since minor tick marks have no text label.</p>\n"},"observer":{},"orientation":{"!doc":"<p>The orientation of the gauge. The value can be &#39;clockwise&#39; or &#39;cclockwise&#39; (default is &#39;clockwise&#39;)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"radius":{"!doc":"<p>radius of gauge (default is smaller of cx-25 or cy-25)</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startAngle":{"!doc":"<p>angle (in degrees) for start of gauge (default is -90)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"surface":{"!doc":"<p>The GFX surface that the shapes are drawn on.  Can be accessed/used by indicators to draw themselves</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useRangeStyles":{"!doc":"<p>Indicates whether to use given css classes (dojoxGaugeRangeXX)\nto determine the color (and other style attributes?) of the ranges\nthis value should be the number of dojoxGaugeRange classes that are\ndefined, starting at dojoxGaugeRange1 (0 indicates falling to default\nhardcoded colors)</p>\n"},"useTooltip":{"!doc":"<p>Indicates whether tooltips should be displayed for ranges, indicators, etc.</p>\n"},"width":{"!doc":"<p>The width of the gauge (default is 300)</p>\n"},"addIndicator":{"!type":"fn(indicator: ?) -> +instance","!doc":"<p>This method is used to add an indicator to the gauge.</p>\n"},"addRange":{"!type":"fn(range: ?)","!doc":"<p>This method is used to add a range to the gauge.</p>\n"},"addRanges":{"!type":"fn(ranges: ?)","!doc":"<p>This method is used to add ranges to the gauge.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createSurface":{"!type":"fn()","!doc":"<p>Internal method used by the gauge to create the graphics surface area</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn()","!doc":"<p>This function is used to draw (or redraw) the gauge.</p>\n"},"drawBackground":{"!type":"fn(group: ?)","!doc":"<p>This function is used to draw (or redraw) the background of the gauge.</p>\n"},"drawForeground":{"!type":"fn(group: ?)","!doc":"<p>This function is used to draw (or redraw) the foreground of the gauge.</p>\n"},"drawRange":{"!type":"fn(group: ?, range: ?)","!doc":"<p>This function is used to draw (or redraw) a range</p>\n"},"drawText":{"!type":"fn(group: ?, txt: ?, x: ?, y: ?, align?: ?, color?: ?, font?: ?) -> ?","!doc":"<p>This function is used draw text onto the gauge.  The text object\nis also returned by the function so that may be removed later\nby calling removeText</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRangeUnderMouse":{"!type":"fn(e: ?) -> ?","!doc":"<p>Determines which range the mouse is currently over</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to move indicators</p>\n"},"handleMouseMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to show the tooltips</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support\nhover text</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support \nhover text</p>\n"},"handleTouchEnd":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStartIndicator":{"!type":"fn(indicator: ?, e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has children, i.e. if this.containerNode contains something.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"moveIndicatorToFront":{"!type":"fn(indicator: ?)","!doc":"<p>This function is used to move an indicator the the front (top)\nof the gauge</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeIndicator":{"!type":"fn(indicator: ?)","!doc":"<p>Removes the given indicator from the gauge by calling it&#39;s remove function \nand removing it from the local cache.</p>\n"},"removeText":{"!type":"fn(t: ?)","!doc":"<p>Removes a text element from the gauge.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setBackground":{"!type":"fn(background: ?)","!doc":"<p>This method is used to set the background of the gauge after it is created.</p>\n"},"setMajorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the major tick marks based on the passed object (expecting the same format\nas the majorTicks object documented above)</p>\n"},"setMinorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the minor tick marks based on the passed object (expecting the same format\nas the minorTicks object documented above)</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateTooltip":{"!type":"fn(txt: ?, e: ?)","!doc":"<p>Updates the tooltip for the gauge to display the given text.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/_Gauge":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>The abstract base class for gauges.\n</p>\n","_attrPairNames":{},"_backgroundDefault":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_defaultIndicator":{"!doc":"<p>Should be overridden by any extending classes and used to indicate what the &#39;default&#39; indicator is.\nThis object is used as the indicator when creating tick marks or when an anonymous object is passed into\naddIndicator.</p>\n"},"_drag":{},"_focusManager":{},"_img":{},"_indicatorData":{},"_lastHover":{},"_overOverlay":{},"_rangeData":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"background":{"!doc":"<p>The color of the background.  This must be an object of one of two forms:\n{&#39;color&#39;: &#39;color-name&#39;}\nOR\n(for a gradient:)\n{&#39;type&#39;: &#39;linear&#39;, &#39;x1&#39;: 0, &#39;x2&#39;: 0, &#39;y1&#39;: 0, &#39;y2&#39;: 200, &#39;colors&#39;: [{offset: 0, color:&#39;#C0C0C0&#39;}, {offset: 1, color: &#39;#E0E0E0&#39;}] }</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultColors":{"!doc":"<p>Set of default colors to color ranges with.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"gaugeContent":{},"height":{"!doc":"<p>The height of the gauge (default is 200)</p>\n"},"hideValues":{"!doc":"<p>Indicates whether the text boxes showing the value of the indicator (as text\ncontent) should be hidden or shown.  Default is not hidden, aka shown.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"image":{"!doc":"<p>Background image for gauge (default is no image)</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTicks":{"!doc":"<p>An object representing the tick marks that should be added to the gauge. Major tick marks have a text label\nindicating the value.  The object can have the following attributes (required are marked with a *):\n\n</p>\n<ul>\n<li>offset: the distance from the &#39;center&#39; of the gauge.  Used differently for Analog vs. Bar</li>\n<li>width: The width of the mark</li>\n<li>length: The length of the mark</li>\n<li>interval: The interval the ticks should be added on</li>\n<li>color: The color of the mark and text</li>\n<li><p>font: an object with any/all of the following parameters:</p>\n<p>  {family: &quot;Helvetica&quot;, style: &quot;italic&quot;, variant: &#39;small-caps&#39;, weight: &#39;bold&#39;, size: &quot;18pt&quot;}</p>\n</li>\n</ul>\n"},"max":{"!doc":"<p>The maximum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"min":{"!doc":"<p>The minimum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"minorTicks":{"!doc":"<p>An object of the same format as majorTicks, indicating where the minor (label-less) marks should be placed\nThe font parameter is ignored if provided since minor tick marks have no text label.</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"surface":{"!doc":"<p>The GFX surface that the shapes are drawn on.  Can be accessed/used by indicators to draw themselves</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useRangeStyles":{"!doc":"<p>Indicates whether to use given css classes (dojoxGaugeRangeXX)\nto determine the color (and other style attributes?) of the ranges\nthis value should be the number of dojoxGaugeRange classes that are\ndefined, starting at dojoxGaugeRange1 (0 indicates falling to default\nhardcoded colors)</p>\n"},"useTooltip":{"!doc":"<p>Indicates whether tooltips should be displayed for ranges, indicators, etc.</p>\n"},"width":{"!doc":"<p>The width of the gauge (default is 300)</p>\n"},"addIndicator":{"!type":"fn(indicator: ?) -> +instance","!doc":"<p>This method is used to add an indicator to the gauge.</p>\n"},"addRange":{"!type":"fn(range: ?)","!doc":"<p>This method is used to add a range to the gauge.</p>\n"},"addRanges":{"!type":"fn(ranges: ?)","!doc":"<p>This method is used to add ranges to the gauge.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createSurface":{"!type":"fn()","!doc":"<p>Internal method used by the gauge to create the graphics surface area</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn()","!doc":"<p>This function is used to draw (or redraw) the gauge.</p>\n"},"drawBackground":{"!type":"fn(group: ?)","!doc":"<p>This function is used to draw (or redraw) the background of the gauge.</p>\n"},"drawForeground":{"!type":"fn(group: ?)","!doc":"<p>This function is used to draw (or redraw) the foreground of the gauge.</p>\n"},"drawText":{"!type":"fn(group: ?, txt: ?, x: ?, y: ?, align?: ?, color?: ?, font?: ?) -> ?","!doc":"<p>This function is used draw text onto the gauge.  The text object\nis also returned by the function so that may be removed later\nby calling removeText</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to move indicators</p>\n"},"handleMouseMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to show the tooltips</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support\nhover text</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support \nhover text</p>\n"},"handleTouchEnd":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStartIndicator":{"!type":"fn(indicator: ?, e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has children, i.e. if this.containerNode contains something.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"moveIndicatorToFront":{"!type":"fn(indicator: ?)","!doc":"<p>This function is used to move an indicator the the front (top)\nof the gauge</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeIndicator":{"!type":"fn(indicator: ?)","!doc":"<p>Removes the given indicator from the gauge by calling it&#39;s remove function \nand removing it from the local cache.</p>\n"},"removeText":{"!type":"fn(t: ?)","!doc":"<p>Removes a text element from the gauge.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setBackground":{"!type":"fn(background: ?)","!doc":"<p>This method is used to set the background of the gauge after it is created.</p>\n"},"setMajorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the major tick marks based on the passed object (expecting the same format\nas the majorTicks object documented above)</p>\n"},"setMinorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the minor tick marks based on the passed object (expecting the same format\nas the minorTicks object documented above)</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateTooltip":{"!type":"fn(txt: ?, e: ?)","!doc":"<p>Updates the tooltip for the gauge to display the given text.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/Range":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>a range to be used in a _Gauge</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>the color of the range.  This must be an object of one of two forms:\n{&#39;color&#39;: &#39;color-name&#39;}\nOR\n(for a gradient:)\n{&#39;type&#39;: &#39;linear&#39;, &#39;colors&#39;: [{offset: 0, color:&#39;#C0C0C0&#39;}, {offset: 1, color: &#39;#E0E0E0&#39;}] }</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"high":{"!doc":"<p>the high value of the range</p>\n"},"hover":{"!doc":"<p>the text to put in the tooltip for the gauge</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"low":{"!doc":"<p>the low value of the range</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"size":{"!doc":"<p>for a circular gauge (such as an AnalogGauge), this dictates the size of the arc</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/AnalogLineIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/AnalogIndicatorBase.prototype","!doc":"<p>An indicator for the AnalogGauge that draws a segment of line that has for length the length of the indicator\nand that starts at an offset from the center of the gauge. The line is drawn on the angle that corresponds\nto the value of the indicator.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{"!doc":"<p>The width of the indicator.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/AnalogNeedleIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/AnalogIndicatorBase.prototype","!doc":"<p>An indicator for the AnalogGauge that draws a needle. The needle is drawn on the angle that corresponds\nto the value of the indicator.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{"!doc":"<p>The width of the indicator.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/BarCircleIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/BarLineIndicator.prototype","!doc":"<p>An indicator for the BarGauge that draws a circle at a position that corresponds to the\nindicator value. This indicator is mainly used to draw round ticks for the scale.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/BarLineIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/_Indicator.prototype","!doc":"<p>An indicator for the BarGauge that draws a segment a line corresponding to the indicator value. </p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/BarGauge":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/_Gauge.prototype","!doc":"<p>a bar graph built using the dojox.gfx package.\n</p>\n","_attrPairNames":{},"_backgroundDefault":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_drag":{},"_focusManager":{},"_img":{},"_indicatorData":{},"_lastHover":{},"_overOverlay":{},"_rangeData":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"background":{"!doc":"<p>The color of the background.  This must be an object of one of two forms:\n{&#39;color&#39;: &#39;color-name&#39;}\nOR\n(for a gradient:)\n{&#39;type&#39;: &#39;linear&#39;, &#39;x1&#39;: 0, &#39;x2&#39;: 0, &#39;y1&#39;: 0, &#39;y2&#39;: 200, &#39;colors&#39;: [{offset: 0, color:&#39;#C0C0C0&#39;}, {offset: 1, color: &#39;#E0E0E0&#39;}] }</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dataHeight":{"!doc":"<p>height of data area (default is bar graph width - 10)</p>\n"},"dataWidth":{"!doc":"<p>width of data area (default is bar graph width - 10)</p>\n"},"dataX":{"!doc":"<p>x position of data area (default 5)</p>\n"},"dataY":{"!doc":"<p>y position of data area (default 5)</p>\n"},"defaultColors":{"!doc":"<p>Set of default colors to color ranges with.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"gaugeContent":{},"height":{"!doc":"<p>The height of the gauge (default is 200)</p>\n"},"hideValues":{"!doc":"<p>Indicates whether the text boxes showing the value of the indicator (as text\ncontent) should be hidden or shown.  Default is not hidden, aka shown.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"image":{"!doc":"<p>Background image for gauge (default is no image)</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTicks":{"!doc":"<p>An object representing the tick marks that should be added to the gauge. Major tick marks have a text label\nindicating the value.  The object can have the following attributes (required are marked with a *):\n\n</p>\n<ul>\n<li>offset: the distance from the &#39;center&#39; of the gauge.  Used differently for Analog vs. Bar</li>\n<li>width: The width of the mark</li>\n<li>length: The length of the mark</li>\n<li>interval: The interval the ticks should be added on</li>\n<li>color: The color of the mark and text</li>\n<li><p>font: an object with any/all of the following parameters:</p>\n<p>  {family: &quot;Helvetica&quot;, style: &quot;italic&quot;, variant: &#39;small-caps&#39;, weight: &#39;bold&#39;, size: &quot;18pt&quot;}</p>\n</li>\n</ul>\n"},"max":{"!doc":"<p>The maximum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"min":{"!doc":"<p>The minimum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"minorTicks":{"!doc":"<p>An object of the same format as majorTicks, indicating where the minor (label-less) marks should be placed\nThe font parameter is ignored if provided since minor tick marks have no text label.</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"surface":{"!doc":"<p>The GFX surface that the shapes are drawn on.  Can be accessed/used by indicators to draw themselves</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useRangeStyles":{"!doc":"<p>Indicates whether to use given css classes (dojoxGaugeRangeXX)\nto determine the color (and other style attributes?) of the ranges\nthis value should be the number of dojoxGaugeRange classes that are\ndefined, starting at dojoxGaugeRange1 (0 indicates falling to default\nhardcoded colors)</p>\n"},"useTooltip":{"!doc":"<p>Indicates whether tooltips should be displayed for ranges, indicators, etc.</p>\n"},"width":{"!doc":"<p>The width of the gauge (default is 300)</p>\n"},"addIndicator":{"!type":"fn(indicator: ?) -> +instance","!doc":"<p>This method is used to add an indicator to the gauge.</p>\n"},"addRange":{"!type":"fn(range: ?)","!doc":"<p>This method is used to add a range to the gauge.</p>\n"},"addRanges":{"!type":"fn(ranges: ?)","!doc":"<p>This method is used to add ranges to the gauge.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createSurface":{"!type":"fn()","!doc":"<p>Internal method used by the gauge to create the graphics surface area</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn()","!doc":"<p>This function is used to draw (or redraw) the gauge.</p>\n"},"drawBackground":{"!type":"fn(group: ?)","!doc":"<p>This function is used to draw (or redraw) the background of the gauge.</p>\n"},"drawForeground":{"!type":"fn(group: ?)","!doc":"<p>This function is used to draw (or redraw) the foreground of the gauge.</p>\n"},"drawRange":{"!type":"fn(group: ?, range: ?)","!doc":"<p>This function is used to draw (or redraw) a range</p>\n"},"drawText":{"!type":"fn(group: ?, txt: ?, x: ?, y: ?, align?: ?, color?: ?, font?: ?) -> ?","!doc":"<p>This function is used draw text onto the gauge.  The text object\nis also returned by the function so that may be removed later\nby calling removeText</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRangeUnderMouse":{"!type":"fn(e: ?) -> ?","!doc":"<p>Determines which range the mouse is currently over</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to move indicators</p>\n"},"handleMouseMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to show the tooltips</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support\nhover text</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support \nhover text</p>\n"},"handleTouchEnd":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStartIndicator":{"!type":"fn(indicator: ?, e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has children, i.e. if this.containerNode contains something.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"moveIndicatorToFront":{"!type":"fn(indicator: ?)","!doc":"<p>This function is used to move an indicator the the front (top)\nof the gauge</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeIndicator":{"!type":"fn(indicator: ?)","!doc":"<p>Removes the given indicator from the gauge by calling it&#39;s remove function \nand removing it from the local cache.</p>\n"},"removeText":{"!type":"fn(t: ?)","!doc":"<p>Removes a text element from the gauge.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setBackground":{"!type":"fn(background: ?)","!doc":"<p>This method is used to set the background of the gauge after it is created.</p>\n"},"setMajorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the major tick marks based on the passed object (expecting the same format\nas the majorTicks object documented above)</p>\n"},"setMinorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the minor tick marks based on the passed object (expecting the same format\nas the minorTicks object documented above)</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateTooltip":{"!type":"fn(txt: ?, e: ?)","!doc":"<p>Updates the tooltip for the gauge to display the given text.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/BarIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/BarLineIndicator.prototype","!doc":"<p>An indicator for the BarGauge that draws a bar corresponding to the indicator value.    </p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/GlossyCircularGauge":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/GlossyCircularGaugeBase.prototype","!doc":"<p>Represents a circular gauge with a glossy appearance.\n</p>\n","_attrPairNames":{},"_backgroundDefault":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_designCx":{},"_designCy":{},"_designHeight":{},"_designTextIndicatorX":{},"_designTextIndicatorY":{},"_designWidth":{},"_drag":{},"_focusManager":{},"_font":{},"_img":{},"_indicatorData":{},"_lastHover":{},"_majorTicksLength":{"!doc":"<p>Major tick size, at design</p>\n"},"_majorTicksOffset":{"!doc":"<p>Distance, at design, from gauge&#39;s center to major ticks</p>\n"},"_minorTicksLength":{"!doc":"<p>Minor tick size, at design</p>\n"},"_minorTicksOffset":{"!doc":"<p>Distance, at design, from gauge&#39;s center to minor ticks</p>\n"},"_needle":{"!doc":"<p>the needle of this circular gauge</p>\n"},"_overOverlay":{},"_range":{"!doc":"<p>the range of this gauge</p>\n"},"_rangeData":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_textIndicator":{"!doc":"<p>the text displaying the gauge&#39;s value</p>\n"},"_textIndicatorAdded":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"background":{"!doc":"<p>The color of the background.  This must be an object of one of two forms:\n{&#39;color&#39;: &#39;color-name&#39;}\nOR\n(for a gradient:)\n{&#39;type&#39;: &#39;linear&#39;, &#39;x1&#39;: 0, &#39;x2&#39;: 0, &#39;y1&#39;: 0, &#39;y2&#39;: 200, &#39;colors&#39;: [{offset: 0, color:&#39;#C0C0C0&#39;}, {offset: 1, color: &#39;#E0E0E0&#39;}] }</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The main color of the gauge.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cx":{"!doc":"<p>center of gauge x coordinate (default is gauge width / 2)</p>\n"},"cy":{"!doc":"<p>center of gauge x coordinate (default is gauge height / 2)</p>\n"},"defaultColors":{"!doc":"<p>Set of default colors to color ranges with.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"endAngle":{"!doc":"<p>angle (in degrees) for end of gauge (default is 90)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge</p>\n"},"gaugeContent":{},"height":{"!doc":"<p>The height of the gauge (default is 200)</p>\n"},"hideValues":{"!doc":"<p>Indicates whether the text boxes showing the value of the indicator (as text\ncontent) should be hidden or shown.  Default is not hidden, aka shown.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"image":{"!doc":"<p>Background image for gauge (default is no image)</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTicks":{"!doc":"<p>An object representing the tick marks that should be added to the gauge. Major tick marks have a text label\nindicating the value.  The object can have the following attributes (required are marked with a *):\n\n</p>\n<ul>\n<li>offset: the distance from the &#39;center&#39; of the gauge.  Used differently for Analog vs. Bar</li>\n<li>width: The width of the mark</li>\n<li>length: The length of the mark</li>\n<li>interval: The interval the ticks should be added on</li>\n<li>color: The color of the mark and text</li>\n<li><p>font: an object with any/all of the following parameters:</p>\n<p>  {family: &quot;Helvetica&quot;, style: &quot;italic&quot;, variant: &#39;small-caps&#39;, weight: &#39;bold&#39;, size: &quot;18pt&quot;}</p>\n</li>\n</ul>\n"},"majorTicksColor":{"!doc":"<p>Color of major tick marks</p>\n"},"majorTicksInterval":{"!doc":"<p>Interval between major ticks</p>\n"},"majorTicksLabelPlacement":{"!doc":"<p>Placement of major tick labels</p>\n"},"max":{"!doc":"<p>The maximum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"min":{"!doc":"<p>The minimum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"minorTicks":{"!doc":"<p>An object of the same format as majorTicks, indicating where the minor (label-less) marks should be placed\nThe font parameter is ignored if provided since minor tick marks have no text label.</p>\n"},"minorTicksColor":{"!doc":"<p>Color of minor tick marks</p>\n"},"minorTicksInterval":{"!doc":"<p>Interval between minor ticks</p>\n"},"needleColor":{"!doc":"<p>The main color of the needle.</p>\n"},"noChange":{"!doc":"<p>Indicates if the gauge reacts to touch events</p>\n"},"observer":{},"orientation":{"!doc":"<p>The orientation of the gauge. The value can be &#39;clockwise&#39; or &#39;cclockwise&#39; (default is &#39;clockwise&#39;)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"radius":{"!doc":"<p>radius of gauge (default is smaller of cx-25 or cy-25)</p>\n"},"scalePrecision":{"!doc":"<p>The precision for the formatting of numbers in the scale (default is 0)</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startAngle":{"!doc":"<p>angle (in degrees) for start of gauge (default is -90)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"surface":{"!doc":"<p>The GFX surface that the shapes are drawn on.  Can be accessed/used by indicators to draw themselves</p>\n"},"textIndicatorColor":{"!doc":"<p>The color of the text indicator</p>\n"},"textIndicatorFont":{"!doc":"<p>The font of the text indicator</p>\n"},"textIndicatorPrecision":{"!doc":"<p>The precision for the formatting of numbers in the text indicator (default is 0)</p>\n"},"textIndicatorVisible":{"!doc":"<p>Indicates if the text indicator is visible</p>\n"},"title":{"!doc":"<p>The title displayed in the needle&#39;s tooltip</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useRangeStyles":{"!doc":"<p>Indicates whether to use given css classes (dojoxGaugeRangeXX)\nto determine the color (and other style attributes?) of the ranges\nthis value should be the number of dojoxGaugeRange classes that are\ndefined, starting at dojoxGaugeRange1 (0 indicates falling to default\nhardcoded colors)</p>\n"},"useTooltip":{"!doc":"<p>Indicates whether tooltips should be displayed for ranges, indicators, etc.</p>\n"},"value":{"!doc":"<p>The value of the gauge.</p>\n"},"width":{"!doc":"<p>The width of the gauge (default is 300)</p>\n"},"addIndicator":{"!type":"fn(indicator: ?) -> +instance","!doc":"<p>This method is used to add an indicator to the gauge.</p>\n"},"addRange":{"!type":"fn(range: ?)","!doc":"<p>This method is used to add a range to the gauge.</p>\n"},"addRanges":{"!type":"fn(ranges: ?)","!doc":"<p>This method is used to add ranges to the gauge.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Creates a new GlossyCircularGauge.</p>\n"},"createSurface":{"!type":"fn()","!doc":"<p>Internal method used by the gauge to create the graphics surface area</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn()","!doc":"<p>This function is used to draw (or redraw) the gauge.</p>\n"},"drawBackground":{"!type":"fn(group: ?)","!doc":"<p>Draws the background of the gauge.</p>\n"},"drawForeground":{"!type":"fn(group: ?)","!doc":"<p>Draws the foreground of the gauge.</p>\n"},"drawRange":{"!type":"fn(group: ?, range: ?)","!doc":"<p>This function is used to draw (or redraw) a range</p>\n"},"drawText":{"!type":"fn(group: ?, txt: ?, x: ?, y: ?, align?: ?, color?: ?, font?: ?) -> ?","!doc":"<p>This function is used draw text onto the gauge.  The text object\nis also returned by the function so that may be removed later\nby calling removeText</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getMajorTicksLength":{"!type":"fn() -> ?","!doc":"<p>Returns the size of the major ticks.</p>\n"},"getMajorTicksOffset":{"!type":"fn() -> ?","!doc":"<p>Return the distance from gauge&#39;s center to major ticks</p>\n"},"getMinorTicksLength":{"!type":"fn() -> ?","!doc":"<p>Return the size of the minor ticks.</p>\n"},"getMinorTicksOffset":{"!type":"fn() -> ?","!doc":"<p>Returns the distance from gauge&#39;s center to minor ticks</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRangeUnderMouse":{"!type":"fn(e: ?) -> ?","!doc":"<p>Determines which range the mouse is currently over</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to move indicators</p>\n"},"handleMouseMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to show the tooltips</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support\nhover text</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support \nhover text</p>\n"},"handleTouchEnd":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStartIndicator":{"!type":"fn(indicator: ?, e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has children, i.e. if this.containerNode contains something.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"moveIndicatorToFront":{"!type":"fn(indicator: ?)","!doc":"<p>This function is used to move an indicator the the front (top)\nof the gauge</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeIndicator":{"!type":"fn(indicator: ?)","!doc":"<p>Removes the given indicator from the gauge by calling it&#39;s remove function \nand removing it from the local cache.</p>\n"},"removeText":{"!type":"fn(t: ?)","!doc":"<p>Removes a text element from the gauge.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setBackground":{"!type":"fn(background: ?)","!doc":"<p>This method is used to set the background of the gauge after it is created.</p>\n"},"setMajorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the major tick marks based on the passed object (expecting the same format\nas the majorTicks object documented above)</p>\n"},"setMajorTicksLength":{"!type":"fn(length: ?) -> fn","!doc":"<p>Sets the size of the major ticks.</p>\n"},"setMajorTicksOffset":{"!type":"fn(offset: ?) -> fn","!doc":"<p>Sets the distance from gauge&#39;s center to major ticks</p>\n"},"setMinorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the minor tick marks based on the passed object (expecting the same format\nas the minorTicks object documented above)</p>\n"},"setMinorTicksLength":{"!type":"fn(length: ?) -> fn","!doc":"<p>Sets the size of the minor ticks.</p>\n"},"setMinorTicksOffset":{"!type":"fn(offset: ?) -> fn","!doc":"<p>Sets the distance from gauge&#39;s center to minor ticks</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Overrides AnalogGauge.startup</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateTooltip":{"!type":"fn(txt: ?, e: ?)","!doc":"<p>Updates the tooltip for the gauge to display the given text.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/GlossyCircularGaugeBase":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/AnalogGauge.prototype","!doc":"<p>The base class for GlossyCircularGauge and GlossySemiCircularGauge.</p>\n","_attrPairNames":{},"_backgroundDefault":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_drag":{},"_focusManager":{},"_font":{},"_img":{},"_indicatorData":{},"_lastHover":{},"_majorTicksLength":{"!doc":"<p>Major tick size, at design</p>\n"},"_majorTicksOffset":{"!doc":"<p>Distance, at design, from gauge&#39;s center to major ticks</p>\n"},"_minorTicksLength":{"!doc":"<p>Minor tick size, at design</p>\n"},"_minorTicksOffset":{"!doc":"<p>Distance, at design, from gauge&#39;s center to minor ticks</p>\n"},"_needle":{"!doc":"<p>the needle of this circular gauge</p>\n"},"_overOverlay":{},"_range":{"!doc":"<p>the range of this gauge</p>\n"},"_rangeData":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_textIndicator":{"!doc":"<p>the text displaying the gauge&#39;s value</p>\n"},"_textIndicatorAdded":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"background":{"!doc":"<p>The color of the background.  This must be an object of one of two forms:\n{&#39;color&#39;: &#39;color-name&#39;}\nOR\n(for a gradient:)\n{&#39;type&#39;: &#39;linear&#39;, &#39;x1&#39;: 0, &#39;x2&#39;: 0, &#39;y1&#39;: 0, &#39;y2&#39;: 200, &#39;colors&#39;: [{offset: 0, color:&#39;#C0C0C0&#39;}, {offset: 1, color: &#39;#E0E0E0&#39;}] }</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The main color of the gauge.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cx":{"!doc":"<p>center of gauge x coordinate (default is gauge width / 2)</p>\n"},"cy":{"!doc":"<p>center of gauge x coordinate (default is gauge height / 2)</p>\n"},"defaultColors":{"!doc":"<p>Set of default colors to color ranges with.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"endAngle":{"!doc":"<p>angle (in degrees) for end of gauge (default is 90)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge</p>\n"},"gaugeContent":{},"height":{"!doc":"<p>The height of the gauge (default is 200)</p>\n"},"hideValues":{"!doc":"<p>Indicates whether the text boxes showing the value of the indicator (as text\ncontent) should be hidden or shown.  Default is not hidden, aka shown.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"image":{"!doc":"<p>Background image for gauge (default is no image)</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTicks":{"!doc":"<p>An object representing the tick marks that should be added to the gauge. Major tick marks have a text label\nindicating the value.  The object can have the following attributes (required are marked with a *):\n\n</p>\n<ul>\n<li>offset: the distance from the &#39;center&#39; of the gauge.  Used differently for Analog vs. Bar</li>\n<li>width: The width of the mark</li>\n<li>length: The length of the mark</li>\n<li>interval: The interval the ticks should be added on</li>\n<li>color: The color of the mark and text</li>\n<li><p>font: an object with any/all of the following parameters:</p>\n<p>  {family: &quot;Helvetica&quot;, style: &quot;italic&quot;, variant: &#39;small-caps&#39;, weight: &#39;bold&#39;, size: &quot;18pt&quot;}</p>\n</li>\n</ul>\n"},"majorTicksColor":{"!doc":"<p>Color of major tick marks</p>\n"},"majorTicksInterval":{"!doc":"<p>Interval between major ticks</p>\n"},"majorTicksLabelPlacement":{"!doc":"<p>Placement of major tick labels</p>\n"},"max":{"!doc":"<p>The maximum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"min":{"!doc":"<p>The minimum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"minorTicks":{"!doc":"<p>An object of the same format as majorTicks, indicating where the minor (label-less) marks should be placed\nThe font parameter is ignored if provided since minor tick marks have no text label.</p>\n"},"minorTicksColor":{"!doc":"<p>Color of minor tick marks</p>\n"},"minorTicksInterval":{"!doc":"<p>Interval between minor ticks</p>\n"},"needleColor":{"!doc":"<p>The main color of the needle.</p>\n"},"noChange":{"!doc":"<p>Indicates if the gauge reacts to touch events</p>\n"},"observer":{},"orientation":{"!doc":"<p>The orientation of the gauge. The value can be &#39;clockwise&#39; or &#39;cclockwise&#39; (default is &#39;clockwise&#39;)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"radius":{"!doc":"<p>radius of gauge (default is smaller of cx-25 or cy-25)</p>\n"},"scalePrecision":{"!doc":"<p>The precision for the formatting of numbers in the scale (default is 0)</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startAngle":{"!doc":"<p>angle (in degrees) for start of gauge (default is -90)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"surface":{"!doc":"<p>The GFX surface that the shapes are drawn on.  Can be accessed/used by indicators to draw themselves</p>\n"},"textIndicatorColor":{"!doc":"<p>The color of the text indicator</p>\n"},"textIndicatorFont":{"!doc":"<p>The font of the text indicator</p>\n"},"textIndicatorPrecision":{"!doc":"<p>The precision for the formatting of numbers in the text indicator (default is 0)</p>\n"},"textIndicatorVisible":{"!doc":"<p>Indicates if the text indicator is visible</p>\n"},"title":{"!doc":"<p>The title displayed in the needle&#39;s tooltip</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useRangeStyles":{"!doc":"<p>Indicates whether to use given css classes (dojoxGaugeRangeXX)\nto determine the color (and other style attributes?) of the ranges\nthis value should be the number of dojoxGaugeRange classes that are\ndefined, starting at dojoxGaugeRange1 (0 indicates falling to default\nhardcoded colors)</p>\n"},"useTooltip":{"!doc":"<p>Indicates whether tooltips should be displayed for ranges, indicators, etc.</p>\n"},"value":{"!doc":"<p>The value of the gauge.</p>\n"},"width":{"!doc":"<p>The width of the gauge (default is 300)</p>\n"},"addIndicator":{"!type":"fn(indicator: ?) -> +instance","!doc":"<p>This method is used to add an indicator to the gauge.</p>\n"},"addRange":{"!type":"fn(range: ?)","!doc":"<p>This method is used to add a range to the gauge.</p>\n"},"addRanges":{"!type":"fn(ranges: ?)","!doc":"<p>This method is used to add ranges to the gauge.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"createSurface":{"!type":"fn()","!doc":"<p>Internal method used by the gauge to create the graphics surface area</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn()","!doc":"<p>This function is used to draw (or redraw) the gauge.</p>\n"},"drawBackground":{"!type":"fn(group: ?)","!doc":"<p>This function is used to draw (or redraw) the background of the gauge.</p>\n"},"drawForeground":{"!type":"fn(group: ?)","!doc":"<p>This function is used to draw (or redraw) the foreground of the gauge.</p>\n"},"drawRange":{"!type":"fn(group: ?, range: ?)","!doc":"<p>This function is used to draw (or redraw) a range</p>\n"},"drawText":{"!type":"fn(group: ?, txt: ?, x: ?, y: ?, align?: ?, color?: ?, font?: ?) -> ?","!doc":"<p>This function is used draw text onto the gauge.  The text object\nis also returned by the function so that may be removed later\nby calling removeText</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getMajorTicksLength":{"!type":"fn() -> ?","!doc":"<p>Returns the size of the major ticks.</p>\n"},"getMajorTicksOffset":{"!type":"fn() -> ?","!doc":"<p>Return the distance from gauge&#39;s center to major ticks</p>\n"},"getMinorTicksLength":{"!type":"fn() -> ?","!doc":"<p>Return the size of the minor ticks.</p>\n"},"getMinorTicksOffset":{"!type":"fn() -> ?","!doc":"<p>Returns the distance from gauge&#39;s center to minor ticks</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRangeUnderMouse":{"!type":"fn(e: ?) -> ?","!doc":"<p>Determines which range the mouse is currently over</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to move indicators</p>\n"},"handleMouseMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to show the tooltips</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support\nhover text</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support \nhover text</p>\n"},"handleTouchEnd":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStartIndicator":{"!type":"fn(indicator: ?, e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has children, i.e. if this.containerNode contains something.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"moveIndicatorToFront":{"!type":"fn(indicator: ?)","!doc":"<p>This function is used to move an indicator the the front (top)\nof the gauge</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeIndicator":{"!type":"fn(indicator: ?)","!doc":"<p>Removes the given indicator from the gauge by calling it&#39;s remove function \nand removing it from the local cache.</p>\n"},"removeText":{"!type":"fn(t: ?)","!doc":"<p>Removes a text element from the gauge.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setBackground":{"!type":"fn(background: ?)","!doc":"<p>This method is used to set the background of the gauge after it is created.</p>\n"},"setMajorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the major tick marks based on the passed object (expecting the same format\nas the majorTicks object documented above)</p>\n"},"setMajorTicksLength":{"!type":"fn(length: ?) -> fn","!doc":"<p>Sets the size of the major ticks.</p>\n"},"setMajorTicksOffset":{"!type":"fn(offset: ?) -> fn","!doc":"<p>Sets the distance from gauge&#39;s center to major ticks</p>\n"},"setMinorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the minor tick marks based on the passed object (expecting the same format\nas the minorTicks object documented above)</p>\n"},"setMinorTicksLength":{"!type":"fn(length: ?) -> fn","!doc":"<p>Sets the size of the minor ticks.</p>\n"},"setMinorTicksOffset":{"!type":"fn(offset: ?) -> fn","!doc":"<p>Sets the distance from gauge&#39;s center to minor ticks</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Overrides AnalogGauge.startup</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateTooltip":{"!type":"fn(txt: ?, e: ?)","!doc":"<p>Updates the tooltip for the gauge to display the given text.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/TextIndicator":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/_Indicator.prototype","!doc":"<p>A gauge indicator the simply draws its value as text.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"align":{"!doc":"<p>The horizontal alignment of the text, the value can be &#39;middle&#39; (the default), &#39;left&#39; or &#39;right&#39;</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"fixedPrecision":{"!doc":"<p>Indicates that the number is displayed in fixed precision or not (precision is defined by the &#39;precision&#39; property (default is true).</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{"!doc":"<p>The interactionMode can have two values: &quot;indicator&quot; (the default) or &quot;gauge&quot;.\nWhen the value is &quot;indicator&quot;, the user must click on the indicator to change the value.\nWhen the value is &quot;gauge&quot;, the user can click on the gauge to change the indicator value.\nIf a gauge contains several indicators with the indicatorMode property set to &quot;gauge&quot;, then\nonly the first indicator will be moved when clicking the gauge.</p>\n"},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"precision":{"!doc":"<p>The number of tailing digits to display the value of the indicator when the &#39;fixedPrecision&#39; property is set to true (default is 0).</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{"!doc":"<p>The width of the indicator.</p>\n"},"x":{"!doc":"<p>The x coordinate of the indicator</p>\n"},"y":{"!doc":"<p>The y coordinate of the indicator</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/GlossyCircularGaugeNeedle":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/AnalogIndicatorBase.prototype","!doc":"<p>The needle for the dojox.gauges.GlossyCircularGauge and\ndojox.gauges.GlossySemiCircularGauge.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{"!doc":"<p>The width of the indicator.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/GlossyHorizontalGauge":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/BarGauge.prototype","!doc":"<p>Represents an horizontal bar gauge with a glossy appearance.\n</p>\n","_attrPairNames":{},"_backgroundDefault":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_designHeight":{},"_drag":{},"_focusManager":{},"_font":{},"_img":{},"_indicatorData":{},"_lastHover":{},"_majorTicksLength":{"!doc":"<p>Major tick size, at design</p>\n"},"_margin":{},"_maxBorderWidth":{},"_minBorderWidth":{},"_minorTicksLength":{"!doc":"<p>Minor tick size, at design</p>\n"},"_overOverlay":{},"_rangeData":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tickLabelOffset":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"background":{"!doc":"<p>The color of the background.  This must be an object of one of two forms:\n{&#39;color&#39;: &#39;color-name&#39;}\nOR\n(for a gradient:)\n{&#39;type&#39;: &#39;linear&#39;, &#39;x1&#39;: 0, &#39;x2&#39;: 0, &#39;y1&#39;: 0, &#39;y2&#39;: 200, &#39;colors&#39;: [{offset: 0, color:&#39;#C0C0C0&#39;}, {offset: 1, color: &#39;#E0E0E0&#39;}] }</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The main color of the gauge.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dataHeight":{"!doc":"<p>height of data area (default is bar graph width - 10)</p>\n"},"dataWidth":{"!doc":"<p>width of data area (default is bar graph width - 10)</p>\n"},"dataX":{"!doc":"<p>x position of data area (default 5)</p>\n"},"dataY":{"!doc":"<p>y position of data area (default 5)</p>\n"},"defaultColors":{"!doc":"<p>Set of default colors to color ranges with.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge</p>\n"},"gaugeContent":{},"height":{"!doc":"<p>The height of the gauge (default is 200)</p>\n"},"hideValues":{"!doc":"<p>Indicates whether the text boxes showing the value of the indicator (as text\ncontent) should be hidden or shown.  Default is not hidden, aka shown.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"image":{"!doc":"<p>Background image for gauge (default is no image)</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTicks":{"!doc":"<p>An object representing the tick marks that should be added to the gauge. Major tick marks have a text label\nindicating the value.  The object can have the following attributes (required are marked with a *):\n\n</p>\n<ul>\n<li>offset: the distance from the &#39;center&#39; of the gauge.  Used differently for Analog vs. Bar</li>\n<li>width: The width of the mark</li>\n<li>length: The length of the mark</li>\n<li>interval: The interval the ticks should be added on</li>\n<li>color: The color of the mark and text</li>\n<li><p>font: an object with any/all of the following parameters:</p>\n<p>  {family: &quot;Helvetica&quot;, style: &quot;italic&quot;, variant: &#39;small-caps&#39;, weight: &#39;bold&#39;, size: &quot;18pt&quot;}</p>\n</li>\n</ul>\n"},"majorTicksColor":{"!doc":"<p>Color of major tick marks</p>\n"},"majorTicksInterval":{"!doc":"<p>Interval between major ticks</p>\n"},"markerColor":{},"max":{"!doc":"<p>The maximum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"min":{"!doc":"<p>The minimum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"minorTicks":{"!doc":"<p>An object of the same format as majorTicks, indicating where the minor (label-less) marks should be placed\nThe font parameter is ignored if provided since minor tick marks have no text label.</p>\n"},"minorTicksColor":{"!doc":"<p>Color of minor tick marks</p>\n"},"minorTicksInterval":{"!doc":"<p>Interval between minor ticks</p>\n"},"noChange":{"!doc":"<p>Indicates if the gauge reacts to touch events</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scalePrecision":{"!doc":"<p>The precision for the formatting of numbers in the scale (default is 0)</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"surface":{"!doc":"<p>The GFX surface that the shapes are drawn on.  Can be accessed/used by indicators to draw themselves</p>\n"},"title":{"!doc":"<p>The title displayed in the needle&#39;s tooltip</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useRangeStyles":{"!doc":"<p>Indicates whether to use given css classes (dojoxGaugeRangeXX)\nto determine the color (and other style attributes?) of the ranges\nthis value should be the number of dojoxGaugeRange classes that are\ndefined, starting at dojoxGaugeRange1 (0 indicates falling to default\nhardcoded colors)</p>\n"},"useTooltip":{"!doc":"<p>Indicates whether tooltips should be displayed for ranges, indicators, etc.</p>\n"},"value":{"!doc":"<p>The value of the gauge.</p>\n"},"width":{"!doc":"<p>The width of the gauge (default is 300)</p>\n"},"addIndicator":{"!type":"fn(indicator: ?) -> +instance","!doc":"<p>This method is used to add an indicator to the gauge.</p>\n"},"addRange":{"!type":"fn(range: ?)","!doc":"<p>This method is used to add a range to the gauge.</p>\n"},"addRanges":{"!type":"fn(ranges: ?)","!doc":"<p>This method is used to add ranges to the gauge.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"createSurface":{"!type":"fn()","!doc":"<p>Internal method used by the gauge to create the graphics surface area</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn()","!doc":"<p>This function is used to draw (or redraw) the gauge.</p>\n"},"drawBackground":{"!type":"fn(group: ?)","!doc":"<p>Draws the background of the gauge</p>\n"},"drawForeground":{"!type":"fn(group: ?)","!doc":"<p>This function is used to draw (or redraw) the foreground of the gauge.</p>\n"},"drawRange":{"!type":"fn(group: ?, range: ?)","!doc":"<p>This function is used to draw (or redraw) a range</p>\n"},"drawText":{"!type":"fn(group: ?, txt: ?, x: ?, y: ?, align?: ?, color?: ?, font?: ?) -> ?","!doc":"<p>This function is used draw text onto the gauge.  The text object\nis also returned by the function so that may be removed later\nby calling removeText</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getMajorTicksLength":{"!type":"fn() -> ?","!doc":"<p>Returns the size of the major ticks.</p>\n"},"getMinorTicksLength":{"!type":"fn() -> ?","!doc":"<p>Gets the size of the minor ticks.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRangeUnderMouse":{"!type":"fn(e: ?) -> ?","!doc":"<p>Determines which range the mouse is currently over</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to move indicators</p>\n"},"handleMouseMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to show the tooltips</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support\nhover text</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support \nhover text</p>\n"},"handleTouchEnd":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStartIndicator":{"!type":"fn(indicator: ?, e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has children, i.e. if this.containerNode contains something.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"moveIndicatorToFront":{"!type":"fn(indicator: ?)","!doc":"<p>This function is used to move an indicator the the front (top)\nof the gauge</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeIndicator":{"!type":"fn(indicator: ?)","!doc":"<p>Removes the given indicator from the gauge by calling it&#39;s remove function \nand removing it from the local cache.</p>\n"},"removeText":{"!type":"fn(t: ?)","!doc":"<p>Removes a text element from the gauge.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setBackground":{"!type":"fn(background: ?)","!doc":"<p>This method is used to set the background of the gauge after it is created.</p>\n"},"setMajorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the major tick marks based on the passed object (expecting the same format\nas the majorTicks object documented above)</p>\n"},"setMajorTicksLength":{"!type":"fn(length: ?) -> fn","!doc":"<p>Sets the size of the major ticks.</p>\n"},"setMinorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the minor tick marks based on the passed object (expecting the same format\nas the minorTicks object documented above)</p>\n"},"setMinorTicksLength":{"!type":"fn(length: ?) -> fn","!doc":"<p>Sets the size of the minor ticks.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateTooltip":{"!type":"fn(txt: ?, e: ?)","!doc":"<p>Updates the tooltip for the gauge to display the given text.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/GlossyHorizontalGaugeMarker":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/BarLineIndicator.prototype","!doc":"<p>The marker for the dojox.gauges.GlossyHorizontalGauge.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_gauge":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The color of the indicator.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>indicates how long an animation of the indicator should take</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font for the indicator. The font is enerally in a format similar to:\n{family: &quot;Helvetica&quot;, weight: &quot;bold&quot;, style: &quot;italic&quot;, size: &quot;18pt&quot;, rotated: true}</p>\n"},"front":{"!doc":"<p>Keep this indicator at the front</p>\n"},"hideValue":{},"hover":{"!doc":"<p>The string to put in the tooltip when this indicator is hovered over.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interactionMode":{},"label":{"!doc":"<p>The text label for the indicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"length":{"!doc":"<p>The length of the indicator.  In the above example, the radius of the AnalogGauge\nis 125, but the length of the indicator is 135, meaning it would project beyond\nthe edge of the AnalogGauge</p>\n"},"noChange":{"!doc":"<p>Indicates whether the indicator&#39;s value can be changed.  Useful for\na static target indicator.  Default is false (that the value can be changed).</p>\n"},"observer":{},"offset":{"!doc":"<p>The offset of the indicator</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"strokeColor":{"!doc":"<p>The color to stroke the outline of the indicator.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>The title of the indicator, to be displayed next to it&#39;s input box for the text-representation.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>The type of indicator to draw.  Varies by gauge type.  Some examples include\n&quot;line&quot;, &quot;arrow&quot;, and &quot;bar&quot;</p>\n"},"value":{"!doc":"<p>The value (on the gauge) that this indicator should be placed at</p>\n"},"width":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn(group: ?, dontAnimate?: ?)","!doc":"<p>Override of dojox.gauges._Indicator.draw</p>\n"},"easing":{"!type":"fn(n?: ?) -> number","!doc":"<p>indicates the easing function to be used when animating the of an indicator.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-down events in the indicator.</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-out events in the indicator.</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>Handles mouse-over events in the indicator.</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>Handles touch start events in the indicator.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the indicator&#39;s shape from the gauge surface.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn(value: ?, animate: ?)","!doc":"<p>Updates the value of the indicator, including moving/re-drawing at it&#39;s new location and\nupdating the text box</p>\n"},"valueChanged":{"!type":"fn()","!doc":"<p>Invoked every time the value of the indicator changes.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gauges/GlossySemiCircularGauge":{"!type":"fn()","prototype":{"!proto":"dojox/gauges/GlossyCircularGaugeBase.prototype","!doc":"<p>Represents a semi circular gauge with a glossy appearance.\n</p>\n","_attrPairNames":{},"_backgroundDefault":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_designCx":{},"_designCy":{},"_designHeight":{},"_designTextIndicatorX":{},"_designTextIndicatorY":{},"_designWidth":{},"_drag":{},"_focusManager":{},"_font":{},"_img":{},"_indicatorData":{},"_lastHover":{},"_majorTicksLength":{"!doc":"<p>Major tick size, at design</p>\n"},"_majorTicksOffset":{"!doc":"<p>Distance, at design, from gauge&#39;s center to major ticks</p>\n"},"_minorTicksLength":{"!doc":"<p>Minor tick size, at design</p>\n"},"_minorTicksOffset":{"!doc":"<p>Distance, at design, from gauge&#39;s center to minor ticks</p>\n"},"_needle":{"!doc":"<p>the needle of this circular gauge</p>\n"},"_overOverlay":{},"_range":{"!doc":"<p>the range of this gauge</p>\n"},"_rangeData":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_textIndicator":{"!doc":"<p>the text displaying the gauge&#39;s value</p>\n"},"_textIndicatorAdded":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"background":{"!doc":"<p>The color of the background.  This must be an object of one of two forms:\n{&#39;color&#39;: &#39;color-name&#39;}\nOR\n(for a gradient:)\n{&#39;type&#39;: &#39;linear&#39;, &#39;x1&#39;: 0, &#39;x2&#39;: 0, &#39;y1&#39;: 0, &#39;y2&#39;: 200, &#39;colors&#39;: [{offset: 0, color:&#39;#C0C0C0&#39;}, {offset: 1, color: &#39;#E0E0E0&#39;}] }</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"color":{"!doc":"<p>The main color of the gauge.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cx":{"!doc":"<p>center of gauge x coordinate (default is gauge width / 2)</p>\n"},"cy":{"!doc":"<p>center of gauge x coordinate (default is gauge height / 2)</p>\n"},"defaultColors":{"!doc":"<p>Set of default colors to color ranges with.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"endAngle":{"!doc":"<p>angle (in degrees) for end of gauge (default is 90)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"font":{"!doc":"<p>The font of the gauge</p>\n"},"gaugeContent":{},"height":{"!doc":"<p>The height of the gauge (default is 200)</p>\n"},"hideValues":{"!doc":"<p>Indicates whether the text boxes showing the value of the indicator (as text\ncontent) should be hidden or shown.  Default is not hidden, aka shown.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"image":{"!doc":"<p>Background image for gauge (default is no image)</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"majorTicks":{"!doc":"<p>An object representing the tick marks that should be added to the gauge. Major tick marks have a text label\nindicating the value.  The object can have the following attributes (required are marked with a *):\n\n</p>\n<ul>\n<li>offset: the distance from the &#39;center&#39; of the gauge.  Used differently for Analog vs. Bar</li>\n<li>width: The width of the mark</li>\n<li>length: The length of the mark</li>\n<li>interval: The interval the ticks should be added on</li>\n<li>color: The color of the mark and text</li>\n<li><p>font: an object with any/all of the following parameters:</p>\n<p>  {family: &quot;Helvetica&quot;, style: &quot;italic&quot;, variant: &#39;small-caps&#39;, weight: &#39;bold&#39;, size: &quot;18pt&quot;}</p>\n</li>\n</ul>\n"},"majorTicksColor":{"!doc":"<p>Color of major tick marks</p>\n"},"majorTicksInterval":{"!doc":"<p>Interval between major ticks</p>\n"},"majorTicksLabelPlacement":{"!doc":"<p>Placement of major tick labels</p>\n"},"max":{"!doc":"<p>The maximum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"min":{"!doc":"<p>The minimum value of the gauge.  Normally not set explicitly, as it will be determined by\nthe ranges that are added.</p>\n"},"minorTicks":{"!doc":"<p>An object of the same format as majorTicks, indicating where the minor (label-less) marks should be placed\nThe font parameter is ignored if provided since minor tick marks have no text label.</p>\n"},"minorTicksColor":{"!doc":"<p>Color of minor tick marks</p>\n"},"minorTicksInterval":{"!doc":"<p>Interval between minor ticks</p>\n"},"needleColor":{"!doc":"<p>The main color of the needle.</p>\n"},"noChange":{"!doc":"<p>Indicates if the gauge reacts to touch events</p>\n"},"observer":{},"orientation":{"!doc":"<p>The orientation of the gauge. The value can be &#39;clockwise&#39; or &#39;cclockwise&#39; (default is &#39;clockwise&#39;)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"radius":{"!doc":"<p>radius of gauge (default is smaller of cx-25 or cy-25)</p>\n"},"scalePrecision":{"!doc":"<p>The precision for the formatting of numbers in the scale (default is 0)</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startAngle":{"!doc":"<p>angle (in degrees) for start of gauge (default is -90)</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"surface":{"!doc":"<p>The GFX surface that the shapes are drawn on.  Can be accessed/used by indicators to draw themselves</p>\n"},"textIndicatorColor":{"!doc":"<p>The color of the text indicator</p>\n"},"textIndicatorFont":{"!doc":"<p>The font of the text indicator</p>\n"},"textIndicatorPrecision":{"!doc":"<p>The precision for the formatting of numbers in the text indicator (default is 0)</p>\n"},"textIndicatorVisible":{"!doc":"<p>Indicates if the text indicator is visible</p>\n"},"title":{"!doc":"<p>The title displayed in the needle&#39;s tooltip</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useRangeStyles":{"!doc":"<p>Indicates whether to use given css classes (dojoxGaugeRangeXX)\nto determine the color (and other style attributes?) of the ranges\nthis value should be the number of dojoxGaugeRange classes that are\ndefined, starting at dojoxGaugeRange1 (0 indicates falling to default\nhardcoded colors)</p>\n"},"useTooltip":{"!doc":"<p>Indicates whether tooltips should be displayed for ranges, indicators, etc.</p>\n"},"value":{"!doc":"<p>The value of the gauge.</p>\n"},"width":{"!doc":"<p>The width of the gauge (default is 300)</p>\n"},"addIndicator":{"!type":"fn(indicator: ?) -> +instance","!doc":"<p>This method is used to add an indicator to the gauge.</p>\n"},"addRange":{"!type":"fn(range: ?)","!doc":"<p>This method is used to add a range to the gauge.</p>\n"},"addRanges":{"!type":"fn(ranges: ?)","!doc":"<p>This method is used to add ranges to the gauge.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Overrides _Widget.buildRendering</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Creates a new GlossySemiCircularGauge</p>\n"},"createSurface":{"!type":"fn()","!doc":"<p>Internal method used by the gauge to create the graphics surface area</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"draw":{"!type":"fn()","!doc":"<p>This function is used to draw (or redraw) the gauge.</p>\n"},"drawBackground":{"!type":"fn(group: ?)","!doc":"<p>Draws the background of the gauge</p>\n"},"drawForeground":{"!type":"fn(group: ?)","!doc":"<p>Draws the foreground of the gauge</p>\n"},"drawRange":{"!type":"fn(group: ?, range: ?)","!doc":"<p>This function is used to draw (or redraw) a range</p>\n"},"drawText":{"!type":"fn(group: ?, txt: ?, x: ?, y: ?, align?: ?, color?: ?, font?: ?) -> ?","!doc":"<p>This function is used draw text onto the gauge.  The text object\nis also returned by the function so that may be removed later\nby calling removeText</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getMajorTicksLength":{"!type":"fn() -> ?","!doc":"<p>Returns the size of the major ticks.</p>\n"},"getMajorTicksOffset":{"!type":"fn() -> ?","!doc":"<p>Return the distance from gauge&#39;s center to major ticks</p>\n"},"getMinorTicksLength":{"!type":"fn() -> ?","!doc":"<p>Return the size of the minor ticks.</p>\n"},"getMinorTicksOffset":{"!type":"fn() -> ?","!doc":"<p>Returns the distance from gauge&#39;s center to minor ticks</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRangeUnderMouse":{"!type":"fn(e: ?) -> ?","!doc":"<p>Determines which range the mouse is currently over</p>\n"},"handleMouseDown":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to move indicators</p>\n"},"handleMouseMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\nthe mouse to show the tooltips</p>\n"},"handleMouseOut":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support\nhover text</p>\n"},"handleMouseOver":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support \nhover text</p>\n"},"handleTouchEnd":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchMove":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStart":{"!type":"fn(e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"handleTouchStartIndicator":{"!type":"fn(indicator: ?, e: ?)","!doc":"<p>This is an internal handler used by the gauge to support using\ntouch events to drag an indicator to modify it&#39;s value</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has children, i.e. if this.containerNode contains something.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"moveIndicatorToFront":{"!type":"fn(indicator: ?)","!doc":"<p>This function is used to move an indicator the the front (top)\nof the gauge</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeIndicator":{"!type":"fn(indicator: ?)","!doc":"<p>Removes the given indicator from the gauge by calling it&#39;s remove function \nand removing it from the local cache.</p>\n"},"removeText":{"!type":"fn(t: ?)","!doc":"<p>Removes a text element from the gauge.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setBackground":{"!type":"fn(background: ?)","!doc":"<p>This method is used to set the background of the gauge after it is created.</p>\n"},"setMajorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the major tick marks based on the passed object (expecting the same format\nas the majorTicks object documented above)</p>\n"},"setMajorTicksLength":{"!type":"fn(length: ?) -> fn","!doc":"<p>Sets the size of the major ticks.</p>\n"},"setMajorTicksOffset":{"!type":"fn(offset: ?) -> fn","!doc":"<p>Sets the distance from gauge&#39;s center to major ticks</p>\n"},"setMinorTicks":{"!type":"fn(ticks: ?)","!doc":"<p>Creates and draws the minor tick marks based on the passed object (expecting the same format\nas the minorTicks object documented above)</p>\n"},"setMinorTicksLength":{"!type":"fn(length: ?) -> fn","!doc":"<p>Sets the size of the minor ticks.</p>\n"},"setMinorTicksOffset":{"!type":"fn(offset: ?) -> fn","!doc":"<p>Sets the distance from gauge&#39;s center to minor ticks</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Overrides AnalogGauge.startup</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateTooltip":{"!type":"fn(txt: ?, e: ?)","!doc":"<p>Updates the tooltip for the gauge to display the given text.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/geo/charting/Feature":{"!type":"fn()","prototype":{"!doc":"<p>A class to encapsulate a map element.</p>\n","_isZoomIn":{},"isSelected":{},"markerText":{},"init":{"!type":"fn()","!doc":"<p>Initializes this feature.</p>\n"},"select":{"!type":"fn(selected: ?)","!doc":"<p>Sets the selected state of this feature to the given value.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>sets a numeric value on this Feature object (used together with series to apply a color).</p>\n"},"unsetColor":{"!type":"fn()","!doc":"<p>clears the colors on this Feature object.</p>\n"},"unsetValue":{"!type":"fn()","!doc":"<p>clears the numeric value on this Feature object (removes color).</p>\n"}}},"dojox/geo/charting/KeyboardInteractionSupport":{"!type":"fn()","prototype":{"!doc":"<p>class to handle keyboard interactions on a dojox.geo.charting.Map component.\n\n</p>\n<p>The sections on the leading edge should receive the focus in response to a TAB event.\nThen use cursor keys to the peer sections. The cursor event should go the adjacent section\nin that direction. With the focus, the section zooms in upon SPACE. The map should zoom out\non ESC. Finally, while it has the focus, the map should lose the focus on TAB.</p>\n","_map":{},"_zoomEnabled":{},"connect":{"!type":"fn()","!doc":"<p>connects this keyboard support class to the Map component</p>\n"},"constructor":{"!type":"fn(map: ?, options?: ?)","!doc":"<p>Constructs a new _KeyboardInteractionSupport instance</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>disconnects any installed listeners</p>\n"},"keydownHandler":{"!type":"fn(e: ?)","!doc":"<p>Handles a keydown event.</p>\n"}}},"dojox/geo/charting/Map":{"!type":"fn()","prototype":{"!doc":"<p>Map widget interacted with charting.</p>\n","_idAttributes":{},"_onDeleteListener":{},"_onNewListener":{},"_onSetListener":{},"colorAnimationDuration":{},"dataBindingAttribute":{},"dataBindingValueFunction":{},"dataStore":{},"defaultColor":{"!doc":"<p>Default map feature color, e.g: &quot;#B7B7B7&quot;</p>\n"},"enableFeatureZoom":{},"highlightColor":{"!doc":"<p>Map feature color when mouse over it, e.g: &quot;#&quot;</p>\n"},"series":{"!doc":"<p>stack to data range, e.g: [{name:&#39;label 1&#39;, min:20, max:70, color:&#39;#DDDDDD&#39;},{...},...]</p>\n"},"showTooltips":{},"addSeries":{"!type":"fn(series: ?)","!doc":"<p>sets ranges of data values (associated with label, color) to style map data values</p>\n"},"constructor":{"!type":"fn(container: ?, shapeData: ?)","!doc":"<p>Constructs a new Map instance.</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>deselect all features of map</p>\n"},"fitToMapArea":{"!type":"fn(mapArea: ?, pixelMargin: ?, animate: ?, onAnimationEnd: ?)","!doc":"<p>set this component&#39;s transformation so that the specified area fits in the component (centered)</p>\n"},"fitToMapContents":{"!type":"fn(pixelMargin: ?, animate: ?, onAnimationEnd: ?)","!doc":"<p>set this component&#39;s transformation so that the whole map data fits in the component (centered)</p>\n"},"getMapCenter":{"!type":"fn() -> +point","!doc":"<p>returns the map coordinates of the center of this Map component.</p>\n"},"getMapScale":{"!type":"fn() -> +float","!doc":"<p>returns the scale of this Map component.</p>\n"},"mapCoordsToScreenCoords":{"!type":"fn(mapX: ?, mapY: ?) -> +any","!doc":"<p>converts map coordinates to screen coordinates given the current transform of this Map component</p>\n"},"resize":{"!type":"fn(adjustMapCenter: ?, adjustMapScale: ?, animate: ?)","!doc":"<p>resize the underlying GFX surface to accommodate to parent DOM Node size change</p>\n"},"screenCoordsToMapCoords":{"!type":"fn(screenX: ?, screenY: ?) -> +any","!doc":"<p>converts screen coordinates to map coordinates given the current transform of this Map component</p>\n"},"setDataBindingAttribute":{"!type":"fn(prop: ?)","!doc":"<p>sets the property name of the dataStore items to use as value (see Feature.setValue function)</p>\n"},"setDataBindingValueFunction":{"!type":"fn(valueFunction: ?)","!doc":"<p>sets the function that extracts values from dataStore items,to use as Feature values (see Feature.setValue function)</p>\n"},"setDataStore":{"!type":"fn(dataStore: ?, dataBindingProp: ?)","!doc":"<p>populate data for each map feature from fetched data store</p>\n"},"setMapCenter":{"!type":"fn(centerX: ?, centerY: ?, animate: ?, onAnimationEnd: ?)","!doc":"<p>set this component&#39;s transformation so that the map is centered on the specified map coordinates</p>\n"},"setMapCenterAndScale":{"!type":"fn(centerX: ?, centerY: ?, scale: ?, animate: ?, onAnimationEnd: ?)","!doc":"<p>set this component&#39;s transformation so that the map is centered on the specified map coordinates\nand scaled to the specified scale.</p>\n"},"setMapScale":{"!type":"fn(scale: ?, animate: ?, onAnimationEnd: ?)","!doc":"<p>set this component&#39;s transformation so that the map is scaled to the specified scale.</p>\n"},"setMapScaleAt":{"!type":"fn(scale: ?, fixedMapX: ?, fixedMapY: ?, animate: ?, onAnimationEnd: ?)","!doc":"<p>set this component&#39;s transformation so that the map is scaled to the specified scale, and the specified\npoint (in map coordinates) stays fixed on this Map component</p>\n"},"setMarkerData":{"!type":"fn(markerFile: ?)","!doc":"<p>import markers from outside file, associate with map feature by feature id\nwhich identified in map shape file, e.g: &quot;NY&quot;:&quot;New York&quot;</p>\n"}}},"dojox/geo/charting/_base":{"hideTooltip":{"!type":"fn(gfxObject: ?)","!doc":"<p>Hides the tooltip displayed around the given shape.</p>\n"},"showTooltip":{"!type":"fn(innerHTML: ?, gfxObject: ?, positions?: ?)","!doc":"<p>Show a Tooltip displaying the given HTML message around the given gfx shape.</p>\n"}},"dojox/geo/charting/_Marker":{"!type":"fn()","prototype":{"_map":{},"_needTooltipRefresh":{},"constructor":{"!type":"fn(markerData: ?, map: ?)"},"hide":{"!type":"fn()"},"show":{"!type":"fn(featureId: ?, evt: ?)"}}},"dojox/geo/charting/MouseInteractionSupport":{"!type":"fn()","prototype":{"!doc":"<p>class to handle mouse interactions on a dojox.geo.charting.Map widget</p>\n","_cancelMouseClick":{},"_currentFeature":{},"_map":{},"_mapClickLocation":{},"_mouseDownListener":{},"_mouseDragListener":{},"_mouseMoveListener":{},"_mouseUpClickListener":{},"_mouseUpListener":{},"_mouseWheelListener":{},"_onDragStartListener":{},"_onSelectStartListener":{},"_panEnabled":{},"_screenClickLocation":{},"_zoomEnabled":{},"mouseClickThreshold":{},"connect":{"!type":"fn()","!doc":"<p>connects this mouse support class to the Map component</p>\n"},"constructor":{"!type":"fn(map: ?, options?: ?)","!doc":"<p>Constructs a new _MouseInteractionSupport instance</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>disconnects any installed listeners</p>\n"},"setEnablePan":{"!type":"fn(enable: ?)","!doc":"<p>enables mouse panning on the map</p>\n"},"setEnableZoom":{"!type":"fn(enable: ?)","!doc":"<p>enables mouse zoom on the map</p>\n"}}},"dojox/geo/charting/TouchInteractionSupport":{"!type":"fn()","prototype":{"!doc":"<p>A class to handle touch interactions on a dojox/geo/charting/Map component.</p>\n","_centerTouchLocation":{},"_doubleTapPerformed":{},"_initialFingerSpacing":{},"_initialScale":{},"_lastTap":{},"_map":{},"_oneFingerTouch":{},"_tapCancel":{},"_tapCount":{},"_tapThreshold":{},"_touchEndListener":{},"_touchEndTapListener":{},"_touchMoveListener":{},"_touchStartListener":{},"connect":{"!type":"fn()","!doc":"<p>install touch listeners</p>\n"},"constructor":{"!type":"fn(map: ?)","!doc":"<p>Constructs a new _TouchInteractionSupport instance</p>\n"},"disconnect":{"!type":"fn()","!doc":"<p>disconnects any installed listeners. Must be called only when disposing of this instance</p>\n"}}},"dojox/geo/charting/widget/Legend":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A legend widget displaying association between colors and Feature value ranges.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"horizontal":{},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"legendBody":{},"map":{},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"swatchSize":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn()","!doc":"<p>Refreshes this legend contents when Map series has changed.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/geo/charting/widget/Map":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A map viewer widget based on the dojox.geo.charting.Map component</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyboardInteractionSupport":{},"_mouseInteractionSupport":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_touchInteractionSupport":{},"adjustMapCenterOnResize":{},"adjustMapScaleOnResize":{},"animateOnResize":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"colorAnimationDuration":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dataBindingAttribute":{},"dataBindingValueFunction":{},"dataStore":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"enableFeatureZoom":{},"enableKeyboardSupport":{},"enableMousePan":{},"enableMouseSupport":{},"enableMouseZoom":{},"enableTouchSupport":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"markerData":{},"mouseClickThreshold":{},"observer":{},"onFeatureClick":{},"onFeatureOver":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"series":{},"shapeData":{},"showTooltips":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(options: ?, div: ?)","!doc":"<p>Constructs a new Map widget</p>\n"},"create":{"!type":"fn(params?: ?, srcNodeRef?: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getInnerMap":{"!type":"fn() -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"resize":{"!type":"fn(b?: ?, height?: ?)","!doc":"<p>Resize the widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/geo/openlayers/Collection":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Geometry.prototype","!doc":"<p>A collection of geometries. </p>\n","coordinates":{"!doc":"<p>An array of geometries.</p>\n"},"shape":{"!doc":"<p>The associated shape when rendered</p>\n"},"constructor":{"!type":"fn(coords: ?)","!doc":"<p>Constructs a new geometry</p>\n"},"getGeometries":{"!type":"fn() -> +any","!doc":"<p>Returns the geometries.</p>\n"},"setGeometries":{"!type":"fn(g: ?)","!doc":"<p>Sets the geometries</p>\n"}}},"dojox/geo/openlayers/Geometry":{"!type":"fn()","prototype":{"!doc":"<p>A Geometry handles description of shapes to be rendered in a GfxLayer\nusing a GeometryFeature feature.\nA Geometry can be:\n\n</p>\n<ul>\n<li>A point geometry of type dojox.geo.openlayers.Point. Coordinates are a an \nObject {x, y}</li>\n<li>A line string geometry of type dojox.geo.openlayers.LineString. Coordinates are\nan array of {x, y} objects</li>\n<li>A collection geometry of type dojox.geo.openlayers.Collection. Coordinates are an array of geometries.</li>\n</ul>\n","coordinates":{"!doc":"<p>The coordinates of the geometry, Object like {x, y} or Array.</p>\n"},"shape":{"!doc":"<p>The associated shape when rendered</p>\n"},"constructor":{"!type":"fn(coords: ?)","!doc":"<p>Constructs a new geometry</p>\n"}}},"dojox/geo/openlayers/Feature":{"!type":"fn()","prototype":{"!doc":"<p>A Feature encapsulates an item so that it can be added to a Layer.\nThis class is not attended to be used as it, but serve as a base class\nfor specific features such as GeometryFeature which can display georeferenced \ngeometries and WidgetFeature which can display georeferenced widgets. </p>\n","constructor":{"!type":"fn()","!doc":"<p>Construct a new Feature</p>\n"},"getCoordinateSystem":{"!type":"fn() -> +any","!doc":"<p>Returns the coordinate system in which coordinates of this feature are expressed.</p>\n"},"getLayer":{"!type":"fn() -> +any","!doc":"<p>Returns the Layer to which this feature belongs.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Subclasses implements specific behavior.\nCalled when removed from the layer.</p>\n"},"render":{"!type":"fn()","!doc":"<p>subclasses implements drawing specific behavior.</p>\n"},"setCoordinateSystem":{"!type":"fn(cs: ?)","!doc":"<p>Set the coordinate system in which coordinates of this feature are expressed.</p>\n"}}},"dojox/geo/openlayers/_base":{"__JsonImportArgs":{"!doc":"<p>The keyword arguments that can be passed in a JsonImport constructor.</p>\n"},"__MapArgs":{"!doc":"<p>The keyword arguments that can be passed in a Map constructor.</p>\n"},"__WidgetFeatureArgs":{"!doc":"<p>The keyword arguments that can be passed in a WidgetFeature constructor.\nYou must define a least one widget retrieval parameter and the geo-localization parameters.</p>\n"},"BaseLayerType":{"!doc":"<p>Defines the base layer types to be used at Map construction time or\nwith the setBaseLayerType function.</p>\n"},"EPSG4326":{},"GreatCircle":{},"widget":{},"Collection":{"!type":"fn()"},"Feature":{"!type":"fn()"},"Geometry":{"!type":"fn()"},"GeometryFeature":{"!type":"fn()"},"GfxLayer":{"!type":"fn()"},"JsonImport":{"!type":"fn()"},"Layer":{"!type":"fn()"},"LineString":{"!type":"fn()"},"Map":{"!type":"fn()"},"parseDMS":{"!type":"fn(v: ?, toDecimal: ?) -> +Float","!doc":"<p>Parses the specified string and returns degree minute second or decimal degree.</p>\n"},"Point":{"!type":"fn()"},"TouchInteractionSupport":{"!type":"fn()"},"WidgetFeature":{"!type":"fn()"}},"dojox/geo/openlayers/_base.Geometry":{"!type":"fn()","prototype":{"!doc":"<p>A Geometry handles description of shapes to be rendered in a GfxLayer\nusing a GeometryFeature feature.\nA Geometry can be:\n\n</p>\n<ul>\n<li>A point geometry of type dojox.geo.openlayers.Point. Coordinates are a an \nObject {x, y}</li>\n<li>A line string geometry of type dojox.geo.openlayers.LineString. Coordinates are\nan array of {x, y} objects</li>\n<li>A collection geometry of type dojox.geo.openlayers.Collection. Coordinates are an array of geometries.</li>\n</ul>\n","coordinates":{"!doc":"<p>The coordinates of the geometry, Object like {x, y} or Array.</p>\n"},"shape":{"!doc":"<p>The associated shape when rendered</p>\n"},"constructor":{"!type":"fn(coords: ?)","!doc":"<p>Constructs a new geometry</p>\n"}}},"dojox/geo/openlayers/_base.Collection":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Geometry.prototype","!doc":"<p>A collection of geometries. </p>\n","coordinates":{"!doc":"<p>An array of geometries.</p>\n"},"shape":{"!doc":"<p>The associated shape when rendered</p>\n"},"constructor":{"!type":"fn(coords: ?)","!doc":"<p>Constructs a new geometry</p>\n"},"getGeometries":{"!type":"fn() -> +any","!doc":"<p>Returns the geometries.</p>\n"},"setGeometries":{"!type":"fn(g: ?)","!doc":"<p>Sets the geometries</p>\n"}}},"dojox/geo/openlayers/_base.BaseLayerType":{"!doc":"<p>Defines the base layer types to be used at Map construction time or\nwith the setBaseLayerType function.</p>\n","ARCGIS":{"!doc":"<p>The ESRI ARCGis base layer selector.</p>\n"},"BING":{"!doc":"<p>Same as Virtual Earth</p>\n"},"GOOGLE":{"!doc":"<p>The Google base layer type selector.</p>\n"},"OSM":{"!doc":"<p>The Open Street Map base layer type selector.</p>\n"},"VIRTUAL_EARTH":{"!doc":"<p>The Virtual Earth base layer type selector.</p>\n"},"WMS":{"!doc":"<p>The Web Map Server base layer type selector.</p>\n"},"YAHOO":{"!doc":"<p>The Yahoo base layer type selector.</p>\n"}},"dojox/geo/openlayers/_base.Feature":{"!type":"fn()","prototype":{"!doc":"<p>A Feature encapsulates an item so that it can be added to a Layer.\nThis class is not attended to be used as it, but serve as a base class\nfor specific features such as GeometryFeature which can display georeferenced \ngeometries and WidgetFeature which can display georeferenced widgets. </p>\n","constructor":{"!type":"fn()","!doc":"<p>Construct a new Feature</p>\n"},"getCoordinateSystem":{"!type":"fn() -> +any","!doc":"<p>Returns the coordinate system in which coordinates of this feature are expressed.</p>\n"},"getLayer":{"!type":"fn() -> +any","!doc":"<p>Returns the Layer to which this feature belongs.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Subclasses implements specific behavior.\nCalled when removed from the layer.</p>\n"},"render":{"!type":"fn()","!doc":"<p>subclasses implements drawing specific behavior.</p>\n"},"setCoordinateSystem":{"!type":"fn(cs: ?)","!doc":"<p>Set the coordinate system in which coordinates of this feature are expressed.</p>\n"}}},"dojox/geo/openlayers/_base.Point":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Geometry.prototype","!doc":"<p>A Point geometry handles description of points to be rendered in a GfxLayer</p>\n","coordinates":{"!doc":"<p>The coordinates of the geometry, Object like {x, y} or Array.</p>\n"},"shape":{"!doc":"<p>The associated shape when rendered</p>\n"},"constructor":{"!type":"fn(coords: ?)","!doc":"<p>Constructs a new geometry</p>\n"},"getPoint":{"!type":"fn() -> +any","!doc":"<p>Gets the point defining this geometry.</p>\n"},"setPoint":{"!type":"fn(p: ?)","!doc":"<p>Sets the point for this geometry.</p>\n"}}},"dojox/geo/openlayers/_base.LineString":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Geometry.prototype","!doc":"<p>The <code>dojox.geo.openlayers.LineString</code> geometry. This geometry holds an array\nof coordinates.</p>\n","coordinates":{"!doc":"<p>The coordinates of the geometry, Object like {x, y} or Array.</p>\n"},"shape":{"!doc":"<p>The associated shape when rendered</p>\n"},"constructor":{"!type":"fn(coords: ?)","!doc":"<p>Constructs a new geometry</p>\n"},"getPoints":{"!type":"fn() -> +any","!doc":"<p>Gets the points of this geometry.</p>\n"},"setPoints":{"!type":"fn(p: ?)","!doc":"<p>Sets the points for this geometry.</p>\n"}}},"dojox/geo/openlayers/_base.GeometryFeature":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Feature.prototype","!doc":"<p>A Feature encapsulating a geometry.</p>\n","_defaults":{},"constructor":{"!type":"fn(geometry: ?)","!doc":"<p>Constructs a GeometryFeature for the specified geometry.</p>\n"},"createShape":{"!type":"fn(s: ?, g?: ?) -> +any","!doc":"<p>Called when the shape rendering the geometry has to be created.\nThis default implementation creates a circle for a point geometry, a polyline for\na LineString geometry and is recursively called when creating a collection.\nUser may replace this method to produce a custom shape.</p>\n"},"getCoordinateSystem":{"!type":"fn() -> +any","!doc":"<p>Returns the coordinate system in which coordinates of this feature are expressed.</p>\n"},"getFill":{"!type":"fn() -> +any","!doc":"<p>Returns the fill style</p>\n"},"getLayer":{"!type":"fn() -> +any","!doc":"<p>Returns the Layer to which this feature belongs.</p>\n"},"getShape":{"!type":"fn() -> +any","!doc":"<p>Returns the shape rendering the geometry</p>\n"},"getShapeProperties":{"!type":"fn() -> ?","!doc":"<p>Returns the shape properties. </p>\n"},"getStroke":{"!type":"fn() -> +any","!doc":"<p>Returns the stroke style</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the shape from the Surface. \nCalled when the feature is removed from the layer.</p>\n"},"render":{"!type":"fn(g?: ?)","!doc":"<p>Render a geometry. \nCalled by the Layer on which the feature is added. </p>\n"},"renderCollection":{"!type":"fn(g?: ?)","!doc":"<p>Renders a geometry collection.</p>\n"},"renderLineString":{"!type":"fn(g?: ?)","!doc":"<p>Renders a line string geometry.</p>\n"},"renderPoint":{"!type":"fn(g?: ?)","!doc":"<p>Renders a point geometry.</p>\n"},"setCoordinateSystem":{"!type":"fn(cs: ?)","!doc":"<p>Set the coordinate system in which coordinates of this feature are expressed.</p>\n"},"setFill":{"!type":"fn(f: ?) -> fn","!doc":"<p>Set the fill style to be applied on the rendered shape.</p>\n"},"setShapeProperties":{"!type":"fn(s: ?) -> fn","!doc":"<p>Sets the shape properties. </p>\n"},"setStroke":{"!type":"fn(s: ?) -> fn","!doc":"<p>Set the stroke style to be applied on the rendered shape.</p>\n"}}},"dojox/geo/openlayers/_base.Layer":{"!type":"fn()","prototype":{"!doc":"<p>Base layer class for dojox.geo.openlayers.Map specific layers extending OpenLayers.Layer class.\nThis layer class accepts Features which encapsulates graphic objects to be added to the map.\nThis layer class encapsulates an OpenLayers.Layer.\nThis class provides Feature management such as add, remove and feature access.</p>\n","added":{"!type":"fn()","!doc":"<p>Called when the layer is added to the map</p>\n"},"addFeature":{"!type":"fn(f: ?)","!doc":"<p>Add a feature or an array of features to the layer.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Removes all the features from this layer.</p>\n"},"constructor":{"!type":"fn(name: ?, options: ?)","!doc":"<p>Constructs a new Layer.</p>\n"},"getDojoMap":{"!type":"fn() -> ?"},"getFeatureAt":{"!type":"fn(i: ?) -> +any","!doc":"<p>Returns the i-th feature of this layer.</p>\n"},"getFeatureCount":{"!type":"fn() -> +any","!doc":"<p>Returns the number of the features contained by this layer.</p>\n"},"getFeatures":{"!type":"fn() -> +any","!doc":"<p>Returns the feature hold by this layer.</p>\n"},"moveTo":{"!type":"fn(event: ?)","!doc":"<p>Called when the layer is panned or zoomed.</p>\n"},"redraw":{"!type":"fn()","!doc":"<p>Redraws this layer</p>\n"},"removeFeature":{"!type":"fn(f: ?)","!doc":"<p>Removes a feature or an array of features from the layer.</p>\n"},"removeFeatureAt":{"!type":"fn(index: ?)","!doc":"<p>Remove the feature at the specified index.</p>\n"},"renderFeature":{"!type":"fn(f: ?)","!doc":"<p>Called when rendering a feature is necessary.</p>\n"}}},"dojox/geo/openlayers/_base.GfxLayer":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Layer.prototype","!doc":"<p>A layer dedicated to render dojox.geo.openlayers.GeometryFeature</p>\n","_viewport":{},"added":{"!type":"fn()","!doc":"<p>Called when added to a map.</p>\n"},"addFeature":{"!type":"fn(f: ?)","!doc":"<p>Add a feature or an array of features to the layer.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Removes all the features from this layer.</p>\n"},"constructor":{"!type":"fn(name: ?, options: ?)","!doc":"<p>Constructs a new GFX layer.</p>\n"},"getDojoMap":{"!type":"fn() -> ?"},"getFeatureAt":{"!type":"fn(i: ?) -> +any","!doc":"<p>Returns the i-th feature of this layer.</p>\n"},"getFeatureCount":{"!type":"fn() -> +any","!doc":"<p>Returns the number of the features contained by this layer.</p>\n"},"getFeatures":{"!type":"fn() -> +any","!doc":"<p>Returns the feature hold by this layer.</p>\n"},"getSurface":{"!type":"fn() -> +any","!doc":"<p>Get the underlying dojox.gfx.Surface</p>\n"},"getViewport":{"!type":"fn() -> ?","!doc":"<p>Gets the viewport</p>\n"},"moveTo":{"!type":"fn(event: ?)","!doc":"<p>Called when this layer is moved or zoomed.</p>\n"},"redraw":{"!type":"fn()","!doc":"<p>Redraws this layer</p>\n"},"removeFeature":{"!type":"fn(f: ?)","!doc":"<p>Removes a feature or an array of features from the layer.</p>\n"},"removeFeatureAt":{"!type":"fn(index: ?)","!doc":"<p>Remove the feature at the specified index.</p>\n"},"renderFeature":{"!type":"fn(f: ?)","!doc":"<p>Called when rendering a feature is necessary.</p>\n"},"setMap":{"!type":"fn(map: ?)","!doc":"<p>Sets the map for this layer.</p>\n"},"setViewport":{"!type":"fn(g: ?)","!doc":"<p>Sets the viewport</p>\n"}}},"dojox/geo/openlayers/_base.GreatCircle":{"DEG2RAD":{},"RAD2DEG":{},"TOLERANCE":{},"toGeometryFeature":{"!type":"fn(p1: ?, p2: ?, increment: ?) -> +any","!doc":"<p>Create a geodetic line as an array of dojox.geo.openlayers.GeometryFeature.</p>\n"},"toLineString":{"!type":"fn(p1: ?, p2: ?, increment: ?) -> +instance","!doc":"<p>Create a geodetic line as an array of OpenLayers.Geometry.LineString.</p>\n"},"toPointArray":{"!type":"fn(p1: ?, p2: ?, increment: ?) -> []","!doc":"<p>Create a geodetic line as an array of OpenLayers.Point.</p>\n"}},"dojox/geo/openlayers/_base.__JsonImportArgs":{"!doc":"<p>The keyword arguments that can be passed in a JsonImport constructor.</p>\n","error":{"!doc":"<p>Error callback called if something fails.</p>\n"},"nextFeature":{"!doc":"<p>The function called each time a feature is read. The function is called with a GeometryFeature as argument.</p>\n"},"url":{"!doc":"<p>The url pointing to the JSON file to load.</p>\n"}},"dojox/geo/openlayers/_base.JsonImport":{"!type":"fn()","prototype":{"!doc":"<p>Class to load JSON formated ShapeFile as output of the JSon Custom Map Converter.</p>\n","constructor":{"!type":"fn(params: ?)","!doc":"<p>Construct a new JSON importer.</p>\n"},"loadData":{"!type":"fn()","!doc":"<p>Triggers the loading.</p>\n"}}},"dojox/geo/openlayers/_base.TouchInteractionSupport":{"!type":"fn()","prototype":{"!doc":"<p>class to handle touch interactions on a OpenLayers.Map widget</p>\n","_centerTouchLocation":{},"_initialFingerSpacing":{},"_initialScale":{},"_lastTap":{},"_map":{},"_tapCount":{},"_tapThreshold":{},"_touchEndListener":{},"_touchMoveListener":{},"constructor":{"!type":"fn(map: ?)","!doc":"<p>Constructs a new TouchInteractionSupport instance</p>\n"}}},"dojox/geo/openlayers/_base.__MapArgs":{"!doc":"<p>The keyword arguments that can be passed in a Map constructor.</p>\n","baseLayerName":{"!doc":"<p>The name of the base layer.</p>\n"},"baseLayerOptions":{"!doc":"<p>Additional specific options passed to OpensLayers layer, such as The list of layer to display, for Web Map Server layer.</p>\n"},"baseLayerType":{"!doc":"<p>type of the base layer. Can be any of\n\n</p>\n<ul>\n<li><code>dojox.geo.openlayers.BaseLayerType.OSM</code>: Open Street Map base layer</li>\n<li><code>dojox.geo.openlayers.BaseLayerType.WMS</code>: Web Map Service layer</li>\n<li><code>dojox.geo.openlayers.BaseLayerType.GOOGLE</code>: Google layer</li>\n<li><code>dojox.geo.openlayers.BaseLayerType.VIRTUAL_EARTH</code>: Virtual Earth layer</li>\n<li><code>dojox.geo.openlayers.BaseLayerType.BING</code>: Bing layer</li>\n<li><code>dojox.geo.openlayers.BaseLayerType.YAHOO</code>: Yahoo layer</li>\n<li><code>dojox.geo.openlayers.BaseLayerType.ARCGIS</code>: ESRI ArgGIS layer</li>\n</ul>\n"},"baseLayerUrl":{"!doc":"<p>Some layer may need an url such as Web Map Server.</p>\n"}},"dojox/geo/openlayers/_base.Map":{"!type":"fn()","prototype":{"!doc":"<p>A map viewer based on the OpenLayers library.\n</p>\n","_tp":{},"olMap":{"!doc":"<p>The underlying OpenLayers.Map object.\nShould be accessed on read mode only.</p>\n"},"addLayer":{"!type":"fn(layer: ?)","!doc":"<p>Add the specified layer to the map.</p>\n"},"constructor":{"!type":"fn(div: ?, options: ?)","!doc":"<p>Constructs a new Map object</p>\n"},"fitTo":{"!type":"fn(o: ?)","!doc":"<p>Fits the map on a point,or an area</p>\n"},"getBaseLayerType":{"!type":"fn() -> +any","!doc":"<p>Returns the base layer type.</p>\n"},"getLayer":{"!type":"fn(property: ?, value: ?) -> +any","!doc":"<p>Returns the layer whose property matches the value.</p>\n"},"getLayerCount":{"!type":"fn() -> +any","!doc":"<p>Returns the count of layers of this map.</p>\n"},"getOLMap":{"!type":"fn() -> +any","!doc":"<p>gets the underlying OpenLayers map object.</p>\n"},"getScale":{"!type":"fn(geodesic: ?) -> +null","!doc":"<p>Returns the current scale</p>\n"},"initialFit":{"!type":"fn(params: ?)","!doc":"<p>Performs an initial fit to contents.</p>\n"},"layerIndex":{"!type":"fn(layer: ?, index?: ?) -> +any","!doc":"<p>Set or retrieve the layer index.</p>\n"},"removeLayer":{"!type":"fn(layer: ?)","!doc":"<p>Remove the specified layer from the map.</p>\n"},"setBaseLayerType":{"!type":"fn(type: ?) -> +any","!doc":"<p>Set the base layer type, replacing the existing base layer</p>\n"},"transform":{"!type":"fn(p: ?, from: ?, to: ?) -> ?","!doc":"<p>Transforms the point passed as argument, expressed in the <em>from</em> \ncoordinate system to the map coordinate system.</p>\n"},"transformXY":{"!type":"fn(x: ?, y: ?, from?: ?, to?: ?) -> +any","!doc":"<p>Transforms the coordinates passed as argument, expressed in the <em>from</em> \ncoordinate system to the map coordinate system.</p>\n"}}},"dojox/geo/openlayers/_base.__WidgetFeatureArgs":{"!doc":"<p>The keyword arguments that can be passed in a WidgetFeature constructor.\nYou must define a least one widget retrieval parameter and the geo-localization parameters.</p>\n","createWidget":{"!doc":"<p>Function for widget creation. Must return a `dijit._Widget.</p>\n"},"dijitId":{"!doc":"<p>The digitId of an existing widget.</p>\n"},"dojoType":{"!doc":"<p>The class of a widget to create.</p>\n"},"height":{"!doc":"<p>The height of the widget.</p>\n"},"latitude":{"!doc":"<p>The latitude, in decimal degrees where to place the widget.</p>\n"},"longitude":{"!doc":"<p>The longitude, in decimal degrees where to place the widget.</p>\n"},"widget":{"!doc":"<p>An already created widget.</p>\n"},"width":{"!doc":"<p>The width of the widget.</p>\n"}},"dojox/geo/openlayers/_base.WidgetFeature":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Feature.prototype","!doc":"<p>Wraps a Dojo widget, provide geolocalisation of the widget and interface\nto Layer class.</p>\n","_bbox":{},"_widget":{},"constructor":{"!type":"fn(params: ?)","!doc":"<p>Constructs a new <code>dojox.geo.openlayers.WidgetFeature</code></p>\n"},"getCoordinateSystem":{"!type":"fn() -> +any","!doc":"<p>Returns the coordinate system in which coordinates of this feature are expressed.</p>\n"},"getLayer":{"!type":"fn() -> +any","!doc":"<p>Returns the Layer to which this feature belongs.</p>\n"},"getParameters":{"!type":"fn() -> +dojox.geo.openlayers.__WidgetFeatureArgs","!doc":"<p>Returns the parameters describing the widget.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>removes this feature.</p>\n"},"render":{"!type":"fn()","!doc":"<p>renders the widget.</p>\n"},"setCoordinateSystem":{"!type":"fn(cs: ?)","!doc":"<p>Set the coordinate system in which coordinates of this feature are expressed.</p>\n"},"setParameters":{"!type":"fn(params: ?)","!doc":"<p>Sets the parameters describing the widget.</p>\n"}}},"dojox/geo/openlayers/_base.widget":{"Map":{"!type":"fn()"}},"dojox/geo/openlayers/GeometryFeature":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Feature.prototype","!doc":"<p>A Feature encapsulating a geometry.</p>\n","_defaults":{},"constructor":{"!type":"fn(geometry: ?)","!doc":"<p>Constructs a GeometryFeature for the specified geometry.</p>\n"},"createShape":{"!type":"fn(s: ?, g?: ?) -> +any","!doc":"<p>Called when the shape rendering the geometry has to be created.\nThis default implementation creates a circle for a point geometry, a polyline for\na LineString geometry and is recursively called when creating a collection.\nUser may replace this method to produce a custom shape.</p>\n"},"getCoordinateSystem":{"!type":"fn() -> +any","!doc":"<p>Returns the coordinate system in which coordinates of this feature are expressed.</p>\n"},"getFill":{"!type":"fn() -> +any","!doc":"<p>Returns the fill style</p>\n"},"getLayer":{"!type":"fn() -> +any","!doc":"<p>Returns the Layer to which this feature belongs.</p>\n"},"getShape":{"!type":"fn() -> +any","!doc":"<p>Returns the shape rendering the geometry</p>\n"},"getShapeProperties":{"!type":"fn() -> ?","!doc":"<p>Returns the shape properties. </p>\n"},"getStroke":{"!type":"fn() -> +any","!doc":"<p>Returns the stroke style</p>\n"},"remove":{"!type":"fn()","!doc":"<p>Removes the shape from the Surface. \nCalled when the feature is removed from the layer.</p>\n"},"render":{"!type":"fn(g?: ?)","!doc":"<p>Render a geometry. \nCalled by the Layer on which the feature is added. </p>\n"},"renderCollection":{"!type":"fn(g?: ?)","!doc":"<p>Renders a geometry collection.</p>\n"},"renderLineString":{"!type":"fn(g?: ?)","!doc":"<p>Renders a line string geometry.</p>\n"},"renderPoint":{"!type":"fn(g?: ?)","!doc":"<p>Renders a point geometry.</p>\n"},"setCoordinateSystem":{"!type":"fn(cs: ?)","!doc":"<p>Set the coordinate system in which coordinates of this feature are expressed.</p>\n"},"setFill":{"!type":"fn(f: ?) -> fn","!doc":"<p>Set the fill style to be applied on the rendered shape.</p>\n"},"setShapeProperties":{"!type":"fn(s: ?) -> fn","!doc":"<p>Sets the shape properties. </p>\n"},"setStroke":{"!type":"fn(s: ?) -> fn","!doc":"<p>Set the stroke style to be applied on the rendered shape.</p>\n"}}},"dojox/geo/openlayers/Point":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Geometry.prototype","!doc":"<p>A Point geometry handles description of points to be rendered in a GfxLayer</p>\n","coordinates":{"!doc":"<p>The coordinates of the geometry, Object like {x, y} or Array.</p>\n"},"shape":{"!doc":"<p>The associated shape when rendered</p>\n"},"constructor":{"!type":"fn(coords: ?)","!doc":"<p>Constructs a new geometry</p>\n"},"getPoint":{"!type":"fn() -> +any","!doc":"<p>Gets the point defining this geometry.</p>\n"},"setPoint":{"!type":"fn(p: ?)","!doc":"<p>Sets the point for this geometry.</p>\n"}}},"dojox/geo/openlayers/LineString":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Geometry.prototype","!doc":"<p>The <code>dojox.geo.openlayers.LineString</code> geometry. This geometry holds an array\nof coordinates.</p>\n","coordinates":{"!doc":"<p>The coordinates of the geometry, Object like {x, y} or Array.</p>\n"},"shape":{"!doc":"<p>The associated shape when rendered</p>\n"},"constructor":{"!type":"fn(coords: ?)","!doc":"<p>Constructs a new geometry</p>\n"},"getPoints":{"!type":"fn() -> +any","!doc":"<p>Gets the points of this geometry.</p>\n"},"setPoints":{"!type":"fn(p: ?)","!doc":"<p>Sets the points for this geometry.</p>\n"}}},"dojox/geo/openlayers/GfxLayer":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Layer.prototype","!doc":"<p>A layer dedicated to render dojox.geo.openlayers.GeometryFeature</p>\n","_viewport":{},"added":{"!type":"fn()","!doc":"<p>Called when added to a map.</p>\n"},"addFeature":{"!type":"fn(f: ?)","!doc":"<p>Add a feature or an array of features to the layer.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Removes all the features from this layer.</p>\n"},"constructor":{"!type":"fn(name: ?, options: ?)","!doc":"<p>Constructs a new GFX layer.</p>\n"},"getDojoMap":{"!type":"fn() -> ?"},"getFeatureAt":{"!type":"fn(i: ?) -> +any","!doc":"<p>Returns the i-th feature of this layer.</p>\n"},"getFeatureCount":{"!type":"fn() -> +any","!doc":"<p>Returns the number of the features contained by this layer.</p>\n"},"getFeatures":{"!type":"fn() -> +any","!doc":"<p>Returns the feature hold by this layer.</p>\n"},"getSurface":{"!type":"fn() -> +any","!doc":"<p>Get the underlying dojox.gfx.Surface</p>\n"},"getViewport":{"!type":"fn() -> ?","!doc":"<p>Gets the viewport</p>\n"},"moveTo":{"!type":"fn(event: ?)","!doc":"<p>Called when this layer is moved or zoomed.</p>\n"},"redraw":{"!type":"fn()","!doc":"<p>Redraws this layer</p>\n"},"removeFeature":{"!type":"fn(f: ?)","!doc":"<p>Removes a feature or an array of features from the layer.</p>\n"},"removeFeatureAt":{"!type":"fn(index: ?)","!doc":"<p>Remove the feature at the specified index.</p>\n"},"renderFeature":{"!type":"fn(f: ?)","!doc":"<p>Called when rendering a feature is necessary.</p>\n"},"setMap":{"!type":"fn(map: ?)","!doc":"<p>Sets the map for this layer.</p>\n"},"setViewport":{"!type":"fn(g: ?)","!doc":"<p>Sets the viewport</p>\n"}}},"dojox/geo/openlayers/Layer":{"!type":"fn()","prototype":{"!doc":"<p>Base layer class for dojox.geo.openlayers.Map specific layers extending OpenLayers.Layer class.\nThis layer class accepts Features which encapsulates graphic objects to be added to the map.\nThis layer class encapsulates an OpenLayers.Layer.\nThis class provides Feature management such as add, remove and feature access.</p>\n","added":{"!type":"fn()","!doc":"<p>Called when the layer is added to the map</p>\n"},"addFeature":{"!type":"fn(f: ?)","!doc":"<p>Add a feature or an array of features to the layer.</p>\n"},"clear":{"!type":"fn()","!doc":"<p>Removes all the features from this layer.</p>\n"},"constructor":{"!type":"fn(name: ?, options: ?)","!doc":"<p>Constructs a new Layer.</p>\n"},"getDojoMap":{"!type":"fn() -> ?"},"getFeatureAt":{"!type":"fn(i: ?) -> +any","!doc":"<p>Returns the i-th feature of this layer.</p>\n"},"getFeatureCount":{"!type":"fn() -> +any","!doc":"<p>Returns the number of the features contained by this layer.</p>\n"},"getFeatures":{"!type":"fn() -> +any","!doc":"<p>Returns the feature hold by this layer.</p>\n"},"moveTo":{"!type":"fn(event: ?)","!doc":"<p>Called when the layer is panned or zoomed.</p>\n"},"redraw":{"!type":"fn()","!doc":"<p>Redraws this layer</p>\n"},"removeFeature":{"!type":"fn(f: ?)","!doc":"<p>Removes a feature or an array of features from the layer.</p>\n"},"removeFeatureAt":{"!type":"fn(index: ?)","!doc":"<p>Remove the feature at the specified index.</p>\n"},"renderFeature":{"!type":"fn(f: ?)","!doc":"<p>Called when rendering a feature is necessary.</p>\n"}}},"dojox/geo/openlayers/GreatCircle":{"DEG2RAD":{},"RAD2DEG":{},"TOLERANCE":{},"toGeometryFeature":{"!type":"fn(p1: ?, p2: ?, increment: ?) -> +any","!doc":"<p>Create a geodetic line as an array of dojox.geo.openlayers.GeometryFeature.</p>\n"},"toLineString":{"!type":"fn(p1: ?, p2: ?, increment: ?) -> +instance","!doc":"<p>Create a geodetic line as an array of OpenLayers.Geometry.LineString.</p>\n"},"toPointArray":{"!type":"fn(p1: ?, p2: ?, increment: ?) -> []","!doc":"<p>Create a geodetic line as an array of OpenLayers.Point.</p>\n"}},"dojox/geo/openlayers/JsonImport":{"!type":"fn()","prototype":{"!doc":"<p>Class to load JSON formated ShapeFile as output of the JSon Custom Map Converter.</p>\n","constructor":{"!type":"fn(params: ?)","!doc":"<p>Construct a new JSON importer.</p>\n"},"loadData":{"!type":"fn()","!doc":"<p>Triggers the loading.</p>\n"}}},"dojox/geo/openlayers/Map":{"!type":"fn()","prototype":{"!doc":"<p>A map viewer based on the OpenLayers library.\n</p>\n","_tp":{},"olMap":{"!doc":"<p>The underlying OpenLayers.Map object.\nShould be accessed on read mode only.</p>\n"},"addLayer":{"!type":"fn(layer: ?)","!doc":"<p>Add the specified layer to the map.</p>\n"},"constructor":{"!type":"fn(div: ?, options: ?)","!doc":"<p>Constructs a new Map object</p>\n"},"fitTo":{"!type":"fn(o: ?)","!doc":"<p>Fits the map on a point,or an area</p>\n"},"getBaseLayerType":{"!type":"fn() -> +any","!doc":"<p>Returns the base layer type.</p>\n"},"getLayer":{"!type":"fn(property: ?, value: ?) -> +any","!doc":"<p>Returns the layer whose property matches the value.</p>\n"},"getLayerCount":{"!type":"fn() -> +any","!doc":"<p>Returns the count of layers of this map.</p>\n"},"getOLMap":{"!type":"fn() -> +any","!doc":"<p>gets the underlying OpenLayers map object.</p>\n"},"getScale":{"!type":"fn(geodesic: ?) -> +null","!doc":"<p>Returns the current scale</p>\n"},"initialFit":{"!type":"fn(params: ?)","!doc":"<p>Performs an initial fit to contents.</p>\n"},"layerIndex":{"!type":"fn(layer: ?, index?: ?) -> +any","!doc":"<p>Set or retrieve the layer index.</p>\n"},"removeLayer":{"!type":"fn(layer: ?)","!doc":"<p>Remove the specified layer from the map.</p>\n"},"setBaseLayerType":{"!type":"fn(type: ?) -> +any","!doc":"<p>Set the base layer type, replacing the existing base layer</p>\n"},"transform":{"!type":"fn(p: ?, from: ?, to: ?) -> ?","!doc":"<p>Transforms the point passed as argument, expressed in the <em>from</em> \ncoordinate system to the map coordinate system.</p>\n"},"transformXY":{"!type":"fn(x: ?, y: ?, from?: ?, to?: ?) -> +any","!doc":"<p>Transforms the coordinates passed as argument, expressed in the <em>from</em> \ncoordinate system to the map coordinate system.</p>\n"}}},"dojox/geo/openlayers/TouchInteractionSupport":{"!type":"fn()","prototype":{"!doc":"<p>class to handle touch interactions on a OpenLayers.Map widget</p>\n","_centerTouchLocation":{},"_initialFingerSpacing":{},"_initialScale":{},"_lastTap":{},"_map":{},"_tapCount":{},"_tapThreshold":{},"_touchEndListener":{},"_touchMoveListener":{},"constructor":{"!type":"fn(map: ?)","!doc":"<p>Constructs a new TouchInteractionSupport instance</p>\n"}}},"dojox/geo/openlayers/Patch":{"patchGFX":{"!type":"fn()"}},"dojox/geo/openlayers/WidgetFeature":{"!type":"fn()","prototype":{"!proto":"dojox/geo/openlayers/Feature.prototype","!doc":"<p>Wraps a Dojo widget, provide geolocalisation of the widget and interface\nto Layer class.</p>\n","_bbox":{},"_widget":{},"constructor":{"!type":"fn(params: ?)","!doc":"<p>Constructs a new <code>dojox.geo.openlayers.WidgetFeature</code></p>\n"},"getCoordinateSystem":{"!type":"fn() -> +any","!doc":"<p>Returns the coordinate system in which coordinates of this feature are expressed.</p>\n"},"getLayer":{"!type":"fn() -> +any","!doc":"<p>Returns the Layer to which this feature belongs.</p>\n"},"getParameters":{"!type":"fn() -> +dojox.geo.openlayers.__WidgetFeatureArgs","!doc":"<p>Returns the parameters describing the widget.</p>\n"},"remove":{"!type":"fn()","!doc":"<p>removes this feature.</p>\n"},"render":{"!type":"fn()","!doc":"<p>renders the widget.</p>\n"},"setCoordinateSystem":{"!type":"fn(cs: ?)","!doc":"<p>Set the coordinate system in which coordinates of this feature are expressed.</p>\n"},"setParameters":{"!type":"fn(params: ?)","!doc":"<p>Sets the parameters describing the widget.</p>\n"}}},"dojox/geo/openlayers/widget/Map":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A widget version of the <code>dojox.geo.openlayers.Map</code> component.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"baseLayerType":{"!doc":"<p>Base layer type as defined in <code>dojox.geo.openlayer.BaseLayerType</code>. Can be one of:\n\n</p>\n<ul>\n<li><code>OSM</code></li>\n<li><code>WMS</code></li>\n<li><code>Google</code></li>\n<li><code>VirtualEarth</code></li>\n<li><code>Yahoo</code></li>\n<li><code>ArcGIS</code></li>\n</ul>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"initialLocation":{"!doc":"<p>The part of the map shown at startup time. It is the string description of the location shown at\nstartup time. Format is the same as for the <code>dojox.geo.openlayers.widget.Map.fitTo</code>\nmethod.\n\n</p>\n<pre><code>{\n    bounds: [ulx, uly, lrx, lry]\n}</code></pre>\n<p>The map is fit on the specified bounds expressed as decimal degrees latitude and longitude.\n\n</p>\n<p>The bounds are defined with their upper left and lower right corners coordinates.\n\n\n</p>\n<pre><code>{\n    position: [longitude, latitude],\n    extent: degrees\n}</code></pre>\n<p>The map is fit on the specified position showing the extent <code>&lt;extent&gt;</code> around\n\n</p>\n<p>the specified center position.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"map":{"!doc":"<p>The underlying <code>dojox/geo/openlayers/Map</code> object.</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"touchHandler":{"!doc":"<p>Tells if the touch handler should be attached to the map or not.\nTouch handler handles touch events so that the widget can be used\non mobile applications.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, creates the real dojox.geo.openlayers.Map object.     </p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"resize":{"!type":"fn(b?: ?, h?: ?)","!doc":"<p>Resize the widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/gfx/Moveable":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(shape: ?, params: ?)","!doc":"<p>an object, which makes a shape moveable</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops watching for possible move, deletes all references, so the object can be garbage-collected</p>\n"}}},"dojox/gfx/Mover":{"!type":"fn()","prototype":{"constructor":{"!type":"fn(shape: ?, e: ?, host?: ?)","!doc":"<p>an object, which makes a shape follow the mouse,\nused as a default mover, and as a base class for custom movers</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>stops the move, deletes all references, so the object can be garbage-collected</p>\n"}}},"dojox/gfx/VectorText":{"!type":"fn()","prototype":{"!doc":"<p>An implementation of the SVG Font 1.1 spec, using dojox/gfx.\n\n</p>\n<p>Basic interface:\n\n</p>\n<pre><code>var f = new gfx.Font(url|string);\nsurface||group.createVectorText(text)\n.setFill(fill)\n.setStroke(stroke)\n.setFont(fontStyleObject);</code></pre>\n<p>The arguments passed to createVectorText are the same as you would\npass to surface||group.createText; the difference is that this\nis entirely renderer-agnostic, and the return value is a subclass\nof dojox/gfx.Group.\n\n</p>\n<p>Note also that the &quot;defaultText&quot; object is slightly different:\n{ type:&quot;vectortext&quot;, x:0, y:0, width:null, height: null,\ntext: &quot;&quot;, align: &quot;start&quot;, decoration: &quot;none&quot; }\n\n</p>\n<p>...as well as the &quot;defaultVectorFont&quot; object:\n{ type:&quot;vectorfont&quot;, size:&quot;10pt&quot; }\n\n</p>\n<p>The reason for this should be obvious: most of the style for the font is defined\nby the font object itself.\n\n</p>\n<p>Note that this will only render IF and WHEN you set the font.</p>\n","_entityRe":{},"constructor":{"!type":"fn(url: ?)","!doc":"<p>Create this font object based on the SVG Font definition at url.</p>\n"},"draw":{"!type":"fn(group: ?, textArgs: ?, fontArgs: ?, fillArgs: ?, strokeArgs: ?) -> +dojox","!doc":"<p>based on the passed parameters, draw the given text using paths\ndefined by this font.</p>\n"},"getBaseline":{"!type":"fn(scale?: ?) -> number","!doc":"<p>Find the baseline coord for alignment; adjust for scale if passed.</p>\n"},"getCenterline":{"!type":"fn(scale?: ?) -> number","!doc":"<p>return the y coordinate that is the center of the viewbox.</p>\n"},"getLineHeight":{"!type":"fn(scale?: ?) -> number","!doc":"<p>return the height of a single line, sans leading, based on scale.</p>\n"},"getWidth":{"!type":"fn(text: ?, scale?: ?) -> number","!doc":"<p>Get the width of the rendered text without actually rendering it.</p>\n"},"initialized":{"!type":"fn() -> bool","!doc":"<p>Return if we&#39;ve loaded a font def, and the parsing was successful.</p>\n"},"load":{"!type":"fn(url: ?) -> fn","!doc":"<p>Load the passed SVG and send it to the parser for parsing.</p>\n"}}},"dojox/xml/DomParser":{"prototype":{"constructor":{"!type":"fn()"}}},"dojox/gfx/arc":{"!doc":"<p>This module contains the core graphics Arc functions.</p>\n","curvePI4":{"!doc":"<p>an object with properties of an arc around a unit circle from 0 to pi/4</p>\n"},"arcAsBezier":{"!type":"fn(last: ?, rx: ?, ry: ?, xRotg: ?, large: ?, sweep: ?, x: ?, y: ?) -> []","!doc":"<p>calculates an arc as a series of Bezier curves\ngiven the last point and a standard set of SVG arc parameters,\nit returns an array of arrays of parameters to form a series of\nabsolute Bezier curves.</p>\n"},"unitArcAsBezier":{"!type":"fn(alpha: ?)","!doc":"<p>return a start point, 1st and 2nd control points, and an end point of\na an arc, which is reflected on the x axis</p>\n"}},"dojox/gfx/attach":{},"dojox/gfx/bezierutils":{"computeLength":{"!type":"fn(points: ?) -> number","!doc":"<p>Returns the length of the given bezier curve.</p>\n"},"distance":{"!type":"fn(x1: ?, y1: ?, x2: ?, y2: ?) -> ?","!doc":"<p>Returns the distance between the specified points.</p>\n"},"splitBezierAtT":{"!type":"fn(points: ?, t: ?) -> ?"},"tAtLength":{"!type":"fn(points: ?, length: ?) -> number","!doc":"<p>Returns the t corresponding to the given length for the specified bezier curve.</p>\n"}},"dojox/gfx/canvas":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of this module alone will only add in drawing support.\nThe additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.\nThe id of the Canvas renderer is &#39;canvas&#39;.  This id can be used when switch Dojo&#39;s\ngraphics context between renderer implementations.  See dojox/gfx/_base.switchRenderer\nAPI.</p>\n","attachNode":{"!type":"fn()"},"attachSurface":{"!type":"fn()"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +instance","!doc":"<p>creates a surface (Canvas)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/canvas.Shape":{"!type":"fn()","prototype":{"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape (Canvas), which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","canvasClip":{},"add":{"!type":"fn(shape: ?) -> ?"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn() -> ?"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch.</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +instance","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered disposed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> ?"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a rectangle shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>an ellipse shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn() -> fn"},"setStroke":{"!type":"fn() -> fn"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a circle shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas.Line":{"!type":"fn()","prototype":{"!doc":"<p>a line shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a polyline/polygon shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn() -> fn"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas.Image":{"!type":"fn()","prototype":{"!doc":"<p>an image shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn() -> fn"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas.Text":{"!type":"fn()","prototype":{"canvasClip":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextWidth":{"!type":"fn() -> ?","!doc":"<p>get the text width in pixels</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas.Path":{"!type":"fn()","prototype":{"!doc":"<p>a path shape (Canvas)</p>\n","_2PI":{},"_validSegments":{},"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn() -> ?"},"setStroke":{"!type":"fn() -> fn"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/canvas.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a text shape (Canvas)</p>\n","_2PI":{},"_validSegments":{},"canvasClip":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/canvas.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings (Canvas)</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> ?"},"clear":{"!type":"fn() -> ?"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch.</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +instance","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn()"},"downloadImage":{"!type":"fn(img: ?, url: ?)","!doc":"<p>internal method, which starts an image download and renders, when it is ready</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> +null","!doc":"<p>returns an object with properties &quot;width&quot; and &quot;height&quot;</p>\n"},"getEventSource":{"!type":"fn() -> +null"},"getTextDir":{"!type":"fn() -> ?"},"makeDirty":{"!type":"fn()","!doc":"<p>internal method, which is called when we may need to redraw</p>\n"},"on":{"!type":"fn()"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> ?"},"render":{"!type":"fn(ctx: ?)","!doc":"<p>Renders the gfx scene.</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/decompose":{"!doc":"<p>Decompose a 2D matrix into translation, scaling, and rotation components.</p>\n","prototype":{"constructor":{"!type":"fn(matrix: ?)","!doc":"<p>Decompose a 2D matrix into translation, scaling, and rotation components.</p>\n"}}},"dojox/gfx/canvasWithEvents":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers which extends\nthe basic canvas drawing renderer bridge to add additional support for graphics events\non Shapes.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of the canvas module alone will only add in drawing support.\nThis additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.</p>\n","Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)","!doc":"<p>creates a surface (Canvas)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/canvasWithEvents.Shape":{"!type":"fn()","prototype":{"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvasWithEvents.Group":{"!type":"fn()","prototype":{"canvasClip":{},"add":{"!type":"fn(shape: ?) -> ?"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn() -> ?"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch.</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +instance","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> ?"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvasWithEvents.Image":{"!type":"fn()","prototype":{"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvasWithEvents.Text":{"!type":"fn()","prototype":{"canvasClip":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextWidth":{"!type":"fn() -> ?","!doc":"<p>get the text width in pixels</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvasWithEvents.Rect":{"!type":"fn()","prototype":{"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvasWithEvents.Circle":{"!type":"fn()","prototype":{"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvasWithEvents.Ellipse":{"!type":"fn()","prototype":{"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvasWithEvents.Line":{"!type":"fn()","prototype":{"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvasWithEvents.Polyline":{"!type":"fn()","prototype":{"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvasWithEvents.Path":{"!type":"fn()","prototype":{"_2PI":{},"_validSegments":{},"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/canvasWithEvents.TextPath":{"!type":"fn()","prototype":{"_2PI":{},"_validSegments":{},"canvasClip":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this shape. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects an event handler. Use <code>handle.remove</code> instead.</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/canvasWithEvents.Surface":{"!type":"fn()","prototype":{"isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> ?"},"clear":{"!type":"fn() -> ?"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch.</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>Deprecated. Connects a handler to an event on this surface. Use <code>on</code> instead.</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +instance","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated. Disconnects a handler. Use <code>handle.remove</code> instead.</p>\n"},"downloadImage":{"!type":"fn(img: ?, url: ?)","!doc":"<p>internal method, which starts an image download and renders, when it is ready</p>\n"},"fixTarget":{"!type":"fn(listener: ?) -> fn","!doc":"<p>Corrects the <code>target</code> properties of the event object passed to the actual listener.</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> +null","!doc":"<p>returns an object with properties &quot;width&quot; and &quot;height&quot;</p>\n"},"getEventSource":{"!type":"fn() -> ?"},"getTextDir":{"!type":"fn() -> ?"},"makeDirty":{"!type":"fn()","!doc":"<p>internal method, which is called when we may need to redraw</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this surface.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> ?"},"render":{"!type":"fn(ctx: ?)","!doc":"<p>Renders the gfx scene.</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/canvas_attach":{"!doc":"<p>This the graphics rendering bridge for W3C Canvas compliant browsers.\nSince Canvas is an immediate mode graphics api, with no object graph or\neventing capabilities, use of this module alone will only add in drawing support.\nThe additional module, canvasWithEvents extends this module with additional support\nfor handling events on Canvas.  By default, the support for events is now included\nhowever, if only drawing capabilities are needed, canvas event module can be disabled\nusing the dojoConfig option, canvasEvents:true|false.\nThe id of the Canvas renderer is &#39;canvas&#39;.  This id can be used when switch Dojo&#39;s\ngraphics context between renderer implementations.  See dojox/gfx/_base.switchRenderer\nAPI.</p>\n","attachNode":{"!type":"fn()"},"attachSurface":{"!type":"fn()"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +instance","!doc":"<p>creates a surface (Canvas)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/canvas_attach.Shape":{"!type":"fn()","prototype":{"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas_attach.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape (Canvas), which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","canvasClip":{},"add":{"!type":"fn(shape: ?) -> ?"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn() -> ?"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch.</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +instance","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered disposed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> ?"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas_attach.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a rectangle shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas_attach.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>an ellipse shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn() -> fn"},"setStroke":{"!type":"fn() -> fn"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas_attach.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a circle shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas_attach.Line":{"!type":"fn()","prototype":{"!doc":"<p>a line shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas_attach.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a polyline/polygon shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn() -> fn"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas_attach.Image":{"!type":"fn()","prototype":{"!doc":"<p>an image shape (Canvas)</p>\n","canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn() -> fn"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas_attach.Text":{"!type":"fn()","prototype":{"canvasClip":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextWidth":{"!type":"fn() -> ?","!doc":"<p>get the text width in pixels</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/canvas_attach.Path":{"!type":"fn()","prototype":{"!doc":"<p>a path shape (Canvas)</p>\n","_2PI":{},"_validSegments":{},"canvasClip":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setShape":{"!type":"fn() -> ?"},"setStroke":{"!type":"fn() -> fn"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/canvas_attach.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a text shape (Canvas)</p>\n","_2PI":{},"_validSegments":{},"canvasClip":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn()"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> +null"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn()"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object\n(the default implementation simply ignores it)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object\n(the default implementation simply ignores it)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}}},"dojox/gfx/canvas_attach.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings (Canvas)</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> ?"},"clear":{"!type":"fn() -> ?"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch.</p>\n"},"connect":{"!type":"fn()"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +instance","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn()"},"downloadImage":{"!type":"fn(img: ?, url: ?)","!doc":"<p>internal method, which starts an image download and renders, when it is ready</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> +null","!doc":"<p>returns an object with properties &quot;width&quot; and &quot;height&quot;</p>\n"},"getEventSource":{"!type":"fn() -> +null"},"getTextDir":{"!type":"fn() -> ?"},"makeDirty":{"!type":"fn()","!doc":"<p>internal method, which is called when we may need to redraw</p>\n"},"on":{"!type":"fn()"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently: ?) -> ?"},"render":{"!type":"fn(ctx: ?)","!doc":"<p>Renders the gfx scene.</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/canvasext":{"!doc":"<p>A module that adds canvas-specific features to the gfx api. You should require this module\nwhen your application specifically targets the HTML5 Canvas renderer.</p>\n"},"dojox/gfx/filters":{"!doc":"<p>A module that defines a minimal API to create SVG filter definition objects to be used with the\ndojox/gfx/svgext/Shape.setFilter() API, as well as a set of predefined filters.</p>\n"},"dojox/gfx/gradient":{"project":{"!type":"fn(matrix: ?, gradient: ?, tl: ?, rb: ?, ttl: ?, trb: ?) -> ?","!doc":"<p>Returns a new gradient using the &quot;VML algorithm&quot; and suitable for VML.</p>\n"},"rescale":{"!type":"fn(stops: ?, from: ?, to: ?) -> []","!doc":"<p>Recalculates a gradient from 0-1 window to\n&quot;from&quot;-&quot;to&quot; window blending and replicating colors,\nif necessary.</p>\n"}},"dojox/gfx/move":{},"dojox/gfx/registry":{"byId":{"!type":"fn(id: ?) -> +dojox","!doc":"<p>Returns the shape that matches the specified id.</p>\n"},"dispose":{"!type":"fn(s: ?, recurse?: ?)","!doc":"<p>Removes the specified shape from the registry.</p>\n"},"register":{"!type":"fn(s: ?) -> number","!doc":"<p>Register the specified shape into the graphics registry.</p>\n"}},"dojox/gfx/renderer":{"!doc":"<p>This module is an AMD loader plugin that loads the appropriate graphics renderer\nimplementation based on detected environment and current configuration settings.</p>\n"},"dojox/gfx/silverlight":{"!doc":"<p>This the graphics rendering bridge for the Microsoft Silverlight plugin.\nSilverlight is a faster implementation on IE6-8 than the default 2d graphics, VML</p>\n","attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)","!doc":"<p>creates a surface (Silverlight)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/silverlight.Shape":{"!type":"fn()","prototype":{"!doc":"<p>Silverlight-specific implementation of dojox/gfx/shape.Shape methods</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(name: ?, listener: ?) -> ?"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/silverlight.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape (Silverlight), which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","add":{"!type":"fn(shape: ?) -> fn","!doc":"<p>adds a shape to a group/surface</p>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered disposed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>sets a raw Silverlight node to be used by this shape</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a rectangle shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a rectangle shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>an ellipse shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets an ellipse shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a circle shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a circle shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.Line":{"!type":"fn()","prototype":{"!doc":"<p>a line shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a line shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a polyline/polygon shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(points: ?, closed: ?) -> fn","!doc":"<p>sets a polyline/polygon shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.Image":{"!type":"fn()","prototype":{"!doc":"<p>an image (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets an image shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.Text":{"!type":"fn()","prototype":{"!doc":"<p>an anchored text (Silverlight)</p>\n","textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextWidth":{"!type":"fn() -> ?","!doc":"<p>get the text width in pixels</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a text shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn() -> fn","!doc":"<p>ignore setting a stroke style</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.Path":{"!type":"fn()","prototype":{"!doc":"<p>a path shape (Silverlight)</p>\n","_2PI":{},"_validSegments":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a textpath shape (Silverlight)</p>\n","_2PI":{},"_validSegments":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?)","!doc":"<p>forms a path using a shape (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings (Silverlight)</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> fn","!doc":"<p>adds a shape to a group/surface</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> +null","!doc":"<p>returns an object with properties &quot;width&quot; and &quot;height&quot;</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(name: ?, listener: ?) -> ?"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/silverlight_attach":{"!doc":"<p>This the graphics rendering bridge for the Microsoft Silverlight plugin.\nSilverlight is a faster implementation on IE6-8 than the default 2d graphics, VML</p>\n","attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)","!doc":"<p>creates a surface (Silverlight)</p>\n"},"Ellipse":{"!type":"fn()"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/silverlight_attach.Shape":{"!type":"fn()","prototype":{"!doc":"<p>Silverlight-specific implementation of dojox/gfx/shape.Shape methods</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(name: ?, listener: ?) -> ?"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/silverlight_attach.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape (Silverlight), which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","add":{"!type":"fn(shape: ?) -> fn","!doc":"<p>adds a shape to a group/surface</p>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered disposed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>sets a raw Silverlight node to be used by this shape</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a rectangle shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a rectangle shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>an ellipse shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets an ellipse shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a circle shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a circle shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.Line":{"!type":"fn()","prototype":{"!doc":"<p>a line shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a line shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a polyline/polygon shape (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(points: ?, closed: ?) -> fn","!doc":"<p>sets a polyline/polygon shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.Image":{"!type":"fn()","prototype":{"!doc":"<p>an image (Silverlight)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets an image shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.Text":{"!type":"fn()","prototype":{"!doc":"<p>an anchored text (Silverlight)</p>\n","textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextWidth":{"!type":"fn() -> ?","!doc":"<p>get the text width in pixels</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a text shape object (Silverlight)</p>\n"},"setStroke":{"!type":"fn() -> fn","!doc":"<p>ignore setting a stroke style</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.Path":{"!type":"fn()","prototype":{"!doc":"<p>a path shape (Silverlight)</p>\n","_2PI":{},"_validSegments":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a textpath shape (Silverlight)</p>\n","_2PI":{},"_validSegments":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (Silverlight)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?)","!doc":"<p>forms a path using a shape (Silverlight)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (Silverlight)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/silverlight_attach.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings (Silverlight)</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> fn","!doc":"<p>adds a shape to a group/surface</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> +null","!doc":"<p>returns an object with properties &quot;width&quot; and &quot;height&quot;</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(name: ?, listener: ?) -> ?"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/svg":{"!doc":"<p>This the graphics rendering bridge for browsers compliant with W3C SVG1.0.\nThis is the preferred renderer to use for interactive and accessible graphics.</p>\n","dasharray":{},"useSvgWeb":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"getRef":{"!type":"fn(name: ?) -> +any","!doc":"<p>looks up a node by its external name</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/svg.xmlns":{"svg":{},"xlink":{}},"dojox/gfx/svg.dasharray":{"dash":{},"dashdot":{},"dot":{},"longdash":{},"longdashdot":{},"longdashdotdot":{},"shortdash":{},"shortdashdot":{},"shortdashdotdot":{},"shortdot":{},"solid":{}},"dojox/gfx/svg.Shape":{"!type":"fn()","prototype":{"!doc":"<p>SVG-specific implementation of dojox/gfx/shape.Shape methods</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> []"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?)"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/svg.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape (SVG), which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","add":{"!type":"fn(shape: ?) -> fn","!doc":"<p>adds a shape to a group/surface</p>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered disposed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>sets a raw SVG node to be used by this shape</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a rectangle shape (SVG)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a rectangle shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg.Ellipse":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg.Circle":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg.Line":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a polyline/polygon shape (SVG)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(points: ?, closed: ?) -> fn","!doc":"<p>sets a polyline/polygon shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg.Image":{"!type":"fn()","prototype":{"!doc":"<p>an image (SVG)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets an image shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg.Text":{"!type":"fn()","prototype":{"!doc":"<p>an anchored text (SVG)</p>\n","textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextWidth":{"!type":"fn() -> number","!doc":"<p>get the text width in pixels</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a text shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg.Path":{"!type":"fn()","prototype":{"!doc":"<p>a path shape (SVG)</p>\n","_2PI":{},"_validSegments":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/svg.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a textpath shape (SVG)</p>\n","_2PI":{},"_validSegments":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/svg.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings (SVG)</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> fn","!doc":"<p>adds a shape to a group/surface</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> []"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?)"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> +null","!doc":"<p>returns an object with properties &quot;width&quot; and &quot;height&quot;</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/svg_attach":{"!doc":"<p>This the graphics rendering bridge for browsers compliant with W3C SVG1.0.\nThis is the preferred renderer to use for interactive and accessible graphics.</p>\n","dasharray":{},"useSvgWeb":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?)"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"getRef":{"!type":"fn(name: ?) -> +any","!doc":"<p>looks up a node by its external name</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/svg_attach.xmlns":{"svg":{},"xlink":{}},"dojox/gfx/svg_attach.dasharray":{"dash":{},"dashdot":{},"dot":{},"longdash":{},"longdashdot":{},"longdashdotdot":{},"shortdash":{},"shortdashdot":{},"shortdashdotdot":{},"shortdot":{},"solid":{}},"dojox/gfx/svg_attach.Shape":{"!type":"fn()","prototype":{"!doc":"<p>SVG-specific implementation of dojox/gfx/shape.Shape methods</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> []"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?)"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/svg_attach.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape (SVG), which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","add":{"!type":"fn(shape: ?) -> fn","!doc":"<p>adds a shape to a group/surface</p>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered disposed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>sets a raw SVG node to be used by this shape</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a rectangle shape (SVG)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a rectangle shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.Ellipse":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.Circle":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.Line":{"!type":"fn()","prototype":{"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a polyline/polygon shape (SVG)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(points: ?, closed: ?) -> fn","!doc":"<p>sets a polyline/polygon shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.Image":{"!type":"fn()","prototype":{"!doc":"<p>an image (SVG)</p>\n","applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets an image shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.Text":{"!type":"fn()","prototype":{"!doc":"<p>an anchored text (SVG)</p>\n","textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextWidth":{"!type":"fn() -> number","!doc":"<p>get the text width in pixels</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a text shape object (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.Path":{"!type":"fn()","prototype":{"!doc":"<p>a path shape (SVG)</p>\n","_2PI":{},"_validSegments":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a textpath shape (SVG)</p>\n","_2PI":{},"_validSegments":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (SVG)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (SVG)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (SVG)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/svg_attach.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings (SVG)</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> fn","!doc":"<p>adds a shape to a group/surface</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> []"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> ?","!doc":"<p>creates a image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> ?","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?)"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> +null","!doc":"<p>returns an object with properties &quot;width&quot; and &quot;height&quot;</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/svgext":{"!doc":"<p>A module that adds svg-specific features to the gfx api. You should require this module\nwhen your application specifically targets the SVG renderer.</p>\n"},"dojox/gfx/vml":{"!doc":"<p>This the default graphics rendering bridge for IE6-7.\nThis renderer is very slow.  For best performance on IE6-8, use Silverlight plugin.\nIE9+ defaults to the standard W3C SVG renderer.</p>\n","_bool":{},"text_alignment":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +any","!doc":"<p>creates a surface (VML)</p>\n"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/vml.text_alignment":{"end":{},"middle":{},"start":{}},"dojox/gfx/vml._bool":{"t":{},"true":{}},"dojox/gfx/vml.Shape":{"!type":"fn()","prototype":{"!doc":"<p>VML-specific implementation of dojox/gfx/shape.Shape methods</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/vml.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape (VML), which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","_capMap":{},"_capMapReversed":{},"add":{"!type":"fn(shape: ?) -> fn"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a VML group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> +null","!doc":"<p>creates a VML image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> +null","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered disposed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a rectangle shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a rectangle shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>an ellipse shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets an ellipse shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a circle shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a circle shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml.Line":{"!type":"fn()","prototype":{"!doc":"<p>a line shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a line shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a polyline/polygon shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(points: ?, closed?: ?) -> ?","!doc":"<p>sets a polyline/polygon shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml.Image":{"!type":"fn()","prototype":{"!doc":"<p>an image (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets an image shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml.Text":{"!type":"fn()","prototype":{"!doc":"<p>an anchored text (VML)</p>\n","_alignment":{},"_capMap":{},"_capMapReversed":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextWidth":{"!type":"fn() -> ?","!doc":"<p>get the text width, in px</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a text shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml.Path":{"!type":"fn()","prototype":{"!doc":"<p>a path shape (VML)</p>\n","_2PI":{},"_capMap":{},"_capMapReversed":{},"_pathVmlToSvgMap":{},"_validSegments":{},"renderers":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/vml.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a textpath shape (VML)</p>\n","_2PI":{},"_capMap":{},"_capMapReversed":{},"_pathVmlToSvgMap":{},"_validSegments":{},"renderers":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/vml.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings (VML)</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> fn"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a VML group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> +null","!doc":"<p>creates a VML image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> +null","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> +null","!doc":"<p>returns an object with properties &quot;width&quot; and &quot;height&quot;</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/gfx/vml_attach":{"!doc":"<p>This the default graphics rendering bridge for IE6-7.\nThis renderer is very slow.  For best performance on IE6-8, use Silverlight plugin.\nIE9+ defaults to the standard W3C SVG renderer.</p>\n","_bool":{},"text_alignment":{},"xmlns":{},"attachNode":{"!type":"fn(node: ?)","!doc":"<p>creates a shape from a Node</p>\n"},"attachSurface":{"!type":"fn(node: ?)","!doc":"<p>creates a surface from a Node</p>\n"},"Circle":{"!type":"fn()"},"createSurface":{"!type":"fn(parentNode: ?, width: ?, height: ?) -> +any","!doc":"<p>creates a surface (VML)</p>\n"},"Ellipse":{"!type":"fn()"},"fixTarget":{"!type":"fn(event: ?, gfxElement: ?)","!doc":"<p>Adds the gfxElement to event.gfxTarget if none exists. This new\nproperty will carry the GFX element associated with this event.</p>\n"},"Group":{"!type":"fn()"},"Image":{"!type":"fn()"},"Line":{"!type":"fn()"},"Path":{"!type":"fn()"},"Polyline":{"!type":"fn()"},"Rect":{"!type":"fn()"},"Shape":{"!type":"fn()"},"Surface":{"!type":"fn()"},"Text":{"!type":"fn()"},"TextPath":{"!type":"fn()"}},"dojox/gfx/vml_attach.text_alignment":{"end":{},"middle":{},"start":{}},"dojox/gfx/vml_attach._bool":{"t":{},"true":{}},"dojox/gfx/vml_attach.Shape":{"!type":"fn()","prototype":{"!doc":"<p>VML-specific implementation of dojox/gfx/shape.Shape methods</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}}},"dojox/gfx/vml_attach.Group":{"!type":"fn()","prototype":{"!doc":"<p>a group shape (VML), which can be used\nto logically group shapes (e.g, to propagate matricies)</p>\n","_capMap":{},"_capMapReversed":{},"add":{"!type":"fn(shape: ?) -> fn"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a VML group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> +null","!doc":"<p>creates a VML image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> +null","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered disposed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(shape: ?) -> fn","!doc":"<p>sets a shape object\n(the default implementation simply ignores it)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.Rect":{"!type":"fn()","prototype":{"!doc":"<p>a rectangle shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a rectangle shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.Ellipse":{"!type":"fn()","prototype":{"!doc":"<p>an ellipse shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets an ellipse shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.Circle":{"!type":"fn()","prototype":{"!doc":"<p>a circle shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>sets a circle shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.Line":{"!type":"fn()","prototype":{"!doc":"<p>a line shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a line shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.Polyline":{"!type":"fn()","prototype":{"!doc":"<p>a polyline/polygon shape (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(points: ?, closed?: ?) -> ?","!doc":"<p>sets a polyline/polygon shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.Image":{"!type":"fn()","prototype":{"!doc":"<p>an image (VML)</p>\n","_capMap":{},"_capMapReversed":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets an image shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.Text":{"!type":"fn()","prototype":{"!doc":"<p>an anchored text (VML)</p>\n","_alignment":{},"_capMap":{},"_capMapReversed":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"bidiPreprocess":{"!type":"fn(newShape: ?) -> ?"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTextWidth":{"!type":"fn() -> ?","!doc":"<p>get the text width, in px</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> ?","!doc":"<p>sets a text shape object (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.Path":{"!type":"fn()","prototype":{"!doc":"<p>a path shape (VML)</p>\n","_2PI":{},"_capMap":{},"_capMapReversed":{},"_pathVmlToSvgMap":{},"_validSegments":{},"renderers":{},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.TextPath":{"!type":"fn()","prototype":{"!doc":"<p>a textpath shape (VML)</p>\n","_2PI":{},"_capMap":{},"_capMapReversed":{},"_pathVmlToSvgMap":{},"_validSegments":{},"renderers":{},"textDir":{"!doc":"<p>Used for displaying bidi scripts in right layout.\nDefines the base direction of text that displayed, can have 3 values:\n\n</p>\n<ol>\n<li>&quot;ltr&quot; - base direction is left to right.</li>\n<li>&quot;rtl&quot; - base direction is right to left.</li>\n<li>&quot;auto&quot; - base direction is contextual (defined by first strong character).</li>\n</ol>\n"},"applyLeftTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on left side\n(matrix * this.matrix)</p>\n"},"applyRightTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>multiplies the existing matrix with an argument on right side\n(this.matrix * matrix)</p>\n"},"applyTransform":{"!type":"fn(matrix: ?) -> fn","!doc":"<p>a shortcut for dojox/gfx/shape.Shape.applyRightTransform</p>\n"},"arcTo":{"!type":"fn() -> fn","!doc":"<p>forms an elliptic arc segment</p>\n"},"bidiPreprocess":{"!type":"fn(newText: ?) -> ?"},"closePath":{"!type":"fn() -> fn","!doc":"<p>closes a path</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn(rawNode: ?)"},"curveTo":{"!type":"fn() -> fn","!doc":"<p>forms a curve segment</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Releases all internal resources owned by this shape. Once this method has been called,\nthe instance is considered destroyed and should not be used anymore.</p>\n"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"formatText":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Applies the right transform on text, according to renderer.</p>\n"},"getAbsoluteMode":{"!type":"fn() -> ?","!doc":"<p>returns a current value of the absolute mode</p>\n"},"getBoundingBox":{"!type":"fn() -> ?","!doc":"<p>Returns the bounding box Rectangle for this shape or null if a BoundingBox cannot be\ncalculated for the shape on the current renderer or for shapes with no geometric area (points).\nA bounding box is a rectangular geometric region\ndefining the X and Y extent of the shape.\n(see dojox/gfx.defaultRect)\nNote that this method returns a direct reference to the attribute of this instance. Therefore you should\nnot modify its value directly but clone it instead.</p>\n"},"getClip":{"!type":"fn() -> ?"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a Node, which is used as\na source of events for this shape</p>\n"},"getFill":{"!type":"fn() -> ?","!doc":"<p>Returns the current fill object or null\n(see dojox/gfx.defaultLinearGradient,\ndojox/gfx.defaultRadialGradient,\ndojox/gfx.defaultPattern,\nor dojo/Color)</p>\n"},"getFont":{"!type":"fn() -> ?","!doc":"<p>returns the current font object or null</p>\n"},"getLastPosition":{"!type":"fn() -> +null","!doc":"<p>returns the last point in the path, or null</p>\n"},"getNode":{"!type":"fn() -> ?","!doc":"<p>Different graphics rendering subsystems implement shapes in different ways.  This\nmethod provides access to the underlying graphics subsystem object.  Clients calling this\nmethod and using the return value must be careful not to try sharing or using the underlying node\nin a general way across renderer implementation.\nReturns the underlying graphics Node, or null if no underlying graphics node is used by this shape.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent Shape, Group or null if this Shape is unparented.\n(see dojox/gfx/shape.Surface,\nor dojox/gfx.Group)</p>\n"},"getShape":{"!type":"fn() -> ?","!doc":"<p>returns the current Shape object or null\n(see dojox/gfx.defaultPath,\ndojox/gfx.defaultPolyline,\ndojox/gfx.defaultRect,\ndojox/gfx.defaultEllipse,\ndojox/gfx.defaultCircle,\ndojox/gfx.defaultLine,\nor dojox/gfx.defaultImage)</p>\n"},"getStroke":{"!type":"fn() -> ?","!doc":"<p>Returns the current stroke object or null\n(see dojox/gfx.defaultStroke)</p>\n"},"getText":{"!type":"fn() -> ?","!doc":"<p>returns the current text object or null</p>\n"},"getTransform":{"!type":"fn() -> ?","!doc":"<p>Returns the current transformation matrix applied to this Shape or null</p>\n"},"getTransformedBoundingBox":{"!type":"fn() -> +null","!doc":"<p>returns an array of four points or null\nfour points represent four corners of the untransformed bounding box</p>\n"},"hLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a horizontal line segment</p>\n"},"lineTo":{"!type":"fn() -> fn","!doc":"<p>forms a line segment</p>\n"},"moveTo":{"!type":"fn() -> fn","!doc":"<p>forms a move segment</p>\n"},"moveToBack":{"!type":"fn() -> fn","!doc":"<p>moves a shape to back of its parent&#39;s list of shapes</p>\n"},"moveToFront":{"!type":"fn() -> fn","!doc":"<p>moves a shape to front of its parent&#39;s list of shapes</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"qCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic curve segment</p>\n"},"qSmoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a quadratic smooth curve segment</p>\n"},"removeShape":{"!type":"fn(silently: ?) -> fn","!doc":"<p>removes the shape from its parent&#39;s list of shapes</p>\n"},"setAbsoluteMode":{"!type":"fn(mode: ?) -> fn","!doc":"<p>sets an absolute or relative mode for path points</p>\n"},"setClip":{"!type":"fn(clip: ?) -> fn","!doc":"<p>sets the clipping area of this shape.</p>\n"},"setFill":{"!type":"fn(fill: ?) -> fn","!doc":"<p>sets a fill object (VML)</p>\n"},"setFont":{"!type":"fn(newFont: ?) -> fn","!doc":"<p>sets a font for text</p>\n"},"setRawNode":{"!type":"fn(rawNode: ?)","!doc":"<p>assigns and clears the underlying node that will represent this\nshape. Once set, transforms, gradients, etc, can be applied.\n(no fill &amp; stroke by default)</p>\n"},"setShape":{"!type":"fn(newShape: ?) -> fn","!doc":"<p>forms a path using a shape (VML)</p>\n"},"setStroke":{"!type":"fn(stroke: ?) -> fn","!doc":"<p>sets a stroke object (VML)</p>\n"},"setText":{"!type":"fn(newText: ?) -> fn","!doc":"<p>sets a text to be drawn along the path</p>\n"},"setTransform":{"!type":"fn(matrix: ?) -> ?","!doc":"<p>sets a transformation matrix</p>\n"},"smoothCurveTo":{"!type":"fn() -> fn","!doc":"<p>forms a smooth curve segment</p>\n"},"vLineTo":{"!type":"fn() -> fn","!doc":"<p>forms a vertical line segment</p>\n"}},"nodeType":{}},"dojox/gfx/vml_attach.Surface":{"!type":"fn()","prototype":{"!doc":"<p>a surface object to be used for drawings (VML)</p>\n","isLoaded":{},"textDir":{"!doc":"<p>Will be used as default for Text/TextPath/Group objects that created by this surface\nand textDir wasn&#39;t directly specified for them, though the bidi support was loaded.\nCan be set in two ways:\n\n</p>\n<ol>\n<li>When the surface is created and textDir value passed to it as fourth\nparameter.</li>\n<li>Using the setTextDir(String) function, when this function is used the value\nof textDir propagates to all of it&#39;s children and the children of children (for Groups) etc.</li>\n</ol>\n"},"add":{"!type":"fn(shape: ?) -> fn"},"clear":{"!type":"fn() -> ?","!doc":"<p>removes all shapes from a group/surface</p>\n"},"closeBatch":{"!type":"fn() -> fn","!doc":"<p>submits the current batch, append all pending child shapes to DOM</p>\n"},"connect":{"!type":"fn(name: ?, object: ?, method: ?) -> ?","!doc":"<p>connects a handler to an event on this shape</p>\n"},"constructor":{"!type":"fn()"},"createCircle":{"!type":"fn(circle: ?) -> ?","!doc":"<p>creates a circle shape</p>\n"},"createEllipse":{"!type":"fn(ellipse: ?) -> ?","!doc":"<p>creates an ellipse shape</p>\n"},"createGroup":{"!type":"fn() -> ?","!doc":"<p>creates a VML group shape</p>\n"},"createImage":{"!type":"fn(image: ?) -> +null","!doc":"<p>creates a VML image shape</p>\n"},"createLine":{"!type":"fn(line: ?) -> ?","!doc":"<p>creates a line shape</p>\n"},"createObject":{"!type":"fn(shapeType: ?, rawShape: ?) -> +null","!doc":"<p>creates an instance of the passed shapeType class</p>\n"},"createPath":{"!type":"fn(path: ?) -> ?","!doc":"<p>creates a path shape</p>\n"},"createPolyline":{"!type":"fn(points: ?) -> ?","!doc":"<p>creates a polyline/polygon shape</p>\n"},"createRect":{"!type":"fn(rect: ?) -> +null","!doc":"<p>creates a rectangle shape</p>\n"},"createShape":{"!type":"fn(shape: ?) -> +any","!doc":"<p>creates a shape object based on its type; it is meant to be used\nby group-like objects</p>\n"},"createText":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createTextPath":{"!type":"fn(text: ?) -> ?","!doc":"<p>creates a text shape</p>\n"},"createViewport":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(token: ?) -> ?","!doc":"<p>connects a handler by token from an event on this shape</p>\n"},"getBoundingBox":{"!type":"fn() -> +null","!doc":"<p>Returns the bounding box Rectangle for this shape.</p>\n"},"getDimensions":{"!type":"fn() -> +null","!doc":"<p>returns an object with properties &quot;width&quot; and &quot;height&quot;</p>\n"},"getEventSource":{"!type":"fn() -> ?","!doc":"<p>returns a node, which can be used to attach event listeners</p>\n"},"getTextDir":{"!type":"fn() -> ?"},"on":{"!type":"fn(type: ?, listener: ?) -> ?","!doc":"<p>Connects an event to this shape.</p>\n"},"openBatch":{"!type":"fn() -> fn","!doc":"<p>starts a new batch, subsequent new child shapes will be held in\nthe batch instead of appending to the container directly.</p>\n"},"remove":{"!type":"fn(shape: ?, silently?: ?) -> fn","!doc":"<p>remove a shape from a group/surface</p>\n"},"setDimensions":{"!type":"fn(width: ?, height: ?) -> fn","!doc":"<p>sets the width and height of the rawNode</p>\n"},"setTextDir":{"!type":"fn(newTextDir: ?)","!doc":"<p>Used for propagation and change of textDir.\nnewTextDir will be forced as textDir for all of it&#39;s children (Group/Text/TextPath).</p>\n"},"whenLoaded":{"!type":"fn(context: ?, method: ?)"}}},"dojox/grid/BidiSupport":{},"dojox/grid/bidi/_BidiMixin":{"!doc":"<p>Module that deals with BIDI for Grid</p>\n"},"dojox/grid/_Builder":{},"dojox/grid/util":{"!doc":"<p>grid utility library</p>\n","gridViewTag":{},"keyEvents":{},"mouseEvents":{},"na":{},"rowIndexTag":{},"arrayCompare":{"!type":"fn(inA: ?, inB: ?)"},"arrayInsert":{"!type":"fn(inArray: ?, inIndex: ?, inValue: ?)"},"arrayRemove":{"!type":"fn(inArray: ?, inIndex: ?)"},"arraySwap":{"!type":"fn(inArray: ?, inI: ?, inJ: ?)"},"fire":{"!type":"fn(ob: ?, ev: ?, args: ?) -> ?"},"funnelEvents":{"!type":"fn(inNode: ?, inObject: ?, inMethod: ?, inEvents: ?)"},"removeNode":{"!type":"fn(inNode: ?) -> ?"},"setStyleHeightPx":{"!type":"fn(inElement: ?, inHeight: ?)"}},"dojox/grid/_Grid":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A grid widget with virtual scrolling, cell editing, complex rows,\nsorting, fixed columns, sizeable columns, etc.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_click":{},"_focusManager":{},"_placeholders":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setSummaryAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoHeight":{"!doc":"<p>If autoHeight is true, grid height is automatically set to fit the data.\nIf it is an integer, the height will be automatically set to fit the data\nif there are fewer than that many rows - and the height will be set to show\nthat many rows if there are more</p>\n"},"autoRender":{"!doc":"<p>If autoRender is true, grid will render itself after initialization.</p>\n"},"autoWidth":{"!doc":"<p>If autoWidth is true, grid width is automatically set to fit the data.</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"cellOverClass":{"!doc":"<p>css class to apply to grid cells over which the cursor is placed.</p>\n"},"class":{},"classTag":{"!doc":"<p>CSS class applied to the grid&#39;s domNode</p>\n"},"columnReordering":{"!doc":"<p>If set to true, will add drag and drop reordering to views with one row of columns.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultHeight":{"!doc":"<p>default height of the grid, measured in any valid css unit.</p>\n"},"delayScroll":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>indicates if the grid contains editable cells, default is false\nset to true if editable cell encountered during rendering</p>\n"},"elasticView":{"!doc":"<p>Override defaults and make the indexed grid view elastic, thus filling available horizontal space.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows when the grid encounters an error loading</p>\n"},"escapeHTMLInData":{"!doc":"<p>This will escape HTML brackets from the data to prevent HTML from\nuser-inputted data being rendered with may contain JavaScript and result in\nXSS attacks. This is true by default, and it is recommended that it remain\ntrue. Setting this to false will allow data to be displayed in the grid without\nfiltering, and should be only used if it is known that the data won&#39;t contain\nmalicious scripts. If HTML is needed in grid cells, it is recommended that\nyou use the formatter function to generate the HTML (the output of\nformatter functions is not filtered, even with escapeHTMLInData set to true).</p>\n"},"fastScroll":{"!doc":"<p>flag modifies vertical scrolling behavior. Defaults to true but set to false for slower\nscroll performance but more immediate scrolling feedback</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatterScope":{"!doc":"<p>An object to execute format functions within.  If not set, the\nformat functions will execute within the scope of the cell that\nhas a format function.</p>\n"},"headerMenu":{"!doc":"<p>If set to a dijit.Menu, will use this as a context menu for the grid headers.</p>\n"},"height":{"!doc":"<p>explicit height of the grid, measured in any valid css unit.  This will be populated (and overridden)\nif the height: css attribute exists on the source node.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"initialWidth":{"!doc":"<p>A css string to use to set our initial width (only used if autoWidth\nis true).  The first rendering of the grid will be this width, any\nresizing of columns, etc will result in the grid switching to\nautoWidth mode.  Note, this width will override any styling in a\nstylesheet or directly on the node.</p>\n"},"invalidated":{},"keepRows":{"!doc":"<p>Number of rows to keep in the rendering cache.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while the grid is loading</p>\n"},"noDataMessage":{"!doc":"<p>Message that shows if the grid has no data - wrap it in a\nspan with class &#39;dojoxGridNoData&#39; if you want it to be\nstyled similar to the loading and error messages</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeholderLabel":{"!doc":"<p>Label of placeholders to search for in the header menu to replace with column toggling\nmenu items.</p>\n"},"rowCount":{},"rowHeight":{"!doc":"<p>If rowHeight is set to a positive number, it will define the height of the rows\nin pixels. This can provide a significant performance advantage, since it\neliminates the need to measure row sizes during rendering, which is one\nthe primary bottlenecks in the DataGrid&#39;s performance.</p>\n"},"rowSelector":{"!doc":"<p>If set to true, will add a row selector view to this grid.  If set to a CSS width, will add\na row selector of that width to this grid.</p>\n"},"rowsPerPage":{"!doc":"<p>Number of rows to render at a time.</p>\n"},"scrollRedrawThreshold":{"!doc":"<p>pixel distance a user must scroll vertically to trigger grid scrolling.</p>\n"},"searchContainerNode":{},"selectable":{"!doc":"<p>Set to true if you want to be able to select the text within the grid.</p>\n"},"selectionMode":{"!doc":"<p>Set the selection mode of grid&#39;s Selection.  Value must be &#39;single&#39;, &#39;multiple&#39;,\nor &#39;extended&#39;.  Default is &#39;extended&#39;.</p>\n"},"singleClickEdit":{"!doc":"<p>Single-click starts editing. Default is double-click</p>\n"},"sortInfo":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"structure":{"!doc":"<p>View layout defintion.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"summary":{"!doc":"<p>Customizable summary descriptions which will be added to grid.domNode</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"updating":{},"addRow":{"!type":"fn()","!doc":"<p>Add a row to the grid.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"beginUpdate":{"!type":"fn()","!doc":"<p>Use to make multiple changes to rows while queueing row updating.</p>\n"},"buildRendering":{"!type":"fn()"},"buildViews":{"!type":"fn()"},"canSort":{"!type":"fn(inSortInfo: ?) -> bool","!doc":"<p>Determines if the grid can be sorted</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createLayout":{"!type":"fn()","!doc":"<p>Creates a new Grid layout</p>\n"},"createManagers":{"!type":"fn()","!doc":"<p>create grid managers for various tasks including rows, focus, selection, editing</p>\n"},"createScroller":{"!type":"fn()","!doc":"<p>Creates a new virtual scroller</p>\n"},"createSelection":{"!type":"fn()"},"createView":{"!type":"fn(inClass: ?, idx: ?) -> +instance"},"createViews":{"!type":"fn()"},"defaultUpdate":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchContentEvent":{"!type":"fn(e: ?)"},"dispatchHeaderEvent":{"!type":"fn(e: ?)"},"dispatchKeyEvent":{"!type":"fn(e: ?)"},"doApplyCellEdit":{"!type":"fn(inValue: ?, inRowIndex: ?, inFieldIndex: ?)"},"doApplyEdit":{"!type":"fn(inRowIndex: ?)"},"doCancelEdit":{"!type":"fn(inRowIndex: ?)"},"doclick":{"!type":"fn(e: ?)"},"docontextmenu":{"!type":"fn(e: ?)"},"dodblclick":{"!type":"fn(e: ?)"},"doheaderclick":{"!type":"fn(e: ?)"},"doheadercontextmenu":{"!type":"fn(e: ?)"},"doheaderdblclick":{"!type":"fn(e: ?)"},"dokeydown":{"!type":"fn(e: ?)"},"doKeyEvent":{"!type":"fn(e: ?)"},"doStartEdit":{"!type":"fn(inCell: ?, inRowIndex: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"endUpdate":{"!type":"fn()","!doc":"<p>Use after calling beginUpdate to render any changes made to rows.</p>\n"},"finishScrollJob":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getCell":{"!type":"fn(inIndex: ?) -> +any","!doc":"<p>Retrieves the cell object for a given grid column.</p>\n"},"getCellName":{"!type":"fn(inCell: ?) -> string","!doc":"<p>Returns the cell name of a passed cell</p>\n"},"getCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?) -> ?"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnTogglingItems":{"!type":"fn() -> ?","!doc":"<p>returns an array of dijit.CheckedMenuItem widgets that can be\nadded to a menu for toggling columns on and off.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getItem":{"!type":"fn(inRowIndex: ?) -> +null"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowNode":{"!type":"fn(inRowIndex: ?) -> ?","!doc":"<p>find the rowNode that is not a rowSelector</p>\n"},"getSortAsc":{"!type":"fn(inSortInfo: ?) -> ?","!doc":"<p>Returns true if grid is sorted in an ascending direction.</p>\n"},"getSortIndex":{"!type":"fn(inSortInfo: ?) -> number","!doc":"<p>Returns the index of the column on which the grid is sorted</p>\n"},"hasLayout":{"!type":"fn() -> ?"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"postrender":{"!type":"fn()"},"postresize":{"!type":"fn()"},"prerender":{"!type":"fn()"},"removeSelectedRows":{"!type":"fn()","!doc":"<p>Remove the selected rows from the grid.</p>\n"},"render":{"!type":"fn()","!doc":"<p>Render the grid, headers, and views. Edit and scrolling states are reset. To retain edit and\nscrolling states, see Update.</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>Update the grid&#39;s rendering dimensions and resize it</p>\n"},"rowHeightChanged":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update grid when the height of a row has changed. Row height is handled automatically as rows\nare rendered. Use this function only to update a row&#39;s height outside the normal rendering process.</p>\n"},"scrollTo":{"!type":"fn(inTop: ?)","!doc":"<p>Vertically scroll the grid to a given pixel position</p>\n"},"scrollToRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Scroll the grid to a specific row.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?, textDir: ?)"},"setCellWidth":{"!type":"fn(inIndex: ?, inUnitWidth: ?)"},"setHeaderMenu":{"!type":"fn(menu: ?)"},"setScrollTop":{"!type":"fn(inTop: ?)"},"setSortIndex":{"!type":"fn(inIndex: ?, inAsc: ?)","!doc":"<p>Sort the grid on a column in a specified direction</p>\n"},"setSortInfo":{"!type":"fn(inSortInfo: ?)"},"setStructure":{"!type":"fn(inStructure: ?)","!doc":"<p>Install a new structure and rebuild the grid.</p>\n"},"setupHeaderMenu":{"!type":"fn()"},"showMessage":{"!type":"fn(message: ?)"},"sizeChange":{"!type":"fn()"},"sort":{"!type":"fn()"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"textSizeChanged":{"!type":"fn()"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn()","!doc":"<p>Update the grid, retaining edit and scrolling states.</p>\n"},"updateRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Render a single row.</p>\n"},"updateRowCount":{"!type":"fn(inRowCount: ?)","!doc":"<p>Change the number of rows.</p>\n"},"updateRows":{"!type":"fn(startIndex: ?, howMany: ?)","!doc":"<p>Render consecutive rows at once.</p>\n"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update the styles for a row after it&#39;s state has changed.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"markupFactory":{"!type":"fn(props: ?, node: ?, ctor: ?, cellFunc: ?)"}},"dojox/grid/_Events":{"!type":"fn()","prototype":{"!doc":"<p>_Grid mixin that provides default implementations for grid events.</p>\n","cellOverClass":{"!doc":"<p>css class to apply to grid cells over which the cursor is placed.</p>\n"}}},"dojox/grid/_Scroller":{"!type":"fn()","prototype":{"averageRowHeight":{},"contentNode":{},"defaultPageHeight":{},"defaultRowHeight":{},"dummy":{},"firstVisibleRow":{},"keepPages":{},"keepRows":{},"lastVisibleRow":{},"pacifying":{},"pacifyTicks":{},"page":{},"pageCount":{},"pageTop":{},"rowCount":{},"scrollboxNode":{},"windowHeight":{},"buildPage":{"!type":"fn(inPageIndex: ?, inReuseNode: ?, inPos: ?)"},"calcLastPageHeight":{"!type":"fn() -> number"},"constructor":{"!type":"fn(inContentNodes: ?)"},"createPageNode":{"!type":"fn() -> ?"},"destroy":{"!type":"fn()"},"destroyPage":{"!type":"fn(inPageIndex: ?)"},"doPacify":{"!type":"fn() -> bool"},"endPacify":{"!type":"fn()"},"findPage":{"!type":"fn(inTop: ?)"},"findScrollTop":{"!type":"fn(inRow: ?) -> number"},"findTopRow":{"!type":"fn(inScrollTop: ?) -> number"},"getDefaultNodes":{"!type":"fn() -> ?"},"getDefaultPageNode":{"!type":"fn(inPageIndex: ?) -> ?"},"getFirstVisibleRow":{"!type":"fn(inPage: ?, inPageTop: ?, inScrollTop: ?) -> number"},"getLastPageRow":{"!type":"fn(inPage: ?) -> number"},"getLastVisibleRow":{"!type":"fn(inPage: ?, inBottom: ?, inScrollBottom: ?) -> number"},"getPageHeight":{"!type":"fn(inPageIndex: ?) -> ?"},"getPageNodePosition":{"!type":"fn(inNode: ?) -> ?"},"getPageRow":{"!type":"fn(inPage: ?) -> number"},"getScrollBottom":{"!type":"fn(inTop: ?) -> number"},"init":{"!type":"fn(inRowCount: ?, inKeepRows: ?, inRowsPerPage: ?)"},"installPage":{"!type":"fn(inPageIndex: ?)"},"invalidate":{"!type":"fn()"},"invalidateNodes":{"!type":"fn()"},"invalidatePageNode":{"!type":"fn(inPageIndex: ?, inNodes: ?) -> ?"},"measurePage":{"!type":"fn(inPageIndex: ?) -> number"},"needPage":{"!type":"fn(inPageIndex: ?, inPos: ?) -> ?"},"onscroll":{"!type":"fn()"},"pacify":{"!type":"fn(inShouldPacify: ?)"},"pageExists":{"!type":"fn(inPageIndex: ?) -> ?"},"popPage":{"!type":"fn() -> ?"},"positionPage":{"!type":"fn(inPageIndex: ?, inPos: ?)"},"positionPageNode":{"!type":"fn(inNode: ?, inPos: ?)"},"preparePage":{"!type":"fn(inPageIndex: ?, inReuseNode: ?)"},"processEvent":{"!type":"fn(e: ?) -> ?"},"processNodeEvent":{"!type":"fn(e: ?, inNode: ?) -> bool"},"pushPage":{"!type":"fn(inPageIndex: ?) -> ?"},"removePage":{"!type":"fn(inPageIndex: ?)"},"removeRow":{"!type":"fn(inRowIndex: ?)"},"renderPage":{"!type":"fn(inPageIndex: ?)"},"renderRow":{"!type":"fn(inRowIndex: ?, inPageNode: ?)"},"repositionPages":{"!type":"fn(inPageIndex: ?)"},"resize":{"!type":"fn()"},"rowHeightChanged":{"!type":"fn(inRowIndex: ?, fromAsynRendering: ?)"},"scroll":{"!type":"fn(inTop: ?)"},"setContentNodes":{"!type":"fn(inNodes: ?)"},"setKeepInfo":{"!type":"fn(inKeepRows: ?)"},"setPacifying":{"!type":"fn(inPacifying: ?)"},"startPacify":{"!type":"fn()"},"updateContentHeight":{"!type":"fn(inDh: ?)"},"updatePageHeight":{"!type":"fn(inPageIndex: ?, fromBuild: ?, fromAsynRendering: ?) -> ?"},"updateRowCount":{"!type":"fn(inRowCount: ?)"}}},"dojox/grid/_Layout":{"!type":"fn()","prototype":{"!doc":"<p>Controls grid cell layout. Owned by grid and used internally.</p>\n","cells":{},"defaultWidth":{},"structure":{},"addCellDef":{"!type":"fn(inRowIndex: ?, inCellIndex: ?, inDef: ?) -> ?"},"addRowDef":{"!type":"fn(inRowIndex: ?, inDef: ?) -> []"},"addRowsDef":{"!type":"fn(inDef: ?) -> []"},"addViewDef":{"!type":"fn(inDef: ?) -> ?"},"constructor":{"!type":"fn(inGrid: ?)"},"moveColumn":{"!type":"fn(sourceViewIndex: ?, destViewIndex: ?, cellIndex: ?, targetIndex: ?, before: ?)"},"setColumnVisibility":{"!type":"fn(columnIndex: ?, visible: ?) -> bool"},"setStructure":{"!type":"fn(inStructure: ?)"}}},"dojox/grid/cells":{},"dojox/grid/cells/_base":{"!type":"fn()","prototype":{"!doc":"<p>Represents a grid cell and contains information about column options and methods\nfor retrieving cell related information.\nEach column in a grid layout has a cell object and most events and many methods\nprovide access to these objects.</p>\n","_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"hidden":{},"noresize":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn(inProps: ?)"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)","!doc":"<p>focus the grid editor</p>\n"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> +any","!doc":"<p>formats the cell for editing</p>\n"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)","!doc":"<p>format the editing dom node. Use when editor is a widget.</p>\n"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>returns value entered into editor</p>\n"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)","!doc":"<p>set the value of the grid editor</p>\n"}},"AlwaysEdit":{"!type":"fn()"},"Bool":{"!type":"fn()"},"Cell":{"!type":"fn()"},"markupFactory":{"!type":"fn(node: ?, cellDef: ?)"},"RowIndex":{"!type":"fn()"},"Select":{"!type":"fn()"}},"dojox/grid/cells/_base.Cell":{"!type":"fn()","prototype":{"!doc":"<p>grid cell that provides a standard text input box upon editing</p>\n","_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"hidden":{},"KEY_HANDLER":{},"keyFilter":{"!doc":"<p>optional regex for disallowing keypresses</p>\n"},"LRE":{},"noresize":{},"PDF":{},"RLE":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn()"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"doKey":{"!type":"fn(e: ?)"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)","!doc":"<p>focus the grid editor</p>\n"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> string"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>returns value entered into editor</p>\n"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)","!doc":"<p>set the value of the grid editor</p>\n"}},"markupFactory":{"!type":"fn(node: ?, cellDef: ?)"}},"dojox/grid/cells/_base.RowIndex":{"!type":"fn()","prototype":{"_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"hidden":{},"keyFilter":{"!doc":"<p>optional regex for disallowing keypresses</p>\n"},"name":{},"noresize":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn()"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"doKey":{"!type":"fn(e: ?)"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)","!doc":"<p>focus the grid editor</p>\n"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> string"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"get":{"!type":"fn(inRowIndex: ?) -> number"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>returns value entered into editor</p>\n"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"postscript":{"!type":"fn()"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)","!doc":"<p>set the value of the grid editor</p>\n"}},"markupFactory":{"!type":"fn(node: ?, cellDef: ?)"}},"dojox/grid/cells/_base.Select":{"!type":"fn()","prototype":{"!doc":"<p>grid cell that provides a standard select for editing</p>\n","_formatPending":{},"_getValueCallOrig":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"hidden":{},"keyFilter":{"!doc":"<p>optional regex for disallowing keypresses</p>\n"},"noresize":{},"options":{"!doc":"<p>text of each item</p>\n"},"returnIndex":{"!doc":"<p>editor returns only the index of the selected option and not the value</p>\n"},"styles":{},"value":{},"values":{"!doc":"<p>value for each item</p>\n"},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn(inCell: ?)"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"doKey":{"!type":"fn(e: ?)"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)","!doc":"<p>focus the grid editor</p>\n"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> ?"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn(inRowIndex: ?) -> ?"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)","!doc":"<p>set the value of the grid editor</p>\n"}},"markupFactory":{"!type":"fn(node: ?, cell: ?)"}},"dojox/grid/cells/_base.AlwaysEdit":{"!type":"fn()","prototype":{"!doc":"<p>grid cell that is always in an editable state, regardless of grid editing state</p>\n","_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"hidden":{},"keyFilter":{"!doc":"<p>optional regex for disallowing keypresses</p>\n"},"noresize":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"applyStaticValue":{"!type":"fn(inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn()"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"doKey":{"!type":"fn(e: ?)"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)","!doc":"<p>focus the grid editor</p>\n"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> string"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>returns value entered into editor</p>\n"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)","!doc":"<p>set the value of the grid editor</p>\n"}},"markupFactory":{"!type":"fn(node: ?, cell: ?)"}},"dojox/grid/cells/_base.Bool":{"!type":"fn()","prototype":{"!doc":"<p>grid cell that provides a standard checkbox that is always on for editing</p>\n","_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"hidden":{},"keyFilter":{"!doc":"<p>optional regex for disallowing keypresses</p>\n"},"noresize":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"applyStaticValue":{"!type":"fn(inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn()"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"doclick":{"!type":"fn(e: ?)"},"doKey":{"!type":"fn(e: ?)"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)","!doc":"<p>focus the grid editor</p>\n"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> string"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>returns value entered into editor</p>\n"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)","!doc":"<p>set the value of the grid editor</p>\n"}},"markupFactory":{"!type":"fn(node: ?, cell: ?)"}},"dojox/grid/_RowSelector":{"!type":"fn()","prototype":{"!proto":"dojox/grid/_View.prototype","!doc":"<p>Custom grid view. If used in a grid structure, provides a small selectable region for grid rows.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_nativeScroll":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_togglingColumn":{"!doc":"<p>Width of the column being toggled (-1 for none)</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"classTag":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultWidth":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"firstScroll":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lastTop":{},"marginBottom":{},"noscroll":{},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"padBorderWidth":{},"rowPad":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"viewWidth":{"!doc":"<p>Width for the view, in valid css unit</p>\n"},"adaptHeight":{"!type":"fn(minusScroll: ?)"},"adaptWidth":{"!type":"fn()"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"buildRow":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"buildRowContent":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"convertColPctToFixed":{"!type":"fn() -> bool"},"createRowNode":{"!type":"fn(inRowIndex: ?) -> ?"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchContentEvent":{"!type":"fn(e: ?) -> ?"},"dispatchHeaderEvent":{"!type":"fn(e: ?) -> ?"},"doContentEvent":{"!type":"fn(e: ?)"},"doHeaderEvent":{"!type":"fn(e: ?)"},"domouseout":{"!type":"fn(e: ?)"},"domouseover":{"!type":"fn(e: ?)"},"doscroll":{"!type":"fn(inEvent: ?)"},"doStyleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getCellNode":{"!type":"fn(inRowIndex: ?, inCellIndex: ?) -> ?"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnsWidth":{"!type":"fn() -> number"},"getContentWidth":{"!type":"fn() -> string"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getHeaderCellNode":{"!type":"fn(inCellIndex: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowNode":{"!type":"fn(inRowIndex: ?) -> ?"},"getScrollbarWidth":{"!type":"fn() -> ?"},"getWidth":{"!type":"fn() -> ?"},"hasHScrollbar":{"!type":"fn(reset: ?) -> bool"},"hasVScrollbar":{"!type":"fn(reset: ?) -> bool"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"render":{"!type":"fn()"},"renderHeader":{"!type":"fn()"},"renderRow":{"!type":"fn(inRowIndex: ?) -> ?"},"resize":{"!type":"fn()"},"rowRemoved":{"!type":"fn(inRowIndex: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setColumnsWidth":{"!type":"fn(width: ?)"},"setColWidth":{"!type":"fn(inIndex: ?, inWidth: ?)"},"setScrollTop":{"!type":"fn(inTop: ?) -> ?"},"setSize":{"!type":"fn(w: ?, h: ?)"},"setStructure":{"!type":"fn(inStructure: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleRow":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"styleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"testFlexCells":{"!type":"fn() -> ?"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn()"},"updateRow":{"!type":"fn()"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)"},"updateStructure":{"!type":"fn()"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/grid/_View":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A collection of grid columns. A grid is comprised of a set of views that stack horizontally.\nGrid creates views automatically based on grid&#39;s layout structure.\nUsers should typically not need to access individual views directly.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_nativeScroll":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_togglingColumn":{"!doc":"<p>Width of the column being toggled (-1 for none)</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"classTag":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultWidth":{"!doc":"<p>Default width of the view</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"firstScroll":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lastTop":{},"marginBottom":{},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"rowPad":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"viewWidth":{"!doc":"<p>Width for the view, in valid css unit</p>\n"},"adaptHeight":{"!type":"fn(minusScroll: ?)"},"adaptWidth":{"!type":"fn()"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"buildRow":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"buildRowContent":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"convertColPctToFixed":{"!type":"fn() -> bool"},"createRowNode":{"!type":"fn(inRowIndex: ?) -> ?"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchContentEvent":{"!type":"fn(e: ?) -> ?"},"dispatchHeaderEvent":{"!type":"fn(e: ?) -> ?"},"doContentEvent":{"!type":"fn(e: ?)"},"doHeaderEvent":{"!type":"fn(e: ?)"},"doscroll":{"!type":"fn(inEvent: ?)"},"doStyleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getCellNode":{"!type":"fn(inRowIndex: ?, inCellIndex: ?) -> ?"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnsWidth":{"!type":"fn() -> number"},"getContentWidth":{"!type":"fn() -> string"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getHeaderCellNode":{"!type":"fn(inCellIndex: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowNode":{"!type":"fn(inRowIndex: ?) -> ?"},"getScrollbarWidth":{"!type":"fn() -> ?"},"getWidth":{"!type":"fn() -> string"},"hasHScrollbar":{"!type":"fn(reset: ?) -> bool"},"hasVScrollbar":{"!type":"fn(reset: ?) -> bool"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"render":{"!type":"fn()"},"renderHeader":{"!type":"fn()"},"renderRow":{"!type":"fn(inRowIndex: ?) -> ?"},"resize":{"!type":"fn()"},"rowRemoved":{"!type":"fn(inRowIndex: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setColumnsWidth":{"!type":"fn(width: ?)"},"setColWidth":{"!type":"fn(inIndex: ?, inWidth: ?)"},"setScrollTop":{"!type":"fn(inTop: ?) -> ?"},"setSize":{"!type":"fn(w: ?, h: ?)"},"setStructure":{"!type":"fn(inStructure: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleRow":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"styleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"testFlexCells":{"!type":"fn() -> ?"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn()"},"updateRow":{"!type":"fn(inRowIndex: ?) -> ?"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)"},"updateStructure":{"!type":"fn()"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/grid/_ViewManager":{"!type":"fn()","prototype":{"!doc":"<p>A collection of grid views. Owned by grid and used internally for managing grid views.</p>\n","defaultWidth":{},"views":{},"addView":{"!type":"fn(inView: ?)"},"arrange":{"!type":"fn(l: ?, w: ?) -> number"},"constructor":{"!type":"fn(inGrid: ?)"},"destroyViews":{"!type":"fn()"},"findClient":{"!type":"fn(inAutoWidth: ?) -> ?"},"forEach":{"!type":"fn(inCallback: ?)"},"getContentNodes":{"!type":"fn() -> []"},"getFirstScrollingView":{"!type":"fn() -> ?","!doc":"<p>Returns the first grid view with a scroll bar</p>\n"},"getViewWidth":{"!type":"fn(inIndex: ?) -> ?"},"measureContent":{"!type":"fn() -> number"},"measureHeader":{"!type":"fn() -> number"},"normalizeHeaderNodeHeight":{"!type":"fn()"},"normalizeRowNodeHeights":{"!type":"fn(inRowNodes: ?)"},"render":{"!type":"fn()"},"renderRow":{"!type":"fn(inRowIndex: ?, inNodes: ?, skipRenorm: ?)"},"renormalizeRow":{"!type":"fn(inRowIndex: ?)"},"resetHeaderNodeHeight":{"!type":"fn()"},"resize":{"!type":"fn()"},"rowRemoved":{"!type":"fn(inRowIndex: ?)"},"setScrollTop":{"!type":"fn(inTop: ?) -> ?"},"updateRow":{"!type":"fn(inRowIndex: ?, skipRenorm: ?)"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)"}}},"dojox/grid/_RowManager":{"!type":"fn()","prototype":{"linesToEms":{},"overRow":{},"applyStyles":{"!type":"fn(inRow: ?)"},"constructor":{"!type":"fn(inGrid: ?)"},"isOver":{"!type":"fn(inRowIndex: ?) -> bool"},"prepareStylingRow":{"!type":"fn(inRowIndex: ?, inRowNode: ?) -> ?"},"setOverRow":{"!type":"fn(inRowIndex: ?)"},"styleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"updateStyles":{"!type":"fn(inRowIndex: ?)"}}},"dojox/grid/_FocusManager":{"!type":"fn()","prototype":{"!doc":"<p>Controls grid cell focus. Owned by grid and used internally for focusing.\nNote: grid cell actually receives keyboard input only when cell is being edited.</p>\n","_colHeadFocusIdx":{},"_colHeadNode":{},"_contextMenuBindNode":{},"focusClass":{},"focusView":{},"tabbingOut":{},"blurHeader":{"!type":"fn()"},"colSizeAdjust":{"!type":"fn(e: ?, colIdx: ?, delta: ?)"},"constructor":{"!type":"fn(inGrid: ?)"},"destroy":{"!type":"fn()"},"doBlur":{"!type":"fn(e: ?)"},"doColHeaderBlur":{"!type":"fn(e: ?)"},"doColHeaderFocus":{"!type":"fn(e: ?)"},"doContextMenu":{"!type":"fn(e: ?)"},"doFocus":{"!type":"fn(e: ?)"},"doLastNodeBlur":{"!type":"fn(e: ?)"},"doLastNodeFocus":{"!type":"fn(e: ?)"},"findAndFocusGridCell":{"!type":"fn() -> bool","!doc":"<p>find the first focusable grid cell</p>\n"},"focusGrid":{"!type":"fn(inSkipFocusCell: ?)"},"focusGridView":{"!type":"fn()"},"focusHeader":{"!type":"fn()"},"getHeaderIndex":{"!type":"fn() -> +any","!doc":"<p>if one of the column headers currently has focus, return its index.</p>\n"},"initFocusView":{"!type":"fn()"},"isFirstFocusCell":{"!type":"fn() -> bool"},"isFocusCell":{"!type":"fn(inCell: ?, inRowIndex: ?) -> +any","!doc":"<p>states if the given cell is focused</p>\n"},"isLastFocusCell":{"!type":"fn() -> bool"},"isNavHeader":{"!type":"fn() -> +any","!doc":"<p>states whether currently navigating among column headers.</p>\n"},"isNoFocusCell":{"!type":"fn() -> bool"},"move":{"!type":"fn(inRowDelta: ?, inColDelta: ?)","!doc":"<p>focus grid cell or  simulate focus to column header based on position relative to current focus</p>\n"},"next":{"!type":"fn()","!doc":"<p>focus next grid cell</p>\n"},"nextKey":{"!type":"fn(e: ?)"},"previous":{"!type":"fn()","!doc":"<p>focus previous grid cell</p>\n"},"previousKey":{"!type":"fn(e: ?)"},"scrollIntoView":{"!type":"fn() -> +null"},"setFocusCell":{"!type":"fn(inCell: ?, inRowIndex: ?)","!doc":"<p>focuses the given grid cell</p>\n"},"setFocusIndex":{"!type":"fn(inRowIndex: ?, inCellIndex: ?)","!doc":"<p>focuses the given grid cell</p>\n"},"styleRow":{"!type":"fn(inRow: ?)"},"tabOut":{"!type":"fn(inFocusNode: ?)"}}},"dojox/grid/_EditManager":{"!type":"fn()","prototype":{"!doc":"<p>Controls grid cell editing process. Owned by grid and used internally for editing.</p>\n","_boomerangWindow":{},"info":{},"apply":{"!type":"fn()","!doc":"<p>Apply a grid edit</p>\n"},"applyCellEdit":{"!type":"fn(inValue: ?, inCell: ?, inRowIndex: ?)"},"applyRowEdit":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancel a grid edit</p>\n"},"cellFocus":{"!type":"fn(inCell: ?, inRowIndex: ?)","!doc":"<p>Invoke editing when cell is focused</p>\n"},"constructor":{"!type":"fn(inGrid: ?)"},"destroy":{"!type":"fn()"},"dispatchEvent":{"!type":"fn(e: ?) -> number"},"editorApply":{"!type":"fn()"},"editorCancel":{"!type":"fn()"},"focusEditor":{"!type":"fn()"},"isEditCell":{"!type":"fn(inRowIndex: ?, inCellIndex: ?) -> bool","!doc":"<p>Indicates if the given cell is being edited.</p>\n"},"isEditing":{"!type":"fn() -> bool","!doc":"<p>Indicates editing state of the grid.</p>\n"},"isEditRow":{"!type":"fn(inRowIndex: ?) -> bool","!doc":"<p>Indicates if the given row is being edited.</p>\n"},"restore":{"!type":"fn(inView: ?, inRowIndex: ?)","!doc":"<p>Restores the grid editing state</p>\n"},"rowClick":{"!type":"fn(e: ?)"},"save":{"!type":"fn(inRowIndex: ?, inView: ?)","!doc":"<p>Save the grid editing state</p>\n"},"setEditCell":{"!type":"fn(inCell: ?, inRowIndex: ?)","!doc":"<p>Set the given cell to be edited</p>\n"},"start":{"!type":"fn(inCell: ?, inRowIndex: ?, inEditing: ?)"},"styleRow":{"!type":"fn(inRow: ?)"}}},"dojox/grid/Selection":{"!type":"fn()","prototype":{"!doc":"<p>Manages row selection for grid. Owned by grid and used internally\nfor selection. Override to implement custom selection.</p>\n","mode":{},"rangeStartIndex":{},"selected":{},"selectedIndex":{},"updating":{},"addToSelection":{"!type":"fn(inIndex: ?)"},"clear":{"!type":"fn()"},"clickSelect":{"!type":"fn(inIndex: ?, inCtrlKey: ?, inShiftKey: ?)"},"clickSelectEvent":{"!type":"fn(e: ?)"},"constructor":{"!type":"fn(inGrid: ?)"},"deselect":{"!type":"fn(inIndex: ?)"},"deselectAll":{"!type":"fn(inExcept: ?)"},"deselectRange":{"!type":"fn(inFrom: ?, inTo: ?)"},"getFirstSelected":{"!type":"fn() -> number"},"getNextSelected":{"!type":"fn(inPrev: ?) -> number"},"getSelected":{"!type":"fn() -> []"},"getSelectedCount":{"!type":"fn() -> number"},"insert":{"!type":"fn(inIndex: ?)"},"isSelected":{"!type":"fn(inIndex: ?) -> bool"},"remove":{"!type":"fn(inIndex: ?)"},"select":{"!type":"fn(inIndex: ?)"},"selectRange":{"!type":"fn(inFrom: ?, inTo: ?)"},"setMode":{"!type":"fn(mode: ?)"},"setSelected":{"!type":"fn(inIndex: ?, inSelect: ?)"},"toggleSelect":{"!type":"fn(inIndex: ?)"}}},"dojox/grid/cells/dijit":{"CheckBox":{"!type":"fn()"},"ComboBox":{"!type":"fn()"},"DateTextBox":{"!type":"fn()"},"Editor":{"!type":"fn()"}},"dojox/grid/cells/dijit._Widget":{"!type":"fn()","prototype":{"!proto":"dojox/grid/cells/_base.prototype","_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"getWidgetPropsCallOrig":{},"hidden":{},"noresize":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"attachWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn(inCell: ?)"},"createWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?) -> +instance"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> string"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?) -> ?"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn(inRowIndex: ?) -> ?"},"getWidgetProps":{"!type":"fn(inDatum: ?) -> ?"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)"},"sizeWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"widgetClass":{"!type":"fn()"}},"markupFactory":{"!type":"fn(node: ?, cell: ?)"}},"dojox/grid/cells/dijit.ComboBox":{"!type":"fn()","prototype":{"!proto":"dojox/grid/cells/_base.prototype","_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"getWidgetPropsCallOrig":{},"hidden":{},"noresize":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"attachWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn(inCell: ?)"},"createWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?) -> +instance"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> string"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?) -> ?"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn() -> ?"},"getWidgetProps":{"!type":"fn(inDatum: ?) -> ?"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)"},"sizeWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"widgetClass":{"!type":"fn()"}},"markupFactory":{"!type":"fn(node: ?, cell: ?)"}},"dojox/grid/cells/dijit.DateTextBox":{"!type":"fn()","prototype":{"!proto":"dojox/grid/cells/_base.prototype","_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"hidden":{},"noresize":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"attachWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn(inCell: ?)"},"createWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?) -> +instance"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> string"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?) -> ?"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn(inRowIndex: ?) -> ?"},"getWidgetProps":{"!type":"fn(inDatum: ?) -> ?"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)"},"sizeWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"widgetClass":{"!type":"fn()"}},"markupFactory":{"!type":"fn(node: ?, cell: ?)"}},"dojox/grid/cells/dijit.CheckBox":{"!type":"fn()","prototype":{"!proto":"dojox/grid/cells/_base.prototype","_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"hidden":{},"noresize":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"attachWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn(inCell: ?)"},"createWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?) -> +instance"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> string"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?) -> ?"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn() -> ?"},"getWidgetProps":{"!type":"fn(inDatum: ?) -> ?"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)"},"sizeWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"widgetClass":{"!type":"fn()"}},"markupFactory":{"!type":"fn(node: ?, cell: ?)"}},"dojox/grid/cells/dijit.Editor":{"!type":"fn()","prototype":{"!proto":"dojox/grid/cells/_base.prototype","_formatPending":{},"_valueProp":{},"alwaysEditing":{},"classes":{},"defaultValue":{},"draggable":{},"editable":{},"formatter":{},"hidden":{},"noresize":{},"styles":{},"value":{},"apply":{"!type":"fn(inRowIndex: ?)","!doc":"<p>apply edit from cell editor</p>\n"},"applyEdit":{"!type":"fn(inValue: ?, inRowIndex: ?)"},"attachWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"cancel":{"!type":"fn(inRowIndex: ?)","!doc":"<p>cancel cell edit</p>\n"},"cancelEdit":{"!type":"fn(inRowIndex: ?)"},"cancelFormatNode":{"!type":"fn()"},"canResize":{"!type":"fn() -> ?"},"constructor":{"!type":"fn(inCell: ?)"},"createWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?) -> +instance"},"dispatchEvent":{"!type":"fn(m: ?, e: ?) -> ?"},"focus":{"!type":"fn(inRowIndex: ?, inNode: ?)"},"format":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>provides the html for a given grid cell.</p>\n"},"formatEditing":{"!type":"fn(inDatum: ?, inRowIndex: ?) -> string"},"formatNode":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"getEditNode":{"!type":"fn(inRowIndex: ?) -> number"},"getHeaderNode":{"!type":"fn() -> ?"},"getNode":{"!type":"fn(inRowIndex: ?) -> +any","!doc":"<p>gets the dom node for a given grid cell.</p>\n"},"getValue":{"!type":"fn(inRowIndex: ?) -> ?"},"getWidgetProps":{"!type":"fn(inDatum: ?) -> ?"},"isFlex":{"!type":"fn() -> ?"},"needFormatNode":{"!type":"fn(inDatum: ?, inRowIndex: ?)"},"populateEditor":{"!type":"fn()"},"registerOnBlur":{"!type":"fn(inNode: ?, inRowIndex: ?)"},"restore":{"!type":"fn(inRowIndex: ?)","!doc":"<p>restore editor state</p>\n"},"save":{"!type":"fn(inRowIndex: ?)","!doc":"<p>save editor state</p>\n"},"setValue":{"!type":"fn(inRowIndex: ?, inValue: ?)"},"sizeWidget":{"!type":"fn(inNode: ?, inDatum: ?, inRowIndex: ?)"},"widgetClass":{"!type":"fn()"}},"markupFactory":{"!type":"fn(node: ?, cell: ?)"}},"dojox/grid/DataGrid":{"!type":"fn()","prototype":{"!proto":"dojox/grid/_Grid.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_bop":{},"_by_idty":{},"_by_idx":{},"_cache":{},"_click":{},"_eop":{},"_focusManager":{},"_isLoaded":{},"_isLoading":{},"_pages":{},"_pending_requests":{},"_placeholders":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_requests":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setSummaryAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_store_connects":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoHeight":{"!doc":"<p>If autoHeight is true, grid height is automatically set to fit the data.\nIf it is an integer, the height will be automatically set to fit the data\nif there are fewer than that many rows - and the height will be set to show\nthat many rows if there are more</p>\n"},"autoRender":{"!doc":"<p>If autoRender is true, grid will render itself after initialization.</p>\n"},"autoWidth":{"!doc":"<p>If autoWidth is true, grid width is automatically set to fit the data.</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"cellOverClass":{"!doc":"<p>css class to apply to grid cells over which the cursor is placed.</p>\n"},"class":{},"classTag":{"!doc":"<p>CSS class applied to the grid&#39;s domNode</p>\n"},"columnReordering":{"!doc":"<p>If set to true, will add drag and drop reordering to views with one row of columns.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultHeight":{"!doc":"<p>default height of the grid, measured in any valid css unit.</p>\n"},"delayScroll":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>indicates if the grid contains editable cells, default is false\nset to true if editable cell encountered during rendering</p>\n"},"elasticView":{"!doc":"<p>Override defaults and make the indexed grid view elastic, thus filling available horizontal space.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows when the grid encounters an error loading</p>\n"},"escapeHTMLInData":{"!doc":"<p>This will escape HTML brackets from the data to prevent HTML from\nuser-inputted data being rendered with may contain JavaScript and result in\nXSS attacks. This is true by default, and it is recommended that it remain\ntrue. Setting this to false will allow data to be displayed in the grid without\nfiltering, and should be only used if it is known that the data won&#39;t contain\nmalicious scripts. If HTML is needed in grid cells, it is recommended that\nyou use the formatter function to generate the HTML (the output of\nformatter functions is not filtered, even with escapeHTMLInData set to true).</p>\n"},"fastScroll":{"!doc":"<p>flag modifies vertical scrolling behavior. Defaults to true but set to false for slower\nscroll performance but more immediate scrolling feedback</p>\n"},"fetchText":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatterScope":{"!doc":"<p>An object to execute format functions within.  If not set, the\nformat functions will execute within the scope of the cell that\nhas a format function.</p>\n"},"headerMenu":{"!doc":"<p>If set to a dijit.Menu, will use this as a context menu for the grid headers.</p>\n"},"height":{"!doc":"<p>explicit height of the grid, measured in any valid css unit.  This will be populated (and overridden)\nif the height: css attribute exists on the source node.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"initialWidth":{"!doc":"<p>A css string to use to set our initial width (only used if autoWidth\nis true).  The first rendering of the grid will be this width, any\nresizing of columns, etc will result in the grid switching to\nautoWidth mode.  Note, this width will override any styling in a\nstylesheet or directly on the node.</p>\n"},"invalidated":{},"items":{},"keepRows":{"!doc":"<p>Number of rows to keep in the rendering cache.</p>\n"},"keepSelection":{"!doc":"<p>Whether keep selection after sort, filter etc.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while the grid is loading</p>\n"},"noDataMessage":{"!doc":"<p>Message that shows if the grid has no data - wrap it in a\nspan with class &#39;dojoxGridNoData&#39; if you want it to be\nstyled similar to the loading and error messages</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeholderLabel":{"!doc":"<p>Label of placeholders to search for in the header menu to replace with column toggling\nmenu items.</p>\n"},"query":{},"queryOptions":{},"rowCount":{},"rowHeight":{"!doc":"<p>If rowHeight is set to a positive number, it will define the height of the rows\nin pixels. This can provide a significant performance advantage, since it\neliminates the need to measure row sizes during rendering, which is one\nthe primary bottlenecks in the DataGrid&#39;s performance.</p>\n"},"rowSelector":{"!doc":"<p>If set to true, will add a row selector view to this grid.  If set to a CSS width, will add\na row selector of that width to this grid.</p>\n"},"rowsPerPage":{"!doc":"<p>Number of rows to render at a time.</p>\n"},"scrollRedrawThreshold":{"!doc":"<p>pixel distance a user must scroll vertically to trigger grid scrolling.</p>\n"},"searchContainerNode":{},"selectable":{"!doc":"<p>Set to true if you want to be able to select the text within the grid.</p>\n"},"selectionMode":{"!doc":"<p>Set the selection mode of grid&#39;s Selection.  Value must be &#39;single&#39;, &#39;multiple&#39;,\nor &#39;extended&#39;.  Default is &#39;extended&#39;.</p>\n"},"singleClickEdit":{"!doc":"<p>Single-click starts editing. Default is double-click</p>\n"},"sortFields":{},"sortInfo":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{},"structure":{"!doc":"<p>View layout definition.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"summary":{"!doc":"<p>Customizable summary descriptions which will be added to grid.domNode</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"updateDelay":{"!doc":"<p>Time, in milliseconds, to delay updates automatically so that multiple\ncalls to onSet/onNew/onDelete don&#39;t keep rerendering the grid.  Set\nto 0 to immediately cause updates.  A higher value will result in\nbetter performance at the expense of responsiveness of the grid.</p>\n"},"updating":{},"addRow":{"!type":"fn()","!doc":"<p>Add a row to the grid.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"beginUpdate":{"!type":"fn()","!doc":"<p>Use to make multiple changes to rows while queueing row updating.</p>\n"},"buildRendering":{"!type":"fn()"},"buildViews":{"!type":"fn()"},"canEdit":{"!type":"fn(inCell: ?, inRowIndex: ?) -> ?"},"canSort":{"!type":"fn() -> bool"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createLayout":{"!type":"fn()","!doc":"<p>Creates a new Grid layout</p>\n"},"createManagers":{"!type":"fn()","!doc":"<p>create grid managers for various tasks including rows, focus, selection, editing</p>\n"},"createScroller":{"!type":"fn()","!doc":"<p>Creates a new virtual scroller</p>\n"},"createSelection":{"!type":"fn()"},"createView":{"!type":"fn(inClass: ?, idx: ?) -> +instance"},"createViews":{"!type":"fn()"},"defaultUpdate":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchContentEvent":{"!type":"fn(e: ?)"},"dispatchHeaderEvent":{"!type":"fn(e: ?)"},"dispatchKeyEvent":{"!type":"fn(e: ?)"},"doApplyCellEdit":{"!type":"fn(inValue: ?, inRowIndex: ?, inAttrName: ?)"},"doApplyEdit":{"!type":"fn(inRowIndex: ?, inDataAttr: ?)"},"doCancelEdit":{"!type":"fn(inRowIndex: ?)"},"doclick":{"!type":"fn(e: ?)"},"docontextmenu":{"!type":"fn(e: ?)"},"dodblclick":{"!type":"fn(e: ?)"},"doheaderclick":{"!type":"fn(e: ?)"},"doheadercontextmenu":{"!type":"fn(e: ?)"},"doheaderdblclick":{"!type":"fn(e: ?)"},"dokeydown":{"!type":"fn(e: ?)"},"doKeyEvent":{"!type":"fn(e: ?)"},"doStartEdit":{"!type":"fn(inCell: ?, inRowIndex: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"endUpdate":{"!type":"fn()","!doc":"<p>Use after calling beginUpdate to render any changes made to rows.</p>\n"},"filter":{"!type":"fn(query: ?, reRender: ?)"},"finishScrollJob":{"!type":"fn()"},"get":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>Default data getter.</p>\n"},"getCell":{"!type":"fn(inIndex: ?) -> +any","!doc":"<p>Retrieves the cell object for a given grid column.</p>\n"},"getCellName":{"!type":"fn(inCell: ?) -> ?"},"getCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?) -> ?"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnTogglingItems":{"!type":"fn() -> ?","!doc":"<p>returns an array of dijit.CheckedMenuItem widgets that can be\nadded to a menu for toggling columns on and off.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getItem":{"!type":"fn(idx: ?) -> +null"},"getItemIndex":{"!type":"fn(item: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowNode":{"!type":"fn(inRowIndex: ?) -> ?","!doc":"<p>find the rowNode that is not a rowSelector</p>\n"},"getSortAsc":{"!type":"fn(inSortInfo: ?) -> ?","!doc":"<p>Returns true if grid is sorted in an ascending direction.</p>\n"},"getSortIndex":{"!type":"fn(inSortInfo: ?) -> number","!doc":"<p>Returns the index of the column on which the grid is sorted</p>\n"},"getSortProps":{"!type":"fn() -> ?"},"hasLayout":{"!type":"fn() -> ?"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"postrender":{"!type":"fn()"},"postresize":{"!type":"fn()"},"prerender":{"!type":"fn()"},"removeSelectedRows":{"!type":"fn()","!doc":"<p>Remove the selected rows from the grid.</p>\n"},"render":{"!type":"fn()","!doc":"<p>Render the grid, headers, and views. Edit and scrolling states are reset. To retain edit and\nscrolling states, see Update.</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>Update the grid&#39;s rendering dimensions and resize it</p>\n"},"rowHeightChanged":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update grid when the height of a row has changed. Row height is handled automatically as rows\nare rendered. Use this function only to update a row&#39;s height outside the normal rendering process.</p>\n"},"scrollTo":{"!type":"fn(inTop: ?)","!doc":"<p>Vertically scroll the grid to a given pixel position</p>\n"},"scrollToRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Scroll the grid to a specific row.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?, textDir: ?)"},"setCellWidth":{"!type":"fn(inIndex: ?, inUnitWidth: ?)"},"setHeaderMenu":{"!type":"fn(menu: ?)"},"setItems":{"!type":"fn(items: ?)"},"setQuery":{"!type":"fn(query: ?, queryOptions: ?)"},"setScrollTop":{"!type":"fn(inTop: ?)"},"setSortIndex":{"!type":"fn(inIndex: ?, inAsc: ?)","!doc":"<p>Sort the grid on a column in a specified direction</p>\n"},"setSortInfo":{"!type":"fn(inSortInfo: ?)"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions: ?)"},"setStructure":{"!type":"fn(inStructure: ?)","!doc":"<p>Install a new structure and rebuild the grid.</p>\n"},"setupHeaderMenu":{"!type":"fn()"},"showMessage":{"!type":"fn(message: ?)"},"sizeChange":{"!type":"fn()"},"sort":{"!type":"fn()"},"startup":{"!type":"fn()"},"styleRowState":{"!type":"fn(inRow: ?)","!doc":"<p>Perform row styling</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"textSizeChanged":{"!type":"fn()"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn()","!doc":"<p>Update the grid, retaining edit and scrolling states.</p>\n"},"updateRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Render a single row.</p>\n"},"updateRowCount":{"!type":"fn(inRowCount: ?)","!doc":"<p>Change the number of rows.</p>\n"},"updateRows":{"!type":"fn(startIndex: ?, howMany: ?)","!doc":"<p>Render consecutive rows at once.</p>\n"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update the styles for a row after it&#39;s state has changed.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"cell_markupFactory":{"!type":"fn(cellFunc: ?, node: ?, cellDef: ?)"},"markupFactory":{"!type":"fn(props: ?, node: ?, ctor: ?, cellFunc: ?)"}},"dojox/grid/DataSelection":{"!type":"fn()","prototype":{"!proto":"dojox/grid/Selection.prototype","mode":{},"rangeStartIndex":{},"selected":{},"selectedIndex":{},"updating":{},"addToSelection":{"!type":"fn(inItemOrIndex: ?)"},"clear":{"!type":"fn()"},"clickSelect":{"!type":"fn(inIndex: ?, inCtrlKey: ?, inShiftKey: ?)"},"clickSelectEvent":{"!type":"fn(e: ?)"},"constructor":{"!type":"fn(grid: ?)"},"deselect":{"!type":"fn(inItemOrIndex: ?)"},"deselectAll":{"!type":"fn(inItemOrIndex: ?)"},"deselectRange":{"!type":"fn(inFrom: ?, inTo: ?)"},"destroy":{"!type":"fn()"},"getFirstSelected":{"!type":"fn() -> +null"},"getNextSelected":{"!type":"fn(inPrev: ?) -> +null"},"getSelected":{"!type":"fn() -> []"},"getSelectedCount":{"!type":"fn() -> number"},"insert":{"!type":"fn(inIndex: ?)"},"isSelected":{"!type":"fn(inIndex: ?) -> bool"},"remove":{"!type":"fn(inIndex: ?)"},"select":{"!type":"fn(inIndex: ?)"},"selectRange":{"!type":"fn(inFrom: ?, inTo: ?)"},"setMode":{"!type":"fn(mode: ?)"},"setSelected":{"!type":"fn(inIndex: ?, inSelect: ?)"},"toggleSelect":{"!type":"fn(inIndex: ?)"}}},"dojox/grid/_SelectionPreserver":{"!type":"fn()","prototype":{"!doc":"<p>Preserve selections across various user actions.\n</p>\n","constructor":{"!type":"fn(selection: ?)"},"destroy":{"!type":"fn()"},"reset":{"!type":"fn()"}}},"dojox/grid/EnhancedGrid":{"!type":"fn()","prototype":{"!proto":"dojox/grid/DataGrid.prototype","!doc":"<p>Provides enhanced features based on DataGrid\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_bop":{},"_by_idty":{},"_by_idx":{},"_cache":{},"_click":{},"_eop":{},"_focusManager":{},"_isLoaded":{},"_isLoading":{},"_pages":{},"_pending_requests":{},"_placeholders":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_requests":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setSummaryAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_store_connects":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoHeight":{"!doc":"<p>If autoHeight is true, grid height is automatically set to fit the data.\nIf it is an integer, the height will be automatically set to fit the data\nif there are fewer than that many rows - and the height will be set to show\nthat many rows if there are more</p>\n"},"autoRender":{"!doc":"<p>If autoRender is true, grid will render itself after initialization.</p>\n"},"autoWidth":{"!doc":"<p>If autoWidth is true, grid width is automatically set to fit the data.</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"cellOverClass":{"!doc":"<p>css class to apply to grid cells over which the cursor is placed.</p>\n"},"class":{},"classTag":{"!doc":"<p>CSS class applied to the grid&#39;s domNode</p>\n"},"columnReordering":{"!doc":"<p>If set to true, will add drag and drop reordering to views with one row of columns.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultHeight":{"!doc":"<p>default height of the grid, measured in any valid css unit.</p>\n"},"delayScroll":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>indicates if the grid contains editable cells, default is false\nset to true if editable cell encountered during rendering</p>\n"},"elasticView":{"!doc":"<p>Override defaults and make the indexed grid view elastic, thus filling available horizontal space.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows when the grid encounters an error loading</p>\n"},"escapeHTMLInData":{"!doc":"<p>This will escape HTML brackets from the data to prevent HTML from\nuser-inputted data being rendered with may contain JavaScript and result in\nXSS attacks. This is true by default, and it is recommended that it remain\ntrue. Setting this to false will allow data to be displayed in the grid without\nfiltering, and should be only used if it is known that the data won&#39;t contain\nmalicious scripts. If HTML is needed in grid cells, it is recommended that\nyou use the formatter function to generate the HTML (the output of\nformatter functions is not filtered, even with escapeHTMLInData set to true).</p>\n"},"fastScroll":{"!doc":"<p>flag modifies vertical scrolling behavior. Defaults to true but set to false for slower\nscroll performance but more immediate scrolling feedback</p>\n"},"fetchText":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatterScope":{"!doc":"<p>An object to execute format functions within.  If not set, the\nformat functions will execute within the scope of the cell that\nhas a format function.</p>\n"},"headerMenu":{"!doc":"<p>If set to a dijit.Menu, will use this as a context menu for the grid headers.</p>\n"},"height":{"!doc":"<p>explicit height of the grid, measured in any valid css unit.  This will be populated (and overridden)\nif the height: css attribute exists on the source node.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"initialWidth":{"!doc":"<p>A css string to use to set our initial width (only used if autoWidth\nis true).  The first rendering of the grid will be this width, any\nresizing of columns, etc will result in the grid switching to\nautoWidth mode.  Note, this width will override any styling in a\nstylesheet or directly on the node.</p>\n"},"invalidated":{},"items":{},"keepRows":{"!doc":"<p>Number of rows to keep in the rendering cache.</p>\n"},"keepSelection":{"!doc":"<p>Whether keep selection after sort, filter etc.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while the grid is loading</p>\n"},"noDataMessage":{"!doc":"<p>Message that shows if the grid has no data - wrap it in a\nspan with class &#39;dojoxGridNoData&#39; if you want it to be\nstyled similar to the loading and error messages</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeholderLabel":{"!doc":"<p>Label of placeholders to search for in the header menu to replace with column toggling\nmenu items.</p>\n"},"pluginMgr":{"!doc":"<p>Singleton plugin manager</p>\n"},"plugins":{"!doc":"<p>Plugin properties, e.g. {nestedSorting: true, dnd: true, ...}</p>\n"},"query":{},"queryOptions":{},"rowCount":{},"rowHeight":{"!doc":"<p>If rowHeight is set to a positive number, it will define the height of the rows\nin pixels. This can provide a significant performance advantage, since it\neliminates the need to measure row sizes during rendering, which is one\nthe primary bottlenecks in the DataGrid&#39;s performance.</p>\n"},"rowSelector":{"!doc":"<p>If set to true, will add a row selector view to this grid.  If set to a CSS width, will add\na row selector of that width to this grid.</p>\n"},"rowsPerPage":{"!doc":"<p>Number of rows to render at a time.</p>\n"},"scrollRedrawThreshold":{"!doc":"<p>pixel distance a user must scroll vertically to trigger grid scrolling.</p>\n"},"searchContainerNode":{},"selectable":{"!doc":"<p>Set to true if you want to be able to select the text within the grid.</p>\n"},"selectionMode":{"!doc":"<p>Set the selection mode of grid&#39;s Selection.  Value must be &#39;single&#39;, &#39;multiple&#39;,\nor &#39;extended&#39;.  Default is &#39;extended&#39;.</p>\n"},"singleClickEdit":{"!doc":"<p>Single-click starts editing. Default is double-click</p>\n"},"sortFields":{},"sortInfo":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{},"structure":{"!doc":"<p>View layout definition.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"summary":{"!doc":"<p>Customizable summary descriptions which will be added to grid.domNode</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"updateDelay":{"!doc":"<p>Time, in milliseconds, to delay updates automatically so that multiple\ncalls to onSet/onNew/onDelete don&#39;t keep rerendering the grid.  Set\nto 0 to immediately cause updates.  A higher value will result in\nbetter performance at the expense of responsiveness of the grid.</p>\n"},"updating":{},"addRow":{"!type":"fn()","!doc":"<p>Add a row to the grid.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"beginUpdate":{"!type":"fn()","!doc":"<p>Use to make multiple changes to rows while queueing row updating.</p>\n"},"buildRendering":{"!type":"fn()"},"buildViews":{"!type":"fn()"},"canEdit":{"!type":"fn(inCell: ?, inRowIndex: ?) -> ?"},"canSort":{"!type":"fn(colIndex: ?, field: ?) -> bool","!doc":"<p>Overwritten</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createLayout":{"!type":"fn()","!doc":"<p>Creates a new Grid layout</p>\n"},"createManagers":{"!type":"fn()","!doc":"<p>create grid managers for various tasks including rows, focus, selection, editing</p>\n"},"createScroller":{"!type":"fn()","!doc":"<p>Creates a new virtual scroller</p>\n"},"createSelection":{"!type":"fn()"},"createView":{"!type":"fn() -> ?","!doc":"<p>Overwrite: rewrite getCellX of view.header</p>\n"},"createViews":{"!type":"fn()"},"defaultUpdate":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all resources</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchContentEvent":{"!type":"fn(e: ?)"},"dispatchHeaderEvent":{"!type":"fn(e: ?)"},"dispatchKeyEvent":{"!type":"fn(e: ?)"},"doApplyCellEdit":{"!type":"fn(inValue: ?, inRowIndex: ?, inAttrName: ?)","!doc":"<p>Overwritten, see DataGrid.doApplyCellEdit()</p>\n"},"doApplyEdit":{"!type":"fn(inRowIndex: ?, inDataAttr: ?)"},"doCancelEdit":{"!type":"fn(inRowIndex: ?)"},"doclick":{"!type":"fn(e: ?)"},"docontextmenu":{"!type":"fn(e: ?)"},"dodblclick":{"!type":"fn(e: ?)"},"doheaderclick":{"!type":"fn(e: ?)"},"doheadercontextmenu":{"!type":"fn(e: ?)"},"doheaderdblclick":{"!type":"fn(e: ?)"},"dokeydown":{"!type":"fn(e: ?)"},"doKeyEvent":{"!type":"fn(e: ?)","!doc":"<p>Overwritten, see _Grid.doKeyEvent()</p>\n"},"doStartEdit":{"!type":"fn(inCell: ?, inRowIndex: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"endUpdate":{"!type":"fn()","!doc":"<p>Use after calling beginUpdate to render any changes made to rows.</p>\n"},"filter":{"!type":"fn(query: ?, reRender: ?)"},"finishScrollJob":{"!type":"fn()"},"get":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>Default data getter.</p>\n"},"getCell":{"!type":"fn(inIndex: ?) -> +any","!doc":"<p>Retrieves the cell object for a given grid column.</p>\n"},"getCellByField":{"!type":"fn(field: ?) -> ?"},"getCellName":{"!type":"fn(inCell: ?) -> ?"},"getCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?) -> ?"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnTogglingItems":{"!type":"fn() -> ?","!doc":"<p>returns an array of dijit.CheckedMenuItem widgets that can be\nadded to a menu for toggling columns on and off.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getItem":{"!type":"fn(idx: ?) -> +null"},"getItemIndex":{"!type":"fn(item: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowNode":{"!type":"fn(inRowIndex: ?) -> ?","!doc":"<p>find the rowNode that is not a rowSelector</p>\n"},"getSortAsc":{"!type":"fn(inSortInfo: ?) -> ?","!doc":"<p>Returns true if grid is sorted in an ascending direction.</p>\n"},"getSortIndex":{"!type":"fn(inSortInfo: ?) -> number","!doc":"<p>Returns the index of the column on which the grid is sorted</p>\n"},"getSortProps":{"!type":"fn() -> ?"},"hasLayout":{"!type":"fn() -> ?"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"mixin":{"!type":"fn(target: ?, source: ?)"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"plugin":{"!type":"fn(name: ?) -> ?","!doc":"<p>An easier way for getting a plugin, e.g. grid.plugin(&#39;dnd&#39;)</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"postrender":{"!type":"fn()"},"postresize":{"!type":"fn()"},"prerender":{"!type":"fn()"},"removeSelectedRows":{"!type":"fn()","!doc":"<p>Remove the selected rows from the grid.</p>\n"},"render":{"!type":"fn()","!doc":"<p>Render the grid, headers, and views. Edit and scrolling states are reset. To retain edit and\nscrolling states, see Update.</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>Update the grid&#39;s rendering dimensions and resize it</p>\n"},"rowHeightChanged":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update grid when the height of a row has changed. Row height is handled automatically as rows\nare rendered. Use this function only to update a row&#39;s height outside the normal rendering process.</p>\n"},"scrollTo":{"!type":"fn(inTop: ?)","!doc":"<p>Vertically scroll the grid to a given pixel position</p>\n"},"scrollToRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Scroll the grid to a specific row.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?, textDir: ?)"},"setCellWidth":{"!type":"fn(inIndex: ?, inUnitWidth: ?)"},"setHeaderMenu":{"!type":"fn(menu: ?)"},"setItems":{"!type":"fn(items: ?)"},"setQuery":{"!type":"fn(query: ?, queryOptions: ?)"},"setScrollTop":{"!type":"fn(inTop: ?)"},"setSortIndex":{"!type":"fn(inIndex: ?, inAsc: ?)","!doc":"<p>Sort the grid on a column in a specified direction</p>\n"},"setSortInfo":{"!type":"fn(inSortInfo: ?)"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions: ?)"},"setStructure":{"!type":"fn(inStructure: ?)","!doc":"<p>Install a new structure and rebuild the grid.</p>\n"},"setupHeaderMenu":{"!type":"fn()"},"showMessage":{"!type":"fn(message: ?)"},"sizeChange":{"!type":"fn()"},"sort":{"!type":"fn()"},"startup":{"!type":"fn()"},"styleRowState":{"!type":"fn(inRow: ?)","!doc":"<p>Perform row styling</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"textSizeChanged":{"!type":"fn()"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn()","!doc":"<p>Update the grid, retaining edit and scrolling states.</p>\n"},"updateRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Render a single row.</p>\n"},"updateRowCount":{"!type":"fn(inRowCount: ?)","!doc":"<p>Change the number of rows.</p>\n"},"updateRows":{"!type":"fn(startIndex: ?, howMany: ?)","!doc":"<p>Render consecutive rows at once.</p>\n"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update the styles for a row after it&#39;s state has changed.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"markupFactory":{"!type":"fn(props: ?, node: ?, ctor: ?, cellFunc: ?)"},"registerPlugin":{"!type":"fn(clazz: ?, props: ?)"}},"dojox/grid/enhanced/_PluginManager":{"!type":"fn()","prototype":{"!doc":"<p>Singleton plugin manager</p>\n","_connects":{"!doc":"<p>Connection list</p>\n"},"_options":{"!doc":"<p>Normalized plugin options</p>\n"},"_plugins":{"!doc":"<p>Plugin list</p>\n"},"constructor":{"!type":"fn(inGrid: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all resources</p>\n"},"forEach":{"!type":"fn(func: ?, args: ?)"},"getPlugin":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get plugin(&quot;name&quot;)</p>\n"},"getPluginClazz":{"!type":"fn(clazz: ?) -> +Class","!doc":"<p>Load target plugin which must be already required (require(..))</p>\n"},"isFixedCell":{"!type":"fn(cell: ?) -> bool","!doc":"<p>See if target cell(column) is fixed or not.</p>\n"},"loadPlugin":{"!type":"fn(name: ?) -> ?","!doc":"<p>Load required plugin(&quot;name&quot;)</p>\n"},"pluginExisted":{"!type":"fn(name: ?) -> bool","!doc":"<p>Check if plugin(&quot;name&quot;) existed</p>\n"},"postInit":{"!type":"fn()","!doc":"<p>Load plugins after DataGrid.postCreate() - the default phase when plugins are created\nSee EnhancedGrid.postCreate()</p>\n"},"preInit":{"!type":"fn()","!doc":"<p>Load appropriate plugins before DataGrid.postCreate().\nSee EnhancedGrid.postCreate()</p>\n"},"startup":{"!type":"fn()"}},"registry":{},"registerPlugin":{"!type":"fn(clazz: ?, props?: ?)","!doc":"<p>Register plugins - TODO, a better way rather than global registry?</p>\n"}},"dojox/grid/enhanced/_Events":{"!type":"fn()","prototype":{"!doc":"<p>Overwrite some default events of DataGrid\n</p>\n","_events":{"!doc":"<p>Method map cached from dojox.grid._Events().</p>\n"},"cellActiveClass":{"!doc":"<p>css class to apply to grid content cells when activated(mouse down)</p>\n"},"headerCellActiveClass":{"!doc":"<p>css class to apply to grid header cells when activated(mouse down)</p>\n"},"rowActiveClass":{"!doc":"<p>css class to apply to grid rows when activated(mouse down)</p>\n"},"constructor":{"!type":"fn(inGrid: ?)"},"dokeyup":{"!type":"fn(e: ?)","!doc":"<p>Grid key up event handler.</p>\n"},"domousedown":{"!type":"fn(e: ?)"},"domouseup":{"!type":"fn(e: ?)"}}},"dojox/grid/enhanced/_FocusManager":{"!type":"fn()","prototype":{"!proto":"dojox/grid/_FocusManager.prototype","_blurFromEditableCell":{},"_colHeadFocusIdx":{},"_colHeadNode":{},"_contextMenuBindNode":{},"_isNavigating":{},"_navElems":{},"focusClass":{},"focusView":{},"tabbingOut":{},"addArea":{"!type":"fn(area: ?)"},"blurHeader":{"!type":"fn()"},"colSizeAdjust":{"!type":"fn(e: ?, colIdx: ?, delta: ?)"},"constructor":{"!type":"fn(grid: ?)"},"contentMouseEvent":{"!type":"fn(evt: ?)"},"currentArea":{"!type":"fn(areaName: ?, toBlurOld: ?) -> ?","!doc":"<p>Set current area to the one areaName refers.</p>\n"},"destroy":{"!type":"fn()"},"doBlur":{"!type":"fn(e: ?)"},"doColHeaderBlur":{"!type":"fn(e: ?)"},"doColHeaderFocus":{"!type":"fn(e: ?)"},"doContextMenu":{"!type":"fn(e: ?)"},"doFocus":{"!type":"fn(e: ?)","!doc":"<p>Overwritten\ntrap focus only for grid dom node\ndo not focus for scrolling if grid is about to blur</p>\n"},"doLastNodeBlur":{"!type":"fn(e: ?)"},"doLastNodeFocus":{"!type":"fn(e: ?)","!doc":"<p>Overwritten</p>\n"},"findAndFocusGridCell":{"!type":"fn()","!doc":"<p>Overwritten</p>\n"},"focusArea":{"!type":"fn(areaId: ?, evt: ?)"},"focusGrid":{"!type":"fn(inSkipFocusCell: ?)"},"focusGridView":{"!type":"fn()"},"focusHeader":{"!type":"fn(evt: ?, step: ?) -> bool"},"getArea":{"!type":"fn(areaName: ?) -> ?"},"getHeaderIndex":{"!type":"fn() -> +any","!doc":"<p>if one of the column headers currently has focus, return its index.</p>\n"},"headerMouseEvent":{"!type":"fn(evt: ?)"},"ignoreArea":{"!type":"fn(name: ?)"},"initFocusView":{"!type":"fn()","!doc":"<p>Overwritten</p>\n"},"isFirstFocusCell":{"!type":"fn() -> bool"},"isFocusCell":{"!type":"fn(inCell: ?, inRowIndex: ?) -> +any","!doc":"<p>states if the given cell is focused</p>\n"},"isLastFocusCell":{"!type":"fn() -> bool"},"isNavHeader":{"!type":"fn() -> +any","!doc":"<p>Overwritten\nCheck whether currently navigating among column headers.</p>\n"},"isNoFocusCell":{"!type":"fn() -> bool"},"move":{"!type":"fn(inRowDelta: ?, inColDelta: ?)","!doc":"<p>focus grid cell or  simulate focus to column header based on position relative to current focus</p>\n"},"next":{"!type":"fn()","!doc":"<p>focus next grid cell</p>\n"},"nextKey":{"!type":"fn(e: ?)","!doc":"<p>Overwritten</p>\n"},"placeArea":{"!type":"fn(name: ?, pos: ?, otherAreaName: ?)","!doc":"<p>Place the area refered by <em>name</em> at some logical position relative to an existing area.</p>\n"},"previous":{"!type":"fn()","!doc":"<p>focus previous grid cell</p>\n"},"previousKey":{"!type":"fn(e: ?)","!doc":"<p>Overwritten</p>\n"},"removeArea":{"!type":"fn(areaName: ?)"},"scrollIntoView":{"!type":"fn() -> +null"},"setFocusCell":{"!type":"fn(inCell: ?, inRowIndex: ?)","!doc":"<p>Overwritten - focuses the given grid cell</p>\n"},"setFocusIndex":{"!type":"fn(inRowIndex: ?, inCellIndex: ?)","!doc":"<p>focuses the given grid cell</p>\n"},"styleRow":{"!type":"fn(inRow: ?)"},"tab":{"!type":"fn(step: ?, evt: ?)"},"tabOut":{"!type":"fn(inFocusNode: ?)"}}},"dojox/grid/enhanced/plugins/_SelectionPreserver":{"!type":"fn()","prototype":{"!proto":"dojox/grid/_SelectionPreserver.prototype","!doc":"<p>Preserve selections across various user actions.\n</p>\n","constructor":{"!type":"fn(selection: ?)"},"destroy":{"!type":"fn()"},"reset":{"!type":"fn()"}}},"dojox/grid/LazyTreeGrid":{"!type":"fn()","prototype":{"!proto":"dojox/grid/TreeGrid.prototype","!doc":"<p>An enhanced TreeGrid widget which supports lazy-loading for nested children items\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_bop":{},"_by_idty":{},"_by_idx":{},"_cache":{},"_click":{},"_eop":{},"_focusManager":{},"_isLoaded":{},"_isLoading":{},"_pages":{},"_pending_requests":{},"_placeholders":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_requests":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setSummaryAttr":{},"_size":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_store_connects":{},"aggregator":{"!doc":"<p>The aggregator class - it will be populated automatically if we\nare a collapsible grid</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoHeight":{"!doc":"<p>If autoHeight is true, grid height is automatically set to fit the data.\nIf it is an integer, the height will be automatically set to fit the data\nif there are fewer than that many rows - and the height will be set to show\nthat many rows if there are more</p>\n"},"autoRender":{"!doc":"<p>If autoRender is true, grid will render itself after initialization.</p>\n"},"autoWidth":{"!doc":"<p>If autoWidth is true, grid width is automatically set to fit the data.</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"cellOverClass":{"!doc":"<p>css class to apply to grid cells over which the cursor is placed.</p>\n"},"class":{},"classTag":{"!doc":"<p>CSS class applied to the grid&#39;s domNode</p>\n"},"colSpans":{"!doc":"<p>A json object that defines column span of each level rows.  Attributes:\n\n</p>\n<ul>\n<li>0/1/..: which level need to colspan</li>\n<li>start: start column index of colspan</li>\n<li>end: end column index of colspan</li>\n<li>primary: index of column which content will be displayed (default is value of start).</li>\n</ul>\n<p>example:\n\n</p>\n<pre><code>colSpans = {\n0:  [\n        {start: 0, end: 1, primary: 0},\n        {start: 2, end: 4, primary: 3}\n    ],\n1:  [\n        {start: 0, end: 3, primary: 1}\n    ]\n};</code></pre>\n"},"columnReordering":{"!doc":"<p>If set to true, will add drag and drop reordering to views with one row of columns.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultHeight":{"!doc":"<p>default height of the grid, measured in any valid css unit.</p>\n"},"defaultOpen":{"!doc":"<p>Whether or not we default to open (all levels).  This defaults to\nfalse for grids with a treeModel.</p>\n"},"defaultState":{"!doc":"<p>Used to restore the state of LazyTreeGrid.\nThis object should ONLY be obtained from <code>LazyTreeGrid.getState()</code>.</p>\n"},"delayScroll":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>indicates if the grid contains editable cells, default is false\nset to true if editable cell encountered during rendering</p>\n"},"elasticView":{"!doc":"<p>Override defaults and make the indexed grid view elastic, thus filling available horizontal space.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows when the grid encounters an error loading</p>\n"},"escapeHTMLInData":{"!doc":"<p>This will escape HTML brackets from the data to prevent HTML from\nuser-inputted data being rendered with may contain JavaScript and result in\nXSS attacks. This is true by default, and it is recommended that it remain\ntrue. Setting this to false will allow data to be displayed in the grid without\nfiltering, and should be only used if it is known that the data won&#39;t contain\nmalicious scripts. If HTML is needed in grid cells, it is recommended that\nyou use the formatter function to generate the HTML (the output of\nformatter functions is not filtered, even with escapeHTMLInData set to true).</p>\n"},"expandoCell":{"!doc":"<p>When used in conjunction with a treeModel (see above), this is a 0-based\nindex of the cell in which to place the actual expando</p>\n"},"fastScroll":{"!doc":"<p>flag modifies vertical scrolling behavior. Defaults to true but set to false for slower\nscroll performance but more immediate scrolling feedback</p>\n"},"fetchText":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatterScope":{"!doc":"<p>An object to execute format functions within.  If not set, the\nformat functions will execute within the scope of the cell that\nhas a format function.</p>\n"},"headerMenu":{"!doc":"<p>If set to a dijit.Menu, will use this as a context menu for the grid headers.</p>\n"},"height":{"!doc":"<p>explicit height of the grid, measured in any valid css unit.  This will be populated (and overridden)\nif the height: css attribute exists on the source node.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"initialWidth":{"!doc":"<p>A css string to use to set our initial width (only used if autoWidth\nis true).  The first rendering of the grid will be this width, any\nresizing of columns, etc will result in the grid switching to\nautoWidth mode.  Note, this width will override any styling in a\nstylesheet or directly on the node.</p>\n"},"invalidated":{},"items":{},"keepRows":{"!doc":"<p>Number of rows to keep in the rendering cache.</p>\n"},"keepSelection":{"!doc":"<p>Whether keep selection after sort, filter etc.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while the grid is loading</p>\n"},"noDataMessage":{"!doc":"<p>Message that shows if the grid has no data - wrap it in a\nspan with class &#39;dojoxGridNoData&#39; if you want it to be\nstyled similar to the loading and error messages</p>\n"},"observer":{},"openAtLevels":{"!doc":"<p>Which levels we are open at (overrides defaultOpen for the values\nthat exist here).  Its values can be a boolean (true/false) or an\ninteger (for the # of children to be closed if there are more than\nthat)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeholderLabel":{"!doc":"<p>Label of placeholders to search for in the header menu to replace with column toggling\nmenu items.</p>\n"},"query":{},"queryOptions":{},"rowCount":{},"rowHeight":{"!doc":"<p>If rowHeight is set to a positive number, it will define the height of the rows\nin pixels. This can provide a significant performance advantage, since it\neliminates the need to measure row sizes during rendering, which is one\nthe primary bottlenecks in the DataGrid&#39;s performance.</p>\n"},"rowSelector":{"!doc":"<p>If set to true, will add a row selector view to this grid.  If set to a CSS width, will add\na row selector of that width to this grid.</p>\n"},"rowsPerPage":{"!doc":"<p>Number of rows to render at a time.</p>\n"},"scrollRedrawThreshold":{"!doc":"<p>pixel distance a user must scroll vertically to trigger grid scrolling.</p>\n"},"searchContainerNode":{},"selectable":{"!doc":"<p>Set to true if you want to be able to select the text within the grid.</p>\n"},"selectionMode":{"!doc":"<p>Set the selection mode of grid&#39;s Selection.  Value must be &#39;single&#39;, &#39;multiple&#39;,\nor &#39;extended&#39;.  Default is &#39;extended&#39;.</p>\n"},"singleClickEdit":{"!doc":"<p>Single-click starts editing. Default is double-click</p>\n"},"sortChildItems":{"!doc":"<p>If true, child items will be returned sorted according to the sorting\nproperties of the grid.</p>\n"},"sortFields":{},"sortInfo":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{},"structure":{"!doc":"<p>View layout definition.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"summary":{"!doc":"<p>Customizable summary descriptions which will be added to grid.domNode</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"treeModel":{"!doc":"<p>A tree store model object.</p>\n"},"updateDelay":{"!doc":"<p>Time, in milliseconds, to delay updates automatically so that multiple\ncalls to onSet/onNew/onDelete don&#39;t keep rerendering the grid.  Set\nto 0 to immediately cause updates.  A higher value will result in\nbetter performance at the expense of responsiveness of the grid.</p>\n"},"updating":{},"addRow":{"!type":"fn()","!doc":"<p>Add a row to the grid.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"beginUpdate":{"!type":"fn()","!doc":"<p>Use to make multiple changes to rows while queueing row updating.</p>\n"},"buildRendering":{"!type":"fn()"},"buildViews":{"!type":"fn()"},"canEdit":{"!type":"fn(inCell: ?, inRowIndex: ?) -> ?"},"canSort":{"!type":"fn() -> bool"},"collapse":{"!type":"fn(itemId?: ?)","!doc":"<p>Collapse the row with the given itemId.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createLayout":{"!type":"fn()","!doc":"<p>Creates a new Grid layout</p>\n"},"createManagers":{"!type":"fn()"},"createScroller":{"!type":"fn()"},"createSelection":{"!type":"fn()"},"createView":{"!type":"fn(inClass: ?, idx: ?) -> +instance"},"createViews":{"!type":"fn()"},"defaultUpdate":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchContentEvent":{"!type":"fn(e: ?)"},"dispatchHeaderEvent":{"!type":"fn(e: ?)"},"dispatchKeyEvent":{"!type":"fn(e: ?)"},"doApplyCellEdit":{"!type":"fn(inValue: ?, inRowIndex: ?, inAttrName: ?)"},"doApplyEdit":{"!type":"fn(inRowIndex: ?, inDataAttr: ?)"},"doCancelEdit":{"!type":"fn(inRowIndex: ?)"},"doclick":{"!type":"fn(e: ?)"},"docontextmenu":{"!type":"fn(e: ?)"},"dodblclick":{"!type":"fn(e: ?)"},"doheaderclick":{"!type":"fn(e: ?)"},"doheadercontextmenu":{"!type":"fn(e: ?)"},"doheaderdblclick":{"!type":"fn(e: ?)"},"dokeydown":{"!type":"fn(e: ?)"},"doKeyEvent":{"!type":"fn(e: ?)"},"doStartEdit":{"!type":"fn(inCell: ?, inRowIndex: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"endUpdate":{"!type":"fn()","!doc":"<p>Use after calling beginUpdate to render any changes made to rows.</p>\n"},"expand":{"!type":"fn(itemId?: ?)","!doc":"<p>Expand the row with the given itemId.</p>\n"},"expandoFetch":{"!type":"fn(rowIndex: ?, open: ?)","!doc":"<p>Function for fetch children of a given row</p>\n"},"filter":{"!type":"fn(query: ?, reRender: ?)"},"finishScrollJob":{"!type":"fn()"},"get":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>Default data getter.</p>\n"},"getCell":{"!type":"fn(inIndex: ?) -> +any","!doc":"<p>Retrieves the cell object for a given grid column.</p>\n"},"getCellName":{"!type":"fn(inCell: ?) -> ?"},"getCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?) -> ?"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnTogglingItems":{"!type":"fn() -> ?","!doc":"<p>returns an array of dijit.CheckedMenuItem widgets that can be\nadded to a menu for toggling columns on and off.</p>\n"},"getDefaultOpenState":{"!type":"fn(cellDef: ?, item: ?) -> ?","!doc":"<p>Returns the default open state for the given definition and item\nIt reads from the openAtLevels and defaultOpen values of the\ngrid to calculate if the given item should default to open or\nnot.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getItem":{"!type":"fn(idx: ?) -> ?","!doc":"<p>overridden so that you can pass in a &#39;/&#39; delimited string of indexes to get the\nitem based off its path...that is, passing in &quot;1/3/2&quot; will get the\n3rd (0-based) child from the 4th child of the 2nd top-level item.</p>\n"},"getItemIndex":{"!type":"fn(item: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowNode":{"!type":"fn(inRowIndex: ?) -> ?","!doc":"<p>find the rowNode that is not a rowSelector</p>\n"},"getSortAsc":{"!type":"fn(inSortInfo: ?) -> ?","!doc":"<p>Returns true if grid is sorted in an ascending direction.</p>\n"},"getSortIndex":{"!type":"fn(inSortInfo: ?) -> number","!doc":"<p>Returns the index of the column on which the grid is sorted</p>\n"},"getSortProps":{"!type":"fn() -> ?"},"getState":{"!type":"fn() -> ?","!doc":"<p>Get the current state of LazyTreeGrid including expanding, sorting, selection and scroll top state.</p>\n"},"hasLayout":{"!type":"fn() -> ?"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"postrender":{"!type":"fn()"},"postresize":{"!type":"fn()"},"prerender":{"!type":"fn()"},"refresh":{"!type":"fn(keepState: ?)","!doc":"<p>Refresh, and persist the expand/collapse state when keepState equals true</p>\n"},"removeSelectedRows":{"!type":"fn()","!doc":"<p>Remove the selected rows from the grid.</p>\n"},"render":{"!type":"fn()"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>Update the grid&#39;s rendering dimensions and resize it</p>\n"},"rowHeightChanged":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update grid when the height of a row has changed. Row height is handled automatically as rows\nare rendered. Use this function only to update a row&#39;s height outside the normal rendering process.</p>\n"},"scrollTo":{"!type":"fn(inTop: ?)","!doc":"<p>Vertically scroll the grid to a given pixel position</p>\n"},"scrollToRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Scroll the grid to a specific row.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?, textDir: ?)"},"setCellWidth":{"!type":"fn(inIndex: ?, inUnitWidth: ?)"},"setHeaderMenu":{"!type":"fn(menu: ?)"},"setItems":{"!type":"fn(items: ?)"},"setModel":{"!type":"fn(treeModel: ?)"},"setQuery":{"!type":"fn(query: ?, queryOptions: ?)"},"setScrollTop":{"!type":"fn(inTop: ?)"},"setSortIndex":{"!type":"fn(inIndex: ?, inAsc: ?)"},"setSortInfo":{"!type":"fn(inSortInfo: ?)"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions: ?)"},"setStructure":{"!type":"fn(inStructure: ?)","!doc":"<p>Install a new structure and rebuild the grid.</p>\n"},"setupHeaderMenu":{"!type":"fn()"},"showMessage":{"!type":"fn(message: ?)"},"sizeChange":{"!type":"fn()"},"sort":{"!type":"fn()"},"startup":{"!type":"fn()"},"styleRowNode":{"!type":"fn(rowIndex: ?, rowNode: ?)"},"styleRowState":{"!type":"fn(inRow: ?)","!doc":"<p>Perform row styling</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"textSizeChanged":{"!type":"fn()"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn()","!doc":"<p>Update the grid, retaining edit and scrolling states.</p>\n"},"updateRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Render a single row.</p>\n"},"updateRowCount":{"!type":"fn(inRowCount: ?)","!doc":"<p>Change the number of rows.</p>\n"},"updateRows":{"!type":"fn(startIndex: ?, howMany: ?)","!doc":"<p>Render consecutive rows at once.</p>\n"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update the styles for a row after it&#39;s state has changed.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"markupFactory":{"!type":"fn(props: ?, node: ?, ctor: ?, cellFunc: ?)"}},"dojox/grid/TreeGrid":{"!type":"fn()","prototype":{"!proto":"dojox/grid/DataGrid.prototype","!doc":"<p>A grid that supports nesting rows - it provides an expando function\nsimilar to dijit.Tree.  It also provides mechanisms for aggregating\nthe values of subrows</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_bop":{},"_by_idty":{},"_by_idx":{},"_cache":{},"_click":{},"_eop":{},"_focusManager":{},"_isLoaded":{},"_isLoading":{},"_pages":{},"_pending_requests":{},"_placeholders":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_requests":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setSummaryAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_store_connects":{},"aggregator":{"!doc":"<p>The aggregator class - it will be populated automatically if we\nare a collapsible grid</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoHeight":{"!doc":"<p>If autoHeight is true, grid height is automatically set to fit the data.\nIf it is an integer, the height will be automatically set to fit the data\nif there are fewer than that many rows - and the height will be set to show\nthat many rows if there are more</p>\n"},"autoRender":{"!doc":"<p>If autoRender is true, grid will render itself after initialization.</p>\n"},"autoWidth":{"!doc":"<p>If autoWidth is true, grid width is automatically set to fit the data.</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"cellOverClass":{"!doc":"<p>css class to apply to grid cells over which the cursor is placed.</p>\n"},"class":{},"classTag":{"!doc":"<p>CSS class applied to the grid&#39;s domNode</p>\n"},"columnReordering":{"!doc":"<p>If set to true, will add drag and drop reordering to views with one row of columns.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultHeight":{"!doc":"<p>default height of the grid, measured in any valid css unit.</p>\n"},"defaultOpen":{"!doc":"<p>Whether or not we default to open (all levels).  This defaults to\nfalse for grids with a treeModel.</p>\n"},"delayScroll":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>indicates if the grid contains editable cells, default is false\nset to true if editable cell encountered during rendering</p>\n"},"elasticView":{"!doc":"<p>Override defaults and make the indexed grid view elastic, thus filling available horizontal space.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows when the grid encounters an error loading</p>\n"},"escapeHTMLInData":{"!doc":"<p>This will escape HTML brackets from the data to prevent HTML from\nuser-inputted data being rendered with may contain JavaScript and result in\nXSS attacks. This is true by default, and it is recommended that it remain\ntrue. Setting this to false will allow data to be displayed in the grid without\nfiltering, and should be only used if it is known that the data won&#39;t contain\nmalicious scripts. If HTML is needed in grid cells, it is recommended that\nyou use the formatter function to generate the HTML (the output of\nformatter functions is not filtered, even with escapeHTMLInData set to true).</p>\n"},"expandoCell":{"!doc":"<p>When used in conjunction with a treeModel (see above), this is a 0-based\nindex of the cell in which to place the actual expando</p>\n"},"fastScroll":{"!doc":"<p>flag modifies vertical scrolling behavior. Defaults to true but set to false for slower\nscroll performance but more immediate scrolling feedback</p>\n"},"fetchText":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"formatterScope":{"!doc":"<p>An object to execute format functions within.  If not set, the\nformat functions will execute within the scope of the cell that\nhas a format function.</p>\n"},"headerMenu":{"!doc":"<p>If set to a dijit.Menu, will use this as a context menu for the grid headers.</p>\n"},"height":{"!doc":"<p>explicit height of the grid, measured in any valid css unit.  This will be populated (and overridden)\nif the height: css attribute exists on the source node.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"initialWidth":{"!doc":"<p>A css string to use to set our initial width (only used if autoWidth\nis true).  The first rendering of the grid will be this width, any\nresizing of columns, etc will result in the grid switching to\nautoWidth mode.  Note, this width will override any styling in a\nstylesheet or directly on the node.</p>\n"},"invalidated":{},"items":{},"keepRows":{"!doc":"<p>Number of rows to keep in the rendering cache.</p>\n"},"keepSelection":{"!doc":"<p>Whether keep selection after sort, filter etc.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while the grid is loading</p>\n"},"noDataMessage":{"!doc":"<p>Message that shows if the grid has no data - wrap it in a\nspan with class &#39;dojoxGridNoData&#39; if you want it to be\nstyled similar to the loading and error messages</p>\n"},"observer":{},"openAtLevels":{"!doc":"<p>Which levels we are open at (overrides defaultOpen for the values\nthat exist here).  Its values can be a boolean (true/false) or an\ninteger (for the # of children to be closed if there are more than\nthat)</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeholderLabel":{"!doc":"<p>Label of placeholders to search for in the header menu to replace with column toggling\nmenu items.</p>\n"},"query":{},"queryOptions":{},"rowCount":{},"rowHeight":{"!doc":"<p>If rowHeight is set to a positive number, it will define the height of the rows\nin pixels. This can provide a significant performance advantage, since it\neliminates the need to measure row sizes during rendering, which is one\nthe primary bottlenecks in the DataGrid&#39;s performance.</p>\n"},"rowSelector":{"!doc":"<p>If set to true, will add a row selector view to this grid.  If set to a CSS width, will add\na row selector of that width to this grid.</p>\n"},"rowsPerPage":{"!doc":"<p>Number of rows to render at a time.</p>\n"},"scrollRedrawThreshold":{"!doc":"<p>pixel distance a user must scroll vertically to trigger grid scrolling.</p>\n"},"searchContainerNode":{},"selectable":{"!doc":"<p>Set to true if you want to be able to select the text within the grid.</p>\n"},"selectionMode":{"!doc":"<p>Set the selection mode of grid&#39;s Selection.  Value must be &#39;single&#39;, &#39;multiple&#39;,\nor &#39;extended&#39;.  Default is &#39;extended&#39;.</p>\n"},"singleClickEdit":{"!doc":"<p>Single-click starts editing. Default is double-click</p>\n"},"sortChildItems":{"!doc":"<p>If true, child items will be returned sorted according to the sorting\nproperties of the grid.</p>\n"},"sortFields":{},"sortInfo":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{},"structure":{"!doc":"<p>View layout definition.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"summary":{"!doc":"<p>Customizable summary descriptions which will be added to grid.domNode</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"treeModel":{"!doc":"<p>A dijit.Tree model that will be used instead of using aggregates.\nSetting this value will make the TreeGrid behave like a columnar\ntree.  When setting this value, defaultOpen will default to false,\nand openAtLevels will be ignored.</p>\n"},"updateDelay":{"!doc":"<p>Time, in milliseconds, to delay updates automatically so that multiple\ncalls to onSet/onNew/onDelete don&#39;t keep rerendering the grid.  Set\nto 0 to immediately cause updates.  A higher value will result in\nbetter performance at the expense of responsiveness of the grid.</p>\n"},"updating":{},"addRow":{"!type":"fn()","!doc":"<p>Add a row to the grid.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"beginUpdate":{"!type":"fn()","!doc":"<p>Use to make multiple changes to rows while queueing row updating.</p>\n"},"buildRendering":{"!type":"fn()"},"buildViews":{"!type":"fn()"},"canEdit":{"!type":"fn(inCell: ?, inRowIndex: ?) -> ?"},"canSort":{"!type":"fn() -> bool"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createLayout":{"!type":"fn()","!doc":"<p>Creates a new Grid layout</p>\n"},"createManagers":{"!type":"fn()","!doc":"<p>create grid managers for various tasks including rows, focus, selection, editing</p>\n"},"createScroller":{"!type":"fn()"},"createSelection":{"!type":"fn()"},"createView":{"!type":"fn(inClass: ?, idx: ?) -> +instance"},"createViews":{"!type":"fn()"},"defaultUpdate":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchContentEvent":{"!type":"fn(e: ?)"},"dispatchHeaderEvent":{"!type":"fn(e: ?)"},"dispatchKeyEvent":{"!type":"fn(e: ?)"},"doApplyCellEdit":{"!type":"fn(inValue: ?, inRowIndex: ?, inAttrName: ?)"},"doApplyEdit":{"!type":"fn(inRowIndex: ?, inDataAttr: ?)"},"doCancelEdit":{"!type":"fn(inRowIndex: ?)"},"doclick":{"!type":"fn(e: ?)"},"docontextmenu":{"!type":"fn(e: ?)"},"dodblclick":{"!type":"fn(e: ?)"},"doheaderclick":{"!type":"fn(e: ?)"},"doheadercontextmenu":{"!type":"fn(e: ?)"},"doheaderdblclick":{"!type":"fn(e: ?)"},"dokeydown":{"!type":"fn(e: ?)"},"doKeyEvent":{"!type":"fn(e: ?)"},"doStartEdit":{"!type":"fn(inCell: ?, inRowIndex: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"endUpdate":{"!type":"fn()","!doc":"<p>Use after calling beginUpdate to render any changes made to rows.</p>\n"},"filter":{"!type":"fn(query: ?, reRender: ?)"},"finishScrollJob":{"!type":"fn()"},"get":{"!type":"fn(inRowIndex: ?, inItem: ?) -> +any","!doc":"<p>Default data getter.</p>\n"},"getCell":{"!type":"fn(inIndex: ?) -> +any","!doc":"<p>Retrieves the cell object for a given grid column.</p>\n"},"getCellName":{"!type":"fn(inCell: ?) -> ?"},"getCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?) -> ?"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnTogglingItems":{"!type":"fn() -> ?","!doc":"<p>returns an array of dijit.CheckedMenuItem widgets that can be\nadded to a menu for toggling columns on and off.</p>\n"},"getDefaultOpenState":{"!type":"fn(cellDef: ?, item: ?) -> ?","!doc":"<p>Returns the default open state for the given definition and item\nIt reads from the openAtLevels and defaultOpen values of the\ngrid to calculate if the given item should default to open or\nnot.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getItem":{"!type":"fn(idx: ?) -> ?","!doc":"<p>overridden so that you can pass in a &#39;/&#39; delimited string of indexes to get the\nitem based off its path...that is, passing in &quot;1/3/2&quot; will get the\n3rd (0-based) child from the 4th child of the 2nd top-level item.</p>\n"},"getItemIndex":{"!type":"fn(item: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowNode":{"!type":"fn(inRowIndex: ?) -> ?","!doc":"<p>find the rowNode that is not a rowSelector</p>\n"},"getSortAsc":{"!type":"fn(inSortInfo: ?) -> ?","!doc":"<p>Returns true if grid is sorted in an ascending direction.</p>\n"},"getSortIndex":{"!type":"fn(inSortInfo: ?) -> number","!doc":"<p>Returns the index of the column on which the grid is sorted</p>\n"},"getSortProps":{"!type":"fn() -> ?"},"hasLayout":{"!type":"fn() -> ?"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"postrender":{"!type":"fn()"},"postresize":{"!type":"fn()"},"prerender":{"!type":"fn()"},"removeSelectedRows":{"!type":"fn()","!doc":"<p>Remove the selected rows from the grid.</p>\n"},"render":{"!type":"fn()","!doc":"<p>Render the grid, headers, and views. Edit and scrolling states are reset. To retain edit and\nscrolling states, see Update.</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>Update the grid&#39;s rendering dimensions and resize it</p>\n"},"rowHeightChanged":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update grid when the height of a row has changed. Row height is handled automatically as rows\nare rendered. Use this function only to update a row&#39;s height outside the normal rendering process.</p>\n"},"scrollTo":{"!type":"fn(inTop: ?)","!doc":"<p>Vertically scroll the grid to a given pixel position</p>\n"},"scrollToRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Scroll the grid to a specific row.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setCellNodeTextDirection":{"!type":"fn(inColIndex: ?, inRowIndex: ?, textDir: ?)"},"setCellWidth":{"!type":"fn(inIndex: ?, inUnitWidth: ?)"},"setHeaderMenu":{"!type":"fn(menu: ?)"},"setItems":{"!type":"fn(items: ?)"},"setModel":{"!type":"fn(treeModel: ?)"},"setQuery":{"!type":"fn(query: ?, queryOptions: ?)"},"setScrollTop":{"!type":"fn(inTop: ?)"},"setSortIndex":{"!type":"fn(inIndex: ?, inAsc: ?)","!doc":"<p>Sort the grid on a column in a specified direction</p>\n"},"setSortInfo":{"!type":"fn(inSortInfo: ?)"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions: ?)"},"setStructure":{"!type":"fn(inStructure: ?)","!doc":"<p>Install a new structure and rebuild the grid.</p>\n"},"setupHeaderMenu":{"!type":"fn()"},"showMessage":{"!type":"fn(message: ?)"},"sizeChange":{"!type":"fn()"},"sort":{"!type":"fn()"},"startup":{"!type":"fn()"},"styleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"styleRowState":{"!type":"fn(inRow: ?)","!doc":"<p>Perform row styling</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"textSizeChanged":{"!type":"fn()"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn()","!doc":"<p>Update the grid, retaining edit and scrolling states.</p>\n"},"updateRow":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Render a single row.</p>\n"},"updateRowCount":{"!type":"fn(inRowCount: ?)","!doc":"<p>Change the number of rows.</p>\n"},"updateRows":{"!type":"fn(startIndex: ?, howMany: ?)","!doc":"<p>Render consecutive rows at once.</p>\n"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)","!doc":"<p>Update the styles for a row after it&#39;s state has changed.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"markupFactory":{"!type":"fn(props: ?, node: ?, ctor: ?, cellFunc: ?)"}},"dojox/grid/TreeSelection":{"!type":"fn()","prototype":{"!proto":"dojox/grid/DataSelection.prototype","mode":{},"rangeStartIndex":{},"selected":{},"selectedIndex":{},"updating":{},"addToSelection":{"!type":"fn(inItemOrIndex: ?)"},"clear":{"!type":"fn()"},"clickSelect":{"!type":"fn(inIndex: ?, inCtrlKey: ?, inShiftKey: ?)"},"clickSelectEvent":{"!type":"fn(e: ?)"},"constructor":{"!type":"fn(grid: ?)"},"deselect":{"!type":"fn(inItemOrIndex: ?)"},"deselectAll":{"!type":"fn(inItemOrIndex: ?)"},"deselectRange":{"!type":"fn(inFrom: ?, inTo: ?)"},"destroy":{"!type":"fn()"},"getFirstSelected":{"!type":"fn() -> number"},"getNextSelected":{"!type":"fn(inPrev: ?) -> number"},"getSelected":{"!type":"fn() -> []"},"getSelectedCount":{"!type":"fn() -> number"},"insert":{"!type":"fn(inIndex: ?)"},"isSelected":{"!type":"fn(inIndex: ?) -> bool"},"remove":{"!type":"fn(inIndex: ?)"},"select":{"!type":"fn(inIndex: ?)"},"selectRange":{"!type":"fn(inFrom: ?, inTo: ?)"},"setMode":{"!type":"fn(mode: ?)"},"setSelected":{"!type":"fn(inIndex: ?, inSelect: ?)"},"toggleSelect":{"!type":"fn(inIndex: ?)"}}},"dojox/grid/cells/tree":{"formatAggregate":{"!type":"fn(inItem: ?, level: ?, inRowIndexes: ?) -> ?"},"formatAtLevel":{"!type":"fn(inRowIndexes: ?, inItem: ?, level: ?, summaryRow: ?, toggleClass: ?, cellClasses: ?) -> string"},"formatIndexes":{"!type":"fn(inRowIndexes: ?, inItem: ?) -> ?"},"getOpenState":{"!type":"fn(itemId: ?) -> ?"}},"dojox/grid/_TreeView":{"!type":"fn()","prototype":{"!proto":"dojox/grid/_View.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_nativeScroll":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_togglingColumn":{"!doc":"<p>Width of the column being toggled (-1 for none)</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"classTag":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultWidth":{"!doc":"<p>Default width of the view</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"firstScroll":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lastTop":{},"marginBottom":{},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"rowPad":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"viewWidth":{"!doc":"<p>Width for the view, in valid css unit</p>\n"},"adaptHeight":{"!type":"fn(minusScroll: ?)"},"adaptWidth":{"!type":"fn()"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"buildRow":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"buildRowContent":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"convertColPctToFixed":{"!type":"fn() -> bool"},"createRowNode":{"!type":"fn(inRowIndex: ?) -> ?"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchContentEvent":{"!type":"fn(e: ?) -> ?"},"dispatchHeaderEvent":{"!type":"fn(e: ?) -> ?"},"doContentEvent":{"!type":"fn(e: ?)"},"doHeaderEvent":{"!type":"fn(e: ?)"},"doscroll":{"!type":"fn(inEvent: ?)"},"doStyleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getCellNode":{"!type":"fn(inRowIndex: ?, inCellIndex: ?) -> ?"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnsWidth":{"!type":"fn() -> number"},"getContentWidth":{"!type":"fn() -> string"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getHeaderCellNode":{"!type":"fn(inCellIndex: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowNode":{"!type":"fn(inRowIndex: ?) -> ?"},"getScrollbarWidth":{"!type":"fn() -> ?"},"getWidth":{"!type":"fn() -> string"},"hasHScrollbar":{"!type":"fn(reset: ?) -> bool"},"hasVScrollbar":{"!type":"fn(reset: ?) -> bool"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"render":{"!type":"fn()"},"renderHeader":{"!type":"fn()"},"renderRow":{"!type":"fn(inRowIndex: ?) -> ?"},"resize":{"!type":"fn()"},"rowRemoved":{"!type":"fn(inRowIndex: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setColumnsWidth":{"!type":"fn(width: ?)"},"setColWidth":{"!type":"fn(inIndex: ?, inWidth: ?)"},"setScrollTop":{"!type":"fn(inTop: ?) -> ?"},"setSize":{"!type":"fn(w: ?, h: ?)"},"setStructure":{"!type":"fn(inStructure: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleRow":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"styleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"testFlexCells":{"!type":"fn() -> ?"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn()"},"updateRow":{"!type":"fn(inRowIndex: ?) -> ?"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)"},"updateStructure":{"!type":"fn()"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/grid/LazyTreeGridStoreModel":{"!type":"fn()","prototype":{"!proto":"dijit/tree/ForestStoreModel.prototype","childrenAttrs":{"!doc":"<p>One or more attribute names (attributes in the dojo.data item) that specify that item&#39;s children</p>\n"},"deferItemLoadingUntilExpand":{"!doc":"<p>Setting this to true will cause the TreeStoreModel to defer calling loadItem on nodes\nuntil they are expanded. This allows for lazying loading where only one\nloadItem (and generally one network call, consequently) per expansion\n(rather than one for each child).\nThis relies on partial loading of the children items; each children item of a\nfully loaded item should contain the label and info about having children.</p>\n"},"labelAttr":{"!doc":"<p>If specified, get label for tree node from this attribute, rather\nthan by calling store.getLabel()</p>\n"},"newItemIdAttr":{"!doc":"<p>Name of attribute in the Object passed to newItem() that specifies the id.\n\n</p>\n<p>If newItemIdAttr is set then it&#39;s used when newItem() is called to see if an\nitem with the same id already exists, and if so just links to the old item\n(so that the old item ends up with two parents).\n\n</p>\n<p>Setting this to null or &quot;&quot; will make every drop create a new item.</p>\n"},"query":{"!doc":"<p>Specifies the set of children of the root item.</p>\n"},"root":{"!doc":"<p>Pointer to the root item (read only, not a parameter)</p>\n"},"rootId":{"!doc":"<p>ID of fabricated root item</p>\n"},"rootLabel":{"!doc":"<p>Label of fabricated root item</p>\n"},"serverStore":{},"store":{"!doc":"<p>Underlying store</p>\n"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()"},"fetchItemByIdentity":{"!type":"fn(keywordArgs: ?)"},"getChildren":{"!type":"fn(parentItem: ?, onComplete: ?, onError: ?, queryObj: ?)"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getLabel":{"!type":"fn(item: ?) -> ?"},"getRoot":{"!type":"fn(onItem: ?, onError: ?)","!doc":"<p>Calls onItem with the root item for the tree, possibly a fabricated item.\nCalls onError on error.</p>\n"},"isItem":{"!type":"fn(something: ?) -> bool"},"mayHaveChildren":{"!type":"fn(item: ?) -> ?"},"newItem":{"!type":"fn(args: ?, parent: ?, insertIndex?: ?) -> ?","!doc":"<p>Creates a new item.   See dojo/data/api/Write for details on args.\nUsed in drag &amp; drop when item from external source dropped onto tree.</p>\n"},"pasteItem":{"!type":"fn(childItem: ?, oldParentItem: ?, newParentItem: ?, bCopy: ?, insertIndex?: ?)","!doc":"<p>Move or copy an item from one parent item to another.\nUsed in drag &amp; drop</p>\n"}}},"dojox/grid/_CheckBoxSelector":{},"dojox/grid/_Selector":{"!type":"fn()","prototype":{"!proto":"dojox/grid/_View.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_nativeScroll":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_togglingColumn":{"!doc":"<p>Width of the column being toggled (-1 for none)</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"classTag":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultWidth":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"firstScroll":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inputType":{},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lastTop":{},"marginBottom":{},"noscroll":{},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"padBorderWidth":{},"rowPad":{},"searchContainerNode":{},"selectionMode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"viewWidth":{"!doc":"<p>Width for the view, in valid css unit</p>\n"},"adaptHeight":{"!type":"fn(minusScroll: ?)"},"adaptWidth":{"!type":"fn()"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"buildRow":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"buildRowContent":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"convertColPctToFixed":{"!type":"fn() -> bool"},"createRowNode":{"!type":"fn(inRowIndex: ?) -> ?"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchContentEvent":{"!type":"fn(e: ?) -> ?"},"dispatchHeaderEvent":{"!type":"fn(e: ?) -> ?"},"doContentEvent":{"!type":"fn(e: ?)"},"doHeaderEvent":{"!type":"fn(e: ?)"},"doscroll":{"!type":"fn(inEvent: ?)"},"doStyleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getCellNode":{"!type":"fn(inRowIndex: ?, inCellIndex: ?) -> ?"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnsWidth":{"!type":"fn() -> number"},"getContentWidth":{"!type":"fn() -> string"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getHeaderCellNode":{"!type":"fn(inCellIndex: ?) -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getRowNode":{"!type":"fn(inRowIndex: ?) -> ?"},"getScrollbarWidth":{"!type":"fn() -> ?"},"getWidth":{"!type":"fn() -> ?"},"hasHScrollbar":{"!type":"fn(reset: ?) -> bool"},"hasVScrollbar":{"!type":"fn(reset: ?) -> bool"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"render":{"!type":"fn()"},"renderHeader":{"!type":"fn()"},"renderRow":{"!type":"fn(inRowIndex: ?) -> ?"},"resize":{"!type":"fn()"},"rowRemoved":{"!type":"fn(inRowIndex: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setColumnsWidth":{"!type":"fn(width: ?)"},"setColWidth":{"!type":"fn(inIndex: ?, inWidth: ?)"},"setScrollTop":{"!type":"fn(inTop: ?) -> ?"},"setSize":{"!type":"fn(w: ?, h: ?)"},"setStructure":{"!type":"fn(s: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleRow":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"styleRowNode":{"!type":"fn(inRowIndex: ?, inRowNode: ?)"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"testFlexCells":{"!type":"fn() -> ?"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"update":{"!type":"fn()"},"updateRow":{"!type":"fn(inRowIndex: ?) -> ?"},"updateRowStyles":{"!type":"fn(inRowIndex: ?)"},"updateStructure":{"!type":"fn()"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/grid/_RadioSelector":{},"dojox/grid/enhanced/_Plugin":{"!type":"fn()","prototype":{"!doc":"<p>Base class for all plugins.</p>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>Plugin name, e.g. &#39;nestedSorting&#39;, &#39;dnd&#39;...</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(inGrid: ?, option: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all resources.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"init":{"!type":"fn()"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/AutoScroll":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Provides horizontal and vertical auto-scroll for grid.</p>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"autoScrollInterval":{"!doc":"<p>The time interval (in miliseconds) between 2 scrolling.</p>\n"},"autoScrollMargin":{"!doc":"<p>The width (in pixel) of the margin area where autoscroll can be triggered.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>Plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?, args: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all resources.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"init":{"!type":"fn()"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/CellMerge":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>This plugin provides functions to merge(un-merge) adjacent cells within one row.\nAcceptable plugin parameters:\n\n</p>\n<ul>\n<li>mergedCells: Array: An array of objects with structure:</li>\n</ul>\n<pre><code>{\n    row: function(Integer)|Integer\n        If it&#39;s a function, it&#39;s a predicate to decide which rows are to be merged.\n        It takes an integer (the row index), and should return true or false;\n    start: Integer\n        The column index of the left most cell that shall be merged.\n    end: Integer\n        The column index of the right most cell that shall be merged.\n    major: Integer\n        The column index of the cell whose content should be used as the content of the merged cell.\n        It must be larger than or equal to the startColumnIndex, and less than or equal to the endColumnIndex.\n        If it is omitted, the content of the leading edge (left-most for ltr, right most for rtl) cell will be used.\n}</code></pre>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>Plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?, args: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all resources.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"getMergedCells":{"!type":"fn() -> []","!doc":"<p>Get all records of currently merged cells.</p>\n"},"getMergedCellsByRow":{"!type":"fn(rowIndex: ?) -> []","!doc":"<p>Get the records of currently merged cells at the given row.</p>\n"},"init":{"!type":"fn()"},"mergeCells":{"!type":"fn(rowTester: ?, startColumnIndex: ?, endColumnIndex: ?, majorColumnIndex?: ?) -> +Object","!doc":"<p>Merge cells from <em>startColumnIndex</em> to <em>endColumnIndex</em> at rows that make <em>rowTester</em> return true,\nusing the content of the cell at <em>majorColumnIndex</em></p>\n"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unmergeCells":{"!type":"fn(mergeHandler: ?)","!doc":"<p>Unmerge the cells that are merged by the <em>mergeHandler</em>, which represents a call to the function mergeCells.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/Cookie":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>This plugin provides a way to persist some grid features in cookie.\nDefault persistable features are:\ncolumn width:   &quot;columnWidth&quot; (handler name)\ncolumn order:   &quot;columnOrder&quot;\nsorting order:  &quot;sortOrder&quot;\n\n</p>\n<p>Grid users can define new persistable features\nby calling the following before grid is initialized (that is, during &quot;preInit&quot;);\n\n</p>\n<pre><code>grid.addCookieHandler({\n    name: &quot;a name for the new persistable feature&quot;,\n    onLoad: function(savedObject, grid){\n        //load the cookie.\n    },\n    onSave: function(grid){\n        //save the cookie.\n    }\n});</code></pre>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_cookieEnabled":{},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>Plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"addCookieHandler":{"!type":"fn(args: ?)","!doc":"<p>If a grid plugin wants cookie service, call this.\nThis must be called during preInit.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?, args: ?)"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"getCookieEnabled":{"!type":"fn(cookieName?: ?) -> ?","!doc":"<p>A getter to check cookie support of a particular Grid feature.</p>\n"},"init":{"!type":"fn()"},"removeCookie":{"!type":"fn()","!doc":"<p>Remove cookie for this grid.</p>\n"},"setCookieEnabled":{"!type":"fn(cookieName?: ?, enabled: ?)","!doc":"<p>A setter to enable|disable cookie support for a particular Grid feature.</p>\n"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/Dialog":{"!type":"fn()","prototype":{"!proto":"dijit/Dialog.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_firstFocusItem":{"!doc":"<p>The pointer to the first focusable node in the dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_focusManager":{},"_lastFocusItem":{"!doc":"<p>The pointer to which node has focus prior to our dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autofocus":{"!doc":"<p>A Toggle to modify the default focus behavior of a Dialog, which\nis to focus on the first dialog element after opening the dialog.\nFalse will disable autofocusing. Default: true</p>\n"},"baseClass":{},"class":{},"closable":{"!doc":"<p>Dialog show [x] icon to close itself, and ESC key will close the dialog.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"cssStateNodes":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"draggable":{"!doc":"<p>Toggles the movable aspect of the Dialog. If true, Dialog\ncan be dragged by it&#39;s title. If false it will remain centered\nin the viewport.</p>\n"},"duration":{"!doc":"<p>The time in milliseconds it takes the dialog to fade in and out</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"maxRatio":{"!doc":"<p>Maximum size to allow the dialog to expand to, relative to viewport size</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"open":{"!doc":"<p>True if Dialog is currently displayed on screen.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refNode":{},"refocus":{"!doc":"<p>A Toggle to modify the default focus behavior of a Dialog, which\nis to re-focus the element which had focus before being opened.\nFalse will disable refocusing. Default: true</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Will be &quot;Error&quot; if one or more of the child widgets has an invalid value,\n&quot;Incomplete&quot; if not all of the required child widgets are filled in.  Otherwise, &quot;&quot;,\nwhich indicates that the form is ready to be submitted.</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectChildren":{"!type":"fn(inStartup: ?)","!doc":"<p>You can call this function directly, ex. in the event that you\nprogrammatically add a widget to the form <em>after</em> the form has been\ninitialized.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"disconnectChildren":{"!type":"fn()","!doc":"<p>Deprecated method.   Applications no longer need to call this.   Remove for 2.0.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"execute":{"!type":"fn(formContents: ?)","!doc":"<p>Callback when the user hits the submit button.\nOverride this method to handle Dialog execution.</p>\n"},"focus":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValues":{"!type":"fn() -> ?"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn() -> +dojo","!doc":"<p>Hide the dialog</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns true if all of the widgets are valid.\nDeprecated, will be removed in 2.0.  Use get(&quot;state&quot;) instead.</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"setValues":{"!type":"fn(val: ?) -> ?"},"show":{"!type":"fn() -> +dojo","!doc":"<p>Display the dialog</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?","!doc":"<p>returns if the form is valid - same as isValid - but\nprovides a few additional (ui-specific) features:\n\n</p>\n<ol>\n<li>it will highlight any sub-widgets that are not valid</li>\n<li>it will call focus() on the first invalid sub-widget</li>\n</ol>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/grid/enhanced/plugins/DnD":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Provide drag and drop for grid columns/rows/cells within grid and out of grid.\nThe store of grid must implement dojo.data.api.Write.\n\n</p>\n<p>DnD selected columns:\nSupport moving within grid, moving/copying out of grid to a non-grid DnD target.\n\n</p>\n<p>DnD selected rows:\nSupport moving within grid, moving/copying out of grid to any DnD target.\n\n</p>\n<p>DnD selected cells (in rectangle shape only):\nSupport moving/copying within grid, moving/copying out of grid to any DnD target.</p>\n","_config":{},"_connects":{"!doc":"<p>List of all connections.</p>\n"},"_copyOnly":{},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"_targetAnchorBorderWidth":{},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>plugin name;</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?, args: ?)"},"copyOnly":{"!type":"fn(isCopyOnly: ?) -> bool","!doc":"<p>Setter/getter of this._copyOnly.</p>\n"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"init":{"!type":"fn()"},"setupConfig":{"!type":"fn(config: ?)","!doc":"<p>Configure which DnD functionalities are needed.\nCombination of any item from type set (&quot;row&quot;, &quot;col&quot;, &quot;cell&quot;)\nand any item from mode set(&quot;within&quot;, &quot;in&quot;, &quot;out&quot;) is configurable.\n\n</p>\n<p>&quot;row&quot;, &quot;col&quot;, &quot;cell&quot; are straightforward, while the other 3 are explained below:\n\n</p>\n<ul>\n<li>&quot;within&quot;: DnD within grid, that is, column/row reordering and cell moving/copying.</li>\n<li>&quot;in&quot;: Whether allowed to accept rows/cells (currently not support columns) from another grid.</li>\n<li>&quot;out&quot;: Whether allowed to drag out of grid, to another grid or even to any other DnD target.</li>\n</ul>\n<p>If not provided in the config, will use the default.\nWhen declared together, Mode set has higher priority than type set.</p>\n"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/Selector":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Provides standard extended selection for grid.\nSupports mouse/keyboard selection, multi-selection, and de-selection.\n\n</p>\n<p>Acceptable plugin parameters:\nThe whole plugin parameter object is a config object passed to the setupConfig function.\n\n</p>\n<p>Acceptable cell parameters defined in layout:\n\n</p>\n<ol>\n<li>notselectable: Boolean: Whether this column is (and all the cells in it are) selectable.</li>\n</ol>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"clear":{"!type":"fn(type?: ?)","!doc":"<p>Clear all selections.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?, args: ?)"},"deselect":{"!type":"fn(type: ?, rowIndex: ?, colIndex: ?)"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"getLastSelectedRange":{"!type":"fn(type: ?) -> ?","!doc":"<p>Get last selected range of the given type.</p>\n"},"getSelected":{"!type":"fn(type: ?, includeExceptions: ?) -> [+__SelectItem]","!doc":"<p>Get an array of selected locations.</p>\n"},"getSelectedCount":{"!type":"fn(type: ?, includeExceptions: ?) -> +Integer","!doc":"<p>Get the number of selected items.</p>\n"},"getSelectedType":{"!type":"fn() -> string","!doc":"<p>Get the type of selected items.</p>\n"},"init":{"!type":"fn()"},"isSelected":{"!type":"fn(type: ?, rowIndex: ?, colIndex?: ?) -> bool","!doc":"<p>Check whether a location (a cell, a column or a row) is selected.</p>\n"},"isSelecting":{"!type":"fn(type: ?) -> bool","!doc":"<p>Check whether the user is currently selecting something.</p>\n"},"select":{"!type":"fn(type: ?, rowIndex: ?, colIndex?: ?)","!doc":"<p>Select a location (a cell, a column or a row).</p>\n"},"selectEnabled":{"!type":"fn(toEnable?: ?) -> bool","!doc":"<p>Turn on/off this selection functionality if <em>toEnable</em> is provided.\nCheck whether this selection functionality is enabled if nothing is passed in.</p>\n"},"selectRange":{"!type":"fn(type: ?, start: ?, end: ?, toSelect: ?)","!doc":"<p>Select a continuous range (a block of cells, a set of continuous columns or rows)</p>\n"},"setupConfig":{"!type":"fn(config: ?)","!doc":"<p>Set selection mode for row/col/cell.</p>\n"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"toggleSelect":{"!type":"fn(type: ?, rowIndex: ?, colIndex: ?)"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/Rearrange":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Provides a set of method to re-arrange the structure of grid.</p>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"changeCells":{"!type":"fn(sourceGrid: ?, cellsToMove: ?, target: ?)"},"clearCells":{"!type":"fn(cellsToClear: ?)"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?, args: ?)"},"copyCells":{"!type":"fn(cellsToMove: ?, target: ?)"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"init":{"!type":"fn()"},"insertRows":{"!type":"fn(sourceGrid: ?, rowsToMove: ?, targetPos: ?)"},"moveCells":{"!type":"fn(cellsToMove: ?, target: ?)"},"moveColumns":{"!type":"fn(colsToMove: ?, targetPos: ?)","!doc":"<p>Move a set of columns to a given position.</p>\n"},"moveRows":{"!type":"fn(rowsToMove: ?, targetPos: ?)","!doc":"<p>Move a set of rows to a given position</p>\n"},"removeRows":{"!type":"fn(rowsToRemove: ?)"},"setArgs":{"!type":"fn(args: ?)"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/_RowMapLayer":{"!type":"fn()","prototype":{"__enabled":{"!doc":"<p>To control whether this layer is valid.</p>\n"},"_originFetch":{"!doc":"<p>The original fetch function of the store.</p>\n"},"_store":{"!doc":"<p>The wrapped store.</p>\n"},"layerFuncName":{},"tags":{},"clearMapping":{"!type":"fn()"},"constructor":{"!type":"fn(grid: ?)"},"enabled":{"!type":"fn(toEnable?: ?) -> bool","!doc":"<p>The get/set function of the enabled status of this layer</p>\n"},"initialize":{"!type":"fn(store: ?)"},"invalidate":{"!type":"fn()"},"name":{"!type":"fn() -> +any","!doc":"<p>Get the name of this store layer.\nThe default name retrieved from class name, which should have a pattern of &quot;{name}Layer&quot;.\nIf this pattern does not exist, the whole class name will be this layer&#39;s name.\nIt&#39;s better to override this method if your class name is too complicated.</p>\n"},"originFetch":{"!type":"fn() -> ?"},"setMapping":{"!type":"fn(mapping: ?)","!doc":"<p>Remember the row mapping.</p>\n"},"uninitialize":{"!type":"fn()"}}},"dojox/grid/enhanced/plugins/_StoreLayer":{"wrap":{"!type":"fn(store: ?, funcName: ?, layer: ?, layerFuncName: ?) -> +any","!doc":"<p>Wrap the store with the given layer.</p>\n"}},"dojox/grid/enhanced/plugins/_StoreLayer._StoreLayer":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class of store layers, provides basic utilities and some interfaces.</p>\n","__enabled":{"!doc":"<p>To control whether this layer is valid.</p>\n"},"_originFetch":{"!doc":"<p>The original fetch function of the store.</p>\n"},"_store":{"!doc":"<p>The wrapped store.</p>\n"},"layerFuncName":{},"tags":{},"constructor":{"!type":"fn()"},"enabled":{"!type":"fn(toEnable?: ?) -> bool","!doc":"<p>The get/set function of the enabled status of this layer</p>\n"},"initialize":{"!type":"fn(store: ?)"},"invalidate":{"!type":"fn()"},"name":{"!type":"fn() -> +any","!doc":"<p>Get the name of this store layer.\nThe default name retrieved from class name, which should have a pattern of &quot;{name}Layer&quot;.\nIf this pattern does not exist, the whole class name will be this layer&#39;s name.\nIt&#39;s better to override this method if your class name is too complicated.</p>\n"},"originFetch":{"!type":"fn() -> ?"},"uninitialize":{"!type":"fn(store: ?)"}}},"dojox/grid/enhanced/plugins/_StoreLayer._ServerSideLayer":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all server side store layers.</p>\n","__cmds":{},"__enabled":{"!doc":"<p>To control whether this layer is valid.</p>\n"},"_originFetch":{"!doc":"<p>The original fetch function of the store.</p>\n"},"_store":{"!doc":"<p>The wrapped store.</p>\n"},"_url":{"!doc":"<p>The url of the server</p>\n"},"layerFuncName":{},"tags":{},"command":{"!type":"fn(cmdName: ?, cmdContent?: ?) -> +any","!doc":"<p>get/set a command (a name-value pair)</p>\n"},"constructor":{"!type":"fn(args: ?)"},"enabled":{"!type":"fn(toEnable?: ?) -> ?","!doc":"<p>Overrided from _StoreLayer.enabled</p>\n"},"initialize":{"!type":"fn(store: ?)"},"invalidate":{"!type":"fn()"},"name":{"!type":"fn() -> +any","!doc":"<p>Get the name of this store layer.\nThe default name retrieved from class name, which should have a pattern of &quot;{name}Layer&quot;.\nIf this pattern does not exist, the whole class name will be this layer&#39;s name.\nIt&#39;s better to override this method if your class name is too complicated.</p>\n"},"originFetch":{"!type":"fn() -> ?"},"uninitialize":{"!type":"fn(store: ?)"},"useCommands":{"!type":"fn(toUse?: ?) -> bool","!doc":"<p>If you only want to modify the user request, instead of sending a separate command\nto server before fetch, just call:\n\n</p>\n<pre><code>this.useCommand(false);</code></pre>\n"}}},"dojox/grid/enhanced/plugins/Exporter":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Provide functions to export the grid data into a given format.\n\n</p>\n<p>Acceptable plugin parameters:\n\n</p>\n<ol>\n<li>exportFormatter: function(data, cell, rowIndex, item)\n Provide a way to customize how data should look in exported string.\n Note that usually the formatter of grid cell should not be used here (it can return HTML or even widget).</li>\n</ol>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>Plugin name.</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?, args: ?)","!doc":"<p>only newed by _Plugin</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all resources.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"exportGrid":{"!type":"fn(type: ?, args?: ?, onExported: ?)","!doc":"<p>Export required rows(fetchArgs) to a kind of format(type)\nusing the corresponding writer with given arguments(writerArgs),\nthen pass the exported text to a given function(onExported).</p>\n"},"exportSelected":{"!type":"fn(type: ?, writerArgs?: ?, onExported: ?) -> string","!doc":"<p>Only export selected rows.</p>\n"},"init":{"!type":"fn()"},"setExportFormatter":{"!type":"fn(formatter: ?)"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}},"writerNames":{},"registerWriter":{"!type":"fn(fileType: ?, writerClsName: ?)","!doc":"<p>Register a writer(writerClsName) to a export format type(fileType).\nThis function separates the Exporter from all kinds of writers.</p>\n"}},"dojox/grid/enhanced/plugins/Filter":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Provide filter functionality for grid.\n\n</p>\n<p>Acceptable plugin parameters:\n\n</p>\n<ol>\n<li>itemsName: string:\n the name shown on the filter bar.</li>\n<li>statusTipTimeout: number:\n when does the status tip show.</li>\n<li>ruleCount: number:\n default to 3, should not change to more. The Claro theme limits it.</li>\n<li>disabledConditions: object:\n If you don&#39;t need all of the conditions provided for a data type,\n you can explicitly declare them here:\n e.g.: disabledConditions: {string: [&quot;contains&quot;, &quot;is&quot;], number: [&quot;equalto&quot;], ...}</li>\n<li>isServerSide: boolean:\n Whether to use server side filtering. Default to false.</li>\n<li>isStateful: boolean:\n If isServerSide is true, set the server side filter to be stateful or not. default to false.</li>\n<li>url: string:\n If using stateful, this is the url to send commands. default to store.url.</li>\n<li>ruleCountToConfirmClearFilter: Integer | null |Infinity:\n If the filter rule count is larger than or equal to this value, then a confirm dialog will show when clearing filter.\n If set to less than 1 or null, then always show the confirm dialog.\n If set to Infinity, then never show the confirm dialog.\n Default value is 2.</li>\n</ol>\n<p>Acceptable cell parameters defined in layout:\n\n</p>\n<ol>\n<li>filterable: boolean:\n The column is not filterable only when this is set to false explicitly.</li>\n<li>datatype: string:\n The data type of this column. Can be &quot;string&quot;, &quot;number&quot;, &quot;date&quot;, &quot;time&quot;, &quot;boolean&quot;.\n Default to &quot;string&quot;.</li>\n<li>autoComplete: boolean:\n If need auto-complete in the ComboBox for String type, set this to true.</li>\n<li><p>dataTypeArgs: object:\n Some arguments helping convert store data to something the filter UI understands.\n Different data type arguments can be provided to different data types.\n For date/time, this is a dojo.date.locale.__FormatOptions, so the DataTimeBox can understand the store data.\n For boolean, this object contains:</p>\n<ul>\n<li>trueLabel: string:\n  A label to display in the filter definition dialog for true value. Default to &quot;True&quot;.</li>\n<li>falseLabel: string:\n  A label to display in the filter definition dialog for false value. Default to &quot;False&quot;.</li>\n</ul>\n</li>\n<li><p>disabledConditions: object:\n If you don&#39;t need all of the conditions provided by the filter UI on this column, you can explicitly say it out here.\n e.g.: disabledConditions: [&quot;contains&quot;, &quot;is&quot;]\n This will disable the &quot;contains&quot; condition for this column, if this column is of string type.\n For full set of conditions, please refer to dojox.grid.enhanced.plugins.filter.FilterDefDialog._setupData.</p>\n</li>\n</ol>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?, args: ?)","!doc":"<p>See constructor of dojox.grid.enhanced._Plugin.</p>\n"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"init":{"!type":"fn()"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/filter/FilterLayer":{"ClientSideFilterLayer":{"!type":"fn()"},"ServerSideFilterLayer":{"!type":"fn()"},"wrap":{"!type":"fn(store: ?, funcName: ?, layer: ?, layerFuncName: ?) -> +any","!doc":"<p>Wrap the store with the given layer.</p>\n"}},"dojox/grid/enhanced/plugins/filter/FilterLayer.ServerSideFilterLayer":{"!type":"fn()","prototype":{"__cmds":{},"__enabled":{"!doc":"<p>To control whether this layer is valid.</p>\n"},"_filter":{"!doc":"<p>The filter definition</p>\n"},"_originFetch":{"!doc":"<p>The original fetch function of the store.</p>\n"},"_store":{"!doc":"<p>The wrapped store.</p>\n"},"_url":{"!doc":"<p>The url of the server</p>\n"},"layerFuncName":{},"tags":{},"command":{"!type":"fn(cmdName: ?, cmdContent?: ?) -> +any","!doc":"<p>get/set a command (a name-value pair)</p>\n"},"constructor":{"!type":"fn(args: ?)"},"enabled":{"!type":"fn(toEnable?: ?) -> ?","!doc":"<p>Overrided from _StoreLayer.enabled</p>\n"},"filterDef":{"!type":"fn(filter?: ?) -> +null","!doc":"<p>See _FilterLayerMixin.filterDef</p>\n"},"initialize":{"!type":"fn(store: ?)"},"invalidate":{"!type":"fn()"},"name":{"!type":"fn() -> +any","!doc":"<p>Get the name of this store layer.\nThe default name retrieved from class name, which should have a pattern of &quot;{name}Layer&quot;.\nIf this pattern does not exist, the whole class name will be this layer&#39;s name.\nIt&#39;s better to override this method if your class name is too complicated.</p>\n"},"originFetch":{"!type":"fn() -> ?"},"uninitialize":{"!type":"fn(store: ?)"},"useCommands":{"!type":"fn(toUse?: ?) -> bool","!doc":"<p>If you only want to modify the user request, instead of sending a separate command\nto server before fetch, just call:\n\n</p>\n<pre><code>this.useCommand(false);</code></pre>\n"}}},"dojox/grid/enhanced/plugins/filter/FilterLayer.ClientSideFilterLayer":{"!type":"fn()","prototype":{"!doc":"<p>Add a client side filter layer on top of the data store,\nso any filter expression can be applied to the store.</p>\n","__enabled":{"!doc":"<p>To control whether this layer is valid.</p>\n"},"_fetchAll":{},"_filter":{"!doc":"<p>The filter definition</p>\n"},"_getter":{"!doc":"<p>A user defined way to get data from store</p>\n"},"_indexMap":{"!doc":"<p>A map from the row index of this._items to the row index of the original store.</p>\n"},"_items":{"!doc":"<p>Cached items (may contain holes)</p>\n"},"_nextUnfetchedIdx":{"!doc":"<p>The index of the next item in the store that is never fetched.</p>\n"},"_originFetch":{"!doc":"<p>The original fetch function of the store.</p>\n"},"_result":{"!doc":"<p>Current fetch result</p>\n"},"_resultStartIdx":{"!doc":"<p>The index in cache of the first result item</p>\n"},"_store":{"!doc":"<p>The wrapped store.</p>\n"},"_storeSize":{"!doc":"<p>The actual size of the original store</p>\n"},"layerFuncName":{},"tags":{},"constructor":{"!type":"fn(args: ?)"},"enabled":{"!type":"fn(toEnable?: ?) -> bool","!doc":"<p>The get/set function of the enabled status of this layer</p>\n"},"fetchAllOnFirstFilter":{"!type":"fn(toFetchAll?: ?) -> +any","!doc":"<p>The get/set function for fetchAll.</p>\n"},"filterDef":{"!type":"fn(filter?: ?) -> +(_ConditionExpr","!doc":"<p>See _FilterLayerMixin.filterDef</p>\n"},"initialize":{"!type":"fn(store: ?)"},"name":{"!type":"fn() -> +any","!doc":"<p>Get the name of this store layer.\nThe default name retrieved from class name, which should have a pattern of &quot;{name}Layer&quot;.\nIf this pattern does not exist, the whole class name will be this layer&#39;s name.\nIt&#39;s better to override this method if your class name is too complicated.</p>\n"},"originFetch":{"!type":"fn() -> ?"},"setGetter":{"!type":"fn(getter: ?)","!doc":"<p>Set the user defined way to retrieve data from store.</p>\n"},"uninitialize":{"!type":"fn(store: ?)"}}},"dojox/grid/enhanced/plugins/filter/FilterLayer._StoreLayer":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class of store layers, provides basic utilities and some interfaces.</p>\n","__enabled":{"!doc":"<p>To control whether this layer is valid.</p>\n"},"_originFetch":{"!doc":"<p>The original fetch function of the store.</p>\n"},"_store":{"!doc":"<p>The wrapped store.</p>\n"},"layerFuncName":{},"tags":{},"constructor":{"!type":"fn()"},"enabled":{"!type":"fn(toEnable?: ?) -> bool","!doc":"<p>The get/set function of the enabled status of this layer</p>\n"},"initialize":{"!type":"fn(store: ?)"},"invalidate":{"!type":"fn()"},"name":{"!type":"fn() -> +any","!doc":"<p>Get the name of this store layer.\nThe default name retrieved from class name, which should have a pattern of &quot;{name}Layer&quot;.\nIf this pattern does not exist, the whole class name will be this layer&#39;s name.\nIt&#39;s better to override this method if your class name is too complicated.</p>\n"},"originFetch":{"!type":"fn() -> ?"},"uninitialize":{"!type":"fn(store: ?)"}}},"dojox/grid/enhanced/plugins/filter/FilterLayer._ServerSideLayer":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all server side store layers.</p>\n","__cmds":{},"__enabled":{"!doc":"<p>To control whether this layer is valid.</p>\n"},"_originFetch":{"!doc":"<p>The original fetch function of the store.</p>\n"},"_store":{"!doc":"<p>The wrapped store.</p>\n"},"_url":{"!doc":"<p>The url of the server</p>\n"},"layerFuncName":{},"tags":{},"command":{"!type":"fn(cmdName: ?, cmdContent?: ?) -> +any","!doc":"<p>get/set a command (a name-value pair)</p>\n"},"constructor":{"!type":"fn(args: ?)"},"enabled":{"!type":"fn(toEnable?: ?) -> ?","!doc":"<p>Overrided from _StoreLayer.enabled</p>\n"},"initialize":{"!type":"fn(store: ?)"},"invalidate":{"!type":"fn()"},"name":{"!type":"fn() -> +any","!doc":"<p>Get the name of this store layer.\nThe default name retrieved from class name, which should have a pattern of &quot;{name}Layer&quot;.\nIf this pattern does not exist, the whole class name will be this layer&#39;s name.\nIt&#39;s better to override this method if your class name is too complicated.</p>\n"},"originFetch":{"!type":"fn() -> ?"},"uninitialize":{"!type":"fn(store: ?)"},"useCommands":{"!type":"fn(toUse?: ?) -> bool","!doc":"<p>If you only want to modify the user request, instead of sending a separate command\nto server before fetch, just call:\n\n</p>\n<pre><code>this.useCommand(false);</code></pre>\n"}}},"dojox/grid/enhanced/plugins/filter/FilterBar":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>The filter bar UI.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_curColIdx":{},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_handle_statusTooltip":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_timeout_statusTooltip":{},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"plugin":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColumnIdx":{"!type":"fn(coordX: ?) -> number"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFilterBarShown":{"!type":"fn() -> bool"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"showFilterBar":{"!type":"fn(toShow: ?, useAnim: ?, animArgs: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleClearFilterBtn":{"!type":"fn(toHide: ?)"},"toggleFilterBar":{"!type":"fn(useAnim: ?, animArgs: ?)"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn()"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/grid/enhanced/plugins/filter/FilterDefDialog":{"!type":"fn()","prototype":{"!doc":"<p>Create the filter definition UI.</p>\n","_relOpCls":{},"_savedCriterias":{},"curColIdx":{},"plugin":{},"addCriteriaBoxes":{"!type":"fn(cnt: ?)","!doc":"<p>Add <em>cnt</em> criteria boxes to the filter definition pane.\nCheck overflow if necessary.</p>\n"},"canFilter":{"!type":"fn() -> bool"},"clearFilter":{"!type":"fn(noRefresh: ?)","!doc":"<p>Clear filter definition.</p>\n"},"closeDialog":{"!type":"fn()","!doc":"<p>Close the filter definition dialog.</p>\n"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()"},"getColumnLabelByValue":{"!type":"fn(v: ?) -> ?"},"getColumnType":{"!type":"fn(colIndex: ?) -> ?"},"getConditionLabelByValue":{"!type":"fn(type: ?, c: ?) -> ?"},"getCriteria":{"!type":"fn(idx: ?) -> number","!doc":"<p>Get the <em>idx</em>-th criteria.</p>\n"},"getExprForColumn":{"!type":"fn(value: ?, colIdx: ?, type: ?, condition: ?) -> ?"},"getExprForCriteria":{"!type":"fn(rule: ?) -> ?"},"getFilter":{"!type":"fn() -> ?"},"removeCriteriaBoxes":{"!type":"fn(cnt: ?, isIdx?: ?)","!doc":"<p>Remove criteria boxes from the filter definition pane.</p>\n"},"setFilter":{"!type":"fn(rules: ?, ruleRelation: ?)"},"showDialog":{"!type":"fn(colIndex: ?)","!doc":"<p>Show the filter defintion dialog.</p>\n"}}},"dojox/grid/enhanced/plugins/filter/FilterBuilder":{"!type":"fn()","prototype":{"!doc":"<p>Create filter expression from a JSON object.</p>\n","defaultArgs":{},"supportedOps":{"!doc":"<p>The builders of all supported operations</p>\n"},"supportedTypes":{},"buildExpression":{"!type":"fn(def: ?) -> ?"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr":{"BooleanExpr":{"!type":"fn()"},"Contains":{"!type":"fn()"},"DateExpr":{"!type":"fn()"},"EndsWith":{"!type":"fn()"},"EqualTo":{"!type":"fn()"},"IsEmpty":{"!type":"fn()"},"LargerThan":{"!type":"fn()"},"LargerThanOrEqualTo":{"!type":"fn()"},"LessThan":{"!type":"fn()"},"LessThanOrEqualTo":{"!type":"fn()"},"LogicALL":{"!type":"fn()"},"LogicAND":{"!type":"fn()"},"LogicANY":{"!type":"fn()"},"LogicNOT":{"!type":"fn()"},"LogicOR":{"!type":"fn()"},"LogicXOR":{"!type":"fn()"},"Matches":{"!type":"fn()"},"NumberExpr":{"!type":"fn()"},"StartsWith":{"!type":"fn()"},"StringExpr":{"!type":"fn()"},"TimeExpr":{"!type":"fn()"}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LogicAND":{"!type":"fn()","prototype":{"!doc":"<p>A logic AND condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LogicOR":{"!type":"fn()","prototype":{"!doc":"<p>A logic OR condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LogicXOR":{"!type":"fn()","prototype":{"!doc":"<p>A logic XOR condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LogicNOT":{"!type":"fn()","prototype":{"!doc":"<p>A logic NOT condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;only one operand&quot; and confirm the operand is a valid _ConditionExpr.\nThen do the calculation of this operator.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LogicALL":{"!type":"fn()","prototype":{"!doc":"<p>A logic ALL condition expression, equals a sequence of logic ANDs</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> +instance","!doc":"<p>Override from _ConditionExpr</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LogicANY":{"!type":"fn()","prototype":{"!doc":"<p>A logic ANY condition expression, equals a sequence of logic ORs</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> +instance"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.EqualTo":{"!type":"fn()","prototype":{"!doc":"<p>An &quot;equal to&quot; condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LessThan":{"!type":"fn()","prototype":{"!doc":"<p>A &quot;less than&quot; condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LessThanOrEqualTo":{"!type":"fn()","prototype":{"!doc":"<p>A &quot;less than or equal to&quot; condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LargerThan":{"!type":"fn()","prototype":{"!doc":"<p>A &quot;larger than&quot; condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.LargerThanOrEqualTo":{"!type":"fn()","prototype":{"!doc":"<p>A &quot;larger than or equal to&quot; condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.Contains":{"!type":"fn()","prototype":{"!doc":"<p>A &quot;contains&quot; condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.StartsWith":{"!type":"fn()","prototype":{"!doc":"<p>A &quot;starts with&quot; condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.EndsWith":{"!type":"fn()","prototype":{"!doc":"<p>An &quot;ends with&quot; condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.Matches":{"!type":"fn()","prototype":{"!doc":"<p>A &quot;regular expression match&quot; condition expression.\nThe second operand&#39;s value will be regarded as an regular expression string.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.IsEmpty":{"!type":"fn()","prototype":{"!doc":"<p>Check empty</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;only one operand&quot; and confirm the operand is a valid _ConditionExpr.\nThen do the calculation of this operator.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.BooleanExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for boolean values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> string","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.StringExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for string values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> string","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.NumberExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for number values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> string","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.DateExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for date values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _DataExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr.TimeExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for time values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _DataExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr._ConditionExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all condition expressions.\nA condition expression can be applied on a data row (e.g. an item in a store)\nand generate a result condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> +any","!doc":"<p><em>Unimplemented Interface</em>\nApply this condition expression on the given datarow, return a result expression.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> +any","!doc":"<p>Convert this data expression to a simple object. Mainly used for serialization.</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr._DataExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all data expressions.\nA _DataExpr is a condition expression for a single data value.\nIf the data value to be represent is a pure value (literal value, like string/number/Date/...)\nthis _DataExpr is nothing more than a simple wrapper.\nIf the data value to be represent is in a store, then _DataExpr is responsible to extract it\nfrom the store when this condition is applied to a data row.</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> string","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr._OperatorExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all operator expressions.\nAn operator expression is a _ConditionExpr that represents an operation.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> +any","!doc":"<p><em>Unimplemented Interface</em>\nApply this condition expression on the given datarow, return a result expression.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr._UniOpExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all uni-operator expressions.\nA uni-operator expression is an _OperatorExpr that only allow one operand.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;only one operand&quot; and confirm the operand is a valid _ConditionExpr.\nThen do the calculation of this operator.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_FilterExpr._BiOpExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all bi-operator expressions.\nA bi-operator expression is an _OperatorExpr that allow and only allow two operands.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs":{"BooleanExpr":{"!type":"fn()"},"DateExpr":{"!type":"fn()"},"NumberExpr":{"!type":"fn()"},"StringExpr":{"!type":"fn()"},"TimeExpr":{"!type":"fn()"}},"dojox/grid/enhanced/plugins/filter/_DataExprs.BooleanExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for boolean values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> string","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs.StringExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for string values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> string","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs.NumberExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for number values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> string","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs.DateExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for date values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _DataExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs.TimeExpr":{"!type":"fn()","prototype":{"!doc":"<p>A condition expression wrapper for time values</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _DataExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs._ConditionExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all condition expressions.\nA condition expression can be applied on a data row (e.g. an item in a store)\nand generate a result condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> +any","!doc":"<p><em>Unimplemented Interface</em>\nApply this condition expression on the given datarow, return a result expression.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> +any","!doc":"<p>Convert this data expression to a simple object. Mainly used for serialization.</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs._DataExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all data expressions.\nA _DataExpr is a condition expression for a single data value.\nIf the data value to be represent is a pure value (literal value, like string/number/Date/...)\nthis _DataExpr is nothing more than a simple wrapper.\nIf the data value to be represent is in a store, then _DataExpr is responsible to extract it\nfrom the store when this condition is applied to a data row.</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> string","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs._OperatorExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all operator expressions.\nAn operator expression is a _ConditionExpr that represents an operation.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> +any","!doc":"<p><em>Unimplemented Interface</em>\nApply this condition expression on the given datarow, return a result expression.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs._UniOpExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all uni-operator expressions.\nA uni-operator expression is an _OperatorExpr that only allow one operand.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;only one operand&quot; and confirm the operand is a valid _ConditionExpr.\nThen do the calculation of this operator.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_DataExprs._BiOpExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all bi-operator expressions.\nA bi-operator expression is an _OperatorExpr that allow and only allow two operands.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_ConditionExpr":{},"dojox/grid/enhanced/plugins/filter/_ConditionExpr._ConditionExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all condition expressions.\nA condition expression can be applied on a data row (e.g. an item in a store)\nand generate a result condition expression.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> +any","!doc":"<p><em>Unimplemented Interface</em>\nApply this condition expression on the given datarow, return a result expression.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> +any","!doc":"<p>Convert this data expression to a simple object. Mainly used for serialization.</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_ConditionExpr._DataExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all data expressions.\nA _DataExpr is a condition expression for a single data value.\nIf the data value to be represent is a pure value (literal value, like string/number/Date/...)\nthis _DataExpr is nothing more than a simple wrapper.\nIf the data value to be represent is in a store, then _DataExpr is responsible to extract it\nfrom the store when this condition is applied to a data row.</p>\n","_colArg":{},"_name":{},"_value":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nIf this is a pure value, simply return self.\nOtherwise, extract the cell data from datarow using the given getter function,\nand then convert this cell data to a _DataExpr and return the expression.</p>\n"},"constructor":{"!type":"fn(dataValue: ?, isColumn?: ?, convertArgs: ?)","!doc":"<p>If a _DataExpr is constructed with only one argument, this argument is regarded as a pure value.\nIf the second argument is exactly a boolean true (no implict type transformation,\nso as to allow derived classes accept more arguments while retain <em>isColumn</em> to be optional),\nthen this _DataExpr represents a column, and it&#39;s applyRow method is not a no-op.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"getValue":{"!type":"fn() -> +any","!doc":"<p>If this is a pure value wrapper, simply return the value.\nOtherwise (it&#39;s a column), return is undefined.</p>\n"},"toObject":{"!type":"fn() -> string","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_ConditionExpr._OperatorExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all operator expressions.\nAn operator expression is a _ConditionExpr that represents an operation.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> +any","!doc":"<p><em>Unimplemented Interface</em>\nApply this condition expression on the given datarow, return a result expression.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_ConditionExpr._UniOpExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all uni-operator expressions.\nA uni-operator expression is an _OperatorExpr that only allow one operand.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;only one operand&quot; and confirm the operand is a valid _ConditionExpr.\nThen do the calculation of this operator.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/grid/enhanced/plugins/filter/_ConditionExpr._BiOpExpr":{"!type":"fn()","prototype":{"!doc":"<p>The most abstract class for all bi-operator expressions.\nA bi-operator expression is an _OperatorExpr that allow and only allow two operands.</p>\n","_name":{},"applyRow":{"!type":"fn(datarow: ?, getter: ?) -> ?","!doc":"<p>Implement _ConditionExpr.applyRow.\nApply the restriction of &quot;two operands&quot; and confirm operands are valid _ConditionExpr&#39;s.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>The arguments are operands (or an array of operands, if the first argument\nis an Array) of this operator, ordering from left to right.\nEvery operand should be a _ConditionExpr.</p>\n"},"getName":{"!type":"fn() -> +any","!doc":"<p>Get the name of this kind of expression.</p>\n"},"toObject":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ConditionExpr.toObject</p>\n"}}},"dojox/html/ellipsis":{"!doc":"<p>offers cross-browser support for text-overflow: ellipsis</p>\n"},"dojox/grid/enhanced/plugins/filter/FilterStatusTip":{"!type":"fn()","prototype":{"!doc":"<p>Create the status tip UI.</p>\n","closeDialog":{"!type":"fn()"},"constructor":{"!type":"fn(args: ?)"},"destroy":{"!type":"fn()"},"showDialog":{"!type":"fn(pos_x: ?, pos_y: ?, columnIdx: ?)"}}},"dojox/grid/enhanced/plugins/filter/ClearFilterConfirm":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>The UI for user to confirm the operation of clearing filter.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"plugin":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn()"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/grid/enhanced/plugins/GridSource":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Source.prototype","!doc":"<p>A special source that can accept grid contents.\nOnly for non-grid widgets or domNodes.</p>\n","accept":{},"allowNested":{"!doc":"<p>Indicates whether to allow dnd item nodes to be nested within other elements.\nBy default this is false, indicating that only direct children of the container can\nbe draggable dnd item nodes</p>\n"},"autoSync":{},"copyOnly":{},"current":{"!doc":"<p>The DOM node the mouse is currently hovered over</p>\n"},"delay":{},"generateText":{},"horizontal":{},"insertNodesForGrid":{"!doc":"<p>If you&#39;d like to insert some sort of nodes into your dnd source, turn this on,\nand override getCellContent/getRowContent/getColumnContent\nto populate the dnd data in your desired format.</p>\n"},"isSource":{},"map":{"!doc":"<p>Map from an item&#39;s id (which is also the DOMNode&#39;s id) to\nthe dojo/dnd/Container.Item itself.</p>\n"},"selection":{"!doc":"<p>The set of id&#39;s that are currently selected, such that this.selection[id] == 1\nif the node w/that id is selected.  Can iterate over selected node&#39;s id&#39;s like:\n\n</p>\n<pre><code>for(var id in this.selection)</code></pre>\n"},"selfAccept":{},"selfCopy":{},"singular":{},"skipForm":{},"withHandles":{},"checkAcceptance":{"!type":"fn(source: ?, nodes: ?) -> bool"},"clearItems":{"!type":"fn()","!doc":"<p>removes all data items from the map</p>\n"},"constructor":{"!type":"fn(node: ?, params?: ?)","!doc":"<p>a constructor of the Source</p>\n"},"copyState":{"!type":"fn(keyPressed: ?, self?: ?) -> bool","!doc":"<p>Returns true if we need to copy items, false to move.\nIt is separated to be overwritten dynamically, if needed.</p>\n"},"creator":{"!type":"fn()","!doc":"<p>creator function, dummy at the moment</p>\n"},"deleteSelectedNodes":{"!type":"fn() -> fn","!doc":"<p>deletes all selected items</p>\n"},"delItem":{"!type":"fn(key: ?)","!doc":"<p>removes a data item from the map by its key (id)</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>prepares the object to be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"forInItems":{"!type":"fn(f: ?, o?: ?) -> string","!doc":"<p>iterates over a data map skipping members that\nare present in the empty object (IE and/or 3rd-party libraries).</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getAllNodes":{"!type":"fn() -> ?","!doc":"<p>returns a list (an array) of all valid child nodes</p>\n"},"getCellContent":{"!type":"fn(grid: ?, leftTopCell: ?, rightBottomCell: ?)","!doc":"<p>Fill node innerHTML for dnd grid cells.</p>\n"},"getColumnContent":{"!type":"fn(grid: ?, colIndexes: ?)","!doc":"<p>Fill node innerHTML for dnd grid columns.</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>returns a data item by its key (id)</p>\n"},"getRowContent":{"!type":"fn(grid: ?, rowIndexes: ?)","!doc":"<p>Fill node innerHTML for dnd grid rows.</p>\n"},"getSelectedNodes":{"!type":"fn() -> +instance","!doc":"<p>returns a list (an array) of selected nodes</p>\n"},"insertNodes":{"!type":"fn(addSelected: ?, data: ?, before: ?, anchor: ?) -> fn","!doc":"<p>inserts new data items (see <code>dojo/dnd/Container.insertNodes()</code> method for details)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"selectAll":{"!type":"fn() -> ?","!doc":"<p>selects all items</p>\n"},"selectNone":{"!type":"fn() -> ?","!doc":"<p>unselects all items</p>\n"},"setItem":{"!type":"fn(key: ?, data: ?)","!doc":"<p>associates a data item with its key (id)</p>\n"},"startup":{"!type":"fn()","!doc":"<p>collects valid child items and populate the map</p>\n"},"sync":{"!type":"fn() -> fn","!doc":"<p>sync up the node list with the data map</p>\n"}}},"dojox/grid/enhanced/plugins/IndirectSelection":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>A handy way for adding check boxe/radio button for rows, and selecting rows by swiping(or keyboard)</p>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>Plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"addRowSelectCell":{"!type":"fn(option: ?)","!doc":"<p>Add indirectSelection cell(mapped to a column of radio button|check boxes)</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"init":{"!type":"fn()"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/Menu":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Provides context menu support, including header menu, row menu, cell menu and selected region menu</p>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>Plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"types":{"!doc":"<p>menu types</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all resources.\n_Grid.destroy() will unbind headerMenu</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"init":{"!type":"fn()"},"showMenu":{"!type":"fn(e: ?)","!doc":"<p>Show appropriate context menu\nFired from dojox.grid.enhanced._Events.onRowContextMenu, &#39;this&#39; scope - Grid</p>\n"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/NestedSorting":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Provides nested sorting feature\n</p>\n","_a11yText":{},"_connects":{"!doc":"<p>List of all connections.</p>\n"},"_currMainSort":{},"_currRegionIdx":{},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>Plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"clearSort":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"getSortProps":{"!type":"fn() -> +null","!doc":"<p>Overwritten, see DataGrid.getSortProps()</p>\n"},"init":{"!type":"fn()"},"initCookieHandler":{"!type":"fn()"},"isAsc":{"!type":"fn(cellIndex: ?) -> bool"},"isDesc":{"!type":"fn(cellIndex: ?) -> bool"},"removeSortData":{"!type":"fn(cellIdx: ?)"},"setSortData":{"!type":"fn(cellIdx: ?, attr: ?, value: ?)","!doc":"<p>Set sorting data for a column.</p>\n"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/Pagination":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>The typical pagination way to deal with huge dataset\nan alternative for the default virtual scrolling manner.</p>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_currentPage":{},"_currentPageSize":{},"_defaultRowsPerPage":{},"_maxSize":{},"_nls":{},"_showAll":{},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"defaultPage":{"!doc":"<p>Which page will be displayed initially, 1st page by default.</p>\n"},"defaultPageSize":{"!doc":"<p>Number of rows in a page, 25 by default.</p>\n"},"description":{"!doc":"<p>Whether the description information will be displayed, true by default.</p>\n"},"gotoButton":{"!doc":"<p>Whether the goto page button will be displayed, false by default.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"maxPageStep":{"!doc":"<p>The max number of page sizes to be displayed, 7 by default.</p>\n"},"name":{},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"pageSizes":{"!doc":"<p>Array of page sizes for switching, e.g. [10, 25, 50, 100, Infinity] by default,\nInfinity or any NaN value will be treated as &quot;all&quot;.</p>\n"},"pageStepper":{"!doc":"<p>Whether the page switch options will be displayed, true by default.</p>\n"},"position":{"!doc":"<p>The position of the pagination bar - &quot;top&quot;|&quot;bottom&quot;, &quot;bottom&quot; by default.</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"sizeSwitch":{"!doc":"<p>Whether the page size switch options will be displayed, true by default.</p>\n"},"changePageSize":{"!type":"fn(size: ?)"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(inGrid: ?, option: ?)"},"currentPage":{"!type":"fn(page: ?) -> +any","!doc":"<p>Shift to the given page, return current page number. If there \nis no valid page was passed in, just return current page num.</p>\n"},"currentPageSize":{"!type":"fn(size: ?) -> +any","!doc":"<p>Change the size of current page or return the current page size.</p>\n"},"destroy":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"firstPage":{"!type":"fn()","!doc":"<p>Go to the first page</p>\n"},"getTotalPageNum":{"!type":"fn() -> ?","!doc":"<p>Get total page number</p>\n"},"getTotalRowCount":{"!type":"fn() -> ?","!doc":"<p>Function for get total row count</p>\n"},"gotoFirstPage":{"!type":"fn()"},"gotoLastPage":{"!type":"fn()"},"gotoPage":{"!type":"fn(page: ?)"},"init":{"!type":"fn()"},"lastPage":{"!type":"fn()","!doc":"<p>Go to the last page</p>\n"},"nextPage":{"!type":"fn()","!doc":"<p>Go to the next page.</p>\n"},"prevPage":{"!type":"fn()","!doc":"<p>Go to the previous page.</p>\n"},"removeSelectedRows":{"!type":"fn()"},"scrollToRow":{"!type":"fn(inRowIndex: ?) -> ?","!doc":"<p>Override the grid.scrollToRow(), could jump to the right page\nand scroll to the specific row</p>\n"},"showGotoPageButton":{"!type":"fn(flag: ?)","!doc":"<p>For show/hide the go to page button dynamically</p>\n"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/Printer":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Provide printGrid function to the grid.</p>\n","__printArgs":{},"_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>Plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?)","!doc":"<p>only newed by _Plugin</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all resources.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"exportSelectedToHTML":{"!type":"fn(args?: ?, onExported: ?)","!doc":"<p>Export selected rows to HTML string, but do NOT print.\nUsers can use this to implement print preview.\nFor meaning of arguments see function <em>printGrid</em></p>\n"},"exportToHTML":{"!type":"fn(args?: ?, onExported: ?)","!doc":"<p>Export to HTML string, but do NOT print.\nUsers can use this to implement print preview.\nFor meaning of the 1st-3rd arguments see function <em>printGrid</em>.</p>\n"},"init":{"!type":"fn()"},"normalizeRowHeight":{"!type":"fn(doc: ?)"},"printGrid":{"!type":"fn(args?: ?)","!doc":"<p>Print all the data in the grid, using title as a title,\ndecorating generated html by cssFiles,\nusing tagName:&quot;attrbuteList&quot; pairs(writerArgs) to control html tags\nin the generated html string.</p>\n"},"printSelected":{"!type":"fn(args?: ?)","!doc":"<p>Print selected data. All other features are the same as printGrid.\nFor meaning of arguments see function <em>printGrid</em></p>\n"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/exporter/TableWriter":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/plugins/exporter/_ExportWriter.prototype","!doc":"<p>Export grid to HTML table format. Primarily used by Printer plugin.</p>\n","argObj":{},"afterContent":{"!type":"fn()","!doc":"<p>Overrided from _ExportWriter</p>\n"},"afterContentRow":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>After handling a line of data (not header).</p>\n"},"afterHeader":{"!type":"fn() -> +any","!doc":"<p>The header line has been handled.</p>\n"},"afterSubrow":{"!type":"fn(arg_obj: ?)","!doc":"<p>Overrided from _ExportWriter</p>\n"},"afterView":{"!type":"fn(arg_obj: ?)","!doc":"<p>Overrided from _ExportWriter</p>\n"},"beforeContent":{"!type":"fn(items: ?) -> +any","!doc":"<p>We are ready to go through all the contents(items).</p>\n"},"beforeContentRow":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>Before handling a line of data (not header).</p>\n"},"beforeHeader":{"!type":"fn(grid: ?) -> +any","!doc":"<p>We are going to start the travel in the grid.\nIs there anything we should do now?</p>\n"},"beforeSubrow":{"!type":"fn(arg_obj: ?) -> bool","!doc":"<p>Overrided from _ExportWriter</p>\n"},"beforeView":{"!type":"fn(arg_obj: ?) -> bool","!doc":"<p>Overrided from _ExportWriter</p>\n"},"constructor":{"!type":"fn(writerArgs?: ?)","!doc":"<p>The generated table only defines the col/rowspan, height and width of\nall the cells in the style attribute, no other attributes\n(like border, cellspacing, etc.) are used.\nUsers can define these attributes in the writerArgs object, like:\n{table:&quot;border=&#39;border&#39;&quot;,thead:&quot;cellspacing=&#39;3&#39;&quot;}</p>\n"},"handleCell":{"!type":"fn(arg_obj: ?)","!doc":"<p>Overrided from _ExportWriter</p>\n"},"toString":{"!type":"fn() -> ?","!doc":"<p>Overrided from _ExportWriter</p>\n"}}},"dojox/grid/enhanced/plugins/exporter/_ExportWriter":{"!type":"fn()","prototype":{"!doc":"<p>This is an abstract class for all kinds of writers used in the Exporter plugin.\nIt utilizes the strategy pattern to break the export work into several stages,\nand provide interfaces for all of them.\n\n</p>\n<p>Implementations might choose some of the functions in this class to override,\nthus providing their own functionalities.\n\n</p>\n<p>The Exporter will go through the grid line by line. So in every line, all the Views\nwill be reached, and the header line is only handled once.\n\n</p>\n<p>An <em>argObj</em> object is passed to most functions of this class.\nIt carries context arguments that make sense when they are called.</p>\n","argObj":{},"afterContent":{"!type":"fn() -> +any","!doc":"<p>We have finished the entire grid travel.\nDo some clean up work if you need to.</p>\n"},"afterContentRow":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>After handling a line of data (not header).</p>\n"},"afterHeader":{"!type":"fn() -> +any","!doc":"<p>The header line has been handled.</p>\n"},"afterSubrow":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>Before handling a subrow in a line (defined in the grid structure).</p>\n"},"afterView":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>After handling a view.</p>\n"},"beforeContent":{"!type":"fn(items: ?) -> +any","!doc":"<p>We are ready to go through all the contents(items).</p>\n"},"beforeContentRow":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>Before handling a line of data (not header).</p>\n"},"beforeHeader":{"!type":"fn(grid: ?) -> +any","!doc":"<p>We are going to start the travel in the grid.\nIs there anything we should do now?</p>\n"},"beforeSubrow":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>Before handling a subrow in a line (defined in the grid structure).</p>\n"},"beforeView":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>Before handling a view.</p>\n"},"constructor":{"!type":"fn(writerArgs?: ?)","!doc":"<p>Writer initializations goes here.</p>\n"},"handleCell":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>Handle a header cell or data cell.</p>\n"},"toString":{"!type":"fn() -> +any","!doc":"<p>Export to a string.</p>\n"}}},"dojox/grid/enhanced/plugins/Search":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/_Plugin.prototype","!doc":"<p>Search the grid using wildcard string or Regular Expression.</p>\n","_connects":{"!doc":"<p>List of all connections.</p>\n"},"_subscribes":{"!doc":"<p>List of all subscribes.</p>\n"},"grid":{"!doc":"<p>Grid that the plugin belongs to</p>\n"},"name":{"!doc":"<p>plugin name</p>\n"},"option":{"!doc":"<p>Plugin properties - leveraged with default and user specified properties.\ne.g. for dnd plugin, it may look like {&quot;class&quot;: dojox.grid.enhanced.plugins.DnD, &quot;dependency&quot;: [&quot;nestedSorting&quot;], ...}</p>\n"},"privates":{"!doc":"<p>Private properties/methods shouldn&#39;t be mixin-ed anytime.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> ?","!doc":"<p>Connects specified obj/event to specified method of this object.</p>\n"},"constructor":{"!type":"fn(grid: ?, args: ?)"},"destroy":{"!type":"fn()","!doc":"<p>Destroy all resources.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Disconnects handle and removes it from connection list.</p>\n"},"init":{"!type":"fn()"},"searchRow":{"!type":"fn(searchArgs: ?, onSearched: ?)"},"subscribe":{"!type":"fn(topic: ?, method: ?) -> ?","!doc":"<p>Subscribes to the specified topic and calls the specified method\nof this object.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Un-subscribes handle and removes it from subscriptions list.</p>\n"}}},"dojox/grid/enhanced/plugins/exporter/CSVWriter":{"!type":"fn()","prototype":{"!proto":"dojox/grid/enhanced/plugins/exporter/_ExportWriter.prototype","!doc":"<p>Export grid to CSV format.</p>\n","_newline":{},"_separator":{},"argObj":{},"afterContent":{"!type":"fn() -> +any","!doc":"<p>We have finished the entire grid travel.\nDo some clean up work if you need to.</p>\n"},"afterContentRow":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>After handling a line of data (not header).</p>\n"},"afterHeader":{"!type":"fn() -> +any","!doc":"<p>The header line has been handled.</p>\n"},"afterSubrow":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>Before handling a subrow in a line (defined in the grid structure).</p>\n"},"afterView":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>After handling a view.</p>\n"},"beforeContent":{"!type":"fn(items: ?) -> +any","!doc":"<p>We are ready to go through all the contents(items).</p>\n"},"beforeContentRow":{"!type":"fn(arg_obj: ?) -> bool","!doc":"<p>Overrided from _ExportWriter</p>\n"},"beforeHeader":{"!type":"fn(grid: ?) -> +any","!doc":"<p>We are going to start the travel in the grid.\nIs there anything we should do now?</p>\n"},"beforeSubrow":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>Before handling a subrow in a line (defined in the grid structure).</p>\n"},"beforeView":{"!type":"fn(argObj: ?) -> +any","!doc":"<p>Before handling a view.</p>\n"},"constructor":{"!type":"fn(writerArgs?: ?)","!doc":"<p>CSV default separator is &#39;,&#39;.\nBut we can also use our own.</p>\n"},"handleCell":{"!type":"fn(arg_obj: ?)","!doc":"<p>Overrided from _ExportWriter</p>\n"},"toString":{"!type":"fn() -> number","!doc":"<p>Overrided from _ExportWriter</p>\n"}}},"dojox/help/console":{},"dojox/help/_base":{},"dojox/highlight/_base":{"!doc":"<p>Syntax highlighting with language auto-detection package</p>\n","constants":{},"languages":{},"Code":{"!type":"fn(props?: ?, node: ?)","!doc":"<p>A class object to allow for dojoType usage with the highlight engine. This is\nNOT a Widget in the conventional sense, and does not have any member functions for\nthe instance. This is provided as a convenience. You likely should be calling\n<code>dojox.highlight.init</code> directly.</p>\n"}},"dojox/highlight/_base.constants":{"APOS_STRING_MODE":{},"BACKSLASH_ESCAPE":{},"C_BLOCK_COMMENT_MODE":{},"C_LINE_COMMENT_MODE":{},"C_NUMBER_MODE":{},"C_NUMBER_RE":{},"HASH_COMMENT_MODE":{},"IDENT_RE":{},"NUMBER_RE":{},"QUOTE_STRING_MODE":{},"UNDERSCORE_IDENT_RE":{}},"dojox/highlight/languages/_all":{},"dojox/highlight/languages/_static":{},"dojox/highlight/languages/cpp":{"!doc":"<p>C++ highlight definitions</p>\n","defaultMode":{},"modes":{}},"dojox/highlight/languages/cpp.defaultMode":{"contains":{},"illegal":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/java":{"!doc":"<p>Java highlight definitions</p>\n","defaultMode":{},"modes":{}},"dojox/highlight/languages/java.defaultMode":{"contains":{},"illegal":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/delphi":{"!doc":"<p>Delphi highlight definitions</p>\n","case_insensitive":{},"defaultMode":{},"modes":{}},"dojox/highlight/languages/delphi.defaultMode":{"contains":{},"illegal":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/_dynamic":{},"dojox/highlight/languages/python":{"!doc":"<p>Python highlight definitions</p>\n","defaultMode":{},"modes":{}},"dojox/highlight/languages/python.defaultMode":{"contains":{},"illegal":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/xquery":{"case_insensitive":{},"defaultMode":{},"modes":{},"XQUERY_COMMENT":{}},"dojox/highlight/languages/xquery.defaultMode":{"contains":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/xquery.XQUERY_COMMENT":{"begin":{},"className":{},"end":{}},"dojox/highlight/languages/groovy":{"!doc":"<p>Groovy highlight definitions</p>\n","defaultMode":{},"GROOVY_KEYWORDS":{},"modes":{}},"dojox/highlight/languages/groovy.defaultMode":{"contains":{},"illegal":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/groovy.GROOVY_KEYWORDS":{"abstract":{},"assert":{},"boolean":{},"break":{},"byte":{},"case":{},"catch":{},"char":{},"class":{},"const":{},"continue":{},"def":{},"do":{},"double":{},"else":{},"extends":{},"false":{},"final":{},"finally":{},"float":{},"for":{},"goto":{},"if":{},"implements":{},"import":{},"instanceof":{},"int":{},"interface":{},"long":{},"native":{},"new":{},"package":{},"private":{},"protected":{},"public":{},"return":{},"short":{},"static":{},"strictfp":{},"super":{},"switch":{},"synchronized":{},"this":{},"throw":{},"throws":{},"transient":{},"true":{},"try":{},"void":{},"volatile":{},"while":{}},"dojox/highlight/languages/_www":{},"dojox/highlight/languages/xml":{"case_insensitive":{},"defaultMode":{},"modes":{},"XML_ATTR":{},"XML_COMMENT":{},"XML_VALUE":{}},"dojox/highlight/languages/xml.defaultMode":{"contains":{}},"dojox/highlight/languages/xml.XML_COMMENT":{"begin":{},"className":{},"end":{}},"dojox/highlight/languages/xml.XML_ATTR":{"begin":{},"className":{},"contains":{},"end":{}},"dojox/highlight/languages/xml.XML_VALUE":{"begin":{},"className":{},"end":{}},"dojox/highlight/languages/html":{"case_insensitive":{},"defaultMode":{},"HTML_ATTR":{},"HTML_DOCTYPE":{},"HTML_TAGS":{},"HTML_VALUE":{},"modes":{}},"dojox/highlight/languages/html.defaultMode":{"contains":{}},"dojox/highlight/languages/html.HTML_TAGS":{"a":{},"abbr":{},"acronym":{},"address":{},"applet":{},"area":{},"b":{},"base":{},"basefont":{},"bdo":{},"big":{},"blockquote":{},"body":{},"br":{},"button":{},"caption":{},"center":{},"cite":{},"code":{},"col":{},"colgroup":{},"dd":{},"del":{},"dfn":{},"dir":{},"div":{},"dl":{},"dt":{},"em":{},"fieldset":{},"font":{},"form":{},"frame":{},"frameset":{},"h1":{},"h2":{},"h3":{},"h4":{},"h5":{},"h6":{},"head":{},"hr":{},"html":{},"i":{},"iframe":{},"img":{},"input":{},"ins":{},"isindex":{},"kbd":{},"label":{},"legend":{},"li":{},"link":{},"map":{},"menu":{},"meta":{},"noframes":{},"noscript":{},"object":{},"ol":{},"optgroup":{},"option":{},"p":{},"param":{},"pre":{},"q":{},"s":{},"samp":{},"script":{},"select":{},"small":{},"span":{},"strike":{},"strong":{},"style":{},"sub":{},"sup":{},"table":{},"tbody":{},"td":{},"textarea":{},"tfoot":{},"th":{},"thead":{},"title":{},"tr":{},"tt":{},"u":{},"ul":{},"var":{},"xml":{},"xmlns":{}},"dojox/highlight/languages/html.HTML_DOCTYPE":{"begin":{},"className":{},"end":{},"relevance":{}},"dojox/highlight/languages/html.HTML_ATTR":{"begin":{},"className":{},"end":{}},"dojox/highlight/languages/html.HTML_VALUE":{"begin":{},"className":{},"end":{}},"dojox/highlight/languages/css":{"!doc":"<p>CSS Language definition file.</p>\n","case_insensitive":{},"defaultMode":{},"modes":{}},"dojox/highlight/languages/css.defaultMode":{"contains":{},"illegal":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/django":{"case_insensitive":{},"defaultMode":{},"modes":{}},"dojox/highlight/languages/django.defaultMode":{"contains":{}},"dojox/highlight/languages/javascript":{"defaultMode":{},"modes":{}},"dojox/highlight/languages/javascript.defaultMode":{"contains":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/pygments/_html":{},"dojox/highlight/languages/pygments/_www":{},"dojox/highlight/languages/pygments/xml":{"defaultMode":{},"modes":{}},"dojox/highlight/languages/pygments/xml.defaultMode":{"contains":{}},"dojox/highlight/languages/pygments/html":{"case_insensitive":{},"defaultMode":{},"modes":{}},"dojox/highlight/languages/pygments/html.defaultMode":{"contains":{}},"dojox/highlight/languages/pygments/css":{"defaultMode":{},"modes":{}},"dojox/highlight/languages/pygments/css.defaultMode":{"contains":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/pygments/javascript":{"defaultMode":{},"modes":{}},"dojox/highlight/languages/pygments/javascript.defaultMode":{"contains":{},"keywords":{},"lexems":{}},"dojox/highlight/languages/sql":{"!doc":"<p>SQL highlight definitions</p>\n","case_insensitive":{},"defaultMode":{},"modes":{}},"dojox/highlight/languages/sql.defaultMode":{"contains":{},"keywords":{},"lexems":{}},"dojox/highlight/widget/Code":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A simple source code formatting widget that adds line numbering, alternating line colors\nand line range support on top of dojox.highlight module.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{},"listType":{},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"range":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"url":{},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setRange":{"!type":"fn(range: ?)","!doc":"<p>update the view to a new passed range</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/highlight":{"!doc":"<p>Deprecated.  Should require dojox/highlight modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/html":{"!doc":"<p>Deprecated.  Should require dojox/html modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/image/Badge":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A simple grid of Images that loops through thumbnails</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"cellMargin":{"!doc":"<p>Size in PX to adjust for cell margins</p>\n"},"cellSize":{"!doc":"<p>Size in PX of each thumbnail</p>\n"},"children":{"!doc":"<p>A CSS3 Selector that determines the node to become a child</p>\n"},"class":{},"cols":{"!doc":"<p>Number of Columns to display</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"delay":{"!doc":"<p>Time (in ms) to show the image before sizing down again</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"easing":{"!doc":"<p>An easing function to use when showing the node (does not apply to shrinking)</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"rows":{"!doc":"<p>Number of Rows to display</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"threads":{"!doc":"<p>how many cycles will be going &quot;simultaneously&quot; (&gt;2 not reccommended)</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/image/FlickrBadge":{"!type":"fn()","prototype":{"_store":{},"apikey":{},"children":{},"searchText":{"!doc":"<p>Free text search.  Photos who&#39;s title, description, or tags contain the text will be displayed</p>\n"},"setid":{"!doc":"<p>The id of the set to display</p>\n"},"tags":{"!doc":"<p>A comma separated list of tags or an array of tags to grab from Flickr</p>\n"},"target":{"!doc":"<p>Where to display the pictures when clicked on.  Valid values are the same as the target attribute\nof the A tag.</p>\n"},"userid":{"!doc":"<p>If you know your Flickr userid, you can set it to prevent a call to fetch the id</p>\n"},"username":{"!doc":"<p>Your Flickr username</p>\n"},"postCreate":{"!type":"fn()"},"startup":{"!type":"fn()"}}},"dojox/image/Lightbox":{"!type":"fn()","prototype":{"!doc":"<p>A dojo-based Lightbox implementation.</p>\n","_allowPassthru":{"!doc":"<p>Privately set this to disable/enable natural link of anchor tags</p>\n"},"_attachedDialog":{"!doc":"<p>The pointer to the global lightbox dialog for this widget</p>\n"},"duration":{"!doc":"<p>Generic time in MS to adjust the feel of widget. could possibly add various\ndurations for the various actions (dialog fadein, sizeing, img fadein ...)</p>\n"},"group":{"!doc":"<p>Grouping images in a page with similar tags will provide a &#39;slideshow&#39; like grouping of images</p>\n"},"href":{},"modal":{"!doc":"<p>If true, this Dialog instance will be truly modal and prevent closing until\nexplicitly told to by calling hide() or clicking the (x) - Defaults to false\nto preserve previous behaviors. (aka: enable click-to-close on the underlay)</p>\n"},"title":{"!doc":"<p>A string of text to be shown in the Lightbox beneath the image (empty if using a store)</p>\n"},"destroy":{"!type":"fn()"},"disable":{"!type":"fn()","!doc":"<p>Disables event clobbering and dialog, and follows natural link</p>\n"},"enable":{"!type":"fn()","!doc":"<p>Enables the dialog (prevents default link)</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hide the Lightbox currently showing</p>\n"},"show":{"!type":"fn()","!doc":"<p>Show the Lightbox with this instance as the starting point</p>\n"},"startup":{"!type":"fn()"}},"LightboxDialog":{"!type":"fn()"}},"dojox/image/Lightbox.LightboxDialog":{"!type":"fn()","prototype":{"!doc":"<p>The &quot;dialog&quot; shared  between any Lightbox instances on the page, publically available\nfor programmatic manipulation.</p>\n","_groups":{"!doc":"<p>an object of arrays, each array (of objects) being a unique &#39;group&#39;</p>\n"},"adjust":{"!doc":"<p>If true, ensure the image always stays within the viewport\nmore difficult than necessary to disable, but enabled by default\nseems sane in most use cases.</p>\n"},"errorImg":{"!doc":"<p>Path to the image used when a 404 is encountered</p>\n"},"errorMessage":{"!doc":"<p>The text to display when an unreachable image is linked</p>\n"},"imageClass":{"!doc":"<p>The classname to apply to the image node in the dialog (for extra styling)</p>\n"},"imgUrl":{"!doc":"<p>The src=&quot;&quot; attribute of our imageNode (can be null at statup)</p>\n"},"inGroup":{"!doc":"<p>Array of objects. this is populated by from the JSON object _groups, and\nshould not be populate manually. it is a placeholder for the currently\nshowing group of images in this master dialog</p>\n"},"modal":{"!doc":"<p>If true, this Dialog instance will be truly modal and prevent closing until\nexplicitly told to by calling hide() or clicking the (x) - Defaults to false\nto preserve previous behaviors. (aka: enable click-to-close on the underlay)</p>\n"},"templateString":{},"title":{"!doc":"<p>The current title, read from object passed to show()</p>\n"},"addImage":{"!type":"fn(child: ?, group?: ?)","!doc":"<p>Add an image to this Master Lightbox</p>\n"},"constructor":{"!type":"fn(args: ?)"},"hide":{"!type":"fn()","!doc":"<p>Hide the Master Lightbox</p>\n"},"removeGroup":{"!type":"fn(group: ?)","!doc":"<p>Remove all images in a passed group</p>\n"},"removeImage":{"!type":"fn(child: ?)","!doc":"<p>Remove an image instance from this LightboxDialog.</p>\n"},"resizeTo":{"!type":"fn(size: ?, forceTitle: ?)","!doc":"<p>Resize our dialog container, and fire _showImage</p>\n"},"show":{"!type":"fn(groupData: ?)","!doc":"<p>Show the Master Dialog. Starts the chain of events to show\nan image in the dialog, including showing the dialog if it is\nnot already visible</p>\n"},"startup":{"!type":"fn() -> +dijit._Widget","!doc":"<p>Add some extra event handlers, and startup our superclass.</p>\n"}}},"dojox/image/LightboxNano":{"!type":"fn()","prototype":{"!doc":"<p>A simple &quot;nano&quot; version of the lightbox.</p>\n","duration":{"!doc":"<p>The delay in milliseconds of the LightboxNano open and close animation.</p>\n"},"href":{"!doc":"<p>URL to the large image to show in the lightbox.</p>\n"},"preloadDelay":{"!doc":"<p>The delay in milliseconds after the LightboxNano is created before preloading the larger image.</p>\n"},"constructor":{"!type":"fn(p?: ?, n?: ?)","!doc":"<p>Initializes the DOM node and connect onload event</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys the LightboxNano and it&#39;s DOM node</p>\n"},"show":{"!type":"fn(args?: ?)","!doc":"<p>Shows this LightboxNano programatically. Allows passing a new href and\na programmatic origin.</p>\n"}}},"dojox/image/Magnifier":{"!type":"fn()","prototype":{"!proto":"dojox/image/MagnifierLite.prototype","!doc":"<p>Adds magnification on a portion of an image element, using <code>dojox.gfx</code>\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"glassSize":{"!doc":"<p>the width and height of the bounding box</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scale":{"!doc":"<p>the multiplier of the Mangification.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(finalize: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/image/MagnifierLite":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Adds magnification on a portion of an image element</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"glassSize":{"!doc":"<p>the width and height of the bounding box</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scale":{"!doc":"<p>the multiplier of the Mangification.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(finalize: ?)"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/image/_base":{},"dojox/image":{"!doc":"<p>Deprecated.  Should require dojox/image modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/io/OAuth":{"!doc":"<p>Helper singleton for signing any kind of Ajax request using the OAuth 1.0 protocol.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Helper singleton for signing any kind of Ajax request using the OAuth 1.0 protocol.</p>\n"}}},"dojox/io/httpParse":{"!doc":"<p>Parses an HTTP stream for a message.</p>\n","prototype":{"constructor":{"!type":"fn(httpStream: ?, topHeaders?: ?, partial?: ?) -> +any","!doc":"<p>Parses an HTTP stream for a message.</p>\n"}}},"dojox/io/proxy/xip":{"!doc":"<p>Object that implements the iframe handling for XMLHttpRequest\nIFrame Proxying.\n\n</p>\n<p>Do not use this object directly. See the Dojo Book page\non XMLHttpRequest IFrame Proxying:\n<a href=\"http://dojotoolkit.org/book/dojo-book-0-4/part-5-connecting-pieces/i-o/cross-domain-xmlhttprequest-using-iframe-proxy\">http://dojotoolkit.org/book/dojo-book-0-4/part-5-connecting-pieces/i-o/cross-domain-xmlhttprequest-using-iframe-proxy</a>\nUsage of XHR IFrame Proxying does not work from local disk in Safari.</p>\n","_callbackName":{},"_isWebKit":{},"_state":{},"_stateIdCounter":{},"urlLimit":{},"xipClientUrl":{},"createFacade":{"!type":"fn() -> +instance"},"destroyState":{"!type":"fn(stateId: ?)"},"fragmentReceived":{"!type":"fn(frag: ?)"},"fragmentReceivedEvent":{"!type":"fn(evt: ?)","!doc":"<p>HTML5 document messaging endpoint. Unpack the event to see if we want to use it.</p>\n"},"frameLoaded":{"!type":"fn(stateId: ?)"},"makeServerUrl":{"!type":"fn(stateId: ?, cmd: ?, message: ?) -> string"},"receive":{"!type":"fn(stateId: ?, urlEncodedData: ?)"},"send":{"!type":"fn(facade: ?) -> +null","!doc":"<p>starts the xdomain request using the provided facade.\nThis method first does some init work, then delegates to _realSend.</p>\n"},"sendRequest":{"!type":"fn(stateId: ?, encodedData: ?)"},"sendRequestPart":{"!type":"fn(stateId: ?)"},"sendRequestStart":{"!type":"fn(stateId: ?)"},"setServerUrl":{"!type":"fn(stateId: ?, cmd: ?, message: ?)"},"unpackMessage":{"!type":"fn(encodedMessage: ?) -> ?"},"XhrIframeFacade":{"!type":"fn(ifpServerUrl: ?)","!doc":"<p>XMLHttpRequest facade object used by dojox.io.proxy.xip.\n\n</p>\n<p>Do not use this object directly. See the Dojo Book page\non XMLHttpRequest IFrame Proxying:\n<a href=\"http://dojotoolkit.org/book/dojo-book-0-4/part-5-connecting-pieces/i-o/cross-domain-xmlhttprequest-using-iframe-proxy\">http://dojotoolkit.org/book/dojo-book-0-4/part-5-connecting-pieces/i-o/cross-domain-xmlhttprequest-using-iframe-proxy</a></p>\n"}},"dojox/io/proxy/xip._state":{},"dojox/io/scriptFrame":{"_loadedIds":{},"_waiters":{}},"dojox/io/scriptFrame._waiters":{},"dojox/io/scriptFrame._loadedIds":{},"dojox/io/windowName":{"_frameNum":{},"send":{"!type":"fn(method: ?, args: ?) -> ?","!doc":"<p>Provides secure cross-domain request capability.\nSends a request using an iframe (POST or GET) and reads the response through the\nframe&#39;s window.name.\n</p>\n"}},"dojox/io/xhrMultiPart":{"prototype":{"constructor":{"!type":"fn(args: ?)"}}},"dojox/uuid/generateRandomUuid":{"!doc":"<p>This function generates random UUIDs, meaning &quot;version 4&quot; UUIDs.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>This function generates random UUIDs, meaning &quot;version 4&quot; UUIDs.</p>\n"}}},"dojox/uuid/_base":{"NIL_UUID":{},"variant":{},"version":{},"assert":{"!type":"fn(booleanValue: ?, message?: ?)","!doc":"<p>Throws an exception if the assertion fails.</p>\n"},"generateNilUuid":{"!type":"fn()","!doc":"<p>This function returns the Nil UUID: &quot;00000000-0000-0000-0000-000000000000&quot;.</p>\n"},"generateRandomUuid":{"!type":"fn()","!doc":"<p>This function generates random UUIDs, meaning &quot;version 4&quot; UUIDs.</p>\n"},"generateTimeBasedUuid":{"!type":"fn(node?: ?)","!doc":"<p>This function generates time-based UUIDs, meaning &quot;version 1&quot; UUIDs.</p>\n"},"getNode":{"!type":"fn(uuidString: ?)","!doc":"<p>If this is a version 1 UUID (a time-based UUID), getNode() returns a\n12-character string with the &quot;node&quot; or &quot;pseudonode&quot; portion of the UUID,\nwhich is the rightmost 12 characters.</p>\n"},"getTimestamp":{"!type":"fn(uuidString: ?, returnType?: ?) -> +any","!doc":"<p>If this is a version 1 UUID (a time-based UUID), this method returns\nthe timestamp value encoded in the UUID.  The caller can ask for the\ntimestamp to be returned either as a JavaScript Date object or as a\n15-character string of hex digits.</p>\n"},"getVariant":{"!type":"fn(uuidString: ?)","!doc":"<p>Returns a variant code that indicates what type of UUID this is.\nReturns one of the enumerated dojox.uuid.variant values.</p>\n"},"getVersion":{"!type":"fn(uuidString: ?)","!doc":"<p>Returns a version number that indicates what type of UUID this is.\nReturns one of the enumerated dojox.uuid.version values.</p>\n"},"isValid":{"!type":"fn(uuidString: ?)","!doc":"<p>Returns true if the UUID was initialized with a valid value.</p>\n"},"Uuid":{"!type":"fn(input?: ?)","!doc":"<p>This is the constructor for the Uuid class.  The Uuid class offers\nmethods for inspecting existing UUIDs.</p>\n"}},"dojox/uuid/_base.version":{"DCE_SECURITY":{},"NAME_BASED_MD5":{},"NAME_BASED_SHA1":{},"RANDOM":{},"TIME_BASED":{},"UNKNOWN":{}},"dojox/uuid/_base.variant":{"DCE":{},"MICROSOFT":{},"NCS":{},"UNKNOWN":{}},"dojox/io/xhrPlugins":{"plainXhr":{},"addCrossSiteXhr":{"!type":"fn(url: ?, httpAdapter: ?)","!doc":"<p>Adds W3C Cross site XHR or XDomainRequest handling for the given URL prefix\n</p>\n"},"addProxy":{"!type":"fn(proxyUrl: ?)","!doc":"<p>adds a server side proxy xhr handler for cross-site URLs</p>\n"},"fullHttpAdapter":{"!type":"fn(plainXhr: ?, noRawBody: ?)","!doc":"<p>Provides a HTTP adaption.</p>\n"},"register":{"!type":"fn()","!doc":"<p>overrides the default xhr handler to implement a registry of\nxhr handlers</p>\n"}},"dojox/io/xhrScriptPlugin":{"!doc":"<p>Adds the script transport (JSONP) as an XHR plugin for the given site. See\ndojox.io.script for more information on the transport. Note, that JSONP\nis <em>not</em> a secure transport, by loading data from a third-party site using JSONP\nthe site has full access to your JavaScript environment.</p>\n","prototype":{"constructor":{"!type":"fn(url: ?, callbackParamName: ?, httpAdapter?: ?)","!doc":"<p>Adds the script transport (JSONP) as an XHR plugin for the given site. See\ndojox.io.script for more information on the transport. Note, that JSONP\nis <em>not</em> a secure transport, by loading data from a third-party site using JSONP\nthe site has full access to your JavaScript environment.</p>\n"}}},"dojox/io/xhrWindowNamePlugin":{"!doc":"<p>Adds the windowName transport as an XHR plugin for the given site. See\ndojox.io.windowName for more information on the transport.</p>\n","prototype":{"constructor":{"!type":"fn(url: ?, httpAdapter?: ?, trusted?: ?)","!doc":"<p>Adds the windowName transport as an XHR plugin for the given site. See\ndojox.io.windowName for more information on the transport.</p>\n"}}},"dojox/secure/capability":{"keywords":{},"validate":{"!type":"fn(script: ?, safeLibraries: ?, safeGlobals: ?)","!doc":"<p>pass in the text of a script. If it passes and it can be eval&#39;ed, it should be safe.\nNote that this does not do full syntax checking, it relies on eval to reject invalid scripts.\nThere are also known false rejections:\n\n</p>\n<ul>\n<li>Nesting vars inside blocks will not declare the variable for the outer block</li>\n<li>Named functions are not treated as declaration so they are generally not allowed unless the name is declared with a var.</li>\n<li>Var declaration that involve multiple comma delimited variable assignments are not accepted</li>\n</ul>\n"}},"dojox/jq":{},"dojox/jsonPath/query":{"!doc":"<p>Perform jsonPath query <code>expr</code> on javascript object or json string <code>obj</code></p>\n","prototype":{"constructor":{"!type":"fn(obj: ?, expr: ?, arg: ?)","!doc":"<p>Perform jsonPath query <code>expr</code> on javascript object or json string <code>obj</code></p>\n"}}},"dojox/jsonPath":{"!doc":"<p>Deprecated.  Should require dojox/jsonPath modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/lang/aspect/cflow":{"!doc":"<p>Returns true if the context stack contains a context for a given\ninstance that satisfies a given method name criteria.\n</p>\n","prototype":{"constructor":{"!type":"fn(instance: ?, method?: ?)","!doc":"<p>Returns true if the context stack contains a context for a given\ninstance that satisfies a given method name criteria.\n</p>\n"}}},"dojox/lang/aspect/counter":{"!doc":"<p>Returns an object, which can be used to count calls to methods.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Returns an object, which can be used to count calls to methods.</p>\n"}}},"dojox/lang/aspect/memoizer":{"!doc":"<p>Returns an object, which can be used to count calls to methods.\n</p>\n","prototype":{"constructor":{"!type":"fn(keyMaker?: ?)","!doc":"<p>Returns an object, which can be used to count calls to methods.\n</p>\n"}}},"dojox/lang/aspect/memoizerGuard":{"!doc":"<p>Invalidates the memoizer&#39;s cache (see dojox.lang.aspect.memoizer)\nafter calling certain methods.\n</p>\n","prototype":{"constructor":{"!type":"fn(method?: ?)","!doc":"<p>Invalidates the memoizer&#39;s cache (see dojox.lang.aspect.memoizer)\nafter calling certain methods.\n</p>\n"}}},"dojox/lang/aspect/profiler":{"!doc":"<p>Returns an object, which can be used to time calls to methods.\n</p>\n","prototype":{"constructor":{"!type":"fn(title?: ?)","!doc":"<p>Returns an object, which can be used to time calls to methods.\n</p>\n"}}},"dojox/lang/aspect/timer":{"!doc":"<p>Returns an object, which can be used to time calls to methods.\n</p>\n","prototype":{"constructor":{"!type":"fn(name?: ?)","!doc":"<p>Returns an object, which can be used to time calls to methods.\n</p>\n"}}},"dojox/lang/aspect/tracer":{"!doc":"<p>Returns an object, which can be used to trace calls with Firebug&#39;s console.\nPrints argument, a return value, or an exception.\n</p>\n","prototype":{"constructor":{"!type":"fn(grouping: ?)","!doc":"<p>Returns an object, which can be used to trace calls with Firebug&#39;s console.\nPrints argument, a return value, or an exception.\n</p>\n"}}},"dojox/lang/aspect":{"advise":{"!type":"fn(obj: ?, method: ?, advice: ?)","!doc":"<p>Attach AOP-style advices to a method.\n</p>\n"},"adviseRaw":{"!type":"fn(obj: ?, methods: ?, advices: ?)","!doc":"<p>Attach AOP-style advices to methods.\n</p>\n"},"getContext":{"!type":"fn()","!doc":"<p>Returns the context information for the advice in effect.</p>\n"},"getContextStack":{"!type":"fn()","!doc":"<p>Returns the context stack, which reflects executing advices\nup to this point. The array is ordered from oldest to newest.\nIn order to get the active context use dojox.lang.aspect.getContext().</p>\n"},"proceed":{"!type":"fn()","!doc":"<p>Call the original function (or the next level around advice) in an around advice code.\n</p>\n"},"unadvise":{"!type":"fn(handle: ?)","!doc":"<p>Detach previously attached AOP-style advices.\n</p>\n"}},"dojox/lang/async/event":{"failOn":{"!type":"fn(src: ?, name: ?)"},"from":{"!type":"fn(src: ?, name: ?)"}},"dojox/lang/async/timeout":{"failOn":{"!type":"fn(ms: ?)"},"from":{"!type":"fn(ms: ?)"}},"dojox/lang/async/topic":{"failOn":{"!type":"fn(topic: ?)"},"from":{"!type":"fn(topic: ?)"}},"dojox/lang/async":{"any":{"!type":"fn(x: ?)","!doc":"<p>Executes functions in parallel. As soon as one of them finishes\ncancels the rest.</p>\n"},"ifThen":{"!type":"fn(cond: ?, ifTrue: ?, ifFalse: ?)","!doc":"<p>Executes a condition, waits for it if necessary, and executes\none of two functions.</p>\n"},"loop":{"!type":"fn(cond: ?, body: ?)","!doc":"<p>Executes a condition, waits for it if necessary, and executes\nthe body, if truthy value was returned.\nThen it repeats the cycle until the condition function returns\na falsy value.</p>\n"},"par":{"!type":"fn(x: ?)","!doc":"<p>Executes functions in parallel. Waits for all of them to finish.</p>\n"},"select":{"!type":"fn(cond: ?, x: ?)","!doc":"<p>Executes a condition, waits for it if necessary, and executes\nNth function from list.</p>\n"},"seq":{"!type":"fn(x: ?)","!doc":"<p>Executes functions sequentially. Waits if any of them returns Deferred.</p>\n"}},"dojox/lang/functional/listcomp":{},"dojox/lang/functional/sequence":{"buildLambda":{"!type":"fn(s: ?) -> string","!doc":"<p>builds a function from a snippet, returns a string, which\nrepresents the function.</p>\n"},"clearLambdaCache":{"!type":"fn()","!doc":"<p>clears internal cache of lambdas</p>\n"},"every":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"everyRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether all elements in the array pass the test\nimplemented by the provided function.</p>\n"},"filter":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"filterIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with all attributes that pass the test\nimplemented by the provided function.</p>\n"},"filterRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with all elements that pass the test\nimplemented by the provided function.</p>\n"},"foldl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns the final value.</p>\n"},"foldr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns the final\nvalue.</p>\n"},"foldr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> ?","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns the final value.</p>\n"},"forEach":{"!type":"fn(a: ?, f: ?, o?: ?) -> string","!doc":"<p>executes a provided function once per array element.</p>\n"},"forEachRev":{"!type":"fn(a: ?, f: ?, o?: ?)","!doc":"<p>executes a provided function once per array element.</p>\n"},"forIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> string","!doc":"<p>iterates over all object attributes.</p>\n"},"keys":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all keys in the object</p>\n"},"lambda":{"!type":"fn(s: ?) -> fn()","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns a function object; functions are passed through\nunmodified.</p>\n"},"map":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"mapIn":{"!type":"fn(obj: ?, f: ?, o?: ?) -> ?","!doc":"<p>creates new object with the results of calling\na provided function on every attribute in this object.</p>\n"},"mapRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>creates a new array with the results of calling\na provided function on every element in this array.</p>\n"},"rawLambda":{"!type":"fn(s: ?) -> ?","!doc":"<p>builds a function from a snippet, or array (composing),\nreturns an object describing the function; functions are\npassed through unmodified.</p>\n"},"reduce":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from left-to-right) as to reduce it to a single value.</p>\n"},"reduceRight":{"!type":"fn(a: ?, f: ?, z?: ?) -> ?","!doc":"<p>apply a function simultaneously against two values of the array\n(from right-to-left) as to reduce it to a single value.</p>\n"},"repeat":{"!type":"fn(n: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>builds an array by repeatedly applying a unary function N times\nwith a seed value Z. N should be greater than 0.</p>\n"},"scanl":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right using a seed value as a starting point; returns an array\nof values produced by foldl() at that point.</p>\n"},"scanl1":{"!type":"fn(a: ?, f: ?, o?: ?) -> []","!doc":"<p>repeatedly applies a binary function to an array from left\nto right; returns an array of values produced by foldl1() at that\npoint.</p>\n"},"scanr":{"!type":"fn(a: ?, f: ?, z: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left using a seed value as a starting point; returns an array\nof values produced by foldr() at that point.</p>\n"},"scanr1":{"!type":"fn(a: ?, f: ?, o?: ?) -> +instance","!doc":"<p>repeatedly applies a binary function to an array from right\nto left; returns an array of values produced by foldr1() at that\npoint.</p>\n"},"some":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"someRev":{"!type":"fn(a: ?, f: ?, o?: ?) -> bool","!doc":"<p>tests whether some element in the array passes the test\nimplemented by the provided function.</p>\n"},"unfold":{"!type":"fn(pr: ?, f: ?, g: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by unfolding a value</p>\n"},"until":{"!type":"fn(pr: ?, f: ?, z: ?, o?: ?) -> []","!doc":"<p>builds an array by repeatedly applying a unary function with\na seed value Z until the predicate is satisfied.</p>\n"},"values":{"!type":"fn(obj: ?) -> []","!doc":"<p>returns an array of all values in the object</p>\n"}},"dojox/lang/functional/zip":{},"dojox/lang/observable":{"!doc":"<p>Creates a wrapper object, which can be observed. The wrapper object\nis a proxy to the wrapped object. If you will be making multiple wrapper\nobjects with the same set of listeners, it is recommended that you\nuse makeObservable, as it is more memory efficient.</p>\n","prototype":{"constructor":{"!type":"fn(wrapped: ?, onRead: ?, onWrite: ?, onInvoke: ?)","!doc":"<p>Creates a wrapper object, which can be observed. The wrapper object\nis a proxy to the wrapped object. If you will be making multiple wrapper\nobjects with the same set of listeners, it is recommended that you\nuse makeObservable, as it is more memory efficient.</p>\n"}}},"dojox/lang/oo/Decorator":{"!doc":"<p>The base class for all decorators.</p>\n","decorator":{"!doc":"<p>a function to handle the custom assignment, or an object with exec()\nmethod. The signature is:\ndecorator(/<em>String</em>/ name, /<em>Function</em>/ newValue, /<em>Function</em>/ oldValue).</p>\n"},"value":{"!doc":"<p>a payload to be processed by the decorator.</p>\n"},"prototype":{"constructor":{"!type":"fn(value: ?, decorator: ?)","!doc":"<p>The base class for all decorators.</p>\n"}}},"dojox/lang/oo/Filter":{"!doc":"<p>Filter to control mixing in objects by skipping\nproperties and renaming them.</p>\n","bag":{"!doc":"<p>object to be filtered</p>\n"},"filter":{"!doc":"<p>a function to handle the name filtering,\nor an object with exec() method</p>\n"},"prototype":{"constructor":{"!type":"fn(bag: ?, filter: ?)","!doc":"<p>Filter to control mixing in objects by skipping\nproperties and renaming them.</p>\n"}}},"dojox/lang/oo/aop":{"after":{},"afterReturning":{},"afterThrowing":{},"around":{},"before":{}},"dojox/lang/oo/general":{"after":{},"augment":{},"before":{},"override":{},"shuffle":{},"tap":{},"wrap":{}},"dojox/lang/oo/mixin":{"!doc":"<p>mixes in two or more objects processing decorators and filters\nusing defaults as a fallback</p>\n","prototype":{"constructor":{"!type":"fn(target: ?, source: ?) -> ?","!doc":"<p>mixes in two or more objects processing decorators and filters\nusing defaults as a fallback</p>\n"}}},"dojox/lang/oo/rearrange":{"!doc":"<p>Process properties in place by removing and renaming them.</p>\n","prototype":{"constructor":{"!type":"fn(bag: ?, map: ?) -> ?","!doc":"<p>Process properties in place by removing and renaming them.</p>\n"}}},"dojox/layout/BorderContainer":{},"dojox/layout/DragPane":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>Makes a pane&#39;s content draggable by/within it&#39;s surface</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invert":{"!doc":"<p>Naturally, the behavior is to invert the axis of the drag.\nSetting invert:false will make the pane drag in the same\ndirection as the mouse.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/layout/ExpandoPane":{"!type":"fn()","prototype":{"!proto":"dijit/layout/ContentPane.prototype","!doc":"<p>An experimental collapsing-pane for dijit.layout.BorderContainer</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTitleAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>duration to run show/hide animations</p>\n"},"easeIn":{"!doc":"<p>easing function use to show pane</p>\n"},"easeOut":{"!doc":"<p>easing function used to hide pane</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"previewOnDblClick":{"!doc":"<p>If true, will override the default behavior of a double-click calling a full toggle.\nIf false, a double-click will cause the preview to popup</p>\n"},"previewOpacity":{"!doc":"<p>A value from 0 .. 1 indicating the opacity to use on the container\nwhen only showing a preview</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startExpanded":{"!doc":"<p>Does this widget start in an open (true) or closed (false) state</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"preview":{"!type":"fn()","!doc":"<p>Expand this pane in preview mode (does not affect surrounding layout)</p>\n"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(newSize?: ?)","!doc":"<p>we aren&#39;t a layout widget, but need to act like one.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggle":{"!type":"fn()","!doc":"<p>Toggle this pane&#39;s visibility</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/layout/GridContainer":{"!type":"fn()","prototype":{"!proto":"dojox/layout/GridContainerLite.prototype","!doc":"<p>A grid containing any kind of objects and acting like web portals.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"acceptTypes":{"!doc":"<p>The GridContainer will only accept the children that fit to the types.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoRefresh":{"!doc":"<p>Enable the refresh of registered areas on drag start.</p>\n"},"baseClass":{"!doc":"<p>This class name is applied to the widget&#39;s domNode\nand also may be used to generate names for sub nodes,\nfor example dijitTabContainer-content.</p>\n"},"class":{},"colWidths":{"!doc":"<p>A comma separated list of column widths. If the column widths do not add up\nto 100, the remaining columns split the rest of the width evenly\nbetween them.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>If true, change the size of my currently displayed child to match my size.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dragHandleClass":{"!doc":"<p>CSS class enabling a drag handle on a child.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hasResizableColumns":{"!doc":"<p>Allow or not resizing of columns by a grip handle.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isAutoOrganized":{"!doc":"<p>If true, widgets are organized automatically,\nelse the attribute colum of child will define the right column.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"isLeftFixed":{"!doc":"<p>Define if the last left column is fixed.\nUsed when you add or remove columns by calling setColumns method.</p>\n"},"isRightFixed":{"!doc":"<p>Define if the last right column is fixed.\nUsed when you add or remove columns by calling setColumns method.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"liveResizeColumns":{"!doc":"<p>Specifies whether columns resize as you drag (true) or only upon mouseup (false)</p>\n"},"minChildWidth":{"!doc":"<p>Minimum children width in pixel (only used for IE6 which doesn&#39;t handle min-width css property)</p>\n"},"minColWidth":{"!doc":"<p>Minimum column width in percentage.</p>\n"},"mode":{"!doc":"<p>Location to add/remove columns, must be set to &#39;left&#39; or &#39;right&#39; (default).</p>\n"},"nbZones":{"!doc":"<p>The number of dropped zones, by default 1.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>template of gridContainer.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, column?: ?, p?: ?) -> +any","!doc":"<p>Add a child in a specific column of the GridContainer widget.</p>\n"},"addService":{"!type":"fn(child: ?, column?: ?, p?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(props: ?, node: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disableDnd":{"!type":"fn()","!doc":"<p>Disable the Drag And Drop for children of GridContainer.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"enableDnd":{"!type":"fn()","!doc":"<p>Enable the Drag And Drop for children of GridContainer.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> +any","!doc":"<p>A specific method which returns children after they were placed in zones.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()","!doc":"<p>Resize of each child</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)"},"resize":{"!type":"fn()","!doc":"<p>Resize the GridContainer widget and columns.\nReplace grips if it&#39;s necessary.</p>\n"},"resizeChildAfterDragStart":{"!type":"fn(node: ?, sourceArea: ?, indexChild: ?) -> bool","!doc":"<p>Resize the GridContainerLite inner table only if the drag source\nis a child of this gridContainer.</p>\n"},"resizeChildAfterDrop":{"!type":"fn(node: ?, targetArea: ?, indexChild: ?)","!doc":"<p>Call when a child is dropped.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setColumns":{"!type":"fn(nbColumns: ?)","!doc":"<p>Set the number of columns.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call the startup of GridContainerLite and place grips\nif user has chosen the hasResizableColumns attribute to true.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/layout/GridContainerLite":{"!type":"fn()","prototype":{"!proto":"dijit/layout/_LayoutWidget.prototype","!doc":"<p>The GridContainerLite is a container of child elements that are placed in a kind of grid.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"acceptTypes":{"!doc":"<p>The GridContainer will only accept the children that fit to the types.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoRefresh":{"!doc":"<p>Enable the refresh of registered areas on drag start.</p>\n"},"baseClass":{"!doc":"<p>This class name is applied to the widget&#39;s domNode\nand also may be used to generate names for sub nodes,\nfor example dijitTabContainer-content.</p>\n"},"class":{},"colWidths":{"!doc":"<p>A comma separated list of column widths. If the column widths do not add up\nto 100, the remaining columns split the rest of the width evenly\nbetween them.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>If true, change the size of my currently displayed child to match my size.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dragHandleClass":{"!doc":"<p>CSS class enabling a drag handle on a child.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isAutoOrganized":{"!doc":"<p>If true, widgets are organized automatically,\nelse the attribute colum of child will define the right column.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"nbZones":{"!doc":"<p>The number of dropped zones, by default 1.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>template of gridContainer.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, column?: ?, p?: ?) -> +any","!doc":"<p>Add a child in a specific column of the GridContainer widget.</p>\n"},"addService":{"!type":"fn(child: ?, column?: ?, p?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(props: ?, node: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disableDnd":{"!type":"fn()","!doc":"<p>Disable the Drag And Drop for children of GridContainer.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"enableDnd":{"!type":"fn()","!doc":"<p>Enable the Drag And Drop for children of GridContainer.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> +any","!doc":"<p>A specific method which returns children after they were placed in zones.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()","!doc":"<p>Resize of each child</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)"},"resize":{"!type":"fn(changeSize?: ?, resultSize?: ?)","!doc":"<p>Call this to resize a widget, or after its size has changed.</p>\n"},"resizeChildAfterDragStart":{"!type":"fn(node: ?, sourceArea: ?, indexChild: ?) -> bool","!doc":"<p>Resize the GridContainerLite inner table only if the drag source\nis a child of this gridContainer.</p>\n"},"resizeChildAfterDrop":{"!type":"fn(node: ?, targetArea: ?, indexChild: ?) -> +any","!doc":"<p>Resize the GridContainerLite inner table and the dropped widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"ChildWidgetProperties":{"!doc":"<p>Properties set on children of a GridContainerLite</p>\n"}},"dojox/layout/GridContainerLite.ChildWidgetProperties":{"!doc":"<p>Properties set on children of a GridContainerLite</p>\n","column":{"!doc":"<p>Column of the grid to place the widget.\nDefined only if dojo.require(&quot;dojox.layout.GridContainerLite&quot;) is done.</p>\n"},"dragRestriction":{"!doc":"<p>If true, the widget can not be draggable.\nDefined only if dojo.require(&quot;dojox.layout.GridContainerLite&quot;) is done.</p>\n"}},"dojox/mdnd/AreaManager":{"!type":"fn()","prototype":{"!doc":"<p>Drag And Drop manager</p>\n","areaClass":{"!doc":"<p>CSS class enabled an area if areaClass is defined</p>\n"},"autoRefresh":{"!doc":"<p>Enable the refresh of registered areas on drag start.</p>\n"},"dragHandleClass":{"!doc":"<p>CSS class enabled a drag handle.</p>\n"},"addDragItem":{"!type":"fn(area: ?, node: ?, index: ?, notCheckParent: ?) -> +any","!doc":"<p>To add an item programmatically.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Constructor of AreaManager class.\nInitialize arrays, connects and subscribes.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroy the component.</p>\n"},"findCurrentIndexArea":{"!type":"fn(coords: ?, size: ?) -> +any","!doc":"<p>find the nearest target area according to coordinates.\nCoordinates are representing by an object : for example, {&#39;x&#39;:10,&#39;y&#39;:10}</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initialize the manager by calling the registerByClass method</p>\n"},"placeDropIndicator":{"!type":"fn(coords: ?, size: ?) -> +any","!doc":"<p>Search the right place to insert the dropIndicator and display the dropIndicator.</p>\n"},"registerByClass":{"!type":"fn()","!doc":"<p>Register all Dnd Areas identified by the attribute areaClass :\ninsert Dnd Areas using the specific sort of dropMode.</p>\n"},"registerByNode":{"!type":"fn(area: ?, notInitAreas: ?)","!doc":"<p>To register Dnd Area : insert the DndArea using the specific sort of dropMode.</p>\n"},"removeDragItem":{"!type":"fn(area: ?, node: ?) -> +any","!doc":"<p>Delete a moveable item programmatically. The node is removed from the area.</p>\n"},"unregister":{"!type":"fn(area: ?) -> +any","!doc":"<p>Unregister a D&amp;D Area and its children into the AreaManager.</p>\n"}}},"dojox/mdnd/Moveable":{"!type":"fn()","prototype":{"!doc":"<p>Allow end-users to track a DOM node into the web page</p>\n","dragDistance":{"!doc":"<p>The user clicks on the handle, but the drag action will really begin\nif he tracks the main node to more than 3 pixels.</p>\n"},"handle":{"!doc":"<p>The node on which the user clicks to drag the main node.</p>\n"},"skip":{"!doc":"<p>A flag to control a drag action if a form element has been focused.\nIf true, the drag action is not executed.</p>\n"},"constructor":{"!type":"fn(params: ?, node: ?)","!doc":"<p>Configure parameters and listen to mousedown events from handle\nnode.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Delecte associated events</p>\n"},"initOffsetDrag":{"!type":"fn(e: ?)","!doc":"<p>Initialize the gap between main node coordinates and the clicked point.\nCall the onDragStart method.</p>\n"},"isFormElement":{"!type":"fn(e: ?) -> +any","!doc":"<p>identify the type of target node associated with a DOM event.</p>\n"}}},"dojox/mdnd/DropIndicator":{"!type":"fn()","prototype":{"!doc":"<p>DropIndicator managment for DnD.</p>\n","node":{"!doc":"<p>the drop indicator node</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()","!doc":"<p>destroy the dropIndicator</p>\n"},"place":{"!type":"fn(area: ?, nodeRef: ?, size: ?) -> +any","!doc":"<p>Place the DropIndicator in the right place</p>\n"},"remove":{"!type":"fn()","!doc":"<p>remove the DropIndicator (not destroy)</p>\n"}}},"dojox/mdnd/dropMode/OverDropMode":{"!type":"fn()","prototype":{"!doc":"<p>Default class to find the nearest target only if the mouse is over an area.</p>\n","_oldBehaviour":{"!doc":"<p>see getDragpoint()</p>\n"},"_oldXPoint":{"!doc":"<p>used to save a X position</p>\n"},"_oldYPoint":{"!doc":"<p>used to save a Y position</p>\n"},"addArea":{"!type":"fn(areas: ?, object: ?) -> +any","!doc":"<p>Add a D&amp;D Area into an array sorting by the x position.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"getDragPoint":{"!type":"fn(coords: ?, size: ?, mousePosition: ?) -> +any","!doc":"<p>return coordinates of the draggable item.\n\n</p>\n<ul>\n<li>For X point : the x position of mouse</li>\n<li>For Y point : the y position of mouse</li>\n</ul>\n"},"getDropIndex":{"!type":"fn(targetArea: ?, coords: ?) -> +any","!doc":"<p>Return the index where the drop has to be placed.</p>\n"},"getTargetArea":{"!type":"fn(areaList: ?, coords: ?, currentIndexArea: ?) -> +any","!doc":"<p>get the nearest D&amp;D area.</p>\n"},"initItems":{"!type":"fn(area: ?)","!doc":"<p>initialize the horizontal line in order to determinate the drop zone.</p>\n"},"refreshItems":{"!type":"fn(area: ?, indexItem: ?, size: ?, added: ?)","!doc":"<p>take into account the drop indicator DOM element in order to compute horizontal lines</p>\n"},"updateAreas":{"!type":"fn(areaList: ?)","!doc":"<p>refresh areas position and size to determinate the nearest area to drop an item</p>\n"}}},"dojox/mdnd/AutoScroll":{"!type":"fn()","prototype":{"!doc":"<p>Activate scrolling while dragging a widget.</p>\n","interval":{"!doc":"<p>default mouse move offset</p>\n"},"marginMouse":{"!doc":"<p>Default mouse margin</p>\n"},"recursiveTimer":{},"checkAutoScroll":{"!type":"fn(e: ?)","!doc":"<p>Check if an autoScroll have to be launched.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"getViewport":{"!type":"fn()","!doc":"<p>Set the visible part of the window. Varies accordion to Navigator.</p>\n"},"init":{"!type":"fn()"},"setAutoScrollMaxPage":{"!type":"fn()","!doc":"<p>Set the hightest heigh and width authorized scroll.</p>\n"},"setAutoScrollNode":{"!type":"fn(node: ?)","!doc":"<p>set the node which is dragged</p>\n"},"stopAutoScroll":{"!type":"fn()","!doc":"<p>Stop the autoscroll.</p>\n"}}},"dojox/layout/RadioGroup":{"!doc":"<p>dojox.layout.RadioGroup - an experimental (probably poorly named) Layout widget extending StackContainer\nthat accepts ContentPanes as children, and applies aesthetically pleasing responsive transition animations\nattached to :hover of the Buttons created.</p>\n"},"dojox/layout/RotatorContainer":{"!type":"fn()","prototype":{"!proto":"dijit/layout/StackContainer.prototype","!doc":"<p>Extends a StackContainer to automatically transition between children\nand display navigation in the form of tabs or a pager.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoStart":{"!doc":"<p>Starts the timer to transition children upon creation.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"cycles":{"!doc":"<p>Number of cycles before pausing.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>If true, change the size of my currently displayed child to match my size</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pagerClass":{"!doc":"<p>The declared Class of the Pager used for this Widget</p>\n"},"pagerId":{"!doc":"<p>ID the pager widget.</p>\n"},"pauseOnManualChange":{"!doc":"<p>Pause the rotator when the tab is changed or the pager&#39;s next/previous\nbuttons are clicked.</p>\n"},"persist":{"!doc":"<p>Remembers the selected child across sessions</p>\n"},"reverse":{"!doc":"<p>Causes the rotator to rotate in reverse order.</p>\n"},"searchContainerNode":{},"selectedChildWidget":{"!doc":"<p>References the currently selected child widget, if any.\nAdjust selected child with selectChild() method.</p>\n"},"showTabs":{"!doc":"<p>Sets the display of the tabs.  The tabs are actually a StackController.\nThe child&#39;s title is used for the tab&#39;s label.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"suspendOnHover":{"!doc":"<p>Pause the rotator when the mouse hovers over it.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>The type of transition to perform when switching children.\nA null transition will transition instantly.</p>\n"},"transitionDelay":{"!doc":"<p>The delay in milliseconds before transitioning to the next child.</p>\n"},"transitionDuration":{"!doc":"<p>The duration of the transition in milliseconds.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"back":{"!type":"fn() -> ?","!doc":"<p>Go back to previous page.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Unsubscribe to all of our topics</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"forward":{"!type":"fn() -> ?","!doc":"<p>Advance to next page.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Initializes the DOM nodes, tabs, and transition stuff.</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(page: ?)"},"resize":{"!type":"fn()"},"selectChild":{"!type":"fn(page: ?, animate: ?) -> ?","!doc":"<p>Show the given widget (which must be one of my children)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Initializes the pagers.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/layout/ScrollPane":{"!type":"fn()","prototype":{"!proto":"dijit/layout/ContentPane.prototype","!doc":"<p>A pane that &quot;scrolls&quot; its content based on the mouse poisition inside</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_line":{"!doc":"<p>storage for our top and bottom most scrollpoints</p>\n"},"_lo":{"!doc":"<p>the height of the visible pane</p>\n"},"_offset":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoHide":{"!doc":"<p>whether the scroll helper should hide when mouseleave</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"orientation":{"!doc":"<p>either &quot;horizontal&quot; or &quot;vertical&quot; for scroll orientation.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  False by default.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(size?: ?)","!doc":"<p>calculates required sizes. Call this if you add/remove\ncontent manually, or reload the content.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/layout/TableContainer":{"!type":"fn()","prototype":{"!proto":"dijit/layout/_LayoutWidget.prototype","!doc":"<p>A container that lays out its child widgets in a table layout.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>This class name is applied to the widget&#39;s domNode\nand also may be used to generate names for sub nodes,\nfor example dijitTabContainer-content.</p>\n"},"class":{},"cols":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"customClass":{"!doc":"<p>A CSS class that will be applied to child elements.  For example, if\nthe class is &quot;myClass&quot;, the table will have &quot;myClass-table&quot; applied to it,\neach label TD will have &quot;myClass-labelCell&quot; applied, and each\nwidget TD will have &quot;myClass-valueCell&quot; applied.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"labelWidth":{"!doc":"<p>Defines the width of a label.  If the value is a number, it is\ntreated as a pixel value.  The other valid value is a percentage,\ne.g. &quot;50%&quot;</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"orientation":{"!doc":"<p>Either &quot;horiz&quot; or &quot;vert&quot; for label orientation.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"showLabels":{"!doc":"<p>True if labels should be displayed, false otherwise.</p>\n"},"spacing":{"!doc":"<p>The cell spacing to apply to the table.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroys all the widgets inside this.containerNode,\nbut not this widget itself</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()","!doc":"<p>Lays out the child widgets.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(child: ?)"},"resize":{"!type":"fn()","!doc":"<p>Resizes all children.  This widget itself\ndoes not resize, as it takes up 100% of the\navailable width.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"ChildWidgetProperties":{"!doc":"<p>Properties to be set on children of TableContainer</p>\n"}},"dojox/layout/TableContainer.ChildWidgetProperties":{"!doc":"<p>Properties to be set on children of TableContainer</p>\n","colspan":{"!doc":"<p>The number of columns this widget should span.</p>\n"},"label":{"!doc":"<p>The label to display for a given widget</p>\n"},"spanLabel":{"!doc":"<p>Setting spanLabel to true makes the widget take up both the\nlabel and value cells. Defaults to false.</p>\n"},"title":{"!doc":"<p>The label to display for a given widget.  This is interchangeable\nwith the &#39;label&#39; parameter, as some widgets already have a use\nfor the &#39;label&#39;, and this can be used instead to avoid conflicts.</p>\n"}},"dojox/layout/ToggleSplitter":{"!type":"fn()","prototype":{"!doc":"<p>A draggable and clickable spacer between two items in a <code>dijit.layout.BorderContainer</code>.</p>\n","_closedSize":{"!doc":"<p>the css height/width value to apply by default when the attached pane is closed</p>\n"},"baseClass":{},"child":{"!doc":"<p>Pointer to the pane associated with this splitter</p>\n"},"container":{"!doc":"<p>Pointer to the parent BorderContainer</p>\n"},"region":{"!doc":"<p>Region of pane associated with this splitter.\n&quot;top&quot;, &quot;bottom&quot;, &quot;left&quot;, &quot;right&quot;.</p>\n"},"state":{"!doc":"<p>the initial and current state of the splitter (and its attached pane)\nIt has three values: full, collapsed (optional), closed</p>\n"},"templateString":{},"postCreate":{"!type":"fn()"},"startup":{"!type":"fn() -> fn"}}},"dojox/layout/dnd/Avatar":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Avatar.prototype","!doc":"<p>An Object, which represents the object being moved in a GridContainer</p>\n","manager":{"!doc":"<p>a DnD manager object</p>\n"},"construct":{"!type":"fn()","!doc":"<p>A constructor function. it is separate so it can be (dynamically)\noverwritten in case of need.</p>\n"},"constructor":{"!type":"fn(manager: ?, opacity: ?)"},"destroy":{"!type":"fn()","!doc":"<p>destructor for the avatar; called to remove all references so it can be garbage-collected</p>\n"},"update":{"!type":"fn()","!doc":"<p>Updates the avatar to reflect the current DnD state.</p>\n"}}},"dojox/layout/dnd/PlottedDnd":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Source.prototype","!doc":"<p>dnd source handling plotted zone to show the dropping area</p>\n","accept":{},"allowNested":{"!doc":"<p>Indicates whether to allow dnd item nodes to be nested within other elements.\nBy default this is false, indicating that only direct children of the container can\nbe draggable dnd item nodes</p>\n"},"autoSync":{},"copyOnly":{},"current":{"!doc":"<p>The DOM node the mouse is currently hovered over</p>\n"},"delay":{},"GC_OFFSET_X":{},"GC_OFFSET_Y":{},"generateText":{},"horizontal":{},"isSource":{},"map":{"!doc":"<p>Map from an item&#39;s id (which is also the DOMNode&#39;s id) to\nthe dojo/dnd/Container.Item itself.</p>\n"},"selection":{"!doc":"<p>The set of id&#39;s that are currently selected, such that this.selection[id] == 1\nif the node w/that id is selected.  Can iterate over selected node&#39;s id&#39;s like:\n\n</p>\n<pre><code>for(var id in this.selection)</code></pre>\n"},"selfAccept":{},"selfCopy":{},"singular":{},"skipForm":{},"withHandles":{},"checkAcceptance":{"!type":"fn(source: ?, nodes: ?) -> bool","!doc":"<p>checks if the target can accept nodes from this source</p>\n"},"clearItems":{"!type":"fn()","!doc":"<p>removes all data items from the map</p>\n"},"constructor":{"!type":"fn(node: ?, params: ?)"},"copyState":{"!type":"fn(keyPressed: ?, self?: ?) -> bool","!doc":"<p>Returns true if we need to copy items, false to move.\nIt is separated to be overwritten dynamically, if needed.</p>\n"},"creator":{"!type":"fn()","!doc":"<p>creator function, dummy at the moment</p>\n"},"deleteDashedZone":{"!type":"fn()","!doc":"<p>hide the dashed zone</p>\n"},"deleteSelectedNodes":{"!type":"fn() -> fn","!doc":"<p>deletes all selected items</p>\n"},"delItem":{"!type":"fn(key: ?)","!doc":"<p>removes a data item from the map by its key (id)</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>prepares the object to be garbage-collected</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"forInItems":{"!type":"fn(f: ?, o?: ?) -> string","!doc":"<p>iterates over a data map skipping members that\nare present in the empty object (IE and/or 3rd-party libraries).</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getAllNodes":{"!type":"fn() -> ?","!doc":"<p>returns a list (an array) of all valid child nodes</p>\n"},"getDraggedWidget":{"!type":"fn(node: ?) -> +null","!doc":"<p>Return one or more widget selected during the drag.</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>returns a data item by its key (id)</p>\n"},"getSelectedNodes":{"!type":"fn() -> +instance","!doc":"<p>returns a list (an array) of selected nodes</p>\n"},"insertDashedZone":{"!type":"fn(before: ?)","!doc":"<p>Insert the dashed zone at the right place</p>\n"},"insertNodes":{"!type":"fn(addSelected: ?, data: ?, before: ?, anchor: ?) -> ?","!doc":"<p>Inserts new data items (see Dojo Container&#39;s insertNodes method for details).</p>\n"},"isAccepted":{"!type":"fn(node: ?) -> ?","!doc":"<p>test if this node can be accepted</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, Ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"selectAll":{"!type":"fn() -> ?","!doc":"<p>selects all items</p>\n"},"selectNone":{"!type":"fn() -> ?","!doc":"<p>unselects all items</p>\n"},"setDndItemSelectable":{"!type":"fn(node: ?, isSelectable: ?)","!doc":"<p>set an item as selectable</p>\n"},"setIndicatorPosition":{"!type":"fn(e: ?) -> bool","!doc":"<p>set the position of the drop indicator</p>\n"},"setItem":{"!type":"fn(key: ?, data: ?)","!doc":"<p>associates a data item with its key (id)</p>\n"},"startup":{"!type":"fn()","!doc":"<p>collects valid child items and populate the map</p>\n"},"sync":{"!type":"fn() -> fn","!doc":"<p>sync up the node list with the data map</p>\n"}}},"dojox/math/curves":{},"dojox/math/random/Secure":{"!type":"fn()","prototype":{"!doc":"<p>Super simple implementation of a random number generator,\nwhich relies on Math.random().</p>\n","constructor":{"!type":"fn(prng: ?, noEvents?: ?)","!doc":"<p>Initializes an instance of a secure random generator.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Disconnects events, if any, preparing the object for GC.</p>\n"},"nextBytes":{"!type":"fn(byteArray: ?)","!doc":"<p>Fills in an array of bytes with random numbers</p>\n"},"seedTime":{"!type":"fn()","!doc":"<p>Mix in the current time (w/milliseconds) into the pool</p>\n"}}},"dojox/math/random/prng4":{"size":{},"prototype":{"constructor":{"!type":"fn()"}}},"dojox/math/stats":{},"dojox/math":{"!doc":"<p>Deprecated.  Should require dojox/math modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/mdnd/LazyManager":{"!type":"fn()","prototype":{"!doc":"<p>This class allows to launch a drag and drop dojo on the fly.</p>\n","cancelDrag":{"!type":"fn()","!doc":"<p>cancel a drag and drop dojo on the fly.</p>\n"},"constructor":{"!type":"fn()"},"destroy":{"!type":"fn()"},"getItem":{"!type":"fn(draggedNode: ?) -> ?"},"startDrag":{"!type":"fn(e: ?, draggedNode?: ?)","!doc":"<p>launch a dojo drag and drop on the fly.</p>\n"}}},"dojox/mdnd/PureSource":{"!type":"fn()","prototype":{"!proto":"dojo/dnd/Selector.prototype","!doc":"<p>A Source Object, which can be used only as a DnD source.\nA Source can contained several dnd items.\nA dnd item is not a source.</p>\n","allowNested":{"!doc":"<p>Indicates whether to allow dnd item nodes to be nested within other elements.\nBy default this is false, indicating that only direct children of the container can\nbe draggable dnd item nodes</p>\n"},"copyOnly":{},"current":{"!doc":"<p>The DOM node the mouse is currently hovered over</p>\n"},"generateText":{},"horizontal":{},"isSource":{},"map":{"!doc":"<p>Map from an item&#39;s id (which is also the DOMNode&#39;s id) to\nthe dojo/dnd/Container.Item itself.</p>\n"},"selection":{"!doc":"<p>The set of id&#39;s that are currently selected, such that this.selection[id] == 1\nif the node w/that id is selected.  Can iterate over selected node&#39;s id&#39;s like:\n\n</p>\n<pre><code>for(var id in this.selection)</code></pre>\n"},"singular":{},"skipForm":{},"targetState":{},"withHandles":{},"clearItems":{"!type":"fn()","!doc":"<p>removes all data items from the map</p>\n"},"constructor":{"!type":"fn(node: ?, params?: ?)","!doc":"<p>Initialize a new PureSource.</p>\n"},"copyState":{"!type":"fn(keyPressed: ?) -> +any","!doc":"<p>Returns true, if we need to copy items, false to move.\nIt is separated to be overwritten dynamically, if needed.</p>\n"},"creator":{"!type":"fn()","!doc":"<p>creator function, dummy at the moment</p>\n"},"deleteSelectedNodes":{"!type":"fn() -> fn","!doc":"<p>deletes all selected items</p>\n"},"delItem":{"!type":"fn(key: ?)","!doc":"<p>removes a data item from the map by its key (id)</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Prepares the object to be garbage-collected.</p>\n"},"emit":{"!type":"fn(type: ?, event: ?) -> ?"},"forInItems":{"!type":"fn(f: ?, o?: ?) -> string","!doc":"<p>iterates over a data map skipping members that\nare present in the empty object (IE and/or 3rd-party libraries).</p>\n"},"forInSelectedItems":{"!type":"fn(f: ?, o?: ?)","!doc":"<p>iterates over selected items;\nsee <code>dojo/dnd/Container.forInItems()</code> for details</p>\n"},"getAllNodes":{"!type":"fn() -> ?","!doc":"<p>returns a list (an array) of all valid child nodes</p>\n"},"getItem":{"!type":"fn(key: ?) -> ?","!doc":"<p>returns a data item by its key (id)</p>\n"},"getSelectedNodes":{"!type":"fn() -> +instance","!doc":"<p>returns a list (an array) of selected nodes</p>\n"},"insertNodes":{"!type":"fn(addSelected: ?, data: ?, before: ?, anchor: ?) -> fn","!doc":"<p>inserts new data items (see <code>dojo/dnd/Container.insertNodes()</code> method for details)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?) -> +any","!doc":"<p>Markup methods.</p>\n"},"on":{"!type":"fn(type: ?, listener: ?) -> ?"},"selectAll":{"!type":"fn() -> ?","!doc":"<p>selects all items</p>\n"},"selectNone":{"!type":"fn() -> ?","!doc":"<p>unselects all items</p>\n"},"setItem":{"!type":"fn(key: ?, data: ?)","!doc":"<p>associates a data item with its key (id)</p>\n"},"startup":{"!type":"fn()","!doc":"<p>collects valid child items and populate the map</p>\n"},"sync":{"!type":"fn() -> fn","!doc":"<p>sync up the node list with the data map</p>\n"}}},"dojox/mdnd/adapter/DndFromDojo":{"!type":"fn()","prototype":{"!doc":"<p>Allow communication between Dojo dnd items and DojoX D&amp;D areas</p>\n","_areaManager":{"!doc":"<p>Reference to the current DojoX Dnd Manager</p>\n"},"_currentArea":{"!doc":"<p>The current Area on mouse over</p>\n"},"_dojoManager":{},"_moveHandler":{"!doc":"<p>The handler of mouse connection</p>\n"},"_oldArea":{"!doc":"<p>The old area the mouse has passed over</p>\n"},"_subscribeHandler":{"!doc":"<p>The list of dojo dnd topics</p>\n"},"dropIndicatorSize":{"!doc":"<p>size by default of dropIndicator (display only into a D&amp;D Area)</p>\n"},"constructor":{"!type":"fn()"},"isAccepted":{"!type":"fn(node: ?, accept: ?) -> +any","!doc":"<p>Check if a dragNode is accepted into a dojo target.</p>\n"},"subscribeDnd":{"!type":"fn()","!doc":"<p>Subscribe to somes topics of dojo drag and drop.</p>\n"},"unsubscribeDnd":{"!type":"fn()","!doc":"<p>Unsubscribe to some topics of dojo drag and drop.</p>\n"}}},"dojox/mdnd/adapter/DndToDojo":{"!type":"fn()","prototype":{"!doc":"<p>Allow communication between an item of dojox D&amp;D area to a target dojo.</p>\n","_currentDojoArea":{"!doc":"<p>Representing the current dojo area</p>\n"},"_dojoList":{"!doc":"<p>Array containing object references the dojo Target list</p>\n"},"_dojoxManager":{"!doc":"<p>The reference to the dojox AreaManager</p>\n"},"_draggedNode":{"!doc":"<p>The current dragged node</p>\n"},"_dragStartHandler":{"!doc":"<p>Handle to keep start subscribe</p>\n"},"_dropHandler":{"!doc":"<p>Handle to keep drop subscribe</p>\n"},"_moveHandler":{"!doc":"<p>Handle to keep move subscribe</p>\n"},"_moveUpHandler":{"!doc":"<p>Handle to kee move up subscribe</p>\n"},"constructor":{"!type":"fn()"},"isAccepted":{"!type":"fn(draggedNode: ?, target: ?) -> bool","!doc":"<p>Return true if the dragged node is accepted.\nThis method has to be overwritten according to registered target.</p>\n"},"refresh":{"!type":"fn()","!doc":"<p>Refresh the coordinates of all registered dojo target.</p>\n"},"refreshByType":{"!type":"fn(type: ?)","!doc":"<p>Refresh the coordinates of registered dojo target with a specific type.</p>\n"},"register":{"!type":"fn(area: ?, type: ?, dojoTarget: ?)","!doc":"<p>Register a target dojo.\nThe target is represented by an object containing :\n\n</p>\n<ul>\n<li>the dojo area node</li>\n<li>the type reference to identify a group node</li>\n<li>the coords of the area to enable refresh position</li>\n</ul>\n"},"unregister":{"!type":"fn()","!doc":"<p>Unregister all targets dojo.</p>\n"},"unregisterByNode":{"!type":"fn(area: ?)","!doc":"<p>Unregister a target dojo.</p>\n"},"unregisterByType":{"!type":"fn(type: ?)","!doc":"<p>Unregister several targets dojo having the same type passing in parameter.</p>\n"}}},"dojox/mdnd/dropMode/DefaultDropMode":{"!type":"fn()","prototype":{"!doc":"<p>Enabled a type of calcul for Dnd.\nDefault class to find the nearest target.</p>\n","_oldBehaviour":{"!doc":"<p>see <code>getDragPoint</code></p>\n"},"_oldXPoint":{"!doc":"<p>used to save a X position</p>\n"},"_oldYPoint":{"!doc":"<p>used to save a Y position</p>\n"},"addArea":{"!type":"fn(areas: ?, object: ?) -> +any","!doc":"<p>Add a DnD Area into an array sorting by the x position.</p>\n"},"destroy":{"!type":"fn()"},"getDragPoint":{"!type":"fn(coords: ?, size: ?, mousePosition: ?) -> +any","!doc":"<p>return coordinates of the draggable item</p>\n"},"getDropIndex":{"!type":"fn(targetArea: ?, coords: ?) -> +any","!doc":"<p>Return the index where the drop has to be placed.</p>\n"},"getTargetArea":{"!type":"fn(areaList: ?, coords: ?, currentIndexArea: ?) -> +any","!doc":"<p>get the nearest DnD area.\nCoordinates are basically provided by the <code>getDragPoint</code> method.</p>\n"},"initItems":{"!type":"fn(area: ?)","!doc":"<p>initialize the horizontal line in order to determinate the drop zone.</p>\n"},"refreshItems":{"!type":"fn(area: ?, indexItem: ?, size: ?, added: ?)","!doc":"<p>take into account the drop indicator DOM element in order to compute horizontal lines</p>\n"},"updateAreas":{"!type":"fn(areaList: ?)","!doc":"<p>Refresh intervals between areas to determinate the nearest area to drop an item.\nAlgorithm :\nthe marker should be the vertical line passing by the\ncentral point between two contiguous areas.\nNote:\nIf the page has only one targetArea, it&#39;s not necessary to calculate coords.</p>\n"}}},"dojox/mdnd/dropMode/VerticalDropMode":{"!type":"fn()","prototype":{"!doc":"<p>Enabled a type of calcul for Dnd.\nDefault class to find the nearest target.</p>\n","_oldBehaviour":{"!doc":"<p>see <code>getDragPoint</code></p>\n"},"_oldXPoint":{"!doc":"<p>used to save a X position</p>\n"},"_oldYPoint":{"!doc":"<p>used to save a Y position</p>\n"},"addArea":{"!type":"fn(areas: ?, object: ?) -> +any","!doc":"<p>Add a DnD Area into an array sorting by the x position.</p>\n"},"destroy":{"!type":"fn()"},"getDragPoint":{"!type":"fn(coords: ?, size: ?, mousePosition: ?) -> +any","!doc":"<p>return coordinates of the draggable item</p>\n"},"getDropIndex":{"!type":"fn(targetArea: ?, coords: ?) -> +any","!doc":"<p>Return the index where the drop has to be placed.</p>\n"},"getTargetArea":{"!type":"fn(areaList: ?, coords: ?, currentIndexArea: ?) -> +any","!doc":"<p>get the nearest DnD area.\nCoordinates are basically provided by the <code>getDragPoint</code> method.</p>\n"},"initItems":{"!type":"fn(area: ?)","!doc":"<p>initialize the horizontal line in order to determinate the drop zone.</p>\n"},"refreshItems":{"!type":"fn(area: ?, indexItem: ?, size: ?, added: ?)","!doc":"<p>take into account the drop indicator DOM element in order to compute horizontal lines</p>\n"},"updateAreas":{"!type":"fn(areaList: ?)","!doc":"<p>Refresh intervals between areas to determinate the nearest area to drop an item.\nAlgorithm :\nthe marker should be the vertical line passing by the\ncentral point between two contiguous areas.\nNote:\nIf the page has only one targetArea, it&#39;s not necessary to calculate coords.</p>\n"}}},"dojox/mobile/Accordion":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A container widget that can display a group of child panes in a stacked format.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_openSpace":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"animation":{"!doc":"<p>If true, animation is used when a pane is opened or\ncollapsed. The animation works only on webkit browsers.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{},"fixedHeight":{"!doc":"<p>If true, the entire accordion widget has fixed height regardless\nof the height of each pane; in this mode, there is always an open pane and\ncollapsing a pane can only be done by opening a different pane.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child widgets.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child widgets.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"roundRect":{"!doc":"<p>If true, the widget shows rounded corners.\nAdding the &quot;mblAccordionRoundRect&quot; class to domNode has the same effect.</p>\n"},"singleOpen":{"!doc":"<p>If true, only one pane is open at a time. The current open pane\nis collapsed, when another pane is opened.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"buildRendering":{"!type":"fn()"},"collapse":{"!type":"fn(pane: ?, noAnimation: ?)","!doc":"<p>Collapses the given pane to close it.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselect":{"!type":"fn(pane: ?)","!doc":"<p>Unhighlights the title bar of the given pane.</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"expand":{"!type":"fn(pane: ?, noAnimation: ?)","!doc":"<p>Expands the given pane to make it visible.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> ?"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSelectedPanes":{"!type":"fn() -> ?"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)"},"resize":{"!type":"fn()"},"select":{"!type":"fn(pane: ?)","!doc":"<p>Highlights the title bar of the given pane.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a dojox/mobile/Accordion.</p>\n"}},"dojox/mobile/Accordion.ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a dojox/mobile/Accordion.</p>\n","alt":{"!doc":"<p>The alternate text of the Accordion title.</p>\n"},"icon1":{"!doc":"<p>The unselected icon of the Accordion title.</p>\n"},"icon2":{"!doc":"<p>The selected icon of the Accordion title.</p>\n"},"iconPos1":{"!doc":"<p>The position (&quot;top,left,width,height&quot;) of the unselected aggregated icon of the Accordion title.</p>\n"},"iconPos2":{"!doc":"<p>The position (&quot;top,left,width,height&quot;) of the selected aggregated icon of the Accordion title.</p>\n"},"label":{"!doc":"<p>The label of the Accordion title.</p>\n"},"lazy":{"!doc":"<p>Specifies that the Accordion child must be lazily loaded.</p>\n"},"selected":{"!doc":"<p>The selected state of the Accordion title.</p>\n"}},"dojox/mobile/iconUtils":{"!doc":"<p>Utilities to create an icon (image, CSS sprite image, or DOM Button).</p>\n","iconWrapper":{},"prototype":{"constructor":{"!type":"fn()","!doc":"<p>Utilities to create an icon (image, CSS sprite image, or DOM Button).</p>\n"}},"createDomButton":{"!type":"fn(refNode: ?, style?: ?, toNode?: ?)","!doc":"<p>Creates a DOM button.</p>\n"},"createIcon":{"!type":"fn(icon: ?, iconPos?: ?, node?: ?, title?: ?, parent?: ?, refNode?: ?, pos?: ?)","!doc":"<p>Creates or updates an icon node</p>\n"},"setIcon":{"!type":"fn(icon: ?, iconPos: ?, iconNode: ?, alt?: ?, parent: ?, refNode?: ?, pos?: ?)","!doc":"<p>A setter function to set an icon.</p>\n"},"setupSpriteIcon":{"!type":"fn(iconNode: ?, iconPos: ?)","!doc":"<p>Sets up CSS sprite for a foreground image.</p>\n"}},"dojox/mobile/lazyLoadUtils":{"!doc":"<p>Utilities to lazy-loading of Dojo widgets.</p>\n","_lazyNodes":{},"prototype":{"constructor":{"!type":"fn()","!doc":"<p>Utilities to lazy-loading of Dojo widgets.</p>\n"}},"instantiateLazyWidgets":{"!type":"fn(root: ?, requires: ?, callback: ?)","!doc":"<p>Instantiates dojo widgets under the root node.</p>\n"}},"dojox/mobile/Audio":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A thin wrapper around the HTML5 <code>&lt;audio&gt;</code> element.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_playable":{"!doc":"<p>Internal flag.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tag":{"!doc":"<p>The name of the tag (&quot;audio&quot;).</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>The height of the embed element.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"source":{"!doc":"<p>An array of src and type,\nex. [{src:&quot;a.mp3&quot;, type:&quot;audio/mpeg&quot;}, {src:&quot;a.ogg&quot;, type:&quot;audio/ogg&quot;}, ...].\nThe src gives the path of the media resource. The type gives the\ntype of the media resource.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"width":{"!doc":"<p>The width of the embed element. \nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Creates a new instance of the class.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Badge":{"!type":"fn()","prototype":{"!doc":"<p>A utility class to create and update a badge node.</p>\n","className":{"!doc":"<p>A CSS class name of a DOM button.</p>\n"},"fontSize":{"!doc":"<p>Font size in pixel. The other style attributes are determined by the DOM\nbutton itself.\nNote that changing the value of the property after the badge\ncreation has no effect.</p>\n"},"value":{"!doc":"<p>A text to show in a badge.\nNote that changing the value of the property after the badge\ncreation has no effect.</p>\n"},"constructor":{"!type":"fn(params?: ?, node?: ?)","!doc":"<p>Creates a new instance of the class.</p>\n"},"getValue":{"!type":"fn() -> string","!doc":"<p>Returns the text shown in the badge.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Set a label text to the badge.</p>\n"}}},"dojox/mobile/Carousel":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A carousel widget that manages a list of images.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>Explicitly specified height of the widget (ex. &quot;300px&quot;). If\n&quot;inherit&quot; is specified, the height is inherited from its offset\nparent.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"itemWidth":{"!doc":"<p>The number of visible items (=numVisible) is determined by\n(carousel_width / itemWidth).\nIf itemWidth is specified, numVisible is automatically calculated.\nIf resize() is called, numVisible is recalculated and the layout\nis changed accordingly.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"navButton":{"!doc":"<p>If true, navigation buttons are displyed on the title bar.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"numVisible":{"!doc":"<p>The number of visible items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageIndicator":{"!doc":"<p>If true, a page indicator, a series of small dots that indicate\nthe current page, is displayed on the title bar.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"selectable":{"!doc":"<p>If true, an item can be selected by clicking it.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>A title of the carousel to be displayed on the title bar.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"fillPages":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexByItemWidget":{"!type":"fn(w: ?) -> number","!doc":"<p>Returns the index of a given item widget.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getItemWidgetByIndex":{"!type":"fn(index: ?) -> +null","!doc":"<p>Returns the index of an item widget at a given index.</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getParentView":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the parent view of the given DOM node.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"handleViewChanged":{"!type":"fn(view: ?)","!doc":"<p>Listens to &quot;/dojox/mobile/viewChanged&quot; events.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"instantiateView":{"!type":"fn(view: ?)","!doc":"<p>Instantiates the given view.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()"},"resizeItems":{"!type":"fn()","!doc":"<p>Resizes the child items of the carousel.</p>\n"},"select":{"!type":"fn(itemWidget: ?)","!doc":"<p>Selects the given widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"ChildSwapViewProperties":{"!doc":"<p>This property can be specified for the SwapView children of a dojox/mobile/Carousel.</p>\n"}},"dojox/mobile/Carousel.ChildSwapViewProperties":{"!doc":"<p>This property can be specified for the SwapView children of a dojox/mobile/Carousel.</p>\n","lazy":{"!doc":"<p>Specifies that the Carousel child must be lazily loaded.</p>\n"}},"dojox/mobile/CarouselItem":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>An item of dojox/mobile/Carousel.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>An alt text for the carousel item image.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerText":{"!doc":"<p>A text that is displayed below the carousel item image.</p>\n"},"headerText":{"!doc":"<p>A text that is displayed above the carousel item image.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"src":{"!doc":"<p>A path for an image to be displayed as a carousel item.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"resize":{"!type":"fn(size: ?)"},"select":{"!type":"fn()","!doc":"<p>Highlights the item.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/PageIndicator":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A current page indicator.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"refId":{"!doc":"<p>An ID of a DOM node to be searched. Siblings of the reference\nnode will be searched for views. If not specified, this.domNode\nwill be the reference node.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isView":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns true if the given node is a view.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Updates the indicator.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/SwapView":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/View.prototype","!doc":"<p>A container that can be swiped horizontally.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_endOfTransitionTimeoutHandle":{"!doc":"<p>The handle (returned by _WidgetBase.defer) for the timeout set on touchEnd in case\nthe end of transition event is not fired by the browser.</p>\n"},"_fixedAppFooter":{},"_moved":{"!doc":"<p>Flag that signals if the user have moved in (one of) the scroll\ndirection(s) since touch start (a move under the threshold is ignored).</p>\n"},"_parentPadBorderExtentsBottom":{"!doc":"<p>For Tooltip.js.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"allowNestedScrolls":{"!doc":"<p>Flag to allow scrolling in nested containers, e.g. to allow ScrollableView in a SwapView.</p>\n"},"appBars":{"!doc":"<p>Enables the search for application-specific bars (header or footer).</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraint":{"!doc":"<p>bounce back to the content area</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dirLock":{"!doc":"<p>disable the move handler if scroll starts in the unexpected direction</p>\n"},"disableFlashScrollBar":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fadeScrollBar":{},"fixedFooter":{"!doc":"<p>Id of the fixed footer.</p>\n"},"fixedFooterHeight":{"!doc":"<p>height of a fixed footer</p>\n"},"fixedHeader":{"!doc":"<p>Id of the fixed header.</p>\n"},"fixedHeaderHeight":{"!doc":"<p>height of a fixed header</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>explicitly specified height of this widget (ex. &quot;300px&quot;)</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLocalFooter":{"!doc":"<p>footer is view-local (as opposed to application-wide)</p>\n"},"keepScrollPos":{"!doc":"<p>If true, the scroll position is kept when transition occurs between views.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"propagatable":{"!doc":"<p>let touchstart event propagate up</p>\n"},"scrollableParams":{"!doc":"<p>Parameters for dojox/mobile/scrollable.init().</p>\n"},"scrollBar":{"!doc":"<p>show scroll bar or not</p>\n"},"scrollDir":{"!doc":"<p>Scroll direction, used by dojox/mobile/scrollable (always &quot;f&quot; for this class).</p>\n"},"scrollType":{"!doc":"<ul>\n<li>1: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)animation for slide animation</li>\n<li>2: use top/left style,</li>\n<li>3: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)transition for slide animation</li>\n<li>0: use default value (3 for Android, iOS6+, and BlackBerry; otherwise 1)</li>\n</ul>\n"},"selected":{"!doc":"<p>If true, the view is displayed at startup time.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>The name of the HTML tag to create as domNode. The default value is &quot;div&quot;.</p>\n"},"threshold":{"!doc":"<p>drag threshold value in pixels</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"touchNode":{"!doc":"<p>a node that will have touch event handlers</p>\n"},"weight":{"!doc":"<p>Frictional weight used to compute scrolling speed.</p>\n"},"abort":{"!type":"fn()","!doc":"<p>Aborts scrolling.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"adjustDestination":{"!type":"fn(to: ?, pos: ?, dim: ?) -> bool","!doc":"<p>A stub function to be overridden by subclasses.</p>\n"},"buildRendering":{"!type":"fn()"},"calcScrollBarPos":{"!type":"fn(to: ?) -> ?","!doc":"<p>Calculates the scroll bar position.</p>\n"},"calcSpeed":{"!type":"fn(distance: ?, time: ?) -> number","!doc":"<p>Calculate the speed given the distance and time.</p>\n"},"checkFixedBar":{"!type":"fn(node: ?, local: ?) -> ?","!doc":"<p>Checks if the given node is a fixed bar or not.</p>\n"},"cleanup":{"!type":"fn()","!doc":"<p>Uninitialize the module.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, node?: ?)","!doc":"<p>Creates a new instance of the class.</p>\n"},"convertToId":{"!type":"fn(moveTo: ?) -> ?"},"createMask":{"!type":"fn()","!doc":"<p>Creates a mask for a scroll bar edge.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"findAppBars":{"!type":"fn()","!doc":"<p>Search for application-specific header or footer.</p>\n"},"findDisp":{"!type":"fn(node: ?) -> ?","!doc":"<p>Overrides dojox/mobile/scrollable.findDisp().</p>\n"},"flashScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar instantly.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDim":{"!type":"fn() -> ?","!doc":"<p>Returns various internal dimensional information needed for calculation.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPos":{"!type":"fn() -> ?","!doc":"<p>Gets the top position in the midst of animation.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getScreenSize":{"!type":"fn() -> ?","!doc":"<p>Returns the dimensions of the browser window.</p>\n"},"getShowingView":{"!type":"fn() -> ?","!doc":"<p>Find the currently showing view from my sibling views.</p>\n"},"getSiblingViews":{"!type":"fn() -> []","!doc":"<p>Returns an array of the sibling views.</p>\n"},"getSpeed":{"!type":"fn() -> ?","!doc":"<p>Returns an object that indicates the scrolling speed.</p>\n"},"goTo":{"!type":"fn(dir: ?, moveTo?: ?)","!doc":"<p>Moves to the next or previous view.</p>\n"},"handleNextPage":{"!type":"fn(w: ?)","!doc":"<p>Called when the &quot;/dojox/mobile/nextPage&quot; topic is published.</p>\n"},"handlePrevPage":{"!type":"fn(w: ?)","!doc":"<p>Called when the &quot;/dojox/mobile/prevPage&quot; topic is published.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hides this view without a transition animation.</p>\n"},"hideScrollBar":{"!type":"fn()","!doc":"<p>Hides the scroll bar.</p>\n"},"init":{"!type":"fn(params?: ?)","!doc":"<p>Initialize according to the given params.</p>\n"},"invokeCallback":{"!type":"fn()","!doc":"<p>A function to be called after performing a transition to\ncall a specified callback.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isFormElement":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the given node is a form control.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isSwapView":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns true if the given node is a SwapView widget.</p>\n"},"isTopLevel":{"!type":"fn() -> bool","!doc":"<p>Returns true if this is a top-level widget.</p>\n"},"isVisible":{"!type":"fn(checkAncestors?: ?) -> bool","!doc":"<p>Return true if this view is visible</p>\n"},"makeTranslateStr":{"!type":"fn(to: ?) -> string","!doc":"<p>Constructs a string value that is passed to the -webkit-transform property.</p>\n"},"nextView":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the next view.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"performTransition":{"!type":"fn(moveTo: ?, transitionDir: ?, transition: ?, context: ?, method: ?)","!doc":"<p>Function to perform the various types of view transitions, such as fade, slide, and flip.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"previousView":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the previous view.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"resetScrollBar":{"!type":"fn()","!doc":"<p>Resets the scroll bar length, position, etc.</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, alignWithTop?: ?, duration?: ?)","!doc":"<p>Scrolls the pane until the searching node is in the view.</p>\n"},"scrollScrollBarTo":{"!type":"fn(to: ?)","!doc":"<p>Moves the scroll bar(s) to the given position without animation.</p>\n"},"scrollTo":{"!type":"fn(to: ?)","!doc":"<p>Overrides dojox/mobile/scrollable.scrollTo().</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setKeyframes":{"!type":"fn(from: ?, to: ?, idx: ?)","!doc":"<p>Programmatically sets key frames for the scroll animation.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)","!doc":"<p>Sets the given node as selectable or unselectable.</p>\n"},"show":{"!type":"fn(noEvent?: ?, doNotHideOthers?: ?)","!doc":"<p>Shows this view without a transition animation.</p>\n"},"showScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar.</p>\n"},"slideScrollBarTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Moves the scroll bar(s) to the given position with the slide animation.</p>\n"},"slideTo":{"!type":"fn(to: ?, duration: ?, easing: ?, fake_pos?: ?)","!doc":"<p>Overrides dojox/mobile/scrollable.slideTo().</p>\n"},"startup":{"!type":"fn()"},"stopAnimation":{"!type":"fn()","!doc":"<p>Stops the currently running animation.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/View":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget that represents a view that occupies the full screen</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"keepScrollPos":{"!doc":"<p>If true, the scroll position is kept when transition occurs between views.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"selected":{"!doc":"<p>If true, the view is displayed at startup time.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>The name of the HTML tag to create as domNode. The default value is &quot;div&quot;.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addTransitionInfo":{"!type":"fn()"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, node?: ?)","!doc":"<p>Creates a new instance of the class.</p>\n"},"convertToId":{"!type":"fn(moveTo: ?) -> ?"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getShowingView":{"!type":"fn() -> ?","!doc":"<p>Find the currently showing view from my sibling views.</p>\n"},"getSiblingViews":{"!type":"fn() -> []","!doc":"<p>Returns an array of the sibling views.</p>\n"},"getTransitionInfo":{"!type":"fn()"},"handleFragIds":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hides this view without a transition animation.</p>\n"},"invokeCallback":{"!type":"fn()","!doc":"<p>A function to be called after performing a transition to\ncall a specified callback.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isVisible":{"!type":"fn(checkAncestors?: ?) -> bool","!doc":"<p>Return true if this view is visible</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"performTransition":{"!type":"fn(moveTo: ?, transitionDir: ?, transition: ?, context: ?, method: ?)","!doc":"<p>Function to perform the various types of view transitions, such as fade, slide, and flip.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setFragIds":{"!type":"fn()"},"show":{"!type":"fn(noEvent?: ?, doNotHideOthers?: ?)","!doc":"<p>Shows this view without a transition animation.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"wakeUp":{"!type":"fn(node: ?)","!doc":"<p>Function to force IE to redraw a node since its layout\ncode tends to misrender in partial draws.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ViewController":{"!type":"fn()","prototype":{"!doc":"<p>A singleton class that controls view transition.</p>\n","dataHandlerClass":{"!doc":"<p>The data handler class used to load external views,\nby default &quot;dojox/mobile/dh/DataHandler&quot;\n(see the Data Handlers page in the reference documentation).</p>\n"},"dataSourceClass":{"!doc":"<p>The data source class used to load external views,\nby default &quot;dojox/mobile/dh/UrlDataSource&quot;\n(see the Data Handlers page in the reference documentation).</p>\n"},"fileTypeMapClass":{"!doc":"<p>The file type map class used to load external views,\nby default &quot;dojox/mobile/dh/SuffixFileTypeMap&quot;\n(see the Data Handlers page in the reference documentation).</p>\n"},"findTransitionViews":{"!type":"fn(moveTo: ?) -> []","!doc":"<p>Parses the moveTo argument and determines a starting view and a destination view.</p>\n"},"openExternalView":{"!type":"fn(transOpts: ?, target: ?) -> +dojo","!doc":"<p>Loads an external view and performs a transition to it.</p>\n"}},"_instance":{},"getInstance":{"!type":"fn()"}},"dojox/mobile/ProgressIndicator":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A progress indication widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.   </p>\n"},"center":{"!doc":"<p>If true, the indicator is displayed as center aligned.</p>\n"},"class":{},"colors":{"!doc":"<p>An array of indicator colors. 12 colors have to be given.\nIf colors are not specified, CSS styles\n(mblProg0Color - mblProg11Color) are used.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"interval":{"!doc":"<p>The time interval in milliseconds for updating the spinning\nindicator.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"removeOnStop":{"!doc":"<p>If true, this widget is removed from the parent node\nwhen stop() is called.</p>\n"},"size":{"!doc":"<p>The size of the indicator in pixels.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"startSpinning":{"!doc":"<p>If true, calls start() to run the indicator at startup.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Creates a new instance of the class.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"scale":{"!type":"fn(size: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setImage":{"!type":"fn(file: ?)","!doc":"<p>Sets an indicator icon image file (typically animated GIF).\nIf null is specified, restores the default spinner.</p>\n"},"start":{"!type":"fn()","!doc":"<p>Starts the spinning of the ProgressIndicator.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"stop":{"!type":"fn()","!doc":"<p>Stops the spinning of the ProgressIndicator.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"_instance":{},"getInstance":{"!type":"fn(props: ?) -> +instance"}},"dojox/mobile/TransitionEvent":{"!type":"fn()","prototype":{"!doc":"<p>A class used to trigger view transitions.</p>\n","constructor":{"!type":"fn(target: ?, transitionOptions: ?, triggerEvent?: ?)","!doc":"<p>Creates a transition event.</p>\n"},"dispatch":{"!type":"fn()","!doc":"<p>Dispatches this transition event. Emits a &quot;startTransition&quot; event on the target.</p>\n"}}},"dojox/mobile/viewRegistry":{"!doc":"<p>A registry of existing views.</p>\n","hash":{"!doc":"<p>The object used to register views.</p>\n"},"initialView":{},"length":{"!doc":"<p>The number of registered views.</p>\n"},"add":{"!type":"fn(view: ?)","!doc":"<p>Adds a view to the registry.</p>\n"},"getChildViews":{"!type":"fn(parent: ?) -> []","!doc":"<p>Gets the children views of the specified view.</p>\n"},"getEnclosingScrollable":{"!type":"fn(node: ?) -> +dojox","!doc":"<p>Gets the dojox/mobile/scrollable object containing the specified DOM node.</p>\n"},"getEnclosingView":{"!type":"fn(node: ?) -> +dojox","!doc":"<p>Gets the view containing the specified DOM node.</p>\n"},"getParentView":{"!type":"fn(view: ?) -> +dojox","!doc":"<p>Gets the parent view of the specified view.</p>\n"},"getViews":{"!type":"fn() -> []","!doc":"<p>Gets all registered views.</p>\n"},"remove":{"!type":"fn(id: ?)","!doc":"<p>Removes a view from the registry.</p>\n"}},"dojox/mobile/viewRegistry.hash":{"!doc":"<p>The object used to register views.</p>\n"},"dojox/mobile/common":{"!doc":"<p>A common module for dojox/mobile.</p>\n","toView":{},"createDomButton":{"!type":"fn() -> ?"},"fromView":{"!type":"fn(moveTo: ?, transitionDir: ?, transition: ?, context: ?, method: ?)","!doc":"<p>Function to perform the various types of view transitions, such as fade, slide, and flip.</p>\n"}},"dojox/mobile/uacss":{"!doc":"<p>Requiring this module adds CSS classes to your document&#39;s <code>&lt;html</code>&gt; tag:\n\n</p>\n<ul>\n<li>&quot;dj_android&quot; when running on Android;</li>\n<li>&quot;dj_bb&quot; when running on BlackBerry;</li>\n<li>&quot;dj_ios&quot; when running on iOS (iPhone, iPad or iPod);</li>\n<li>&quot;dj_ios6&quot; when running on iOS6+; this class is intended for the iphone theme to detect if it must use the iOS 6 variant of the theme. Currently applies on iOS 6 or later.</li>\n<li>&quot;dj_iphone&quot; when running on iPhone, iPad or iPod (Note: will be changed in future versions to be set only on iPhone);</li>\n<li>&quot;dj_ipod&quot; when running on iPod;</li>\n<li>&quot;dj_ipad&quot; when running on iPad.</li>\n</ul>\n"},"dojox/mobile/transition":{"!doc":"<p>This is the wrapper module which loads\ndojox/css3/transit conditionally. If mblCSS3Transition\nis set to &#39;dojox/css3/transit&#39;, it will be loaded as\nthe module to conduct view transitions, otherwise this module returns null.</p>\n"},"dojox/mobile/_ScrollableMixin":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/scrollable.prototype","!doc":"<p>Mixin for widgets to have a touch scrolling capability.</p>\n","_fixedAppFooter":{},"_moved":{"!doc":"<p>Flag that signals if the user have moved in (one of) the scroll\ndirection(s) since touch start (a move under the threshold is ignored).</p>\n"},"_parentPadBorderExtentsBottom":{"!doc":"<p>For Tooltip.js.</p>\n"},"allowNestedScrolls":{"!doc":"<p>Flag to allow scrolling in nested containers, e.g. to allow ScrollableView in a SwapView.</p>\n"},"appBars":{"!doc":"<p>Enables the search for application-specific bars (header or footer).</p>\n"},"constraint":{"!doc":"<p>bounce back to the content area</p>\n"},"dirLock":{"!doc":"<p>disable the move handler if scroll starts in the unexpected direction</p>\n"},"disableFlashScrollBar":{},"fadeScrollBar":{},"fixedFooter":{"!doc":"<p>Id of the fixed footer.</p>\n"},"fixedFooterHeight":{"!doc":"<p>height of a fixed footer</p>\n"},"fixedHeader":{"!doc":"<p>Id of the fixed header.</p>\n"},"fixedHeaderHeight":{"!doc":"<p>height of a fixed header</p>\n"},"height":{"!doc":"<p>explicitly specified height of this widget (ex. &quot;300px&quot;)</p>\n"},"isLocalFooter":{"!doc":"<p>footer is view-local (as opposed to application-wide)</p>\n"},"propagatable":{"!doc":"<p>let touchstart event propagate up</p>\n"},"scrollableParams":{"!doc":"<p>Parameters for dojox/mobile/scrollable.init().</p>\n"},"scrollBar":{"!doc":"<p>show scroll bar or not</p>\n"},"scrollDir":{"!doc":"<p>v: vertical, h: horizontal, vh: both, f: flip</p>\n"},"scrollType":{"!doc":"<ul>\n<li>1: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)animation for slide animation</li>\n<li>2: use top/left style,</li>\n<li>3: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)transition for slide animation</li>\n<li>0: use default value (3 for Android, iOS6+, and BlackBerry; otherwise 1)</li>\n</ul>\n"},"threshold":{"!doc":"<p>drag threshold value in pixels</p>\n"},"touchNode":{"!doc":"<p>a node that will have touch event handlers</p>\n"},"weight":{"!doc":"<p>frictional drag</p>\n"},"abort":{"!type":"fn()","!doc":"<p>Aborts scrolling.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"adjustDestination":{"!type":"fn(to: ?, pos: ?, dim: ?) -> bool","!doc":"<p>A stub function to be overridden by subclasses.</p>\n"},"calcScrollBarPos":{"!type":"fn(to: ?) -> ?","!doc":"<p>Calculates the scroll bar position.</p>\n"},"calcSpeed":{"!type":"fn(distance: ?, time: ?) -> number","!doc":"<p>Calculate the speed given the distance and time.</p>\n"},"checkFixedBar":{"!type":"fn(node: ?, local: ?) -> ?","!doc":"<p>Checks if the given node is a fixed bar or not.</p>\n"},"cleanup":{"!type":"fn()","!doc":"<p>Uninitialize the module.</p>\n"},"createMask":{"!type":"fn()","!doc":"<p>Creates a mask for a scroll bar edge.</p>\n"},"destroy":{"!type":"fn()"},"findAppBars":{"!type":"fn()","!doc":"<p>Search for application-specific header or footer.</p>\n"},"findDisp":{"!type":"fn(node: ?) -> +null","!doc":"<p>Finds the currently displayed view node from my sibling nodes.</p>\n"},"flashScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar instantly.</p>\n"},"getDim":{"!type":"fn() -> ?","!doc":"<p>Returns various internal dimensional information needed for calculation.</p>\n"},"getPos":{"!type":"fn() -> ?","!doc":"<p>Gets the top position in the midst of animation.</p>\n"},"getScreenSize":{"!type":"fn() -> ?","!doc":"<p>Returns the dimensions of the browser window.</p>\n"},"getSpeed":{"!type":"fn() -> ?","!doc":"<p>Returns an object that indicates the scrolling speed.</p>\n"},"hideScrollBar":{"!type":"fn()","!doc":"<p>Hides the scroll bar.</p>\n"},"init":{"!type":"fn(params?: ?)","!doc":"<p>Initialize according to the given params.</p>\n"},"isFormElement":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the given node is a form control.</p>\n"},"isTopLevel":{"!type":"fn() -> bool","!doc":"<p>Returns true if this is a top-level widget.</p>\n"},"makeTranslateStr":{"!type":"fn(to: ?) -> string","!doc":"<p>Constructs a string value that is passed to the -webkit-transform property.</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"resetScrollBar":{"!type":"fn()","!doc":"<p>Resets the scroll bar length, position, etc.</p>\n"},"resize":{"!type":"fn(e: ?)","!doc":"<p>Adjusts the height of the widget.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, alignWithTop?: ?, duration?: ?)","!doc":"<p>Scrolls the pane until the searching node is in the view.</p>\n"},"scrollScrollBarTo":{"!type":"fn(to: ?)","!doc":"<p>Moves the scroll bar(s) to the given position without animation.</p>\n"},"scrollTo":{"!type":"fn(to: ?, doNotMoveScrollBar?: ?, node?: ?)","!doc":"<p>Scrolls to the given position immediately without animation.</p>\n"},"setKeyframes":{"!type":"fn(from: ?, to: ?, idx: ?)","!doc":"<p>Programmatically sets key frames for the scroll animation.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)","!doc":"<p>Sets the given node as selectable or unselectable.</p>\n"},"showScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar.</p>\n"},"slideScrollBarTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Moves the scroll bar(s) to the given position with the slide animation.</p>\n"},"slideTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Scrolls to the given position with the slide animation.</p>\n"},"startup":{"!type":"fn()"},"stopAnimation":{"!type":"fn()","!doc":"<p>Stops the currently running animation.</p>\n"}}},"dojox/mobile/CheckBox":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/ToggleButton.prototype","!doc":"<p>A non-templated checkbox widget that can be in two states \n(checked or not checked).</p>\n","_aria_attr":{},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"checked":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.\nIn markup, specified as &quot;checked=&#39;checked&#39;&quot; or just &quot;checked&quot;.\nTrue if the button is depressed, or the checkbox is checked,\nor the radio button is selected, etc.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>The duration of selection, in milliseconds, or -1 for no post-click CSS styling.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>The label of the button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> bool"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ToggleButton":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Button.prototype","!doc":"<p>A non-templated button widget that can be in two states (checked or not).\nCan be base class for things like tabs or checkbox or radio buttons</p>\n","_aria_attr":{},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{"!doc":"<p>Overrides the automatic assignment of type to nodes, because it causes\nexception on IE. Instead, the type must be specified as this.type\nwhen the node is created, as part of the original DOM.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"checked":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.\nIn markup, specified as &quot;checked=&#39;checked&#39;&quot; or just &quot;checked&quot;.\nTrue if the button is depressed, or the checkbox is checked,\nor the radio button is selected, etc.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>The duration of selection, in milliseconds, or -1 for no post-click CSS styling.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>The label of the button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> bool"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ComboBox":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/TextBox.prototype","!doc":"<p>A non-templated auto-completing text box widget.</p>\n","__skipInputEvent":{},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_stopClickEvents":{},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autoComplete":{"!doc":"<p>Flag which enables the auto-completion.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dropDown":{"!doc":"<p>The widget to display as a popup. This widget <em>must</em> be\ndefined before the startup function is called.</p>\n"},"dropDownClass":{"!doc":"<p>Name of the drop-down widget class used to select a date/time.\nShould be specified by subclasses.</p>\n"},"dropDownPosition":{"!doc":"<p>This variable controls the position of the drop-down.\nIt is an array of strings with the following values:\n\n</p>\n<ul>\n<li>before: places drop down to the left of the target node/widget, or to the right in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>after: places drop down to the right of the target node/widget, or to the left in\nthe case of RTL scripts like Hebrew and Arabic</li>\n<li>above: drop down goes above target node</li>\n<li>below: drop down goes below target node</li>\n</ul>\n<p>The list is positions is tried, in order, until a position is found where the drop down fits\nwithin the viewport.</p>\n"},"fetchProperties":{"!doc":"<p>Mixin to the store&#39;s fetch.\nFor example, to set the sort order of the ComboBox menu, pass:\n\n</p>\n<pre><code>{ sort: [{attribute:&quot;name&quot;,descending: true}] }</code></pre>\n<p>To override the default queryOptions so that deep=false, do:\n\n\n</p>\n<pre><code>{ queryOptions: {ignoreCase: true, deep: false} }</code></pre>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"highlightMatch":{"!doc":"<p>One of: &quot;first&quot;, &quot;all&quot; or &quot;none&quot;.\n\n</p>\n<p>If the ComboBox/FilteringSelect opens with the search results and the searched\nstring can be found, it will be highlighted.  If set to &quot;all&quot;\nthen will probably want to change <code>queryExpr</code> parameter to &#39;<em>${0}</em>&#39;\n\n</p>\n<p>Highlighting is only performed when <code>labelType</code> is &quot;text&quot;, so as to not\ninterfere with any HTML markup an HTML label might contain.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ignoreCase":{"!doc":"<p>Set true if the query should ignore case when matching possible items</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"item":{"!doc":"<p>This is the item returned by the dojo/store/api/Store implementation that\nprovides the data for this ComboBox, it&#39;s the currently selected item.</p>\n"},"labelAttr":{"!doc":"<p>The entries in the drop down list come from this attribute in the\ndojo.data items.\nIf not specified, the searchAttr attribute is used instead.</p>\n"},"labelType":{"!doc":"<p>Specifies how to interpret the labelAttr in the data store items.\nCan be &quot;html&quot; or &quot;text&quot;.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxHeight":{"!doc":"<p>The maximum height for the drop-down.\nAny drop-down taller than this value will have scrollbars.\nSet to -1 to limit the height to the available space in the viewport.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageSize":{"!doc":"<p>Argument to data provider.\nSpecifies maximum number of search results to return per query</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"query":{"!doc":"<p>A query that can be passed to <code>store</code> to initially filter the items.\nComboBox overwrites any reference to the <code>searchAttr</code> and sets it to the <code>queryExpr</code> with the user&#39;s input substituted.</p>\n"},"queryExpr":{"!doc":"<p>This specifies what query is sent to the data store,\nbased on what the user has typed.  Changing this expression will modify\nwhether the results are only exact matches, a &quot;starting with&quot; match,\netc.\n<code>${0}</code> will be substituted for the user text.\n<code>*</code> is used for wildcards.\n<code>${0}*</code> means &quot;starts with&quot;, <code>*${0}*</code> means &quot;contains&quot;, <code>${0}</code> means &quot;is&quot;</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchAttr":{"!doc":"<p>Search for items in the data store where this attribute (in the item)\nmatches what the user typed</p>\n"},"searchDelay":{"!doc":"<p>Delay in milliseconds between when user types something and we start\nsearching based on that value</p>\n"},"selectOnClick":{"!doc":"<p>Flag which enables the selection on click.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this ComboBox.\nThe store must accept an object hash of properties for its query. See <code>query</code> and <code>queryExpr</code> for details.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"buildRendering":{"!type":"fn()"},"closeDropDown":{"!type":"fn()","!doc":"<p>Closes the drop down on this widget</p>\n"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"doHighlight":{"!type":"fn(label: ?, find: ?) -> ?","!doc":"<p>Highlights the string entered by the user in the menu.  By default this\nhighlights the first occurrence found. Override this method\nto implement your custom highlighting.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isLoaded":{"!type":"fn() -> bool"},"loadDropDown":{"!type":"fn(callback: ?)"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"openDropDown":{"!type":"fn() -> +any","!doc":"<p>Opens the dropdown for this widget. To be called only when this.dropDown\nhas been created and is ready to display (that is, its data is loaded).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_ComboBoxMenu":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Focus-less menu for internal use in dojox/mobile/ComboBox.\nAbstract methods that must be defined externally:\n\n</p>\n<ul>\n<li>onChange: item was explicitly chosen (mousedown somewhere on the menu and mouseup somewhere on the menu);</li>\n<li>onPage: next(1) or previous(-1) button pressed.</li>\n</ul>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_messages":{"!doc":"<p>Holds &quot;next&quot; and &quot;previous&quot; text for paging buttons on drop down</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"bgIframe":{"!doc":"<p>Flag to prevent the creation of a background iframe, when appropriate. For internal usage. </p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"selected":{"!doc":"<p>currently selected node</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"clearResultList":{"!type":"fn()","!doc":"<p>Clears the entries in the drop down list, but of course keeps the previous and next buttons.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createOptions":{"!type":"fn(results: ?, options: ?, labelFunc: ?)","!doc":"<p>Fills in the items in the drop down list</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn()"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getHighlightedOption":{"!type":"fn() -> ?"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"highlightFirstOption":{"!type":"fn()","!doc":"<p>Highlight the first real item in the list (not Previous Choices).</p>\n"},"highlightLastOption":{"!type":"fn()","!doc":"<p>Highlight the last real item in the list (not More Choices).</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"selectFirstNode":{"!type":"fn()","!doc":"<p>Select the first displayed item in the list.</p>\n"},"selectLastNode":{"!type":"fn()","!doc":"<p>Select the last displayed item in the list</p>\n"},"selectNextNode":{"!type":"fn()","!doc":"<p>Select the item just below the current selection.\nIf nothing selected, select first node.</p>\n"},"selectPreviousNode":{"!type":"fn()","!doc":"<p>Select the item just above the current selection.\nIf nothing selected, select last node (if\nyou select Previous and try to keep scrolling up the list).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_ListTouchMixin":{"!type":"fn()","prototype":{"!proto":"dijit/form/_ListBase.prototype","!doc":"<p>Focus-less menu to handle touch events consistently.</p>\n","selected":{"!doc":"<p>currently selected node</p>\n"},"postCreate":{"!type":"fn()"},"selectFirstNode":{"!type":"fn()","!doc":"<p>Select the first displayed item in the list.</p>\n"},"selectLastNode":{"!type":"fn()","!doc":"<p>Select the last displayed item in the list</p>\n"},"selectNextNode":{"!type":"fn()","!doc":"<p>Select the item just below the current selection.\nIf nothing selected, select first node.</p>\n"},"selectPreviousNode":{"!type":"fn()","!doc":"<p>Select the item just above the current selection.\nIf nothing selected, select last node (if\nyou select Previous and try to keep scrolling up the list).</p>\n"}}},"dojox/mobile/TextBox":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A non-templated base class for textbox form inputs</p>\n","__skipInputEvent":{},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Container":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Pane.prototype","!doc":"<p>A simple container-type widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Pane":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A simple pane widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ContentPane":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Container.prototype","!doc":"<p>A very simple content pane to embed an HTML fragment.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>An HTML fragment to embed.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"executeScripts":{"!doc":"<p>If true, executes scripts that is found in the content.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>URL of the content to embed.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lazy":{"!doc":"<p>If true, external content specified with the href property is\nnot loaded at startup time. It can be loaded by calling load().</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>If true, runs the parser when the load completes.</p>\n"},"prog":{"!doc":"<p>If true, shows progress indicator while loading an HTML fragment\nspecified by href.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"errorHandler":{"!type":"fn(err: ?)","!doc":"<p>An error handler called when load fails.</p>\n"},"execScript":{"!type":"fn(html: ?) -> string","!doc":"<p>Finds script tags and executes the script.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"load":{"!type":"fn()","!doc":"<p>Loads external content specified with href.</p>\n"},"loadHandler":{"!type":"fn(response: ?)","!doc":"<p>A handler called when load completes.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_ContentPaneMixin":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_ExecScriptMixin.prototype","!doc":"<p>Mixin for a very simple content pane to embed an HTML fragment.</p>\n","content":{"!doc":"<p>An HTML fragment to embed.</p>\n"},"executeScripts":{"!doc":"<p>If true, executes scripts that is found in the content.</p>\n"},"href":{"!doc":"<p>URL of the content to embed.</p>\n"},"lazy":{"!doc":"<p>If true, external content specified with the href property is\nnot loaded at startup time. It can be loaded by calling load().</p>\n"},"parseOnLoad":{"!doc":"<p>If true, runs the parser when the load completes.</p>\n"},"prog":{"!doc":"<p>If true, shows progress indicator while loading an HTML fragment\nspecified by href.</p>\n"},"errorHandler":{"!type":"fn(err: ?)","!doc":"<p>An error handler called when load fails.</p>\n"},"execScript":{"!type":"fn(html: ?) -> string","!doc":"<p>Finds script tags and executes the script.</p>\n"},"load":{"!type":"fn()","!doc":"<p>Loads external content specified with href.</p>\n"},"loadHandler":{"!type":"fn(response: ?)","!doc":"<p>A handler called when load completes.</p>\n"}}},"dojox/mobile/_ExecScriptMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for providing script execution capability to content handlers.</p>\n","execScript":{"!type":"fn(html: ?) -> string","!doc":"<p>Finds script tags and executes the script.</p>\n"}}},"dojox/mobile/DataCarousel":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Carousel.prototype","!doc":"<p>A dojo/data-enabled Carousel.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>Explicitly specified height of the widget (ex. &quot;300px&quot;). If\n&quot;inherit&quot; is specified, the height is inherited from its offset\nparent.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"itemWidth":{"!doc":"<p>The number of visible items (=numVisible) is determined by\n(carousel_width / itemWidth).\nIf itemWidth is specified, numVisible is automatically calculated.\nIf resize() is called, numVisible is recalculated and the layout\nis changed accordingly.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"navButton":{"!doc":"<p>If true, navigation buttons are displyed on the title bar.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"numVisible":{"!doc":"<p>The number of visible items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageIndicator":{"!doc":"<p>If true, a page indicator, a series of small dots that indicate\nthe current page, is displayed on the title bar.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"selectable":{"!doc":"<p>If true, an item can be selected by clicking it.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>A title of the carousel to be displayed on the title bar.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"fillPages":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexByItemWidget":{"!type":"fn(w: ?) -> number","!doc":"<p>Returns the index of a given item widget.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getItemWidgetByIndex":{"!type":"fn(index: ?) -> +null","!doc":"<p>Returns the index of an item widget at a given index.</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getParentView":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the parent view of the given DOM node.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"handleViewChanged":{"!type":"fn(view: ?)","!doc":"<p>Listens to &quot;/dojox/mobile/viewChanged&quot; events.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"instantiateView":{"!type":"fn(view: ?)","!doc":"<p>Instantiates the given view.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()"},"resizeItems":{"!type":"fn()","!doc":"<p>Resizes the child items of the carousel.</p>\n"},"select":{"!type":"fn(itemWidget: ?)","!doc":"<p>Selects the given widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions?: ?) -> ?","!doc":"<p>Sets a query.</p>\n"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions?: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_DataMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for widgets to enable dojo/data data store.</p>\n","query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions?: ?) -> ?","!doc":"<p>Sets a query.</p>\n"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions?: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"}}},"dojox/mobile/DatePicker":{"!doc":"<p>A wrapper widget around SpinWheelDatePicker or ValuePickerDatePicker.\nIt should be used with the automatic theme loader, dojox/mobile/deviceTheme.\nReturns ValuePickerDatePicker when the current theme is &quot;android&quot; or &quot;holodark&quot;.\nReturns SpinWheelDatePicker otherwise.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>A wrapper widget around SpinWheelDatePicker or ValuePickerDatePicker.\nIt should be used with the automatic theme loader, dojox/mobile/deviceTheme.\nReturns ValuePickerDatePicker when the current theme is &quot;android&quot; or &quot;holodark&quot;.\nReturns SpinWheelDatePicker otherwise.</p>\n"}}},"dojox/mobile/EdgeToEdgeCategory":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/RoundRectCategory.prototype","!doc":"<p>A category header for an edge-to-edge list.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A label of the category. If the label is not specified,\ninnerHTML is used as a label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/RoundRectCategory":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A category header for a rounded rectangle list.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A label of the category. If the label is not specified,\ninnerHTML is used as a label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/EdgeToEdgeDataList":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/EdgeToEdgeList.prototype","!doc":"<p>A dojo/data-enabled version of EdgeToEdgeList.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"append":{"!doc":"<p>If true, refresh() does not clear the existing items.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>If true, the list can be reordered.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"editableMixinClass":{"!doc":"<p>The name of the mixin class.</p>\n"},"filterBoxClass":{"!doc":"<p>The name of the CSS class added to the DOM node inside which is placed the \ndojox/mobile/SearchBox created when mixing dojox/mobile/FilteredListMixin.\nThe default value is &quot;mblFilteredEdgeToEdgeListSearchBox&quot;. </p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"itemMap":{"!doc":"<p>An optional parameter mapping field names from the store to ItemList name.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"select":{"!doc":"<p>Selection mode of the list. The check mark is shown for the\nselected list item(s). The value can be &quot;single&quot;, &quot;multiple&quot;, or &quot;&quot;.\nIf &quot;single&quot;, there can be only one selected item at a time.\nIf &quot;multiple&quot;, there can be multiple selected items at a time.\nIf &quot;&quot;, the check mark is not shown.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stateful":{"!doc":"<p>If true, the last selected item remains highlighted.</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"syncWithViews":{"!doc":"<p>If true, this widget listens to view transition events to be\nsynchronized with view&#39;s visibility.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>The default animated transition effect for child items.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createListItem":{"!type":"fn(item: ?) -> +instance","!doc":"<p>Creates a list item widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>Deselects all the items.</p>\n"},"deselectItem":{"!type":"fn(item: ?)","!doc":"<p>Deselects the given item.</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"generateList":{"!type":"fn(items: ?, dataObject: ?)","!doc":"<p>Given the data, generates a list of items.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"itemRenderer":{"!type":"fn()","!doc":"<p>The class used to create list items. Default is dojox/mobile/ListItem.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"selectItem":{"!type":"fn(item: ?)","!doc":"<p>Selects the given item.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions?: ?) -> ?","!doc":"<p>Sets a query.</p>\n"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions?: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/EdgeToEdgeList":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/RoundRectList.prototype","!doc":"<p>An edge-to-edge layout list.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>If true, the list can be reordered.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"editableMixinClass":{"!doc":"<p>The name of the mixin class.</p>\n"},"filterBoxClass":{"!doc":"<p>The name of the CSS class added to the DOM node inside which is placed the \ndojox/mobile/SearchBox created when mixing dojox/mobile/FilteredListMixin.\nThe default value is &quot;mblFilteredEdgeToEdgeListSearchBox&quot;. </p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"select":{"!doc":"<p>Selection mode of the list. The check mark is shown for the\nselected list item(s). The value can be &quot;single&quot;, &quot;multiple&quot;, or &quot;&quot;.\nIf &quot;single&quot;, there can be only one selected item at a time.\nIf &quot;multiple&quot;, there can be multiple selected items at a time.\nIf &quot;&quot;, the check mark is not shown.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stateful":{"!doc":"<p>If true, the last selected item remains highlighted.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"syncWithViews":{"!doc":"<p>If true, this widget listens to view transition events to be\nsynchronized with view&#39;s visibility.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>The default animated transition effect for child items.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>Deselects all the items.</p>\n"},"deselectItem":{"!type":"fn(item: ?)","!doc":"<p>Deselects the given item.</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"selectItem":{"!type":"fn(item: ?)","!doc":"<p>Selects the given item.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/RoundRectList":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A rounded rectangle list.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>If true, the list can be reordered.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"editableMixinClass":{"!doc":"<p>The name of the mixin class.</p>\n"},"filterBoxClass":{"!doc":"<p>The name of the CSS class added to the DOM node inside which is placed the \ndojox/mobile/SearchBox created when mixing dojox/mobile/FilteredListMixin.\nThe default value is &quot;mblFilteredRoundRectListSearchBox&quot;.  </p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"select":{"!doc":"<p>Selection mode of the list. The check mark is shown for the\nselected list item(s). The value can be &quot;single&quot;, &quot;multiple&quot;, or &quot;&quot;.\nIf &quot;single&quot;, there can be only one selected item at a time.\nIf &quot;multiple&quot;, there can be multiple selected items at a time.\nIf &quot;&quot;, the check mark is not shown.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stateful":{"!doc":"<p>If true, the last selected item remains highlighted.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"syncWithViews":{"!doc":"<p>If true, this widget listens to view transition events to be\nsynchronized with view&#39;s visibility.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>The default animated transition effect for child items.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Function to simulate the borderRadius appearance on\nIE, since IE does not support this CSS style.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>Deselects all the items.</p>\n"},"deselectItem":{"!type":"fn(item: ?)","!doc":"<p>Deselects the given item.</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"redrawBorders":{"!type":"fn()","!doc":"<p>Function to adjust the creation of RoundRectLists on IE.\nRemoved undesired styles.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"selectItem":{"!type":"fn(item: ?)","!doc":"<p>Selects the given item.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_DataListMixin":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_DataMixin.prototype","!doc":"<p>Mixin for widgets to generate the list items corresponding to\nthe data provider object.</p>\n","append":{"!doc":"<p>If true, refresh() does not clear the existing items.</p>\n"},"itemMap":{"!doc":"<p>An optional parameter mapping field names from the store to ItemList name.</p>\n"},"query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"createListItem":{"!type":"fn(item: ?) -> +instance","!doc":"<p>Creates a list item widget.</p>\n"},"generateList":{"!type":"fn(items: ?, dataObject: ?)","!doc":"<p>Given the data, generates a list of items.</p>\n"},"itemRenderer":{"!type":"fn()","!doc":"<p>The class used to create list items. Default is dojox/mobile/ListItem.</p>\n"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions?: ?) -> ?","!doc":"<p>Sets a query.</p>\n"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions?: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"}}},"dojox/mobile/ListItem":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_ItemBase.prototype","!doc":"<p>An item of either RoundRectList or EdgeToEdgeList.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_delayedSelection":{},"_duration":{"!doc":"<p>Duration of selection, milliseconds.</p>\n"},"_handleClick":{"!doc":"<p>If true, this widget listens to touch events.</p>\n"},"_selClass":{},"_selEndMethod":{},"_selStartMethod":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{"!doc":"<p>Sets tabIndex to domNode.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>An alternate text for the icon image.</p>\n"},"anchorLabel":{"!doc":"<p>If true, the label text becomes a clickable anchor text. When\nthe user clicks on the text, the onAnchorLabelClicked handler is\ncalled. You can override or connect to the handler and implement\nany action. The handler has no default action.</p>\n"},"arrowClass":{"!doc":"<p>An icon to display as an arrow. The value can be either a path\nfor an image file or a class name of a DOM button.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"back":{"!doc":"<p>If true, history.back() is called when clicked.</p>\n"},"baseClass":{},"busy":{"!doc":"<p>If true, a progress indicator spins.</p>\n"},"callback":{"!doc":"<p>A callback function that is called when the transition has been\nfinished. A function reference, or name of a function in\ncontext.</p>\n"},"checkClass":{"!doc":"<p>An icon to display as a check mark. The value can be either a\npath for an image file or a class name of a DOM button.</p>\n"},"checked":{"!doc":"<p>If true, a check mark is displayed at the right of the item.</p>\n"},"class":{},"clickable":{"!doc":"<p>If true, this item becomes clickable even if a transition\ndestination (moveTo, etc.) is not specified.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"deleteIcon":{"!doc":"<p>A delete icon to display at the left of the item. The value can\nbe either a path for an image file or a class name of a DOM\nbutton.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"header":{"!doc":"<p>If true, this item is rendered as a category header.</p>\n"},"href":{"!doc":"<p>A URL of another web page to go to.</p>\n"},"hrefTarget":{"!doc":"<p>A target that specifies where to open a page specified by\nhref. The value will be passed to the 2nd argument of\nwindow.open().</p>\n"},"icon":{"!doc":"<p>An icon image to display. The value can be either a path for an\nimage file or a class name of a DOM button. If icon is not\nspecified, the iconBase parameter of the parent widget is used.</p>\n"},"iconPos":{"!doc":"<p>The position of an aggregated icon. IconPos is comma separated\nvalues like top,left,width,height (ex. &quot;0,0,29,29&quot;). If iconPos\nis not specified, the iconPos parameter of the parent widget is\nused.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A label of the item. If the label is not specified, innerHTML is\nused as a label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"moveTo":{"!doc":"<p>The id of the transition destination view which resides in the\ncurrent page.\n\n</p>\n<p>If the value has a hash sign (&#39;#&#39;) before the id (e.g. #view1)\nand the dojo/hash module is loaded by the user application, the\nview transition updates the hash in the browser URL so that the\nuser can bookmark the destination view. In this case, the user\ncan also use the browser&#39;s back/forward button to navigate\nthrough the views in the browser history.\n\n</p>\n<p>If null, transitions to a blank view.\nIf &#39;#&#39;, returns immediately without transition.</p>\n"},"noArrow":{"!doc":"<p>If true, the right hand side arrow is not displayed.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"paramsToInherit":{},"progStyle":{"!doc":"<p>A css class name to add to the progress indicator.</p>\n"},"rightIcon":{"!doc":"<p>An icon to display at the right hand side of the item. The value\ncan be either a path for an image file or a class name of a DOM\nbutton.</p>\n"},"rightIcon2":{"!doc":"<p>An icon to display at the left of the rightIcon. The value can\nbe either a path for an image file or a class name of a DOM\nbutton.</p>\n"},"rightIcon2Title":{"!doc":"<p>An alt text for the right icon2.</p>\n"},"rightIconTitle":{"!doc":"<p>An alt text for the right icon.</p>\n"},"rightText":{"!doc":"<p>A right-aligned text to display on the item.</p>\n"},"scene":{"!doc":"<p>The name of a scene. Used from dojox/mobile/app.</p>\n"},"selected":{"!doc":"<p>If true, the item is highlighted to indicate it is selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the item so users can hit the tab key to\nfocus on it.</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"toggle":{"!doc":"<p>If true, the item acts like a toggle button.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>A type of animated transition effect. You can choose from the\nstandard transition types, &quot;slide&quot;, &quot;fade&quot;, &quot;flip&quot;, or from the\nextended transition types, &quot;cover&quot;, &quot;coverv&quot;, &quot;dissolve&quot;,\n&quot;reveal&quot;, &quot;revealv&quot;, &quot;scaleIn&quot;, &quot;scaleOut&quot;, &quot;slidev&quot;,\n&quot;swirl&quot;, &quot;zoomIn&quot;, &quot;zoomOut&quot;, &quot;cube&quot;, and &quot;swap&quot;. If &quot;none&quot; is\nspecified, transition occurs immediately without animation.</p>\n"},"transitionDir":{"!doc":"<p>The transition direction. If 1, transition forward. If -1,\ntransition backward. For example, the slide transition slides\nthe view from right to left when dir == 1, and from left to\nright when dir == -1.</p>\n"},"transitionOptions":{"!doc":"<p>A hash object that holds transition options.</p>\n"},"uncheckClass":{"!doc":"<p>An icon to display as an uncheck mark. The value can be either a\npath for an image file or a class name of a DOM button.</p>\n"},"url":{"!doc":"<p>A URL of an html fragment page or JSON data that represents a\nnew view content. The view content is loaded with XHR and\ninserted in the current page. Then a view transition occurs to\nthe newly created view. The view is cached so that subsequent\nrequests would not load the content again.</p>\n"},"urlTarget":{"!doc":"<p>Node id under which a new view will be created according to the\nurl parameter. If not specified, The new view will be created as\na sibling of the current view.</p>\n"},"variableHeight":{"!doc":"<p>If true, the height of the item varies according to its content.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an ongoing selection (if any).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defaultClickAction":{"!type":"fn(e: ?)","!doc":"<p>The default action of this item.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getTransOpts":{"!type":"fn() -> string","!doc":"<p>Copies from the parent and returns the values of parameters<br>specified by the property paramsToInherit.</p>\n"},"handleSelection":{"!type":"fn(e: ?)","!doc":"<p>Handles this items selection state.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"inheritParams":{"!type":"fn() -> bool","!doc":"<p>Copies from the parent the values of parameters specified \nby the property paramsToInherit.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layoutChildren":{"!type":"fn()"},"layoutVariableHeight":{"!type":"fn()","!doc":"<p>Lays out the current item with variable height.</p>\n"},"makeTransition":{"!type":"fn(e: ?)","!doc":"<p>Makes a transition.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()"},"set":{"!type":"fn(key: ?, value: ?)"},"setArrow":{"!type":"fn()","!doc":"<p>Sets the arrow icon if necessary.</p>\n"},"setTransitionPos":{"!type":"fn(e: ?)","!doc":"<p>Stores the clicked position for later use.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"transitionTo":{"!type":"fn(moveTo: ?, href: ?, url: ?, scene: ?)","!doc":"<p>Performs a view transition.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"userClickAction":{"!type":"fn(e: ?)","!doc":"<p>User-defined click action.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a dojox/mobile/ListItem.</p>\n"}},"dojox/mobile/ListItem.ChildWidgetProperties":{"!doc":"<p>These properties can be specified for the children of a dojox/mobile/ListItem.</p>\n","layout":{"!doc":"<p>Specifies the position of the ListItem child (&quot;left&quot;, &quot;center&quot; or &quot;right&quot;).</p>\n"},"preventTouch":{"!doc":"<p>Disables touch events on the ListItem child.</p>\n"}},"dojox/mobile/_ItemBase":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A base class for item classes (e.g. ListItem, IconItem, etc.).</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_delayedSelection":{"!doc":"<p>If true, selection is delayed 100ms and canceled if dragged in\norder to avoid selection when flick operation is performed.</p>\n"},"_duration":{"!doc":"<p>Duration of selection, milliseconds.</p>\n"},"_handleClick":{"!doc":"<p>If true, this widget listens to touch events.</p>\n"},"_selEndMethod":{"!doc":"<p>Specifies how the item leaves the selected state.\n\n</p>\n<ul>\n<li>&quot;touch&quot;: Use touch events to leave the selected state.</li>\n<li>&quot;timer&quot;: Use setTimeout to leave the selected state.</li>\n<li>&quot;none&quot;: Do not change the selected state.</li>\n</ul>\n"},"_selStartMethod":{"!doc":"<p>Specifies how the item enters the selected state.\n\n</p>\n<ul>\n<li>&quot;touch&quot;: Use touch events to enter the selected state.</li>\n<li>&quot;none&quot;: Do not change the selected state.</li>\n</ul>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{"!doc":"<p>Sets tabIndex to domNode.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>An alternate text for the icon image.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"back":{"!doc":"<p>If true, history.back() is called when clicked.</p>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"callback":{"!doc":"<p>A callback function that is called when the transition has been\nfinished. A function reference, or name of a function in\ncontext.</p>\n"},"class":{},"clickable":{"!doc":"<p>If true, this item becomes clickable even if a transition\ndestination (moveTo, etc.) is not specified.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>A URL of another web page to go to.</p>\n"},"hrefTarget":{"!doc":"<p>A target that specifies where to open a page specified by\nhref. The value will be passed to the 2nd argument of\nwindow.open().</p>\n"},"icon":{"!doc":"<p>An icon image to display. The value can be either a path for an\nimage file or a class name of a DOM button. If icon is not\nspecified, the iconBase parameter of the parent widget is used.</p>\n"},"iconPos":{"!doc":"<p>The position of an aggregated icon. IconPos is comma separated\nvalues like top,left,width,height (ex. &quot;0,0,29,29&quot;). If iconPos\nis not specified, the iconPos parameter of the parent widget is\nused.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A label of the item. If the label is not specified, innerHTML is\nused as a label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"moveTo":{"!doc":"<p>The id of the transition destination view which resides in the\ncurrent page.\n\n</p>\n<p>If the value has a hash sign (&#39;#&#39;) before the id (e.g. #view1)\nand the dojo/hash module is loaded by the user application, the\nview transition updates the hash in the browser URL so that the\nuser can bookmark the destination view. In this case, the user\ncan also use the browser&#39;s back/forward button to navigate\nthrough the views in the browser history.\n\n</p>\n<p>If null, transitions to a blank view.\nIf &#39;#&#39;, returns immediately without transition.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"paramsToInherit":{"!doc":"<p>Comma separated parameters to inherit from the parent.</p>\n"},"scene":{"!doc":"<p>The name of a scene. Used from dojox/mobile/app.</p>\n"},"selected":{"!doc":"<p>If true, the item is highlighted to indicate it is selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the item so users can hit the tab key to\nfocus on it.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"toggle":{"!doc":"<p>If true, the item acts like a toggle button.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>A type of animated transition effect. You can choose from the\nstandard transition types, &quot;slide&quot;, &quot;fade&quot;, &quot;flip&quot;, or from the\nextended transition types, &quot;cover&quot;, &quot;coverv&quot;, &quot;dissolve&quot;,\n&quot;reveal&quot;, &quot;revealv&quot;, &quot;scaleIn&quot;, &quot;scaleOut&quot;, &quot;slidev&quot;,\n&quot;swirl&quot;, &quot;zoomIn&quot;, &quot;zoomOut&quot;, &quot;cube&quot;, and &quot;swap&quot;. If &quot;none&quot; is\nspecified, transition occurs immediately without animation.</p>\n"},"transitionDir":{"!doc":"<p>The transition direction. If 1, transition forward. If -1,\ntransition backward. For example, the slide transition slides\nthe view from right to left when dir == 1, and from left to\nright when dir == -1.</p>\n"},"transitionOptions":{"!doc":"<p>A hash object that holds transition options.</p>\n"},"url":{"!doc":"<p>A URL of an html fragment page or JSON data that represents a\nnew view content. The view content is loaded with XHR and\ninserted in the current page. Then a view transition occurs to\nthe newly created view. The view is cached so that subsequent\nrequests would not load the content again.</p>\n"},"urlTarget":{"!doc":"<p>Node id under which a new view will be created according to the\nurl parameter. If not specified, The new view will be created as\na sibling of the current view.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an ongoing selection (if any).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defaultClickAction":{"!type":"fn(e: ?)","!doc":"<p>The default action of this item.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getTransOpts":{"!type":"fn() -> string","!doc":"<p>Copies from the parent and returns the values of parameters<br>specified by the property paramsToInherit.</p>\n"},"handleSelection":{"!type":"fn(e: ?)","!doc":"<p>Handles this items selection state.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"inheritParams":{"!type":"fn() -> bool","!doc":"<p>Copies from the parent the values of parameters specified \nby the property paramsToInherit.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeTransition":{"!type":"fn(e: ?)","!doc":"<p>Makes a transition.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setTransitionPos":{"!type":"fn(e: ?)","!doc":"<p>Stores the clicked position for later use.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"transitionTo":{"!type":"fn(moveTo: ?, href: ?, url: ?, scene: ?)","!doc":"<p>Performs a view transition.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"userClickAction":{"!type":"fn(e: ?)","!doc":"<p>User-defined click action.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/EdgeToEdgeStoreList":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/EdgeToEdgeList.prototype","!doc":"<p>A dojo/store-enabled version of EdgeToEdgeList.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"append":{"!doc":"<p>If true, refresh() does not clear the existing items.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"childrenProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s children.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>If true, the list can be reordered.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"editableMixinClass":{"!doc":"<p>The name of the mixin class.</p>\n"},"filterBoxClass":{"!doc":"<p>The name of the CSS class added to the DOM node inside which is placed the \ndojox/mobile/SearchBox created when mixing dojox/mobile/FilteredListMixin.\nThe default value is &quot;mblFilteredEdgeToEdgeListSearchBox&quot;. </p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"itemMap":{"!doc":"<p>An optional parameter mapping field names from the store to ItemList names.\nExample: itemMap:{text:&#39;label&#39;, profile_image_url:&#39;icon&#39;}</p>\n"},"labelProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"select":{"!doc":"<p>Selection mode of the list. The check mark is shown for the\nselected list item(s). The value can be &quot;single&quot;, &quot;multiple&quot;, or &quot;&quot;.\nIf &quot;single&quot;, there can be only one selected item at a time.\nIf &quot;multiple&quot;, there can be multiple selected items at a time.\nIf &quot;&quot;, the check mark is not shown.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stateful":{"!doc":"<p>If true, the last selected item remains highlighted.</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"syncWithViews":{"!doc":"<p>If true, this widget listens to view transition events to be\nsynchronized with view&#39;s visibility.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>The default animated transition effect for child items.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createListItem":{"!type":"fn(item: ?) -> +instance","!doc":"<p>Creates a list item widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>Deselects all the items.</p>\n"},"deselectItem":{"!type":"fn(item: ?)","!doc":"<p>Deselects the given item.</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"generateList":{"!type":"fn(items: ?)","!doc":"<p>Given the data, generates a list of items.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"itemRenderer":{"!type":"fn()","!doc":"<p>The class used to create list items. Default is dojox/mobile/ListItem.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"selectItem":{"!type":"fn(item: ?)","!doc":"<p>Selects the given item.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions: ?) -> ?"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_StoreListMixin":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_StoreMixin.prototype","!doc":"<p>Mixin for widgets to generate the list items corresponding to\nthe dojo/store data provider object.</p>\n","append":{"!doc":"<p>If true, refresh() does not clear the existing items.</p>\n"},"childrenProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s children.</p>\n"},"itemMap":{"!doc":"<p>An optional parameter mapping field names from the store to ItemList names.\nExample: itemMap:{text:&#39;label&#39;, profile_image_url:&#39;icon&#39;}</p>\n"},"labelProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s label.</p>\n"},"query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"createListItem":{"!type":"fn(item: ?) -> +instance","!doc":"<p>Creates a list item widget.</p>\n"},"destroy":{"!type":"fn()"},"generateList":{"!type":"fn(items: ?)","!doc":"<p>Given the data, generates a list of items.</p>\n"},"itemRenderer":{"!type":"fn()","!doc":"<p>The class used to create list items. Default is dojox/mobile/ListItem.</p>\n"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions: ?) -> ?"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"}}},"dojox/mobile/_StoreMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for widgets to enable dojo/store data store.</p>\n","childrenProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s children.</p>\n"},"labelProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s label.</p>\n"},"query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"destroy":{"!type":"fn()"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions: ?) -> ?"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"}}},"dojox/mobile/ExpandingTextArea":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/TextArea.prototype","!doc":"<p>Non-templated TEXTAREA widget with the capability to adjust its \nheight according to the amount of data.</p>\n","__skipInputEvent":{},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Resizes the textarea vertically (should be called after a style/value change)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/TextArea":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/TextBox.prototype","!doc":"<p>Non-templated TEXTAREA widget.</p>\n","__skipInputEvent":{},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5).\nThis should only contain plain text (no html markup).</p>\n"},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/FilteredListMixin":{"!type":"fn()","prototype":{"!doc":"<p>Mixin for filtered lists.</p>\n","_createdFilterBox":{"!doc":"<p>The instance of dojox/mobile/SearchBox created by this mixin, or null if none\nhas been created. Stored for being able to destroy it together with the list widget.</p>\n"},"_createdScrollableView":{"!doc":"<p>The instance of dojox/mobile/ScrollableView created by this mixin, if any. \nStored for getScrollableView() and for being able to destroy it together \nwith the list widget.</p>\n"},"_filterBox":{"!doc":"<p>The instance of dojox/mobile/SearchBox used by this mixin. \nStored for getFilterBox().</p>\n"},"filterBoxRef":{"!doc":"<p>The reference for the search box allowing to enter the filtering criteria.\nOnly used at construction time:\n- If unspecified, the mixin creates a dojox/mobile/SearchBox and \na dojox/mobile/ScrollableView. The list is placed inside the ScrollableView and the\nSearchBox, wrapped in a DIV, is placed on top of the ScrollableView.\n- If the string is the id of a widget which is an instance of dojox/mobile/SearchBox \nor a subclass, the mixin uses this SearchBox for filtering the list.\n- If the id is specified but does not reference a dojox/mobile/SearchBox or \nsubclass, an error is thrown. </p>\n"},"filterBoxVisible":{"!doc":"<p>A flag which allows to show or hide the dojox/mobile/SearchBox associated with\nthe list.</p>\n"},"placeHolder":{"!doc":"<p>Defines a hint to help users fill out the input field (as defined in HTML 5) of the \ndojox/mobile/SearchBox. This should only contain plain text (no HTML markup).\nWhen the SearchBox is provided by the user (not created by this mixin), its placeHolder\nproperty takes precedence.</p>\n"},"destroy":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the widget. If the list has created dojox/mobile/SearchBox \nor dojox/mobile/ScrollableView widgets, these widgets are also destroyed.</p>\n"},"getFilterBox":{"!type":"fn() -> ?","!doc":"<p>Returns the dojox/mobile/SearchBox widget used for entering the filtering criteria.\nIf an instance has been referenced at construction time using the property filterBoxRef,\nthis instance is returned. Otherwise, returns the instance created by the mixin.\nThis function allows the user to get the instance of SearchBox in order to customize\nits parameters. </p>\n"},"getScrollableView":{"!type":"fn() -> ?","!doc":"<p>Returns the instance of dojox/mobile/ScrollableView created by this mixin,\nor null if none has been created. The mixin creates a ScrollableView if and\nonly if the property filterBoxRef is unspecified.\nThis function allows the user to get the instance of ScrollableView in order to\ncustomize its parameters.</p>\n"},"startup":{"!type":"fn()"}}},"dojox/mobile/SearchBox":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/TextBox.prototype","!doc":"<p>A non-templated base class for INPUT type=&quot;search&quot;.</p>\n","__skipInputEvent":{},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_blankValue":{},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"displayedValue":{"!doc":"<p>For subclasses like ComboBox where the displayed value\n(ex: Kentucky) and the serialized value (ex: KY) are different,\nthis represents the displayed value.\n\n</p>\n<p>Setting &#39;displayedValue&#39; through set(&#39;displayedValue&#39;, ...)\nupdates &#39;value&#39;, and vice-versa.  Otherwise &#39;value&#39; is updated\nfrom &#39;displayedValue&#39; periodically, like onBlur etc.\n\n</p>\n<p>TODO: move declaration to MappedTextBox?\nProblem is that ComboBox references displayedValue,\nfor benefit of FilteringSelect.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fetchProperties":{"!doc":"<p>Mixin to the store&#39;s fetch.\nFor example, to set the sort order of the ComboBox menu, pass:\n\n</p>\n<pre><code>{ sort: [{attribute:&quot;name&quot;,descending: true}] }</code></pre>\n<p>To override the default queryOptions so that deep=false, do:\n\n\n</p>\n<pre><code>{ queryOptions: {ignoreCase: true, deep: false} }</code></pre>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ignoreCase":{"!doc":"<p>Set true if the query should ignore case when matching possible items</p>\n"},"incremental":{"!doc":"<p>Set true to search on every key or false to only search after \npressing ENTER or cancel.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lowercase":{"!doc":"<p>Converts all characters to lowercase if true.  Default is false.</p>\n"},"maxLength":{"!doc":"<p>HTML INPUT tag maxLength declaration.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageSize":{"!doc":"<p>Argument to data provider.\nSpecifies maximum number of search results to return per query</p>\n"},"placeHolder":{},"propercase":{"!doc":"<p>Converts the first character of each word to uppercase if true.</p>\n"},"query":{"!doc":"<p>A query that can be passed to <code>store</code> to initially filter the items.\nComboBox overwrites any reference to the <code>searchAttr</code> and sets it to the <code>queryExpr</code> with the user&#39;s input substituted.</p>\n"},"queryExpr":{"!doc":"<p>This specifies what query is sent to the data store,\nbased on what the user has typed.  Changing this expression will modify\nwhether the results are only exact matches, a &quot;starting with&quot; match,\netc.\n<code>${0}</code> will be substituted for the user text.\n<code>*</code> is used for wildcards.\n<code>${0}*</code> means &quot;starts with&quot;, <code>*${0}*</code> means &quot;contains&quot;, <code>${0}</code> means &quot;is&quot;</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"searchAttr":{"!doc":"<p>Search for items in the data store where this attribute (in the item)\nmatches what the user typed</p>\n"},"searchDelay":{"!doc":"<p>Delay in milliseconds between when user types something and we start\nsearching based on that value</p>\n"},"selectOnClick":{"!doc":"<p>If true, all text will be selected when focused with mouse</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this ComboBox.\nThe store must accept an object hash of properties for its query. See <code>query</code> and <code>queryExpr</code> for details.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"trim":{"!doc":"<p>Removes leading and trailing whitespace if true.  Default is false.</p>\n"},"type":{"!doc":"<p>Corresponds to the type attribute of the HTML <code>&lt;input&gt;</code> element.\nThe value is &quot;search&quot;.</p>\n"},"uppercase":{"!doc":"<p>Converts all characters to uppercase if true.  Default is false.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"filter":{"!type":"fn(val: ?) -> ?","!doc":"<p>Auto-corrections (such as trimming) that are applied to textbox\nvalue on blur or form submit.</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"format":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a value to a properly formatted string.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parse":{"!type":"fn(value: ?, constraints: ?) -> string","!doc":"<p>Replaceable function to convert a formatted string to a value</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ScrollableView":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/View.prototype","!doc":"<p>A container that has a touch scrolling capability.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_fixedAppFooter":{},"_moved":{"!doc":"<p>Flag that signals if the user have moved in (one of) the scroll\ndirection(s) since touch start (a move under the threshold is ignored).</p>\n"},"_parentPadBorderExtentsBottom":{"!doc":"<p>For Tooltip.js.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"allowNestedScrolls":{"!doc":"<p>Flag to allow scrolling in nested containers, e.g. to allow ScrollableView in a SwapView.</p>\n"},"appBars":{"!doc":"<p>Enables the search for application-specific bars (header or footer).</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraint":{"!doc":"<p>bounce back to the content area</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dirLock":{"!doc":"<p>disable the move handler if scroll starts in the unexpected direction</p>\n"},"disableFlashScrollBar":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fadeScrollBar":{},"fixedFooter":{"!doc":"<p>Id of the fixed footer.</p>\n"},"fixedFooterHeight":{"!doc":"<p>height of a fixed footer</p>\n"},"fixedHeader":{"!doc":"<p>Id of the fixed header.</p>\n"},"fixedHeaderHeight":{"!doc":"<p>height of a fixed header</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>explicitly specified height of this widget (ex. &quot;300px&quot;)</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLocalFooter":{"!doc":"<p>footer is view-local (as opposed to application-wide)</p>\n"},"keepScrollPos":{"!doc":"<p>Overrides dojox/mobile/View/keepScrollPos.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"propagatable":{"!doc":"<p>let touchstart event propagate up</p>\n"},"scrollableParams":{"!doc":"<p>Parameters for dojox/mobile/scrollable.init().</p>\n"},"scrollBar":{"!doc":"<p>show scroll bar or not</p>\n"},"scrollDir":{"!doc":"<p>v: vertical, h: horizontal, vh: both, f: flip</p>\n"},"scrollType":{"!doc":"<ul>\n<li>1: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)animation for slide animation</li>\n<li>2: use top/left style,</li>\n<li>3: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)transition for slide animation</li>\n<li>0: use default value (3 for Android, iOS6+, and BlackBerry; otherwise 1)</li>\n</ul>\n"},"selected":{"!doc":"<p>If true, the view is displayed at startup time.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>The name of the HTML tag to create as domNode. The default value is &quot;div&quot;.</p>\n"},"threshold":{"!doc":"<p>drag threshold value in pixels</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"touchNode":{"!doc":"<p>a node that will have touch event handlers</p>\n"},"weight":{"!doc":"<p>frictional drag</p>\n"},"abort":{"!type":"fn()","!doc":"<p>Aborts scrolling.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"addFixedBar":{"!type":"fn(widget: ?)","!doc":"<p>Adds a view local fixed bar to this widget.</p>\n"},"adjustDestination":{"!type":"fn(to: ?, pos: ?, dim: ?) -> bool","!doc":"<p>A stub function to be overridden by subclasses.</p>\n"},"buildRendering":{"!type":"fn()"},"calcScrollBarPos":{"!type":"fn(to: ?) -> ?","!doc":"<p>Calculates the scroll bar position.</p>\n"},"calcSpeed":{"!type":"fn(distance: ?, time: ?) -> number","!doc":"<p>Calculate the speed given the distance and time.</p>\n"},"checkFixedBar":{"!type":"fn(node: ?, local: ?) -> ?","!doc":"<p>Checks if the given node is a fixed bar or not.</p>\n"},"cleanup":{"!type":"fn()","!doc":"<p>Uninitialize the module.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Creates a new instance of the class.</p>\n"},"convertToId":{"!type":"fn(moveTo: ?) -> ?"},"createMask":{"!type":"fn()","!doc":"<p>Creates a mask for a scroll bar edge.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"findAppBars":{"!type":"fn()","!doc":"<p>Search for application-specific header or footer.</p>\n"},"findDisp":{"!type":"fn(node: ?) -> +null","!doc":"<p>Finds the currently displayed view node from my sibling nodes.</p>\n"},"flashScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar instantly.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> ?","!doc":"<p>Overrides _WidgetBase.getChildren to add local fixed bars,\nwhich are not under containerNode, to the children array.</p>\n"},"getDim":{"!type":"fn() -> ?","!doc":"<p>Returns various internal dimensional information needed for calculation.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPos":{"!type":"fn() -> ?","!doc":"<p>Gets the top position in the midst of animation.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getScreenSize":{"!type":"fn() -> ?","!doc":"<p>Returns the dimensions of the browser window.</p>\n"},"getShowingView":{"!type":"fn() -> ?","!doc":"<p>Find the currently showing view from my sibling views.</p>\n"},"getSiblingViews":{"!type":"fn() -> []","!doc":"<p>Returns an array of the sibling views.</p>\n"},"getSpeed":{"!type":"fn() -> ?","!doc":"<p>Returns an object that indicates the scrolling speed.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hides this view without a transition animation.</p>\n"},"hideScrollBar":{"!type":"fn()","!doc":"<p>Hides the scroll bar.</p>\n"},"init":{"!type":"fn(params?: ?)","!doc":"<p>Initialize according to the given params.</p>\n"},"invokeCallback":{"!type":"fn()","!doc":"<p>A function to be called after performing a transition to\ncall a specified callback.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isFormElement":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the given node is a form control.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isTopLevel":{"!type":"fn(e: ?) -> bool","!doc":"<p>Returns true if this is a top-level widget.\nOverrides dojox/mobile/scrollable.isTopLevel.</p>\n"},"isVisible":{"!type":"fn(checkAncestors?: ?) -> bool","!doc":"<p>Return true if this view is visible</p>\n"},"makeTranslateStr":{"!type":"fn(to: ?) -> string","!doc":"<p>Constructs a string value that is passed to the -webkit-transform property.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"performTransition":{"!type":"fn(moveTo: ?, transitionDir: ?, transition: ?, context: ?, method: ?)","!doc":"<p>Function to perform the various types of view transitions, such as fade, slide, and flip.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"reparent":{"!type":"fn()","!doc":"<p>Moves all the children, except header and footer, to\ncontainerNode.</p>\n"},"resetScrollBar":{"!type":"fn()","!doc":"<p>Resets the scroll bar length, position, etc.</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, alignWithTop?: ?, duration?: ?)","!doc":"<p>Scrolls the pane until the searching node is in the view.</p>\n"},"scrollScrollBarTo":{"!type":"fn(to: ?)","!doc":"<p>Moves the scroll bar(s) to the given position without animation.</p>\n"},"scrollTo":{"!type":"fn(to: ?, doNotMoveScrollBar?: ?, node?: ?)","!doc":"<p>Scrolls to the given position immediately without animation.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setKeyframes":{"!type":"fn(from: ?, to: ?, idx: ?)","!doc":"<p>Programmatically sets key frames for the scroll animation.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)","!doc":"<p>Sets the given node as selectable or unselectable.</p>\n"},"show":{"!type":"fn(noEvent?: ?, doNotHideOthers?: ?)","!doc":"<p>Shows this view without a transition animation.</p>\n"},"showScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar.</p>\n"},"slideScrollBarTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Moves the scroll bar(s) to the given position with the slide animation.</p>\n"},"slideTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Scrolls to the given position with the slide animation.</p>\n"},"startup":{"!type":"fn()"},"stopAnimation":{"!type":"fn()","!doc":"<p>Stops the currently running animation.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/FixedSplitter":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A layout container that splits the window horizontally or\nvertically.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"orientation":{"!doc":"<p>The direction of split. If &quot;H&quot; is specified, panes are split\nhorizontally. If &quot;V&quot; is specified, panes are split vertically.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"screenSizeAware":{"!doc":"<p>If true, dynamically load a screen-size-aware module.</p>\n"},"screenSizeAwareClass":{"!doc":"<p>A screen-size-aware module to load.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"variablePane":{"!doc":"<p>The index of a pane that fills the remainig space.\nIf -1, the last child pane fills the remaining space.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/FixedSplitterPane":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Container.prototype","!doc":"<p>Deprecated widget. Use dojox/mobile/Container instead.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/FormLayout":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Container.prototype","!doc":"<p>A responsive container to create mobile forms.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"columns":{"!doc":"<p>This property controls how a <label> and its <fieldset> are positioned. The <label> can be on the same line\nthan its <fieldset> (two columns) or on top of it (single column).\nIf set to &quot;auto&quot;, the number of columns depends on the width of the screen: Two columns\nif the width of the screen is larger than 500px, one column otherwise. The width of the screen is determined using CSS\nMedia Queries.\nSetting this property to &quot;single&quot; or &quot;two&quot; allows to force the layout used whatever the width of the screen.\nDefault value for this property is &quot;auto&quot;.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"rightAlign":{"!doc":"<p>This property controls the horizontal position of control(s) in a <fieldset>. It applies only\nto forms that have two columns (see &#39;columns&#39; property).\nDefault value for this property is false.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/GridLayout":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/IconMenu.prototype","!doc":"<p>A container widget that places its children in a grid layout.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_createTerminator":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tags":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"childItemClass":{"!doc":"<p>The name of the CSS class of grid items.</p>\n"},"class":{},"cols":{"!doc":"<p>The number of child items in a row.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"selectOne":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>The default animated transition effect for child items.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn()"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/IconMenu":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A pop-up menu.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_createTerminator":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"childItemClass":{"!doc":"<p>The name of the CSS class of menu items.</p>\n"},"class":{},"cols":{"!doc":"<p>The number of child items in a row.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"selectOne":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>The default animated transition effect for child items.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn()"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/IconMenuItem":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_ItemBase.prototype","!doc":"<p>An item of IconMenu.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_delayedSelection":{"!doc":"<p>If true, selection is delayed 100ms and canceled if dragged in\norder to avoid selection when flick operation is performed.</p>\n"},"_duration":{"!doc":"<p>Duration of selection, milliseconds.</p>\n"},"_handleClick":{"!doc":"<p>If true, this widget listens to touch events.</p>\n"},"_selEndMethod":{},"_selStartMethod":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{"!doc":"<p>Sets tabIndex to domNode.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>An alternate text for the icon image.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"back":{"!doc":"<p>If true, history.back() is called when clicked.</p>\n"},"baseClass":{},"callback":{"!doc":"<p>A callback function that is called when the transition has been\nfinished. A function reference, or name of a function in\ncontext.</p>\n"},"class":{},"clickable":{"!doc":"<p>If true, this item becomes clickable even if a transition\ndestination (moveTo, etc.) is not specified.</p>\n"},"closeOnAction":{"!doc":"<p>If true, the internal handler of click events calls the hide() method \nof the parent widget, which is typically a dojox/mobile/SimpleDialog.\nThe default value is false.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>A URL of another web page to go to.</p>\n"},"hrefTarget":{"!doc":"<p>A target that specifies where to open a page specified by\nhref. The value will be passed to the 2nd argument of\nwindow.open().</p>\n"},"icon":{"!doc":"<p>An icon image to display. The value can be either a path for an\nimage file or a class name of a DOM button. If icon is not\nspecified, the iconBase parameter of the parent widget is used.</p>\n"},"iconPos":{"!doc":"<p>The position of an aggregated icon. IconPos is comma separated\nvalues like top,left,width,height (ex. &quot;0,0,29,29&quot;). If iconPos\nis not specified, the iconPos parameter of the parent widget is\nused.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A label of the item. If the label is not specified, innerHTML is\nused as a label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"moveTo":{"!doc":"<p>The id of the transition destination view which resides in the\ncurrent page.\n\n</p>\n<p>If the value has a hash sign (&#39;#&#39;) before the id (e.g. #view1)\nand the dojo/hash module is loaded by the user application, the\nview transition updates the hash in the browser URL so that the\nuser can bookmark the destination view. In this case, the user\ncan also use the browser&#39;s back/forward button to navigate\nthrough the views in the browser history.\n\n</p>\n<p>If null, transitions to a blank view.\nIf &#39;#&#39;, returns immediately without transition.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"paramsToInherit":{"!doc":"<p>Comma separated parameters to inherit from the parent.</p>\n"},"scene":{"!doc":"<p>The name of a scene. Used from dojox/mobile/app.</p>\n"},"selColor":{},"selected":{"!doc":"<p>If true, the item is highlighted to indicate it is selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the item so users can hit the tab key to\nfocus on it.</p>\n"},"tag":{"!doc":"<p>The name of the HTML tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"toggle":{"!doc":"<p>If true, the item acts like a toggle button.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>A type of animated transition effect. You can choose from the\nstandard transition types, &quot;slide&quot;, &quot;fade&quot;, &quot;flip&quot;, or from the\nextended transition types, &quot;cover&quot;, &quot;coverv&quot;, &quot;dissolve&quot;,\n&quot;reveal&quot;, &quot;revealv&quot;, &quot;scaleIn&quot;, &quot;scaleOut&quot;, &quot;slidev&quot;,\n&quot;swirl&quot;, &quot;zoomIn&quot;, &quot;zoomOut&quot;, &quot;cube&quot;, and &quot;swap&quot;. If &quot;none&quot; is\nspecified, transition occurs immediately without animation.</p>\n"},"transitionDir":{"!doc":"<p>The transition direction. If 1, transition forward. If -1,\ntransition backward. For example, the slide transition slides\nthe view from right to left when dir == 1, and from left to\nright when dir == -1.</p>\n"},"transitionOptions":{"!doc":"<p>A hash object that holds transition options.</p>\n"},"url":{"!doc":"<p>A URL of an html fragment page or JSON data that represents a\nnew view content. The view content is loaded with XHR and\ninserted in the current page. Then a view transition occurs to\nthe newly created view. The view is cached so that subsequent\nrequests would not load the content again.</p>\n"},"urlTarget":{"!doc":"<p>Node id under which a new view will be created according to the\nurl parameter. If not specified, The new view will be created as\na sibling of the current view.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an ongoing selection (if any).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defaultClickAction":{"!type":"fn(e: ?)","!doc":"<p>The default action of this item.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getTransOpts":{"!type":"fn() -> string","!doc":"<p>Copies from the parent and returns the values of parameters<br>specified by the property paramsToInherit.</p>\n"},"handleSelection":{"!type":"fn(e: ?)","!doc":"<p>Handles this items selection state.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"inheritParams":{"!type":"fn() -> bool","!doc":"<p>Copies from the parent the values of parameters specified \nby the property paramsToInherit.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeTransition":{"!type":"fn(e: ?)","!doc":"<p>Makes a transition.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setTransitionPos":{"!type":"fn(e: ?)","!doc":"<p>Stores the clicked position for later use.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"transitionTo":{"!type":"fn(moveTo: ?, href: ?, url: ?, scene: ?)","!doc":"<p>Performs a view transition.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"userClickAction":{"!type":"fn(e: ?)","!doc":"<p>User-defined click action.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Heading":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget that represents a navigation bar.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"back":{"!doc":"<p>A label for the navigational control to return to the previous View.</p>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.   </p>\n"},"busy":{"!doc":"<p>If true, a progress indicator spins on this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>A URL to open when the navigational control is pressed.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A title text of the heading. If the label is not specified, the\ninnerHTML of the node is used as a label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"moveTo":{"!doc":"<p>The id of the transition destination of the navigation control.\nIf the value has a hash sign (&#39;#&#39;) before the id (e.g. #view1)\nand the dojox/mobile/bookmarkable module is loaded by the user application,\nthe view transition updates the hash in the browser URL so that the\nuser can bookmark the destination view. In this case, the user\ncan also use the browser&#39;s back/forward button to navigate\nthrough the views in the browser history.\n\n</p>\n<p>If null, transitions to a blank view.\nIf &#39;#&#39;, returns immediately without transition.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"progStyle":{"!doc":"<p>A css class name to add to the progress indicator.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>A name of HTML tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>A type of animated transition effect. You can choose from the\nstandard transition types, &quot;slide&quot;, &quot;fade&quot;, &quot;flip&quot;, or from the\nextended transition types, &quot;cover&quot;, &quot;coverv&quot;, &quot;dissolve&quot;,\n&quot;reveal&quot;, &quot;revealv&quot;, &quot;scaleIn&quot;, &quot;scaleOut&quot;, &quot;slidev&quot;,\n&quot;swirl&quot;, &quot;zoomIn&quot;, &quot;zoomOut&quot;, &quot;cube&quot;, and &quot;swap&quot;. If &quot;none&quot; is\nspecified, transition occurs immediately without animation.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ToolBarButton":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_ItemBase.prototype","!doc":"<p>A button widget which is placed in the Heading widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_delayedSelection":{"!doc":"<p>If true, selection is delayed 100ms and canceled if dragged in\norder to avoid selection when flick operation is performed.</p>\n"},"_duration":{"!doc":"<p>Duration of selection, milliseconds.</p>\n"},"_handleClick":{"!doc":"<p>If true, this widget listens to touch events.</p>\n"},"_selEndMethod":{},"_selStartMethod":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{"!doc":"<p>Sets tabIndex to domNode.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>An alternate text for the icon image.</p>\n"},"arrow":{"!doc":"<p>Specifies &quot;right&quot; or &quot;left&quot; to be an arrow button.\nNote that changing the value of the property after the widget \ncreation has no effect.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"back":{"!doc":"<p>If true, history.back() is called when clicked.</p>\n"},"baseClass":{},"callback":{"!doc":"<p>A callback function that is called when the transition has been\nfinished. A function reference, or name of a function in\ncontext.</p>\n"},"class":{},"clickable":{"!doc":"<p>If true, this item becomes clickable even if a transition\ndestination (moveTo, etc.) is not specified.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultColor":{"!doc":"<p>CSS class for the default color.\nNote: If this button has an arrow (typically back buttons on iOS),\nthe class selector used for it is the value of defaultColor + &quot;45&quot;.\nFor example, by default the arrow selector is &quot;mblColorDefault45&quot;.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>A URL of another web page to go to.</p>\n"},"hrefTarget":{"!doc":"<p>A target that specifies where to open a page specified by\nhref. The value will be passed to the 2nd argument of\nwindow.open().</p>\n"},"icon":{"!doc":"<p>An icon image to display. The value can be either a path for an\nimage file or a class name of a DOM button. If icon is not\nspecified, the iconBase parameter of the parent widget is used.</p>\n"},"iconPos":{"!doc":"<p>The position of an aggregated icon. IconPos is comma separated\nvalues like top,left,width,height (ex. &quot;0,0,29,29&quot;). If iconPos\nis not specified, the iconPos parameter of the parent widget is\nused.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A label of the item. If the label is not specified, innerHTML is\nused as a label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"light":{"!doc":"<p>If true, this widget produces only a single <code>&lt;span&gt;</code> node when it\nhas only an icon or only a label, and has no arrow. In that\ncase, you cannot have both icon and label, or arrow even if you\ntry to set them.\nNote that changing the value of the property after the widget \ncreation has no effect.</p>\n"},"moveTo":{"!doc":"<p>The id of the transition destination view which resides in the\ncurrent page.\n\n</p>\n<p>If the value has a hash sign (&#39;#&#39;) before the id (e.g. #view1)\nand the dojo/hash module is loaded by the user application, the\nview transition updates the hash in the browser URL so that the\nuser can bookmark the destination view. In this case, the user\ncan also use the browser&#39;s back/forward button to navigate\nthrough the views in the browser history.\n\n</p>\n<p>If null, transitions to a blank view.\nIf &#39;#&#39;, returns immediately without transition.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"paramsToInherit":{"!doc":"<p>Comma separated parameters to inherit from the parent.</p>\n"},"scene":{"!doc":"<p>The name of a scene. Used from dojox/mobile/app.</p>\n"},"selColor":{"!doc":"<p>CSS class for the selected color.\nNote: If this button has an arrow (typically back buttons on iOS),\nthe class selector used for it is the value of selColor + &quot;45&quot;.\nFor example, by default the selected arrow selector is &quot;mblColorDefaultSel45&quot;.</p>\n"},"selected":{"!doc":"<p>If true, the button is in the selected state.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the item so users can hit the tab key to\nfocus on it.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"toggle":{"!doc":"<p>If true, the item acts like a toggle button.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>A type of animated transition effect. You can choose from the\nstandard transition types, &quot;slide&quot;, &quot;fade&quot;, &quot;flip&quot;, or from the\nextended transition types, &quot;cover&quot;, &quot;coverv&quot;, &quot;dissolve&quot;,\n&quot;reveal&quot;, &quot;revealv&quot;, &quot;scaleIn&quot;, &quot;scaleOut&quot;, &quot;slidev&quot;,\n&quot;swirl&quot;, &quot;zoomIn&quot;, &quot;zoomOut&quot;, &quot;cube&quot;, and &quot;swap&quot;. If &quot;none&quot; is\nspecified, transition occurs immediately without animation.</p>\n"},"transitionDir":{"!doc":"<p>The transition direction. If 1, transition forward. If -1,\ntransition backward. For example, the slide transition slides\nthe view from right to left when dir == 1, and from left to\nright when dir == -1.</p>\n"},"transitionOptions":{"!doc":"<p>A hash object that holds transition options.</p>\n"},"url":{"!doc":"<p>A URL of an html fragment page or JSON data that represents a\nnew view content. The view content is loaded with XHR and\ninserted in the current page. Then a view transition occurs to\nthe newly created view. The view is cached so that subsequent\nrequests would not load the content again.</p>\n"},"urlTarget":{"!doc":"<p>Node id under which a new view will be created according to the\nurl parameter. If not specified, The new view will be created as\na sibling of the current view.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an ongoing selection (if any).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defaultClickAction":{"!type":"fn(e: ?)","!doc":"<p>The default action of this item.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselect":{"!type":"fn()"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getTransOpts":{"!type":"fn() -> string","!doc":"<p>Copies from the parent and returns the values of parameters<br>specified by the property paramsToInherit.</p>\n"},"handleSelection":{"!type":"fn(e: ?)","!doc":"<p>Handles this items selection state.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"inheritParams":{"!type":"fn() -> bool","!doc":"<p>Copies from the parent the values of parameters specified \nby the property paramsToInherit.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeTransition":{"!type":"fn(e: ?)","!doc":"<p>Makes a transition.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"select":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setTransitionPos":{"!type":"fn(e: ?)","!doc":"<p>Stores the clicked position for later use.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"transitionTo":{"!type":"fn(moveTo: ?, href: ?, url: ?, scene: ?)","!doc":"<p>Performs a view transition.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"userClickAction":{"!type":"fn(e: ?)","!doc":"<p>User-defined click action.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Icon":{"!type":"fn()","prototype":{"!doc":"<p>A wrapper for image icon, CSS sprite icon, or DOM Button.</p>\n","alt":{"!doc":"<p>An alt text for the icon image.\nNote that changing the value of the property after the icon\ncreation has no effect.</p>\n"},"icon":{"!doc":"<p>An icon to display. The value can be either a path for an image\nfile or a class name of a DOM button.\nNote that changing the value of the property after the icon\ncreation has no effect.</p>\n"},"iconPos":{"!doc":"<p>The position of an aggregated icon. IconPos is comma separated\nvalues like top,left,width,height (ex. &quot;0,0,29,29&quot;).\nNote that changing the value of the property after the icon\ncreation has no effect.</p>\n"},"tag":{"!doc":"<p>The name of the HTML tag to create as this.domNode.</p>\n"},"constructor":{"!type":"fn(args?: ?, node?: ?)","!doc":"<p>Creates a new instance of the class.</p>\n"}}},"dojox/mobile/IconContainer":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A container widget which can hold multiple icons.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"back":{"!doc":"<p>A label for the navigational control.</p>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultIcon":{"!doc":"<p>The default fallback icon, which is displayed only when the\nspecified icon has failed to load.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>If true, the icons can be removed or reordered. You can enter\ninto edit mode by pressing on a child IconItem until it starts shaking.\nThe default value is false. Note that changing the value of the property after\nthe widget creation has no effect.</p>\n"},"editableMixinClass":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconItemPaneClass":{},"iconItemPaneContainerClass":{},"iconItemPaneContainerProps":{},"iconItemPaneProps":{},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A title text of the heading.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pressedIconOpacity":{"!doc":"<p>The opacity of the pressed icon image. The default value is 0.4.</p>\n"},"single":{"!doc":"<p>If true, only one icon content can be opened at a time.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>A type of animated transition effect. You can choose from the\nstandard transition types: &quot;slide&quot;, &quot;fade&quot;, &quot;flip&quot;, or from the\nextended transition types: &quot;cover&quot;, &quot;coverv&quot;, &quot;dissolve&quot;,\n&quot;reveal&quot;, &quot;revealv&quot;, &quot;scaleIn&quot;, &quot;scaleOut&quot;, &quot;slidev&quot;,\n&quot;swirl&quot;, &quot;zoomIn&quot;, &quot;zoomOut&quot;, &quot;cube&quot;, and &quot;swap&quot;. If &quot;none&quot; is\nspecified, the transition occurs immediately without animation. If\n&quot;below&quot; is specified, the application contents are displayed\nbelow the icons. The default value is &quot;below&quot;. Note that changing \nthe value of the property after the widget creation has no effect.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"buildRendering":{"!type":"fn()"},"closeAll":{"!type":"fn()","!doc":"<p>Closes all the icon items.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/IconItem":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_ItemBase.prototype","!doc":"<p>An icon item widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_delayedSelection":{"!doc":"<p>If true, selection is delayed 100ms and canceled if dragged in\norder to avoid selection when flick operation is performed.</p>\n"},"_duration":{"!doc":"<p>Duration of selection, milliseconds.</p>\n"},"_handleClick":{"!doc":"<p>If true, this widget listens to touch events.</p>\n"},"_selEndMethod":{},"_selStartMethod":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{"!doc":"<p>Sets tabIndex to domNode.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>An alternate text for the icon image.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"back":{"!doc":"<p>If true, history.back() is called when clicked.</p>\n"},"badge":{"!doc":"<p>A text to show in a badge (ex. &quot;55&quot;).</p>\n"},"badgeClass":{"!doc":"<p>A class name of a DOM button for a badge.</p>\n"},"baseClass":{},"callback":{"!doc":"<p>A callback function that is called when the transition has been\nfinished. A function reference, or name of a function in\ncontext.</p>\n"},"class":{},"clickable":{"!doc":"<p>If true, this item becomes clickable even if a transition\ndestination (moveTo, etc.) is not specified.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>An HTML fragment to embed as icon content.</p>\n"},"deletable":{"!doc":"<p>If true, you can delete this IconItem by clicking on the delete\nicon during edit mode.\nIf false, the delete icon is not displayed during edit mode so\nthat it cannot be deleted.</p>\n"},"deleteIcon":{"!doc":"<p>A delete icon to display at the top-left corner of the item\nduring edit mode. The value can be either a path for an image\nfile or a class name of a DOM button.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>A URL of another web page to go to.</p>\n"},"hrefTarget":{"!doc":"<p>A target that specifies where to open a page specified by\nhref. The value will be passed to the 2nd argument of\nwindow.open().</p>\n"},"icon":{"!doc":"<p>An icon image to display. The value can be either a path for an\nimage file or a class name of a DOM button. If icon is not\nspecified, the iconBase parameter of the parent widget is used.</p>\n"},"iconPos":{"!doc":"<p>The position of an aggregated icon. IconPos is comma separated\nvalues like top,left,width,height (ex. &quot;0,0,29,29&quot;). If iconPos\nis not specified, the iconPos parameter of the parent widget is\nused.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A label of the item. If the label is not specified, innerHTML is\nused as a label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lazy":{"!doc":"<p>If true, the content of the widget, which includes dojo markup,\nis instantiated lazily. That is, only when the widget is opened\nby the user, the required modules are loaded and the content\nwidgets are instantiated.\nThis option works both in the sync and async loader mode.</p>\n"},"moveTo":{"!doc":"<p>The id of the transition destination view which resides in the\ncurrent page.\n\n</p>\n<p>If the value has a hash sign (&#39;#&#39;) before the id (e.g. #view1)\nand the dojo/hash module is loaded by the user application, the\nview transition updates the hash in the browser URL so that the\nuser can bookmark the destination view. In this case, the user\ncan also use the browser&#39;s back/forward button to navigate\nthrough the views in the browser history.\n\n</p>\n<p>If null, transitions to a blank view.\nIf &#39;#&#39;, returns immediately without transition.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"paramsToInherit":{},"requires":{"!doc":"<p>Comma-separated required module names to be lazily loaded. This\nproperty is effective only when lazy=true. All the modules\nspecified with data-dojo-type and their depending modules are\nautomatically loaded by the IconItem when it is opened.\nHowever, if you need other extra modules to be loaded, use this parameter.\nThis option works both in the sync and async loader mode.</p>\n"},"scene":{"!doc":"<p>The name of a scene. Used from dojox/mobile/app.</p>\n"},"selected":{"!doc":"<p>If true, the item is highlighted to indicate it is selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the item so users can hit the tab key to\nfocus on it.</p>\n"},"tag":{"!doc":"<p>A name of the HTML tag to create as domNode.</p>\n"},"timeout":{"!doc":"<p>Duration of highlight in seconds.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"toggle":{"!doc":"<p>If true, the item acts like a toggle button.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>A type of animated transition effect. You can choose from the\nstandard transition types, &quot;slide&quot;, &quot;fade&quot;, &quot;flip&quot;, or from the\nextended transition types, &quot;cover&quot;, &quot;coverv&quot;, &quot;dissolve&quot;,\n&quot;reveal&quot;, &quot;revealv&quot;, &quot;scaleIn&quot;, &quot;scaleOut&quot;, &quot;slidev&quot;,\n&quot;swirl&quot;, &quot;zoomIn&quot;, &quot;zoomOut&quot;, &quot;cube&quot;, and &quot;swap&quot;. If &quot;none&quot; is\nspecified, transition occurs immediately without animation.</p>\n"},"transitionDir":{"!doc":"<p>The transition direction. If 1, transition forward. If -1,\ntransition backward. For example, the slide transition slides\nthe view from right to left when dir == 1, and from left to\nright when dir == -1.</p>\n"},"transitionOptions":{"!doc":"<p>A hash object that holds transition options.</p>\n"},"url":{"!doc":"<p>A URL of an html fragment page or JSON data that represents a\nnew view content. The view content is loaded with XHR and\ninserted in the current page. Then a view transition occurs to\nthe newly created view. The view is cached so that subsequent\nrequests would not load the content again.</p>\n"},"urlTarget":{"!doc":"<p>Node id under which a new view will be created according to the\nurl parameter. If not specified, The new view will be created as\na sibling of the current view.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an ongoing selection (if any).</p>\n"},"close":{"!type":"fn(noAnimation?: ?)","!doc":"<p>Closes the icon content.</p>\n"},"closeIconClicked":{"!type":"fn(e: ?)","!doc":"<p>User-defined function to handle clicks for the close icon.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defaultClickAction":{"!type":"fn(e: ?)","!doc":"<p>The default action of this item.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getTransOpts":{"!type":"fn() -> string","!doc":"<p>Copies from the parent and returns the values of parameters<br>specified by the property paramsToInherit.</p>\n"},"handleSelection":{"!type":"fn(e: ?)","!doc":"<p>Handles this items selection state.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"highlight":{"!type":"fn(timeout?: ?)","!doc":"<p>Shakes the icon 10 seconds.</p>\n"},"inheritParams":{"!type":"fn() -> bool","!doc":"<p>Copies from the parent the values of parameters specified \nby the property paramsToInherit.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isOpen":{"!type":"fn(e: ?) -> ?","!doc":"<p>Returns true if the icon is open.</p>\n"},"makeTransition":{"!type":"fn(e: ?)","!doc":"<p>Makes a transition.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"open":{"!type":"fn(e: ?)","!doc":"<p>Opens the icon content, or makes a transition.</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"scrollIntoView":{"!type":"fn(node: ?)","!doc":"<p>Scrolls until the given node is in the view.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setTransitionPos":{"!type":"fn(e: ?)","!doc":"<p>Stores the clicked position for later use.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"transitionTo":{"!type":"fn(moveTo: ?, href: ?, url: ?, scene: ?)","!doc":"<p>Performs a view transition.</p>\n"},"unhighlight":{"!type":"fn()","!doc":"<p>Stops shaking the icon.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"userClickAction":{"!type":"fn(e: ?)","!doc":"<p>User-defined click action.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/LongListMixin":{"!type":"fn()","prototype":{"!doc":"<p>This mixin enhances performance of very long lists contained in scrollable views.</p>\n","maxPages":{"!doc":"<p>When this limit is reached, previous pages will be unloaded.</p>\n"},"pageSize":{"!doc":"<p>Items are loaded in the DOM by chunks of this size.</p>\n"},"unloadPages":{"!doc":"<p>Number of pages that will be unloaded when maxPages is reached.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"generateList":{"!type":"fn(items: ?)","!doc":"<p>Overrides dojox.mobile._StoreListMixin when the list is a store list.</p>\n"},"getChildren":{"!type":"fn() -> ?"},"removeChild":{"!type":"fn(widget: ?)"},"resize":{"!type":"fn()"},"startup":{"!type":"fn()"}}},"dojox/mobile/Opener":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Tooltip.prototype","!doc":"<p>A non-templated popup widget that will use either Tooltip or \nOverlay depending on screen size.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"lazy":{"!doc":"<p>If true, the content of the widget, which includes dojo markup,\nis instantiated lazily. That is, only when the widget is opened\nby the user, the required modules are loaded and the content\nwidgets are instantiated.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"requires":{"!doc":"<p>Comma-separated required module names to be lazily loaded. This\nis effective only when lazy=true. All the modules specified with\ndojoType and their depending modules are automatically loaded\nwhen the widget is opened. However, if you need other extra\nmodules to be loaded, use this parameter.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn(val: ?)"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"show":{"!type":"fn(node: ?, positions: ?) -> ?"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Tooltip":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A non-templated popup bubble widget</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Pop down the tooltip</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"show":{"!type":"fn(aroundNode: ?, positions: ?) -> ?","!doc":"<p>Pop up the tooltip and point to aroundNode using the best position</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Overlay":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A non-templated widget that animates up from the bottom, \noverlaying the current content.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Scroll the overlay down and then make it invisible</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"show":{"!type":"fn(aroundNode?: ?) -> ?","!doc":"<p>Scroll the overlay up into view</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ProgressBar":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget that shows the progress of a task.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A text to be shown at the center of the progress bar.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maximum":{"!doc":"<p>Maximum value.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>Number (&quot;0&quot; to maximum) or percentage (&quot;0%&quot; to &quot;100%&quot;)\nindicating the degree of completion of the task.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/RadioButton":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/CheckBox.prototype","!doc":"<p>A non-templated radio button widget that can be in two states (checked or not checked).</p>\n","_aria_attr":{},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_setTypeAttr":{"!doc":"<p>Overrides the automatic assignment of type to nodes, because it causes\nexception on IE. Instead, the type must be specified as this.type\nwhen the node is created, as part of the original DOM.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"checked":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.\nIn markup, specified as &quot;checked=&#39;checked&#39;&quot; or just &quot;checked&quot;.\nTrue if the button is depressed, or the checkbox is checked,\nor the radio button is selected, etc.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>The duration of selection, in milliseconds, or -1 for no post-click CSS styling.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"label":{"!doc":"<p>The label of the button.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> bool"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Rating":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget that displays a rating, usually with stars.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>An alternate text for the icon image.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"image":{"!doc":"<p>Path to a star image, which includes three stars, full star,\nempty star, and half star, from left to right.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"numStars":{"!doc":"<p>The number of stars to show.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The current value of the Rating.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/RoundRect":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Container.prototype","!doc":"<p>A simple round rectangle container.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"shadow":{"!doc":"<p>If true, adds a shadow effect to the container element by adding\nthe CSS class &quot;mblShadow&quot; to widget&#39;s domNode. The default value\nis false. Note that changing the value of the property after\nthe widget creation has no effect.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Function to simulate the borderRadius appearance on\nIE, since IE does not support this CSS style.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/RoundRectDataList":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/RoundRectList.prototype","!doc":"<p>A dojo/data-enabled version of RoundRectList.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"append":{"!doc":"<p>If true, refresh() does not clear the existing items.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>If true, the list can be reordered.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"editableMixinClass":{"!doc":"<p>The name of the mixin class.</p>\n"},"filterBoxClass":{"!doc":"<p>The name of the CSS class added to the DOM node inside which is placed the \ndojox/mobile/SearchBox created when mixing dojox/mobile/FilteredListMixin.\nThe default value is &quot;mblFilteredRoundRectListSearchBox&quot;.  </p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"itemMap":{"!doc":"<p>An optional parameter mapping field names from the store to ItemList name.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"select":{"!doc":"<p>Selection mode of the list. The check mark is shown for the\nselected list item(s). The value can be &quot;single&quot;, &quot;multiple&quot;, or &quot;&quot;.\nIf &quot;single&quot;, there can be only one selected item at a time.\nIf &quot;multiple&quot;, there can be multiple selected items at a time.\nIf &quot;&quot;, the check mark is not shown.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stateful":{"!doc":"<p>If true, the last selected item remains highlighted.</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"syncWithViews":{"!doc":"<p>If true, this widget listens to view transition events to be\nsynchronized with view&#39;s visibility.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>The default animated transition effect for child items.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createListItem":{"!type":"fn(item: ?) -> +instance","!doc":"<p>Creates a list item widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>Deselects all the items.</p>\n"},"deselectItem":{"!type":"fn(item: ?)","!doc":"<p>Deselects the given item.</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"generateList":{"!type":"fn(items: ?, dataObject: ?)","!doc":"<p>Given the data, generates a list of items.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"itemRenderer":{"!type":"fn()","!doc":"<p>The class used to create list items. Default is dojox/mobile/ListItem.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"selectItem":{"!type":"fn(item: ?)","!doc":"<p>Selects the given item.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions?: ?) -> ?","!doc":"<p>Sets a query.</p>\n"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions?: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/RoundRectStoreList":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/RoundRectList.prototype","!doc":"<p>A dojo/store-enabled version of RoundRectList.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"append":{"!doc":"<p>If true, refresh() does not clear the existing items.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"childrenProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s children.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"editable":{"!doc":"<p>If true, the list can be reordered.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"editableMixinClass":{"!doc":"<p>The name of the mixin class.</p>\n"},"filterBoxClass":{"!doc":"<p>The name of the CSS class added to the DOM node inside which is placed the \ndojox/mobile/SearchBox created when mixing dojox/mobile/FilteredListMixin.\nThe default value is &quot;mblFilteredRoundRectListSearchBox&quot;.  </p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"itemMap":{"!doc":"<p>An optional parameter mapping field names from the store to ItemList names.\nExample: itemMap:{text:&#39;label&#39;, profile_image_url:&#39;icon&#39;}</p>\n"},"labelProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"select":{"!doc":"<p>Selection mode of the list. The check mark is shown for the\nselected list item(s). The value can be &quot;single&quot;, &quot;multiple&quot;, or &quot;&quot;.\nIf &quot;single&quot;, there can be only one selected item at a time.\nIf &quot;multiple&quot;, there can be multiple selected items at a time.\nIf &quot;&quot;, the check mark is not shown.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stateful":{"!doc":"<p>If true, the last selected item remains highlighted.</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"syncWithViews":{"!doc":"<p>If true, this widget listens to view transition events to be\nsynchronized with view&#39;s visibility.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>The default animated transition effect for child items.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createListItem":{"!type":"fn(item: ?) -> +instance","!doc":"<p>Creates a list item widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"deselectAll":{"!type":"fn()","!doc":"<p>Deselects all the items.</p>\n"},"deselectItem":{"!type":"fn(item: ?)","!doc":"<p>Deselects the given item.</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"generateList":{"!type":"fn(items: ?)","!doc":"<p>Given the data, generates a list of items.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"itemRenderer":{"!type":"fn()","!doc":"<p>The class used to create list items. Default is dojox/mobile/ListItem.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"selectItem":{"!type":"fn(item: ?)","!doc":"<p>Selects the given item.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions: ?) -> ?"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ScreenSizeAware":{"!type":"fn()","prototype":{"!doc":"<p>A module to make a screen size aware application.</p>\n","leftListId":{"!doc":"<p>The id of the list widget in the left view.</p>\n"},"leftPaneId":{"!doc":"<p>The id of the left pane.</p>\n"},"leftViewId":{"!doc":"<p>The id of the left View.</p>\n"},"rightPaneId":{"!doc":"<p>The id of the right pane.</p>\n"},"splitterId":{"!doc":"<p>The id of the FixedSplitter.</p>\n"},"constructor":{"!type":"fn(options?: ?)","!doc":"<p>Creates a new instance of the class.</p>\n"},"getDestinationId":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the id of the target view of the given item.</p>\n"},"getShowingView":{"!type":"fn() -> +null","!doc":"<p>Returns the view currently shown.</p>\n"},"init":{"!type":"fn()","!doc":"<p>Initializes the application.</p>\n"},"isPhone":{"!type":"fn() -> bool","!doc":"<p>Returns true if the current mode set by transformUI(mode) is &quot;phone&quot;.</p>\n"},"leftItemSelected":{"!type":"fn(e: ?)","!doc":"<p>Function called when an item in the left-side list is selected.</p>\n"},"moveList":{"!type":"fn()","!doc":"<p>Places the list widget. If the current mode is &quot;phone&quot;, it \nplaces the list widget in the right pane, otherwise in the left pane. </p>\n"},"showLeftView":{"!type":"fn()","!doc":"<p>Shows the left-side view.</p>\n"},"showRightView":{"!type":"fn()","!doc":"<p>Shows the right-side view.</p>\n"},"transformUI":{"!type":"fn(mode: ?)","!doc":"<p>Applies an UI mode.</p>\n"},"updateBackButton":{"!type":"fn()","!doc":"<p>Updates the back button.</p>\n"},"updateSelectedItem":{"!type":"fn()","!doc":"<p>Updates the selected item.</p>\n"},"updateStateful":{"!type":"fn()","!doc":"<p>Updates the stateful property of the list widget in the left-side pane.</p>\n"},"updateTransition":{"!type":"fn()","!doc":"<p>Updates the transition property of the items in the left-side widget. </p>\n"}},"_instance":{},"getInstance":{"!type":"fn()"}},"dojox/mobile/ScrollablePane":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Pane.prototype","!doc":"<p>A pane that has the touch-scrolling capability.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_fixedAppFooter":{},"_moved":{"!doc":"<p>Flag that signals if the user have moved in (one of) the scroll\ndirection(s) since touch start (a move under the threshold is ignored).</p>\n"},"_parentPadBorderExtentsBottom":{"!doc":"<p>For Tooltip.js.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"allowNestedScrolls":{"!doc":"<p>Flag to allow scrolling in nested containers, e.g. to allow ScrollableView in a SwapView.</p>\n"},"appBars":{"!doc":"<p>Enables the search for application-specific bars (header or footer).</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"constraint":{"!doc":"<p>bounce back to the content area</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dirLock":{"!doc":"<p>disable the move handler if scroll starts in the unexpected direction</p>\n"},"disableFlashScrollBar":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fadeScrollBar":{},"fixedFooter":{"!doc":"<p>Id of the fixed footer.</p>\n"},"fixedFooterHeight":{"!doc":"<p>height of a fixed footer</p>\n"},"fixedHeader":{"!doc":"<p>Id of the fixed header.</p>\n"},"fixedHeaderHeight":{"!doc":"<p>height of a fixed header</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>explicitly specified height of this widget (ex. &quot;300px&quot;)</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLocalFooter":{"!doc":"<p>footer is view-local (as opposed to application-wide)</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"propagatable":{"!doc":"<p>let touchstart event propagate up</p>\n"},"radius":{"!doc":"<p>Radius of the rounded corner mask.</p>\n"},"roundCornerMask":{"!doc":"<p>If true, creates a rounded corner mask to clip corners of a \nchild widget or DOM node. Works only on WebKit-based browsers.</p>\n"},"scrollableParams":{"!doc":"<p>Parameters for dojox/mobile/scrollable.init().</p>\n"},"scrollBar":{"!doc":"<p>show scroll bar or not</p>\n"},"scrollDir":{"!doc":"<p>v: vertical, h: horizontal, vh: both, f: flip</p>\n"},"scrollType":{"!doc":"<ul>\n<li>1: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)animation for slide animation</li>\n<li>2: use top/left style,</li>\n<li>3: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)transition for slide animation</li>\n<li>0: use default value (3 for Android, iOS6+, and BlackBerry; otherwise 1)</li>\n</ul>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"threshold":{"!doc":"<p>drag threshold value in pixels</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"touchNode":{"!doc":"<p>a node that will have touch event handlers</p>\n"},"weight":{"!doc":"<p>frictional drag</p>\n"},"abort":{"!type":"fn()","!doc":"<p>Aborts scrolling.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"adjustDestination":{"!type":"fn(to: ?, pos: ?, dim: ?) -> bool","!doc":"<p>A stub function to be overridden by subclasses.</p>\n"},"buildRendering":{"!type":"fn()"},"calcScrollBarPos":{"!type":"fn(to: ?) -> ?","!doc":"<p>Calculates the scroll bar position.</p>\n"},"calcSpeed":{"!type":"fn(distance: ?, time: ?) -> number","!doc":"<p>Calculate the speed given the distance and time.</p>\n"},"checkFixedBar":{"!type":"fn(node: ?, local: ?) -> ?","!doc":"<p>Checks if the given node is a fixed bar or not.</p>\n"},"cleanup":{"!type":"fn()","!doc":"<p>Uninitialize the module.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createMask":{"!type":"fn()","!doc":"<p>Creates a mask for a scroll bar edge.</p>\n"},"createRoundMask":{"!type":"fn()","!doc":"<p>Creates a rounded corner rectangle mask.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"findAppBars":{"!type":"fn()","!doc":"<p>Search for application-specific header or footer.</p>\n"},"findDisp":{"!type":"fn(node: ?) -> +null","!doc":"<p>Finds the currently displayed view node from my sibling nodes.</p>\n"},"flashScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar instantly.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDim":{"!type":"fn() -> ?","!doc":"<p>Returns various internal dimensional information needed for calculation.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPos":{"!type":"fn() -> ?","!doc":"<p>Gets the top position in the midst of animation.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getScreenSize":{"!type":"fn() -> ?","!doc":"<p>Returns the dimensions of the browser window.</p>\n"},"getSpeed":{"!type":"fn() -> ?","!doc":"<p>Returns an object that indicates the scrolling speed.</p>\n"},"hideScrollBar":{"!type":"fn()","!doc":"<p>Hides the scroll bar.</p>\n"},"init":{"!type":"fn(params?: ?)","!doc":"<p>Initialize according to the given params.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isFormElement":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the given node is a form control.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isTopLevel":{"!type":"fn(e: ?) -> bool","!doc":"<p>Returns true if this is a top-level widget.\nOverrides dojox/mobile/scrollable.</p>\n"},"makeTranslateStr":{"!type":"fn(to: ?) -> string","!doc":"<p>Constructs a string value that is passed to the -webkit-transform property.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"resetScrollBar":{"!type":"fn()","!doc":"<p>Resets the scroll bar length, position, etc.</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, alignWithTop?: ?, duration?: ?)","!doc":"<p>Scrolls the pane until the searching node is in the view.</p>\n"},"scrollScrollBarTo":{"!type":"fn(to: ?)","!doc":"<p>Moves the scroll bar(s) to the given position without animation.</p>\n"},"scrollTo":{"!type":"fn(to: ?, doNotMoveScrollBar?: ?, node?: ?)","!doc":"<p>Scrolls to the given position immediately without animation.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setKeyframes":{"!type":"fn(from: ?, to: ?, idx: ?)","!doc":"<p>Programmatically sets key frames for the scroll animation.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)","!doc":"<p>Sets the given node as selectable or unselectable.</p>\n"},"showScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar.</p>\n"},"slideScrollBarTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Moves the scroll bar(s) to the given position with the slide animation.</p>\n"},"slideTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Scrolls to the given position with the slide animation.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"stopAnimation":{"!type":"fn()","!doc":"<p>Stops the currently running animation.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/SimpleDialog":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Pane.prototype","!doc":"<p>A dialog box for mobile.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_cover":{"!doc":"<p>Array for sharing the cover instances.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{"!doc":"<p>Sets tabIndex to domNode.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"closeButton":{"!doc":"<p>If true, a button to close the dialog box is displayed at the\ntop-right corner.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"closeButtonClass":{"!doc":"<p>A class name of a DOM button to be used as a close button.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"left":{"!doc":"<p>The left edge position of the widget. If &quot;auto&quot;, the widget is\nplaced at the center of the screen. Otherwise, the value\n(ex. &quot;20px&quot;) is used as the left style of widget&#39;s domNode.</p>\n"},"modal":{"!doc":"<p>If true, a translucent cover is added over the entire page to\nprevent the user from interacting with elements on the page.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the item so users can hit the tab key to\nfocus on it.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"top":{"!doc":"<p>The top edge position of the widget. If &quot;auto&quot;, the widget is\nplaced at the middle of the screen. Otherwise, the value\n(ex. &quot;20px&quot;) is used as the top style of widget&#39;s domNode.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hides the dialog.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn()","!doc":"<p>Refreshes the layout of the dialog.</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"show":{"!type":"fn()","!doc":"<p>Shows the dialog.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Slider":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A non-templated Slider widget similar to the HTML5 INPUT type=range.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_onChangeActive":{"!doc":"<p>Indicates that changes to the value should call onChange() callback.\nThis is false during widget initialization, to avoid calling onChange()\nwhen the initial value is set.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"aria-label":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;disabled=&#39;disabled&#39;&quot;, or just &quot;disabled&quot;.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"flip":{"!doc":"<p>Specifies if the slider should change its default: ascending &lt;--&gt; descending.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"halo":{"!doc":"<p>Size of the boundary that extends beyond the edges of the slider\nto make it easier to touch.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"intermediateChanges":{"!doc":"<p>Fires onChange for each value change or only on demand</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"max":{"!doc":"<p>The last value the slider can be set to.</p>\n"},"min":{"!doc":"<p>The first value the slider can be set to.</p>\n"},"name":{"!doc":"<p>Name used when submitting form; same as &quot;name&quot; attribute or plain HTML elements</p>\n"},"orientation":{"!doc":"<p>The slider direction.\n\n</p>\n<ul>\n<li>&quot;H&quot;: horizontal</li>\n<li>&quot;V&quot;: vertical</li>\n<li>&quot;auto&quot;: use width/height comparison at instantiation time (default is &quot;H&quot; if width/height are 0)</li>\n</ul>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>Should this widget respond to user input?\nIn markup, this is specified as &quot;readOnly&quot;.\nSimilar to disabled except readOnly form values are submitted.</p>\n"},"scrollOnFocus":{"!doc":"<p>On focus, should this widget scroll into view?</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"step":{"!doc":"<p>The delta from 1 value to another.\nThis causes the slider handle to snap/jump to the closest possible value.\nA value of 0 means continuous (as much as allowed by pixel resolution).</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"type":{"!doc":"<p>Corresponds to the native HTML <code>&lt;input&gt;</code> element&#39;s attribute.</p>\n"},"value":{"!doc":"<p>The current slider value.</p>\n"},"buildRendering":{"!type":"fn()"},"compare":{"!type":"fn(val1: ?, val2: ?) -> number","!doc":"<p>Compare 2 values (as returned by get(&#39;value&#39;) for this widget).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Put focus on this widget</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"undo":{"!type":"fn()","!doc":"<p>Restore the value to the last value passed to onChange</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/SpinWheel":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_PickerBase.prototype","!doc":"<p>A value picker widget that has spin wheels.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"colors":{"!doc":"<p>An array of slot colors.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"slotClasses":{"!doc":"<p>An array of slot classes. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotOrder":{"!doc":"<p>An array of index of slotClasses and slotProps.\nIf there are three slots and slotOrder=[2,1,0], the slots are\ndisplayed in reversed order. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotProps":{"!doc":"<p>An array of property objects for each slot class specified in\nslotClasses. This property is intended to be used when you\ncreate a subclass of this widget that has specific slots.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"values":{"!doc":"<p>An array of slot values.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSlots":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child slot widgets.</p>\n"},"getValue":{"!type":"fn() -> ?"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets the picker to show the initial values.</p>\n"},"resize":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setValue":{"!type":"fn(newValue: ?) -> ?"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_PickerBase":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A base class for picker classes (e.g. SpinWheel, ValuePicker).</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"colors":{"!doc":"<p>An array of slot colors.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"slotClasses":{"!doc":"<p>An array of slot classes. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotOrder":{"!doc":"<p>An array of index of slotClasses and slotProps.\nIf there are three slots and slotOrder=[2,1,0], the slots are\ndisplayed in reversed order. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotProps":{"!doc":"<p>An array of property objects for each slot class specified in\nslotClasses. This property is intended to be used when you\ncreate a subclass of this widget that has specific slots.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"values":{"!doc":"<p>An array of slot values.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSlots":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child slot widgets.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets the picker to show the initial values.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/SpinWheelSlot":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A slot of a SpinWheel.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_moved":{"!doc":"<p>Flag that signals if the user have moved in (one of) the scroll\ndirection(s) since touch start (a move under the threshold is ignored).</p>\n"},"_parentPadBorderExtentsBottom":{"!doc":"<p>For Tooltip.js.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"androidWorkaroud":{},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"centerPos":{"!doc":"<p>Inherited from parent.</p>\n"},"class":{},"constraint":{"!doc":"<p>False: no scroll constraint.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dirLock":{"!doc":"<p>disable the move handler if scroll starts in the unexpected direction</p>\n"},"disableFlashScrollBar":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fadeScrollBar":{},"fixedFooterHeight":{"!doc":"<p>height of a fixed footer</p>\n"},"fixedHeaderHeight":{"!doc":"<p>height of a fixed header</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>explicitly specified height of this widget (ex. &quot;300px&quot;)</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLocalFooter":{"!doc":"<p>footer is view-local (as opposed to application-wide)</p>\n"},"items":{"!doc":"<p>An array of array of key-label pairs\n(e.g. [[0, &quot;Jan&quot;], [1, &quot;Feb&quot;], ...]). If key values for each label\nare not necessary, labels can be used instead.</p>\n"},"labelFrom":{"!doc":"<p>The start value of display values of the slot. This parameter is\nespecially useful when the slot has serial values.</p>\n"},"labels":{"!doc":"<p>An array of labels to be displayed on the slot\n(e.g. [&quot;Jan&quot;, &quot;Feb&quot;, ...]). This is a simplified version of the\nitems property.</p>\n"},"labelTo":{"!doc":"<p>The end value of display values of the slot.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"maxSpeed":{"!doc":"<p>Maximum speed.</p>\n"},"minItems":{"!doc":"<p>Minimum number of items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"propagatable":{"!doc":"<p>False: stop touchstart event propagation.</p>\n"},"scrollBar":{},"scrollDir":{"!doc":"<p>v: vertical, h: horizontal, vh: both, f: flip</p>\n"},"scrollType":{"!doc":"<ul>\n<li>1: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)animation for slide animation</li>\n<li>2: use top/left style,</li>\n<li>3: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)transition for slide animation</li>\n<li>0: use default value (3 for Android, iOS6+, and BlackBerry; otherwise 1)</li>\n</ul>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"step":{"!doc":"<p>The steps between labelFrom and labelTo.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for this widget so users can hit the tab key to\nfocus on it.</p>\n"},"threshold":{"!doc":"<p>drag threshold value in pixels</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"touchNode":{"!doc":"<p>a node that will have touch event handlers</p>\n"},"value":{"!doc":"<p>The initial value of the slot.</p>\n"},"weight":{"!doc":"<p>frictional drag</p>\n"},"zeroPad":{"!doc":"<p>Length of zero padding numbers.\nEx. zeroPad=2 -&gt; &quot;00&quot;, &quot;01&quot;, ...\nEx. zeroPad=3 -&gt; &quot;000&quot;, &quot;001&quot;, ...</p>\n"},"abort":{"!type":"fn()","!doc":"<p>Aborts scrolling.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"adjust":{"!type":"fn()","!doc":"<p>Adjusts the position of slot panels.</p>\n"},"adjustDestination":{"!type":"fn(to: ?, pos: ?, dim: ?) -> bool","!doc":"<p>Overrides dojox/mobile/scrollable.adjustDestination().</p>\n"},"buildRendering":{"!type":"fn()"},"calcScrollBarPos":{"!type":"fn(to: ?) -> ?","!doc":"<p>Calculates the scroll bar position.</p>\n"},"calcSpeed":{"!type":"fn(d: ?, t: ?) -> number","!doc":"<p>Overrides dojox/mobile/scrollable.calcSpeed().</p>\n"},"cleanup":{"!type":"fn()","!doc":"<p>Uninitialize the module.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"createMask":{"!type":"fn()","!doc":"<p>Creates a mask for a scroll bar edge.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disableValues":{"!type":"fn(n: ?)","!doc":"<p>Grays out the items with an index higher or equal to the specified number.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"findDisp":{"!type":"fn(node: ?) -> +null","!doc":"<p>Finds the currently displayed view node from my sibling nodes.</p>\n"},"flashScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar instantly.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getCenterItem":{"!type":"fn() -> ?","!doc":"<p>Gets the currently selected item.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDim":{"!type":"fn() -> ?","!doc":"<p>Returns various internal dimensional information needed for calculation.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getKey":{"!type":"fn() -> ?"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPos":{"!type":"fn() -> ?","!doc":"<p>Gets the top position in the midst of animation.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getScreenSize":{"!type":"fn() -> ?","!doc":"<p>Returns the dimensions of the browser window.</p>\n"},"getSpeed":{"!type":"fn() -> ?","!doc":"<p>Overrides dojox/mobile/scrollable.getSpeed().</p>\n"},"getValue":{"!type":"fn() -> ?"},"hideScrollBar":{"!type":"fn()","!doc":"<p>Hides the scroll bar.</p>\n"},"init":{"!type":"fn(params?: ?)","!doc":"<p>Initialize according to the given params.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isFormElement":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the given node is a form control.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isTopLevel":{"!type":"fn() -> bool","!doc":"<p>Returns true if this is a top-level widget.</p>\n"},"makeTranslateStr":{"!type":"fn(to: ?) -> string","!doc":"<p>Constructs a string value that is passed to the -webkit-transform property.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"resetScrollBar":{"!type":"fn()","!doc":"<p>Resets the scroll bar length, position, etc.</p>\n"},"resize":{"!type":"fn(e: ?)"},"scrollIntoView":{"!type":"fn(node: ?, alignWithTop?: ?, duration?: ?)","!doc":"<p>Scrolls the pane until the searching node is in the view.</p>\n"},"scrollScrollBarTo":{"!type":"fn(to: ?)","!doc":"<p>Moves the scroll bar(s) to the given position without animation.</p>\n"},"scrollTo":{"!type":"fn(to: ?, doNotMoveScrollBar?: ?, node?: ?)","!doc":"<p>Scrolls to the given position immediately without animation.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setColor":{"!type":"fn(value: ?, color?: ?)","!doc":"<p>Sets the color of the specified item as blue.</p>\n"},"setInitialValue":{"!type":"fn()","!doc":"<p>Sets the initial value using this.value or the first item.</p>\n"},"setKeyframes":{"!type":"fn(from: ?, to: ?, idx: ?)","!doc":"<p>Programmatically sets key frames for the scroll animation.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)","!doc":"<p>Sets the given node as selectable or unselectable.</p>\n"},"setValue":{"!type":"fn(newValue: ?) -> ?"},"showScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar.</p>\n"},"slideScrollBarTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Moves the scroll bar(s) to the given position with the slide animation.</p>\n"},"slideTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Overrides dojox/mobile/scrollable.slideTo().</p>\n"},"spin":{"!type":"fn(steps: ?)","!doc":"<p>Spins the slot as specified by steps.</p>\n"},"startup":{"!type":"fn()"},"stopAnimation":{"!type":"fn()","!doc":"<p>Stops the currently running animation.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/SpinWheelDatePicker":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/SpinWheel.prototype","!doc":"<p>A SpinWheel-based date picker widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"colors":{"!doc":"<p>An array of slot colors.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dayPattern":{"!doc":"<p>A pattern to be used to format day.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"monthPattern":{"!doc":"<p>A pattern to be used to format month.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"slotClasses":{},"slotOrder":{"!doc":"<p>An array of index of slotClasses and slotProps.\nIf there are three slots and slotOrder=[2,1,0], the slots are\ndisplayed in reversed order. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotProps":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>A string representing the date value.\nThe setter of this property first converts the value argument by calling \nthe fromISOString method of the dojo/date/stamp module, then sets the \nvalues of the picker according to the resulting Date object. \nIf the string cannot be parsed by fromISOString, the method does nothing.\nExample: set(&quot;value&quot;, &quot;2012-1-20&quot;); // January 20, 2012\nThe getter returns the string formatted as described in the dojo/date/stamp \nmodule.</p>\n"},"values":{"!doc":"<p>An array of slot values.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"yearPattern":{"!doc":"<p>A pattern to be used to format year.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disableValues":{"!type":"fn(daysInMonth: ?)","!doc":"<p>Disables the end days of the month to match the specified\nnumber of days of the month.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSlots":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child slot widgets.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"initSlots":{"!type":"fn()","!doc":"<p>Initializes the slots.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reorderSlots":{"!type":"fn()","!doc":"<p>Reorders the slots.         </p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets the picker to show the initial values.</p>\n"},"resize":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_DatePickerMixin":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for date picker widget.</p>\n","dayPattern":{"!doc":"<p>A pattern to be used to format day.</p>\n"},"monthPattern":{"!doc":"<p>A pattern to be used to format month.</p>\n"},"value":{"!doc":"<p>A string representing the date value.\nThe setter of this property first converts the value argument by calling \nthe fromISOString method of the dojo/date/stamp module, then sets the \nvalues of the picker according to the resulting Date object. \nIf the string cannot be parsed by fromISOString, the method does nothing.\nExample: set(&quot;value&quot;, &quot;2012-1-20&quot;); // January 20, 2012\nThe getter returns the string formatted as described in the dojo/date/stamp \nmodule.</p>\n"},"yearPattern":{"!doc":"<p>A pattern to be used to format year.</p>\n"},"initSlots":{"!type":"fn()","!doc":"<p>Initializes the slots.</p>\n"},"reorderSlots":{"!type":"fn()","!doc":"<p>Reorders the slots.         </p>\n"},"reset":{"!type":"fn()","!doc":"<p>Goes to today.</p>\n"}}},"dojox/mobile/SpinWheelTimePicker":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/SpinWheel.prototype","!doc":"<p>A SpinWheel-based time picker widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"colors":{"!doc":"<p>An array of slot colors.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"date":{"!doc":"<p>A Date object corresponding to the current values of the picker.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"slotClasses":{},"slotOrder":{"!doc":"<p>An array of index of slotClasses and slotProps.\nIf there are three slots and slotOrder=[2,1,0], the slots are\ndisplayed in reversed order. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotProps":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"values":{"!doc":"<p>An array of slot values.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSlots":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child slot widgets.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets the picker to show the initial values.</p>\n"},"resize":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_TimePickerMixin":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for time picker widget.</p>\n","date":{"!doc":"<p>A Date object corresponding to the current values of the picker.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Goes to now. Resets the hour and minutes to the current time.</p>\n"}}},"dojox/mobile/StoreCarousel":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Carousel.prototype","!doc":"<p>A dojo/store enabled Carousel.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"childrenProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s children.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>Explicitly specified height of the widget (ex. &quot;300px&quot;). If\n&quot;inherit&quot; is specified, the height is inherited from its offset\nparent.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"itemWidth":{"!doc":"<p>The number of visible items (=numVisible) is determined by\n(carousel_width / itemWidth).\nIf itemWidth is specified, numVisible is automatically calculated.\nIf resize() is called, numVisible is recalculated and the layout\nis changed accordingly.</p>\n"},"labelProperty":{"!doc":"<p>A property name (a property in the dojo/store item) that specifies that item&#39;s label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"navButton":{"!doc":"<p>If true, navigation buttons are displyed on the title bar.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"numVisible":{"!doc":"<p>The number of visible items.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pageIndicator":{"!doc":"<p>If true, a page indicator, a series of small dots that indicate\nthe current page, is displayed on the title bar.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"query":{"!doc":"<p>A query that can be passed to &#39;store&#39; to initially filter the items.</p>\n"},"queryOptions":{"!doc":"<p>An optional parameter for the query.</p>\n"},"selectable":{"!doc":"<p>If true, an item can be selected by clicking it.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>Reference to data provider object used by this widget.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>A title of the carousel to be displayed on the title bar.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"fillPages":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexByItemWidget":{"!type":"fn(w: ?) -> number","!doc":"<p>Returns the index of a given item widget.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getItemWidgetByIndex":{"!type":"fn(index: ?) -> +null","!doc":"<p>Returns the index of an item widget at a given index.</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getParentView":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the parent view of the given DOM node.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"handleViewChanged":{"!type":"fn(view: ?)","!doc":"<p>Listens to &quot;/dojox/mobile/viewChanged&quot; events.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"instantiateView":{"!type":"fn(view: ?)","!doc":"<p>Instantiates the given view.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"refresh":{"!type":"fn() -> +null","!doc":"<p>Fetches the data and generates the list items.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn()"},"resizeItems":{"!type":"fn()","!doc":"<p>Resizes the child items of the carousel.</p>\n"},"select":{"!type":"fn(itemWidget: ?)","!doc":"<p>Selects the given widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setQuery":{"!type":"fn(query: ?, queryOptions: ?) -> ?"},"setStore":{"!type":"fn(store: ?, query: ?, queryOptions: ?) -> +null","!doc":"<p>Sets the store to use with this widget.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Switch":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A toggle switch with a sliding knob.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"leftLabel":{"!doc":"<p>The left-side label of the switch.</p>\n"},"name":{"!doc":"<p>A name for a hidden input field, which holds the current value.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"rightLabel":{"!doc":"<p>The right-side label of the switch.</p>\n"},"role":{"!doc":"<p>The accessibility role.</p>\n"},"shape":{"!doc":"<p>The shape of the switch.\n&quot;mblSwDefaultShape&quot;, &quot;mblSwSquareShape&quot;, &quot;mblSwRoundShape1&quot;,\n&quot;mblSwRoundShape2&quot;, &quot;mblSwArcShape1&quot; or &quot;mblSwArcShape2&quot;.\nThe default value is &quot;mblSwDefaultShape&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for this widget so users can hit the tab key to\nfocus on it.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The initial state of the switch: &quot;on&quot; or &quot;off&quot;. The default\nvalue is &quot;on&quot;.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Reset the widget&#39;s value to what it was at initialization time</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/TabBar":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A bar widget that has buttons to control visibility of views.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_fixedButtonMargin":{},"_fixedButtonWidth":{},"_largeScreenWidth":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"barType":{"!doc":"<p>&quot;tabBar&quot;, &quot;segmentedControl&quot;, &quot;standardTab&quot;, &quot;slimTab&quot;, &quot;flatTab&quot;,\nor &quot;tallTab&quot;</p>\n"},"baseClass":{},"center":{"!doc":"<p>If true, place the tabs in the center of the bar.\nThis property is NOT effective for &quot;tabBar&quot;.</p>\n"},"class":{},"closable":{"!doc":"<p>If true, user can close (destroy) a child tab by clicking the X on the tab.\nThis property is NOT effective for &quot;tabBar&quot; and &quot;tallBar&quot;.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fill":{"!doc":"<p>Define if the TabBar should resize its children so that they evenly fill all the available space in the bar.\n\n</p>\n<p>Allowed values:\n\n</p>\n<ol>\n<li>&quot;auto&quot; is the default behavior from version 1.8: bar buttons are resized to evenly fill the entire bar only on small devices (width &lt; 500px) and when barType is &quot;tabBar&quot;</li>\n<li>&quot;always&quot;: bar buttons are always resized to evenly fill the entire bar</li>\n<li>&quot;never&quot;: bar buttons are never resized to evenly fill the entire bar</li>\n</ol>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconBase":{"!doc":"<p>The default icon path for child items.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"selectOne":{"!doc":"<p>Specifies that only one item can be selected.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"syncWithViews":{"!doc":"<p>If true, this widget listens to view transition events to be\nsynchronized with view&#39;s visibility.</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSelectedTab":{"!type":"fn() -> ?","!doc":"<p>Returns the first selected child.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets the widget to its initial state.</p>\n"},"resize":{"!type":"fn(size: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/TabBarButton":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_ItemBase.prototype","!doc":"<p>A button widget that is placed in the TabBar widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_delayedSelection":{"!doc":"<p>If true, selection is delayed 100ms and canceled if dragged in\norder to avoid selection when flick operation is performed.</p>\n"},"_duration":{"!doc":"<p>Duration of selection, milliseconds.</p>\n"},"_handleClick":{"!doc":"<p>If true, this widget listens to touch events.</p>\n"},"_moveTo":{"!doc":"<p>id of destination view</p>\n"},"_selEndMethod":{},"_selStartMethod":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{"!doc":"<p>Sets tabIndex to domNode.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"alt":{"!doc":"<p>An alternate text for the icon image.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"back":{"!doc":"<p>If true, history.back() is called when clicked.</p>\n"},"badge":{"!doc":"<p>A string to show on a badge. (ex. &quot;12&quot;)</p>\n"},"baseClass":{},"callback":{"!doc":"<p>A callback function that is called when the transition has been\nfinished. A function reference, or name of a function in\ncontext.</p>\n"},"class":{},"clickable":{"!doc":"<p>If true, this item becomes clickable even if a transition\ndestination (moveTo, etc.) is not specified.</p>\n"},"closeIcon":{"!doc":"<p>CSS class for the close icon.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>A URL of another web page to go to.</p>\n"},"hrefTarget":{"!doc":"<p>A target that specifies where to open a page specified by\nhref. The value will be passed to the 2nd argument of\nwindow.open().</p>\n"},"icon":{"!doc":"<p>An icon image to display. The value can be either a path for an\nimage file or a class name of a DOM button. If icon is not\nspecified, the iconBase parameter of the parent widget is used.</p>\n"},"icon1":{"!doc":"<p>A path for the unselected (typically dark) icon. If icon is not\nspecified, the iconBase parameter of the parent widget is used.</p>\n"},"icon2":{"!doc":"<p>A path for the selected (typically highlight) icon. If icon is\nnot specified, the iconBase parameter of the parent widget or\nicon1 is used.</p>\n"},"iconPos":{"!doc":"<p>The position of an aggregated icon. IconPos is comma separated\nvalues like top,left,width,height (ex. &quot;0,0,29,29&quot;). If iconPos\nis not specified, the iconPos parameter of the parent widget is\nused.</p>\n"},"iconPos1":{"!doc":"<p>The position of an aggregated unselected (typically dark)\nicon. IconPos1 is a comma-separated list of values like\ntop,left,width,height (ex. &quot;0,0,29,29&quot;). If iconPos1 is not\nspecified, the iconPos parameter of the parent widget is used.</p>\n"},"iconPos2":{"!doc":"<p>The position of an aggregated selected (typically highlight)\nicon. IconPos2 is a comma-separated list of values like\ntop,left,width,height (ex. &quot;0,0,29,29&quot;). If iconPos2 is not\nspecified, the iconPos parameter of the parent widget or\niconPos1 is used.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>A label of the item. If the label is not specified, innerHTML is\nused as a label.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"moveTo":{"!doc":"<p>The id of the transition destination view which resides in the\ncurrent page.\n\n</p>\n<p>If the value has a hash sign (&#39;#&#39;) before the id (e.g. #view1)\nand the dojo/hash module is loaded by the user application, the\nview transition updates the hash in the browser URL so that the\nuser can bookmark the destination view. In this case, the user\ncan also use the browser&#39;s back/forward button to navigate\nthrough the views in the browser history.\n\n</p>\n<p>If null, transitions to a blank view.\nIf &#39;#&#39;, returns immediately without transition.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"paramsToInherit":{"!doc":"<p>Comma separated parameters to inherit from the parent.</p>\n"},"scene":{"!doc":"<p>The name of a scene. Used from dojox/mobile/app.</p>\n"},"selected":{"!doc":"<p>If true, the button is in the selected state.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the item so users can hit the tab key to\nfocus on it.</p>\n"},"tag":{"!doc":"<p>A name of html tag to create as domNode.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"toggle":{"!doc":"<p>If true, the item acts like a toggle button.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"transition":{"!doc":"<p>A type of animated transition effect.</p>\n"},"transitionDir":{"!doc":"<p>The transition direction. If 1, transition forward. If -1,\ntransition backward. For example, the slide transition slides\nthe view from right to left when dir == 1, and from left to\nright when dir == -1.</p>\n"},"transitionOptions":{"!doc":"<p>A hash object that holds transition options.</p>\n"},"url":{"!doc":"<p>A URL of an html fragment page or JSON data that represents a\nnew view content. The view content is loaded with XHR and\ninserted in the current page. Then a view transition occurs to\nthe newly created view. The view is cached so that subsequent\nrequests would not load the content again.</p>\n"},"urlTarget":{"!doc":"<p>Node id under which a new view will be created according to the\nurl parameter. If not specified, The new view will be created as\na sibling of the current view.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an ongoing selection (if any).</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defaultClickAction":{"!type":"fn(e: ?)","!doc":"<p>The default action of this item.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getTransOpts":{"!type":"fn() -> string","!doc":"<p>Copies from the parent and returns the values of parameters<br>specified by the property paramsToInherit.</p>\n"},"handleSelection":{"!type":"fn(e: ?)","!doc":"<p>Handles this items selection state.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"inheritParams":{"!type":"fn()","!doc":"<p>Overrides dojox/mobile/_ItemBase.inheritParams().</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"makeTransition":{"!type":"fn(e: ?)","!doc":"<p>Makes a transition.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setTransitionPos":{"!type":"fn(e: ?)","!doc":"<p>Stores the clicked position for later use.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"transitionTo":{"!type":"fn(moveTo: ?, href: ?, url: ?, scene: ?)","!doc":"<p>Performs a view transition.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"userClickAction":{"!type":"fn(e: ?)","!doc":"<p>User-defined click action.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/TimePicker":{"!doc":"<p>A wrapper widget around SpinWheelTimePicker or ValuePickerTimePicker.\nReturns ValuePickerTimePicker when the current theme is &quot;android&quot; or &quot;holodark&quot;.\nReturns SpinWheelTimePicker otherwise.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>A wrapper widget around SpinWheelTimePicker or ValuePickerTimePicker.\nReturns ValuePickerTimePicker when the current theme is &quot;android&quot; or &quot;holodark&quot;.\nReturns SpinWheelTimePicker otherwise.</p>\n"}}},"dojox/mobile/TreeView":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/ScrollableView.prototype","!doc":"<p>A scrollable view with tree-style navigation.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_fixedAppFooter":{},"_moved":{"!doc":"<p>Flag that signals if the user have moved in (one of) the scroll\ndirection(s) since touch start (a move under the threshold is ignored).</p>\n"},"_parentPadBorderExtentsBottom":{"!doc":"<p>For Tooltip.js.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"allowNestedScrolls":{"!doc":"<p>Flag to allow scrolling in nested containers, e.g. to allow ScrollableView in a SwapView.</p>\n"},"appBars":{"!doc":"<p>Enables the search for application-specific bars (header or footer).</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"constraint":{"!doc":"<p>bounce back to the content area</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dirLock":{"!doc":"<p>disable the move handler if scroll starts in the unexpected direction</p>\n"},"disableFlashScrollBar":{},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"fadeScrollBar":{},"fixedFooter":{"!doc":"<p>Id of the fixed footer.</p>\n"},"fixedFooterHeight":{"!doc":"<p>height of a fixed footer</p>\n"},"fixedHeader":{"!doc":"<p>Id of the fixed header.</p>\n"},"fixedHeaderHeight":{"!doc":"<p>height of a fixed header</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>explicitly specified height of this widget (ex. &quot;300px&quot;)</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLocalFooter":{"!doc":"<p>footer is view-local (as opposed to application-wide)</p>\n"},"keepScrollPos":{"!doc":"<p>Overrides dojox/mobile/View/keepScrollPos.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"propagatable":{"!doc":"<p>let touchstart event propagate up</p>\n"},"scrollableParams":{"!doc":"<p>Parameters for dojox/mobile/scrollable.init().</p>\n"},"scrollBar":{"!doc":"<p>show scroll bar or not</p>\n"},"scrollDir":{"!doc":"<p>v: vertical, h: horizontal, vh: both, f: flip</p>\n"},"scrollType":{"!doc":"<ul>\n<li>1: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)animation for slide animation</li>\n<li>2: use top/left style,</li>\n<li>3: use (-webkit-)transform:translate3d(x,y,z) style, use (-webkit-)transition for slide animation</li>\n<li>0: use default value (3 for Android, iOS6+, and BlackBerry; otherwise 1)</li>\n</ul>\n"},"selected":{"!doc":"<p>If true, the view is displayed at startup time.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tag":{"!doc":"<p>The name of the HTML tag to create as domNode. The default value is &quot;div&quot;.</p>\n"},"threshold":{"!doc":"<p>drag threshold value in pixels</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"touchNode":{"!doc":"<p>a node that will have touch event handlers</p>\n"},"weight":{"!doc":"<p>frictional drag</p>\n"},"abort":{"!type":"fn()","!doc":"<p>Aborts scrolling.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addCover":{"!type":"fn()","!doc":"<p>Adds the transparent DIV cover.</p>\n"},"addFixedBar":{"!type":"fn(widget: ?)","!doc":"<p>Adds a view local fixed bar to this widget.</p>\n"},"adjustDestination":{"!type":"fn(to: ?, pos: ?, dim: ?) -> bool","!doc":"<p>A stub function to be overridden by subclasses.</p>\n"},"buildRendering":{"!type":"fn()"},"calcScrollBarPos":{"!type":"fn(to: ?) -> ?","!doc":"<p>Calculates the scroll bar position.</p>\n"},"calcSpeed":{"!type":"fn(distance: ?, time: ?) -> number","!doc":"<p>Calculate the speed given the distance and time.</p>\n"},"checkFixedBar":{"!type":"fn(node: ?, local: ?) -> ?","!doc":"<p>Checks if the given node is a fixed bar or not.</p>\n"},"cleanup":{"!type":"fn()","!doc":"<p>Uninitialize the module.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Creates a new instance of the class.</p>\n"},"convertToId":{"!type":"fn(moveTo: ?) -> ?"},"createMask":{"!type":"fn()","!doc":"<p>Creates a mask for a scroll bar edge.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"findAppBars":{"!type":"fn()","!doc":"<p>Search for application-specific header or footer.</p>\n"},"findDisp":{"!type":"fn(node: ?) -> +null","!doc":"<p>Finds the currently displayed view node from my sibling nodes.</p>\n"},"flashScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar instantly.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> ?","!doc":"<p>Overrides _WidgetBase.getChildren to add local fixed bars,\nwhich are not under containerNode, to the children array.</p>\n"},"getDim":{"!type":"fn() -> ?","!doc":"<p>Returns various internal dimensional information needed for calculation.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPos":{"!type":"fn() -> ?","!doc":"<p>Gets the top position in the midst of animation.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getScreenSize":{"!type":"fn() -> ?","!doc":"<p>Returns the dimensions of the browser window.</p>\n"},"getShowingView":{"!type":"fn() -> ?","!doc":"<p>Find the currently showing view from my sibling views.</p>\n"},"getSiblingViews":{"!type":"fn() -> []","!doc":"<p>Returns an array of the sibling views.</p>\n"},"getSpeed":{"!type":"fn() -> ?","!doc":"<p>Returns an object that indicates the scrolling speed.</p>\n"},"handleClick":{"!type":"fn(li: ?)","!doc":"<p>Called when the user clicks a tree item.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hides this view without a transition animation.</p>\n"},"hideScrollBar":{"!type":"fn()","!doc":"<p>Hides the scroll bar.</p>\n"},"init":{"!type":"fn(params?: ?)","!doc":"<p>Initialize according to the given params.</p>\n"},"invokeCallback":{"!type":"fn()","!doc":"<p>A function to be called after performing a transition to\ncall a specified callback.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isFormElement":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the given node is a form control.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isTopLevel":{"!type":"fn(e: ?) -> bool","!doc":"<p>Returns true if this is a top-level widget.\nOverrides dojox/mobile/scrollable.isTopLevel.</p>\n"},"isVisible":{"!type":"fn(checkAncestors?: ?) -> bool","!doc":"<p>Return true if this view is visible</p>\n"},"makeTranslateStr":{"!type":"fn(to: ?) -> string","!doc":"<p>Constructs a string value that is passed to the -webkit-transform property.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"performTransition":{"!type":"fn(moveTo: ?, transitionDir: ?, transition: ?, context: ?, method: ?)","!doc":"<p>Function to perform the various types of view transitions, such as fade, slide, and flip.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"removeCover":{"!type":"fn()","!doc":"<p>Removes the transparent DIV cover.</p>\n"},"reparent":{"!type":"fn()","!doc":"<p>Moves all the children, except header and footer, to\ncontainerNode.</p>\n"},"resetScrollBar":{"!type":"fn()","!doc":"<p>Resets the scroll bar length, position, etc.</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"scrollIntoView":{"!type":"fn(node: ?, alignWithTop?: ?, duration?: ?)","!doc":"<p>Scrolls the pane until the searching node is in the view.</p>\n"},"scrollScrollBarTo":{"!type":"fn(to: ?)","!doc":"<p>Moves the scroll bar(s) to the given position without animation.</p>\n"},"scrollTo":{"!type":"fn(to: ?, doNotMoveScrollBar?: ?, node?: ?)","!doc":"<p>Scrolls to the given position immediately without animation.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setKeyframes":{"!type":"fn(from: ?, to: ?, idx: ?)","!doc":"<p>Programmatically sets key frames for the scroll animation.</p>\n"},"setSelectable":{"!type":"fn(node: ?, selectable: ?)","!doc":"<p>Sets the given node as selectable or unselectable.</p>\n"},"show":{"!type":"fn(noEvent?: ?, doNotHideOthers?: ?)","!doc":"<p>Shows this view without a transition animation.</p>\n"},"showScrollBar":{"!type":"fn()","!doc":"<p>Shows the scroll bar.</p>\n"},"slideScrollBarTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Moves the scroll bar(s) to the given position with the slide animation.</p>\n"},"slideTo":{"!type":"fn(to: ?, duration: ?, easing: ?)","!doc":"<p>Scrolls to the given position with the slide animation.</p>\n"},"startup":{"!type":"fn()"},"stopAnimation":{"!type":"fn()","!doc":"<p>Stops the currently running animation.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ValuePicker":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/_PickerBase.prototype","!doc":"<p>A value picker that has a stepper.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"colors":{"!doc":"<p>An array of slot colors.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"slotClasses":{"!doc":"<p>An array of slot classes. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotOrder":{"!doc":"<p>An array of index of slotClasses and slotProps.\nIf there are three slots and slotOrder=[2,1,0], the slots are\ndisplayed in reversed order. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotProps":{"!doc":"<p>An array of property objects for each slot class specified in\nslotClasses. This property is intended to be used when you\ncreate a subclass of this widget that has specific slots.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"values":{"!doc":"<p>An array of slot values.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSlots":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child slot widgets.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets the picker to show the initial values.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ValuePickerSlot":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget representing one slot of a ValuePicker widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"items":{"!doc":"<p>An array of array of key-label pairs\n(e.g. [[0, &quot;Jan&quot;], [1,&quot;Feb&quot;], ...]). If key values for each label\nare not necessary, labels can be used instead.</p>\n"},"key":{"!doc":"<p>The key of the currently selected value in the items array. This is a read-only property.\nWarning: Do not use this property directly, make sure to call the get() method.</p>\n"},"labelFrom":{"!doc":"<p>The start value of display values of the value picker. This\nparameter is especially useful when value picker has serial\nvalues.</p>\n"},"labels":{"!doc":"<p>An array of labels to be displayed on the value picker\n(e.g. [&quot;Jan&quot;,&quot;Feb&quot;,...]). This is a simplified version of the\nitems property.</p>\n"},"labelTo":{"!doc":"<p>The end value of display values of the value picker.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"minusBtnLabel":{"!doc":"<p>(Accessibility) Text label for minus button</p>\n"},"minusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for minus button</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"plusBtnLabel":{"!doc":"<p>(Accessibility) Text label for plus button</p>\n"},"plusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for plus button</p>\n"},"readOnly":{"!doc":"<p>A flag used to indicate if the input field is readonly or not.\nNote that changing the value of the property after the widget \ncreation has no effect.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"step":{"!doc":"<p>The steps between labelFrom and labelTo.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for this widget so users can hit the tab key to\nfocus on it.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The initial value of the value picker.</p>\n"},"zeroPad":{"!doc":"<p>Length of zero padding numbers.\nEx. zeroPad=2 -&gt; &quot;00&quot;, &quot;01&quot;, ...\nEx. zeroPad=3 -&gt; &quot;000&quot;, &quot;001&quot;, ...</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setInitialValue":{"!type":"fn()","!doc":"<p>Sets the initial value using this.value or the first item.</p>\n"},"spin":{"!type":"fn(steps: ?)","!doc":"<p>Spins the slot as specified by steps.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn(value: ?) -> ?"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ValuePickerDatePicker":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/ValuePicker.prototype","!doc":"<p>A ValuePicker-based date picker widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"colors":{"!doc":"<p>An array of slot colors.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dayMinusBtnLabel":{"!doc":"<p>(Accessibility) Label for day minus button</p>\n"},"dayMinusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for the day minus button</p>\n"},"dayPattern":{"!doc":"<p>A pattern to be used to format day.</p>\n"},"dayPlusBtnLabel":{"!doc":"<p>(Accessibility) Label for day plus button</p>\n"},"dayPlusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for the day plus button</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"monthMinusBtnLabel":{"!doc":"<p>(Accessibility) Label for month minus button</p>\n"},"monthMinusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for the month minus button</p>\n"},"monthPattern":{"!doc":"<p>A pattern to be used to format month.</p>\n"},"monthPlusBtnLabel":{"!doc":"<p>(Accessibility) Label for month plus button</p>\n"},"monthPlusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for the month plus button</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>If true, slot input fields are read-only. Only the plus and\nminus buttons can be used to change the values.\nNote that changing the value of the property after the widget \ncreation has no effect.</p>\n"},"slotClasses":{},"slotOrder":{"!doc":"<p>An array of index of slotClasses and slotProps.\nIf there are three slots and slotOrder=[2,1,0], the slots are\ndisplayed in reversed order. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotProps":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>A string representing the date value.\nThe setter of this property first converts the value argument by calling \nthe fromISOString method of the dojo/date/stamp module, then sets the \nvalues of the picker according to the resulting Date object. \nIf the string cannot be parsed by fromISOString, the method does nothing.\nExample: set(&quot;value&quot;, &quot;2012-1-20&quot;); // January 20, 2012\nThe getter returns the string formatted as described in the dojo/date/stamp \nmodule.</p>\n"},"values":{"!doc":"<p>An array of slot values.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"yearMinusBtnLabel":{},"yearMinusBtnLabelRef":{},"yearPattern":{"!doc":"<p>A pattern to be used to format year.</p>\n"},"yearPlusBtnLabel":{"!doc":"<p>(Accessibility) Label for year plus button</p>\n"},"yearPlusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for the year plus button</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disableValues":{"!type":"fn(daysInMonth: ?)","!doc":"<p>Disables the end days of the month to match the specified\nnumber of days of the month.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSlots":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child slot widgets.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"initSlots":{"!type":"fn()","!doc":"<p>Initializes the slots.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reorderSlots":{"!type":"fn()","!doc":"<p>Reorders the slots.         </p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets the picker to show the initial values.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/ValuePickerTimePicker":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/ValuePicker.prototype","!doc":"<p>A ValuePicker-based time picker widget.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"colors":{"!doc":"<p>An array of slot colors.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"date":{"!doc":"<p>A Date object corresponding to the current values of the picker.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hourMinusBtnLabel":{"!doc":"<p>(Accessibility) Label for hour minus button</p>\n"},"hourMinusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for the hour minus button</p>\n"},"hourPlusBtnLabel":{"!doc":"<p>(Accessibility) Label for hour plus button</p>\n"},"hourPlusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for the hour plus button</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"is24h":{"!doc":"<p>If true, the time is displayed in 24h format.\nOtherwise, displayed in AM/PM mode.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"minuteMinusBtnLabel":{"!doc":"<p>(Accessibility) Label for minute minus button</p>\n"},"minuteMinusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for the minute minus button</p>\n"},"minutePlusBtnLabel":{"!doc":"<p>(Accessibility) Label for minute plus button</p>\n"},"minutePlusBtnLabelRef":{"!doc":"<p>(Accessibility) Reference to a node id containing text label for the minute plus button</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"readOnly":{"!doc":"<p>If true, slot input fields are read-only. Only the plus and\nminus buttons can be used to change the values.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"slotClasses":{},"slotOrder":{"!doc":"<p>An array of index of slotClasses and slotProps.\nIf there are three slots and slotOrder=[2,1,0], the slots are\ndisplayed in reversed order. This property is intended to be used\nwhen you create a subclass of this widget that has specific slots.</p>\n"},"slotProps":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"values":{"!doc":"<p>The time value, as an array in 24h format: [hour24, minute] (ex. [&quot;22&quot;,&quot;06&quot;]).\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"values12":{"!doc":"<p>The time value, as an array in 12h format: [hour12, minute, ampm] (ex. [&quot;10&quot;,&quot;06&quot;,&quot;PM&quot;]).\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"getSlots":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child slot widgets.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets the picker to show the initial values.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/Video":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Audio.prototype","!doc":"<p>A thin wrapper around the HTML5 <code>&lt;video&gt;</code> element.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_playable":{"!doc":"<p>Internal flag.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_tag":{"!doc":"<p>The name of the tag (&quot;video&quot;).</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"height":{"!doc":"<p>The height of the embed element.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"source":{"!doc":"<p>An array of src and type,\nex. [{src:&quot;a.mp3&quot;, type:&quot;audio/mpeg&quot;}, {src:&quot;a.ogg&quot;, type:&quot;audio/ogg&quot;}, ...].\nThe src gives the path of the media resource. The type gives the\ntype of the media resource.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"width":{"!doc":"<p>The width of the embed element.\nNote that changing the value of the property after the widget\ncreation has no effect.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Creates a new instance of the class.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_EditableIconMixin":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for IconContainer to make it editable.</p>\n","deleteIconForEdit":{},"threshold":{},"deleteIconClicked":{"!type":"fn(e: ?)","!doc":"<p>User-defined function to handle clicks for the delete icon.</p>\n"},"deleteItem":{"!type":"fn(item: ?)","!doc":"<p>Deletes the given item.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys the container.</p>\n"},"endEdit":{"!type":"fn()","!doc":"<p>Ends the editing.</p>\n"},"moveChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Moves a child without animation.</p>\n"},"moveChildWithAnimation":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Moves a child with animation.   </p>\n"},"removeChildWithAnimation":{"!type":"fn(widget: ?)","!doc":"<p>Removes the given child with animation.</p>\n"},"scaleItem":{"!type":"fn(widget: ?, ratio: ?)","!doc":"<p>Scales an item according to the specified ratio.</p>\n"},"startEdit":{"!type":"fn()","!doc":"<p>Starts the editing.</p>\n"}}},"dojox/mobile/_EditableListMixin":{"!type":"fn()","prototype":{"!doc":"<p>A rounded rectangle list.</p>\n","deleteIconForEdit":{},"isEditing":{"!doc":"<p>A read-only flag that indicates whether the widget is in the editing mode.</p>\n"},"rightIconForEdit":{},"destroy":{"!type":"fn()","!doc":"<p>Destroys the widget.</p>\n"},"endEdit":{"!type":"fn()","!doc":"<p>Ends the editing.</p>\n"},"startEdit":{"!type":"fn()","!doc":"<p>Starts the editing.</p>\n"}}},"dojox/mobile/_IconItemPane":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/Pane.prototype","!doc":"<p>An internal widget used for IconContainer.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTabIndexAttr":{"!doc":"<p>Sets tabIndex to closeIconNode.</p>\n"},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The name of the CSS class of this widget.</p>\n"},"class":{},"closeIcon":{"!doc":"<p>CSS class for the close icon.</p>\n"},"closeIconRole":{"!doc":"<p>The HTML role of the close icon. Example: &quot;button&quot;.</p>\n"},"closeIconTitle":{"!doc":"<p>The title of the close icon.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconPos":{"!doc":"<p>The default icon position for child items.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>The label of the item.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index for the close button, such that users can hit the tab\nkey to focus on it.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hides the widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isOpen":{"!type":"fn(e: ?) -> bool","!doc":"<p>Tests whether the widget is open.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"resize":{"!type":"fn()","!doc":"<p>Calls resize() of each child widget.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"show":{"!type":"fn()","!doc":"<p>Shows the widget.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mobile/_PickerChooser":{"!doc":"<p>This widget chooses a picker class according to the current theme.\nImports ValuePicker-based date/time picker when the current theme is &quot;android&quot;.\nImports SpinWheel-based date/time picker otherwise.</p>\n","load":{"!type":"fn(id: ?, parentRequire: ?, loaded: ?)","!doc":"<p>Imports a picker class according to the current theme.</p>\n"}},"dojox/mobile/_base":{"!doc":"<p>Includes the basic dojox/mobile modules: common, View, Heading, \nRoundRect, RoundRectCategory, EdgeToEdgeCategory, RoundRectList,\nEdgeToEdgeList, ListItem, Container, Pane, Switch, ToolBarButton, \nand ProgressIndicator.</p>\n"},"dojox/mobile/_compat":{"!doc":"<p>CSS3 compatibility module.</p>\n"},"dojox/mobile/bidi/Accordion":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile Accordion widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/common":{"MARK":{},"enforceTextDirWithUcc":{"!type":"fn(text: ?, textDir: ?) -> string","!doc":"<p>Wraps by UCC (Unicode control characters) displayed text according to textDir.</p>\n"},"removeUCCFromText":{"!type":"fn(text: ?) -> ?","!doc":"<p>Removes UCC from input string.</p>\n"},"setTextDirForButtons":{"!type":"fn(widget: ?)","!doc":"<p>Sets textDir property to children.</p>\n"}},"dojox/mobile/bidi/common.MARK":{"LRE":{},"PDF":{},"RLE":{}},"dojox/mobile/bidi/Badge":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for Badge, using Unicode Control Characters to control text direction.</p>\n","textDir":{"!doc":"<p>Mobile widgets, derived from dijit._WidgetBase has this attribute for text direction support (bidi support).\nThe text direction can be different than the GUI direction.\nValues: &quot;ltr&quot;, &quot;rtl&quot;, &quot;auto&quot;(the direction of a text defined by first strong letter).</p>\n"},"setTextDir":{"!type":"fn(textDir: ?)"},"setValue":{"!type":"fn(value: ?)"}}},"dojox/mobile/bidi/Button":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile Button widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/Carousel":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile Carousel widget, using Unicode Control Characters to control text direction.</p>\n","buildRendering":{"!type":"fn()"}}},"dojox/mobile/bidi/CarouselItem":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile CarouselItem widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/Heading":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile Heading widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/IconItem":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile IconItem widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/IconMenu":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile IconMenu widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/ListItem":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile ListItem widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/RoundRectCategory":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile RoundRectCategory widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/SpinWheelSlot":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile SpinWheelSlot widget, using Unicode Control Characters to control text direction.</p>\n","postCreate":{"!type":"fn()"}}},"dojox/mobile/bidi/SwapView":{"!type":"fn()","prototype":{"_callParentFunction":{},"callParentFunction":{"!doc":"<p>Boolean value indicate whether to call the parent version of the function or the child one.\nUsed to support mirroring.</p>\n"},"nextView":{"!type":"fn(node: ?) -> ?"},"previousView":{"!type":"fn(node: ?) -> ?"}}},"dojox/mobile/bidi/Switch":{"!type":"fn()","prototype":{"!doc":"<p>Bidi support for mobile Switch widget, using Unicode Control Characters to control text direction.</p>\n","buildRendering":{"!type":"fn()"},"postCreate":{"!type":"fn()"}}},"dojox/mobile/bidi/TabBar":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile TabBar widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/TabBarButton":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile TabBarButton widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/TextBox":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile TextBox widget.</p>\n"}},"dojox/mobile/bidi/ToolBarButton":{"!type":"fn()","prototype":{"buildRendering":{"!type":"fn()"}}},"dojox/mobile/bidi/Tooltip":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile Tooltip widget, using Unicode Control Characters to control text direction.</p>\n","buildRendering":{"!type":"fn()"},"postCreate":{"!type":"fn()"}}},"dojox/mobile/bidi/TreeView":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile TreeView widget, using Unicode Control Characters to control text direction.</p>\n"}},"dojox/mobile/bidi/ValuePickerSlot":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile ValuePickerSlot widget, using Unicode Chontrol Characters to control text direction.</p>\n","postCreate":{"!type":"fn()"}}},"dojox/mobile/bidi/_ComboBoxMenu":{"!type":"fn()","prototype":{"buildRendering":{"!type":"fn()"}}},"dojox/mobile/bidi/_ItemBase":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile _ItemBase widget, using Unicode Control Characters to control text direction.</p>\n","getTransOpts":{"!type":"fn() -> ?"}}},"dojox/mobile/bidi/_StoreListMixin":{"!type":"fn()","prototype":{"!doc":"<p>Support for control over text direction for mobile _StoreListMixin and _DataListMixin.</p>\n","createListItem":{"!type":"fn(item: ?) -> ?"}}},"dojox/mobile/bookmarkable":{"!doc":"<p>Utilities to make the view transitions bookmarkable.</p>\n","settingHash":{"!doc":"<p>Whether the browser URL needs to be updated to include the hash.</p>\n"},"transitionInfo":{"!doc":"<p>An array containing information about the transition.</p>\n"},"addTransitionInfo":{"!type":"fn(fromViewId: ?, toViewId: ?, args: ?)","!doc":"<p>Adds transition information.</p>\n"},"findTransitionViews":{"!type":"fn(moveTo: ?) -> []","!doc":"<p>Searches for a starting view and a destination view.</p>\n"},"getTransitionInfo":{"!type":"fn(fromViewId: ?, toViewId: ?) -> ?","!doc":"<p>Returns an array containing the transition information.</p>\n"},"handleFragIds":{"!type":"fn(fragIds: ?) -> ?","!doc":"<p>Analyzes the given hash (fragment id).</p>\n"},"setFragIds":{"!type":"fn(toView: ?)","!doc":"<p>Updates the hash (fragment id) in the browser URL.</p>\n"}},"dojox/mobile/compat":{"!doc":"<p>CSS3 compatibility module.</p>\n"},"dojox/mobile/deviceTheme":{"!doc":"<p>Automatic theme loader.</p>\n","themeMap":{},"prototype":{"constructor":{"!type":"fn()","!doc":"<p>Automatic theme loader.</p>\n"}},"loadCssFile":{"!type":"fn(file: ?)","!doc":"<p>Loads the given CSS file programmatically.</p>\n"},"loadDeviceTheme":{"!type":"fn(userAgent?: ?)","!doc":"<p>Loads a device-specific theme according to the user-agent\nstring.</p>\n"},"setDm":{"!type":"fn(_dm: ?)","!doc":"<p>Replaces the dojox/mobile object.</p>\n"},"toUrl":{"!type":"fn(path: ?) -> ?","!doc":"<p>A wrapper for require.toUrl to support non-dojo usage.</p>\n"}},"dojox/mobile/dh/ContentTypeMap":{"!doc":"<p>A component that provides a map for determining the content handler\nclass from a content-type.</p>\n","map":{},"add":{"!type":"fn(contentType: ?, handlerClass: ?)","!doc":"<p>Adds a handler class for the given content type.</p>\n"},"getHandlerClass":{"!type":"fn(contentType: ?) -> ?","!doc":"<p>Returns the handler class for the given content type.</p>\n"}},"dojox/mobile/dh/ContentTypeMap.map":{"html":{},"json":{}},"dojox/mobile/dh/DataHandler":{"!type":"fn()","prototype":{"!doc":"<p>A component that provides an interface between data and handlers.</p>\n","ds":{"!doc":"<p>A DataSource instance.</p>\n"},"refNode":{"!doc":"<p>An optional reference DOM node before which a new view is created.</p>\n"},"target":{"!doc":"<p>A DOM node under which a new view is created.</p>\n"},"constructor":{"!type":"fn(ds: ?, target: ?, refNode?: ?)","!doc":"<p>Creates a new instance of the class.</p>\n"},"processData":{"!type":"fn(contentType: ?, callback: ?)","!doc":"<p>Fetches data through DataSource and passes it to a content\nhandler.</p>\n"}}},"dojox/mobile/dh/HtmlContentHandler":{"!type":"fn()","prototype":{"!doc":"<p>A HTML content handler.</p>\n","parse":{"!type":"fn(content: ?, target: ?, refNode?: ?) -> ?","!doc":"<p>Parses the given data and creates a new view at the given position.</p>\n"}}},"dojox/mobile/dh/HtmlScriptContentHandler":{"!type":"fn()","prototype":{"!proto":"dojox/mobile/dh/HtmlContentHandler.prototype","!doc":"<p>An HTML content handler that has script execution capability.</p>\n","execScript":{"!type":"fn(html: ?) -> string","!doc":"<p>Finds script tags and executes the script.</p>\n"},"parse":{"!type":"fn(content: ?, target: ?, refNode?: ?) -> ?","!doc":"<p>Parses the given data and creates a new view at the given position.</p>\n"}}},"dojox/mobile/dh/JsonContentHandler":{"!type":"fn()","prototype":{"!doc":"<p>A JSON content handler.</p>\n","parse":{"!type":"fn(content: ?, target: ?, refNode?: ?) -> ?","!doc":"<p>Parses the given data and creates a new view at the given position.</p>\n"}}},"dojox/mobile/dh/PatternFileTypeMap":{"!doc":"<p>A component that provides a map for determining content-type from\nthe pattern of the URL.</p>\n","map":{},"add":{"!type":"fn(key: ?, contentType: ?)","!doc":"<p>Adds a handler class for the given content type.        </p>\n"},"getContentType":{"!type":"fn(fileName: ?)","!doc":"<p>Returns the handler class for the given content type.       </p>\n"}},"dojox/mobile/dh/PatternFileTypeMap.map":{".*.html":{},".*.json":{}},"dojox/mobile/dh/StringDataSource":{"!type":"fn()","prototype":{"!doc":"<p>A component that simply returns the given text.</p>\n","text":{},"constructor":{"!type":"fn(text: ?)","!doc":"<p>Creates a new instance of the class.</p>\n"},"getData":{"!type":"fn() -> ?","!doc":"<p>Returns the given text.         </p>\n"}}},"dojox/mobile/dh/SuffixFileTypeMap":{"!doc":"<p>A component that provides a map for determining content-type from\nthe suffix of the URL.</p>\n","map":{},"add":{"!type":"fn(key: ?, contentType: ?)","!doc":"<p>Adds a handler class for the given content type.</p>\n"},"getContentType":{"!type":"fn(fileName: ?)","!doc":"<p>Returns the handler class for the given content type.       </p>\n"}},"dojox/mobile/dh/SuffixFileTypeMap.map":{"html":{},"json":{}},"dojox/mobile/dh/UrlDataSource":{"!type":"fn()","prototype":{"!doc":"<p>A component that accesses the given URL and fetches the data as text.</p>\n","_url":{},"text":{},"constructor":{"!type":"fn(url: ?)","!doc":"<p>Creates a new instance of the class.</p>\n"},"getData":{"!type":"fn() -> ?","!doc":"<p>Returns a Deferred that accesses the given URL and fetches the data as text.</p>\n"}}},"dojox/mobile/i18n":{"!doc":"<p>An internationalization utility for applications based on dojox/mobile.</p>\n","I18NProperties":{"!doc":"<p>These properties can be specified for any widget once the dojox/mobile/i18n module is loaded.</p>\n"},"load":{"!type":"fn(packageName: ?, bundleName: ?, locale?: ?)","!doc":"<p>Loads an nls resource bundle and returns an array of localized\nresources.</p>\n"},"registerBundle":{"!type":"fn(bundle: ?)","!doc":"<p>Accumulates the given localized resources in an array and returns\nit.</p>\n"}},"dojox/mobile/i18n.I18NProperties":{"!doc":"<p>These properties can be specified for any widget once the dojox/mobile/i18n module is loaded.</p>\n","mblNoConv":{"!doc":"<p>Disables localization by dojox/mobile/i18n for the widget on which the property is set.</p>\n"}},"dojox/mobile/migrationAssist":{"!doc":"<p>Dojo Mobile 1.6/1.7 to 1.8 migration assistance.</p>\n","checkCarousel":{"!type":"fn(w: ?)"},"checkFixedSplitter":{"!type":"fn(w: ?)"},"checkFixedSplitterPane":{"!type":"fn(w: ?)"},"checkListItem":{"!type":"fn(w: ?)"},"checkSpinWheelSlot":{"!type":"fn(w: ?)"},"checkSwapView":{"!type":"fn(w: ?)"},"checkSwitch":{"!type":"fn(w: ?)"},"checkTabBar":{"!type":"fn(w: ?)"},"checkTabBarButton":{"!type":"fn(w: ?)"},"checkToolBarButton":{"!type":"fn(w: ?)"},"prototype":{"constructor":{"!type":"fn()","!doc":"<p>Dojo Mobile 1.6/1.7 to 1.8 migration assistance.</p>\n"}},"dispatch":{"!type":"fn(cls: ?, w: ?)"}},"dojox/mobile/mobile-all":{"!doc":"<p>A roll-up that includes every mobile module. You probably don&#39;t need\nthis. Demo purposes only.</p>\n"},"dojox/mobile/pageTurningUtils":{"!doc":"<p>Utilities to provide page turning effects just like turning a real book.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>Utilities to provide page turning effects just like turning a real book.</p>\n"}}},"dojox/mobile/parser":{"!doc":"<p>A lightweight parser.</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>A lightweight parser.</p>\n"}},"instantiate":{"!type":"fn(nodes: ?, mixin?: ?, options?: ?)","!doc":"<p>Function for instantiating a list of widget nodes.</p>\n"},"parse":{"!type":"fn(rootNode: ?, options?: ?)","!doc":"<p>Function to handle parsing for widgets in the current document.\nIt is not as powerful as the full parser, but it will handle basic\nuse cases fine.</p>\n"}},"dojox/mobile":{"!doc":"<p>Deprecated.  Should require dojox/mobile classes directly rather than trying to access them through\nthis module.</p>\n"},"dojox/mvc/Bind":{"at":{"!type":"fn(target: ?, targetProp: ?) -> +any","!doc":"<p>Returns a pointer to data binding target (a dojo/Stateful property), called at handle, which is used for start synchronization with data binding source (another dojo/Stateful property).</p>\n"},"bind":{"!type":"fn(source: ?, sourceProp: ?, target: ?, targetProp: ?, func?: ?, bindOnlyIfUnequal?: ?) -> ?","!doc":"<p>Bind the specified property of the target to the specified\nproperty of the source with the supplied transformation.</p>\n"},"bindInputs":{"!type":"fn(sourceBindArray: ?, func: ?) -> []","!doc":"<p>Bind the values at the sources specified in the first argument\narray such that a composing function in the second argument is\ncalled when any of the values changes.</p>\n"},"EditModelRefController":{"!type":"fn()"},"EditStoreRefController":{"!type":"fn()"},"EditStoreRefListController":{"!type":"fn()"},"Element":{"!type":"fn()"},"equals":{"!type":"fn(dst: ?, src: ?, options: ?) -> bool","!doc":"<p>Compares two dojo/Stateful objects, by diving into the leaves.</p>\n"},"Generate":{"!type":"fn()"},"getPlainValue":{"!type":"fn(value: ?, options: ?) -> +any","!doc":"<p>Create a raw value from a dojo/Stateful object.</p>\n"},"getStateful":{"!type":"fn(value: ?, options: ?) -> +Anything","!doc":"<p>Create a dojo/Stateful object from a raw value.</p>\n"},"Group":{"!type":"fn()"},"ListController":{"!type":"fn()"},"ModelRefController":{"!type":"fn()"},"newStatefulModel":{"!type":"fn(args: ?)","!doc":"<p>Factory method that instantiates a new data model that view\ncomponents may bind to.</p>\n"},"Output":{"!type":"fn()"},"Repeat":{"!type":"fn()"},"resolve":{"!type":"fn(target: ?, parent?: ?) -> ?","!doc":"<p>Find a dojo/Stateful for the target.</p>\n"},"StatefulArray":{"!type":"fn(a: ?) -> ?","!doc":"<p>An inheritance of native JavaScript array, that adds dojo/Stateful capability.</p>\n"},"StatefulModel":{"!type":"fn()"},"StatefulSeries":{"!type":"fn()"},"StoreRefController":{"!type":"fn()"},"sync":{"!type":"fn(source: ?, sourceProp: ?, target: ?, targetProp: ?, options: ?) -> +any","!doc":"<p>Synchronize two dojo/Stateful properties.</p>\n"},"Templated":{"!type":"fn()"},"WidgetList":{"!type":"fn()"}},"dojox/mvc/Bind._Controller":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?) -> ?","!doc":"<p>If this object is not called from Dojo parser, starts this up right away.\nAlso, if widget registry is available, register this object.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Bind.ModelRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/_Controller.prototype","!doc":"<p>A controller that keeps a reference to dojo/Stateful-based data model.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/Bind.EditModelRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/ModelRefController.prototype","!doc":"<p>A child class of dojox/mvc/ModelRefController.\nKeeps a copy (originalModel) of given data model (sourceModel) so that it can manage the data model of before/after the edit.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refEditModelProp":{"!doc":"<p>The property name for the data model, that is being edited.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refOriginalModelProp":{"!doc":"<p>The property name for the data model, that serves as the original data.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getPlainValueOptions":{"!doc":"<p>The options to get plain value from stateful object.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"holdModelUntilCommit":{"!doc":"<p>True not to send the change in model back to sourceModel until commit() is called.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"originalModel":{"!doc":"<p>The data model, that serves as the original data.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"sourceModel":{},"cloneModel":{"!type":"fn(value: ?) -> ?","!doc":"<p>Create a clone object of the data source.\nChild classes of this controller can override it to achieve its specific needs.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Send the change back to the data source.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets certain properties before setting models.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Change the model back to its original state.</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Set a property to this.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/Bind.StoreRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/ModelRefController.prototype","!doc":"<p>A child class of dojox.mvc.ModelRefController, which keeps a reference to Dojo Object Store (in store property).</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"store":{"!doc":"<p>The Dojo Object Store in use.</p>\n"},"addStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"getStore":{"!type":"fn(id: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by its identity.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"putStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object.</p>\n"},"queryStore":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"removeStore":{"!type":"fn(id: ?, options: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/Bind.EditStoreRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/StoreRefController.prototype","!doc":"<p>A child class of dojox/mvc/StoreRefController, managing edits.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refEditModelProp":{"!doc":"<p>The property name for the data model, that is being edited.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refOriginalModelProp":{"!doc":"<p>The property name for the data model, that serves as the original data.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"_removals":{"!doc":"<p>The list of removed elements.</p>\n"},"_resultsWatchHandle":{"!doc":"<p>The watch handle for model array elements.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getPlainValueOptions":{"!doc":"<p>The options to get plain value from stateful object.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"holdModelUntilCommit":{"!doc":"<p>True not to send the change in model back to sourceModel until commit() is called.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"originalModel":{"!doc":"<p>The data model, that serves as the original data.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"sourceModel":{},"store":{"!doc":"<p>The Dojo Object Store in use.</p>\n"},"addStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists.</p>\n"},"cloneModel":{"!type":"fn(value: ?) -> ?","!doc":"<p>Create a clone object of the data source.\nChild classes of this controller can override it to achieve its specific needs.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Send the change back to the data source.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up model watch handle as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"getStore":{"!type":"fn(id: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by its identity.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"putStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object.</p>\n"},"queryStore":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"removeStore":{"!type":"fn(id: ?, options: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Change the model back to its original state.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/Bind.ListController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/ModelRefController.prototype","!doc":"<p>A controller working with array model, managing its cursor.\nNOTE - If this class is used with a widget by data-dojo-mixins, make sure putting the widget in data-dojo-type and putting this class to data-dojo-mixins.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_listModelWatchHandle":{"!doc":"<p>The watch handle of model, watching for array elements.</p>\n"},"_refCursorProp":{"!doc":"<p>The property name for the data model of the current selection.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_tableModelWatchHandle":{"!doc":"<p>The watch handle of model.</p>\n"},"cursor":{"!doc":"<p>The selected element in the model array.</p>\n"},"cursorId":{"!doc":"<p>The ID of the selected element in the model array.</p>\n"},"cursorIndex":{"!doc":"<p>The index of the selected element in the model.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"idProperty":{"!doc":"<p>The property name in element in the model array, that works as its identifier.</p>\n"},"model":{"!doc":"<p>The data model working as an array.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Set a property to this.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/Bind.EditStoreRefListController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/EditStoreRefController.prototype","!doc":"<p>A child class of dojox/mvc/EditStoreRefController, mixed with ListController.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_listModelWatchHandle":{"!doc":"<p>The watch handle of model, watching for array elements.</p>\n"},"_refCursorProp":{"!doc":"<p>The property name for the data model of the current selection.</p>\n"},"_refEditModelProp":{"!doc":"<p>The property name for the data model, that is being edited.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refOriginalModelProp":{"!doc":"<p>The property name for the data model, that serves as the original data.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"_removals":{"!doc":"<p>The list of removed elements.</p>\n"},"_resultsWatchHandle":{"!doc":"<p>The watch handle for model array elements.</p>\n"},"_tableModelWatchHandle":{"!doc":"<p>The watch handle of model.</p>\n"},"cursor":{"!doc":"<p>The selected element in the model array.</p>\n"},"cursorId":{"!doc":"<p>The ID of the selected element in the model array.</p>\n"},"cursorIndex":{"!doc":"<p>The index of the selected element in the model.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getPlainValueOptions":{"!doc":"<p>The options to get plain value from stateful object.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"holdModelUntilCommit":{"!doc":"<p>True not to send the change in model back to sourceModel until commit() is called.</p>\n"},"idProperty":{"!doc":"<p>The property name in element in the model array, that works as its identifier.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"originalModel":{"!doc":"<p>The data model, that serves as the original data.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"sourceModel":{},"store":{"!doc":"<p>The Dojo Object Store in use.</p>\n"},"addStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists.</p>\n"},"cloneModel":{"!type":"fn(value: ?) -> ?","!doc":"<p>Create a clone object of the data source.\nChild classes of this controller can override it to achieve its specific needs.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Send the change back to the data source.</p>\n"},"commitCurrent":{"!type":"fn()","!doc":"<p>Send the change back to the data source for the current index.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up model watch handle as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"getStore":{"!type":"fn(id: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by its identity.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"putStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object.</p>\n"},"queryStore":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"removeStore":{"!type":"fn(id: ?, options: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Change the model back to its original state.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/Bind.Element":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget implicitly created by dojox/mvc/parserExtension.\nMaps &quot;value&quot; attribute to form element value, innerText/innerHTML to element&#39;s innerText/innerHTML, and other attributes to DOM attributes.\nAlso, for form element, updates value (or checked for check box) as user edits.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setInnerHTMLAttr":{},"_setInnerTextAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Set onchange event handler for form elements.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Bind._Container":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_containedWidgets":{"!doc":"<p>The array of contained widgets at any given point in time within this container.</p>\n"},"_parser":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.index}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inlineTemplateString":{"!doc":"<p>Same as templateString. Used when this widget is mixed with a regular templated widget.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside the container.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or content for this container. It is usually obtained from the\nbody of the container and may be modified or repeated over a collection/array.\nIn this simple implementation, attach points, attach events and WAI\nattributes are not supported in the template.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Bind.Group":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A simple model-bound container widget with single-node binding to a data model.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"target":{"!doc":"<p>The data model used for relative data binding.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Bind.Generate":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/_Container.prototype","!doc":"<p>A container that generates a view based on the data model its bound to.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_containedWidgets":{"!doc":"<p>The array of contained widgets at any given point in time within this container.</p>\n"},"_counter":{"!doc":"<p>A count maintained internally to always generate predictable widget\nIDs in the view generated by this container.</p>\n"},"_defaultClassMapping":{},"_defaultIdNameMapping":{},"_defaultWidgetMapping":{},"_parser":{},"_relTargetProp":{"!doc":"<p>The name of the property that is used by child widgets for relative data binding.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"children":{"!doc":"<p>The array of data model that is used to render child nodes.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.index}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inlineTemplateString":{"!doc":"<p>Same as templateString. Used when this widget is mixed with a regular templated widget.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside the container.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or content for this container. It is usually obtained from the\nbody of the container and may be modified or repeated over a collection/array.\nIn this simple implementation, attach points, attach events and WAI\nattributes are not supported in the template.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Bind.Output":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A simple widget that displays templated output, parts of which may\nbe data-bound.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.value}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or data-bound output content.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postscript":{"!type":"fn(params: ?, srcNodeRef: ?)","!doc":"<p>Override and save template from body.</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Override and refresh output on value change.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Bind.Repeat":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/_Container.prototype","!doc":"<p>A model-bound container which binds to a collection within a data model\nand produces a repeating user-interface from a template for each\niteration within the collection.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_containedWidgets":{"!doc":"<p>The array of contained widgets at any given point in time within this container.</p>\n"},"_parser":{},"_relTargetProp":{"!doc":"<p>The name of the property that is used by child widgets for relative data binding.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"children":{"!doc":"<p>The array of data model that is used to render child nodes.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.index}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"index":{"!doc":"<p>An index used to track the current iteration when the repeating UI is\nproduced. This may be used to parameterize the content in the repeat\ntemplate for the current iteration.\n\n</p>\n<p>For example, consider a collection of search or query results where\neach item contains a &quot;Name&quot; property used to prime the &quot;Results&quot; data\nmodel. Then, the following CRUD-style UI displays all the names in\nthe search results in text boxes where they may be updated or such.\n\n\n</p>\n<pre><code>&lt;div dojoType=&quot;dojox/mvc/Repeat&quot; ref=&quot;Results&quot;&gt;\n    &lt;div class=&quot;row&quot; dojoType=&quot;dojox/mvc/Group&quot; ref=&quot;${this.index}&quot;&gt;\n        &lt;label for=&quot;nameInput${this.index}&quot;&gt;Name:&lt;/label&gt;\n        &lt;input dojoType=&quot;dijit/form/TextBox&quot; id=&quot;nameInput${this.index}&quot; ref=&quot;&#39;Name&#39;&quot;&gt;&lt;/input&gt;\n    &lt;/div&gt;\n&lt;/div&gt;</code></pre>\n"},"inlineTemplateString":{"!doc":"<p>Same as templateString. Used when this widget is mixed with a regular templated widget.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"removeRepeatNode":{"!doc":"<p>When true the dom node for the Repeat and Groups within the Repeat\nwill be removed, their children will be placed into the parent node \nof the Repeat node.  This should be set to true when working with \na Repeat inside of a dojox.mobile list.     </p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside the container.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or content for this container. It is usually obtained from the\nbody of the container and may be modified or repeated over a collection/array.\nIn this simple implementation, attach points, attach events and WAI\nattributes are not supported in the template.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useParent":{"!doc":"<p>id of the DOM node to use as the parent for the repeating items, similar to useParentId processed a little differently </p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postscript":{"!type":"fn(params: ?, srcNodeRef: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Bind.StatefulModel":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>Deprecated.  Use dojox/mvc/getStateful, dojox/mvc/getPlainValue, dojox/mvc/StatefulArray or one of the dojox/mvc/*RefControllers instead.\nThe first-class native JavaScript data model based on dojo/Stateful\nthat wraps any data structure(s) that may be relevant for a view,\na view portion, a dijit or any custom view layer component.\n</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"data":{"!doc":"<p>The plain JavaScript object / data structure used to initialize\nthis model. At any point in time, it holds the lasted saved model\nstate.\nEither data or store property must be provided.</p>\n"},"store":{"!doc":"<p>The data store from where to retrieve initial data for this model.\nAn optional query may also be provided along with this store.\nEither data or store property must be provided.</p>\n"},"valid":{"!doc":"<p>Whether this model deems the associated data to be valid.</p>\n"},"value":{"!doc":"<p>The associated value (if this is a leaf node). The value of\nintermediate nodes in the model is not defined.</p>\n"},"add":{"!type":"fn(name: ?, stateful: ?)","!doc":"<p>Adds a dojo/Stateful tree represented by the given\ndojox/mvc/StatefulModel at the given property name.</p>\n"},"commit":{"!type":"fn(store: ?)","!doc":"<p>Commits this data model:\n\n</p>\n<ul>\n<li>Saves the current state such that a subsequent reset will not\nundo any prior changes.</li>\n<li>Persists client-side changes to the data store, if a store\nhas been supplied as a parameter or at instantiation.</li>\n</ul>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"remove":{"!type":"fn(name: ?)","!doc":"<p>Removes the dojo/Stateful tree at the given property name.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets this data model values to its original state.\nStructural changes to the data model (such as adds or removes)\nare not restored.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"splice":{"!type":"fn(idx: ?, n: ?) -> +dojox","!doc":"<p>Removes and then adds some elements to this array.\nUpdates the removed/added elements, as well as the length, as stateful.</p>\n"},"toPlainObject":{"!type":"fn() -> +any","!doc":"<p>Produces a plain JavaScript object representation of the data\ncurrently within this data model.</p>\n"},"toString":{"!type":"fn() -> +any","!doc":"<p>Returns the string representation of the data currently within this data model.</p>\n"},"valueOf":{"!type":"fn() -> +any","!doc":"<p>Returns the value representation of the data currently within this data model.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"getPlainValueOptions":{"!doc":"<p>An object that defines how plain value should be created from model object.</p>\n"},"getStatefulOptions":{"!doc":"<p>An object that defines how model object should be created from plain object hierarchy.</p>\n"}},"dojox/mvc/Bind.StatefulSeries":{"!type":"fn()","prototype":{"!doc":"<p>Chart data plugin (&quot;series&quot;) that watches for properties specified in dojox/mvc/at handles in the given data.\nAt initialization, and when the properties are updated, creates the data from data given and updates the chart.</p>\n","constructor":{"!type":"fn(items: ?)"},"destroy":{"!type":"fn()"},"setSeriesObject":{"!type":"fn(series: ?)"}}},"dojox/mvc/Bind.Templated":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A templated widget, mostly the same as dijit/_Templated, but without deprecated features in it.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"bindings":{"!doc":"<p>The data binding declaration (or simple parameters) for child widgets.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Bind.WidgetList":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget that creates child widgets repeatedly based on the children attribute (the repeated data) and childType/childMixins/childParams attributes (determines how to create each child widget).</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_relTargetProp":{"!doc":"<p>The name of the property that is used by child widgets for relative data binding.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"childBindings":{"!doc":"<p>Data bindings for child widget.</p>\n"},"childClz":{"!doc":"<p>The class of the child widget. Takes precedence over childType/childMixins.</p>\n"},"childMixins":{"!doc":"<p>The list of module IDs (separated by comma), or a functinon that takes child data as the argument and returns it, of the classes that will be mixed into child widget. childClz takes precedence over childType/this.\nCan be specified via data-mvc-child-mixins attribute of widget declaration.</p>\n"},"childParams":{"!doc":"<p>The mixin properties for child widget.\nCan be specified via data-mvc-child-props attribute of widget declaration.\n&quot;this&quot; in data-mvc-child-props will have the following properties:\n\n</p>\n<ul>\n<li>parent - This widget&#39;s instance.</li>\n<li>target - The data item in children.</li>\n</ul>\n"},"children":{"!doc":"<p>The array of data model that is used to render child nodes.</p>\n"},"childType":{"!doc":"<p>The module ID of child widget, or a function that takes child data as the argument and returns the module ID of child widget. childClz takes precedence over this/childMixins.\nCan be specified via data-mvc-child-type attribute of widget declaration.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"data-mvc-child-bindings":{},"data-mvc-child-mixins":{},"data-mvc-child-props":{},"data-mvc-child-type":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"partialRebuild":{"!doc":"<p>If true, only rebuild repeat items for changed elements. Otherwise, rebuild everything if there is a change in children.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template string for each child items. templateString in child widgets take precedence over this.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Bind._DataBindingMixin":{"!type":"fn()","prototype":{"!doc":"<p>Deprecated.  Use dojox/mvc/at for data binding.\nProvides the ability for dijits or custom view components to become\ndata binding aware.\n</p>\n","binding":{"!doc":"<p>The read only value of the resolved data binding for this widget.\nThis may be a result of resolving various relative refs along\nthe parent axis.</p>\n"},"ref":{"!doc":"<p>The value of the data binding expression passed declaratively by\nthe developer. This usually references a location within an\nexisting datamodel and may be a relative reference based on the\nparent / container data binding (dot-separated string).</p>\n"},"isValid":{"!type":"fn() -> +any","!doc":"<p>Returns the validity of the data binding.</p>\n"}}},"dojox/mvc/Bind._InlineTemplateMixin":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for template widget, which will look for <code>&lt;script type=&quot;dojox/mvc/InlineTemplate&quot;&gt;</code>\nand treat the HTML in there as the template string.</p>\n","buildRendering":{"!type":"fn()"}}},"dojox/mvc/EditModelRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/ModelRefController.prototype","!doc":"<p>A child class of dojox/mvc/ModelRefController.\nKeeps a copy (originalModel) of given data model (sourceModel) so that it can manage the data model of before/after the edit.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refEditModelProp":{"!doc":"<p>The property name for the data model, that is being edited.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refOriginalModelProp":{"!doc":"<p>The property name for the data model, that serves as the original data.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getPlainValueOptions":{"!doc":"<p>The options to get plain value from stateful object.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"holdModelUntilCommit":{"!doc":"<p>True not to send the change in model back to sourceModel until commit() is called.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"originalModel":{"!doc":"<p>The data model, that serves as the original data.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"sourceModel":{},"cloneModel":{"!type":"fn(value: ?) -> ?","!doc":"<p>Create a clone object of the data source.\nChild classes of this controller can override it to achieve its specific needs.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Send the change back to the data source.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets certain properties before setting models.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Change the model back to its original state.</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Set a property to this.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/getPlainValue":{"!doc":"<p>Create a raw value from a dojo/Stateful object.</p>\n","prototype":{"constructor":{"!type":"fn(value: ?, options: ?) -> +any","!doc":"<p>Create a raw value from a dojo/Stateful object.</p>\n"}},"getPlainArray":{"!type":"fn(a: ?) -> +any","!doc":"<p>Returns the stateful version of the given array.</p>\n"},"getPlainObject":{"!type":"fn(o: ?) -> ?","!doc":"<p>Returns the stateful version of the given object.</p>\n"},"getPlainValue":{"!type":"fn(v: ?) -> +Anything","!doc":"<p>Just returns the given value.</p>\n"},"getType":{"!type":"fn(v: ?) -> +any","!doc":"<p>Returns the type of the given value.</p>\n"}},"dojox/mvc/ModelRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/_Controller.prototype","!doc":"<p>A controller that keeps a reference to dojo/Stateful-based data model.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/_Controller":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?) -> ?","!doc":"<p>If this object is not called from Dojo parser, starts this up right away.\nAlso, if widget registry is available, register this object.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_atBindingMixin":{"!type":"fn()","prototype":{"!doc":"<p>The mixin for dijit/_WidgetBase to support data binding.</p>\n","data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"}},"mixin":{"!doc":"<p>The mixin for dijit/_WidgetBase to support data binding.</p>\n"}},"dojox/mvc/_atBindingMixin.mixin":{"!doc":"<p>The mixin for dijit/_WidgetBase to support data binding.</p>\n","data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"}},"dojox/mvc/resolve":{"!doc":"<p>Find a dojo/Stateful for the target.</p>\n","prototype":{"constructor":{"!type":"fn(target: ?, parent?: ?) -> ?","!doc":"<p>Find a dojo/Stateful for the target.</p>\n"}}},"dojox/mvc/sync":{"!doc":"<p>Synchronize two dojo/Stateful properties.</p>\n","both":{"!doc":"<p>Data binding goes in both directions (dojox/mvc/Bind.from | dojox/mvc/Bind.to)</p>\n"},"from":{"!doc":"<p>Data binding goes from the source to the target</p>\n"},"to":{"!doc":"<p>Data binding goes from the target to the source</p>\n"},"prototype":{"constructor":{"!type":"fn(source: ?, sourceProp: ?, target: ?, targetProp: ?, options: ?) -> +any","!doc":"<p>Synchronize two dojo/Stateful properties.</p>\n"}},"equals":{"!type":"fn(dst: ?, src: ?) -> bool","!doc":"<p>Returns if the given two values are equal.</p>\n"}},"dojox/mvc/EditStoreRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/StoreRefController.prototype","!doc":"<p>A child class of dojox/mvc/StoreRefController, managing edits.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refEditModelProp":{"!doc":"<p>The property name for the data model, that is being edited.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refOriginalModelProp":{"!doc":"<p>The property name for the data model, that serves as the original data.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"_removals":{"!doc":"<p>The list of removed elements.</p>\n"},"_resultsWatchHandle":{"!doc":"<p>The watch handle for model array elements.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getPlainValueOptions":{"!doc":"<p>The options to get plain value from stateful object.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"holdModelUntilCommit":{"!doc":"<p>True not to send the change in model back to sourceModel until commit() is called.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"originalModel":{"!doc":"<p>The data model, that serves as the original data.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"sourceModel":{},"store":{"!doc":"<p>The Dojo Object Store in use.</p>\n"},"addStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists.</p>\n"},"cloneModel":{"!type":"fn(value: ?) -> ?","!doc":"<p>Create a clone object of the data source.\nChild classes of this controller can override it to achieve its specific needs.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Send the change back to the data source.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up model watch handle as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"getStore":{"!type":"fn(id: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by its identity.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"putStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object.</p>\n"},"queryStore":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"removeStore":{"!type":"fn(id: ?, options: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Change the model back to its original state.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/StoreRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/ModelRefController.prototype","!doc":"<p>A child class of dojox.mvc.ModelRefController, which keeps a reference to Dojo Object Store (in store property).</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"store":{"!doc":"<p>The Dojo Object Store in use.</p>\n"},"addStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"getStore":{"!type":"fn(id: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by its identity.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"putStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object.</p>\n"},"queryStore":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"removeStore":{"!type":"fn(id: ?, options: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/EditStoreRefListController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/EditStoreRefController.prototype","!doc":"<p>A child class of dojox/mvc/EditStoreRefController, mixed with ListController.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_listModelWatchHandle":{"!doc":"<p>The watch handle of model, watching for array elements.</p>\n"},"_refCursorProp":{"!doc":"<p>The property name for the data model of the current selection.</p>\n"},"_refEditModelProp":{"!doc":"<p>The property name for the data model, that is being edited.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refOriginalModelProp":{"!doc":"<p>The property name for the data model, that serves as the original data.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"_removals":{"!doc":"<p>The list of removed elements.</p>\n"},"_resultsWatchHandle":{"!doc":"<p>The watch handle for model array elements.</p>\n"},"_tableModelWatchHandle":{"!doc":"<p>The watch handle of model.</p>\n"},"cursor":{"!doc":"<p>The selected element in the model array.</p>\n"},"cursorId":{"!doc":"<p>The ID of the selected element in the model array.</p>\n"},"cursorIndex":{"!doc":"<p>The index of the selected element in the model.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getPlainValueOptions":{"!doc":"<p>The options to get plain value from stateful object.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"holdModelUntilCommit":{"!doc":"<p>True not to send the change in model back to sourceModel until commit() is called.</p>\n"},"idProperty":{"!doc":"<p>The property name in element in the model array, that works as its identifier.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"originalModel":{"!doc":"<p>The data model, that serves as the original data.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"sourceModel":{},"store":{"!doc":"<p>The Dojo Object Store in use.</p>\n"},"addStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists.</p>\n"},"cloneModel":{"!type":"fn(value: ?) -> ?","!doc":"<p>Create a clone object of the data source.\nChild classes of this controller can override it to achieve its specific needs.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Send the change back to the data source.</p>\n"},"commitCurrent":{"!type":"fn()","!doc":"<p>Send the change back to the data source for the current index.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up model watch handle as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"getStore":{"!type":"fn(id: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by its identity.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"putStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object.</p>\n"},"queryStore":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"removeStore":{"!type":"fn(id: ?, options: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Change the model back to its original state.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/ListController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/ModelRefController.prototype","!doc":"<p>A controller working with array model, managing its cursor.\nNOTE - If this class is used with a widget by data-dojo-mixins, make sure putting the widget in data-dojo-type and putting this class to data-dojo-mixins.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_listModelWatchHandle":{"!doc":"<p>The watch handle of model, watching for array elements.</p>\n"},"_refCursorProp":{"!doc":"<p>The property name for the data model of the current selection.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_tableModelWatchHandle":{"!doc":"<p>The watch handle of model.</p>\n"},"cursor":{"!doc":"<p>The selected element in the model array.</p>\n"},"cursorId":{"!doc":"<p>The ID of the selected element in the model array.</p>\n"},"cursorIndex":{"!doc":"<p>The index of the selected element in the model.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"idProperty":{"!doc":"<p>The property name in element in the model array, that works as its identifier.</p>\n"},"model":{"!doc":"<p>The data model working as an array.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Set a property to this.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/Element":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget implicitly created by dojox/mvc/parserExtension.\nMaps &quot;value&quot; attribute to form element value, innerText/innerHTML to element&#39;s innerText/innerHTML, and other attributes to DOM attributes.\nAlso, for form element, updates value (or checked for check box) as user edits.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setInnerHTMLAttr":{},"_setInnerTextAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Set onchange event handler for form elements.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Generate":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/_Container.prototype","!doc":"<p>A container that generates a view based on the data model its bound to.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_containedWidgets":{"!doc":"<p>The array of contained widgets at any given point in time within this container.</p>\n"},"_counter":{"!doc":"<p>A count maintained internally to always generate predictable widget\nIDs in the view generated by this container.</p>\n"},"_defaultClassMapping":{},"_defaultIdNameMapping":{},"_defaultWidgetMapping":{},"_parser":{},"_relTargetProp":{"!doc":"<p>The name of the property that is used by child widgets for relative data binding.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"children":{"!doc":"<p>The array of data model that is used to render child nodes.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.index}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inlineTemplateString":{"!doc":"<p>Same as templateString. Used when this widget is mixed with a regular templated widget.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside the container.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or content for this container. It is usually obtained from the\nbody of the container and may be modified or repeated over a collection/array.\nIn this simple implementation, attach points, attach events and WAI\nattributes are not supported in the template.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_Container":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_containedWidgets":{"!doc":"<p>The array of contained widgets at any given point in time within this container.</p>\n"},"_parser":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.index}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inlineTemplateString":{"!doc":"<p>Same as templateString. Used when this widget is mixed with a regular templated widget.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside the container.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or content for this container. It is usually obtained from the\nbody of the container and may be modified or repeated over a collection/array.\nIn this simple implementation, attach points, attach events and WAI\nattributes are not supported in the template.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/at":{"!doc":"<p>Returns a pointer to data binding target (a dojo/Stateful property), called at handle, which is used for start synchronization with data binding source (another dojo/Stateful property).</p>\n","both":{"!doc":"<p>Data binding goes in both directions (dojox/mvc/Bind.from | dojox/mvc/Bind.to)</p>\n"},"from":{"!doc":"<p>Data binding goes from the source to the target</p>\n"},"handle":{"!doc":"<p>A handle of data binding target (a dojo/Stateful property), which is used for start synchronization with data binding source (another dojo/Stateful property).</p>\n"},"to":{"!doc":"<p>Data binding goes from the target to the source</p>\n"},"prototype":{"constructor":{"!type":"fn(target: ?, targetProp: ?) -> +any","!doc":"<p>Returns a pointer to data binding target (a dojo/Stateful property), called at handle, which is used for start synchronization with data binding source (another dojo/Stateful property).</p>\n"}}},"dojox/mvc/at.handle":{"!doc":"<p>A handle of data binding target (a dojo/Stateful property), which is used for start synchronization with data binding source (another dojo/Stateful property).</p>\n","bindDirection":{"!doc":"<p>The data binding bindDirection, choose from: dojox/mvc/sync.from, dojox/mvc/sync.to or dojox/mvc/sync.both.</p>\n"},"converter":{"!doc":"<p>Class/object containing the converter functions used when the data goes between data binding target (e.g. data model or controller) to data binding origin (e.g. widget).</p>\n"},"target":{"!doc":"<p>The data binding literal or dojo/Stateful to be synchronized.</p>\n"},"targetProp":{"!doc":"<p>The property name in target to be synchronized.</p>\n"},"direction":{"!type":"fn(bindDirection: ?)","!doc":"<p>Sets data binding bindDirection.</p>\n"},"equals":{"!type":"fn(equals: ?)","!doc":"<p>Sets a function to check if a value has really been changed when source/target dojo/Stateful changes.</p>\n"},"transform":{"!type":"fn(converter: ?)","!doc":"<p>Attach a data converter.</p>\n"}},"dojox/mvc/_atBindingExtension":{},"dojox/mvc/atBindingExtension":{"!doc":"<p>Monkey-patch the given widget so that they looks at dojox/mvc/at set in them and start data binding specified there.</p>\n","prototype":{"constructor":{"!type":"fn(w: ?) -> ?","!doc":"<p>Monkey-patch the given widget so that they looks at dojox/mvc/at set in them and start data binding specified there.</p>\n"}}},"dojox/mvc/Group":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A simple model-bound container widget with single-node binding to a data model.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"target":{"!doc":"<p>The data model used for relative data binding.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Output":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A simple widget that displays templated output, parts of which may\nbe data-bound.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.value}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or data-bound output content.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postscript":{"!type":"fn(params: ?, srcNodeRef: ?)","!doc":"<p>Override and save template from body.</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Override and refresh output on value change.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/Repeat":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/_Container.prototype","!doc":"<p>A model-bound container which binds to a collection within a data model\nand produces a repeating user-interface from a template for each\niteration within the collection.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_containedWidgets":{"!doc":"<p>The array of contained widgets at any given point in time within this container.</p>\n"},"_parser":{},"_relTargetProp":{"!doc":"<p>The name of the property that is used by child widgets for relative data binding.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"children":{"!doc":"<p>The array of data model that is used to render child nodes.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.index}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"index":{"!doc":"<p>An index used to track the current iteration when the repeating UI is\nproduced. This may be used to parameterize the content in the repeat\ntemplate for the current iteration.\n\n</p>\n<p>For example, consider a collection of search or query results where\neach item contains a &quot;Name&quot; property used to prime the &quot;Results&quot; data\nmodel. Then, the following CRUD-style UI displays all the names in\nthe search results in text boxes where they may be updated or such.\n\n\n</p>\n<pre><code>&lt;div dojoType=&quot;dojox/mvc/Repeat&quot; ref=&quot;Results&quot;&gt;\n    &lt;div class=&quot;row&quot; dojoType=&quot;dojox/mvc/Group&quot; ref=&quot;${this.index}&quot;&gt;\n        &lt;label for=&quot;nameInput${this.index}&quot;&gt;Name:&lt;/label&gt;\n        &lt;input dojoType=&quot;dijit/form/TextBox&quot; id=&quot;nameInput${this.index}&quot; ref=&quot;&#39;Name&#39;&quot;&gt;&lt;/input&gt;\n    &lt;/div&gt;\n&lt;/div&gt;</code></pre>\n"},"inlineTemplateString":{"!doc":"<p>Same as templateString. Used when this widget is mixed with a regular templated widget.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"removeRepeatNode":{"!doc":"<p>When true the dom node for the Repeat and Groups within the Repeat\nwill be removed, their children will be placed into the parent node \nof the Repeat node.  This should be set to true when working with \na Repeat inside of a dojox.mobile list.     </p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside the container.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or content for this container. It is usually obtained from the\nbody of the container and may be modified or repeated over a collection/array.\nIn this simple implementation, attach points, attach events and WAI\nattributes are not supported in the template.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useParent":{"!doc":"<p>id of the DOM node to use as the parent for the repeating items, similar to useParentId processed a little differently </p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postscript":{"!type":"fn(params: ?, srcNodeRef: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/StatefulModel":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>Deprecated.  Use dojox/mvc/getStateful, dojox/mvc/getPlainValue, dojox/mvc/StatefulArray or one of the dojox/mvc/*RefControllers instead.\nThe first-class native JavaScript data model based on dojo/Stateful\nthat wraps any data structure(s) that may be relevant for a view,\na view portion, a dijit or any custom view layer component.\n</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"data":{"!doc":"<p>The plain JavaScript object / data structure used to initialize\nthis model. At any point in time, it holds the lasted saved model\nstate.\nEither data or store property must be provided.</p>\n"},"store":{"!doc":"<p>The data store from where to retrieve initial data for this model.\nAn optional query may also be provided along with this store.\nEither data or store property must be provided.</p>\n"},"valid":{"!doc":"<p>Whether this model deems the associated data to be valid.</p>\n"},"value":{"!doc":"<p>The associated value (if this is a leaf node). The value of\nintermediate nodes in the model is not defined.</p>\n"},"add":{"!type":"fn(name: ?, stateful: ?)","!doc":"<p>Adds a dojo/Stateful tree represented by the given\ndojox/mvc/StatefulModel at the given property name.</p>\n"},"commit":{"!type":"fn(store: ?)","!doc":"<p>Commits this data model:\n\n</p>\n<ul>\n<li>Saves the current state such that a subsequent reset will not\nundo any prior changes.</li>\n<li>Persists client-side changes to the data store, if a store\nhas been supplied as a parameter or at instantiation.</li>\n</ul>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"remove":{"!type":"fn(name: ?)","!doc":"<p>Removes the dojo/Stateful tree at the given property name.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets this data model values to its original state.\nStructural changes to the data model (such as adds or removes)\nare not restored.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"splice":{"!type":"fn(idx: ?, n: ?) -> +dojox","!doc":"<p>Removes and then adds some elements to this array.\nUpdates the removed/added elements, as well as the length, as stateful.</p>\n"},"toPlainObject":{"!type":"fn() -> +any","!doc":"<p>Produces a plain JavaScript object representation of the data\ncurrently within this data model.</p>\n"},"toString":{"!type":"fn() -> +any","!doc":"<p>Returns the string representation of the data currently within this data model.</p>\n"},"valueOf":{"!type":"fn() -> +any","!doc":"<p>Returns the value representation of the data currently within this data model.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"getPlainValueOptions":{"!doc":"<p>An object that defines how plain value should be created from model object.</p>\n"},"getStatefulOptions":{"!doc":"<p>An object that defines how model object should be created from plain object hierarchy.</p>\n"}},"dojox/mvc/StatefulModel.getStatefulOptions":{"!doc":"<p>An object that defines how model object should be created from plain object hierarchy.</p>\n","getStatefulArray":{"!type":"fn(a: ?) -> +instance","!doc":"<p>Create a stateful array from a plain array.</p>\n"},"getStatefulObject":{"!type":"fn(o: ?) -> +instance","!doc":"<p>Create a stateful object from a plain object.</p>\n"},"getStatefulValue":{"!type":"fn(v: ?) -> +instance","!doc":"<p>Create a stateful value from a plain value.</p>\n"},"getType":{"!type":"fn(v: ?) -> string","!doc":"<p>Returns the type of the given value.</p>\n"}},"dojox/mvc/StatefulModel.getPlainValueOptions":{"!doc":"<p>An object that defines how plain value should be created from model object.</p>\n","getPlainArray":{"!type":"fn(a: ?) -> ?"},"getPlainObject":{"!type":"fn(o: ?) -> ?"},"getPlainValue":{"!type":"fn(v: ?) -> +Anything"},"getType":{"!type":"fn(v: ?) -> string","!doc":"<p>Returns the type of the given value.</p>\n"}},"dojox/mvc/StatefulSeries":{"!type":"fn()","prototype":{"!doc":"<p>Chart data plugin (&quot;series&quot;) that watches for properties specified in dojox/mvc/at handles in the given data.\nAt initialization, and when the properties are updated, creates the data from data given and updates the chart.</p>\n","constructor":{"!type":"fn(items: ?)"},"destroy":{"!type":"fn()"},"setSeriesObject":{"!type":"fn(series: ?)"}}},"dojox/mvc/Templated":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A templated widget, mostly the same as dijit/_Templated, but without deprecated features in it.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"bindings":{"!doc":"<p>The data binding declaration (or simple parameters) for child widgets.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/WidgetList":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget that creates child widgets repeatedly based on the children attribute (the repeated data) and childType/childMixins/childParams attributes (determines how to create each child widget).</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_relTargetProp":{"!doc":"<p>The name of the property that is used by child widgets for relative data binding.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"childBindings":{"!doc":"<p>Data bindings for child widget.</p>\n"},"childClz":{"!doc":"<p>The class of the child widget. Takes precedence over childType/childMixins.</p>\n"},"childMixins":{"!doc":"<p>The list of module IDs (separated by comma), or a functinon that takes child data as the argument and returns it, of the classes that will be mixed into child widget. childClz takes precedence over childType/this.\nCan be specified via data-mvc-child-mixins attribute of widget declaration.</p>\n"},"childParams":{"!doc":"<p>The mixin properties for child widget.\nCan be specified via data-mvc-child-props attribute of widget declaration.\n&quot;this&quot; in data-mvc-child-props will have the following properties:\n\n</p>\n<ul>\n<li>parent - This widget&#39;s instance.</li>\n<li>target - The data item in children.</li>\n</ul>\n"},"children":{"!doc":"<p>The array of data model that is used to render child nodes.</p>\n"},"childType":{"!doc":"<p>The module ID of child widget, or a function that takes child data as the argument and returns the module ID of child widget. childClz takes precedence over this/childMixins.\nCan be specified via data-mvc-child-type attribute of widget declaration.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"data-mvc-child-bindings":{},"data-mvc-child-mixins":{},"data-mvc-child-props":{},"data-mvc-child-type":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"partialRebuild":{"!doc":"<p>If true, only rebuild repeat items for changed elements. Otherwise, rebuild everything if there is a change in children.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template string for each child items. templateString in child widgets take precedence over this.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_DataBindingExtension":{},"dojox/mvc/_DataBindingMixin":{"!type":"fn()","prototype":{"!doc":"<p>Deprecated.  Use dojox/mvc/at for data binding.\nProvides the ability for dijits or custom view components to become\ndata binding aware.\n</p>\n","binding":{"!doc":"<p>The read only value of the resolved data binding for this widget.\nThis may be a result of resolving various relative refs along\nthe parent axis.</p>\n"},"ref":{"!doc":"<p>The value of the data binding expression passed declaratively by\nthe developer. This usually references a location within an\nexisting datamodel and may be a relative reference based on the\nparent / container data binding (dot-separated string).</p>\n"},"isValid":{"!type":"fn() -> +any","!doc":"<p>Returns the validity of the data binding.</p>\n"}}},"dojox/mvc/_InlineTemplateMixin":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for template widget, which will look for <code>&lt;script type=&quot;dojox/mvc/InlineTemplate&quot;&gt;</code>\nand treat the HTML in there as the template string.</p>\n","buildRendering":{"!type":"fn()"}}},"dojox/mvc/_TextBoxExtensions":{},"dojox/mvc/_base":{"at":{"!type":"fn(target: ?, targetProp: ?) -> +any","!doc":"<p>Returns a pointer to data binding target (a dojo/Stateful property), called at handle, which is used for start synchronization with data binding source (another dojo/Stateful property).</p>\n"},"bind":{"!type":"fn(source: ?, sourceProp: ?, target: ?, targetProp: ?, func?: ?, bindOnlyIfUnequal?: ?) -> ?","!doc":"<p>Bind the specified property of the target to the specified\nproperty of the source with the supplied transformation.</p>\n"},"bindInputs":{"!type":"fn(sourceBindArray: ?, func: ?) -> []","!doc":"<p>Bind the values at the sources specified in the first argument\narray such that a composing function in the second argument is\ncalled when any of the values changes.</p>\n"},"EditModelRefController":{"!type":"fn()"},"EditStoreRefController":{"!type":"fn()"},"EditStoreRefListController":{"!type":"fn()"},"Element":{"!type":"fn()"},"equals":{"!type":"fn(dst: ?, src: ?, options: ?) -> bool","!doc":"<p>Compares two dojo/Stateful objects, by diving into the leaves.</p>\n"},"Generate":{"!type":"fn()"},"getPlainValue":{"!type":"fn(value: ?, options: ?) -> +any","!doc":"<p>Create a raw value from a dojo/Stateful object.</p>\n"},"getStateful":{"!type":"fn(value: ?, options: ?) -> +Anything","!doc":"<p>Create a dojo/Stateful object from a raw value.</p>\n"},"Group":{"!type":"fn()"},"ListController":{"!type":"fn()"},"ModelRefController":{"!type":"fn()"},"newStatefulModel":{"!type":"fn(args: ?)","!doc":"<p>Factory method that instantiates a new data model that view\ncomponents may bind to.</p>\n"},"Output":{"!type":"fn()"},"Repeat":{"!type":"fn()"},"resolve":{"!type":"fn(target: ?, parent?: ?) -> ?","!doc":"<p>Find a dojo/Stateful for the target.</p>\n"},"StatefulArray":{"!type":"fn(a: ?) -> ?","!doc":"<p>An inheritance of native JavaScript array, that adds dojo/Stateful capability.</p>\n"},"StatefulModel":{"!type":"fn()"},"StatefulSeries":{"!type":"fn()"},"StoreRefController":{"!type":"fn()"},"sync":{"!type":"fn(source: ?, sourceProp: ?, target: ?, targetProp: ?, options: ?) -> +any","!doc":"<p>Synchronize two dojo/Stateful properties.</p>\n"},"Templated":{"!type":"fn()"},"WidgetList":{"!type":"fn()"}},"dojox/mvc/_base._Controller":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?) -> ?","!doc":"<p>If this object is not called from Dojo parser, starts this up right away.\nAlso, if widget registry is available, register this object.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_base.ModelRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/_Controller.prototype","!doc":"<p>A controller that keeps a reference to dojo/Stateful-based data model.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/_base.EditModelRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/ModelRefController.prototype","!doc":"<p>A child class of dojox/mvc/ModelRefController.\nKeeps a copy (originalModel) of given data model (sourceModel) so that it can manage the data model of before/after the edit.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refEditModelProp":{"!doc":"<p>The property name for the data model, that is being edited.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refOriginalModelProp":{"!doc":"<p>The property name for the data model, that serves as the original data.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getPlainValueOptions":{"!doc":"<p>The options to get plain value from stateful object.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"holdModelUntilCommit":{"!doc":"<p>True not to send the change in model back to sourceModel until commit() is called.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"originalModel":{"!doc":"<p>The data model, that serves as the original data.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"sourceModel":{},"cloneModel":{"!type":"fn(value: ?) -> ?","!doc":"<p>Create a clone object of the data source.\nChild classes of this controller can override it to achieve its specific needs.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Send the change back to the data source.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets certain properties before setting models.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Change the model back to its original state.</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Set a property to this.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/_base.StoreRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/ModelRefController.prototype","!doc":"<p>A child class of dojox.mvc.ModelRefController, which keeps a reference to Dojo Object Store (in store property).</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"store":{"!doc":"<p>The Dojo Object Store in use.</p>\n"},"addStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Stops data binding as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"getStore":{"!type":"fn(id: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by its identity.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"putStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object.</p>\n"},"queryStore":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"removeStore":{"!type":"fn(id: ?, options: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/_base.EditStoreRefController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/StoreRefController.prototype","!doc":"<p>A child class of dojox/mvc/StoreRefController, managing edits.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_refEditModelProp":{"!doc":"<p>The property name for the data model, that is being edited.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refOriginalModelProp":{"!doc":"<p>The property name for the data model, that serves as the original data.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"_removals":{"!doc":"<p>The list of removed elements.</p>\n"},"_resultsWatchHandle":{"!doc":"<p>The watch handle for model array elements.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getPlainValueOptions":{"!doc":"<p>The options to get plain value from stateful object.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"holdModelUntilCommit":{"!doc":"<p>True not to send the change in model back to sourceModel until commit() is called.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"originalModel":{"!doc":"<p>The data model, that serves as the original data.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"sourceModel":{},"store":{"!doc":"<p>The Dojo Object Store in use.</p>\n"},"addStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists.</p>\n"},"cloneModel":{"!type":"fn(value: ?) -> ?","!doc":"<p>Create a clone object of the data source.\nChild classes of this controller can override it to achieve its specific needs.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Send the change back to the data source.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up model watch handle as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"getStore":{"!type":"fn(id: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by its identity.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"putStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object.</p>\n"},"queryStore":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"removeStore":{"!type":"fn(id: ?, options: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Change the model back to its original state.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/_base.ListController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/ModelRefController.prototype","!doc":"<p>A controller working with array model, managing its cursor.\nNOTE - If this class is used with a widget by data-dojo-mixins, make sure putting the widget in data-dojo-type and putting this class to data-dojo-mixins.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_listModelWatchHandle":{"!doc":"<p>The watch handle of model, watching for array elements.</p>\n"},"_refCursorProp":{"!doc":"<p>The property name for the data model of the current selection.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_tableModelWatchHandle":{"!doc":"<p>The watch handle of model.</p>\n"},"cursor":{"!doc":"<p>The selected element in the model array.</p>\n"},"cursorId":{"!doc":"<p>The ID of the selected element in the model array.</p>\n"},"cursorIndex":{"!doc":"<p>The index of the selected element in the model.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"idProperty":{"!doc":"<p>The property name in element in the model array, that works as its identifier.</p>\n"},"model":{"!doc":"<p>The data model working as an array.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"destroy":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Set a property to this.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/_base.EditStoreRefListController":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/EditStoreRefController.prototype","!doc":"<p>A child class of dojox/mvc/EditStoreRefController, mixed with ListController.</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"_listModelWatchHandle":{"!doc":"<p>The watch handle of model, watching for array elements.</p>\n"},"_refCursorProp":{"!doc":"<p>The property name for the data model of the current selection.</p>\n"},"_refEditModelProp":{"!doc":"<p>The property name for the data model, that is being edited.</p>\n"},"_refInModelProp":{"!doc":"<p>The property name for the data model, used as the input.\nUsed when this controller needs data model (as input) that is different from the data model this controller provides.</p>\n"},"_refModelProp":{"!doc":"<p>The property name for the data model.</p>\n"},"_refOriginalModelProp":{"!doc":"<p>The property name for the data model, that serves as the original data.</p>\n"},"_refSourceModelProp":{"!doc":"<p>The property name for the data model, that serves as the data source.</p>\n"},"_removals":{"!doc":"<p>The list of removed elements.</p>\n"},"_resultsWatchHandle":{"!doc":"<p>The watch handle for model array elements.</p>\n"},"_tableModelWatchHandle":{"!doc":"<p>The watch handle of model.</p>\n"},"cursor":{"!doc":"<p>The selected element in the model array.</p>\n"},"cursorId":{"!doc":"<p>The ID of the selected element in the model array.</p>\n"},"cursorIndex":{"!doc":"<p>The index of the selected element in the model.</p>\n"},"data-mvc-bindings":{},"dataBindAttr":{"!doc":"<p>The attribute name for data binding.</p>\n"},"getPlainValueOptions":{"!doc":"<p>The options to get plain value from stateful object.</p>\n"},"getStatefulOptions":{"!doc":"<p>The options to get stateful object from plain value.</p>\n"},"holdModelUntilCommit":{"!doc":"<p>True not to send the change in model back to sourceModel until commit() is called.</p>\n"},"idProperty":{"!doc":"<p>The property name in element in the model array, that works as its identifier.</p>\n"},"model":{"!doc":"<p>The data model.</p>\n"},"originalModel":{"!doc":"<p>The data model, that serves as the original data.</p>\n"},"ownProps":{"!doc":"<p>List of property names owned by this controller, instead of the data model.</p>\n"},"sourceModel":{},"store":{"!doc":"<p>The Dojo Object Store in use.</p>\n"},"addStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Creates an object, throws an error if the object already exists.</p>\n"},"cloneModel":{"!type":"fn(value: ?) -> ?","!doc":"<p>Create a clone object of the data source.\nChild classes of this controller can override it to achieve its specific needs.</p>\n"},"commit":{"!type":"fn()","!doc":"<p>Send the change back to the data source.</p>\n"},"commitCurrent":{"!type":"fn()","!doc":"<p>Send the change back to the data source for the current index.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Clean up model watch handle as this object is destroyed.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>If getter function is there, use it. Otherwise, get the data from data model of this object.</p>\n"},"getStore":{"!type":"fn(id: ?, options: ?) -> ?","!doc":"<p>Retrieves an object by its identity.</p>\n"},"hasControllerProperty":{"!type":"fn(name: ?) -> bool","!doc":"<p>Returns true if this controller itself owns the given property.</p>\n"},"postscript":{"!type":"fn(params?: ?, srcNodeRef?: ?)","!doc":"<p>Sets _relTargetProp so that the property specified by _refModelProp is used for relative data binding.</p>\n"},"putStore":{"!type":"fn(object: ?, options?: ?) -> number","!doc":"<p>Stores an object.</p>\n"},"queryStore":{"!type":"fn(query: ?, options?: ?) -> +dojo","!doc":"<p>Queries the store for objects.</p>\n"},"removeStore":{"!type":"fn(id: ?, options: ?) -> bool","!doc":"<p>Deletes an object by its identity</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Change the model back to its original state.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>If the value given is dojox/mvc/at handle, use it for data binding.\nOtherwise, if setter function is there, use it.\nOtherwise, set the value to the data model or to this object.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Starts up data binding as this object starts up.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> ?","!doc":"<p>Watch a property in the data model or in this object.</p>\n"}}},"dojox/mvc/_base.Element":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget implicitly created by dojox/mvc/parserExtension.\nMaps &quot;value&quot; attribute to form element value, innerText/innerHTML to element&#39;s innerText/innerHTML, and other attributes to DOM attributes.\nAlso, for form element, updates value (or checked for check box) as user edits.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setInnerHTMLAttr":{},"_setInnerTextAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Set onchange event handler for form elements.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_base._Container":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_containedWidgets":{"!doc":"<p>The array of contained widgets at any given point in time within this container.</p>\n"},"_parser":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.index}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inlineTemplateString":{"!doc":"<p>Same as templateString. Used when this widget is mixed with a regular templated widget.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside the container.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or content for this container. It is usually obtained from the\nbody of the container and may be modified or repeated over a collection/array.\nIn this simple implementation, attach points, attach events and WAI\nattributes are not supported in the template.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_base.Group":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A simple model-bound container widget with single-node binding to a data model.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"target":{"!doc":"<p>The data model used for relative data binding.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_base.Generate":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/_Container.prototype","!doc":"<p>A container that generates a view based on the data model its bound to.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_containedWidgets":{"!doc":"<p>The array of contained widgets at any given point in time within this container.</p>\n"},"_counter":{"!doc":"<p>A count maintained internally to always generate predictable widget\nIDs in the view generated by this container.</p>\n"},"_defaultClassMapping":{},"_defaultIdNameMapping":{},"_defaultWidgetMapping":{},"_parser":{},"_relTargetProp":{"!doc":"<p>The name of the property that is used by child widgets for relative data binding.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"children":{"!doc":"<p>The array of data model that is used to render child nodes.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.index}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"inlineTemplateString":{"!doc":"<p>Same as templateString. Used when this widget is mixed with a regular templated widget.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside the container.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or content for this container. It is usually obtained from the\nbody of the container and may be modified or repeated over a collection/array.\nIn this simple implementation, attach points, attach events and WAI\nattributes are not supported in the template.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_base.Output":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A simple widget that displays templated output, parts of which may\nbe data-bound.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.value}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or data-bound output content.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postscript":{"!type":"fn(params: ?, srcNodeRef: ?)","!doc":"<p>Override and save template from body.</p>\n"},"set":{"!type":"fn(name: ?, value: ?)","!doc":"<p>Override and refresh output on value change.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_base.Repeat":{"!type":"fn()","prototype":{"!proto":"dojox/mvc/_Container.prototype","!doc":"<p>A model-bound container which binds to a collection within a data model\nand produces a repeating user-interface from a template for each\niteration within the collection.\n</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_containedWidgets":{"!doc":"<p>The array of contained widgets at any given point in time within this container.</p>\n"},"_parser":{},"_relTargetProp":{"!doc":"<p>The name of the property that is used by child widgets for relative data binding.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"children":{"!doc":"<p>The array of data model that is used to render child nodes.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"exprchar":{"!doc":"<p>Character to use for a substitution expression, for a substitution string like ${this.index}\nIf this class is declared in a template HTML and exprchar is used in in-line template of this class, something other than <code>$</code> should be specified to avoid conflict with exprchar of outer-template.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"index":{"!doc":"<p>An index used to track the current iteration when the repeating UI is\nproduced. This may be used to parameterize the content in the repeat\ntemplate for the current iteration.\n\n</p>\n<p>For example, consider a collection of search or query results where\neach item contains a &quot;Name&quot; property used to prime the &quot;Results&quot; data\nmodel. Then, the following CRUD-style UI displays all the names in\nthe search results in text boxes where they may be updated or such.\n\n\n</p>\n<pre><code>&lt;div dojoType=&quot;dojox/mvc/Repeat&quot; ref=&quot;Results&quot;&gt;\n    &lt;div class=&quot;row&quot; dojoType=&quot;dojox/mvc/Group&quot; ref=&quot;${this.index}&quot;&gt;\n        &lt;label for=&quot;nameInput${this.index}&quot;&gt;Name:&lt;/label&gt;\n        &lt;input dojoType=&quot;dijit/form/TextBox&quot; id=&quot;nameInput${this.index}&quot; ref=&quot;&#39;Name&#39;&quot;&gt;&lt;/input&gt;\n    &lt;/div&gt;\n&lt;/div&gt;</code></pre>\n"},"inlineTemplateString":{"!doc":"<p>Same as templateString. Used when this widget is mixed with a regular templated widget.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"removeRepeatNode":{"!doc":"<p>When true the dom node for the Repeat and Groups within the Repeat\nwill be removed, their children will be placed into the parent node \nof the Repeat node.  This should be set to true when working with \na Repeat inside of a dojox.mobile list.     </p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{"!doc":"<p>Flag to parser to not try and parse widgets declared inside the container.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template or content for this container. It is usually obtained from the\nbody of the container and may be modified or repeated over a collection/array.\nIn this simple implementation, attach points, attach events and WAI\nattributes are not supported in the template.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useParent":{"!doc":"<p>id of the DOM node to use as the parent for the repeating items, similar to useParentId processed a little differently </p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postscript":{"!type":"fn(params: ?, srcNodeRef: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_base.StatefulModel":{"!type":"fn()","prototype":{"!proto":"dojo/Stateful.prototype","!doc":"<p>Deprecated.  Use dojox/mvc/getStateful, dojox/mvc/getPlainValue, dojox/mvc/StatefulArray or one of the dojox/mvc/*RefControllers instead.\nThe first-class native JavaScript data model based on dojo/Stateful\nthat wraps any data structure(s) that may be relevant for a view,\na view portion, a dijit or any custom view layer component.\n</p>\n","_attrPairNames":{"!doc":"<p>Used across all instances a hash to cache attribute names and their getter \nand setter names.</p>\n"},"data":{"!doc":"<p>The plain JavaScript object / data structure used to initialize\nthis model. At any point in time, it holds the lasted saved model\nstate.\nEither data or store property must be provided.</p>\n"},"store":{"!doc":"<p>The data store from where to retrieve initial data for this model.\nAn optional query may also be provided along with this store.\nEither data or store property must be provided.</p>\n"},"valid":{"!doc":"<p>Whether this model deems the associated data to be valid.</p>\n"},"value":{"!doc":"<p>The associated value (if this is a leaf node). The value of\nintermediate nodes in the model is not defined.</p>\n"},"add":{"!type":"fn(name: ?, stateful: ?)","!doc":"<p>Adds a dojo/Stateful tree represented by the given\ndojox/mvc/StatefulModel at the given property name.</p>\n"},"commit":{"!type":"fn(store: ?)","!doc":"<p>Commits this data model:\n\n</p>\n<ul>\n<li>Saves the current state such that a subsequent reset will not\nundo any prior changes.</li>\n<li>Persists client-side changes to the data store, if a store\nhas been supplied as a parameter or at instantiation.</li>\n</ul>\n"},"get":{"!type":"fn(name: ?) -> +any","!doc":"<p>Get a property on a Stateful instance.</p>\n"},"postscript":{"!type":"fn(params?: ?)"},"remove":{"!type":"fn(name: ?)","!doc":"<p>Removes the dojo/Stateful tree at the given property name.</p>\n"},"reset":{"!type":"fn()","!doc":"<p>Resets this data model values to its original state.\nStructural changes to the data model (such as adds or removes)\nare not restored.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> +any","!doc":"<p>Set a property on a Stateful instance</p>\n"},"splice":{"!type":"fn(idx: ?, n: ?) -> +dojox","!doc":"<p>Removes and then adds some elements to this array.\nUpdates the removed/added elements, as well as the length, as stateful.</p>\n"},"toPlainObject":{"!type":"fn() -> +any","!doc":"<p>Produces a plain JavaScript object representation of the data\ncurrently within this data model.</p>\n"},"toString":{"!type":"fn() -> +any","!doc":"<p>Returns the string representation of the data currently within this data model.</p>\n"},"valueOf":{"!type":"fn() -> +any","!doc":"<p>Returns the value representation of the data currently within this data model.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}},"getPlainValueOptions":{"!doc":"<p>An object that defines how plain value should be created from model object.</p>\n"},"getStatefulOptions":{"!doc":"<p>An object that defines how model object should be created from plain object hierarchy.</p>\n"}},"dojox/mvc/_base.StatefulSeries":{"!type":"fn()","prototype":{"!doc":"<p>Chart data plugin (&quot;series&quot;) that watches for properties specified in dojox/mvc/at handles in the given data.\nAt initialization, and when the properties are updated, creates the data from data given and updates the chart.</p>\n","constructor":{"!type":"fn(items: ?)"},"destroy":{"!type":"fn()"},"setSeriesObject":{"!type":"fn(series: ?)"}}},"dojox/mvc/_base.Templated":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A templated widget, mostly the same as dijit/_Templated, but without deprecated features in it.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"bindings":{"!doc":"<p>The data binding declaration (or simple parameters) for child widgets.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>A string that represents the widget template.\nUse in conjunction with dojo.cache() to load from a file.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_base.WidgetList":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A widget that creates child widgets repeatedly based on the children attribute (the repeated data) and childType/childMixins/childParams attributes (determines how to create each child widget).</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_relTargetProp":{"!doc":"<p>The name of the property that is used by child widgets for relative data binding.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"childBindings":{"!doc":"<p>Data bindings for child widget.</p>\n"},"childClz":{"!doc":"<p>The class of the child widget. Takes precedence over childType/childMixins.</p>\n"},"childMixins":{"!doc":"<p>The list of module IDs (separated by comma), or a functinon that takes child data as the argument and returns it, of the classes that will be mixed into child widget. childClz takes precedence over childType/this.\nCan be specified via data-mvc-child-mixins attribute of widget declaration.</p>\n"},"childParams":{"!doc":"<p>The mixin properties for child widget.\nCan be specified via data-mvc-child-props attribute of widget declaration.\n&quot;this&quot; in data-mvc-child-props will have the following properties:\n\n</p>\n<ul>\n<li>parent - This widget&#39;s instance.</li>\n<li>target - The data item in children.</li>\n</ul>\n"},"children":{"!doc":"<p>The array of data model that is used to render child nodes.</p>\n"},"childType":{"!doc":"<p>The module ID of child widget, or a function that takes child data as the argument and returns the module ID of child widget. childClz takes precedence over this/childMixins.\nCan be specified via data-mvc-child-type attribute of widget declaration.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"data-mvc-child-bindings":{},"data-mvc-child-mixins":{},"data-mvc-child-props":{},"data-mvc-child-type":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"partialRebuild":{"!doc":"<p>If true, only rebuild repeat items for changed elements. Otherwise, rebuild everything if there is a change in children.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templateString":{"!doc":"<p>The template string for each child items. templateString in child widgets take precedence over this.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/mvc/_base._DataBindingMixin":{"!type":"fn()","prototype":{"!doc":"<p>Deprecated.  Use dojox/mvc/at for data binding.\nProvides the ability for dijits or custom view components to become\ndata binding aware.\n</p>\n","binding":{"!doc":"<p>The read only value of the resolved data binding for this widget.\nThis may be a result of resolving various relative refs along\nthe parent axis.</p>\n"},"ref":{"!doc":"<p>The value of the data binding expression passed declaratively by\nthe developer. This usually references a location within an\nexisting datamodel and may be a relative reference based on the\nparent / container data binding (dot-separated string).</p>\n"},"isValid":{"!type":"fn() -> +any","!doc":"<p>Returns the validity of the data binding.</p>\n"}}},"dojox/mvc/_base._InlineTemplateMixin":{"!type":"fn()","prototype":{"!doc":"<p>A mixin for template widget, which will look for <code>&lt;script type=&quot;dojox/mvc/InlineTemplate&quot;&gt;</code>\nand treat the HTML in there as the template string.</p>\n","buildRendering":{"!type":"fn()"}}},"dojox/mvc/_patches":{},"dojox/mvc/equals":{"!doc":"<p>Compares two dojo/Stateful objects, by diving into the leaves.</p>\n","prototype":{"constructor":{"!type":"fn(dst: ?, src: ?, options: ?) -> bool","!doc":"<p>Compares two dojo/Stateful objects, by diving into the leaves.</p>\n"}},"equalsArray":{"!type":"fn(dst: ?, src: ?) -> bool","!doc":"<p>Returns if the given two stateful arrays are equal.</p>\n"},"equalsDate":{"!type":"fn(dst: ?, src: ?) -> bool"},"equalsObject":{"!type":"fn(dst: ?, src: ?) -> bool","!doc":"<p>Returns if the given two stateful objects are equal.</p>\n"},"equalsValue":{"!type":"fn(dst: ?, src: ?) -> bool","!doc":"<p>Returns if the given two values are equal.</p>\n"},"getType":{"!type":"fn(v: ?) -> string","!doc":"<p>Returns the type of the given value.</p>\n"}},"dojox/mvc/parserExtension":{},"dojox/mvc":{"!doc":"<p>Deprecated.  Should require dojox/mvc modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/rails":{"live":{"!type":"fn(selector: ?, evtName: ?, fn: ?)"}},"dojox/robot/recorder":{},"dojox/rpc/JsonRPC":{},"dojox/rpc/OfflineRest":{"!doc":"<p>Makes the REST service be able to store changes in local\nstorage so it can be used offline automatically.</p>\n","stores":{},"addStore":{"!type":"fn(store: ?, baseQuery?: ?)","!doc":"<p>Adds a store to the monitored store for local storage</p>\n"},"downloadChanges":{"!type":"fn()"},"sendChanges":{"!type":"fn()"},"sync":{"!type":"fn()"},"turnOffAutoSync":{"!type":"fn()"}},"dojox/storage":{"!doc":"<p>Deprecated.  Should require dojox/storage modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/storage/_common":{},"dojox/storage/Provider":{"!type":"fn()","prototype":{"!doc":"<p>A singleton for working with dojox.storage.</p>\n","DEFAULT_NAMESPACE":{"!doc":"<p>The namespace for all storage operations. This is useful if several\napplications want access to the storage system from the same domain but\nwant different storage silos.</p>\n"},"FAILED":{"!doc":"<p>Flag that indicates a put() call to\na storage provider failed.</p>\n"},"onHideSettingsUI":{"!doc":"<p>If a function is assigned to this property, then when the settings\nprovider&#39;s UI is closed this function is called. Useful, for example,\nif the user has just cleared out all storage for this provider using\nthe settings UI, and you want to update your UI.</p>\n"},"PENDING":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider is pending user approval.</p>\n"},"SIZE_NO_LIMIT":{"!doc":"<p>Returned by getMaximumSize() if this storage provider has no theoretical\nlimit on the amount of data it can store.</p>\n"},"SIZE_NOT_AVAILABLE":{"!doc":"<p>Returned by getMaximumSize() if this storage provider can not determine\nthe maximum amount of data it can support.</p>\n"},"SUCCESS":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider was successful.</p>\n"},"clear":{"!type":"fn(namespace?: ?)","!doc":"<p>Completely clears this storage system of all of it&#39;s values and\nkeys. If &#39;namespace&#39; is provided just clears the keys in that\nnamespace.</p>\n"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(key: ?, namespace?: ?)"},"getKeys":{"!type":"fn(namespace?: ?)"},"getMaximumSize":{"!type":"fn()"},"getMultiple":{"!type":"fn(keys: ?, namespace?: ?) -> []"},"getNamespaces":{"!type":"fn()"},"getResourceList":{"!type":"fn() -> []"},"hasKey":{"!type":"fn(key: ?, namespace?: ?) -> bool","!doc":"<p>Determines whether the storage has the given key.</p>\n"},"hasSettingsUI":{"!type":"fn() -> bool"},"hideSettingsUI":{"!type":"fn()","!doc":"<p>If this provider has a settings UI, hides it.</p>\n"},"initialize":{"!type":"fn()","!doc":"<p>Allows this storage provider to initialize itself. This is\ncalled after the page has finished loading, so you can not do\ndocument.writes(). Storage Provider subclasses should initialize\nthemselves inside of here rather than in their function\nconstructor.</p>\n"},"isAvailable":{"!type":"fn()"},"isPermanent":{"!type":"fn()"},"isValidKey":{"!type":"fn(keyName: ?) -> bool"},"isValidKeyArray":{"!type":"fn(keys: ?) -> bool"},"put":{"!type":"fn(key: ?, value: ?, resultsHandler: ?, namespace?: ?)","!doc":"<p>Puts a key and value into this storage system.</p>\n"},"putMultiple":{"!type":"fn(keys: ?, values: ?, resultsHandler: ?, namespace?: ?)","!doc":"<p>Puts multiple keys and values into this storage system.</p>\n"},"remove":{"!type":"fn(key: ?, namespace?: ?)","!doc":"<p>Removes the given key from this storage system.</p>\n"},"removeMultiple":{"!type":"fn(keys: ?, namespace?: ?)","!doc":"<p>Removes the given keys from this storage system.</p>\n"},"showSettingsUI":{"!type":"fn()","!doc":"<p>If this provider has a settings UI, determined\nby calling hasSettingsUI(), it is shown.</p>\n"}}},"dojox/storage/manager":{"!doc":"<p>A singleton class in charge of the dojox.storage system</p>\n","prototype":{"constructor":{"!type":"fn()","!doc":"<p>A singleton class in charge of the dojox.storage system</p>\n"}}},"dojox/storage/LocalStorageProvider":{"!type":"fn()","prototype":{"!proto":"dojox/storage/Provider.prototype","DEFAULT_NAMESPACE":{"!doc":"<p>The namespace for all storage operations. This is useful if several\napplications want access to the storage system from the same domain but\nwant different storage silos.</p>\n"},"FAILED":{"!doc":"<p>Flag that indicates a put() call to\na storage provider failed.</p>\n"},"onHideSettingsUI":{"!doc":"<p>If a function is assigned to this property, then when the settings\nprovider&#39;s UI is closed this function is called. Useful, for example,\nif the user has just cleared out all storage for this provider using\nthe settings UI, and you want to update your UI.</p>\n"},"PENDING":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider is pending user approval.</p>\n"},"SIZE_NO_LIMIT":{"!doc":"<p>Returned by getMaximumSize() if this storage provider has no theoretical\nlimit on the amount of data it can store.</p>\n"},"SIZE_NOT_AVAILABLE":{"!doc":"<p>Returned by getMaximumSize() if this storage provider can not determine\nthe maximum amount of data it can support.</p>\n"},"store":{},"SUCCESS":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider was successful.</p>\n"},"clear":{"!type":"fn(namespace?: ?)"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(key: ?, namespace?: ?) -> ?"},"getFullKey":{"!type":"fn(key: ?, namespace: ?) -> string"},"getKeys":{"!type":"fn(namespace?: ?) -> []"},"getMaximumSize":{"!type":"fn() -> ?"},"getMultiple":{"!type":"fn(keys: ?, namespace?: ?) -> []"},"getNamespaces":{"!type":"fn() -> []"},"getResourceList":{"!type":"fn() -> []"},"hasKey":{"!type":"fn(key: ?, namespace?: ?) -> bool","!doc":"<p>Determines whether the storage has the given key.</p>\n"},"hasSettingsUI":{"!type":"fn() -> bool"},"hideSettingsUI":{"!type":"fn()","!doc":"<p>If this provider has a settings UI, hides it.</p>\n"},"initialize":{"!type":"fn()"},"isAvailable":{"!type":"fn() -> bool"},"isPermanent":{"!type":"fn() -> bool"},"isValidKey":{"!type":"fn(keyName: ?) -> bool"},"isValidKeyArray":{"!type":"fn(keys: ?) -> bool"},"isValidNamespace":{"!type":"fn(keyName: ?) -> bool"},"put":{"!type":"fn(key: ?, value: ?, resultsHandler: ?, namespace?: ?)"},"putMultiple":{"!type":"fn(keys: ?, values: ?, resultsHandler: ?, namespace?: ?)","!doc":"<p>Puts multiple keys and values into this storage system.</p>\n"},"remove":{"!type":"fn(key: ?, namespace?: ?)"},"removeMultiple":{"!type":"fn(keys: ?, namespace?: ?)","!doc":"<p>Removes the given keys from this storage system.</p>\n"},"showSettingsUI":{"!type":"fn()","!doc":"<p>If this provider has a settings UI, determined\nby calling hasSettingsUI(), it is shown.</p>\n"}}},"dojox/storage/GearsStorageProvider":{"!type":"fn()","prototype":{"!proto":"dojox/storage/Provider.prototype","!doc":"<p>Storage provider that uses the features of Google Gears\nto store data (it is saved into the local SQL database\nprovided by Gears, using dojox.sql)</p>\n","_available":{},"_storageReady":{},"DEFAULT_NAMESPACE":{"!doc":"<p>The namespace for all storage operations. This is useful if several\napplications want access to the storage system from the same domain but\nwant different storage silos.</p>\n"},"FAILED":{"!doc":"<p>Flag that indicates a put() call to\na storage provider failed.</p>\n"},"initialized":{},"onHideSettingsUI":{"!doc":"<p>If a function is assigned to this property, then when the settings\nprovider&#39;s UI is closed this function is called. Useful, for example,\nif the user has just cleared out all storage for this provider using\nthe settings UI, and you want to update your UI.</p>\n"},"PENDING":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider is pending user approval.</p>\n"},"SIZE_NO_LIMIT":{"!doc":"<p>Returned by getMaximumSize() if this storage provider has no theoretical\nlimit on the amount of data it can store.</p>\n"},"SIZE_NOT_AVAILABLE":{"!doc":"<p>Returned by getMaximumSize() if this storage provider can not determine\nthe maximum amount of data it can support.</p>\n"},"SUCCESS":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider was successful.</p>\n"},"TABLE_NAME":{},"clear":{"!type":"fn(namespace: ?)"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(key: ?, namespace: ?) -> +null"},"getKeys":{"!type":"fn(namespace: ?) -> []"},"getMaximumSize":{"!type":"fn() -> ?"},"getMultiple":{"!type":"fn(keys: ?, namespace: ?) -> []"},"getNamespaces":{"!type":"fn() -> []"},"getResourceList":{"!type":"fn() -> []"},"hasKey":{"!type":"fn(key: ?, namespace?: ?) -> bool","!doc":"<p>Determines whether the storage has the given key.</p>\n"},"hasSettingsUI":{"!type":"fn() -> bool"},"hideSettingsUI":{"!type":"fn()"},"initialize":{"!type":"fn()"},"isAvailable":{"!type":"fn() -> ?"},"isPermanent":{"!type":"fn() -> bool"},"isValidKey":{"!type":"fn(keyName: ?) -> bool"},"isValidKeyArray":{"!type":"fn(keys: ?) -> bool"},"put":{"!type":"fn(key: ?, value: ?, resultsHandler: ?, namespace: ?)"},"putMultiple":{"!type":"fn(keys: ?, values: ?, resultsHandler: ?, namespace: ?)"},"remove":{"!type":"fn(key: ?, namespace: ?)"},"removeMultiple":{"!type":"fn(keys: ?, namespace: ?)"},"showSettingsUI":{"!type":"fn()"}}},"dojox/sql":{"!doc":"<p>Deprecated.  Should require dojox/sql modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/sql/_base":{},"dojox/sql/_crypto":{},"dojox/storage/WhatWGStorageProvider":{"!type":"fn()","prototype":{"!proto":"dojox/storage/Provider.prototype","!doc":"<p>Storage provider that uses WHAT Working Group features in Firefox 2\nto achieve permanent storage.</p>\n","_allNamespaces":{},"_available":{},"_domain":{},"_statusHandler":{},"_storageEventListener":{},"DEFAULT_NAMESPACE":{"!doc":"<p>The namespace for all storage operations. This is useful if several\napplications want access to the storage system from the same domain but\nwant different storage silos.</p>\n"},"FAILED":{"!doc":"<p>Flag that indicates a put() call to\na storage provider failed.</p>\n"},"initialized":{},"onHideSettingsUI":{"!doc":"<p>If a function is assigned to this property, then when the settings\nprovider&#39;s UI is closed this function is called. Useful, for example,\nif the user has just cleared out all storage for this provider using\nthe settings UI, and you want to update your UI.</p>\n"},"PENDING":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider is pending user approval.</p>\n"},"SIZE_NO_LIMIT":{"!doc":"<p>Returned by getMaximumSize() if this storage provider has no theoretical\nlimit on the amount of data it can store.</p>\n"},"SIZE_NOT_AVAILABLE":{"!doc":"<p>Returned by getMaximumSize() if this storage provider can not determine\nthe maximum amount of data it can support.</p>\n"},"SUCCESS":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider was successful.</p>\n"},"clear":{"!type":"fn(namespace: ?)"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(key: ?, namespace: ?) -> +null"},"getFullKey":{"!type":"fn(key: ?, namespace: ?) -> ?"},"getKeys":{"!type":"fn(namespace: ?) -> []"},"getMaximumSize":{"!type":"fn() -> ?"},"getMultiple":{"!type":"fn(keys: ?, namespace?: ?) -> []"},"getNamespaces":{"!type":"fn() -> []"},"getResourceList":{"!type":"fn() -> []"},"hasKey":{"!type":"fn(key: ?, namespace?: ?) -> bool","!doc":"<p>Determines whether the storage has the given key.</p>\n"},"hasSettingsUI":{"!type":"fn() -> bool"},"hideSettingsUI":{"!type":"fn()"},"initialize":{"!type":"fn()"},"isAvailable":{"!type":"fn() -> ?"},"isPermanent":{"!type":"fn() -> bool"},"isValidKey":{"!type":"fn(keyName: ?) -> bool"},"isValidKeyArray":{"!type":"fn(keys: ?) -> bool"},"put":{"!type":"fn(key: ?, value: ?, resultsHandler: ?, namespace: ?)"},"putMultiple":{"!type":"fn(keys: ?, values: ?, resultsHandler: ?, namespace?: ?)","!doc":"<p>Puts multiple keys and values into this storage system.</p>\n"},"remove":{"!type":"fn(key: ?, namespace: ?)"},"removeMultiple":{"!type":"fn(keys: ?, namespace?: ?)","!doc":"<p>Removes the given keys from this storage system.</p>\n"},"showSettingsUI":{"!type":"fn()"}}},"dojox/storage/FlashStorageProvider":{"!type":"fn()","prototype":{"!proto":"dojox/storage/Provider.prototype","!doc":"<p>Storage provider that uses features in Flash to achieve permanent\nstorage</p>\n","_available":{},"_flashReady":{},"_pageReady":{},"_statusHandler":{},"DEFAULT_NAMESPACE":{"!doc":"<p>The namespace for all storage operations. This is useful if several\napplications want access to the storage system from the same domain but\nwant different storage silos.</p>\n"},"FAILED":{"!doc":"<p>Flag that indicates a put() call to\na storage provider failed.</p>\n"},"initialized":{},"onHideSettingsUI":{"!doc":"<p>If a function is assigned to this property, then when the settings\nprovider&#39;s UI is closed this function is called. Useful, for example,\nif the user has just cleared out all storage for this provider using\nthe settings UI, and you want to update your UI.</p>\n"},"PENDING":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider is pending user approval.</p>\n"},"SIZE_NO_LIMIT":{"!doc":"<p>Returned by getMaximumSize() if this storage provider has no theoretical\nlimit on the amount of data it can store.</p>\n"},"SIZE_NOT_AVAILABLE":{"!doc":"<p>Returned by getMaximumSize() if this storage provider can not determine\nthe maximum amount of data it can support.</p>\n"},"SUCCESS":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider was successful.</p>\n"},"clear":{"!type":"fn(namespace: ?)"},"constructor":{"!type":"fn()"},"flush":{"!type":"fn(namespace: ?)"},"get":{"!type":"fn(key: ?, namespace: ?) -> +null"},"getFlushDelay":{"!type":"fn() -> ?"},"getKeys":{"!type":"fn(namespace: ?) -> ?"},"getMaximumSize":{"!type":"fn() -> ?"},"getMultiple":{"!type":"fn(keys: ?, namespace?: ?) -> ?"},"getNamespaces":{"!type":"fn() -> ?"},"getResourceList":{"!type":"fn() -> []"},"hasKey":{"!type":"fn(key: ?, namespace?: ?) -> bool","!doc":"<p>Determines whether the storage has the given key.</p>\n"},"hasSettingsUI":{"!type":"fn() -> bool"},"hideSettingsUI":{"!type":"fn()"},"initialize":{"!type":"fn()"},"isAvailable":{"!type":"fn() -> bool"},"isPermanent":{"!type":"fn() -> bool"},"isValidKey":{"!type":"fn(keyName: ?) -> bool"},"isValidKeyArray":{"!type":"fn(keys: ?) -> bool"},"put":{"!type":"fn(key: ?, value: ?, resultsHandler: ?, namespace: ?)"},"putMultiple":{"!type":"fn(keys: ?, values: ?, resultsHandler: ?, namespace: ?)"},"remove":{"!type":"fn(key: ?, namespace: ?)"},"removeMultiple":{"!type":"fn(keys: ?, namespace?: ?)"},"setFlushDelay":{"!type":"fn(newDelay: ?)"},"showSettingsUI":{"!type":"fn()"}}},"dojox/storage/BehaviorStorageProvider":{"!type":"fn()","prototype":{"!proto":"dojox/storage/Provider.prototype","DEFAULT_NAMESPACE":{"!doc":"<p>The namespace for all storage operations. This is useful if several\napplications want access to the storage system from the same domain but\nwant different storage silos.</p>\n"},"FAILED":{"!doc":"<p>Flag that indicates a put() call to\na storage provider failed.</p>\n"},"keys":{},"onHideSettingsUI":{"!doc":"<p>If a function is assigned to this property, then when the settings\nprovider&#39;s UI is closed this function is called. Useful, for example,\nif the user has just cleared out all storage for this provider using\nthe settings UI, and you want to update your UI.</p>\n"},"PENDING":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider is pending user approval.</p>\n"},"SIZE_NO_LIMIT":{"!doc":"<p>Returned by getMaximumSize() if this storage provider has no theoretical\nlimit on the amount of data it can store.</p>\n"},"SIZE_NOT_AVAILABLE":{"!doc":"<p>Returned by getMaximumSize() if this storage provider can not determine\nthe maximum amount of data it can support.</p>\n"},"store":{},"storeName":{},"SUCCESS":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider was successful.</p>\n"},"clear":{"!type":"fn(namespace?: ?)"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(key: ?, namespace?: ?) -> ?"},"getFullKey":{"!type":"fn(key: ?, namespace: ?) -> string"},"getKeys":{"!type":"fn(namespace?: ?) -> []"},"getMaximumSize":{"!type":"fn() -> number"},"getMultiple":{"!type":"fn(keys: ?, namespace?: ?) -> []"},"getNamespaces":{"!type":"fn() -> []"},"getResourceList":{"!type":"fn() -> []"},"hasKey":{"!type":"fn(key: ?, namespace?: ?) -> bool","!doc":"<p>Determines whether the storage has the given key.</p>\n"},"hasSettingsUI":{"!type":"fn() -> bool"},"hideSettingsUI":{"!type":"fn()","!doc":"<p>If this provider has a settings UI, hides it.</p>\n"},"initialize":{"!type":"fn()"},"isAvailable":{"!type":"fn() -> ?"},"isPermanent":{"!type":"fn() -> bool"},"isValidKey":{"!type":"fn(keyName: ?) -> bool"},"isValidKeyArray":{"!type":"fn(keys: ?) -> bool"},"isValidNamespace":{"!type":"fn(keyName: ?) -> bool"},"put":{"!type":"fn(key: ?, value: ?, resultsHandler: ?, namespace?: ?)"},"putMultiple":{"!type":"fn(keys: ?, values: ?, resultsHandler: ?, namespace?: ?)","!doc":"<p>Puts multiple keys and values into this storage system.</p>\n"},"remove":{"!type":"fn(key: ?, namespace?: ?)"},"removeMultiple":{"!type":"fn(keys: ?, namespace?: ?)","!doc":"<p>Removes the given keys from this storage system.</p>\n"},"showSettingsUI":{"!type":"fn()","!doc":"<p>If this provider has a settings UI, determined\nby calling hasSettingsUI(), it is shown.</p>\n"}}},"dojox/storage/CookieStorageProvider":{"!type":"fn()","prototype":{"!proto":"dojox/storage/Provider.prototype","cookieName":{},"DEFAULT_NAMESPACE":{"!doc":"<p>The namespace for all storage operations. This is useful if several\napplications want access to the storage system from the same domain but\nwant different storage silos.</p>\n"},"FAILED":{"!doc":"<p>Flag that indicates a put() call to\na storage provider failed.</p>\n"},"onHideSettingsUI":{"!doc":"<p>If a function is assigned to this property, then when the settings\nprovider&#39;s UI is closed this function is called. Useful, for example,\nif the user has just cleared out all storage for this provider using\nthe settings UI, and you want to update your UI.</p>\n"},"PENDING":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider is pending user approval.</p>\n"},"SIZE_NO_LIMIT":{"!doc":"<p>Returned by getMaximumSize() if this storage provider has no theoretical\nlimit on the amount of data it can store.</p>\n"},"SIZE_NOT_AVAILABLE":{"!doc":"<p>Returned by getMaximumSize() if this storage provider can not determine\nthe maximum amount of data it can support.</p>\n"},"storageLife":{},"store":{},"SUCCESS":{"!doc":"<p>Flag that indicates a put() call to a\nstorage provider was successful.</p>\n"},"clear":{"!type":"fn(namespace?: ?)"},"constructor":{"!type":"fn()"},"get":{"!type":"fn(key: ?, namespace?: ?) -> +null"},"getFullKey":{"!type":"fn(key: ?, namespace: ?) -> string"},"getKeys":{"!type":"fn(namespace?: ?) -> []"},"getMaximumSize":{"!type":"fn() -> number"},"getMultiple":{"!type":"fn(keys: ?, namespace?: ?) -> []"},"getNamespaces":{"!type":"fn() -> []"},"getResourceList":{"!type":"fn() -> []"},"hasKey":{"!type":"fn(key: ?, namespace?: ?) -> bool","!doc":"<p>Determines whether the storage has the given key.</p>\n"},"hasSettingsUI":{"!type":"fn() -> bool"},"hideSettingsUI":{"!type":"fn()","!doc":"<p>If this provider has a settings UI, hides it.</p>\n"},"initialize":{"!type":"fn()"},"isAvailable":{"!type":"fn() -> ?"},"isPermanent":{"!type":"fn() -> bool"},"isValidKey":{"!type":"fn(keyName: ?) -> bool"},"isValidKeyArray":{"!type":"fn(keys: ?) -> bool"},"isValidNamespace":{"!type":"fn(keyName: ?) -> bool"},"put":{"!type":"fn(key: ?, value: ?, resultsHandler: ?, namespace?: ?)"},"putMultiple":{"!type":"fn(keys: ?, values: ?, resultsHandler: ?, namespace?: ?)","!doc":"<p>Puts multiple keys and values into this storage system.</p>\n"},"remove":{"!type":"fn(key: ?, namespace?: ?)"},"removeMultiple":{"!type":"fn(keys: ?, namespace?: ?)","!doc":"<p>Removes the given keys from this storage system.</p>\n"},"showSettingsUI":{"!type":"fn()","!doc":"<p>If this provider has a settings UI, determined\nby calling hasSettingsUI(), it is shown.</p>\n"}}},"dojox/secure/DOM":{"prototype":{"constructor":{"!type":"fn(element: ?)"}}},"dojox/secure/fromJson":{},"dojox/secure/sandbox":{"!doc":"<p>Creates a secure sandbox from which scripts and HTML can be loaded that\nwill only be able to access the provided element and it&#39;s descendants, the\nrest of the DOM and JS environment will not be accessible to the sandboxed\nscripts and HTML.\n</p>\n","prototype":{"constructor":{"!type":"fn(element: ?)","!doc":"<p>Creates a secure sandbox from which scripts and HTML can be loaded that\nwill only be able to access the provided element and it&#39;s descendants, the\nrest of the DOM and JS environment will not be accessible to the sandboxed\nscripts and HTML.\n</p>\n"}}},"dojox/sketch/Anchor":{"count":{},"prototype":{"constructor":{"!type":"fn(an: ?, id: ?, isControl: ?)"}}},"dojox/sketch/Annotation":{"prototype":{"calculate":{},"apply":{"!type":"fn(obj: ?)"},"beginEdit":{"!type":"fn(type: ?)"},"constructor":{"!type":"fn(figure: ?, id: ?)"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn()"},"drawBBox":{"!type":"fn()"},"endEdit":{"!type":"fn()"},"getBBox":{"!type":"fn()"},"getTextBox":{"!type":"fn(zoomfactor: ?)"},"getType":{"!type":"fn()"},"initialize":{"!type":"fn()"},"property":{"!type":"fn(name: ?, value?: ?)"},"readCommonAttrs":{"!type":"fn(obj: ?)"},"serialize":{"!type":"fn()"},"setBinding":{"!type":"fn(pt: ?)"},"setMode":{"!type":"fn(m: ?)"},"type":{"!type":"fn()"},"writeCommonAttrs":{"!type":"fn()"},"zoom":{"!type":"fn(pct: ?)"}},"Modes":{},"register":{"!type":"fn(name: ?, toolclass: ?)"}},"dojox/sketch/Annotation.Modes":{"Edit":{},"View":{}},"dojox/sketch/_Plugin":{"!type":"fn()","prototype":{"!doc":"<p>This represents a &quot;plugin&quot; to the dojox.sketch.Figure, which is basically\na single button on the Toolbar and some associated code</p>\n","button":{},"figure":{},"iconClassPrefix":{},"itemGroup":{},"queryCommand":{},"shape":{},"useDefaultCommand":{},"activate":{"!type":"fn(e?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?"},"buttonClass":{"!type":"fn()"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)"},"constructor":{"!type":"fn(args?: ?)"},"destroy":{"!type":"fn(f: ?)"},"setFigure":{"!type":"fn(figure: ?)"},"setToolbar":{"!type":"fn(toolbar: ?)"}}},"dojox/sketch/DoubleArrowAnnotation":{"anchors":{},"control":{},"end":{},"endArrow":{},"endArrowGroup":{},"endRotation":{},"labelShape":{},"pathShape":{},"start":{},"startArrow":{},"startArrowGroup":{},"startRotation":{},"textAlign":{},"textOffset":{},"textPosition":{},"textYOffset":{},"transform":{},"prototype":{"apply":{"!type":"fn(obj: ?)"},"constructor":{"!type":"fn(figure: ?, id: ?)"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(obj: ?)"},"getBBox":{"!type":"fn()"},"getType":{"!type":"fn()"},"initialize":{"!type":"fn(obj: ?)"},"serialize":{"!type":"fn()"},"type":{"!type":"fn()"},"zoom":{"!type":"fn(pct: ?)"}}},"dojox/sketch/DoubleArrowAnnotation.transform":{"dx":{},"dy":{}},"dojox/sketch/DoubleArrowAnnotation.start":{"x":{},"y":{}},"dojox/sketch/DoubleArrowAnnotation.control":{"x":{},"y":{}},"dojox/sketch/DoubleArrowAnnotation.end":{"x":{},"y":{}},"dojox/sketch/DoubleArrowAnnotation.textPosition":{"x":{},"y":{}},"dojox/sketch/Figure":{"prototype":{"gridSize":{},"add":{"!type":"fn(annotation: ?)"},"constructor":{"!type":"fn(mixin: ?)"},"convert":{"!type":"fn(ann: ?, t: ?)"},"destroy":{"!type":"fn(isLoading: ?)"},"draw":{"!type":"fn()"},"getAnnotator":{"!type":"fn(id: ?)"},"getFit":{"!type":"fn()"},"getValue":{"!type":"fn()"},"initialize":{"!type":"fn(node: ?)"},"initUndoStack":{"!type":"fn()"},"load":{"!type":"fn(obj: ?, n: ?)"},"nextKey":{"!type":"fn()"},"redo":{"!type":"fn()"},"remove":{"!type":"fn(annotation: ?)"},"serialize":{"!type":"fn()"},"setTool":{"!type":"fn(t: ?)"},"setValue":{"!type":"fn(text: ?)"},"undo":{"!type":"fn()"},"unzoom":{"!type":"fn()"},"zoom":{"!type":"fn(pct: ?)"}}},"dojox/sketch/UndoStack":{"!type":"fn()","prototype":{"add":{"!type":"fn(cmd: ?, ann?: ?, before?: ?)"},"apply":{"!type":"fn(state: ?, from: ?, to: ?)"},"constructor":{"!type":"fn(figure: ?)"},"destroy":{"!type":"fn()"},"redo":{"!type":"fn()"},"undo":{"!type":"fn()"}}},"dojox/sketch/LeadAnnotation":{"prototype":{"apply":{"!type":"fn(obj: ?)"},"constructor":{"!type":"fn(figure: ?, id: ?)"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(obj: ?)"},"getBBox":{"!type":"fn()"},"getType":{"!type":"fn()"},"initialize":{"!type":"fn(obj: ?)"},"serialize":{"!type":"fn()"},"type":{"!type":"fn()"}}},"dojox/sketch/PreexistingAnnotation":{"prototype":{"apply":{"!type":"fn(obj: ?)"},"constructor":{"!type":"fn(figure: ?, id: ?)"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(obj: ?)"},"getBBox":{"!type":"fn()"},"getType":{"!type":"fn()"},"initialize":{"!type":"fn(obj: ?)"},"serialize":{"!type":"fn()"},"type":{"!type":"fn()"},"zoom":{"!type":"fn(pct: ?)"}}},"dojox/sketch/SingleArrowAnnotation":{"prototype":{"apply":{"!type":"fn(obj: ?)"},"constructor":{"!type":"fn(figure: ?, id: ?)"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(obj: ?)"},"getBBox":{"!type":"fn()"},"getType":{"!type":"fn()"},"initialize":{"!type":"fn(obj: ?)"},"serialize":{"!type":"fn()"},"type":{"!type":"fn()"},"zoom":{"!type":"fn(pct: ?)"}}},"dojox/sketch/Slider":{"!type":"fn()","prototype":{"!proto":"dojox/sketch/_Plugin.prototype","button":{},"figure":{},"iconClassPrefix":{},"itemGroup":{},"queryCommand":{},"shape":{},"useDefaultCommand":{},"activate":{"!type":"fn(e?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?"},"buttonClass":{"!type":"fn()"},"connect":{"!type":"fn(o: ?, f: ?, tf: ?)"},"constructor":{"!type":"fn(args?: ?)"},"destroy":{"!type":"fn(f: ?)"},"reset":{"!type":"fn()"},"setFigure":{"!type":"fn(figure: ?)"},"setToolbar":{"!type":"fn(t: ?)"}}},"dojox/sketch/Toolbar":{"!type":"fn()","prototype":{"!proto":"dijit/Toolbar.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_keyNavCodes":{"!doc":"<p>Hash mapping key code (arrow keys and home/end key) to functions to handle those keys.\nUsually not used directly, as subclasses can instead override _onLeftArrow() etc.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_searchString":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"figure":{},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"focusedChild":{"!doc":"<p>The currently focused child widget, or null if there isn&#39;t one</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"multiCharSearchDuration":{"!doc":"<p>If multiple characters are typed where each keystroke happens within\nmultiCharSearchDuration of the previous keystroke,\nsearch for nodes matching all the keystrokes.\n\n</p>\n<p>For example, typing &quot;ab&quot; will search for entries starting with\n&quot;ab&quot; unless the delay between &quot;a&quot; and &quot;b&quot; is greater than multiCharSearchDuration.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"plugins":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tab index of the container; same as HTML tabIndex attribute.\nNote then when user tabs into the container, focus is immediately\nmoved to the first item in the container.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)"},"addGroupItem":{"!type":"fn(item: ?, group: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"childSelector":{"!type":"fn(node: ?) -> ?"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectKeyNavHandlers":{"!type":"fn(prevKeyCodes: ?, nextKeyCodes: ?)","!doc":"<p>Deprecated.  You can call this in postCreate() to attach the keyboard handlers to the container,\nbut the preferred method is to override _onLeftArrow() and _onRightArrow(), or\n_onUpArrow() and _onDownArrow(), to call focusPrev() and focusNext().</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Default focus() implementation: focus the first child.</p>\n"},"focusChild":{"!type":"fn(widget: ?, last: ?)","!doc":"<p>Focus specified child widget.</p>\n"},"focusFirstChild":{"!type":"fn()","!doc":"<p>Focus the first focusable child in the container.</p>\n"},"focusLastChild":{"!type":"fn()","!doc":"<p>Focus the last focusable child in the container.</p>\n"},"focusNext":{"!type":"fn()","!doc":"<p>Focus the next widget</p>\n"},"focusPrev":{"!type":"fn()","!doc":"<p>Focus the last focusable node in the previous widget\n(ex: go to the ComboButton icon section rather than button section)</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setFigure":{"!type":"fn(f: ?)"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"startupKeyNavChildren":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/sketch/UnderlineAnnotation":{"prototype":{"apply":{"!type":"fn(obj: ?)"},"constructor":{"!type":"fn(figure: ?, id: ?)"},"destroy":{"!type":"fn()"},"draw":{"!type":"fn(obj: ?)"},"getBBox":{"!type":"fn()"},"getType":{"!type":"fn()"},"initialize":{"!type":"fn(obj: ?)"},"serialize":{"!type":"fn()"},"type":{"!type":"fn()"},"zoom":{"!type":"fn(pct: ?)"}}},"dojox/sketch":{"!doc":"<p>Deprecated.  Should require dojox/sketch modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/socket/Reconnect":{"!doc":"<p>Provides auto-reconnection to a websocket after it has been closed</p>\n","prototype":{"constructor":{"!type":"fn(socket: ?, options: ?) -> +any","!doc":"<p>Provides auto-reconnection to a websocket after it has been closed</p>\n"}}},"dojox/socket":{"!doc":"<p>Provides a simple socket connection using WebSocket, or alternate\ncommunication mechanisms in legacy browsers for comet-style communication. This is based\non the WebSocket API and returns an object that implements the WebSocket interface:\n<a href=\"http://dev.w3.org/html5/websockets/#websocket\">http://dev.w3.org/html5/websockets/#websocket</a></p>\n","prototype":{"constructor":{"!type":"fn(argsOrUrl: ?) -> +any","!doc":"<p>Provides a simple socket connection using WebSocket, or alternate\ncommunication mechanisms in legacy browsers for comet-style communication. This is based\non the WebSocket API and returns an object that implements the WebSocket interface:\n<a href=\"http://dev.w3.org/html5/websockets/#websocket\">http://dev.w3.org/html5/websockets/#websocket</a></p>\n"}},"LongPoll":{"!type":"fn(args: ?) -> +any","!doc":"<p>Provides a simple long-poll based comet-style socket/connection to a server and returns an\nobject implementing the WebSocket interface:\n<a href=\"http://dev.w3.org/html5/websockets/#websocket\">http://dev.w3.org/html5/websockets/#websocket</a></p>\n"},"replace":{"!type":"fn(socket: ?, newSocket: ?, listenForOpen: ?)"},"WebSocket":{"!type":"fn(args: ?, fallback: ?)","!doc":"<p>A wrapper for WebSocket, than handles standard args and relative URLs</p>\n"}},"dojox/string/BidiComplex":{"attachInput":{"!type":"fn(field: ?, pattern: ?)","!doc":"<p>Attach key listeners to the INPUT field to accomodate dynamic complex BiDi expressions</p>\n"},"createDisplayString":{"!type":"fn(str: ?, pattern: ?)","!doc":"<p>Create the display string by adding the Unicode direction Markers</p>\n"},"stripSpecialCharacters":{"!type":"fn(str: ?)","!doc":"<p>removes all Unicode directional markers from the string</p>\n"}},"dojox/testing/DocTest":{"!type":"fn()","prototype":{"!doc":"<p>This class executes doctests.</p>\n","errors":{},"getTests":{"!type":"fn(moduleName: ?) -> ?","!doc":"<p>Extract the tests from the given module or string.</p>\n"},"getTestsFromString":{"!type":"fn(data: ?) -> ?"},"run":{"!type":"fn(moduleName: ?)","!doc":"<p>Run the doctests in the module given.</p>\n"},"runTest":{"!type":"fn(commands: ?, expected: ?) -> ?"}}},"dojox/timing/Sequence":{"!type":"fn()","prototype":{"!doc":"<p>This class provides functionality to really sequentialize\nfunction calls. You need to provide a list of functions and\nsome parameters for each (like: pauseBefore) and they will\nbe run one after another. This can be very useful for slideshows\nor alike things.\n</p>\n","_defsResolved":{"!doc":"<p>The resolved sequence, for easier handling.</p>\n"},"_goOnPause":{"!doc":"<p>The pause to wait before really going on.</p>\n"},"_running":{},"constructor":{"!type":"fn()"},"go":{"!type":"fn(defs: ?, doneFunction?: ?)","!doc":"<p>Run the passed sequence definition</p>\n"},"goOn":{"!type":"fn()","!doc":"<p>This method just provides a hook from the outside, so that\nan interrupted sequence can be continued.</p>\n"},"stop":{"!type":"fn()","!doc":"<p>Stop the currently running sequence.</p>\n"}}},"dojox/timing/Streamer":{"!doc":"<p>Streamer will take an input function that pushes N datapoints into a\nqueue, and will pass the next point in that queue out to an\noutput function at the passed interval; this way you can emulate\na constant buffered stream of data.</p>\n","prototype":{"constructor":{"!type":"fn(input: ?, output: ?, interval: ?, minimum: ?, initialData: ?)","!doc":"<p>Streamer will take an input function that pushes N datapoints into a\nqueue, and will pass the next point in that queue out to an\noutput function at the passed interval; this way you can emulate\na constant buffered stream of data.</p>\n"}}},"dojox/timing/ThreadPool":{"prototype":{"constructor":{"!type":"fn(mxthrs: ?, intvl: ?)"}}},"dojox/timing":{"!doc":"<p>Deprecated.  Should require dojox/timing modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/treemap/DrillDownUp":{"!type":"fn()","prototype":{"!doc":"<p>Specializes TreeMap to support drill down and up operations.</p>\n","drillDown":{"!type":"fn(renderer: ?)","!doc":"<p>Drill up from the given renderer.</p>\n"},"drillUp":{"!type":"fn(renderer: ?)","!doc":"<p>Drill up from the given renderer.</p>\n"},"postCreate":{"!type":"fn()"}}},"dojox/treemap/GroupLabel":{"!type":"fn()","prototype":{"!doc":"<p>Specializes TreeMap to remove leaf labels and display group labels centered on group\ncontent instead of display them in headers.</p>\n","createRenderer":{"!type":"fn(item: ?, level: ?, kind: ?) -> ?"},"styleRenderer":{"!type":"fn(renderer: ?, item: ?, level: ?, kind: ?)"}}},"dojox/treemap/Keyboard":{"!type":"fn()","prototype":{"!proto":"dijit/_FocusMixin.prototype","!doc":"<p>Specializes TreeMap to support keyboard navigation and accessibility.</p>\n","_focusManager":{},"_setTabIndexAttr":{},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"constructor":{"!type":"fn()"},"createRenderer":{"!type":"fn(item: ?, level: ?, kind: ?) -> ?"},"postCreate":{"!type":"fn()"}}},"dojox/treemap/_utils":{"find":{"!type":"fn(array: ?, callback: ?) -> ?"},"group":{"!type":"fn(items: ?, groupingFunctions: ?, measureFunction: ?) -> ?"},"initElements":{"!type":"fn(items: ?, areaFunc: ?) -> ?"},"solve":{"!type":"fn(items: ?, width: ?, height: ?, areaFunc: ?, rtl: ?) -> ?"}},"dojox/treemap/ScaledLabel":{"!type":"fn()","prototype":{"!doc":"<p>Specializes TreeMap to display scaled leaf labels instead of constant size labels.</p>\n","createRenderer":{"!type":"fn(item: ?, level: ?, kind: ?) -> ?"},"styleRenderer":{"!type":"fn(renderer: ?, item: ?, level: ?, kind: ?)"}}},"dojox/treemap/TreeMap":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A treemap widget.</p>\n","_areaChanged":{},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_coloringChanged":{},"_dataChanged":{},"_groupFuncs":{},"_groupingChanged":{},"_rootItemChanged":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"areaAttr":{"!doc":"<p>The attribute of the store item that contains the data used to compute the area of a treemap cell.<br>Default is &quot;&quot;. </p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"colorAttr":{"!doc":"<p>The attribute of the store item that contains the data used to compute the color of a treemap cell.\nDefault is &quot;&quot;. </p>\n"},"colorModel":{"!doc":"<p>The optional color model that converts data to color.<br>Default is null.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"groupAttrs":{"!doc":"<p>An array of data attributes used to group data in the treemap.<br>Default is []. </p>\n"},"groupFuncs":{"!doc":"<p>An array of grouping functions used to group data in the treemap.\nWhen null, groupAttrs is to compute grouping functions.\nDefault is null.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"invalidatingProperties":{},"invalidRendering":{},"itemToRenderer":{"!doc":"<p>The associated array item to renderer list.</p>\n"},"labelAttr":{"!doc":"<p>The attribute of the store item that contains the label of a treemap cell.<br>Default is &quot;label&quot;. </p>\n"},"labelThreshold":{"!doc":"<p>The starting depth level at which the labels are not displayed anymore on cells.<br>If NaN no threshold is applied. The depth is the visual depth of the items on the screen not\nin the data (i.e. after drill down the depth of an item might change).\nDefault is NaN.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"query":{"!doc":"<p>A query that can be passed to when querying the store.</p>\n"},"queryOptions":{"!doc":"<p>Options to be applied when querying the store.</p>\n"},"rootItem":{"!doc":"<p>The root item of the treemap, that is the first visible item.\nIf null the entire treemap hierarchy is shown.<br>Default is null.</p>\n"},"selectedItem":{"!doc":"<p>In single selection mode, the selected item or in multiple selection mode the last selected item.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectedItems":{"!doc":"<p>The list of selected items.\nWarning: Do not use this property directly, make sure to call set() or get() methods.</p>\n"},"selectionMode":{"!doc":"<p>Valid values are:\n\n</p>\n<ol>\n<li>&quot;none&quot;: No selection can be done.</li>\n<li>&quot;single&quot;: Only one item can be selected at a time.</li>\n<li>&quot;multiple&quot;: Several item can be selected using the control key modifier.\nDefault value is &quot;single&quot;.</li>\n</ol>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"store":{"!doc":"<p>The store that contains the items to display.</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"tooltipAttr":{"!doc":"<p>The attribute of the store item that contains the tooltip text of a treemap cell.<br>Default is &quot;&quot;. </p>\n"},"addInvalidatingProperties":{"!type":"fn(properties: ?)","!doc":"<p>Add properties to the watched properties to trigger invalidation. This method must be called in\nthe constructor. It is typically used by subclasses of a _Invalidating class to add more properties\nto watch for.</p>\n"},"areaFunc":{"!type":"fn(item: ?, store: ?) -> number","!doc":"<p>A function that returns the value use to compute the area of cell from a store item.\nDefault implementation is using areaAttr.   </p>\n"},"buildRendering":{"!type":"fn()"},"colorFunc":{"!type":"fn(item: ?, store: ?) -> ?","!doc":"<p>A function that returns from a store item the color value of cell or the value used by the \nColorModel to compute the cell color. If a color must be returned it must be in form accepted by the\ndojo/_base/Color constructor. If a value must be returned it must be a Number.\nDefault implementation is using colorAttr.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"createRenderer":{"!type":"fn(item: ?, level: ?, kind: ?) -> +Node","!doc":"<p>Creates an item renderer of the specified kind. This is called only when the treemap\nis created. Default implementation always create div nodes. It also sets overflow\nto hidden and position to absolute on non-header renderers.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"dispatchChange":{"!type":"fn(oldSelectedItem: ?, newSelectedItem: ?, renderer: ?, triggerEvent: ?)","!doc":"<p>Dispatch a selection change event.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getColorForItem":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the color for a given item. This either use the colorModel if not null\nor just the result of the colorFunc.</p>\n"},"getIdentity":{"!type":"fn(item: ?) -> ?"},"getLabelForItem":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns the label for a given item.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"invalidateRendering":{"!type":"fn()","!doc":"<p>Invalidating the rendering for the next executation frame.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isItemSelected":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns wether an item is selected or not.</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn()"},"labelFunc":{"!type":"fn(item: ?, store: ?) -> +null","!doc":"<p>A function that returns the label of cell from a store item.<br>Default implementation is using labelAttr.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"refreshRendering":{"!type":"fn()"},"resize":{"!type":"fn(box: ?)"},"selectFromEvent":{"!type":"fn(e: ?, item: ?, renderer: ?, dispatch: ?) -> bool","!doc":"<p>Applies selection triggered by an user interaction</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setItemSelected":{"!type":"fn(item: ?, value: ?)","!doc":"<p>Change the selection state of an item.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"styleRenderer":{"!type":"fn(renderer: ?, item: ?, level: ?, kind: ?)","!doc":"<p>Style the item renderer. This is called each time the treemap is refreshed.\nFor leaf items it colors them with the color computed from the color model. \nFor other items it does nothing.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"tooltipFunc":{"!type":"fn(item: ?, store: ?) -> +null","!doc":"<p>A function that returns the tooltip of cell from a store item.<br>Default implementation is using tooltipAttr.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"updateRenderers":{"!type":"fn(items: ?)","!doc":"<p>Updates the renderer(s) that represent the specified item(s).</p>\n"},"validateRendering":{"!type":"fn()","!doc":"<p>Immediately validate the rendering if it has been invalidated. You generally do not call that method yourself.</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/uuid/Uuid":{"!doc":"<p>This is the constructor for the Uuid class.  The Uuid class offers\nmethods for inspecting existing UUIDs.</p>\n","prototype":{"compare":{"!type":"fn(otherUuid: ?)","!doc":"<p>Compares this UUID to another UUID, and returns 0, 1, or -1.</p>\n"},"constructor":{"!type":"fn(input?: ?)","!doc":"<p>This is the constructor for the Uuid class.  The Uuid class offers\nmethods for inspecting existing UUIDs.</p>\n"},"getNode":{"!type":"fn()","!doc":"<p>If this is a version 1 UUID (a time-based UUID), getNode() returns a\n12-character string with the &quot;node&quot; or &quot;pseudonode&quot; portion of the UUID,\nwhich is the rightmost 12 characters.</p>\n"},"getTimestamp":{"!type":"fn(returnType?: ?) -> +any","!doc":"<p>If this is a version 1 UUID (a time-based UUID), this method returns\nthe timestamp value encoded in the UUID.  The caller can ask for the\ntimestamp to be returned either as a JavaScript Date object or as a\n15-character string of hex digits.</p>\n"},"getVariant":{"!type":"fn()","!doc":"<p>Returns a variant code that indicates what type of UUID this is.\nReturns one of the enumerated dojox.uuid.variant values.</p>\n"},"getVersion":{"!type":"fn()","!doc":"<p>Returns a version number that indicates what type of UUID this is.\nReturns one of the enumerated dojox.uuid.version values.</p>\n"},"isEqual":{"!type":"fn(otherUuid: ?)","!doc":"<p>Returns true if this UUID is equal to the otherUuid, or false otherwise.</p>\n"},"isValid":{"!type":"fn()","!doc":"<p>Returns true if the UUID was initialized with a valid value.</p>\n"},"toString":{"!type":"fn()","!doc":"<p>This method returns a standard 36-character string representing\nthe UUID, such as &quot;3b12f1df-5232-4804-897e-917bf397618a&quot;.</p>\n"}},"compare":{"!type":"fn(uuidOne: ?, uuidTwo: ?)","!doc":"<p>Given two UUIDs to compare, this method returns 0, 1, or -1.</p>\n"},"getGenerator":{"!type":"fn()","!doc":"<p>Returns the default generator.  See setGenerator().</p>\n"},"setGenerator":{"!type":"fn(generator?: ?)","!doc":"<p>Sets the default generator, which will be used by the\n&quot;new dojox.uuid.Uuid()&quot; constructor if no parameters\nare passed in.</p>\n"}},"dojox/uuid/generateTimeBasedUuid":{"!doc":"<p>This function generates time-based UUIDs, meaning &quot;version 1&quot; UUIDs.</p>\n","_generator":{},"prototype":{"constructor":{"!type":"fn(node?: ?)","!doc":"<p>This function generates time-based UUIDs, meaning &quot;version 1&quot; UUIDs.</p>\n"}},"getNode":{"!type":"fn()","!doc":"<p>Returns the &#39;node&#39; value that will be included in generated UUIDs.</p>\n"},"isValidNode":{"!type":"fn(node?: ?)"},"setNode":{"!type":"fn(node?: ?)","!doc":"<p>Sets the &#39;node&#39; value that will be included in generated UUIDs.</p>\n"}},"dojox/uuid":{"!doc":"<p>Deprecated.  Should require dojox/uuid modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/validate/_base":{"_cardInfo":{"!doc":"<p>A dictionary list of credit card abbreviations</p>\n"},"_isInRangeCache":{},"check":{"!type":"fn(form: ?, profile: ?)","!doc":"<p>validates user input of an HTML form based on input profile</p>\n"},"computeCnpjDv":{"!type":"fn(value: ?)","!doc":"<p>Generate the DV code (checksum part) for a Cnpj number</p>\n"},"computeCpfDv":{"!type":"fn(value: ?)","!doc":"<p>Generate the DV code (checksum part) for a CPF number</p>\n"},"evaluateConstraint":{"!type":"fn(profile: ?, constraint: ?, fieldName: ?, elem: ?)","!doc":"<p>Evaluates dojo.validate.check() constraints that are specified as array\narguments</p>\n"},"getEmailAddressList":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Check if value is an email address list. If an empty list\nis returned, the value didn&#39;t pass the test or it was empty.</p>\n"},"isEmailAddress":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid email address</p>\n"},"isEmailAddressList":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid email address list.</p>\n"},"isInRange":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates whether a string denoting a number\nis between a max and min.</p>\n"},"isIpAddress":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates an IP address</p>\n"},"isNumberFormat":{"!type":"fn(value: ?, flags?: ?) -> ?","!doc":"<p>Validates any sort of number based format</p>\n"},"isPhoneNumber":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validates 10 US digit phone number for several common formats</p>\n"},"isSocialSecurityNumber":{"!type":"fn(value: ?)","!doc":"<p>Validates social security number</p>\n"},"isState":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates US state and territory abbreviations.</p>\n"},"isText":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string has non whitespace characters.\nParameters allow you to constrain the length.</p>\n"},"isUrl":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid URL</p>\n"},"isValidCnpj":{"!type":"fn(value: ?)","!doc":"<p>Validates a CNPJ/CGC number</p>\n"},"isValidCpf":{"!type":"fn(value: ?)","!doc":"<p>Validates a CPF number</p>\n"},"isValidCreditCard":{"!type":"fn(value: ?, ccType: ?)","!doc":"<p>Validate a credit card number by type with Luhn checking.</p>\n"},"isValidCreditCardNumber":{"!type":"fn(value: ?, ccType?: ?)","!doc":"<p>Checks if value matches the pattern for that card or any card types if none is specified</p>\n"},"isValidCvv":{"!type":"fn(value: ?, ccType: ?)","!doc":"<p>Validate the security code (CCV) for a passed credit-card type.</p>\n"},"isValidIsbn":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validate ISBN-10 or ISBN-13 based on the length of value</p>\n"},"isValidLuhn":{"!type":"fn(value: ?)","!doc":"<p>Validate a String value against the Luhn algorithm.</p>\n"},"isZipCode":{"!type":"fn(value: ?)","!doc":"<p>Validates U.S. zip-code</p>\n"}},"dojox/validate/_base._isInRangeCache":{},"dojox/validate/_base._cardInfo":{"!doc":"<p>A dictionary list of credit card abbreviations</p>\n","ax":{},"bl":{},"dc":{},"di":{},"ec":{},"er":{},"jcb":{},"mc":{},"vi":{}},"dojox/validate/regexp":{"ca":{},"us":{},"emailAddress":{"!type":"fn(flags?: ?) -> string","!doc":"<p>Builds a regular expression that matches an email address</p>\n"},"emailAddressList":{"!type":"fn(flags?: ?) -> string","!doc":"<p>Builds a regular expression that matches a list of email addresses.</p>\n"},"host":{"!type":"fn(flags?: ?) -> string","!doc":"<p>Builds a RE that matches a host</p>\n"},"ipAddress":{"!type":"fn(flags?: ?) -> string","!doc":"<p>Builds a RE that matches an IP Address</p>\n"},"numberFormat":{"!type":"fn(flags?: ?) -> ?","!doc":"<p>Builds a regular expression to match any sort of number based format</p>\n"},"url":{"!type":"fn(flags?: ?) -> string","!doc":"<p>Builds a regular expression that matches a URL</p>\n"}},"dojox/validate/regexp.ca":{"postalCode":{"!type":"fn() -> string","!doc":"<p>String regular Express to match Canadain Postal Codes</p>\n"},"province":{"!type":"fn() -> string","!doc":"<p>a regular expression to match Canadian Province Abbreviations</p>\n"}},"dojox/validate/regexp.us":{"state":{"!type":"fn(flags?: ?) -> string","!doc":"<p>A regular expression to match US state and territory abbreviations</p>\n"}},"dojox/validate/br":{"_cardInfo":{"!doc":"<p>A dictionary list of credit card abbreviations</p>\n"},"_isInRangeCache":{},"check":{"!type":"fn(form: ?, profile: ?)","!doc":"<p>validates user input of an HTML form based on input profile</p>\n"},"computeCnpjDv":{"!type":"fn(value: ?)","!doc":"<p>Generate the DV code (checksum part) for a Cnpj number</p>\n"},"computeCpfDv":{"!type":"fn(value: ?)","!doc":"<p>Generate the DV code (checksum part) for a CPF number</p>\n"},"evaluateConstraint":{"!type":"fn(profile: ?, constraint: ?, fieldName: ?, elem: ?)","!doc":"<p>Evaluates dojo.validate.check() constraints that are specified as array\narguments</p>\n"},"getEmailAddressList":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Check if value is an email address list. If an empty list\nis returned, the value didn&#39;t pass the test or it was empty.</p>\n"},"isEmailAddress":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid email address</p>\n"},"isEmailAddressList":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid email address list.</p>\n"},"isInRange":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates whether a string denoting a number\nis between a max and min.</p>\n"},"isIpAddress":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates an IP address</p>\n"},"isNumberFormat":{"!type":"fn(value: ?, flags?: ?) -> ?","!doc":"<p>Validates any sort of number based format</p>\n"},"isPhoneNumber":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validates 10 US digit phone number for several common formats</p>\n"},"isSocialSecurityNumber":{"!type":"fn(value: ?)","!doc":"<p>Validates social security number</p>\n"},"isState":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates US state and territory abbreviations.</p>\n"},"isText":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string has non whitespace characters.\nParameters allow you to constrain the length.</p>\n"},"isUrl":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid URL</p>\n"},"isValidCnpj":{"!type":"fn(value: ?)","!doc":"<p>Validates a CNPJ/CGC number</p>\n"},"isValidCpf":{"!type":"fn(value: ?)","!doc":"<p>Validates a CPF number</p>\n"},"isValidCreditCard":{"!type":"fn(value: ?, ccType: ?)","!doc":"<p>Validate a credit card number by type with Luhn checking.</p>\n"},"isValidCreditCardNumber":{"!type":"fn(value: ?, ccType?: ?)","!doc":"<p>Checks if value matches the pattern for that card or any card types if none is specified</p>\n"},"isValidCvv":{"!type":"fn(value: ?, ccType: ?)","!doc":"<p>Validate the security code (CCV) for a passed credit-card type.</p>\n"},"isValidIsbn":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validate ISBN-10 or ISBN-13 based on the length of value</p>\n"},"isValidLuhn":{"!type":"fn(value: ?)","!doc":"<p>Validate a String value against the Luhn algorithm.</p>\n"},"isZipCode":{"!type":"fn(value: ?)","!doc":"<p>Validates U.S. zip-code</p>\n"}},"dojox/validate/br._isInRangeCache":{},"dojox/validate/br._cardInfo":{"!doc":"<p>A dictionary list of credit card abbreviations</p>\n","ax":{},"bl":{},"dc":{},"di":{},"ec":{},"er":{},"jcb":{},"mc":{},"vi":{}},"dojox/validate/ca":{"!doc":"<p>Module which includes Canadian-specific methods for dojox.validate</p>\n","isPhoneNumber":{"!type":"fn(value: ?) -> ?","!doc":"<p>Validates Canadian 10-digit phone number for several common formats</p>\n"},"isPostalCode":{"!type":"fn(value: ?) -> ?","!doc":"<p>Validates Canadian 6 digit postal code</p>\n"},"isProvince":{"!type":"fn(value: ?) -> ?","!doc":"<p>Validates Canadian province abbreviations (2 characters)</p>\n"},"isSocialInsuranceNumber":{"!type":"fn(value: ?) -> ?","!doc":"<p>Validates Canadian 9 digit social insurance number for several\ncommon formats</p>\n"}},"dojox/validate/us":{"_cardInfo":{"!doc":"<p>A dictionary list of credit card abbreviations</p>\n"},"_isInRangeCache":{},"check":{"!type":"fn(form: ?, profile: ?)","!doc":"<p>validates user input of an HTML form based on input profile</p>\n"},"computeCnpjDv":{"!type":"fn(value: ?)","!doc":"<p>Generate the DV code (checksum part) for a Cnpj number</p>\n"},"computeCpfDv":{"!type":"fn(value: ?)","!doc":"<p>Generate the DV code (checksum part) for a CPF number</p>\n"},"evaluateConstraint":{"!type":"fn(profile: ?, constraint: ?, fieldName: ?, elem: ?)","!doc":"<p>Evaluates dojo.validate.check() constraints that are specified as array\narguments</p>\n"},"getEmailAddressList":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Check if value is an email address list. If an empty list\nis returned, the value didn&#39;t pass the test or it was empty.</p>\n"},"isEmailAddress":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid email address</p>\n"},"isEmailAddressList":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid email address list.</p>\n"},"isInRange":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates whether a string denoting a number\nis between a max and min.</p>\n"},"isIpAddress":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates an IP address</p>\n"},"isNumberFormat":{"!type":"fn(value: ?, flags?: ?) -> ?","!doc":"<p>Validates any sort of number based format</p>\n"},"isPhoneNumber":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validates 10 US digit phone number for several common formats</p>\n"},"isSocialSecurityNumber":{"!type":"fn(value: ?)","!doc":"<p>Validates social security number</p>\n"},"isState":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates US state and territory abbreviations.</p>\n"},"isText":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string has non whitespace characters.\nParameters allow you to constrain the length.</p>\n"},"isUrl":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid URL</p>\n"},"isValidCnpj":{"!type":"fn(value: ?)","!doc":"<p>Validates a CNPJ/CGC number</p>\n"},"isValidCpf":{"!type":"fn(value: ?)","!doc":"<p>Validates a CPF number</p>\n"},"isValidCreditCard":{"!type":"fn(value: ?, ccType: ?)","!doc":"<p>Validate a credit card number by type with Luhn checking.</p>\n"},"isValidCreditCardNumber":{"!type":"fn(value: ?, ccType?: ?)","!doc":"<p>Checks if value matches the pattern for that card or any card types if none is specified</p>\n"},"isValidCvv":{"!type":"fn(value: ?, ccType: ?)","!doc":"<p>Validate the security code (CCV) for a passed credit-card type.</p>\n"},"isValidIsbn":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validate ISBN-10 or ISBN-13 based on the length of value</p>\n"},"isValidLuhn":{"!type":"fn(value: ?)","!doc":"<p>Validate a String value against the Luhn algorithm.</p>\n"},"isZipCode":{"!type":"fn(value: ?)","!doc":"<p>Validates U.S. zip-code</p>\n"}},"dojox/validate/us._isInRangeCache":{},"dojox/validate/us._cardInfo":{"!doc":"<p>A dictionary list of credit card abbreviations</p>\n","ax":{},"bl":{},"dc":{},"di":{},"ec":{},"er":{},"jcb":{},"mc":{},"vi":{}},"dojox/validate/check":{"!doc":"<p>validates user input of an HTML form based on input profile</p>\n","prototype":{"constructor":{"!type":"fn(form: ?, profile: ?)","!doc":"<p>validates user input of an HTML form based on input profile</p>\n"}}},"dojox/validate/creditCard":{"!doc":"<p>Module provides validation functions for Credit Cards, using account number\nrules in conjunction with the Luhn algorigthm, with a pluggable card info database.</p>\n"},"dojox/validate/isbn":{"!doc":"<p>Validate ISBN-10 or ISBN-13 based on the length of value</p>\n","prototype":{"constructor":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validate ISBN-10 or ISBN-13 based on the length of value</p>\n"}}},"dojox/validate/web":{"_cardInfo":{"!doc":"<p>A dictionary list of credit card abbreviations</p>\n"},"_isInRangeCache":{},"check":{"!type":"fn(form: ?, profile: ?)","!doc":"<p>validates user input of an HTML form based on input profile</p>\n"},"computeCnpjDv":{"!type":"fn(value: ?)","!doc":"<p>Generate the DV code (checksum part) for a Cnpj number</p>\n"},"computeCpfDv":{"!type":"fn(value: ?)","!doc":"<p>Generate the DV code (checksum part) for a CPF number</p>\n"},"evaluateConstraint":{"!type":"fn(profile: ?, constraint: ?, fieldName: ?, elem: ?)","!doc":"<p>Evaluates dojo.validate.check() constraints that are specified as array\narguments</p>\n"},"getEmailAddressList":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Check if value is an email address list. If an empty list\nis returned, the value didn&#39;t pass the test or it was empty.</p>\n"},"isEmailAddress":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid email address</p>\n"},"isEmailAddressList":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid email address list.</p>\n"},"isInRange":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates whether a string denoting a number\nis between a max and min.</p>\n"},"isIpAddress":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates an IP address</p>\n"},"isNumberFormat":{"!type":"fn(value: ?, flags?: ?) -> ?","!doc":"<p>Validates any sort of number based format</p>\n"},"isPhoneNumber":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validates 10 US digit phone number for several common formats</p>\n"},"isSocialSecurityNumber":{"!type":"fn(value: ?)","!doc":"<p>Validates social security number</p>\n"},"isState":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Validates US state and territory abbreviations.</p>\n"},"isText":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string has non whitespace characters.\nParameters allow you to constrain the length.</p>\n"},"isUrl":{"!type":"fn(value: ?, flags?: ?)","!doc":"<p>Checks if a string could be a valid URL</p>\n"},"isValidCnpj":{"!type":"fn(value: ?)","!doc":"<p>Validates a CNPJ/CGC number</p>\n"},"isValidCpf":{"!type":"fn(value: ?)","!doc":"<p>Validates a CPF number</p>\n"},"isValidCreditCard":{"!type":"fn(value: ?, ccType: ?)","!doc":"<p>Validate a credit card number by type with Luhn checking.</p>\n"},"isValidCreditCardNumber":{"!type":"fn(value: ?, ccType?: ?)","!doc":"<p>Checks if value matches the pattern for that card or any card types if none is specified</p>\n"},"isValidCvv":{"!type":"fn(value: ?, ccType: ?)","!doc":"<p>Validate the security code (CCV) for a passed credit-card type.</p>\n"},"isValidIsbn":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validate ISBN-10 or ISBN-13 based on the length of value</p>\n"},"isValidLuhn":{"!type":"fn(value: ?)","!doc":"<p>Validate a String value against the Luhn algorithm.</p>\n"},"isZipCode":{"!type":"fn(value: ?)","!doc":"<p>Validates U.S. zip-code</p>\n"}},"dojox/validate/web._isInRangeCache":{},"dojox/validate/web._cardInfo":{"!doc":"<p>A dictionary list of credit card abbreviations</p>\n","ax":{},"bl":{},"dc":{},"di":{},"ec":{},"er":{},"jcb":{},"mc":{},"vi":{}},"dojox/validate":{"!doc":"<p>Deprecated.  Should require dojox/validate modules directly rather than trying to access them through\nthis module.</p>\n"},"dojox/widget/AutoRotator":{"!type":"fn()","prototype":{"!proto":"dojox/widget/Rotator.prototype","!doc":"<p>A rotator that automatically transitions between child nodes.</p>\n","autoStart":{"!doc":"<p>Starts the timer to transition children upon creation.</p>\n"},"cycles":{"!doc":"<p>Number of cycles before pausing.</p>\n"},"duration":{"!doc":"<p>The time in milliseconds before transitioning to the next pane.  The\ndefault value is 4000 (4 seconds).</p>\n"},"panes":{"!doc":"<p>Array of panes to be created in the Rotator. Each array element\nwill be passed as attributes to a html.create() call.</p>\n"},"pauseOnManualChange":{"!doc":"<p>Pause the rotator when the pane is changed or a controller&#39;s next or\nprevious buttons are clicked.</p>\n"},"random":{"!doc":"<p>Determines if the panes should cycle randomly.</p>\n"},"reverse":{"!doc":"<p>Causes the rotator to rotate in reverse order.</p>\n"},"suspendOnHover":{"!doc":"<p>Pause the rotator when the mouse hovers over it.</p>\n"},"transition":{"!doc":"<p>The name of a function that is passed two panes nodes and a duration,\nthen returns a dojo.Animation object. The default value is\n&quot;dojox.widget.rotator.swap&quot;.</p>\n"},"transitionParams":{"!doc":"<p>Parameters for the transition. The string is read in and eval&#39;d as an\nobject.  If the duration is absent, the default value will be used.</p>\n"},"constructor":{"!type":"fn()","!doc":"<p>Initializes the timer and connect to the rotator.</p>\n"},"control":{"!type":"fn(action: ?)","!doc":"<p>Dispatches an action, first to this engine, then to the Rotator.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Disconnect the AutoRotator&#39;s events.</p>\n"},"go":{"!type":"fn(p?: ?) -> +null","!doc":"<p>Transitions the Rotator to the specified pane index.</p>\n"},"next":{"!type":"fn() -> ?","!doc":"<p>Transitions the Rotator to the next pane.</p>\n"},"pause":{"!type":"fn()","!doc":"<p>Sets the state to &quot;not playing&quot; and clears the cycle timer.</p>\n"},"play":{"!type":"fn(skipCycleDecrement?: ?, skipDuration?: ?)","!doc":"<p>Sets the state to &quot;playing&quot; and schedules the next cycle to run.</p>\n"},"prev":{"!type":"fn() -> ?","!doc":"<p>Transitions the Rotator to the previous pane.</p>\n"},"resize":{"!type":"fn(width: ?, height: ?)"}}},"dojox/widget/Rotator":{"!type":"fn()","prototype":{"!doc":"<p>A widget for rotating through child nodes using transitions.\n</p>\n","panes":{"!doc":"<p>Array of panes to be created in the Rotator. Each array element\nwill be passed as attributes to a html.create() call.</p>\n"},"transition":{"!doc":"<p>The name of a function that is passed two panes nodes and a duration,\nthen returns a dojo.Animation object. The default value is\n&quot;dojox.widget.rotator.swap&quot;.</p>\n"},"transitionParams":{"!doc":"<p>Parameters for the transition. The string is read in and eval&#39;d as an\nobject.  If the duration is absent, the default value will be used.</p>\n"},"constructor":{"!type":"fn(params: ?, node: ?)","!doc":"<p>Initializes the panes and events.</p>\n"},"control":{"!type":"fn(action: ?)","!doc":"<p>Dispatches an action, first to this engine, then to the Rotator.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Destroys the Rotator and its DOM node.</p>\n"},"go":{"!type":"fn(p?: ?) -> +null","!doc":"<p>Transitions the Rotator to the specified pane index.</p>\n"},"next":{"!type":"fn() -> ?","!doc":"<p>Transitions the Rotator to the next pane.</p>\n"},"prev":{"!type":"fn() -> ?","!doc":"<p>Transitions the Rotator to the previous pane.</p>\n"},"resize":{"!type":"fn(width: ?, height: ?)"}}},"dojox/widget/Calendar2Pane":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarBase.prototype","!doc":"<p>A Calendar with two panes, the second one containing both month and year</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parent":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/Calendar3Pane":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarBase.prototype","!doc":"<p>A Calendar with three panes, includes day, month, and year views</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parent":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/CalendarFisheye":{"!type":"fn()","prototype":{"!proto":"dojox/widget/Calendar.prototype","!doc":"<p>The standard Calendar. It includes day, month and year views.\nFisheyeLite effects are included.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parent":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/_FisheyeFX":{"!type":"fn()","prototype":{"!doc":"<p>A mixin to add a FisheyeLite effect to the calendar</p>\n","addFx":{"!type":"fn(theQuery: ?, fromNode: ?)"}}},"dojox/widget/FisheyeLite":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A Light-weight Fisheye Component, or an exhanced version\nof dojo/fx/Toggler ...</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"durationIn":{"!doc":"<p>The time (in ms) the run the show animation</p>\n"},"durationOut":{"!doc":"<p>The Time (in ms) to run the hide animation</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"properties":{"!doc":"<p>An object of &quot;property&quot;:scale pairs, or &quot;property&quot;:{} pairs.\ndefaults to font-size with a scale of 2.75\nIf a named property is an integer or float, the &quot;scale multiplier&quot;\nis used. If the named property is an object, that object is mixed\ninto the animation directly. eg: height:{ end:20, units:&quot;em&quot; }</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"units":{"!doc":"<p>Sometimes, you need to specify a unit. Should be part of\nproperties attrib, but was trying to shorthand the logic there</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(props: ?, node: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"easeIn":{"!type":"fn(n?: ?) -> number","!doc":"<p>An easing function to use for the show animation</p>\n"},"easeOut":{"!type":"fn(n?: ?) -> +Decimal","!doc":"<p>An easing function to use for the hide animation</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hide this fisheye item on mouse leave</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"show":{"!type":"fn()","!doc":"<p>Show this Fisheye item.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/CalendarFx":{"!type":"fn()","prototype":{"!proto":"dojox/widget/CalendarFisheye.prototype","!doc":"<p>The visual effects extensions for dojox/widget/Calendar.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parent":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/CalendarViews":{},"dojox/widget/Dialog":{"!type":"fn()","prototype":{"!doc":"<p>A Lightbox-like Modal-dialog for HTML Content</p>\n","dimensions":{"!doc":"<p>A two-element array of [width,height] to animate the Dialog to if sizeToViewport=&quot;false&quot;\nDefaults to [300,300]</p>\n"},"draggable":{"!doc":"<p>Make the pane draggable. Differs from dijit.Dialog by setting default to false</p>\n"},"easing":{"!doc":"<p>An easing function to apply to the sizing animation.</p>\n"},"modal":{"!doc":"<p>If true, this Dialog instance will be truly modal and prevent closing until\nexplicitly told to by calling hide() - Defaults to false to preserve previous\nbehaviors.</p>\n"},"showTitle":{"!doc":"<p>Toogle to show or hide the Title area. Can only be set at startup.</p>\n"},"sizeDuration":{"!doc":"<p>Time (in ms) to use in the Animation for sizing.</p>\n"},"sizeMethod":{"!doc":"<p>To be passed to dojox.fx.sizeTo, one of &quot;chain&quot; or &quot;combine&quot; to effect\nthe animation sequence.</p>\n"},"sizeToViewport":{"!doc":"<p>If true, fix the size of the dialog to the Viewport based on\nviewportPadding value rather than the calculated or natural\nstyle. If false, base the size on a passed dimension attribute.\nEither way, the viewportPadding value is used if the the content\nextends beyond the viewport size for whatever reason.</p>\n"},"templateString":{},"viewportPadding":{"!doc":"<p>If sizeToViewport=&quot;true&quot;, this is the amount of padding in pixels to leave\nbetween the dialog border and the viewport edge.\nThis value is also used when sizeToViewport=&quot;false&quot; and dimensions exceeded\nby dialog content to ensure dialog does not go outside viewport boundary</p>\n"},"constructor":{"!type":"fn(props: ?, node: ?)"},"layout":{"!type":"fn(e: ?)"},"show":{"!type":"fn()"}}},"dojox/widget/DialogSimple":{"!type":"fn()","prototype":{"!proto":"dojox/layout/ContentPane.prototype","!doc":"<p>A Simple Dialog Mixing the <code>dojox.layout.ContentPane</code> functionality over\ntop of a vanilla <code>dijit.Dialog</code>. See <code>dojox.widget.Dialog</code> for a more flexible\ndialog option allowing animations and different styles/theme support.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_firstFocusItem":{"!doc":"<p>The pointer to the first focusable node in the dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_focusManager":{},"_lastFocusItem":{"!doc":"<p>The pointer to which node has focus prior to our dialog.\nSet by <code>dijit/_DialogMixin._getFocusItems()</code>.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"adjustPaths":{"!doc":"<p>Adjust relative paths in html string content to point to this page.\nOnly useful if you grab content from a another folder then the current one</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"autofocus":{"!doc":"<p>A Toggle to modify the default focus behavior of a Dialog, which\nis to focus on the first dialog element after opening the dialog.\nFalse will disable autofocusing. Default: true</p>\n"},"baseClass":{},"class":{},"cleanContent":{"!doc":"<p>Cleans content to make it less likely to generate DOM/JS errors.\nUseful if you send ContentPane a complete page, instead of a html fragment\nscans for:\n\n</p>\n<ul>\n<li>title Node, remove</li>\n<li>DOCTYPE tag, remove</li>\n</ul>\n"},"closable":{"!doc":"<p>Dialog show [x] icon to close itself, and ESC key will close the dialog.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"cssStateNodes":{},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"draggable":{"!doc":"<p>Toggles the movable aspect of the Dialog. If true, Dialog\ncan be dragged by it&#39;s title. If false it will remain centered\nin the viewport.</p>\n"},"duration":{"!doc":"<p>The time in milliseconds it takes the dialog to fade in and out</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"executeScripts":{"!doc":"<p>Execute (eval) scripts that is found in the content</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"maxRatio":{"!doc":"<p>Maximum size to allow the dialog to expand to, relative to viewport size</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"open":{"!doc":"<p>True if Dialog is currently displayed on screen.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refocus":{"!doc":"<p>A Toggle to modify the default focus behavior of a Dialog, which\nis to re-focus the element which had focus before being opened.\nFalse will disable refocusing. Default: true</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"renderStyles":{"!doc":"<p>trigger/load styles in the content</p>\n"},"scriptHasHooks":{"!doc":"<p>replace keyword &#39;<em>container</em>&#39; in scripts with &#39;dijit.byId(this.id)&#39;\nNOTE this name might change in the near future</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"state":{"!doc":"<p>Will be &quot;Error&quot; if one or more of the child widgets has an invalid value,\n&quot;Incomplete&quot; if not all of the required child widgets are filled in.  Otherwise, &quot;&quot;,\nwhich indicates that the form is ready to be submitted.</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"connectChildren":{"!type":"fn(inStartup: ?)","!doc":"<p>You can call this function directly, ex. in the event that you\nprogrammatically add a widget to the form <em>after</em> the form has been\ninitialized.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"disconnectChildren":{"!type":"fn()","!doc":"<p>Deprecated method.   Applications no longer need to call this.   Remove for 2.0.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"execute":{"!type":"fn(formContents: ?)","!doc":"<p>Callback when the user hits the submit button.\nOverride this method to handle Dialog execution.</p>\n"},"focus":{"!type":"fn()"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getValues":{"!type":"fn() -> ?"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"hide":{"!type":"fn() -> +dojo","!doc":"<p>Hide the dialog</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Sends an HTTP GET request to the server.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns true if all of the widgets are valid.\nDeprecated, will be removed in 2.0.  Use get(&quot;state&quot;) instead.</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"reset":{"!type":"fn()"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"setValues":{"!type":"fn(val: ?) -> ?"},"show":{"!type":"fn() -> +dojo","!doc":"<p>Display the dialog</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"validate":{"!type":"fn() -> ?","!doc":"<p>returns if the form is valid - same as isValid - but\nprovides a few additional (ui-specific) features:\n\n</p>\n<ol>\n<li>it will highlight any sub-widgets that are not valid</li>\n<li>it will call focus() on the first invalid sub-widget</li>\n</ol>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/FisheyeList":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Menu similar to the fish eye menu on the Mac OS</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachEdge":{"!doc":"<p>Controls the border that the menu items don&#39;t expand past;\nfor example, if set to &quot;top&quot;, then the menu items will drop downwards as they expand.\nValues: &quot;center&quot;, &quot;left&quot;, &quot;right&quot;, &quot;top&quot;, &quot;bottom&quot;.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"conservativeTrigger":{"!doc":"<p>if true, don&#39;t start enlarging menu items until mouse is over an image;\nif false, start enlarging menu items as the mouse moves near them.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"EDGE":{},"effectUnits":{"!doc":"<p>controls how much reaction the menu makes, relative to the distance of the mouse from the menu</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"imgNode":{},"isFixed":{"!doc":"<p>toggle to enable additional listener (window scroll) if FisheyeList is in a fixed postion</p>\n"},"itemHeight":{"!doc":"<p>height of menu item (in pixels) in it&#39;s dormant state (when the mouse is far away)</p>\n"},"itemMaxHeight":{"!doc":"<p>height of menu item (in pixels) in it&#39;s fully enlarged state (when the mouse is directly over it)</p>\n"},"itemMaxWidth":{"!doc":"<p>width of menu item (in pixels) in it&#39;s fully enlarged state (when the mouse is directly over it)</p>\n"},"itemPadding":{"!doc":"<p>padding (in pixels) between each menu item</p>\n"},"itemWidth":{"!doc":"<p>width of menu item (in pixels) in it&#39;s dormant state (when the mouse is far away)</p>\n"},"labelEdge":{"!doc":"<p>Controls were the labels show up in relation to the menu item icons.\nValues: &quot;center&quot;, &quot;left&quot;, &quot;right&quot;, &quot;top&quot;, &quot;bottom&quot;.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"orientation":{"!doc":"<p>orientation of the menu, either &quot;horizontal&quot; or &quot;vertical&quot;</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"searchContainerNode":{},"snarfChildDomOutput":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>create our connections and setup our FisheyeList</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/FisheyeListItem":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Menu item inside of a FisheyeList.\nSee FisheyeList documentation for details on usage.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconSrc":{"!doc":"<p>pathname to image file (jpg, gif, png, etc.) of icon for this menu item</p>\n"},"id":{"!doc":"<p>will be set to the id of the orginal div element</p>\n"},"label":{"!doc":"<p>label to print next to the icon, when it is moused-over</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/MonthAndYearlyCalendar":{"!type":"fn()","prototype":{"!proto":"dojox/widget/_CalendarBase.prototype","!doc":"<p>A calendar with only a daily view.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"_views":{"!doc":"<p>The list of mixin views available on this calendar.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"constraints":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"footerFormat":{"!doc":"<p>The date format of the date displayed in the footer.    Can be\n&#39;short&#39;, &#39;medium&#39;, and &#39;long&#39;</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template to be used to construct the widget.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"useFx":{"!doc":"<p>Specifies if visual effects should be applied to the widget.\nThe default behavior of the widget does not contain any effects.\nThe dojox.widget.CalendarFx package is needed for these.</p>\n"},"value":{"!doc":"<p>The currently selected Date</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"addFx":{"!type":"fn(query: ?, fromNode: ?)"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn()"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isDisabledDate":{"!type":"fn(date: ?, locale?: ?) -> ?","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseInitialValue":{"!type":"fn(value: ?) -> +instance"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Instantiates the mixin views</p>\n"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/MultiSelectCalendar":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A simple GUI for choosing several dates in the context of a monthly calendar.\n</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"cssStateNodes":{},"currentFocus":{"!doc":"<p>Date object containing the currently focused date, or the date which would be focused\nif the calendar itself was focused.   Also indicates which year and month to display,\ni.e. the current &quot;page&quot; the calendar is on.</p>\n"},"datePackage":{"!doc":"<p>JavaScript namespace to find Calendar routines.  Uses Gregorian Calendar routines\nat dojo.date by default.</p>\n"},"dayWidth":{"!doc":"<p>How to represent the days of the week in the calendar header. See dojo.date.locale</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dndType":{"!doc":"<p>Defines a type of widget.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"returnIsoRanges":{},"searchContainerNode":{},"slideFrom":{"!doc":"<p>A parameter needed by RadioGroupSlide only. An optional paramter to force\nthe ContentPane to slide in from a set direction. Defaults\nto &quot;random&quot;, or specify one of &quot;top&quot;, &quot;left&quot;, &quot;right&quot;, &quot;bottom&quot;\nto slideFrom top, left, right, or bottom.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Order fields are traversed when user hits the tab key</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"value":{"!doc":"<p>The currently selected Dates, initially set to an empty object to indicate no selection.</p>\n"},"widgetsInTemplate":{},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(args: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus the calendar by focusing one of the calendar cells</p>\n"},"focusOnLastDate":{"!type":"fn(lastElement: ?)"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getClassForDate":{"!type":"fn(dateObject: ?, locale?: ?) -> string","!doc":"<p>May be overridden to return CSS classes to associate with the date entry for the given dateObject,\nfor example to indicate a holiday in specified locale.</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"goToToday":{"!type":"fn()","!doc":"<p>We go to today but we do no select it</p>\n"},"handleKey":{"!type":"fn(evt: ?) -> +any","!doc":"<p>Provides keyboard navigation of calendar.</p>\n"},"isDisabledDate":{"!type":"fn(dateObject: ?, locale?: ?) -> bool","!doc":"<p>May be overridden to disable certain dates in the calendar e.g. <code>isDisabledDate=dojo.date.locale.isWeekend</code></p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"selectDate":{"!type":"fn(dateToSelect: ?, selectedDates: ?)"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggleDate":{"!type":"fn(dateToToggle: ?, selectedDates: ?, unselectedDates: ?)"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unselectDate":{"!type":"fn(dateToUnselect: ?, unselectedDates: ?)"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/MultiSelectCalendar._MonthDropDown":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>The month drop down</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dndType":{"!doc":"<p>Defines a type of widget.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"months":{"!doc":"<p>List of names of months, possibly w/some undefined entries for Hebrew leap months\n(ex: [&quot;January&quot;, &quot;February&quot;, undefined, &quot;April&quot;, ...])</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"slideFrom":{"!doc":"<p>A parameter needed by RadioGroupSlide only. An optional paramter to force\nthe ContentPane to slide in from a set direction. Defaults\nto &quot;random&quot;, or specify one of &quot;top&quot;, &quot;left&quot;, &quot;right&quot;, &quot;bottom&quot;\nto slideFrom top, left, right, or bottom.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/Pager":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>A Pager, displaying a list of sized nodes</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>Time in milliseconds to transition the pages</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"iconNext":{"!doc":"<p>The url of the next page icon</p>\n"},"iconPage":{},"iconPageActive":{},"iconPrevious":{"!doc":"<p>The url of the previous page icon</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"itemSpace":{"!doc":"<p>Spacing between items? TODOC</p>\n"},"itemsPage":{"!doc":"<p>The numbers of items to display in each &quot;Page&quot;</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"orientation":{"!doc":"<p>Either &quot;horizontal or &quot;vertical&quot; to define the direction the pages will slide</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"pagerPos":{"!doc":"<p>TODOC</p>\n"},"resizeChildren":{"!doc":"<p>TODOC</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"statusPos":{"!doc":"<p>A string describing where to put the Pager &quot;current page&quot; indicator. Options are\n&quot;leading&quot; or &quot;trailing&quot;. In the case of horiztonal orientation, &quot;leading&quot; indicates\npositioned above the PageItems. In the case of vertical, &quot;leading&quot; indicates &quot;before&quot;.</p>\n"},"store":{"!doc":"<p>A dojo.data Data store</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"generatePagerItem":{"!type":"fn(item: ?, cnt: ?) -> +instance","!doc":"<p>this method instantiates pagerItems to be used by the pager, it is overridable to allow\ncustomizing these items</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn()"},"itemClass":{"!type":"fn()","!doc":"<p>The full dotted named of a Class or Widget constructor to use for the internal Pager Items.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/PagerItem":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"parseChildren":{"!type":"fn()"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is created</p>\n"},"resizeChildren":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/PlaceholderMenuItem":{"!type":"fn()","prototype":{"!proto":"dijit/MenuItem.prototype","!doc":"<p>A menu item that can be used as a placeholder.  Set the label\nof this item to a unique key and you can then use it to add new\nitems at that location.  This item is not displayed.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_isPlaceholder":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_replaced":{},"_replacedWith":{},"_setClassAttr":{},"_setDirAttr":{},"_setIconClassAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"accelKey":{"!doc":"<p>Text for the accelerator (shortcut) key combination, a control, alt, etc. modified keystroke meant to\nexecute the menu item regardless of where the focus is on the page.\n\n</p>\n<p>Note that although Menu can display accelerator keys, there is no infrastructure to actually catch and\nexecute those accelerators.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"disabled":{"!doc":"<p>If true, the menu item is disabled.\nIf false, the menu item is enabled.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"iconClass":{"!doc":"<p>Class to apply to DOMNode to make it display an icon.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"label":{"!doc":"<p>Menu text as HTML</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"shortcutKey":{"!doc":"<p>Single character (underlined when the parent Menu is focused) used to navigate directly to this widget,\nalso known as <a href=\"http://en.wikipedia.org/wiki/Mnemonics_(keyboard%29\">a mnemonic</a>.\nThis is denoted in the label by surrounding the single character with {}.\nFor example, if label=&quot;{F}ile&quot;, then shortcutKey=&quot;F&quot;.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"focus":{"!type":"fn()","!doc":"<p>Focus on this MenuItem</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"replace":{"!type":"fn(menuItems: ?) -> +any","!doc":"<p>replaces this menu item with the given menuItems.  The original\nmenu item is not actually removed from the menu - so if you want\nit removed, you must do that explicitly.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?)","!doc":"<p>Deprecated.   Use set(&#39;disabled&#39;, bool) instead.</p>\n"},"setLabel":{"!type":"fn(content: ?)","!doc":"<p>Deprecated.   Use set(&#39;label&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unReplace":{"!type":"fn(destroy?: ?) -> +any","!doc":"<p>Removes menu items added by calling replace().  It returns the\narray of items that were actually removed (in case you want to\nclean them up later)</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/Portlet":{"!type":"fn()","prototype":{"!proto":"dijit/TitlePane.prototype","!doc":"<p>A container widget that is designed to be contained\nin a dojox.layout.GridContainer. Child widgets can insert\nan icon into the title bar of the Portlet, which when\nclicked, executes the &quot;toggle&quot; method of the child widget.\nA child widget must specify the attribute\n&quot;portletIconClass&quot;, and the optional class\n&quot;portletIconHoverClass&quot;, as well as the\n&quot;toggle&quot; function.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_parents":{"!doc":"<p>An array of all the StackContainer widgets that this Portlet\nis contained in.    These are used to determine if the portlet\nis visible or not.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_setTooltipAttr":{},"_size":{"!doc":"<p>Cache of the previous size of the portlet, used to determine\nif the size has changed and if the child widgets should be\nresized.</p>\n"},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"active":{"!doc":"<p>True if mouse was pressed while over this widget, and hasn&#39;t been released yet</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>The root className to be placed on this widget&#39;s domNode.</p>\n"},"class":{},"closable":{"!doc":"<p>If true, a close button is placed in the title bar,\nand the Portlet can be hidden. If false, the Portlet\ncannot be closed.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"cssStateNodes":{"!doc":"<p>Subclasses may define a cssStateNodes property that lists sub-nodes within the widget that\nneed CSS classes applied on mouse hover/press and focus.\n\n</p>\n<p>Each entry in this optional hash is a an attach-point name (like &quot;upArrowButton&quot;) mapped to a CSS class name\n(like &quot;dijitUpArrowButton&quot;). Example:\n\n</p>\n<pre><code>{\n    &quot;upArrowButton&quot;: &quot;dijitUpArrowButton&quot;,\n    &quot;downArrowButton&quot;: &quot;dijitDownArrowButton&quot;\n}</code></pre>\n<p>The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it\n\n</p>\n<p>is hovered, etc.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>Don&#39;t change this parameter from the default value.\nThis ContentPane parameter doesn&#39;t make sense for TitlePane, since TitlePane\nis never a child of a layout container, nor should TitlePane try to control\nthe size of an inner widget.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"dragRestriction":{"!doc":"<p>To remove the drag capability.</p>\n"},"duration":{"!doc":"<p>Time in milliseconds to fade in/fade out</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hovering":{"!doc":"<p>True if cursor is over this widget</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"open":{"!doc":"<p>Whether pane is opened or closed.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"resizeChildren":{"!doc":"<p>If true, when the Portlet is resized, any child widgets\nwith a &#39;resize&#39; method have that method called.</p>\n"},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"tabIndex":{"!doc":"<p>Tabindex setting for the title (so users can tab to the title then\nuse space/enter to open/close the title pane)</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>Title of the pane</p>\n"},"toggleable":{"!doc":"<p>Whether pane can be opened or closed by clicking the title bar.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(child: ?)","!doc":"<p>Adds a child widget to the portlet.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"setTitle":{"!type":"fn(title: ?)","!doc":"<p>Deprecated.  Use set(&#39;title&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/PortletSettings":{"!type":"fn()","prototype":{"!proto":"dijit/_Container.prototype","!doc":"<p>A settings widget to be used with a dojox.widget.Portlet.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"portletIconClass":{"!doc":"<p>The CSS class to apply to the icon in the Portlet title bar that is used\nto toggle the visibility of this widget.</p>\n"},"portletIconHoverClass":{"!doc":"<p>The CSS class to apply to the icon in the Portlet title bar that is used\nto toggle the visibility of this widget when the mouse hovers over it.</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Sets the require CSS classes on the widget.</p>\n"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggle":{"!type":"fn()","!doc":"<p>Toggles the visibility of this widget.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/PortletDialogSettings":{"!type":"fn()","prototype":{"!proto":"dojox/widget/PortletSettings.prototype","!doc":"<p>A settings widget to be used with a dojox.widget.Portlet, which displays\nthe contents of this widget in a dijit.Dialog box.</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"dimensions":{"!doc":"<p>The size of the dialog to display.  This defaults to [300, 300]</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"portletIconClass":{"!doc":"<p>The CSS class to apply to the icon in the Portlet title bar that is used\nto toggle the visibility of this widget.</p>\n"},"portletIconHoverClass":{"!doc":"<p>The CSS class to apply to the icon in the Portlet title bar that is used\nto toggle the visibility of this widget when the mouse hovers over it.</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(props: ?, node: ?)"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()","!doc":"<p>Sets the require CSS classes on the widget.</p>\n"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Call startup() on all children including non _Widget ones like dojo/dnd/Source objects</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toggle":{"!type":"fn()","!doc":"<p>Shows and hides the Dialog box.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/Roller":{"!type":"fn()","prototype":{"!doc":"<p>A simple widget to take an unordered-list of Text and roll through them</p>\n","_idx":{"!doc":"<p>Index of the the currently visible item in the list of items[]</p>\n"},"autoStart":{"!doc":"<p>Toggle to control starup behavior. Call .start() manually\nif set to <code>false</code></p>\n"},"delay":{"!doc":"<p>Interval between rolls</p>\n"},"durationIn":{"!doc":"<p>Speed (in ms) to apply to the &quot;in&quot; animation (show the node)</p>\n"},"durationOut":{"!doc":"<p>Speed (in ms) to apply to the &quot;out&quot; animation (hide the showing node)</p>\n"},"items":{"!doc":"<p>If populated prior to instantiation, is used as the Items over the children</p>\n"},"itemSelector":{"!doc":"<p>A CSS selector to be used by <code>dojo.query</code> to find the children\nitems in this widget. Defaults to &quot;&gt; li&quot;, finding only first-children\nlist-items in the list, allowing for embedded lists to occur.</p>\n"},"makeAnims":{"!type":"fn()","!doc":"<p>Animation creator function. Need to create an &#39;in&#39; and &#39;out&#39;\nAnimation stored in _anim Object, which the rest of the widget\nwill reuse.</p>\n"},"postCreate":{"!type":"fn()"},"start":{"!type":"fn()","!doc":"<p>Starts to Roller looping</p>\n"},"stop":{"!type":"fn()","!doc":"<p>Stops the Roller from looping anymore.</p>\n"}},"RollerSlide":{"!type":"fn()"}},"dojox/widget/Roller.RollerSlide":{"!type":"fn()","prototype":{"!doc":"<p>An add-on to the Roller to modify animations. This produces\na slide-from-bottom like effect. See <code>dojox.widget.Roller</code> for\nfull API information.</p>\n","_idx":{"!doc":"<p>Index of the the currently visible item in the list of items[]</p>\n"},"autoStart":{"!doc":"<p>Toggle to control starup behavior. Call .start() manually\nif set to <code>false</code></p>\n"},"delay":{"!doc":"<p>Interval between rolls</p>\n"},"durationIn":{"!doc":"<p>Speed (in ms) to apply to the &quot;in&quot; animation (show the node)</p>\n"},"durationOut":{},"items":{"!doc":"<p>If populated prior to instantiation, is used as the Items over the children</p>\n"},"itemSelector":{"!doc":"<p>A CSS selector to be used by <code>dojo.query</code> to find the children\nitems in this widget. Defaults to &quot;&gt; li&quot;, finding only first-children\nlist-items in the list, allowing for embedded lists to occur.</p>\n"},"makeAnims":{"!type":"fn()","!doc":"<p>Animation creator function. Need to create an &#39;in&#39; and &#39;out&#39;\nAnimation stored in _anim Object, which the rest of the widget\nwill reuse.</p>\n"},"postCreate":{"!type":"fn()"},"start":{"!type":"fn()","!doc":"<p>Starts to Roller looping</p>\n"},"stop":{"!type":"fn()","!doc":"<p>Stops the Roller from looping anymore.</p>\n"}}},"dojox/widget/Roller._Hover":{"!type":"fn()","prototype":{"!doc":"<p>A mixin class to provide a way to automate the &quot;stop on hover&quot; functionality.\n</p>\n","postCreate":{"!type":"fn()"}}},"dojox/widget/Standby":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A widget designed to act as a Standby/Busy/Disable/Blocking widget to indicate a\nparticular DOM node is processing and cannot be clicked on at this time.\nThis widget uses absolute positioning to apply the overlay and image.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_centerNode":{"!doc":"<p>Which node to use as the center node, the image or the text node.</p>\n"},"_displayed":{"!doc":"<p>Flag to indicate if the overlay is displayed or not.</p>\n"},"_focusManager":{},"_imageNode":{"!doc":"<p>The image node where we attach and define the image to display.</p>\n"},"_parent":{"!doc":"<p>Wrapping div for the widget, also used for IE 7 in dealing with the\nzoom issue.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_resizeCheck":{"!doc":"<p>Handle to interval function that checks the target for changes.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>Trap flag to ensure startup only processes once.</p>\n"},"_textNode":{"!doc":"<p>The div to attach text/HTML in the overlay center item.</p>\n"},"_underlayNode":{"!doc":"<p>The node that is the translucent underlay for the\nimage that blocks access to the target.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"centerIndicator":{"!doc":"<p>Property to define if the image and its alt text should be used, or\na simple Text/HTML node should be used.  Allowable values are &#39;image&#39;\nand &#39;text&#39;.\nDefault is &#39;image&#39;.</p>\n"},"class":{},"color":{"!doc":"<p>The color to set the overlay.  Should be in #XXXXXX form.\nDefault color for the translucent overlay is light gray.</p>\n"},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dndType":{"!doc":"<p>Defines a type of widget.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>Integer defining how long the show and hide effects should take in milliseconds.\nDefaults to 500</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"image":{"!doc":"<p>A URL to an image to center within the blocking overlay.\nThe default is a basic spinner.</p>\n"},"imageText":{"!doc":"<p>Text to set on the ALT tag of the image.\nThe default is &#39;Please wait...&#39;</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"opacity":{"!doc":"<p>The opacity to make the overlay when it is displayed/faded in.\nThe default is 0.75.  This does not affect the image opacity, only the\noverlay.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"searchContainerNode":{},"slideFrom":{"!doc":"<p>A parameter needed by RadioGroupSlide only. An optional paramter to force\nthe ContentPane to slide in from a set direction. Defaults\nto &quot;random&quot;, or specify one of &quot;top&quot;, &quot;left&quot;, &quot;right&quot;, &quot;bottom&quot;\nto slideFrom top, left, right, or bottom.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"target":{"!doc":"<p>The target to overlay when active.  Can be a widget id, a\ndom id, or a direct node reference.</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{"!doc":"<p>The template string defining out the basics of the widget.  No need for an external\nfile.</p>\n"},"text":{"!doc":"<p>Text/HTML to display in the center of the overlay\nThis is used if image center is disabled.\nDefaults to &#39;Please Wait...&#39;</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"zIndex":{"!doc":"<p>Control that lets you specify if the zIndex for the overlay\nshould be auto-computed based off parent zIndex, or should be set\nto a particular value.  This is useful when you want to overlay\nthings in digit.Dialogs, you can specify a base zIndex to append from.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Function to hide the blocking overlay and status icon or text.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isVisible":{"!type":"fn() -> +any","!doc":"<p>Helper function so you can test if the widget is already visible or not.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"show":{"!type":"fn()","!doc":"<p>Function to display the blocking overlay and busy/status icon or text.</p>\n"},"startup":{"!type":"fn(args: ?)","!doc":"<p>Over-ride of the basic widget startup function.\nConfigures the target node and sets the image to use.</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn()","!doc":"<p>Over-ride to hide the widget, which clears intervals, before cleanup.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/TitleGroup":{"!type":"fn()","prototype":{"!proto":"dijit/_Widget.prototype","!doc":"<p>A container which controls a series of <code>dijit.TitlePane</code>s,\nallowing one to be visible and hiding siblings</p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"dndType":{"!doc":"<p>Defines a type of widget.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"observer":{},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"slideFrom":{"!doc":"<p>A parameter needed by RadioGroupSlide only. An optional paramter to force\nthe ContentPane to slide in from a set direction. Defaults\nto &quot;random&quot;, or specify one of &quot;top&quot;, &quot;left&quot;, &quot;right&quot;, &quot;bottom&quot;\nto slideFrom top, left, right, or bottom.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, position?: ?) -> ?","!doc":"<p>Add a passed widget reference to this container at an optional\nposition index.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()","!doc":"<p>Called after the parameters to the widget have been read-in,\nbut before the widget template is instantiated. Especially\nuseful to set properties that are referenced in the widget\ntemplate.</p>\n"},"removeChild":{"!type":"fn(widget: ?) -> ?","!doc":"<p>Remove the passed widget from this container. Does not destroy\nchild.</p>\n"},"selectChild":{"!type":"fn(widget: ?) -> ?","!doc":"<p>close all found titlePanes within this group, excluding\nthe one the we pass to select</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/Toaster":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Message that slides in from the corner of the screen, used for notifications\nlike &quot;new email&quot;.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"defaultType":{"!doc":"<p>If message type isn&#39;t specified (see &quot;messageTopic&quot; parameter),\nthen display message as this type.\nPossible values in messageTypes enumeration (&quot;message&quot;, &quot;warning&quot;, &quot;error&quot;, &quot;fatal&quot;)</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"duration":{"!doc":"<p>Number of milliseconds to show message</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"messageTopic":{"!doc":"<p>Name of topic; anything published to this topic will be displayed as a message.\nMessage format is either String or an object like\n{message: &quot;hello word&quot;, type: &quot;error&quot;, duration: 500}</p>\n"},"messageTypes":{"!doc":"<p>Possible message types.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"positionDirection":{"!doc":"<p>Position from which message slides into screen, one of\n[&quot;br-up&quot;, &quot;br-left&quot;, &quot;bl-up&quot;, &quot;bl-right&quot;, &quot;tr-down&quot;, &quot;tr-left&quot;, &quot;tl-down&quot;, &quot;tl-right&quot;]</p>\n"},"positionDirectionTypes":{"!doc":"<p>Possible values for positionDirection parameter</p>\n"},"postMixInProperties":{},"searchContainerNode":{},"separator":{"!doc":"<p>String used to separate messages if consecutive calls are made to setContent before previous messages go away</p>\n"},"slideDuration":{"!doc":"<p>Number of milliseconds for the slide animation, increasing will cause the Toaster\nto slide in more slowly.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>hide the Toaster</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setContent":{"!type":"fn(message: ?, messageType: ?, duration?: ?)","!doc":"<p>sets and displays the given message and show duration</p>\n"},"show":{"!type":"fn()","!doc":"<p>&#39;\nshow the Toaster</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/UpgradeBar":{"!type":"fn()","prototype":{"!proto":"dijit/_WidgetBase.prototype","!doc":"<p>Shows a bar at the top of the screen when the user is to\nbe notified that they should upgrade their browser or a\nplugin.</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{"!doc":"<p>Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate\nwidget state.</p>\n"},"buttonCancel":{"!doc":"<p>The HTML tip show when hovering over the close button.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"noRemindButton":{"!doc":"<p>The text link shown that when clicked, permanently dismisses\nthe message (sets a cookie). If this string is blank, this\nlink is not displayed.</p>\n"},"notifications":{"!doc":"<p>An array of objects that hold the criteria for upgrades:\n\n</p>\n<ul>\n<li>message: String: The message to display in the bar. Can be HTML.</li>\n<li>validate: Function: The expression to evaluate to determine if the\n  bar should show or not. Should be a simple expression\n  if used in HTML:</li>\n</ul>\n<pre><code>&lt;div validate=&quot;!google.gears&quot;&gt;\n&lt;div validate=&quot;has(&#39;ie&#39;)&lt;8&quot;&gt;</code></pre>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"postMixInProperties":{},"searchContainerNode":{},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"buildRendering":{"!type":"fn()","!doc":"<p>Construct the UI for this widget, setting this.domNode.\nMost widgets will mixin <code>dijit._TemplatedMixin</code>, which implements this method.</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(props: ?, node: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy this widget, but not its descendants.  Descendants means widgets inside of\nthis.containerNode.   Will also destroy any resources (including widgets) registered via this.own().\n\n</p>\n<p>This method will also destroy internal widgets such as those created from a template,\nassuming those widgets exist inside of this.domNode but outside of this.containerNode.\n\n</p>\n<p>For 2.0 it&#39;s planned that this method will also destroy descendant widgets, so apps should not\ndepend on the current ability to destroy a widget without destroying its descendants.   Generally\nthey should use destroyRecursive() for widgets with children.</p>\n"},"destroyDescendants":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Recursively destroy the children of this widget and their\ndescendants.</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hide":{"!type":"fn()","!doc":"<p>Hides the bar. May be called externally.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"isValid":{"!type":"fn()"},"notify":{"!type":"fn(msg: ?)","!doc":"<p>Triggers the bar to display. An internal function,\nbut could be called externally for fun.</p>\n"},"on":{"!type":"fn(type: ?, func: ?) -> ?","!doc":"<p>Call specified function when event occurs, ex: myWidget.on(&quot;click&quot;, function(){ ... }).</p>\n"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"startup":{"!type":"fn()","!doc":"<p>Processing after the DOM fragment is added to the document</p>\n"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/Wizard":{"!type":"fn()","prototype":{"!proto":"dijit/layout/StackContainer.prototype","!doc":"<p>A set of panels that display sequentially, typically notating a step-by-step\nprocedure like an install</p>\n","_attachEvents":{"!doc":"<p>List of connections associated with data-dojo-attach-event=... in the\ntemplate</p>\n"},"_attachPoints":{"!doc":"<p>List of widget attribute names associated with data-dojo-attach-point=... in the\ntemplate, ex: [&quot;containerNode&quot;, &quot;labelNode&quot;]</p>\n"},"_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_earlyTemplatedStartup":{"!doc":"<p>A fallback to preserve the 1.0 - 1.3 behavior of children in\ntemplates having their startup called before the parent widget\nfires postCreate. Defaults to &#39;false&#39;, causing child widgets to\nhave their .startup() called immediately before a parent widget\n.startup(), but always after the parent .postCreate(). Set to\n&#39;true&#39; to re-enable to previous, arguably broken, behavior.</p>\n"},"_focusManager":{},"_rendered":{"!doc":"<p>Not normally use, but this flag can be set by the app if the server has already rendered the template,\ni.e. already inlining the template for the widget into the main page.   Reduces _TemplatedMixin to\njust function like _AttachMixin.</p>\n"},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_skipNodeCache":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attachScope":{"!doc":"<p>Object to which attach points and events will be scoped.  Defaults\nto &#39;this&#39;.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"cancelButtonLabel":{"!doc":"<p>Label override for the &quot;Cancel&quot; button.</p>\n"},"cancelFunction":{"!doc":"<p>Name of function to call if user presses cancel button.\nCancel button is not displayed if function is not specified.</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"contextRequire":{"!doc":"<p>Used to provide a context require to the dojo/parser in order to be\nable to use relative MIDs (e.g. <code>./Widget</code>) in the widget&#39;s template.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<p>If true, change the size of my currently displayed child to match my size</p>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"doneButtonLabel":{"!doc":"<p>Label override for the &quot;Done&quot; button.</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"hideDisabled":{"!doc":"<p>If true, disabled buttons are hidden; otherwise, they are assigned the\n&quot;WizardButtonDisabled&quot; CSS class</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget is going to call resize() on its\nchildren widgets, setting their size, when they become visible.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"nextButtonLabel":{"!doc":"<p>Label override for the &quot;Next&quot; button.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"persist":{"!doc":"<p>Remembers the selected child across sessions</p>\n"},"previousButtonLabel":{"!doc":"<p>Label override for the &quot;Previous&quot; button.</p>\n"},"searchContainerNode":{},"selectedChildWidget":{"!doc":"<p>References the currently selected child widget, if any.\nAdjust selected child with selectChild() method.</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"templatePath":{"!doc":"<p>Path to template (HTML file) for this widget relative to dojo.baseUrl.\nDeprecated: use templateString with require([... &quot;dojo/text!...&quot;], ...) instead</p>\n"},"templateString":{},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"widgetsInTemplate":{"!doc":"<p>Should we parse the template to find widgets that might be\ndeclared in markup inside it?  (Remove for 2.0 and assume true)</p>\n"},"addChild":{"!type":"fn(child: ?, insertIndex?: ?)"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"back":{"!type":"fn() -> ?","!doc":"<p>Go back to previous page.</p>\n"},"buildRendering":{"!type":"fn()"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)"},"destroyRecursive":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroy this widget and its descendants</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"done":{"!type":"fn()","!doc":"<p>Finish the wizard&#39;s operation</p>\n"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"forward":{"!type":"fn() -> ?","!doc":"<p>Advance to next page.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexInParent":{"!type":"fn() -> number","!doc":"<p>Returns the index of this widget within its container parent.\nIt returns -1 if the parent does not exist, or if the parent\nis not a dijit/_Container</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getNextSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the last child of the parent,\notherwise returns the next element sibling to the &quot;right&quot;.</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"getPreviousSibling":{"!type":"fn() -> ?","!doc":"<p>Returns null if this is the first child of the parent,\notherwise returns the next element sibling to the &quot;left&quot;.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"layout":{"!type":"fn()"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"removeChild":{"!type":"fn(page: ?)"},"resize":{"!type":"fn()"},"selectChild":{"!type":"fn(page: ?, animate: ?) -> ?","!doc":"<p>Show the given widget (which must be one of my children)</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/WizardPane":{"!type":"fn()","prototype":{"!proto":"dijit/layout/ContentPane.prototype","!doc":"<p>A panel in a <code>dojox.widget.Wizard</code></p>\n","_attrPairNames":{},"_blankGif":{"!doc":"<p>Path to a blank 1x1 image.\nUsed by <code>&lt;img&gt;</code> nodes in templates that really get their image via CSS background-image.</p>\n"},"_focusManager":{},"_setClassAttr":{},"_setDirAttr":{},"_setIdAttr":{},"_setLangAttr":{},"_setTitleAttr":{},"_started":{"!doc":"<p>startup() has completed.</p>\n"},"attributeMap":{"!doc":"<p>Deprecated. Instead of attributeMap, widget should have a _setXXXAttr attribute\nfor each XXX attribute to be mapped to the DOM.\n\n</p>\n<p>attributeMap sets up a &quot;binding&quot; between attributes (aka properties)\nof the widget and the widget&#39;s DOM.\nChanges to widget attributes listed in attributeMap will be\nreflected into the DOM.\n\n</p>\n<p>For example, calling set(&#39;title&#39;, &#39;hello&#39;)\non a TitlePane will automatically cause the TitlePane&#39;s DOM to update\nwith the new title.\n\n</p>\n<p>attributeMap is a hash where the key is an attribute of the widget,\nand the value reflects a binding to a:\n\n</p>\n<ul>\n<li><p>DOM node attribute</p>\n<p>  focus: {node: &quot;focusNode&quot;, type: &quot;attribute&quot;}\nMaps this.focus to this.focusNode.focus</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerHTML</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerHTML&quot; }\nMaps this.title to this.titleNode.innerHTML</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node innerText</p>\n<p>  title: { node: &quot;titleNode&quot;, type: &quot;innerText&quot; }\nMaps this.title to this.titleNode.innerText</p>\n</li>\n</ul>\n<ul>\n<li><p>DOM node CSS class</p>\n<p>  myClass: { node: &quot;domNode&quot;, type: &quot;class&quot; }\nMaps this.myClass to this.domNode.className</p>\n</li>\n</ul>\n<p>If the value is an array, then each element in the array matches one of the\nformats of the above list.\n\n</p>\n<p>There are also some shorthands for backwards compatibility:\n\n</p>\n<ul>\n<li>string --&gt; { node: string, type: &quot;attribute&quot; }, for example:</li>\n</ul>\n<pre><code>&quot;focusNode&quot; ---&gt; { node: &quot;focusNode&quot;, type: &quot;attribute&quot; }</code></pre>\n<ul>\n<li>&quot;&quot; --&gt; { node: &quot;domNode&quot;, type: &quot;attribute&quot; }</li>\n</ul>\n"},"baseClass":{},"canGoBack":{"!doc":"<p>If true, then can move back to a previous panel (by clicking the &quot;Previous&quot; button)</p>\n"},"class":{},"containerNode":{"!doc":"<p>Designates where children of the source DOM node will be placed.\n&quot;Children&quot; in this case refers to both DOM nodes and widgets.\nFor example, for myWidget:\n\n\n</p>\n<pre><code>&lt;div data-dojo-type=myWidget&gt;\n    &lt;b&gt; here&#39;s a plain DOM node\n    &lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n    &lt;i&gt; and another plain DOM node &lt;/i&gt;\n&lt;/div&gt;</code></pre>\n<p>containerNode would point to:\n\n\n</p>\n<pre><code>&lt;b&gt; here&#39;s a plain DOM node\n&lt;span data-dojo-type=subWidget&gt;and a widget&lt;/span&gt;\n&lt;i&gt; and another plain DOM node &lt;/i&gt;</code></pre>\n<p>In templated widgets, &quot;containerNode&quot; is set via a\ndata-dojo-attach-point assignment.\n\n</p>\n<p>containerNode must be defined for any widget that accepts innerHTML\n(like ContentPane or BorderContainer or even Button), and conversely\nis null for widgets that don&#39;t, like TextBox.</p>\n"},"content":{"!doc":"<p>The innerHTML of the ContentPane.\nNote that the initialization parameter / argument to set(&quot;content&quot;, ...)\ncan be a String, DomNode, Nodelist, or _Widget.</p>\n"},"dir":{"!doc":"<p>Bi-directional support, as defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir\">HTML DIR</a>\nattribute. Either left-to-right &quot;ltr&quot; or right-to-left &quot;rtl&quot;.  If undefined, widgets renders in page&#39;s\ndefault direction.</p>\n"},"doLayout":{"!doc":"<ul>\n<li>false - don&#39;t adjust size of children</li>\n<li>true - if there is a single visible child widget, set it&#39;s size to however big the ContentPane is</li>\n</ul>\n"},"domNode":{"!doc":"<p>This is our visible representation of the widget! Other DOM\nNodes may by assigned to other properties, usually through the\ntemplate system&#39;s data-dojo-attach-point syntax, but the domNode\nproperty is the canonical &quot;top level&quot; node in widget UI.</p>\n"},"doneFunction":{"!doc":"<p>Name of function that is run if you press the &quot;Done&quot; button from this panel</p>\n"},"errorMessage":{"!doc":"<p>Message that shows if an error occurs</p>\n"},"extractContent":{"!doc":"<p>Extract visible content from inside of <code>&lt;body&gt; .... &lt;/body&gt;</code>.\nI.e., strip <code>&lt;html&gt;</code> and <code>&lt;head&gt;</code> (and it&#39;s contents) from the href</p>\n"},"focused":{"!doc":"<p>This widget or a widget it contains has focus, or is &quot;active&quot; because\nit was recently clicked.</p>\n"},"href":{"!doc":"<p>The href of the content that displays now.\nSet this at construction if you want to load data externally when the\npane is shown.  (Set preload=true to load it immediately.)\nChanging href after creation doesn&#39;t have any effect; Use set(&#39;href&#39;, ...);</p>\n"},"id":{"!doc":"<p>A unique, opaque ID string that can be assigned by users or by the\nsystem. If the developer passes an ID which is known not to be\nunique, the specified ID is ignored and the system-generated ID is\nused instead.</p>\n"},"ioArgs":{"!doc":"<p>Parameters to pass to xhrGet() request, for example:\n\n</p>\n<pre><code>&lt;div data-dojo-type=&quot;dijit/layout/ContentPane&quot; data-dojo-props=&quot;href: &#39;./bar&#39;, ioArgs: {timeout: 500}&quot;&gt;</code></pre>\n"},"isLayoutContainer":{"!doc":"<p>Indicates that this widget will call resize() on it&#39;s child widgets\nwhen they become visible.</p>\n"},"isLoaded":{"!doc":"<p>True if the ContentPane has data in it, either specified\nduring initialization (via href or inline content), or set\nvia set(&#39;content&#39;, ...) / set(&#39;href&#39;, ...)\n\n</p>\n<p>False if it doesn&#39;t have any content, or if ContentPane is\nstill in the process of downloading href.</p>\n"},"lang":{"!doc":"<p>Rarely used.  Overrides the default Dojo locale used to render this widget,\nas defined by the <a href=\"http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang\">HTML LANG</a> attribute.\nValue must be among the list of locales specified during by the Dojo bootstrap,\nformatted according to <a href=\"http://www.ietf.org/rfc/rfc3066.txt\">RFC 3066</a> (like en-us).</p>\n"},"loadingMessage":{"!doc":"<p>Message that shows while downloading</p>\n"},"onLoadDeferred":{"!doc":"<p>This is the <code>dojo.Deferred</code> returned by set(&#39;href&#39;, ...) and refresh().\nCalling onLoadDeferred.then() registers your\ncallback to be called only once, when the prior set(&#39;href&#39;, ...) call or\nthe initial href parameter to the constructor finishes loading.\n\n</p>\n<p>This is different than an onLoad() handler which gets called any time any href\nor content is loaded.</p>\n"},"ownerDocument":{"!doc":"<p>The document this widget belongs to.  If not specified to constructor, will default to\nsrcNodeRef.ownerDocument, or if no sourceRef specified, then to the document global</p>\n"},"parseOnLoad":{"!doc":"<p>Parse content and create the widgets, if any.</p>\n"},"parserScope":{"!doc":"<p>Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,\nwill search for data-dojo-type (or dojoType).  For backwards compatibility\nreasons defaults to dojo._scopeName (which is &quot;dojo&quot; except when\nmulti-version support is used, when it will be something like dojo16, dojo20, etc.)</p>\n"},"passFunction":{"!doc":"<p>Name of function that checks if it&#39;s OK to advance to the next panel.\nIf it&#39;s not OK (for example, mandatory field hasn&#39;t been entered), then\nreturns an error message (String) explaining the reason. Can return null (pass)\nor a Boolean (true == pass)</p>\n"},"preload":{"!doc":"<p>Force load of data on initialization even if pane is hidden.</p>\n"},"preventCache":{"!doc":"<p>Prevent caching of data from href&#39;s by appending a timestamp to the href.</p>\n"},"refreshOnShow":{"!doc":"<p>Refresh (re-download) content when pane goes from hidden to shown</p>\n"},"srcNodeRef":{"!doc":"<p>pointer to original DOM node</p>\n"},"stopParser":{},"style":{"!doc":"<p>HTML style attributes as cssText string or name/value hash</p>\n"},"template":{"!doc":"<p>Flag from the parser that this ContentPane is inside a template\nso the contents are pre-parsed.</p>\n"},"title":{"!doc":"<p>HTML title attribute.\n\n</p>\n<p>For form widgets this specifies a tooltip to display when hovering over\nthe widget (just like the native HTML title attribute).\n\n</p>\n<p>For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,\netc., it&#39;s used to specify the tab label, accordion pane title, etc.  In this case it&#39;s\ninterpreted as HTML.</p>\n"},"tooltip":{"!doc":"<p>When this widget&#39;s title attribute is used to for a tab label, accordion pane title, etc.,\nthis specifies the tooltip to appear when the mouse is hovered over that text.</p>\n"},"addChild":{"!type":"fn(widget: ?, insertIndex?: ?)","!doc":"<p>Makes the given widget a child of this widget.</p>\n"},"attr":{"!type":"fn(name: ?, value?: ?) -> ?","!doc":"<p>This method is deprecated, use get() or set() directly.</p>\n"},"buildRendering":{"!type":"fn()"},"cancel":{"!type":"fn()","!doc":"<p>Cancels an in-flight download of content</p>\n"},"connect":{"!type":"fn(obj: ?, event: ?, method: ?) -> +any","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(on(...)) or this.own(aspect.after(...)) instead.\n\n</p>\n<p>Connects specified obj/event to specified method of this object\nand registers for disconnect() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.connect, except with the\nimplicit use of this widget as the target object.\nEvents connected with <code>this.connect</code> are disconnected upon\ndestruction.</p>\n"},"constructor":{"!type":"fn(params: ?, srcNodeRef?: ?)","!doc":"<p>Create the widget.</p>\n"},"create":{"!type":"fn(params: ?, srcNodeRef: ?)"},"defer":{"!type":"fn(fcn: ?, delay?: ?) -> ?","!doc":"<p>Wrapper to setTimeout to avoid deferred functions executing\nafter the originating widget has been destroyed.\nReturns an object handle with a remove method (that returns null) (replaces clearTimeout).</p>\n"},"destroy":{"!type":"fn()"},"destroyDescendants":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy all the widgets inside the ContentPane and empty containerNode</p>\n"},"destroyRecursive":{"!type":"fn(preserveDom: ?)","!doc":"<p>Destroy the ContentPane and its contents</p>\n"},"destroyRendering":{"!type":"fn(preserveDom?: ?)","!doc":"<p>Destroys the DOM nodes associated with this widget.</p>\n"},"disconnect":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Disconnects handle created by <code>connect</code>.</p>\n"},"done":{"!type":"fn()"},"emit":{"!type":"fn(type: ?, eventObj?: ?, callbackArgs?: ?) -> ?","!doc":"<p>Used by widgets to signal that a synthetic event occurred, ex:\n\n</p>\n<pre><code>myWidget.emit(&quot;attrmodified-selectedChildWidget&quot;, {}).</code></pre>\n<p>Emits an event on this.domNode named type.toLowerCase(), based on eventObj.\nAlso calls onType() method, if present, and returns value from that method.\nBy default passes eventObj to callback, but will pass callbackArgs instead, if specified.\nModifies eventObj by adding missing parameters (bubbles, cancelable, widget).</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Get a property from a widget.</p>\n"},"getChildren":{"!type":"fn() -> []","!doc":"<p>Returns all direct children of this widget, i.e. all widgets underneath this.containerNode whose parent\nis this widget.   Note that it does not return all descendants, but rather just direct children.\nAnalogous to <a href=\"https://developer.mozilla.org/en-US/docs/DOM/Node.childNodes\">Node.childNodes</a>,\nexcept containing widgets rather than DOMNodes.\n\n</p>\n<p>The result intentionally excludes internally created widgets (a.k.a. supporting widgets)\noutside of this.containerNode.\n\n</p>\n<p>Note that the array returned is a simple array.  Application code should not assume\nexistence of methods like forEach().</p>\n"},"getDescendants":{"!type":"fn() -> []","!doc":"<p>Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.\nThis method should generally be avoided as it returns widgets declared in templates, which are\nsupposed to be internal/hidden, but it&#39;s left here for back-compat reasons.</p>\n"},"getIndexOfChild":{"!type":"fn(child: ?) -> ?","!doc":"<p>Gets the index of the child in this container or -1 if not found</p>\n"},"getParent":{"!type":"fn() -> ?","!doc":"<p>Returns the parent widget of this widget.</p>\n"},"hasChildren":{"!type":"fn() -> bool","!doc":"<p>Returns true if widget has child widgets, i.e. if this.containerNode contains widgets.</p>\n"},"ioMethod":{"!type":"fn(args: ?) -> ?","!doc":"<p>Function that should grab the content specified via href.</p>\n"},"isFocusable":{"!type":"fn() -> ?","!doc":"<p>Return true if this widget can currently be focused\nand false if not</p>\n"},"isLeftToRight":{"!type":"fn() -> ?","!doc":"<p>Return this widget&#39;s explicit or implicit orientation (true for LTR, false for RTL)</p>\n"},"markupFactory":{"!type":"fn(params: ?, node: ?, ctor: ?) -> +instance"},"on":{"!type":"fn(type: ?, func: ?) -> ?"},"own":{"!type":"fn() -> +any","!doc":"<p>Track specified handles and remove/destroy them when this instance is destroyed, unless they were\nalready removed/destroyed manually.</p>\n"},"placeAt":{"!type":"fn(reference: ?, position?: ?) -> +dijit","!doc":"<p>Place this widget somewhere in the DOM based\non standard domConstruct.place() conventions.</p>\n"},"postCreate":{"!type":"fn()"},"postMixInProperties":{"!type":"fn()"},"refresh":{"!type":"fn() -> +instance","!doc":"<p>[Re]download contents of href and display</p>\n"},"removeChild":{"!type":"fn(widget: ?)","!doc":"<p>Removes the passed widget instance from this widget but does\nnot destroy it.  You can also pass in an integer indicating\nthe index within the container to remove (ie, removeChild(5) removes the sixth widget).</p>\n"},"resize":{"!type":"fn(changeSize: ?, resultSize: ?)","!doc":"<p>See <code>dijit/layout/_LayoutWidget.resize()</code> for description.\nAlthough ContentPane doesn&#39;t extend _LayoutWidget, it does implement\nthe same API.</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> fn","!doc":"<p>Set a property on a widget</p>\n"},"setAttribute":{"!type":"fn(attr: ?, value: ?)","!doc":"<p>Deprecated.  Use set() instead.</p>\n"},"setContent":{"!type":"fn(data: ?)","!doc":"<p>Deprecated.   Use set(&#39;content&#39;, ...) instead.</p>\n"},"setHref":{"!type":"fn(href: ?) -> ?","!doc":"<p>Deprecated.   Use set(&#39;href&#39;, ...) instead.</p>\n"},"startup":{"!type":"fn()"},"subscribe":{"!type":"fn(t: ?, method: ?) -> ?","!doc":"<p>Deprecated, will be removed in 2.0, use this.own(topic.subscribe()) instead.\n\n</p>\n<p>Subscribes to the specified topic and calls the specified method\nof this object and registers for unsubscribe() on widget destroy.\n\n</p>\n<p>Provide widget-specific analog to dojo.subscribe, except with the\nimplicit use of this widget as the target object.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>Returns a string that represents the widget.</p>\n"},"uninitialize":{"!type":"fn() -> bool","!doc":"<p>Deprecated. Override destroy() instead to implement custom widget tear-down\nbehavior.</p>\n"},"unsubscribe":{"!type":"fn(handle: ?)","!doc":"<p>Deprecated, will be removed in 2.0, use handle.remove() instead.\n\n</p>\n<p>Unsubscribes handle created by this.subscribe.\nAlso removes handle from this widget&#39;s list of subscriptions</p>\n"},"watch":{"!type":"fn(name?: ?, callback: ?) -> +any","!doc":"<p>Watches a property for changes</p>\n"}}},"dojox/widget/rotator/Controller":{"!type":"fn()","prototype":{"!doc":"<p>A controller that manipulates a Rotator or AutoRotator.</p>\n","commands":{"!doc":"<p>A comma-separated list of commands. Valid commands are:\n  prev          An icon button to go to the previous pane.\n  next          An icon button to go to the next pane.\n  play/pause    A play and pause toggle icon button.\n  info          Displays the current and total panes. (ie &quot;1 / 4&quot;)\n</p>\n<h1>Displays a number button for each pane. (ie &quot;1 2 3 4&quot;)</h1>\n<p>  titles        Displays each pane&#39;s title as a tab. (ie &quot;Home Services Contact Blog&quot;)</p>\n"},"rotator":{"!doc":"<p>An instance of a Rotator widget.</p>\n"},"constructor":{"!type":"fn(params: ?, node: ?)","!doc":"<p>Initializes the pager and connect to the rotator.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Disconnect from the rotator.</p>\n"}}},"dojox/widget/rotator/Fade":{"crossFade":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that cross fades two rotator panes.</p>\n"},"fade":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that fades out the current pane, then fades in\nthe next pane.</p>\n"}},"dojox/widget/rotator/Pan":{"pan":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that either pans left or right to the next pane.\nThe actual direction depends on the order of the panes.\n\n</p>\n<p>If panning forward from index 1 to 3, it will perform a pan left. If panning\nbackwards from 5 to 1, then it will perform a pan right.\n\n</p>\n<p>If the parameter &quot;continuous&quot; is set to true, it will return an animation\nchain of several pan animations of each intermediate pane panning. For\nexample, if you pan forward from 1 to 3, it will return an animation panning\nleft from 1 to 2 and then 2 to 3.\n\n</p>\n<p>If an easing is specified, it will be applied to each pan transition.  For\nexample, if you are panning from pane 1 to pane 5 and you set the easing to\n&quot;dojo.fx.easing.elasticInOut&quot;, then it will &quot;wobble&quot; 5 times, once for each\npan transition.\n\n</p>\n<p>If the parameter &quot;wrap&quot; is set to true, it will pan to the next pane using\nthe shortest distance in the array of panes. For example, if there are 6\npanes, then panning from 5 to 1 will pan forward (left) from pane 5 to 6 and\n6 to 1.  If the distance is the same either going forward or backwards, then\nit will always pan forward (left).\n\n</p>\n<p>A continuous pan will use the target pane&#39;s duration to pan all intermediate\npanes.  To use the target&#39;s pane duration for each intermediate pane, then\nset the &quot;quick&quot; parameter to &quot;false&quot;.</p>\n"},"panDown":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that pans in the next rotator pane from the top.</p>\n"},"panLeft":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that pans in the next rotator pane from the left.</p>\n"},"panRight":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that pans in the next rotator pane from the right.</p>\n"},"panUp":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that pans in the next rotator pane from the bottom.</p>\n"}},"dojox/widget/rotator/PanFade":{"panFade":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that either pans left or right to the next pane.\nThe actual direction depends on the order of the panes.\n\n</p>\n<p>If panning forward from index 1 to 3, it will perform a pan left. If panning\nbackwards from 5 to 1, then it will perform a pan right.\n\n</p>\n<p>If the parameter &quot;continuous&quot; is set to true, it will return an animation\nchain of several pan animations of each intermediate pane panning. For\nexample, if you pan forward from 1 to 3, it will return an animation panning\nleft from 1 to 2 and then 2 to 3.\n\n</p>\n<p>If an easing is specified, it will be applied to each pan transition.  For\nexample, if you are panning from pane 1 to pane 5 and you set the easing to\n&quot;dojo.fx.easing.elasticInOut&quot;, then it will &quot;wobble&quot; 5 times, once for each\npan transition.\n\n</p>\n<p>If the parameter &quot;wrap&quot; is set to true, it will pan to the next pane using\nthe shortest distance in the array of panes. For example, if there are 6\npanes, then panning from 5 to 1 will pan forward (left) from pane 5 to 6 and\n6 to 1.  If the distance is the same either going forward or backwards, then\nit will always pan forward (left).\n\n</p>\n<p>A continuous pan will use the target pane&#39;s duration to pan all intermediate\npanes.  To use the target&#39;s pane duration for each intermediate pane, then\nset the &quot;quick&quot; parameter to &quot;false&quot;.</p>\n"},"panFadeDown":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that pans in the next rotator pane from the top.</p>\n"},"panFadeLeft":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that pans in the next rotator pane from the left.</p>\n"},"panFadeRight":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that pans in the next rotator pane from the right.</p>\n"},"panFadeUp":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that pans in the next rotator pane from the bottom.</p>\n"}},"dojox/widget/rotator/Slide":{"slideDown":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that slides in the next rotator pane from the top.</p>\n"},"slideLeft":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that slides in the next rotator pane from the left.</p>\n"},"slideRight":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that slides in the next rotator pane from the right.</p>\n"},"slideUp":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that slides in the next rotator pane from the bottom.</p>\n"}},"dojox/widget/rotator/ThumbnailController":{"!type":"fn()","prototype":{"!doc":"<p>A rotator controller that displays thumbnails of each rotator pane.</p>\n","rotator":{"!doc":"<p>An instance of a Rotator widget.</p>\n"},"constructor":{"!type":"fn(params: ?, node: ?)","!doc":"<p>Initializes the thumbnails and connect to the rotator.</p>\n"},"destroy":{"!type":"fn()","!doc":"<p>Disconnect from the rotator.</p>\n"}}},"dojox/widget/rotator/Wipe":{"wipeDown":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that wipes in the next rotator pane from the top.</p>\n"},"wipeLeft":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that wipes in the next rotator pane from the left.</p>\n"},"wipeRight":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that wipes in the next rotator pane from the right.</p>\n"},"wipeUp":{"!type":"fn(args: ?) -> ?","!doc":"<p>Returns a dojo.Animation that wipes in the next rotator pane from the bottom.</p>\n"}},"dojox/xml/Script":{},"dojox/xml/widgetParser":{},"doh/_nodeRunner":{"!doc":"<p>Module for running DOH tests in node (as opposed to a browser).\nAugments return value from doh/runner.</p>\n"},"doh/_parseURLargs":{"isDebug":{},"noGlobals":{},"scopeMap":{}},"doh/_rhinoRunner":{"!doc":"<p>Module for running DOH tests in rhino (as opposed to a browser).\nAugments return value from doh/runner.</p>\n"},"doh/plugins/alwaysAudio":{},"doh/plugins/android-webdriver-robot":{"downKey":{"!type":"fn(sec: ?, charCode: ?, keyCode: ?, delay: ?)"},"moveMouse":{"!type":"fn(sec: ?, x: ?, y: ?, delay: ?, duration: ?)"},"pressMouse":{"!type":"fn(sec: ?, left: ?, middle: ?, right: ?, delay: ?)"},"releaseMouse":{"!type":"fn(sec: ?, left: ?, middle: ?, right: ?, delay: ?)"},"setDocumentBounds":{"!type":"fn(sec: ?, docScreenX: ?, docScreenY: ?, width: ?, height: ?)"},"typeKey":{"!type":"fn(sec: ?, charCode: ?, keyCode: ?, alt: ?, ctrl: ?, shift: ?, meta: ?, delay: ?, async: ?)"},"typeKeys":{"!type":"fn(chars: ?, delay: ?, duration: ?)"},"upKey":{"!type":"fn(sec: ?, charCode: ?, keyCode: ?, delay: ?)"}},"doh/plugins/remoteRobot":{"!doc":"<p>Plugin that bridges the doh.robot and WebDriver APIs.</p>\n"},"doh/plugins/hello":{}},"require":{"!type":"fn(configuration?: ?, dependencies?: [string], callback?: fn()) -> !custom:requireDojo","!doc":"require is used to configure the loader and load AMD modules."},"define":{"!type":"fn(moduleId?: string, dependencies?: [string], factory?: fn()) -> !custom:defineDojo","!doc":"define is very similar to require and is used to define AMD modules."}}

});  
